/* BaseDeFiches - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import net.mapeadores.util.servlets.HttpAccessException;
import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.commands2.CommandInstruction;
import fr.exemole.bdfserver.extension.BdfExtension;
import fr.exemole.bdfserver.extension.BdfExtensionFactoryException;
import fr.exemole.bdfserver.extension.BdfExtensionManager;
import fr.exemole.bdfserver.get.GetInstructionFactory;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.instruction.InstructionUtils;
import fr.exemole.bdfserver.response.SessionResponseParameters;
import fr.exemole.bdfserver.response.XMLReportResponsePrinter;
import fr.exemole.bdfserver.servlets.session.htmlpages.SessionHtmlPage;
import fr.exemole.bdfserver.session.LoginException;
import fr.exemole.bdfserver.session.SessionManager;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.logging.CommandMessageJsonProducer;
import net.mapeadores.util.servlets.MultiPartParser;
import net.mapeadores.util.servlets.ResponseDataType;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.servlets.multipart.DefaultMultiPartParser;


/**
 *
 * @author Vincent Calame
 */
public class BaseDeFichesServlet extends HttpServlet implements ResponseDataType, BdfInstructionConstants {

    private MultiPartParser multiPartParser = new DefaultMultiPartParser();

    @Override
    public void doGet(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response, true);
    }

    @Override
    public void doPost(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response, false);
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response, boolean isGet) throws IOException, ServletException {
        BdfContext bdfContext = ServletContextUtils.getBdfContext(getServletContext(), request);
        BdfServer bdfServer = bdfContext.getBdfServer();
        BdfRequestMap bdfRequestMap = new BdfRequestMap(request, multiPartParser);
        BdfInstruction bdfInstruction = null;
        String first = request.getServletPath().substring(1);
        if (first.startsWith("BDF-")) {
            printNotFound(request, response);
            return;
        }
        if (Character.isUpperCase(first.charAt(0))) {
            if (first.equals("Get")) {
                bdfInstruction = GetInstructionFactory.getBdfInstruction(bdfContext, bdfRequestMap);
            } else if (CommandInstruction.isValidActionName(first)) {
                bdfInstruction = new CommandInstruction(bdfContext, bdfRequestMap, first);
            }
        } else if (first.equals("ext")) {
            String next = request.getPathInfo();
            if (next == null) {
                next = "";
            } else {
                next = next.substring(1);
            }
            if (next.length() == 0) {
                printForbiddenAccess(request, response);
                return;
            }
            int idxext = next.indexOf('/');
            if (idxext == -1) {
                printRedirect(response, next + "/");
                return;
            }
            String extensionName = next.substring(0, idxext);
            String extensionPath = next.substring(idxext);
            BdfExtensionManager bdfExtensionManager = (BdfExtensionManager) bdfContext.getContextObject(BdfContextConstants.BDFEXTENSIONMANAGER_NAME);
            try {
                BdfExtension bdfExtension = bdfExtensionManager.getBdfExtension(bdfContext, extensionName);
                if (bdfExtension == null) {
                    printNotFound(request, response);
                    return;
                }
                bdfInstruction = bdfExtension.getBdfInstruction(extensionPath, bdfRequestMap);
            } catch (BdfExtensionFactoryException befe) {
                Throwable cause = befe.getCause();
                if (cause != null) {
                    printInternalError(response, cause.getClass().getName() + ": " + cause.getMessage());
                } else {
                    printInternalError(response, "BdfExtensionFactoryException: " + befe.getMessage());
                }
                return;
            }
        } else {
            String filePath = request.getPathInfo();
            if (filePath == null) {
                filePath = "";
            } else {
                filePath = filePath.substring(1);
            }
            bdfInstruction = GetInstructionFactory.getBdfInstruction(bdfContext, bdfRequestMap, first, filePath);
        }
        if (bdfInstruction == null) {
            printNotFound(request, response);
            return;
        }
        BdfUser bdfUser = null;
        short needBdfUser = bdfInstruction.getBdfUserNeed();
        if (needBdfUser != NONE_BDFUSER) {
            SessionManager sessionManager = (SessionManager) bdfContext.getContextObject(BdfContextConstants.SESSIONMANAGER_NAME);
            try {
                bdfUser = sessionManager.getBdfUser(bdfRequestMap);
                if (needBdfUser == EXIT_BDFUSER) {
                    sessionManager.invalidateBdfUser(bdfUser);
                    throw new LoginException(LoginException.EXIT_ERROR);
                }

            } catch (LoginException le) {
                short dataType = bdfRequestMap.getResponseDataType();
                LocalisationInfo localisationInfo = InstructionUtils.getLocalisationInfo(bdfRequestMap, bdfServer);
                ResourceLocalisation resourceLocalisation = ServletCommons.getResourceLocalisation(localisationInfo.getFormatLocale());
                SessionResponseParameters sessionResponseParameters = new SessionResponseParameters(bdfServer, localisationInfo);
                String errorKey = le.getLoginErrorKey();
                String errorMessage = le.getMessage(resourceLocalisation);
                if (dataType == JSON_DATATYPE) {
                    CommandMessageJsonProducer jsonSource = new CommandMessageJsonProducer(errorKey, errorMessage, true);
                    ResponseHandler responseHandler = new JsonResponseHandler(jsonSource, false);
                    responseHandler.handleResponse(response);
                } else if (dataType == JSON_IN_HTML_DATATYPE) {
                    CommandMessageJsonProducer jsonSource = new CommandMessageJsonProducer(errorKey, errorMessage, true);
                    ResponseHandler responseHandler = new JsonResponseHandler(jsonSource, true);
                    responseHandler.handleResponse(response);
                } else if (dataType == XML_DATATYPE) {
                    XMLReportResponsePrinter reportXmlData = XMLReportResponsePrinter.newLoginErrorReport(errorKey, errorMessage);
                    reportXmlData.handleResponse(response);
                } else {
                    sessionResponseParameters.setHtmlPageCode(SessionResponseParameters.LOGIN_PAGE);
                    sessionResponseParameters.setProfondeur(ServletCommons.getProfondeur(request));
                    sessionResponseParameters.putParameter(SessionResponseParameters.HTTPSERVLETREQUEST_PARAMNAME, bdfRequestMap.getRequest());
                    sessionResponseParameters.putParameter(SessionResponseParameters.LOGINEXCEPTION_PARAMNAME, le);
                    SessionHtmlPage.getInstance(sessionResponseParameters).handleResponse(response);
                }
                return;
            }
        }
        try {
            ResponseHandler responseHandler = bdfInstruction.runInstruction(bdfUser);
            if (responseHandler == null) {
                printNotFound(request, response);
                return;
            }
            long lastModified = responseHandler.getLastModified();
            if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
                response.setHeader("Cache-Control", "no-store");
            } else if (lastModified > 0) {
                response.setDateHeader("Last-Modified", lastModified);
                try {
                    long since = request.getDateHeader("If-Modified-Since");
                    if (since >= lastModified) {
                        response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                        return;
                    }
                } catch (IllegalArgumentException iae) {
                }
            }

            responseHandler.handleResponse(response);
        } catch (HttpAccessException hae) {
            int type = hae.getHttpType();
            switch (type) {
                case HttpAccessException.NOT_FOUND:
                    printNotFound(request, response);
                    break;
                case HttpAccessException.FORBIDDEN_ACCESS:
                    printForbiddenAccess(request, response);
                    break;
                case HttpAccessException.INTERNAL_ERROR:
                    printInternalError(response, hae.getMessage());
                    break;
                default:
                    throw new SwitchException("unknown httpType : " + type);
            }
        }
    }

    private void printNotFound(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.sendError(HttpServletResponse.SC_NOT_FOUND, request.getServletPath() + request.getPathInfo());
    }

    private void printForbiddenAccess(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.sendError(HttpServletResponse.SC_FORBIDDEN, request.getServletPath() + request.getPathInfo());
    }

    private void printInternalError(HttpServletResponse response, String msg) throws IOException {
        response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, msg);
    }

    private void printRedirect(HttpServletResponse response, String path) throws IOException {
        response.sendRedirect(path);
    }

}
