/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.commands.BdfCommand;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.servlets.multipart.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;


/**
 * Cette servlet effectue toutes les demandes de modifications d'un élément de l'instance de BdfServer
 * liée au contexte de l'application.
 *
 * @author  Vincent Calame
 */
public class CommandServlet extends HttpServlet {

    private MultiPartParser multiPartParser = new DefaultMultiPartParser();

    public void doGet(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response, true);
    }

    public void doPost(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response, false);
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response, boolean isGet) throws IOException, ServletException {
        BdfRequestMap bdfRequestMap = null;
        /*try {*/
        bdfRequestMap = new BdfRequestMap(request, multiPartParser);
        /*} catch(XMLException xmlException) {
        ReportXmlData reportXmlData = ReportXmlData.newExceptionReport(xmlException.getException());
        reportXmlData.printResponse(response);
        return;
        }*/
        boolean xmlOutput = bdfRequestMap.asksForXml();
        if ((isGet) && (bdfRequestMap.getCommandParameter() != null)) {
            ParameterException pe = new ParameterException(ParameterException.GET_METHOD_FORBIDDEN, null);
            if (xmlOutput) {
                reportXmlParameterException(pe, response);
            } else {
                printParameterException(pe, response);
            }
            return;
        }
        try {
            BdfCommand command = null;
            try {
                command = CommandManager.getCommandInstance(bdfRequestMap);
            } catch (ParameterException pe) {
                if (xmlOutput) {
                    reportXmlParameterException(pe, response);
                } else {
                    printParameterException(pe, response);
                }
                return;
            }
            BdfServer bdfServer = ServletContextUtils.getBdfContext(getServletContext(), request).getBdfServer();
            boolean bdfUserNeed = (command.getCommandType() != BdfCommand.NO_USER_COMMAND);
            BdfBundle bdfBundle = ServletCommons.getBdfBundle(this, bdfRequestMap, response, bdfUserNeed);
            if (bdfBundle == null) {
                return;
            }
            try {
                ResponseHandler page = CommandManager.doCommand(command, bdfBundle, bdfRequestMap);
                page.handleResponse(response);
            } catch (ParameterException pe) {
                if (xmlOutput) {
                    reportXmlParameterException(pe, response);
                } else {
                    BdfUser bdfUser = bdfBundle.getBdfUser();
                    if ((pe.isPotentialHumanError()) && (bdfUser != null)) {
                        CommandMessage commandMessage = pe.getCmdMessage();
                        if (commandMessage != null) {
                            printHumanError(bdfServer, bdfUser, response, commandMessage);
                        } else {
                            printParameterException(pe, response);
                        }
                    } else {
                        printParameterException(pe, response);
                    }
                }
            }
        } catch (RuntimeException e) {
            if (xmlOutput) {
                reportXmlException(e, response);
            } else {
                throw e;
            }
        }
    }

    private void reportXmlException(Exception e, HttpServletResponse response) throws IOException {
        XMLReportResponsePrinter reportXmlData = XMLReportResponsePrinter.newExceptionReport(e);
        reportXmlData.handleResponse(response);
    }

    private void reportXmlParameterException(ParameterException e, HttpServletResponse response) throws IOException {
        XMLReportResponsePrinter reportXmlData = XMLReportResponsePrinter.newParameterExceptionReport(e);
        reportXmlData.handleResponse(response);
    }

    private void printHumanError(BdfServer bdfServer, BdfUser bdfUser, HttpServletResponse response, CommandMessage commandMessage) throws IOException {
        BdfResponseParameters responseParameters = new BdfResponseParameters(bdfServer, bdfUser);
        responseParameters.setCommandMessage(commandMessage);
        MessagePage messagePage = new MessagePage(responseParameters);
        messagePage.handleResponse(response);
    }

    private void printParameterException(ParameterException pe, HttpServletResponse response) throws IOException {
        response.setContentType("text/plain;charset=UTF-8");
        PrintWriter pw = response.getWriter();
        pw.print("Parameter error : ");
        pw.print(pe.getText());
    }

}
