/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007-2011 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.balayages.*;
import javax.servlet.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.runners.BalayageRunner;
import fr.exemole.bdfserver.runners.ScrutariExportRunner;
import fr.exemole.bdfserver.runners.SqlExportRunner;
import net.mapeadores.basedefiches.scrutariexport.*;
import java.io.*;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.util.logging.*;


/**
 *
 * @author Vincent Calame
 */
public class RunServlet extends HttpServlet {

    public void doGet(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response);
    }

    public void doPost(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response);
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response) throws IOException {
        BdfServer bdfServer = ServletContextUtils.getBdfContext(getServletContext(), request).getBdfServer();
        String[] sqlExportArray = request.getParameterValues("sqlexport");
        int result = -1;
        if (sqlExportArray != null) {
            if (result == -1) {
                result = 0;
            }
            result = result + runSqlExportArray(bdfServer, sqlExportArray);
        }
        String[] scrutariExportArray = request.getParameterValues("scrutariexport");
        if (scrutariExportArray != null) {
            if (result == -1) {
                result = 0;
            }
            result = result + runScrutariExportArray(bdfServer, scrutariExportArray);
        }
        String[] balayageArray = request.getParameterValues("balayage");
        if (balayageArray != null) {
            if (result == -1) {
                result = 0;
            }
            result = result + runBalayageArray(bdfServer, balayageArray);
        }
        PrintWriter pw = response.getWriter();
        pw.print(result);
        pw.close();
    }

    private int runSqlExportArray(BdfServer bdfServer, String[] array) {
        int errorCount = 0;
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        for (int i = 0; i < array.length; i++) {
            SqlExportDef sqlExportDef = sqlExportManager.getSqlExportDef(array[i]);
            if (sqlExportDef != null) {
                try {
                    CommandMessage message = SqlExportRunner.run(bdfServer, sqlExportDef);
                    if (message.isErrorMessage()) {
                        errorCount++;
                    }
                } catch (IOException ioe) {
                    errorCount++;
                }
            } else {
                errorCount++;
            }
        }
        return errorCount;
    }

    private int runScrutariExportArray(BdfServer bdfServer, String[] array) {
        int result = 0;
        ScrutariExportManager scrutariExportManager = bdfServer.getScrutariExportManager();
        for (int i = 0; i < array.length; i++) {
            ScrutariExportDef scrutariExportDef = scrutariExportManager.getScrutariExportDef(array[i]);
            if (scrutariExportDef != null) {
                try {
                    ScrutariExportRunner.run(scrutariExportDef, bdfServer);
                } catch (IOException ioe) {
                    result++;
                }
            } else {
                result++;
            }
        }
        return result;
    }

    private int runBalayageArray(BdfServer bdfServer, String[] array) {
        int result = 0;
        BalayageManager balayageManager = bdfServer.getBalayageManager();
        for (int i = 0; i < array.length; i++) {
            String balayageName = array[i];
            String mode = null;
            int idx = balayageName.indexOf('/');
            if (idx != -1) {
                mode = balayageName.substring(idx + 1);
                balayageName = balayageName.substring(0, idx);
            }
            Balayage balayage = balayageManager.getBalayage(balayageName);
            if (balayage != null) {
                BalayageRunner.Result balaygeResult = BalayageRunner.run(balayage, mode, bdfServer);
                if (!balaygeResult.hasInitError()) {
                    result++;
                }
            } else {
                result++;
            }
        }
        return result;
    }

}
