/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2011 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.session.*;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.managers.transformation.*;
import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.instruction.InstructionUtils;
import fr.exemole.bdfserver.servlets.session.htmlpages.*;
import java.util.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.localisation.*;


/**
 *
 * @author Vincent Calame
 */
public class ServletCommons implements SessionConstants {

    private static Map<Locale, ResourceLocalisation> localisationMap = new HashMap<Locale, ResourceLocalisation>();

    static {
        Locale.setDefault(Locale.FRENCH);
    }

    private ServletCommons() {
    }

    public static BdfBundle getBdfBundle(HttpServlet servlet, BdfRequestMap bdfRequestMap, HttpServletResponse response, boolean bdfUserNeed) throws IOException {
        response.setHeader("Cache-Control", "no-store");
        BdfContext bdfContext = ServletContextUtils.getBdfContext(servlet.getServletContext(), bdfRequestMap.getRequest());
        BdfServer bdfServer = bdfContext.getBdfServer();
        SessionManager sessionManager = (SessionManager) bdfContext.getContextObject(BdfContextConstants.SESSIONMANAGER_NAME);
        BdfBundle bdfBundle = null;
        try {
            BdfUser bdfUser = sessionManager.getBdfUser(bdfRequestMap);
            bdfBundle = new BdfBundle(bdfServer, bdfUser);

        } catch (LoginException le) {
            if (!bdfUserNeed) {
                bdfBundle = new BdfBundle(bdfServer, null);
            } else {
                LocalisationInfo localisationInfo = InstructionUtils.getLocalisationInfo(bdfRequestMap, bdfServer);
                if (bdfRequestMap.asksForXml()) {
                    XMLReportResponsePrinter reportXmlData = XMLReportResponsePrinter.newLoginErrorReport(le.getLoginErrorKey(), le.getMessage(getResourceLocalisation(localisationInfo.getFormatLocale())));
                    reportXmlData.handleResponse(response);
                } else {
                    SessionResponseParameters sessionResponseParameters = new SessionResponseParameters(bdfServer, localisationInfo);
                    sessionResponseParameters.setHtmlPageCode(SessionResponseParameters.LOGIN_PAGE);
                    sessionResponseParameters.setProfondeur(getProfondeur(bdfRequestMap.getRequest()));
                    sessionResponseParameters.putParameter(SessionResponseParameters.HTTPSERVLETREQUEST_PARAMNAME, bdfRequestMap.getRequest());
                    sessionResponseParameters.putParameter(SessionResponseParameters.LOGINEXCEPTION_PARAMNAME, le);
                    SessionHtmlPage.getInstance(sessionResponseParameters).handleResponse(response);

                }
                return null;
            }
        }
        bdfBundle.putObject(SESSIONMANAGER_KEY, sessionManager);
        bdfBundle.putObject(SERVLETCONTEXT_KEY, servlet.getServletContext());
        return bdfBundle;
    }

    public static int getProfondeur(HttpServletRequest httpServletRequest) {
        String servletPath = httpServletRequest.getServletPath();
        int p = 0;
        int servletPathLength = servletPath.length();
        for (int i = 1; i < servletPathLength; i++) {
            char carac = servletPath.charAt(i);
            if (carac == '/') {
                p++;
            }
        }
        String pathInfo = httpServletRequest.getPathInfo();
        if (pathInfo != null) {
            int pathInfoLength = pathInfo.length();
            for (int i = 0; i < pathInfoLength; i++) {
                char carac = pathInfo.charAt(i);
                if (carac == '/') {
                    p++;
                }
            }
        }
        return p;
    }

    public static BdfBundle getBdfBundle(HttpServlet servlet, BdfRequestMap bdfRequestMap, boolean bdfUserNeed) throws IOException, LoginException {
        ServletContext servletContext = servlet.getServletContext();
        BdfContext bdfContext = ServletContextUtils.getBdfContext(servletContext, bdfRequestMap.getRequest());
        BdfServer bdfServer = bdfContext.getBdfServer();
        SessionManager sessionManager = (SessionManager) bdfContext.getContextObject(BdfContextConstants.SESSIONMANAGER_NAME);
        BdfUser bdfUser = sessionManager.getBdfUser(bdfRequestMap);
        BdfBundle bdfBundle = new BdfBundle(bdfServer, bdfUser);
        bdfBundle.putObject(SESSIONMANAGER_KEY, sessionManager);
        bdfBundle.putObject(SERVLETCONTEXT_KEY, servlet.getServletContext());
        return bdfBundle;
    }

    public static InputStream getTemplateInputStream(String name) {
        return ServletCommons.class.getResourceAsStream("resources/templates/" + name + ".html");
    }

    public static ResourceLocalisation getResourceLocalisation(Locale uiLocale) {
        return BundleLocalisation.getBundleLocalisation("fr.exemole.bdfserver.servlets.resources.l10n.servlets", uiLocale);
    }

    /*private static void initListeners(BdfServer bdfServer, File extensionDirectory, ServletContext servletContext) {
    if (!extensionDirectory.exists()) {
    return;
    }
    File list = new File(extensionDirectory, "listenerfactory-list.conf");
    if ((!list.exists()) || (list.isDirectory())) {
    return;
    }
    HashMap objectMap = new HashMap();
    objectMap.put(SERVLETCONTEXT_KEY, servletContext);
    ListenerInitializer initializer = new ListenerInitializer(bdfServer, objectMap);
    try {
    initializer.init(list);
    } catch (IOException ioe) {
    throw new NestedIOException(ioe);
    }
    }*/
    

    public static String getTemplatePath(XslTransformerKey xslTransformerKey) {
        String templateName = xslTransformerKey.getTransformerName();
        StringBuffer buf = new StringBuffer();
        buf.append("xslt/");
        if (templateName == null) {
            buf.append("_default_");
            buf.append(xslTransformerKey.getTransformationKey().toString());
        } else {
            buf.append(xslTransformerKey.getTransformationKey().toString());
            buf.append("/");
            buf.append(templateName);
        }
        buf.append(".xsl");
        return buf.toString();
    }

    /*private static Directories initDefaultDirectories(File contextDirectory) {
    Directories directories = new Directories();
    directories.setContextDirectory(contextDirectory);
    directories.setBdfStorageDirectory(new File(contextDirectory, "BDF-INF"));
    directories.setTransformationDirectory(new File(contextDirectory, "_xslt"));
    directories.setBalayageDirectory(new File(contextDirectory, "_balayages"));
    directories.setResourceDirectory(new File(contextDirectory, "_rsc"));
    directories.setTableExportDirectory(new File(contextDirectory, "_tableexport"));
    directories.setBdfExtDirectory(new File(contextDirectory, "BDF-EXT"));
    directories.setOutputDirectory(new File(contextDirectory, "BDF-OUT"));
    return directories;
    }*/
}
