/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.commands.BdfCommand;
import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.extension.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.xml.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.localisation.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class TestServlet extends HttpServlet {

    public void doGet(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response);
    }

    public void doPost(HttpServletRequest requete, HttpServletResponse response) throws IOException, ServletException {
        processRequest(requete, response);
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response) throws IOException {
        TestXMLWriter testXMLWriter = initTestXMLWriter(response);
        CommandTest commandTest = initCommandTest(request, testXMLWriter);
        testXMLWriter.appendCommandTest(commandTest);
    }

    private CommandTest initCommandTest(HttpServletRequest request, TestXMLWriter testXMLWriter) {
        BdfRequestMap bdfRequestMap = new BdfRequestMap(request);
        try {
            BdfBundle bdfBundle = ServletCommons.getBdfBundle(this, bdfRequestMap, true);
            Locale uiLocale = bdfBundle.getBdfUser().getLocalisationInfo().getFormatLocale();
            ResourceLocalisation resourceLocalisation = ServletCommons.getResourceLocalisation(uiLocale);
            testXMLWriter.setResourceLocalisation(resourceLocalisation);
            BdfCommand command = null;
            try {
                command = CommandManager.getCommandInstance(bdfRequestMap);
            } catch (ParameterException pe) {
                return AbstractBdfCommand.convertToErrorCommandText(pe);
            }
            command.setBdfBundle(bdfBundle);
            command.setRequestMap(bdfRequestMap);
            return command.testCommand();
        } catch (RuntimeException e) {
            e.printStackTrace();
            return null;
        } catch (Exception ex) {
            return null;
        }
    }

    private TestXMLWriter initTestXMLWriter(HttpServletResponse response) throws IOException {
        response.setContentType("text/xml;charset=UTF-8");
        TestXMLWriter testXMLWriter = new TestXMLWriter();
        testXMLWriter.setAppendable(response.getWriter());
        testXMLWriter.setIndentLength(0);
        return testXMLWriter;
    }


    private class TestXMLWriter extends DefaultXMLWriter {

        private ResourceLocalisation messageLocalisation;

        TestXMLWriter() {
        }

        private void setResourceLocalisation(ResourceLocalisation messageLocalisation) {
            this.messageLocalisation = messageLocalisation;
        }

        private void appendCommandTest(CommandTest commandTest) throws IOException {
            int result = 0;
            if (commandTest != null) {
                if (commandTest.mayBeDone()) {
                    result = 1;
                } else {
                    result = -1;
                }
            }
            startOpenTag("test");
            addAttribute("result", result);
            endOpenTag();
            if (result == -1) {
                CommandMessage commandMessage = commandTest.getCommandMessage();
                ResourceItem keyResourceItem = commandMessage.getTitleResourceItem();
                if (commandMessage.hasMultiError()) {
                    startOpenTag("error-list");
                    addAttribute("key", keyResourceItem.getResourceKey());
                    addAttribute("msg", messageLocalisation.toString(keyResourceItem));
                    endOpenTag();
                    List list = commandMessage.getMultiErrorList();
                    for (Iterator it = list.iterator(); it.hasNext();) {
                        ResourceItem resourceItem = (ResourceItem) it.next();
                        appendError(resourceItem);
                    }
                    closeTag("error-list");
                } else {
                    appendError(keyResourceItem);
                }
            }
            closeTag("test");
        }

        private void appendError(ResourceItem keyResourceItem) throws IOException {
            startOpenTag("error");
            addAttribute("key", keyResourceItem.getResourceKey());
            addAttribute("msg", messageLocalisation.toString(keyResourceItem));
            closeEmptyTag();
        }

    }

}
