/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import javax.servlet.http.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.roles.*;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommand extends AbstractAdministrationCommand {

    public final static String DELETEFORM_PAGE_PARAMVALUE = "deleteform";
    public final static String HORSSPHEREFORM_PAGE_PARAMVALUE = "horssphereform";
    public final static String SESSION_PAGE_PARAMVALUE = "session";
    public final static String MEMORYTEST_PAGE_PARAMVALUE = "memorytest";
    public final static String TRANSFORMATIONINDEX_PAGE_PARAMVALUE = "transformationindex";
    public final static String TABLEEXPORTINDEX_PAGE_PARAMVALUE = "tableexportindex";
    public final static String ROLEINDEX_PAGE_PARAMVALUE = "roleindex";
    public final static String ROLECREATION_PAGE_PARAMVALUE = "rolecreation";
    public final static String ROLEREMOVE_PAGE_PARAMVALUE = "roleremove";
    public final static String ROLEFORM_PAGE_PARAMVALUE = "roleform";
    public final static String BALAYAGEINDEX_PAGE_PARAMVALUE = "balayageindex";
    public final static String REINIT_PARAMETER = "reinit";
    private short pageType;
    private boolean reinit = false;

    public DefaultCommand() {
    }

    public static String getCommandString() {
        return null;
    }

    protected void doAdministrationCommand() {
        responseParameters.setHtmlPageCode(pageType);
        if (reinit) {
            doReinit();
        }
    }

    protected void checkAdministrationParameters() throws ParameterException {
        pageType = getPageType();
        if (needRoleInit(pageType)) {
            Role role = initRole();
            responseParameters.setRole(role);
        }
        String reinitString = requestMap.getParameter(REINIT_PARAMETER);
        if ((reinitString != null) && (reinitString.equals("1"))) {
            reinit = true;
        }
    }

    private short getPageType() throws ParameterException {
        if (!requestMap.isPageDefined()) {
            return AdministrationResponseParameters.INDEX_PAGE;
        }
        String page_param = requestMap.getPageParameter();
        if (page_param.equals(DELETEFORM_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.DELETEFORM_PAGE;
        } else if (page_param.equals(SESSION_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.SESSION_PAGE;
        } else if (page_param.equals(HORSSPHEREFORM_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.HORSSPHEREFORM_PAGE;
        } else if (page_param.equals(MEMORYTEST_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.MEMORYTEST_PAGE;
        } else if (page_param.equals(TRANSFORMATIONINDEX_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.TRANSFORMATIONINDEX_PAGE;
        } else if (page_param.equals(TABLEEXPORTINDEX_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.TABLEEXPORTINDEX_PAGE;
        } else if (page_param.equals(ROLEINDEX_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.ROLEINDEX_PAGE;
        } else if (page_param.equals(ROLECREATION_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.ROLECREATION_PAGE;
        } else if (page_param.equals(ROLEREMOVE_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.ROLEREMOVE_PAGE;
        } else if (page_param.equals(ROLEFORM_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.ROLEFORM_PAGE;
        } else if (page_param.equals(BALAYAGEINDEX_PAGE_PARAMVALUE)) {
            return AdministrationResponseParameters.BALAYAGEINDEX_PAGE;
        }
        throw new ParameterException(ParameterException.PAGE_WRONGVALUE, page_param);
    }

    private boolean needRoleInit(short pageType) {
        switch (pageType) {
            case AdministrationResponseParameters.ROLEREMOVE_PAGE:
            case AdministrationResponseParameters.ROLEFORM_PAGE:
                return true;
            default:
                return false;
        }
    }

    private void doReinit() {
        switch (pageType) {
            case AdministrationResponseParameters.TRANSFORMATIONINDEX_PAGE:
                bdfServer.getTransformationManager().update();
                responseParameters.setDoneMessage("admin_done_transformationupdate", null);
                break;
            case AdministrationResponseParameters.TABLEEXPORTINDEX_PAGE:
                bdfServer.getTableExportManager().update();
                responseParameters.setDoneMessage("admin_done_tableexportupdate", null);
                break;
            case AdministrationResponseParameters.BALAYAGEINDEX_PAGE:
                bdfServer.getBalayageManager().update();
                responseParameters.setDoneMessage("admin_done_balayageupdate", null);
                break;
        }
    }

}
