/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.roles.*;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class RoleChangeCommand extends AbstractAdministrationCommand {

    public final static String LIB_PARAMPREFIX = "lib";
    public final static String PERMISSION_PARAMNAME = "permission";
    public final static String CREATEFICHE_PARAMSUFFIX = "|createfiche";
    public final static String READFICHE_PARAMSUFFIX = "|readfiche";
    public final static String WRITEFICHE_PARAMSUFFIX = "|writefiche";
    public final static String SENDFICHE_PARAMSUFFIX = "|sendfiche";
    private Role role;
    private ChangeLibelleHolder changeLibelleHolder;
    private Map corpusPermissionMap = new HashMap();

    public RoleChangeCommand() {
    }

    public void doAdministrationCommand() {
        int libelleCount = changeLibelleHolder.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            role.putLibelle(changeLibelleHolder.getLibelleByIndex(i));
        }
        int removedCount = changeLibelleHolder.getRemovedLangCount();
        for (int i = 0; i < removedCount; i++) {
            role.removeLibelle(changeLibelleHolder.getRemovedLangInteger(i));
        }
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            SubsetKey corpusKey = corpus.getSubsetKey();
            CorpusPermission corpusPermission = (CorpusPermission) corpusPermissionMap.get(corpusKey);
            if (corpusPermission == null) {
                role.removeCorpusPermission(corpusKey);
            } else {
                role.setCorpusPermission(corpusKey, corpusPermission);
            }
        }
        bdfServer.getPermissionManager().saveRole(role);
        responseParameters.setHtmlPageCode(AdministrationResponseParameters.ROLEFORM_PAGE);
        responseParameters.setDoneMessage("admin_done_rolechange", null);
    }

    protected void checkAdministrationParameters() throws ParameterException {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        role = RequestUtils.getRole(requestMap, bdfServer.getPermissionManager());
        changeLibelleHolder = RequestUtils.getChangeLibelleHolder(requestMap, LIB_PARAMPREFIX + "/", workingArray);
        responseParameters.setRole(role);

        String[] permissionArray = requestMap.getParameterValues(PERMISSION_PARAMNAME);
        if (permissionArray != null) {
            for (int i = 0; i < permissionArray.length; i++) {
                try {
                    SubsetKey subsetKey = SubsetKey.parse(permissionArray[i]);
                    if (subsetKey.isCorpusSubset()) {
                        initCorpusPermission(subsetKey);
                    }
                } catch (java.text.ParseException pe) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, PERMISSION_PARAMNAME + " = " + permissionArray[i]);
                }
            }
        }
    }

    private void initCorpusPermission(SubsetKey corpusKey) throws ParameterException {
        String corpusString = corpusKey.toString();
        String createFicheParamName = corpusString + CREATEFICHE_PARAMSUFFIX;
        CorpusPermission corpusPermission = new CorpusPermission();
        String createFicheString = requestMap.getParameter(createFicheParamName);
        if (createFicheString == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, createFicheParamName);
        }
        if (createFicheString.equals("0")) {
            corpusPermission.setCreateFichePermission(false);
        } else if (createFicheString.equals("1")) {
            corpusPermission.setCreateFichePermission(true);
        } else {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, createFicheParamName + " = " + createFicheString);
        }

        String readFicheParamName = corpusString + READFICHE_PARAMSUFFIX;
        short type = getType(readFicheParamName);
        corpusPermission.getReadFichePermission().setType(type);

        String writeFicheParamName = corpusString + WRITEFICHE_PARAMSUFFIX;
        short writeType = getType(writeFicheParamName);
        corpusPermission.getWriteFichePermission().setType(writeType);


        corpusPermissionMap.put(corpusKey, corpusPermission);
    }

    private short getType(String paramName) throws ParameterException {
        String ficheActionString = requestMap.getParameter(paramName);
        if (ficheActionString == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, paramName);
        }
        try {
            short type = FicheActionPermission.typeToShort(ficheActionString);
            return type;
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, paramName + " = " + ficheActionString);
        }
    }

    public static String getCommandString() {
        return "RoleChange";
    }

    public static String getCommandKey() {
        return "ROL-02";
    }

}
