/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration;

import net.mapeadores.basedefiches.tableexport.*;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.servlets.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.managers.tableexport.*;
import java.io.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class TableExportCreateCommand extends AbstractAdministrationCommand {

    TableExportParametersBean tableExportParametersBean;
    String tableExportName;

    public TableExportCreateCommand() {
    }

    public void doAdministrationCommand() {
        TableExportManager manager = bdfServer.getTableExportManager();
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            ColDefList colDefList = TableExportUtils.getColDefListFromCorpusUI(bdfServer, corpus, tableExportParametersBean);
            try {
                manager.saveColDefList(tableExportName, corpus.getSubsetKey(), colDefList);
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }
        manager.update();
        responseParameters.setHtmlPageCode(AdministrationResponseParameters.TABLEEXPORTINDEX_PAGE);
        responseParameters.setDoneMessage("admin_done_tableexportcreate", tableExportName);
    }

    protected void checkAdministrationParameters() throws ParameterException {
        tableExportParametersBean = TableExportParameters.getTableExportParametersBean(requestMap, true, false);
        tableExportName = tableExportParametersBean.getTableExportName().trim();
        if (tableExportName.length() == 0) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.TABLEEXPORTINDEX_PAGE);
            responseParameters.setErrorMessage("admin_err_emptytableexportname", tableExportName);
            return;
        }
        if (!StringUtils.isTechnicalName(tableExportName, true)) {
            responseParameters.setHtmlPageCode(AdministrationResponseParameters.TABLEEXPORTINDEX_PAGE);
            responseParameters.setErrorMessage("admin_err_wrongtableexportname", tableExportName);
            return;
        }
        tableExportParametersBean.setTableExportName(tableExportName);
    }

    public static String getCommandString() {
        return "TableExportCreate";
    }

    public static String getCommandKey() {
        return "EXP-02";
    }

}
