/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.servlets.administration.*;
import fr.exemole.bdfserver.servlets.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.roles.*;
import fr.exemole.bdfserver.servlets.resources.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class AdministrationHtmlPage extends BdfHtmlPage {

    AdministrationResponseParameters responseParameters;
    protected short pageType;

    public AdministrationHtmlPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
        this.responseParameters = responseParameters;
        this.pageType = responseParameters.getResponseCode();
    }

    public static ResponseHandler getInstance(AdministrationResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case BdfResponseParameters.MESSAGE_SERVLETPAGE:
                return new MessagePage(responseParameters);
            case AdministrationResponseParameters.INDEX_PAGE:
                return new IndexPage(responseParameters);
            case AdministrationResponseParameters.SESSION_PAGE:
                return new SessionPage(responseParameters);
            case AdministrationResponseParameters.DELETEFORM_PAGE:
                return new DeleteFormPage(responseParameters);
            case AdministrationResponseParameters.HORSSPHEREFORM_PAGE:
                return new HorsSphereFormPage(responseParameters);
            case AdministrationResponseParameters.DELETECONFIRM_PAGE:
                return new DeleteConfirmPage(responseParameters);
            case AdministrationResponseParameters.BACKUP_PAGE:
                return new BackupPage(responseParameters);
            case AdministrationResponseParameters.MEMORYTEST_PAGE:
                return new MemoryTestPage(responseParameters);
            case AdministrationResponseParameters.TRANSFORMATIONINDEX_PAGE:
                return new TransformationIndexPage(responseParameters);
            case AdministrationResponseParameters.TABLEEXPORTINDEX_PAGE:
                return new TableExportIndexPage(responseParameters);
            case AdministrationResponseParameters.ROLEINDEX_PAGE:
                return new RoleIndexPage(responseParameters);
            case AdministrationResponseParameters.ROLECREATION_PAGE:
                return new RoleCreationPage(responseParameters);
            case AdministrationResponseParameters.ROLEFORM_PAGE:
                return new RoleFormPage(responseParameters);
            case AdministrationResponseParameters.ROLEREMOVE_PAGE:
                return new RoleRemovePage(responseParameters);
            case AdministrationResponseParameters.BALAYAGEINDEX_PAGE:
                return new BalayageIndexPage(responseParameters);
        }
        return null;
    }

    public String getTitle() {
        return "Administration";
    }

    public String getFormAction() {
        return "Administration";
    }

    public String getCssThemeFile() {
        return "administration.css";
    }

    protected void printRoleTitre(Role role) {
        startP(CssClass.P_TOOLBARLINKS);
        appendA("Administration?page=" + DefaultCommand.ROLEINDEX_PAGE_PARAMVALUE, "admin_title_roleindex");
        printTiret();
        printRoleLinks(role);
        endP();
        startH1();
        printLibColon("admin_lib_role");
        printRoleIntitule(role);
        endH1();
        printMessage();
    }

    protected void printRoleLinks(Role role) {
        String href = "Administration?role=" + role.getName();
        if (pageType == AdministrationResponseParameters.ROLEFORM_PAGE) {
            printNoLink("global_link_change");
        } else {
            appendA(href + "&amp;page=" + DefaultCommand.ROLEFORM_PAGE_PARAMVALUE, "global_link_change");
        }
        if (!bdfServer.getPermissionManager().isDefaultRole(role)) {
            printTiret();
            if (pageType == AdministrationResponseParameters.ROLEREMOVE_PAGE) {
                printNoLink("global_link_remove");
            } else {
                appendA(href + "&amp;page=" + DefaultCommand.ROLEREMOVE_PAGE_PARAMVALUE, "global_link_remove");
            }
        }
    }

    protected void printNoLink(String key) {
        startSpan(CssClass.SPAN_CURRENTPAGE);
        localize(key);
        endSpan();
    }

}
