/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.administration.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.roles.*;
import java.util.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.ui.*;


/**
 *
 * @author Vincent Calame
 */
public class RoleFormPage extends AdministrationHtmlPage {

    Role role;
    LangConfig langConfig;
    PermissionManager permissionManager;
    boolean isDefaultRole;

    public RoleFormPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(BDF_JS);
        role = responseParameters.getRole();
        langConfig = bdfServer.getConfigurationManager().getLangConfig();
        permissionManager = bdfServer.getPermissionManager();
        isDefaultRole = permissionManager.isDefaultRole(role);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printRoleTitre(role);
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Administration");
        commandBox.setCommandFamily("ROL");
        commandBox.setCommandString(RoleChangeCommand.getCommandString());
        commandBox.setCommandKey(RoleChangeCommand.getCommandKey());
        CommandBoxUtils.insertRole(commandBox, role);
        startCommandBox(commandBox);
        printIntitules();
        printCorpusPermissionList();
        endCommandBox("admin_submit_rolechange");
        end();
    }

    private void printIntitules() {
        appendH3("admin_title_roleintitules");
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        LibelleHolder libelleHolder = role.getRoleLibelleHolder();
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            String intitule = libelleHolder.getLibelleString(langInteger, false);
            if (intitule == null) {
                intitule = "";
            }
            printLangFieldRow(RoleChangeCommand.LIB_PARAMPREFIX, langInteger, intitule);
        }
        endTable();
    }

    private void printCorpusPermissionList() {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        appendH3("admin_title_corpuspermission");
        startUl();
        SubsetGroupList subsetGroupList = bdfServer.getUIManager().getCorpusSubsetKeyTree();
        int count = subsetGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup subsetKeyGroup = subsetGroupList.getSubsetGroup(i);
            boolean isTemplate = !subsetKeyGroup.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startP();
                startStrong();
                escape(subsetKeyGroup.getLibelleString(workingLangInteger));
                endStrong();
                endP();
                startUl();
            }
            for (Iterator it2 = subsetKeyGroup.getSubsetKeyList().iterator(); it2.hasNext();) {
                SubsetKey subsetKey = (SubsetKey) it2.next();
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                startLi();
                printCorpusPermission(corpus);
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpusPermission(Corpus corpus) {
        startLi();
        SubsetKey corpusKey = corpus.getSubsetKey();
        startP();
        startCode();
        append("[");
        append(corpusKey.toString());
        append("]");
        endCode();
        printTiret();
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        endP();

        CorpusPermission corpusPermission = role.getCorpusPermission(corpusKey);
        boolean permissionDefined = (corpusPermission != null);
        if ((isDefaultRole) && (corpusPermission == null)) {
            corpusPermission = CorpusPermission.newDefaultInstance();
            permissionDefined = true;
        }

        HTMLAttributes checkAttributes = new HTMLAttributes();
        if (isDefaultRole) {
            checkAttributes.setDisabled(true);
            checkAttributes.setChecked(true);
        } else {
            if (isWithJQuery()) {
                checkAttributes.setOnChange("BDF.deployFromCheck(this,'" + corpusKey.toString() + "');");
            }
            checkAttributes.setChecked(permissionDefined);
            checkAttributes.setName(RoleChangeCommand.PERMISSION_PARAMNAME);
            checkAttributes.setValue(corpusKey.toString());
        }
        startP(CssClass.P_SUBTITLE);
        if (isDefaultRole) {
            addParameter(RoleChangeCommand.PERMISSION_PARAMNAME, corpusKey.toString());
        }
        addCheck(checkAttributes, "admin_lib_corpuspermission", null);
        endP();

        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId(corpusKey.toString());
        divAttributes.addClass(DIV_ADM_PERMISSION);
        if (isWithJQuery()) {
            if (!permissionDefined) {
                divAttributes.addClass(INVISIBLE);
            }
        }
        startDiv(divAttributes);

        HTMLAttributes tableAttributes = new HTMLAttributes();
        tableAttributes.setClasses(TABLE_ADM_CORPUSPERMISSION);
        startTable(tableAttributes);
        startRow();
        startCell();
        localize("admin_lib_createfiche");
        endCell();
        startCell();
        localize("admin_lib_readfiche");
        endCell();
        startCell();
        localize("admin_lib_writefiche");
        endCell();
        startCell();
        localize("admin_lib_sendfiche");
        endCell();
        endRow();


        startRow();
        startCell();
        printCreateFiche(corpusKey, corpusPermission);
        endCell();
        startCell();
        printReadFiche(corpusKey, corpusPermission);
        endCell();
        startCell();
        printWriteFiche(corpusKey, corpusPermission);
        endCell();
        startCell();
        printSendFiche(corpusKey, corpusPermission);
        endCell();
        endRow();

        endTable();

        endDiv();

        endLi();
    }

    private void printCreateFiche(SubsetKey corpusKey, CorpusPermission corpusPermission) {
        boolean yes = true;
        if (corpusPermission != null) {
            yes = corpusPermission.canCreateFiche();
        }
        HTMLAttributes radioAttributes = new HTMLAttributes();
        radioAttributes.setName(corpusKey.toString() + RoleChangeCommand.CREATEFICHE_PARAMSUFFIX);
        radioAttributes.setChecked(yes);
        radioAttributes.setValue("1");
        addRadio(radioAttributes, "global_lib_yes", null);
        appendBr();
        radioAttributes.setChecked(!yes);
        radioAttributes.setValue("0");
        addRadio(radioAttributes, "global_lib_no", null);
    }

    private void printReadFiche(SubsetKey corpusKey, CorpusPermission corpusPermission) {
        short current = FicheActionPermission.ALL;
        if (corpusPermission != null) {
            current = corpusPermission.getReadFichePermission().getType();
        }
        printRadios(corpusKey + RoleChangeCommand.READFICHE_PARAMSUFFIX, current);
    }

    private void printWriteFiche(SubsetKey corpusKey, CorpusPermission corpusPermission) {
        short current = FicheActionPermission.OWN;
        if (corpusPermission != null) {
            current = corpusPermission.getWriteFichePermission().getType();
        }
        printRadios(corpusKey + RoleChangeCommand.WRITEFICHE_PARAMSUFFIX, current);
    }

    private void printSendFiche(SubsetKey corpusKey, CorpusPermission corpusPermission) {
        short current = FicheActionPermission.ALL;
        if (corpusPermission != null) {
            current = corpusPermission.getReadFichePermission().getType();
        }
    }

    private void printRadios(String name, short current) {
        HTMLAttributes radioAttributes = new HTMLAttributes();
        radioAttributes.setName(name);

        printRadio(radioAttributes, FicheActionPermission.NONE, current);
        appendBr();
        printRadio(radioAttributes, FicheActionPermission.OWN, current);
        appendBr();
        printRadio(radioAttributes, FicheActionPermission.SPHERE, current);

        appendBr();
        printRadio(radioAttributes, FicheActionPermission.ALL, current);

    }

    private void printRadio(HTMLAttributes radioAttributes, short type, short current) {
        String typeString = FicheActionPermission.typeToString(type);
        radioAttributes.setChecked((type == current));
        radioAttributes.setValue(FicheActionPermission.typeToString(type));
        addRadio(radioAttributes, "admin_lib_ficheaction_" + typeString, null);
    }

}
