/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.administration.htmlpages;

import fr.exemole.bdfserver.response.AdministrationResponseParameters;
import fr.exemole.bdfserver.managers.transformation.*;
import fr.exemole.bdfserver.servlets.administration.*;
import fr.exemole.bdfserver.xsltfactory.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.html.*;


/**
 *
 * @author Vincent Calame
 */
public class TransformationIndexPage extends AdministrationHtmlPage implements XSLTFactoryConstants {

    TransformationManager transformationManager;
    List noTemplateList = new ArrayList();
    List allTransformationList = new ArrayList();
    HTMLAttributes validA = new HTMLAttributes();

    {
        validA.setTarget("_NEW");
    }

    HTMLAttributes invalidA = new HTMLAttributes();

    {
        invalidA.setTarget("_NEW");
    }

    HTMLAttributes currentA = validA;

    public TransformationIndexPage(AdministrationResponseParameters responseParameters) {
        super(responseParameters);
        this.transformationManager = bdfServer.getTransformationManager();
        allowsTest();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendH1("admin_title_transformationindex");
        printMessage();
        startUl(UlAttrs.getLinkAttr());
        printLi(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
        printLi(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY);
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            TransformationKey transformationKey = new TransformationKey(corpus.getSubsetKey());
            printLi(transformationKey);
        }
        int noTemplateSize = noTemplateList.size();
        if (noTemplateSize > 0) {
            startLi();
            startP();
            startSmall();
            HtmlPrinterUtils.printLibColon(this, "admin_info_notransformation");
            endSmall();
            startCode();
            for (int i = 0; i < noTemplateSize; i++) {
                if (i > 0) {
                    append(", ");
                }
                append(noTemplateList.get(i).toString());
            }
            endCode();
            endP();
            endLi();
        }
        startLi();
        startP();
        startSmall();
        append("(");
        appendA("Administration?page=" + DefaultCommand.TRANSFORMATIONINDEX_PAGE_PARAMVALUE + "&reinit=1", "admin_link_transformationupdate");
        append(")");
        endSmall();
        endP();
        endLi();
        endUl();

        printTransformationCreateForm();

        end();
    }

    private void printLi(TransformationKey transformationKey) {
        allTransformationList.add(transformationKey);
        TransformationInfo transformationInfo = transformationManager.getTransformationInfo(transformationKey);
        int xslCount = transformationInfo.getXslTransformerCount();
        int extensionCount = transformationInfo.getExtensionListCount();
        XslTransformerInfo[] unvalidXslArray = transformationInfo.getUnvalidXslTransformerInfoArray();
        TransformationInfo.ListByExtension[] byExtensionErrorArray = transformationInfo.getUnvalidExtensionListArray();
        int errorLength = unvalidXslArray.length + byExtensionErrorArray.length;

        if ((xslCount == 0) && (extensionCount == 0) && (errorLength == 0)) {
            noTemplateList.add(transformationInfo.getTransformationKey());
            return;
        }
        startLi();
        startP();
        startCode();
        append(transformationKey.toString());
        endCode();
        endP();
        if (xslCount > 0) {
            startUl();
            for (int i = 0; i < xslCount; i++) {
                printValidLi(transformationInfo.getXslTransformerInfo(i));
            }
            endUl();
        }
        if (extensionCount > 0) {
            startUl();
            for (int i = 0; i < extensionCount; i++) {
                printByExtensionLi(transformationInfo.getExtensionList(i));
            }
            endUl();
        }
        if (errorLength > 0) {
            startUl();
            for (int i = 0; i < unvalidXslArray.length; i++) {
                printUnvalidLi(unvalidXslArray[i]);
            }
            for (int i = 0; i < byExtensionErrorArray.length; i++) {
                printUnvalidByExtensionLi(byExtensionErrorArray[i]);
            }
            endUl();
        }
        endLi();
    }

    private void printValidLi(XslTransformerInfo xslTransformerInfo) {
        currentA = validA;
        startLi();
        startP();
        startCode();
        printTitle(xslTransformerInfo, true);
        endCode();
        endP();
        printWarning(xslTransformerInfo);
        endLi();
    }

    private void printUnvalidLi(XslTransformerInfo xslTransformerInfo) {
        currentA = invalidA;
        boolean withXslt = true;
        short errorType = xslTransformerInfo.getErrorType();
        String errorLocKey = "admin_lib_error_";
        String uri = null;
        switch (errorType) {
            case XslTransformerInfo.PATTERN_ERROR:
                errorLocKey = errorLocKey + "pattern";
                uri = xslTransformerInfo.getPatternURI();
                withXslt = false;
                break;
            case XslTransformerInfo.XMLOFXSL_ERROR:
                errorLocKey = errorLocKey + "xmlofxsl";
                uri = xslTransformerInfo.getXslURI();
                break;
            case XslTransformerInfo.XSL_ERROR:
                errorLocKey = errorLocKey + "xsl";
                uri = xslTransformerInfo.getXslURI();
                break;
            default:
                throw new SwitchException("unknown xslTransformerInfo.getErrorType() = " + errorType);
        }
        startLi();
        startP();
        startSpan(ERRORMESSAGE_SPANCLASS);
        startCode();
        printTitle(xslTransformerInfo, withXslt);
        endCode();
        endSpan();
        endP();


        startP();
        startSmall();
        localize(errorLocKey, uri);
        HtmlPrinterUtils.printColon(this);
        endSmall();
        appendBr();
        startPre();
        escape(xslTransformerInfo.getErrorMessage(), true);
        endPre();
        endP();


        printWarning(xslTransformerInfo);
        endLi();
    }

    private void printByExtensionLi(TransformationInfo.ListByExtension byExtension) {
        startLi();
        startP();
        startCode();
        startStrong();
        append(".");
        append(byExtension.getExtension());
        endStrong();
        endCode();
        endP();
        startUl();
        int count = byExtension.getStreamTransformerCount();
        for (int i = 0; i < count; i++) {
            printValidLi(byExtension.getStreamTransformerInfo(i));
        }
        StreamTransformerInfo[] array = byExtension.getUnvalidStreamTransformerInfoArray();
        for (int i = 0; i < array.length; i++) {
            printUnvalidLi(array[i]);
        }
        endUl();
        endLi();
    }

    private void printUnvalidByExtensionLi(TransformationInfo.ListByExtension byExtension) {
        startLi();
        startP();
        startSpan(ERRORMESSAGE_SPANCLASS);
        startCode();
        startStrong();
        append(".");
        append(byExtension.getExtension());
        endStrong();
        endCode();
        endSpan();
        endP();
        startUl();
        StreamTransformerInfo[] array = byExtension.getUnvalidStreamTransformerInfoArray();
        for (int i = 0; i < array.length; i++) {
            printUnvalidLi(array[i]);
        }
        endUl();
        endLi();
    }

    private void printValidLi(StreamTransformerInfo streamTransformerInfo) {
        startLi();
        startP();
        startCode();
        printTitle(streamTransformerInfo);
        endCode();
        endP();
        printWarning(streamTransformerInfo);
        endLi();
    }

    private void printUnvalidLi(StreamTransformerInfo streamTransformerInfo) {
        startLi();
        startP();
        startSpan(ERRORMESSAGE_SPANCLASS);
        startCode();
        printTitle(streamTransformerInfo);
        endCode();
        endSpan();
        endP();
        short errorType = streamTransformerInfo.getErrorType();
        String errorLocKey = "admin_lib_error_";
        boolean extensionError = false;
        switch (errorType) {
            case StreamTransformerInfo.UNKNOWN_EXTENSION_ERROR:
                errorLocKey = errorLocKey + "unknownextension";
                extensionError = true;
                break;
            case StreamTransformerInfo.STREAM_TRANSFORMER_ERROR:
                errorLocKey = errorLocKey + "streamtransformer";
                break;
            case StreamTransformerInfo.PROCESS_ERROR:
                errorLocKey = errorLocKey + "process";
                break;
            default:
                throw new SwitchException("unknown streamTransformerInfo.getErrorType() = " + errorType);
        }
        startP();

        if (extensionError) {
            startSmall();
            localize(errorLocKey, streamTransformerInfo.getErrorMessage());
            endSmall();
        } else {
            startSmall();
            localize(errorLocKey);
            HtmlPrinterUtils.printColon(this);
            endSmall();
            appendBr();
            startPre();
            escape(streamTransformerInfo.getErrorMessage(), true);
            endPre();
        }
        endP();
        printWarning(streamTransformerInfo);
        endLi();

    }

    private void printTitle(XslTransformerInfo xslTransformerInfo, boolean withXslt) {
        XslTransformerKey xslTransformerKey = xslTransformerInfo.getXslTransformerKey();
        String name = xslTransformerKey.getTransformerName();
        String path = "xslt/" + xslTransformerKey.getTransformationKey().toString() + "/" + name;
        if (withXslt) {
            String xsltName = name + ".xsl";
            currentA.setHref(path + ".xsl");
            startA(currentA);
            append(xsltName);
            endA();
        }
        if (xslTransformerInfo.isFromPattern()) {
            String txtName = name + ".txt";
            currentA.setHref(path + ".txt");
            append(" (");
            startA(currentA);
            append(txtName);
            endA();
            append(")");
        }
    }

    private void printTitle(StreamTransformerInfo streamTransformerInfo) {
        StreamTransformerKey streamTransformerKey = streamTransformerInfo.getStreamTransformerKey();
        append(streamTransformerKey.getTransformerName());
        append(".");
        append(streamTransformerKey.getExtension());
    }

    private void printWarning(StreamTransformerInfo streamTransformerInfo) {
        int warningCount = streamTransformerInfo.getWarningCount();
        if (warningCount > 0) {
            startP();
            startSmall();
            HtmlPrinterUtils.printLibColon(this, "admin_lib_processwarning");
            endSmall();
            appendBr();
            startPre();
            for (int i = 0; i < warningCount; i++) {
                if (i > 0) {
                    append("\n");
                }
                escape(streamTransformerInfo.getWarning(i), true);
            }
            endPre();
            endP();
        }
    }

    private void printWarning(XslTransformerInfo xslTransformerInfo) {
        if (xslTransformerInfo.isFromPattern()) {
            String patternWarning = xslTransformerInfo.getPatternWarning();
            if (patternWarning != null) {
                startP();
                localize("admin_lib_patternwarning", xslTransformerInfo.getPatternURI());
                HtmlPrinterUtils.printColon(this);
                appendBr();
                startPre();
                escape(patternWarning, true);
                endPre();
                endP();
            }
        }
    }

    void printTransformationCreateForm() {
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Administration");
        commandBox.setCommandString(TransformationCreateCommand.getCommandString());
        commandBox.setCommandKey(TransformationCreateCommand.getCommandKey());
        startCommandBox(commandBox);
        startP();
        HtmlPrinterUtils.printLibColon(this, "admin_lib_transformationkey");
        startSelect(TransformationCreateCommand.TRANSFORMATIONKEY_PARAMNAME);
        for (Iterator it = allTransformationList.iterator(); it.hasNext();) {
            TransformationKey transformationKey = (TransformationKey) it.next();
            String val = transformationKey.toString();
            this.startOption(val, false);
            append(val);
            endOption();
        }
        endSelect();
        endP();
        startP();
        HtmlPrinterUtils.printLibColon(this, "admin_lib_transformername");
        appendMediumInput(TransformationCreateCommand.TRANSFORMERNAME_PARAMNAME, "");
        endP();

        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setName(TransformationCreateCommand.FORMAT_PARAMNAME);

        startP();
        HtmlPrinterUtils.printLibColon(this, "admin_lib_format");
        startSelect(selectAttributes);
        printOption(XSL_FORMAT, true);
        printOption(ODT_FORMAT, false);
        endSelect();
        endP();

        HTMLAttributes checkAttributes = new HTMLAttributes();
        checkAttributes.setName(TransformationCreateCommand.WITHIF_PARAMNAME);
        checkAttributes.setChecked(false);
        checkAttributes.setValue(TransformationCreateCommand.YES_PARAMVALUE);
        startP();
        startSmall();
        addCheck(checkAttributes, "admin_lib_withif", null);
        endSmall();
        endP();
        endCommandBox("admin_submit_transformationcreate");
    }

    void printOption(short formatType, boolean selected) {
        String value = XSLTFactoryUtils.formatTypeToString(formatType);
        HTMLAttributes optionAttributes = new HTMLAttributes();
        optionAttributes.setValue(value);
        optionAttributes.setSelected(selected);
        startOption(optionAttributes);
        append(value);
        endOption();
    }

}
