/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import fr.exemole.bdfserver.response.AssociationResponseParameters;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.croisement.*;
import fr.exemole.bdfserver.request.ParameterException;


/**
 *
 * @author Vincent Calame
 */
public class AssociationAddCommand extends AbstractAssociationCommand {

    public static final String BASENAME_PARAMNAME = "basename";

    public AssociationAddCommand() {
    }

    protected AssociationResponseParameters doAssociationCommand() throws ParameterException {
        String param_basename = requestMap.getParameter(BASENAME_PARAMNAME);
        if (param_basename == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, BASENAME_PARAMNAME);
        }
        Document existingdocument = addenda.getDocumentByBasename(param_basename);
        AssociationResponseParameters responseParameters = initResponseParameters();
        responseParameters.setHtmlPageCode(AssociationResponseParameters.ASSOCIATIONFORM_PAGE);
        if (existingdocument == null) {
            responseParameters.setErrorMessage("association_err_notexistingbasename", param_basename);
        } else {
            if (CroisementUtils.areCrossed(abregeFiche, existingdocument)) {
                responseParameters.setErrorMessage("association_err_existingassociation", param_basename);
            } else {
                DefaultDocumentCroisementList documentList = new DefaultDocumentCroisementList(existingdocument);
                bdfServer.getBasedefichesEditor().getCroisementEditor().updateCroisement(abregeFiche, documentList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
                responseParameters.setDoneMessage("association_done_associationadd", param_basename);
            }

        }
        return responseParameters;
    }

    public boolean needDocumentInit() {
        return false;
    }

    public boolean needAbregeFicheInit() {
        return true;
    }

    public static String getCommandString() {
        return "AssociationAdd";
    }

}
