/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import fr.exemole.bdfserver.response.AssociationResponseParameters;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.util.text.UTF8Bridge;
import fr.exemole.bdfserver.users.BdfUser;
import java.text.*;
import fr.exemole.bdfserver.request.ParameterException;


/**
 *
 * @author Vincent Calame
 */
public class AssociationRemoveCommand extends AbstractAssociationCommand {

    public final static String DOCUMENTREMOVE_PARAMNAME = "documentremove";

    public AssociationRemoveCommand() {
    }

    protected AssociationResponseParameters doAssociationCommand() throws ParameterException {
        String basename = document.getBasename();
        boolean documentRemove = testDocumentRemove();
        boolean associationDone = bdfServer.getBasedefichesEditor().getCroisementEditor().removeCroisement(abregeFiche, document);
        AssociationResponseParameters responseParameters = initResponseParameters();
        responseParameters.setHtmlPageCode(AssociationResponseParameters.ASSOCIATIONFORM_PAGE);
        if (associationDone) {
            boolean removeDone = false;
            if (documentRemove) {
                removeDone = addendaEditor.removeDocument(document);
            }
            if (removeDone) {
                responseParameters.setDoneMessage("association_done_documentremove", basename);
            } else {
                responseParameters.setDoneMessage("association_done_associationremove", basename);
            }
        }
        return responseParameters;
    }

    public static String getCommandString() {
        return "AssociationRemove";
    }

    public boolean needDocumentInit() {
        return true;
    }

    public boolean needAbregeFicheInit() {
        return true;
    }

    private boolean testDocumentRemove() {
        String param = requestMap.getParameter(DOCUMENTREMOVE_PARAMNAME);
        if (param == null) {
            return false;
        }
        if (!param.equals("1")) {
            return false;
        }
        if (!CroisementUtils.isUniqueCroisement(abregeFiche, document)) {
            return false;
        }
        return true;
    }

}
