/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.association;

import fr.exemole.bdfserver.response.AssociationResponseParameters;
import javax.servlet.http.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.association.*;
import net.mapeadores.util.text.UTF8Bridge;
import fr.exemole.bdfserver.users.BdfUser;
import java.text.*;
import fr.exemole.bdfserver.request.ParameterException;


/**
 *
 * @author Vincent Calame
 */
public class VersionRemoveCommand extends AbstractAssociationCommand {

    public static final String VERSION_PARAMNAME = "version";

    public VersionRemoveCommand() {
    }

    protected AssociationResponseParameters doAssociationCommand() throws ParameterException {
        String param_version = requestMap.getParameter(VERSION_PARAMNAME);
        if (param_version == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, VERSION_PARAMNAME);
        }
        AssociationResponseParameters responseParameters = initResponseParameters();
        responseParameters.setHtmlPageCode(AssociationResponseParameters.DOCUMENTFORM_PAGE);
        if (param_version.length() > 0) {
            Version version = document.getVersionByExtension(param_version);
            if (version == null) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, VERSION_PARAMNAME + " = " + param_version);
            }
            boolean done = addendaEditor.removeVersion(version);
            if (done) {
                responseParameters.setDoneMessage("association_done_versionremove", param_version);
            }
        }
        return responseParameters;
    }

    public static String getCommandString() {
        return "VersionRemove";
    }

    public boolean needDocumentInit() {
        return true;
    }

    public boolean needAbregeFicheInit() {
        return false;
    }

}
