/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.response.ExportationResponseParameters;
import fr.exemole.bdfserver.balayages.*;
import fr.exemole.bdfserver.request.ParameterException;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.storage.*;
import net.mapeadores.util.exceptions.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.runners.BalayageRunner;
import fr.exemole.bdfserver.runners.ScrutariExportRunner;
import fr.exemole.bdfserver.runners.SqlExportRunner;
import java.io.*;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.util.logging.CommandMessage;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommand extends AbstractExportationCommand {

    public final static String MODE_PARAMNAME = "mode";
    public final static String SCRUTARIEXPORTINDEX_PAGE_PARAMVALUE = "scrutariexportindex";
    public final static String SCRUTARIEXPORTCREATION_PAGE_PARAMVALUE = "scrutariexportcreation";
    public final static String SCRUTARIEXPORTREMOVE_PAGE_PARAMVALUE = "scrutariexportremove";
    public final static String SQLEXPORTINDEX_PAGE_PARAMVALUE = "sqlexportindex";
    public final static String SQLEXPORTCREATION_PAGE_PARAMVALUE = "sqlexportcreation";
    public final static String SQLEXPORTREMOVE_PAGE_PARAMVALUE = "sqlexportremove";
    public final static String BALAYAGEINDEX_PAGE_PARAMVALUE = "balayageindex";
    public final static String THESAURUSFORM_PAGE_PARAMVALUE = "thesaurusform";
    public final static String RUN_PARAMETER = "run";
    public final static String SCRUTARI_RUN_PARAMVALUE = "scrutariexport";
    public final static String SQLEXPORT_RUN_PARAMVALUE = "sqlexport";
    public final static String BALAYAGE_RUN_PARAMVALUE = "balayage";
    private short pageType;
    private short runType = 0;
    private final static short BALAYAGE_TYPE = 1;
    private final static short SQLEXPORT_TYPE = 2;
    private final static short SCRUTARIEXPORT_TYPE = 3;

    public DefaultCommand() {
    }

    protected void doExportationCommand() {
        switch (runType) {
            case BALAYAGE_TYPE:
                runBalayage();
                break;
            case SCRUTARIEXPORT_TYPE:
                runScrutariExport();
                break;
            case SQLEXPORT_TYPE:
                runSqlExport();
                break;
            default:
                responseParameters.setHtmlPageCode(pageType);
        }
    }

    protected void checkExportationParameters() throws ParameterException {
        String runString = requestMap.getParameter(RUN_PARAMETER);
        boolean needScrutariExport = false;
        boolean needSqlExport = false;
        boolean needBalayage = false;
        if (runString != null) {
            if (runString.equals(SCRUTARI_RUN_PARAMVALUE)) {
                needScrutariExport = true;
                runType = SCRUTARIEXPORT_TYPE;
            } else if (runString.equals(SQLEXPORT_RUN_PARAMVALUE)) {
                needSqlExport = true;
                runType = SQLEXPORT_TYPE;
            } else if (runString.equals(BALAYAGE_RUN_PARAMVALUE)) {
                needBalayage = true;
                runType = BALAYAGE_TYPE;
            } else {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, RUN_PARAMETER + " = " + runString);
            }
        } else {
            pageType = getPageType();
            switch (pageType) {
                case ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE:
                case ExportationResponseParameters.SCRUTARIEXPORTREMOVE_PAGE:
                    needScrutariExport = true;
                    break;
                case ExportationResponseParameters.SQLEXPORTFORM_PAGE:
                case ExportationResponseParameters.SQLEXPORTREMOVE_PAGE:
                    needSqlExport = true;
                    break;
                default:
            }
            if (needThesaurus(pageType)) {
                responseParameters.setThesaurus(RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches()));
            }
        }
        if (needScrutariExport) {
            responseParameters.setScrutariExportDef(RequestUtils.getScrutariExportDef(requestMap, bdfServer.getScrutariExportManager()));
        }
        if (needSqlExport) {
            responseParameters.setSqlExportDef(RequestUtils.getSqlExportDef(requestMap, bdfServer.getSqlExportManager()));
        }
        if (needBalayage) {
            BalayageManager balayageManager = bdfServer.getBalayageManager();
            Balayage balayage = RequestUtils.getBalayage(requestMap, balayageManager);
            responseParameters.setBalayage(balayage);
            String mode = requestMap.getParameter(MODE_PARAMNAME);
            if (mode != null) {
                if (mode.length() == 0) {
                    mode = null;
                } else if (!BalayageUtils.containsMode(balayage, mode)) {
                    throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, MODE_PARAMNAME + " = " + mode);
                } else {
                    responseParameters.setMode(mode);
                }
            }
        }
    }

    public static String getCommandString() {
        return null;
    }

    private short getPageType() throws ParameterException {
        if (!requestMap.isPageDefined()) {
            if (requestMap.getParameter(RequestUtils.SCRUTARIEXPORT_PARAMNAME) != null) {
                return ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE;
            } else if (requestMap.getParameter(RequestUtils.SQLEXPORT_PARAMNAME) != null) {
                return ExportationResponseParameters.SQLEXPORTFORM_PAGE;
            }
            return ExportationResponseParameters.STREAMFORM_PAGE;
        }
        String page_param = requestMap.getPageParameter();
        if (page_param.equals(SCRUTARIEXPORTINDEX_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.SCRUTARIEXPORTINDEX_PAGE;
        } else if (page_param.equals(SCRUTARIEXPORTCREATION_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.SCRUTARIEXPORTCREATION_PAGE;
        } else if (page_param.equals(SCRUTARIEXPORTREMOVE_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.SCRUTARIEXPORTREMOVE_PAGE;
        } else if (page_param.equals(SQLEXPORTINDEX_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.SQLEXPORTINDEX_PAGE;
        } else if (page_param.equals(SQLEXPORTCREATION_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.SQLEXPORTCREATION_PAGE;
        } else if (page_param.equals(SQLEXPORTREMOVE_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.SQLEXPORTREMOVE_PAGE;
        } else if (page_param.equals(BALAYAGEINDEX_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.BALAYAGEINDEX_PAGE;
        } else if (page_param.equals(THESAURUSFORM_PAGE_PARAMVALUE)) {
            return ExportationResponseParameters.THESAURUSFORM_PAGE;
        }
        throw new ParameterException(ParameterException.PAGE_WRONGVALUE, page_param);
    }

    private boolean needThesaurus(short pageType) {
        switch (pageType) {
            case ExportationResponseParameters.THESAURUSFORM_PAGE:
                return true;
            default:
                return false;
        }
    }

    private void runSqlExport() {
        SqlExportDef sqlExportDef = responseParameters.getSqlExportDef();
        CommandMessage message = null;
        try {
            message = SqlExportRunner.run(bdfServer, sqlExportDef);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        if (message.isErrorMessage()) {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLEXPORTFORM_PAGE);
        } else {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLRUN_PAGE);
        }
        responseParameters.setCommandMessage(message);

    }

    private void runScrutariExport() {
        try {
            ScrutariExportRunner.run(responseParameters.getScrutariExportDef(), bdfServer);
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
        responseParameters.setHtmlPageCode(ExportationResponseParameters.SCRUTARIEXPORTRUN_PAGE);
        responseParameters.setDoneMessage("exportation_done_scrutariexportrun", null);

    }

    private void runBalayage() {
        String mode = responseParameters.getMode();
        Balayage balayage = responseParameters.getBalayage();
        BalayageRunner.Result result = BalayageRunner.run(balayage, mode, bdfServer);
        if (result.hasInitError()) {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.BALAYAGEERROR_PAGE);
            responseParameters.setCommandMessage(result.getInitError());
        } else {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.BALAYAGE_PAGE);
            responseParameters.putParameter("balayageLog",result.getBalayageLog());
        }
    }

}
