/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2011 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.response.ExportationResponseParameters;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDefBuilder;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.request.parsers.*;
import java.text.ParseException;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefParser;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.format.patterns.MotcleFormat;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.WrongFormatSourceKeyException;
import net.mapeadores.basedefiches.tableexport.ColDefErrorHandler;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefListParser;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportChangeCommand extends AbstractExportationCommand {

    public final static String BASEICON_PARAMNAME = "baseicon";
    public final static String FICHEHREFPATTERN_PARAMNAME = "fichehrefpattern";
    public final static String MOTCLEHREFPATTERN_PARAMNAME = "motclehrefpattern";
    public final static String BASENAMEACTIVATION_PARAMNAME = "basenameactivation";
    public final static String BASENAMEVALUE_PARAMNAME = "basenamevalue";
    public final static String LANGSUI_PARAMNAME = "langsui";
    public final static String COMPLEMENTKEYLIST_PARAMSUFFIX = "|complementkeylist";
    public final static String DATEKEY_PARAMSUFFIX = "|datekey";
    public final static String SELECTED_PARAMSUFFIX = "|selected";
    public final static String LIBELLEPATTERN_PARAMSUFFIX = "|libellepattern";
    public final static String TITREFORMATDEF_PARAMSUFFIX = "|titreformatdef";
    public final static String SOUSTITREFORMATDEF_PARAMSUFFIX = "|soustitreformatdef";
    public final static String MULTILANG_PARAMSUFFIX = "|multilang";
    public final static String MULTILANGKEY_PARAMSUFFIX = "|multilangkey";
    public final static String HREFPATTERN_PARAMSUFFIX = "|hrefpattern";
    private String name;
    private String baseIconString;
    private int[] array;
    private List<CorpusScrutariDef> corpusScrutariDefList;
    private List<ThesaurusScrutariDef> thesaurusScrutariDefList;
    private AccoladePattern ficheGlobalPattern;
    private AccoladePattern motcleGlobalPattern;
    private String baseName;
    private BasedefichesFormatDefErrorHandler errorHandler = new BasedefichesFormatDefErrorHandler();

    public ScrutariExportChangeCommand() {
    }

    protected void doExportationCommand() {
        ScrutariExportDef scrutariExportDef = toScrutariExportDef();
        bdfServer.getScrutariExportManager().putScrutariExportDef(scrutariExportDef);
        responseParameters.setScrutariExportDef(scrutariExportDef);
        responseParameters.setHtmlPageCode(ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE);
        responseParameters.setDoneMessage("exportation_done_scrutariexportchange", null);
    }

    protected void checkExportationParameters() throws ParameterException {
        ScrutariExportDef currentScrutariExportDef = RequestUtils.getScrutariExportDef(requestMap, bdfServer.getScrutariExportManager());
        name = currentScrutariExportDef.getName();
        baseIconString = getMandatoryParameter(BASEICON_PARAMNAME);
        String langsUi = getMandatoryParameter(LANGSUI_PARAMNAME);
        array = LangIntegerUtils.toCleanLangIntegerArray(langsUi);
        ficheGlobalPattern = getPattern(FICHEHREFPATTERN_PARAMNAME);
        motcleGlobalPattern = getPattern(MOTCLEHREFPATTERN_PARAMNAME);
        String activationParam = getMandatoryParameter(BASENAMEACTIVATION_PARAMNAME);
        if (activationParam.equals("1")) {
            baseName = getMandatoryParameter(BASENAMEVALUE_PARAMNAME);
        }
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        corpusScrutariDefList = new ArrayList<CorpusScrutariDef>();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            CorpusScrutariDef corpusScrutariDef = getCorpusScrutariDef(corpus);
            if (corpusScrutariDef != null) {
                corpusScrutariDefList.add(corpusScrutariDef);
            }
        }
        thesaurusScrutariDefList = new ArrayList<ThesaurusScrutariDef>();
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            ThesaurusScrutariDef thesaurusExportDef = getThesaurusExportDef(thesaurus);
            if (thesaurusExportDef != null) {
                thesaurusScrutariDefList.add(thesaurusExportDef);
            }
        }
    }

    public static String getCommandString() {
        return "ScrutariExportChange";
    }

    private ScrutariExportDef toScrutariExportDef() {
        ScrutariExportDefBuilder scrutariExport = null;
        try {
            scrutariExport = ScrutariExportDefBuilder.newInstance(name);
        } catch (ParseException pe) {
            throw new ShouldNotOccurException("Test done before");
        }
        scrutariExport.setBaseIcon(baseIconString);
        scrutariExport.setFicheHrefPattern(ficheGlobalPattern);
        scrutariExport.setMotcleHrefPattern(motcleGlobalPattern);
        try {
            scrutariExport.setBaseName(baseName);
        } catch (java.text.ParseException pe) {
        }
        scrutariExport.setLangUiArray(array);

        for (CorpusScrutariDef corpusScrutariDef : corpusScrutariDefList) {
            scrutariExport.addCorpusScrutariDef(corpusScrutariDef);
        }
        for (ThesaurusScrutariDef thesaurusExportDef : thesaurusScrutariDefList) {
            scrutariExport.addThesaurusScrutariDef(thesaurusExportDef);
        }
        scrutariExport.setFicheSelectOptions(FicheSelectParser.parse(requestMap, bdfServer, bdfUser));
        return scrutariExport.toScrutariExportDef();
    }

    private AccoladePattern getPattern(String paramName) throws ParameterException {
        String patternParam = requestMap.getParameter(paramName);
        if (patternParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, paramName);
        }
        patternParam = patternParam.trim();
        if (patternParam.length() == 0) {
            return null;
        }
        try {
            AccoladePattern pattern = new AccoladePattern(patternParam);
            return pattern;
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    private CorpusScrutariDef getCorpusScrutariDef(Corpus corpus) throws ParameterException {
        SubsetKey corpusKey = corpus.getSubsetKey();
        String corpusString = corpusKey.toString();
        String dateKeyParam = requestMap.getParameter(corpusString + DATEKEY_PARAMSUFFIX);
        String fieldListParam = requestMap.getParameter(corpusString + COMPLEMENTKEYLIST_PARAMSUFFIX);
        if ((fieldListParam == null) && (dateKeyParam == null)) {
            return null;
        }
        if (dateKeyParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, corpusString + DATEKEY_PARAMSUFFIX);
        }
        if (fieldListParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, corpusString + COMPLEMENTKEYLIST_PARAMSUFFIX);
        }
        CorpusScrutariDefBuilder corpusScrutariDefBuilder = new CorpusScrutariDefBuilder(corpusKey);
        fieldListParam = fieldListParam.trim();
        if (fieldListParam.length() > 0) {
            ColDefList colDefList = ColDefListParser.parse(fieldListParam, null, new InternalColDefErrorHandler());
            corpusScrutariDefBuilder.addComplements(colDefList);
        }
        FormatSourceKey dateFormatSourceKey = null;
        try {
            dateFormatSourceKey = CorpusScrutariDefBuilder.parseDateKey(dateKeyParam);
        } catch (ParseException pe) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, corpusString + DATEKEY_PARAMSUFFIX + " = " + dateKeyParam);
        } catch (WrongFormatSourceKeyException wfske) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, corpusString + DATEKEY_PARAMSUFFIX + " = " + dateKeyParam);
        }
        if (dateFormatSourceKey.isFieldKey()) {
            FieldKey dateFieldKey = dateFormatSourceKey.getFieldKey();
            List<CorpusField> corpusFieldList = CorpusMetadataUtils.getCorpusFieldListByFicheItemType(corpus.getCorpusMetadata(), CorpusField.DATATION_FIELD);
            boolean present = false;
            for (CorpusField corpusField : corpusFieldList) {
                if (corpusField.getFieldKey().equals(dateFieldKey)) {
                    present = true;
                    break;
                }
            }
            if (!present) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, corpusString + DATEKEY_PARAMSUFFIX + " = " + dateKeyParam);
            }
        }
        corpusScrutariDefBuilder.setDateKey(dateFormatSourceKey);
        String multiLangString = requestMap.getParameter(corpusString + MULTILANG_PARAMSUFFIX);
        if ((multiLangString != null) && (multiLangString.equals("1"))) {
            String multiLangKeyString = requestMap.getParameter(corpusString + MULTILANGKEY_PARAMSUFFIX);
            if (multiLangKeyString != null) {
                try {
                    FieldKey multiLangKey = FieldKey.parse(multiLangKeyString);
                    corpusScrutariDefBuilder.setMultiLangKey(multiLangKey);
                } catch (ParseException pe) {
                } catch (IllegalArgumentException iae) {
                }
            }
        }
        AccoladePattern hrefPattern = getPattern(corpusString + HREFPATTERN_PARAMSUFFIX);
        if (hrefPattern != null) {
            corpusScrutariDefBuilder.setHrefPattern(hrefPattern);
        }
        BasedefichesFormatDef titreFormatDef = parseFormatDef(requestMap.getParameter(corpusString + TITREFORMATDEF_PARAMSUFFIX), errorHandler);
        if (titreFormatDef != null) {
            corpusScrutariDefBuilder.setTitreBasedefichesFormatDef(titreFormatDef);
        }
        BasedefichesFormatDef soustitreFormatDef = parseFormatDef(requestMap.getParameter(corpusString + SOUSTITREFORMATDEF_PARAMSUFFIX), errorHandler);
        if (soustitreFormatDef != null) {
            corpusScrutariDefBuilder.setSoustitreBasedefichesFormatDef(soustitreFormatDef);
        }
        return corpusScrutariDefBuilder.toCorpusScrutariDef();
    }

    private ThesaurusScrutariDef getThesaurusExportDef(Thesaurus thesaurus) {
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        ThesaurusScrutariDefBuilder thesaurusScrutariDefBuilder = new ThesaurusScrutariDefBuilder(thesaurusKey);
        String selectedParam = requestMap.getParameter(thesaurusKey.toString() + SELECTED_PARAMSUFFIX);
        boolean selected = false;
        if ((selectedParam != null) && (selectedParam.equals("1"))) {
            selected = true;
        }
        thesaurusScrutariDefBuilder.setSelected(selected);
        String patternParam = requestMap.getParameter(thesaurusKey.toString() + LIBELLEPATTERN_PARAMSUFFIX);
        if (patternParam != null) {
            patternParam = patternParam.trim();
            if (patternParam.length() > 0) {
                MotcleFormat motcleFormat = new MotcleFormat();
                try {
                    motcleFormat.setPattern(patternParam);
                    thesaurusScrutariDefBuilder.setMotcleFormatPattern(patternParam);
                } catch (java.text.ParseException pe) {
                }
            }

        }
        return thesaurusScrutariDefBuilder.toThesaurusScrutariDef();
    }

    private BasedefichesFormatDef parseFormatDef(String s, BasedefichesFormatDefParser.ErrorHandler errorHandler) {
        if (s == null) {
            return null;
        }
        return BasedefichesFormatDefParser.parse(s, errorHandler);
    }


    private static class InternalColDefErrorHandler implements ColDefErrorHandler {

        public void wrongParameterLine(int lineNumber, String line, String key) {
            //throw new IllegalStateException("wrongParameterLine " + lineNumber + " / " + line + " / " + key);
        }

        public void tooManyLineWarning(int lineNumber, String line) {
            //throw new IllegalStateException("tooManyLineWarning " + lineNumber + " / " + line);
        }

        public void missingFormatSourceLineError(int lineNumber) {
            //throw new IllegalStateException("missingFormatSourceLineError " + lineNumber);
        }

        public void formatSourceError(int lineNumber, String key, Object value) {
            //throw new IllegalStateException("formatSourceError " + lineNumber + " / " + key + " / " + value);
        }

        public void formatPatternError(int lineNumber, String key, Object value) {
            //throw new IllegalStateException("formatPatternError " + lineNumber + " / " + key + " / " + value);
        }

        public void formatInstructionError(int lineNumber, String key, Object value) {
            //throw new IllegalStateException("formatInstructionError " + lineNumber + " / " + key + " / " + value);
        }

    }


    private class BasedefichesFormatDefErrorHandler implements BasedefichesFormatDefParser.ErrorHandler {

        public void nullFormatSourceError() {
            handle("nullFormatSourceError");
        }

        public void wrongFormatSourceError() {
            handle("wrongFormatSourceError");
        }

        public void wrongFormatSourcePartError(String formatSourcePart) {
            handle("wrongFormatSourcePartError" + " : " + formatSourcePart);
        }

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart) {
            handle("invalidAsciiCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void invalidEndCharacterInstructionError(String formatInstructionPart) {
            handle("invalidEndCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart) {
            handle("invalidSeparatorCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void unknownArgumentKeyInstructionWarning(String argumentKey) {
            handle("unknownArgumentKeyInstructionWarning" + " : " + argumentKey);
        }

        public void missingArgumentValueInstructionError(String argumentKey) {
            handle("missingArgumentValueInstructionError" + " : " + argumentKey);
        }

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue) {
            handle("wrongArgumentValueInstructionError" + " : " + argumentKey + " / " + argumentValue);
        }

        private void handle(String s) {
            //
        }

    }

}
