/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.response.ExportationResponseParameters;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.scrutariexport.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportCreationCommand extends AbstractExportationCommand {

    public final static String NEWSCRUTARIEXPORT_PARAMNAME = "newscrutariexport";
    private String nameParam;

    public ScrutariExportCreationCommand() {
    }

    protected void doExportationCommand() {
        ScrutariExportManager scrutariExportManager = bdfServer.getScrutariExportManager();
        try {
            ScrutariExportDefBuilder scrutariExportDefBuilder =  ScrutariExportDefBuilder.newInstance(nameParam);
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SCRUTARIEXPORTFORM_PAGE);
            responseParameters.setDoneMessage("exportation_done_scrutariexportcreation", null);
            ScrutariExportDef scrutariExportDef = scrutariExportDefBuilder.toScrutariExportDef();
            responseParameters.setScrutariExportDef(scrutariExportDef);
            scrutariExportManager.putScrutariExportDef(scrutariExportDef);
        } catch (java.text.ParseException pe) {
            throw new ShouldNotOccurException("test has be done");
        }
    }

    protected void checkExportationParameters() throws ParameterException {
        nameParam = getMandatoryParameter(NEWSCRUTARIEXPORT_PARAMNAME);
        ScrutariExportDef scrutariExport = bdfServer.getScrutariExportManager().getScrutariExportDef(nameParam);
        if (scrutariExport != null) {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SCRUTARIEXPORTCREATION_PAGE);
            responseParameters.setErrorMessage("exportation_err_existingscrutariexportname", nameParam);
            return;
        }
        if (!StringUtils.isTechnicalName(nameParam, true)) {
            responseParameters.setHtmlPageCode(ExportationResponseParameters.SCRUTARIEXPORTCREATION_PAGE);
            responseParameters.setErrorMessage("exportation_err_wrongexportname", nameParam);
            return;
        }
    }

    public static String getCommandString() {
        return "ScrutariExportCreation";
    }

}
