/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.managers.SqlExportManager;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import java.text.ParseException;
import net.mapeadores.util.text.*;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.basedefiches.sqlexport.SqlExportDefBuilder;
import net.mapeadores.util.exceptions.ImplementationException;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportChangeCommand extends AbstractExportationCommand {

    public final static String CUSTOMCLASSNAME_PARAMNAME = "customclassname";
    public final static String PATH_PARAMNAME = "path";
    public final static String PARAMS_PARAMNAME = "params";
    public final static String URL_PARAMNAME = "url";
    public final static String TABLEEXPORTNAME_PARAMNAME = "tableexportname";
    public final static String POSTCOMMAND_PARAMNAME = "postcommand";
    private SqlExportDef sqlExportDef;

    public SqlExportChangeCommand() {
    }

    protected void doExportationCommand() {
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        synchronized (bdfServer) {
            sqlExportManager.putSqlExportDef(sqlExportDef);
        }
        responseParameters.setSqlExportDef(sqlExportDef);
        responseParameters.setHtmlPageCode(ExportationResponseParameters.SQLEXPORTFORM_PAGE);
        responseParameters.setDoneMessage("exportation_done_sqlexportchange", sqlExportDef.getName());
    }

    protected void checkExportationParameters() throws ParameterException {
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        SqlExportDef currentSqlExportDef = RequestUtils.getSqlExportDef(requestMap, sqlExportManager);
        SqlExportDefBuilder sqlExportDefBuilder = null;
        try {
            sqlExportDefBuilder = SqlExportDefBuilder.newInstance(currentSqlExportDef.getName());
        } catch (ParseException pe) {
            throw new ImplementationException("name from SqlExportDef instance : " + currentSqlExportDef.getName());
        }
        String customClassName = getMandatoryParameter(CUSTOMCLASSNAME_PARAMNAME).trim();
        sqlExportDefBuilder.setSqlExportClassName(customClassName);
        String path = getMandatoryParameter(PATH_PARAMNAME).trim();
        sqlExportDefBuilder.setPath(path);
        String url = getMandatoryParameter(URL_PARAMNAME).trim();
        sqlExportDefBuilder.setUrl(url);
        String postCommand = getMandatoryParameter(POSTCOMMAND_PARAMNAME).trim();
        sqlExportDefBuilder.setPostCommand(postCommand);
        String tableExportName = getMandatoryParameter(TABLEEXPORTNAME_PARAMNAME).trim();
        if (tableExportName.length() == 0) {
            sqlExportDefBuilder.setTableExportName(null);
        } else {
            sqlExportDefBuilder.setTableExportName(tableExportName);
        }
        String params = getMandatoryParameter(PARAMS_PARAMNAME);
        String[] tokens = StringUtils.getLineTokens(params, StringUtils.EMPTY_EXCLUDE);
        for (int i = 0; i < tokens.length; i++) {
            String token = tokens[i];
            int idx = token.indexOf('=');
            if (idx == -1) {
                continue;
            }
            String name = token.substring(0, idx).trim();
            if (name.length() > 0) {
                String value = token.substring(idx + 1).trim();
                sqlExportDefBuilder.addParam(name, value);
            }
        }
        this.sqlExportDef = sqlExportDefBuilder.toSqlExportDef();
    }

    public static String getCommandString() {
        return "SqlExportChange";
    }

}
