/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation;

import fr.exemole.bdfserver.response.ExportationResponseParameters;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.BdfServer;
import javax.servlet.http.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.tableexport.thesaurus.ThesaurusTableExport;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.request.ParameterException;
import net.mapeadores.basedefiches.tableexport.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.text.*;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusTableStreamCommand extends AbstractExportationCommand {

    public final static String FIELDS_PARAMNAME = "fields";
    public final static String ROOTS_PARAMNAME = "roots";
    public final static String INCLUDEROOTS_PARAMNAME = "includeroots";
    private ThesaurusTableExport thesaurusTableExport;

    public ThesaurusTableStreamCommand() {
    }

    protected void doExportationCommand() {
        responseParameters.setThesaurusTableExport(thesaurusTableExport);
        responseParameters.setContentStreamCode(ExportationResponseParameters.THESAURUSTABLE_SXC_CONTENTSTREAM);
    }

    protected void checkExportationParameters() throws ParameterException {
        thesaurusTableExport = initThesaurusTableExport();
    }

    private ThesaurusTableExport initThesaurusTableExport() throws ParameterException {
        Thesaurus thesaurus = RequestUtils.getThesaurus(requestMap, bdfServer.getBasedefiches());
        String fields = requestMap.getParameter(FIELDS_PARAMNAME);
        if (fields == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, FIELDS_PARAMNAME);
        }
        ThesaurusTableExport thesaurusTableExport = null;
        String[] fieldsArray = StringUtils.getTechnicalTokens(fields);
        if (fieldsArray.length == 0) {
            thesaurusTableExport = ThesaurusTableExport.getDefault(thesaurus);
        } else {
            thesaurusTableExport = new ThesaurusTableExport(thesaurus);
            for (int i = 0; i < fieldsArray.length; i++) {
                try {
                    ThesaurusFieldKey thesaurusFieldKey = ThesaurusFieldKey.parse(fieldsArray[i]);
                    thesaurusTableExport.addThesaurusFieldKey(thesaurusFieldKey);
                } catch (ParseException pe) {
                }
            }
        }
        boolean includeRoots = false;
        String includeRootsString = requestMap.getParameter(INCLUDEROOTS_PARAMNAME);
        if (includeRootsString != null) {
            if (includeRootsString.equals("1")) {
                includeRoots = true;
            }
        }

        String roots = requestMap.getParameter(ROOTS_PARAMNAME);
        if (roots != null) {
            String[] array = StringUtils.getTechnicalTokens(roots);
            boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
            for (int i = 0; i < array.length; i++) {
                String id = array[i];
                Motcle motcle = null;
                if (withIdalpha) {
                    motcle = thesaurus.getMotcleByIdalpha(id);
                } else {
                    try {
                        int idths = Integer.parseInt(id);
                        motcle = thesaurus.getMotcleByIdths(idths);
                    } catch (NumberFormatException nfe) {
                    }
                }
                if (motcle != null) {
                    thesaurusTableExport.addRoot(motcle, includeRoots);
                }
            }
        }


        return thesaurusTableExport;
    }

    public static String getCommandString() {
        return "ThesaurusTableStream";
    }

}
