/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.htmlpages;

import fr.exemole.bdfserver.html.catalogs.SelectFormCatalog;
import fr.exemole.bdfserver.response.ExportationResponseParameters;
import fr.exemole.bdfserver.servlets.util.FicheSelectWriter;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.exportation.*;
import fr.exemole.bdfserver.ui.corpus.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportUtils;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDef;
import net.mapeadores.basedefiches.tableexport.ColDefUtils;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportFormPage extends ExportationHtmlPage {

    private Map<SubsetKey, CorpusScrutariDef> corpusDefMap;
    private Map<SubsetKey, ThesaurusScrutariDef> thesaurusDefMap;

    public ScrutariExportFormPage(ExportationResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(SelectFormCatalog.SELECTFORM_JAVASCRIPT_LIB);
        addJavascript(SelectFormCatalog.SELECTFORM_FICHE);
        corpusDefMap = ScrutariExportUtils.toCorpusScrutariDefMap(scrutariExportDef);
        thesaurusDefMap = ScrutariExportUtils.toThesaurusScrutariDefMap(scrutariExportDef);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        initResponse(response);
        start();
        printScrutariTitre();

        startForm(scrutariExportDef, ScrutariExportChangeCommand.getCommandString());
        printFixedSubmit("exportation_submit_scrutariexportchange");
        appendH2("exportation_title_generaloptions");
        printBaseName();
        printBaseIcon();
        printPattern(ScrutariExportChangeCommand.FICHEHREFPATTERN_PARAMNAME, scrutariExportDef.getFicheHrefPattern(), "exportation_lib_fichehrefpattern");
        printPattern(ScrutariExportChangeCommand.MOTCLEHREFPATTERN_PARAMNAME, scrutariExportDef.getMotcleHrefPattern(), "exportation_lib_motclehrefpattern");
        String langsUi = getArray(scrutariExportDef);
        startP();
        printLib("exportation_lib_langsui");
        appendSmallInput(ScrutariExportChangeCommand.LANGSUI_PARAMNAME, langsUi);
        endP();
        appendH2("exportation_title_corpusoptions");
        startTable("100%");
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            printCorpus(basedefiches.getCorpus(i));
        }
        endTable();
        appendH2("exportation_title_thesaurusoptions");
        startTable("100%");
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            printThesaurus(basedefiches.getThesaurus(i));
        }
        endTable();
        appendH2("exportation_title_selectionoptions");

        FicheSelectWriter selectionFormWriter = new FicheSelectWriter(bdfServer, this, workingLangInteger);
        selectionFormWriter.setFicheSelectOptions(scrutariExportDef.getFicheSelectOptions());
        selectionFormWriter.printFieldSets();

        endForm("exportation_submit_scrutariexportchange");
        end();
    }

    void printBaseName() {
        String baseName = scrutariExportDef.getBaseName();
        startP();
        appendRadioInput(ScrutariExportChangeCommand.BASENAMEACTIVATION_PARAMNAME, "0", (baseName == null), null);
        localize("exportation_lib_basename_default", bdfServer.getBasedefiches().getBasedefichesMetadata().getBasedefichesName());
        endP();
        startP();
        appendRadioInput(ScrutariExportChangeCommand.BASENAMEACTIVATION_PARAMNAME, "1", (baseName != null), "exportation_lib_basename_activation");
        space();
        appendSmallInput(ScrutariExportChangeCommand.BASENAMEVALUE_PARAMNAME, (baseName != null) ? baseName : "");
        endP();
    }

    void printBaseIcon() {
        String baseIcon = scrutariExportDef.getBaseIcon();
        startP();
        printLibColon("exportation_lib_baseicon");
        appendLargeInput(ScrutariExportChangeCommand.BASEICON_PARAMNAME, baseIcon);
        endP();
    }

    void printPattern(String paramName, AccoladePattern pattern, String libKey) {
        String value = (pattern != null) ? pattern.toString() : "";
        startP();
        printLib(libKey);
        appendLargeInput(paramName, value);
        endP();
    }

    void printThesaurus(Thesaurus thesaurus) {
        SubsetKey thesaurusSubsetKey = thesaurus.getSubsetKey();
        ThesaurusScrutariDef thesaurusExportDef = thesaurusDefMap.get(thesaurusSubsetKey);
        if (thesaurusExportDef == null) {
            thesaurusExportDef = ScrutariExportUtils.getDefaultScrutariThesaurusDef(thesaurusSubsetKey);
        }
        String motcleFormatPattern = thesaurusExportDef.getMotcleFormatPattern();
        if (motcleFormatPattern == null) {
            motcleFormatPattern = "";
        }
        startRow();
        startCell(3, 0);
        startH3();
        appendCheckInput(thesaurusSubsetKey.toString() + ScrutariExportChangeCommand.SELECTED_PARAMSUFFIX, "1", thesaurusExportDef.isSelected(), null);
        space();
        append(ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        endH3();
        endCell();
        endRow();
        startRow();
        startCell(3, 0);
        startP();
        startSmall();
        appendMediumInput(thesaurusSubsetKey.toString() + ScrutariExportChangeCommand.LIBELLEPATTERN_PARAMSUFFIX, motcleFormatPattern);
        endSmall();
        endP();
        endCell();
        endRow();
    }

    void printCorpus(Corpus corpus) {
        SubsetKey corpusSubsetKey = corpus.getSubsetKey();
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        CorpusScrutariDef corpusScrutariDef = corpusDefMap.get(corpusSubsetKey);
        if (corpusScrutariDef == null) {
            corpusScrutariDef = ScrutariExportUtils.getDefaultScrutariCorpusDef(corpusSubsetKey);
        }
        String corpusString = corpus.getSubsetKey().toString();
        startRow();
        startCell(3, 0);
        startH3();
        append(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        endH3();
        endCell();
        endRow();
        startRow();
        startCell();
        startP();
        startSmall();
        printLib("exportation_lib_hrefpattern");
        endSmall();
        endP();
        endCell();
        startCell(2, 0);
        AccoladePattern hrefPattern = corpusScrutariDef.getHrefPattern();
        String value = (hrefPattern != null) ? hrefPattern.toString() : "";
        appendLargeInput(corpusString + ScrutariExportChangeCommand.HREFPATTERN_PARAMSUFFIX, value);
        endCell();
        endRow();
        startRow();
        printFormatRow(corpusScrutariDef.getTitreFormatDef(), corpusString + ScrutariExportChangeCommand.TITREFORMATDEF_PARAMSUFFIX, "exportation_lib_titreformatdef");
        printFormatRow(corpusScrutariDef.getSoustitreFormatDef(), corpusString + ScrutariExportChangeCommand.SOUSTITREFORMATDEF_PARAMSUFFIX, "exportation_lib_soustitreformatdef");
        startRow();
        startCellWithTopAlign();
        startP();
        startSmall();
        localize("exportation_lib_complementfieldlist");
        endSmall();
        endP();
        endCell();
        startCell();
        startTextArea(corpusString + ScrutariExportChangeCommand.COMPLEMENTKEYLIST_PARAMSUFFIX, 45, 8);
        StringBuilder buf = new StringBuilder();
        try {
            ColDefUtils.writeColDefList(buf, corpusScrutariDef.getComplementColDefList());
        } catch (IOException ioe) {
        }
        escape(buf.toString(), true);
        endTextArea();
        endCell();
        startCellWithTopAlign();
        startP();
        startCode();
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            append(corpusField.getFieldKey().toString());
            doublespace();
        }
        append(FieldKey.REDACTEURS.toString());
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            doublespace();
            append(corpusField.getFieldKey().toString());
        }
        List<IncludeUI> corpusIncludeList = CorpusUIUtils.getCorpusIncludeUIList(corpusUI);
        for (IncludeUI includeUI : corpusIncludeList) {
            doublespace();
            append(includeUI.getIncludeKey().toString());
        }
        endCode();
        endP();
        endCell();
        endRow();
        startRow();
        startCell();
        startP();
        startSmall();
        printLib("exportation_lib_datekey");
        endSmall();
        endP();
        endCell();
        startCell(2, 0);
        startP();
        FormatSourceKey dateFormatSourceKey = corpusScrutariDef.getDateFormatSourceKey();
        startSelect(corpusString + ScrutariExportChangeCommand.DATEKEY_PARAMSUFFIX);
        printOption(FormatSourceKey.DATE_CREATION, dateFormatSourceKey);
        printOption(FormatSourceKey.DATE_MODIFICATION, dateFormatSourceKey);
        FieldKey dateFieldKey = null;
        if (dateFormatSourceKey.isFieldKey()) {
            dateFieldKey = dateFormatSourceKey.getFieldKey();
        }
        List<CorpusField> fieldList = CorpusMetadataUtils.getCorpusFieldListByFicheItemType(corpusMetadata, CorpusField.DATATION_FIELD);
        for (CorpusField corpusField : fieldList) {
            FieldKey fieldKey = corpusField.getFieldKey();
            boolean selected = false;
            if ((dateFieldKey != null) && (fieldKey.equals(dateFieldKey))) {
                selected = true;
            }
            printOption(corpusField.getFieldKey(), selected);
        }
        endSelect();
        endP();
        endCell();
        startRow();
        startCell(3, 0);
        startP();
        startSmall();
        HTMLAttributes checkAttributes = new HTMLAttributes();
        checkAttributes.setName(corpusString + ScrutariExportChangeCommand.MULTILANG_PARAMSUFFIX);
        checkAttributes.setValue("1");
        checkAttributes.setChecked(corpusScrutariDef.isMultiLang());
        addCheck(checkAttributes, "exportation_lib_multilang", null);
        endSmall();
        endP();
        endCell();
        endRow();
        startRow();
        startCell();
        startP();
        startSmall();
        printLib("exportation_lib_multilangkey");
        endSmall();
        endP();
        endCell();
        startCell(2, 0);
        String multiLangKeyValue = "";
        Object obj = corpusScrutariDef.getMultiLangKey();
        if (obj != null) {
            multiLangKeyValue = obj.toString();
        }
        appendMediumInput(corpusString + ScrutariExportChangeCommand.MULTILANGKEY_PARAMSUFFIX, multiLangKeyValue);
        endCell();
        endRow();
    }

    private void printFormatRow(BasedefichesFormatDef basedefichesFormatDef, String name, String libKey) {
        String value = "";
        if (basedefichesFormatDef != null) {
            value = BasedefichesFormatUtils.toLines(basedefichesFormatDef);
        }
        startCellWithTopAlign();
        startP();
        startSmall();
        localize(libKey);
        endSmall();
        endP();
        endCell();
        startCell(2, 0);
        startTextArea(name, 45, 3);
        escape(value, true);
        endTextArea();
        endCell();
        endRow();
    }

    private void printOption(Object key, Object current) {
        startOption(key.toString(), (current.equals(key)));
        append(key.toString());
        endOption();
    }

    private void printOption(Object key, boolean selected) {
        String keyString = key.toString();
        startOption(keyString, selected);
        append(keyString);
        endOption();
    }

    private String getArray(ScrutariExportDef scrutariExportDef) {
        int count = scrutariExportDef.getLangUICount();
        if (count == 0) {
            return "";
        }
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(';');
            }
            LangInteger.appendString(buf, scrutariExportDef.getLangUI(i));
        }
        return buf.toString();
    }

    public String getCssThemeFile() {
        return "exportation_scrutari.css";
    }

}
