/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2011 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.htmlpages;

import fr.exemole.bdfserver.managers.SqlExportManager;
import fr.exemole.bdfserver.sqlexport.*;
import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.exportation.*;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportFormPage extends ExportationHtmlPage {

    SqlExportManager sqlExportManager;

    public SqlExportFormPage(ExportationResponseParameters responseParameters) {
        super(responseParameters);
        this.sqlExportManager = bdfServer.getSqlExportManager();
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printSqlTitre();
        startForm(sqlExportDef, SqlExportChangeCommand.getCommandString());
        printTableExportSelect(sqlExportDef.getTableExportName());
        printClassName(sqlExportDef.getSqlExportClassName());
        printParams(sqlExportDef);
        printPath(sqlExportDef.getPath());
        printUrl(sqlExportDef.getUrl());
        printPostCommand(sqlExportDef.getPostCommand());
        endForm("exportation_submit_sqlexportchange");
        end();
    }

    private void printClassName(String className) {
        startP();
        printLibColon("exportation_lib_sqlexport_customclassname");
        appendLargeInput(SqlExportChangeCommand.CUSTOMCLASSNAME_PARAMNAME, className);
        endP();
    }

    private void printPath(String path) {
        startP();
        printLibColon("exportation_lib_sqlexport_path");
        appendLargeInput(SqlExportChangeCommand.PATH_PARAMNAME, path);
        endP();
    }

    private void printUrl(String url) {
        if (url == null) {
            url = "";
        }
        startP();
        printLibColon("exportation_lib_sqlexport_url");
        appendLargeInput(SqlExportChangeCommand.URL_PARAMNAME, url);
        endP();
    }

    private void printPostCommand(String postCommand) {
        if (postCommand == null) {
            postCommand = "";
        }
        startP();
        printLibColon("exportation_lib_sqlexport_postcommand");
        appendMediumInput(SqlExportChangeCommand.POSTCOMMAND_PARAMNAME, postCommand);
        endP();
    }

    private void printTableExportSelect(String tableExportName) {
        if (tableExportName == null) {
            tableExportName = "";
        }
        startP();
        printLibColon("exportation_lib_sqlexport_tableexport");
        startSelect(SqlExportChangeCommand.TABLEEXPORTNAME_PARAMNAME);
        startOption("", (tableExportName.equals("")));
        append("---");
        endOption();
        for (String current : bdfServer.getTableExportManager().getTableExportNameSet()) {
            boolean selected = false;
            if (current.equals(tableExportName)) {
                selected = true;
            }
            startOption(current, selected);
            append(current);
            endOption();
        }
        endSelect();
        endP();
    }

    private void printParams(SqlExportDef sqlExportDef) {
        String text = "";
        int paramCount = sqlExportDef.getParamCount();
        if (paramCount > 0) {
            StringBuilder buf = new StringBuilder();
            for (int i = 0; i < paramCount; i++) {
                SqlExportDef.Param param = sqlExportDef.getParam(i);
                buf.append(param.getName());
                buf.append("= ");
                buf.append(param.getValue());
                buf.append("\n");
            }
            text = buf.toString();
        }
        startP();
        printLibColon("exportation_lib_sqlexport_params");
        appendBr();
        startTextArea(SqlExportChangeCommand.PARAMS_PARAMNAME, 60, 7);
        escape(text, true);
        endTextArea();
        endP();
    }

}
