/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.exportation.htmlpages;

import fr.exemole.bdfserver.response.*;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.exportation.*;
import fr.exemole.bdfserver.beans.*;
import fr.exemole.bdfserver.managers.tableexport.*;
import java.util.*;
import fr.exemole.bdfserver.servlets.util.*;


/**
 *
 * @author Vincent Calame
 */
public class StreamFormPage extends ExportationHtmlPage {

    TableExportManager tableExportManager;
    TableExportParametersBean tableExportParametersBean;
    TableExportParameters.Printer printer;

    public StreamFormPage(ExportationResponseParameters responseParameters) {
        super(responseParameters);
        this.tableExportManager = bdfServer.getTableExportManager();
        this.tableExportParametersBean = bdfUser.getTableExportParametersBean();
        if (tableExportParametersBean == null) {
            tableExportParametersBean = new TableExportParametersBean();
        }
        printer = new TableExportParameters.Printer(tableExportParametersBean, this);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        appendH1("exportation_title_stream");
        Set<String> nameSet = tableExportManager.getTableExportNameSet();
        startPostForm(TableStreamCommand.getCommandString());
        if (nameSet.size() > 0) {
            String name = tableExportParametersBean.getTableExportName();
            boolean defaultExport = (name == null);
            printSelectRadio(defaultExport, name, nameSet);
            printDefaultRadio(defaultExport);
        }
        printTableExportParametersBean();
        printHeaderType();
        endForm("exportation_submit_stream");
        end();
    }

    public String getFormAction() {
        return "_exportation.ods";
    }

    void printSelectRadio(boolean defaultExport, String name, Set<String> nameSet) {
        startH2();
        appendRadioInput(TableStreamCommand.DEFAULTEXPORT_PARAMNAME, "0", !defaultExport, "exportation_lib_defaultexport_no");
        startSelect(TableExportParameters.TABLEEXPORTNAME_PARAMNAME);
        for (String current : nameSet) {
            boolean selected = false;
            if ((name != null) && (current.equals(name))) {
                selected = true;
            }
            startOption(current, selected);
            append(current);
            endOption();
        }
        endSelect();
        endH2();
    }

    void printDefaultRadio(boolean defaultExport) {
        startH2();
        appendRadioInput(TableStreamCommand.DEFAULTEXPORT_PARAMNAME, "1", defaultExport, "exportation_lib_defaultexport_yes");
        endH2();
    }

    void printTableExportParametersBean() {
        startP();
        printer.printBooleanParameters();
        appendBr();
        printer.printIndexationRadios();
        endP();
    }

    void printHeaderType() {
        appendH2("exportation_title_headertype");
        startP();
        printer.printHeaderTypeRadios();
        endP();
    }

}
