/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFicheRemoveCommand extends AbstractMassCommand {

    public final static String IDCORPUSLIST_PARAMNAME = "idcorpuslist";

    public MassFicheRemoveCommand() {
    }

    public boolean needCorpus() {
        return true;
    }

    public static String getCommandString() {
        return "MassFicheRemove";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSFICHEREMOVEFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSFICHEREMOVECONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(corpus);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        String[] list = requestMap.getParameterValues(IDCORPUSLIST_PARAMNAME);
        int count = 0;
        if (list != null) {
            BasedefichesEditor basedefichesEditor = bdfServer.getBasedefichesEditor();
            CorpusEditor corpusEditor = basedefichesEditor.getCorpusEditor(corpus.getSubsetKey());
            for (int i = 0; i < list.length; i++) {
                try {
                    int idcorpus = Integer.parseInt(list[i]);
                    AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                    if (abregeFiche != null) {
                        boolean done2 = corpusEditor.removeFiche(abregeFiche);
                        if (done2) {
                            count++;
                        }
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_ficheremove", new Integer(count));
        }
    }


    private static class Parser {

        ParseResult parseResult = new ParseResult(ParseResult.TOKENS);
        Corpus corpus;

        Parser(Corpus corpus) {
            this.corpus = corpus;
        }

        ParseResult getParseResult() {
            return parseResult;
        }

        private void parse(String s) {
            String[] tokens = StringUtils.getTechnicalTokens(s);
            for (int i = 0; i < tokens.length; i++) {
                int tokenNumber = i + 1;
                String token = tokens[i];
                try {
                    int idcorpus = Integer.parseInt(token);
                    AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                    if (abregeFiche == null) {
                        parseResult.addBdfError("MISSING", token, tokenNumber);
                    } else {
                        parseResult.addResult(abregeFiche.getIdcorpus(), abregeFiche, null, tokenNumber);
                    }
                } catch (NumberFormatException nfe) {
                    parseResult.addParseError(token, tokenNumber);
                }
            }

        }

    }

}
