/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.parsers.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFieldCommand extends AbstractMassCommand {

    public final static String[] paramArray = {COUPLESEPARATEUR_PARAMNAME, FUSIONDOUBLON_PARAMNAME, FIELDKEY_PARAMNAME};
    public final static String[] executeParamArray = {FIELDKEY_PARAMNAME};
    public final static String IDCORPUSARRAY_PARAMNAME = "idcorpusarray";

    public MassFieldCommand() {
    }

    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public boolean needCorpus() {
        return true;
    }

    public static String getCommandString() {
        return "MassField";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSFIELDFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSFIELDCONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        FieldKey fieldKey = corpusField.getFieldKey();
        boolean corpsdefiche = fieldKey.ownsToCorpsdefiche();
        if (fieldKey.isUniqueValueField()) {
            fusionDoublon = false;
        }
        Parser parser = new Parser(corpus, coupleSeparateur, fusionDoublon, corpsdefiche);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        String[] idcorpusArray = requestMap.getParameterValues(IDCORPUSARRAY_PARAMNAME);
        if (idcorpusArray == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, IDCORPUSARRAY_PARAMNAME);
        }
        CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
        int count = 0;
        SaisieEngine saisieEngine = new SaisieEngine(bdfServer, bdfUser, requestMap, SaisieEngine.PARTIALFICHE_BITVALUE);
        for (int i = 0; i < idcorpusArray.length; i++) {
            try {
                int idcorpus = Integer.parseInt(idcorpusArray[i]);
                AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche != null) {
                    saisieEngine.updateFiche(abregeFiche, idcorpusArray[i]);
                    count++;
                }
            } catch (NumberFormatException nfe) {
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        responseParameters.setDoneMessage("mass_done_corpusfield", new Integer(count));
    }


    public static class Parser extends CoupleCorpusParser {

        boolean corpsdefiche;
        boolean fusionDoublon;

        Parser(Corpus corpus, char coupleSeparateur, boolean fusionDoublon, boolean corpsdefiche) {
            super(corpus, coupleSeparateur);
            this.corpsdefiche = corpsdefiche;
            this.fusionDoublon = fusionDoublon;
        }

        protected void parseValue(AbregeFiche abregeFiche, String value, int lineNumber) {
            if (!corpsdefiche) {
                value = StringUtils.cleanString(value);
            }
            int id = abregeFiche.getIdcorpus();
            if (parseResult.containsId(id)) {
                if (fusionDoublon) {
                    ParseResult.ResultItem resultItem = parseResult.getResultItem(id);
                    String currentVal = (String) resultItem.getValueObject();
                    if (corpsdefiche) {
                        value = currentVal + "\n\n" + value;
                    } else {
                        value = currentVal + " ; " + value;
                    }
                } else {
                    parseResult.checkDoublon(id, lineNumber);
                    return;
                }
            }
            parseResult.addResult(id, abregeFiche, value, lineNumber);
        }

    }

}
