/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.request.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.croisement.*;
import fr.exemole.bdfserver.request.parsers.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class MassLiageCommand extends AbstractMassCommand {

    public final static String[] paramArray = {COUPLESEPARATEUR_PARAMNAME, CROISEMENTUPDATE_PARAMNAME, OTHERCORPUS_PARAMNAME, FUSIONDOUBLON_PARAMNAME};
    public final static String[] executeParamArray = {CROISEMENTUPDATE_PARAMNAME, OTHERCORPUS_PARAMNAME};

    public MassLiageCommand() {
    }

    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public boolean needCorpus() {
        return true;
    }

    public static String getCommandString() {
        return "MassLiage";
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSLIAGEFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSLIAGECONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(corpus, coupleSeparateur, fusionDoublon, otherCorpus);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        CroisementEditor croisementsEditor = bdfServer.getBasedefichesEditor().getCroisementEditor();
        int count = 0;
        for (Iterator it = requestMap.getParameterNameSet().iterator(); it.hasNext();) {
            String name = (String) it.next();
            try {
                int idcorpus = Integer.parseInt(name);
                AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche != null) {
                    DefaultFicheCroisementList list;
                    if (requestMap.getParameter(name).length() == 0) {
                        list = new DefaultFicheCroisementList(otherCorpus.getSubsetKey(), 0);
                    } else {
                        String[] values = requestMap.getParameterValues(name);
                        list = new DefaultFicheCroisementList(otherCorpus.getSubsetKey(), values.length);
                        for (int i = 0; i < values.length; i++) {
                            PoidsIntegerToken poidsIntegerToken = PoidsIntegerToken.parse(values[i]);
                            if (poidsIntegerToken != null) {
                                AbregeFiche abr2 = otherCorpus.getAbregeFicheByIdcorpus(poidsIntegerToken.getInteger());
                                if (abr2 != null) {
                                    list.add(abr2, poidsIntegerToken.getPoids());
                                }
                            }
                        }
                    }
                    count++;
                    croisementsEditor.updateCroisement(abregeFiche, list, updateType);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_liage", new Integer(count));
        }
    }


    public static class Parser extends CoupleCorpusParser {

        Corpus liageCorpus;
        boolean fusionDoublon;

        Parser(Corpus corpus, char coupleSeparateur, boolean fusionDoublon, Corpus liageCorpus) {
            super(corpus, coupleSeparateur);
            this.liageCorpus = liageCorpus;
            this.fusionDoublon = fusionDoublon;
        }

        protected void parseValue(AbregeFiche abregeFiche, String value, int lineNumber) {
            String[] idtokens = StringUtils.getTechnicalTokens(value, false);
            int length = idtokens.length;
            DefaultFicheCroisementList abregeFicheList = new DefaultFicheCroisementList(liageCorpus.getSubsetKey(), length);
            boolean error = false;
            for (int i = 0; i < length; i++) {
                PoidsIntegerToken poidsIntegerToken = PoidsIntegerToken.parse(idtokens[i]);
                if (poidsIntegerToken == null) {
                    parseResult.addParseError(idtokens[i], lineNumber);
                    error = true;
                } else {
                    AbregeFiche abrg2 = liageCorpus.getAbregeFicheByIdcorpus(poidsIntegerToken.getInteger());
                    if (abrg2 == null) {
                        parseResult.addBdfError("UNKNOWN LIAGE IDCORPUS", String.valueOf(poidsIntegerToken.getInteger()), lineNumber);
                        error = true;
                    } else if (abrg2.equals(abregeFiche)) {
                        parseResult.addBdfError("IDCORPUS MATCHING", String.valueOf(abregeFiche.getIdcorpus()), lineNumber);
                        error = true;
                    } else {
                        abregeFicheList.add(abrg2, poidsIntegerToken.getPoids());
                    }
                }
            }
            if (!error) {
                int id = abregeFiche.getIdcorpus();
                if (parseResult.containsId(id)) {
                    if (fusionDoublon) {
                        ParseResult.ResultItem resultItem = parseResult.getResultItem(id);
                        DefaultFicheCroisementList currentAbregeFicheList = (DefaultFicheCroisementList) resultItem.getValueObject();
                        currentAbregeFicheList.append(abregeFicheList);
                    } else {
                        parseResult.checkDoublon(id, lineNumber);
                    }
                } else {
                    parseResult.addResult(id, abregeFiche, abregeFicheList, lineNumber);
                }
            }
        }

    }

}
