/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.util.text.*;


/**
 *
 * @author Vincent Calame
 */
public class MassMotcleRemoveCommand extends AbstractMassCommand {

    public final static String[] paramArray = {TYPEID_PARAMNAME};
    public final static String[] executeParamArray = new String[0];
    public final static String IDTHSLIST_PARAMNAME = "idthslist";

    public MassMotcleRemoveCommand() {
    }

    public boolean needCorpus() {
        return false;
    }

    public boolean needThesaurus() {
        return true;
    }

    public static String getCommandString() {
        return "MassMotcleRemove";
    }

    public String[] getParseStepParamArray() {
        return paramArray;
    }

    public static String[] getSuppParamArray() {
        return paramArray;
    }

    public String[] getExecuteStepParamArray() {
        return executeParamArray;
    }

    public short getFormPage() {
        return ImportationResponseParameters.MASSMOTCLEREMOVEFORM_PAGE;
    }

    public short getConfirmPage() {
        return ImportationResponseParameters.MASSMOTCLEREMOVECONFIRM_PAGE;
    }

    protected ParseResult doParseStep(String text) {
        Parser parser = new Parser(thesaurus, typeId);
        parser.parse(text);
        return parser.getParseResult();
    }

    protected void doExecuteStep() throws ParameterException {
        String[] list = requestMap.getParameterValues(IDTHSLIST_PARAMNAME);
        int count = 0;
        if (list != null) {
            BasedefichesEditor basedefichesEditor = bdfServer.getBasedefichesEditor();
            ThesaurusEditor thesaurusEditor = basedefichesEditor.getThesaurusEditor(thesaurus.getSubsetKey());
            for (int i = 0; i < list.length; i++) {
                try {
                    int idths = Integer.parseInt(list[i]);
                    Motcle motcle = thesaurus.getMotcleByIdths(idths);
                    if (motcle != null) {
                        ThesaurusUtils.prepareToRemove(basedefichesEditor, motcle);
                        boolean done2 = thesaurusEditor.removeMotcle(motcle);
                        if (done2) {
                            count++;
                        }
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }
        responseParameters.setHtmlPageCode(ImportationResponseParameters.MASSINDEX_PAGE);
        if (count > 0) {
            responseParameters.setDoneMessage("mass_done_motcleremove", new Integer(count));
        }
    }


    private static class Parser {

        ParseResult parseResult = new ParseResult(ParseResult.TOKENS);
        Thesaurus thesaurus;
        String parseType;

        Parser(Thesaurus thesaurus, String parseType) {
            this.thesaurus = thesaurus;
            this.parseType = parseType;
        }

        ParseResult getParseResult() {
            return parseResult;
        }

        private void parse(String s) {
            if ((parseType == null) || (parseType.equals(IDTHS_TYPEID_PARAMVALUE))) {
                String[] tokens = StringUtils.getTechnicalTokens(s);
                for (int i = 0; i < tokens.length; i++) {
                    parseIdths(tokens[i], i + 1);
                }
            } else {
                String[] tokens = StringUtils.getTechnicalTokens(s, false);
                for (int i = 0; i < tokens.length; i++) {
                    parseIdalpha(tokens[i], i + 1);
                }
            }
        }

        private void parseIdalpha(String idalpha, int tokenNumber) {
            if (!ThesaurusUtils.isValidIdalpha(idalpha)) {
                parseResult.addParseError(idalpha, tokenNumber);
            } else {
                Motcle motcle = thesaurus.getMotcleByIdalpha(idalpha);
                if (motcle == null) {
                    parseResult.addBdfError("MISSING", idalpha, tokenNumber);
                } else {
                    parseResult.addResult(motcle.getIdths(), motcle, null, tokenNumber);
                }
            }
        }

        private void parseIdths(String idthsString, int tokenNumber) {
            try {
                int idths = Integer.parseInt(idthsString);
                Motcle motcle = thesaurus.getMotcleByIdths(idths);
                if (motcle == null) {
                    parseResult.addBdfError("MISSING", idthsString, tokenNumber);
                } else {
                    parseResult.addResult(motcle.getIdths(), motcle, null, tokenNumber);
                }
            } catch (NumberFormatException nfe) {
                parseResult.addParseError(idthsString, tokenNumber);
            }
        }

    }

}
