/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import java.util.*;
import fr.exemole.bdfserver.ui.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.tableparser.*;
import net.mapeadores.util.text.coupleparser.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.managers.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class ImportationHtmlPage extends BdfHtmlPage {

    protected ParseResult parseResult;
    ImportationResponseParameters responseParameters;
    Corpus corpus;
    Thesaurus thesaurus;
    Basedefiches basedefiches;
    LangConfig langConfig;
    Integer langInteger;
    Corpus otherCorpus;
    Thesaurus otherThesaurus;

    public ImportationHtmlPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        parseResult = (ParseResult) responseParameters.getParameter(ImportationResponseParameters.PARSERESULT_KEY);
        this.responseParameters = responseParameters;
        this.corpus = responseParameters.getCorpus();
        this.thesaurus = responseParameters.getThesaurus();
        this.basedefiches = bdfServer.getBasedefiches();
        this.langConfig = bdfServer.getConfigurationManager().getLangConfig();
        this.langInteger = (Integer) responseParameters.getParameter(AbstractMassCommand.LANG_PARAMNAME);
        this.otherCorpus = (Corpus) responseParameters.getParameter(AbstractMassCommand.OTHERCORPUS_PARAMNAME);
        this.otherThesaurus = (Thesaurus) responseParameters.getParameter(AbstractMassCommand.OTHERTHESAURUS_PARAMNAME);
    }

    public static ResponseHandler getInstance(ImportationResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case ImportationResponseParameters.MESSAGE_SERVLETPAGE:
                return new MessagePage(responseParameters);
            case ImportationResponseParameters.MASSINDEX_PAGE:
                return new MassIndexPage(responseParameters);
            case ImportationResponseParameters.MASSFIELDCONFIRM_PAGE:
                return new MassFieldConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSFIELDFORM_PAGE:
                return new MassFieldFormPage(responseParameters);
            case ImportationResponseParameters.MASSLIBCONFIRM_PAGE:
                return new MassLibConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSLIBFORM_PAGE:
                return new MassLibFormPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLEREMOVECONFIRM_PAGE:
                return new MassMotcleRemoveConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLEREMOVEFORM_PAGE:
                return new MassMotcleRemoveFormPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLEMERGECONFIRM_PAGE:
                return new MassMotcleMergeConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLEMERGEFORM_PAGE:
                return new MassMotcleMergeFormPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLEMOVECONFIRM_PAGE:
                return new MassMotcleMoveConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLEMOVEFORM_PAGE:
                return new MassMotcleMoveFormPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLECREATIONCONFIRM_PAGE:
                return new MassMotcleCreationConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSMOTCLECREATIONFORM_PAGE:
                return new MassMotcleCreationFormPage(responseParameters);
            case ImportationResponseParameters.MASSLIAGECONFIRM_PAGE:
                return new MassLiageConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSLIAGEFORM_PAGE:
                return new MassLiageFormPage(responseParameters);
            case ImportationResponseParameters.MASSINDEXATIONFICHECONFIRM_PAGE:
                return new MassIndexationFicheConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSINDEXATIONFICHEFORM_PAGE:
                return new MassIndexationFicheFormPage(responseParameters);
            case ImportationResponseParameters.MASSFICHECREATIONFORM_PAGE:
                return new MassFicheCreationFormPage(responseParameters);
            case ImportationResponseParameters.MASSFICHECREATIONCONFIRM_PAGE:
                return new MassFicheCreationConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSUIFORM_PAGE:
                return new MassUIFormPage(responseParameters);
            case ImportationResponseParameters.MASSUICONFIRM_PAGE:
                return new MassUIConfirmPage(responseParameters);
            case ImportationResponseParameters.MASSFICHEREMOVEFORM_PAGE:
                return new MassFicheRemoveFormPage(responseParameters);
            case ImportationResponseParameters.MASSFICHEREMOVECONFIRM_PAGE:
                return new MassFicheRemoveConfirmPage(responseParameters);
        }
        return null;
    }

    protected void printTitre(String key) {
        startP(CssClass.P_TOOLBARLINKS);
        appendA("Importation", "saisie_title_massindex");
        endP();
        appendH1(key);
    }

    protected void printTextArea() {
        endTable();
        startP();
        printLib("saisie_lib_textparse");
        endP();
        startTextArea(AbstractMassCommand.TEXT_PARAMNAME, 75, 30);
        String text = (String) responseParameters.getParameter(AbstractMassCommand.TEXT_PARAMNAME);
        if (text != null) {
            escape(text, true);
        }
        endTextArea();
    }

    protected void startParseForm(String command) {
        startPostForm(command);
        addStepParameter(AbstractMassCommand.PARSE_STEP_PARAMVALUE);
        startTable();
    }

    protected void startParseCorpusForm(String command) {
        startPostForm(corpus, command);
        addStepParameter(AbstractMassCommand.PARSE_STEP_PARAMVALUE);
        startTable();
        printSelectedCorpusRow();
    }

    protected void startParseThesaurusForm(String command) {
        startPostForm(thesaurus, command);
        addStepParameter(AbstractMassCommand.PARSE_STEP_PARAMVALUE);
        startTable();
        printSelectedThesaurusRow();
    }

    protected void printSelectedThesaurusRow() {
        startRow();
        printLibCells("saisie_lib_thesaurusselection", false);
        startCell();
        startSpan(CssClass.SPAN_HIGHLIGHT);
        escape(ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        endSpan();
        endCell();
        endRow();
    }

    protected void printLangRow() {
        if (langInteger != null) {
            startRow();
            printLibCells("saisie_lib_lang", false);
            startCell();
            startCode();
            append(LangInteger.toString(langInteger.intValue()));
            endCode();
            endCell();
            endRow();
        }
    }

    protected void printSelectedCorpusRow() {
        startRow();
        printLibCells("saisie_lib_corpusselection", false);
        startCell();
        startSpan(CssClass.SPAN_HIGHLIGHT);
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, this.workingLangInteger));
        endSpan();
        endCell();
        endRow();
    }

    protected void printFields() {
        String value = (String) responseParameters.getParameter(AbstractMassCommand.FIELDS_PARAMNAME);
        if (value == null) {
            value = "";
        }
        startRow();
        printLibCells("saisie_lib_fields", true);
        startCell();
        startTextArea(AbstractMassCommand.FIELDS_PARAMNAME, 65, 3);
        escape(value);
        endTextArea();
        endCell();
        endRow();

        startRow();
        printLibCells("saisie_lib_potentialfields", true);
        startCell();
        startCode();
        printPotentialFields();
        endCode();
        endCell();
        endRow();
    }

    protected void printPotentialFields() {
    }

    protected void printReturn() {
        if (corpus != null) {
            startPostForm(corpus, null);
        } else if (thesaurus != null) {
            startPostForm(thesaurus, null);
        } else {
            startPostForm(null);
        }
        addPageParameter(getReturnPageValue());
        addSupplementaryParameters();
        String fieldsParam = (String) responseParameters.getParameter(AbstractMassCommand.FIELDS_PARAMNAME);
        if (fieldsParam != null) {
            addParameter(AbstractMassCommand.FIELDS_PARAMNAME, fieldsParam);
        }
        TableParser.Parameters tbparams = (TableParser.Parameters) responseParameters.getParameter(AbstractMassCommand.PARSETYPE_PARAMNAME);
        if (tbparams != null) {
            printHiddenTableParameters(tbparams);
        }
        String text = (String) responseParameters.getParameter(AbstractMassCommand.TEXT_PARAMNAME);
        if (text != null) {
            appendHiddenInput(AbstractMassCommand.TEXT_PARAMNAME, text, true);
        } else {
            addParameter(AbstractMassCommand.TEXT_PARAMNAME, "");
        }
        String typeid = (String) responseParameters.getParameter(AbstractMassCommand.TYPEID_PARAMNAME);
        if (typeid != null) {
            addParameter(AbstractMassCommand.TYPEID_PARAMNAME, typeid);
        }
        Character coupleSeparateur = (Character) responseParameters.getParameter(AbstractMassCommand.COUPLESEPARATEUR_PARAMNAME);
        if (coupleSeparateur != null) {
            addParameter(AbstractMassCommand.COUPLESEPARATEUR_PARAMNAME, StringUtils.charToString(coupleSeparateur.charValue()));
        }
        Short croisementUpdate = (Short) responseParameters.getParameter(AbstractMassCommand.CROISEMENTUPDATE_PARAMNAME);
        if (croisementUpdate != null) {
            addParameter(AbstractMassCommand.CROISEMENTUPDATE_PARAMNAME, AbstractMassCommand.croisementUpdateToString(croisementUpdate.shortValue()));
        }
        Boolean bool = (Boolean) responseParameters.getParameter(AbstractMassCommand.FUSIONDOUBLON_PARAMNAME);
        if (bool != null) {
            String val = (bool.booleanValue()) ? "1" : "0";
            addParameter(AbstractMassCommand.FUSIONDOUBLON_PARAMNAME, val);
        }
        addLangParameter();
        addOtherCorpusParameter();
        addOtherThesaurusParameter();
        endForm("saisie_submit_back");
    }

    private void printHiddenTableParameters(TableParser.Parameters tbparams) {
        String typeValue = null;
        if (tbparams instanceof OOCopyParameters) {
            typeValue = AbstractMassCommand.OOCOPY_PARAMVALUE;
        } else if (tbparams instanceof CsvParameters) {
            typeValue = AbstractMassCommand.CSV_PARAMVALUE;
            CsvParameters csvParams = (CsvParameters) tbparams;
            appendHiddenInput(AbstractMassCommand.CSV_DELIMITER_PARAMNAME, StringUtils.charToString(csvParams.getDelimiter()));
            appendHiddenInput(AbstractMassCommand.CSV_QUOTE_PARAMNAME, StringUtils.charToString(csvParams.getQuote()));
            appendHiddenInput(AbstractMassCommand.CSV_ESCAPED_PARAMNAME, csvParams.isEscapedCSV() ? "1" : "0");
        } else if (tbparams instanceof MysqlParameters) {
            typeValue = AbstractMassCommand.MYSQL_PARAMVALUE;
            MysqlParameters mysqlParams = (MysqlParameters) tbparams;
            appendHiddenInput(AbstractMassCommand.MYSQL_FIELDSTERMINATEDBY_PARAMNAME, StringUtils.charToString(mysqlParams.getFieldsTerminatedBy()));
            appendHiddenInput(AbstractMassCommand.MYSQL_LINESTERMINATEDBY_PARAMNAME, StringUtils.charToString(mysqlParams.getLinesTerminatedBy()));
        }
        appendHiddenInput(AbstractMassCommand.PARSETYPE_PARAMNAME, typeValue);
    }

    protected void addSupplementaryParameters() {
    }

    protected String getReturnPageValue() {
        return null;
    }

    protected void endParseForm() {
        endForm("saisie_submit_textparse");
    }

    protected void printMessage() {
        super.printMessage();
        if (parseResult != null) {
            printParseError(parseResult);
        }
    }

    private void printParseError(ParseResult parseResult) {
        String type = parseResult.getType();
        boolean line = (type.equals(ParseResult.LINES));
        printParseErrorList(parseResult.getParseErrorList(), "saisie_lib_parseerrorlist_" + type, line);
        if (corpus != null) {
            printBdfErrorList(parseResult.getBdfErrorList(), "saisie_lib_bdferrorlist_" + type, line);
        } else {
            printBdfErrorList(parseResult.getBdfErrorList(), "saisie_lib_bdferrorlist_" + type, line);
        }
    }

    private void printParseErrorList(List list, String key, boolean line) {
        if (list.size() == 0) {
            return;
        }
        appendP(null, key);
        startPre();
        for (Iterator it = list.iterator(); it.hasNext();) {
            ParseResult.ParseError parseError = (ParseResult.ParseError) it.next();
            if (line) {
                append(String.valueOf(parseError.getLineNumber()));
                append(": ");
            }
            escape(parseError.getRawText());
            if (line) {
                append("\n");
            } else {
                append("; ");
            }
        }
        endPre();
    }

    private void printBdfErrorList(List list, String key, boolean line) {
        if (list.size() == 0) {
            return;
        }
        appendP(null, key);
        startPre();
        for (Iterator it = list.iterator(); it.hasNext();) {
            ParseResult.BdfError bdfError = (ParseResult.BdfError) it.next();
            if (line) {
                append(String.valueOf(bdfError.getLineNumber()));
                append(": ");
            }
            append(bdfError.getKey());
            append(" : ");
            escape(bdfError.getText());
            if (line) {
                append("\n");
            } else {
                append("; ");
            }
        }
        endPre();
    }

    protected void printResultItemListTitle(String key, int size) {
        startH2();
        localize(key);
        space();
        append("[");
        append(String.valueOf(size));
        append("]");
        endH2();
    }

    protected void printTypeIdRadiosRow() {
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            startRow();
            printLibCells("saisie_lib_typeid", false);
            startCell();
            String selectedType = (String) responseParameters.getParameter(AbstractMassCommand.TYPEID_PARAMNAME);
            if (selectedType == null) {
                selectedType = AbstractMassCommand.IDTHS_TYPEID_PARAMVALUE;
            }
            printTypeIdRadio(AbstractMassCommand.IDTHS_TYPEID_PARAMVALUE, selectedType);
            space();
            printTypeIdRadio(AbstractMassCommand.IDALPHA_TYPEID_PARAMVALUE, selectedType);
            endCell();
            endRow();
        }
    }

    private void printTypeIdRadio(String type, String selectedValue) {
        boolean selected = (selectedValue.equals(type));
        this.appendRadioInput(MassLibCommand.TYPEID_PARAMNAME, type, selected, "saisie_lib_typeid_" + type);
    }

    protected void printThesaurusDestinationSelectRow() {
        startRow();
        printLibCells("saisie_lib_thesaurusdestination", false);
        startCell();
        SubsetGroupList thesaurusGroupList = bdfServer.getUIManager().getThesaurusSubsetKeyTree();
        Thesaurus ths = (Thesaurus) responseParameters.getParameter(AbstractMassCommand.OTHERTHESAURUS_PARAMNAME);
        SubsetKey thesaurusKey = (ths != null) ? ths.getSubsetKey() : thesaurus.getSubsetKey();
        BdfHtmlUtils.printSubsetSelect(this, thesaurusGroupList, AbstractMassCommand.OTHERTHESAURUS_PARAMNAME, thesaurusKey);
        endCell();
        endRow();
    }

    protected void printThesaurusDestinationIntituleRow(Thesaurus thesaurusDestination) {
        startRow();
        printLibCells("saisie_lib_thesaurusdestination", false);
        startCell();
        escape(ThesaurusUtils.getThesaurusIntitule(thesaurusDestination, workingLangInteger));
        endCell();
        endRow();
    }

    protected void printFusionDoublonCheckBoxRow() {
        boolean yes = false;
        Boolean bool = (Boolean) responseParameters.getParameter(AbstractMassCommand.FUSIONDOUBLON_PARAMNAME);
        if (bool != null) {
            yes = bool.booleanValue();
        }
        HTMLAttributes checkAttributes = new HTMLAttributes();
        checkAttributes.setName(AbstractMassCommand.FUSIONDOUBLON_PARAMNAME);
        HTMLAttributes labelAttributes = new HTMLAttributes();
        startRow();
        appendEmptyCell();
        appendEmptyCell();
        startCell();
        checkAttributes.setChecked(yes);
        checkAttributes.setValue("1");
        addCheck(checkAttributes, "saisie_lib_fusiondoublon", labelAttributes);

        endCell();
        endRow();
    }

    protected void printCoupleSeparateurSelectRow() {
        char current = '\n';
        Character cp = (Character) responseParameters.getParameter(AbstractMassCommand.COUPLESEPARATEUR_PARAMNAME);
        if (cp != null) {
            current = cp.charValue();
        }
        startRow();
        printLibCells("saisie_lib_coupleseparateur", true);
        startCell();
        printSmallSelect(AbstractMassCommand.COUPLESEPARATEUR_PARAMNAME, CoupleParser.getAllowedCoupleSeparateurArrayToString(), StringUtils.charToString(current));
        endCell();
        endRow();
    }

    protected void printCroisementUpdateSelectRow() {
        Short currentShort = (Short) responseParameters.getParameter(AbstractMassCommand.CROISEMENTUPDATE_PARAMNAME);
        String current = AbstractMassCommand.CLEAR_PARAMVALUE;
        if (currentShort != null) {
            try {
                current = AbstractMassCommand.croisementUpdateToString(currentShort.shortValue());
            } catch (IllegalArgumentException iae) {
            }
        }
        startRow();
        printLibCells("saisie_lib_updatetype", true);
        startCell();
        HTMLAttributes radioAttributes = new HTMLAttributes();
        HTMLAttributes labelAttributes = new HTMLAttributes();
        radioAttributes.setName(AbstractMassCommand.CROISEMENTUPDATE_PARAMNAME);
        printCroisementUpdateRadio(AbstractMassCommand.CLEAR_PARAMVALUE, current, radioAttributes, labelAttributes);
        printCroisementUpdateRadio(AbstractMassCommand.APPEND_REPLACE_PARAMVALUE, current, radioAttributes, labelAttributes);
        printCroisementUpdateRadio(AbstractMassCommand.APPEND_MAX_PARAMVALUE, current, radioAttributes, labelAttributes);
        printCroisementUpdateRadio(AbstractMassCommand.APPEND_NO_PARAMVALUE, current, radioAttributes, labelAttributes);
        endCell();
        endRow();
    }

    private void printCroisementUpdateRadio(String key, String current, HTMLAttributes radioAttributes, HTMLAttributes labelAttributes) {
        radioAttributes.setValue(key);
        radioAttributes.setChecked((key.equals(current)));
        addRadio(radioAttributes, "saisie_lib_updatetype_" + key, labelAttributes);
        appendBr();
    }

    protected void printParseTypeRadiosRow() {
        TableParser.Parameters tbparams = (TableParser.Parameters) responseParameters.getParameter(AbstractMassCommand.PARSETYPE_PARAMNAME);
        if (tbparams == null) {
            tbparams = new OOCopyParameters();
        }
        startRow();
        printLibCells("saisie_lib_parsetype", true);
        startCell();
        startP();
        HTMLAttributes radioAttributes = new HTMLAttributes();
        HTMLAttributes labelAttributes = new HTMLAttributes();
        radioAttributes.setName(AbstractMassCommand.PARSETYPE_PARAMNAME);

        radioAttributes.setValue(AbstractMassCommand.OOCOPY_PARAMVALUE);
        radioAttributes.setChecked((tbparams instanceof OOCopyParameters));
        addRadio(radioAttributes, "saisie_lib_parsetype_oocopy", labelAttributes);
        endP();
        startP();

        radioAttributes.setValue(AbstractMassCommand.CSV_PARAMVALUE);
        radioAttributes.setChecked((tbparams instanceof CsvParameters));
        addRadio(radioAttributes, "saisie_lib_parsetype_csv", labelAttributes);
        appendBr();
        startSmall();
        printCSVOptions(tbparams);
        endSmall();
        endP();

        startP();

        radioAttributes.setValue(AbstractMassCommand.MYSQL_PARAMVALUE);
        radioAttributes.setChecked((tbparams instanceof MysqlParameters));
        addRadio(radioAttributes, "saisie_lib_parsetype_mysql", labelAttributes);
        appendBr();
        startSmall();
        printMyslOptions(tbparams);
        endSmall();
        endP();

        endCell();
        endRow();
    }

    private void printCSVOptions(TableParser.Parameters parameters) {
        char currentDelimiter = CsvParameters.getDefaultDelimiter();
        char currentQuote = CsvParameters.getDefaultQuote();
        boolean escapedCSV = false;
        if (parameters instanceof CsvParameters) {
            CsvParameters csvParameters = (CsvParameters) parameters;
            currentDelimiter = csvParameters.getDelimiter();
            currentQuote = csvParameters.getQuote();
            escapedCSV = csvParameters.isEscapedCSV();
        }
        HTMLAttributes checkAttributes = new HTMLAttributes();
        checkAttributes.setChecked(escapedCSV);
        checkAttributes.setName(AbstractMassCommand.CSV_ESCAPED_PARAMNAME);
        checkAttributes.setValue("1");
        addCheck(checkAttributes, "global_lib_csv_escaped", null);
        appendBr();
        printLibColon("global_lib_csv_delimiter");
        printSmallSelect(AbstractMassCommand.CSV_DELIMITER_PARAMNAME, CsvParameters.getAvalaibleDelimiterArrayToString(), StringUtils.charToString(currentDelimiter));
        appendBr();
        printLibColon("global_lib_csv_quote");
        printSmallSelect(AbstractMassCommand.CSV_QUOTE_PARAMNAME, CsvParameters.getAvalaibleQuoteArrayToString(), StringUtils.charToString(currentQuote));
    }

    private void printMyslOptions(TableParser.Parameters parameters) {
        char field = MysqlParameters.getDefaultFieldsTerminatedBy();
        char line = MysqlParameters.getDefaultLinesTerminatedBy();
        if (parameters instanceof MysqlParameters) {
            MysqlParameters mysqlParameters = (MysqlParameters) parameters;
            field = mysqlParameters.getFieldsTerminatedBy();
            line = mysqlParameters.getLinesTerminatedBy();
        }
        printLibColon("global_lib_mysql_fieldsdelimitedby");
        printSmallSelect(AbstractMassCommand.MYSQL_FIELDSTERMINATEDBY_PARAMNAME, MysqlParameters.getAvalaibleFieldsTerminatedByArrayToString(), StringUtils.charToString(field));
        appendBr();
        printLibColon("global_lib_mysql_linesdelimitedby");
        printSmallSelect(AbstractMassCommand.MYSQL_LINESTERMINATEDBY_PARAMNAME, MysqlParameters.getAvalaibleLinesTerminatedByArrayToString(), StringUtils.charToString(line));

    }

    private void printSmallSelect(String name, String[] value, String current) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        startSelect(name);
        for (int i = 0; i < value.length; i++) {
            String val = value[i];
            htmlAttributes.setSelected(val.equals(current));
            htmlAttributes.setValue(val);
            startOption(htmlAttributes);
            append(val);
            endOption();
        }
        endSelect();
    }

    public String getTitle() {
        return "Importation";
    }

    public String getCssThemeFile() {
        return "saisie.css";
    }

    public String getFormAction() {
        return "Importation";
    }

    protected void printLangSelect(int[] array) {
        Integer selectedLang = (Integer) responseParameters.getParameter(AbstractMassCommand.LANG_PARAMNAME);
        startSelect(AbstractMassCommand.LANG_PARAMNAME);
        for (int i = 0; i < array.length; i++) {
            int langInteger = array[i];
            appendOption(LangInteger.toString(langInteger), testSelection(langInteger, selectedLang));
        }
        endSelect();
    }

    private boolean testSelection(int current, Integer selected) {
        if (selected == null) {
            return false;
        }
        return current == (selected.intValue());
    }

    protected void addLangParameter() {
        if (langInteger != null) {
            addParameter(AbstractMassCommand.LANG_PARAMNAME, LangInteger.toString(langInteger.intValue()));
        }
    }

    protected void addOtherCorpusParameter() {
        if (otherCorpus != null) {
            addParameter(AbstractMassCommand.OTHERCORPUS_PARAMNAME, otherCorpus.getSubsetKey().getSubsetName());
        }
    }

    protected void addOtherThesaurusParameter() {
        if (otherThesaurus != null) {
            addParameter(AbstractMassCommand.OTHERTHESAURUS_PARAMNAME, otherThesaurus.getSubsetKey().getSubsetName());
        }
    }

    protected void addExecuteStepParameter() {
        addStepParameter(AbstractMassCommand.EXECUTE_STEP_PARAMVALUE);
    }

}
