/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.ui.*;
import fr.exemole.bdfserver.ui.corpus.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFicheCreationConfirmPage extends ImportationHtmlPage {

    private Object[] keyArray;
    private int idcorpusIndex = -1;
    private int langIndex = -1;
    private boolean[] textInputArray;
    private String[] libelleArray;
    private String[] keyStringArray;

    public MassFicheCreationConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        keyArray = (Object[]) responseParameters.getParameter(MassFicheCreationCommand.KEYARRAY_PARAM);
        int length = keyArray.length;
        textInputArray = new boolean[length];
        libelleArray = new String[length];
        keyStringArray = new String[length];
        for (int i = 0; i < length; i++) {
            Object obj = keyArray[i];
            keyStringArray[i] = obj.toString();
            textInputArray[i] = true;
            if (obj instanceof FieldKey) {
                FieldKey fieldKey = (FieldKey) obj;
                CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
                if (fieldKey.equals(FieldKey.IDCORPUS)) {
                    idcorpusIndex = i;
                } else if (fieldKey.equals(FieldKey.LANG)) {
                    langIndex = i;
                } else if (fieldKey.ownsToCorpsdefiche()) {
                    textInputArray[i] = false;
                }
                libelleArray[i] = corpusField.getLibelleString(workingLangInteger, true);
            } else {
                libelleArray[i] = obj.toString();
            }
        }
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massfichecreationconfirm");
        startTable();
        printSelectedCorpusRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSFICHECREATIONFORM_PAGE_PARAMVALUE;
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        printResultItemListTitle("saisie_title_list_fiches", size);
        startPostForm(corpus, MassFicheCreationCommand.getCommandString());
        addExecuteStepParameter();
        startUl();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            startLi();
            printResultItem(resultItem);
            endLi();
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

    private void printResultItem(ParseResult.ResultItem resultItem) {
        Integer itg = (Integer) resultItem.getKeyObject();
        String[] line = (String[]) resultItem.getValueObject();
        int length = line.length;
        String itgS = itg.toString();
        String prefix = itgS + "_";
        startH3();
        appendHiddenInput(MassFicheCreationCommand.IDCORPUSARRAY_PARAMNAME, itgS);
        append(itgS);
        if (langIndex == -1) {
            appendHiddenInput(prefix + FieldKey.LANG, LangInteger.toString(workingLangInteger));
        }
        endH3();
        startTable();
        for (int i = 0; i < length; i++) {
            String name = prefix + keyStringArray[i];
            String libelle = libelleArray[i];
            String value = line[i];
            if (i == idcorpusIndex) {
                continue;
            } else if (i == langIndex) {
                printPredeterminedLine(name, libelle, value);
            } else if (textInputArray[i]) {
                printOneLine(name, libelle, value, 60, null);
            } else {
                printMultiLine(name, libelle, value, 4, null);
            }
        }
        endTable();
    }

}
