/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;


/**
 *
 * @author Vincent Calame
 */
public class MassFieldFormPage extends ImportationHtmlPage {

    public MassFieldFormPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massfield");
        printMessage();
        startParseCorpusForm(MassFieldCommand.getCommandString());
        startRow();
        printLibCells("saisie_lib_fieldkey", false);
        startCell();
        printFieldSelect((FieldKey) responseParameters.getParameter(MassFieldCommand.FIELDKEY_PARAMNAME));
        endCell();
        endRow();
        printCoupleSeparateurSelectRow();
        printFusionDoublonCheckBoxRow();
        printTextArea();
        endParseForm();
        end();
    }

    private void printFieldSelect(FieldKey selectedFieldKey) {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        startSelect(MassFieldCommand.FIELDKEY_PARAMNAME);
        appendOption(FieldKey.TITRE.toString(), testSelection(FieldKey.TITRE, selectedFieldKey));
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            appendOption(FieldKey.SOUSTITRE.toString(), testSelection(FieldKey.SOUSTITRE, selectedFieldKey));
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            FieldKey current = corpusField.getFieldKey();
            appendOption(current.toString(), testSelection(current, selectedFieldKey));
        }
        appendOption(FieldKey.REDACTEURS.toString(), testSelection(FieldKey.REDACTEURS, selectedFieldKey));
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            FieldKey current = corpusField.getFieldKey();
            appendOption(current.toString(), testSelection(current, selectedFieldKey));
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        for (int i = 0; i < sectionCount; i++) {
            CorpusField corpusField = corpusMetadata.getSectionField(i);
            FieldKey current = corpusField.getFieldKey();
            appendOption(current.toString(), testSelection(current, selectedFieldKey));
        }
        endSelect();
    }

    private boolean testSelection(FieldKey current, FieldKey selected) {
        if (selected == null) {
            return false;
        }
        return current.equals(selected);
    }

}
