/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.croisement.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.primitives.*;


/**
 *
 * @author Vincent Calame
 */
public class MassLiageConfirmPage extends ImportationHtmlPage {

    short updateType;
    String updateTypeString;

    public MassLiageConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        updateType = ((Short) responseParameters.getParameter(MassLiageCommand.CROISEMENTUPDATE_PARAMNAME)).shortValue();
        updateTypeString = MassLiageCommand.croisementUpdateToString(updateType);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massliageconfirm");
        startTable();
        printSelectedCorpusRow();
        startRow();
        printLibCells("saisie_lib_liagecorpus", false);
        startCell();
        escape(CorpusMetadataUtils.getCorpusIntituleString(otherCorpus, workingLangInteger));
        space();
        startCode();
        append("[");
        append(otherCorpus.getSubsetKey().toString());
        append("]");
        endCode();
        endCell();
        endRow();
        startRow();
        printLibCells("saisie_lib_updatetype", false);
        startCell();
        localize("saisie_lib_updatetype_" + updateTypeString);
        endCell();
        endRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSLIAGEFORM_PAGE_PARAMVALUE;
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), otherCorpus.getSubsetKey());
        printResultItemListTitle("saisie_title_list_fiches", size);
        startPostForm(corpus, MassLiageCommand.getCommandString());
        addExecuteStepParameter();
        addOtherCorpusParameter();
        addParameter(MassLiageCommand.CROISEMENTUPDATE_PARAMNAME, updateTypeString);
        startUl();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            AbregeFiche abregeFiche = (AbregeFiche) resultItem.getKeyObject();

            startLi();
            startP();
            escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
            endP();
            startUl();
            startLi();
            appendH3("saisie_lib_existingliage");
            startUl();
            CroisementItem croisementItem = croisement.getCroisementItem(corpus.getSubsetKey(), abregeFiche.getTicket());
            int existLength = croisementItem.getCount();
            if (existLength == 0) {
                printNoFicheLi();
            } else {
                for (int k = 0; k < existLength; k++) {
                    int ticket = croisementItem.getTicket(k);
                    int poids = croisementItem.getPoids(k);
                    AbregeFiche abr2 = (AbregeFiche) otherCorpus.getSubsetItemByTicket(ticket);
                    if (abr2 != null) {
                        printFicheLi(abr2, poids);
                    }
                }
            }
            endUl();
            endLi();

            startLi();
            if (updateType == CroisementEditor.CLEAR_EXISTING) {
                appendH3("saisie_lib_newliage_clear");
            } else {
                appendH3("saisie_lib_newliage_append");
            }
            startUl();

            FicheCroisementList abregeFicheList = (FicheCroisementList) resultItem.getValueObject();
            int abrgSize = abregeFicheList.getSubsetItemCount();

            if (abrgSize == 0) {
                if (updateType == CroisementEditor.CLEAR_EXISTING) {
                    appendHiddenInput(String.valueOf(abregeFiche.getIdcorpus()), "");
                    printNoFicheLi();
                } else {
                    printNoAppendFicheLi();
                }
            } else {
                for (int j = 0; j < abrgSize; j++) {
                    AbregeFiche abr2 = abregeFicheList.getAbregeFiche(j);
                    int poids = abregeFicheList.getPoids(j);
                    StringBuffer buf = new StringBuffer();
                    buf.append(abr2.getIdcorpus());

                    if (poids > 1) {
                        buf.append('+');
                        buf.append(poids - 1);
                    }
                    appendHiddenInput(String.valueOf(abregeFiche.getIdcorpus()), buf.toString());
                    printFicheLi(abr2, poids);
                }
            }
            endUl();
            endLi();
            endUl();
            endLi();
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

    private void printFicheLi(AbregeFiche abr2, int poids) {
        startLi();
        startP();
        startCode();
        if (poids > 1) {
            append("(+");
            append(String.valueOf(poids - 1));
            append(")");
            space();
        }
        escape(BdfUserUtils.getAbregeFicheString(bdfUser, abr2));
        endCode();
        endP();
        endLi();
    }

    private void printNoFicheLi() {
        startLi();
        startP();
        startEm();
        localize("saisie_info_nofiche");
        endEm();
        endP();
        endLi();
    }

    private void printNoAppendFicheLi() {
        startLi();
        startP();
        startEm();
        localize("saisie_info_noappendfiche");
        endEm();
        endP();
        endLi();
    }

}
