/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class MassLibConfirmPage extends ImportationHtmlPage {

    String vonew;

    public MassLibConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
        vonew = (String) responseParameters.getParameter(MassLibCommand.VONEW_PARAMNAME);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_masslibconfirm");
        startTable();
        printSelectedThesaurusRow();
        printLangRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSLIBFORM_PAGE_PARAMVALUE;
    }

    protected void addSupplementaryParameters() {
        addVoParameters();
    }

    private void addVoParameters() {
        if (vonew != null) {
            addParameter(MassLibCommand.VONEW_PARAMNAME, vonew);
        }
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        printResultItemListTitle("saisie_title_list_motscles", size);
        startPostForm(thesaurus, MassLibCommand.getCommandString());
        addExecuteStepParameter();
        addLangParameter();
        addVoParameters();
        startUl();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            Motcle motcle = (Motcle) resultItem.getKeyObject();
            startLi();
            startP();
            escape(BdfUserUtils.getMotcleString(bdfUser, motcle));
            endP();
            startUl();
            startP();
            printLib("saisie_lib_oldvalue");
            startCode();
            escape(getOldValue(motcle));
            endCode();
            endP();
            startP();
            printLib("saisie_lib_newvalue");
            appendLargeInput(String.valueOf(motcle.getIdths()), (String) resultItem.getValueObject());
            endP();
            endUl();
            endLi();
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

    private String getOldValue(Motcle motcle) {
        String lib = null;
        if (langInteger == null) {
            lib = motcle.getLibelleString(workingLangInteger, false);
        } else {
            lib = motcle.getLibelleString(langInteger.intValue(), false);
        }
        if (lib == null) {
            lib = "";
        }
        return lib;
    }

}
