/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.langinteger.*;


/**
 *
 * @author Vincent Calame
 */
public class MassMotcleCreationConfirmPage extends ImportationHtmlPage {

    public MassMotcleCreationConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massmotclecreationconfirm");
        startTable();
        printSelectedThesaurusRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSMOTCLECREATIONFORM_PAGE_PARAMVALUE;
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        printResultItemListTitle("saisie_title_list_motscles", size);
        startPostForm(thesaurus, MassMotcleCreationCommand.getCommandString());
        addExecuteStepParameter();
        startUl();
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            printIdalpha(list);
        } else if (ThesaurusUtils.isBabelienType(thesaurus)) {
            printBabelienList(list);
        } else {
            printMultiList(list);
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

    private void printIdalpha(List list) {
        int size = list.size();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            MassMotcleCreationCommand.CreationValue creationValue = (MassMotcleCreationCommand.CreationValue) resultItem.getValueObject();
            String idalpha = (String) creationValue.getIdObject();
            String parentIdalpha = (String) creationValue.getParentIdObject();
            String prefix = String.valueOf(i) + "_";
            startLi();
            startP();
            appendHiddenInput(prefix + "idalpha", idalpha);
            append(idalpha);
            if (parentIdalpha != null) {
                appendHiddenInput(prefix + "parent_idalpha", parentIdalpha);
                escape(" (-> ");
                append(parentIdalpha);
                append(")");
            }
            endP();
            printLibMap(prefix, creationValue.getLibMap());
            endLi();
        }
    }

    private void printMultiList(List list) {
        int size = list.size();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            MassMotcleCreationCommand.CreationValue creationValue = (MassMotcleCreationCommand.CreationValue) resultItem.getValueObject();
            String prefix = String.valueOf(i) + "_";
            startLi();
            printIdths(prefix, creationValue);
            printLibMap(prefix, creationValue.getLibMap());
            endLi();
        }
    }

    private void printBabelienList(List list) {
        int size = list.size();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            MassMotcleCreationCommand.CreationValue creationValue = (MassMotcleCreationCommand.CreationValue) resultItem.getValueObject();
            String prefix = String.valueOf(i) + "_";
            startLi();
            printIdths(prefix, creationValue);
            String lib = creationValue.getLib();
            if (lib != null) {
                appendHiddenInput(prefix + "lib", lib);
                startP();
                startCode();
                escape(lib);
                endCode();
                endP();
            }
            endLi();
        }
    }

    private void printIdths(String prefix, MassMotcleCreationCommand.CreationValue creationValue) {
        Integer idths = (Integer) creationValue.getIdObject();
        Integer parentIdths = (Integer) creationValue.getParentIdObject();
        if ((idths == null) && (parentIdths == null)) {
            return;
        }
        startP();
        if (idths != null) {
            appendHiddenInput(prefix + "idths", idths.toString());
            append(idths.toString());
        }
        if (parentIdths != null) {
            appendHiddenInput(prefix + "parent_idths", parentIdths.toString());
            escape(" (-> ");
            append(parentIdths.toString());
            append(")");
        }
        endP();
    }

    private void printLibMap(String prefix, LinkedIntHashMap libMap) {
        for (int i = 0; i < libMap.size(); i++) {
            int langInteger = libMap.getKeyAt(i);
            String lib = (String) libMap.getValueAt(i);
            startP();
            appendHiddenInput(prefix + "lib_" + LangInteger.toString(langInteger), lib);
            append(LangInteger.toString(langInteger));
            append(" : ");
            startCode();
            escape(lib);
            endCode();
            endP();
        }

    }

}
