/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.importation.htmlpages;

import fr.exemole.bdfserver.response.ImportationResponseParameters;
import fr.exemole.bdfserver.servlets.importation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.bdfserver.users.*;
import fr.exemole.bdfserver.servlets.resources.*;


/**
 *
 * @author Vincent Calame
 */
public class MassMotcleRemoveConfirmPage extends ImportationHtmlPage {

    public MassMotcleRemoveConfirmPage(ImportationResponseParameters responseParameters) {
        super(responseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTitre("saisie_title_massmotcleremoveconfirm");
        startTable();
        printSelectedThesaurusRow();
        endTable();
        printMessage();
        printReturn();
        printResultItemList(parseResult.getResultItemList());
        end();
    }

    protected String getReturnPageValue() {
        return DefaultCommand.MASSMOTCLEREMOVEFORM_PAGE_PARAMVALUE;
    }

    private void printResultItemList(List list) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        String[] intitules = new String[corpusCount];
        for (int i = 0; i < corpusCount; i++) {
            intitules[i] = CorpusMetadataUtils.getCorpusIntituleString(basedefiches.getCorpus(i), workingLangInteger);
        }
        Croisement[] croisements = CroisementUtils.getCroisementWithCorpusArray(thesaurus);

        printResultItemListTitle("saisie_title_list_motscles", size);
        startPostForm(thesaurus, MassMotcleRemoveCommand.getCommandString());
        addExecuteStepParameter();
        startUl();
        for (int i = 0; i < size; i++) {
            ParseResult.ResultItem resultItem = (ParseResult.ResultItem) list.get(i);
            Motcle motcle = (Motcle) resultItem.getKeyObject();
            startLi();
            startP();
            appendCheckInput(MassMotcleRemoveCommand.IDTHSLIST_PARAMNAME, String.valueOf(motcle.getIdths()), true, null);
            escape(BdfUserUtils.getMotcleString(bdfUser, motcle));
            endP();
            startP(CssClass.P_SUBTITLE);
            boolean premier = true;
            for (int j = 0; j < croisements.length; j++) {
                Croisement croisement = croisements[j];
                int count = croisement.getCroisementItem(thesaurus.getSubsetKey(), motcle.getTicket()).getCount();
                if (count > 0) {
                    if (premier) {
                        premier = false;
                    } else {
                        printTiret();
                    }
                    escape(intitules[j]);
                    append("&nbsp: ");
                    append(String.valueOf(count));
                }
            }
            if (premier) {
                localize("saisie_info_noindexation");
            }
            endP();
            endLi();
        }
        endUl();
        endForm("saisie_submit_massconfirm");
    }

}
