/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.session.htmlpages;

import fr.exemole.bdfserver.response.SessionResponseParameters;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.servlets.session.*;
import java.io.*;
import javax.servlet.http.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.metadata.*;
import net.mapeadores.util.langinteger.*;
import fr.exemole.bdfserver.users.*;


/**
 *
 * @author Vincent Calame
 */
public class FramePage extends SessionHtmlPage implements BdfUserConstants {

    public FramePage(SessionResponseParameters sessionResponseParameters) {
        super(sessionResponseParameters);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        startHtml(localisationInfo.getFormatLocaleInteger());
        startHead();
        startTitle();
        escape(BasedefichesMetadataUtils.getLongIntituleString(bdfServer.getBasedefiches(), workingLangInteger));
        endTitle();
        HTMLAttributes metaAttributes = new HTMLAttributes();
        metaAttributes.setHttpEquiv("Content-Type");
        metaAttributes.setContent("text/html; charset=UTF-8");
        appendMeta(metaAttributes);
        HTMLAttributes linkAttributes = new HTMLAttributes();
        linkAttributes.setHref("rsc/icon.png");
        linkAttributes.setType("image/png");
        linkAttributes.setRel("icon");
        appendLink(linkAttributes);
        endHead();
        startRowsFrameset("55,*");
        appendFrame("Menu", "Affichage?page=menu");
        String listeWidth = getListeWidth();
        if (LangInteger.isRTLScript(localisationInfo.getFormatLocaleInteger())) {
            startColsFrameset("*," + listeWidth);
            appendSaisieFrame();
            appendListeFrame();
            endFrameset();
        } else {
            startColsFrameset(listeWidth + ",*");
            appendListeFrame();
            appendSaisieFrame();
            endFrameset();
        }
        endFrameset();
        endHtml();
    }

    private void appendListeFrame() throws IOException {
        appendFrame("Liste", "Affichage?page=fiches");
    }

    private void appendSaisieFrame() throws IOException {
        appendFrame("Saisie", "Affichage?page=accueil");
    }

    private String getListeWidth() {
        if (bdfUser == null) {
            return "25%";
        }
        switch (bdfUser.getScreenWidth()) {
            case SCREENWIDTH_800:
                return "200";
            case SCREENWIDTH_1024:
                return "270";
            case SCREENWIDTH_1280:
                return "350";
            default:
                return "25%";
        }
    }

    public String getTitle() {
        return null;
    }

    public String getFormAction() {
        return null;
    }

    public String getCssThemeFile() {
        return null;
    }

}
