/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class CompleteTransferCommand extends AbstractThesaurusCommand {

    public final static String DESTINATIONTHESAURUS_PARAMNAME = "destinationthesaurus";
    Thesaurus destination;
    ThesaurusEditor destinationEditor;
    int step;

    public CompleteTransferCommand() {
    }

    protected void doThesaurusCommand() {
        destinationEditor = bdfServer.getBasedefichesEditor().getThesaurusEditor(destination.getSubsetKey());
        if (step == 1) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.COMPLETETRANSFERCONFIRMATION_PAGE);
        } else if (step == 2) {
            int langInteger = bdfUser.getWorkingLangInteger();
            int transferCount = thesaurus.size();
            transfer();
            Object[] objstt = new Object[3];
            objstt[0] = new Integer(transferCount);
            objstt[1] = ThesaurusUtils.getThesaurusIntitule(thesaurus, langInteger);
            objstt[2] = ThesaurusUtils.getThesaurusIntitule(destination, langInteger);
            responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
            responseParameters.setDoneMessage("thesaurus_msg_transferttotal2", objstt);
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        step = getStepNumber();
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            throw new ParameterException(ParameterException.CMD_UNVALAIBLE_IDALPHA, getCommandString());
        }
        String param_destination = requestMap.getParameter(DESTINATIONTHESAURUS_PARAMNAME);
        if (param_destination == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, DESTINATIONTHESAURUS_PARAMNAME);
        }
        destination = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), param_destination);
        if (destination == null) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, DESTINATIONTHESAURUS_PARAMNAME + " = " + param_destination);
        }
        if (destination.equals(thesaurus)) {
            throw new ParameterException(ParameterException.PARAMETER_EQUALSTHESAURUS, DESTINATIONTHESAURUS_PARAMNAME);
        }
        responseParameters.putParameter(DESTINATIONTHESAURUS_PARAMNAME, destination);
        if (step == 2) {
            if (!checkConfirmPassword()) {
                responseParameters.setHtmlPageCode(ThesaurusResponseParameters.COMPLETETRANSFERCONFIRMATION_PAGE);
                responseParameters.setErrorMessage("global_err_password", null);
                return;
            }
        }
    }

    /*
     * Lancement du transfert.
     */
    private void transfer() {
        MotcleList firstLevel = thesaurus.getFirstLevelList();
        for (int i = 0; i < firstLevel.getMotcleCount(); i++) {
            copy(firstLevel.getMotcle(i));
        }
    }

    /*
     * Copie itérative des mots-clés.
     */
    private Motcle copy(Motcle sourceMotcle) {
        Motcle destinationMotcle = ThesaurusUtils.copy(destinationEditor, sourceMotcle);
        MotcleList currentChildren = sourceMotcle.getChildList();
        for (int i = 0; i < currentChildren.getMotcleCount(); i++) {
            Motcle childCopy = copy(currentChildren.getMotcle(i));
            try {
                destinationEditor.setParent(childCopy, destinationMotcle);
            } catch (ParentRecursivityException pre) {
                throw new ImplementationException("test before", pre);
            }
        }
        ThesaurusUtils.transfereIndexation(bdfServer.getBasedefichesEditor(), sourceMotcle, destinationMotcle);
        return destinationMotcle;
    }

    public static String getCommandString() {
        return "CompleteTransfer";
    }

    public boolean needMotcleInit() {
        return false;
    }

}
