/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.request.ParameterException;
import fr.exemole.bdfserver.request.RequestUtils;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.tools.thesaurus.*;


/**
 *
 * @author Vincent Calame
 */
public class IdalphaSortCommand extends AbstractThesaurusCommand {

    public static String RECURSIVE_PARAMNAME = "recursive";
    public static String DESCENDING_PARAMNAME = "descending";
    public static String IGNORECASE_PARAMNAME = "ignorecase";
    public static String IGNOREPUNCTUATION_PARAMNAME = "ignorepunctuation";
    boolean recursive = false;
    boolean descending = false;
    boolean ignoreCase = false;
    boolean ignorePunctuation = false;

    public IdalphaSortCommand() {
    }

    protected void doThesaurusCommand() {
        ThesaurusIdalphaSort idalphaSort = new ThesaurusIdalphaSort(thesaurusEditor);
        idalphaSort.setRecursive(recursive);
        idalphaSort.setDescending(descending);
        idalphaSort.setIgnoreCase(ignoreCase);
        idalphaSort.setIgnorePunctuation(ignorePunctuation);
        idalphaSort.sort(motcle);
        short pageType = (motcle != null) ? ThesaurusResponseParameters.ADVANCED_PAGE : ThesaurusResponseParameters.THESAURUSFORM_PAGE;
        responseParameters.setHtmlPageCode(pageType);
        responseParameters.setDoneMessage("thesaurus_done_idalphasort", null);
    }

    protected void checkThesaurusParameters() throws ParameterException {
        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            throw new ParameterException(ParameterException.IDALPHA_NOTAVAILABLE, thesaurus.getSubsetKey().getSubsetName());
        }
        boolean fromMotcle = RequestUtils.hasMotcleParameter(requestMap);
        if (fromMotcle) {
            initMotcle();
        }
        recursive = getParam(RECURSIVE_PARAMNAME);
        descending = getParam(DESCENDING_PARAMNAME);
        ignoreCase = getParam(IGNORECASE_PARAMNAME);
        ignorePunctuation = getParam(IGNOREPUNCTUATION_PARAMNAME);
    }

    public static String getCommandString() {
        return "IdalphaSort";
    }

    public boolean needMotcleInit() {
        return false;
    }

    private boolean getParam(String paramName) throws ParameterException {
        String param = requestMap.getParameter(paramName);
        if (param == null) {
            return false;
        }
        if (param.length() == 0) {
            return false;
        }
        if (param.equals("0")) {
            return false;
        }
        if (param.equals("1")) {
            return true;
        }
        throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, paramName + "=" + param);
    }

}
