/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 *
 * @author Vincent Calame
 */
public class LibelleCommand extends AbstractThesaurusCommand {

    private TextParser textParser;
    String param_mtcllib;
    int langInteger = LangInteger.UNDETERMINED;

    public LibelleCommand() {
    }

    protected void doThesaurusCommand() {
        initTextParser();
        boolean done = false;
        if (ThesaurusUtils.isBabelienType(thesaurus)) {
            done = setBabelienLibelle();
        } else {
            done = setAllLibelles();
        }
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
        if (done) {
            responseParameters.setDoneMessage("thesaurus_msg_modiflibelle", null);
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        if (ThesaurusUtils.isBabelienType(thesaurus)) {
            String param_mtcllgpref = requestMap.getParameter("mtcl_lgpref");
            param_mtcllib = requestMap.getParameter("mtcl_lib");
            if (param_mtcllgpref == null) {
                throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, "mtcl_lgpref");
            }
            if (param_mtcllib == null) {
                throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, "mtcl_lib");
            }
            if (param_mtcllgpref.length() > 0) {
                try {
                    langInteger = LangInteger.parse(param_mtcllgpref);
                } catch (LangIntegerException mcle) {
                    responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLE_PAGE);
                    responseParameters.setErrorMessage("thesaurus_err_langcode", param_mtcllgpref);
                    return;
                }
            }
        }
    }

    protected void initTextParser() {
        textParser = new TextParser();
    }

    public static String getCommandString() {
        return "Libelle";
    }

    public boolean needMotcleInit() {
        return true;
    }

    protected boolean setBabelienLibelle() {
        TypoOptions typoOptions = TypoOptions.getTypoOptions(langInteger);
        textParser.setQuoteOptions(typoOptions.getQuoteOptions());
        CleanedString cleanedString = CleanedString.newInstance(textParser.parseString(param_mtcllib));
        boolean done = false;
        if (cleanedString == null) {
            thesaurusEditor.removeLibelle(motcle, langInteger);
        } else {
            thesaurusEditor.putLibelle(motcle, LibelleUtils.toLibelle(langInteger, cleanedString));
        }
        return done;
    }

    public boolean setAllLibelles() {
        int[] array = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
        boolean done = false;
        for (int i = 0; i < array.length; i++) {
            int langInteger = array[i];
            String param = requestMap.getParameter("mtcl_lib_" + LangInteger.toString(langInteger));
            if (param == null) {
                continue;
            }
            done = true;
            TypoOptions typoOptions = TypoOptions.getTypoOptions(langInteger);
            textParser.setQuoteOptions(typoOptions.getQuoteOptions());
            CleanedString cleanedString = CleanedString.newInstance(textParser.parseString(param));
            if (cleanedString == null) {
                thesaurusEditor.removeLibelle(motcle, langInteger);
            } else {
                thesaurusEditor.putLibelle(motcle, LibelleUtils.toLibelle(langInteger, cleanedString));
            }
        }
        return done;
    }

}
