/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.parsers.*;
import net.mapeadores.basedefiches.croisement.*;


/**
 *
 * @author Vincent Calame
 */
public class MotcleIndexationCommand extends AbstractThesaurusCommand {

    public MotcleIndexationCommand() {
    }

    protected void doThesaurusCommand() {
        updateIndexation();
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.MOTCLEINDEXATION_PAGE);
        responseParameters.setDoneMessage("thesaurus_done_motcleindexation", null);
        responseParameters.setMajLinkNeeded(false);
    }

    protected void checkThesaurusParameters() throws ParameterException {
    }

    public static String getCommandString() {
        return "MotcleIndexation";
    }

    public boolean needMotcleInit() {
        return true;
    }

    public void updateIndexation() {
        CroisementEditor croisementEditor = bdfServer.getBasedefichesEditor().getCroisementEditor();
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            if (isSlaveCorpus(corpus)) {
                continue;
            }
            String param_value = requestMap.getParameter(corpus.getSubsetKey().toString());
            if (param_value == null) {
                continue;
            }
            FicheCroisementList abregeFicheList = parse(corpus, param_value);
            croisementEditor.updateCroisement(motcle, abregeFicheList, CroisementEditor.CLEAR_EXISTING);
        }
    }

    private FicheCroisementList parse(Corpus corpus, String chaine) {
        String[] tokens = StringUtils.getTechnicalTokens(chaine, false);
        DefaultFicheCroisementList abregeFicheList = new DefaultFicheCroisementList(corpus.getSubsetKey(), tokens.length);
        for (int i = 0; i < tokens.length; i++) {
            PoidsIntegerToken poidsIntegerToken = PoidsIntegerToken.parse(tokens[i]);
            if (poidsIntegerToken == null) {
                continue;
            }
            AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(poidsIntegerToken.getInteger());
            if (abregeFiche == null) {
                continue;
            }
            abregeFicheList.add(abregeFiche, poidsIntegerToken.getPoids());
        }
        return abregeFicheList;
    }

    public static String croisementToPlainString(Motcle motcle, Corpus corpus) {
        SubsetKey thesaurusKey = motcle.getThesaurus().getSubsetKey();
        Croisement croisement = corpus.getBasedefiches().getCroisement(corpus.getSubsetKey(), thesaurusKey);
        CroisementItem croisementItem = CroisementUtils.sortByPoids(croisement.getCroisementItem(thesaurusKey, motcle.getTicket()));
        int count = croisementItem.getCount();
        if (count == 0) {
            return "";
        }
        StringBuffer buf = new StringBuffer(count * 8);
        for (int i = 0; i < count; i++) {
            SubsetItem subsetItem = corpus.getSubsetItemByTicket(croisementItem.getTicket(i));
            buf.append(subsetItem.getIdsubset());
            int poids = croisementItem.getPoids(i);
            if (poids > 1) {
                buf.append(" <");
                buf.append(poids);
                buf.append(">");
            }
            buf.append("; ");
        }
        return buf.toString();
    }

}
