/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.commands.AbstractBdfCommand;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.extension.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusCreationCommand extends AbstractBdfCommand {

    public final static String NEWTHESAURUS_PARAMNAME = "newthesaurus";
    public final static String THESAURUSTYPE_PARAMNAME = "thesaurustype";

    public ThesaurusCreationCommand() {
    }

    public BdfResponseParameters doCommand() throws ParameterException {
        int langInteger = bdfUser.getWorkingLangInteger();
        String nameParam = requestMap.getParameter(NEWTHESAURUS_PARAMNAME);
        if (nameParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, NEWTHESAURUS_PARAMNAME);
        }
        String typeParam = requestMap.getParameter(THESAURUSTYPE_PARAMNAME);
        if (typeParam == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, THESAURUSTYPE_PARAMNAME);
        }
        short thesaurusType = getThesaurusType(typeParam);

        ThesaurusResponseParameters responseParameters = new ThesaurusResponseParameters(bdfServer, bdfUser, langInteger);
        try {
            SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, nameParam);
            ThesaurusEditor thesaurusEditor = bdfServer.getBasedefichesEditor().createThesaurus(thesaurusKey, thesaurusType);
            if (thesaurusType != ThesaurusMetadata.BABELIEN_TYPE) {
                thesaurusEditor.getThesaurusMetadataEditor().setAuthorizedLangIntegerArray(bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray());
            }
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.THESAURUSFORM_PAGE);
            responseParameters.setDoneMessage("thesaurus_done_thesauruscreation", thesaurusKey.toString());
            responseParameters.setThesaurus(thesaurusEditor.getThesaurus());
        } catch (ExistingSubsetException ese) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.THESAURUSCREATIONFORM_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_existingthesaurusname", nameParam);
            return responseParameters;
        } catch (java.text.ParseException pe) {
            responseParameters.setHtmlPageCode(ThesaurusResponseParameters.THESAURUSCREATIONFORM_PAGE);
            responseParameters.setErrorMessage("thesaurus_err_wrongthesaurusname", nameParam);
            return responseParameters;
        }
        return responseParameters;
    }

    public static String getCommandString() {
        return "ThesaurusCreation";
    }

    private short getThesaurusType(String typeParam) throws ParameterException {
        try {
            short type = ThesaurusUtils.thesaurusTypeToShort(typeParam);
            return type;
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, THESAURUSTYPE_PARAMNAME + " = " + typeParam);
        }
    }

    public boolean needSynchronisation() {
        return true;
    }

    public short getCommandType() {
        return ADMIN_ONLY_COMMAND;
    }

    public ResponsePrinterFactory getResponsePrinterFactory() {
        return new ThesaurusResponsePrinterFactory();
    }

}
