/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusIntituleCommand extends AbstractThesaurusCommand {

    ThesaurusMetadataEditor thesaurusMetadataEditor;

    public ThesaurusIntituleCommand() {
    }

    public static String getCommandString() {
        return "ThesaurusIntitule";
    }

    protected void doThesaurusCommand() {
        thesaurusMetadataEditor = thesaurusEditor.getThesaurusMetadataEditor();
        TextParser textParser = new TextParser();
        boolean done = false;
        Set set = requestMap.getParameterNameSet();
        for (Iterator it = set.iterator(); it.hasNext();) {
            String paramName = (String) it.next();
            int idx = paramName.lastIndexOf('/');
            if (idx == -1) {
                continue;
            }
            String fieldName = paramName.substring(0, idx);
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(idx + 1));
                TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLangInteger);
                textParser.setQuoteOptions(typoOptions.getQuoteOptions());
                String nvlibelle = textParser.parseString(requestMap.getParameter(paramName));
                CleanedString cleanedString = CleanedString.newInstance(nvlibelle);
                if (fieldName.startsWith("intitule_")) {
                    String intituleName = fieldName.substring(9);
                    Intitule thesaurusIntitule = BasedefichesUtils.getOrCreateIntitule(thesaurusMetadataEditor, intituleName);
                    boolean stepDone = false;
                    if (cleanedString == null) {
                        stepDone = thesaurusMetadataEditor.removeLibelle(thesaurusIntitule, currentLangInteger);
                    } else {
                        stepDone = thesaurusMetadataEditor.putLibelle(thesaurusIntitule, LibelleUtils.toLibelle(currentLangInteger, cleanedString));
                    }
                    if (stepDone) {
                        done = true;
                    }
                }
            } catch (LangIntegerException ile) {
            }
        }
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.THESAURUSFORM_PAGE);
        if (done) {
            responseParameters.setDoneMessage("thesaurus_done_intitules", null);
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
    }

    public boolean needMotcleInit() {
        return false;
    }

}
