/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.request.ParameterException;
import fr.exemole.bdfserver.managers.policies.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusPolicyCommand extends AbstractThesaurusCommand {

    public final static String POLICY_PARAMNAME = "policy";
    public final static String TRANSFERKEY_PARAMNAME = "transferkey";
    public final static String VERIFKEYLIST_PARAMNAME = "verifkeylist";
    private Object policyParameter = null;
    private short ajoutPolicy;

    public ThesaurusPolicyCommand() {
    }

    public static String getCommandString() {
        return "ThesaurusPolicy";
    }

    public boolean needMotcleInit() {
        return false;
    }

    protected void doThesaurusCommand() {
        boolean done = bdfServer.getPolicyManager().getThesaurusPolicy(thesaurus.getSubsetKey()).setAjoutPolicy(ajoutPolicy, policyParameter);
        responseParameters.setHtmlPageCode(ThesaurusResponseParameters.THESAURUSFORM_PAGE);
        if (done) {
            responseParameters.setDoneMessage("thesaurus_done_policy", null);
            bdfServer.savePolicyManager();
        }
    }

    protected void checkThesaurusParameters() throws ParameterException {
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            throw new ParameterException(ParameterException.CMD_UNVALAIBLE_IDALPHA, getCommandString());
        }
        ajoutPolicy = getAjoutPolicy();
        if (ajoutPolicy == ThesaurusPolicy.AJOUT_TRANSFERT) {
            policyParameter = getTransferKey();
        } else if (ajoutPolicy == ThesaurusPolicy.AJOUT_VERIFICATION) {
            policyParameter = getVerifList();
        }
    }

    private short getAjoutPolicy() throws ParameterException {
        String policyParam = getMandatoryParameter(POLICY_PARAMNAME);
        try {
            short ajoutPolicy = ThesaurusPolicy.ajoutPolicyToShort(policyParam);
            if ((ThesaurusUtils.isMultiType(thesaurus)) && (!ThesaurusPolicy.availablePolicyForMultiThesaurus(ajoutPolicy))) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, POLICY_PARAMNAME + " = " + policyParam);
            }
            return ajoutPolicy;
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, POLICY_PARAMNAME + " = " + policyParam);
        }
    }

    private SubsetKey getTransferKey() throws ParameterException {
        String transferKeyParam = getMandatoryParameter(TRANSFERKEY_PARAMNAME);
        Thesaurus transferThesaurus = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), transferKeyParam);
        if ((transferThesaurus == null) || (!ThesaurusUtils.isBabelienType(transferThesaurus))) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, TRANSFERKEY_PARAMNAME + " = " + transferKeyParam);
        }
        return transferThesaurus.getSubsetKey();
    }

    private List<SubsetKey> getVerifList() throws ParameterException {
        String[] listArray = requestMap.getParameterValues(VERIFKEYLIST_PARAMNAME);
        if (listArray == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, VERIFKEYLIST_PARAMNAME);
        }
        ArrayList<SubsetKey> arrayList = new ArrayList<SubsetKey>();
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        for (int i = 0; i < listArray.length; i++) {
            Thesaurus verifThesaurus = BasedefichesUtils.getThesaurus(basedefiches, listArray[i]);
            if ((verifThesaurus != null) && (!ThesaurusUtils.isIdalphaType(verifThesaurus))) {
                arrayList.add(verifThesaurus.getSubsetKey());
            }
        }
        if (arrayList.size() == 0) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, VERIFKEYLIST_PARAMNAME + " = " + listArray.toString());
        }
        return arrayList;
    }

}
