/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus;

import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.thesaurus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.langinteger.LangInteger;
import java.util.Locale;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class ToLowerCaseCommand extends AbstractThesaurusCommand {

    public static String NOCHANGE_PARAMNAME = "nochange";
    public static String FIRST_NOCHANGE_PARAMVALUE = "first";

    public ToLowerCaseCommand() {
    }

    protected void doThesaurusCommand() {
        String nochangeParam = requestMap.getParameter(NOCHANGE_PARAMNAME);
        if (nochangeParam == null) {
            nochangeParam = "";
        }
        int changeCount = 0;
        for (int i = 0; i < thesaurus.getTicketLength(); i++) {
            Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
            if (motcle != null) {
                int libelleCount = motcle.getLibelleCount();
                for (int j = 0; j < libelleCount; j++) {
                    Libelle libelle = motcle.getLibelleByIndex(j);
                    String lib = libelle.getLibelleString();
                    Locale locale = LangInteger.getLocale(libelle.getLangInteger());
                    String lower = toLowerCase(lib, locale, nochangeParam);
                    if (!(lower.equals(lib))) {
                        CleanedString cleanedString = CleanedString.newInstance(lower);
                        thesaurusEditor.putLibelle(motcle, LibelleUtils.toLibelle(libelle.getLangInteger(), cleanedString));
                        changeCount++;
                    }
                }

            }
        }
        responseParameters.setHtmlPageCode(BdfResponseParameters.MESSAGE_SERVLETPAGE);
        responseParameters.setDoneMessage("thesaurus_done_tolowercase", String.valueOf(changeCount));
    }

    protected void checkThesaurusParameters() throws ParameterException {
    }

    private String toLowerCase(String lib, Locale locale, String nochangeParam) {
        int length = lib.length();
        if (length == 0) {
            return "";
        }
        if (nochangeParam.equals(FIRST_NOCHANGE_PARAMVALUE)) {
            StringBuilder buf = new StringBuilder(length);
            buf.append(lib.charAt(0));
            buf.append(lib.substring(1).toLowerCase(locale));
            return buf.toString();
        } else {
            return lib.toLowerCase(locale);
        }
    }

    public static String getCommandString() {
        return "ToLowerCase";
    }

    public boolean needMotcleInit() {
        return false;
    }

}
