/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.get.instructions.pioche.PiocheConstants;
import fr.exemole.bdfserver.response.*;
import fr.exemole.bdfserver.servlets.thesaurus.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.util.*;
import fr.exemole.bdfserver.managers.*;
import fr.exemole.bdfserver.managers.policies.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.servlets.util.PiocheParameters;


/**
 *
 * @author Vincent Calame
 */
public class AdvancedPage extends ThesaurusHtmlPage {

    public AdvancedPage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(THESAURUS_ADVANCED_JS);
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int thesaurusCount = basedefiches.getThesaurusCount();
        PolicyManager policyManager = bdfServer.getPolicyManager();
        ThesaurusPolicy thesaurusPolicy = policyManager.getThesaurusPolicy(thesaurus.getSubsetKey());
        SubsetKey moveSelectedKey = (SubsetKey) bdfUser.getParameterValue("thesaurus.move.last");
        if (moveSelectedKey == null) {
            moveSelectedKey = getPreferredThesaurusKey();
        }
        SubsetKey mergeSelectedKey = (SubsetKey) bdfUser.getParameterValue("thesaurus.merge.last");
        if (mergeSelectedKey == null) {
            mergeSelectedKey = moveSelectedKey;
        }
        start();
        printTitre();

        if (!ThesaurusUtils.isIdalphaType(thesaurus)) {
            StringBuffer bufoptions = new StringBuffer();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus ths = basedefiches.getThesaurus(i);
                if (!ths.equals(thesaurus)) {
                    ThesaurusPolicy thsPolicy = policyManager.getThesaurusPolicy(ths.getSubsetKey());
                    if ((!ThesaurusUtils.isIdalphaType(ths)) && (thsPolicy.isEditable())) {
                        SubsetKey thsKey = ths.getSubsetKey();
                        bufoptions.append("<OPTION value=\"");
                        bufoptions.append(thsKey.getSubsetName());
                        bufoptions.append("\"");
                        if (thsKey.equals(moveSelectedKey)) {
                            bufoptions.append(" SELECTED");
                        }
                        bufoptions.append(">");
                        HTMLUtils.appendHTML(bufoptions, ThesaurusUtils.getThesaurusIntitule(ths, workingLangInteger));
                        bufoptions.append("</OPTION>");
                    }
                }
            }
            String choptions = bufoptions.toString();
            if (choptions.length() > 0) {
                startThesaurusForm(MoveCommand.getCommandString());
                addStepParameter("1");
                appendP(null, "thesaurus_frm_titredeplacement");
                startP();
                printLib("thesaurus_frm_titrethesaurusdeplacement");
                startSelect(MoveCommand.DESTINATIONTHESAURUS_PARAMNAME);
                pw.print(choptions);
                endSelect();
                endP();
                endForm("thesaurus_submit_move");
            }
        }

        startThesaurusForm(MergeCommand.getCommandString());
        startP();
        localize("thesaurus_frm_titrefusion");
        endP();
        addStepParameter("1");
        startP();
        printLib("thesaurus_frm_titreidentifiantfusion");
        String appelantId = "inputfusion";
        HTMLAttributes txtAtt = new HTMLAttributes();
        txtAtt.setName(MergeCommand.DESTINATIONIDMTCL_PARAMNAME);
        txtAtt.setSize("30");
        txtAtt.setId(appelantId);
        appendTextInput(txtAtt);

        if (isWithJQuery()) {
            AppelantWriter appelantWriter = new AppelantWriter(bdfServer.getBasedefiches(), this, workingLangInteger);
            PiocheParameters piocheParameters = new PiocheParameters();
            piocheParameters.setAppelantId(appelantId);
            piocheParameters.setJavascriptFunction("piocheFusion");
            piocheParameters.setLimite(1);
            piocheParameters.setParam(PiocheConstants.IDTHS_PARAMVALUE);
            appelantWriter.printPiocheur(piocheParameters);
        }

        endP();
        startP();
        printLib("thesaurus_frm_titrethesaurusfusion");
        HTMLAttributes fusionAtt = new HTMLAttributes();
        fusionAtt.setName(MergeCommand.DESTINATIONTHESAURUS_PARAMNAME);
        fusionAtt.setId("thsfusion");
        startSelect(fusionAtt);
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus ths = basedefiches.getThesaurus(i);
            SubsetKey thsKey = ths.getSubsetKey();
            startOption(thsKey.getSubsetName(), thsKey.equals(mergeSelectedKey));
            escape(ThesaurusUtils.getThesaurusIntitule(ths, workingLangInteger));
            endOption();
        }
        endSelect();
        endP();
        endForm("thesaurus_frm_fusion");

        if ((bdfServer.getBasedefiches().isRemoveable(motcle)) || (ThesaurusPolicy.isLaxPolicy(thesaurusPolicy.getAjoutPolicy()))) {
            startThesaurusForm(MotcleRemoveCommand.getCommandString());
            appendP(null, "thesaurus_frm_titresuppression");
            endFormWithConfirm("thesaurus_frm_suppression");
        }

        if ((ThesaurusUtils.isIdalphaType(thesaurus)) && (motcle.getChildList().getMotcleCount() > 0)) {
            startThesaurusForm(IdalphaSortCommand.getCommandString());
            appendP(null, "thesaurus_title_idalphasort");
            startP();
            appendCheckInput(IdalphaSortCommand.RECURSIVE_PARAMNAME, "1", false, "thesaurus_lib_recursive");
            endP();
            startP();
            appendCheckInput(IdalphaSortCommand.DESCENDING_PARAMNAME, "1", false, "thesaurus_lib_descending");
            endP();
            startP();
            appendCheckInput(IdalphaSortCommand.IGNORECASE_PARAMNAME, "1", false, "thesaurus_lib_ignorecase");
            endP();
            startP();
            appendCheckInput(IdalphaSortCommand.IGNOREPUNCTUATION_PARAMNAME, "1", false, "thesaurus_lib_ignorepunctuation");
            endP();
            endForm("thesaurus_submit_idalphasort");
        }
        end();
    }

}
