/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.servlets.thesaurus.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.util.html.*;

import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.policies.ThesaurusPolicy;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import fr.exemole.bdfserver.request.parsers.PoidsToken;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import net.mapeadores.basedefiches.croisement.*;
import net.mapeadores.util.primitives.PrimUtils;


/**
 *
 * @author Vincent Calame
 */
public class PonderationPage extends ThesaurusHtmlPage {

    private Basedefiches basedefiches;
    private String triParamValue;
    private boolean asc = false;
    private boolean parCorpus = false;
    private SubsetKey thesaurusKey;
    private HTMLAttributes plusAttributes;
    private HTMLAttributes moinsAttributes;

    public PonderationPage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(THESAURUS_PONDERATION_JS);
        triParamValue = (String) responseParameters.getParameter(DefaultCommand.TRI_PARAMNAME);
        asc = (triParamValue.equals(DefaultCommand.CORPUSASC_TRI_PARAMVALUE)) || (triParamValue.equals(DefaultCommand.ASC_TRI_PARAMVALUE));
        parCorpus = (triParamValue.equals(DefaultCommand.CORPUSASC_TRI_PARAMVALUE)) || (triParamValue.equals(DefaultCommand.CORPUSDESC_TRI_PARAMVALUE));
        basedefiches = bdfServer.getBasedefiches();
        thesaurusKey = thesaurus.getSubsetKey();
        plusAttributes = new HTMLAttributes();
        plusAttributes.setOnClick("plus(this);return false;");
        plusAttributes.setClasses("PlusMoins");
        plusAttributes.setHref("#");
        moinsAttributes = new HTMLAttributes();
        moinsAttributes.setOnClick("moins(this);return false;");
        moinsAttributes.setClasses("PlusMoins");
        moinsAttributes.setHref("#");
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        HTMLAttributes scriptAttributes = new HTMLAttributes();
        scriptAttributes.setType("text/javascript");
        startScript(scriptAttributes);
        append("\nvar errorMessage=\"");
        printLib("thesaurus_err_ponderationvalue");
        append("\"; var isAscending=");
        if (asc) {
            append("true");
        } else {
            append("false");
        }
        append(";\n");
        endScript();
        printTitre();
        printTriLinks();
        startForm(motcle, PonderationCommand.getCommandString());
        addParameter(DefaultCommand.TRI_PARAMNAME, triParamValue);
        printFixedSubmit("thesaurus_submit_ponderation");
        if (parCorpus) {
            printCorpusTri();
        } else {
            printAllTri();
        }
        endForm("thesaurus_submit_ponderation");
        end();
    }

    private void printTriLinks() {
        startP(CssClass.P_TOOLBARLINKS);
        printTriLink(DefaultCommand.CORPUSASC_TRI_PARAMVALUE);
        printTiret();
        printTriLink(DefaultCommand.CORPUSDESC_TRI_PARAMVALUE);
        printTiret();
        printTriLink(DefaultCommand.ASC_TRI_PARAMVALUE);
        printTiret();
        printTriLink(DefaultCommand.DESC_TRI_PARAMVALUE);
        endP();
    }

    private void printTriLink(String paramValue) {
        if (paramValue.equals(triParamValue)) {
            startSpan(CssClass.SPAN_CURRENTPAGE);
            localize("thesaurus_link_tri_" + paramValue);
            endSpan();
        } else {
            String href = getString + "&amp;page=" + DefaultCommand.PONDERATION_PAGE_PARAMVALUE + "&amp;" + DefaultCommand.TRI_PARAMNAME + "=" + paramValue;
            appendA(href, "thesaurus_link_tri_" + paramValue);
        }
    }

    private void printCorpusTri() {
        int corpusCount = basedefiches.getCorpusCount();
        for (int k = 0; k < corpusCount; k++) {
            Corpus corpus = basedefiches.getCorpus(k);
            if (isSlaveCorpus(corpus)) {
                continue;
            }
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurusKey);
            CroisementItem croisementItem = CroisementUtils.sortByPoids(croisement.getCroisementItem(thesaurusKey, motcle.getTicket()));
            int length = croisementItem.getCount();
            if (length > 0) {
                startH2();
                escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
                endH2();
                startTable();
                if (asc) {
                    for (int i = 0; i < length; i++) {
                        AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(croisementItem.getTicket(i));
                        printRow(abregeFiche, croisementItem.getPoids(i));
                    }
                } else {
                    for (int i = length - 1; i >= 0; i--) {
                        AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(croisementItem.getTicket(i));
                        printRow(abregeFiche, croisementItem.getPoids(i));
                    }
                }
                endTable();
            }

        }
    }

    private void printAllTri() {
        ArrayList ponderationKeyList = new ArrayList();
        int corpusCount = basedefiches.getCorpusCount();
        for (int k = 0; k < corpusCount; k++) {
            Corpus corpus = basedefiches.getCorpus(k);
            if (isSlaveCorpus(corpus)) {
                continue;
            }
            Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurusKey);
            CroisementItem croisementItem = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
            int length = croisementItem.getCount();
            for (int i = 0; i < length; i++) {
                int poids = croisementItem.getPoids(i);
                int ticket = croisementItem.getTicket(i);
                ponderationKeyList.add(new PonderationKey(corpus, i, ticket, poids));
            }
        }
        Collections.sort(ponderationKeyList);
        int size = ponderationKeyList.size();
        startTable();
        if (asc) {
            for (int i = 0; i < size; i++) {
                PonderationKey ponderationKey = (PonderationKey) ponderationKeyList.get(i);
                printPonderationRow(ponderationKey);
            }
        } else {
            for (int i = size - 1; i >= 0; i--) {
                PonderationKey ponderationKey = (PonderationKey) ponderationKeyList.get(i);
                printPonderationRow(ponderationKey);
            }
        }
        endTable();
    }

    private void printPonderationRow(PonderationKey ponderationKey) {
        AbregeFiche abregeFiche = ponderationKey.getAbregeFiche();
        printRow(abregeFiche, ponderationKey.getPoids());
    }

    private void printRow(AbregeFiche abregeFiche, int poids) {
        String value = String.valueOf(poids);
        startRow();
        startCell();
        appendSmallestInput(abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "_" + String.valueOf(abregeFiche.getIdcorpus()), value);
        endCell();
        startCell();
        startA(plusAttributes);
        append("+");
        endA();
        startA(moinsAttributes);
        append("-");
        endA();
        endCell();
        startCell();
        startCode();
        escape(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
        endCode();
        endCell();
        endRow();
    }


    private class PonderationKey implements Comparable {

        private int poids;
        private Corpus corpus;
        private int ticket;
        private int position;

        PonderationKey(Corpus corpus, int position, int ticket, int poids) {
            this.corpus = corpus;
            this.ticket = ticket;
            this.poids = poids;
            this.position = position;
        }

        AbregeFiche getAbregeFiche() {
            return (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
        }

        int getPoids() {
            return poids;
        }

        public int compareTo(Object obj) {
            PonderationKey other = (PonderationKey) obj;
            if (this.poids < other.poids) {
                return -1;
            }
            if (this.poids > other.poids) {
                return 1;
            }
            int result = this.corpus.getSubsetKey().compareTo(other.corpus.getSubsetKey());
            if (result != 0) {
                return result;
            }
            if (this.position < other.position) {
                return -1;
            }
            if (this.position > other.position) {
                return 1;
            }
            if (this.ticket < other.ticket) {
                return -1;
            }
            if (this.ticket > other.ticket) {
                return 1;
            }
            return 0;
        }

    }

}
