/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.htmlproducers.saisie.SaisieHtmlConstants;
import fr.exemole.bdfserver.response.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.thesaurus.metadata.*;
import fr.exemole.bdfserver.servlets.*;
import fr.exemole.bdfserver.managers.policies.*;
import fr.exemole.bdfserver.servlets.resources.*;
import fr.exemole.bdfserver.servlets.thesaurus.*;
import fr.exemole.bdfserver.request.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.corpus.*;



/**
 *
 * @author Vincent Calame
 */
public abstract class ThesaurusHtmlPage extends BdfHtmlPage {

    public static final String ACTION = "Thesaurus";
    protected short pageType;
    protected Thesaurus thesaurus;
    protected ThesaurusMetadata thesaurusMetadata;
    protected Motcle motcle;
    protected int thesaurusLangInteger;
    protected String getString;
    protected boolean majLink;
    protected ThesaurusResponseParameters responseParameters;

    public ThesaurusHtmlPage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
        this.thesaurusLangInteger = responseParameters.getThesaurusLangInteger();
        this.majLink = responseParameters.isMajLinkNeeded();
        this.pageType = responseParameters.getResponseCode();
        this.responseParameters = responseParameters;
        if (responseParameters.getMotcle() != null) {
            initMotcle(responseParameters.getMotcle());
        } else {
            initThesaurus(responseParameters.getThesaurus());
        }
    }

    private void initThesaurus(Thesaurus thesaurus) {
        this.thesaurus = thesaurus;
        if (thesaurus != null) {
            thesaurusMetadata = thesaurus.getThesaurusMetadata();
            getString = "Thesaurus?" + RequestUtils.toParameterString(thesaurus);
        }
    }

    /**
     * Fait appelle automatiquement setThesaurus(motcle.getThesaurus())
     */
    private void initMotcle(Motcle motcle) {
        initThesaurus(motcle.getThesaurus());
        this.motcle = motcle;
        getString = "Thesaurus?" + RequestUtils.toParameterString(motcle);
    }

    /**
     * Crèe une instance de HtmlThesaurusPage en se basant sur la valeur de responseParameters.getPageType().
     */
    public static ResponseHandler getInstance(ThesaurusResponseParameters responseParameters) {
        switch (responseParameters.getResponseCode()) {
            case BdfResponseParameters.MESSAGE_SERVLETPAGE:
                return new MessagePage(responseParameters);
            case ThesaurusResponseParameters.MOTCLE_PAGE:
                return new MotclePage(responseParameters);
            case ThesaurusResponseParameters.ADVANCED_PAGE:
                return new AdvancedPage(responseParameters);
            case ThesaurusResponseParameters.NVIDTHS_PAGE:
                return new NvIdthsPage(responseParameters);
            case ThesaurusResponseParameters.NVMTCLBYIDALPHA_PAGE:
                return new NvMtclByIdalphaPage(responseParameters);
            case ThesaurusResponseParameters.MOVECONFIRMATION_PAGE:
                return new MoveConfirmationPage(responseParameters);
            case ThesaurusResponseParameters.MERGECONFIRMATION_PAGE:
                return new MergeConfirmationPage(responseParameters);
            case ThesaurusResponseParameters.COMPLETETRANSFERCONFIRMATION_PAGE:
                return new CompleteTransferConfirmationPage(responseParameters);
            case ThesaurusResponseParameters.CLEANINGCONFIRMATION_PAGE:
                return new CleaningConfirmationPage(responseParameters);
            case ThesaurusResponseParameters.MOTCLEINDEXATION_PAGE:
                return new MotcleIndexationPage(responseParameters);
            case ThesaurusResponseParameters.THESAURUSCREATIONFORM_PAGE:
                return new ThesaurusCreationFormPage(responseParameters);
            case ThesaurusResponseParameters.THESAURUSFORM_PAGE:
                return new ThesaurusFormPage(responseParameters);
            case ThesaurusResponseParameters.PONDERATION_PAGE:
                return new PonderationPage(responseParameters);
            case ThesaurusResponseParameters.IDALPHASORT_PAGE:
                return new IdalphaSortPage(responseParameters);
            case ThesaurusResponseParameters.COMPARAISON_PAGE:
                return new ComparaisonPage(responseParameters);
            case ThesaurusResponseParameters.RECOUVREMENT_PAGE:
                return new RecouvrementPage(responseParameters);
            case ThesaurusResponseParameters.THESAURUS_PAGE:
                return new ThesaurusPage(responseParameters);
        }
        return null;
    }

    /*
     * Écrit le début de la page avec les liens, le libellé du mot-clé, etc.
     * Appelé par les différentes versions de printPage()
     */
    protected void printTitre() {
        startP(CssClass.P_TOOLBARLINKS);
        String pageActu = DefaultCommand.getPageTypeString(pageType);
        printLink("", "thesaurus_link_motcle", pageActu);
        printTiret();
        printLink(DefaultCommand.ADVANCED_PAGE_PARAMVALUE, "thesaurus_link_options", pageActu);
        printTiret();
        printLink(DefaultCommand.MOTCLEINDEXATION_PAGE_PARAMVALUE, "thesaurus_link_indexation", pageActu);
        printTiret();
        printLink(DefaultCommand.PONDERATION_PAGE_PARAMVALUE, "thesaurus_link_ponderation", pageActu);
        printTiret();
        String indexationHref = "Saisie?page=" + SaisieHtmlConstants.SELECTIONINDEXATION_FORM_PAGE + "&amp;" + RequestUtils.toParameterString(motcle);
        appendA(indexationHref, "thesaurus_link_masse");
        endP();
        startH1();
        printMotcleLibelle(motcle, thesaurusLangInteger);
        endH1();
        printMessage();
    }

    protected void printLink(String page, String l10nKey, String pageActu) {
        if (pageActu.equals(page)) {
            startSpan(CssClass.SPAN_CURRENTPAGE);
            localize(l10nKey);
            endSpan();
        } else {
            String href = getString;
            if (page.length() > 0) {
                href = href + "&amp;page=" + page;
            }
            appendA(href, l10nKey);
        }
    }

    /*
     * Écrit un message d'erreur ou d'info. Appelé par la méthode printTitre() ou directement dans certaines pages.
     */
    protected void printMessage() {
        if (message == null) {
            return;
        }
        super.printMessage();
        if (majLink) {
            startP();
            if (thesaurus != null) {
                printTiret();
                String href = "motscles/" + thesaurus.getSubsetKey().getSubsetName() + ".html";
                appendA(href, "thesaurus_link_thesaurusmaj", "Liste");
            } else {
                printTiret();
                appendA("Affichage?page=recensements", "global_link_recensements_maj", "Liste");
            }
            endP();
        }

    }

    protected void startThesaurusForm(String command) {
        startThesaurusForm(command, true);
    }

    protected void startThesaurusForm(String command, boolean withHr) {
        if (withHr) {
            appendHr();
        }
        if (motcle != null) {
            startForm(motcle, command);
        } else if (thesaurus != null) {
            startPostForm(thesaurus, command);
        }
    }

    protected SubsetKey getPreferredThesaurusKey() {
        ThesaurusPolicy thesaurusPolicy = bdfServer.getPolicyManager().getThesaurusPolicy(thesaurus.getSubsetKey());
        if (thesaurusPolicy.getAjoutPolicy() == ThesaurusPolicy.AJOUT_VERIFICATION) {
            return (SubsetKey) thesaurusPolicy.getVerifThesaurusKeyList().get(0);
        } else {
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            if (basedefiches.getThesaurusCount() == 1) {
                return thesaurus.getSubsetKey();
            }
            Thesaurus first = basedefiches.getThesaurus(0);
            if ((first.equals(thesaurus))) {
                first = basedefiches.getThesaurus(1);
            }
            return first.getSubsetKey();
        }
    }

    protected void printJavascriptPiocheFusion() {
        pw.println("<script>");
        pw.println("function pioche() {");
        pw.println("var slc = document.getElementById(\"thsfusion\");");
        pw.println("var option = slc.options[slc.selectedIndex];");
        pw.println("var thesaurus = option.value;");
        pw.print("var url = \"Pioche?category=thesaurus&");
        pw.print(RequestUtils.THESAURUS_PARAMNAME);
        pw.print("=\" + thesaurus + \"&appelant=inputfusion&idths=1&limite=1\";");
        pw.println("var w = window.open(url,\"pioche_\"+thesaurus,\"resizable=yes,width=430,height=470\");");
        pw.println("w.focus()}");
        pw.println("</script>");
    }

    public String getTitle() {
        return "Thesaurus";
    }

    public String getFormAction() {
        return "Thesaurus";
    }

    public String getCssThemeFile() {
        return "thesaurus.css";
    }

    public boolean isSlaveCorpus(Corpus corpus) {
        Subset masterSubset = corpus.getMasterSubset();
        if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
            return true;
        }
        return false;
    }

}
