/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.thesaurus.htmlpages;

import fr.exemole.bdfserver.get.instructions.pioche.PiocheConstants;
import fr.exemole.bdfserver.response.ThesaurusResponseParameters;
import fr.exemole.bdfserver.servlets.*;
import java.io.*;
import javax.servlet.http.*;
import fr.exemole.bdfserver.request.RequestUtils;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.*;
import fr.exemole.bdfserver.servlets.thesaurus.*;
import net.mapeadores.util.html.*;
import fr.exemole.bdfserver.servlets.resources.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.basedefiches.selection.eligibility.*;
import fr.exemole.bdfserver.servlets.util.*;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusPage extends ThesaurusHtmlPage {

    int thesaurusLangInteger;
    String eligibility;
    MotcleEligibility motcleEligibility;
    int eligibleCount = 0;

    public ThesaurusPage(ThesaurusResponseParameters responseParameters) {
        super(responseParameters);
        addJavascript(BDF_JS);
        addJavascript(APPELANT_PIOCHE_JS);
        addJavascript(THESAURUS_PAGE_JS);
        Integer itg = (Integer) responseParameters.getParameter(DefaultCommand.LIBLANG_PARAMNAME);
        if (itg == null) {
            thesaurusLangInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, workingLangInteger);
        } else {
            thesaurusLangInteger = itg.intValue();
        }
        initEligibility((String) responseParameters.getParameter(DefaultCommand.FILTRE_PARAMNAME));

    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        String intitule = ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger);
        String thesaurusParameter = RequestUtils.toParameterString(thesaurus);
        SubsetKey thesaurusKey = thesaurus.getSubsetKey();
        String thesaurusName = thesaurus.getSubsetKey().getSubsetName();
        start();
        if (isWithJQuery()) {
            HTMLAttributes divSubsetSelectAttributes = new HTMLAttributes();
            divSubsetSelectAttributes.addClass(DIV_GLB_SUBSETSELECT);
            startDiv(divSubsetSelectAttributes);
            BdfHtmlUtils.printChangeSubsetSelect(this, thesaurus, null);
            endDiv();
        }
        startP(CssClass.P_SMALLLINKS);
        BdfHtmlUtils.printOds(this, "motscles/" + thesaurusName + "-table");
        printTiret();
        StringBuffer buf = new StringBuffer();
        buf.append("Exportation?page=");
        buf.append(fr.exemole.bdfserver.servlets.exportation.DefaultCommand.THESAURUSFORM_PAGE_PARAMVALUE);
        buf.append("&amp;");
        buf.append(RequestUtils.THESAURUS_PARAMNAME);
        buf.append("=");
        buf.append(thesaurusKey.getSubsetName());
        startA(buf.toString(), "Saisie");
        append("export");
        endA();
        endP();
        BdfHtmlUtils.printDeployTitle(this, thesaurusName, "global_link_statistiques");
        BdfHtmlUtils.printThesaurusLinksDeployBlock(this, thesaurusName, "motscles/");
        if (isAdmin()) {
            AppelantWriter appelantWriter = new AppelantWriter(bdfServer.getBasedefiches(), this, workingLangInteger);
            startP(CssClass.P_SMALLLINKS);
            String creationHref = "Thesaurus?" + thesaurusParameter + "&amp;page=" + DefaultCommand.MOTCLECREATIONFORM_PAGE_PARAMVALUE;
            appendA(creationHref, "thesaurus_link_newmotcle", "Saisie");
            printTiret();
            printThesaurusLink(DefaultCommand.FORM_PAGE_PARAMVALUE, "thesaurus_title_thesaurusmanagement");
            printTiret();
            printThesaurusLink(null, "global_link_maj");
            appendBr();
            startGetForm(thesaurus, null, "Saisie");
            String appelantId = "piocheur";
            HTMLAttributes txtAtt = new HTMLAttributes();
            txtAtt.setSize("10");
            txtAtt.setName("idmtcl");
            txtAtt.setId(appelantId);
            appendTextInput(txtAtt);

            PiocheParameters piocheParameters = new PiocheParameters();
            piocheParameters.setAppelantId(appelantId);
            piocheParameters.setLimite(1);
            piocheParameters.setSubset(thesaurus);
            piocheParameters.setParam(PiocheConstants.IDTHS_PARAMVALUE);
            appelantWriter.printPiocheur(piocheParameters);
            addSubmit("global_submit_go", false);
            appendBr();
            startSmall();
            HTMLAttributes chAtt = new HTMLAttributes();
            chAtt.setName("page");
            chAtt.setValue(DefaultCommand.ADVANCED_PAGE_PARAMVALUE);
            if (isWithJQuery()) {
                chAtt.setOnClick("setPage(this,'" + DefaultCommand.ADVANCED_PAGE_PARAMVALUE + "');");
            }
            addCheck(chAtt, "thesaurus_link_options", null);
            endSmall();
            endForm();
            endP();
        }

        if (!ThesaurusUtils.isBabelienType(thesaurus)) {
            printFilterLinks();
        }

        startH1();
        print16pxStaticIcon("global_subset_thesaurus", "..");
        space();
        escape(intitule);
        if (motcleEligibility == null) {
            append(" [");
            append(String.valueOf(thesaurus.size()));
            append("]");
        }
        endH1();
        MotcleList firstLevel = thesaurus.getFirstLevelList();
        int size = firstLevel.getMotcleCount();
        if (size > 0) {
            startUl();
            for (int i = 0; i < size; i++) {
                printMotcle(firstLevel.getMotcle(i));
            }
            endUl();
        }
        if (motcleEligibility != null) {
            startP();
            printLib("thesaurus_lib_filterresult");
            append(String.valueOf(eligibleCount));
            append(" / ");
            append(String.valueOf(thesaurus.size()));
            endP();
        }
        end();
    }

    private void printThesaurusLink(String pageName, String titleKey) {
        String href = "Thesaurus?" + RequestUtils.toParameterString(thesaurus);
        if (pageName != null) {
            href = href + "&amp;page=" + pageName;
        }
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        if (pageName != null) {
            htmlAttributes.setTarget("Saisie");
        }
        htmlAttributes.setHref(href);
        startA(htmlAttributes);
        localize(titleKey);
        endA();
    }

    private void initEligibility(String elig) {
        if (elig == null) {
            eligibility = "";
            return;
        }
        if (elig.equals(DefaultCommand.NOLIB_OPTION)) {
            eligibility = elig;
            motcleEligibility = new NoLibEligibility(thesaurusLangInteger);
        } else if (elig.equals(DefaultCommand.LIB_OPTION)) {
            eligibility = elig;
            motcleEligibility = new LibEligibility(thesaurusLangInteger);
        } else {
            eligibility = "";
        }
    }

    private void printFilterLinks() {
        String hrefStart = "Thesaurus?" + RequestUtils.toParameterString(thesaurus) + "&amp;" + DefaultCommand.LIBLANG_PARAMNAME + "=";
        int[] langArray = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
        int length = langArray.length;
        if (length > 1) {
            startP(CssClass.P_SMALLLINKS);
            printLib("global_lib_langlink");
            for (int i = 0; i < length; i++) {
                int lgInt = langArray[i];
                if (i > 0) {
                    printTiret();
                }
                String lg = LangInteger.toString(lgInt);
                if (lgInt == thesaurusLangInteger) {
                    startSpan(CssClass.SPAN_CURRENTPAGE);
                    append(lg);
                    endSpan();
                } else {
                    startA(hrefStart + lg);
                    append(lg);
                    endA();
                }
            }
            endP();
        }
        hrefStart = hrefStart + LangInteger.toString(thesaurusLangInteger);
        startP(CssClass.P_SMALLLINKS);
        printLib("global_lib_filter");
        testLink(hrefStart, "", "global_lib_filter_none");
        printTiret();
        testLink(hrefStart, DefaultCommand.LIB_OPTION, "thesaurus_link_filter_lib");
        printTiret();
        testLink(hrefStart, DefaultCommand.NOLIB_OPTION, "thesaurus_link_filter_nolib");
        endP();
    }

    private void testLink(String hrefStart, String key, String locKey) {
        if (key.equals(eligibility)) {
            startSpan(CssClass.SPAN_CURRENTPAGE);
            localize(locKey);
            endSpan();
        } else {
            String href = hrefStart;
            if (key.length() != 0) {
                href = href + "&amp;" + DefaultCommand.FILTRE_PARAMNAME + "=" + key;
            }
            startA(href);
            localize(locKey);
            endA();
        }
    }

    private void printMotcle(Motcle motcle) {
        boolean withHierachy = (motcleEligibility == null);
        boolean accepted = true;
        if (motcleEligibility != null) {
            accepted = motcleEligibility.accept(motcle);
            if (accepted) {
                eligibleCount++;
            }
        }
        if (accepted) {
            startLi();
            startP();
            if (isAdmin()) {
                HTMLAttributes aAttributes = new HTMLAttributes();
                aAttributes.setHref("Thesaurus?" + RequestUtils.toParameterString(motcle));
                aAttributes.setTarget("Saisie");
                startA(aAttributes);
                printMotcleLibelle(motcle, thesaurusLangInteger);
                endA();
            } else {
                printMotcleLibelle(motcle, thesaurusLangInteger);
            }
            endP();
        }
        MotcleList childList = motcle.getChildList();
        int size = childList.getMotcleCount();
        if (size > 0) {
            if (withHierachy) {
                startUl();
            }
            for (int i = 0; i < size; i++) {
                printMotcle(childList.getMotcle(i));
            }
            if (withHierachy) {
                endUl();
            }
        }
        if (accepted) {
            endLi();
        }
    }

}
