/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.util;

import javax.servlet.http.*;
import java.io.*;
import fr.exemole.bdfserver.servlets.resources.CssClass;
import net.mapeadores.basedefiches.extraction.*;
import net.mapeadores.basedefiches.selection.*;
import net.mapeadores.util.html.*;
import net.mapeadores.basedefiches.*;
import java.util.*;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.basedefiches.thesaurus.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.corpus.metadata.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.conditions.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class AppelantWriter {

    private HtmlPrinter hp;
    private Basedefiches basedefiches;
    private int workingLangInteger;

    public AppelantWriter(Basedefiches basedefiches, HtmlPrinter htmlPrinter, int workingLangInteger) {
        this.hp = htmlPrinter;
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
    }

    public void printPiocheur(PiocheParameters piocheParameters) {
        String javascriptFunction = piocheParameters.getJavascriptFunction();
        if (javascriptFunction == null) {
            javascriptFunction = "pioche";
        }
        String path = piocheParameters.getPath();
        if (path == null) {
            path = "";
        } else {
            path = path + "/";
        }
        List subsetList = piocheParameters.getSubsetList();
        StringBuffer buf = new StringBuffer();
        buf.append("javascript:");
        buf.append(javascriptFunction);
        buf.append("('");
        buf.append("motcle");
        buf.append("','");
        buf.append(piocheParameters.getAppelantId());
        buf.append("','");
        buf.append(getSubsetsString(subsetList));
        buf.append("',");
        buf.append(piocheParameters.getLimite());
        buf.append(",'");
        buf.append(path);
        buf.append("'");
        Object param = piocheParameters.getParam();
        if (param != null) {
            buf.append(",'");
            buf.append(param.toString());
            buf.append("'");
        }
        buf.append(")");
        HTMLAttributes aAtt = new HTMLAttributes();
        aAtt.setHref(buf.toString());
        aAtt.setId(piocheParameters.getLinkId());
        hp.startA(aAtt);
        String rscKey = getRscKey(piocheParameters);
        hp.appendImg(path + "theme/icons/32x16/" + rscKey + ".png", "global_link_" + rscKey, getValue(subsetList), 32, 16, "Icon");
        hp.endA();
    }

    public static String getSubsetsString(List subsetList) {
        if (subsetList == null) {
            return "";
        }
        StringBuffer buf = new StringBuffer();
        int size = subsetList.size();
        for (int i = 0; i < size; i++) {
            Subset subset = (Subset) subsetList.get(i);
            if (i > 0) {
                buf.append(",");
            }
            buf.append(subset.getSubsetKey().getSubsetName());
        }
        return buf.toString();
    }

    private String getRscKey(PiocheParameters piocheParameters) {
        List subsetList = piocheParameters.getSubsetList();
        if (subsetList == null) {
                return "pioche_motcle_all";
            } else if (subsetList.size() > 1) {
                return "pioche_motcle_selection";
            } else {
                return "pioche_motcle_one";
            }
    }

    private String getValue(List subsetList) {
        if (subsetList == null) {
            return null;
        }
        if (subsetList.size() > 1) {
            return null;
        }
        Subset subset = (Subset) subsetList.get(0);
        if (subset instanceof Corpus) {
            return CorpusMetadataUtils.getCorpusIntituleString((Corpus) subset, workingLangInteger);
        }
        if (subset instanceof Thesaurus) {
            return ThesaurusUtils.getThesaurusIntitule((Thesaurus) subset, workingLangInteger);
        }
        return null;
    }

}
