/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.servlets.util;

import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.beans.*;
import net.mapeadores.basedefiches.tableexport.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class TableExportParameters {

    public final static String TABLEEXPORTNAME_PARAMNAME = "tableexportname";
    public final static String HEADERTYPE_PARAMNAME = "headertype";
    public final static String WITHTEXTES_PARAMNAME = "withtextes";
    public final static String WITHCHRONO_PARAMNAME = "withchrono";
    public final static String WITHLIAGE_PARAMNAME = "withliage";
    public final static String WITHASSOCIATION_PARAMNAME = "withassociation";
    public final static String INDEXATIONINCLUDETYPE_PARAMNAME = "indexationincludetype";

    private TableExportParameters() {
    }

    public static TableExportParametersBean getTableExportParametersBean(RequestMap requestMap, boolean mandatoryName, boolean withHeader) throws ParameterException {
        TableExportParametersBean tableExportParametersBean = new TableExportParametersBean();
        if (mandatoryName) {
            String name = requestMap.getParameter(TABLEEXPORTNAME_PARAMNAME);
            if (name == null) {
                throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, TABLEEXPORTNAME_PARAMNAME);
            }
            tableExportParametersBean.setTableExportName(name.trim());
        }
        tableExportParametersBean.setWithTextes(getBoolean(requestMap, WITHTEXTES_PARAMNAME));
        tableExportParametersBean.setWithChrono(getBoolean(requestMap, WITHCHRONO_PARAMNAME));
        tableExportParametersBean.setWithAssociation(getBoolean(requestMap, WITHASSOCIATION_PARAMNAME));
        tableExportParametersBean.setWithLiage(getBoolean(requestMap, WITHLIAGE_PARAMNAME));
        String iitValue = requestMap.getParameter(INDEXATIONINCLUDETYPE_PARAMNAME);
        if (iitValue == null) {
            throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, INDEXATIONINCLUDETYPE_PARAMNAME);
        }
        try {
            short indexationIncludeType = TableExportCommon.indexationIncludeTypeToShort(iitValue);
            tableExportParametersBean.setIndexationIncludeType(indexationIncludeType);
        } catch (IllegalArgumentException iae) {
            throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, INDEXATIONINCLUDETYPE_PARAMNAME + " = " + iitValue);
        }

        if (withHeader) {
            String headerTypeValue = requestMap.getParameter(HEADERTYPE_PARAMNAME);
            if (headerTypeValue == null) {
                throw new ParameterException(ParameterException.PARAMETER_UNDEFINED, HEADERTYPE_PARAMNAME);
            }
            try {
                short headerType = TableExportCommon.headerTypeToShort(headerTypeValue);
                tableExportParametersBean.setHeaderType(headerType);
            } catch (IllegalArgumentException iae) {
                throw new ParameterException(ParameterException.PARAMETER_WRONGVALUE, HEADERTYPE_PARAMNAME + " = " + headerTypeValue);
            }
        }
        return tableExportParametersBean;
    }

    private static boolean getBoolean(RequestMap requestMap, String paramName) {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            return false;
        }
        if (paramValue.equals("1")) {
            return true;
        }
        return false;
    }


    public static class Printer {

        TableExportParametersBean tableExportParametersBean;
        HtmlPrinter htmlPrinter;
        short indexationIncludeType;

        public Printer(TableExportParametersBean tableExportParametersBean, HtmlPrinter htmlPrinter) {
            this.tableExportParametersBean = tableExportParametersBean;
            this.htmlPrinter = htmlPrinter;
            indexationIncludeType = tableExportParametersBean.getIndexationIncludeType();
        }

        public void printBooleanParameters() {
            printCheckbox(TableExportParameters.WITHTEXTES_PARAMNAME, tableExportParametersBean.isWithTextes());
            printCheckbox(TableExportParameters.WITHCHRONO_PARAMNAME, tableExportParametersBean.isWithChrono());
            printCheckbox(TableExportParameters.WITHASSOCIATION_PARAMNAME, tableExportParametersBean.isWithAssociation());
            printCheckbox(TableExportParameters.WITHLIAGE_PARAMNAME, tableExportParametersBean.isWithLiage());
        }

        public void printIndexationRadios() {
            printRadio(TableExportCommon.ONLY_UI_INDEXATION);
            printRadio(TableExportCommon.ALL_INDEXATION);
            printRadio(TableExportCommon.NO_INDEXATION);
        }

        public void printHeaderTypeRadios() {
            printHeaderTypeRadio(TableExportCommon.LIB_HEADER);
            printHeaderTypeRadio(TableExportCommon.KEY_HEADER);
            printHeaderTypeRadio(TableExportCommon.NO_HEADER);
        }

        void printCheckbox(String paramName, boolean checked) {
            HTMLAttributes att = new HTMLAttributes();
            att.setName(paramName);
            att.setValue("1");
            att.setChecked(checked);
            htmlPrinter.addCheck(att, "exportation_lib_" + paramName, null);
            htmlPrinter.appendBr();
        }

        void printHeaderTypeRadio(short headerType) {
            String value = TableExportCommon.headerTypeToString(headerType);
            HTMLAttributes radioAttributes = new HTMLAttributes();
            radioAttributes.setName(TableExportParameters.HEADERTYPE_PARAMNAME);
            radioAttributes.setChecked((headerType == tableExportParametersBean.getHeaderType()));
            radioAttributes.setValue(value);
            htmlPrinter.addRadio(radioAttributes, "exportation_lib_headertype_" + value, null);
            htmlPrinter.appendBr();
        }

        void printRadio(short type) {
            boolean checked = (type == indexationIncludeType);
            String value = TableExportCommon.indexationIncludeTypeToString(type);
            HTMLAttributes radioAttributes = new HTMLAttributes();
            radioAttributes.setChecked(checked);
            radioAttributes.setValue(value);
            radioAttributes.setName(TableExportParameters.INDEXATIONINCLUDETYPE_PARAMNAME);
            htmlPrinter.addRadio(radioAttributes, "exportation_lib_indexation_" + value, null);
            htmlPrinter.appendBr();
        }

    }

}
