/* BasedefichesLib_API - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.beans;

import java.io.Serializable;
import java.text.ParseException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class CourrielBean implements Serializable {

    private static final long serialVersionUID = 1L;
    private String addrSpec = "localhost@localdomain";
    private String nomComplet = "";
    final static short VALIDE_SPEC = 2;
    final static short HORS_SPEC = 3;
    final static short LT = 4;
    final static short GT = 5;

    public CourrielBean() {
    }

    public CourrielBean(CourrielBean courrielBean) {
        this.addrSpec = courrielBean.getAddrSpec();
        this.nomComplet = courrielBean.getNomComplet();
    }

    public String getAddrSpec() {
        return addrSpec;
    }

    /**
     *
     *@throws IllegalArgumentException si l'adresse est incorrecte (ou null)
     */
    public void setAddrSpec(String addrSpec) {
        if (addrSpec == null) {
            throw new IllegalArgumentException("addrSpec cannot be null");
        }
        try {
            this.addrSpec = checkAddrspec(addrSpec, 0);
        } catch (ParseException pe) {
            throw new IllegalArgumentException("illegal addrSpec = " + addrSpec + " ( " + pe.getMessage() + ")");
        }
    }

    public String getNomComplet() {
        return nomComplet;
    }

    public void setNomComplet(String nomComplet) {
        if (nomComplet == null) {
            throw new IllegalArgumentException("NomComplet cannot be null");
        }
        this.nomComplet = StringUtils.cleanString(nomComplet);
    }

    @Override
    public String toString() {
        if (nomComplet.length() == 0) {
            return addrSpec;
        }
        StringBuffer buf = new StringBuffer();
        boolean needQuotes = false;
        boolean whitespace = false;
        for (int i = 0; i < nomComplet.length(); i++) {
            char carac = nomComplet.charAt(i);
            if (Character.isWhitespace(carac)) {
                whitespace = true;
            }
            if (Character.isISOControl(carac)) {
                continue;
            }
            if (whitespace) {
                whitespace = false;
                buf.append(' ');
            }
            if ((carac == '\"') || (carac == '\\')) {
                needQuotes = true;
                buf.append('\\');
            } else if ((carac == '>') || (carac == '<')) {
                needQuotes = true;
            }
            buf.append(carac);
        }
        if (needQuotes) {
            buf.insert(0, '"');
            buf.append('"');
        }
        buf.append(" <");
        buf.append(addrSpec);
        buf.append(">");
        return buf.toString();
    }

    static public CourrielBean parse(String s) throws ParseException {
        if (s == null) {
            throw new ParseException("s is null", 0);
        }
        s = StringUtils.cleanString(s);
        if (s.length() == 0) {
            throw new ParseException("s length is null", 0);
        }
        if (s.charAt(0) == '\"') {
            return parseWithQuote(s);
        }
        return parseWithoutQuote(s);

    }

    static private CourrielBean parseWithQuote(String chaine) throws ParseException {
        StringBuffer nomCompletBuffer = new StringBuffer();
        int departlt = 0;
        int taillechaine = chaine.length();
        boolean whitespace = false;
        for (int i = 1; i < taillechaine; i++) {
            char carac = chaine.charAt(i);
            if (Character.isWhitespace(carac)) {
                whitespace = true;
            } else if (carac == '\\') {
                if (i == taillechaine - 1) {
                    continue;
                } else {
                    if (whitespace) {
                        whitespace = false;
                        nomCompletBuffer.append(' ');
                    }
                    nomCompletBuffer.append(chaine.charAt(i + 1));
                    i++;
                }
            } else if (carac == '\"') {
                departlt = i + 1;
                break;
            } else {
                if (whitespace) {
                    whitespace = false;
                    nomCompletBuffer.append(' ');
                }
                nomCompletBuffer.append(carac);
            }
        }
        if (departlt == 0) {
            throw new ParseException("no closing quote", chaine.length() - 1);
        }
        int departaddrspec = -1;
        for (int i = departlt; i < taillechaine; i++) {
            char carac = chaine.charAt(i);
            if (carac == '<') {
                departaddrspec = i + 1;
            }
        }
        if (departaddrspec == -1) {
            throw new ParseException("no begining <", chaine.length() - 1);
        }
        CourrielBean courrielBean = new CourrielBean();
        courrielBean.nomComplet = nomCompletBuffer.toString();
        courrielBean.addrSpec = checkAddrspec(chaine, departaddrspec);
        return courrielBean;
    }

    static private CourrielBean parseWithoutQuote(String chaine) throws ParseException {
        StringBuffer buf = new StringBuffer();
        boolean avecArobase = false;
        boolean valide = true;
        int departaddrspec = chaine.indexOf('<') + 1;
        CourrielBean courrielBean = new CourrielBean();
        courrielBean.addrSpec = checkAddrspec(chaine, departaddrspec);
        if (departaddrspec > 0) {
            courrielBean.nomComplet = chaine.substring(0, departaddrspec - 1).trim();
        } else {
            courrielBean.nomComplet = "";
        }
        return courrielBean;
    }

    static private String checkAddrspec(String chaine, int depart) throws ParseException {
        StringBuffer bufaddrspec = new StringBuffer();
        int taillechaine = chaine.length();
        boolean avecarobase = false;
        for (int i = depart; i < taillechaine; i++) {
            char carac = chaine.charAt(i);
            if (Character.isISOControl(carac)) {
                continue;
            }
            if (Character.isSpaceChar(carac)) {
                continue;
            }
            if (carac == '@') {
                if (avecarobase) {
                    throw new ParseException("invalid @ character", i);
                } else {
                    if (bufaddrspec.length() == 0) {
                        throw new ParseException(chaine.substring(depart) + " starts with @ character", 0);
                    }
                    avecarobase = true;
                    bufaddrspec.append('@');
                    continue;
                }
            }
            short test = testeChar(carac);
            if (test == VALIDE_SPEC) {
                bufaddrspec.append(carac);
            } else if (test == GT) {
                if (avecarobase) {
                    return bufaddrspec.toString();
                } else {
                    throw new ParseException("closing > character without @", i);
                }
            } else {
                throw new ParseException("invalid character", i);
            }
        }
        if (avecarobase) {
            return bufaddrspec.toString();
        } else {
            throw new ParseException("missing @ character", chaine.length() - 1);
        }
    }

    static private short testeChar(char c) {
        if (c == '<') {
            return LT;
        }
        if (c == '>') {
            return GT;
        }
        if (c > '~') {
            return HORS_SPEC;
        }
        if (c == '.') {
            return VALIDE_SPEC;
        }

        if ((c >= '0') && (c <= '9')) {
            return VALIDE_SPEC;
        }
        if ((c >= 'A') && (c <= 'Z')) {
            return VALIDE_SPEC;
        }
        if ((c >= '^') && (c <= '~')) {
            return VALIDE_SPEC;
        }
        switch (c) {
            case '!':
            case '#':
            case '$':
            case '%':
            case '&':
            case '\'':
            case '*':
            case '+':
            case '-':
            case '/':
            case '=':
            case '?':
                return VALIDE_SPEC;
            default:
                return HORS_SPEC;
        }
    }

    @Override
    public int hashCode() {
        return addrSpec.hashCode() + nomComplet.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof CourrielBean)) {
            return false;
        }
        CourrielBean other = (CourrielBean) obj;
        return ((other.addrSpec.equals(this.addrSpec)) && (other.nomComplet.equals(this.nomComplet)));
    }

}
