/* BasedefichesLib_API - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import net.mapeadores.basedefiches.beans.CourrielBean;
import java.io.Serializable;


/**
 * Un courriel est composé d'une adresse et d'un nom complet.
 *
 * @author Vincent Calame
 */
public final class Courriel implements FicheItem, Serializable {

    private static final long serialVersionUID = 2L;
    private CourrielBean courrielBean;

    public Courriel(CourrielBean courrielBean) {
        this.courrielBean = new CourrielBean(courrielBean);
    }

    public Courriel(Courriel courriel, String nomComplet) {
        this.courrielBean = new CourrielBean(courriel.courrielBean);
        courrielBean.setNomComplet(nomComplet);
    }

    public String getAddrSpec() {
        return courrielBean.getAddrSpec();
    }

    public String getNomComplet() {
        return courrielBean.getNomComplet();
    }

    public boolean hasNomComplet() {
        return (getNomComplet().length() == 0);
    }

    @Override
    public String toString() {
        return courrielBean.toString();
    }

    @Override
    public int hashCode() {
        return courrielBean.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof Courriel)) {
            return false;
        }
        return ((Courriel) obj).courrielBean.equals(this.courrielBean);
    }

    public CourrielBean toCourrielBean() {
        return new CourrielBean(courrielBean);
    }

}
