/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public class DefaultFicheBlockList implements FicheBlockList, Serializable {

    private static final long serialVersionUID = 2L;
    private FicheBlock[] blocks;
    private int size = 0;

    public DefaultFicheBlockList() {
        this(8);
    }

    public DefaultFicheBlockList(int size) {
        blocks = new FicheBlock[size];
    }

    public DefaultFicheBlockList(FicheBlock ficheBlock) {
        if (ficheBlock == null) {
            throw new IllegalArgumentException("null not allowed");
        }
        blocks = new FicheBlock[2];
        size = 1;
        blocks[0] = ficheBlock;
    }

    public int getFicheBlockCount() {
        return size;
    }

    public FicheBlock getFicheBlock(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return blocks[index];
    }

    protected void innerClear() {
        for (int i = 0; i < size; i++) {
            blocks[i].clear();
            blocks[i] = null;
        }
        size = 0;
    }

    protected void innerAddFicheBlock(FicheBlock ficheBlock) {
        if (!isValidFicheBlock(ficheBlock)) {
            throw new IllegalArgumentException("ficheBlock is not valid");
        }
        checkLength();
        blocks[size] = ficheBlock;
        size++;
    }

    public boolean isValidFicheBlock(FicheBlock ficheBlock) {
        if (ficheBlock == null) {
            return false;
        }
        return true;
    }

    protected void innerAddFicheBlockList(FicheBlockList ficheBlockList) {
        int othersize = ficheBlockList.getFicheBlockCount();
        checkLength(othersize);
        for (int i = 0; i < othersize; i++) {
            FicheBlock ficheBlock = ficheBlockList.getFicheBlock(i);
            innerAddFicheBlock(ficheBlock);
        }
    }

    private void checkLength(int addlength) {
        if (size + addlength >= blocks.length) {
            FicheBlock[] nv = new FicheBlock[size + addlength];
            System.arraycopy(blocks, 0, nv, 0, size);
            blocks = nv;
        }
    }

    private void checkLength() {
        if (size == blocks.length) {
            FicheBlock[] nv = new FicheBlock[size * 2];
            System.arraycopy(blocks, 0, nv, 0, size);
            blocks = nv;
        }
    }

}
