/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import java.io.Serializable;
import java.util.HashMap;
import java.util.ArrayList;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;


/**
 *
 * @author Vincent Calame
 */
public class Fiche implements Serializable {

    private static final long serialVersionUID = 9L;
    private HashMap<String, Object> fieldMap = new HashMap<String, Object>();
    private ArrayList<InternalInformation> informationList;
    private ArrayList<InternalPropriete> proprieteList;
    private ArrayList<InternalSection> sectionList;
    private DefaultFicheItemList redacteurs = null;
    private String titre = "";
    private Para soustitre = null;
    private int langInteger = LangInteger.UNDETERMINED;

    public Fiche() {
    }

    public void setLangInteger(int langInteger) {
        this.langInteger = langInteger;
    }

    public int getLangInteger() {
        return langInteger;
    }

    /**
     * Ne retourne jamais <em>null</em>
     */
    public String getTitre() {
        return titre;
    }

    /**
     *
     *@throws IllegalArgumentException si titre est nul
     */
    public void setTitre(String titre) {
        if (titre == null) {
            throw new IllegalArgumentException("argument is null");
        }
        this.titre = StringUtils.cleanString(titre);
    }

    /**
     * Peut retourner <em>null</em>
     */
    public Para getSoustitre() {
        return soustitre;
    }

    /**
     *
     *@throws IllegalArgumentException si soustitre est nul
     */
    public void setSoustitre(Para soustitrePara) {
        if ((soustitrePara == null) || (soustitrePara.getTextContentCount() == 0)) {
            this.soustitre = null;
        } else {
            this.soustitre = soustitrePara;
        }
    }

    /**
     * <em>null</em> ou non vide
     */
    public FicheItemList getRedacteurs() {
        return redacteurs;
    }

    public void appendRedacteurs(FicheItemList ficheItemList) {
        setRedacteurs(ficheItemList, true);
    }

    public void setRedacteurs(FicheItemList ficheItemList) {
        setRedacteurs(ficheItemList, false);
    }

    private void setRedacteurs(FicheItemList ficheItemList, boolean append) {
        if ((ficheItemList != null) && (ficheItemList.getFicheItemCount() == 0)) {
            ficheItemList = null;
        }
        if (ficheItemList == null) {
            if (!append) {
                redacteurs = null;
            }
            return;
        }
        if (redacteurs == null) {
            redacteurs = new DefaultFicheItemList();
        } else if (!append) {
            redacteurs.innerClear();
        }
        int ficheItemCount = ficheItemList.getFicheItemCount();
        for (int i = 0; i < ficheItemCount; i++) {
            FicheItem ficheItem = ficheItemList.getFicheItem(i);
            if (ficheItem instanceof RedacteurItem) {
                redacteurs.innerAddFicheItem(ficheItem);
            }
        }
        if (redacteurs.getFicheItemCount() == 0) {
            redacteurs = null;
        }
    }

    /**
     *
     *@throws IllegalArgumentException si fieldKey.isInformationFieldKey() == false
     */
    public Information getInformation(FieldKey fieldKey) {
        if (!fieldKey.isInformationFieldKey()) {
            throw new IllegalArgumentException("fieldKey is not an InformationFieldKey");
        }
        return (Information) getField(fieldKey);
    }

    /**
     *
     *@throws IllegalArgumentException si fieldKey.isProprieteFieldKey() == false
     */
    public Propriete getPropriete(FieldKey fieldKey) {
        if (!fieldKey.isProprieteFieldKey()) {
            throw new IllegalArgumentException("fieldKey is not an ProprieteFieldKey");
        }
        return (Propriete) getField(fieldKey);
    }

    /**
     *
     *@throws IllegalArgumentException si fieldKey.isSectionFieldKey() == false
     */
    public Section getSection(FieldKey fieldKey) {
        if (!fieldKey.isSectionFieldKey()) {
            throw new IllegalArgumentException("fieldKey is not an SectionFieldKey");
        }
        return (Section) getField(fieldKey);
    }

    public int getProprieteCount() {
        if (proprieteList == null) {
            return 0;
        }
        return proprieteList.size();
    }

    public Propriete getPropriete(int i) {
        return proprieteList.get(i);
    }

    public int getInformationCount() {
        if (informationList == null) {
            return 0;
        }
        return informationList.size();
    }

    public Information getInformation(int i) {
        return informationList.get(i);
    }

    public int getSectionCount() {
        if (sectionList == null) {
            return 0;
        }
        return sectionList.size();
    }

    public Section getSection(int i) {
        return sectionList.get(i);
    }

    public void appendInformation(FieldKey fieldKey, FicheItemList ficheItemList) {
        setInformation(fieldKey, ficheItemList, true);
    }

    public void setInformation(FieldKey fieldKey, FicheItemList ficheItemList) {
        setInformation(fieldKey, ficheItemList, false);
    }

    private void setInformation(FieldKey fieldKey, FicheItemList ficheItemList, boolean append) {
        if (!fieldKey.isInformationFieldKey()) {
            throw new IllegalArgumentException("Not a Information FieldKey");
        }
        if ((ficheItemList != null) && (ficheItemList.getFicheItemCount() == 0)) {
            ficheItemList = null;
        }
        InternalInformation current = (InternalInformation) getField(fieldKey);
        if (current == null) {
            if (ficheItemList == null) {
                return;
            }
            if (informationList == null) {
                informationList = new ArrayList<InternalInformation>();
            }
            InternalInformation information = new InternalInformation(fieldKey);
            information.innerAddFicheItemList(ficheItemList);
            putField(fieldKey, information);
            informationList.add(information);
        } else {
            if (ficheItemList == null) {
                if (!append) {
                    removeField(fieldKey);
                    informationList.remove(current);
                }
            } else {
                if (!append) {
                    current.innerClear();
                }
                current.innerAddFicheItemList(ficheItemList);
            }
        }
    }

    public void appendSection(FieldKey fieldKey, FicheBlockList ficheBlockList) {
        setSection(fieldKey, ficheBlockList, true);
    }

    public void setSection(FieldKey fieldKey, FicheBlockList ficheBlockList) {
        setSection(fieldKey, ficheBlockList, false);
    }

    private void setSection(FieldKey fieldKey, FicheBlockList ficheBlockList, boolean append) {
        if (!fieldKey.isSectionFieldKey()) {
            throw new IllegalArgumentException("Not a Section FieldKey");
        }
        if ((ficheBlockList != null) && (ficheBlockList.getFicheBlockCount() == 0)) {
            ficheBlockList = null;
        }
        InternalSection current = (InternalSection) getField(fieldKey);
        if (current == null) {
            if (ficheBlockList == null) {
                return;
            }
            if (sectionList == null) {
                sectionList = new ArrayList<InternalSection>();
            }
            InternalSection section = new InternalSection(fieldKey);
            section.innerAddFicheBlockList(ficheBlockList);
            putField(fieldKey, section);
            sectionList.add(section);
        } else {
            if (ficheBlockList == null) {
                if (!append) {
                    removeField(fieldKey);
                    sectionList.remove(current);
                }
            } else {
                if (!append) {
                    current.innerClear();
                }
                current.innerAddFicheBlockList(ficheBlockList);
            }
        }
    }

    public void setPropriete(FieldKey fieldKey, FicheItem ficheItem) {
        if (!fieldKey.isProprieteFieldKey()) {
            throw new IllegalArgumentException("Not a Propriete FieldKey");
        }
        InternalPropriete current = (InternalPropriete) getField(fieldKey);
        if (current == null) {
            if (ficheItem == null) {
                return;
            }
            if (proprieteList == null) {
                proprieteList = new ArrayList<InternalPropriete>();
            }
            InternalPropriete propriete = new InternalPropriete(fieldKey);
            propriete.setFicheItem(ficheItem);
            putField(fieldKey, propriete);
            proprieteList.add(propriete);
        } else {
            if (ficheItem == null) {
                removeField(fieldKey);
                proprieteList.remove(current);
            } else {
                current.setFicheItem(ficheItem);
            }
        }
    }

    public void clear() {
        titre = "";
        soustitre = null;
        redacteurs = null;
        if (informationList != null) {
            informationList.clear();
            informationList = null;
        }
        if (proprieteList != null) {
            proprieteList.clear();
            proprieteList = null;
        }
        if (sectionList != null) {
            sectionList.clear();
            sectionList = null;
        }
        clearFieldMap();
    }

    private Object getField(FieldKey fieldKey) {
        return fieldMap.get(fieldKey.toString());
    }

    private void putField(FieldKey fieldKey, Object obj) {
        fieldMap.put(fieldKey.toString(), obj);
    }

    private void removeField(FieldKey fieldKey) {
        fieldMap.remove(fieldKey.toString());
    }

    private void clearFieldMap() {
        fieldMap.clear();
    }


    private static class InternalPropriete implements Propriete, Serializable {

        private FieldKey fieldKey;
        private FicheItem item;

        private InternalPropriete(FieldKey fieldKey) {
            this.fieldKey = fieldKey;
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

        public FicheItem getFicheItem() {
            return item;
        }

        private void setFicheItem(FicheItem ficheitem) {
            this.item = ficheitem;
        }

    }


    private static class InternalInformation extends DefaultFicheItemList implements Information, Serializable {

        private FieldKey fieldKey;

        private InternalInformation(FieldKey fieldKey) {
            this.fieldKey = fieldKey;
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

    }


    private static class InternalSection extends DefaultFicheBlockList implements Section, Serializable {

        private FieldKey fieldKey;

        private InternalSection(FieldKey fieldKey) {
            this.fieldKey = fieldKey;
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

    }

}
