/* BasedefichesLib_API - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class Insert extends ZoneBlock {

    private static final long serialVersionUID = 3L;
    public final static short IMAGE_TYPE = 1;
    public final static short LETTRINE_TYPE = 2;
    public final static short POSITION_UNDETERMINED = 0;
    public final static short POSITION_LEFT = 1;
    public final static short POSITION_RIGHT = 2;
    private short type;
    private int width = -1;
    private int height = -1;
    private String src = "";
    private String ref = "";
    private String credit = "";
    private SubsetKey albumSubsetKey = null;
    private int idalbum = 0;
    private String albumDimName = null;
    private short position = POSITION_UNDETERMINED;

    public Insert(short type) {
        if ((type < 1) && (type > 2)) {
            throw new IllegalArgumentException("wrong type value");
        }
        this.type = type;
        if (type == LETTRINE_TYPE) {
            position = POSITION_LEFT;
        }
    }

    @Override
    public void clear() {
        super.clear();
        src = "";
        ref = "";
        credit = "";
    }

    public short getType() {
        return type;
    }

    public String getSrc() {
        return src;
    }

    public String getRef() {
        return ref;
    }

    public String getCredit() {
        return credit;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public short getPosition() {
        return position;
    }

    public void setWidth(int width) {
        if (width < 0) {
            this.width = -1;
        } else {
            this.width = width;
        }
    }

    public void setHeight(int height) {
        if (height < 0) {
            this.height = -1;
        } else {
            this.height = height;
        }
    }

    public void setPosition(short position) {
        if (type != LETTRINE_TYPE) {
            return;
        }
        if (position != POSITION_RIGHT) {
            this.position = POSITION_LEFT;
        } else {
            this.position = POSITION_RIGHT;
        }
    }

    public void setSrc(String src) {
        if (src == null) {
            this.src = "";
        } else {
            this.src = src;
        }
    }

    public void setRef(String ref) {
        if (ref == null) {
            this.ref = "";
        } else {
            this.ref = ref;
        }
    }

    public void setCredit(String credit) {
        if (credit == null) {
            this.credit = "";
        } else {
            this.credit = credit;
        }
    }

    public void setIllustration(SubsetKey albumSubsetKey, int idalbum, String albumDimName) {
        this.albumSubsetKey = albumSubsetKey;
        this.idalbum = idalbum;
        if (albumDimName == null) {
            this.albumDimName = "";
        } else {
            this.albumDimName = albumDimName;
        }
    }

    public SubsetKey getAlbumSubsetKey() {
        return albumSubsetKey;
    }

    public int getIdalbum() {
        return idalbum;
    }

    public String getAlbumDimName() {
        return albumDimName;
    }

    public static String typeToString(short type) {
        switch (type) {
            case IMAGE_TYPE:
                return "image";
            case LETTRINE_TYPE:
                return "lettrine";
            default:
                throw new IllegalArgumentException("wrong type value");
        }
    }

    public static char typeToInitiale(short type) {
        switch (type) {
            case IMAGE_TYPE:
                return 'i';
            case LETTRINE_TYPE:
                return 'l';
            default:
                throw new IllegalArgumentException("wrong type value");
        }
    }

    public static String positionToString(short position) {
        switch (position) {
            case POSITION_LEFT:
                return "left";
            case POSITION_RIGHT:
                return "right";
            case POSITION_UNDETERMINED:
                return "";
            default:
                throw new IllegalArgumentException("wrong position value");
        }
    }

    public static char positionToInitiale(short position) {
        switch (position) {
            case POSITION_LEFT:
                return 'l';
            case POSITION_RIGHT:
                return 'r';
            case POSITION_UNDETERMINED:
                return '0';
            default:
                throw new IllegalArgumentException("wrong position value");
        }
    }

    public static short typeToShort(String chtype) {
        if (chtype.equals("image")) {
            return IMAGE_TYPE;
        }
        if (chtype.equals("lettrine")) {
            return LETTRINE_TYPE;
        }
        throw new IllegalArgumentException("wrong type value");
    }

    public static short typeToShort(char initiale) {
        switch (initiale) {
            case 'i':
                return IMAGE_TYPE;
            case 'l':
                return LETTRINE_TYPE;
            default:
                return IMAGE_TYPE;
        }
    }

    public static short positionToShort(char initiale) {
        switch (initiale) {
            case 'r':
                return POSITION_RIGHT;
            case 'l':
                return POSITION_LEFT;
            default:
                return POSITION_UNDETERMINED;
        }
    }

    public static short positionToString(String chtype) {
        if (chtype.equals("left")) {
            return POSITION_LEFT;
        }
        if (chtype.equals("right")) {
            return POSITION_RIGHT;
        }
        return POSITION_UNDETERMINED;
    }

}
