/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public abstract class ParagraphBlock implements FicheBlock, TextContentBuilder, TextContentList, Serializable {

    private static final long serialVersionUID = 2L;
    protected List<Object> contentList = new ArrayList<Object>();
    private String className = "";
    private String htmlId = "";

    public ParagraphBlock() {
    }

    public int getTextContentCount() {
        return contentList.size();
    }

    public Object getTextContent(int i) {
        return contentList.get(i);
    }

    public void addText(String text) {
        contentList.add(text);
    }

    public void addS(S s) {
        contentList.add(s);
    }

    public String getClassName() {
        return className;
    }

    public String getHtmlId() {
        return htmlId;
    }

    public void setClassName(String className) {
        if (className == null) {
            this.className = "";
        } else {
            this.className = className;
        }
    }

    public void setHtmlId(String id) {
        if (id == null) {
            this.htmlId = "";
        } else {
            this.htmlId = id;
        }
    }

    public void clear() {
        for (int i = 0; i < contentList.size(); i++) {
            Object obj = contentList.get(i);
            if (obj instanceof S) {
                ((S) obj).clear();
            }
        }
        className = "";
        htmlId = "";
        contentList.clear();
    }

    public String toRawString() {
        StringBuilder buf = new StringBuilder();
        int size = contentList.size();
        for (int i = 0; i < size; i++) {
            Object obj = contentList.get(i);
            if (obj instanceof String) {
                buf.append((String) obj);
            } else if (obj instanceof S) {
                S s = (S) obj;
                buf.append(s.getValue());
            }
        }
        return buf.toString();
    }

}
