/* BasedefichesLib_API - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.CountryUtils;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import java.util.Locale;
import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class Pays implements FicheItem, Serializable {

    private static final long serialVersionUID = 2L;
    private int countryInteger;

    public Pays(String countryCode) {
        try {
            countryInteger = CountryUtils.parseCountryCode(countryCode);
        } catch (LangIntegerException ile) {
            throw new IllegalArgumentException("");
        }

    }

    private Pays(int countryInteger) {
        this.countryInteger = countryInteger;
    }

    public String getCountryCode() {
        return LangInteger.getCountry(countryInteger);
    }

    /**
     * Rappel : un « countryInteger » est un langInteger composé de NO_LINGUISTIC_CONTENT et du code pays (zxx-{codepays}).
     */
    public int getCountryInteger() {
        return countryInteger;
    }

    @Override
    public String toString() {
        return getCountryCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof Pays)) {
            return false;
        }
        return (((Pays) obj).countryInteger == this.countryInteger);
    }

    @Override
    public int hashCode() {
        return countryInteger;
    }

    public static Pays parsePays(String s, Locale locale) {
        try {
            int countryInteger = CountryUtils.parseCountryCode(s);
            return new Pays(countryInteger);
        } catch (LangIntegerException ile) {
            if (locale != null) {
                Integer itg = LangIntegerLocalisation.getLangInteger(s, locale);
                if (itg == null) {
                    return null;
                }
                int countryInteger = itg.intValue();
                if (LangInteger.getLangPart(countryInteger) != LangInteger.NO_LINGUISTIC_CONTENT) {
                    return null;
                }
                if (countryInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
                    return null;
                }
                return new Pays(countryInteger);
            } else {
                return null;
            }
        }
    }

}
