/* BasedefichesLib_API - Copyright (c) 2006-2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;

import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public class S implements Serializable {

    private static final long serialVersionUID = 2L;
    public final static int SRC_BITVALUE = 1;
    public final static int REF_BITVALUE = 2;
    public final static int VALUE_BITVALUE = 4;
    public final static int TITLE_BITVALUE = 8;
    public final static int LANG_BITVALUE = 16;
    public final static int IDENTIFIERS_BITVALUE = 32;
    public final static int SUBTYPE_BITVALUE = 64;
    public final static short TYPE_UNKNOWN = 0;
    public final static short TYPE_EM = 1;
    public final static short TYPE_STRG = 2;
    public final static short TYPE_LINK = 3;
    public final static short TYPE_EMSTRG = 4;
    public final static short TYPE_IREF = 5;
    public final static short TYPE_ANCHOR = 6;
    public final static short TYPE_IMG = 7;
    public final static short TYPE_FICHE = 8;
    public final static short TYPE_CITE = 9;
    public final static short TYPE_DFN = 10;
    public final static short TYPE_SAMP = 11;
    public final static short TYPE_KBD = 12;
    public final static short TYPE_VAR = 13;
    public final static short TYPE_ABBR = 14;
    public final static short TYPE_ACRONYM = 15;
    public final static short TYPE_SPAN = 16;
    public final static short TYPE_DEL = 17;
    public final static short TYPE_CODE = 18;
    public final static short TYPE_INS = 19;
    public final static short TYPE_BR = 20;
    public final static short TYPE_SUP = 21;
    public final static short TYPE_SUB = 22;
    public final static short TYPE_QUOTE = 23;
    public final static short TYPE_MOTCLE = 24;
    private short type;
    private String subtype;
    private String value = "";
    private String className = "";
    private String ref;
    private String src;
    private String lang;
    private String title;
    private String htmlId = "";

    public S(short type) {
        this.type = type;
        init();
    }

    /**
     *
     *@throws IllegalArgumentException si value est null
     */
    public S(short type, String value) {
        this.type = type;
        if (value == null) {
            throw new IllegalArgumentException("value is null");
        }
        this.value = value;
        init();
    }

    private void init() {
        int mask = getAttributeMask(type);
        if ((mask & SRC_BITVALUE) != 0) {
            src = "";
        }
        if ((mask & REF_BITVALUE) != 0) {
            ref = "";
        }
        if ((mask & TITLE_BITVALUE) != 0) {
            title = "";
        }
        if ((mask & LANG_BITVALUE) != 0) {
            lang = "";
        }
        if ((mask & SUBTYPE_BITVALUE) != 0) {
            subtype = "";
        }
    }

    public short getType() {
        return type;
    }

    public String getTypeString() {
        return getTypeToString(type);
    }

    public String getRef() {
        return ref;
    }

    public String getSrc() {
        return src;
    }

    public String getTitle() {
        return title;
    }

    public String getLang() {
        return lang;
    }

    public String getClassName() {
        return className;
    }

    public String getHtmlId() {
        return htmlId;
    }

    public String getSubtype() {
        return subtype;
    }

    public void setValue(String s) {
        if (s == null) {
            this.value = "";
        } else {
            this.value = s;
        }
    }

    public boolean hasIdentifiers() {
        return (className.length() != 0) || (htmlId.length() != 0);
    }

    public String getIdentifiers() {
        if (htmlId.length() > 0) {
            return className + "!" + htmlId;
        }
        return className;
    }

    public void setClassName(String s) {
        if (s == null) {
            this.className = "";
        } else {
            this.className = s;
        }
    }

    public void setHtmlId(String s) {
        if (s == null) {
            this.htmlId = "";
        } else {
            this.htmlId = s;
        }
    }

    public void setIdentifiers(String s) {
        if (s == null) {
            this.className = "";
            this.htmlId = "";
        } else {
            int idx = s.indexOf('!');
            if (idx == -1) {
                this.className = s;
                this.htmlId = "";
            } else {
                this.className = s.substring(0, idx);
                this.htmlId = s.substring(idx + 1);
            }
        }
    }

    public void setRef(String ref) {
        int mask = getAttributeMask();
        if ((mask & REF_BITVALUE) != 0) {
            if (ref != null) {
                this.ref = ref;
            } else {
                this.ref = "";
            }
        } else {
            //throw new UnsupportedOperationException("Attribute not allowed");
        }
    }

    public void setTitle(String title) {
        int mask = getAttributeMask();
        if ((mask & TITLE_BITVALUE) != 0) {
            if (title != null) {
                this.title = title;
            } else {
                this.title = "";
            }
        } else {
            //throw new UnsupportedOperationException("Attribute not allowed");
        }
    }

    public void setLang(String lang) {
        int mask = getAttributeMask();
        if ((mask & LANG_BITVALUE) != 0) {
            if (lang != null) {
                this.lang = lang;
            } else {
                this.lang = "";
            }
        } else {
            //throw new UnsupportedOperationException("Attribute not allowed");
        }
    }

    public void setSrc(String src) {
        int mask = getAttributeMask();
        if ((mask & SRC_BITVALUE) != 0) {
            if (src != null) {
                this.src = src;
            } else {
                this.src = "";
            }
        } else {
            //throw new UnsupportedOperationException("Attribute not allowed");
        }
    }

    public void setSubtype(String subtype) {
        int mask = getAttributeMask();
        if ((mask & SUBTYPE_BITVALUE) != 0) {
            if (subtype != null) {
                this.subtype = subtype;
            } else {
                this.subtype = "";
            }
        } else {
            //throw new UnsupportedOperationException("Attribute not allowed");
        }
    }

    public void setAttribute(int bitvalue, String s) {
        switch (bitvalue) {
            case SRC_BITVALUE:
                setSrc(s);
                break;
            case REF_BITVALUE:
                setRef(s);
                break;
            case VALUE_BITVALUE:
                setValue(s);
                break;
            case IDENTIFIERS_BITVALUE:
                setIdentifiers(s);
                break;
            case TITLE_BITVALUE:
                setTitle(s);
                break;
            case LANG_BITVALUE:
                setLang(s);
                break;
            case SUBTYPE_BITVALUE:
                setSubtype(s);
                break;
            default:
                throw new IllegalArgumentException("Unknown bitvalue = " + bitvalue);
        }
    }

    public String getAttribute(int bitValue) {
        switch (bitValue) {
            case SRC_BITVALUE:
                return getSrc();
            case REF_BITVALUE:
                return getRef();
            case VALUE_BITVALUE:
                return getValue();
            case IDENTIFIERS_BITVALUE:
                return getIdentifiers();
            case TITLE_BITVALUE:
                return getTitle();
            case LANG_BITVALUE:
                return getLang();
            case SUBTYPE_BITVALUE:
                return getSubtype();
            default:
                throw new IllegalArgumentException("Unknown bitValue = " + bitValue);
        }
    }

    public int getAttributeMask() {
        return getAttributeMask(type);
    }

    public void checkHttpValue(boolean fill) {
        if ((type == TYPE_IMG) || (type == TYPE_LINK)) {
            String defval = (src != null) ? src : ref;
            if (fill) {
                if (value.length() == 0) {
                    if (defval != null) {
                        if (defval.startsWith("http://")) {
                            defval = defval.substring(7);
                        }
                        value = defval;
                    }
                }
            } else {
                if ((value.equals(defval)) || (("http://" + value).equals(defval))) {
                    value = "";
                }
            }
        }
    }

    static public String getTypeToString(short type) {
        switch (type) {
            case TYPE_EM:
                return "em";
            case TYPE_STRG:
                return "strg";
            case TYPE_EMSTRG:
                return "emstrg";
            case TYPE_LINK:
                return "link";
            case TYPE_ANCHOR:
                return "anchor";
            case TYPE_IREF:
                return "iref";
            case TYPE_IMG:
                return "img";
            case TYPE_FICHE:
                return "fiche";
            case TYPE_CITE:
                return "cite";
            case TYPE_DFN:
                return "dfn";
            case TYPE_SAMP:
                return "samp";
            case TYPE_KBD:
                return "kbd";
            case TYPE_VAR:
                return "var";
            case TYPE_ABBR:
                return "abbr";
            case TYPE_ACRONYM:
                return "acronym";
            case TYPE_SPAN:
                return "span";
            case TYPE_DEL:
                return "del";
            case TYPE_INS:
                return "ins";
            case TYPE_BR:
                return "br";
            case TYPE_SUP:
                return "sup";
            case TYPE_SUB:
                return "sub";
            case TYPE_CODE:
                return "code";
            case TYPE_QUOTE:
                return "quote";
            case TYPE_MOTCLE:
                return "motcle";
            default:
                return "";
        }
    }

    static public short getTypeToShort(String chtype) {
        if (chtype == null) {
            return TYPE_UNKNOWN;
        }
        if (chtype.equals("em")) {
            return TYPE_EM;
        }
        if (chtype.equals("strg")) {
            return TYPE_STRG;
        }
        if (chtype.equals("emstrg")) {
            return TYPE_EMSTRG;
        }
        if (chtype.equals("link")) {
            return TYPE_LINK;
        }
        if (chtype.equals("note")) {
            return TYPE_IREF;
        }
        if (chtype.equals("iref")) {
            return TYPE_IREF;
        }
        if (chtype.equals("anchor")) {
            return TYPE_ANCHOR;
        }
        if (chtype.equals("img")) {
            return TYPE_IMG;
        }
        if (chtype.equals("fiche")) {
            return TYPE_FICHE;
        }
        if (chtype.equals("cite")) {
            return TYPE_CITE;
        }
        if (chtype.equals("dfn")) {
            return TYPE_DFN;
        }
        if (chtype.equals("samp")) {
            return TYPE_SAMP;
        }
        if (chtype.equals("kbd")) {
            return TYPE_KBD;
        }
        if (chtype.equals("var")) {
            return TYPE_VAR;
        }
        if (chtype.equals("abbr")) {
            return TYPE_ABBR;
        }
        if (chtype.equals("acronym")) {
            return TYPE_ACRONYM;
        }
        if (chtype.equals("span")) {
            return TYPE_SPAN;
        }
        if (chtype.equals("code")) {
            return TYPE_CODE;
        }
        if (chtype.equals("ins")) {
            return TYPE_INS;
        }
        if (chtype.equals("del")) {
            return TYPE_DEL;
        }
        if (chtype.equals("quote")) {
            return TYPE_QUOTE;
        }
        if (chtype.equals("motcle")) {
            return TYPE_MOTCLE;
        }
        if (chtype.equals("br")) {
            return TYPE_BR;
        }
        if (chtype.equals("sup")) {
            return TYPE_SUP;
        }
        if (chtype.equals("sub")) {
            return TYPE_SUB;
        }
        return TYPE_UNKNOWN;
    }

    public String toString() {
        return value;
    }

    public String getValue() {
        return value;
    }

    /**
     * Propose un code d'une lettre pour chaque élément. Ce n'est pas forcément l'initial du mot anglais.
     */
    static public char getTypeToInitial(short type) {
        switch (type) {
            case TYPE_EM:
                return 'e';
            case TYPE_STRG:
                return 'b';
            case TYPE_EMSTRG:
                return 'w';
            case TYPE_LINK:
                return 'a';
            case TYPE_ANCHOR:
                return 'o';
            case TYPE_IREF:
                return 'g';
            case TYPE_IMG:
                return 'i';
            case TYPE_FICHE:
                return 'f';
            case TYPE_CITE:
                return 'r';
            case TYPE_DFN:
                return 'd';
            case TYPE_SAMP:
                return 's';
            case TYPE_KBD:
                return 'k';
            case TYPE_VAR:
                return 'v';
            case TYPE_ABBR:
                return 't';
            case TYPE_ACRONYM:
                return 'y';
            case TYPE_SPAN:
                return 'z';
            case TYPE_DEL:
                return 'x';
            case TYPE_CODE:
                return 'c';
            case TYPE_INS:
                return 'u';
            case TYPE_BR:
                return 'l';
            case TYPE_SUP:
                return 'h';
            case TYPE_SUB:
                return 'j';
            case TYPE_QUOTE:
                return 'q';
            case TYPE_MOTCLE:
                return 'm';
            default:
                throw new IllegalArgumentException("Wrong type value");
        }
    }

    static public char getSubtypeToInitial(String subtype) {
        if (subtype == null) {
            return '0';
        }
        if (subtype.equals("CLS")) {
            return 'c';
        }
        if (subtype.equals("XML")) {
            return 'x';
        }
        if (subtype.equals("UI")) {
            return 'u';
        }
        if (subtype.equals("STG")) {
            return 's';
        }
        if (subtype.equals("CMD")) {
            return 'u';
        }
        if (subtype.equals("RSC")) {
            return 'r';
        }
        if (subtype.equals("PTH")) {
            return 'p';
        }
        if (subtype.equals("KEY")) {
            return 'k';
        }
        if (subtype.equals("MTD")) {
            return 'm';
        }
        return '0';
    }

    static public String getInitialToSubtype(short type, char carac) {
        switch (carac) {
            /*case  'a':
            return TYPE_LINK;
            case 'b' :
            return TYPE_STRG;*/
            case 'c':
                return "CLS";
            /*case 'd' :
            return TYPE_DFN;
            case 'e' :
            return TYPE_EM;*/
            /*case 'f' :
            return "FNC";/*
            case 'g' :
            return TYPE_IREF;
            case 'h' :
            return TYPE_SUP;
             **/
            /*case 'i' :
            return "ICP";/*
            case 'j' :
            return TYPE_SUB;*/
            case 'k':
                return "KEY";/*
            case 'l' :
            return TYPE_BR;*/
            case 'm':
                return "MTD";/*
            case 'o' :
            return TYPE_ANCHOR;
             **/
            case 'p':
                return "PTH";
            case 'r':
                return "RSC";
            case 's':
                return "STG";
            /* case 't' :
            return TYPE_ABBR;*/
            case 'u':
                return "UI";
            /*case 'v' :
            return TYPE_VAR;
            case  'w':
            return TYPE_EMSTRG;*/
            case 'x':
                return "XML";
            /*case 'y' :
            return TYPE_ACRONYM;
            case 'z' :
            return TYPE_SPAN;*/
            default:
                return null;
        }
    }

    static public short getInitialToShort(char carac) {
        switch (carac) {
            case 'a':
                return TYPE_LINK;
            case 'b':
                return TYPE_STRG;
            case 'c':
                return TYPE_CODE;
            case 'd':
                return TYPE_DFN;
            case 'e':
                return TYPE_EM;
            case 'f':
                return TYPE_FICHE;
            case 'g':
                return TYPE_IREF;
            case 'h':
                return TYPE_SUP;
            case 'i':
                return TYPE_IMG;
            case 'j':
                return TYPE_SUB;
            case 'k':
                return TYPE_KBD;
            case 'l':
                return TYPE_BR;
            case 'm':
                return TYPE_MOTCLE;
            case 'o':
                return TYPE_ANCHOR;
            case 'q':
                return TYPE_QUOTE;
            case 'r':
                return TYPE_CITE;
            case 's':
                return TYPE_SAMP;
            case 't':
                return TYPE_ABBR;
            case 'u':
                return TYPE_INS;
            case 'v':
                return TYPE_VAR;
            case 'w':
                return TYPE_EMSTRG;
            case 'x':
                return TYPE_DEL;
            case 'y':
                return TYPE_ACRONYM;
            case 'z':
                return TYPE_SPAN;
            default:
                throw new IllegalArgumentException("Wrong carac value");
        }
    }

    public static boolean hasAttribute(short type, int bitvalue) {
        return (getAttributeMask(type) & bitvalue) != 0;
    }

    public static int getAttributeMask(short type) {
        int mask = VALUE_BITVALUE | IDENTIFIERS_BITVALUE;
        int linkmask = REF_BITVALUE | LANG_BITVALUE;
        switch (type) {
            case TYPE_IMG:
                return mask | linkmask | SRC_BITVALUE;
            case TYPE_CITE:
            case TYPE_DFN:
                return mask | linkmask | SUBTYPE_BITVALUE;
            case TYPE_FICHE:
            case TYPE_LINK:
            case TYPE_MOTCLE:
                return mask | linkmask;
            case TYPE_ANCHOR:
            case TYPE_IREF:
                return mask | REF_BITVALUE;
            case TYPE_ABBR:
            case TYPE_ACRONYM:
                return mask | TITLE_BITVALUE;
            case TYPE_SAMP:
            case TYPE_VAR:
            case TYPE_KBD:
            case TYPE_CODE:
                return mask | SUBTYPE_BITVALUE;
            case TYPE_EM:
            case TYPE_STRG:
            case TYPE_EMSTRG:
            case TYPE_SPAN:
            case TYPE_BR:
            case TYPE_SUP:
            case TYPE_SUB:
            case TYPE_QUOTE:
            case TYPE_INS:
            case TYPE_DEL:
            default:
                return mask;
        }
    }

    public void clear() {
        value = "";
        className = "";
        htmlId = "";
    }

}
