/* BasedefichesLib_API - Copyright (c) 2006-20007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.fiche;


/**
 *
 * @author Vincent Calame
 */
public class Table extends ZoneBlock {

    private static final long serialVersionUID = 2L;
    public final static int HORIZONTALHEADER_BITVALUE = 1;
    public final static int VERTICALHEADER_BITVALUE = 2;
    private Tr[] trs = new Tr[4];
    private int size = 0;
    private int headerMask = 0;
    private int cols = 0;
    private int rows = 0;

    public Table() {
    }

    public int getTrCount() {
        return size;
    }

    public Tr getTr(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return trs[index];
    }

    @Override
    public void clear() {
        super.clear();
        for (int i = 0; i < size; i++) {
            trs[i].clear();
            trs[i] = null;
        }
        size = 0;
    }

    public int getRows() {
        return rows;
    }

    public int getCols() {
        return cols;
    }

    public int getHeaderMask() {
        return headerMask;
    }

    public void addTr(Tr tr) {
        if (size == 0) {
            testHorizontalHeader(tr);
        } else if (size == 1) {
            testVerticalHeader(tr);
        }
        checkLength();
        trs[size] = tr;
        size++;
        rows++;
        checkCols(tr);
    }

    public void checkSize() {
        cols = 0;
        for (int i = 0; i < size; i++) {
            checkCols(trs[i]);
        }
    }

    private void checkCols(Tr tr) {
        int curcols = 0;
        for (int i = 0; i < tr.getFicheBlockCount(); i++) {
            Td td = tr.getTd(i);
            curcols = curcols + td.getColSpan();
        }
        cols = Math.max(cols, curcols);
    }

    private void checkLength() {
        if (size == trs.length) {
            Tr[] nv = new Tr[size * 2];
            System.arraycopy(trs, 0, nv, 0, size);
            trs = nv;
        }
    }

    private void testHorizontalHeader(Tr tr) {
        int trsize = tr.getFicheBlockCount();
        if (trsize == 0) {
            return;
        }
        Td td = tr.getTd(0);
        if (td.getType() != Td.HEADER_TYPE) {
            return;
        }
        headerMask = headerMask | VERTICALHEADER_BITVALUE | HORIZONTALHEADER_BITVALUE;
        if (trsize == 1) {
            return;
        }
        Td td2 = tr.getTd(1);
        if (td.getType() != Td.HEADER_TYPE) {
            headerMask = headerMask & ~HORIZONTALHEADER_BITVALUE;
        }
    }

    private void testVerticalHeader(Tr tr) {
        if ((headerMask & VERTICALHEADER_BITVALUE) == 0) {
            return;
        }
        int trsize = tr.getFicheBlockCount();
        if (trsize == 0) {
            return;
        }
        Td td = tr.getTd(0);
        if (td.getType() != Td.HEADER_TYPE) {
            headerMask = headerMask & ~VERTICALHEADER_BITVALUE;
        }
    }

}
