/* BasedefichesLib_API - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class CorpusFieldListUtils {

    private CorpusFieldListUtils() {
    }

    public static FieldKey[] toFieldKeyArray(CorpusFieldList corpusFieldList) {
        int length = corpusFieldList.getCorpusFieldCount();
        FieldKey[] result = new FieldKey[length];
        for (int i = 0; i < length; i++) {
            result[i] = corpusFieldList.getCorpusField(i).getFieldKey();
        }
        return result;
    }

    public static CorpusField[] toArray(CorpusFieldList corpusFieldList) {
        int length = corpusFieldList.getCorpusFieldCount();
        CorpusField[] result = new CorpusField[length];
        for (int i = 0; i < length; i++) {
            result[i] = corpusFieldList.getCorpusField(i);
        }
        return result;
    }

    public static List<CorpusField> toList(CorpusFieldList corpusFieldList) {
        List<CorpusField> list = new ArrayList<CorpusField>();
        int count = corpusFieldList.getCorpusFieldCount();
        for (int i = 0; i < count; i++) {
            list.add(corpusFieldList.getCorpusField(i));
        }
        return list;
    }

    public static boolean isEqual(CorpusFieldList corpusFieldList, List<CorpusField> list) {
        int length = list.size();
        if (length != corpusFieldList.getCorpusFieldCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            if (!list.get(i).equals(corpusFieldList.getCorpusField(i))) {
                return false;
            }
        }
        return true;
    }

    public static CorpusFieldList fromList(List<CorpusField> list) {
        int size = list.size();
        return new ArrayCorpusFieldList(list.toArray(new CorpusField[size]));
    }

    /**
     * retourne nul si la liste devient vide.
     */
    public static CorpusFieldList removeCorpusField(CorpusFieldList corpusFieldList, CorpusField corpusField) {
        if (corpusFieldList == null) {
            return null;
        }
        List<CorpusField> result = new ArrayList<CorpusField>();
        int count = corpusFieldList.getCorpusFieldCount();
        boolean done = false;
        for (int i = 0; i < count; i++) {
            CorpusField otherCorpusField = corpusFieldList.getCorpusField(i);
            if (otherCorpusField.equals(corpusField)) {
                done = true;
            } else {
                result.add(otherCorpusField);
            }
        }
        if (!done) {
            return corpusFieldList;
        }
        int size = result.size();
        if (result.size() == 0) {
            return null;
        }
        return new ArrayCorpusFieldList(result.toArray(new CorpusField[size]));
    }


    private static class ArrayCorpusFieldList implements CorpusFieldList {

        private CorpusField[] array;

        private ArrayCorpusFieldList(CorpusField[] array) {
            this.array = array;
        }

        public int getCorpusFieldCount() {
            return array.length;
        }

        public CorpusField getCorpusField(int i) {
            return array[i];
        }

    }

}
