/* BasedefichesLib_API - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;


/**
 * Cette classe décrit les options qu'applique obligatoirement une implémentation de BaseDeFiches
 * Cette classe ne contient que des méthodes statiques.
 *
 * @author Vincent Calame
 */
public class FieldOptionsUtils implements FieldOptionsConstants {

    

    private FieldOptionsUtils() {
    }

    /**
     * Retourne la valeur de l'option INFORMATIONDISPLAY_OPTION qui doit être utilisée par défaut (il s'agit de INLINE_DISPLAY).
     */
    public static short getDefaultInformationDisplay() {
        return INLINE_DISPLAY;
    }

    /**
     *
     *@throws IllegalArgumentException
     */
    public static String informationDisplayToString(short informationDisplay) {
        switch (informationDisplay) {
            case INLINE_DISPLAY:
                return "inline";
            case BLOCK_DISPLAY:
                return "block";
            default:
                throw new IllegalArgumentException("wrong value");
        }
    }

    /**
     *
     *@throws IllegalArgumentException
     */
    public static short informationDisplayToShort(String informationDisplay) {
        if (informationDisplay.length() == 0) {
            return getDefaultInformationDisplay();
        }
        if (informationDisplay.equals("inline")) {
            return INLINE_DISPLAY;
        }
        if (informationDisplay.equals("block")) {
            return BLOCK_DISPLAY;
        }
        throw new IllegalArgumentException("wrong value");
    }

    public static boolean haveSubfield(int ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
            case CorpusField.GEOPOINT_FIELD:
            case CorpusField.IMAGE_FIELD:
                return true;
            default:
                return false;
        }
    }

    public static boolean testFieldOption(String optionName, CorpusField corpusField) {
        try {
            checkFieldOption(optionName, corpusField.getFieldKey(), corpusField.getFicheItemType());
            return true;
        } catch (FieldOptionException foe) {
            return false;
        }
    }

    public static void checkFieldOption(String optionName, FieldKey fieldKey, int ficheItemType) throws FieldOptionException {
        if (optionName.equals(INFORMATIONDISPLAY_OPTION)) {
            if (!fieldKey.isInformationFieldKey()) {
                throw new FieldOptionException("informationDisplay Option applies only to Information Field");
            }
        } else if (optionName.equals(DEFAULTSPHEREKEY_OPTION)) {
            if (ficheItemType != CorpusField.PERSONNE_FIELD) {
                throw new FieldOptionException("ficheItemType must be PERSONNE_FIELD");
            }
        } else if (optionName.equals(SUBFIELDDISPLAY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("mainPersonne Option applies only to Propriete Field");
            }
            if (!haveSubfield(ficheItemType)) {
                throw new FieldOptionException("wrong ficheItemType = " + ficheItemType);
            }
        } else if (optionName.equals(CURRENCYARRAY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("currencyArray Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.MONTANT_FIELD) {
                throw new FieldOptionException("ficheItemType must be MONTANT_FIELD");
            }
        } else if (optionName.equals(ADDRESSFIELDARRAY_OPTION)) {
            if (!fieldKey.isProprieteFieldKey()) {
                throw new FieldOptionException("addressFields Option applies only to Propriete Field");
            }
            if (ficheItemType != CorpusField.GEOPOINT_FIELD) {
                throw new FieldOptionException("ficheItemType must be GEOPOINT_FIELD");
            }
        } else if (optionName.equals(BASEURL_OPTION)) {
            if (ficheItemType != CorpusField.IMAGE_FIELD) {
                throw new FieldOptionException("ficheItemType must be IMAGE_FIELD");
            }
        } else if (optionName.equals(LANGARRAY_OPTION)) {
            if (fieldKey.equals(FieldKey.LANG)) {
                return;
            }
            if ((ficheItemType != CorpusField.PARA_FIELD) && (ficheItemType != CorpusField.LANGUE_FIELD)) {
                throw new FieldOptionException("ficheItemType must be PARA_FIELD or LANGUE_FIELD");
            }
        }
    }

}
