/* BasedefichesLib_API - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.corpus.metadata;

import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public final class SubfieldKey {

    public static final short NOM_SUBTYPE = 1;
    public static final short PRENOM_SUBTYPE = 2;
    public static final short NOMAVANT_SUBTYPE = 3;
    public static final short ORIGINAL_SUBTYPE = 4;
    public static final short VALUE_SUBTYPE = 11;
    public static final short CUR_SUBTYPE = 12;
    public static final short LATITUDE_SUBTYPE = 21;
    public static final short LONGITUDE_SUBTYPE = 22;
    public static final short SRC_SUBTYPE = 31;
    public static final short ALT_SUBTYPE = 32;
    public static final short TITLE_SUBTYPE = 33;
    private FieldKey fieldKey;
    private short subtype;

    public SubfieldKey(FieldKey fieldKey, short subtype) {
        this.fieldKey = fieldKey;
        this.subtype = subtype;
    }

    public FieldKey getFieldKey() {
        return fieldKey;
    }

    public short getSubtype() {
        return subtype;
    }

    @Override
    public int hashCode() {
        return fieldKey.hashCode() + subtype;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof SubfieldKey)) {
            return false;
        }
        SubfieldKey fk = (SubfieldKey) obj;
        if (fk.subtype != this.subtype) {
            return false;
        }
        return fk.fieldKey.equals(this.fieldKey);
    }

    @Override
    public String toString() {
        return fieldKey.toString() + ":" + subtypeToString(subtype);
    }

    public static SubfieldKey parse(String s) throws ParseException {
        int idx = s.indexOf(":");
        if (idx == -1) {
            throw new ParseException("No ':' character", 0);
        }
        FieldKey fieldKey = FieldKey.parse(s.substring(0, idx));
        try {
            short subtype = subtypeToShort(s.substring(idx + 1));
            return new SubfieldKey(fieldKey, subtype);
        } catch (IllegalArgumentException iae) {
            throw new ParseException("Wrong subtype = " + s.substring(idx + 1), idx = +1);
        }
    }

    /**
     * Équivalent de parse() mais en envoyant IllegalArgumentException plutôt que ParseException.
     * à utiliser quand on est sûr de la syntaxe et permet d'éviter un try {} catch {}
     */
    public static SubfieldKey build(String s) {
        try {
            return parse(s);
        } catch (ParseException pe) {
            throw new IllegalArgumentException(pe.getMessage());
        }
    }

    public static String subtypeToString(short subtype) {
        switch (subtype) {
            case NOM_SUBTYPE:
                return "nom";
            case PRENOM_SUBTYPE:
                return "prenom";
            case ORIGINAL_SUBTYPE:
                return "original";
            case NOMAVANT_SUBTYPE:
                return "nomavant";
            case VALUE_SUBTYPE:
                return "value";
            case CUR_SUBTYPE:
                return "cur";
            case LATITUDE_SUBTYPE:
                return "lat";
            case LONGITUDE_SUBTYPE:
                return "lon";
            case SRC_SUBTYPE:
                return "src";
            case ALT_SUBTYPE:
                return "alt";
            case TITLE_SUBTYPE:
                return "title";
            default:
                throw new IllegalArgumentException("Wrong subtype = " + subtype);
        }
    }

    public static short subtypeToShort(String subtype) {
        if (subtype.equals("nom")) {
            return NOM_SUBTYPE;
        } else if (subtype.equals("prenom")) {
            return PRENOM_SUBTYPE;
        } else if (subtype.equals("nomavant")) {
            return NOMAVANT_SUBTYPE;
        } else if (subtype.equals("original")) {
            return ORIGINAL_SUBTYPE;
        } else if (subtype.equals("value")) {
            return VALUE_SUBTYPE;
        } else if (subtype.equals("cur")) {
            return CUR_SUBTYPE;
        } else if (subtype.equals("lat")) {
            return LATITUDE_SUBTYPE;
        } else if (subtype.equals("lon")) {
            return LONGITUDE_SUBTYPE;
        } else if (subtype.equals("src")) {
            return SRC_SUBTYPE;
        } else if (subtype.equals("alt")) {
            return ALT_SUBTYPE;
        } else if (subtype.equals("title")) {
            return TITLE_SUBTYPE;
        }
        throw new IllegalArgumentException("Wrong subtype = " + subtype);
    }

    public static boolean isLegalSubfield(short subfieldType, int ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
                if (subfieldType < 10) {
                    return true;
                } else {
                    return false;
                }
            case CorpusField.MONTANT_FIELD:
                if ((subfieldType > 10) && (subfieldType < 20)) {
                    return true;
                } else {
                    return false;
                }
            case CorpusField.GEOPOINT_FIELD:
                if ((subfieldType > 20) && (subfieldType < 30)) {
                    return true;
                } else {
                    return false;
                }
            case CorpusField.IMAGE_FIELD:
                if ((subfieldType > 30) && (subfieldType < 40)) {
                    return true;
                } else {
                    return false;
                }
            default:
                return false;
        }
    }

}
