/* BasedefichesLib_API - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.croisement;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.RedacteurItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.Map;
import java.util.HashMap;
import net.mapeadores.basedefiches.sphere.Redacteur;


/**
 *
 * @author Vincent Calame
 */
public class CorpusSphereCroisementUtils {

    private CorpusSphereCroisementUtils() {
    }

    public static Map<SubsetKey, RedacteurCroisementList> getSphereCroisementListMap(Corpus corpus, Fiche fiche) {
        CroisementBuilder builder = new CroisementBuilder();
        builder.check(corpus, fiche);
        return builder.sphereMapOfMap;
    }


    private static class CroisementBuilder {

        Basedefiches basedefiches;
        Map<SubsetKey, RedacteurCroisementList> sphereMapOfMap = new HashMap<SubsetKey, RedacteurCroisementList>();

        public void check(Corpus corpus, Fiche fiche) {
            this.basedefiches = corpus.getBasedefiches();
            CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
            FicheItemList redacList = fiche.getRedacteurs();
            if (redacList != null) {
                updateRedacteurCroisement(1, redacList);
            }
            int proprieteCount = fiche.getProprieteCount();
            for (int i = 0; i < proprieteCount; i++) {
                Propriete propriete = fiche.getPropriete(i);
                CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(propriete.getFieldKey());
                if (corpusField != null) {
                    if (corpusField.getFicheItemType() == CorpusField.PERSONNE_FIELD) {
                        updateRedacteurCroisement(2, propriete.getFicheItem());
                    }
                }
            }
            int informationCount = fiche.getInformationCount();
            for (int i = 0; i < informationCount; i++) {
                Information information = fiche.getInformation(i);
                if (information.getFicheItemCount() > 0) {
                    FieldKey fieldKey = information.getFieldKey();
                    CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
                    if (corpusField != null) {
                        if (corpusField.getFicheItemType() == CorpusField.PERSONNE_FIELD) {
                            updateRedacteurCroisement(4, information);
                        }
                    }
                }
            }
        }

        private void updateRedacteurCroisement(int rank, FicheItemList list) {
            int count = list.getFicheItemCount();
            for (int i = 0; i < count; i++) {
                updateRedacteurCroisement(rank, list.getFicheItem(i));
            }
        }

        private void updateRedacteurCroisement(int rank, FicheItem ficheItem) {
            if (ficheItem == null) {
                return;
            }
            if (ficheItem instanceof RedacteurItem) {
                String redacteurKeyString = ((RedacteurItem) ficheItem).getRedacteurKeyString();
                if (redacteurKeyString != null) {
                    try {
                        RedacteurKey redacteurKey = RedacteurKey.parse(redacteurKeyString);
                        addRedacteurKey(redacteurKey, rank);
                    } catch (java.text.ParseException pe) {
                    }
                }
            }
        }

        private void addRedacteurKey(RedacteurKey redacteurKey, int mask) {
            SubsetKey sphereKey = redacteurKey.getSphereKey();
            RedacteurCroisementList rcl = (RedacteurCroisementList) sphereMapOfMap.get(sphereKey);
            if (rcl == null) {
                Sphere sphere = (Sphere) basedefiches.getSubset(sphereKey);
                if (sphere == null) {
                    return;
                }
                rcl = new RedacteurCroisementList(sphere);
                sphereMapOfMap.put(sphereKey, rcl);
            }
            rcl.add(redacteurKey, mask);
        }

    }


    private static class RedacteurCroisementList implements CroisementList {

        Sphere sphere;
        LinkedIntHashMap map = new LinkedIntHashMap();

        RedacteurCroisementList(Sphere sphere) {
            this.sphere = sphere;
        }

        private void add(RedacteurKey redacteurKey, int mask) {
            Redacteur redacteur = sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString());
            if (redacteur == null) {
                return;
            }
            int ticket = redacteur.getTicket();
            Integer itg = (Integer) map.get(ticket);
            if (itg == null) {
                map.put(ticket, new Integer(mask));
            } else {
                int nvmsk = mask | itg.intValue();
                map.put(ticket, new Integer(nvmsk));
            }
        }

        public int getSubsetItemCount() {
            return map.size();
        }

        public int getTicket(int index) {
            return map.getKeyAt(index);
        }

        public int getPoids(int index) {
            return ((Integer) map.getValueAt(index)).intValue();
        }

        public SubsetKey getSubsetKey() {
            return sphere.getSubsetKey();
        }

    }

}
