/* BasedefichesLib_API - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.croisement;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Illustration;


/**
 *
 * @author Vincent Calame
 */
public class MutableIllustrationCroisementList implements IllustrationCroisementList {

    private SubsetKey albumSubsetKey;
    private Illustration[] illustrationArray;
    private int[] poidsArray;
    private int size;

    public MutableIllustrationCroisementList(SubsetKey albumSubsetKey, int initialCapacity) {
        this.albumSubsetKey = albumSubsetKey;
        illustrationArray = new Illustration[initialCapacity];
        poidsArray = new int[initialCapacity];
    }

    public int getSubsetItemCount() {
        return size;
    }

    public Illustration getIllustration(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return illustrationArray[index];
    }

    public int getPoids(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException();
        }
        return poidsArray[index];
    }

    public int getTicket(int index) {
        return getIllustration(index).getTicket();
    }

    /**
     *
     *@throws IllegalArgumentException si poids < 1
     */
    public void add(Illustration illustration, int poids) {
        if (!illustration.getAlbum().getSubsetKey().equals(albumSubsetKey)) {
            throw new IllegalArgumentException("subsetKey must be the same");
        }
        if (poids < 1) {
            throw new IllegalArgumentException("poids < 1");
        }
        for (int i = 0; i < size; i++) {
            if (illustrationArray[i].equals(illustration)) {
                poidsArray[i] = Math.max(poidsArray[i], poids);
                return;
            }
        }
        checkLength();
        illustrationArray[size] = illustration;
        poidsArray[size] = poids;
        size++;
    }

    public void append(IllustrationCroisementList illustrationList) {
        if (!illustrationList.getSubsetKey().equals(albumSubsetKey)) {
            throw new IllegalArgumentException("subsetKey must be the same");
        }
        int currentSize = size;
        int otherSize = illustrationList.getSubsetItemCount();
        int newsize = currentSize + otherSize;
        if (newsize > illustrationArray.length) {
            extendsLength(newsize);
        }
        for (int i = 0; i < otherSize; i++) {
            Illustration otherIllustration = illustrationList.getIllustration(i);
            int otherPoids = illustrationList.getPoids(i);
            boolean here = false;
            for (int j = 0; j < currentSize; j++) {
                if (illustrationArray[j].equals(otherIllustration)) {
                    poidsArray[j] = Math.max(poidsArray[j], otherPoids);
                    here = true;
                    break;
                }
            }
            if (!here) {
                illustrationArray[size] = otherIllustration;
                poidsArray[size] = otherPoids;
                size++;
            }
        }
    }

    public SubsetKey getSubsetKey() {
        return albumSubsetKey;
    }

    private void checkLength() {
        if (size == illustrationArray.length) {
            extendsLength(Math.max(size * 2, 4));
        }
    }

    private void extendsLength(int newLength) {
        Illustration[] nv = new Illustration[newLength];
        System.arraycopy(illustrationArray, 0, nv, 0, size);
        illustrationArray = nv;
        int[] nvi = new int[newLength];
        System.arraycopy(poidsArray, 0, nvi, 0, size);
        poidsArray = nvi;
    }

}
