/* BasedefichesLib_API - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format;

import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.format.FormatDefBuilder;
import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.format.FormatDef;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesFormatDefBuilder extends FormatDefBuilder implements BasedefichesFormatConstants {

    private InternalBasedefichesFormatDef internalBasedefichesFormatDef = new InternalBasedefichesFormatDef();

    public BasedefichesFormatDefBuilder() {
    }

    public BasedefichesFormatDef toBasedefichesFormatDef() {
        if (internalBasedefichesFormatDef.getFormatSourceKeyCount() == 0) {
            throw new IllegalStateException("getFormatSourceKeyCount() == 0");
        }
        return internalBasedefichesFormatDef;
    }

    public void addSource(FormatSourceKey formatSourceKey) {
        internalBasedefichesFormatDef.addSource(formatSourceKey);
    }

    public void setDefaultProprieteKey(FieldKey defaultProprieteKey) {
        if ((defaultProprieteKey != null) && (!defaultProprieteKey.isProprieteFieldKey())) {
            throw new IllegalArgumentException("defaultProprieteKey is not a Propriete : " + defaultProprieteKey);
        }
        if (defaultProprieteKey == null) {
            remove(BDF_DEFAULTPROPRIETEKEY_PARAMETER);
        } else {
            put(BDF_DEFAULTPROPRIETEKEY_PARAMETER, defaultProprieteKey);
        }
    }

    public void setNoItem(boolean noItem) {
        put(BDF_NOITEM_PARAMETER, Boolean.valueOf(noItem));
    }

    protected void put(String paramName, Object paramValue) {
        internalBasedefichesFormatDef.put(paramName, paramValue);
    }

    protected void remove(String paramName) {
        internalBasedefichesFormatDef.remove(paramName);
    }

    protected void addInnerSeparator(FormatDef.InnerSeparator innerSeparator) {
        internalBasedefichesFormatDef.addInnerSeparator(innerSeparator);
    }

    protected void addSourceSeparator(FormatDef.SourceSeparator sourceSeparator) {
        internalBasedefichesFormatDef.addSourceSeparator(sourceSeparator);
    }

    protected void addCleanedFormatPattern(String formatPattern) {
        internalBasedefichesFormatDef.addCleanedFormatPattern(formatPattern);
    }


    private static class InternalBasedefichesFormatDef implements BasedefichesFormatDef {

        private Map<String, Object> paramMap = new HashMap<String, Object>();
        private List<FormatDef.InnerSeparator> innerSeparatorList = new ArrayList<FormatDef.InnerSeparator>();
        private List<String> formatPatternList = new ArrayList<String>();
        private List<FormatDef.SourceSeparator> sourceSeparatorList = new ArrayList<FormatDef.SourceSeparator>();
        private List<FormatSourceKey> formatSourceKeyList = new ArrayList<FormatSourceKey>();

        public Object getParameterValue(String paramName) {
            return paramMap.get(paramName);
        }

        public int getInnerSeparatorCount() {
            return innerSeparatorList.size();
        }

        public InnerSeparator getInnerSeparator(int i) {
            return innerSeparatorList.get(i);
        }

        public int getFormatPatternCount() {
            return formatPatternList.size();
        }

        public String getFormatPattern(int i) {
            return formatPatternList.get(i);
        }

        public int getSourceSeparatorCount() {
            return sourceSeparatorList.size();
        }

        public SourceSeparator getSourceSeparator(int i) {
            return sourceSeparatorList.get(i);
        }

        public int getFormatSourceKeyCount() {
            return formatSourceKeyList.size();
        }

        public FormatSourceKey getFormatSourceKey(int i) {
            return formatSourceKeyList.get(i);
        }

        private void put(String paramName, Object paramValue) {
            paramMap.put(paramName, paramValue);
        }

        private void remove(String paramName) {
            paramMap.remove(paramName);
        }

        private void addInnerSeparator(FormatDef.InnerSeparator innerSeparator) {
            innerSeparatorList.add(innerSeparator);
        }

        private void addSourceSeparator(FormatDef.SourceSeparator sourceSeparator) {
            sourceSeparatorList.add(sourceSeparator);
        }

        private void addCleanedFormatPattern(String formatPattern) {
            formatPatternList.add(formatPattern);
        }

        private void addSource(FormatSourceKey formatSourceKey) {
            formatSourceKeyList.add(formatSourceKey);
        }

    }

}
