/* BasedefichesLib_API - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format;

import java.io.IOException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.format.FormatConstants;
import net.mapeadores.util.format.FormatDef;
import net.mapeadores.util.format.FormatUtils;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesFormatUtils implements FormatConstants, BasedefichesFormatConstants {

    private BasedefichesFormatUtils() {
    }

    public static boolean isNoItem(FormatDef formatDef) {
        Object value = formatDef.getParameterValue(BDF_NOITEM_PARAMETER);
        if (value == null) {
            return false;
        }
        return ((Boolean) value).booleanValue();
    }

    public static FieldKey getDefaultProprieteKey(FormatDef formatDef) {
        return (FieldKey) formatDef.getParameterValue(BDF_DEFAULTPROPRIETEKEY_PARAMETER);
    }

    public static String toLines(BasedefichesFormatDef formatDef) {
        StringBuilder buf = new StringBuilder();
        try {
            appendLines(formatDef, buf);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static void appendLines(BasedefichesFormatDef formatDef, Appendable appendable) throws IOException {
        String[] array = BasedefichesFormatUtils.toStringArray(formatDef);
        appendable.append(array[0]);
        appendable.append('\n');
        String format = array[1];
        String param = array[2];
        if (format == null) {
            if (param != null) {
                appendable.append("-\n");
            }
        } else {
            appendable.append(format);
            appendable.append('\n');
        }
        if (param != null) {
            appendable.append(param);
            appendable.append('\n');
        }
    }

    public static String[] toStringArray(BasedefichesFormatDef formatDef) {
        StringBuilder buf = new StringBuilder();
        int size = formatDef.getFormatSourceKeyCount();
        for (int i = 0; i < size; i++) {
            if (i > 0) {
                buf.append(',');
            }
            buf.append(formatDef.getFormatSourceKey(i).toString());
        }
        String[] result = new String[3];
        result[0] = buf.toString();
        result[1] = formatPatternListToString(formatDef);
        result[2] = paramToString(formatDef);
        return result;
    }

    public static String getFormatPatternBySourceIndex(BasedefichesFormatDef formatDef, int sourceIndex) {
        int count = formatDef.getFormatPatternCount();
        if (count == 0) {
            return null;
        }
        if (sourceIndex >= count) {
            return getFormatPattern(formatDef, count - 1);
        } else {
            return getFormatPattern(formatDef, sourceIndex);
        }
    }

    public static String getFormatPattern(BasedefichesFormatDef formatDef, int i) {
        String formatPattern = formatDef.getFormatPattern(i);
        if (formatPattern != null) {
            return formatPattern;
        }
        if (i == 0) {
            return null;
        }
        return getFormatPattern(formatDef, i - 1);
    }

    public static String formatPatternListToString(BasedefichesFormatDef formatDef) {
        int count = formatDef.getFormatPatternCount();
        if (count == 0) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < count; i++) {
            String fp = formatDef.getFormatPattern(i);
            if (i > 0) {
                buf.append("||");
            }
            if (fp == null) {
                buf.append(' ');
            } else {
                buf.append(fp);
            }
        }
        return buf.toString();
    }

    public static String paramToString(BasedefichesFormatDef formatDef) {
        ParamStringBuilder stringBuilder = new ParamStringBuilder(formatDef);
        String result = stringBuilder.toString();
        if (result.length() == 0) {
            return null;
        }
        return result;
    }

    public static boolean isSame(BasedefichesFormatDef formatDef1, BasedefichesFormatDef formatDef2) {
        String stg1 = toLines(formatDef1);
        String stg2 = toLines(formatDef2);
        return stg1.equals(stg2);
    }


    private static class ParamStringBuilder {

        StringBuilder buf;
        boolean suite = false;
        BasedefichesFormatDef formatDef;

        ParamStringBuilder(BasedefichesFormatDef formatDef) {
            this.formatDef = formatDef;
            buf = new StringBuilder();
            initCast();
            initPosition();
            initLast();
            initLimit();
            initSeparator();
            initDefaultValue();
            initconvertEmptyToNull();
            int innerSeparatorCount = formatDef.getInnerSeparatorCount();
            for (int i = 0; i < innerSeparatorCount; i++) {
                FormatDef.InnerSeparator innerSeparator = formatDef.getInnerSeparator(i);
                testSuite();
                buf.append("sep");
                buf.append(innerSeparator.getSourceIndex() + 1);
                buf.append('=');
                appendString(innerSeparator.getSeparator());
            }
            int sourceSeparatorCount = formatDef.getSourceSeparatorCount();
            for (int i = 0; i < sourceSeparatorCount; i++) {
                FormatDef.SourceSeparator sourceSeparator = formatDef.getSourceSeparator(i);
                testSuite();
                buf.append("sep");
                buf.append(sourceSeparator.getSourceIndex1() + 1);
                buf.append('_');
                buf.append(sourceSeparator.getSourceIndex2() + 1);
                buf.append('=');
                appendString(sourceSeparator.getSeparator());
            }
            Object fieldKeyValue = formatDef.getParameterValue(BDF_DEFAULTPROPRIETEKEY_PARAMETER);
            if (fieldKeyValue != null) {
                testSuite();
                buf.append("defprop=");
                buf.append(((FieldKey) fieldKeyValue).toString());
            }
            Object noItemValue = formatDef.getParameterValue(BDF_NOITEM_PARAMETER);
            if (noItemValue != null) {
                if (((Boolean) noItemValue).booleanValue()) {
                    testSuite();
                    buf.append("noitem");
                }
            }
        }

        public String toString() {
            return buf.toString();
        }

        private void initDefaultValue() {
            Object value = formatDef.getParameterValue(DEFAULTVALUE_PARAMETER);
            if (value != null) {
                testSuite();
                buf.append("defval=");
                appendString((String) value);
            }
        }

        private void initCast() {
            Object value = formatDef.getParameterValue(CAST_PARAMETER);
            if (value != null) {
                testSuite();
                buf.append("cast=");
                buf.append(FormatUtils.castTypeToString((Short) value));
            }
        }

        private void initLimit() {
            Object value = formatDef.getParameterValue(LIMIT_PARAMETER);
            if (value != null) {
                testSuite();
                buf.append("limit=");
                buf.append(((Integer) value).toString());
            }
        }

        private void initPosition() {
            Object value = formatDef.getParameterValue(POSITION_PARAMETER);
            if (value != null) {
                testSuite();
                int position = ((Integer) value).intValue();
                buf.append("pos=");
                if (position == FormatConstants.MAX_POSITION) {
                    buf.append("last");
                } else {
                    buf.append(position + 1);
                }
            }
        }

        private void initLast() {
            Object value = formatDef.getParameterValue(FILLBYLAST_PARAMETER);
            if (value != null) {
                if (((Boolean) value).booleanValue()) {
                    Object positionValue = formatDef.getParameterValue(POSITION_PARAMETER);
                    if ((positionValue == null) || (((Integer) positionValue).intValue() != FormatConstants.MAX_POSITION)) {
                        testSuite();
                        buf.append("last");
                    }
                }
            }
        }

        private void initconvertEmptyToNull() {
            Object value = formatDef.getParameterValue(EMPTYTONULL_PARAMETER);
            if (value != null) {
                if (((Boolean) value).booleanValue()) {
                    testSuite();
                    buf.append("null");
                }
            }
        }

        private void initSeparator() {
            Object value = formatDef.getParameterValue(SEPARATOR_PARAMETER);
            if (value != null) {
                String separator = (String) value;
                testSuite();
                buf.append("sep=");
                if (separator.equals("\n")) {
                    appendString("NL");
                } else {
                    appendString(separator);
                }
            }
        }

        private void appendString(String s) {
            if (s.length() == 0) {
                buf.append("\"\"");
            } else if (StringUtils.isValidAsciiString(s)) {
                buf.append(s);
            } else {
                buf.append('\"');
                StringUtils.escapeDoubleQuote(s, buf);
                buf.append('\"');
            }
        }

        private void testSuite() {
            if (suite) {
                buf.append(',');
            } else {
                suite = true;
            }
        }

    }

}
