/* BasedefichesLib_API - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public final class FormatSourceKey {

    private FieldKey fieldKey;
    private SubfieldKey subfieldKey;
    private IncludeKey includeKey;
    private SubsetKey slaveCorpusKey;
    private ThesaurusFieldKey thesaurusFieldKey;
    private FieldKey typePrefixFieldKey;
    public final static FormatSourceKey DATE_CREATION = new FormatSourceKey(IncludeKey.DATE_CREATION);
    public final static FormatSourceKey DATE_MODIFICATION = new FormatSourceKey(IncludeKey.DATE_MODIFICATION);

    private FormatSourceKey() {
    }

    private FormatSourceKey(IncludeKey includeKey) {
        this.includeKey = includeKey;
    }

    public static FormatSourceKey newFieldKeyInstance(FieldKey fieldKey) {
        return newFieldKeyInstance(fieldKey, null);
    }

    public static FormatSourceKey newFieldKeyInstance(FieldKey fieldKey, SubsetKey slaveCorpusKey) {
        if (fieldKey == null) {
            throw new IllegalArgumentException("fieldKey is null");
        }
        FormatSourceKey formatSourceKey = new FormatSourceKey();
        formatSourceKey.fieldKey = fieldKey;
        formatSourceKey.slaveCorpusKey = slaveCorpusKey;
        return formatSourceKey;
    }

    public static FormatSourceKey newSubfieldKeyInstance(SubfieldKey subfieldKey) {
        return newSubfieldKeyInstance(subfieldKey, null);
    }

    public static FormatSourceKey newSubfieldKeyInstance(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey) {
        if (subfieldKey == null) {
            throw new IllegalArgumentException("subfieldKey is null");
        }
        FormatSourceKey formatSourceKey = new FormatSourceKey();
        formatSourceKey.subfieldKey = subfieldKey;
        formatSourceKey.slaveCorpusKey = slaveCorpusKey;
        return formatSourceKey;
    }

    public static FormatSourceKey newIncludeKeyInstance(IncludeKey includeKey) {
        return newIncludeKeyInstance(includeKey, null);
    }

    public static FormatSourceKey newIncludeKeyInstance(IncludeKey includeKey, SubsetKey slaveCorpusKey) {
        if (includeKey == null) {
            throw new IllegalArgumentException("includeKey is null");
        }
        FormatSourceKey formatSourceKey = new FormatSourceKey();
        formatSourceKey.includeKey = includeKey;
        formatSourceKey.slaveCorpusKey = slaveCorpusKey;
        return formatSourceKey;
    }

    public static FormatSourceKey newThesaurusFieldKeyInstance(ThesaurusFieldKey thesaurusFieldKey) {
        if (thesaurusFieldKey == null) {
            throw new IllegalArgumentException("thesaurusFieldKey is null");
        }
        FormatSourceKey formatSourceKey = new FormatSourceKey();
        formatSourceKey.thesaurusFieldKey = thesaurusFieldKey;
        return formatSourceKey;
    }

    public static FormatSourceKey newTypePrefixFieldKeyInstance(FieldKey typePrefixFieldKey) {
        return newTypePrefixFieldKeyInstance(typePrefixFieldKey, null);
    }

    public static FormatSourceKey newTypePrefixFieldKeyInstance(FieldKey typePrefixFieldKey, SubsetKey slaveCorpusKey) {
        if (typePrefixFieldKey == null) {
            throw new IllegalArgumentException("typePrefixFieldKey is null");
        }
        if (typePrefixFieldKey.isSpecialFieldKey()) {
            throw new IllegalArgumentException("typePrefixFieldKey is a special field: " + typePrefixFieldKey.toString());
        }
        FormatSourceKey formatSourceKey = new FormatSourceKey();
        formatSourceKey.typePrefixFieldKey = typePrefixFieldKey;
        formatSourceKey.slaveCorpusKey = slaveCorpusKey;
        return formatSourceKey;
    }

    public boolean isFieldKey() {
        return (fieldKey != null);
    }

    public boolean isSubfieldKey() {
        return (subfieldKey != null);
    }

    public boolean isIncludeKey() {
        return (includeKey != null);
    }

    public boolean isThesaurusFieldKey() {
        return (thesaurusFieldKey != null);
    }

    public boolean isTypePrefixFieldKey() {
        return (typePrefixFieldKey != null);
    }

    public FieldKey getFieldKey() {
        return fieldKey;
    }

    public SubfieldKey getSubfieldKey() {
        return subfieldKey;
    }

    public IncludeKey getIncludeKey() {
        return includeKey;
    }

    public boolean fromSlaveCorpusKey() {
        return (slaveCorpusKey != null);
    }

    public SubsetKey getSlaveCorpusKey() {
        return slaveCorpusKey;
    }

    public ThesaurusFieldKey getThesaurusFieldKey() {
        return thesaurusFieldKey;
    }

    public FieldKey getTypePrefixFieldKey() {
        return typePrefixFieldKey;
    }

    @Override
    public String toString() {
        StringBuilder buf = new StringBuilder();
        if (slaveCorpusKey != null) {
            buf.append(slaveCorpusKey.toString());
            buf.append('/');
        }
        if (fieldKey != null) {
            buf.append(fieldKey.toString());
        } else if (subfieldKey != null) {
            buf.append(subfieldKey.toString());
        } else if (includeKey != null) {
            buf.append(includeKey.toString());
        } else if (thesaurusFieldKey != null) {
            buf.append(thesaurusFieldKey.toString());
        } else if (typePrefixFieldKey != null) {
            buf.append(typePrefixFieldKey.toString());
            buf.append('*');
        } else {
            throw new SwitchException("All key are null !");
        }
        return buf.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj == this) {
            return true;
        }
        if (obj.getClass() != this.getClass()) {
            return false;
        }
        FormatSourceKey other = (FormatSourceKey) obj;
        if (slaveCorpusKey != null) {
            if (other.fromSlaveCorpusKey()) {
                return slaveCorpusKey.equals(other.slaveCorpusKey);
            } else {
                return false;
            }
        } else if (other.fromSlaveCorpusKey()) {
            return false;
        }
        if (fieldKey != null) {
            if (other.isFieldKey()) {
                return fieldKey.equals(other.fieldKey);
            } else {
                return false;
            }
        } else if (subfieldKey != null) {
            if (other.isSubfieldKey()) {
                return subfieldKey.equals(other.subfieldKey);
            } else {
                return false;
            }
        } else if (includeKey != null) {
            if (other.isIncludeKey()) {
                return includeKey.equals(other.includeKey);
            } else {
                return false;
            }
        } else if (thesaurusFieldKey != null) {
            if (other.isThesaurusFieldKey()) {
                return thesaurusFieldKey.equals(other.thesaurusFieldKey);
            } else {
                return false;
            }
        } else if (typePrefixFieldKey != null) {
            if (other.isTypePrefixFieldKey()) {
                return typePrefixFieldKey.equals(other.typePrefixFieldKey);
            } else {
                return false;
            }
        } else {
            throw new SwitchException("All key are null !");
        }
    }

    @Override
    public int hashCode() {
        int hashCode = 0;
        if (slaveCorpusKey != null) {
            hashCode = slaveCorpusKey.hashCode();
        }
        if (fieldKey != null) {
            hashCode = hashCode + fieldKey.hashCode();
        } else if (subfieldKey != null) {
            hashCode = hashCode + subfieldKey.hashCode();
        } else if (includeKey != null) {
            hashCode = hashCode + includeKey.hashCode();
        } else if (thesaurusFieldKey != null) {
            hashCode = hashCode + thesaurusFieldKey.hashCode();
        } else if (typePrefixFieldKey != null) {
            hashCode = hashCode + typePrefixFieldKey.hashCode();
        } else {
            throw new SwitchException("All key are null !");
        }
        return hashCode;
    }

}
