/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.InsertionHandler;
import net.mapeadores.util.text.insertpattern.api.InstructionErrorHandler;
import net.mapeadores.util.text.insertpattern.api.CharacterErrorException;
import net.mapeadores.util.text.insertpattern.impl.InsertpatternParser;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import java.util.Locale;
import java.util.ArrayList;
import java.io.StringReader;
import java.io.IOException;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractFormat {

    private FormatPart[] formatPartArray;

    public AbstractFormat() {
    }

    public static Locale getFormatLocale(LocalisationInfo localisationInfo) {
        if (localisationInfo == null) {
            return (new LocalisationInfo(LangInteger.fromLocale(Locale.getDefault()))).getFormatLocale();
        } else {
            return localisationInfo.getFormatLocale();
        }
    }

    public static int getWorkingLangInteger(LocalisationInfo localisationInfo) {
        if (localisationInfo == null) {
            return (new LocalisationInfo(LangInteger.fromLocale(Locale.getDefault()))).getWorkingLangInteger();
        } else {
            return localisationInfo.getWorkingLangInteger();
        }
    }

    public void setPattern(String s) throws ParseException {
        FormatBuilder formatBuilder = new FormatBuilder(this);
        InsertpatternParser parser = new InsertpatternParser();
        parser.setInsertionHandler(formatBuilder);
        parser.setErrorHandler(formatBuilder);
        try {
            StringReader reader = new StringReader(s);
            parser.parse(reader);
        } catch (IOException ioe) {
            throw new ShouldNotOccurException();
        } catch (CharacterErrorException cee) {
            throw new ParseException(s, cee.getCol());
        }
        this.formatPartArray = formatBuilder.finish();
    }

    /**
     * Retourne UnsupportedOperationException si l'objet ne peut ê
     * LocalisationInfo peut être nul.
     */
    public abstract String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException;

    protected String processFormat(ObjectFormatPartResolver objectFormatPartResolver) {
        StringBuilder buf = new StringBuilder();
        int length = formatPartArray.length;
        for (int i = 0; i < length; i++) {
            FormatPart formatPart = formatPartArray[i];
            short category = formatPart.getCategory();
            switch (category) {
                case FormatPart.STRING_CATEGORY:
                    buf.append(formatPart.getString());
                    break;
                case FormatPart.OBJECT_CATEGORY:
                    try {
                        objectFormatPartResolver.append(buf, formatPart);
                    } catch (IOException ioe) {
                    }
                    break;
            }
        }
        return buf.toString();
    }


    public static interface ObjectFormatPartResolver {

        public void append(Appendable buf, FormatPart formatPart) throws IOException;

    }

    public abstract FormatPart convertToFormatPart(Instruction[] instructionArray);


    static class FormatBuilder implements InsertionHandler, InstructionErrorHandler {

        ArrayList<FormatPart> formatPartList = new ArrayList<FormatPart>();
        AbstractFormat abstractFormat;

        FormatBuilder(AbstractFormat abstractFormat) {
            this.abstractFormat = abstractFormat;
        }

        public void appendString(String s) {
            formatPartList.add(new FormatPart(s));
        }

        public void doInsertion(Instruction[] instructionArray) {
            FormatPart formatPart = abstractFormat.convertToFormatPart(instructionArray);
            if (formatPart != null) {
                formatPartList.add(formatPart);
            } else {
                String firstKey = instructionArray[0].getArgument(0).getKey();
                String error = "#ERR : {" + firstKey + " ... }#";
                formatPartList.add(new FormatPart(error));
            }

        }

        public void startIf(String clause, boolean unique) {
        }

        public void startElseIf(String clause) {
        }

        public void startElse() {
        }

        public void endIf(boolean unique, boolean onElse) {
        }

        public void characterError(char carac, String key, int row, int col) {
            throw new CharacterErrorException(carac, key, row, col);
        }

        public void warning(String key, int row, int col) {
        }

        FormatPart[] finish() {
            int size = formatPartList.size();
            FormatPart[] fpa = new FormatPart[size];
            for (int i = 0; i < size; i++) {
                fpa[i] = (FormatPart) formatPartList.get(i);
            }
            abstractFormat = null;
            formatPartList = null;
            return fpa;
        }

    }

}
