/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Courriel;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * {addr}, {value}, {name}
 *
 * @author Vincent Calame
 */
public class CourrielFormat extends FicheItemFormat {

    private static final int VALUE_SUBTYPE = 1;
    private static final int ADDR_SUBTYPE = 2;
    private static final int NAME_SUBTYPE = 3;

    public CourrielFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Courriel) {
            return formatCourriel((Courriel) obj);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatCourriel(Courriel courriel) {
        if (courriel == null) {
            return null;
        }
        return processFormat(new CourrielResolver(courriel));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("value")) {
            return new FormatPart(VALUE_SUBTYPE, null);
        } else if (key.equals("addr")) {
            return new FormatPart(ADDR_SUBTYPE, null);
        } else if (key.equals("name")) {
            return new FormatPart(NAME_SUBTYPE, null);
        } else {
            return null;
        }
    }


    private class CourrielResolver implements ObjectFormatPartResolver {

        private CourrielBean courrielBean;

        CourrielResolver(Courriel courriel) {
            this.courrielBean = courriel.toCourrielBean();
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case VALUE_SUBTYPE:
                    buf.append(courrielBean.toString());
                    break;
                case ADDR_SUBTYPE:
                    buf.append(courrielBean.getAddrSpec());
                    break;
                case NAME_SUBTYPE:
                    buf.append(courrielBean.getNomComplet());
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
