/* BasedefichesLib_API - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.exceptions.SwitchException;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
public abstract class FicheItemFormat extends AbstractFormat {

    public FicheItemFormat() {
    }

    /**
     * Fait appel à format(Object obj) mais n'envoie pas d'exception UnsupportedObjectException, retourne null en lieu et place.
     */
    public String formatFicheItem(FicheItem ficheItem, LocalisationInfo localisationInfo) {
        if (ficheItem == null) {
            return null;
        }
        try {
            return format(ficheItem, localisationInfo);
        } catch (UnsupportedObjectException uoe) {
            return null;
        }
    }

    /**
     *
     * Ne pas oublier d'ajouter l'instance de Basedefiches si c'est un formatage de personnes.
     */
    public static FicheItemFormat newInstance(int ficheItemType, String formatPattern) throws ParseException {
        if (formatPattern == null) {
            return newDefaultInstance(ficheItemType);
        }
        FicheItemFormat ficheItemFormat = null;
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
                ficheItemFormat = new PersonneFormat();
                break;
            case CorpusField.LANGUE_FIELD:
                ficheItemFormat = new CodeFormat();
                break;
            case CorpusField.DATATION_FIELD:
                ficheItemFormat = new DatationFormat();
                break;
            case CorpusField.PAYS_FIELD:
                ficheItemFormat = new CodeFormat();
                break;
            case CorpusField.COURRIEL_FIELD:
                ficheItemFormat = new CourrielFormat();
                break;
            case CorpusField.LINK_FIELD:
                ficheItemFormat = new LinkFormat();
                break;
            case CorpusField.NOMBRE_FIELD:
                ficheItemFormat = new MontantFormat();
                break;
            case CorpusField.MONTANT_FIELD:
                ficheItemFormat = new MontantFormat();
                break;
            case CorpusField.ITEM_FIELD:
                ficheItemFormat = new ValueFormat();
                break;
            case CorpusField.GEOPOINT_FIELD:
                ficheItemFormat = new GeopointFormat();
                break;
            case CorpusField.PARA_FIELD:
                ficheItemFormat = new ParaFormat();
                break;
            case CorpusField.IMAGE_FIELD:
                ficheItemFormat = new ImageFormat();
                break;
            default:
                throw new SwitchException("ficheItemType = " + ficheItemType);
        }
        ficheItemFormat.setPattern(formatPattern);
        return ficheItemFormat;
    }

    public static FicheItemFormat newDefaultInstance(int ficheItemType) {
        try {
            return newInstance(ficheItemType, getDefaultPattern(ficheItemType));
        } catch (ParseException pe) {
            throw new ShouldNotOccurException(pe);
        }
    }

    public static String getDefaultPattern(int ficheItemType) {
        switch (ficheItemType) {
            case CorpusField.PERSONNE_FIELD:
                return "{normal}";
            case CorpusField.LANGUE_FIELD:
                return "{code}";
            case CorpusField.DATATION_FIELD:
                return "{code}";
            case CorpusField.PAYS_FIELD:
                return "{code}";
            case CorpusField.NOMBRE_FIELD:
                return "{code}";
            case CorpusField.MONTANT_FIELD:
                return "{code}";
            case CorpusField.COURRIEL_FIELD:
                return "{addr}";
            case CorpusField.LINK_FIELD:
                return "{href}";
            case CorpusField.ITEM_FIELD:
                return "{value}";
            case CorpusField.GEOPOINT_FIELD:
                return "{lat} {lon}";
            case CorpusField.PARA_FIELD:
                return "{raw}";
            case CorpusField.IMAGE_FIELD:
                return "{src}";
            default:
                throw new SwitchException("ficheItemType = " + ficheItemType);
        }
    }

}
