/* BasedefichesLib_API - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.primitives.DegreSexagesimal;
import net.mapeadores.util.primitives.PrimL10nUtils;
import net.mapeadores.util.localisation.LocalisationInfo;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class GeopointFormat extends FicheItemFormat {

    private static final int LAT_SUBTYPE = 1;
    private static final int LATLIB_SUBTYPE = 2;
    private static final int LON_SUBTYPE = 3;
    private static final int LONLIB_SUBTYPE = 4;

    public GeopointFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Geopoint) {
            return formatGeopoint((Geopoint) obj, localisationInfo);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatGeopoint(Geopoint geopoint, LocalisationInfo localisationInfo) {
        if (geopoint == null) {
            return null;
        }
        return processFormat(new GeopointResolver(geopoint, localisationInfo));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("lat")) {
            return new FormatPart(LAT_SUBTYPE, null);
        } else if (key.equals("latlib")) {
            return new FormatPart(LATLIB_SUBTYPE, new LibParameters(instructionArray));
        } else if (key.equals("lon")) {
            return new FormatPart(LON_SUBTYPE, null);
        } else if (key.equals("lonlib")) {
            return new FormatPart(LONLIB_SUBTYPE, new LibParameters(instructionArray));
        } else {
            return null;
        }
    }


    private class GeopointResolver implements ObjectFormatPartResolver {

        Geopoint geopoint;
        LocalisationInfo localisationInfo;

        GeopointResolver(Geopoint geopoint, LocalisationInfo localisationInfo) {
            this.geopoint = geopoint;
            this.localisationInfo = localisationInfo;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case LAT_SUBTYPE:
                    buf.append(geopoint.getLatitude().toString());
                    break;
                case LATLIB_SUBTYPE:
                    appendLib(buf, geopoint.getLatitude(), formatPart, true);
                    break;
                case LON_SUBTYPE:
                    buf.append(geopoint.getLongitude().toString());
                    break;
                case LONLIB_SUBTYPE:
                    appendLib(buf, geopoint.getLongitude(), formatPart, false);
                    break;
            }
        }

        private void appendLib(Appendable buf, DegreDecimal degreDecimal, FormatPart formatPart, boolean latitude) throws IOException {
            LibParameters libParameters = (LibParameters) formatPart.getObject();
            DegreSexagesimal sexa = DegreSexagesimal.fromDegreDecimal(degreDecimal);
            int[] array = libParameters.getlangIntegerArray();
            if (array == null) {
                buf.append(PrimL10nUtils.getLib(sexa, getFormatLocale(localisationInfo), latitude));
            } else {
                for (int i = 0; i < array.length; i++) {
                    if (i > 0) {
                        buf.append(libParameters.getSeparator());
                    }
                    buf.append(PrimL10nUtils.getLib(sexa, LangInteger.getLocale(array[i]), latitude));
                }
            }
        }

    }

}
