/* BasedefichesLib_API - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class ImageFormat extends FicheItemFormat {

    private static final int SRC_SUBTYPE = 1;
    private static final int ALT_SUBTYPE = 2;
    private static final int TITLE_SUBTYPE = 3;

    public ImageFormat() {
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Image) {
            return formatImage((Image) obj);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatImage(Image image) {
        if (image == null) {
            return null;
        }
        return processFormat(new ImageResolver(image));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        if (key.equals("src")) {
            return new FormatPart(SRC_SUBTYPE, null);
        } else if (key.equals("alt")) {
            return new FormatPart(ALT_SUBTYPE, null);
        } else if (key.equals("title")) {
            return new FormatPart(TITLE_SUBTYPE, null);
        } else {
            return null;
        }
    }


    private class ImageResolver implements ObjectFormatPartResolver {

        Image image;

        ImageResolver(Image image) {
            this.image = image;
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case SRC_SUBTYPE:
                    buf.append(image.getSrc());
                    break;
                case ALT_SUBTYPE:
                    buf.append(image.getAlt());
                    break;
                case TITLE_SUBTYPE:
                    buf.append(image.getTitle());
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
