/* BasedefichesLib_API - Copyright (c) 2007-2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.patterns;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.sphere.Idsphere;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.exceptions.SwitchException;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class PersonneFormat extends FicheItemFormat {

    private Basedefiches basedefiches;
    private static final int NOM_SUBTYPE = 1;
    private static final int PRENOM_SUBTYPE = 2;
    private static final int ORIGINAL_SUBTYPE = 3;
    private static final int NOMAVANT_SUBTYPE = 4;
    private static final int ORGANISME_SUBTYPE = 5;
    private static final int STYLE_SUBTYPE = 6;
    private static final int IDSPHERE_SUBTYPE = 7;
    private static final int SPHERE_SUBTYPE = 8;
    private static final int NOM_UP_SUBTYPE = 9;
    private static final int CODE_SUBTYPE = 10;

    public PersonneFormat() {
    }

    public void setBasedefiches(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    public String format(Object obj, LocalisationInfo localisationInfo) throws UnsupportedObjectException {
        if (obj == null) {
            return null;
        }
        if (obj instanceof Personne) {
            return formatPersonne((Personne) obj);
        } else if (obj instanceof Item) {
            return formatItem((Item) obj);
        }
        throw new UnsupportedObjectException(obj);
    }

    public String formatPersonne(Personne personne) {
        if (personne == null) {
            return null;
        }
        return processFormat(new PersonneResolver(personne));
    }

    public String formatItem(Item item) {
        if (item == null) {
            return null;
        }
        return item.getValue();
    }

    public FormatPart convertToFormatPart(Instruction[] instructionArray) {
        Instruction inst1 = instructionArray[0];
        Argument arg1 = inst1.getArgument(0);
        String key = arg1.getKey();
        try {
            short style = PersonneBean.styleToShort(key);
            return new FormatPart(STYLE_SUBTYPE, new Short(style));
        } catch (IllegalArgumentException iae) {
        }
        if (key.equals("nom")) {
            return new FormatPart(NOM_SUBTYPE, null);
        } else if (key.equals("prenom")) {
            return new FormatPart(PRENOM_SUBTYPE, null);
        } else if (key.equals("original")) {
            return new FormatPart(ORIGINAL_SUBTYPE, null);
        } else if (key.equals("nomavant")) {
            return new FormatPart(NOMAVANT_SUBTYPE, null);
        } else if (key.equals("organisme")) {
            return new FormatPart(ORGANISME_SUBTYPE, null);
        } else if (key.equals("sphere")) {
            return new FormatPart(SPHERE_SUBTYPE, null);
        } else if (key.equals("idsphere")) {
            return new FormatPart(IDSPHERE_SUBTYPE, null);
        } else if (key.equals("nom_up")) {
            return new FormatPart(NOM_UP_SUBTYPE, null);
        } else if (key.equals("code")) {
            return new FormatPart(CODE_SUBTYPE, null);
        } else {
            return null;
        }
    }


    private class PersonneResolver implements ObjectFormatPartResolver {

        PersonneBean personneBean;
        String organisme;
        RedacteurKey redacteurKey;

        PersonneResolver(Personne personne) {
            if (personne.isRedacteurSphere()) {
                try {
                    redacteurKey = RedacteurKey.parse(personne.getRedacteurKeyString());
                    PersonneBean pb = SphereUtils.getPersonneBean(basedefiches, redacteurKey);
                    if (pb == null) {
                        pb = new PersonneBean();
                        pb.setNom(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                    }
                    this.personneBean = pb;
                } catch (java.text.ParseException pe) {
                    redacteurKey = null;
                    personneBean = new PersonneBean();
                    personneBean.setNom(personne.getRedacteurKeyString());
                }
            } else {
                personneBean = personne.getPersonneBean();
                organisme = personne.getOrganisme();
            }
        }

        public void append(Appendable buf, FormatPart formatPart) throws IOException {
            int subType = formatPart.getSubType();
            switch (subType) {
                case NOM_SUBTYPE:
                    buf.append(personneBean.getNom());
                    break;
                case NOM_UP_SUBTYPE:
                    buf.append(personneBean.getNom().toUpperCase());
                    break;
                case PRENOM_SUBTYPE:
                    buf.append(personneBean.getPrenom());
                    break;
                case NOMAVANT_SUBTYPE:
                    boolean b = personneBean.isNomAvant();
                    buf.append((b) ? "1" : "0");
                    break;
                case ORIGINAL_SUBTYPE:
                    buf.append(personneBean.getOriginal());
                    break;
                case ORGANISME_SUBTYPE:
                    if (organisme != null) {
                        buf.append(organisme);
                    }
                    break;
                case IDSPHERE_SUBTYPE:
                    if (redacteurKey != null) {
                        buf.append(redacteurKey.getIdsphereString());
                    } else {
                        buf.append("?idsphere?");
                    }
                    break;
                case SPHERE_SUBTYPE:
                    if (redacteurKey != null) {
                        SubsetKey sphereKey = redacteurKey.getSphereKey();
                        if (sphereKey != null) {
                            buf.append(sphereKey.getSubsetName());
                        }
                    } else {
                        buf.append("?sphere?");
                    }
                    break;
                case STYLE_SUBTYPE:
                    Short style = (Short) formatPart.getObject();
                    buf.append(personneBean.getNomComplet(style.shortValue()));
                    break;
                case CODE_SUBTYPE:
                    if (redacteurKey != null) {
                        buf.append(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                    } else {
                        buf.append(personneBean.getNomComplet(PersonneBean.NORMAL_STYLE));
                    }
                    break;
                default:
                    throw new SwitchException("formatPart.getSubType() = " + subType);
            }
        }

    }

}
