/* BasedefichesLib_API - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.include;

import net.mapeadores.util.primitives.Range;
import net.mapeadores.util.primitives.RangeList;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class IncludeSummaryBuilder {

    private InternalIncludeSummary internal = new InternalIncludeSummary();

    public IncludeSummaryBuilder() {
    }

    public IncludeSummary getIncludeSummary() {
        return internal;
    }

    public void append(IncludeKey includeKey) {
        internal.add(includeKey);
    }


    private class InternalIncludeSummary implements IncludeSummary {

        List<InternalSubsetInfo> corpusList = new ArrayList<InternalSubsetInfo>();
        List<InternalSubsetInfo> thesaurusList = new ArrayList<InternalSubsetInfo>();
        List<InternalSubsetInfo> albumList = new ArrayList<InternalSubsetInfo>();
        Map<SubsetKey, InternalSubsetInfo> subsetInfoMap = new HashMap<SubsetKey, InternalSubsetInfo>();

        public int getSubsetInfoCount(short type) {
            switch (type) {
                case CORPUS_TYPE:
                    return corpusList.size();
                case THESAURUS_TYPE:
                    return thesaurusList.size();
                case ALBUM_TYPE:
                    return albumList.size();
                default:
                    throw new IllegalArgumentException("Unknown Type");
            }
        }

        public SubsetInfo getSubsetInfo(short type, int i) {
            switch (type) {
                case CORPUS_TYPE:
                    return corpusList.get(i);
                case THESAURUS_TYPE:
                    return thesaurusList.get(i);
                case ALBUM_TYPE:
                    return albumList.get(i);
                default:
                    throw new IllegalArgumentException("Unknown Type");
            }
        }

        public SubsetInfo getSubsetInfo(SubsetKey subsetKey) {
            return subsetInfoMap.get(subsetKey);
        }

        private void add(IncludeKey includeKey) {
            SubsetKey subsetKey = includeKey.getSubsetKey();
            if (subsetKey == null) {
                return;
            }
            if ((!subsetKey.isThesaurusSubset()) && (!subsetKey.isCorpusSubset()) && (!subsetKey.isAlbumSubset())) {
                return;
            }
            InternalSubsetInfo subsetInfo = subsetInfoMap.get(subsetKey);
            if (subsetInfo == null) {
                subsetInfo = new InternalSubsetInfo(subsetKey);
                subsetInfoMap.put(subsetKey, subsetInfo);
                if (subsetKey.isThesaurusSubset()) {
                    thesaurusList.add(subsetInfo);
                } else if (subsetKey.isCorpusSubset()) {
                    corpusList.add(subsetInfo);
                }
            }
            subsetInfo.add(includeKey.getPoidsFilter());
        }


        private class InternalSubsetInfo implements SubsetInfo {

            private boolean anyPoids = false;
            private RangeList rangeList = null;
            private SubsetKey subsetKey;

            private InternalSubsetInfo(SubsetKey subsetKey) {
                this.subsetKey = subsetKey;
            }

            public SubsetKey getSubsetKey() {
                return subsetKey;
            }

            public boolean hasPoidsFilter() {
                return (rangeList != null);
            }

            public boolean hasAnyPoidsInclude() {
                return anyPoids;
            }

            public RangeList getPoidsFilterRangeList() {
                return rangeList;
            }

            private void add(int poidsFilter) {
                if (poidsFilter < 1) {
                    anyPoids = true;
                } else {
                    Range range = new Range(poidsFilter, poidsFilter);
                    if (rangeList == null) {
                        rangeList = new RangeList();
                    }
                    rangeList.addRange(range);
                }
            }

        }

    }

}
