/* BasedefichesLib_API - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.l10n;

import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.List;
import java.util.Properties;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.text.ChangeLibelleHolderBuilder;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesL10n {

    private static final Map<String, LibelleHolder> libelleHolderMap;

    static {
        List<Object> keyList = new ArrayList<Object>();
        keyList.add(FieldKey.IDCORPUS);
        keyList.add(FieldKey.LANG);
        keyList.add(FieldKey.TITRE);
        keyList.add(FieldKey.SOUSTITRE);
        keyList.add(FieldKey.REDACTEURS);
        keyList.add(IncludeKey.ASSOCIATION);
        keyList.add(IncludeKey.LIAGE);
        keyList.add(IncludeKey.RATTACHEMENT);
        keyList.add(IncludeKey.DATE_CREATION);
        keyList.add(IncludeKey.DATE_MODIFICATION);
        Initializer initializer = new Initializer(keyList);
        initializer.initLang("fr");
        initializer.initLang("es");
        initializer.initLang("en");
        initializer.initLang("zh");
        initializer.initLang("pt");
        libelleHolderMap = initializer.toMap();
    }

    private BasedefichesL10n() {
    }

    public static LibelleHolder getSpecialFieldKeyLibelle(FieldKey fieldKey) {
        if (!fieldKey.isSpecialFieldKey()) {
            throw new IllegalArgumentException("fieldKey is not a special key");
        }
        LibelleHolder libelleHolder = libelleHolderMap.get(fieldKey.toString());
        if (libelleHolder == null) {
            throw new IllegalArgumentException("libelleList not defined for fieldKey = " + fieldKey.toString());
        }
        return libelleHolder;
    }

    public static Libelle getLibelle(IncludeKey includeKey, int langInteger) {
        LibelleHolder libelleHolder = libelleHolderMap.get(includeKey.toString());
        if (libelleHolder == null) {
            return null;
        }
        Libelle libelle = libelleHolder.getLibelleByLang(langInteger);
        if (libelle != null) {
            return libelle;
        }
        int langPart = LangInteger.getLangPart(langInteger);
        if (langPart != langInteger) {
            return libelleHolder.getLibelleByLang(langInteger);
        }
        return null;
    }


    private static class Initializer {

        private Map<String, ChangeLibelleHolderBuilder> builderMap = new HashMap<String, ChangeLibelleHolderBuilder>();

        private Initializer(List<Object> keyList) {
            for (Object key : keyList) {
                String keyString = key.toString();
                builderMap.put(key.toString(), new ChangeLibelleHolderBuilder(keyString));
            }
        }

        private void initLang(String langString) {
            int langInteger = 0;
            try {
                langInteger = LangInteger.parse(langString);
            } catch (LangIntegerException lie) {
                throw new InternalResourceException(lie);
            }
            Properties properties = new Properties();
            try {
                properties.load(BasedefichesL10n.class.getResourceAsStream("basedefiches_" + langString + ".properties"));
            } catch (IOException ioe) {
                throw new InternalResourceException(ioe);
            }
            for (Map.Entry<String, ChangeLibelleHolderBuilder> entry : builderMap.entrySet()) {
                String key = entry.getKey();
                String libString = properties.getProperty(key);
                CleanedString cleanedString = CleanedString.newInstance(libString);
                if (cleanedString == null) {
                    continue;
                }
                entry.getValue().putLibelle(LibelleUtils.toLibelle(langInteger, cleanedString));
            }
        }

        private Map<String, LibelleHolder> toMap() {
            Map<String, LibelleHolder> result = new HashMap<String, LibelleHolder>();
            for (Map.Entry<String, ChangeLibelleHolderBuilder> entry : builderMap.entrySet()) {
                result.put(entry.getKey(), entry.getValue().toChangeLibelleHolder());
            }
            return result;
        }

    }

}
