/* BasedefichesLib_API - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.sphere;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.List;
import java.util.ArrayList;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurStats {

    private RedacteurKey redacteurKey;
    private Sphere sphere;
    private List<ByCorpusStats> byCorpusStatsList = new ArrayList<ByCorpusStats>();
    private int count = 0;

    public RedacteurStats(Redacteur redacteur) {
        this(redacteur.getSphere(), redacteur.getRedacteurKey());
    }

    public RedacteurStats(Sphere sphere, RedacteurKey redacteurKey) {
        this.sphere = sphere;
        this.redacteurKey = redacteurKey;
        init();
    }

    public boolean isEmpty() {
        return byCorpusStatsList.isEmpty();
    }

    public int getFicheCount() {
        return count;
    }

    private void init() {
        Basedefiches basedefiches = sphere.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            List<CorpusField> personneList = CorpusMetadataUtils.getCorpusFieldListByFicheItemType(corpus.getCorpusMetadata(), CorpusField.PERSONNE_FIELD);
            ByCorpusStats byCorpusStats = null;
            if (personneList.size() == 1) {
                byCorpusStats = initRedacteur(corpus);
            } else {
                byCorpusStats = initRedacteurAndFields(corpus, personneList);
            }
            if (byCorpusStats != null) {
                byCorpusStatsList.add(byCorpusStats);
                count = count + byCorpusStats.getFicheCount();
            }
        }
    }

    private ByCorpusStats initRedacteur(Corpus corpus) {
        ByCorpusStats byCorpusStats = null;
        int length = corpus.getTicketLength();
        for (int i = 0; i < length; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
            if (abregeFiche != null) {
                if (abregeFiche.hasRedacteur(redacteurKey)) {
                    if (byCorpusStats == null) {
                        byCorpusStats = new ByCorpusStats(corpus);
                    }
                    byCorpusStats.add(abregeFiche.getIdcorpus(), 1);
                }
            }
        }
        return byCorpusStats;
    }

    private ByCorpusStats initRedacteurAndFields(Corpus corpus, List<CorpusField> personneList) {
        ByCorpusStats byCorpusStats = null;
        int size = personneList.size();
        int length = corpus.getTicketLength();
        for (int i = 0; i < length; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
            if (abregeFiche != null) {
                int poids = 0;
                if (abregeFiche.hasRedacteur(redacteurKey)) {
                    poids = 1;
                }
                if (size > 1) {
                    Fiche fiche = corpus.getEnteteFiche(abregeFiche);
                    for (int j = 1; j < size; j++) {
                        CorpusField corpusField = personneList.get(j);
                        FieldKey fieldKey = corpusField.getFieldKey();
                        if (fieldKey.isProprieteFieldKey()) {
                            Propriete propriete = fiche.getPropriete(fieldKey);
                            if (propriete != null) {
                                boolean here = testFicheItem(propriete.getFicheItem());
                                if (here) {
                                    poids = poids | 2;
                                }
                            }
                        } else if (fieldKey.isInformationFieldKey()) {
                            Information information = fiche.getInformation(fieldKey);
                            if (information != null) {
                                int size3 = information.getFicheItemCount();
                                for (int k = 0; k < size3; k++) {
                                    boolean here = testFicheItem(information.getFicheItem(k));
                                    if (here) {
                                        poids = poids | 4;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
                if (poids > 0) {
                    if (byCorpusStats == null) {
                        byCorpusStats = new ByCorpusStats(corpus);
                    }
                    byCorpusStats.add(abregeFiche.getIdcorpus(), poids);
                }
            }
        }
        return byCorpusStats;
    }

    private boolean testFicheItem(FicheItem ficheItem) {
        if (!(ficheItem instanceof Personne)) {
            return false;
        }
        String redacteurKeyString = ((Personne) ficheItem).getRedacteurKeyString();
        if (redacteurKeyString == null) {
            return false;
        }
        try {
            RedacteurKey prk = RedacteurKey.parse(redacteurKeyString);
            return prk.equals(redacteurKey);
        } catch (java.text.ParseException pe) {
            return false;
        }
    }


    public static class ByCorpusStats {

        private Corpus corpus;
        private SortedMap<Integer, ByFicheStats> treeMap = new TreeMap<Integer, ByFicheStats>();

        ByCorpusStats(Corpus corpus) {
            this.corpus = corpus;
        }

        public int getFicheCount() {
            return treeMap.size();
        }

        public Corpus getCorpus() {
            return corpus;
        }

        private void add(int idcorpus, int poids) {
            ByFicheStats byFicheStats = new ByFicheStats(idcorpus, poids);
            treeMap.put(new Integer(idcorpus), byFicheStats);
        }

        public List<ByFicheStats> getByFicheStatsList() {
            return new ArrayList<ByFicheStats>(treeMap.values());
        }

    }


    public static class ByFicheStats {

        private int idcorpus;
        private int poids;

        ByFicheStats(int idcorpus, int poids) {
            this.idcorpus = idcorpus;
            this.poids = poids;
        }

        public int getIdcorpus() {
            return idcorpus;
        }

        public int getPoids() {
            return poids;
        }

    }

    public List<ByCorpusStats> getByCorpusStatsList() {
        return byCorpusStatsList;
    }

}
