/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.conversion;

import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.corpus.fiche.Courriel;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlock;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Image;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Link;
import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.fiche.Nombre;
import net.mapeadores.basedefiches.corpus.fiche.P;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.Pays;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.fiche.S;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.DateInteger;


/**
 *
 * @author Vincent Calame
 */
public class ConversionUtils {

    private ConversionUtils() {
    }

    public static FicheBlock toFicheBlock(FicheItem ficheItem) {
        P prg = new P(P.TYPE_NORMAL);
        if (ficheItem instanceof Item) {
            prg.addText(((Item) ficheItem).getValue());
        } else if (ficheItem instanceof Para) {
            Para para = (Para) ficheItem;
            int textContentCount = para.getTextContentCount();
            for (int i = 0; i < textContentCount; i++) {
                Object obj = para.getTextContent(i);
                if (obj instanceof String) {
                    prg.addText((String) obj);
                } else if (obj instanceof S) {
                    prg.addS((S) obj);
                }
            }
        } else if (ficheItem instanceof Pays) {
            prg.addText(((Pays) ficheItem).getCountryCode());
        } else if (ficheItem instanceof Langue) {
            prg.addText(LangInteger.toString(((Langue) ficheItem).getLangInteger()));
        } else if (ficheItem instanceof Datation) {
            prg.addText(DateInteger.toISOString(((Datation) ficheItem).getDateInteger()));
        } else if (ficheItem instanceof Geopoint) {
            Geopoint geopoint = (Geopoint) ficheItem;
            prg.addText(geopoint.getLatitude().toString() + " " + geopoint.getLongitude().toString());
        } else if (ficheItem instanceof Image) {
            Image image = (Image) ficheItem;
            S span = new S(S.TYPE_IMG);
            span.setSrc(image.getSrc());
            span.setValue(image.getAlt());
            prg.addS(span);
        } else if (ficheItem instanceof Link) {
            Link link = (Link) ficheItem;
            S span = new S(S.TYPE_LINK);
            span.setRef(link.getHref());
            span.setValue(link.getTitle());
            prg.addS(span);
        } else if (ficheItem instanceof Courriel) {
            CourrielBean courrielBean = ((Courriel) ficheItem).toCourrielBean();
            S span = new S(S.TYPE_LINK);
            span.setRef("mailto:" + courrielBean.getAddrSpec());
            span.setValue(courrielBean.getNomComplet());
            prg.addS(span);
        } else if (ficheItem instanceof Montant) {
            Montant montant = (Montant) ficheItem;
            prg.addText(CurrencyInteger.toString(montant.getCurrencyInteger()) + " " + montant.getDecimal().toString());
        } else if (ficheItem instanceof Nombre) {
            prg.addText(((Nombre) ficheItem).getDecimal().toString());
        } else if (ficheItem instanceof Personne) {
            Personne personne = (Personne) ficheItem;
            if (personne.isRedacteurSphere()) {
                prg.addText(personne.getRedacteurKeyString());
            } else {
                prg.addText(personne.getPersonneBean().getNomComplet(PersonneBean.NORMAL_STYLE));
            }
        } else {
            prg.addText("Unknown : " + ficheItem.getClass().getName());
        }
        return prg;
    }

}
