/* BasedefichesLib_Explore - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.conversion;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;


/**
 *
 * @author Vincent Calame
 */
public abstract class FicheConverter {

    MutableProcessLog mutableProcessLog = new MutableProcessLog();

    public FicheConverter() {
    }

    public static FicheConverter getInstance(CorpusField source, CorpusField destination) {
        return getInstance(source.getFieldKey(), destination.getFieldKey());
    }

    public static FicheConverter getInstance(FieldKey sourceKey, FieldKey destinationKey) {
        int type = 0;
        if (destinationKey.isFicheItemField(0)) {
            type = 1;
        } else if (destinationKey.ownsToCorpsdefiche()) {
            type = 2;
        } else {
            throw new IllegalArgumentException("wrong destination : " + destinationKey.toString());
        }
        if (sourceKey.isProprieteFieldKey()) {
            if (type == 1) {
                return new ProprieteToFicheItemField(sourceKey, destinationKey);
            } else if (type == 2) {
                return new FicheItemFieldToFicheBlockField(sourceKey, destinationKey);
            }
        } else if ((sourceKey.isInformationFieldKey()) || (sourceKey.equals(FieldKey.REDACTEURS))) {
            if (type == 1) {
                return new InformationToFicheItemField(sourceKey, destinationKey);
            } else if (type == 2) {
                return new FicheItemFieldToFicheBlockField(sourceKey, destinationKey);
            }
        } else if (sourceKey.isSectionFieldKey()) {
            if (type == 1) {
                return new SectionToFicheItemField(sourceKey, destinationKey);
            } else if (type == 2) {
                return new SectionToFicheBlockField(sourceKey, destinationKey);
            }
        } else if (sourceKey.isSpecialFieldKey()) {
            if (type == 1) {
                return new SimpleToFicheItemField(sourceKey, destinationKey);
            } else {
                throw new IllegalArgumentException("No section Key : " + destinationKey.toString());
            }
        }
        throw new SwitchException("wrong category : " + sourceKey.toString());
    }

    public void convert(AbregeFiche abregeFiche, Fiche fiche) {
        convert(abregeFiche, fiche, false);
    }

    public void convert(AbregeFiche abregeFiche, Fiche fiche, boolean removeOldField) {
        String uri = abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "/" + abregeFiche.getIdcorpus();
        mutableProcessLog.setCurrentURI(ProcessURI.fromString(uri));
        convertContent(abregeFiche, fiche, removeOldField);
    }

    public ProcessLog getProcessLog() {
        return mutableProcessLog;
    }

    abstract void convertContent(AbregeFiche abregeFiche, Fiche fiche, boolean removeOldField);

    void addLog(String category, String key, Object object) {
        ProcessLogItem logItem = mutableProcessLog.newLogItemInstance();
        logItem.setKey(key);
        logItem.setCategory(category);
        logItem.setObject(object);
    }

}
