/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.conversion;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;


/**
 *
 * @author Vincent Calame
 */
public class FicheItemFieldToFicheBlockField extends FicheConverter {

    private FieldKey sourceKey;
    private FieldKey destinationKey;

    public FicheItemFieldToFicheBlockField(FieldKey sourceKey, FieldKey destinationKey) {
        this.sourceKey = sourceKey;
        this.destinationKey = destinationKey;
    }

    void convertContent(AbregeFiche abregeFiche, Fiche fiche, boolean removeOldField) {
        FicheBlockList list = null;
        if (sourceKey.isProprieteFieldKey()) {
            list = getProprieteFicheBlockList(fiche);
        } else {
            list = getInformationFicheBlockList(fiche);
        }
        if (list == null) {
            return;
        }
        fiche.appendSection(destinationKey, list);
        if (removeOldField) {
            if (sourceKey.isProprieteFieldKey()) {
                fiche.setPropriete(sourceKey, null);
            } else {
                fiche.setInformation(sourceKey, null);
            }
        }
    }

    private FicheBlockList getProprieteFicheBlockList(Fiche fiche) {
        Propriete propriete = fiche.getPropriete(sourceKey);
        if (propriete == null) {
            return null;
        }
        FicheItem ficheItem = propriete.getFicheItem();
        if (ficheItem == null) {
            return null;

        }
        MutableFicheBlockList mutableFicheBlockList = new MutableFicheBlockList();
        mutableFicheBlockList.addFicheBlock(ConversionUtils.toFicheBlock(ficheItem));
        return mutableFicheBlockList;
    }

    private FicheBlockList getInformationFicheBlockList(Fiche fiche) {
        Information information = fiche.getInformation(sourceKey);
        if (information == null) {
            return null;
        }
        MutableFicheBlockList mutableFicheBlockList = new MutableFicheBlockList();
        int ficheItemCount = information.getFicheItemCount();
        for (int i = 0; i < ficheItemCount; i++) {
            mutableFicheBlockList.addFicheBlock(ConversionUtils.toFicheBlock(information.getFicheItem(i)));
        }
        return mutableFicheBlockList;

    }

}
