/* BasedefichesLib_Explore - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.conversion;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.fiche.CdataDiv;
import net.mapeadores.basedefiches.corpus.fiche.Code;
import net.mapeadores.basedefiches.corpus.fiche.Div;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlock;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Li;
import net.mapeadores.basedefiches.corpus.fiche.P;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.ParagraphBlock;
import net.mapeadores.basedefiches.corpus.fiche.S;
import net.mapeadores.basedefiches.corpus.fiche.Table;
import net.mapeadores.basedefiches.corpus.fiche.Tr;
import net.mapeadores.basedefiches.corpus.fiche.Ul;


/**
 *
 * @author Vincent Calame
 */
public class SectionToFicheItemField extends FicheConverter {

    private FieldKey sourceKey;
    private FieldKey destinationKey;

    public SectionToFicheItemField(FieldKey sourceKey, FieldKey destinationKey) {
        this.sourceKey = sourceKey;
        this.destinationKey = destinationKey;
    }

    void convertContent(AbregeFiche abregeFiche, Fiche fiche, boolean removeOldField) {
        FicheBlockList list = fiche.getSection(sourceKey);
        if ((list == null) || (list.getFicheBlockCount() == 0)) {
            return;
        }
        MutableFicheItemList ficheItemList = new MutableFicheItemList();
        boolean onlyP = appendPara(ficheItemList, list);
        if (!onlyP) {
            addLog("WRN", "warning_contains_notonly_p", null);
        }
        int size = ficheItemList.getFicheItemCount();
        if (size == 0) {
            return;
        }
        if (destinationKey.isProprieteFieldKey()) {
            FicheItem ficheItem = ficheItemList.getFicheItem(0);
            fiche.setPropriete(destinationKey, ficheItem);
            if (ficheItemList.getFicheItemCount() > 1) {
                addLog("WRN", "warning_ficheblock_size", sourceKey.toString());
            }
        } else {
            fiche.appendInformation(destinationKey, ficheItemList);
        }
        if (removeOldField) {
            fiche.setSection(sourceKey, null);
        }
    }

    private boolean appendPara(MutableFicheItemList ficheItemList, FicheBlockList ficheBlockList) {
        boolean onlyP = true;
        int size = ficheBlockList.getFicheBlockCount();
        for (int i = 0; i < size; i++) {
            FicheBlock ficheBlock = ficheBlockList.getFicheBlock(i);
            if (!(ficheBlock instanceof P)) {
                onlyP = false;
            }
            if (ficheBlock instanceof ParagraphBlock) {
                Para para = toPara((ParagraphBlock) ficheBlock);
                if (para != null) {
                    ficheItemList.addFicheItem(para);
                }
            } else if (ficheBlock instanceof Div) {
                appendPara(ficheItemList, ((Div) ficheBlock).getFicheBlockList());
            } else if (ficheBlock instanceof CdataDiv) {
                String s = ((CdataDiv) ficheBlock).getCdata();
                if (s.length() > 0) {
                    Para.Builder builder = new Para.Builder();
                    builder.addText(s);
                    ficheItemList.addFicheItem(builder.toPara());
                }
            } else if (ficheBlock instanceof Ul) {
                Ul ul = (Ul) ficheBlock;
                int liCount = ul.getLiCount();
                for (int j = 0; j < liCount; j++) {
                    Li li = ul.getLi(j);
                    appendPara(ficheItemList, li);
                }
            } else if (ficheBlock instanceof Code) {
                appendPara(ficheItemList, (Code) ficheBlock);
            } else if (ficheBlock instanceof Table) {
                Table table = (Table) ficheBlock;
                int trCount = table.getTrCount();
                for (int j = 0; j < trCount; j++) {
                    Tr tr = table.getTr(j);
                    appendPara(ficheItemList, tr);
                }
            }
        }
        return onlyP;
    }

    private Para toPara(ParagraphBlock paragraphBlock) {
        Para.Builder builder = new Para.Builder();
        int count = paragraphBlock.getTextContentCount();
        if (count == 0) {
            return null;
        }
        for (int i = 0; i < count; i++) {
            Object obj = paragraphBlock.getTextContent(i);
            if (obj instanceof S) {
                builder.addS((S) obj);
            } else if (obj instanceof String) {
                builder.addText((String) obj);
            }
        }
        return builder.toPara();
    }

}
