/* BasedefichesLib_Explore - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.conversion;

import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class SimpleToFicheItemField extends FicheConverter {

    private FieldKey destinationKey;
    private FieldKey sourceKey;

    public SimpleToFicheItemField(FieldKey sourceKey, FieldKey destinationKey) {
        this.destinationKey = destinationKey;
        this.sourceKey = sourceKey;
    }

    void convertContent(AbregeFiche abregeFiche, Fiche fiche, boolean removeOldField) {
        FicheItem ficheItem = getFicheItem(abregeFiche, fiche);
        if (ficheItem == null) {
            return;
        }
        if (destinationKey.isProprieteFieldKey()) {
            fiche.setPropriete(destinationKey, ficheItem);
        } else {
            fiche.appendInformation(destinationKey, new MutableFicheItemList(ficheItem));
        }
        if (removeOldField) {
            if (sourceKey.equals(FieldKey.SOUSTITRE)) {
                fiche.setSoustitre(null);
            } else if (sourceKey.equals(FieldKey.TITRE)) {
                fiche.setTitre("");
            }
        }
    }

    private FicheItem getFicheItem(AbregeFiche abregeFiche, Fiche fiche) {
        if (sourceKey.equals(FieldKey.TITRE)) {
            return getItem(fiche.getTitre());
        } else if (sourceKey.equals(FieldKey.SOUSTITRE)) {
            return fiche.getSoustitre();
        } else if (sourceKey.equals(FieldKey.LANG)) {
            int langInteger = fiche.getLangInteger();
            Langue langue = new Langue(langInteger);
            return langue;
        } else if (sourceKey.equals(FieldKey.IDCORPUS)) {
            return new Item(String.valueOf(abregeFiche.getIdcorpus()));
        } else {
            throw new SwitchException("sourceKey = " + sourceKey.toString());
        }
    }

    private Item getItem(String s) {
        if (s.length() == 0) {
            return null;
        }
        return new Item(s);
    }

}
