/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.extraction.api.DocumentFilter;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;


/**
 *
 * @author Vincent Calame
 */
public class AddendaExtractor {

    private Map<SubsetKey, AddendaExtractBundle> addendaKeyMap = new HashMap<SubsetKey, AddendaExtractBundle>();
    private List<AddendaExtractBundle> addendaList = new ArrayList<AddendaExtractBundle>();
    private ExtraitAddendaDef extraitAddendaDef;

    public AddendaExtractor(ExtraitAddendaDef extraitAddendaDef) {
        this.extraitAddendaDef = extraitAddendaDef;
    }

    public ExtraitAddendaDef getExtraitAddendaDef() {
        return extraitAddendaDef;
    }

    public void add(Document document, int poids) {
        if (poids < 0) {
            poids = 0;
        }
        Addenda addenda = document.getAddenda();
        AddendaExtractBundle bundle = addendaKeyMap.get(addenda.getSubsetKey());
        DocumentFilter documentFilter = extraitAddendaDef.getDocumentFilter();
        if (bundle == null) {
            bundle = new AddendaExtractBundle(addenda);
            addendaKeyMap.put(addenda.getSubsetKey(), bundle);
            addendaList.add(bundle);
        }
        bundle.add(new InternalDocumentExtractInfo(document, documentFilter, poids));
    }

    public boolean containsDocument() {
        return (addendaKeyMap.size() != 0);
    }

    public int getAddendaCount() {
        return addendaList.size();
    }

    public Addenda getAddenda(int index) {
        AddendaExtractBundle bundle = addendaList.get(index);
        return bundle.addenda;
    }

    public List<DocumentExtractInfo> getDocumentExtractInfoList(int index) {
        AddendaExtractBundle bundle = addendaList.get(index);
        return bundle.infoList;
    }


    private class AddendaExtractBundle {

        private Addenda addenda;
        private List<DocumentExtractInfo> infoList = new ArrayList<DocumentExtractInfo>();

        private AddendaExtractBundle(Addenda addenda) {
            this.addenda = addenda;
        }

        private void add(DocumentExtractInfo info) {
            infoList.add(info);
        }

    }


    private static class InternalDocumentExtractInfo implements DocumentExtractInfo {

        private Document document;
        private DocumentFilter documentFilter;
        private int poids = 0;

        public InternalDocumentExtractInfo(Document document, DocumentFilter documentFilter, int poids) {
            this.document = document;
            this.documentFilter = documentFilter;
            this.poids = poids;
        }

        public Document getDocument() {
            return document;
        }

        public DocumentFilter getDocumentFilter() {
            return documentFilter;
        }

        public int getPoids() {
            return poids;
        }

    }

}
