/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.IllustrationFilter;


/**
 *
 * @author Vincent Calame
 */
public class AlbumExtractor {

    private Map<SubsetKey, AlbumExtractBundle> albumKeyMap = new HashMap<SubsetKey, AlbumExtractBundle>();
    private List<AlbumExtractBundle> albumList = new ArrayList<AlbumExtractBundle>();
    private ExtraitAlbumDef extraitAlbumDef;

    public AlbumExtractor(ExtraitAlbumDef extraitAlbumDef) {
        this.extraitAlbumDef = extraitAlbumDef;
    }

    public ExtraitAlbumDef getExtraitAlbumDef() {
        return extraitAlbumDef;
    }

    public void add(Illustration illustration, int poids) {
        if (poids < 0) {
            poids = 0;
        }
        Album album = illustration.getAlbum();
        AlbumExtractBundle bundle = albumKeyMap.get(album.getSubsetKey());
        IllustrationFilter illustrationFilter = extraitAlbumDef.getIllustrationFilter();
        if (bundle == null) {
            bundle = new AlbumExtractBundle(album);
            albumKeyMap.put(album.getSubsetKey(), bundle);
            albumList.add(bundle);
        }
        bundle.add(new InternalIllustrationExtractInfo(illustration, illustrationFilter, poids));
    }

    public boolean containsIllustration() {
        return (albumKeyMap.size() != 0);
    }

    public int getAlbumCount() {
        return albumList.size();
    }

    public Album getAlbum(int index) {
        AlbumExtractBundle bundle = albumList.get(index);
        return bundle.album;
    }

    public List<IllustrationExtractInfo> getIllustrationExtractInfoList(int index) {
        AlbumExtractBundle bundle = albumList.get(index);
        return bundle.infoList;
    }


    private class AlbumExtractBundle {

        private Album album;
        private List<IllustrationExtractInfo> infoList = new ArrayList<IllustrationExtractInfo>();

        private AlbumExtractBundle(Album album) {
            this.album = album;
        }

        private void add(IllustrationExtractInfo info) {
            infoList.add(info);
        }

    }


    private static class InternalIllustrationExtractInfo implements IllustrationExtractInfo {

        private Illustration illustration;
        private IllustrationFilter illustrationFilter;
        private int poids = 0;

        public InternalIllustrationExtractInfo(Illustration illustration, IllustrationFilter illustrationFilter, int poids) {
            this.illustration = illustration;
            this.illustrationFilter = illustrationFilter;
            this.poids = poids;
        }

        public Illustration getIllustration() {
            return illustration;
        }

        public IllustrationFilter getIllustrationFilter() {
            return illustrationFilter;
        }

        public int getPoids() {
            return poids;
        }

    }

}
