/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.misc.LinkAnalyser;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionContextUtils {

    private static final FicheEligibility allFicheEligibility = new AllFicheEligibility();

    private ExtractionContextUtils() {
    }

    public static ExtractionContext newExtractionContext(Basedefiches basedefiches, LocalisationLists localisationLists) {
        return new InternalExtractionContext(basedefiches, localisationLists, null, null);
    }

    public static ExtractionContext newExtractionContext(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
        return new InternalExtractionContext(basedefiches, localisationLists, ficheEligibility, null);
    }

    public static ExtractionContext newExtractionContext(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility, FicheEligibility staticFicheEligibility) {
        return new InternalExtractionContext(basedefiches, localisationLists, ficheEligibility, staticFicheEligibility);
    }


    private static class InternalExtractionContext implements ExtractionContext {

        private Basedefiches basedefiches;
        private LocalisationLists localisationLists;
        private FicheEligibility ficheEligibility;
        private FicheEligibility staticFicheEligibility;

        private InternalExtractionContext(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility, FicheEligibility staticFicheEligibility) {
            this.basedefiches = basedefiches;
            this.localisationLists = localisationLists;
            this.ficheEligibility = ficheEligibility;
            this.staticFicheEligibility = staticFicheEligibility;
        }

        public Basedefiches getBasedefiches() {
            return basedefiches;
        }

        public LocalisationLists getLocalisationLists() {
            return localisationLists;
        }

        public FicheEligibility getFicheEligibility() {
            return ficheEligibility;
        }

        public FicheEligibility getStaticFicheEligibility() {
            return staticFicheEligibility;
        }

        public LinkAnalyser getLinkAnalyser() {
            return null;
        }

    }


    private static class AllFicheEligibility implements FicheEligibility {

        public boolean accept(AbregeFiche abregeFiche) {
            return true;
        }

    }

}
