/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction;

import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusExtractor {

    private Map<SubsetKey, ThesaurusExtractBundle> thesaurusKeyMap = new HashMap<SubsetKey, ThesaurusExtractBundle>();
    private List<ThesaurusExtractBundle> thesaurusList = new ArrayList<ThesaurusExtractBundle>();
    private Subset filterSubset;
    private ExtraitThesaurusDef extraitThesaurusDef;

    public ThesaurusExtractor(ExtraitThesaurusDef extraitThesaurusDef, Subset filterSubset) {
        this.filterSubset = filterSubset;
        this.extraitThesaurusDef = extraitThesaurusDef;
    }

    public ExtraitThesaurusDef getExtraitThesaurusDef() {
        return extraitThesaurusDef;
    }

    public void add(Motcle motcle, int poids) {
        if (poids < 0) {
            poids = 0;
        }
        Thesaurus thesaurus = motcle.getThesaurus();
        ThesaurusExtractBundle bundle = thesaurusKeyMap.get(thesaurus.getSubsetKey());
        MotcleFilter motcleFilter = extraitThesaurusDef.getMotcleFilter();
        if (bundle == null) {
            bundle = new ThesaurusExtractBundle(thesaurus);
            thesaurusKeyMap.put(thesaurus.getSubsetKey(), bundle);
            thesaurusList.add(bundle);
        }
        bundle.add(new InternalMotcleExtractInfo(motcle, motcleFilter, poids));
    }

    public boolean containsMotcle() {
        return (thesaurusKeyMap.size() != 0);
    }

    public int getThesaurusCount() {
        return thesaurusList.size();
    }

    public Thesaurus getThesaurus(int index) {
        ThesaurusExtractBundle bundle = thesaurusList.get(index);
        return bundle.thesaurus;
    }

    public String getRattachementType(int index) {
        ThesaurusExtractBundle bundle = thesaurusList.get(index);
        return bundle.rattachementType;
    }

    public List<MotcleExtractInfo> getMotcleExtractInfoList(int index) {
        ThesaurusExtractBundle bundle = thesaurusList.get(index);
        return bundle.infoList;
    }


    private class ThesaurusExtractBundle {

        private String rattachementType = null;
        private Thesaurus thesaurus;
        private List<MotcleExtractInfo> infoList = new ArrayList<MotcleExtractInfo>();

        private ThesaurusExtractBundle(Thesaurus thesaurus) {
            this.thesaurus = thesaurus;
            if (filterSubset != null) {
                Subset masterSubset = filterSubset.getMasterSubset();
                if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
                    rattachementType = "master";
                }
            }
        }

        private void add(MotcleExtractInfo info) {
            infoList.add(info);
        }

    }


    private static class InternalMotcleExtractInfo implements MotcleExtractInfo {

        private Motcle motcle;
        private MotcleFilter motcleFilter;
        private int poids;

        private InternalMotcleExtractInfo(Motcle motcle, MotcleFilter motcleFilter, int poids) {
            this.motcle = motcle;
            this.motcleFilter = motcleFilter;
            this.poids = poids;
        }

        public Motcle getMotcle() {
            return motcle;
        }

        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        public int getPoids() {
            return poids;
        }

    }

}
