/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.extraction.api.CorpusExtractorList;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtractionSource;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDefList;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.api.GroupClause;
import net.mapeadores.basedefiches.extraction.api.IllustrationFilter;
import net.mapeadores.basedefiches.extraction.api.IntituleClause;
import net.mapeadores.basedefiches.extraction.api.PackClause;
import net.mapeadores.basedefiches.extraction.api.ThesaurusExtractorList;
import net.mapeadores.basedefiches.selection.BufferedFicheEligibility;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelectOptionsList;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptionsList;
import net.mapeadores.basedefiches.selection.MotcleSelectOptionsList;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class BuilderUtils implements ExtractionConstants {

    public final static ExtraitCorpusDefList EMPTY_EXTRAITCORPUSDEFLIST = new EmptyExtraitCorpusDefList();
    public final static ExtraitThesaurusDefList EMPTY_EXTRAITTHESAURUSDEFLIST = new EmptyExtraitThesaurusDefList();
    public final static CorpusExtractorList EMPTY_CORPUSEXTRACTORLIST = new EmptyCorpusExtractorList();
    public final static ThesaurusExtractorList EMPTY_THESAURUSEXTRACTORLIST = new EmptyThesaurusExtractorList();

    private BuilderUtils() {
    }

    public static String testTagName(String tagName, String defaultTagName) {
        if (tagName == null) {
            return defaultTagName;
        }
        if (tagName.length() == 0) {
            return defaultTagName;
        }
        if (tagName.equals("NULL")) {
            return null;
        }
        if (testCharacters(tagName)) {
            return tagName;
        }
        return defaultTagName;
    }

    public static ExtractionSource getExtractionSource(Object dynamicObject) {
        return new SimpleExtractionSource(dynamicObject);
    }

    public static ExtractionSource getExtractionSource(Object dynamicObject, ExtractionContext extractionContext, ExtractionDef extractionDef) {
        ExtraitCorpusDefList extraitCorpusDefList = extractionDef.getStaticExtraitCorpusDefList();
        ExtraitThesaurusDefList extraitThesaurusDefList = extractionDef.getStaticExtraitThesaurusDefList();
        if ((extraitCorpusDefList.getExtraitCorpusDefCount() == 0) && (extraitThesaurusDefList.getExtraitThesaurusDefCount() == 0)) {
            return new SimpleExtractionSource(dynamicObject);
        }
        CorpusExtractorList corpusExtractorList = toCorpusExtractorList(extraitCorpusDefList, extractionContext);
        ThesaurusExtractorList thesaurusExtractorList = toThesaurusExtractorList(extraitThesaurusDefList, extractionContext);
        return new CompleteExtractionSource(dynamicObject, corpusExtractorList, thesaurusExtractorList);
    }

    public static ExtraitCorpusDef getExtraitCorpusDef(FicheFilter ficheFilter, IntituleClause intituleClause) {
        return new InternalExtraitCorpusDef(ficheFilter, intituleClause);
    }

    public static ExtraitThesaurusDef getExtraitThesaurusDef(MotcleFilter motcleFilter) {
        return new InternalExtraitThesaurusDef(motcleFilter);
    }

    public static ExtraitAlbumDef getExtraitAlbumDef(IllustrationFilter illustrationFilter) {
        return new InternalExtraitAlbumDef(illustrationFilter);
    }

    public static ExtraitCorpusDefList toExtraitCorpusDefList(ExtraitCorpusDef extraitCorpusDef) {
        return new SingletonExtraitCorpusDefList(extraitCorpusDef);
    }

    public static ExtraitCorpusDefList toExtraitCorpusDefList(List<ExtraitCorpusDef> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_EXTRAITCORPUSDEFLIST;
        } else if (size == 1) {
            return new SingletonExtraitCorpusDefList(list.get(0));
        } else {
            return new ArrayExtraitCorpusDefList(list.toArray(new ExtraitCorpusDef[size]));
        }
    }

    public static ExtraitThesaurusDefList toExtraitThesaurusDefList(List<ExtraitThesaurusDef> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_EXTRAITTHESAURUSDEFLIST;
        } else if (size == 1) {
            return new SingletonExtraitThesaurusDefList(list.get(0));
        } else {
            return new ArrayExtraitThesaurusDefList(list.toArray(new ExtraitThesaurusDef[size]));
        }
    }

    public static CorpusExtractorList toCorpusExtractorList(List<CorpusExtractor> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_CORPUSEXTRACTORLIST;
        } else if (size == 1) {
            return new SingletonCorpusExtractorList(list.get(0));
        } else {
            return new ArrayCorpusExtractorList(list.toArray(new CorpusExtractor[size]));
        }
    }

    public static ThesaurusExtractorList toThesaurusExtractorList(List<ThesaurusExtractor> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_THESAURUSEXTRACTORLIST;
        } else if (size == 1) {
            return new SingletonThesaurusExtractorList(list.get(0));
        } else {
            return new ArrayThesaurusExtractorList(list.toArray(new ThesaurusExtractor[size]));
        }
    }

    public static CorpusExtractorList toCorpusExtractorList(ExtraitCorpusDefList extraitCorpusDefList, ExtractionContext extractionContext) {
        int count = extraitCorpusDefList.getExtraitCorpusDefCount();
        if (count == 0) {
            return BuilderUtils.EMPTY_CORPUSEXTRACTORLIST;
        }
        FicheEligibility staticFicheEligibility = extractionContext.getStaticFicheEligibility();
        List<AbregeFiche> bufferList = null;
        if ((staticFicheEligibility != null) && (staticFicheEligibility instanceof BufferedFicheEligibility)) {
            bufferList = ((BufferedFicheEligibility) staticFicheEligibility).toList();
        }
        List<CorpusExtractor> list = new ArrayList<CorpusExtractor>();
        for (int k = 0; k < count; k++) {
            ExtraitCorpusDef extraitCorpusDef = extraitCorpusDefList.getExtraitCorpusDef(k);
            FicheSelector ficheSelector = new FicheSelector(extractionContext.getBasedefiches(), extractionContext.getLocalisationLists().getWorkingLangInteger(0));
            ficheSelector.addFicheSelectOptionsList(extraitCorpusDef.getFicheSelectOptionsList());
            int corpusCount = ficheSelector.getCorpusCount();
            if (corpusCount == 0) {
                continue;
            }
            CorpusExtractor corpusExtractor = new CorpusExtractor(extraitCorpusDef);
            if (bufferList != null) {
                for (AbregeFiche abregeFiche : bufferList) {
                    if (ficheSelector.isSelected(abregeFiche)) {
                        corpusExtractor.add(abregeFiche, 0);
                    }
                }
            } else {
                ficheSelector.setFicheEligibility(staticFicheEligibility);
                for (int i = 0; i < corpusCount; i++) {
                    Corpus corpus = ficheSelector.getCorpus(i);
                    for (int j = 0; j < corpus.getTicketLength(); j++) {
                        AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                        if (abregeFiche != null) {
                            if (ficheSelector.isSelected(abregeFiche)) {
                                corpusExtractor.add(abregeFiche, 0);
                            }
                        }
                    }
                }
            }
            list.add(corpusExtractor);
        }
        return BuilderUtils.toCorpusExtractorList(list);
    }

    public static ThesaurusExtractorList toThesaurusExtractorList(ExtraitThesaurusDefList extraitThesaurusDefList, ExtractionContext extractionContext) {
        int count = extraitThesaurusDefList.getExtraitThesaurusDefCount();
        if (count == 0) {
            return BuilderUtils.EMPTY_THESAURUSEXTRACTORLIST;
        }
        List<ThesaurusExtractor> list = new ArrayList<ThesaurusExtractor>();
        for (int k = 0; k < count; k++) {
            ExtraitThesaurusDef extraitThesaurusDef = extraitThesaurusDefList.getExtraitThesaurusDef(k);
            MotcleSelector motcleSelector = new MotcleSelector(extractionContext.getBasedefiches(), extractionContext.getLocalisationLists().getWorkingLangInteger(0));
            motcleSelector.addMotcleSelectOptionsList(extraitThesaurusDef.getMotcleSelectOptionsList());
            int thesaurusCount = motcleSelector.getThesaurusCount();
            if (thesaurusCount == 0) {
                continue;
            }
            ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(extraitThesaurusDef, null);
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = motcleSelector.getThesaurus(i);
                for (int j = 0; j < thesaurus.getTicketLength(); j++) {
                    Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(j);
                    if (motcle != null) {
                        if (motcleSelector.isSelected(motcle)) {
                            thesaurusExtractor.add(motcle, 0);
                        }
                    }
                }
            }
            list.add(thesaurusExtractor);
        }
        return BuilderUtils.toThesaurusExtractorList(list);
    }

    private static boolean testCharacters(String s) {
        int longueur = s.length();
        for (int i = 0; i < longueur; i++) {
            char carac = s.charAt(i);
            if (!isCharacter(carac)) {
                if (i == 0) {
                    return false;
                }
                if (!isNumber(carac)) {
                    if (i == (longueur - 1)) {
                        return false;
                    }
                    if (!isSpecialChar(carac)) {
                        return false;
                    }
                }

            }
        }
        return true;
    }

    private static boolean isCharacter(char carac) {
        if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        }
        if ((carac >= 'A') && (carac <= 'Z')) {
            return true;
        }
        return false;
    }

    private static boolean isNumber(char carac) {
        if ((carac >= '0') && (carac <= '9')) {
            return true;
        }
        return false;
    }

    private static boolean isSpecialChar(char carac) {
        switch (carac) {
            case '-':
            case '_':
                return true;
            default:
                return false;
        }
    }


    private static class InternalExtraitCorpusDef implements ExtraitCorpusDef {

        private IntituleClause intituleClause;
        private FicheFilter ficheFilter;

        InternalExtraitCorpusDef(FicheFilter ficheFilter, IntituleClause intituleClause) {
            this.ficheFilter = ficheFilter;
            this.intituleClause = intituleClause;
        }

        public String getTagName() {
            return EXTRAITCORPUS_DEFAULT_TAGNAME;
        }

        public GroupClause getGroupClause() {
            return null;
        }

        public PackClause getPackClause() {
            return null;
        }

        public String getExtraitName() {
            return null;
        }

        public IntituleClause getIntituleClause() {
            return intituleClause;
        }

        public FicheFilter getFicheFilter() {
            return ficheFilter;
        }

        public FicheSelectOptionsList getFicheSelectOptionsList() {
            return SelectionUtils.EMPTY_FICHESELECTOPTIONSLIST;
        }

    }


    private static class InternalExtraitThesaurusDef implements ExtraitThesaurusDef {

        private MotcleFilter motcleFilter;

        private InternalExtraitThesaurusDef(MotcleFilter motcleFilter) {
            this.motcleFilter = motcleFilter;
        }

        public String getTagName() {
            return EXTRAITTHESAURUS_DEFAULT_TAGNAME;
        }

        public String getExtraitName() {
            return null;
        }

        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        public MotcleSelectOptionsList getMotcleSelectOptionsList() {
            return SelectionUtils.EMPTY_MOTCLESELECTOPTIONSLIST;
        }

    }


    private static class InternalExtraitAlbumDef implements ExtraitAlbumDef {

        private IllustrationFilter illustrationFilter;

        private InternalExtraitAlbumDef(IllustrationFilter illustrationFilter) {
            this.illustrationFilter = illustrationFilter;
        }

        public String getTagName() {
            return EXTRAITALBUM_DEFAULT_TAGNAME;
        }

        public String getExtraitName() {
            return null;
        }

        public IllustrationFilter getIllustrationFilter() {
            return illustrationFilter;
        }

        public IllustrationSelectOptionsList getIllustrationSelectOptionsList() {
            return SelectionUtils.EMPTY_ILLUSTRATIONSELECTOPTIONSLIST;
        }

    }


    private static class EmptyExtraitCorpusDefList implements ExtraitCorpusDefList {

        public int getExtraitCorpusDefCount() {
            return 0;
        }

        public ExtraitCorpusDef getExtraitCorpusDef(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonExtraitCorpusDefList implements ExtraitCorpusDefList {

        private ExtraitCorpusDef extraitCorpusDef;

        private SingletonExtraitCorpusDefList(ExtraitCorpusDef extraitCorpusDef) {
            this.extraitCorpusDef = extraitCorpusDef;
        }

        public int getExtraitCorpusDefCount() {
            return 1;
        }

        public ExtraitCorpusDef getExtraitCorpusDef(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return extraitCorpusDef;
        }

    }


    private static class ArrayExtraitCorpusDefList implements ExtraitCorpusDefList {

        private ExtraitCorpusDef[] extraitCorpusDefArray;

        private ArrayExtraitCorpusDefList(ExtraitCorpusDef[] extraitCorpusDefArray) {
            this.extraitCorpusDefArray = extraitCorpusDefArray;
        }

        public int getExtraitCorpusDefCount() {
            return extraitCorpusDefArray.length;
        }

        public ExtraitCorpusDef getExtraitCorpusDef(int i) {
            return extraitCorpusDefArray[i];
        }

    }


    private static class EmptyExtraitThesaurusDefList implements ExtraitThesaurusDefList {

        public int getExtraitThesaurusDefCount() {
            return 0;
        }

        public ExtraitThesaurusDef getExtraitThesaurusDef(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonExtraitThesaurusDefList implements ExtraitThesaurusDefList {

        private ExtraitThesaurusDef extraitThesaurusDef;

        private SingletonExtraitThesaurusDefList(ExtraitThesaurusDef extraitThesaurusDef) {
            this.extraitThesaurusDef = extraitThesaurusDef;
        }

        public int getExtraitThesaurusDefCount() {
            return 1;
        }

        public ExtraitThesaurusDef getExtraitThesaurusDef(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return extraitThesaurusDef;
        }

    }


    private static class ArrayExtraitThesaurusDefList implements ExtraitThesaurusDefList {

        private ExtraitThesaurusDef[] extraitThesaurusDefArray;

        private ArrayExtraitThesaurusDefList(ExtraitThesaurusDef[] extraitThesaurusDefArray) {
            this.extraitThesaurusDefArray = extraitThesaurusDefArray;
        }

        public int getExtraitThesaurusDefCount() {
            return extraitThesaurusDefArray.length;
        }

        public ExtraitThesaurusDef getExtraitThesaurusDef(int i) {
            return extraitThesaurusDefArray[i];
        }

    }


    private static class EmptyCorpusExtractorList implements CorpusExtractorList {

        public int getCorpusExtractorCount() {
            return 0;
        }

        public CorpusExtractor getCorpusExtractor(int i) {
            throw new IndexOutOfBoundsException();
        }

        public boolean containsFiche() {
            return false;
        }

    }


    private static class SingletonCorpusExtractorList implements CorpusExtractorList {

        private CorpusExtractor corpusExtractor;

        private SingletonCorpusExtractorList(CorpusExtractor corpusExtractor) {
            this.corpusExtractor = corpusExtractor;
        }

        public int getCorpusExtractorCount() {
            return 1;
        }

        public CorpusExtractor getCorpusExtractor(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return corpusExtractor;
        }

        public boolean containsFiche() {
            return corpusExtractor.containsFiche();
        }

    }


    private static class ArrayCorpusExtractorList implements CorpusExtractorList {

        private CorpusExtractor[] corpusExtractorArray;

        private ArrayCorpusExtractorList(CorpusExtractor[] corpusExtractorArray) {
            this.corpusExtractorArray = corpusExtractorArray;
        }

        public int getCorpusExtractorCount() {
            return corpusExtractorArray.length;
        }

        public CorpusExtractor getCorpusExtractor(int i) {
            return corpusExtractorArray[i];
        }

        public boolean containsFiche() {
            int length = corpusExtractorArray.length;
            if (length == 0) {
                return false;
            }
            for (int i = 0; i < length; i++) {
                if (corpusExtractorArray[i].containsFiche()) {
                    return true;
                }
            }
            return false;
        }

    }


    private static class EmptyThesaurusExtractorList implements ThesaurusExtractorList {

        public int getThesaurusExtractorCount() {
            return 0;
        }

        public ThesaurusExtractor getThesaurusExtractor(int i) {
            throw new IndexOutOfBoundsException();
        }

        public boolean containsMotcle() {
            return false;
        }

    }


    private static class SingletonThesaurusExtractorList implements ThesaurusExtractorList {

        private ThesaurusExtractor thesaurusExtractor;

        private SingletonThesaurusExtractorList(ThesaurusExtractor thesaurusExtractor) {
            this.thesaurusExtractor = thesaurusExtractor;
        }

        public int getThesaurusExtractorCount() {
            return 1;
        }

        public ThesaurusExtractor getThesaurusExtractor(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return thesaurusExtractor;
        }

        public boolean containsMotcle() {
            return thesaurusExtractor.containsMotcle();
        }

    }


    private static class ArrayThesaurusExtractorList implements ThesaurusExtractorList {

        private ThesaurusExtractor[] thesaurusExtractorArray;

        private ArrayThesaurusExtractorList(ThesaurusExtractor[] thesaurusExtractorArray) {
            this.thesaurusExtractorArray = thesaurusExtractorArray;
        }

        public int getThesaurusExtractorCount() {
            return thesaurusExtractorArray.length;
        }

        public ThesaurusExtractor getThesaurusExtractor(int i) {
            return thesaurusExtractorArray[i];
        }

        public boolean containsMotcle() {
            int length = thesaurusExtractorArray.length;
            if (length == 0) {
                return false;
            }
            for (int i = 0; i < length; i++) {
                if (thesaurusExtractorArray[i].containsMotcle()) {
                    return true;
                }
            }
            return false;
        }

    }


    private static class SimpleExtractionSource implements ExtractionSource {

        private Object dynamicObject;

        private SimpleExtractionSource(Object dynamicObject) {
            this.dynamicObject = dynamicObject;
        }

        public Object getDynamicObject() {
            return dynamicObject;
        }

        public CorpusExtractorList getStaticCorpusExtractorList() {
            return EMPTY_CORPUSEXTRACTORLIST;
        }

        public ThesaurusExtractorList getStaticThesaurusExtractorList() {
            return EMPTY_THESAURUSEXTRACTORLIST;
        }

    }


    private static class CompleteExtractionSource implements ExtractionSource {

        private Object dynamicObject;
        private CorpusExtractorList corpusExtractorList;
        private ThesaurusExtractorList thesaurusExtractorList;

        private CompleteExtractionSource(Object dynamicObject, CorpusExtractorList corpusExtractorList, ThesaurusExtractorList thesaurusExtractorList) {
            this.dynamicObject = dynamicObject;
            this.corpusExtractorList = corpusExtractorList;
            this.thesaurusExtractorList = thesaurusExtractorList;
        }

        public Object getDynamicObject() {
            return dynamicObject;
        }

        public CorpusExtractorList getStaticCorpusExtractorList() {
            return corpusExtractorList;
        }

        public ThesaurusExtractorList getStaticThesaurusExtractorList() {
            return thesaurusExtractorList;
        }

    }

}
