/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import java.util.List;
import net.mapeadores.basedefiches.extraction.api.DocumentFilter;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.basedefiches.selection.DocumentSelectOptions;
import net.mapeadores.basedefiches.selection.DocumentSelectOptionsList;
import net.mapeadores.basedefiches.selection.SelectionUtils;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitAddendaDefBuilder implements ExtractionConstants {

    public static ExtraitAddendaDef ASSOCIATION_EXTRAITADDENDADEF = new AssociationExtraitAddendaDef();
    private InternalExtraitAddendaDef internalExtraitAddendaDef = new InternalExtraitAddendaDef();

    public ExtraitAddendaDefBuilder() {
    }

    public ExtraitAddendaDef toExtraitAddendaDef() {
        if (internalExtraitAddendaDef.DocumentFilter == null) {
            throw new IllegalStateException("DocumentFilter cannot be null");
        }
        return internalExtraitAddendaDef;
    }

    public void setDocumentFilter(DocumentFilter DocumentFilter) {
        internalExtraitAddendaDef.DocumentFilter = DocumentFilter;
    }

    public void setTagName(String tagName) {
        internalExtraitAddendaDef.tagName = BuilderUtils.testTagName(tagName, EXTRAITADDENDA_DEFAULT_TAGNAME);
    }

    public void setExtraitName(String extraitName) {
        if ((extraitName != null) && (extraitName.length() == 0)) {
            extraitName = null;
        }
        internalExtraitAddendaDef.extraitName = extraitName;
    }

    public void setDocumentSelectOptionsList(List<DocumentSelectOptions> list) {
        internalExtraitAddendaDef.DocumentSelectOptionsList = SelectionUtils.toDocumentSelectOptionsList(list);
    }

    public void setDocumentSelectOptionsList(DocumentSelectOptions documentSelectOptions) {
        internalExtraitAddendaDef.DocumentSelectOptionsList = SelectionUtils.toDocumentSelectOptionsList(documentSelectOptions);
    }

    public void setDocumentSelectOptionsList(DocumentSelectOptionsList DocumentSelectOptionsList) {
        if (DocumentSelectOptionsList == null) {
            DocumentSelectOptionsList = SelectionUtils.EMPTY_DOCUMENTSELECTOPTIONSLIST;
        }
        internalExtraitAddendaDef.DocumentSelectOptionsList = DocumentSelectOptionsList;
    }


    private static class InternalExtraitAddendaDef implements ExtraitAddendaDef {

        private String extraitName = null;
        private String tagName = EXTRAITADDENDA_DEFAULT_TAGNAME;
        private DocumentFilter DocumentFilter;
        private DocumentSelectOptionsList DocumentSelectOptionsList = SelectionUtils.EMPTY_DOCUMENTSELECTOPTIONSLIST;

        public String getTagName() {
            return tagName;
        }

        public String getExtraitName() {
            return extraitName;
        }

        public DocumentFilter getDocumentFilter() {
            return DocumentFilter;
        }

        public DocumentSelectOptionsList getDocumentSelectOptionsList() {
            return DocumentSelectOptionsList;
        }

    }


    private static class AssociationExtraitAddendaDef implements ExtraitAddendaDef {

        private AssociationExtraitAddendaDef() {
        }

        public String getTagName() {
            return null;
        }

        public String getExtraitName() {
            return null;
        }

        public DocumentFilter getDocumentFilter() {
            return DocumentFilterBuilder.FICHE_DOCUMENTFILTER;
        }

        public DocumentSelectOptionsList getDocumentSelectOptionsList() {
            return SelectionUtils.EMPTY_DOCUMENTSELECTOPTIONSLIST;
        }

    }

}
