/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import java.util.List;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.IllustrationFilter;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptions;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptionsList;
import net.mapeadores.basedefiches.selection.SelectionUtils;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitAlbumDefBuilder implements ExtractionConstants {

    private InternalExtraitAlbumDef internalExtraitAlbumDef = new InternalExtraitAlbumDef();

    public ExtraitAlbumDefBuilder() {
    }

    public ExtraitAlbumDef toExtraitAlbumDef() {
        if (internalExtraitAlbumDef.illustrationFilter == null) {
            throw new IllegalStateException("illustrationFilter cannot be null");
        }
        return internalExtraitAlbumDef;
    }

    public void setIllustrationFilter(IllustrationFilter illustrationFilter) {
        internalExtraitAlbumDef.illustrationFilter = illustrationFilter;
    }

    public void setTagName(String tagName) {
        internalExtraitAlbumDef.tagName = BuilderUtils.testTagName(tagName, EXTRAITALBUM_DEFAULT_TAGNAME);
    }

    public void setExtraitName(String extraitName) {
        if ((extraitName != null) && (extraitName.length() == 0)) {
            extraitName = null;
        }
        internalExtraitAlbumDef.extraitName = extraitName;
    }

    public void setIllustrationSelectOptionsList(List<IllustrationSelectOptions> list) {
        internalExtraitAlbumDef.illustrationSelectOptionsList = SelectionUtils.toIllustrationSelectOptionsList(list);
    }

    public void setIllustrationSelectOptionsList(IllustrationSelectOptions illustrationSelectOptions) {
        internalExtraitAlbumDef.illustrationSelectOptionsList = SelectionUtils.toIllustrationSelectOptionsList(illustrationSelectOptions);
    }

    public void setIllustrationSelectOptionsList(IllustrationSelectOptionsList illustrationSelectOptionsList) {
        if (illustrationSelectOptionsList == null) {
            illustrationSelectOptionsList = SelectionUtils.EMPTY_ILLUSTRATIONSELECTOPTIONSLIST;
        }
        internalExtraitAlbumDef.illustrationSelectOptionsList = illustrationSelectOptionsList;
    }


    private static class InternalExtraitAlbumDef implements ExtraitAlbumDef {

        private String extraitName = null;
        private String tagName = EXTRAITALBUM_DEFAULT_TAGNAME;
        private IllustrationFilter illustrationFilter;
        private IllustrationSelectOptionsList illustrationSelectOptionsList = SelectionUtils.EMPTY_ILLUSTRATIONSELECTOPTIONSLIST;

        public String getTagName() {
            return tagName;
        }

        public String getExtraitName() {
            return extraitName;
        }

        public IllustrationFilter getIllustrationFilter() {
            return illustrationFilter;
        }

        public IllustrationSelectOptionsList getIllustrationSelectOptionsList() {
            return illustrationSelectOptionsList;
        }

    }

}
