/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import java.util.List;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.api.GroupClause;
import net.mapeadores.basedefiches.extraction.api.IntituleClause;
import net.mapeadores.basedefiches.extraction.api.PackClause;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelectOptionsList;
import net.mapeadores.basedefiches.selection.SelectionUtils;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitCorpusDefBuilder implements ExtractionConstants {

    private InternalExtraitCorpusDef internalExtraitCorpusDef = new InternalExtraitCorpusDef();

    public ExtraitCorpusDefBuilder() {
    }

    public ExtraitCorpusDef toExtraitCorpusDef() {
        if (internalExtraitCorpusDef.ficheFilter == null) {
            throw new IllegalStateException("ficheFilter cannot be null");
        }
        return internalExtraitCorpusDef;
    }

    public void setTagName(String tagName) {
        internalExtraitCorpusDef.tagName = BuilderUtils.testTagName(tagName, EXTRAITCORPUS_DEFAULT_TAGNAME);
    }

    public void setGroupClause(GroupClause groupClause) {
        internalExtraitCorpusDef.groupClause = groupClause;
    }

    public void setPackClause(PackClause packClause) {
        internalExtraitCorpusDef.packClause = packClause;
    }

    public void setExtraitName(String extraitName) {
        if ((extraitName != null) && (extraitName.length() == 0)) {
            extraitName = null;
        }
        internalExtraitCorpusDef.extraitName = extraitName;
    }

    public void setIntituleClause(IntituleClause intituleClause) {
        internalExtraitCorpusDef.intituleClause = intituleClause;
    }

    public void setFicheFilter(FicheFilter ficheFilter) {
        if (ficheFilter == null) {
            throw new IllegalArgumentException("ficheFilter is null");
        }
        internalExtraitCorpusDef.ficheFilter = ficheFilter;
    }

    public void setFicheSelectOptionsList(List<FicheSelectOptions> list) {
        internalExtraitCorpusDef.ficheSelectOptionsList = SelectionUtils.toFicheSelectOptionsList(list);
    }

    public void setFicheSelectOptionsList(FicheSelectOptions ficheSelectOptions) {
        internalExtraitCorpusDef.ficheSelectOptionsList = SelectionUtils.toFicheSelectOptionsList(ficheSelectOptions);
    }

    public void setFicheSelectOptionsList(FicheSelectOptionsList ficheSelectOptionsList) {
        if (ficheSelectOptionsList == null) {
            ficheSelectOptionsList = SelectionUtils.EMPTY_FICHESELECTOPTIONSLIST;
        }
        internalExtraitCorpusDef.ficheSelectOptionsList = ficheSelectOptionsList;
    }


    private static class InternalExtraitCorpusDef implements ExtraitCorpusDef {

        private FicheFilter ficheFilter;
        private String extraitName = null;
        private String tagName = EXTRAITCORPUS_DEFAULT_TAGNAME;
        private GroupClause groupClause = null;
        private PackClause packClause = null;
        private IntituleClause intituleClause = null;
        private FicheSelectOptionsList ficheSelectOptionsList = SelectionUtils.EMPTY_FICHESELECTOPTIONSLIST;

        private InternalExtraitCorpusDef() {
        }

        public String getTagName() {
            return tagName;
        }

        public GroupClause getGroupClause() {
            return groupClause;
        }

        public PackClause getPackClause() {
            return packClause;
        }

        public String getExtraitName() {
            return extraitName;
        }

        public IntituleClause getIntituleClause() {
            return intituleClause;
        }

        public FicheFilter getFicheFilter() {
            return ficheFilter;
        }

        public FicheSelectOptionsList getFicheSelectOptionsList() {
            return ficheSelectOptionsList;
        }

    }

}
