/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.filterunit.CorpsdeficheFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.FieldKeyFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitList;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitListBuilder;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitUtils;
import net.mapeadores.basedefiches.extraction.filterunit.TypePrefixFilterUnit;
import net.mapeadores.basedefiches.selection.MotcleSelector;


/**
 *
 * @author Vincent Calame
 */
public class FicheFilterBuilder implements ExtractionConstants {

    public final static FicheFilter TITRE_FICHEFILTER = new TitreFicheFilter();
    private InternalFicheFilter internalFicheFilter;

    public FicheFilterBuilder() {
        internalFicheFilter = new InternalFicheFilter();
    }

    public FicheFilter toFicheFilter() {
        return internalFicheFilter;
    }

    public void parseTagName(String tagName) {
        tagName = BuilderUtils.testTagName(tagName, FICHE_DEFAULT_TAGNAME);
        internalFicheFilter.setTagName(tagName);
    }

    public void updateFilterUnitList(FilterUnitList filterUnitList) {
        internalFicheFilter.updateFilterUnitList(filterUnitList);
    }

    public static FicheFilter newFicheInstance(String tagName) {
        FicheFilter liageFicheFilter = TITRE_FICHEFILTER;
        ExtraitCorpusDef extraitCorpusDef = BuilderUtils.getExtraitCorpusDef(liageFicheFilter, IntituleClauseBuilder.DEFAULT);
        ExtraitThesaurusDef extraitThesaurusDef = BuilderUtils.getExtraitThesaurusDef(MotcleFilterBuilder.INDEXATION_MOTCLEFILTER);
        ExtraitAlbumDef extraitAlbumDef = BuilderUtils.getExtraitAlbumDef(IllustrationFilterBuilder.FICHE_ILLUSTRATIONFILTER);
        ExtraitAddendaDef extraitAddendaDef = ExtraitAddendaDefBuilder.ASSOCIATION_EXTRAITADDENDADEF;
        FilterUnitListBuilder filterUnitListBuilder = new FilterUnitListBuilder();
        filterUnitListBuilder.addEnteteFilter();
        filterUnitListBuilder.addCorpsdeficheFilter();
        filterUnitListBuilder.addExtraitAddendaFilter(extraitAddendaDef);
        filterUnitListBuilder.addExtraitThesaurusFilter(extraitThesaurusDef);
        filterUnitListBuilder.addExtraitCorpusFilter(extraitCorpusDef);
        filterUnitListBuilder.addExtraitAlbumFilter(extraitAlbumDef);
        filterUnitListBuilder.addChronoFilter();
        filterUnitListBuilder.addFieldKey(FieldKey.LANG, null);
        filterUnitListBuilder.addIntituleFicheFilter();
        FicheFilterBuilder builder = new FicheFilterBuilder();
        builder.parseTagName(tagName);
        builder.updateFilterUnitList(filterUnitListBuilder.toFilterUnitList());
        return builder.toFicheFilter();
    }

    public static FicheFilter fromFilterUnitList(FilterUnitList filterUnitList) {
        FicheFilterBuilder builder = new FicheFilterBuilder();
        builder.updateFilterUnitList(filterUnitList);
        return builder.toFicheFilter();
    }


    private static class InternalFicheFilter implements FicheFilter {

        private FilterUnit[] filterUnitArray;
        private boolean withCorpsdefiche = false;
        private String tagName = FICHE_DEFAULT_TAGNAME;
        private MotcleSelector indexationMotcleSelector = null;

        InternalFicheFilter() {
        }

        public boolean isEmptyFicheFilter() {
            if (filterUnitArray != null) {
                return false;
            }
            if (indexationMotcleSelector != null) {
                return false;
            }
            return false;
        }

        public boolean isWithCorpsdefiche() {
            return withCorpsdefiche;
        }

        public int getFilterUnitCount() {
            if (filterUnitArray == null) {
                return 0;
            }
            return filterUnitArray.length;
        }

        public FilterUnit getFilterUnit(int i) {
            return filterUnitArray[i];
        }

        public String getTagName() {
            return tagName;
        }

        private void setTagName(String tagName) {
            this.tagName = tagName;
        }

        private void updateFilterUnitList(FilterUnitList filterUnitList) {
            if (filterUnitList == null) {
                filterUnitArray = null;
            }
            int count = filterUnitList.getFilterUnitCount();
            filterUnitArray = new FilterUnit[count];
            for (int i = 0; i < count; i++) {
                FilterUnit filterUnit = filterUnitList.getFilterUnit(i);
                if (filterUnit instanceof CorpsdeficheFilterUnit) {
                    withCorpsdefiche = true;
                } else if (filterUnit instanceof FieldKeyFilterUnit) {
                    FieldKey fieldKey = ((FieldKeyFilterUnit) filterUnit).getFieldKey();
                    if (fieldKey.ownsToCorpsdefiche()) {
                        withCorpsdefiche = true;
                    }
                } else if (filterUnit instanceof TypePrefixFilterUnit) {
                    short category = ((TypePrefixFilterUnit) filterUnit).getCategory();
                    if (category == FieldKey.SECTION_CATEGORY) {
                        withCorpsdefiche = true;
                    }
                }
                filterUnitArray[i] = filterUnit;
            }
        }

    }


    private static class TitreFicheFilter implements FicheFilter {

        private FilterUnit titreFilterUnit;

        private TitreFicheFilter() {
            titreFilterUnit = FilterUnitUtils.getFieldKeyFilterUnit(FieldKey.TITRE, null);
        }

        public boolean isEmptyFicheFilter() {
            return false;
        }

        public boolean isWithCorpsdefiche() {
            return false;
        }

        public int getFilterUnitCount() {
            return 2;
        }

        public FilterUnit getFilterUnit(int i) {
            if (i == 0) {
                return titreFilterUnit;
            }
            if (i == 1) {
                return FilterUnitUtils.INTITULEFICHE_FILTERUNIT;
            }
            throw new IndexOutOfBoundsException();
        }

        public String getTagName() {
            return FICHE_DEFAULT_TAGNAME;
        }

    }

}
