/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.GroupClause;
import net.mapeadores.basedefiches.extraction.api.GroupParams;
import net.mapeadores.basedefiches.extraction.params.GroupParamsParser;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.impl.InstructionParser;


/**
 *
 * @author Vincent Calame
 */
public class GroupClauseBuilder implements ExtractionConstants {

    private InternalGroupClause internalGroupClause;

    public GroupClauseBuilder(short type) {
        this.internalGroupClause = new InternalGroupClause(type);
    }

    public boolean parseSortOrder(String sortString) {
        try {
            short sort = sortToShort(sortString);
            internalGroupClause.setSortOrder(sort);
            return true;
        } catch (IllegalArgumentException iae) {
            return false;
        }
    }

    public boolean parseGroupParams(String paramsString) {
        short type = internalGroupClause.getGroupType();
        GroupParams groupParams = null;
        InstructionErrorHandler eieh = new InstructionErrorHandler();
        Instruction instruction = InstructionParser.parse(paramsString, eieh);
        switch (type) {
            case POIDS_TYPE:
                break;
            case TITRE_TYPE:
                groupParams = GroupParamsParser.parseTitreGroupParams(instruction);
                break;
            case ANNEE_TYPE:
            case LANG_TYPE:
            case PAYS_TYPE:
                groupParams = GroupParamsParser.parseFicheItemGroupParams(instruction);
                break;
            case FIELDS_TYPE:
                groupParams = GroupParamsParser.parseFieldsGroupParams(instruction);
                break;
        }
        if (groupParams != null) {
            internalGroupClause.setGroupParams(groupParams);
            return true;
        } else {
            return false;
        }
    }

    public boolean parseTagName(String tagName) {
        tagName = BuilderUtils.testTagName(tagName, GROUP_DEFAULT_TAGNAME);
        internalGroupClause.setTagName(tagName);
        return true;
    }

    public void setSubGroupClause(GroupClause groupClause) {
        internalGroupClause.setSubGroupClause(groupClause);
    }

    public GroupClause toGroupClause() {
        short type = internalGroupClause.getGroupType();
        GroupParams groupParams = internalGroupClause.getGroupParams();
        switch (type) {
            case FIELDS_TYPE:
                if (groupParams == null) {
                    return null;
                }
        }
        return internalGroupClause;
    }

    public static GroupClauseBuilder newInstance(String typeString) {
        try {
            short type = typeToShort(typeString);
            return new GroupClauseBuilder(type);

        } catch (IllegalArgumentException iae) {
            return null;
        }
    }

    public static GroupClause getGroupClause(String groupClauseString, boolean hidden) {
        int idx = groupClauseString.indexOf('-');
        if (idx == -1) {
            return null;
        }
        String typeString = groupClauseString.substring(0, idx);
        String sortString = groupClauseString.substring(idx + 1);
        short type = 0;
        try {
            type = typeToShort(typeString);

        } catch (IllegalArgumentException iae) {
            return null;
        }
        switch (type) {
            case POIDS_TYPE:
            case TITRE_TYPE:
            case ANNEE_TYPE:
            case LANG_TYPE:
                break;
            default:
                return null;
        }
        short sort = ASCENDING_SORT;
        try {
            sort = sortToShort(sortString);
        } catch (IllegalArgumentException iae) {
        }
        String tagName = (hidden) ? null : GROUP_DEFAULT_TAGNAME;
        InternalGroupClause groupClause = new InternalGroupClause(type);
        groupClause.setSortOrder(sort);
        groupClause.setTagName(tagName);
        return groupClause;
    }

    public static short sortToShort(String sort) {
        if ((sort == null) || (sort.length() == 0)) {
            return ASCENDING_SORT;
        }
        if (sort.equals("asc")) {
            return ASCENDING_SORT;
        }
        if (sort.equals("desc")) {
            return DESCENDING_SORT;
        }
        throw new IllegalArgumentException("unknwon sort : " + sort);
    }

    public static short typeToShort(String type) {
        if ((type == null) || (type.length() == 0)) {
            throw new IllegalArgumentException("undefined type");
        }
        if (type.equals("poids")) {
            return POIDS_TYPE;
        }
        if (type.equals("titre")) {
            return TITRE_TYPE;
        }
        if (type.equals("annee")) {
            return ANNEE_TYPE;
        }
        if (type.equals("lang")) {
            return LANG_TYPE;
        }
        if (type.equals("pays")) {
            return PAYS_TYPE;
        }
        if (type.equals("fields")) {
            return FIELDS_TYPE;
        }
        throw new IllegalArgumentException("unknwon type : " + type);
    }


    private static class InstructionErrorHandler implements InstructionParser.ErrorHandler {

        public void invalidAsciiCharacterError(String part, int row, int col) {
        }

        public void invalidEndCharacterError(String part, int row, int col) {
        }

        public void invalidSeparatorCharacterError(String part, int row, int col) {
        }

    }


    private static class InternalGroupClause implements GroupClause {

        private short groupType;
        private short sortOrder = ASCENDING_SORT;
        private String tagName = GROUP_DEFAULT_TAGNAME;
        private GroupParams groupParams = null;
        private GroupClause subGroupClause = null;

        InternalGroupClause(short groupType) {
            this.groupType = groupType;
        }

        public short getGroupType() {
            return groupType;
        }

        public short getSortOrder() {
            return sortOrder;
        }

        public void setSortOrder(short sortOrder) {
            this.sortOrder = sortOrder;
        }

        public String getTagName() {
            return tagName;
        }

        public void setTagName(String tagName) {
            this.tagName = tagName;
        }

        public GroupParams getGroupParams() {
            return groupParams;
        }

        public void setGroupParams(GroupParams groupParams) {
            this.groupParams = groupParams;
        }

        public GroupClause getSubGroupClause() {
            return subGroupClause;
        }

        public void setSubGroupClause(GroupClause subGroupClause) {
            this.subGroupClause = subGroupClause;
        }

    }

}
