/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;


/**
 *
 * @author Vincent Calame
 */
public class MotcleFilterBuilder implements ExtractionConstants {

    public final static MotcleFilter INDEXATION_MOTCLEFILTER = new IndexationMotcleFilter();
    private InternalMotcleFilter internalMotcleFilter = new InternalMotcleFilter();

    public MotcleFilterBuilder() {
    }

    public void setWithLevel(boolean withLevel) {
        internalMotcleFilter.withLevel = withLevel;
    }

    public void setChildrenFilter(MotcleFilter childrenFilter) {
        internalMotcleFilter.childrenFilter = childrenFilter;
    }

    public void setParentFilter(MotcleFilter parentFilter) {
        internalMotcleFilter.parentFilter = parentFilter;
    }

    public void setNextFilter(MotcleFilter nextFilter) {
        internalMotcleFilter.nextFilter = nextFilter;
    }

    public void setWithLib(boolean withLib) {
        internalMotcleFilter.withLib = withLib;
    }

    public void setPreviousFilter(MotcleFilter previousFilter) {
        internalMotcleFilter.previousFilter = previousFilter;
    }

    public void setWithIntituleThesaurus(boolean withIntituleThesaurus) {
        internalMotcleFilter.withIntituleThesaurus = withIntituleThesaurus;
    }

    public void setIndexationExtraitCorpusDefList(ExtraitCorpusDefList indexationExtraitCorpusDefList) {
        internalMotcleFilter.indexationExtraitCorpusDefList = indexationExtraitCorpusDefList;
    }

    public void setChildrenRecursive() {
        internalMotcleFilter.childrenFilter = internalMotcleFilter;
    }

    public void setParentRecursive() {
        internalMotcleFilter.parentFilter = internalMotcleFilter;
    }

    public void parseMotcleTagName(String tagName, String defaultTagName) {
        tagName = BuilderUtils.testTagName(tagName, defaultTagName);
        internalMotcleFilter.tagName = tagName;
    }

    public MotcleFilter toMotcleFilter() {
        return internalMotcleFilter;
    }

    public static ExtraitCorpusDefList getDefaultExtraitCorpusDefList() {
        FicheFilter indexationFicheFilter = FicheFilterBuilder.TITRE_FICHEFILTER;
        ExtraitCorpusDef extraitCorpusDef = BuilderUtils.getExtraitCorpusDef(indexationFicheFilter, IntituleClauseBuilder.DEFAULT);
        return BuilderUtils.toExtraitCorpusDefList(extraitCorpusDef);
    }

    /**
     * Propose une instance par défaut pour les cas où le mot-clé est demandé seul.
     */
    public static MotcleFilter getAloneInstance() {
        MotcleFilterBuilder builder = new MotcleFilterBuilder();
        builder.setWithLib(true);
        builder.setIndexationExtraitCorpusDefList(getDefaultExtraitCorpusDefList());
        return builder.toMotcleFilter();
    }

    /**
     * Propose une instance par défaut pour les cas où le mot-clé est demandé à l'intérieur d'un thésaurus.
     */
    public static MotcleFilter getThesaurusInstance() {
        MotcleFilterBuilder builder = new MotcleFilterBuilder();
        builder.setWithLib(true);
        builder.setIndexationExtraitCorpusDefList(getDefaultExtraitCorpusDefList());
        builder.setChildrenRecursive();
        builder.setWithIntituleThesaurus(true);
        return builder.toMotcleFilter();
    }

    public static MotcleFilter getIndexationInstance() {
        MotcleFilterBuilder builder = new MotcleFilterBuilder();
        builder.setWithLib(true);
        return builder.toMotcleFilter();
    }


    private static class InternalMotcleFilter implements MotcleFilter {

        private ExtraitCorpusDefList indexationExtraitCorpusDefList = BuilderUtils.EMPTY_EXTRAITCORPUSDEFLIST;
        private boolean withLevel, withLib;
        private MotcleFilter previousFilter;
        private MotcleFilter nextFilter;
        private MotcleFilter parentFilter;
        private MotcleFilter childrenFilter;
        private boolean withIntituleThesaurus = false;
        private String tagName = MOTCLE_DEFAULT_TAGNAME;

        private InternalMotcleFilter() {
        }

        public boolean isNoneFiltering() {
            if (withLib) {
                return false;
            }
            if (indexationExtraitCorpusDefList.getExtraitCorpusDefCount() != 0) {
                return false;
            }
            if (withLevel) {
                return false;
            }
            if (childrenFilter != null) {
                return false;
            }
            if (parentFilter != null) {
                return false;
            }
            if (nextFilter != null) {
                return false;
            }
            if (previousFilter != null) {
                return false;
            }
            return true;
        }

        public boolean withLevel() {
            return withLevel;
        }

        public boolean withLib() {
            return withLib;
        }

        public MotcleFilter getChildrenFilter() {
            return childrenFilter;
        }

        public MotcleFilter getParentFilter() {
            return parentFilter;
        }

        public MotcleFilter getNextFilter() {
            return nextFilter;
        }

        public MotcleFilter getPreviousFilter() {
            return previousFilter;
        }

        public boolean isWithIntituleThesaurus() {
            return withIntituleThesaurus;
        }

        public ExtraitCorpusDefList getIndexationExtraitCorpusDefList() {
            return indexationExtraitCorpusDefList;
        }

        public String getTagName() {
            return tagName;
        }

    }


    private static class IndexationMotcleFilter implements MotcleFilter {

        private IndexationMotcleFilter() {
        }

        public boolean isNoneFiltering() {
            return false;
        }

        public boolean withLevel() {
            return false;
        }

        public boolean withLib() {
            return true;
        }

        public MotcleFilter getChildrenFilter() {
            return null;
        }

        public MotcleFilter getParentFilter() {
            return null;
        }

        public MotcleFilter getNextFilter() {
            return null;
        }

        public MotcleFilter getPreviousFilter() {
            return null;
        }

        public boolean isWithIntituleThesaurus() {
            return false;
        }

        public ExtraitCorpusDefList getIndexationExtraitCorpusDefList() {
            return BuilderUtils.EMPTY_EXTRAITCORPUSDEFLIST;
        }

        public String getTagName() {
            return MOTCLE_DEFAULT_TAGNAME;
        }

    }

}
