/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.builders;

import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.PackClause;


/**
 *
 * @author Vincent Calame
 */
public class PackClauseBuilder implements ExtractionConstants {

    private InternalPackClause internalPackClause;

    public PackClauseBuilder(int packSize) {
        if (packSize < 1) {
            packSize = 1;
        }
        internalPackClause = new InternalPackClause(packSize);
    }

    public PackClause toPackClause() {
        return internalPackClause;
    }

    public boolean parseTagName(String tagName) {
        tagName = BuilderUtils.testTagName(tagName, PACK_DEFAULT_TAGNAME);
        internalPackClause.setTagName(tagName);
        return true;
    }

    public boolean parseModulo(String moduloString) {
        if ((moduloString == null) || (moduloString.length() == 0)) {
            return false;
        }
        try {
            int modulo = Integer.parseInt(moduloString);
            internalPackClause.setModulo(modulo);
            return true;
        } catch (NumberFormatException nfe) {
            return false;
        }
    }

    public static PackClauseBuilder newInstance(String packSizeString) {
        if ((packSizeString == null) || (packSizeString.length() == 0)) {
            return null;
        }
        try {
            int packSize = Integer.parseInt(packSizeString);
            return new PackClauseBuilder(packSize);
        } catch (NumberFormatException nfe) {
            return null;
        }
    }


    private static class InternalPackClause implements PackClause {

        private int packSize;
        private int modulo = 0;
        private String tagName = PACK_DEFAULT_TAGNAME;

        InternalPackClause(int packSize) {
            this.packSize = packSize;
        }

        public String getTagName() {
            return tagName;
        }

        private void setTagName(String tagName) {
            this.tagName = tagName;
        }

        public int getModulo() {
            return modulo;
        }

        private void setModulo(int modulo) {
            if (modulo < 2) {
                modulo = 0;
            }
            this.modulo = modulo;
        }

        public int getPackSize() {
            return packSize;
        }

    }

}
