/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.filterunit;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.util.attr.AttributeHolder;


/**
 *
 * @author Vincent Calame
 */
public class FilterUnitListBuilder {

    private List<FilterUnit> list = new ArrayList<FilterUnit>();

    public FilterUnitListBuilder() {
    }

    public void addFieldKey(FieldKey fieldKey, AttributeHolder attributes) {
        list.add(FilterUnitUtils.getFieldKeyFilterUnit(fieldKey, attributes));
    }

    public void addTypeFilter(short category, String type, AttributeHolder attributes) {
        if (type.endsWith("*")) {
            int length = type.length();
            if (length == 1) {
                return;
            }
            String prefix = type.substring(0, length - 1);
            list.add(FilterUnitUtils.getTypePrefixFilterUnit(category, prefix, attributes));
        } else {
            try {
                FieldKey fieldKey = FieldKey.parse(category, type);
                addFieldKey(fieldKey, attributes);
            } catch (java.text.ParseException pe) {
            }
        }
    }

    public void addLotFilter(String tagName, FilterUnitList filterUnitList) {
        if (tagName != null) {
            tagName = tagName.trim();
            if (tagName.length() == 0) {
                tagName = null;
            }
        }
        list.add(FilterUnitUtils.getLotFilterUnit(tagName, filterUnitList));
    }

    public void addEnteteFilter() {
        list.add(FilterUnitUtils.ENTETE_FILTERUNIT);
    }

    public void addCorpsdeficheFilter() {
        list.add(FilterUnitUtils.CORPSDEFICHE_FILTERUNIT);
    }

    public void addChronoFilter() {
        list.add(FilterUnitUtils.CHRONO_FILTERUNIT);
    }

    public void addIntituleFicheFilter() {
        list.add(FilterUnitUtils.INTITULEFICHE_FILTERUNIT);
    }

    public void addExtraitCorpusFilter(ExtraitCorpusDef extraitCorpusDef) {
        list.add(FilterUnitUtils.getExtraitCorpusFilterUnit(extraitCorpusDef));
    }

    public void addExtraitThesaurusFilter(ExtraitThesaurusDef extraitThesaurusDef) {
        list.add(FilterUnitUtils.getExtraitThesaurusFilterUnit(extraitThesaurusDef));
    }

    public void addExtraitAlbumFilter(ExtraitAlbumDef extraitAlbumDef) {
        list.add(FilterUnitUtils.getExtraitAlbumFilterUnit(extraitAlbumDef));
    }

    public void addExtraitAddendaFilter(ExtraitAddendaDef extraitAddendaDef) {
        list.add(FilterUnitUtils.getExtraitAddendaFilterUnit(extraitAddendaDef));
    }

    public FilterUnitList toFilterUnitList() {
        return new InternalFilterUnitList(list);
    }


    private static class InternalFilterUnitList implements FilterUnitList {

        private List<FilterUnit> list;

        private InternalFilterUnitList(List<FilterUnit> list) {
            this.list = list;
        }

        public int getFilterUnitCount() {
            return list.size();
        }

        public FilterUnit getFilterUnit(int i) {
            return list.get(i);
        }

    }

}
