/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.filterunit;

import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.util.attr.AttributeHolder;


/**
 *
 * @author Vincent Calame
 */
public class FilterUnitUtils {

    public final static EnteteFilterUnit ENTETE_FILTERUNIT = new InternalEnteteFilterUnit();
    public final static CorpsdeficheFilterUnit CORPSDEFICHE_FILTERUNIT = new InternalCorpsdeficheFilterUnit();
    public final static ChronoFilterUnit CHRONO_FILTERUNIT = new InternalChronoFilterUnit();
    public final static IntituleFicheFilterUnit INTITULEFICHE_FILTERUNIT = new InternalIntituleFicheFilterUnit();

    public static FieldKeyFilterUnit getFieldKeyFilterUnit(FieldKey fieldKey, AttributeHolder attributes) {
        return new InternalFieldKeyFilterUnit(fieldKey, attributes);
    }

    public static LotFilterUnit getLotFilterUnit(String tagName, FilterUnitList filterUnitList) {
        return new InternalLotFilterUnit(tagName, filterUnitList);
    }

    public static TypePrefixFilterUnit getTypePrefixFilterUnit(short category, String prefix, AttributeHolder attributes) {
        return new InternalTypePrefixFilterUnit(category, prefix, attributes);
    }

    public static ExtraitCorpusFilterUnit getExtraitCorpusFilterUnit(ExtraitCorpusDef extraitCorpusDef) {
        return new InternalExtraitCorpusFilterUnit(extraitCorpusDef);
    }

    public static ExtraitThesaurusFilterUnit getExtraitThesaurusFilterUnit(ExtraitThesaurusDef extraitThesaurusDef) {
        return new InternalExtraitThesaurusFilterUnit(extraitThesaurusDef);
    }

    public static ExtraitAlbumFilterUnit getExtraitAlbumFilterUnit(ExtraitAlbumDef extraitAlbumDef) {
        return new InternalExtraitAlbumFilterUnit(extraitAlbumDef);
    }

    public static ExtraitAddendaFilterUnit getExtraitAddendaFilterUnit(ExtraitAddendaDef extraitAlbumDef) {
        return new InternalExtraitAddendaFilterUnit(extraitAlbumDef);
    }


    private static class InternalEnteteFilterUnit implements EnteteFilterUnit {

        public InternalEnteteFilterUnit() {
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalCorpsdeficheFilterUnit implements CorpsdeficheFilterUnit {

        public InternalCorpsdeficheFilterUnit() {
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalChronoFilterUnit implements ChronoFilterUnit {

        public InternalChronoFilterUnit() {
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalIntituleFicheFilterUnit implements IntituleFicheFilterUnit {

        public InternalIntituleFicheFilterUnit() {
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalFieldKeyFilterUnit implements FieldKeyFilterUnit {

        private FieldKey fieldKey;
        private AttributeHolder attributes;

        public InternalFieldKeyFilterUnit(FieldKey fieldKey, AttributeHolder attributes) {
            this.fieldKey = fieldKey;
            this.attributes = attributes;
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

        public AttributeHolder getAttributes() {
            return attributes;
        }

    }


    private static class InternalLotFilterUnit implements LotFilterUnit {

        private String tagName;
        private FilterUnitList filterUnitList;

        public InternalLotFilterUnit(String tagName, FilterUnitList filterUnitList) {
            this.tagName = tagName;
            this.filterUnitList = filterUnitList;
        }

        public String getTagName() {
            return tagName;
        }

        public FilterUnitList getFilterUnitList() {
            return filterUnitList;
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalTypePrefixFilterUnit implements TypePrefixFilterUnit {

        private short category;
        private String prefix;
        private AttributeHolder attributes;

        public InternalTypePrefixFilterUnit(short category, String prefix, AttributeHolder attributes) {
            this.category = category;
            this.prefix = prefix;
            this.attributes = attributes;
        }

        public short getCategory() {
            return category;
        }

        public String getPrefix() {
            return prefix;

        }

        public AttributeHolder getAttributes() {
            return attributes;
        }

    }


    private static class InternalExtraitCorpusFilterUnit implements ExtraitCorpusFilterUnit {

        private ExtraitCorpusDef extraitCorpusDef;

        public InternalExtraitCorpusFilterUnit(ExtraitCorpusDef extraitCorpusDef) {
            this.extraitCorpusDef = extraitCorpusDef;
        }

        public ExtraitCorpusDef getExtraitCorpusDef() {
            return extraitCorpusDef;
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalExtraitThesaurusFilterUnit implements ExtraitThesaurusFilterUnit {

        private ExtraitThesaurusDef extraitThesaurusDef;

        public InternalExtraitThesaurusFilterUnit(ExtraitThesaurusDef extraitThesaurusDef) {
            this.extraitThesaurusDef = extraitThesaurusDef;
        }

        public ExtraitThesaurusDef getExtraitThesaurusDef() {
            return extraitThesaurusDef;
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalExtraitAlbumFilterUnit implements ExtraitAlbumFilterUnit {

        private ExtraitAlbumDef extraitAlbumDef;

        public InternalExtraitAlbumFilterUnit(ExtraitAlbumDef extraitAlbumDef) {
            this.extraitAlbumDef = extraitAlbumDef;
        }

        public ExtraitAlbumDef getExtraitAlbumDef() {
            return extraitAlbumDef;
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }


    private static class InternalExtraitAddendaFilterUnit implements ExtraitAddendaFilterUnit {

        private ExtraitAddendaDef extraitAddendaDef;

        public InternalExtraitAddendaFilterUnit(ExtraitAddendaDef extraitAddendaDef) {
            this.extraitAddendaDef = extraitAddendaDef;
        }

        public ExtraitAddendaDef getExtraitAddendaDef() {
            return extraitAddendaDef;
        }

        public AttributeHolder getAttributes() {
            return null;
        }

    }

}
