/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.extraction.groups;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.extraction.FicheExtractInfo;


/**
 *
 * @author Vincent Calame
 */
public class InnerSortFicheGroup implements MutableFicheGroup {

    private String name;
    private String tagName;
    private List<FicheExtractBundle> bundleList = new ArrayList<FicheExtractBundle>();
    private FicheItem[] matchingFicheItemArray;
    private boolean asc;

    public InnerSortFicheGroup(String name, String tagName, boolean asc) {
        this.name = name;
        this.tagName = tagName;
        this.asc = asc;
    }

    public String getName() {
        return name;
    }

    public String getTagName() {
        return tagName;
    }

    public void setMatchingFicheItemArray(FicheItem[] matchingFicheItemArray) {
        this.matchingFicheItemArray = matchingFicheItemArray;
    }

    public FicheItem[] getMatchingFicheItemArray() {
        return matchingFicheItemArray;
    }

    public boolean isBottomGroup() {
        return true;
    }

    public Map<String, String> getAttributesMap() {
        return null;
    }

    public FicheExtractInfo[] getFicheExtractInfoArray() {
        Collections.sort(bundleList, new InternalComparator(asc));
        int size = bundleList.size();
        FicheExtractInfo[] array = new FicheExtractInfo[size];
        for (int i = 0; i < size; i++) {
            array[i] = bundleList.get(i).toFicheExtractInfo();
        }
        return array;
    }

    public void add(FicheExtractBundle ficheExtractBundle) {
        bundleList.add(ficheExtractBundle);
    }

    public FicheGroup[] getSubgroupArray() {
        return null;
    }


    private static class InternalComparator implements Comparator<FicheExtractBundle> {

        boolean asc;

        InternalComparator(boolean asc) {
            this.asc = asc;
        }

        public int compare(FicheExtractBundle feb1, FicheExtractBundle feb2) {
            int result = internalCompare(feb1, feb2);
            if (!asc) {
                return -result;
            }
            return result;
        }

        private int internalCompare(FicheExtractBundle feb1, FicheExtractBundle feb2) {
            int in1 = feb1.getInnerSortValue();
            int in2 = feb2.getInnerSortValue();
            if (in1 < in2) {
                return -1;
            }
            if (in1 > in2) {
                return 1;
            }
            int id1 = feb1.getAbregeFiche().getIdcorpus();
            int id2 = feb2.getAbregeFiche().getIdcorpus();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

}
