/* BasedefichesLib_Explore - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute;

import net.mapeadores.basedefiches.format.compute.impl.ImplBasedefichesFormatter;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.format.compute.resolvers.AssociationFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.ChronoFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.CorpusFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.EmptyFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.FicheBlockFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.FicheItemFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.SimpleValueFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.SpecialLiageResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.SpecialSlaveCorpusResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.SubfieldFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.ThesaurusFieldFormatResolver;
import net.mapeadores.basedefiches.format.compute.resolvers.ThesaurusFormatResolver;
import net.mapeadores.basedefiches.format.patterns.AbregeFicheFormat;
import net.mapeadores.basedefiches.format.patterns.DocumentFormat;
import net.mapeadores.basedefiches.format.patterns.FicheItemFormat;
import net.mapeadores.basedefiches.format.patterns.MotcleFormat;
import net.mapeadores.basedefiches.format.patterns.PersonneFormat;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class BasedefichesFormatDefEngine {

    private ImplBasedefichesFormatter formatter;
    private ErrorHandler errorHandler;
    private Subset currentSubset;
    private Corpus currentCorpus;
    private SubsetKey slaveCorpusKey;
    private Basedefiches basedefiches;
    private boolean noItem;
    private List<Object> resolverList = new ArrayList<Object>();
    private boolean withMap = false;

    private BasedefichesFormatDefEngine(Subset currentSubset, ErrorHandler errorHandler) {
        this.currentSubset = currentSubset;
        this.basedefiches = currentSubset.getBasedefiches();
        this.errorHandler = errorHandler;
    }

    public static BasedefichesFormatter compute(Subset subset, BasedefichesFormatDef basedefichesFormatDef, ErrorHandler errorHandler) {
        BasedefichesFormatDefEngine cffd = new BasedefichesFormatDefEngine(subset, errorHandler);
        boolean done = cffd.init(basedefichesFormatDef);
        if (done) {
            return cffd.formatter;
        } else {
            return null;
        }
    }

    private boolean init(BasedefichesFormatDef basedefichesFormatDef) {
        this.noItem = BasedefichesFormatUtils.isNoItem(basedefichesFormatDef);
        int size = basedefichesFormatDef.getFormatSourceKeyCount();
        for (int i = 0; i < size; i++) {
            FormatSourceKey formatSourceKey = basedefichesFormatDef.getFormatSourceKey(i);
            String formatPattern = BasedefichesFormatUtils.getFormatPatternBySourceIndex(basedefichesFormatDef, i);
            if (!init(formatSourceKey, formatPattern)) {
                return false;
            }
        }
        this.formatter = new ImplBasedefichesFormatter(currentSubset, basedefichesFormatDef, toFormatResolverListProvider());
        return true;
    }

    private boolean init(FormatSourceKey formatSourceKey, String formatPattern) {
        if (!initCurrentCorpus(formatSourceKey)) {
            return false;
        }
        if (formatSourceKey.isFieldKey()) {
            return initFieldKey(formatSourceKey, formatPattern);
        } else if (formatSourceKey.isSubfieldKey()) {
            return initSubfieldKey(formatSourceKey, formatPattern);
        } else if (formatSourceKey.isIncludeKey()) {
            return initIncludeKey(formatSourceKey, formatPattern);
        } else if (formatSourceKey.isThesaurusFieldKey()) {
            return initThesaurusFieldKey(formatSourceKey, formatPattern);
        } else if (formatSourceKey.isTypePrefixFieldKey()) {
            return initTypePrefixFieldKey(formatSourceKey, formatPattern);
        } else {
            throw new SwitchException("FormatSourceKey ?");
        }
    }

    private boolean initCurrentCorpus(FormatSourceKey formatSourceKey) {
        currentCorpus = null;
        slaveCorpusKey = formatSourceKey.getSlaveCorpusKey();
        if (slaveCorpusKey != null) {
            currentCorpus = (Corpus) basedefiches.getSubset(slaveCorpusKey);
            if (currentCorpus == null) {
                errorHandler.unknownSubsetKey(slaveCorpusKey);
                return false;
            }
            Subset masterSubset = currentCorpus.getMasterSubset();
            if ((masterSubset == null) || (!masterSubset.equals(currentSubset))) {
                currentCorpus = null;
                errorHandler.notASlaveCorpusKey(slaveCorpusKey);
                return false;
            }
        } else {
            if (currentSubset instanceof Corpus) {
                currentCorpus = (Corpus) currentSubset;
            }
        }
        return true;
    }

    private boolean initThesaurusFieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
        if (!(currentSubset instanceof Thesaurus)) {
            errorHandler.illegalFormatSourceKey(formatSourceKey.toString());
            return false;
        }
        addFormatResolver(new ThesaurusFieldFormatResolver((Thesaurus) currentSubset, formatSourceKey.getThesaurusFieldKey()));
        return true;
    }

    private boolean initFieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
        FieldKey fieldKey = formatSourceKey.getFieldKey();
        if (!testCurrentCorpus(fieldKey)) {
            return false;
        }
        CorpusField corpusField = currentCorpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            errorHandler.unknownFieldKey(fieldKey);
            return false;
        }
        if (fieldKey.ownsToCorpsdefiche()) {
            addFormatResolver(new FicheBlockFormatResolver(corpusField, slaveCorpusKey));
        } else if ((fieldKey.equals(FieldKey.IDCORPUS)) || (fieldKey.equals(FieldKey.TITRE))) {
            addFormatResolver(new SimpleValueFormatResolver(corpusField, slaveCorpusKey));
        } else {
            FicheItemFormat ficheItemFormat = null;
            try {
                ficheItemFormat = FicheItemFormat.newInstance(corpusField.getFicheItemType(), formatPattern);
                if (ficheItemFormat instanceof PersonneFormat) {
                    ((PersonneFormat) ficheItemFormat).setBasedefiches(currentSubset.getBasedefiches());
                }
            } catch (ParseException pe) {
                errorHandler.wrongFormatPattern(formatPattern);
                return false;
            }
            FormatResolver formatResolver = createFicheItemFormatResolver(corpusField, formatPattern);
            if (formatResolver == null) {
                return false;
            }
            addFormatResolver(formatResolver);
        }
        return true;
    }

    private boolean initTypePrefixFieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
        FieldKey typePrefixFieldKey = formatSourceKey.getTypePrefixFieldKey();
        if (currentCorpus == null) {
            errorHandler.illegalFormatSourceKey(typePrefixFieldKey.toString() + "*");
            return false;
        }
        Map<String, FormatResolver> resolverMap = new HashMap<String, FormatResolver>();
        String prefix = typePrefixFieldKey.getTypeString();
        int prefixLength = prefix.length();
        CorpusMetadata corpusMetadata = currentCorpus.getCorpusMetadata();
        if (typePrefixFieldKey.isSectionFieldKey()) {
            int sectionCount = corpusMetadata.getSectionFieldCount();
            for (int i = 0; i < sectionCount; i++) {
                CorpusField corpusField = corpusMetadata.getSectionField(i);
                String type = corpusField.getFieldKey().getTypeString();
                if (type.startsWith(prefix)) {
                    String part2 = type.substring(prefixLength);
                    FormatResolver formatResolver = new FicheBlockFormatResolver(corpusField, slaveCorpusKey);
                    resolverMap.put(part2, formatResolver);
                }
            }
        } else if (typePrefixFieldKey.isInformationFieldKey()) {
            int informationCount = corpusMetadata.getInformationFieldCount();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                String type = corpusField.getFieldKey().getTypeString();
                if (type.startsWith(prefix)) {
                    String part2 = type.substring(prefixLength);
                    FormatResolver formatResolver = createFicheItemFormatResolver(corpusField, formatPattern);
                    if (formatResolver == null) {
                        return false;
                    }
                    resolverMap.put(part2, formatResolver);
                }
            }
        } else if (typePrefixFieldKey.isProprieteFieldKey()) {
            int proprieteCount = corpusMetadata.getProprieteFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                String type = corpusField.getFieldKey().getTypeString();
                if (type.startsWith(prefix)) {
                    String part2 = type.substring(prefixLength);
                    FormatResolver formatResolver = createFicheItemFormatResolver(corpusField, formatPattern);
                    if (formatResolver == null) {
                        return false;
                    }
                    resolverMap.put(part2, formatResolver);
                }
            }
        }
        if (resolverMap.size() == 0) {
            errorHandler.noTypePrefixFieldKey(typePrefixFieldKey);
            return false;
        }
        addFormatResolverMap(resolverMap);
        return true;
    }

    private FormatResolver createFicheItemFormatResolver(CorpusField corpusField, String formatPattern) {
        FicheItemFormat ficheItemFormat = null;
        try {
            ficheItemFormat = FicheItemFormat.newInstance(corpusField.getFicheItemType(), formatPattern);
            if (ficheItemFormat instanceof PersonneFormat) {
                ((PersonneFormat) ficheItemFormat).setBasedefiches(currentSubset.getBasedefiches());
            }
        } catch (ParseException pe) {
            errorHandler.wrongFormatPattern(formatPattern);
            return null;
        }
        return FicheItemFormatResolver.newInstance(corpusField, ficheItemFormat, slaveCorpusKey, noItem);
    }

    private boolean initSubfieldKey(FormatSourceKey formatSourceKey, String formatPattern) {
        SubfieldKey subfieldKey = formatSourceKey.getSubfieldKey();
        if (!testCurrentCorpus(subfieldKey)) {
            return false;
        }
        FieldKey fieldKey = subfieldKey.getFieldKey();
        CorpusField corpusField = currentCorpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            errorHandler.unknownSubfieldKey(subfieldKey);
            return false;
        }
        if (!SubfieldKey.isLegalSubfield(subfieldKey.getSubtype(), corpusField.getFicheItemType())) {
            errorHandler.illegalSubfieldKey(subfieldKey);
            return false;
        }
        SubfieldFormatResolver formatResolver = SubfieldFormatResolver.newInstance(subfieldKey, slaveCorpusKey, noItem);
        formatResolver.setBasedefiches(basedefiches);
        addFormatResolver(formatResolver);
        return true;
    }

    private boolean testCurrentCorpus(Object obj) {
        if (currentCorpus == null) {
            errorHandler.illegalFormatSourceKey(obj.toString());
            return false;
        } else {
            return true;
        }
    }

    private boolean initIncludeKey(FormatSourceKey formatSourceKey, String formatPattern) {
        IncludeKey includeKey = formatSourceKey.getIncludeKey();
        if (includeKey.isChronoInclude()) {
            if (!testCurrentCorpus(includeKey)) {
                return false;
            }
            FicheItemFormat ficheItemFormat;
            try {
                ficheItemFormat = FicheItemFormat.newInstance(CorpusField.DATATION_FIELD, formatPattern);
            } catch (ParseException pe) {
                errorHandler.wrongFormatPattern(formatPattern);
                return false;
            }
            addFormatResolver(new ChronoFormatResolver(includeKey, ficheItemFormat, slaveCorpusKey));
        } else if (includeKey.equals(IncludeKey.ASSOCIATION)) {
            if (!initCurrentCorpus(formatSourceKey)) {
                return false;
            }
            DocumentFormat documentFormat = null;
            if (formatPattern == null) {
                formatPattern = DocumentFormat.getDefaultPattern();
            }
            try {
                documentFormat = new DocumentFormat();
                documentFormat.setPattern(formatPattern);
            } catch (ParseException pe) {
                errorHandler.wrongFormatPattern(formatPattern);
                return false;
            }
            addFormatResolver(new AssociationFormatResolver(documentFormat, slaveCorpusKey));
        } else if (includeKey.equals(IncludeKey.LIAGE)) {
            if (!initCurrentCorpus(formatSourceKey)) {
                return false;
            }
            AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
            if (abregeFicheFormat == null) {
                return false;
            }
            addFormatResolver(new SpecialLiageResolver(abregeFicheFormat, slaveCorpusKey));
        } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
            if (!initCurrentCorpus(formatSourceKey)) {
                return false;
            }
            Subset masterSubset = currentSubset.getMasterSubset();
            Subset[] slaveSubsetArray = currentSubset.getSlaveSubsetArray();
            if (masterSubset != null) {
                if (masterSubset instanceof Corpus) {
                    AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
                    if (abregeFicheFormat == null) {
                        return false;
                    }
                    addFormatResolver(new CorpusFormatResolver((Corpus) masterSubset, abregeFicheFormat, -1, slaveCorpusKey));
                } else if (masterSubset instanceof Thesaurus) {
                    Thesaurus thesaurus = (Thesaurus) masterSubset;
                    MotcleFormat motcleFormat = initMotcleFormat(thesaurus, formatPattern, errorHandler);
                    if (motcleFormat == null) {
                        return false;
                    }
                    addFormatResolver(new ThesaurusFormatResolver(thesaurus, motcleFormat, includeKey.getPoidsFilter(), slaveCorpusKey));
                }
            } else if (slaveSubsetArray != null) {
                AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
                if (abregeFicheFormat == null) {
                    return false;
                }
                addFormatResolver(new SpecialSlaveCorpusResolver(abregeFicheFormat, slaveCorpusKey));
            } else {
                addFormatResolver(new EmptyFormatResolver());
            }

        } else if (includeKey.isCorpusInclude()) {
            if (!initCurrentCorpus(formatSourceKey)) {
                return false;
            }
            Corpus corpus = (Corpus) basedefiches.getSubset(includeKey.getSubsetKey());
            if (corpus == null) {
                errorHandler.unknownSubsetKey(includeKey.getSubsetKey());
                return false;
            }
            AbregeFicheFormat abregeFicheFormat = initAbregeFicheFormat(formatPattern, errorHandler);
            if (abregeFicheFormat == null) {
                return false;
            }
            addFormatResolver(new CorpusFormatResolver(corpus, abregeFicheFormat, includeKey.getPoidsFilter(), slaveCorpusKey));
        } else if (includeKey.isThesaurusInclude()) {
            if (!initCurrentCorpus(formatSourceKey)) {
                return false;
            }
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(includeKey.getSubsetKey());
            if (thesaurus == null) {
                errorHandler.unknownSubsetKey(includeKey.getSubsetKey());
                return false;
            }
            MotcleFormat motcleFormat = initMotcleFormat(thesaurus, formatPattern, errorHandler);
            if (motcleFormat == null) {
                return false;
            }
            addFormatResolver(new ThesaurusFormatResolver(thesaurus, motcleFormat, includeKey.getPoidsFilter(), slaveCorpusKey));
        }
        return true;
    }

    private AbregeFicheFormat initAbregeFicheFormat(String formatPattern, ErrorHandler errorHandler) {
        AbregeFicheFormat abregeFicheFormat = null;
        if (formatPattern == null) {
            formatPattern = AbregeFicheFormat.getDefaultPattern();
        }
        try {
            abregeFicheFormat = new AbregeFicheFormat();
            abregeFicheFormat.setPattern(formatPattern);
            return abregeFicheFormat;
        } catch (ParseException pe) {
            errorHandler.wrongFormatPattern(formatPattern);
            return null;
        }
    }

    private MotcleFormat initMotcleFormat(Thesaurus thesaurus, String formatPattern, ErrorHandler errorHandler) {
        MotcleFormat motcleFormat = null;
        if (formatPattern == null) {
            formatPattern = MotcleFormat.getDefaultPattern(ThesaurusUtils.isIdalphaType(thesaurus));
        }
        try {
            motcleFormat = new MotcleFormat();
            motcleFormat.setPattern(formatPattern);
            return motcleFormat;
        } catch (ParseException pe) {
            errorHandler.wrongFormatPattern(formatPattern);
            return null;
        }
    }

    private void addFormatResolver(FormatResolver formatResolver) {
        resolverList.add(formatResolver);
    }

    private void addFormatResolverMap(Map<String, FormatResolver> resolverMap) {
        withMap = true;
        resolverList.add(resolverMap);
    }

    private FormatResolverListProvider toFormatResolverListProvider() {
        if (!withMap) {
            int size = resolverList.size();
            FormatResolver[] array = new FormatResolver[size];
            for (int i = 0; i < size; i++) {
                array[i] = (FormatResolver) resolverList.get(i);
            }
            FormatResolverList formatResolverList = ComputeUtils.toFormatResolverList(array);
            return ComputeUtils.getSimpleFormatResolverListProvider(formatResolverList);
        } else {
            return new MapFormatResolverListProvider(resolverList);
        }
    }


    public static interface ErrorHandler {

        public void unknownSubfieldKey(SubfieldKey subfieldKey);

        public void illegalSubfieldKey(SubfieldKey subfieldKey);

        public void wrongFormatPattern(String formatPattern);

        public void unknownFieldKey(FieldKey fieldKey);

        public void unknownSubsetKey(SubsetKey subsetKey);

        public void notASlaveCorpusKey(SubsetKey subsetKey);

        public void illegalFormatSourceKey(String keyString);

        public void noTypePrefixFieldKey(FieldKey fieldKey);

    }


    private static class MapFormatResolverListProvider implements FormatResolverListProvider {

        private Map<Integer, FormatResolverList> resolverMap = new HashMap<Integer, FormatResolverList>();
        private List<Object> resolverList;

        private MapFormatResolverListProvider(List<Object> resolverList) {
            this.resolverList = resolverList;
        }

        public FormatResolverList getFormatResolverList(LocalisationInfo localisationInfo) {
            int langInteger = localisationInfo.getWorkingLangInteger();
            String lang = LangInteger.toString(langInteger);
            FormatResolverList formatResolverList = resolverMap.get(langInteger);
            if (formatResolverList == null) {
                int size = resolverList.size();
                FormatResolver[] array = new FormatResolver[size];
                for (int i = 0; i < size; i++) {
                    Object obj = resolverList.get(i);
                    if (obj instanceof FormatResolver) {
                        array[i] = (FormatResolver) obj;
                    } else {
                        Map<String, FormatResolver> map = (Map<String, FormatResolver>) obj;
                        FormatResolver formatResolver = map.get(lang);
                        if (formatResolver == null) {
                            formatResolver = map.get("");
                        }
                        if (formatResolver == null) {
                            formatResolver = (FormatResolver) map.values().iterator().next();
                        }
                        array[i] = formatResolver;
                    }
                }
                formatResolverList = ComputeUtils.toFormatResolverList(array);
                resolverMap.put(langInteger, formatResolverList);
            }
            return formatResolverList;
        }

    }

}
