/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.impl;

import java.util.Collections;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatter;
import net.mapeadores.basedefiches.format.compute.FormatComputeUtils;
import net.mapeadores.basedefiches.format.compute.FormatResolver;
import net.mapeadores.basedefiches.format.compute.FormatResolverList;
import net.mapeadores.basedefiches.format.compute.FormatResolverListProvider;
import net.mapeadores.basedefiches.format.compute.FormatResult;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.resolvers.FicheItemFormatResolver;
import net.mapeadores.basedefiches.format.patterns.FicheItemFormat;
import net.mapeadores.basedefiches.include.IncludeSummaryMap;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.SubsetItemPointeur;
import net.mapeadores.util.format.FormatDef;
import net.mapeadores.util.format.FormatUtils;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class ImplBasedefichesFormatter implements BasedefichesFormatter {

    private boolean lineSeparator;
    private short castType;
    private Subset subset;
    private FieldKey defaultProprieteKey;
    private int position;
    private boolean fillByLast;
    private FicheItemFormat defaultPropItemFormat;
    private String defaultValue;
    private FormatDef formatDef;
    private FormatResolverListProvider formatResolverListProvider;

    public ImplBasedefichesFormatter(Subset subset, FormatDef formatDef, FormatResolverListProvider formatResolverListProvider) {
        this.subset = subset;
        this.lineSeparator = FormatUtils.isLineSeparator(formatDef);
        this.castType = FormatUtils.getCast(formatDef);
        this.defaultProprieteKey = BasedefichesFormatUtils.getDefaultProprieteKey(formatDef);
        this.position = FormatUtils.getPosition(formatDef);
        this.fillByLast = FormatUtils.isFillByLast(formatDef);
        this.defaultValue = FormatUtils.getDefaultValue(formatDef);
        this.formatDef = formatDef;
        this.formatResolverListProvider = formatResolverListProvider;
        endInit();
    }

    private void endInit() {
        if (subset instanceof Corpus) {
            if (defaultProprieteKey != null) {
                CorpusField corpusField = ((Corpus) subset).getCorpusMetadata().getCorpusFieldByFieldKey(defaultProprieteKey);
                if (corpusField != null) {
                    int ficheItemType = corpusField.getFicheItemType();
                    FormatResolver formatResolver = formatResolverListProvider.getFormatResolverList(null).getFormatResolver(0);
                    if (formatResolver instanceof FicheItemFormatResolver) {
                        FicheItemFormatResolver fifr = (FicheItemFormatResolver) formatResolver;
                        if (fifr.isSameType(ficheItemType)) {
                            defaultPropItemFormat = fifr.getFicheItemFormat();
                        } else {
                            defaultPropItemFormat = FicheItemFormat.newDefaultInstance(ficheItemType);
                        }
                    } else {
                        defaultPropItemFormat = FicheItemFormat.newDefaultInstance(ficheItemType);
                    }

                }
            }
        }
    }

    public boolean isLineSeparator() {
        return lineSeparator;
    }

    public short getCastType() {
        return castType;
    }

    /**
     * Retourne une value nulle ou une liste avec au moins un élement non nul.
     */
    public List<String> formatToList(SubsetItemPointeur subsetItemPointeur, IncludeSummaryMap includeSummaryMap, LocalisationInfo localisationInfo) {
        testSubsetItemPointeur(subsetItemPointeur);
        FormatResult formatResult = resolve(subsetItemPointeur, includeSummaryMap, localisationInfo);
        List<String> result = formatResult.getResultList();
        if (result == null) {
            String s = getDefaultString(subsetItemPointeur, localisationInfo);
            if (s != null) {
                result = Collections.singletonList(s);
            }
        }
        return result;
    }

    public String formatToString(SubsetItemPointeur subsetItemPointeur, IncludeSummaryMap includeSummaryMap, LocalisationInfo localisationInfo) {
        testSubsetItemPointeur(subsetItemPointeur);
        FormatResult formatResult = resolve(subsetItemPointeur, includeSummaryMap, localisationInfo);
        String result = formatResult.getResultString();
        if (result == null) {
            result = getDefaultString(subsetItemPointeur, localisationInfo);
        }
        return result;
    }

    private void testSubsetItemPointeur(SubsetItemPointeur subsetItemPointeur) {
        if (subset instanceof Corpus) {
            if (!(subsetItemPointeur instanceof FichePointeur)) {
                throw new IllegalArgumentException("subsetItemPointeur must be an instance of FichePointeur");
            }
        }
    }

    private FormatResult resolve(SubsetItemPointeur subsetItemPointeur, IncludeSummaryMap includeSummaryMap, LocalisationInfo localisationInfo) {
        FormatSource formatSource = FormatComputeUtils.toFormatSource(subsetItemPointeur, includeSummaryMap, localisationInfo);
        FormatResolverList formatResolverList = formatResolverListProvider.getFormatResolverList(localisationInfo);
        if (position != -1) {
            ImplPositionFormatBuilder positionFormatBuilder = new ImplPositionFormatBuilder(formatSource, position, fillByLast);
            return positionFormatBuilder.resolve(formatResolverList);
        } else {
            ImplLimitFormatBuilder limitFormatBuilder = new ImplLimitFormatBuilder(formatSource, formatDef);
            return limitFormatBuilder.resolve(formatResolverList);
        }
    }

    private String getDefaultString(SubsetItemPointeur subsetItemPointeur, LocalisationInfo localisationInfo) {
        String defaultString = null;
        if (defaultPropItemFormat != null) {
            Object defObject = ((FichePointeur) subsetItemPointeur).getValue(defaultProprieteKey);
            if (defObject != null) {
                defaultString = defaultPropItemFormat.formatFicheItem((FicheItem) defObject, localisationInfo);
            }
        }
        if (defaultString == null) {
            defaultString = defaultValue;
        }
        return defaultString;
    }

}
