/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.impl;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.format.compute.FormatResolver;
import net.mapeadores.basedefiches.format.compute.FormatResolverList;
import net.mapeadores.basedefiches.format.compute.FormatResult;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.util.format.FormatDef;
import net.mapeadores.util.format.FormatUtils;


/**
 *
 * @author Vincent Calame
 */
class ImplLimitFormatBuilder implements LimitFormatBuilder {

    private int limit;
    private List<String> resultList = new ArrayList<String>();
    private List<Integer> positionList = new ArrayList<Integer>();
    private boolean resolved = false;
    private Integer currentPosition = null;
    private FormatDef formatDef;
    private FormatSource formatSource;

    ImplLimitFormatBuilder(FormatSource formatSource, FormatDef formatDef) {
        this.formatSource = formatSource;
        this.formatDef = formatDef;
        this.limit = FormatUtils.getLimit(formatDef);
    }

    public boolean add(String s) {
        if (resolved) {
            return false;
        }
        if (s == null) {
            return true;
        }
        positionList.add(currentPosition);
        resultList.add(s);
        if (limit != -1) {
            if (resultList.size() == limit) {
                resolved = true;
                return false;
            }
        }
        return true;
    }

    FormatResult resolve(FormatResolverList formatResolverList) {
        int size = formatResolverList.getFormatResolverCount();
        for (int i = 0; i < size; i++) {
            FormatResolver formatResolver = formatResolverList.getFormatResolver(i);
            currentPosition = new Integer(i);
            formatResolver.addInList(this, formatSource);
            if (resolved) {
                break;
            }
        }
        return new InternalFormatResult();
    }


    private class InternalFormatResult implements FormatResult {

        public List<String> getResultList() {
            if (resultList.size() == 0) {
                return null;
            }
            return resultList;
        }

        public String getResultString() {
            if (resultList.size() == 0) {
                return null;
            }
            StringBuilder buf = new StringBuilder();
            int size = resultList.size();
            Integer previousPosition = null;
            for (int i = 0; i < size; i++) {
                Integer position = positionList.get(i);
                if (i > 0) {
                    if (position.equals(previousPosition)) {
                        String separator = FormatUtils.getInnerSeparator(formatDef, position.intValue());
                        buf.append(separator);
                    } else {
                        String separator = FormatUtils.getSourceSeparator(formatDef, previousPosition.intValue(), position.intValue());
                        buf.append(separator);
                    }
                }
                previousPosition = position;
                buf.append(resultList.get(i));
            }
            return buf.toString();
        }

    }

}
