/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.impl;

import java.util.Collections;
import java.util.List;
import net.mapeadores.basedefiches.format.compute.FormatResolver;
import net.mapeadores.basedefiches.format.compute.FormatResolverList;
import net.mapeadores.basedefiches.format.compute.FormatResult;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.util.format.FormatConstants;


/**
 *
 * @author Vincent Calame
 */
class ImplPositionFormatBuilder implements PositionFormatBuilder {

    private int position;
    private String result;
    private boolean resolved = false;
    private boolean fillByLast;
    private FormatSource formatSource;

    ImplPositionFormatBuilder(FormatSource formatSource, int position, boolean fillByLast) {
        this.formatSource = formatSource;
        this.position = position;
        this.fillByLast = fillByLast;
    }

    FormatResult resolve(FormatResolverList formatResolverList) {
        if (position != FormatConstants.MAX_POSITION) {
            int size = formatResolverList.getFormatResolverCount();
            for (int i = 0; i < size; i++) {
                FormatResolver formatResolver = formatResolverList.getFormatResolver(i);
                formatResolver.getInPosition(this, formatSource);
                if (resolved) {
                    break;
                }
            }
            if ((!resolved) && (fillByLast)) {
                position = FormatConstants.MAX_POSITION;
                resolveLastPosition(formatResolverList);
            }
        } else {
            resolveLastPosition(formatResolverList);
        }
        return new InternalFormatResult();
    }

    private void resolveLastPosition(FormatResolverList formatResolverList) {
        int size = formatResolverList.getFormatResolverCount();
        for (int i = (size - 1); i >= 0; i--) {
            FormatResolver formatResolver = formatResolverList.getFormatResolver(i);
            formatResolver.getInPosition(this, formatSource);
            if (resolved) {
                break;
            }
        }
    }

    public int getPosition() {
        return position;
    }

    public int checkPosition(int length) {
        if (resolved) {
            return -1;
        }
        if (position == FormatConstants.MAX_POSITION) {
            return (length - 1);
        }
        if (position < length) {
            return position;
        } else {
            position = position - length;
            return -1;
        }
    }

    public void set(String result) {
        this.result = result;
        resolved = true;
    }


    private class InternalFormatResult implements FormatResult {

        public List<String> getResultList() {
            if (result == null) {
                return null;
            }
            return Collections.singletonList(result);
        }

        public String getResultString() {
            return result;
        }

    }

}
