/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.basedefiches.format.patterns.AbregeFicheFormat;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusFormatResolver extends AbstractFormatResolver {

    private Corpus corpus;
    private AbregeFicheFormat abregeFicheFormat;
    private int poidsFilter;

    public CorpusFormatResolver(Corpus corpus, AbregeFicheFormat abregeFicheFormat, int poidsFilter, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.corpus = corpus;
        this.abregeFicheFormat = abregeFicheFormat;
        this.poidsFilter = poidsFilter;
    }

    public void getInPosition(PositionFormatBuilder formatBuilder, FormatSource formatSource) {
        FicheCroisementList ficheCroisementList = getFicheCroisementList(formatSource);
        int size = ficheCroisementList.getSubsetItemCount();
        int position = formatBuilder.checkPosition(size);
        if (position != -1) {
            formatBuilder.set(abregeFicheFormat.formatAbregeFiche(ficheCroisementList.getAbregeFiche(position), ficheCroisementList.getPoids(position)));
        }
    }

    public void addInList(LimitFormatBuilder formatBuilder, FormatSource formatSource) {
        FicheCroisementList abregeFicheList = getFicheCroisementList(formatSource);
        int size = abregeFicheList.getSubsetItemCount();
        for (int i = 0; i < size; i++) {
            boolean cont = formatBuilder.add(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(i), abregeFicheList.getPoids(i)));
            if (!cont) {
                break;
            }
        }
    }

    private FicheCroisementList getFicheCroisementList(FormatSource formatSource) {
        FicheCroisementList ficheCroisementList = getFichePointeur(formatSource).getFicheCroisementList(corpus);
        if (poidsFilter > -1) {
            return CroisementUtils.poidsFilter(ficheCroisementList, poidsFilter);
        } else {
            IncludeSummary includeSummary = getIncludeSummary(formatSource);
            RangeList rangeList = null;
            if (includeSummary != null) {
                IncludeSummary.SubsetInfo info = includeSummary.getSubsetInfo(corpus.getSubsetKey());
                if (info != null) {
                    rangeList = info.getPoidsFilterRangeList();
                    if (rangeList != null) {
                        return CroisementUtils.excludePoids(ficheCroisementList, rangeList);
                    }
                }
            }
            return ficheCroisementList;
        }
    }

}
