/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.basedefiches.format.patterns.FicheItemFormat;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public abstract class FicheItemFormatResolver extends AbstractFormatResolver {

    protected CorpusField corpusField;
    protected FicheItemFormat ficheItemFormat;
    private boolean noItem;

    private FicheItemFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey, boolean noItem) {
        super(slaveCorpusKey);
        this.corpusField = corpusField;
        this.ficheItemFormat = ficheItemFormat;
        this.noItem = noItem;
    }

    String formatFicheItem(FicheItem ficheItem, LocalisationInfo localisationInfo) {
        if ((noItem) && (ficheItem instanceof Item)) {
            if (corpusField.getFicheItemType() != CorpusField.ITEM_FIELD) {
                return null;
            }
        }
        return ficheItemFormat.formatFicheItem(ficheItem, localisationInfo);
    }

    public boolean isSameType(int ficheItemType) {
        return (corpusField.getFicheItemType() == ficheItemType);
    }

    public FicheItemFormat getFicheItemFormat() {
        return ficheItemFormat;
    }

    public static FicheItemFormatResolver newInstance(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey, boolean noItem) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if ((fieldKey.isProprieteFieldKey()) || (fieldKey.equals(FieldKey.LANG)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
            return new ProprieteFormatResolver(corpusField, ficheItemFormat, slaveCorpusKey, noItem);
        } else {
            return new InformationFormatResolver(corpusField, ficheItemFormat, slaveCorpusKey, noItem);
        }
    }


    private static class ProprieteFormatResolver extends FicheItemFormatResolver {

        private ProprieteFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey, boolean noItem) {
            super(corpusField, ficheItemFormat, slaveCorpusKey, noItem);
        }

        public void getInPosition(PositionFormatBuilder positionFormatBuilder, FormatSource formatSource) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(formatSource).getValue(corpusField.getFieldKey());
            if (ficheItem != null) {
                int position = positionFormatBuilder.checkPosition(1);
                if (position != -1) {
                    positionFormatBuilder.set(formatFicheItem(ficheItem, formatSource.getLocalisationInfo()));
                }
            }
        }

        public void addInList(LimitFormatBuilder limitFormatBuilder, FormatSource formatSource) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(formatSource).getValue(corpusField.getFieldKey());
            if (ficheItem != null) {
                limitFormatBuilder.add(formatFicheItem(ficheItem, formatSource.getLocalisationInfo()));
            }
        }

    }


    private static class InformationFormatResolver extends FicheItemFormatResolver {

        private InformationFormatResolver(CorpusField corpusField, FicheItemFormat ficheItemFormat, SubsetKey slaveCorpusKey, boolean noItem) {
            super(corpusField, ficheItemFormat, slaveCorpusKey, noItem);
        }

        public void getInPosition(PositionFormatBuilder positionFormatBuilder, FormatSource formatSource) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(formatSource).getValue(corpusField.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            int position = positionFormatBuilder.checkPosition(size);
            if (position != -1) {
                positionFormatBuilder.set(formatFicheItem(ficheItemList.getFicheItem(position), formatSource.getLocalisationInfo()));
            }

        }

        public void addInList(LimitFormatBuilder limitFormatBuilder, FormatSource formatSource) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(formatSource).getValue(corpusField.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            for (int i = 0; i < size; i++) {
                boolean cont = limitFormatBuilder.add(formatFicheItem(ficheItemList.getFicheItem(i), formatSource.getLocalisationInfo()));
                if (!cont) {
                    break;
                }
            }

        }

    }

}
