/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.basedefiches.format.patterns.AbregeFicheFormat;


/**
 *
 * @author Vincent Calame
 */
public abstract class SpecialCorpusArrayResolver extends AbstractFormatResolver {

    private AbregeFicheFormat abregeFicheFormat;

    public SpecialCorpusArrayResolver(AbregeFicheFormat abregeFicheFormat, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.abregeFicheFormat = abregeFicheFormat;
    }

    public void getInPosition(PositionFormatBuilder formatBuilder, FormatSource formatSource) {
        FicheCroisementList[] array = getArray(formatSource);
        int size = getSize(array);
        int position = formatBuilder.checkPosition(size);
        if (position != -1) {
            int length = array.length;
            for (int i = 0; i < length; i++) {
                FicheCroisementList abregeFicheList = array[i];
                if (position >= abregeFicheList.getSubsetItemCount()) {
                    position = position - abregeFicheList.getSubsetItemCount();
                } else {
                    formatBuilder.set(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(position), abregeFicheList.getPoids(position)));
                    break;
                }
            }
        }
    }

    public void addInList(LimitFormatBuilder formatBuilder, FormatSource formatSource) {
        FicheCroisementList[] array = getArray(formatSource);
        int size = getSize(array);
        int length = array.length;
        int p = 0;
        for (int i = 0; i < length; i++) {
            FicheCroisementList abregeFicheList = array[i];
            int curSize = abregeFicheList.getSubsetItemCount();
            curSize = Math.min(size - p, curSize);
            boolean brekCont = false;
            for (int j = 0; j < curSize; j++) {
                boolean cont = formatBuilder.add(abregeFicheFormat.formatAbregeFiche(abregeFicheList.getAbregeFiche(j), abregeFicheList.getPoids(j)));
                if (!cont) {
                    brekCont = true;
                    break;
                }
                p++;
            }
            if (brekCont) {
                break;
            }
        }
    }

    protected abstract FicheCroisementList[] getArray(FormatSource formatSource);

    private int getSize(FicheCroisementList[] array) {
        int size = 0;
        int length = array.length;
        for (int i = 0; i < length; i++) {
            size = size + array[i].getSubsetItemCount();
        }
        return size;
    }

}
