/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.basedefiches.format.patterns.FormatResolutionUtils;


/**
 *
 * @author Vincent Calame
 */
public abstract class SubfieldFormatResolver extends AbstractFormatResolver {

    private Basedefiches basedefiches;
    protected SubfieldKey subfieldKey;
    private boolean noItem;

    private SubfieldFormatResolver(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey, boolean noItem) {
        super(slaveCorpusKey);
        this.subfieldKey = subfieldKey;
        this.noItem = noItem;
    }

    public void setBasedefiches(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    protected String format(FicheItem ficheItem) {
        if ((noItem) && (ficheItem instanceof Item)) {
            return null;
        }
        return FormatResolutionUtils.formatSubfield(subfieldKey.getSubtype(), ficheItem, basedefiches);
    }

    public static SubfieldFormatResolver newInstance(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey, boolean noItem) {
        FieldKey fieldKey = subfieldKey.getFieldKey();
        if (fieldKey.isProprieteFieldKey()) {
            return new ProprieteSubfieldFormatResolver(subfieldKey, slaveCorpusKey, noItem);
        } else {
            return new InformationSubfieldFormatResolver(subfieldKey, slaveCorpusKey, noItem);
        }
    }


    private static class ProprieteSubfieldFormatResolver extends SubfieldFormatResolver {

        private ProprieteSubfieldFormatResolver(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey, boolean noItem) {
            super(subfieldKey, slaveCorpusKey, noItem);

        }

        public void getInPosition(PositionFormatBuilder formatBuilder, FormatSource formatSource) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(formatSource).getValue(subfieldKey.getFieldKey());
            if (ficheItem != null) {
                int position = formatBuilder.checkPosition(1);
                if (position != -1) {
                    formatBuilder.set(format(ficheItem));
                }
            }
        }

        public void addInList(LimitFormatBuilder formatBuilder, FormatSource formatSource) {
            FicheItem ficheItem = (FicheItem) getFichePointeur(formatSource).getValue(subfieldKey.getFieldKey());
            if (ficheItem != null) {
                formatBuilder.add(format(ficheItem));
            }
        }

    }


    private static class InformationSubfieldFormatResolver extends SubfieldFormatResolver {

        private InformationSubfieldFormatResolver(SubfieldKey subfieldKey, SubsetKey slaveCorpusKey, boolean noItem) {
            super(subfieldKey, slaveCorpusKey, noItem);
        }

        public void getInPosition(PositionFormatBuilder formatBuilder, FormatSource formatSource) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(formatSource).getValue(subfieldKey.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            int position = formatBuilder.checkPosition(size);
            if (position != -1) {
                formatBuilder.set(format(ficheItemList.getFicheItem(position)));
            }
        }

        public void addInList(LimitFormatBuilder formatBuilder, FormatSource formatSource) {
            FicheItemList ficheItemList = (FicheItemList) getFichePointeur(formatSource).getValue(subfieldKey.getFieldKey());
            if (ficheItemList == null) {
                return;
            }
            int size = ficheItemList.getFicheItemCount();
            for (int i = 0; i < size; i++) {
                boolean cont = formatBuilder.add(format(ficheItemList.getFicheItem(i)));
                if (!cont) {
                    break;
                }
            }
        }

    }

}
