/* BasedefichesLib_Explore - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.format.compute.resolvers;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.format.compute.FormatSource;
import net.mapeadores.basedefiches.format.compute.LimitFormatBuilder;
import net.mapeadores.basedefiches.format.compute.PositionFormatBuilder;
import net.mapeadores.basedefiches.format.patterns.MotcleFormat;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusFormatResolver extends AbstractFormatResolver {

    private Thesaurus thesaurus;
    private MotcleFormat motcleFormat;
    private int poidsFilter;

    public ThesaurusFormatResolver(Thesaurus thesaurus, MotcleFormat motcleFormat, int poidsFilter, SubsetKey slaveCorpusKey) {
        super(slaveCorpusKey);
        this.thesaurus = thesaurus;
        this.motcleFormat = motcleFormat;
        this.poidsFilter = poidsFilter;
    }

    public void getInPosition(PositionFormatBuilder positionFormatBuilder, FormatSource formatSource) {
        MotcleCroisementList motcleList = getMotcleList(formatSource);
        int size = motcleList.getSubsetItemCount();
        int position = positionFormatBuilder.checkPosition(size);
        if (position != -1) {
            positionFormatBuilder.set(motcleFormat.formatMotcle(motcleList.getMotcle(position), motcleList.getPoids(position), formatSource.getLocalisationInfo()));
        }
    }

    public void addInList(LimitFormatBuilder limitFormatBuilder, FormatSource formatSource) {
        MotcleCroisementList motcleList = getMotcleList(formatSource);
        int size = motcleList.getSubsetItemCount();
        for (int i = 0; i < size; i++) {
            boolean cont = limitFormatBuilder.add(motcleFormat.formatMotcle(motcleList.getMotcle(i), motcleList.getPoids(i), formatSource.getLocalisationInfo()));
            if (!cont) {
                break;
            }
        }
    }

    private MotcleCroisementList getMotcleList(FormatSource formatSource) {
        MotcleCroisementList motcleCroisementList = getFichePointeur(formatSource).getMotcleCroisementList(thesaurus);
        if (poidsFilter > -1) {
            return CroisementUtils.poidsFilter(motcleCroisementList, poidsFilter);
        } else {
            RangeList rangeList = null;
            IncludeSummary includeSummary = getIncludeSummary(formatSource);
            if (includeSummary != null) {
                IncludeSummary.SubsetInfo info = includeSummary.getSubsetInfo(thesaurus.getSubsetKey());
                if (info != null) {
                    rangeList = info.getPoidsFilterRangeList();
                    if (rangeList != null) {
                        return CroisementUtils.excludePoids(motcleCroisementList, rangeList);
                    }
                }
            }
            return motcleCroisementList;
        }
    }

}
