/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.misc;

import java.util.MissingResourceException;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusResourceLocalisation implements ResourceLocalisation {

    private Thesaurus thesaurus;
    private int workingLangInteger;
    private ResourceLocalisation defaultResourceLocalisation;

    public ThesaurusResourceLocalisation(Thesaurus thesaurus, int workingLangInteger) {
        this.thesaurus = thesaurus;
        this.workingLangInteger = workingLangInteger;
    }

    public String toString(ResourceItem resourceItem) {
        return toString(resourceItem.getResourceKey(), resourceItem.getValue());
    }

    public String toString(String resourceKey, Object value) {
        Motcle motcle = thesaurus.getMotcleByIdalpha(resourceKey);
        if (motcle != null) {
            String lib = motcle.getLibelleString(workingLangInteger, false);
            if (lib == null) {
                return resourceKey;
            } else {
                return StringUtils.getMessage(lib, value);
            }
        } else if (defaultResourceLocalisation != null) {
            try {
                return defaultResourceLocalisation.toString(resourceKey, value);
            } catch (MissingResourceException mre) {
                return "?" + resourceKey + "?";
            }
        } else {
            return "?" + resourceKey + "?";
        }
    }

    public boolean containsKey(String resourceKey) {
        Motcle motcle = thesaurus.getMotcleByIdalpha(resourceKey);
        if (motcle != null) {
            return true;
        } else if (defaultResourceLocalisation != null) {
            return defaultResourceLocalisation.containsKey(resourceKey);
        } else {
            return false;
        }
    }

    public ResourceLocalisation getDefaultResourceLocalisation() {
        return defaultResourceLocalisation;
    }

    public void setDefaultResourceLocalisation(ResourceLocalisation defaultResourceLocalisation) {
        this.defaultResourceLocalisation = defaultResourceLocalisation;
    }

}
