/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.pointeurs;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.croisement.CroisementList;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.DocumentCroisementList;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.croisement.IllustrationCroisementList;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractPointeur implements SubsetItemPointeur {

    private Subset mainSubset;
    private Map<SubsetKey, CroisementListBuffer> croisementListBufferMap = new HashMap<SubsetKey, CroisementListBuffer>();
    private SubsetItem currentSubsetItem;

    public AbstractPointeur(Subset subset) {
        this.mainSubset = subset;
    }

    public Subset getSubset() {
        return mainSubset;
    }

    public SubsetItem getCurrentSubsetItem() {
        return currentSubsetItem;
    }

    public void setCurrentSubsetItem(SubsetItem subsetItem) {
        this.currentSubsetItem = subsetItem;
        for (CroisementListBuffer buffer : croisementListBufferMap.values()) {
            buffer.setCurrent(subsetItem);
        }
        initCurrentSubsetItem(subsetItem);
    }

    protected abstract void initCurrentSubsetItem(SubsetItem subsetItem);

    public FicheCroisementList getFicheCroisementList(Corpus corpus) {
        return (FicheCroisementList) getCroisementList(corpus);
    }

    public MotcleCroisementList getMotcleCroisementList(Thesaurus thesaurus) {
        return (MotcleCroisementList) getCroisementList(thesaurus);
    }

    public IllustrationCroisementList getIllustrationCroisementList(Album album) {
        return (IllustrationCroisementList) getCroisementList(album);
    }

    public DocumentCroisementList getDocumentCroisementList() {
        return (DocumentCroisementList) getCroisementList(SubsetKey.ADDENDA);
    }

    private CroisementList getCroisementList(Subset subset) {
        SubsetKey subsetKey = subset.getSubsetKey();
        CroisementListBuffer buffer = croisementListBufferMap.get(subsetKey);
        if (buffer == null) {
            buffer = new CroisementListBuffer(mainSubset, subset);
            buffer.setCurrent(currentSubsetItem);
            croisementListBufferMap.put(subsetKey, buffer);
        }
        return buffer.getCroisementList();
    }

    private CroisementList getCroisementList(SubsetKey subsetKey) {
        CroisementListBuffer buffer = croisementListBufferMap.get(subsetKey);
        if (buffer == null) {
            Subset subset = mainSubset.getBasedefiches().getSubset(subsetKey);
            buffer = new CroisementListBuffer(mainSubset, subset);
            buffer.setCurrent(currentSubsetItem);
            croisementListBufferMap.put(subsetKey, buffer);
        }
        return buffer.getCroisementList();
    }


    private static class CroisementListBuffer {

        private CroisementList currentCroisementList;
        private Subset otherSubset;
        private Croisement croisement;
        private int currentIdsubset;

        CroisementListBuffer(Subset mainSubset, Subset otherSubset) {
            this.croisement = mainSubset.getBasedefiches().getCroisement(mainSubset.getSubsetKey(), otherSubset.getSubsetKey());
            this.otherSubset = otherSubset;
            initToNull();
        }

        private CroisementList getCroisementList() {
            return currentCroisementList;
        }

        private void initToNull() {
            currentIdsubset = -1;
            currentCroisementList = CroisementUtils.getEmptyCroisementList(otherSubset.getSubsetKey());
        }

        private void setCurrent(SubsetItem subsetItem) {
            if (subsetItem == null) {
                if (currentIdsubset != -1) {
                    initToNull();
                }
            } else {
                int newIdsubset = subsetItem.getIdsubset();
                if (newIdsubset != currentIdsubset) {
                    currentIdsubset = newIdsubset;
                    CroisementItem croisementItem = croisement.getCroisementItem(subsetItem.getSubset().getSubsetKey(), subsetItem.getTicket());
                    if (otherSubset instanceof Corpus) {
                        currentCroisementList = CroisementUtils.toFicheCroisementList(croisementItem, (Corpus) otherSubset);
                    } else if (otherSubset instanceof Thesaurus) {
                        currentCroisementList = CroisementUtils.toMotcleCroisementList(croisementItem, (Thesaurus) otherSubset);
                    } else if (otherSubset instanceof Album) {
                        currentCroisementList = CroisementUtils.toIllustrationCroisementList(croisementItem, (Album) otherSubset);
                    } else if (otherSubset instanceof Addenda) {
                        currentCroisementList = CroisementUtils.toDocumentCroisementList(croisementItem, (Addenda) otherSubset);
                    }
                }
            }
        }

    }

}
