/* BasedefichesLib_Explore - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.pointeurs;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Section;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class FichePointeur extends AbstractPointeur {

    private boolean ficheComplete;
    private boolean withSlaveCorpusPointeur;
    private Corpus corpus;
    private Fiche fiche;
    private AbregeFiche currentAbregeFiche;
    private boolean isSlaveInit = false;
    private Map<SubsetKey, FichePointeur> slaveFichePointeurMap;
    private Subset masterSubset;
    private Map<FieldKey, Object> fieldBufferMap = new HashMap<FieldKey, Object>();
    private Corpus[] rattachementCorpusArray;
    private boolean cacheEnable = false;
    private Map<Integer, Fiche> cacheFicheMap;

    public FichePointeur(Corpus corpus) {
        this(corpus, false, false);
    }

    public FichePointeur(Corpus corpus, boolean ficheComplete) {
        this(corpus, ficheComplete, false);
    }

    public FichePointeur(Corpus corpus, boolean ficheComplete, boolean withSlaveCorpusPointeur) {
        super(corpus);
        this.ficheComplete = ficheComplete;
        this.withSlaveCorpusPointeur = withSlaveCorpusPointeur;
        this.corpus = corpus;
        this.masterSubset = corpus.getMasterSubset();
    }

    FichePointeur(AbregeFiche abregeFiche, Fiche fiche, boolean ficheComplete) {
        this(abregeFiche.getCorpus(), ficheComplete, false);
        setCurrentSubsetItem(abregeFiche);
        this.fiche = fiche;
    }

    protected void initCurrentSubsetItem(SubsetItem subsetItem) {
        AbregeFiche abregeFiche = (AbregeFiche) subsetItem;
        this.currentAbregeFiche = abregeFiche;
        fieldBufferMap.clear();
        this.fiche = null;
        if (abregeFiche != null) {
            String titre = abregeFiche.getTitre();
            if (titre.length() > 0) {
                fieldBufferMap.put(FieldKey.TITRE, titre);
            } else {
                fieldBufferMap.put(FieldKey.TITRE, Boolean.FALSE);
            }
            fieldBufferMap.put(FieldKey.IDCORPUS, new Integer(abregeFiche.getIdcorpus()));
        }
        initAbregeFicheSlave();
    }

    public boolean isFicheComplete() {
        return ficheComplete;
    }

    public void enableCache(boolean enable) {
        if (cacheEnable == enable) {
            return;
        }
        this.cacheEnable = enable;
        if (enable) {
            cacheFicheMap = new HashMap<Integer, Fiche>();
        } else {
            cacheFicheMap.clear();
            cacheFicheMap = null;
        }
    }

    private void initAbregeFicheSlave() {
        if (!isSlaveInit) {
            return;
        }
        if (slaveFichePointeurMap == null) {
            return;
        }
        int idcorpus = -1;
        if (currentAbregeFiche != null) {
            idcorpus = currentAbregeFiche.getIdcorpus();
        }
        for (FichePointeur fichePointeur : slaveFichePointeurMap.values()) {
            if (currentAbregeFiche == null) {
                fichePointeur.setCurrentSubsetItem(null);
            } else {
                SubsetItem slaveAbregeFiche = fichePointeur.getSubset().getSubsetItemById(idcorpus);
                fichePointeur.setCurrentSubsetItem(slaveAbregeFiche);
            }
        }
    }

    public Fiche getFiche() {
        if (currentAbregeFiche == null) {
            return null;
        }
        initFiche();
        return fiche;
    }

    public FichePointeur getSlaveFichePointeur(SubsetKey slaveCorpusKey) {
        testSlaveInit();
        if (slaveFichePointeurMap == null) {
            return null;
        }
        return slaveFichePointeurMap.get(slaveCorpusKey);
    }

    public boolean isRattachementCorpus(Corpus corpus) {
        testSlaveInit();
        if (rattachementCorpusArray == null) {
            return false;
        }
        int size = rattachementCorpusArray.length;
        for (int i = 0; i < size; i++) {
            if (rattachementCorpusArray[i].equals(corpus)) {
                return true;
            }
        }
        return false;
    }

    public int getRattachementCorpusCount() {
        testSlaveInit();
        if (rattachementCorpusArray == null) {
            return 0;
        }
        return rattachementCorpusArray.length;
    }

    public Corpus getRattachementCorpus(int i) {
        testSlaveInit();
        return rattachementCorpusArray[i];
    }

    private void testSlaveInit() {
        if (isSlaveInit) {
            return;
        }
        isSlaveInit = true;
        Subset[] slaveCorpusArray = corpus.getSlaveSubsetArray();
        if (masterSubset != null) {
            if (masterSubset instanceof Corpus) {
                rattachementCorpusArray = new Corpus[1];
                rattachementCorpusArray[0] = (Corpus) masterSubset;
            }
        } else if (slaveCorpusArray != null) {
            if (withSlaveCorpusPointeur) {
                slaveFichePointeurMap = new HashMap<SubsetKey, FichePointeur>();
            }
            int length = slaveCorpusArray.length;
            rattachementCorpusArray = new Corpus[length];
            for (int i = 0; i < length; i++) {
                Corpus slaveCorpus = (Corpus) slaveCorpusArray[i];
                rattachementCorpusArray[i] = slaveCorpus;
                if (withSlaveCorpusPointeur) {
                    slaveFichePointeurMap.put(slaveCorpus.getSubsetKey(), new FichePointeur(slaveCorpus, ficheComplete));
                }
            }
        }
        initAbregeFicheSlave();
    }

    public SubsetItem getCurrentMasterSubsetItem() {
        if (currentAbregeFiche == null) {
            return null;
        }
        if (masterSubset == null) {
            return null;
        }
        return masterSubset.getSubsetItemById(currentAbregeFiche.getIdcorpus());
    }

    public Datation getChronoToDatationDate(IncludeKey includeKey) {
        if (currentAbregeFiche == null) {
            return null;
        }
        int dateInteger = 0;
        if (includeKey.equals(IncludeKey.DATE_CREATION)) {
            dateInteger = currentAbregeFiche.getCreationDate();
        } else if (includeKey.equals(IncludeKey.DATE_MODIFICATION)) {
            dateInteger = currentAbregeFiche.getModificationDate();
        } else {
            throw new IllegalArgumentException("wrong includeKey : " + includeKey.toString());
        }
        if (dateInteger == 0) {
            return null;
        }
        return new Datation(dateInteger);
    }

    /**
     * Valeurs possibles : String (si titre non vide), FicheItem, FicheItemList (avec au moins un élément), FicheBlockList (avec au moins un élément), Integer (si idcorpus).
     * Si LANG, c'est un ficheItem de type Langue
     *
     */
    public Object getValue(FieldKey fieldKey) {
        if (currentAbregeFiche == null) {
            return null;
        }
        Object obj = fieldBufferMap.get(fieldKey);
        if (obj != null) {
            if (obj.equals(Boolean.FALSE)) {
                return null;
            } else {
                return obj;
            }
        }
        if ((!ficheComplete) && (fieldKey.ownsToCorpsdefiche())) {
            ficheComplete = true;
            if (fiche != null) {
                fiche = null;
                if (cacheEnable) {
                    cacheFicheMap.remove(currentAbregeFiche.getIdcorpus());
                }
            }
        }
        initFiche();
        obj = getValue(fiche, fieldKey);
        if (obj == null) {
            fieldBufferMap.put(fieldKey, Boolean.FALSE);
        } else {
            fieldBufferMap.put(fieldKey, obj);
        }
        return obj;
    }

    private void initFiche() {
        if (fiche != null) {
            return;
        }
        if (cacheEnable) {
            Fiche cacheFiche = cacheFicheMap.get(currentAbregeFiche.getIdcorpus());
            if (cacheFiche != null) {
                fiche = cacheFiche;
                return;
            }
        }
        if (ficheComplete) {
            fiche = corpus.getFiche(currentAbregeFiche);
        } else {
            fiche = corpus.getEnteteFiche(currentAbregeFiche);
        }
        if (cacheEnable) {
            cacheFicheMap.put(currentAbregeFiche.getIdcorpus(), fiche);
        }
    }

    /*
     * Aucun test n'est fait pour FieldKey.TITRE et FieldKey.IDCORPUS car ils sont
     * déjà inclus dans fieldBufferMap avant l'appel à cette méthode
     */
    private static Object getValue(Fiche fiche, FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            return fiche.getSoustitre();
        } else if (fieldKey.equals(FieldKey.LANG)) {
            int langInteger = fiche.getLangInteger();
            if (langInteger == LangInteger.UNDETERMINED) {
                return null;
            } else {
                return new Langue(langInteger);
            }
        } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return fiche.getRedacteurs();
        } else if (fieldKey.isProprieteFieldKey()) {
            Propriete prop = fiche.getPropriete(fieldKey);
            if (prop == null) {
                return null;
            }
            return prop.getFicheItem();
        } else if (fieldKey.isInformationFieldKey()) {
            Information information = fiche.getInformation(fieldKey);
            if (information == null) {
                return null;
            }
            if (information.getFicheItemCount() == 0) {
                return null;
            }
            return information;
        } else if (fieldKey.isSectionFieldKey()) {
            Section section = fiche.getSection(fieldKey);
            if (section == null) {
                return null;
            }
            if (section.getFicheBlockCount() == 0) {
                return null;
            }
            return section;
        } else {
            throw new SwitchException("unknown fieldKey = " + fieldKey.toString());
        }
    }

}
