/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.pointeurs;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.DocumentCroisementList;
import net.mapeadores.basedefiches.croisement.IllustrationCroisementList;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;


/**
 *
 * @author Vincent Calame
 */
public class MotclePointeur extends AbstractPointeur {

    private Thesaurus thesaurus;
    private Motcle currentMotcle;
    private Map<SubsetKey, FichePointeur> slaveFichePointeurMap;
    private Corpus[] rattachementCorpusArray;

    public MotclePointeur(Thesaurus thesaurus) {
        super(thesaurus);
        this.thesaurus = thesaurus;
        Subset[] slaveCorpusArray = thesaurus.getSlaveSubsetArray();
        if (slaveCorpusArray != null) {
            slaveFichePointeurMap = new HashMap<SubsetKey, FichePointeur>();
            int length = slaveCorpusArray.length;
            rattachementCorpusArray = new Corpus[length];
            for (int i = 0; i < length; i++) {
                Corpus slaveCorpus = (Corpus) slaveCorpusArray[i];
                rattachementCorpusArray[i] = slaveCorpus;
                slaveFichePointeurMap.put(slaveCorpus.getSubsetKey(), new FichePointeur(slaveCorpus, false));
            }
        }
    }

    @Override
    public MotcleCroisementList getMotcleCroisementList(Thesaurus thesaurus) {
        throw new UnsupportedOperationException();
    }

    @Override
    public IllustrationCroisementList getIllustrationCroisementList(Album album) {
        throw new UnsupportedOperationException();
    }

    @Override
    public DocumentCroisementList getDocumentCroisementList() {
        throw new UnsupportedOperationException();
    }

    protected void initCurrentSubsetItem(SubsetItem subsetItem) {
        Motcle motcle = (Motcle) subsetItem;
        this.currentMotcle = motcle;
        if (slaveFichePointeurMap != null) {
            for (FichePointeur fichePointeur : slaveFichePointeurMap.values()) {
                if (motcle == null) {
                    fichePointeur.setCurrentSubsetItem(null);
                } else {
                    SubsetItem slaveAbregeFiche = fichePointeur.getSubset().getSubsetItemById(motcle.getIdths());
                    fichePointeur.setCurrentSubsetItem(slaveAbregeFiche);
                }
            }
        }
    }

    public FichePointeur getSlaveFichePointeur(SubsetKey slaveCorpusKey) {
        if (slaveFichePointeurMap == null) {
            return null;
        }
        return slaveFichePointeurMap.get(slaveCorpusKey);
    }

    public Object getFieldValue(ThesaurusFieldKey thesaurusFieldKey) {
        if (currentMotcle == null) {
            return null;
        }
        short category = thesaurusFieldKey.getCategory();
        if (category == ThesaurusFieldKey.SPECIAL_CATEGORY) {
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.IDTHS)) {
                return new Integer(currentMotcle.getIdths());
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.IDALPHA)) {
                return currentMotcle.getIdalpha();
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.PARENT_IDTHS)) {
                Motcle parent = currentMotcle.getParent();
                if (parent == null) {
                    return null;
                }
                return new Integer(parent.getIdths());
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.PARENT_IDALPHA)) {
                Motcle parent = currentMotcle.getParent();
                if (parent == null) {
                    return null;
                }
                return parent.getIdalpha();
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.POSITION_LOCAL)) {
                return new Integer(currentMotcle.getChildIndex() + 1);
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.POSITION_GLOBAL)) {
                return ThesaurusUtils.getGlobalPosition(currentMotcle);
            }
            if (thesaurusFieldKey.equals(ThesaurusFieldKey.LEVEL)) {
                return new Integer(currentMotcle.getLevel());
            }
            throw new IllegalArgumentException("Unkwnon special thesaurusFieldKey : " + thesaurusFieldKey.toString());
        }
        if (category == ThesaurusFieldKey.LIB_CATEGORY) {
            return currentMotcle.getLibelleString(thesaurusFieldKey.getLangInteger(), false);
        }
        throw new IllegalArgumentException("Unkwnon thesaurusFieldKey : " + thesaurusFieldKey.toString());
    }

}
