/* BasedefichesLib_Explore - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.pointeurs;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Code;
import net.mapeadores.basedefiches.corpus.fiche.Courriel;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.Div;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlock;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Image;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Li;
import net.mapeadores.basedefiches.corpus.fiche.Link;
import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.fiche.Nombre;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.ParagraphBlock;
import net.mapeadores.basedefiches.corpus.fiche.Pays;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.fiche.Table;
import net.mapeadores.basedefiches.corpus.fiche.Tr;
import net.mapeadores.basedefiches.corpus.fiche.Ul;
import net.mapeadores.basedefiches.corpus.fiche.ZoneBlock;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.util.conditions.TextTestEngine;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class PointeurUtils {

    private PointeurUtils() {
    }

    public static Object getValue(AbregeFiche abregeFiche, FieldKey fieldKey) {
        boolean ficheComplete = false;
        if (fieldKey.ownsToCorpsdefiche()) {
            ficheComplete = true;
        }
        FichePointeur fichePointeur = new FichePointeur(abregeFiche.getCorpus(), ficheComplete, false);
        fichePointeur.setCurrentSubsetItem(abregeFiche);
        return fichePointeur.getValue(fieldKey);
    }

    public static FichePointeur toFichePointeur(AbregeFiche abregeFiche) {
        FichePointeur fichePointeur = new FichePointeur(abregeFiche.getCorpus(), false, false);
        fichePointeur.setCurrentSubsetItem(abregeFiche);
        return fichePointeur;
    }

    public static FichePointeur toFichePointeur(AbregeFiche abregeFiche, Fiche fiche, boolean ficheComplete) {
        return new FichePointeur(abregeFiche, fiche, ficheComplete);
    }

    public static MotclePointeur toMotclePointeur(Motcle motcle) {
        MotclePointeur motclePointeur = new MotclePointeur(motcle.getThesaurus());
        motclePointeur.setCurrentSubsetItem(motcle);
        return motclePointeur;
    }

    public static FicheCroisementList[] getNoList(FichePointeur fichePointeur, IncludeSummary includeSummary) {
        List<FicheCroisementList> listOfLists = new ArrayList<FicheCroisementList>();
        Basedefiches basedefiches = fichePointeur.getSubset().getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus otherCorpus = basedefiches.getCorpus(i);
            if (fichePointeur.isRattachementCorpus(otherCorpus)) {
                continue;
            }
            boolean done = false;
            if (includeSummary != null) {
                IncludeSummary.SubsetInfo info = includeSummary.getSubsetInfo(otherCorpus.getSubsetKey());
                if (info != null) {
                    if (info.hasAnyPoidsInclude()) {
                        done = true;
                    } else {
                        RangeList rangeList = info.getPoidsFilterRangeList();
                        if (rangeList != null) {
                            FicheCroisementList list = CroisementUtils.excludePoids(fichePointeur.getFicheCroisementList(otherCorpus), rangeList);
                            if (list.getSubsetItemCount() > 0) {
                                listOfLists.add(list);
                            }
                            done = true;
                        }
                    }
                }
            }
            if (!done) {
                FicheCroisementList list = fichePointeur.getFicheCroisementList(otherCorpus);
                if (list.getSubsetItemCount() > 0) {
                    listOfLists.add(list);
                }
            }
        }
        int size = listOfLists.size();
        FicheCroisementList[] result = new FicheCroisementList[size];
        for (int i = 0; i < size; i++) {
            result[i] = listOfLists.get(i);
        }
        return result;
    }

    public static FicheCroisementList[] getRattachementList(FichePointeur fichePointeur) {
        int size = fichePointeur.getRattachementCorpusCount();
        FicheCroisementList[] result = new FicheCroisementList[size];
        if (size == 0) {
            return result;
        }
        int p = 0;
        for (int i = 0; i < size; i++) {
            Corpus otherCorpus = fichePointeur.getRattachementCorpus(i);
            FicheCroisementList list = fichePointeur.getFicheCroisementList(otherCorpus);
            if (list.getSubsetItemCount() > 0) {
                result[p] = list;
                p++;
            }
        }
        if (p < size) {
            FicheCroisementList[] newResult = new FicheCroisementList[p];
            System.arraycopy(result, 0, newResult, 0, p);
            return newResult;
        } else {
            return result;
        }
    }

    public static void testContent(TextTestEngine textTestEngine, Object obj) {
        if (obj instanceof Integer) {
            textTestEngine.addString(obj.toString());
        } else if (obj instanceof String) {
            textTestEngine.addString(obj.toString());
        } else if (obj instanceof FicheItem) {
            testFicheItemContent(textTestEngine, (FicheItem) obj);
        } else if (obj instanceof FicheItemList) {
            FicheItemList ficheItemList = (FicheItemList) obj;
            int count = ficheItemList.getFicheItemCount();
            for (int i = 0; i < count; i++) {
                FicheItem ficheItem = ficheItemList.getFicheItem(i);
                testFicheItemContent(textTestEngine, ficheItem);
                if (textTestEngine.canStop()) {
                    return;
                }
            }
        } else if (obj instanceof FicheBlockList) {
            testFicheBlockListContent(textTestEngine, (FicheBlockList) obj);
        }
    }

    public static void testFicheBlockListContent(TextTestEngine contentSelector, FicheBlockList ficheBlockList) {
        int count = ficheBlockList.getFicheBlockCount();
        for (int i = 0; i < count; i++) {
            FicheBlock ficheBlock = ficheBlockList.getFicheBlock(i);
            if (ficheBlock instanceof ParagraphBlock) {
                contentSelector.addString(((ParagraphBlock) ficheBlock).toRawString());
            } else if (ficheBlock instanceof Ul) {
                Ul ul = (Ul) ficheBlock;
                int liCount = ul.getLiCount();
                for (int j = 0; j < liCount; j++) {
                    Li li = ul.getLi(j);
                    testFicheBlockListContent(contentSelector, li);
                    if (contentSelector.canStop()) {
                        return;
                    }
                }
            } else if (ficheBlock instanceof ZoneBlock) {
                ZoneBlock zoneBlock = (ZoneBlock) ficheBlock;
                ParagraphBlock numero = zoneBlock.getNumero();
                if (numero != null) {
                    contentSelector.addString(numero.toRawString());
                    if (contentSelector.canStop()) {
                        return;
                    }
                }
                ParagraphBlock legende = zoneBlock.getLegende();
                if (legende != null) {
                    contentSelector.addString(legende.toRawString());
                    if (contentSelector.canStop()) {
                        return;
                    }
                }
                if (zoneBlock instanceof Code) {
                    testFicheBlockListContent(contentSelector, (FicheBlockList) zoneBlock);
                } else if (zoneBlock instanceof Div) {
                    testFicheBlockListContent(contentSelector, ((Div) zoneBlock).getFicheBlockList());
                } else if (zoneBlock instanceof Table) {
                    Table table = (Table) zoneBlock;
                    int trCount = table.getTrCount();
                    for (int j = 0; j < trCount; j++) {
                        Tr tr = table.getTr(j);
                        testFicheBlockListContent(contentSelector, tr);
                        if (contentSelector.canStop()) {
                            return;
                        }
                    }
                }
            }
            if (contentSelector.canStop()) {
                return;
            }
        }
    }

    public static void testFicheItemContent(TextTestEngine contentSelector, FicheItem ficheItem) {
        if (ficheItem instanceof Item) {
            contentSelector.addString(((Item) ficheItem).getValue());
            return;
        }
        if (ficheItem instanceof Para) {
            contentSelector.addString(((Para) ficheItem).contentToString());
            return;
        }
        if (ficheItem instanceof Pays) {
            Pays pays = (Pays) ficheItem;
            contentSelector.addString(pays.getCountryCode());
            if (contentSelector.canStop()) {
                return;
            }
            contentSelector.addString(LangIntegerLocalisation.getCountryLibelle(pays.getCountryInteger(), contentSelector.getLocale()));
            return;
        }
        if (ficheItem instanceof Courriel) {
            Courriel courriel = (Courriel) ficheItem;
            contentSelector.addString(courriel.getAddrSpec());
            if (contentSelector.canStop()) {
                return;
            }
            if (courriel.hasNomComplet()) {
                contentSelector.addString(courriel.getNomComplet());
            }
            return;
        }
        if (ficheItem instanceof Langue) {
            Langue langue = (Langue) ficheItem;
            int langInteger = langue.getLangInteger();
            contentSelector.addString(LangInteger.toString(langInteger));
            if (contentSelector.canStop()) {
                return;
            }
            contentSelector.addString(LangIntegerLocalisation.getLangLibelle(langInteger, contentSelector.getLocale()));
            return;
        }
        if (ficheItem instanceof Geopoint) {
            Geopoint geopoint = (Geopoint) ficheItem;
            contentSelector.addString(geopoint.getLatitude().toString());
            if (contentSelector.canStop()) {
                return;
            }
            contentSelector.addString(geopoint.getLongitude().toString());
            return;
        }
        if (ficheItem instanceof Montant) {
            Montant montant = (Montant) ficheItem;
            contentSelector.addString(montant.getDecimal().toString());
            if (contentSelector.canStop()) {
                return;
            }
            contentSelector.addString(CurrencyInteger.toString(montant.getCurrencyInteger()));
            return;
        }
        if (ficheItem instanceof Personne) {
            Personne personne = (Personne) ficheItem;
            if (personne.isRedacteurSphere()) {
                contentSelector.addString(personne.getRedacteurKeyString());
            } else {
                String nom = personne.getNom();
                if (nom.length() > 0) {
                    contentSelector.addString(nom);
                    if (contentSelector.canStop()) {
                        return;
                    }
                }
                String prenom = personne.getPrenom();
                if (prenom.length() > 0) {
                    contentSelector.addString(prenom);
                    if (contentSelector.canStop()) {
                        return;
                    }
                }
                String original = personne.getOriginal();
                if (original.length() > 0) {
                    contentSelector.addString(original);
                    if (contentSelector.canStop()) {
                        return;
                    }
                }
                String organisme = personne.getOrganisme();
                if ((organisme != null) && (organisme.length() > 0)) {
                    contentSelector.addString(organisme);
                }
            }
            return;
        }
        if (ficheItem instanceof Link) {
            Link link = (Link) ficheItem;
            String href = link.getHref();
            if (href.length() > 0) {
                contentSelector.addString(href);
                if (contentSelector.canStop()) {
                    return;
                }
            }
            String title = link.getTitle();
            if (title.length() > 0) {
                contentSelector.addString(title);
                if (contentSelector.canStop()) {
                    return;
                }
            }
            String comment = link.getComment();
            if (comment.length() > 0) {
                contentSelector.addString(comment);
            }
            return;
        }
        if (ficheItem instanceof Image) {
            Image image = (Image) ficheItem;
            String src = image.getSrc();
            if (src.length() > 0) {
                contentSelector.addString(src);
                if (contentSelector.canStop()) {
                    return;
                }
            }
            String alt = image.getAlt();
            if (alt.length() > 0) {
                contentSelector.addString(alt);
                if (contentSelector.canStop()) {
                    return;
                }
            }
            String title = image.getTitle();
            if (title.length() > 0) {
                contentSelector.addString(title);
            }
            return;
        }
        if (ficheItem instanceof Datation) {
            contentSelector.addString(DateInteger.toString(((Datation) ficheItem).getDateInteger()));
            return;
        }
        if (ficheItem instanceof Nombre) {
            contentSelector.addString(((Nombre) ficheItem).getDecimal().toString());
            return;
        }
    }

}
