/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.AddendaList;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.pointeurs.DocumentPointeur;

/**
 *
 * @author Vincent Calame
 */
public class DocumentSelector {

    private Basedefiches basedefiches;
    private Map<SubsetKey, SelectorByAddenda> selectorByAddendaMap = new HashMap<SubsetKey, SelectorByAddenda>();
    private List<SelectorByAddenda> selectorByAddendaList = new ArrayList<SelectorByAddenda>();

    public DocumentSelector(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    public boolean containsAddenda(SubsetKey documentKey) {
        return selectorByAddendaMap.containsKey(documentKey);
    }

    public int getAddendaCount() {
        return selectorByAddendaList.size();
    }

    public Addenda getAddenda(int i) {
        SelectorByAddenda selectorByAddenda = selectorByAddendaList.get(i);
        return selectorByAddenda.getAddenda();
    }

    public boolean isSelected(Document document) {
        return isSelected(document, -1);
    }

    public boolean isSelected(Document document, int poids) {
        SelectorByAddenda selectorByAddenda = selectorByAddendaMap.get(document.getAddenda().getSubsetKey());
        if (selectorByAddenda == null) {
            return false;
        }
        return selectorByAddenda.isSelected(document, poids);
    }

    public void addDocumentSelectOptions(DocumentSelectOptions selectOptions) {
        add(selectOptions);
    }

    public void addDocumentSelectOptionsList(DocumentSelectOptionsList selectOptionsList) {
        int count = selectOptionsList.getDocumentSelectOptionsCount();
        if (count == 0) {
            add(new DocumentSelectOptions());
        } else {
            for (int i = 0; i < count; i++) {
                add(selectOptionsList.getDocumentSelectOptions(i));
            }
        }
    }

    private void add(DocumentSelectOptions documentSelectOptions) {
        AddendaList addendaList = SelectionUtils.getAddendaList(basedefiches, documentSelectOptions);
        int documentCount = addendaList.getAddendaCount();
        for (int i = 0; i < documentCount; i++) {
            Addenda document = addendaList.getAddenda(i);
            SelectorByAddenda selectorByAddenda = getSelectorByAddenda(document);
            selectorByAddenda.add(documentSelectOptions);
        }
    }

    private SelectorByAddenda getSelectorByAddenda(Addenda album) {
        SubsetKey albumKey = album.getSubsetKey();
        SelectorByAddenda selectorByAddenda = (SelectorByAddenda) selectorByAddendaMap.get(albumKey);
        if (selectorByAddenda == null) {
            selectorByAddenda = new SelectorByAddenda(album);
            selectorByAddendaMap.put(albumKey, selectorByAddenda);
            selectorByAddendaList.add(selectorByAddenda);
        }
        return selectorByAddenda;
    }


    private class SelectorByAddenda {

        private List<DocumentSelectUnit> documentSelectUnitList = new ArrayList<DocumentSelectUnit>();
        private DocumentPointeur documentPointeur;

        private SelectorByAddenda(Addenda addenda) {
            documentPointeur = new DocumentPointeur(addenda);
        }

        private Addenda getAddenda() {
            return (Addenda) documentPointeur.getSubset();
        }

        private void add(DocumentSelectOptions documentSelectOptions) {
            DocumentSelectUnit documentSelectUnit = DocumentSelectUnit.newInstance(documentPointeur, documentSelectOptions);
            documentSelectUnitList.add(documentSelectUnit);
        }

        private boolean isSelected(Document document, int poids) {
            documentPointeur.setCurrentSubsetItem(document);
            int size = documentSelectUnitList.size();
            for (int i = 0; i < size; i++) {
                if (documentSelectUnitList.get(i).isCurrentDocumentSelected(poids)) {
                    return true;
                }
            }
            return false;
        }

    }
}
