/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelectOptions {

    public final static short TITRE_SCOPE = 1;
    public final static short ENTETE_SCOPE = 2;
    public final static short FICHE_SCOPE = 3;
    public final static short SELECTION_SCOPE = 4;
    private RangeList rangeList;
    private MotcleSelectOptions[] motcleSelectOptionsArray;
    private Condition fieldCondition;
    private RedacteurKey[] redacteurKeyArray;
    private SubsetKey[] sphereKeyArray;
    private SubsetKey[] corpusKeyArray;
    private short motcleSelectMode = ConditionConstants.MODE_AND;
    private short fieldConditionScope;
    private FieldKey[] fieldKeyArray;
    private PoidsCondition poidsCondition;
    private boolean withCurrentCorpus;

    public FicheSelectOptions() {
    }

    public FicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        this.rangeList = ficheSelectOptions.rangeList;
        this.motcleSelectOptionsArray = ficheSelectOptions.motcleSelectOptionsArray;
        this.fieldCondition = ficheSelectOptions.fieldCondition;
        this.redacteurKeyArray = ficheSelectOptions.redacteurKeyArray;
        this.sphereKeyArray = ficheSelectOptions.sphereKeyArray;
        this.corpusKeyArray = ficheSelectOptions.corpusKeyArray;
        this.motcleSelectMode = ficheSelectOptions.motcleSelectMode;
        this.fieldConditionScope = ficheSelectOptions.fieldConditionScope;
        this.fieldKeyArray = ficheSelectOptions.fieldKeyArray;
        this.poidsCondition = ficheSelectOptions.poidsCondition;
        this.withCurrentCorpus = ficheSelectOptions.withCurrentCorpus;
    }

    public RangeList getRangeList() {
        return rangeList;
    }

    /**
     *
     *@throws IllegalArgumentException si rangeList n'est pas positif
     */
    public void setRangeList(RangeList rangeList) {
        if ((rangeList != null) && (!RangeUtils.isPositiveRangeList(rangeList))) {
            throw new IllegalArgumentException("rangeList is not positive");
        }
        this.rangeList = rangeList;
    }

    public MotcleSelectOptions[] getMotcleSelectOptionsArray() {
        return motcleSelectOptionsArray;
    }

    /**
     * Si motcleSelectOptions est vide, cela revient à donner la valeur nulle.
     */
    public void setMotcleSelectOptionsArray(MotcleSelectOptions[] motcleSelectOptionsArray) {
        if ((motcleSelectOptionsArray == null) || (motcleSelectOptionsArray.length == 0)) {
            this.motcleSelectOptionsArray = null;
        } else {
            this.motcleSelectOptionsArray = motcleSelectOptionsArray;
        }
    }

    public void setMotcleSelectMode(short motcleSelectMode) {
        this.motcleSelectMode = motcleSelectMode;
    }

    public short getMotcleSelectMode() {
        return motcleSelectMode;
    }

    public Condition getFieldCondition() {
        return fieldCondition;
    }

    public short getFieldConditionScope() {
        return fieldConditionScope;
    }

    public String getFieldConditionScopeString() {
        return fieldConditionToString(fieldConditionScope);
    }

    public void setFieldCondition(Condition fieldCondition, short fieldConditionScope, FieldKey[] fieldKeyArray) {
        switch (fieldConditionScope) {
            case TITRE_SCOPE:
            case ENTETE_SCOPE:
            case FICHE_SCOPE:
            case SELECTION_SCOPE:
                this.fieldConditionScope = fieldConditionScope;
                break;
            default:
                throw new IllegalArgumentException("Unknown fieldConditionScope : " + fieldConditionScope);
        }
        this.fieldCondition = fieldCondition;
        if (fieldConditionScope == SELECTION_SCOPE) {
            if (fieldKeyArray == null) {
                throw new IllegalArgumentException("fieldKeyArray is null");
            }
            if (fieldKeyArray.length == 0) {
                throw new IllegalArgumentException("fieldKeyArray is empty");
            }
            this.fieldKeyArray = fieldKeyArray;
        }
    }

    public FieldKey[] getFieldKeyArray() {
        return fieldKeyArray;
    }

    public RedacteurKey[] getRedacteurKeyArray() {
        return redacteurKeyArray;
    }

    public void setRedacteurKeyArray(RedacteurKey[] redacteurKeyArray) {
        this.redacteurKeyArray = redacteurKeyArray;
    }

    public SubsetKey[] getSphereKeyArray() {
        return sphereKeyArray;
    }

    public void setSphereKeyArray(SubsetKey[] sphereKeyArray) {
        this.sphereKeyArray = sphereKeyArray;
    }

    public SubsetKey[] getCorpusKeyArray() {
        return corpusKeyArray;
    }

    public void setCorpusKeyArray(SubsetKey[] corpusKeyArray) {
        this.corpusKeyArray = corpusKeyArray;
    }

    public PoidsCondition getPoidsCondition() {
        return poidsCondition;
    }

    public void setPoidsCondition(PoidsCondition poidsCondition) {
        this.poidsCondition = poidsCondition;
    }

    public boolean isWithCurrentCorpus() {
        return withCurrentCorpus;
    }

    public void setWithCurrentCorpus(boolean withCurrentCorpus) {
        this.withCurrentCorpus = withCurrentCorpus;
    }

    /**
     * Retourne <em>true</em> s'il y a au moins une option.
     */
    public boolean containsOption() {
        if (rangeList != null) {
            return true;
        }
        if (motcleSelectOptionsArray != null) {
            return true;
        }
        if (fieldCondition != null) {
            return true;
        }
        if (redacteurKeyArray != null) {
            return true;
        }
        if (getSphereKeyArray() != null) {
            return true;
        }
        if (corpusKeyArray != null) {
            return true;
        }
        if (poidsCondition != null) {
            return true;
        }
        return false;
    }

    public static String fieldConditionToString(short fieldConditionScope) {
        switch (fieldConditionScope) {
            case TITRE_SCOPE:
                return "titre";
            case ENTETE_SCOPE:
                return "entete";
            case FICHE_SCOPE:
                return "fiche";
            case SELECTION_SCOPE:
                return "selection";
            default:
                throw new IllegalArgumentException("Unknwon scope : " + fieldConditionScope);
        }
    }

    public static short fieldConditionToShort(String fieldConditionScopeString) {
        if (fieldConditionScopeString.equals("titre")) {
            return TITRE_SCOPE;
        }
        if (fieldConditionScopeString.equals("entete")) {
            return ENTETE_SCOPE;
        }
        if (fieldConditionScopeString.equals("fiche")) {
            return FICHE_SCOPE;
        }
        if (fieldConditionScopeString.equals("selection")) {
            return SELECTION_SCOPE;
        }
        throw new IllegalArgumentException("Unknwon scope : " + fieldConditionScopeString);
    }

}
