/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;


/**
 *
 * @author Vincent Calame
 */
public class FicheSelector {

    private FicheEligibility ficheEligibility;
    private Basedefiches basedefiches;
    private Map<SubsetKey, SelectorByCorpus> selectorByCorpusMap = new HashMap<SubsetKey, SelectorByCorpus>();
    private List<SelectorByCorpus> selectorByCorpusList = new ArrayList<SelectorByCorpus>();
    private int langInteger;
    private Corpus currentCorpus;

    public FicheSelector(Basedefiches basedefiches, int langInteger) {
        this.basedefiches = basedefiches;
        this.langInteger = langInteger;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public int getLangInteger() {
        return langInteger;
    }

    /**
     * Précise le filtre d'éligibilité.
     */
    public void setFicheEligibility(FicheEligibility ficheEligibility) {
        this.ficheEligibility = ficheEligibility;
    }

    public void setCurrentCorpus(Corpus currentCorpus) {
        this.currentCorpus = currentCorpus;
    }

    public void addFicheSelectOptionsList(FicheSelectOptionsList ficheSelectOptionsList) {
        int count = ficheSelectOptionsList.getFicheSelectOptionsCount();
        if (count == 0) {
            add(new FicheSelectOptions());
        } else {
            for (int i = 0; i < count; i++) {
                add(ficheSelectOptionsList.getFicheSelectOptions(i));
            }
        }
    }

    public void addFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        add(ficheSelectOptions);
    }

    private void add(FicheSelectOptions ficheSelectOptions) {
        IndexationSelector indexationSelector = null;
        MotcleSelectOptions[] motcleSelectOptionsArray = ficheSelectOptions.getMotcleSelectOptionsArray();
        if (motcleSelectOptionsArray != null) {
            indexationSelector = IndexationSelector.newInstance(basedefiches, langInteger, motcleSelectOptionsArray, ficheSelectOptions.getMotcleSelectMode());
        }
        boolean withCurrentCorpus = false;
        if ((currentCorpus != null) && (ficheSelectOptions.isWithCurrentCorpus())) {
            SelectorByCorpus selectorByCorpus = getSelectorByCorpus(currentCorpus);
            selectorByCorpus.add(indexationSelector, ficheSelectOptions);
            withCurrentCorpus = true;
        }
        SubsetKey[] corpusKeyArray = ficheSelectOptions.getCorpusKeyArray();
        if (corpusKeyArray != null) {
            for (int i = 0; i < corpusKeyArray.length; i++) {
                SelectorByCorpus selectorByCorpus = getSelectorByCorpus(corpusKeyArray[i]);
                if (selectorByCorpus != null) {
                    selectorByCorpus.add(indexationSelector, ficheSelectOptions);
                }
            }
        } else if (!withCurrentCorpus) {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                SelectorByCorpus selectorByCorpus = getSelectorByCorpus(corpus);
                selectorByCorpus.add(indexationSelector, ficheSelectOptions);
            }
        }
    }

    public boolean containsCorpus(SubsetKey corpusKey) {
        return selectorByCorpusMap.containsKey(corpusKey);
    }

    public boolean isSelected(AbregeFiche abregeFiche, int poids) {
        if (ficheEligibility != null) {
            if (!(ficheEligibility.accept(abregeFiche))) {
                return false;
            }
        }
        SelectorByCorpus selectorByCorpus = selectorByCorpusMap.get(abregeFiche.getCorpus().getSubsetKey());
        if (selectorByCorpus == null) {
            return false;
        }
        return selectorByCorpus.isSelected(abregeFiche, poids);
    }

    public boolean isSelected(AbregeFiche abregeFiche) {
        return isSelected(abregeFiche, -1);
    }

    public int getCorpusCount() {
        return selectorByCorpusList.size();
    }

    public Corpus getCorpus(int index) {
        SelectorByCorpus selectorByCorpus = (SelectorByCorpus) selectorByCorpusList.get(index);
        return selectorByCorpus.getCorpus();
    }

    private SelectorByCorpus getSelectorByCorpus(SubsetKey corpusKey) {
        SelectorByCorpus selectorByCorpus = (SelectorByCorpus) selectorByCorpusMap.get(corpusKey);
        if (selectorByCorpus == null) {
            Corpus corpus = (Corpus) basedefiches.getSubset(corpusKey);
            if (corpus == null) {
                return null;
            }
            return createSelectorByCorpus(corpus);
        }
        return selectorByCorpus;
    }

    private SelectorByCorpus getSelectorByCorpus(Corpus corpus) {
        SubsetKey corpusKey = corpus.getSubsetKey();
        SelectorByCorpus selectorByCorpus = (SelectorByCorpus) selectorByCorpusMap.get(corpusKey);
        if (selectorByCorpus == null) {
            return createSelectorByCorpus(corpus);
        }
        return selectorByCorpus;
    }

    private SelectorByCorpus createSelectorByCorpus(Corpus corpus) {
        SelectorByCorpus selectorByCorpus = new SelectorByCorpus(corpus);
        selectorByCorpusMap.put(corpus.getSubsetKey(), selectorByCorpus);
        selectorByCorpusList.add(selectorByCorpus);
        return selectorByCorpus;
    }


    private class SelectorByCorpus {

        private List<FicheSelectUnit> ficheSelectUnitList = new ArrayList<FicheSelectUnit>();
        private FichePointeur fichePointeur;

        private SelectorByCorpus(Corpus corpus) {
            this.fichePointeur = new FichePointeur(corpus);
        }

        private Corpus getCorpus() {
            return (Corpus) fichePointeur.getSubset();
        }

        private void add(IndexationSelector indexationSelector, FicheSelectOptions ficheSelectOptions) {
            FicheSelectUnit ficheSelectUnit = FicheSelectUnit.newInstance(fichePointeur, ficheSelectOptions, langInteger);
            ficheSelectUnit.setFicheByMotcleSelector(indexationSelector);
            ficheSelectUnitList.add(ficheSelectUnit);
        }

        private boolean isSelected(AbregeFiche abregeFiche, int poids) {
            fichePointeur.setCurrentSubsetItem(abregeFiche);
            int size = ficheSelectUnitList.size();
            for (int i = 0; i < size; i++) {
                if (ficheSelectUnitList.get(i).isCurrentAbregeFicheSelected(poids)) {
                    return true;
                }
            }
            return false;
        }

    }

}
