/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.AlbumList;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.pointeurs.IllustrationPointeur;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationSelector {

    private Basedefiches basedefiches;
    private Map<SubsetKey, SelectorByAlbum> selectorByAlbumMap = new HashMap<SubsetKey, SelectorByAlbum>();
    private List<SelectorByAlbum> selectorByAlbumList = new ArrayList<SelectorByAlbum>();

    public IllustrationSelector(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    public boolean containsAlbum(SubsetKey illustrationKey) {
        return selectorByAlbumMap.containsKey(illustrationKey);
    }

    public int getAlbumCount() {
        return selectorByAlbumList.size();
    }

    public Album getAlbum(int i) {
        SelectorByAlbum selectorByAlbum = selectorByAlbumList.get(i);
        return selectorByAlbum.getAlbum();
    }

    public boolean isSelected(Illustration illustration) {
        return isSelected(illustration, -1);
    }

    public boolean isSelected(Illustration illustration, int poids) {
        SelectorByAlbum selectorByAlbum = selectorByAlbumMap.get(illustration.getAlbum().getSubsetKey());
        if (selectorByAlbum == null) {
            return false;
        }
        return selectorByAlbum.isSelected(illustration, poids);
    }

    public void addIllustrationSelectOptions(IllustrationSelectOptions selectOptions) {
        add(selectOptions);
    }

    public void addIllustrationSelectOptionsList(IllustrationSelectOptionsList selectOptionsList) {
        int count = selectOptionsList.getIllustrationSelectOptionsCount();
        if (count == 0) {
            add(new IllustrationSelectOptions());
        } else {
            for (int i = 0; i < count; i++) {
                add(selectOptionsList.getIllustrationSelectOptions(i));
            }
        }
    }

    private void add(IllustrationSelectOptions illustrationSelectOptions) {
        AlbumList albumList = SelectionUtils.getAlbumList(basedefiches, illustrationSelectOptions);
        int illustrationCount = albumList.getAlbumCount();
        for (int i = 0; i < illustrationCount; i++) {
            Album illustration = albumList.getAlbum(i);
            SelectorByAlbum selectorByAlbum = getSelectorByAlbum(illustration);
            selectorByAlbum.add(illustrationSelectOptions);
        }
    }

    private SelectorByAlbum getSelectorByAlbum(Album album) {
        SubsetKey albumKey = album.getSubsetKey();
        SelectorByAlbum selectorByAlbum = (SelectorByAlbum) selectorByAlbumMap.get(albumKey);
        if (selectorByAlbum == null) {
            selectorByAlbum = new SelectorByAlbum(album);
            selectorByAlbumMap.put(albumKey, selectorByAlbum);
            selectorByAlbumList.add(selectorByAlbum);
        }
        return selectorByAlbum;
    }


    private class SelectorByAlbum {

        private List<IllustrationSelectUnit> illustrationSelectUnitList = new ArrayList<IllustrationSelectUnit>();
        private IllustrationPointeur illustrationPointeur;

        private SelectorByAlbum(Album album) {
            illustrationPointeur = new IllustrationPointeur(album);
        }

        private Album getAlbum() {
            return (Album) illustrationPointeur.getSubset();
        }

        private void add(IllustrationSelectOptions illustrationSelectOptions) {
            IllustrationSelectUnit illustrationSelectUnit = IllustrationSelectUnit.newInstance(illustrationPointeur, illustrationSelectOptions);
            illustrationSelectUnitList.add(illustrationSelectUnit);
        }

        private boolean isSelected(Illustration illustration, int poids) {
            illustrationPointeur.setCurrentSubsetItem(illustration);
            int size = illustrationSelectUnitList.size();
            for (int i = 0; i < size; i++) {
                if (illustrationSelectUnitList.get(i).isCurrentIllustrationSelected(poids)) {
                    return true;
                }
            }
            return false;
        }

    }

}
