/* BasedefichesLib_Explore - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class IndexationSelector {

    private Basedefiches basedefiches;
    private int langInteger;
    private short optionsMode = ConditionConstants.MODE_OR;
    private List<SelectorByOption> selectorByOptionsList = new ArrayList<SelectorByOption>();

    private IndexationSelector(Basedefiches basedefiches, int langInteger, short mode) {
        this.basedefiches = basedefiches;
        this.langInteger = langInteger;
        this.optionsMode = mode;
    }

    public static IndexationSelector newInstance(Basedefiches basedefiches, int langInteger, MotcleSelectOptions motcleSelectOptions) {
        IndexationSelector indexationSelector = new IndexationSelector(basedefiches, langInteger, ConditionConstants.MODE_AND);
        indexationSelector.addMotcleSelectOptions(motcleSelectOptions);
        return indexationSelector;
    }

    public static IndexationSelector newInstance(Basedefiches basedefiches, int langInteger, MotcleSelectOptions[] motcleSelectOptionsArray, short mode) {
        IndexationSelector indexationSelector = new IndexationSelector(basedefiches, langInteger, mode);
        int length = motcleSelectOptionsArray.length;
        for (int i = 0; i < length; i++) {
            indexationSelector.addMotcleSelectOptions(motcleSelectOptionsArray[i]);
        }
        return indexationSelector;
    }

    private void addMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
        if (motcleSelectOptions.isEmpty()) {
            return;
        }
        selectorByOptionsList.add(new SelectorByOption(motcleSelectOptions));
    }

    public boolean isSelected(AbregeFiche abregeFiche) {
        int size = selectorByOptionsList.size();
        if (size == 0) {
            return true;
        }
        if (optionsMode == ConditionConstants.MODE_OR) {
            for (int i = 0; i < size; i++) {
                if (selectorByOptionsList.get(i).isSelected(abregeFiche)) {
                    return true;
                }
            }
            return false;
        } else {
            for (int i = 0; i < size; i++) {
                if (!selectorByOptionsList.get(i).isSelected(abregeFiche)) {
                    return false;
                }
            }
            return true;
        }

    }


    private class SelectorByOption {

        private Thesaurus[] thesaurusArray;
        private Map<SubsetKey, IndexationSelectUnit> unitMap = new HashMap<SubsetKey, IndexationSelectUnit>();
        private short conditionState;
        MotcleTestEngine motcleTestEngine;
        MotcleRangeTestCache motcleRangeTestCache;

        private SelectorByOption(MotcleSelectOptions motcleSelectOptions) {
            RangeList levelRangeList = motcleSelectOptions.getLevelRangeList();
            RangeList idthsRangeList = motcleSelectOptions.getIdthsRangeList();
            if ((levelRangeList != null) || (idthsRangeList != null)) {
                motcleRangeTestCache = new MotcleRangeTestCache(levelRangeList, idthsRangeList);
            }
            initThesaurusArray(motcleSelectOptions.getThesaurusKeyArray());
            Condition condition = motcleSelectOptions.getFieldCondition();
            if (condition != null) {
                conditionState = condition.getState();
                if ((conditionState == ConditionConstants.P) || (conditionState == ConditionConstants.PA)) {
                    motcleTestEngine = new MotcleTestEngine(condition, langInteger);
                }
            } else {
                conditionState = ConditionConstants.U;
            }

        }

        private boolean isSelected(AbregeFiche abregeFiche) {
            IndexationSelectUnit indexationSelectUnit = getIndexationSelectUnit(abregeFiche.getCorpus());
            return indexationSelectUnit.isSelected(abregeFiche);
        }

        private IndexationSelectUnit getIndexationSelectUnit(Corpus corpus) {
            SubsetKey corpusKey = corpus.getSubsetKey();
            IndexationSelectUnit indexationSelectUnit = unitMap.get(corpusKey);
            if (indexationSelectUnit == null) {
                indexationSelectUnit = IndexationSelectUnit.newInstance(corpus, thesaurusArray, conditionState, motcleTestEngine, motcleRangeTestCache);
                unitMap.put(corpusKey, indexationSelectUnit);
            }
            return indexationSelectUnit;
        }

        private void initThesaurusArray(SubsetKey[] thesaurusKeyArray) {
            if (thesaurusKeyArray == null) {
                int thesaurusCount = basedefiches.getThesaurusCount();
                thesaurusArray = new Thesaurus[thesaurusCount];
                for (int i = 0; i < thesaurusCount; i++) {
                    thesaurusArray[i] = basedefiches.getThesaurus(i);
                }
            } else {
                int length = thesaurusKeyArray.length;
                Thesaurus[] temp = new Thesaurus[length];
                int p = 0;
                for (int i = 0; i < length; i++) {
                    Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKeyArray[i]);
                    if (thesaurus != null) {
                        temp[p] = thesaurus;
                        p++;
                    }
                }
                if (p != length) {
                    Thesaurus[] nvtmp = new Thesaurus[p];
                    System.arraycopy(temp, 0, nvtmp, 0, p);
                    thesaurusArray = nvtmp;
                } else {
                    thesaurusArray = temp;
                }
            }

        }

    }

}
