/* BasedefichesLib_Explore - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class MotcleRangeTestCache {

    private Map<SubsetKey, Map<Integer, Boolean>> cacheMap = new HashMap<SubsetKey, Map<Integer, Boolean>>();
    private Map<Integer, Boolean> currentMap;
    private Thesaurus currentThesaurus;
    private RangeList idthsRangeList;
    private RangeList levelRangeList;

    public MotcleRangeTestCache(RangeList levelRangeList, RangeList idthsRangeList) {
        this.levelRangeList = levelRangeList;
        this.idthsRangeList = idthsRangeList;
    }

    public void setCurrentThesaurus(Thesaurus thesaurus) {
        currentMap = cacheMap.get(thesaurus.getSubsetKey());
        if (currentMap == null) {
            currentMap = new HashMap<Integer, Boolean>();
            cacheMap.put(thesaurus.getSubsetKey(), currentMap);
        }
        currentThesaurus = thesaurus;
    }

    public boolean isSelected(int motcleTicket) {
        Boolean b = currentMap.get(motcleTicket);
        if (b == null) {
            Motcle motcle = (Motcle) currentThesaurus.getSubsetItemByTicket(motcleTicket);
            if (motcle == null) {
                b = Boolean.FALSE;
            } else {
                if (isRangeSelected(motcle)) {
                    b = Boolean.TRUE;
                } else {
                    b = Boolean.FALSE;
                }
            }
            currentMap.put(motcleTicket, b);
        }
        return b.booleanValue();
    }

    private boolean isRangeSelected(Motcle motcle) {
        if (levelRangeList != null) {
            if (!levelRangeList.contains(motcle.getLevel())) {
                return false;
            }
        }
        if (idthsRangeList != null) {
            if (!idthsRangeList.contains(motcle.getIdths())) {
                return false;
            }
        }
        return true;
    }

}
