/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSelectOptions {

    private RangeList idthsRangeList;
    private RangeList levelRangeList;
    private SubsetKey[] thesaurusKeyArray;
    private boolean idalphaOnly;
    private Condition fieldCondition;
    private PoidsCondition poidsCondition;

    public MotcleSelectOptions() {
    }

    public MotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
        this.idthsRangeList = motcleSelectOptions.idthsRangeList;
        this.levelRangeList = motcleSelectOptions.levelRangeList;
        this.thesaurusKeyArray = motcleSelectOptions.thesaurusKeyArray;
        this.fieldCondition = motcleSelectOptions.fieldCondition;
        this.idalphaOnly = motcleSelectOptions.idalphaOnly;
        this.poidsCondition = motcleSelectOptions.poidsCondition;
    }

    /**
     * null ou contient au moins un élément
     */
    public RangeList getIdthsRangeList() {
        return idthsRangeList;
    }

    /**
     *
     *@throws IllegalArgumentException si rangeList n'est pas positif
     */
    public void setIdthsRangeList(RangeList rangeList) {
        if ((rangeList == null) || (rangeList.size() == 0)) {
            this.idthsRangeList = null;
            return;
        }
        if (!RangeUtils.isPositiveRangeList(rangeList)) {
            throw new IllegalArgumentException("rangeList is not positive");
        }
        this.idthsRangeList = rangeList;
    }

    /**
     * null ou contient au moins un élément
     */
    public RangeList getLevelRangeList() {
        return levelRangeList;
    }

    public void setLevelRangeList(RangeList levelRangeList) {
        if ((levelRangeList == null) || (levelRangeList.size() == 0)) {
            this.levelRangeList = null;
            return;
        }
        this.levelRangeList = levelRangeList;
    }

    public SubsetKey[] getThesaurusKeyArray() {
        return thesaurusKeyArray;
    }

    public void setThesaurusKeyArray(SubsetKey[] thesaurusKeyArray) {
        if ((thesaurusKeyArray != null) && (thesaurusKeyArray.length == 0)) {
            thesaurusKeyArray = null;
        } else {
            this.thesaurusKeyArray = thesaurusKeyArray;
        }
    }

    /**
     * Indique que la condition est vide.
     */
    public boolean isEmpty() {
        if (fieldCondition != null) {
            return false;
        }
        if (idthsRangeList != null) {
            return false;
        }
        if (levelRangeList != null) {
            return false;
        }
        if (poidsCondition != null) {
            return false;
        }
        return true;
    }

    public void setFieldCondition(Condition fieldCondition) {
        this.fieldCondition = fieldCondition;
    }

    public Condition getFieldCondition() {
        return fieldCondition;
    }

    public boolean isIdalphaOnly() {
        return idalphaOnly;
    }

    public void setIdalphaOnly(boolean idalphaOnly) {
        this.idalphaOnly = idalphaOnly;
    }

    public PoidsCondition getPoidsCondition() {
        return poidsCondition;
    }

    public void setPoidsCondition(PoidsCondition poidsCondition) {
        this.poidsCondition = poidsCondition;
    }

    /**
     * Retourne <em>true</em> s'il y a au moins une option.
     */
    public boolean containsOption() {
        if (idthsRangeList != null) {
            return true;
        }
        if (levelRangeList != null) {
            return true;
        }
        if (thesaurusKeyArray != null) {
            return true;
        }
        if (fieldCondition != null) {
            return true;
        }
        if (poidsCondition != null) {
            return true;
        }
        return false;
    }

}
