/* BasedefichesLib_Explore - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurSelector {

    private Basedefiches basedefiches;
    private RedacteurEligibility redacteurEligibility;
    private int langInteger;
    private Map<SubsetKey, SphereSelectBundle> sphereSelectBundleMap = new HashMap<SubsetKey, SphereSelectBundle>();

    public RedacteurSelector(Basedefiches basedefiches, int langInteger) {
        this.basedefiches = basedefiches;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public int getLangInteger() {
        return langInteger;
    }

    public void setRedacteurEligibility(RedacteurEligibility redacteurEligibility) {
        this.redacteurEligibility = redacteurEligibility;
    }

    public void addRedacteurSelectOptions(RedacteurSelectOptions selectOptions) {
        int sphereCount = basedefiches.getSphereCount();
        for (int i = 0; i < sphereCount; i++) {
            Sphere sphere = basedefiches.getSphere(i);
            SphereSelectBundle sphereSelectBundle = getSphereSelectBundle(sphere);
            sphereSelectBundle.add(selectOptions);
        }
    }

    public boolean isSelected(Redacteur redacteur) {
        if (redacteurEligibility != null) {
            if (!(redacteurEligibility.accept(redacteur))) {
                return false;
            }
        }
        SphereSelectBundle sphereSelectBundle = sphereSelectBundleMap.get(redacteur.getSphere().getSubsetKey());
        if (sphereSelectBundle == null) {
            return false;
        }
        return sphereSelectBundle.isSelected(redacteur);
    }

    private SphereSelectBundle getSphereSelectBundle(Sphere sphere) {
        SubsetKey sphereKey = sphere.getSubsetKey();
        SphereSelectBundle sphereSelectBundle = sphereSelectBundleMap.get(sphereKey);
        if (sphereSelectBundle == null) {
            return createSphereSelectBundle(sphere);
        }
        return sphereSelectBundle;
    }

    private SphereSelectBundle createSphereSelectBundle(Sphere sphere) {
        SphereSelectBundle sphereSelectBundle = new SphereSelectBundle(sphere);
        sphereSelectBundleMap.put(sphere.getSubsetKey(), sphereSelectBundle);
        return sphereSelectBundle;
    }


    private class SphereSelectBundle {

        private Sphere sphere;
        private RedacteurSelectUnit[] redacteurSelectUnitArray = new RedacteurSelectUnit[2];
        private int size = 0;

        private SphereSelectBundle(Sphere sphere) {
            this.sphere = sphere;
        }

        private void add(RedacteurSelectOptions selectOptions) {
            if (size == redacteurSelectUnitArray.length) {
                RedacteurSelectUnit[] nvFSU = new RedacteurSelectUnit[size * 2];
                System.arraycopy(redacteurSelectUnitArray, 0, nvFSU, 0, size);
                redacteurSelectUnitArray = nvFSU;
            }
            RedacteurSelectUnit redacteurSelectUnit = new RedacteurSelectUnit(basedefiches);
            redacteurSelectUnit.setNomcompletCondition(selectOptions.getNomcompletCondition(), langInteger);
            redacteurSelectUnitArray[size] = redacteurSelectUnit;
            size++;
        }

        private boolean isSelected(Redacteur redacteur) {
            for (int i = 0; i < size; i++) {
                if (redacteurSelectUnitArray[i].isSelected(redacteur)) {
                    return true;
                }
            }
            return false;
        }

    }

}
