/* BasedefichesLib_Explore - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection;

import java.text.Collator;
import net.mapeadores.basedefiches.AlbumList;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.CorpusList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.ThesaurusList;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.selection.util.MutableAlbumList;
import net.mapeadores.basedefiches.selection.util.MutableCorpusList;
import net.mapeadores.basedefiches.selection.util.MutableThesaurusList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import net.mapeadores.basedefiches.AddendaList;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.selection.util.MutableAddendaList;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.Range;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class SelectionUtils implements SelectionConstants {

    public static final FicheSelectOptionsList EMPTY_FICHESELECTOPTIONSLIST = new EmptyFicheSelectOptionsList();
    public static final MotcleSelectOptionsList EMPTY_MOTCLESELECTOPTIONSLIST = new EmptyMotcleSelectOptionsList();
    public static final IllustrationSelectOptionsList EMPTY_ILLUSTRATIONSELECTOPTIONSLIST = new EmptyIllustrationSelectOptionsList();
    public static final DocumentSelectOptionsList EMPTY_DOCUMENTSELECTOPTIONSLIST = new EmptyDocumentSelectOptionsList();

    private SelectionUtils() {
    }

    public static AbregeFicheSortedList sortByIdcorpus(FicheSelector ficheSelector) {
        return getAbregeFicheSortedList(ficheSelector, IDCORPUS_SORTTYPE);
    }

    public static AbregeFicheSortedList noSort(FicheSelector ficheSelector) {
        return getAbregeFicheSortedList(ficheSelector, NONE_SORTTYPE);
    }

    public static AbregeFicheSortedList getAbregeFicheSortedList(FicheSelector ficheSelector, short sortType) {
        List<AbregeFiche> list = new ArrayList<AbregeFiche>();
        for (int i = 0; i < ficheSelector.getCorpusCount(); i++) {
            Corpus corpus = ficheSelector.getCorpus(i);
            for (int j = 0; j < corpus.getTicketLength(); j++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                if (abregeFiche != null) {
                    if (ficheSelector.isSelected(abregeFiche)) {
                        list.add(abregeFiche);
                    }
                }
            }
        }
        return sort(list, sortType, ficheSelector.getLangInteger());
    }

    public static AbregeFicheSortedList sort(Collection<AbregeFiche> list, short sortType, int langInteger) {
        switch (sortType) {
            case TITRE_SORTTYPE:
                return AbregeFicheSortedList.sortByTitre(list, langInteger);
            case CREATIONDATE_SORTTYPE:
                return AbregeFicheSortedList.sortByCreationDate(list);
            case MODIFICATIONDATE_SORTTYPE:
                return AbregeFicheSortedList.sortByModificationDate(list);
            case IDCORPUS_SORTTYPE:
                return AbregeFicheSortedList.sortByIdcorpus(list);
            case NONE_SORTTYPE:
                return AbregeFicheSortedList.noSort(list);
            default:
                throw new SwitchException("sortType=" + sortType);
        }
    }

    public static AbregeFicheSortedList sort(Collection<AbregeFiche> list, short sortType, Collator collator) {
        switch (sortType) {
            case TITRE_SORTTYPE:
                return AbregeFicheSortedList.sortByTitre(list, collator);
            case CREATIONDATE_SORTTYPE:
                return AbregeFicheSortedList.sortByCreationDate(list);
            case MODIFICATIONDATE_SORTTYPE:
                return AbregeFicheSortedList.sortByModificationDate(list);
            case IDCORPUS_SORTTYPE:
                return AbregeFicheSortedList.sortByIdcorpus(list);
            case NONE_SORTTYPE:
                return AbregeFicheSortedList.noSort(list);
            default:
                throw new SwitchException("sortType=" + sortType);
        }
    }

    public static ThesaurusList getThesaurusList(Basedefiches basedefiches, MotcleSelectOptions motcleSelectOptions) {
        if (motcleSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] thesaurusKeyArray = motcleSelectOptions.getThesaurusKeyArray();
        if (thesaurusKeyArray == null) {
            return basedefiches;
        }
        MutableThesaurusList list = new MutableThesaurusList();
        int length = thesaurusKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey thesaurusKey = thesaurusKeyArray[i];
            if (thesaurusKey == null) {
                continue;
            }
            if (!thesaurusKey.isThesaurusSubset()) {
                continue;
            }
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
            if (thesaurus != null) {
                list.add(thesaurus);
            }
        }
        return list;
    }

    public static CorpusList getCorpusList(Basedefiches basedefiches, FicheSelectOptions ficheSelectOptions) {
        if (ficheSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] corpusKeyArray = ficheSelectOptions.getCorpusKeyArray();
        if (corpusKeyArray == null) {
            return basedefiches;
        }
        MutableCorpusList list = new MutableCorpusList();
        int length = corpusKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey corpusKey = corpusKeyArray[i];
            if (corpusKey == null) {
                continue;
            }
            if (!corpusKey.isCorpusSubset()) {
                continue;
            }
            Corpus corpus = (Corpus) basedefiches.getSubset(corpusKey);
            if (corpus != null) {
                list.add(corpus);
            }
        }
        return list;
    }

    public static AlbumList getAlbumList(Basedefiches basedefiches, IllustrationSelectOptions illustrationSelectOptions) {
        if (illustrationSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] albumKeyArray = illustrationSelectOptions.getAlbumKeyArray();
        if (albumKeyArray == null) {
            return basedefiches;
        }
        MutableAlbumList list = new MutableAlbumList();
        int length = albumKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey albumKey = albumKeyArray[i];
            if (albumKey == null) {
                continue;
            }
            if (!albumKey.isAlbumSubset()) {
                continue;
            }
            Album album = (Album) basedefiches.getSubset(albumKey);
            if (album != null) {
                list.add(album);
            }
        }
        return list;
    }

    public static AddendaList getAddendaList(Basedefiches basedefiches, DocumentSelectOptions documentSelectOptions) {
        if (documentSelectOptions == null) {
            return basedefiches;
        }
        SubsetKey[] addendaKeyArray = documentSelectOptions.getAddendaKeyArray();
        if (addendaKeyArray == null) {
            return basedefiches;
        }
        MutableAddendaList list = new MutableAddendaList();
        int length = addendaKeyArray.length;
        for (int i = 0; i < length; i++) {
            SubsetKey addendaKey = addendaKeyArray[i];
            if (addendaKey == null) {
                continue;
            }
            if (!addendaKey.isAddendaSubset()) {
                continue;
            }
            Addenda addenda = (Addenda) basedefiches.getSubset(addendaKey);
            if (addenda != null) {
                list.add(addenda);
            }
        }
        return list;
    }

    public static short sortTypeToShort(String s) {
        if (s.equals("idcorpus")) {
            return IDCORPUS_SORTTYPE;
        }
        if (s.equals("titre")) {
            return TITRE_SORTTYPE;
        }
        if (s.equals("creationdate")) {
            return CREATIONDATE_SORTTYPE;
        }
        if (s.equals("modificationdate")) {
            return MODIFICATIONDATE_SORTTYPE;
        }
        if (s.equals("none")) {
            return NONE_SORTTYPE;
        }
        throw new IllegalArgumentException("wrong shortType value");
    }

    public static String sortTypeToString(short sortType) {
        switch (sortType) {
            case TITRE_SORTTYPE:
                return "titre";
            case CREATIONDATE_SORTTYPE:
                return "creationdate";
            case MODIFICATIONDATE_SORTTYPE:
                return "modificationdate";
            case IDCORPUS_SORTTYPE:
                return "idcorpus";
            case NONE_SORTTYPE:
                return "none";
            default:
                throw new IllegalArgumentException("wrong shortType value");
        }
    }

    public static boolean isValidSortType(short sortType) {
        switch (sortType) {
            case TITRE_SORTTYPE:
            case CREATIONDATE_SORTTYPE:
            case MODIFICATIONDATE_SORTTYPE:
            case IDCORPUS_SORTTYPE:
            case NONE_SORTTYPE:
                return true;
            default:
                return false;
        }
    }

    public static PoidsCondition newPoidsCondition(int poids) {
        RangeList rangeList = new RangeList();
        rangeList.addRange(new Range(poids, poids));
        return new InternalPoidsCondition(rangeList, false);
    }

    public static PoidsCondition newPoidsCondition(RangeList rangeList, boolean isExclude) {
        return new InternalPoidsCondition(rangeList, isExclude);
    }


    private static class InternalPoidsCondition implements PoidsCondition {

        private boolean exclusion;
        private RangeList poidsRangeList;

        private InternalPoidsCondition(RangeList poidsRangeList, boolean exclusion) {
            this.poidsRangeList = poidsRangeList;
            this.exclusion = exclusion;
        }

        public RangeList getPoidsRangeList() {
            return poidsRangeList;
        }

        public boolean isExclusionRangeList() {
            return exclusion;
        }

    }

    public static MotcleSelectOptionsList toMotcleSelectOptionsList(MotcleSelectOptions motcleSelectOptions) {
        if (motcleSelectOptions == null) {
            return EMPTY_MOTCLESELECTOPTIONSLIST;
        } else {
            return new SingletonMotcleSelectOptionsList(motcleSelectOptions);
        }
    }

    public static MotcleSelectOptionsList toMotcleSelectOptionsList(List<MotcleSelectOptions> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_MOTCLESELECTOPTIONSLIST;
        } else if (size == 1) {
            return new SingletonMotcleSelectOptionsList(list.get(0));
        } else {
            return new ArrayMotcleSelectOptionsList(list.toArray(new MotcleSelectOptions[size]));
        }
    }

    public static FicheSelectOptionsList toFicheSelectOptionsList(FicheSelectOptions ficheSelectOptions) {
        if (ficheSelectOptions == null) {
            return EMPTY_FICHESELECTOPTIONSLIST;
        } else {
            return new SingletonFicheSelectOptionsList(ficheSelectOptions);
        }
    }

    public static FicheSelectOptionsList toFicheSelectOptionsList(List<FicheSelectOptions> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_FICHESELECTOPTIONSLIST;
        } else if (size == 1) {
            return new SingletonFicheSelectOptionsList(list.get(0));
        } else {
            return new ArrayFicheSelectOptionsList(list.toArray(new FicheSelectOptions[size]));
        }
    }

    public static IllustrationSelectOptionsList toIllustrationSelectOptionsList(IllustrationSelectOptions illustrationSelectOptions) {
        if (illustrationSelectOptions == null) {
            return EMPTY_ILLUSTRATIONSELECTOPTIONSLIST;
        } else {
            return new SingletonIllustrationSelectOptionsList(illustrationSelectOptions);
        }
    }

    public static IllustrationSelectOptionsList toIllustrationSelectOptionsList(List<IllustrationSelectOptions> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_ILLUSTRATIONSELECTOPTIONSLIST;
        } else if (size == 1) {
            return new SingletonIllustrationSelectOptionsList(list.get(0));
        } else {
            return new ArrayIllustrationSelectOptionsList(list.toArray(new IllustrationSelectOptions[size]));
        }
    }

    public static DocumentSelectOptionsList toDocumentSelectOptionsList(DocumentSelectOptions documentSelectOptions) {
        if (documentSelectOptions == null) {
            return EMPTY_DOCUMENTSELECTOPTIONSLIST;
        } else {
            return new SingletonDocumentSelectOptionsList(documentSelectOptions);
        }
    }

    public static DocumentSelectOptionsList toDocumentSelectOptionsList(List<DocumentSelectOptions> list) {
        int size = list.size();
        if (size == 0) {
            return EMPTY_DOCUMENTSELECTOPTIONSLIST;
        } else if (size == 1) {
            return new SingletonDocumentSelectOptionsList(list.get(0));
        } else {
            return new ArrayDocumentSelectOptionsList(list.toArray(new DocumentSelectOptions[size]));
        }
    }


    private static class EmptyMotcleSelectOptionsList implements MotcleSelectOptionsList {

        public int getMotcleSelectOptionsCount() {
            return 0;
        }

        public MotcleSelectOptions getMotcleSelectOptions(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonMotcleSelectOptionsList implements MotcleSelectOptionsList {

        private MotcleSelectOptions motcleSelectOptions;

        private SingletonMotcleSelectOptionsList(MotcleSelectOptions motcleSelectOptions) {
            this.motcleSelectOptions = motcleSelectOptions;
        }

        public int getMotcleSelectOptionsCount() {
            return 1;
        }

        public MotcleSelectOptions getMotcleSelectOptions(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return motcleSelectOptions;
        }

    }


    private static class ArrayMotcleSelectOptionsList implements MotcleSelectOptionsList {

        private MotcleSelectOptions[] motcleSelectOptionsArray;

        private ArrayMotcleSelectOptionsList(MotcleSelectOptions[] motcleSelectOptionsArray) {
            this.motcleSelectOptionsArray = motcleSelectOptionsArray;
        }

        public int getMotcleSelectOptionsCount() {
            return motcleSelectOptionsArray.length;
        }

        public MotcleSelectOptions getMotcleSelectOptions(int i) {
            return motcleSelectOptionsArray[i];
        }

    }


    private static class EmptyFicheSelectOptionsList implements FicheSelectOptionsList {

        public int getFicheSelectOptionsCount() {
            return 0;
        }

        public FicheSelectOptions getFicheSelectOptions(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonFicheSelectOptionsList implements FicheSelectOptionsList {

        private FicheSelectOptions ficheSelectOptions;

        private SingletonFicheSelectOptionsList(FicheSelectOptions ficheSelectOptions) {
            this.ficheSelectOptions = ficheSelectOptions;
        }

        public int getFicheSelectOptionsCount() {
            return 1;
        }

        public FicheSelectOptions getFicheSelectOptions(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return ficheSelectOptions;
        }

    }


    private static class ArrayFicheSelectOptionsList implements FicheSelectOptionsList {

        private FicheSelectOptions[] ficheSelectOptionsArray;

        private ArrayFicheSelectOptionsList(FicheSelectOptions[] ficheSelectOptionsArray) {
            this.ficheSelectOptionsArray = ficheSelectOptionsArray;
        }

        public int getFicheSelectOptionsCount() {
            return ficheSelectOptionsArray.length;
        }

        public FicheSelectOptions getFicheSelectOptions(int i) {
            return ficheSelectOptionsArray[i];
        }

    }


    private static class EmptyIllustrationSelectOptionsList implements IllustrationSelectOptionsList {

        public int getIllustrationSelectOptionsCount() {
            return 0;
        }

        public IllustrationSelectOptions getIllustrationSelectOptions(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonIllustrationSelectOptionsList implements IllustrationSelectOptionsList {

        private IllustrationSelectOptions illustrationSelectOptions;

        private SingletonIllustrationSelectOptionsList(IllustrationSelectOptions illustrationSelectOptions) {
            this.illustrationSelectOptions = illustrationSelectOptions;
        }

        public int getIllustrationSelectOptionsCount() {
            return 1;
        }

        public IllustrationSelectOptions getIllustrationSelectOptions(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return illustrationSelectOptions;
        }

    }


    private static class ArrayIllustrationSelectOptionsList implements IllustrationSelectOptionsList {

        private IllustrationSelectOptions[] illustrationSelectOptionsArray;

        private ArrayIllustrationSelectOptionsList(IllustrationSelectOptions[] illustrationSelectOptionsArray) {
            this.illustrationSelectOptionsArray = illustrationSelectOptionsArray;
        }

        public int getIllustrationSelectOptionsCount() {
            return illustrationSelectOptionsArray.length;
        }

        public IllustrationSelectOptions getIllustrationSelectOptions(int i) {
            return illustrationSelectOptionsArray[i];
        }

    }


    private static class EmptyDocumentSelectOptionsList implements DocumentSelectOptionsList {

        public int getDocumentSelectOptionsCount() {
            return 0;
        }

        public DocumentSelectOptions getDocumentSelectOptions(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class SingletonDocumentSelectOptionsList implements DocumentSelectOptionsList {

        private DocumentSelectOptions documentSelectOptions;

        private SingletonDocumentSelectOptionsList(DocumentSelectOptions documentSelectOptions) {
            this.documentSelectOptions = documentSelectOptions;
        }

        public int getDocumentSelectOptionsCount() {
            return 1;
        }

        public DocumentSelectOptions getDocumentSelectOptions(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return documentSelectOptions;
        }

    }


    private static class ArrayDocumentSelectOptionsList implements DocumentSelectOptionsList {

        private DocumentSelectOptions[] documentSelectOptionsArray;

        private ArrayDocumentSelectOptionsList(DocumentSelectOptions[] documentSelectOptionsArray) {
            this.documentSelectOptionsArray = documentSelectOptionsArray;
        }

        public int getDocumentSelectOptionsCount() {
            return documentSelectOptionsArray.length;
        }

        public DocumentSelectOptions getDocumentSelectOptions(int i) {
            return documentSelectOptionsArray[i];
        }

    }

}
