/* BasedefichesLib_Explore - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.selection.util;

import java.text.ParseException;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.text.insertpattern.impl.InstructionParser;


/**
 *
 * @author Vincent Calame
 */
public class SelectOptionsParser {

    public static boolean isMotcleSelectOptionsInstruction(Instruction instruction) {
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            if (key.equals("thesaurus")) {
                return true;
            }
            if (key.equals("motcle-q")) {
                return true;
            }
            if (key.equals("motcle-q-a")) {
                return true;
            }
        }
        return false;
    }

    public static MotcleSelectOptions parseInstruction(Instruction instruction) {
        Set<SubsetKey> thesaurusKeySet = new LinkedHashSet<SubsetKey>();
        Condition condition = null;
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Argument argument = instruction.getArgument(i);
            String key = argument.getKey();
            String value = argument.getValue();
            if (key.equals("thesaurus")) {
                if (value == null) {
                    continue;
                }
                String[] tokens = StringUtils.getTechnicalTokens(value, true);
                for (int j = 0; j < tokens.length; j++) {
                    try {
                        SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, tokens[i]);
                        thesaurusKeySet.add(thesaurusKey);
                    } catch (ParseException pe) {
                    }
                }
            } else if (key.equals("motcle-q")) {
                if (value != null) {
                    condition = ConditionUtils.parseCondition(value, ConditionConstants.MODE_OR);
                }
            } else if (key.equals("motcle-q-a")) {
                if (value != null) {
                    condition = ConditionUtils.parseCondition(value, ConditionConstants.MODE_AND);
                }
            }
        }
        int thesaurusSize = thesaurusKeySet.size();
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        if (thesaurusSize > 0) {
            SubsetKey[] thesaurusKeyArray = new SubsetKey[thesaurusSize];
            int p = 0;
            for (SubsetKey thesaurusKey : thesaurusKeySet) {
                thesaurusKeyArray[p] = thesaurusKey;
                p++;
            }
            motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
        }
        motcleSelectOptions.setFieldCondition(condition);
        return motcleSelectOptions;
    }

    public static MotcleSelectOptions parse(String s) {
        Instruction instruction = InstructionParser.parse(s, null);
        if (instruction == null) {
            return null;
        }
        return parseInstruction(instruction);
    }

}
