/* BasedefichesLib_Export - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.scrutariexport;

import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import java.util.List;
import java.util.ArrayList;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefBuilder;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefUtils;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class CorpusScrutariDefBuilder {

    private InternalCorpusScrutariDef internalCorpusScrutariDef;

    public CorpusScrutariDefBuilder(SubsetKey corpusSubsetKey) {
        internalCorpusScrutariDef = new InternalCorpusScrutariDef(corpusSubsetKey);
    }

    public CorpusScrutariDef toCorpusScrutariDef() {
        return internalCorpusScrutariDef;
    }

    public void addComplementKey(Object key) {
        if (key == null) {
            throw new IllegalArgumentException("fieldKey cannot be null");
        }
        FormatSourceKey formatSourceKey = null;
        if (key instanceof FieldKey) {
            FieldKey fieldKey = (FieldKey) key;
            if (fieldKey.isSectionFieldKey()) {
                return;
            }
            if ((fieldKey.isSpecialFieldKey()) && (!fieldKey.equals(FieldKey.REDACTEURS))) {
                return;
            }
            formatSourceKey = FormatSourceKey.newFieldKeyInstance(fieldKey);
        } else if (key instanceof IncludeKey) {
            IncludeKey includeKey = ((IncludeKey) key);
            if (!includeKey.isCorpusInclude()) {
                return;
            }
            formatSourceKey = FormatSourceKey.newIncludeKeyInstance(includeKey);
        } else {
            throw new ClassCastException("key must be an instance of FieldKey or IncludeBundle");
        }
        if (formatSourceKey != null) {
            internalCorpusScrutariDef.addComplement(formatSourceKey);
        }
    }

    /**
     *  complementName peut être nul (ou vide)
     */
    public void addComplement(String complementName, LibelleHolder customLibelleHolder, BasedefichesFormatDef basedefichesFormatDef) {
        internalCorpusScrutariDef.addComplement(complementName, customLibelleHolder, basedefichesFormatDef);
    }

    public void addComplements(ColDefList colDefList) {
        int count = colDefList.getColDefCount();
        for (int i = 0; i < count; i++) {
            internalCorpusScrutariDef.addComplement(colDefList.getColDef(i));
        }
    }

    /**
     *@throws IllegalArgumentException si dateKey est nul
     */
    public void setDateKey(FormatSourceKey dateFormatSourceKey) {
        if (dateFormatSourceKey == null) {
            throw new IllegalArgumentException("dateKey argument is null");
        }
        internalCorpusScrutariDef.dateFormatSourceKey = dateFormatSourceKey;
    }

    public void setDateKey(String s) throws ParseException, WrongFormatSourceKeyException {
        FormatSourceKey dateFormatSourceKey = parseDateKey(s);
        internalCorpusScrutariDef.dateFormatSourceKey = dateFormatSourceKey;
    }

    public void setTitreBasedefichesFormatDef(BasedefichesFormatDef titreBasedefichesFormatDef) {
        internalCorpusScrutariDef.setTitreBasedefichesFormatDef(titreBasedefichesFormatDef);
    }

    public void setSoustitreBasedefichesFormatDef(BasedefichesFormatDef soustitreBasedefichesFormatDef) {
        internalCorpusScrutariDef.setSoustitreBasedefichesFormatDef(soustitreBasedefichesFormatDef);
    }

    public void setHrefPattern(AccoladePattern hrefPattern) {
        internalCorpusScrutariDef.setHrefPattern(hrefPattern);
    }

    public void setMultiLangKey(Object multiLangKey) {
        if (multiLangKey != null) {
            if (multiLangKey instanceof FieldKey) {
                FieldKey fieldKey = (FieldKey) multiLangKey;
                if ((!fieldKey.isInformationFieldKey()) && (!fieldKey.isProprieteFieldKey())) {
                    throw new IllegalArgumentException("multiLangKey as FieldKey must be Propriete or Information");
                }
            } else if (multiLangKey instanceof IncludeKey) {
                if (!((IncludeKey) multiLangKey).isThesaurusInclude()) {
                    throw new IllegalArgumentException("multiLangKey as IncludeKey must be a thesaurus");
                }
            } else {
                throw new IllegalArgumentException("multiLangKey must be an instance of FieldKey or IncludeKey: " + multiLangKey.getClass().getName());
            }
        }
        internalCorpusScrutariDef.multiLangKey = multiLangKey;
    }

    public static FormatSourceKey parseDateKey(String s) throws ParseException, WrongFormatSourceKeyException {
        if (s.equals("date_creation")) {
            return FormatSourceKey.DATE_CREATION;
        }
        if (s.equals("date_modification")) {
            return FormatSourceKey.DATE_MODIFICATION;
        }
        FieldKey fieldKey = FieldKey.parse(s);
        if ((!fieldKey.isProprieteFieldKey()) && (!fieldKey.isInformationFieldKey())) {
            throw new WrongFormatSourceKeyException();
        }
        return FormatSourceKey.newFieldKeyInstance(fieldKey);
    }


    private static class InternalCorpusScrutariDef implements CorpusScrutariDef {

        private SubsetKey corpusSubsetKey;
        private FormatSourceKey dateFormatSourceKey = FormatSourceKey.DATE_CREATION;
        private InternalColDefList colDefList = new InternalColDefList();
        private BasedefichesFormatDef titreBasedefichesFormatDef;
        private BasedefichesFormatDef soustitreBasedefichesFormatDef;
        private AccoladePattern hrefPattern;
        private Object multiLangKey;

        private InternalCorpusScrutariDef(SubsetKey corpusSubsetKey) {
            this.corpusSubsetKey = corpusSubsetKey;
        }

        public BasedefichesFormatDef getTitreFormatDef() {
            return titreBasedefichesFormatDef;
        }

        public BasedefichesFormatDef getSoustitreFormatDef() {
            return soustitreBasedefichesFormatDef;
        }

        public SubsetKey getCorpusSubsetKey() {
            return corpusSubsetKey;
        }

        public FormatSourceKey getDateFormatSourceKey() {
            return dateFormatSourceKey;
        }

        public ColDefList getComplementColDefList() {
            return colDefList;
        }

        public AccoladePattern getHrefPattern() {
            return hrefPattern;
        }

        public boolean isMultiLang() {
            return (multiLangKey != null);
        }

        public Object getMultiLangKey() {
            return multiLangKey;
        }

        private void addComplement(ColDef colDef) {
            colDefList.addComplement(colDef);
        }

        private void addComplement(FormatSourceKey formatSourceKey) {
            int numero = colDefList.getColDefCount() + 1;
            String colName = "comp_" + numero;
            BasedefichesFormatDefBuilder builder = new BasedefichesFormatDefBuilder();
            builder.addSource(formatSourceKey);
            builder.setSeparator(", ");
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(colName, null, builder.toBasedefichesFormatDef());
            colDefList.addComplement(colDef);
        }

        private void addComplement(String complementName, LibelleHolder customLibelleHolder, BasedefichesFormatDef basedefichesFormatDef) {
            if ((complementName == null) || (complementName.length() == 0)) {
                int numero = colDefList.getColDefCount() + 1;
                complementName = "comp_" + numero;
            }
            ColDef colDef = ColDefUtils.newBasedefichesFormatDefInstance(complementName, customLibelleHolder, basedefichesFormatDef);
            colDefList.addComplement(colDef);
        }

        private void setTitreBasedefichesFormatDef(BasedefichesFormatDef titreBasedefichesFormatDef) {
            this.titreBasedefichesFormatDef = titreBasedefichesFormatDef;
        }

        private void setSoustitreBasedefichesFormatDef(BasedefichesFormatDef soustitreBasedefichesFormatDef) {
            this.soustitreBasedefichesFormatDef = soustitreBasedefichesFormatDef;
        }

        public void setHrefPattern(AccoladePattern hrefPattern) {
            this.hrefPattern = hrefPattern;
        }

    }


    private static class InternalColDefList implements ColDefList {

        private List<ColDef> list = new ArrayList<ColDef>();

        public int getColDefCount() {
            return list.size();
        }

        public ColDef getColDef(int i) {
            return list.get(i);
        }

        public String getParameterValue(String parameterName) {
            return null;
        }

        private void addComplement(ColDef colDef) {
            list.add(colDef);
        }

    }

}
