/* BasedefichesLib_Export - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.scrutariexport;

import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.StringUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportDefBuilder {

    private InternalScrutariExportDef internalScrutariExportDef;
    private Set<SubsetKey> subsetKeySet = new HashSet<SubsetKey>();

    private ScrutariExportDefBuilder(String name) {
        this.internalScrutariExportDef = new InternalScrutariExportDef(name);
    }

    public static ScrutariExportDefBuilder newInstance(String name) throws ParseException {
        StringUtils.checkTechnicalName(name, true);
        return new ScrutariExportDefBuilder(name);
    }

    public ScrutariExportDef toScrutariExportDef() {
        return internalScrutariExportDef;
    }

    public void setBaseName(String baseName) throws ParseException {
        internalScrutariExportDef.setBaseName(baseName);
    }

    public void setBaseIcon(String baseIcon) {
        internalScrutariExportDef.setBaseIcon(baseIcon);
    }

    public void setFicheHrefPattern(AccoladePattern ficheHrefPattern) {
        internalScrutariExportDef.setFicheHrefPattern(ficheHrefPattern);
    }

    public void setMotcleHrefPattern(AccoladePattern motcleHrefPattern) {
        internalScrutariExportDef.setMotcleHrefPattern(motcleHrefPattern);
    }

    public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        internalScrutariExportDef.setFicheSelectOptions(ficheSelectOptions);
    }

    public void addCorpusScrutariDef(CorpusScrutariDef corpusScrutariDef) {
        if (!subsetKeySet.contains(corpusScrutariDef.getCorpusSubsetKey())) {
            internalScrutariExportDef.addCorpusScrutariDef(corpusScrutariDef);
        }
    }

    public void addThesaurusScrutariDef(ThesaurusScrutariDef thesaurusScrutariDef) {
        if (!subsetKeySet.contains(thesaurusScrutariDef.getThesaurusSubsetKey())) {
            internalScrutariExportDef.addThesaurusExportDef(thesaurusScrutariDef);
        }
    }

    public void setLangUiArray(int[] array) {
        internalScrutariExportDef.setLangUiArray(array);
    }


    private static class InternalScrutariExportDef implements ScrutariExportDef {

        private String name;
        private String baseName = null;
        private String baseIcon = null;
        private FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
        private AccoladePattern ficheHrefPattern;
        private AccoladePattern motcleHrefPattern;
        private List<CorpusScrutariDef> corpusScrutariDefList = new ArrayList<CorpusScrutariDef>();
        private List<ThesaurusScrutariDef> thesaurusScrutariDefList = new ArrayList<ThesaurusScrutariDef>();
        private int[] langUiArray;

        private InternalScrutariExportDef(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public String getBaseName() {
            return baseName;
        }

        public String getBaseIcon() {
            return baseIcon;
        }

        public AccoladePattern getFicheHrefPattern() {
            return ficheHrefPattern;
        }

        public AccoladePattern getMotcleHrefPattern() {
            return motcleHrefPattern;
        }

        public int getCorpusScrutariDefCount() {
            return corpusScrutariDefList.size();
        }

        public CorpusScrutariDef getCorpusScrutariDef(int i) {
            return corpusScrutariDefList.get(i);
        }

        public int getThesaurusScrutariDefCount() {
            return thesaurusScrutariDefList.size();
        }

        public ThesaurusScrutariDef getThesaurusScrutariDef(int i) {
            return thesaurusScrutariDefList.get(i);
        }

        public FicheSelectOptions getFicheSelectOptions() {
            return ficheSelectOptions;
        }

        public int getLangUICount() {
            if (langUiArray == null) {
                return 0;
            }
            return langUiArray.length;
        }

        public int getLangUI(int i) {
            if (langUiArray == null) {
                throw new IndexOutOfBoundsException();
            }
            return langUiArray[i];
        }

        private void setBaseName(String baseName) throws ParseException {
            if (baseName == null) {
                this.baseName = null;
            } else {
                StringUtils.checkTechnicalName(baseName, true);
                this.baseName = baseName;
            }
        }

        private void setBaseIcon(String baseIcon) {
            this.baseIcon = baseIcon;
        }

        private void setFicheHrefPattern(AccoladePattern ficheHrefPattern) {
            this.ficheHrefPattern = ficheHrefPattern;
        }

        private void setMotcleHrefPattern(AccoladePattern motcleHrefPattern) {
            this.motcleHrefPattern = motcleHrefPattern;
        }

        private void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
            this.ficheSelectOptions = ficheSelectOptions;
        }

        private void addCorpusScrutariDef(CorpusScrutariDef corpusScrutariDef) {
            corpusScrutariDefList.add(corpusScrutariDef);
        }

        private void addThesaurusExportDef(ThesaurusScrutariDef thesaurusScrutariDef) {
            thesaurusScrutariDefList.add(thesaurusScrutariDef);
        }

        private void setLangUiArray(int[] array) {
            if ((array == null) || (array.length == 0)) {
                this.langUiArray = null;
            } else {
                int length = array.length;
                langUiArray = new int[length];
                System.arraycopy(array, 0, langUiArray, 0, length);
            }
        }

    }

}
