/* BasedefichesLib_Export - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.scrutariexport;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.tableexport.ColDefUtils;
import net.mapeadores.util.text.AccoladePattern;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportUtils {

    private ScrutariExportUtils() {
    }

    public static CorpusScrutariDef getDefaultScrutariCorpusDef(SubsetKey corpusSubsetKey) {
        return new DefaultCorpusScrutariDef(corpusSubsetKey);
    }

    public static ThesaurusScrutariDef getDefaultScrutariThesaurusDef(SubsetKey thesaurusSubsetKey) {
        return new DefaultThesaurusScrutariDef(thesaurusSubsetKey);
    }

    public static Map<SubsetKey, CorpusScrutariDef> toCorpusScrutariDefMap(ScrutariExportDef scrutariExportDef) {
        Map<SubsetKey, CorpusScrutariDef> map = new HashMap<SubsetKey, CorpusScrutariDef>();
        int count = scrutariExportDef.getCorpusScrutariDefCount();
        for (int i = 0; i < count; i++) {
            CorpusScrutariDef corpusScrutariDef = scrutariExportDef.getCorpusScrutariDef(i);
            map.put(corpusScrutariDef.getCorpusSubsetKey(), corpusScrutariDef);
        }
        return map;
    }

    public static Map<SubsetKey, ThesaurusScrutariDef> toThesaurusScrutariDefMap(ScrutariExportDef scrutariExportDef) {
        Map<SubsetKey, ThesaurusScrutariDef> map = new HashMap<SubsetKey, ThesaurusScrutariDef>();
        int count = scrutariExportDef.getThesaurusScrutariDefCount();
        for (int i = 0; i < count; i++) {
            ThesaurusScrutariDef thesaurusScrutariDef = scrutariExportDef.getThesaurusScrutariDef(i);
            map.put(thesaurusScrutariDef.getThesaurusSubsetKey(), thesaurusScrutariDef);
        }
        return map;
    }


    private static class DefaultCorpusScrutariDef implements CorpusScrutariDef {

        private SubsetKey corpusSubsetKey;

        private DefaultCorpusScrutariDef(SubsetKey corpusSubsetKey) {
            this.corpusSubsetKey = corpusSubsetKey;
        }

        public BasedefichesFormatDef getTitreFormatDef() {
            return null;
        }

        public BasedefichesFormatDef getSoustitreFormatDef() {
            return null;
        }

        public SubsetKey getCorpusSubsetKey() {
            return corpusSubsetKey;
        }

        public FormatSourceKey getDateFormatSourceKey() {
            return FormatSourceKey.DATE_CREATION;
        }

        public ColDefList getComplementColDefList() {
            return ColDefUtils.EMPTY_COLDEFLIST;
        }

        public AccoladePattern getHrefPattern() {
            return null;
        }

        public boolean isMultiLang() {
            return false;
        }

        public Object getMultiLangKey() {
            return null;
        }

    }


    private static class DefaultThesaurusScrutariDef implements ThesaurusScrutariDef {

        private SubsetKey thesaurusSubsetKey;

        private DefaultThesaurusScrutariDef(SubsetKey thesaurusSubsetKey) {
            this.thesaurusSubsetKey = thesaurusSubsetKey;
        }

        public SubsetKey getThesaurusSubsetKey() {
            return thesaurusSubsetKey;
        }

        public boolean isSelected() {
            return false;
        }

        public String getMotcleFormatPattern() {
            return null;
        }

    }

}
