/* BasedefichesLib_Export - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.sqlexport;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportDefBuilder {

    private InternalSqlExportDef internalSqlExportDef;

    private SqlExportDefBuilder(String name) {
        internalSqlExportDef = new InternalSqlExportDef(name);
    }

    public static SqlExportDefBuilder newInstance(String name) throws ParseException {
        StringUtils.checkTechnicalName(name, true);
        return new SqlExportDefBuilder(name);
    }

    public SqlExportDef toSqlExportDef() {
        return internalSqlExportDef;
    }

    public void setSqlExportClassName(String sqlExportClassName) {
        if (sqlExportClassName == null) {
            throw new IllegalArgumentException("sqlExportClassName argument is null");
        }
        if (sqlExportClassName.equals("fr.exemole.bdf.sqlexport.DefaultSqlExport")) {
            internalSqlExportDef.sqlExportClassName = "";
        } else {
            internalSqlExportDef.sqlExportClassName = sqlExportClassName;
        }
    }

    public void setPath(String path) {
        if (path == null) {
            throw new IllegalArgumentException("path argument is null");
        }
        internalSqlExportDef.path = path;
    }

    public void setName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("name argument is null");
        }
        internalSqlExportDef.name = name;
    }

    public void setUrl(String url) {
        if ((url == null) || (url.length() == 0)) {
            internalSqlExportDef.url = null;
        } else {
            internalSqlExportDef.url = url;
        }
    }

    public void setTableExportName(String tableExportName) {
        internalSqlExportDef.tableExportName = tableExportName;
    }

    public void setPostCommand(String postCommand) {
        internalSqlExportDef.postCommand = postCommand;
    }

    public void addParam(String name, String value) {
        if ((name == null) || (name.length() == 0)) {
            throw new IllegalArgumentException("name is null or empty");
        }
        if (value == null) {
            value = "";
        }
        internalSqlExportDef.paramList.add(new InternalParam(name, value));
    }


    private static class InternalSqlExportDef implements SqlExportDef {

        private String name;
        private String sqlExportClassName = "";
        private String path = "";
        private String url = null;
        private String tableExportName = null;
        private String postCommand = null;
        private List<InternalParam> paramList = new ArrayList<InternalParam>();

        private InternalSqlExportDef(String name) {
            this.name = name;
        }

        public String getSqlExportClassName() {
            return sqlExportClassName;
        }

        public String getPath() {
            return path;
        }

        public String getName() {
            return name;
        }

        public String getUrl() {
            return url;
        }

        public String getTableExportName() {
            return tableExportName;
        }

        public String getPostCommand() {
            return postCommand;
        }

        public int getParamCount() {
            return paramList.size();
        }

        public Param getParam(int i) {
            return paramList.get(i);
        }

    }


    private static class InternalParam implements SqlExportDef.Param {

        private String name;
        private String value;

        private InternalParam(String name, String value) {
            this.name = name;
            this.value = value;
        }

        public String getName() {
            return name;
        }

        public String getValue() {
            return value;
        }

    }

}
