/* BasedefichesLib_Export - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tableexport;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatter;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatDefEngine;
import net.mapeadores.basedefiches.include.IncludeSummaryMap;
import net.mapeadores.basedefiches.include.IncludeSummaryMapBuilder;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.MotclePointeur;
import net.mapeadores.basedefiches.pointeurs.SubsetItemPointeur;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.exceptions.NestedClassLoaderException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.format.FormatConstants;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class SubsetTableExportBuilder implements TableExportConstants {

    private ComputedColDefList computedColDefList = new ComputedColDefList();
    private Subset subset;
    private LocalisationInfo localisationInfo;
    private IncludeSummaryMapBuilder includeSummaryMapBuilder;
    private TableExportExtension tableExportExtension;

    public SubsetTableExportBuilder(Subset subset, LocalisationInfo localisationInfo) {
        this.subset = subset;
        this.localisationInfo = localisationInfo;
        includeSummaryMapBuilder = new IncludeSummaryMapBuilder();
    }

    public void setTableExportExtension(TableExportExtension tableExportExtension) {
        this.tableExportExtension = tableExportExtension;
    }

    public ColDefTestResultList addColDefList(ColDefList colDefList) {
        InternalColDefTestResultList internalColDefTestResultList = new InternalColDefTestResultList();
        computedColDefList.addSourceColDefList(colDefList);
        int count = colDefList.getColDefCount();
        for (int i = 0; i < count; i++) {
            ColDef colDef = colDefList.getColDef(i);
            ColDefTestResult addResult = addColDef(colDef);
            internalColDefTestResultList.addAddResult(addResult);
        }
        return internalColDefTestResultList;
    }

    public ColDefTestResult addColDef(ColDef colDef) {
        short type = colDef.getType();
        switch (type) {
            case ColDef.FICHEFORMAT_TYPE:
                ComputeErrorHandler computeErrorHandler = new ComputeErrorHandler();
                BasedefichesFormatter formatter = BasedefichesFormatDefEngine.compute(subset, colDef.getBasedefichesFormatDef(), computeErrorHandler);
                if (formatter != null) {
                    computedColDefList.addComputedColDef(new ComputedColDef(colDef, formatter));
                    BasedefichesFormatDef bfd = colDef.getBasedefichesFormatDef();
                    int size = bfd.getFormatSourceKeyCount();
                    for (int i = 0; i < size; i++) {
                        FormatSourceKey formatSourceKey = bfd.getFormatSourceKey(i);
                        if (formatSourceKey.isIncludeKey()) {
                            includeSummaryMapBuilder.append(formatSourceKey.getIncludeKey(), formatSourceKey.getSlaveCorpusKey());
                        }
                    }
                    return ColDefTestResult.newDoneResult();
                } else {
                    return computeErrorHandler.addResult;
                }
            case ColDef.EXTENSION_TYPE:
                computedColDefList.addComputedColDef(new ComputedColDef(colDef, null));
                return ColDefTestResult.newDoneResult();
            default:
                throw new SwitchException("Unknown type = " + type);
        }
    }

    public SubsetTableExport toSubsetTableExport() {
        InternalSubsetTableExport internalSubsetTableExport = new InternalSubsetTableExport();
        internalSubsetTableExport.includeSummaryMap = includeSummaryMapBuilder.getIncludeSummaryMap();
        internalSubsetTableExport.subset = subset;
        internalSubsetTableExport.localisationInfo = localisationInfo;
        internalSubsetTableExport.computedColDefList = computedColDefList;
        if (tableExportExtension == null) {
            String extensionFactoryName = computedColDefList.getParameterValue(EXTENSION_FACTORY);
            if ((extensionFactoryName != null) && (extensionFactoryName.length() > 0)) {
                try {
                    Class extensionClass = Class.forName(extensionFactoryName);
                    TableExportExtensionFactory teef = (TableExportExtensionFactory) extensionClass.newInstance();
                    tableExportExtension = teef.newInstance(subset, localisationInfo);
                } catch (ClassNotFoundException cnfe) {
                    throw new NestedClassLoaderException(cnfe);
                } catch (InstantiationException ie) {
                    throw new ImplementationException(ie);
                } catch (IllegalAccessException ia) {
                    throw new ImplementationException(ia);
                }
            }
        }
        internalSubsetTableExport.tableExportExtension = tableExportExtension;
        return internalSubsetTableExport;
    }


    private static class InternalColDefTestResultList implements ColDefTestResultList {

        private List<ColDefTestResult> addResultList = new ArrayList<ColDefTestResult>();
        private boolean withError = false;

        void addAddResult(ColDefTestResult addResult) {
            addResultList.add(addResult);
            if (addResult.isError()) {
                withError = true;
            }
        }

        public boolean containsError() {
            return withError;
        }

        public int getAddResultCount() {
            return addResultList.size();
        }

        public ColDefTestResult getAddResult(int i) {
            return addResultList.get(i);
        }

    }


    private static class ComputedColDefList implements ColDefList {

        private ArrayList<ComputedColDef> computedColDefList = new ArrayList<ComputedColDef>();
        private ColDefList sourceColDefList;

        public int getColDefCount() {
            return computedColDefList.size();
        }

        public ColDef getColDef(int i) {
            return computedColDefList.get(i).getColDef();
        }

        public String getParameterValue(String parameterName) {
            return sourceColDefList.getParameterValue(parameterName);
        }

        ComputedColDef getComputedColDef(int i) {
            return computedColDefList.get(i);
        }

        void addComputedColDef(ComputedColDef computedColDef) {
            computedColDefList.add(computedColDef);
        }

        void addSourceColDefList(ColDefList sourceColDefList) {
            this.sourceColDefList = sourceColDefList;
        }

    }


    private static class InternalSubsetTableExport implements SubsetTableExport {

        private Subset subset;
        private LocalisationInfo localisationInfo;
        private IncludeSummaryMap includeSummaryMap;
        private ComputedColDefList computedColDefList;
        private TableExportExtension tableExportExtension;

        InternalSubsetTableExport() {
        }

        public ColDefList getColDefList() {
            return computedColDefList;
        }

        public Subset getSubset() {
            return subset;
        }

        public LocalisationInfo getLocalisationInfo() {
            return localisationInfo;
        }

        public void export(ColDefTableWriter tableWriter) {
            ExportRun er = new ExportRun(this, tableWriter);
            er.run();
        }

        public void export(ColDefTableWriter corpusTableWriter, SubsetItemList subsetItemList) {
            ExportRun er = new ExportRun(this, corpusTableWriter);
            er.setSubsetItemList(subsetItemList);
            er.run();
        }

    }


    private static class ExportRun {

        ComputedColDef[] colDefArray;
        SubsetItemList subsetItemList;
        SubsetItemPointeur subsetItemPointeur;
        ColDefTableWriter tableWriter;
        InternalSubsetTableExport internalSubsetTableExport;

        ExportRun(InternalSubsetTableExport internalCorpusTableExport, ColDefTableWriter tableWriter) {
            this.internalSubsetTableExport = internalCorpusTableExport;
            ComputedColDefList computedColDefList = internalCorpusTableExport.computedColDefList;
            boolean loadFiche = false;
            boolean withSlaveCorpusPointeur = false;
            int size = computedColDefList.getColDefCount();
            colDefArray = new ComputedColDef[size];
            for (int i = 0; i < size; i++) {
                ComputedColDef compColDef = computedColDefList.getComputedColDef(i);
                colDefArray[i] = compColDef;
                if (compColDef.getType() == ColDef.FICHEFORMAT_TYPE) {
                    BasedefichesFormatDef ffd = compColDef.getFicheFormatDef();
                    int size2 = ffd.getFormatSourceKeyCount();
                    for (int j = 0; j < size2; j++) {
                        FormatSourceKey formatSourceKey = ffd.getFormatSourceKey(j);
                        if (formatSourceKey.isFieldKey()) {
                            if (formatSourceKey.getFieldKey().ownsToCorpsdefiche()) {
                                loadFiche = true;
                            }
                        }
                        if (formatSourceKey.getSlaveCorpusKey() != null) {
                            withSlaveCorpusPointeur = true;
                        }
                    }
                }
            }
            Subset subset = internalCorpusTableExport.subset;
            if (subset instanceof Corpus) {
                this.subsetItemPointeur = new FichePointeur((Corpus) subset, loadFiche, withSlaveCorpusPointeur);
            } else if (subset instanceof Thesaurus) {
                this.subsetItemPointeur = new MotclePointeur((Thesaurus) subset);
            }
            this.tableWriter = tableWriter;
        }

        void setSubsetItemList(SubsetItemList subsetItemList) {
            this.subsetItemList = subsetItemList;
        }

        void run() {
            if (subsetItemList != null) {
                int count = subsetItemList.getSubsetItemCount();
                for (int i = 0; i < count; i++) {
                    add(subsetItemList.getSubsetItem(i));
                }
            } else {
                Subset subset = internalSubsetTableExport.subset;
                int length = subset.getTicketLength();
                for (int i = 0; i < length; i++) {
                    SubsetItem subsetItem = subset.getSubsetItemByTicket(i);
                    if (subsetItem != null) {
                        add(subsetItem);
                    }
                }
            }

        }

        void add(SubsetItem subsetItem) {
            TableExportExtension tableExportExtension = internalSubsetTableExport.tableExportExtension;
            subsetItemPointeur.setCurrentSubsetItem(subsetItem);
            tableWriter.startRow();
            int length = colDefArray.length;
            for (int i = 0; i < length; i++) {
                ComputedColDef compColDef = colDefArray[i];
                ColDef colDef = compColDef.getColDef();
                short type = colDef.getType();
                switch (type) {
                    case ColDef.FICHEFORMAT_TYPE:
                        BasedefichesFormatter formatter = compColDef.getComputedFicheFormatDef();
                        String chaine = null;
                        if (formatter.isLineSeparator()) {
                            List<String> liste = formatter.formatToList(subsetItemPointeur, internalSubsetTableExport.includeSummaryMap, internalSubsetTableExport.localisationInfo);
                            if (liste != null) {
                                if (liste.size() == 1) {
                                    chaine = liste.get(0);
                                } else {
                                    short castType = formatter.getCastType();
                                    if (castType == FormatConstants.NO_CAST) {
                                        tableWriter.addLineCell(colDef, liste);
                                        break;
                                    }
                                }
                            }
                        } else {
                            chaine = formatter.formatToString(subsetItemPointeur, internalSubsetTableExport.includeSummaryMap, internalSubsetTableExport.localisationInfo);
                        }
                        ColDefUtils.writeCell(tableWriter, colDef, chaine);
                        break;
                    case ColDef.EXTENSION_TYPE:
                        if (tableExportExtension != null) {
                            tableExportExtension.addColDefCell(tableWriter, colDef, subsetItem);
                        } else {
                            tableWriter.addNullCell(colDef);
                        }
                        break;
                    default:
                        throw new SwitchException("Unknwon type = " + type);
                }

            }
            tableWriter.endRow();
        }

    }


    private static class ComputedColDef {

        ColDef colDef;
        BasedefichesFormatter formatter;

        ComputedColDef(ColDef colDef, BasedefichesFormatter formatter) {
            this.colDef = colDef;
            this.formatter = formatter;
        }

        BasedefichesFormatDef getFicheFormatDef() {
            return colDef.getBasedefichesFormatDef();
        }

        ColDef getColDef() {
            return colDef;
        }

        short getType() {
            return colDef.getType();
        }

        BasedefichesFormatter getComputedFicheFormatDef() {
            return formatter;
        }

    }


    private static class ComputeErrorHandler implements BasedefichesFormatDefEngine.ErrorHandler {

        ColDefTestResult addResult;

        public void unknownSubfieldKey(SubfieldKey subfieldKey) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.UNKNWON_SUBFIELDKEY, subfieldKey);
        }

        public void illegalSubfieldKey(SubfieldKey subfieldKey) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.ILLEGAL_SUBFIELDKEY, subfieldKey);
        }

        public void wrongFormatPattern(String formatPattern) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.WRONG_PATTERN, formatPattern);
        }

        public void unknownFieldKey(FieldKey fieldKey) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.UNKNWON_FIELDKEY, fieldKey);
        }

        public void unknownSubsetKey(SubsetKey subsetKey) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.UNKNWON_SUBSETKEY, subsetKey);
        }

        public void notASlaveCorpusKey(SubsetKey subsetKey) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.NOTASLAVECORPUSKEY, subsetKey);
        }

        public void illegalFormatSourceKey(String keyString) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.ILLEGAL_FORMATSOURCEKEY, keyString);
        }

        public void noTypePrefixFieldKey(FieldKey fieldKey) {
            addResult = ColDefTestResult.newErrorResult(ColDefTestResult.NOTYPEPREFIX_FIELDKEY, fieldKey);
        }

    }

}
