/* BasedefichesLib_Export - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tableexport.logging;

import java.util.HashMap;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.tableexport.ColDefErrorHandler;
import net.mapeadores.util.logging.AbstractProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;


/**
 *
 * @author Vincent Calame
 */
public class TableExportLog extends AbstractProcessLog {

    public final static String UNKNOWNSUBSET_URI = "unknwonsubset";
    public final static String EMPTY_URI = "empty";
    private HashMap<SubsetKey, TESourceLog> sourceBySubsetKey = new HashMap<SubsetKey, TESourceLog>();

    public TableExportLog() {
    }

    public void addURILogItem(String key, SubsetProcessURI corpusProcessSource) {
        TESourceLog sourceLog = getSourceLog(corpusProcessSource);
        sourceLog.addURICorpusLogItem(key);
    }

    public void addMetadataErrorLogItem(SubsetKey corpusSubsetKey, String key, Object errorValue) {
        TESourceLog sourceLog = (TESourceLog) sourceBySubsetKey.get(corpusSubsetKey);
        sourceLog.addCorpusMetadataErrorLogItem(key, errorValue);
    }

    public ColDefErrorHandler getColDefListParserErrorHandler(SubsetProcessURI corpusProcessSource) {
        TESourceLog sourceLog = getSourceLog(corpusProcessSource);
        return sourceLog.getColDefListParserErrorHandler();
    }

    private TESourceLog getSourceLog(ProcessURI processSource) {
        TESourceLog sourceLog = (TESourceLog) getURILog(processSource.getProcessURIString());
        if (sourceLog == null) {
            sourceLog = new TESourceLog(processSource);
            putURILog(sourceLog);
        } else {
            sourceLog.testProcessURI(processSource);
        }
        return sourceLog;
    }


    class TESourceLog extends AbstractProcessLog.URILog {

        boolean isSubsetSource;

        TESourceLog(ProcessURI source) {
            super(source);
            indexBySubset();
        }

        private void indexBySubset() {
            if (processURI instanceof SubsetProcessURI) {
                sourceBySubsetKey.put(((SubsetProcessURI) processURI).getSubsetKey(), this);
                isSubsetSource = true;
            }
        }

        void testProcessURI(ProcessURI newProcessURI) {
            if ((!isSubsetSource) && (newProcessURI instanceof SubsetProcessURI)) {
                processURI = newProcessURI;
                indexBySubset();
            }
        }

        void addURICorpusLogItem(String key) {
            ProcessLogItem logItem = newProcessLogItem();
            logItem.setCategory("URI");
            logItem.setKey(key);
            logItem.setObject(((SubsetProcessURI) processURI).getSubsetKey());
            add(logItem);
        }

        void addCorpusMetadataErrorLogItem(String key, Object errorValue) {
            ProcessLogItem logItem = newProcessLogItem();
            logItem.setCategory("CORPUSMETADATA");
            logItem.setKey(key);
            logItem.setObject(errorValue);
            add(logItem);
        }

        ColDefErrorHandler getColDefListParserErrorHandler() {
            return new ColDefListErrorHandler();
        }


        class ColDefListErrorHandler implements ColDefErrorHandler {

            public void wrongParameterLine(int lineNumber, String line, String key) {
                ProcessLogItem logItem = newProcessLogItem();
                logItem.setLineNumber(lineNumber);
                logItem.setCategory("WARNING");
                logItem.setKey("wrongparameterline_" + key);
                logItem.setObject(line);
                add(logItem);
            }

            public void tooManyLineWarning(int lineNumber, String line) {
                ProcessLogItem logItem = newProcessLogItem();
                logItem.setLineNumber(lineNumber);
                logItem.setCategory("WARNING");
                logItem.setKey("toomanyline");
                logItem.setObject(line);
                add(logItem);
            }

            public void missingFormatSourceLineError(int lineNumber) {
                ProcessLogItem logItem = newProcessLogItem();
                logItem.setLineNumber(lineNumber);
                logItem.setCategory("FORMAT_SOURCE");
                logItem.setKey("missingformatsourceline");
                logItem.setObject(null);
                add(logItem);
            }

            public void formatSourceError(int lineNumber, String key, Object value) {
                ProcessLogItem logItem = newProcessLogItem();
                logItem.setLineNumber(lineNumber);
                logItem.setCategory("FORMAT_SOURCE");
                logItem.setKey(key);
                logItem.setObject(value);
                add(logItem);
            }

            public void formatPatternError(int lineNumber, String key, Object value) {
                ProcessLogItem logItem = newProcessLogItem();
                logItem.setLineNumber(lineNumber);
                logItem.setCategory("FORMAT_PATTERN");
                logItem.setKey(key);
                logItem.setObject(value);
                add(logItem);
            }

            public void formatInstructionError(int lineNumber, String key, Object value) {
                ProcessLogItem logItem = newProcessLogItem();
                logItem.setLineNumber(lineNumber);
                logItem.setCategory("FORMAT_INSTRUCTION");
                logItem.setKey(key);
                logItem.setObject(value);
                add(logItem);
            }

        }

    }

}
