/* BasedefichesLib_Export - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tableexport.thesaurus;

import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.util.text.Libelle;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusTableExport {

    private List<ThesaurusFieldKey> thesaurusFieldKeyList = new ArrayList<ThesaurusFieldKey>();
    private Thesaurus thesaurus;
    private Map<Motcle, Boolean> rootMap = new LinkedHashMap<Motcle, Boolean>();
    private ThesaurusTableWriter currentThesaurusTableWriter;
    private ThesaurusFieldKey[] currentArray = null;

    public ThesaurusTableExport(Thesaurus thesaurus) {
        this.thesaurus = thesaurus;
    }

    public Thesaurus getThesaurus() {
        return thesaurus;
    }

    /**
     * Vérifie les doublons.
     */
    public void addThesaurusFieldKey(ThesaurusFieldKey thesaurusFieldKey) {
        if (!thesaurusFieldKeyList.contains(thesaurusFieldKey)) {
            thesaurusFieldKeyList.add(thesaurusFieldKey);
        }
    }

    public void addRoot(Motcle motcle, boolean include) {
        rootMap.put(motcle, new Boolean(include));
    }

    public ThesaurusFieldKey[] getThesaurusFieldKeyArray() {
        int size = thesaurusFieldKeyList.size();
        ThesaurusFieldKey[] array = new ThesaurusFieldKey[size];
        for (int i = 0; i < size; i++) {
            array[i] = thesaurusFieldKeyList.get(i);
        }
        return array;
    }

    public void export(ThesaurusTableWriter thesaurusTableWriter) {
        currentThesaurusTableWriter = thesaurusTableWriter;
        currentArray = getThesaurusFieldKeyArray();
        if (rootMap.size() == 0) {
            MotcleList firstLevel = thesaurus.getFirstLevelList();
            for (int i = 0; i < firstLevel.getMotcleCount(); i++) {
                exportMotcle(firstLevel.getMotcle(i), true);
            }
        } else {
            for (Map.Entry<Motcle, Boolean> entry : rootMap.entrySet()) {
                exportMotcle(entry.getKey(), entry.getValue().booleanValue());
            }
        }
        currentThesaurusTableWriter = null;
        currentArray = null;
    }

    private void exportMotcle(Motcle motcle, boolean include) {
        if (include) {
            currentThesaurusTableWriter.startRow(motcle);
            Motcle parent = motcle.getParent();
            int length = currentArray.length;
            for (int i = 0; i < length; i++) {
                ThesaurusFieldKey key = currentArray[i];
                if (key.equals(ThesaurusFieldKey.IDTHS)) {
                    currentThesaurusTableWriter.addIdthsCell(key, new Integer(motcle.getIdths()));
                } else if (key.equals(ThesaurusFieldKey.PARENT_IDTHS)) {
                    Integer idthsparent = (parent != null) ? new Integer(parent.getIdths()) : null;
                    currentThesaurusTableWriter.addIdthsCell(key, idthsparent);
                } else if (key.equals(ThesaurusFieldKey.IDALPHA)) {
                    currentThesaurusTableWriter.addIdalphaCell(key, motcle.getIdalpha());
                } else if (key.equals(ThesaurusFieldKey.PARENT_IDALPHA)) {
                    String parentIdalpha = (parent != null) ? parent.getIdalpha() : null;
                    currentThesaurusTableWriter.addIdalphaCell(key, parentIdalpha);
                } else if (key.equals(ThesaurusFieldKey.BABELIENLANG)) {
                    int intlang = ThesaurusUtils.getDefaultPreferredLangInteger(motcle);
                    currentThesaurusTableWriter.addBabelienLangCell(intlang);
                } else if (key.equals(ThesaurusFieldKey.BABELIENLIB)) {
                    Libelle motcleLibelle = ThesaurusUtils.getBabelienLibelle(motcle);
                    currentThesaurusTableWriter.addLibCell(key, motcleLibelle.getLibelleString());
                } else if (key.isLibThesaurusFieldKey()) {
                    int langInteger = key.getLangInteger();
                    currentThesaurusTableWriter.addLibCell(key, motcle.getLibelleString(langInteger, false));
                }
            }
            currentThesaurusTableWriter.endRow();
        }
        MotcleList childList = motcle.getChildList();
        for (int i = 0; i < childList.getMotcleCount(); i++) {
            exportMotcle(childList.getMotcle(i), true);
        }
    }

    public static ThesaurusTableExport getDefault(Thesaurus thesaurus) {
        ThesaurusTableExport thesaurusTableExport = new ThesaurusTableExport(thesaurus);
        ThesaurusFieldKey[] tfk = ThesaurusFieldKey.getDefaultFieldsArray(thesaurus.getThesaurusMetadata());
        int length = tfk.length;
        for (int i = 0; i < length; i++) {
            thesaurusTableExport.addThesaurusFieldKey(tfk[i]);
        }
        return thesaurusTableExport;
    }

}
