/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import java.util.List;
import java.util.ArrayList;
import net.mapeadores.util.text.Libelle;


/**
 * Usage interne : implémente toutes les méthodes d'un mot-clé hors de ce qui touche à la langue.
 *
 * @author Vincent Calame
 */
abstract class AbstractMotcle implements Motcle {

    private String idalpha;
    private int level = 1;
    private BdfimplThesaurus thesaurus;
    private AbstractMotcle parent;
    private int idths;
    private int ticket;
    private int childIndex;
    private ChildMotcleList childMotcleList = new ChildMotcleList();
    private List<AbstractMotcle> childrenList;

    public AbstractMotcle(BdfimplThesaurus thesaurus, int idths) {
        this.thesaurus = thesaurus;
        this.idths = idths;
    }

    public int getLevel() {
        return level;
    }

    public Motcle getParent() {
        return parent;
    }

    public Thesaurus getThesaurus() {
        return thesaurus;
    }

    public Subset getSubset() {
        return thesaurus;
    }

    public int getTicket() {
        return ticket;
    }

    public int getIdths() {
        return idths;
    }

    public int getIdsubset() {
        return idths;
    }

    public String getIdalpha() {
        return idalpha;
    }

    public int getChildIndex() {
        return childIndex;
    }

    public MotcleList getChildList() {
        return childMotcleList;
    }

    abstract boolean putLibelle(Libelle libelle);

    abstract boolean removeLibelle(int langInteger);

    void setTicket(int ticket) {
        this.ticket = ticket;
    }

    boolean setIdalpha(String newIdalpha) {
        if (idalpha == null) {
            idalpha = newIdalpha;
            return true;
        } else {
            if (newIdalpha.equals(idalpha)) {
                return false;
            }
            idalpha = newIdalpha;
            return true;
        }
    }

    boolean setParent(AbstractMotcle newParent) {
        boolean todo = parentTest(newParent);
        if (!todo) {
            return false;
        }
        removeFromParent();
        if (newParent != null) {
            level = newParent.getLevel() + 1;
            newParent.addChild(this);
        } else {
            level = 1;
            thesaurus.addInFirstLevel(this);
        }
        parent = newParent;
        return true;
    }

    void removeFromParent() {
        if (parent != null) {
            parent.removeChild(childIndex);
        } else {
            thesaurus.removeFromFirstLevel(childIndex);
        }
        childIndex = -1;
        level = -1;
    }

    private void removeChild(int index) {
        childrenList.remove(index);
        int size = childrenList.size();
        for (int i = index; i < size; i++) {
            AbstractMotcle child = childrenList.get(i);
            child.decreaseIndex();
        }
    }

    private void addChild(AbstractMotcle motcle) {
        if (childrenList == null) {
            childrenList = new ArrayList<AbstractMotcle>();
        }
        int size = childrenList.size();
        childrenList.add(motcle);
        motcle.setChildIndex(size);
    }

    void decreaseIndex() {
        childIndex = childIndex - 1;
    }

    void increaseIndex() {
        childIndex = childIndex + 1;
    }

    void setChildIndex(int childIndex) {
        this.childIndex = childIndex;
    }

    boolean changeIndex(int newIndex) {
        if (newIndex == childIndex) {
            return false;
        }
        List<AbstractMotcle> list = (parent != null) ? parent.childrenList : thesaurus.firstLevel;
        if ((newIndex < 0) || (newIndex >= list.size())) {
            throw new IndexOutOfBoundsException();
        }
        permute(list, childIndex, newIndex);
        this.childIndex = newIndex;
        return true;
    }

    boolean parentTest(AbstractMotcle newParent) {
        if (newParent == null) {
            return (parent != null);
        } else if (parent == null) {
            return true;
        } else {
            return !parent.equals(newParent);
        }
    }

    void permute(List<AbstractMotcle> list, int oldIndex, int newIndex) {
        AbstractMotcle movedMotcle = list.set(newIndex, list.get(oldIndex));
        if (oldIndex > newIndex) {
            for (int i = newIndex + 1; i <= oldIndex; i++) {
                movedMotcle.setChildIndex(i);
                movedMotcle = list.set(i, movedMotcle);
            }
        } else {
            for (int i = newIndex - 1; i >= oldIndex; i--) {
                movedMotcle.setChildIndex(i);
                movedMotcle = list.set(i, movedMotcle);
            }
        }
    }


    private class ChildMotcleList implements MotcleList {

        public Thesaurus getThesaurus() {
            return thesaurus;
        }

        public int getMotcleCount() {
            if (childrenList == null) {
                return 0;
            }
            return childrenList.size();
        }

        public Motcle getMotcle(int i) {
            return childrenList.get(i);
        }

    }

}
