/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
class BabelienMotcle extends AbstractMotcle {

    private static final Libelle noLinguisticContentLibelle = new NoLinguisticContentLibelle();
    private int langInteger = LangInteger.NO_LINGUISTIC_CONTENT;
    private Libelle libelle = noLinguisticContentLibelle;

    public BabelienMotcle(BdfimplThesaurus thesaurus, int idths) {
        super(thesaurus, idths);
    }

    public Libelle getLibelleByLang(int langInteger) {
        if (langInteger == this.langInteger) {
            return libelle;
        }
        return null;
    }

    public int getLibelleCount() {
        return 1;
    }

    public Libelle getLibelleByIndex(int i) {
        if (i != 0) {
            throw new IndexOutOfBoundsException();
        }
        return libelle;
    }

    public String getLibelleString(int langInteger, boolean defaut) {
        if (defaut) {
            if (this.langInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
                return "?";
            }
            return libelle.getLibelleString();
        } else {
            if (langInteger == this.langInteger) {
                return libelle.getLibelleString();
            } else {
                return null;
            }
        }
    }

    boolean putLibelle(Libelle libelle) {
        if (LibelleUtils.areEquals(this.libelle, libelle)) {
            return false;
        }
        this.langInteger = libelle.getLangInteger();
        if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
            this.libelle = noLinguisticContentLibelle;
        } else {
            this.libelle = libelle;
        }
        return true;
    }

    boolean removeLibelle(int langInteger) {
        if (this.libelle == noLinguisticContentLibelle) {
            return false;
        }
        this.libelle = noLinguisticContentLibelle;
        this.langInteger = LangInteger.NO_LINGUISTIC_CONTENT;
        return true;
    }


    private static class NoLinguisticContentLibelle implements Libelle {

        public String getLibelleString() {
            return "";
        }

        public int getLangInteger() {
            return LangInteger.NO_LINGUISTIC_CONTENT;
        }

    }

}
