/* BasedefichesLib_Impl - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.io.IOException;
import java.io.InputStream;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadata;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;
import net.mapeadores.util.collections.AbstractIndexAndIntKeyMap;


/**
 *
 * @author Vincent Calame
 */
class BdfimplAlbum extends AbstractIndexAndIntKeyMap implements Album {

    private BdfimplBasedefiches basedefiches;
    private BdfimplAlbumMetadata albumMetadata;
    private SubsetKey albumKey;
    private BdfimplAlbumEditor bdfimplAlbumEditor = new BdfimplAlbumEditor();
    private int availableIdsubset = 1;

    public BdfimplAlbum(SubsetKey albumKey, BdfimplBasedefiches basedefiches) {
        this.basedefiches = basedefiches;
        this.albumKey = albumKey;
        this.albumMetadata = new BdfimplAlbumMetadata(this);
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public AlbumMetadata getAlbumMetadata() {
        return albumMetadata;
    }

    public Illustration getIllustrationByIdalbum(int idalbum) {
        return (Illustration) getValueByKey(idalbum);
    }

    public SubsetKey getSubsetKey() {
        return albumKey;
    }

    public SubsetItem getSubsetItemById(int idsubset) {
        return (SubsetItem) getValueByKey(idsubset);
    }

    public int getTicketLength() {
        return length();
    }

    public int size() {
        return getEnabledIndexCount();
    }

    public SubsetItem getSubsetItemByTicket(int ticket) {
        return (SubsetItem) getValueByIndex(ticket);
    }

    public Subset getMasterSubset() {
        return null;
    }

    public Subset[] getSlaveSubsetArray() {
        return null;
    }

    AlbumEditor getAlbumEditor() {
        return bdfimplAlbumEditor;
    }

    void fireAlbumChange() {
        basedefiches.fireSubsetChange(albumKey);
    }

    void disable() {
        albumMetadata.disable();
    }


    private class BdfimplAlbumEditor implements AlbumEditor {

        public Album getAlbum() {
            return BdfimplAlbum.this;
        }

        public AlbumMetadataEditor getAlbumMetadataEditor() {
            return albumMetadata.getAlbumMetadataEditor();
        }

        public Illustration createIllustration(int idalbum, short type) throws ExistingIdException {
            if (idalbum < 0) {
                idalbum = availableIdsubset;
            } else {
                int current = BdfimplAlbum.this.getIndexByKey(idalbum);
                if (current != -1) {
                    throw new ExistingIdException();
                }
            }
            if (idalbum >= availableIdsubset) {
                availableIdsubset = idalbum + 1;
            }
            BdfimplIllustration implIllustration = new BdfimplIllustration(idalbum, type);
            implIllustration.ticket = BdfimplAlbum.this.putFromKey(idalbum, implIllustration);
            return implIllustration;
        }

        public void updateIllustration(Illustration illustration, InputStream inputStream, short type) throws IOException {
            BdfimplIllustration implIllustration = (BdfimplIllustration) illustration;
            implIllustration.type = type;
            basedefiches.getAlbumDataSource().update(illustration, inputStream);
        }

        public boolean removeIllustration(Illustration illustration) {
            BdfimplIllustration implIllustration = (BdfimplIllustration) illustration;
            if (!basedefiches.isRemoveable(implIllustration)) {
                return false;
            }
            int ticket = illustration.getTicket();
            int idalbum = illustration.getIdalbum();
            BdfimplAlbum.this.disable(ticket);
            return basedefiches.getAlbumDataSource().deleteIllustration(albumKey, idalbum);
        }

    }


    private class BdfimplIllustration implements Illustration {

        private int ticket;
        private int idalbum;
        private short type;

        private BdfimplIllustration(int idalbum, short type) {
            this.idalbum = idalbum;
            this.type = type;
        }

        public Subset getSubset() {
            return BdfimplAlbum.this;
        }

        public int getIdsubset() {
            return idalbum;
        }

        public int getIdalbum() {
            return idalbum;
        }

        public int getTicket() {
            return ticket;
        }

        public Album getAlbum() {
            return BdfimplAlbum.this;
        }

        public short getFormatType() {
            return type;
        }

        public InputStream getInputStream(short specialDim) {
            switch (specialDim) {
                case AlbumConstants.ORIGINAL_SPECIALDIM:
                    return basedefiches.getAlbumDataSource().getInputStream(this, null, null);
                case AlbumConstants.MINI_SPECIALDIM:
                    return basedefiches.getAlbumDataSource().getInputStream(this, AlbumConstants.MINI_DIMNAME, albumMetadata.getResizeInfo(specialDim));
                default:
                    throw new IllegalArgumentException("Unknown specialDim: " + specialDim);

            }
        }

        public InputStream getInputStream(AlbumDim albumDim) {
            if (!albumMetadata.isFromAlbum(albumDim)) {
                throw new IllegalArgumentException("albumDim is not linked to this Album");
            }
            return basedefiches.getAlbumDataSource().getInputStream(this, albumDim.getName(), albumDim.getResizeInfo());
        }

    }

}
