/* BasedefichesLib_Impl - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.ExistingNameException;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadata;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;
import net.mapeadores.util.awt.ResizeInfo;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
class BdfimplAlbumMetadata implements AlbumMetadata {

    private final static ResizeInfo MINI_RESIZEINFO = ResizeInfo.newMaxDimInstance(96, 64);
    private BdfimplAlbumDim miniAlbumDim = null;
    private BdfimplAlbum album;
    private BdfimplAlbumMetadataEditor albumMetadataEditor = new BdfimplAlbumMetadataEditor();
    private Map<String, BdfimplAlbumDim> albumDimMap = new HashMap<String, BdfimplAlbumDim>();
    private List<BdfimplAlbumDim> albumDimList = new ArrayList<BdfimplAlbumDim>();
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();
    private Map<String, BdfimplAlbumDim> albumDimUsageMap = new LinkedHashMap<String, BdfimplAlbumDim>();

    public BdfimplAlbumMetadata(BdfimplAlbum album) {
        this.album = album;
    }

    public Album getAlbum() {
        return album;
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    public int getAlbumDimCount() {
        return albumDimList.size();
    }

    public AlbumDim getAlbumDim(int i) {
        return albumDimList.get(i);
    }

    public AlbumDim getAlbumDimByName(String name) {
        return albumDimMap.get(name);
    }

    public AlbumDim getAlbumDimForUsage(String usageName) {
        return albumDimUsageMap.get(usageName);
    }

    public String[] getUsageNameArray() {
        int size = albumDimUsageMap.size();
        String[] result = new String[size];
        if (size > 0) {
            result = albumDimUsageMap.keySet().toArray(result);
        }
        return result;
    }

    public ResizeInfo getResizeInfo(short specialDim) {
        switch (specialDim) {
            case AlbumConstants.MINI_SPECIALDIM:
                if (miniAlbumDim != null) {
                    return miniAlbumDim.getResizeInfo();
                } else {
                    return MINI_RESIZEINFO;
                }
            default:
                return null;
        }
    }

    AlbumMetadataEditor getAlbumMetadataEditor() {
        return albumMetadataEditor;
    }

    boolean isFromAlbum(AlbumDim albumDim) {
        try {
            testAlbumDim(albumDim);
            return true;
        } catch (IllegalArgumentException iae) {
            return false;
        }
    }

    void disable() {
        albumDimMap.clear();
        albumDimList.clear();
        intituleMap.clear();
        intituleList.clear();
        albumDimUsageMap.clear();
    }

    private void fireChange() {
        album.fireAlbumChange();
    }

    private BdfimplAlbumDim testAlbumDim(AlbumDim albumDim) throws IllegalArgumentException {
        if (!(albumDim instanceof BdfimplAlbumDim)) {
            throw new IllegalArgumentException("albumDim argument does not come from this AlbumMetadata instance");
        }
        if (albumDim.getAlbumMetadata() != this) {
            throw new IllegalArgumentException("albumDim argument does not come from this AlbumMetadata instance");
        }
        return (BdfimplAlbumDim) albumDim;
    }


    private class BdfimplAlbumMetadataEditor implements AlbumMetadataEditor {

        public AlbumMetadata getAlbumMetadata() {
            return BdfimplAlbumMetadata.this;
        }

        public AlbumDim createAlbumDim(String name, short type) throws ExistingNameException, ParseException {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (albumDimMap.containsKey(name)) {
                throw new ExistingNameException();
            }
            AlbumUtils.testAlbumDimName(name);
            BdfimplAlbumDim bdfimplAlbumDim = new BdfimplAlbumDim(name, type);
            albumDimList.add(bdfimplAlbumDim);
            albumDimMap.put(name, bdfimplAlbumDim);
            fireChange();
            return bdfimplAlbumDim;
        }

        public boolean setDim(AlbumDim albumDim, int width, int height) {
            BdfimplAlbumDim bdfimplAlbumDim = testAlbumDim(albumDim);
            boolean done = bdfimplAlbumDim.setDim(width, height);
            if (done) {
                AlbumDataSource albumDataSource = ((BdfimplBasedefiches) album.getBasedefiches()).getAlbumDataSource();
                if (albumDataSource != null) {
                    albumDataSource.clearDimCache(album.getSubsetKey(), bdfimplAlbumDim.getName());
                }
                fireChange();
            }
            return done;
        }

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (intituleMap.containsKey(name)) {
                return null;
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplAlbumMetadata.this, album.getSubsetKey().toString());
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            fireChange();
            return intitule;
        }

        public Metadata getMetadata() {
            return BdfimplAlbumMetadata.this;
        }

        public boolean putLibelle(Intitule intitule, Libelle libelle) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplAlbumMetadata.this, intitule);
            boolean done = def.putLibelle(libelle);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean removeLibelle(Intitule intitule, int langInteger) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplAlbumMetadata.this, intitule);
            boolean done = def.removeLibelle(langInteger);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean setAlbumDimForUsage(String usageName, AlbumDim albumDim) {
            BdfimplAlbumDim current = albumDimUsageMap.get(usageName);
            if (albumDim == null) {
                if (current == null) {
                    return false;
                } else {
                    albumDimUsageMap.remove(usageName);
                    return true;
                }
            }
            BdfimplAlbumDim bdfimplAlbumDim = testAlbumDim(albumDim);
            if ((current != null) && (current.equals(bdfimplAlbumDim))) {
                return false;
            }
            albumDimUsageMap.put(usageName, bdfimplAlbumDim);
            return true;
        }

    }


    private class BdfimplAlbumDim implements AlbumDim {

        private String name;
        private short type;
        private ResizeInfo resizeInfo = MINI_RESIZEINFO;

        private BdfimplAlbumDim(String name, short type) {
            this.name = name;
            this.type = type;
            resizeInfo = toResizeInfo(type, 100, 100);
        }

        public AlbumMetadata getAlbumMetadata() {
            return BdfimplAlbumMetadata.this;
        }

        public String getName() {
            return name;
        }

        public short getDimType() {
            return type;
        }

        public ResizeInfo getResizeInfo() {
            return resizeInfo;
        }

        private boolean setDim(int width, int height) {
            try {
                ResizeInfo newResizeInfo = toResizeInfo(type, width, height);
                if (newResizeInfo.equals(resizeInfo)) {
                    return false;
                }
                this.resizeInfo = newResizeInfo;
                return true;
            } catch (IllegalArgumentException iae) {
                return false;
            }
        }

    }

    private static ResizeInfo toResizeInfo(short type, int width, int height) {
        switch (type) {
            case AlbumConstants.FIXEDWIDTH_DIMTYPE:
                return ResizeInfo.newFixedWidthInstance(width);
            case AlbumConstants.FIXEDHEIGHT_DIMTYPE:
                return ResizeInfo.newFixedHeightInstance(height);
            case AlbumConstants.MAXWIDTH_DIMTYPE:
                return ResizeInfo.newMaxWidthInstance(width);
            case AlbumConstants.MAXHEIGHT_DIMTYPE:
                return ResizeInfo.newMaxHeightInstance(height);
            case AlbumConstants.MAXDIM_DIMTYPE:
                return ResizeInfo.newMaxDimInstance(width, height);
            default:
                throw new SwitchException("unknownType = " + type);
        }
    }

}
