/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.ExistingSubsetException;
import net.mapeadores.basedefiches.NoRemoveableSubsetException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.AddendaEditor;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.CroisementEntry;
import net.mapeadores.basedefiches.croisement.CroisementList;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.NotEditableCroisementException;
import net.mapeadores.basedefiches.event.BasedefichesListener;
import net.mapeadores.basedefiches.impl.croisements.MasterAndSlaveCroisement;
import net.mapeadores.basedefiches.impl.croisements.MemoryCroisement;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataEditor;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurStats;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.ParentRecursivityException;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplBasedefiches implements Basedefiches {

    private Map<SubsetKey, Subset> subsetMap = new HashMap<SubsetKey, Subset>();
    private Map<SubsetCouple, Croisement> croisementMap = new HashMap<SubsetCouple, Croisement>();
    private Map<SubsetCouple, EditableCroisement> editableCroisementMap = new HashMap<SubsetCouple, EditableCroisement>();
    private BdfimplBasedefichesMetadata basedefichesMetadata;
    private List<BdfimplSphere> sphereList = new ArrayList<BdfimplSphere>();
    private List<BdfimplCorpus> corpusList = new ArrayList<BdfimplCorpus>();
    private List<BdfimplThesaurus> thesaurusList = new ArrayList<BdfimplThesaurus>();
    private List<BdfimplAlbum> albumList = new ArrayList<BdfimplAlbum>();
    private FicheDataSource ficheDataSource;
    private BdfImplBasedefichesEditor basedefichesEditor = new BdfImplBasedefichesEditor();
    private SubsetDataListener subsetDataListener;
    private BdfimplAddenda associationAddenda;
    private ListenerList listenerList = new ListenerList();
    private AlbumDataSource albumDataSource;

    public BdfimplBasedefiches() {
        basedefichesMetadata = new BdfimplBasedefichesMetadata(this);
        associationAddenda = new BdfimplAddenda(this);
        subsetMap.put(SubsetKey.ADDENDA, associationAddenda);
    }

    protected void setSubsetDataListener(SubsetDataListener subsetDataListener) {
        this.subsetDataListener = subsetDataListener;
    }

    protected void setDocumentDataSource(DocumentDataSource documentDataSource) {
        associationAddenda.setDocumentDataSource(documentDataSource);
    }

    protected void setAlbumDataSource(AlbumDataSource albumDataSource) {
        this.albumDataSource = albumDataSource;
    }

    protected void setFicheDataSource(FicheDataSource ficheDataSource) {
        this.ficheDataSource = ficheDataSource;
    }

    public void addBasedefichesListener(BasedefichesListener bl) {
        listenerList.addBasedefichesListener(bl);
    }

    public void removeBasedefichesListener(BasedefichesListener bl) {
        listenerList.removeBasedefichesListener(bl);
    }

    public BasedefichesMetadata getBasedefichesMetadata() {
        return basedefichesMetadata;
    }

    public int getCorpusCount() {
        return corpusList.size();
    }

    public Corpus getCorpus(int i) {
        return corpusList.get(i);
    }

    public int getThesaurusCount() {
        return thesaurusList.size();
    }

    public Thesaurus getThesaurus(int i) {
        return thesaurusList.get(i);
    }

    public int getSphereCount() {
        return sphereList.size();
    }

    public Sphere getSphere(int i) {
        return sphereList.get(i);
    }

    public int getAlbumCount() {
        return albumList.size();
    }

    public Album getAlbum(int i) {
        return albumList.get(i);
    }

    public int getAddendaCount() {
        return 1;
    }

    public Addenda getAddenda(int i) {
        if (i == 0) {
            return associationAddenda;
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    public Subset getSubset(SubsetKey subsetKey) {
        return subsetMap.get(subsetKey);
    }

    public Croisement getCroisement(SubsetKey subsetKey1, SubsetKey subsetKey2) {
        if ((subsetKey1.isSphereSubset()) || (subsetKey2.isSphereSubset())) {
            throw new IllegalArgumentException("Sphere !");
        }
        SubsetCouple subsetCouple = new SubsetCouple(subsetKey1, subsetKey2);
        Croisement croisement = croisementMap.get(subsetCouple);
        if (croisement == null) {
            return CroisementUtils.toEmptyCroisement(this, subsetCouple);
        } else {
            return croisement;
        }
    }

    public boolean isRemoveable(Subset subset) {
        if (subset.getSlaveSubsetArray() != null) {
            return false;
        }
        boolean empty = (subset.size() == 0);
        if (!empty) {
            return false;
        }
        if ((subset instanceof Corpus) || (subset instanceof Thesaurus) || (subset instanceof Sphere) || (subset instanceof Album)) {
            return true;
        } else {
            return false;
        }
    }

    public boolean isRemoveable(SubsetItem subsetItem) {
        if (subsetItem instanceof Redacteur) {
            RedacteurStats stats = new RedacteurStats((Redacteur) subsetItem);
            return stats.isEmpty();
        } else if (subsetItem instanceof Motcle) {
            return ThesaurusUtils.isRemoveable((Motcle) subsetItem);
        } else if (subsetItem instanceof Illustration) {
            return AlbumUtils.isRemoveable((Illustration) subsetItem);
        } else if (subsetItem instanceof AbregeFiche) {
            Subset[] array = subsetItem.getSubset().getSlaveSubsetArray();
            if (array != null) {
                int idsubset = subsetItem.getIdsubset();
                for (int i = 0; i < array.length; i++) {
                    if (array[i].getSubsetItemById(idsubset) != null) {
                        return false;
                    }
                }
            }
            return true;
        } else {
            return true;
        }
    }

    protected FicheDataSource getFicheDataSource() {
        return ficheDataSource;
    }

    protected BasedefichesEditor getBasedefichesEditor() {
        return basedefichesEditor;
    }

    protected void fireSubsetChange(SubsetKey subsetKey) {
        if (subsetDataListener != null) {
            subsetDataListener.subsetChanged(subsetKey);
        }
    }

    protected void fireChronoChange(SubsetKey subsetKey) {
        if (subsetDataListener != null) {
            subsetDataListener.chronoChanged(subsetKey);
        }
    }

    protected void fireSubsetCoupleChange(SubsetCouple subsetCouple) {
        if (subsetDataListener != null) {
            subsetDataListener.subsetCoupleChanged(subsetCouple);
        }
    }

    protected void fireBasedefichesMetadataChange() {
        if (subsetDataListener != null) {
            subsetDataListener.basedefichesMetadataChanged();
        }
    }

    protected ListenerList getListenerList() {
        return listenerList;
    }

    protected void updateFiche(AbregeFiche abregeFiche, Fiche fiche) {
        ((BdfimplCorpus) abregeFiche.getCorpus()).updateFiche(abregeFiche, fiche);
    }

    protected AlbumDataSource getAlbumDataSource() {
        return albumDataSource;
    }


    private class BdfImplBasedefichesEditor implements BasedefichesEditor {

        BdfimplCroisementsEditor bdfimplCroisementEditor = new BdfimplCroisementsEditor();

        public Basedefiches getBasedefiches() {
            return BdfimplBasedefiches.this;
        }

        public BasedefichesMetadataEditor getBasedefichesMetadataEditor() {
            return basedefichesMetadata.getBasedefichesMetadataEditor();
        }

        public CroisementEditor getCroisementEditor() {
            return bdfimplCroisementEditor;
        }

        public CorpusEditor createCorpus(SubsetKey corpusKey, Subset masterSubset) throws ExistingSubsetException {
            if (corpusKey == null) {
                throw new IllegalArgumentException("corpusKey == null");
            }
            if (!corpusKey.isCorpusSubset()) {
                throw new IllegalArgumentException("corpusKey.isCorpusKey() == false");
            }
            if (subsetMap.containsKey(corpusKey)) {
                throw new ExistingSubsetException();
            }
            if (masterSubset != null) {
                if (masterSubset instanceof BdfimplCorpus) {
                    if (masterSubset.getMasterSubset() != null) {
                        throw new IllegalArgumentException("masterSubset : " + masterSubset.getSubsetKey().toString() + " cannot be a slave subset");
                    }
                } else if (!(masterSubset instanceof BdfimplThesaurus)) {
                    throw new IllegalArgumentException("bad implementation of masterSubset : " + masterSubset.getClass().getName());
                }
            }
            BdfimplCorpus corpus = new BdfimplCorpus(corpusKey, BdfimplBasedefiches.this, masterSubset);
            SubsetKey masterSubsetKey = null;
            if (masterSubset != null) {
                masterSubsetKey = masterSubset.getSubsetKey();
                if (masterSubset instanceof BdfimplCorpus) {
                    BdfimplCorpus masterCorpus = (BdfimplCorpus) masterSubset;
                    masterCorpus.addSlaveCorpus(corpus);
                } else {
                    BdfimplThesaurus masterThesaurus = (BdfimplThesaurus) masterSubset;
                    masterThesaurus.addSlaveCorpus(corpus);
                }
                MasterAndSlaveCroisement croisement = new MasterAndSlaveCroisement(masterSubset, corpus);
                croisementMap.put(croisement.getSubsetCouple(), croisement);
            }
            subsetMap.put(corpusKey, corpus);
            addInCorpusList(corpus);
            fireSubsetChange(corpusKey);
            getListenerList().fireCorpusCreated(corpus);
            return corpus.getCorpusEditor();
        }

        public SphereEditor createSphere(SubsetKey sphereKey) throws ExistingSubsetException {
            if (sphereKey == null) {
                throw new IllegalArgumentException("sphereKey == null");
            }
            if (!sphereKey.isSphereSubset()) {
                throw new IllegalArgumentException("sphereKey.isSphereKey() == false");
            }
            if (subsetMap.containsKey(sphereKey)) {
                throw new ExistingSubsetException();
            }
            BdfimplSphere sphere = new BdfimplSphere(sphereKey, BdfimplBasedefiches.this);
            subsetMap.put(sphereKey, sphere);
            addInSphereList(sphere);
            fireSubsetChange(sphereKey);
            getListenerList().fireSphereCreated(sphere);
            return sphere.getSphereEditor();
        }

        public ThesaurusEditor createThesaurus(SubsetKey thesaurusKey, short thesaurusType) throws ExistingSubsetException {
            if (!ThesaurusUtils.testThesaurusType(thesaurusType)) {
                throw new IllegalArgumentException("wrong thesaurusType value");
            }
            if (thesaurusKey == null) {
                throw new IllegalArgumentException("thesaurusKey == null");
            }
            if (!thesaurusKey.isThesaurusSubset()) {
                throw new IllegalArgumentException("thesaurusKey.isThesaurusKey() == false");
            }
            if (subsetMap.containsKey(thesaurusKey)) {
                throw new ExistingSubsetException();
            }
            BdfimplThesaurus thesaurus = new BdfimplThesaurus(thesaurusKey, thesaurusType, BdfimplBasedefiches.this);
            subsetMap.put(thesaurusKey, thesaurus);
            addInThesaurusList(thesaurus);
            fireSubsetChange(thesaurusKey);
            getListenerList().fireThesaurusCreated(thesaurus);
            return thesaurus.getThesaurusEditor();
        }

        public AlbumEditor createAlbum(SubsetKey albumKey) throws ExistingSubsetException {
            if (albumKey == null) {
                throw new IllegalArgumentException("albumKey == null");
            }
            if (!albumKey.isAlbumSubset()) {
                throw new IllegalArgumentException("albumKey.isAlbumKey() == false");
            }
            if (subsetMap.containsKey(albumKey)) {
                throw new ExistingSubsetException();
            }
            BdfimplAlbum album = new BdfimplAlbum(albumKey, BdfimplBasedefiches.this);
            subsetMap.put(albumKey, album);
            addInAlbumList(album);
            fireSubsetChange(albumKey);
            getListenerList().fireAlbumCreated(album);
            return album.getAlbumEditor();
        }

        public CorpusEditor getCorpusEditor(SubsetKey corpusKey) {
            BdfimplCorpus corpus = (BdfimplCorpus) subsetMap.get(corpusKey);
            if (corpus == null) {
                return null;
            }
            return corpus.getCorpusEditor();
        }

        public AddendaEditor getAddendaEditor() {
            return associationAddenda.getAddendaEditor();
        }

        public SphereEditor getSphereEditor(SubsetKey sphereKey) {
            BdfimplSphere sphere = (BdfimplSphere) subsetMap.get(sphereKey);
            if (sphere == null) {
                return null;
            }
            return sphere.getSphereEditor();
        }

        public ThesaurusEditor getThesaurusEditor(SubsetKey thesaurusKey) {
            BdfimplThesaurus thesaurus = (BdfimplThesaurus) subsetMap.get(thesaurusKey);
            if (thesaurus == null) {
                return null;
            }
            return thesaurus.getThesaurusEditor();
        }

        public AlbumEditor getAlbumEditor(SubsetKey albumKey) {
            BdfimplAlbum album = (BdfimplAlbum) subsetMap.get(albumKey);
            if (album == null) {
                return null;
            }
            return album.getAlbumEditor();
        }

        public void merge(Motcle source, Motcle destination) throws net.mapeadores.basedefiches.thesaurus.ParentRecursivityException {
            if (source.getThesaurus().equals(destination.getThesaurus())) {
                ThesaurusUtils.transfereChildren(basedefichesEditor, source, destination);
            } else {
                try {
                    ThesaurusUtils.transfereChildren(basedefichesEditor, source, source.getParent());
                } catch (ParentRecursivityException pre) {
                    throw new ImplementationException("recursivity has been checked before in the script", pre);
                }
            }
            ThesaurusUtils.transfereIndexation(basedefichesEditor, source, destination);
            ThesaurusUtils.transfereLibelles(basedefichesEditor, source, destination);
            AbstractMotcle am = (AbstractMotcle) source;
            ((BdfimplThesaurus) am.getThesaurus()).getThesaurusEditor().removeMotcle(am);
        }

        public void removeSphere(Sphere sphere) throws NoRemoveableSubsetException {
            BdfimplSphere bdfimplSphere = testSphere(sphere);
            if (!isRemoveable(sphere)) {
                throw new NoRemoveableSubsetException();
            }
            SubsetKey sphereKey = bdfimplSphere.getSubsetKey();
            sphereList.remove(bdfimplSphere);
            subsetMap.remove(sphereKey);
            bdfimplSphere.disable();
            fireSubsetChange(sphereKey);
            getListenerList().fireSphereRemoved(sphereKey);
        }

        public void removeThesaurus(Thesaurus thesaurus) throws NoRemoveableSubsetException {
            BdfimplThesaurus bdfimplThesaurus = testThesaurus(thesaurus);
            if (!isRemoveable(thesaurus)) {
                throw new NoRemoveableSubsetException();
            }
            SubsetKey thesaurusKey = bdfimplThesaurus.getSubsetKey();
            thesaurusList.remove(bdfimplThesaurus);
            subsetMap.remove(thesaurusKey);
            bdfimplThesaurus.disable();
            for (int i = 0; i < corpusList.size(); i++) {
                BdfimplCorpus corpus = (BdfimplCorpus) corpusList.get(i);
                removeCroisement(corpus.getSubsetKey(), thesaurusKey, true);
            }
            fireSubsetChange(thesaurusKey);
            getListenerList().fireThesaurusRemoved(thesaurusKey);
        }

        public void removeAlbum(Album album) throws NoRemoveableSubsetException {
            BdfimplAlbum bdfimplAlbum = testAlbum(album);
            if (!isRemoveable(album)) {
                throw new NoRemoveableSubsetException();
            }
            SubsetKey albumKey = bdfimplAlbum.getSubsetKey();
            albumList.remove(bdfimplAlbum);
            subsetMap.remove(albumKey);
            bdfimplAlbum.disable();
            fireSubsetChange(albumKey);
            getListenerList().fireSphereRemoved(albumKey);
        }

        public void removeCorpus(Corpus corpus) throws NoRemoveableSubsetException {
            BdfimplCorpus bdfimplCorpus = testCorpus(corpus);
            if (!isRemoveable(corpus)) {
                throw new NoRemoveableSubsetException();
            }
            SubsetKey corpusKey = bdfimplCorpus.getSubsetKey();
            Subset masterSubset = bdfimplCorpus.getMasterSubset();
            if (masterSubset != null) {
                if (masterSubset instanceof BdfimplCorpus) {
                    ((BdfimplCorpus) masterSubset).removeSlaveCorpus(bdfimplCorpus);
                } else if (masterSubset instanceof BdfimplThesaurus) {
                    ((BdfimplThesaurus) masterSubset).removeSlaveCorpus(bdfimplCorpus);
                } else {
                    throw new SwitchException("masterSubset class = " + masterSubset.getClass().getName());
                }
            }
            corpusList.remove(bdfimplCorpus);
            subsetMap.remove(corpusKey);
            bdfimplCorpus.disable();
            for (int i = 0; i < corpusList.size(); i++) {
                BdfimplCorpus corpus2 = (BdfimplCorpus) corpusList.get(i);
                boolean fire = !testMasterSubset(corpus2, masterSubset);
                removeCroisement(corpus2.getSubsetKey(), corpusKey, fire);
            }
            for (int i = 0; i < thesaurusList.size(); i++) {
                BdfimplThesaurus thesaurus = (BdfimplThesaurus) thesaurusList.get(i);
                boolean fire = !testMasterSubset(thesaurus, masterSubset);
                removeCroisement(thesaurus.getSubsetKey(), corpusKey, fire);
            }
            removeCroisement(corpusKey, SubsetKey.ADDENDA, true);
            fireSubsetChange(corpusKey);
            fireChronoChange(corpusKey);
            getListenerList().fireCorpusRemoved(corpusKey, masterSubset);
        }

        public boolean putAttribute(AttributeHolder atlasAttributeHolder, Attribute attribute) {
            AttributeKey attributeKey = attribute.getAttributeKey();
            Attribute oldAttribute = atlasAttributeHolder.getAttributeByKey(attributeKey);
            if (atlasAttributeHolder instanceof BdfimplBasedefichesMetadata) {
                BdfimplBasedefichesMetadata metadata = (BdfimplBasedefichesMetadata) atlasAttributeHolder;
                boolean done = metadata.putAttribute(attribute);
                if (done) {
                    fireBasedefichesMetadataChange();
                    return true;
                }
            }
            return false;
        }

        public boolean removeAttribute(AttributeHolder atlasAttributeHolder, AttributeKey attributeKey) {
            Attribute oldAttribute = atlasAttributeHolder.getAttributeByKey(attributeKey);
            if (oldAttribute == null) {
                return false;
            }
            if (atlasAttributeHolder instanceof BdfimplBasedefichesMetadata) {
                BdfimplBasedefichesMetadata metadata = (BdfimplBasedefichesMetadata) atlasAttributeHolder;
                boolean done = metadata.removeAttribute(attributeKey);
                if (done) {
                    fireBasedefichesMetadataChange();
                    return true;
                }
            }
            return false;
        }

    }

    private boolean testMasterSubset(Subset subset, Subset masterSubset) {
        if (masterSubset == null) {
            return false;
        }
        return subset.equals(masterSubset);
    }

    private boolean testRattachementSubset(Corpus corpus1, Corpus corpus2) {
        return (testMasterSubset(corpus1, corpus2.getMasterSubset())) || (testMasterSubset(corpus2, corpus1.getMasterSubset()));
    }

    private void removeCroisement(SubsetKey key1, SubsetKey key2, boolean fire) {
        SubsetCouple subsetCouple = new SubsetCouple(key1, key2);
        croisementMap.remove(subsetCouple);
        editableCroisementMap.remove(subsetCouple);
        if (fire) {
            fireSubsetCoupleChange(subsetCouple);
        }
    }

    private void addInSphereList(BdfimplSphere sphere) {
        int size = sphereList.size();
        if (size == 0) {
            sphereList.add(sphere);
            return;
        }
        boolean done = false;
        for (int i = (size - 1); i >= 0; i--) {
            BdfimplSphere other = sphereList.get(i);
            if (other.getSubsetKey().compareTo(sphere.getSubsetKey()) < 0) {
                sphereList.add(i + 1, sphere);
                done = true;
                break;
            }
        }
        if (!done) {
            sphereList.add(0, sphere);
        }
    }

    private void addInAlbumList(BdfimplAlbum album) {
        int size = albumList.size();
        if (size == 0) {
            albumList.add(album);
            return;
        }
        boolean done = false;
        for (int i = (size - 1); i >= 0; i--) {
            BdfimplAlbum other = albumList.get(i);
            if (other.getSubsetKey().compareTo(album.getSubsetKey()) < 0) {
                albumList.add(i + 1, album);
                done = true;
                break;
            }
        }
        if (!done) {
            albumList.add(0, album);
        }
    }

    private void addInThesaurusList(BdfimplThesaurus thesaurus) {
        int size = thesaurusList.size();
        if (size == 0) {
            thesaurusList.add(thesaurus);
            return;
        }
        boolean done = false;
        for (int i = (size - 1); i >= 0; i--) {
            BdfimplThesaurus other = thesaurusList.get(i);
            if (other.getSubsetKey().compareTo(thesaurus.getSubsetKey()) < 0) {
                thesaurusList.add(i + 1, thesaurus);
                done = true;
                break;
            }
        }
        if (!done) {
            thesaurusList.add(0, thesaurus);
        }
    }

    private void addInCorpusList(BdfimplCorpus corpus) {
        int size = corpusList.size();
        if (size == 0) {
            corpusList.add(corpus);
            return;
        }
        boolean done = false;
        for (int i = (size - 1); i >= 0; i--) {
            BdfimplCorpus other = corpusList.get(i);
            if (other.getSubsetKey().compareTo(corpus.getSubsetKey()) < 0) {
                corpusList.add(i + 1, corpus);
                done = true;
                break;
            }
        }
        if (!done) {
            corpusList.add(0, corpus);
        }
    }


    class BdfimplCroisementsEditor implements CroisementEditor {

        public Basedefiches getBasedefiches() {
            return BdfimplBasedefiches.this;
        }

        public void initCroisements(SubsetCouple subsetCouple, List<CroisementEntry> croisementEntryList) {
            EditableCroisement editableCroisement = getEditableCroisement(subsetCouple);
            editableCroisement.initCroisements(croisementEntryList);
            fireSubsetCoupleChange(editableCroisement.getSubsetCouple());
        }

        public void updateCroisement(SubsetItem subsetItem, CroisementList croisementList, short updateType) {
            SubsetKey subsetKey1 = subsetItem.getSubset().getSubsetKey();
            SubsetKey subsetKey2 = croisementList.getSubsetKey();
            EditableCroisement editableCroisement = getEditableCroisement(subsetKey1, subsetKey2);
            editableCroisement.addCroisement(subsetKey1, subsetItem.getTicket(), croisementList, updateType);
            fireSubsetCoupleChange(editableCroisement.getSubsetCouple());
        }

        public boolean removeCroisement(SubsetItem subsetItem1, SubsetItem subsetItem2) {
            SubsetKey subsetKey1 = subsetItem1.getSubset().getSubsetKey();
            SubsetKey subsetKey2 = subsetItem2.getSubset().getSubsetKey();
            EditableCroisement editableCroisement = getEditableCroisement(subsetKey1, subsetKey2);
            boolean done = editableCroisement.removeCroisement(subsetKey1, subsetItem1.getTicket(), subsetKey2, subsetItem2.getTicket());
            if (done) {
                fireSubsetCoupleChange(editableCroisement.getSubsetCouple());
            }
            return done;
        }

        private EditableCroisement getEditableCroisement(SubsetKey subsetKey1, SubsetKey subsetKey2) {
            return getEditableCroisement(new SubsetCouple(subsetKey1, subsetKey2));
        }

        private EditableCroisement getEditableCroisement(SubsetCouple subsetCouple) {
            EditableCroisement editableCroisement = editableCroisementMap.get(subsetCouple);
            if (editableCroisement != null) {
                return editableCroisement;
            }
            if (croisementMap.containsKey(subsetCouple)) {
                throw new NotEditableCroisementException(subsetCouple);
            }
            editableCroisement = new MemoryCroisement(BdfimplBasedefiches.this, subsetCouple);
            croisementMap.put(subsetCouple, editableCroisement);
            editableCroisementMap.put(subsetCouple, editableCroisement);
            return editableCroisement;
        }

    }

    /**
     * Effectue les préparatifs nécessaires avant la suppression d'une fiche.
     * Supprime tous les croisements d'une fiche.
     */
    protected void prepareToRemove(AbregeFiche abregeFiche) {
        Corpus crps = abregeFiche.getCorpus();
        SubsetKey corpusKey = crps.getSubsetKey();
        Subset masterSubset = crps.getMasterSubset();
        int ticket = abregeFiche.getTicket();
        for (int i = 0; i < thesaurusList.size(); i++) {
            Thesaurus thesaurus = (Thesaurus) thesaurusList.get(i);
            if (!testMasterSubset(thesaurus, masterSubset)) {
                int result = clearCroisement(corpusKey, ticket, thesaurus.getSubsetKey());
                if (result > 0) {
                    fireSubsetCoupleChange(new SubsetCouple(corpusKey, thesaurus.getSubsetKey()));
                }
            }
        }
        for (int i = 0; i < corpusList.size(); i++) {
            Corpus crp2 = (Corpus) corpusList.get(i);
            if (!testRattachementSubset(crps, crp2)) {
                int result = clearCroisement(corpusKey, ticket, crp2.getSubsetKey());
                if (result > 0) {
                    fireSubsetCoupleChange(new SubsetCouple(corpusKey, crp2.getSubsetKey()));
                }
            }
        }
        int resultDocument = clearCroisement(corpusKey, ticket, SubsetKey.ADDENDA);
        if (resultDocument > 0) {
            fireSubsetCoupleChange(new SubsetCouple(corpusKey, SubsetKey.ADDENDA));
        }
    }

    private int clearCroisement(SubsetKey corpusKey, int ticket, SubsetKey subsetKey) {
        EditableCroisement croisement = editableCroisementMap.get(new SubsetCouple(corpusKey, subsetKey));
        if (croisement == null) {
            return 0;
        }
        return croisement.clear(corpusKey, ticket);
    }

    private BdfimplSphere testSphere(Sphere sphere) {
        if (sphere == null) {
            throw new IllegalArgumentException("sphere argument cannot be null");
        }
        try {
            BdfimplSphere defsphere = (BdfimplSphere) sphere;
            if (defsphere.getBasedefiches() != this) {
                throw new IllegalArgumentException("sphere argument does not come from this Basedefiches instance");
            }
            return defsphere;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("sphere argument does not come from this Basedefiches instance");
        }
    }

    private BdfimplThesaurus testThesaurus(Thesaurus thesaurus) {
        if (thesaurus == null) {
            throw new IllegalArgumentException("thesaurus argument cannot be null");
        }
        try {
            BdfimplThesaurus defthesaurus = (BdfimplThesaurus) thesaurus;
            if (defthesaurus.getBasedefiches() != this) {
                throw new IllegalArgumentException("thesaurus argument does not come from this Basedefiches instance");
            }
            return defthesaurus;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("thesaurus argument does not come from this Basedefiches instance");
        }
    }

    private BdfimplCorpus testCorpus(Corpus corpus) {
        if (corpus == null) {
            throw new IllegalArgumentException("corpus argument cannot be null");
        }
        try {
            BdfimplCorpus defcorpus = (BdfimplCorpus) corpus;
            if (defcorpus.getBasedefiches() != this) {
                throw new IllegalArgumentException("corpus argument does not come from this Basedefiches instance");
            }
            return defcorpus;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("corpus argument does not come from this Basedefiches instance");
        }
    }

    private BdfimplAlbum testAlbum(Album album) {
        if (album == null) {
            throw new IllegalArgumentException("album argument cannot be null");
        }
        try {
            BdfimplAlbum defalbum = (BdfimplAlbum) album;
            if (defalbum.getBasedefiches() != this) {
                throw new IllegalArgumentException("album argument does not come from this Basedefiches instance");
            }
            return defalbum;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("album argument does not come from this Basedefiches instance");
        }
    }

}
