/* BasedefichesLib_Impl - Copyright (c) 2068-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataEditor;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
class BdfimplBasedefichesMetadata implements BasedefichesMetadata {

    private BdfimplBasedefiches basedefiches;
    private String authorityUUID = "e17a05b0-c45e-11d8-9669-0800200c9a66";
    private String basedefichesName = "basedefiches";
    private BdfimplBasedefichesMetadataEditor basedefichesMetadataEditor = new BdfimplBasedefichesMetadataEditor();
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();
    private Map<AttributeKey, Attribute> attributeMap;
    private List<Attribute> attributeList;

    public BdfimplBasedefichesMetadata(BdfimplBasedefiches basedefiches) {
        this.basedefiches = basedefiches;
    }

    public String getAuthorityUUID() {
        return authorityUUID;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public String getBasedefichesName() {
        return basedefichesName;
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    public Attribute getAttributeByKey(AttributeKey key) {
        if (attributeMap == null) {
            return null;
        }
        return attributeMap.get(key);
    }

    public int getAttributeCount() {
        if (attributeList == null) {
            return 0;
        }
        return attributeList.size();
    }

    public Attribute getAttributeByIndex(int i) {
        if (attributeList == null) {
            throw new IndexOutOfBoundsException();
        }
        return attributeList.get(i);
    }

    protected boolean removeAttribute(AttributeKey attributeKey) {
        if (attributeMap == null) {
            return false;
        }
        Attribute attribute = attributeMap.remove(attributeKey);
        if (attribute != null) {
            attributeList.remove(attribute);
            return true;
        } else {
            return false;
        }
    }

    protected boolean putAttribute(Attribute attribute) {
        AttributeKey attributeKey = attribute.getAttributeKey();
        if (attributeMap == null) {
            attributeMap = new HashMap<AttributeKey, Attribute>();
            attributeMap.put(attributeKey, attribute);
            attributeList = new ArrayList<Attribute>();
            attributeList.add(attribute);
            return true;
        }
        Attribute current = attributeMap.get(attributeKey);
        if (current != null) {
            if (AttributeUtils.isEqual(current, attribute)) {
                return false;
            }
            int idx = attributeList.indexOf(current);
            attributeList.set(idx, attribute);
        } else {
            attributeList.add(attribute);
        }
        attributeMap.put(attributeKey, attribute);
        return true;
    }

    BasedefichesMetadataEditor getBasedefichesMetadataEditor() {
        return basedefichesMetadataEditor;
    }


    private class BdfimplBasedefichesMetadataEditor implements BasedefichesMetadataEditor {

        public Metadata getMetadata() {
            return BdfimplBasedefichesMetadata.this;
        }

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (intituleMap.containsKey(name)) {
                return null;
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplBasedefichesMetadata.this, basedefichesName);
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            basedefiches.fireBasedefichesMetadataChange();
            return intitule;
        }

        public boolean setAuthorityUUID(String s) {
            if (!StringUtils.checkUUID(s)) {
                throw new IllegalArgumentException("not a UUID : " + s);
            }
            boolean done = false;
            if (!s.equals(authorityUUID)) {
                authorityUUID = s;
                done = true;
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

        public boolean setBasedefichesName(String s) {
            if (!StringUtils.isTechnicalName(s, true)) {
                throw new IllegalArgumentException("not a technical name : " + s);
            }
            boolean done = false;
            if (!s.equals(basedefichesName)) {
                basedefichesName = s;
                done = true;
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

        public boolean putLibelle(Intitule intitule, Libelle libelle) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplBasedefichesMetadata.this, intitule);
            boolean done = def.putLibelle(libelle);
            if (done) {
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

        public boolean removeLibelle(Intitule intitule, int langInteger) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplBasedefichesMetadata.this, intitule);
            boolean done = def.removeLibelle(langInteger);
            if (done) {
                basedefiches.fireBasedefichesMetadataChange();
            }
            return done;
        }

    }

}
