/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.NoMasterIdsubsetException;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatDefEngine;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.collections.AbstractIndexAndIntKeyMap;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.StringUtils;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatter;
import net.mapeadores.basedefiches.pointeurs.PointeurUtils;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class BdfimplCorpus extends AbstractIndexAndIntKeyMap implements Corpus {

    BdfimplBasedefiches basedefiches;
    private SubsetKey corpusKey;
    BdfimplCorpusMetadata corpusMetadata;
    BdfimplCorpusEditor corpusEditor = new BdfimplCorpusEditor();
    private int availableIdcorpus = 1;
    private BdfimplCorpus[] slaveSubsetArray = null;
    private Subset masterSubset;

    public BdfimplCorpus(SubsetKey corpusKey, BdfimplBasedefiches basedefiches, Subset masterSubset) {
        this.basedefiches = basedefiches;
        this.corpusKey = corpusKey;
        this.corpusMetadata = new BdfimplCorpusMetadata(this);
        this.masterSubset = masterSubset;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public CorpusMetadata getCorpusMetadata() {
        return corpusMetadata;
    }

    public AbregeFiche getAbregeFicheByIdcorpus(int idcorpus) {
        return (AbregeFiche) getValueByKey(idcorpus);
    }

    public Fiche getEnteteFiche(AbregeFiche abregeFiche) {
        return basedefiches.getFicheDataSource().getFiche(abregeFiche, false);
    }

    public Fiche getFiche(AbregeFiche abregeFiche) {
        return basedefiches.getFicheDataSource().getFiche(abregeFiche, true);
    }

    public SubsetKey getSubsetKey() {
        return corpusKey;
    }

    public SubsetItem getSubsetItemById(int idsubset) {
        return (SubsetItem) getValueByKey(idsubset);
    }

    public int getTicketLength() {
        return length();
    }

    public int size() {
        return getEnabledIndexCount();
    }

    public SubsetItem getSubsetItemByTicket(int ticket) {
        return (SubsetItem) getValueByIndex(ticket);
    }

    public Subset getMasterSubset() {
        return masterSubset;
    }

    public Subset[] getSlaveSubsetArray() {
        return BdfimplUtils.copy(slaveSubsetArray);
    }

    protected CorpusEditor getCorpusEditor() {
        return corpusEditor;
    }

    protected void disable() {
        corpusMetadata.disable();
        masterSubset = null;
    }

    protected ListenerList getListenerList() {
        return basedefiches.getListenerList();
    }

    protected void updateFiche(AbregeFiche abregeFiche, Fiche fiche) {
        corpusEditor.updateFiche(abregeFiche, fiche);
    }

    protected void addSlaveCorpus(BdfimplCorpus bdfimplCorpus) {
        slaveSubsetArray = BdfimplUtils.addSlaveCorpus(slaveSubsetArray, bdfimplCorpus);
    }

    protected void removeSlaveCorpus(BdfimplCorpus bdfimplCorpus) {
        slaveSubsetArray = BdfimplUtils.removeSlaveCorpus(slaveSubsetArray, bdfimplCorpus);
    }


    private class BdfimplCorpusEditor implements CorpusEditor {

        public AbregeFiche createFiche(int idcorpus) throws ExistingIdException, NoMasterIdsubsetException {
            if (masterSubset != null) {
                SubsetItem subsetItem = masterSubset.getSubsetItemById(idcorpus);
                if (subsetItem == null) {
                    throw new NoMasterIdsubsetException();
                }
                if (BdfimplCorpus.this.getIndexByKey(idcorpus) != -1) {
                    throw new ExistingIdException();
                }
            } else {
                if (idcorpus < 0) {
                    idcorpus = availableIdcorpus;
                } else {
                    int current = BdfimplCorpus.this.getIndexByKey(idcorpus);
                    if (current != -1) {
                        throw new ExistingIdException();
                    }
                }
            }
            BdfimplAbregeFiche abregeFiche = new BdfimplAbregeFiche(idcorpus);
            abregeFiche.ticket = BdfimplCorpus.this.putFromKey(idcorpus, abregeFiche);
            if (idcorpus >= availableIdcorpus) {
                availableIdcorpus = idcorpus + 1;
            }
            getListenerList().fireFicheCreated(abregeFiche);
            FicheDataSource ficheDataSource = basedefiches.getFicheDataSource();
            if (ficheDataSource != null) {
                ficheDataSource.saveFiche(abregeFiche, new Fiche());
            }
            return abregeFiche;
        }

        public Corpus getCorpus() {
            return BdfimplCorpus.this;
        }

        public CorpusMetadataEditor getCorpusMetadataEditor() {
            return corpusMetadata.getCorpusMetadataEditor();
        }

        public boolean removeFiche(AbregeFiche abregeFiche) {
            BdfimplAbregeFiche bdfimplAbregeFiche = testAbregeFiche(abregeFiche);
            if (!basedefiches.isRemoveable(bdfimplAbregeFiche)) {
                return false;
            }
            basedefiches.prepareToRemove(abregeFiche);
            BdfimplCorpus.this.disable(abregeFiche.getTicket());
            basedefiches.getFicheDataSource().removeFiche(abregeFiche);
            basedefiches.fireChronoChange(corpusKey);
            return true;
        }

        public void saveFiche(AbregeFiche abregeFiche, Fiche fiche) {
            titreGeneration(abregeFiche, fiche);
            updateFiche(abregeFiche, fiche);
            basedefiches.getFicheDataSource().saveFiche(abregeFiche, fiche);
            getListenerList().fireFicheSaved(abregeFiche, fiche);
        }

        private void updateFiche(AbregeFiche abregeFiche, Fiche fiche) {
            BdfimplAbregeFiche bdfimplAbregeFiche = testAbregeFiche(abregeFiche);
            bdfimplAbregeFiche.update(fiche);
        }

        public void setDate(AbregeFiche abregeFiche, int dateInteger, boolean modification) {
            BdfimplAbregeFiche bdfimplAbregeFiche = testAbregeFiche(abregeFiche);
            bdfimplAbregeFiche.setDate(dateInteger, modification);
        }

        private void titreGeneration(AbregeFiche abregeFiche, Fiche fiche) {
            BasedefichesFormatDef titreFormatDef = corpusMetadata.getTitreFormatDef();
            if (titreFormatDef == null) {
                return;
            }
            FormatDefErrorHandler errorHandler = new FormatDefErrorHandler();
            BasedefichesFormatter formatter = BasedefichesFormatDefEngine.compute(BdfimplCorpus.this, titreFormatDef, errorHandler);
            if (formatter == null) {
                fiche.setTitre("#" + errorHandler.errorKey + "#");
            } else {
                LocalisationInfo localisationInfo = new LocalisationInfo(fiche.getLangInteger());
                FichePointeur fichePointeur = PointeurUtils.toFichePointeur(abregeFiche, fiche, false);
                String result = formatter.formatToString(fichePointeur, null, localisationInfo);
                if (result == null) {
                    fiche.setTitre("");
                } else {
                    fiche.setTitre(StringUtils.cleanString(result));
                }
            }

        }

    }


    private static class FormatDefErrorHandler implements BasedefichesFormatDefEngine.ErrorHandler {

        String errorKey = "error";

        public void unknownSubfieldKey(SubfieldKey subfieldKey) {
            errorKey = "unknownSubfieldKey" + " : " + subfieldKey.toString();
        }

        public void illegalSubfieldKey(SubfieldKey subfieldKey) {
            errorKey = "illegalSubfieldKey" + " : " + subfieldKey.toString();
        }

        public void wrongFormatPattern(String formatPattern) {
            errorKey = "wrongFormatPattern" + " : " + formatPattern;
        }

        public void unknownFieldKey(FieldKey fieldKey) {
            errorKey = "unknownFieldKey" + " : " + fieldKey.toString();
        }

        public void unknownSubsetKey(SubsetKey subsetKey) {
            errorKey = "unknownSubsetKey" + " : " + subsetKey.toString();
        }

        public void notASlaveCorpusKey(SubsetKey subsetKey) {
            errorKey = "notASlaveCorpusKey" + " : " + subsetKey.toString();
        }

        public void illegalFormatSourceKey(String keyString) {
            errorKey = "illegalFormatSourceKey" + " : " + keyString;
        }

        public void noTypePrefixFieldKey(FieldKey fieldKey) {
            errorKey = "noTypePrefixFieldKey" + " : " + fieldKey.toString();
        }

    }


    private class BdfimplAbregeFiche implements AbregeFiche {

        private int ticket;
        private int idcorpus;
        private int langInteger = LangInteger.UNDETERMINED;
        private String titre = "";
        private int creationDate = 0;
        private int modificationDate = 0;
        private boolean removeable = false;
        private RedacteurKey[] redacteurKeyArray;

        BdfimplAbregeFiche(int idcorpus) {
            this.idcorpus = idcorpus;
        }

        public Corpus getCorpus() {
            return BdfimplCorpus.this;
        }

        public Subset getSubset() {
            return BdfimplCorpus.this;
        }

        public int getTicket() {
            return ticket;
        }

        public String getTitre() {
            return titre;
        }

        public int getIdcorpus() {
            return idcorpus;
        }

        public int getIdsubset() {
            return idcorpus;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public int getModificationDate() {
            return modificationDate;
        }

        public int getCreationDate() {
            return creationDate;
        }

        public boolean isRemoveable() {
            return removeable;
        }

        public boolean hasRedacteur(RedacteurKey redacteurKey) {
            if (redacteurKeyArray == null) {
                return false;
            }
            int length = redacteurKeyArray.length;
            for (int i = 0; i < length; i++) {
                if (redacteurKeyArray[i].equals(redacteurKey)) {
                    return true;
                }
            }
            return false;
        }

        public boolean hasRedacteurInSphere(SubsetKey sphereKey) {
            if (redacteurKeyArray == null) {
                return false;
            }
            int length = redacteurKeyArray.length;
            for (int i = 0; i < length; i++) {
                RedacteurKey redacteurKey = redacteurKeyArray[i];
                if (redacteurKey.getSphereKey().equals(sphereKey)) {
                    return true;
                }
            }
            return false;
        }

        void update(Fiche fiche) {
            this.langInteger = fiche.getLangInteger();
            this.titre = fiche.getTitre();
            FicheItemList redacList = fiche.getRedacteurs();
            if (redacList == null) {
                this.removeable = true;
                this.redacteurKeyArray = null;
            } else {
                int size = redacList.getFicheItemCount();
                removeable = false;
                List<RedacteurKey> tempList = new ArrayList<RedacteurKey>();
                for (int i = 0; i < size; i++) {
                    FicheItem ficheItem = redacList.getFicheItem(i);
                    if (ficheItem instanceof Personne) {
                        String redacteurKeyString = ((Personne) ficheItem).getRedacteurKeyString();
                        if (redacteurKeyString != null) {
                            try {
                                RedacteurKey redacteurKey = RedacteurKey.parse(redacteurKeyString);
                                tempList.add(redacteurKey);
                            } catch (java.text.ParseException pe) {
                            }
                        }
                    }
                }
                int size2 = tempList.size();
                if (size2 == 0) {
                    this.redacteurKeyArray = null;
                } else {
                    this.redacteurKeyArray = new RedacteurKey[size2];
                    for (int i = 0; i < size2; i++) {
                        this.redacteurKeyArray[i] = (RedacteurKey) tempList.get(i);
                    }
                }
            }
        }

        void setDate(int dateInteger, boolean modification) {
            int mask = 0;
            if (!modification) {
                if (dateInteger != creationDate) {
                    mask = 1;
                    creationDate = dateInteger;
                    if (modificationDate < dateInteger) {
                        modificationDate = dateInteger;
                        mask = 3;
                    }
                }
            } else {
                if (creationDate == 0) {
                    creationDate = dateInteger;
                    modificationDate = dateInteger;
                    mask = 3;
                } else if ((dateInteger != modificationDate) && (dateInteger >= creationDate)) {
                    modificationDate = dateInteger;
                    mask = 2;
                }
            }
            if (mask > 0) {
                basedefiches.fireChronoChange(getCorpus().getSubsetKey());
            }
        }

    }

    private BdfimplAbregeFiche testAbregeFiche(AbregeFiche abregeFiche) {
        if (abregeFiche == null) {
            throw new IllegalArgumentException("abregeFiche argument cannot be null");
        }
        try {
            BdfimplAbregeFiche bdfimpl = (BdfimplAbregeFiche) abregeFiche;
            if (bdfimpl.getCorpus() != this) {
                throw new IllegalArgumentException("abregeFicher argument does not come from this corpus");
            }
            return bdfimpl;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("abregeFiche argument does not come from this corpus");
        }
    }

}
