/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusFieldList;
import net.mapeadores.basedefiches.corpus.metadata.CorpusFieldListUtils;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.ExistingFieldKeyException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionException;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptions;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsConstants;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.basedefiches.corpus.metadata.MandatoryFieldException;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.util.primitives.IntegerList;
import net.mapeadores.util.primitives.IntegerListUtils;
import net.mapeadores.util.text.DefaultLibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
class BdfimplCorpusMetadata implements CorpusMetadata {

    private BdfimplCorpus corpus;
    private BdfimplCorpusMetadataEditor corpusMetadataEditor;
    private CorpusField geolocalisationField;
    private Map<FieldKey, CorpusField> corpusFieldMap = new HashMap<FieldKey, CorpusField>();
    private List<BdfimplCorpusField> proprieteFieldList = new ArrayList<BdfimplCorpusField>();
    private List<BdfimplCorpusField> informationFieldList = new ArrayList<BdfimplCorpusField>();
    private List<BdfimplCorpusField> sectionFieldList = new ArrayList<BdfimplCorpusField>();
    private BasedefichesFormatDef titreFormatDef;
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();

    public BdfimplCorpusMetadata(BdfimplCorpus corpus) {
        this.corpus = corpus;
        corpusMetadataEditor = new BdfimplCorpusMetadataEditor();
        initField(FieldKey.IDCORPUS);
        initField(FieldKey.LANG);
        initField(FieldKey.TITRE);
        initField(FieldKey.REDACTEURS);
    }

    private void initField(FieldKey fieldKey) {
        corpusMetadataEditor.innerCreateCorpusField(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public Corpus getCorpus() {
        return corpus;
    }

    public CorpusField getCorpusFieldByFieldKey(FieldKey fieldKey) {
        return corpusFieldMap.get(fieldKey);
    }

    public int getProprieteFieldCount() {
        return proprieteFieldList.size();
    }

    public CorpusField getProprieteField(int i) {
        return proprieteFieldList.get(i);
    }

    public int getInformationFieldCount() {
        return informationFieldList.size();
    }

    public CorpusField getInformationField(int i) {
        return informationFieldList.get(i);
    }

    public int getSectionFieldCount() {
        return sectionFieldList.size();
    }

    public CorpusField getSectionField(int i) {
        return sectionFieldList.get(i);
    }

    public CorpusField getGeolocalisationField() {
        return geolocalisationField;
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    CorpusMetadataEditor getCorpusMetadataEditor() {
        return corpusMetadataEditor;
    }

    void disable() {
        sectionFieldList.clear();
        proprieteFieldList.clear();
        informationFieldList.clear();
        corpusFieldMap.clear();
        intituleMap.clear();
        intituleList.clear();
    }

    ListenerList getListenerList() {
        return corpus.getListenerList();
    }

    BasedefichesFormatDef getTitreFormatDef() {
        return titreFormatDef;
    }

    private void fireChange() {
        corpus.basedefiches.fireSubsetChange(corpus.getSubsetKey());
    }

    private BdfimplCorpusField testCorpusField(CorpusField corpusField) throws IllegalArgumentException {
        if (!(corpusField instanceof BdfimplCorpusField)) {
            throw new IllegalArgumentException("corpusField argument does not come from this CorpusMetadata instance");
        }
        BdfimplCorpusField def = (BdfimplCorpusField) corpusField;
        if (def.getCorpusMetadata() != this) {
            throw new IllegalArgumentException("corpusField argument does not come from this CorpusMetadata instance");
        }
        return def;
    }


    private class BdfimplCorpusField extends DefaultLibelleHolder implements CorpusField {

        private FieldKey fieldKey;
        private short ficheItemType = CorpusField.NO_FICHEITEM_FIELD;
        private Map<String, String> stringOptions = new HashMap<String, String>();
        private InternalFieldOptions fieldOptions = new InternalFieldOptions();

        private BdfimplCorpusField(FieldKey fieldKey) {
            this(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
        }

        private BdfimplCorpusField(FieldKey fieldKey, short ficheItemType) {
            super(fieldKey.toString());
            this.fieldKey = fieldKey;
            this.ficheItemType = ficheItemType;
            if (fieldKey.isInformationFieldKey()) {
                fieldOptions.informationDisplay = FieldOptionsConstants.INLINE_DISPLAY;
            }
        }

        public FieldKey getFieldKey() {
            return fieldKey;
        }

        public FieldOptions getFieldOptions() {
            return fieldOptions;
        }

        public CorpusMetadata getCorpusMetadata() {
            return BdfimplCorpusMetadata.this;
        }

        

        public short getFicheItemType() {
            return ficheItemType;
        }

        public String getStringOption(String optionName) {
            return stringOptions.get(optionName);
        }

        public String[] getStringOptionNameArray() {
            int size = stringOptions.size();
            String[] result = new String[size];
            if (size == 0) {
                return result;
            }
            int p = 0;
            for (String key : stringOptions.keySet()) {
                result[p] = key;
                p++;
            }
            return result;
        }

        private boolean putLibelle(Libelle libelle) {
            return innerPutLibelle(libelle);
        }

        private boolean removeLibelle(int langInteger) {
            return innerRemoveLibelle(langInteger);
        }

        private boolean setOption(String optionName, Object optionValue) throws FieldOptionException {
            FieldOptionsUtils.checkFieldOption(optionName, fieldKey, ficheItemType);
            if (optionName.equals(FieldOptionsConstants.INFORMATIONDISPLAY_OPTION)) {
                return fieldOptions.setInformationDisplay(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION)) {
                return fieldOptions.setDefaultSphereKey(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.SUBFIELDDISPLAY_OPTION)) {
                return fieldOptions.setSubfieldDisplay(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.TITREGENERATIONFORMAT_OPTION)) {
                boolean changeResult = fieldOptions.setGenerationFormatDef(optionValue);
                if ((changeResult) && (fieldKey.equals(FieldKey.TITRE))) {
                    titreFormatDef = fieldOptions.getGenerationFormatDef();
                }
                return changeResult;
            } else if (optionName.equals(FieldOptionsConstants.CURRENCYARRAY_OPTION)) {
                return fieldOptions.setCurrencyList(optionValue);
            } else if (optionName.equals(FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION)) {
                return fieldOptions.setAddressFieldList(optionValue);
            } else {
                if ((optionValue != null) && (!(optionValue instanceof String))) {
                    throw new FieldOptionException("optionValue must be null or a String instance");
                }
                return setStringOption(optionName, (String) optionValue);
            }
        }

        private boolean setStringOption(String optionName, String optionValue) {
            if (optionValue != null) {
                optionValue = optionValue.trim();
                if (optionValue.length() == 0) {
                    optionValue = null;
                }
            }
            if (optionValue == null) {
                if (stringOptions.containsKey(optionName)) {
                    stringOptions.remove(optionName);
                    return true;
                } else {
                    return false;
                }
            }
            String currentValue = stringOptions.get(optionName);
            if (currentValue == null) {
                stringOptions.put(optionName, optionValue);
                return true;
            } else if (currentValue.equals(optionValue)) {
                return false;
            } else {
                stringOptions.put(optionName, optionValue);
                return true;
            }
        }


        private class InternalFieldOptions implements FieldOptions {

            private Short informationDisplay = null;
            private SubsetKey sphereKey = null;
            private boolean subfieldDisplay = false;
            private IntegerList currencyList = null;
            private CorpusFieldList addressFieldList = null;
            private BasedefichesFormatDef generationFormatDef;

            public SubsetKey getDefaultSphereKey() {
                return sphereKey;
            }

            public IntegerList getCurrencyList() {
                return currencyList;
            }

            public CorpusFieldList getAddressFieldList() {
                return addressFieldList;
            }

            public BasedefichesFormatDef getGenerationFormatDef() {
                return generationFormatDef;
            }

            public Short getInformationDisplay() {
                return informationDisplay;
            }

            public boolean getSubfieldDisplay() {
                return subfieldDisplay;
            }

            private boolean setSubfieldDisplay(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (subfieldDisplay) {
                        subfieldDisplay = false;
                        return true;
                    } else {
                        return false;
                    }
                } else if (optionValue.equals(Boolean.TRUE)) {
                    if (!subfieldDisplay) {
                        subfieldDisplay = true;
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    throw new FieldOptionException("optionValue must be null or equals to Boolean.TRUE");
                }
            }

            private boolean setInformationDisplay(Object optionValue) throws FieldOptionException {
                if (!fieldKey.isInformationFieldKey()) {
                    throw new FieldOptionException("No pertinent option");
                }
                if (optionValue == null) {
                    throw new FieldOptionException("optionValue cannot be null");
                }
                if (!(optionValue instanceof Short)) {
                    throw new FieldOptionException("optionValue must be a short");
                }
                short newVal = ((Short) optionValue).shortValue();
                short currentVal = informationDisplay.shortValue();
                switch (newVal) {
                    case FieldOptionsUtils.INLINE_DISPLAY:
                    case FieldOptionsUtils.BLOCK_DISPLAY:
                        if (newVal != currentVal) {
                            informationDisplay = ((Short) optionValue);
                            return true;
                        } else {
                            return false;
                        }
                    default:
                        throw new FieldOptionException("unknown informationDisplay value = " + newVal);
                }
            }

            private boolean setDefaultSphereKey(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (sphereKey == null) {
                        return false;
                    } else {
                        sphereKey = null;
                        return true;
                    }
                } else {
                    if (!(optionValue instanceof SubsetKey)) {
                        throw new FieldOptionException("optionValue must be a SubsetKey instance");
                    }
                    SubsetKey subsetKey = (SubsetKey) optionValue;
                    if (!subsetKey.isSphereSubset()) {
                        throw new FieldOptionException("optionValue must be a sphere SubsetKey");
                    }
                    if ((sphereKey != null) && (sphereKey.equals(subsetKey))) {
                        return false;
                    }
                    sphereKey = subsetKey;
                    return true;
                }
            }

            private boolean setGenerationFormatDef(Object optionValue) throws FieldOptionException {
                boolean returnVal = false;
                if (optionValue == null) {
                    if (generationFormatDef != null) {
                        generationFormatDef = null;
                        returnVal = true;
                    }
                } else {
                    if (!(optionValue instanceof BasedefichesFormatDef)) {
                        throw new FieldOptionException("optionValue must be an instance of BasedeficheFormatDef");
                    }
                    BasedefichesFormatDef newFormatDef = (BasedefichesFormatDef) optionValue;
                    if (generationFormatDef == null) {
                        generationFormatDef = newFormatDef;
                        returnVal = true;
                    } else {
                        if (BasedefichesFormatUtils.isSame(newFormatDef, generationFormatDef)) {
                            returnVal = false;
                        } else {
                            generationFormatDef = newFormatDef;
                            returnVal = true;
                        }
                    }
                }
                return returnVal;
            }

            private boolean setCurrencyList(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (currencyList == null) {
                        return false;
                    } else {
                        currencyList = null;
                        return true;
                    }
                }
                int[] newArray = null;
                if (optionValue instanceof int[]) {
                    newArray = (int[]) optionValue;
                } else if (optionValue instanceof IntegerList) {
                    newArray = IntegerListUtils.toArray((IntegerList) optionValue);
                } else {
                    throw new FieldOptionException("optionValue must be an instance of int[] or IntegerList");
                }
                int length = newArray.length;
                if (length == 0) {
                    if (currencyList == null) {
                        return false;
                    } else {
                        currencyList = null;
                        return true;
                    }
                }
                if ((currencyList != null) && (IntegerListUtils.isEqual(currencyList, newArray))) {
                    return false;
                }
                currencyList = IntegerListUtils.fromArray(newArray, true);
                return true;
            }

            private boolean setAddressFieldList(Object optionValue) throws FieldOptionException {
                if (optionValue == null) {
                    if (addressFieldList == null) {
                        return false;
                    } else {
                        addressFieldList = null;
                        return true;
                    }
                }
                FieldKey[] newArray = null;
                if (optionValue instanceof FieldKey[]) {
                    newArray = (FieldKey[]) optionValue;
                } else if (optionValue instanceof CorpusFieldList) {
                    newArray = CorpusFieldListUtils.toFieldKeyArray((CorpusFieldList) optionValue);
                } else {
                    throw new FieldOptionException("optionValue must be an instance of CorpusField[] or CorpusFieldList");
                }
                List<CorpusField> corpusFieldList = new ArrayList<CorpusField>();
                int length = newArray.length;
                for (int i = 0; i < length; i++) {
                    CorpusField corpusField = BdfimplCorpusMetadata.this.getCorpusFieldByFieldKey(newArray[i]);
                    if (corpusField != null) {
                        corpusFieldList.add(corpusField);
                    }
                }
                if (corpusFieldList.size() == 0) {
                    if (addressFieldList == null) {
                        return false;
                    } else {
                        addressFieldList = null;
                        return true;
                    }
                }
                if ((addressFieldList != null) && (CorpusFieldListUtils.isEqual(addressFieldList, corpusFieldList))) {
                    return false;
                }
                addressFieldList = CorpusFieldListUtils.fromList(corpusFieldList);
                return true;
            }

        }

    }


    private class BdfimplCorpusMetadataEditor implements CorpusMetadataEditor {

        public Metadata getMetadata() {
            return BdfimplCorpusMetadata.this;
        }

        public CorpusField createCorpusField(FieldKey fieldKey, short ficheItemType) throws ExistingFieldKeyException {
            if (corpusFieldMap.containsKey(fieldKey)) {
                throw new ExistingFieldKeyException();
            }
            CorpusField corpusField = innerCreateCorpusField(fieldKey, ficheItemType);
            fireChange();
            getListenerList().fireCorpusFieldCreated(corpus, corpusField);
            return corpusField;
        }

        private CorpusField innerCreateCorpusField(FieldKey fieldKey, short ficheItemType) {
            if (!fieldKey.isSpecialFieldKey()) {
                if (!StringUtils.isTechnicalName(fieldKey.getTypeString(), false)) {
                    throw new IllegalArgumentException("fieldKey.getTypeString() is not valid : " + fieldKey.getTypeString());
                }
            }
            BdfimplCorpusField corpusField = null;
            if ((fieldKey.equals(FieldKey.IDCORPUS)) || (fieldKey.equals(FieldKey.TITRE))) {
                corpusField = new BdfimplCorpusField(fieldKey);
            } else if (fieldKey.ownsToCorpsdefiche()) {
                corpusField = new BdfimplCorpusField(fieldKey);
            } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                corpusField = new BdfimplCorpusField(fieldKey, CorpusField.PERSONNE_FIELD);
            } else if (fieldKey.equals(FieldKey.LANG)) {
                corpusField = new BdfimplCorpusField(fieldKey, CorpusField.LANGUE_FIELD);
            } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                corpusField = new BdfimplCorpusField(fieldKey, CorpusField.PARA_FIELD);
            } else {
                ficheItemType = CorpusMetadataUtils.checkFicheItem(fieldKey, ficheItemType);
                corpusField = new BdfimplCorpusField(fieldKey, ficheItemType);
            }
            addInList(corpusField);
            corpusFieldMap.put(fieldKey, corpusField);
            return corpusField;
        }

        private void addInList(BdfimplCorpusField corpusField) {
            FieldKey fieldKey = corpusField.getFieldKey();
            if (fieldKey.isSectionFieldKey()) {
                sectionFieldList.add(corpusField);
            } else if (fieldKey.isProprieteFieldKey()) {
                proprieteFieldList.add(corpusField);
            } else if (fieldKey.isInformationFieldKey()) {
                informationFieldList.add(corpusField);
            }
        }

        public boolean putLibelle(CorpusField corpusField, Libelle libelle) {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            boolean done = defcorpusfield.putLibelle(libelle);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean removeLibelle(CorpusField corpusField, int langInteger) {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            boolean done = defcorpusfield.removeLibelle(langInteger);
            if (done) {
                fireChange();
            }
            return done;
        }

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            String defaultLib = corpus.getSubsetKey().getSubsetName();
            if (name.equals(CorpusConstants.FICHE_EXISTING)) {
                defaultLib = defaultLib + "/";
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplCorpusMetadata.this, defaultLib);
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            fireChange();
            return intitule;
        }

        public boolean putLibelle(Intitule intitule, Libelle libelle) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplCorpusMetadata.this, intitule);
            boolean done = def.putLibelle(libelle);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean removeLibelle(Intitule intitule, int langInteger) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplCorpusMetadata.this, intitule);
            boolean done = def.removeLibelle(langInteger);
            if (done) {
                fireChange();
            }
            return done;
        }

        public boolean setFieldOption(CorpusField corpusField, String optionName, Object optionValue) throws FieldOptionException {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            boolean done = defcorpusfield.setOption(optionName, optionValue);
            if (done) {
                fireChange();
            }
            return done;
        }

        public void removeCorpusField(CorpusField corpusField) throws MandatoryFieldException {
            BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
            FieldKey fieldKey = defcorpusfield.getFieldKey();
            if (fieldKey.isMandatoryFieldKey()) {
                throw new MandatoryFieldException();
            }
            corpusFieldMap.remove(fieldKey);
            if (fieldKey.isProprieteFieldKey()) {
                proprieteFieldList.remove(defcorpusfield);
                if ((geolocalisationField != null) && (geolocalisationField.equals(defcorpusfield))) {
                    geolocalisationField = null;
                }
            } else if (fieldKey.isSectionFieldKey()) {
                sectionFieldList.remove(defcorpusfield);
            } else if (fieldKey.isInformationFieldKey()) {
                informationFieldList.remove(defcorpusfield);
            }
            for (BdfimplCorpusField otherCorpusfield : proprieteFieldList) {
                if (otherCorpusfield.fieldOptions.addressFieldList != null) {
                    otherCorpusfield.fieldOptions.addressFieldList = CorpusFieldListUtils.removeCorpusField(otherCorpusfield.fieldOptions.addressFieldList, corpusField);
                }
            }
            fireChange();
            getListenerList().fireCorpusFieldRemoved(corpus, fieldKey);
        }

        public boolean setGeolocalisationField(CorpusField corpusField) {
            boolean done = false;
            if (corpusField == null) {
                if (geolocalisationField != null) {
                    geolocalisationField = null;
                    done = true;
                }
            } else {
                BdfimplCorpusField defcorpusfield = testCorpusField(corpusField);
                FieldKey fieldKey = defcorpusfield.getFieldKey();
                if (!fieldKey.isProprieteFieldKey()) {
                    throw new IllegalArgumentException("not a Propriete : " + fieldKey.toString());
                }
                if (corpusField.getFicheItemType() != CorpusField.GEOPOINT_FIELD) {
                    throw new IllegalArgumentException("not a GeoPoint Type : " + corpusField.getFicheItemType());
                }
                if (geolocalisationField == null) {
                    geolocalisationField = defcorpusfield;
                    done = true;
                } else if (!geolocalisationField.equals(defcorpusfield)) {
                    geolocalisationField = defcorpusfield;
                    done = true;
                }
            }
            if (done) {
                fireChange();
            }
            return done;
        }

    }

}
