/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadata;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadataEditor;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
class BdfimplSphereMetadata implements SphereMetadata {

    private BdfimplSphere sphere;
    private BdfimplSphereMetadataEditor sphereMetadataEditor = new BdfimplSphereMetadataEditor();
    private Map<String, BdfimplIntitule> intituleMap = new HashMap<String, BdfimplIntitule>();
    private List<BdfimplIntitule> intituleList = new ArrayList<BdfimplIntitule>();

    public BdfimplSphereMetadata(BdfimplSphere sphere) {
        this.sphere = sphere;
    }

    public Intitule getIntituleByName(String name) {
        return intituleMap.get(name);
    }

    public int getIntituleCount() {
        return intituleList.size();
    }

    public Intitule getIntitule(int i) {
        return intituleList.get(i);
    }

    SphereMetadataEditor getSphereMetadataEditor() {
        return sphereMetadataEditor;
    }

    void disable() {
        intituleMap.clear();
        intituleList.clear();
    }


    private class BdfimplSphereMetadataEditor implements SphereMetadataEditor {

        public Intitule createIntitule(String name) {
            if (name == null) {
                throw new IllegalArgumentException("name is null");
            }
            if (name.length() == 0) {
                throw new IllegalArgumentException("name is empty");
            }
            if (intituleMap.containsKey(name)) {
                return null;
            }
            BdfimplIntitule intitule = new BdfimplIntitule(name, BdfimplSphereMetadata.this, sphere.getSubsetKey().toString());
            intituleMap.put(name, intitule);
            intituleList.add(intitule);
            sphere.fireSphereChange();
            return intitule;
        }

        public Metadata getMetadata() {
            return BdfimplSphereMetadata.this;
        }

        public boolean putLibelle(Intitule intitule, Libelle libelle) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplSphereMetadata.this, intitule);
            boolean done = def.putLibelle(libelle);
            if (done) {
                sphere.fireSphereChange();
            }
            return done;
        }

        public boolean removeLibelle(Intitule intitule, int langInteger) {
            BdfimplIntitule def = BdfimplUtils.testIntitule(BdfimplSphereMetadata.this, intitule);
            boolean done = def.removeLibelle(langInteger);
            if (done) {
                sphere.fireSphereChange();
            }
            return done;
        }

    }

}
