/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.thesaurus.ParentRecursivityException;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadataEditor;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.collections.AbstractIndexAndIntKeyMap;
import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.text.Libelle;
import java.util.Locale;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;
import java.text.ParseException;


/**
 *
 * @author Vincent Calame
 */
class BdfimplThesaurus extends AbstractIndexAndIntKeyMap implements Thesaurus {

    private int availableIdths = 1;
    private SubsetKey thesaurusKey;
    BdfimplBasedefiches basedefiches;
    short thesaurusType;
    BdfimplThesaurusMetadata thesaurusMetadata;
    BdfimplThesaurusEditor thesaurusEditor = new BdfimplThesaurusEditor();
    List<AbstractMotcle> firstLevel = new ArrayList<AbstractMotcle>();
    private SortedCollatedKeyMap idalphaMap;
    private SortedCollatedKeyMap babelienMap;
    private IntHashMap libelleMaps;
    private BdfimplCorpus[] slaveSubsetArray = null;
    private FistLevelMotcleList fistLevelMotcleList = new FistLevelMotcleList();

    public BdfimplThesaurus(SubsetKey thesaurusKey, short thesaurusType, BdfimplBasedefiches basedefiches) {
        this.thesaurusKey = thesaurusKey;
        this.basedefiches = basedefiches;
        this.thesaurusType = thesaurusType;
        if (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE) {
            idalphaMap = new SortedCollatedKeyMap(Locale.ENGLISH);
        }
        if (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE) {
            babelienMap = new SortedCollatedKeyMap(Locale.FRENCH);
        } else {
            libelleMaps = new IntHashMap();
        }
        thesaurusMetadata = new BdfimplThesaurusMetadata(this);
    }

    public ThesaurusMetadata getThesaurusMetadata() {
        return thesaurusMetadata;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public SubsetKey getSubsetKey() {
        return thesaurusKey;
    }

    public SubsetItem getSubsetItemById(int idsubset) {
        return (SubsetItem) getValueByKey(idsubset);
    }

    public int getTicketLength() {
        return length();
    }

    public int size() {
        return getEnabledIndexCount();
    }

    public SubsetItem getSubsetItemByTicket(int ticket) {
        return (SubsetItem) getValueByIndex(ticket);
    }

    public MotcleList getFirstLevelList() {
        return fistLevelMotcleList;
    }

    public Motcle getMotcleByIdths(int idths) {
        return (Motcle) getValueByKey(idths);
    }

    public Motcle getMotcleByIdalpha(String idalpha) {
        if (idalphaMap == null) {
            throw new UnsupportedOperationException("thesarus has no idalpha");
        }
        return (Motcle) idalphaMap.getValue(idalpha);
    }

    public Motcle getMotcleByLibelle(String libelle, int langInteger) {
        SortedCollatedKeyMap map = getLibelleMap(langInteger);
        List liste = (List) map.getValue(libelle);
        if (liste == null) {
            return null;
        }
        return (Motcle) liste.get(0);
    }

    public Set<Motcle> getMotcleSetByIdalpha(String s, int condition) {
        if (idalphaMap == null) {
            throw new UnsupportedOperationException("thesarus has no idalpha");
        }
        Set<Motcle> hashSet = new HashSet<Motcle>();
        SearchResultUnitList list = idalphaMap.search(s, condition, null);
        int count = list.getSearchResultUnitCount();
        for (int i = 0; i < count; i++) {
            hashSet.add((Motcle) list.getSearchResultUnit(i).getValue());
        }
        return hashSet;
    }

    public Set<Motcle> getMotcleSetByLibelle(String s, int condition, int langInteger) {
        SortedCollatedKeyMap map = getLibelleMap(langInteger);
        Set<Motcle> hashSet = new HashSet<Motcle>();
        SearchResultUnitList list = map.search(s, condition, null);
        int count = list.getSearchResultUnitCount();
        for (int i = 0; i < count; i++) {
            @SuppressWarnings("unchecked")
            List<Motcle> motcleList = (List<Motcle>) list.getSearchResultUnit(i).getValue();
            hashSet.addAll(motcleList);
        }
        return hashSet;
    }

    public Subset getMasterSubset() {
        return null;
    }

    public Subset[] getSlaveSubsetArray() {
        return BdfimplUtils.copy(slaveSubsetArray);
    }

    protected void addSlaveCorpus(BdfimplCorpus bdfimplCorpus) {
        slaveSubsetArray = BdfimplUtils.addSlaveCorpus(slaveSubsetArray, bdfimplCorpus);
    }

    protected void removeSlaveCorpus(BdfimplCorpus bdfimplCorpus) {
        slaveSubsetArray = BdfimplUtils.removeSlaveCorpus(slaveSubsetArray, bdfimplCorpus);
    }

    protected void fireThesaurusChange() {
        basedefiches.fireSubsetChange(thesaurusKey);
    }

    protected ThesaurusEditor getThesaurusEditor() {
        return thesaurusEditor;
    }

    protected void disable() {
        thesaurusMetadata.disable();
    }


    class BdfimplThesaurusEditor implements ThesaurusEditor {

        public Motcle createMotcle(int idths, String idalpha) throws ExistingIdException, ParseException {
            if (idths < 0) {
                idths = availableIdths;
            } else {
                int current = BdfimplThesaurus.this.getIndexByKey(idths);
                if (current != -1) {
                    throw new ExistingIdException();
                }
            }
            if (idalphaMap != null) {
                ThesaurusUtils.testIdalpha(idalpha);
                if (idalphaMap.getValue(idalpha) != null) {
                    throw new ExistingIdException();
                }
            } else {
                idalpha = null;
            }
            AbstractMotcle motcle = null;
            if (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE) {
                motcle = new BabelienMotcle(BdfimplThesaurus.this, idths);
            } else {
                motcle = new MultiMotcle(BdfimplThesaurus.this, idths);
            }
            if (idalpha != null) {
                motcle.setIdalpha(idalpha);
                idalphaMap.putValue(idalpha, motcle);
            }
            motcle.setTicket(putFromKey(idths, motcle));
            availableIdths = Math.max(availableIdths, idths) + 1;
            addInFirstLevel(motcle);
            fireThesaurusChange();
            return motcle;
        }

        public Thesaurus getThesaurus() {
            return BdfimplThesaurus.this;
        }

        public ThesaurusMetadataEditor getThesaurusMetadataEditor() {
            return thesaurusMetadata.getThesaurusMetadataEditor();
        }

        public boolean removeMotcle(Motcle motcle) {
            AbstractMotcle am = testMotcle(motcle);
            if (!basedefiches.isRemoveable(am)) {
                return false;
            }
            am.removeFromParent();
            if (idalphaMap != null) {
                idalphaMap.removeValue(am.getIdalpha());
            }
            int libelleCount = am.getLibelleCount();
            for (int i = 0; i < libelleCount; i++) {
                Libelle libelle = am.getLibelleByIndex(i);
                SortedCollatedKeyMap map = getLibelleMap(libelle.getLangInteger());
                remove(map, libelle.getLibelleString(), am);
            }
            BdfimplThesaurus.this.disable(am.getTicket());
            fireThesaurusChange();
            return true;
        }

        public boolean setChildIndex(Motcle motcle, int index) {
            AbstractMotcle am = testMotcle(motcle);
            boolean done = am.changeIndex(index);
            if (done) {
                fireThesaurusChange();
            }
            return done;
        }

        public boolean setIdalpha(Motcle motcle, String newIdalpha) throws ExistingIdException, ParseException {
            if (idalphaMap == null) {
                throw new UnsupportedOperationException("this is not a thesaurus with idalpha");
            }
            ThesaurusUtils.testIdalpha(newIdalpha);
            AbstractMotcle am = testMotcle(motcle);
            Object obj = idalphaMap.getValue(newIdalpha);
            if (obj != null) {
                if (!obj.equals(am)) {
                    throw new ExistingIdException();
                }
            }
            String oldIdalpha = am.getIdalpha();
            boolean done = am.setIdalpha(newIdalpha);
            if (done) {
                idalphaMap.removeValue(oldIdalpha);
                idalphaMap.putValue(newIdalpha, motcle);
                fireThesaurusChange();
            }
            return done;
        }

        public boolean putLibelle(Motcle motcle, Libelle libelle) {
            AbstractMotcle am = testMotcle(motcle);
            int langInteger = libelle.getLangInteger();
            Libelle oldLibelle = am.getLibelleByLang(langInteger);
            boolean done = am.putLibelle(libelle);
            if (done) {
                SortedCollatedKeyMap map = getLibelleMap(langInteger);
                if (oldLibelle != null) {
                    remove(map, oldLibelle.getLibelleString(), am);
                }
                add(map, libelle.getLibelleString(), am);
                fireThesaurusChange();
            }
            return done;
        }

        public boolean removeLibelle(Motcle motcle, int langInteger) {
            AbstractMotcle am = testMotcle(motcle);
            Libelle oldLibelle = am.getLibelleByLang(langInteger);
            boolean done = am.removeLibelle(langInteger);
            if (done) {
                if (oldLibelle != null) {
                    SortedCollatedKeyMap map = getLibelleMap(langInteger);
                    remove(map, oldLibelle.getLibelleString(), am);
                }
                fireThesaurusChange();
            }
            return done;
        }

        public boolean setParent(Motcle motcle, Motcle parentMotcle) throws ParentRecursivityException {
            AbstractMotcle am = testMotcle(motcle);
            boolean done = false;
            if (parentMotcle == null) {
                done = am.setParent(null);
            } else {
                AbstractMotcle amparent = testMotcle(parentMotcle);
                if (am.equals(amparent)) {
                    throw new IllegalArgumentException("motcle and parentMotcle are the same");
                }
                if (ThesaurusUtils.isDescendant(amparent, am)) {
                    throw new ParentRecursivityException(am, amparent);
                }
                done = am.setParent(amparent);
            }
            if (done) {
                fireThesaurusChange();
            }
            return done;
        }

    }

    /**
     * Appelé par AbstractMotcle
     */
    void removeFromFirstLevel(int index) {
        firstLevel.remove(index);
        for (int i = index; i < firstLevel.size(); i++) {
            AbstractMotcle child = (AbstractMotcle) firstLevel.get(i);
            child.decreaseIndex();
        }
    }

    void addInFirstLevel(AbstractMotcle motcle) {
        int size = firstLevel.size();
        firstLevel.add(motcle);
        motcle.setChildIndex(size);
    }

    private AbstractMotcle testMotcle(Motcle motcle) {
        if (motcle == null) {
            throw new IllegalArgumentException("motcle argument cannot be null");
        }
        try {
            AbstractMotcle am = (AbstractMotcle) motcle;
            if (am.getThesaurus() != this) {
                throw new IllegalArgumentException("motcle argument does not come from this thesaurus");
            }
            return am;
        } catch (ClassCastException cce) {
            throw new IllegalArgumentException("motcle argument does not come from this thesaurus");
        }
    }

    private void remove(SortedCollatedKeyMap map, String lib, Object obj) {
        List liste = (List) map.getValue(lib);
        if (liste.size() == 1) {
            if (!liste.get(0).equals(obj)) {
                throw new IllegalStateException(obj.toString() + " does not match " + liste.get(0));
            }
            map.removeValue(lib);
        } else {
            liste.remove(obj);
        }
    }

    private void add(SortedCollatedKeyMap map, String lib, Motcle motcle) {
        @SuppressWarnings("unchecked")
        List<Motcle> liste = (List<Motcle>) map.getValue(lib);
        if (liste == null) {
            map.putValue(lib, Collections.singletonList(motcle));
        } else if (liste.size() == 1) {
            List<Motcle> newList = new ArrayList<Motcle>();
            newList.add(liste.get(0));
            newList.add(motcle);
            map.putValue(lib, newList);
        } else {
            liste.add(motcle);
        }
    }

    private SortedCollatedKeyMap getLibelleMap(int langInteger) {
        if (babelienMap != null) {
            return babelienMap;
        } else {
            SortedCollatedKeyMap map = (SortedCollatedKeyMap) libelleMaps.get(langInteger);
            if (map == null) {
                map = new SortedCollatedKeyMap(langInteger);
                libelleMaps.put(langInteger, map);
            }
            return map;
        }
    }


    private class FistLevelMotcleList implements MotcleList {

        public Thesaurus getThesaurus() {
            return BdfimplThesaurus.this;
        }

        public int getMotcleCount() {
            return firstLevel.size();
        }

        public Motcle getMotcle(int i) {
            return firstLevel.get(i);
        }

    }

}
