/* BasedefichesLib_Impl - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.event.BasedefichesListener;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.collections.ArrayUtils;


/**
 *
 * @author Vincent Calame
 */
public class ListenerList {

    private BasedefichesListener[] basedefichesListenerArray = new BasedefichesListener[0];

    public ListenerList() {
    }

    public void addBasedefichesListener(BasedefichesListener bl) {
        int length = basedefichesListenerArray.length;
        basedefichesListenerArray = (BasedefichesListener[]) ArrayUtils.addUnique(basedefichesListenerArray, bl, new BasedefichesListener[length + 1]);
    }

    public void removeBasedefichesListener(BasedefichesListener bl) {
        int length = basedefichesListenerArray.length;
        if (length > 0) {
            basedefichesListenerArray = (BasedefichesListener[]) ArrayUtils.removeUnique(basedefichesListenerArray, bl, new BasedefichesListener[length - 1]);
        }
    }

    public void fireCorpusCreated(Corpus corpus) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].corpusCreated(corpus);
        }
    }

    public void fireCorpusRemoved(SubsetKey corpusKey, Subset masterSubset) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].corpusRemoved(corpusKey, masterSubset);
        }
    }

    public void fireCorpusFieldCreated(Corpus corpus, CorpusField corpusField) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].corpusFieldCreated(corpus, corpusField);
        }
    }

    public void fireCorpusFieldRemoved(Corpus corpus, FieldKey fieldKey) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].corpusFieldRemoved(corpus, fieldKey);
        }
    }

    public void fireFicheCreated(AbregeFiche abregeFiche) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].ficheCreated(abregeFiche);
        }
    }

    public void fireFicheSaved(AbregeFiche abregeFiche, Fiche fiche) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].ficheSaved(abregeFiche, fiche);
        }
    }

    public void fireThesaurusCreated(Thesaurus thesaurus) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].thesaurusCreated(thesaurus);
        }
    }

    public void fireThesaurusRemoved(SubsetKey thesaurusKey) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].thesaurusRemoved(thesaurusKey);
        }
    }

    public void fireSphereCreated(Sphere sphere) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].sphereCreated(sphere);
        }
    }

    public void fireSphereRemoved(SubsetKey sphereKey) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].sphereRemoved(sphereKey);
        }
    }

    public void fireRedacteurRemoved(RedacteurKey redacteurKey) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].redacteurRemoved(redacteurKey);
        }
    }

    public void fireAlbumCreated(Album album) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].albumCreated(album);
        }
    }

    public void fireAlbumRemoved(SubsetKey albumKey) {
        for (int i = 0; i < basedefichesListenerArray.length; i++) {
            basedefichesListenerArray[i].albumRemoved(albumKey);
        }
    }

}
