/* BasedefichesLib_Impl - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl;

import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
class MultiMotcle extends AbstractMotcle {

    private LinkedIntHashMap langIntMap = new LinkedIntHashMap();

    public MultiMotcle(BdfimplThesaurus thesaurus, int idths) {
        super(thesaurus, idths);
    }

    public Libelle getLibelleByLang(int langInteger) {
        return (Libelle) langIntMap.get(langInteger);
    }

    public int getLibelleCount() {
        return langIntMap.size();
    }

    public Libelle getLibelleByIndex(int i) {
        return (Libelle) langIntMap.getValueAt(i);
    }

    public String getLibelleString(int langInteger, boolean defaut) {
        Libelle libelle = getLibelleByLang(langInteger);
        if (libelle != null) {
            return libelle.getLibelleString();
        }
        if (defaut) {
            int[] langArray = this.getThesaurus().getThesaurusMetadata().getAuthorizedLangIntegerArray();
            for (int i = 0; i < langArray.length; i++) {
                int lg2 = langArray[i];
                if (lg2 != langInteger) {
                    libelle = getLibelleByLang(lg2);
                    if (libelle != null) {
                        return libelle.getLibelleString();
                    }
                }
            }
            return "?";
        } else {
            return null;
        }
    }

    boolean putLibelle(Libelle libelle) {
        int langInteger = libelle.getLangInteger();
        Libelle oldLibelle = getLibelleByLang(langInteger);
        if (oldLibelle == null) {
            langIntMap.put(langInteger, libelle);
            return true;
        }
        if (oldLibelle.getLibelleString().equals(libelle.getLibelleString())) {
            return false;
        }
        langIntMap.put(langInteger, libelle);
        return true;
    }

    boolean removeLibelle(int langInteger) {
        Libelle libelle = getLibelleByLang(langInteger);
        if (libelle == null) {
            return false;
        }
        langIntMap.remove(langInteger);
        return true;
    }

}
