/* BasedefichesLib_Impl - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.NotEditableCroisementException;


/**
 *
 * @author Vincent Calame
 */
public class CoupleCroisementItemProvider extends CroisementItemProvider {

    private SubsetCouple subsetCouple;
    private ImplCroisementItem[] croisementItemArray1 = new ImplCroisementItem[64];
    private ImplCroisementItem[] croisementItemArray2 = new ImplCroisementItem[64];

    public CoupleCroisementItemProvider(SubsetCouple subsetCouple) {
        this.subsetCouple = subsetCouple;
    }

    public ImplCroisementItem getCroisementItem(SubsetKey subsetKey, int ticket) {
        int number = subsetCouple.getSubsetNumber(subsetKey);
        if (number == 1) {
            return getCroisementItem(croisementItemArray1, ticket);
        } else {
            return getCroisementItem(croisementItemArray2, ticket);
        }
    }

    public ImplCroisementItem getOrCreateCroisementItem(SubsetKey subsetKey, int ticket) {
        int number = subsetCouple.getSubsetNumber(subsetKey);
        if (number == 1) {
            croisementItemArray1 = testSize(croisementItemArray1, ticket);
            return getOrCreateCroisementItem(croisementItemArray1, ticket);
        } else {
            croisementItemArray2 = testSize(croisementItemArray2, ticket);
            return getOrCreateCroisementItem(croisementItemArray2, ticket);
        }
    }

    public boolean isTicketForbidden() {
        return false;
    }

    public SubsetKey getOtherSubsetKey(SubsetKey subsetKey) {
        return subsetCouple.getOtherSubsetKey(subsetKey);
    }

    public CroisementItemProviderBuilder getCroisementItemProviderBuilder(Basedefiches basedefiches) {
        Subset subset1 = basedefiches.getSubset(subsetCouple.getSubsetKey1());
        Subset subset2 = basedefiches.getSubset(subsetCouple.getSubsetKey2());
        if (subset1 == null) {
            throw new NotEditableCroisementException(subsetCouple.getSubsetKey1());
        }
        if (subset2 == null) {
            throw new NotEditableCroisementException(subsetCouple.getSubsetKey2());
        }
        return new CoupleCroisementItemProviderBuilder(subset1, subset2);
    }


    class CoupleCroisementItemProviderBuilder extends CroisementItemProviderBuilder {

        private Subset subset1;
        private Subset subset2;
        private Map<Integer, CroisementItemBuilder> map1 = new HashMap<Integer, CroisementItemBuilder>();
        private Map<Integer, CroisementItemBuilder> map2 = new HashMap<Integer, CroisementItemBuilder>();

        CoupleCroisementItemProviderBuilder(Subset subset1, Subset subset2) {
            this.subset1 = subset1;
            this.subset2 = subset2;
        }

        public CroisementItemBuilder getCroisementItemBuilder1(int ticket1) {
            return getCroisementItemBuilder(map1, ticket1);
        }

        public CroisementItemBuilder getCroisementItemBuilder2(int ticket2) {
            return getCroisementItemBuilder(map2, ticket2);
        }

        public int getTicket1(int idsubset1) {
            return getTicket(subset1, idsubset1);
        }

        public int getTicket2(int idsubset2) {
            return getTicket(subset2, idsubset2);
        }

        public void build() {
            int max1 = getMaxTicket(map1);
            croisementItemArray1 = new ImplCroisementItem[max1 + 1];
            for (CroisementItemBuilder croisementItemBuilder : map1.values()) {
                int ticket = croisementItemBuilder.ticket;
                ImplCroisementItem croisementItem = croisementItemBuilder.build();
                croisementItemArray1[ticket] = croisementItem;
            }
            int max2 = getMaxTicket(map2);
            croisementItemArray2 = new ImplCroisementItem[max2 + 1];
            for (CroisementItemBuilder croisementItemBuilder : map2.values()) {
                int ticket = croisementItemBuilder.ticket;
                ImplCroisementItem croisementItem = croisementItemBuilder.build();
                croisementItemArray2[ticket] = croisementItem;
            }
        }

    }

}
