/* BasedefichesLib_Impl - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.CroisementList;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public abstract class CroisementItemProvider {

    public abstract ImplCroisementItem getCroisementItem(SubsetKey subsetKey, int ticket);

    public abstract ImplCroisementItem getOrCreateCroisementItem(SubsetKey subsetKey, int ticket);

    public abstract boolean isTicketForbidden();

    public abstract SubsetKey getOtherSubsetKey(SubsetKey subsetKey);

    public abstract CroisementItemProviderBuilder getCroisementItemProviderBuilder(Basedefiches basedefiches);

    public boolean removeCroisement(SubsetKey subsetKey1, int ticket1, SubsetKey subsetKey2, int ticket2) {
        ImplCroisementItem croisementItem1 = getCroisementItem(subsetKey1, ticket1);
        if (croisementItem1 == null) {
            return false;
        }
        ImplCroisementItem croisementItem2 = getCroisementItem(subsetKey2, ticket2);
        if (croisementItem2 == null) {
            return false;
        }
        int idx2 = croisementItem1.indexOfTicket(ticket2);
        if (idx2 == -1) {
            return false;
        }
        int idx1 = croisementItem2.indexOfTicket(ticket1);
        if (idx1 == -1) {
            throw new ShouldNotOccurException("idx = -1");
        }
        croisementItem1.removeIndex(idx2);
        croisementItem2.removeIndex(idx1);
        return true;
    }

    public int clear(SubsetKey subsetKey, int ticket) {
        ImplCroisementItem croisementItem1 = getCroisementItem(subsetKey, ticket);
        if (croisementItem1 == null) {
            return 0;
        }
        ImplCroisementItem.UpdateResult updateResult = croisementItem1.clear();
        int count = remove(updateResult, subsetKey, ticket);
        return count;
    }

    public void addCroisement(SubsetKey subsetKey, int ticket, CroisementList croisementList, short updateType) {
        ImplCroisementItem croisementItem = getOrCreateCroisementItem(subsetKey, ticket);
        int forbiddenTicket = (isTicketForbidden()) ? ticket : -1;
        ImplCroisementItem.UpdateResult updateResult = croisementItem.update(croisementList, forbiddenTicket, updateType);
        remove(updateResult, subsetKey, ticket);
        update(updateResult, subsetKey, ticket);
        add(updateResult, subsetKey, ticket);
    }

    private int remove(ImplCroisementItem.UpdateResult updateResult, SubsetKey subsetKey, int ticket) {
        int count = updateResult.getRemoveCount();
        if (count == 0) {
            return 0;
        }
        SubsetKey otherSubsetKey = getOtherSubsetKey(subsetKey);
        for (int i = 0; i < count; i++) {
            int otherTicket = updateResult.getRemoveTicket(i);
            ImplCroisementItem otherCroisementItem = getCroisementItem(otherSubsetKey, otherTicket);
            otherCroisementItem.removeTicket(ticket);
        }
        return count;
    }

    private int update(ImplCroisementItem.UpdateResult updateResult, SubsetKey subsetKey, int ticket) {
        int count = updateResult.getUpdateCount();
        if (count == 0) {
            return 0;
        }
        SubsetKey otherSubsetKey = getOtherSubsetKey(subsetKey);
        for (int i = 0; i < count; i++) {
            int otherTicket = updateResult.getUpdateTicket(i);
            ImplCroisementItem otherCroisementItem = getCroisementItem(otherSubsetKey, otherTicket);
            otherCroisementItem.updateTicket(ticket, updateResult.getUpdatePoids(i));
        }
        return count;
    }

    private int add(ImplCroisementItem.UpdateResult updateResult, SubsetKey subsetKey, int ticket) {
        int count = updateResult.getAddedCount();
        if (count == 0) {
            return 0;
        }
        SubsetKey otherSubsetKey = getOtherSubsetKey(subsetKey);
        for (int i = 0; i < count; i++) {
            int otherTicket = updateResult.getAddedTicket(i);
            ImplCroisementItem otherCroisementItem = getOrCreateCroisementItem(otherSubsetKey, otherTicket);
            otherCroisementItem.addTicket(ticket, updateResult.getAddedPoids(i));
        }
        return count;
    }

    protected ImplCroisementItem[] enlarge(ImplCroisementItem[] croisementItemArray, int newSize) {
        ImplCroisementItem[] newArray = new ImplCroisementItem[newSize];
        System.arraycopy(croisementItemArray, 0, newArray, 0, croisementItemArray.length);
        return newArray;
    }

    protected ImplCroisementItem[] testSize(ImplCroisementItem[] croisementItemArray, int ticket) {
        if (ticket >= croisementItemArray.length) {
            int newSize = Math.max(croisementItemArray.length * 2, ticket + 1);
            croisementItemArray = enlarge(croisementItemArray, newSize);
        }
        return croisementItemArray;
    }

    protected ImplCroisementItem getCroisementItem(ImplCroisementItem[] croisementItemArray, int ticket) {
        if (ticket >= croisementItemArray.length) {
            return null;
        }
        return croisementItemArray[ticket];
    }

    protected ImplCroisementItem getOrCreateCroisementItem(ImplCroisementItem[] croisementItemArray, int ticket) {
        ImplCroisementItem croisementItem = croisementItemArray[ticket];
        if (croisementItem == null) {
            croisementItem = new ImplCroisementItem();
            croisementItemArray[ticket] = croisementItem;
        }
        return croisementItem;
    }

}
