/* BasedefichesLib_Impl - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.croisement.CroisementEntry;


/**
 *
 * @author Vincent Calame
 */
public abstract class CroisementItemProviderBuilder {

    public CroisementItemProviderBuilder() {
    }

    public abstract CroisementItemBuilder getCroisementItemBuilder1(int ticket1);

    public abstract CroisementItemBuilder getCroisementItemBuilder2(int ticket2);

    public abstract int getTicket1(int idsubet1);

    public abstract int getTicket2(int idsubet2);

    public abstract void build();

    public void initCroisements(List<CroisementEntry> croisementEntryList) {
        for (CroisementEntry croisementEntry : croisementEntryList) {
            addCroisementEntry(croisementEntry);
        }
    }

    private void addCroisementEntry(CroisementEntry croisementEntry) {
        int idsubset1 = croisementEntry.getIdsubset1();
        int ticket1 = getTicket1(idsubset1);
        if (ticket1 == -1) {
            return;
        }
        int idsubset2 = croisementEntry.getIdsubset2();
        int ticket2 = getTicket2(idsubset2);
        if (ticket2 == -1) {
            return;
        }
        CroisementItemBuilder croisementItemBuilder1 = getCroisementItemBuilder1(ticket1);
        CroisementItemBuilder croisementItemBuilder2 = getCroisementItemBuilder2(ticket2);
        if (croisementItemBuilder1.equals(croisementItemBuilder2)) {
            return;
        }
        croisementItemBuilder1.add(ticket2, croisementEntry.getPoids(), croisementEntry.getPosition2());
        croisementItemBuilder2.add(ticket1, croisementEntry.getPoids(), croisementEntry.getPosition1());
    }

    protected int getTicket(Subset subset, int idsubset) {
        SubsetItem subsetItem = subset.getSubsetItemById(idsubset);
        if (subsetItem == null) {
            return -1;
        }
        return subsetItem.getTicket();
    }

    protected int getMaxTicket(Map<Integer, CroisementItemBuilder> map) {
        int maxTicket = -1;
        for (Integer itg : map.keySet()) {
            maxTicket = Math.max(maxTicket, itg.intValue());
        }
        return maxTicket;
    }

    protected CroisementItemBuilder getCroisementItemBuilder(Map<Integer, CroisementItemBuilder> map, int ticket) {
        Integer itg = new Integer(ticket);
        CroisementItemBuilder croisementItemBuilder = (CroisementItemBuilder) map.get(itg);
        if (croisementItemBuilder == null) {
            croisementItemBuilder = new CroisementItemBuilder(ticket);
            map.put(itg, croisementItemBuilder);
        }
        return croisementItemBuilder;
    }


    static protected class CroisementItemBuilder {

        int ticket;
        TreeSet<TempEntry> positionedEntry = new TreeSet<TempEntry>();
        LinkedHashSet<TempEntry> notpositioneEntry = new LinkedHashSet<TempEntry>();

        public CroisementItemBuilder(int ticket) {
            this.ticket = ticket;
        }

        void add(int ticket, int poids, int position) {
            TempEntry tempEntry = new TempEntry(ticket, poids, position);
            if (position < 0) {
                notpositioneEntry.add(tempEntry);
            } else {
                positionedEntry.add(tempEntry);
            }
        }

        public ImplCroisementItem build() {
            int length = positionedEntry.size() + notpositioneEntry.size();
            int[] ticketArray = new int[length];
            int[] poidsArray = new int[length];
            int p = 0;
            for (TempEntry tempEntry : positionedEntry) {
                ticketArray[p] = tempEntry.ticket;
                poidsArray[p] = tempEntry.poids;
                p++;
            }
            for (TempEntry tempEntry : notpositioneEntry) {
                ticketArray[p] = tempEntry.ticket;
                poidsArray[p] = tempEntry.poids;
                p++;
            }
            ImplCroisementItem croisementItem = new ImplCroisementItem(ticketArray, poidsArray);
            return croisementItem;
        }

    }


    static class TempEntry implements Comparable {

        int ticket;
        int poids;
        int position;

        TempEntry(int ticket, int poids, int position) {
            this.ticket = ticket;
            this.poids = poids;
            this.position = position;
        }

        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (!(obj instanceof TempEntry)) {
                return false;
            }
            return (compareTo(obj) == 0);
        }

        public int compareTo(Object obj) {
            TempEntry other = (TempEntry) obj;
            if (this.ticket == other.ticket) {
                return 0;
            }
            if (other.position > this.position) {
                return -1;
            }
            if (other.position < this.position) {
                return 1;
            }
            if (other.poids > this.poids) {
                return -1;
            }
            if (other.poids < this.poids) {
                return 1;
            }
            if (other.ticket > this.ticket) {
                return -1;
            }
            if (other.ticket < this.ticket) {
                return 1;
            }
            return 0;
        }

    }

}
