/* BasedefichesLib_Impl - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.croisement.CroisementUtils;


/**
 *
 * @author Vincent Calame
 */
public class MasterAndSlaveCroisement implements Croisement {

    Subset masterSubset;
    Subset slaveSubset;
    SubsetKey masterSubsetKey;
    SubsetKey slaveSubsetKey;
    SubsetCouple subsetCouple;

    public MasterAndSlaveCroisement(Subset masterSubset, Subset slaveSubset) {
        this.masterSubset = masterSubset;
        this.slaveSubset = slaveSubset;
        masterSubsetKey = masterSubset.getSubsetKey();
        slaveSubsetKey = slaveSubset.getSubsetKey();
        subsetCouple = new SubsetCouple(masterSubsetKey, slaveSubsetKey);
    }

    public Basedefiches getBasedefiches() {
        return masterSubset.getBasedefiches();
    }

    public SubsetCouple getSubsetCouple() {
        return subsetCouple;
    }

    public CroisementItem getCroisementItem(SubsetKey subsetKey, int ticket) {
        if (subsetKey.equals(masterSubsetKey)) {
            return getCroisementItem(masterSubset, ticket, slaveSubset);
        } else {
            return getCroisementItem(slaveSubset, ticket, masterSubset);
        }
    }

    private CroisementItem getCroisementItem(Subset subset1, int ticket, Subset subset2) {
        SubsetItem subsetItem1 = subset1.getSubsetItemByTicket(ticket);
        if (subsetItem1 == null) {
            return CroisementUtils.EMPTY_CROISEMENTITEM;
        }
        SubsetItem subsetItem2 = subset2.getSubsetItemById(subsetItem1.getIdsubset());
        if (subsetItem2 == null) {
            return CroisementUtils.EMPTY_CROISEMENTITEM;
        }
        return CroisementUtils.getSingletonCroisementItem(subsetItem2.getTicket(), 1);
    }

    public int getPoids(SubsetKey subsetKey1, int ticket1, SubsetKey subsetKey2, int ticket2) {
        SubsetItem subsetItem1 = getSubsetItem(subsetKey1, ticket1);
        SubsetItem subsetItem2 = getSubsetItem(subsetKey2, ticket2);
        if (subsetItem1 == null) {
            return 0;
        }
        if (subsetItem2 == null) {
            return 0;
        }
        if (subsetItem1.getIdsubset() == subsetItem2.getIdsubset()) {
            return 1;
        } else {
            return 0;
        }
    }

    private SubsetItem getSubsetItem(SubsetKey subsetKey, int ticket) {
        if (subsetKey.equals(masterSubsetKey)) {
            return masterSubset.getSubsetItemByTicket(ticket);
        } else {
            return slaveSubset.getSubsetItemByTicket(ticket);
        }
    }

}
