/* BasedefichesLib_Impl - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.impl.croisements;

import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.CroisementEntry;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.croisement.CroisementList;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.impl.EditableCroisement;


/**
 *
 * @author Vincent Calame
 */
public class MemoryCroisement implements EditableCroisement {

    private SubsetCouple subsetCouple;
    private CroisementItemProvider croisementItemProvider;
    private Basedefiches basedefiches;
    public final static long[] EMPTY_ARRAY = new long[0];

    public MemoryCroisement(Basedefiches basedefiches, SubsetCouple subsetCouple) {
        this.basedefiches = basedefiches;
        this.subsetCouple = subsetCouple;
        if (subsetCouple.isUnique()) {
            croisementItemProvider = new UniqueCroisementItemProvider(subsetCouple.getSubsetKey1());
        } else {
            croisementItemProvider = new CoupleCroisementItemProvider(subsetCouple);
        }
    }

    public SubsetCouple getSubsetCouple() {
        return subsetCouple;
    }

    public Basedefiches getBasedefiches() {
        return basedefiches;
    }

    public CroisementItem getCroisementItem(SubsetKey subsetKey, int ticket) {
        ImplCroisementItem croisementItem = croisementItemProvider.getCroisementItem(subsetKey, ticket);
        if (croisementItem == null) {
            return CroisementUtils.EMPTY_CROISEMENTITEM;
        } else {
            return croisementItem;
        }
    }

    public int getPoids(SubsetKey subsetKey1, int ticket1, SubsetKey subsetKey2, int ticket2) {
        ImplCroisementItem croisementItem = getImplCroisementItem(subsetKey1, ticket1);
        if (croisementItem == null) {
            return 0;
        }
        int idx = croisementItem.indexOfTicket(ticket2);
        if (idx == -1) {
            return 0;
        }
        return croisementItem.getPoids(idx);
    }

    public boolean removeCroisement(SubsetKey subsetKey1, int ticket1, SubsetKey subsetKey2, int ticket2) {
        return croisementItemProvider.removeCroisement(subsetKey1, ticket1, subsetKey2, ticket2);
    }

    public int clear(SubsetKey subsetKey, int ticket) {
        return croisementItemProvider.clear(subsetKey, ticket);
    }

    public void addCroisement(SubsetKey subsetKey, int ticket, CroisementList croisementList, short updateType) {
        croisementItemProvider.addCroisement(subsetKey, ticket, croisementList, updateType);
    }

    public void initCroisements(List<CroisementEntry> croisementEntryList) {
        CroisementItemProviderBuilder builder = croisementItemProvider.getCroisementItemProviderBuilder(basedefiches);
        builder.initCroisements(croisementEntryList);
        builder.build();
    }

    private int getSubsetNumber(SubsetKey subsetKey) {
        int subsetNumber = subsetCouple.getSubsetNumber(subsetKey);
        if (subsetNumber == 0) {
            throw new IllegalArgumentException("wrong subsetKey argument");
        }
        return subsetNumber;
    }

    protected boolean remove(SubsetKey subsetKey1, int ticket1, SubsetKey subsetKey2, int ticket2) {
        return false;
    }

    private ImplCroisementItem getImplCroisementItem(SubsetKey subsetKey, int ticket) {
        return croisementItemProvider.getCroisementItem(subsetKey, ticket);
    }

}
