/* BasedefichesLib_Tools - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.duplication;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class DuplicationDefBuilder implements DuplicationConstants {

    private InternalDuplicationDef internalDuplicationDef = new InternalDuplicationDef();
    private Set<Integer> idcorpusSet = new HashSet<Integer>();

    public DuplicationDefBuilder(SubsetKey originCorpusSubsetKey) {
        if (!originCorpusSubsetKey.isCorpusSubset()) {
            throw new IllegalArgumentException("originCorpusSubsetKey is not a corpus subset");
        }
        internalDuplicationDef.originCorpusSubsetKey = originCorpusSubsetKey;
        internalDuplicationDef.destinationCorpusSubsetKey = originCorpusSubsetKey;
    }

    public DuplicationDefBuilder(SubsetKey originCorpusSubsetKey, SubsetKey destinationCorpusSubsetKey) {
        if (!originCorpusSubsetKey.isCorpusSubset()) {
            throw new IllegalArgumentException("originCorpusSubsetKey is not a corpus subset");
        }
        if (!destinationCorpusSubsetKey.isCorpusSubset()) {
            throw new IllegalArgumentException("destinationCorpusSubsetKey is not a corpus subset");
        }
        internalDuplicationDef.originCorpusSubsetKey = originCorpusSubsetKey;
        internalDuplicationDef.destinationCorpusSubsetKey = destinationCorpusSubsetKey;
    }

    public DuplicationDef toDuplicationDef() {
        return internalDuplicationDef;
    }

    public void setDestinationIdcorpusStart(int destinationIdcorpusStart) {
        if (destinationIdcorpusStart < 1) {
            switch (destinationIdcorpusStart) {
                case NEWIDCORPUS:
                    break;
                case SAMEIDCORPUS:
                    if (internalDuplicationDef.isSame()) {
                        throw new IllegalArgumentException("SAMEIDCORPUS cannot be applied when destination corpus = origin corpus");
                    }
                    break;
                default:
                    throw new IllegalArgumentException("destinationIdcorpusStart > 0 or = NEWIDCORPUS or SAMEIDCORPUS");
            }
        }
        internalDuplicationDef.destinationIdcorpusStart = destinationIdcorpusStart;
    }

    public void setPoidsWithOrigin(int poids) {
        if ((poids < 1) && (poids != -1)) {
            throw new IllegalArgumentException("poids must be > 0 or = -1");
        }
        internalDuplicationDef.poids = poids;
    }

    public void addOriginIdcorpus(int idcorpus) {
        if (idcorpus < 1) {
            throw new IllegalArgumentException("idcorpus < 1");
        }
        if (!idcorpusSet.contains(idcorpus)) {
            idcorpusSet.add(idcorpus);
            internalDuplicationDef.addOriginIdcorpus(idcorpus);
        }
    }


    private static class InternalDuplicationDef implements DuplicationDef {

        private SubsetKey originCorpusSubsetKey;
        private SubsetKey destinationCorpusSubsetKey;
        private int destinationIdcorpusStart = NEWIDCORPUS;
        private int poids = -1;
        private List<Integer> idcorpusList = new ArrayList<Integer>();

        public SubsetKey getOriginCorpusSubsetKey() {
            return originCorpusSubsetKey;
        }

        public SubsetKey getDestinationCorpusSubsetKey() {
            return destinationCorpusSubsetKey;
        }

        public int getDestinationIdcorpusStart() {
            return destinationIdcorpusStart;
        }

        public int getOriginIdcorpusCount() {
            return idcorpusList.size();
        }

        public int getOriginIdcorpus(int i) {
            return idcorpusList.get(i);
        }

        public int getPoidsWithOrigin() {
            return poids;
        }

        private boolean isSame() {
            return (originCorpusSubsetKey.equals(destinationCorpusSubsetKey));
        }

        private void addOriginIdcorpus(int idcorpus) {
            idcorpusList.add(idcorpus);
        }

    }

}
