/* BasedefichesLib_Tools - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.duplication;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.NoMasterIdsubsetException;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.croisement.CroisementList;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 *
 * @author Vincent Calame
 */
public class DuplicationEngine implements DuplicationConstants {

    private InternalDuplicationLog internalDuplicationLog = new InternalDuplicationLog();
    private Basedefiches basedefiches;
    private DuplicationDef duplicationDef;
    private SubsetKey originCorpusSubsetKey;
    private SubsetKey destinationCorpusSubsetKey;
    private CroisementEditor croisementEditor;
    private BasedefichesEditor basedefichesEditor;

    private DuplicationEngine(BasedefichesEditor basedefichesEditor, DuplicationDef duplicationDef) {
        this.basedefichesEditor = basedefichesEditor;
        this.basedefiches = basedefichesEditor.getBasedefiches();
        this.croisementEditor = basedefichesEditor.getCroisementEditor();
        this.duplicationDef = duplicationDef;
    }

    private DuplicationEngine(Basedefiches basedefiches, DuplicationDef duplicationDef) {
        this.basedefiches = basedefiches;
        this.duplicationDef = duplicationDef;
    }

    public static DuplicationLog run(DuplicationDef duplicationDef, BasedefichesEditor basedefichesEditor) {
        DuplicationEngine duplicationEngine = new DuplicationEngine(basedefichesEditor, duplicationDef);
        duplicationEngine.innerRun(false);
        return duplicationEngine.internalDuplicationLog;
    }

    public static DuplicationLog test(DuplicationDef duplicationDef, Basedefiches basedefiches) {
        DuplicationEngine duplicationEngine = new DuplicationEngine(basedefiches, duplicationDef);
        duplicationEngine.innerRun(true);
        return duplicationEngine.internalDuplicationLog;
    }

    private void innerRun(boolean test) {
        originCorpusSubsetKey = duplicationDef.getOriginCorpusSubsetKey();
        Corpus origineCorpus = (Corpus) basedefiches.getSubset(originCorpusSubsetKey);
        if (origineCorpus == null) {
            addError(ORIGIN_UNKNOWNCORPUS_ERROR);
        }
        destinationCorpusSubsetKey = duplicationDef.getDestinationCorpusSubsetKey();
        Corpus destinationCorpus = (Corpus) basedefiches.getSubset(destinationCorpusSubsetKey);
        if (destinationCorpus == null) {
            addError(DESTINATION_UNKNOWNCORPUS_ERROR);
        }
        if (hasError()) {
            return;
        }
        Subset destinationMaster = destinationCorpus.getMasterSubset();
        CoupleCroisement[] coupleCroisementArray = getCoupleCroisementArray(destinationMaster);
        List<AbregeFiche> originAbregeFicheList = getOriginAbregeFicheList(origineCorpus);
        if (hasError()) {
            return;
        }
        int newIdcorpusStart = duplicationDef.getDestinationIdcorpusStart();
        List<Integer> destinationIdcorpusList = null;
        if (newIdcorpusStart == SAMEIDCORPUS) {
            if (originCorpusSubsetKey.equals(destinationCorpusSubsetKey)) {
                throw new IllegalStateException("SAMEIDCORPUS cannot be used when origin and destination corpus are the same");
            }
            destinationIdcorpusList = testSameIdcorpus(originAbregeFicheList, destinationCorpus);
        } else if (newIdcorpusStart > 0) {
            destinationIdcorpusList = testIdcorpusStart(newIdcorpusStart, destinationCorpus, originAbregeFicheList.size());
        }
        if (hasError()) {
            return;
        }
        if (test) {
            return;
        }
        CorpusEditor destinationEditor = basedefichesEditor.getCorpusEditor(destinationCorpusSubsetKey);
        SortedMap<Integer, Correspondance> correspondanceMap = new TreeMap<Integer, Correspondance>();
        int ficheCount = originAbregeFicheList.size();
        for (int i = 0; i < ficheCount; i++) {
            AbregeFiche origine = originAbregeFicheList.get(i);
            int idcorpus = -1;
            if (destinationIdcorpusList != null) {
                idcorpus = destinationIdcorpusList.get(i);
            }
            AbregeFiche destination = null;
            try {
                destination = destinationEditor.createFiche(idcorpus);
            } catch (ExistingIdException eie) {
                throw new ShouldNotOccurException("Test du doublon fait avant");
            } catch (NoMasterIdsubsetException eie) {
                throw new ShouldNotOccurException("Test du rattachement fait avant");
            }
            Correspondance correspondance = new Correspondance(origine, destination);
            correspondance.setDates(destinationEditor);
            correspondance.copyFiche(destinationEditor);
            int length = coupleCroisementArray.length;
            for (int j = 0; j < length; j++) {
                coupleCroisementArray[j].copy(correspondance);
            }
            correspondanceMap.put(origine.getTicket(), correspondance);
            addResult(origine.getIdcorpus(), destination.getIdcorpus());
        }
        Croisement croisementInitial = basedefiches.getCroisement(originCorpusSubsetKey, originCorpusSubsetKey);
        int poidsWithOrigin = duplicationDef.getPoidsWithOrigin();
        for (Map.Entry<Integer, Correspondance> entry : correspondanceMap.entrySet()) {
            int ticket = entry.getKey();
            Correspondance correspondance = entry.getValue();
            correspondance.convertCroisementInitial(croisementInitial, correspondanceMap.tailMap(ticket));
            if (poidsWithOrigin > 0) {
                correspondance.linkToOrigin(poidsWithOrigin);
            }
        }
    }

    private List<AbregeFiche> getOriginAbregeFicheList(Corpus origineCorpus) {
        List<AbregeFiche> originAbregeFicheList = new ArrayList<AbregeFiche>();
        int ficheCount = duplicationDef.getOriginIdcorpusCount();
        for (int i = 0; i < ficheCount; i++) {
            int originIdcorpus = duplicationDef.getOriginIdcorpus(i);
            AbregeFiche abregeFiche = origineCorpus.getAbregeFicheByIdcorpus(originIdcorpus);
            if (abregeFiche == null) {
                addIdcorpusError(MISSING_ORIGINFICHE_IDCORPUSERROR, originIdcorpus);
                continue;
            } else {
                originAbregeFicheList.add(abregeFiche);
            }
        }
        return originAbregeFicheList;
    }

    private CoupleCroisement[] getCoupleCroisementArray(Subset destinationMaster) {
        int corpusCount = basedefiches.getCorpusCount();
        int thesaurusCount = basedefiches.getThesaurusCount();
        int coupleLength = thesaurusCount + corpusCount + 1;
        SubsetKey destinationMasterKey = null;
        if (destinationMaster != null) {
            coupleLength = coupleLength - 1;
            destinationMasterKey = destinationMaster.getSubsetKey();
        }
        CoupleCroisement[] coupleCroisementArray = new CoupleCroisement[coupleLength];
        int p = 0;
        for (int i = 0; i < corpusCount; i++) {
            SubsetKey subsetKey = basedefiches.getCorpus(i).getSubsetKey();
            if ((destinationMasterKey == null) || (!subsetKey.equals(destinationMasterKey))) {
                coupleCroisementArray[p] = new CoupleCroisement(subsetKey);
                p++;
            }

        }
        for (int i = 0; i < thesaurusCount; i++) {
            SubsetKey subsetKey = basedefiches.getThesaurus(i).getSubsetKey();
            if ((destinationMasterKey == null) || (!subsetKey.equals(destinationMasterKey))) {
                coupleCroisementArray[p] = new CoupleCroisement(subsetKey);
                p++;
            }

        }
        coupleCroisementArray[p] = new CoupleCroisement(SubsetKey.ADDENDA);
        return coupleCroisementArray;
    }

    private List<Integer> testSameIdcorpus(List<AbregeFiche> originAbregeFicheList, Corpus destinationCorpus) {
        List<Integer> result = new ArrayList<Integer>();
        for (AbregeFiche originAbregeFiche : originAbregeFicheList) {
            int idcorpus = originAbregeFiche.getIdcorpus();
            AbregeFiche abregeFiche = destinationCorpus.getAbregeFicheByIdcorpus(idcorpus);
            if (abregeFiche != null) {
                addIdcorpusError(EXISTING_DESTINATIONFICHE_IDCORPUSERROR, idcorpus);
            }
            result.add(idcorpus);
        }
        return result;
    }

    private List<Integer> testIdcorpusStart(int newIdcorpusStart, Corpus destinationCorpus, int size) {
        List<Integer> result = new ArrayList<Integer>();
        int max = newIdcorpusStart + size;
        for (int idcorpus = newIdcorpusStart; idcorpus < max; idcorpus++) {
            AbregeFiche abregeFiche = destinationCorpus.getAbregeFicheByIdcorpus(idcorpus);
            if (abregeFiche != null) {
                addIdcorpusError(EXISTING_DESTINATIONFICHE_IDCORPUSERROR, idcorpus);
            }
            result.add(idcorpus);
        }
        return result;
    }

    private void addError(String errorKey) {
        internalDuplicationLog.addErrorKey(errorKey);
    }

    private boolean hasError() {
        return internalDuplicationLog.hasError();
    }

    private void addIdcorpusError(String errorKey, int idcorpus) {
        internalDuplicationLog.addIdcorpusError(errorKey, idcorpus);
    }

    private void addResult(int originIdcorpus, int destinationIdcorpus) {
        internalDuplicationLog.addResult(originIdcorpus, destinationIdcorpus);
    }


    private class Correspondance {

        private AbregeFiche origine;
        private AbregeFiche destination;

        private Correspondance(AbregeFiche origine, AbregeFiche destination) {
            this.origine = origine;
            this.destination = destination;
        }

        private void setDates(CorpusEditor destinationEditor) {
            destinationEditor.setDate(destination, origine.getCreationDate(), false);
            destinationEditor.setDate(destination, origine.getModificationDate(), true);
        }

        private void copyFiche(CorpusEditor destinationEditor) {
            Fiche fiche = origine.getCorpus().getFiche(origine);
            destinationEditor.saveFiche(destination, fiche);
        }

        private void convertCroisementInitial(Croisement croisementInitial, Map<Integer, Correspondance> correspondanceMap) {
            CroisementItem croisementItem = croisementInitial.getCroisementItem(originCorpusSubsetKey, origine.getTicket());
            if (croisementItem == null) {
                return;
            }
            int count = croisementItem.getCount();
            List<Integer> ticketList = new ArrayList<Integer>();
            List<Integer> poidsList = new ArrayList<Integer>();
            for (int i = 0; i < count; i++) {
                int ticket = croisementItem.getTicket(i);
                int poids = croisementItem.getPoids(i);
                Correspondance correspondance = correspondanceMap.get(ticket);
                if (correspondance == null) {
                    continue;
                }
                ticketList.add(correspondance.destination.getTicket());
                poidsList.add(poids);
            }
            if (ticketList.size() == 0) {
                return;
            }
            CroisementList croisementList = CroisementUtils.toCroisementList(destinationCorpusSubsetKey, ticketList, poidsList);
            croisementEditor.updateCroisement(destination, croisementList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
        }

        private void linkToOrigin(int poidsWithOrigin) {
            CroisementItem croisementItem = CroisementUtils.getSingletonCroisementItem(origine.getTicket(), poidsWithOrigin);
            CroisementList croisementList = CroisementUtils.toCroisementList(originCorpusSubsetKey, croisementItem);
            croisementEditor.updateCroisement(destination, croisementList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
        }

    }


    private class CoupleCroisement {

        SubsetKey subsetKey;
        Croisement origineCroisement;
        Croisement destinationCroisement;

        CoupleCroisement(SubsetKey subsetKey) {
            this.subsetKey = subsetKey;
            origineCroisement = basedefiches.getCroisement(originCorpusSubsetKey, subsetKey);
            destinationCroisement = basedefiches.getCroisement(destinationCorpusSubsetKey, subsetKey);
        }

        public void copy(Correspondance correspondance) {
            CroisementItem croisementItem = origineCroisement.getCroisementItem(originCorpusSubsetKey, correspondance.origine.getTicket());
            if (croisementItem != null) {
                croisementEditor.updateCroisement(correspondance.destination, CroisementUtils.toCroisementList(subsetKey, croisementItem), CroisementEditor.CLEAR_EXISTING);
            }
        }

    }


    private static class InternalDuplicationLog implements DuplicationLog {

        private List<InternalDuplicationError> errorList = new ArrayList<InternalDuplicationError>();
        private Map<String, InternalDuplicationError> errorMap = new HashMap<String, InternalDuplicationError>();
        private List<InternalDuplicationResult> resultList = new ArrayList<InternalDuplicationResult>();

        private boolean hasError() {
            return (!errorList.isEmpty());
        }

        public int getErrorCount() {
            return errorList.size();
        }

        public DuplicationLog.DuplicationError getError(int i) {
            return errorList.get(i);
        }

        public int getResultCount() {
            return resultList.size();
        }

        public DuplicationResult getResult(int i) {
            return resultList.get(i);
        }

        private void addErrorKey(String errorKey) {
            if (!errorMap.containsKey(errorKey)) {
                InternalDuplicationError internalDuplicationError = new InternalDuplicationError(errorKey);
                errorList.add(internalDuplicationError);
                errorMap.put(errorKey, internalDuplicationError);
            }
        }

        private void addIdcorpusError(String errorKey, int idcorpus) {
            InternalDuplicationError internalDuplicationError = errorMap.get(errorKey);
            if (internalDuplicationError == null) {
                internalDuplicationError = new InternalDuplicationError(errorKey);
                errorList.add(internalDuplicationError);
                errorMap.put(errorKey, internalDuplicationError);
            }
            internalDuplicationError.addIcorpus(idcorpus);
        }

        private void addResult(int originIdcorpus, int destinationIdcorpus) {
            resultList.add(new InternalDuplicationResult(originIdcorpus, destinationIdcorpus));
        }

    }


    private static class InternalDuplicationError implements DuplicationLog.DuplicationError {

        private String errorKey;
        private List<Integer> idcorpusList = new ArrayList<Integer>();

        private InternalDuplicationError(String errorKey) {
            this.errorKey = errorKey;
        }

        public String getErrorKey() {
            return errorKey;
        }

        public int getIdcorpusCount() {
            return idcorpusList.size();
        }

        public int getIdcorpus(int i) {
            return idcorpusList.get(i);
        }

        private void addIcorpus(int idcorpus) {
            idcorpusList.add(idcorpus);
        }

    }


    private static class InternalDuplicationResult implements DuplicationLog.DuplicationResult {

        private int originIdcorpus;
        private int destinationIdcorpus;

        private InternalDuplicationResult(int originIdcorpus, int destinationIdcorpus) {
            this.originIdcorpus = originIdcorpus;
            this.destinationIdcorpus = destinationIdcorpus;
        }

        public int getOriginIdcorpus() {
            return originIdcorpus;
        }

        public int getDestinationIdcorpus() {
            return destinationIdcorpus;
        }

    }

}
