/* BasedefichesLib_Tools - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.scrutariexport;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesConstants;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.FormatSourceKey;
import net.mapeadores.basedefiches.format.FormatSourceLibelleProvider;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatDefEngine;
import net.mapeadores.basedefiches.format.compute.BasedefichesFormatter;
import net.mapeadores.basedefiches.format.patterns.MotcleFormat;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportUtils;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDef;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusConstants;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.text.ValueResolver;
import net.scrutari.dataexport.ScrutariDataExportFactory;
import net.scrutari.dataexport.api.BaseMetadataExport;
import net.scrutari.dataexport.api.CorpusMetadataExport;
import net.scrutari.dataexport.api.FicheExport;
import net.scrutari.dataexport.api.MotcleExport;
import net.scrutari.dataexport.api.ScrutariDataExport;
import net.scrutari.dataexport.api.ThesaurusMetadataExport;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportEngine {

    private ScrutariExportDef scrutariExportDef;
    private int[] workingLangArray;
    private AbregeFicheSortedList abregeFicheSortedList;
    private FicheValueResolver fichePatternValueSet = new FicheValueResolver();
    private FormatSourceLibelleProvider formatSourceLibelleProvider;
    private Basedefiches basedefiches;
    private ScrutariDataExport scrutariDataExport;
    private ThesaurusEngine[] thesaurusEngineArray;
    private Map<SubsetKey, CorpusScrutariDef> corpusDefMap;
    private Map<SubsetKey, ThesaurusScrutariDef> thesaurusDefMap;

    public ScrutariExportEngine(Basedefiches basedefiches, Appendable appendable, int nbredetabulations, FormatSourceLibelleProvider formatSourceLibelleProvider) {
        this.basedefiches = basedefiches;
        this.formatSourceLibelleProvider = formatSourceLibelleProvider;
        scrutariDataExport = ScrutariDataExportFactory.newInstance(appendable, 0, false);
    }

    public void addBase(ScrutariExportDef scrutariExportDef, int[] workingLangArray) throws IOException {
        this.scrutariExportDef = scrutariExportDef;
        this.workingLangArray = workingLangArray;
        corpusDefMap = ScrutariExportUtils.toCorpusScrutariDefMap(scrutariExportDef);
        thesaurusDefMap = ScrutariExportUtils.toThesaurusScrutariDefMap(scrutariExportDef);
        FicheSelectOptions ficheSelectOptions = scrutariExportDef.getFicheSelectOptions();
        FicheSelector ficheSelector = new FicheSelector(basedefiches, workingLangArray[0]);
        ficheSelector.addFicheSelectOptions(ficheSelectOptions);
        abregeFicheSortedList = SelectionUtils.sortByIdcorpus(ficheSelector);
        initThesaurusEngineArray();
        BaseMetadataExport basemetaDataExport = scrutariDataExport.startExport();
        addBaseMetadata(basemetaDataExport);
        for (int i = 0; i < abregeFicheSortedList.getCorpusCount(); i++) {
            Corpus corpus = (Corpus) abregeFicheSortedList.getCorpus(i);
            CorpusEngine corpusEngine = new CorpusEngine(corpus);
            corpusEngine.run();
        }
        if (thesaurusEngineArray != null) {
            for (int i = 0; i < thesaurusEngineArray.length; i++) {
                thesaurusEngineArray[i].addThesaurus();
            }
        }
        scrutariDataExport.endExport();
    }

    private void initThesaurusEngineArray() {
        int thesaurusCount = basedefiches.getThesaurusCount();
        List<ThesaurusEngine> selectedList = new ArrayList<ThesaurusEngine>();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            ThesaurusScrutariDef thesaurusScrutariDef = thesaurusDefMap.get(thesaurus.getSubsetKey());
            if ((thesaurusScrutariDef != null) && (thesaurusScrutariDef.isSelected())) {
                selectedList.add(new ThesaurusEngine(thesaurus, thesaurusScrutariDef));
            }
        }
        int size = selectedList.size();
        thesaurusEngineArray = new ThesaurusEngine[size];
        for (int i = 0; i < size; i++) {
            thesaurusEngineArray[i] = (ThesaurusEngine) selectedList.get(i);
        }
    }

    private void addBaseMetadata(BaseMetadataExport basemetaDataExport) throws IOException {
        BasedefichesMetadata basedefichesMetadata = basedefiches.getBasedefichesMetadata();
        basemetaDataExport.setAuthorityUUID(basedefichesMetadata.getAuthorityUUID());
        String baseName = scrutariExportDef.getBaseName();
        if (baseName == null) {
            baseName = basedefichesMetadata.getBasedefichesName();
        }
        basemetaDataExport.setBaseName(baseName);
        basemetaDataExport.setBaseIcon(scrutariExportDef.getBaseIcon());
        addIntitule(basemetaDataExport, basedefichesMetadata, BasedefichesConstants.SHORT, BaseMetadataExport.INTITULE_SHORT);
        addIntitule(basemetaDataExport, basedefichesMetadata, BasedefichesConstants.LONG, BaseMetadataExport.INTITULE_LONG);
        int langUICount = scrutariExportDef.getLangUICount();
        if (langUICount > 0) {
            for (int i = 0; i < langUICount; i++) {
                basemetaDataExport.addLangUI(LangInteger.toString(scrutariExportDef.getLangUI(i)));
            }
        }

    }

    private void addIntitule(BaseMetadataExport basemetaDataExport, BasedefichesMetadata basedefichesMetadata, String name, int intituleType) {
        Intitule intitule = basedefichesMetadata.getIntituleByName(name);
        if (intitule != null) {
            int length = workingLangArray.length;
            boolean done = false;
            for (int i = 0; i < length; i++) {
                int langInteger = workingLangArray[i];
                String libelleString = intitule.getLibelleString(langInteger, false);
                if (libelleString != null) {
                    basemetaDataExport.setIntitule(intituleType, LangInteger.toString(langInteger), libelleString);
                    done = true;
                }
            }
            if (!done) {
                basemetaDataExport.setIntitule(intituleType, "und", intitule.getLibelleString(LangInteger.UNDETERMINED, true));
            }
        }
    }

    private int putTicket(int thesaurusIndex, int ticket) {
        return thesaurusEngineArray[thesaurusIndex].putTicket(ticket);
    }

    private Croisement[] getCroisementArray(SubsetKey corpusKey) {
        if (thesaurusEngineArray == null) {
            return null;
        }
        int length = thesaurusEngineArray.length;
        Croisement[] croisementArray = new Croisement[length];
        for (int i = 0; i < length; i++) {
            croisementArray[i] = basedefiches.getCroisement(corpusKey, thesaurusEngineArray[i].getThesaurus().getSubsetKey());
        }
        return croisementArray;
    }


    private class CorpusEngine {

        private Corpus corpus;
        private SubsetKey corpusKey;
        private SubsetItemList abregeFicheList;
        private ComplementCol[] complementColArray;
        private Croisement[] croisementArray;
        private int datationSourceType = 0;
        private FieldKey datationType = null;
        private AccoladePattern hrefPattern;
        private FieldKey geolocalisationFieldKey;
        private BasedefichesFormatter titreFormatter = null;
        private String titreFormatError = null;
        private BasedefichesFormatter soustitreFormatter = null;
        private String soustitreFormatError = null;
        private boolean multiLang = false;
        private Object multiLangKey = null;

        private CorpusEngine(Corpus corpus) {
            this.corpus = corpus;
            CorpusField geolocalisationField = corpus.getCorpusMetadata().getGeolocalisationField();
            if (geolocalisationField != null) {
                geolocalisationFieldKey = geolocalisationField.getFieldKey();
            }
            corpusKey = corpus.getSubsetKey();
            this.abregeFicheList = abregeFicheSortedList.getAbregeFicheListByCorpus(corpus);
            CorpusScrutariDef corpusScrutariDef = corpusDefMap.get(corpus.getSubsetKey());
            if (corpusScrutariDef == null) {
                corpusScrutariDef = ScrutariExportUtils.getDefaultScrutariCorpusDef(corpus.getSubsetKey());
            }
            FormatDefErrorHandler formatDefErrorHandler = new FormatDefErrorHandler();
            BasedefichesFormatDef titreFormatDef = corpusScrutariDef.getTitreFormatDef();
            if (titreFormatDef != null) {
                titreFormatter = BasedefichesFormatDefEngine.compute(corpus, titreFormatDef, formatDefErrorHandler);
                if (titreFormatter == null) {
                    titreFormatError = "#" + formatDefErrorHandler.errorKey + "#";
                }
            }
            BasedefichesFormatDef soustitreFormatDef = corpusScrutariDef.getSoustitreFormatDef();
            if (soustitreFormatDef != null) {
                soustitreFormatter = BasedefichesFormatDefEngine.compute(corpus, soustitreFormatDef, formatDefErrorHandler);
                if (soustitreFormatter == null) {
                    soustitreFormatError = "#" + formatDefErrorHandler.errorKey + "#";
                }
            }
            ColDefList complementColDefList = corpusScrutariDef.getComplementColDefList();
            int size = complementColDefList.getColDefCount();
            complementColArray = new ComplementCol[size];
            int p = 0;
            for (int i = 0; i < size; i++) {
                ColDef complementColDef = complementColDefList.getColDef(i);
                ComplementCol complementCol = toComplementCol(complementColDef, formatDefErrorHandler);
                if (complementCol != null) {
                    complementColArray[p] = complementCol;
                    p++;
                }
            }
            if (p < size) {
                ComplementCol[] temp = new ComplementCol[p];
                System.arraycopy(complementColArray, 0, temp, 0, p);
                complementColArray = temp;
            }
            croisementArray = getCroisementArray(corpus.getSubsetKey());
            initDatationType(corpusScrutariDef.getDateFormatSourceKey());
            AccoladePattern corpusFichePattern = corpusScrutariDef.getHrefPattern();
            if (corpusFichePattern != null) {
                this.hrefPattern = corpusFichePattern;
            } else {
                this.hrefPattern = scrutariExportDef.getFicheHrefPattern();
            }
            if (corpusScrutariDef.isMultiLang()) {
                multiLang = true;
                multiLangKey = (FieldKey) corpusScrutariDef.getMultiLangKey();
            }
        }

        private ComplementCol toComplementCol(ColDef colDef, FormatDefErrorHandler formatDefErrorHandler) {
            BasedefichesFormatter formatter = BasedefichesFormatDefEngine.compute(corpus, colDef.getBasedefichesFormatDef(), formatDefErrorHandler);
            if (formatter == null) {
                return null;
            }
            return new ComplementCol(corpus, colDef, formatter);
        }

        private void initDatationType(FormatSourceKey dateFormatSourceKey) {
            if (dateFormatSourceKey.isIncludeKey()) {
                IncludeKey includeKey = dateFormatSourceKey.getIncludeKey();
                if (includeKey.equals(IncludeKey.DATE_CREATION)) {
                    datationSourceType = 1;
                } else if (includeKey.equals(IncludeKey.DATE_MODIFICATION)) {
                    datationSourceType = 2;
                }
            } else if (dateFormatSourceKey.isFieldKey()) {
                CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(dateFormatSourceKey.getFieldKey());
                if (corpusField != null) {
                    if (corpusField.getFicheItemType() == CorpusField.DATATION_FIELD) {
                        datationSourceType = 3;
                        datationType = corpusField.getFieldKey();
                    }
                }
            }
        }

        private void run() {
            CorpusMetadataExport corpusMetadataExport = scrutariDataExport.newCorpus(corpus.getSubsetKey().getSubsetName());
            addCorpusMetadata(corpusMetadataExport, corpus.getCorpusMetadata());
            int ficheCount = abregeFicheList.getSubsetItemCount();
            FichePointeur fichePointeur = new FichePointeur(corpus);
            for (int i = 0; i < ficheCount; i++) {
                fichePointeur.setCurrentSubsetItem((AbregeFiche) abregeFicheList.getSubsetItem(i));
                if (multiLang) {
                    addMultiLangFiche(fichePointeur);
                } else {
                    addFiche(fichePointeur);
                }
            }
        }

        private void addCorpusMetadata(CorpusMetadataExport corpusMetadataExport, CorpusMetadata corpusMetadata) {
            addIntitule(corpusMetadataExport, corpusMetadata, CorpusConstants.CORPUS, CorpusMetadataExport.INTITULE_CORPUS);
            addIntitule(corpusMetadataExport, corpusMetadata, CorpusConstants.FICHE_EXISTING, CorpusMetadataExport.INTITULE_FICHE);
            int length = complementColArray.length;
            for (int i = 0; i < length; i++) {
                complementColArray[i].addIntitule(corpusMetadataExport);
            }
        }

        private void addIntitule(CorpusMetadataExport corpusMetadataExport, CorpusMetadata corpusMetadata, String name, int intituleType) {
            Intitule intitule = corpusMetadata.getIntituleByName(name);
            if (intitule != null) {
                int length = workingLangArray.length;
                boolean done = false;
                for (int i = 0; i < length; i++) {
                    int langInteger = workingLangArray[i];
                    String libelleString = intitule.getLibelleString(langInteger, false);
                    if (libelleString != null) {
                        corpusMetadataExport.setIntitule(intituleType, LangInteger.toString(langInteger), libelleString);
                        done = true;
                    }
                }
                if (!done) {
                    corpusMetadataExport.setIntitule(intituleType, "und", intitule.getLibelleString(LangInteger.UNDETERMINED, true));
                }
            }
        }

        private void addMultiLangFiche(FichePointeur fichePointeur) {
            AbregeFiche abregeFiche = (AbregeFiche) fichePointeur.getCurrentSubsetItem();
            fichePatternValueSet.setCurrent(abregeFiche);
            Set<LocalisationInfo> localisationInfoSet = getLocalisationInfoSet(fichePointeur);
            int size = localisationInfoSet.size();
            if (size == 0) {
                return;
            }
            int idcorpus = abregeFiche.getIdcorpus();
            String idcorpusString = String.valueOf(idcorpus);
            int date = getDate(fichePointeur);
            Geopoint geopoint = getGeoloc(fichePointeur);
            String[] idArray = new String[size];
            int p = 0;
            for (LocalisationInfo localisationInfo : localisationInfoSet) {
                fichePatternValueSet.setCurrent(localisationInfo);
                String langString = LangInteger.toString(localisationInfo.getWorkingLangInteger());
                String id = idcorpusString + "_" + langString;
                idArray[p] = id;
                FicheExport ficheExport = scrutariDataExport.newFiche(id);
                ficheExport.setTitre(getTitre(fichePointeur, localisationInfo));
                ficheExport.setSoustitre(getSoustitre(fichePointeur, localisationInfo));
                ficheExport.setLang(langString);
                if (date != 0) {
                    ficheExport.setDate(DateInteger.toString(date));
                }
                int complementLength = complementColArray.length;
                if (complementLength > 0) {
                    for (int i = 0; i < complementLength; i++) {
                        complementColArray[i].addComplement(i + 1, fichePointeur, ficheExport, localisationInfo);
                    }
                }
                if (hrefPattern != null) {
                    ficheExport.setHref(hrefPattern.format(fichePatternValueSet));
                }
                if (geopoint != null) {
                    ficheExport.setGeoloc(geopoint.getLatitude().toString(), geopoint.getLongitude().toString());
                }
            }
            if (croisementArray != null) {
                String corpusName = corpus.getSubsetKey().getSubsetName();
                int ficheTicket = abregeFiche.getTicket();
                int length = croisementArray.length;
                for (int i = 0; i < length; i++) {
                    CroisementItem croisementItem = croisementArray[i].getCroisementItem(corpusKey, ficheTicket);
                    int ciCount = croisementItem.getCount();
                    String thesaurusName = croisementArray[i].getSubsetCouple().getSubsetKey2().getSubsetName();
                    for (int j = 0; j < ciCount; j++) {
                        int poids = croisementItem.getPoids(j);
                        int idths = putTicket(i, croisementItem.getTicket(j));
                        String idthsString = String.valueOf(idths);
                        for (int k = 0; k < size; k++) {
                            scrutariDataExport.addIndexation(corpusName, idthsString, thesaurusName, idthsString, poids);
                        }
                    }
                }
            }
        }

        private Set<LocalisationInfo> getLocalisationInfoSet(FichePointeur fichePointeur) {
            Set<LocalisationInfo> result = new LinkedHashSet<LocalisationInfo>();
            if (multiLangKey instanceof FieldKey) {
                Object value = fichePointeur.getValue((FieldKey) multiLangKey);
                if (value instanceof Langue) {
                    result.add(new LocalisationInfo(((Langue) value).getLangInteger()));
                } else if (value instanceof FicheItemList) {
                    FicheItemList ficheItemList = (FicheItemList) value;
                    int count = ficheItemList.getFicheItemCount();
                    for (int i = 0; i < count; i++) {
                        FicheItem ficheItem = ficheItemList.getFicheItem(i);
                        if (ficheItem instanceof Langue) {
                            result.add(new LocalisationInfo(((Langue) ficheItem).getLangInteger()));
                        }
                    }
                }
            }
            return result;
        }

        private void addFiche(FichePointeur fichePointeur) {
            AbregeFiche abregeFiche = (AbregeFiche) fichePointeur.getCurrentSubsetItem();
            LocalisationInfo localisationInfo = new LocalisationInfo(abregeFiche.getLangInteger());
            fichePatternValueSet.setCurrent(abregeFiche);
            fichePatternValueSet.setCurrent(localisationInfo);
            int idcorpus = abregeFiche.getIdcorpus();
            String idcorpusString = String.valueOf(idcorpus);
            FicheExport ficheExport = scrutariDataExport.newFiche(idcorpusString);
            ficheExport.setTitre(getTitre(fichePointeur, localisationInfo));
            ficheExport.setSoustitre(getSoustitre(fichePointeur, localisationInfo));
            ficheExport.setLang(LangInteger.toString(abregeFiche.getLangInteger()));
            int date = getDate(fichePointeur);
            if (date != 0) {
                ficheExport.setDate(DateInteger.toString(date));
            }
            int complementLength = complementColArray.length;
            if (complementLength > 0) {
                for (int i = 0; i < complementLength; i++) {
                    complementColArray[i].addComplement(i + 1, fichePointeur, ficheExport, localisationInfo);
                }
            }
            if (hrefPattern != null) {
                ficheExport.setHref(hrefPattern.format(fichePatternValueSet));
            }
            Geopoint geopoint = getGeoloc(fichePointeur);
            if (geopoint != null) {
                ficheExport.setGeoloc(geopoint.getLatitude().toString(), geopoint.getLongitude().toString());
            }
            if (croisementArray != null) {
                String corpusName = corpus.getSubsetKey().getSubsetName();
                for (int i = 0; i < croisementArray.length; i++) {
                    CroisementItem croisementItem = croisementArray[i].getCroisementItem(corpusKey, abregeFiche.getTicket());
                    int ciCount = croisementItem.getCount();
                    String thesaurusName = croisementArray[i].getSubsetCouple().getSubsetKey2().getSubsetName();
                    for (int j = 0; j < ciCount; j++) {
                        int poids = croisementItem.getPoids(j);
                        int idths = putTicket(i, croisementItem.getTicket(j));
                        scrutariDataExport.addIndexation(corpusName, idcorpusString, thesaurusName, String.valueOf(idths), poids);
                    }
                }
            }
        }

        private String getTitre(FichePointeur fichePointeur, LocalisationInfo localisationInfo) {
            String titre = null;
            if (titreFormatter != null) {
                titre = titreFormatter.formatToString(fichePointeur, null, localisationInfo);
            } else if (titreFormatError != null) {
                titre = titreFormatError;
            } else {
                titre = (String) fichePointeur.getValue(FieldKey.TITRE);
            }
            return titre;
        }

        private String getSoustitre(FichePointeur fichePointeur, LocalisationInfo localisationInfo) {
            String soustitre = null;
            if (soustitreFormatter != null) {
                soustitre = soustitreFormatter.formatToString(fichePointeur, null, localisationInfo);
            } else if (soustitreFormatError != null) {
                soustitre = soustitreFormatError;
            } else {
                Para para = (Para) fichePointeur.getValue(FieldKey.SOUSTITRE);
                if (para != null) {
                    soustitre = para.contentToString();
                }
            }
            return soustitre;
        }

        private int getDate(FichePointeur fichePointeur) {
            int date = 0;
            AbregeFiche abregeFiche = (AbregeFiche) fichePointeur.getCurrentSubsetItem();
            if (datationSourceType == 1) {
                date = abregeFiche.getCreationDate();
            } else if (datationSourceType == 2) {
                date = abregeFiche.getModificationDate();
            } else if (datationSourceType == 3) {
                Object obj = fichePointeur.getValue(datationType);
                if (obj != null) {
                    if (obj instanceof Datation) {
                        date = ((Datation) obj).getDateInteger();
                    } else if (obj instanceof FicheItemList) {
                        FicheItemList fil = (FicheItemList) obj;
                        if (fil.getFicheItemCount() > 0) {
                            FicheItem fi = fil.getFicheItem(0);
                            if (fi instanceof Datation) {
                                date = ((Datation) fi).getDateInteger();
                            }
                        }
                    }
                }
            }
            return date;
        }

        private Geopoint getGeoloc(FichePointeur fichePointeur) {
            if (geolocalisationFieldKey != null) {
                FicheItem ficheItem = (FicheItem) fichePointeur.getValue(geolocalisationFieldKey);
                if ((ficheItem != null) && (ficheItem instanceof Geopoint)) {
                    return (Geopoint) ficheItem;
                }
            }
            return null;
        }

    }


    private class ThesaurusEngine {

        private Thesaurus thesaurus;
        private Set<Integer> idthsSet = new HashSet<Integer>();
        private LocalisationInfo[] localisationInfoArray;
        private MotcleFormat motcleFormat;

        private ThesaurusEngine(Thesaurus thesaurus, ThesaurusScrutariDef thesaurusScrutariDef) {
            this.thesaurus = thesaurus;
            int[] thesaurusLangArray = thesaurus.getThesaurusMetadata().getAuthorizedLangIntegerArray();
            if (thesaurusLangArray != null) {
                int length = thesaurusLangArray.length;
                localisationInfoArray = new LocalisationInfo[length];
                for (int i = 0; i < length; i++) {
                    localisationInfoArray[i] = new LocalisationInfo(thesaurusLangArray[i]);
                }
            }
            String motcleFormatPattern = thesaurusScrutariDef.getMotcleFormatPattern();
            boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
            if (motcleFormatPattern == null) {
                if (withIdalpha) {
                    motcleFormatPattern = "{idalpha} – {lib}";
                } else {
                    motcleFormatPattern = "{lib}";
                }
            }
            if (motcleFormatPattern != null) {
                try {
                    MotcleFormat mF = new MotcleFormat();
                    mF.setPattern(motcleFormatPattern);
                    this.motcleFormat = mF;
                } catch (java.text.ParseException pe) {
                }
            }
            if (motcleFormat == null) {
                motcleFormat = MotcleFormat.newDefaultInstance(withIdalpha);
            }
        }

        private int putTicket(int ticket) {
            SubsetItem subsetItem = thesaurus.getSubsetItemByTicket(ticket);
            int idths = subsetItem.getIdsubset();
            idthsSet.add(new Integer(idths));
            return idths;
        }

        private Thesaurus getThesaurus() {
            return thesaurus;
        }

        private void addThesaurus() {
            if (idthsSet.isEmpty()) {
                return;
            }
            ThesaurusMetadataExport thesaurusMetadataExport = scrutariDataExport.newThesaurus(thesaurus.getSubsetKey().getSubsetName());
            addThesaurusMetadata(thesaurusMetadataExport, thesaurus.getThesaurusMetadata());
            for (int i = 0; i < thesaurus.getTicketLength(); i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if ((motcle != null) && (idthsSet.contains(motcle.getIdths()))) {
                    addMotcle(motcle);
                }
            }
        }

        private void addMotcle(Motcle motcle) {
            MotcleExport motcleExport = scrutariDataExport.newMotcle(String.valueOf(motcle.getIdths()));
            if (localisationInfoArray != null) {
                int length = localisationInfoArray.length;
                for (int i = 0; i < length; i++) {
                    LocalisationInfo localisationInfo = localisationInfoArray[i];
                    String libelleString = motcleFormat.formatMotcle(motcle, 1, localisationInfo);
                    if (libelleString.length() > 0) {
                        motcleExport.setLibelle(LangInteger.toString(localisationInfo.getWorkingLangInteger()), libelleString);
                    }
                }
            } else {
                int langInteger = ThesaurusUtils.getDefaultPreferredLangInteger(motcle);
                LocalisationInfo localisationInfo = new LocalisationInfo(langInteger);
                String libelleString = motcleFormat.formatMotcle(motcle, 1, localisationInfo);
                if (libelleString.length() > 0) {
                    motcleExport.setLibelle(LangInteger.toString(langInteger), libelleString);
                }
            }
        }

        private void addThesaurusMetadata(ThesaurusMetadataExport thesaurusMetadataExport, ThesaurusMetadata thesaurusMetadata) {
            addIntitule(thesaurusMetadataExport, thesaurusMetadata, ThesaurusConstants.THESAURUS, ThesaurusMetadataExport.INTITULE_THESAURUS);
        }

        private void addIntitule(ThesaurusMetadataExport thesaurusMetadataExport, ThesaurusMetadata thesaurusMetadata, String name, int intituleType) {
            Intitule intitule = thesaurusMetadata.getIntituleByName(name);
            if (intitule != null) {
                int length = workingLangArray.length;
                boolean done = false;
                for (int i = 0; i < length; i++) {
                    int langInteger = workingLangArray[i];
                    String libelleString = intitule.getLibelleString(langInteger, false);
                    if (libelleString != null) {
                        thesaurusMetadataExport.setIntitule(intituleType, LangInteger.toString(langInteger), libelleString);
                        done = true;
                    }
                }
                if (!done) {
                    thesaurusMetadataExport.setIntitule(intituleType, "und", intitule.getLibelleString(LangInteger.UNDETERMINED, true));
                }
            }
        }

    }


    private static class FicheValueResolver implements ValueResolver {

        private AbregeFiche abregeFiche;
        private LocalisationInfo localisationInfo;

        private void setCurrent(AbregeFiche abregeFiche) {
            this.abregeFiche = abregeFiche;
        }

        private void setCurrent(LocalisationInfo localisationInfo) {
            this.localisationInfo = localisationInfo;
        }

        public String getValue(AccoladeArgument accoladeArgument) {
            String name = accoladeArgument.getName();
            if (name.equals("corpus")) {
                return abregeFiche.getCorpus().getSubsetKey().getSubsetName();
            }
            if (name.equals("idcorpus")) {
                return String.valueOf(abregeFiche.getIdcorpus());
            }
            if (name.equals("lang")) {
                return LangInteger.toString(localisationInfo.getWorkingLangInteger());
            }
            return null;
        }

    }


    private class ComplementCol {

        private Corpus corpus;
        private ColDef colDef;
        private BasedefichesFormatter formatter;

        private ComplementCol(Corpus corpus, ColDef colDef, BasedefichesFormatter formatter) {
            this.corpus = corpus;
            this.colDef = colDef;
            this.formatter = formatter;
        }

        private void addIntitule(CorpusMetadataExport corpusMetadataExport) {
            LibelleHolder libelleHolder = colDef.getCustomLibelle();
            if (libelleHolder == null) {
                BasedefichesFormatDef basedefichesFormatDef = colDef.getBasedefichesFormatDef();
                if (basedefichesFormatDef != null) {
                    libelleHolder = formatSourceLibelleProvider.getLibelleHolder(corpus, basedefichesFormatDef.getFormatSourceKey(0));
                }
            }
            int number = corpusMetadataExport.addComplement();
            boolean done = false;
            if (libelleHolder != null) {
                int length = workingLangArray.length;
                for (int i = 0; i < length; i++) {
                    int langInteger = workingLangArray[i];
                    Libelle libelle = libelleHolder.getLibelleByLang(langInteger);
                    if (libelle != null) {
                        corpusMetadataExport.setComplementIntitule(number, LangInteger.toString(langInteger), libelle.getLibelleString());
                        done = true;
                    }
                }
            }
            if (!done) {
                String undeterminedString = colDef.getColName();
                if (libelleHolder != null) {
                    undeterminedString = libelleHolder.getLibelleString(LangInteger.UNDETERMINED, true);
                } else {
                    undeterminedString = colDef.getColName();
                }
                corpusMetadataExport.setComplementIntitule(number, "und", undeterminedString);
            }
        }

        private void addComplement(int complementNumber, FichePointeur fichePointeur, FicheExport ficheExport, LocalisationInfo localisationInfo) {
            String s = formatter.formatToString(fichePointeur, null, localisationInfo);
            ficheExport.addComplement(complementNumber, s);
        }

    }


    private static class FormatDefErrorHandler implements BasedefichesFormatDefEngine.ErrorHandler {

        String errorKey = "error";

        public void unknownSubfieldKey(SubfieldKey subfieldKey) {
            errorKey = "unknownSubfieldKey" + " : " + subfieldKey.toString();
        }

        public void illegalSubfieldKey(SubfieldKey subfieldKey) {
            errorKey = "illegalSubfieldKey" + " : " + subfieldKey.toString();
        }

        public void wrongFormatPattern(String formatPattern) {
            errorKey = "wrongFormatPattern" + " : " + formatPattern;
        }

        public void unknownFieldKey(FieldKey fieldKey) {
            errorKey = "unknownFieldKey" + " : " + fieldKey.toString();
        }

        public void unknownSubsetKey(SubsetKey subsetKey) {
            errorKey = "unknownSubsetKey" + " : " + subsetKey.toString();
        }

        public void notASlaveCorpusKey(SubsetKey subsetKey) {
            errorKey = "notASlaveCorpusKey" + " : " + subsetKey.toString();
        }

        public void illegalFormatSourceKey(String keyString) {
            errorKey = "illegalFormatSourceKey" + " : " + keyString;
        }

        public void noTypePrefixFieldKey(FieldKey fieldKey) {
            errorKey = "noTypePrefixFieldKey" + " : " + fieldKey.toString();
        }

    }

}
