/* BasedefichesLib_Tools - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.tableexport;

import java.io.IOException;
import java.io.Writer;
import java.util.Date;
import java.util.List;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefTableWriter;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.format.FormatConstants;
import net.mapeadores.util.format.FormatUtils;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.sql.InsertWriter;


/**
 *
 * @author Vincent Calame
 */
public class SqlColDefTableWriter implements ColDefTableWriter {

    protected InsertWriter insertWriter;
    protected Writer writer;
    protected String tableName;
    private boolean notNull;

    public SqlColDefTableWriter(Writer writer, String tableName) {
        insertWriter = new InsertWriter(writer);
        this.tableName = tableName;
        this.writer = writer;
    }

    public void setNotNull(boolean notNull) {
        this.notNull = notNull;
    }

    public void startRow() {
        try {
            insertWriter.startInsert(tableName);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void addNullCell(ColDef colDef) {
        try {
            if (notNull) {
                short castType = FormatUtils.getCast(colDef.getBasedefichesFormatDef());
                switch (castType) {
                    case FormatConstants.INTEGER_CAST:
                        insertWriter.addInteger(0);
                        break;
                    case FormatConstants.DECIMAL_CAST:
                        insertWriter.addDecimal(new Decimal(0, (byte) 0, 0));
                        break;
                    default:
                        insertWriter.addText("");
                }
            } else {
                insertWriter.addNull();
            }
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void addIntegerCell(ColDef colDef, Long lg) {
        try {
            if (lg == null) {
                if (notNull) {
                    insertWriter.addInteger(0);
                } else {
                    insertWriter.addNull();
                }
            } else {
                insertWriter.addInteger(lg.longValue());
            }
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void addDecimalCell(ColDef colDef, Decimal decimal) {
        try {
            if (decimal == null) {
                if (notNull) {
                    insertWriter.addDecimal(new Decimal(0, (byte) 0, 0));
                } else {
                    insertWriter.addNull();
                }
            } else {
                insertWriter.addDecimal(decimal);
            }
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void addStringCell(ColDef colDef, String s) {
        try {
            if (s == null) {
                if (notNull) {
                    insertWriter.addText("");
                } else {
                    insertWriter.addNull();
                }
            } else {
                insertWriter.addText(s);
            }
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void addDateCell(ColDef colDef, Date date) {
        try {
            if (date == null) {
                if (notNull) {
                    date = new Date();
                } else {
                    insertWriter.addNull();
                    return;
                }
            }
            insertWriter.addText(DateInteger.toISOString(DateInteger.fromDate(date)));
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void addLineCell(ColDef colDef, List<String> list) {
        try {
            if ((list == null) || (list.size() == 0)) {
                if (notNull) {
                    insertWriter.addText("");
                } else {
                    insertWriter.addNull();
                }
            } else {
                StringBuilder buf = new StringBuilder();
                boolean premier = true;
                for (String s : list) {
                    if (premier) {
                        premier = false;
                    } else {
                        buf.append('\n');
                    }
                    buf.append(s);
                }
                insertWriter.addText(buf.toString());
            }
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public void endRow() {
        try {
            insertWriter.endInsertLine();
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

}
