/* BasedefichesLib_Tools - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.tools.thesaurus;

import java.util.TreeSet;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusIdalphaSort {

    private ThesaurusEditor thesaurusEditor;
    private boolean recursive = false;
    private boolean descending = false;
    private boolean ignoreCase = false;
    private boolean ignorePunctuation = false;

    public ThesaurusIdalphaSort(ThesaurusEditor thesaurusEditor) {
        this.thesaurusEditor = thesaurusEditor;
    }

    public void setRecursive(boolean recursive) {
        this.recursive = recursive;
    }

    public void setDescending(boolean descending) {
        this.descending = descending;
    }

    public void setIgnoreCase(boolean ignoreCase) {
        this.ignoreCase = ignoreCase;
    }

    public void setIgnorePunctuation(boolean ignorePunctuation) {
        this.ignorePunctuation = ignorePunctuation;
    }

    public void sort(Motcle motcle) {
        MotcleList children = (motcle != null) ? motcle.getChildList() : thesaurusEditor.getThesaurus().getFirstLevelList();
        int size = children.getMotcleCount();
        if (size == 0) {
            return;
        }
        if (size == 1) {
            if (recursive) {
                sort((Motcle) children.getMotcle(0));
            }
            return;
        }
        TreeSet<MotcleBundle> treeSet = new TreeSet<MotcleBundle>();
        for (int i = 0; i < size; i++) {
            Motcle child = (Motcle) children.getMotcle(i);
            treeSet.add(new MotcleBundle(child));
        }
        int p = (descending) ? (size - 1) : 0;
        for (MotcleBundle motcleBundle : treeSet) {
            Motcle child = motcleBundle.motcle;
            if (child.getChildIndex() != p) {
                thesaurusEditor.setChildIndex(child, p);
            }
            if (descending) {
                p--;
            } else {
                p++;
            }
        }
        if (recursive) {
            for (int i = 0; i < size; i++) {
                sort(children.getMotcle(i));
            }
        }
    }

    private String charcleanIdalpha(String idalpha) {
        if (ignorePunctuation) {
            int length = idalpha.length();
            StringBuilder buf = new StringBuilder(length);
            for (int i = 0; i < length; i++) {
                char carac = idalpha.charAt(i);
                if (Character.isLetterOrDigit(carac)) {
                    if (ignoreCase) {
                        carac = Character.toUpperCase(carac);
                    }
                    buf.append(carac);
                }
            }
            return buf.toString();
        } else if (ignoreCase) {
            return idalpha.toUpperCase();
        } else {
            return idalpha;
        }
    }

    private String cleanIdalpha(String idalpha) {
        String result = charcleanIdalpha(idalpha);
        try {
            int itg = Integer.parseInt(result);
            if (itg < 0) {
                return result;
            }
            StringBuilder buf = new StringBuilder(16);
            int length = result.length();
            for (int i = 0; i < (11 - length); i++) {
                buf.append('0');
            }
            buf.append(result);
            return buf.toString();
        } catch (NumberFormatException nfe) {
            return result;
        }
    }


    private class MotcleBundle implements Comparable {

        String cleanedIdalpha;
        String idalpha;
        Motcle motcle;

        MotcleBundle(Motcle motcle) {
            this.motcle = motcle;
            this.idalpha = motcle.getIdalpha();
            this.cleanedIdalpha = cleanIdalpha(idalpha);
        }

        public int hashCode() {
            return idalpha.hashCode();
        }

        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (!(obj instanceof MotcleBundle)) {
                return false;
            }
            return ((MotcleBundle) obj).idalpha.equals(this.idalpha);
        }

        public int compareTo(Object obj) {
            MotcleBundle other = (MotcleBundle) obj;
            int val = cleanedIdalpha.compareTo(other.cleanedIdalpha);
            if (val != 0) {
                return val;
            }
            return idalpha.compareTo(other.idalpha);
        }

    }

}

