/* BasedefichesLib_Xml - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.AddendaEditor;
import net.mapeadores.basedefiches.association.ExistingBasenameException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class AddendaDOMReader extends BasedefichesDOMReader {

    private AddendaEditor addendaEditor;
    private AddendaXmlErrorHandler errorHandler;

    public AddendaDOMReader(AddendaEditor addendaEditor, AddendaXmlErrorHandler errorHandler) {
        super(errorHandler);
        this.addendaEditor = addendaEditor;
        this.errorHandler = errorHandler;
    }

    public void fillAddenda(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("document")) {
                    updateDocument(el);
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private void updateDocument(Element element_xml) {
        String idaddendaString = element_xml.getAttribute("idaddenda");
        if (idaddendaString.length() == 0) {
            idaddendaString = element_xml.getAttribute("idsubset");
        }
        try {
            int idaddenda = Integer.parseInt(idaddendaString);
            String basename = element_xml.getAttribute("basename");
            if (basename.length() > 0) {
                Document document = (Document) addendaEditor.getAddenda().getSubsetItemById(idaddenda);
                if (document != null) {
                    try {
                        addendaEditor.setBasename(document, basename);
                    } catch (ExistingBasenameException ebe) {
                        errorHandler.basenameDoublonError(basename);
                    } catch (ParseException pe) {
                        errorHandler.wrongBasenameError(basename);
                    }
                }
            }
        } catch (NumberFormatException nfe) {
            errorHandler.wrongIdaddendaError(idaddendaString);
        }
    }

}
