/* BasedefichesLib_Xml - Copyright (c)2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.ExistingNameException;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class AlbumDOMReader extends BasedefichesDOMReader {

    private AlbumEditor albumEditor;
    private AlbumMetadataEditor albumMetadataEditor;

    public AlbumDOMReader(BasedefichesXmlErrorHandler basedefichesXmlErrorHandler) {
        super(basedefichesXmlErrorHandler);
    }

    public void setAlbumEditor(AlbumEditor albumEditor) {
        this.albumEditor = albumEditor;
        this.albumMetadataEditor = albumEditor.getAlbumMetadataEditor();
    }

    public void fillAlbum(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("metadata")) {
                    fillMetadata(element);
                } else {
                    basedefichesXmlErrorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private void fillMetadata(Element element_xml) {
        Map<String, String> usageMap = new HashMap<String, String>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("intitule")) {
                    addIntitule(el, albumMetadataEditor);
                } else if (tagname.equals("album-dim")) {
                    addAlbumDim(el);
                } else if (tagname.equals("dim-usage")) {
                    appendDimUsage(usageMap, el);
                } else {
                    basedefichesXmlErrorHandler.unknownTagWarning(tagname);
                }
            }
        }
        for (Map.Entry<String, String> mapEntry : usageMap.entrySet()) {
            String usageName = mapEntry.getKey();
            String dimName = mapEntry.getValue();
            AlbumDim albumDim = albumMetadataEditor.getAlbumMetadata().getAlbumDimByName(dimName);
            if (albumDim != null) {
                albumMetadataEditor.setAlbumDimForUsage(usageName, albumDim);
            }
        }
    }

    private void appendDimUsage(Map<String, String> usageMap, Element element_xml) {
        String tagName = "dim-usage";
        String usageName = element_xml.getAttribute("usage-name");
        if (usageName.length() == 0) {
            basedefichesXmlErrorHandler.emptyAttributeError(tagName, "usage-name");
            return;
        }
        String dimName = element_xml.getAttribute("dim-name");
        if (dimName.length() == 0) {
            basedefichesXmlErrorHandler.emptyAttributeError(tagName, "dim-name");
            return;
        }
        usageMap.put(usageName, dimName);
    }

    private void addAlbumDim(Element element_xml) {
        String tagName = "album-dim";
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            basedefichesXmlErrorHandler.emptyAttributeError(tagName, "name");
            return;
        }
        tagName = tagName + " name=\"" + name + "\"";
        String typeString = element_xml.getAttribute("type");
        if (typeString.length() == 0) {
            basedefichesXmlErrorHandler.emptyAttributeError(tagName, "type");
            return;
        }
        short type = 0;
        try {
            type = AlbumUtils.dimTypeToShort(typeString);
        } catch (IllegalArgumentException iae) {
            basedefichesXmlErrorHandler.wrongTypeAttributeError(tagName, typeString);
            return;
        }
        int width = 0;
        int height = 0;
        if (AlbumUtils.needWidth(type)) {
            width = getInt(element_xml, tagName, "width");
            if (width == -1) {
                return;
            }
        }
        if (AlbumUtils.needHeight(type)) {
            height = getInt(element_xml, tagName, "height");
            if (height == -1) {
                return;
            }
        }
        AlbumDim albumDim = null;
        try {
            albumDim = albumMetadataEditor.createAlbumDim(name, type);
        } catch (ExistingNameException ene) {
            return;
        } catch (ParseException pe) {
            return;
        }
        albumMetadataEditor.setDim(albumDim, width, height);

    }

    private int getInt(Element element_xml, String tagName, String attributeName) {
        String intString = element_xml.getAttribute(attributeName);
        int result = 0;
        if (intString.length() == 0) {
            basedefichesXmlErrorHandler.emptyAttributeError(tagName, attributeName);
            return -1;
        }
        try {
            result = Integer.parseInt(intString);
        } catch (NumberFormatException nfe) {
            basedefichesXmlErrorHandler.wrongIntegerAttributeError(tagName, attributeName, intString);
            return -1;
        }
        if (result < 1) {
            basedefichesXmlErrorHandler.wrongIntegerAttributeError(tagName, attributeName, intString);
            return -1;
        }
        return result;
    }

}
