/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import net.mapeadores.basedefiches.BasedefichesAttributes;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeBuilder;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;

/**
 *
 * @author Vincent Calame
 */
public class AttributeDOMReader {

    private BasedefichesEditor basedefichesEditor;
    private BasedefichesXmlErrorHandler basedefichesXmlErrorHandler;

    public AttributeDOMReader(BasedefichesEditor basedefichesEditor, BasedefichesXmlErrorHandler basedefichesXmlErrorHandler) {
        this.basedefichesEditor = basedefichesEditor;
        this.basedefichesXmlErrorHandler = basedefichesXmlErrorHandler;
    }

    public void readAttrElement(Element element, AttributeHolder attributeHolder) {
        String localKey = element.getAttribute("key").trim();
        if (localKey.length() == 0) {
            basedefichesXmlErrorHandler.emptyAttributeError("key", "attr");
            return;
        }
        String nameSpace = element.getAttribute("ns").trim();
        AttributeKey attributeKey = BasedefichesAttributes.toAttributeKey(nameSpace, localKey);
        AttributeHandler attributeHandler = new AttributeHandler(attributeKey);
        DOMUtils.readChildren(element, attributeHandler);
        Attribute attribute = attributeHandler.toAttribute();
        if (attribute != null) {
            basedefichesEditor.putAttribute(attributeHolder, attribute);
        }
    }


    private class AttributeHandler implements ElementHandler {

        private AttributeBuilder attributeBuilder;

        public AttributeHandler(AttributeKey attributeKey) {
            this.attributeBuilder = new AttributeBuilder(attributeKey);
        }

        public Attribute toAttribute() {
            return attributeBuilder.toAttribute();
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("val")) {
                CleanedString cs = XMLUtils.toCleanedString(element);
                if (cs != null) {
                    attributeBuilder.addValue(cs);
                } else {
                    basedefichesXmlErrorHandler.emptyElementError("val");
                }
            } else {
                basedefichesXmlErrorHandler.unknownTagWarning(tagName);
            }
        }

    }


}
