/* BasedefichesLib_Xml - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import java.io.Writer;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class BeanToXML {

    private BeanToXML() {
    }

    public static void addXML(XMLWriter xmlWriter, PersonneBean personneBean) throws IOException {
        xmlWriter.startOpenTag("personne-bean");
        if (personneBean.isNomAvant()) {
            xmlWriter.addAttribute("nom-avant", "true");
        }
        xmlWriter.addAttribute("nom", personneBean.getNom());
        xmlWriter.addAttribute("prenom", personneBean.getPrenom());
        xmlWriter.addAttribute("original", personneBean.getOriginal());
        xmlWriter.closeEmptyTag();
    }

    public static void addXML(XMLWriter xmlWriter, CourrielBean courrielBean) throws IOException {
        xmlWriter.startOpenTag("courriel-bean");
        xmlWriter.addAttribute("addr-spec", courrielBean.getAddrSpec());
        xmlWriter.addAttribute("nom-complet", courrielBean.getNomComplet());
        xmlWriter.closeEmptyTag();
    }

    public static void writeXML(Writer writer, PersonneBean personneBean, int nbretabulations) throws IOException {
        XMLUtils.writeTabs(writer, nbretabulations);
        writer.write("<personne-bean");
        if (personneBean.isNomAvant()) {
            writer.write(" nom-avant=\"true\"");
        }
        writer.write(" nom=\"");
        XMLUtils.writeEscape(personneBean.getNom(), writer);
        writer.write("\" prenom=\"");
        XMLUtils.writeEscape(personneBean.getPrenom(), writer);
        writer.write("\" original=\"");
        XMLUtils.writeEscape(personneBean.getOriginal(), writer);
        writer.write("\"/>");
    }

    public static void writeXML(Writer writer, CourrielBean courrielBean, int nbretabulations) throws IOException {
        XMLUtils.writeTabs(writer, nbretabulations);
        writer.write("<courriel-bean");
        writer.write(" addr-spec=\"");
        XMLUtils.writeEscape(courrielBean.getAddrSpec(), writer);
        writer.write("\" nom-complet=\"");
        XMLUtils.writeEscape(courrielBean.getNomComplet(), writer);
        writer.write("\"/>");
    }

    public static CourrielBean toCourrielBean(Element element) {
        CourrielBean courrielBean = new CourrielBean();
        courrielBean.setAddrSpec(element.getAttribute("addr-spec"));
        courrielBean.setNomComplet(element.getAttribute("nom-complet"));
        return courrielBean;
    }

    public static PersonneBean toPersonneBean(Element element) {
        PersonneBean personneBean = new PersonneBean();
        personneBean.setNom(element.getAttribute("nom"));
        personneBean.setPrenom(element.getAttribute("prenom"));
        personneBean.setOriginal(element.getAttribute("original"));
        String nomAvant = element.getAttribute("nom-avant");
        if (nomAvant.equals("true")) {
            personneBean.setNomAvant(true);
        }
        //deprécié
        String strg_style = element.getAttribute("style");
        if (strg_style.length() > 0) {
            personneBean.setNomAvant(true);
        }
        return personneBean;
    }

}
