/* BasedefichesLib_Xml - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.filters.SubsetItemFilter;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ChronoXMLWriter extends UtilXMLWriter {

    public ChronoXMLWriter() {
    }

    public void appendChrono(Corpus corpus) throws IOException {
        appendChrono(corpus, null);
    }

    public void appendChrono(Corpus corpus, SubsetItemFilter subsetItemFilter) throws IOException {
        openTag("chrono");
        int ticketLength = corpus.getTicketLength();
        for (int i = 0; i < ticketLength; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
            if (testSubsetItem(abregeFiche, subsetItemFilter)) {
                int creationDate = abregeFiche.getCreationDate();
                int modificationDate = abregeFiche.getModificationDate();
                appendCh(abregeFiche.getIdcorpus(), creationDate, modificationDate);
            }
        }
        closeTag("chrono");
    }

    private boolean testSubsetItem(SubsetItem subsetItem, SubsetItemFilter subsetItemFilter) {
        if (subsetItem == null) {
            return false;
        }
        if (subsetItemFilter == null) {
            return true;
        }
        return subsetItemFilter.containsSubsetItem(subsetItem.getIdsubset());
    }

    private void appendCh(int idcorpus, int creationDate, int modificationDate) throws IOException {
        if (creationDate == 0) {
            return;
        }
        startOpenTag("ch");
        addAttribute("id", idcorpus);
        addAttribute("c", DateInteger.toString(creationDate));
        if (modificationDate != creationDate) {
            addAttribute("m", DateInteger.toString(modificationDate));
        }
        closeEmptyTag();
    }

}
