/* BasedefichesLib_Xml - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.ExistingFieldKeyException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionException;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsConstants;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatDefParser;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.CurrencyIntegerException;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusMetadataDOMReader extends BasedefichesDOMReader implements FieldOptionsConstants {

    private CorpusMetadataEditor corpusMetadataEditor;
    private HashMap ficheItemMap = new HashMap();
    private CorpusMetadataXmlErrorHandler errorHandler;

    public CorpusMetadataDOMReader(CorpusMetadataEditor corpusMetadataEditor, CorpusMetadataXmlErrorHandler errorHandler) {
        super(errorHandler);
        this.corpusMetadataEditor = corpusMetadataEditor;
        this.errorHandler = errorHandler;
    }

    public void fillCorpusMetadata(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        Element fieldsElement = null;
        Element optionsElement = null;
        Element l10nElement = null;
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if ((fieldsElement == null) && (tagname.equals("fields"))) {
                    fieldsElement = el;

                } else if ((optionsElement == null) && (tagname.equals("options"))) {
                    optionsElement = el;
                } else if ((l10nElement == null) && (tagname.equals("l10n"))) {
                    l10nElement = el;
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
        /*
         * Vieille compatibilité (avant 0.9.8.4)
         */
        if (optionsElement != null) {
            checkFicheItemType(optionsElement);
        }
        createFields(fieldsElement);
        if (optionsElement != null) {
            initOptions(optionsElement);
        }
        if (l10nElement != null) {
            initL10n(l10nElement);
        }
    }

    private void createFields(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                FieldKey fieldKey = null;
                short ficheItemType = CorpusField.NO_FICHEITEM_FIELD;
                if (tagname.equals("soustitre")) {
                    fieldKey = FieldKey.SOUSTITRE;
                } else if (tagname.equals("texte")) {
                    fieldKey = FieldKey.getSectionFieldKey("texte");
                } else if ((tagname.equals("section")) || (tagname.equals("annexe"))) {
                    String type = getType(el);
                    if (type != null) {
                        fieldKey = FieldKey.getSectionFieldKey(type);
                    }
                } else if (tagname.equals("propriete")) {
                    String type = getType(el);
                    if (type != null) {
                        fieldKey = FieldKey.getProprieteFieldKey(type);
                        ficheItemType = getFicheItemType(fieldKey, el);
                    }
                } else if (tagname.equals("information")) {
                    String type = getType(el);
                    if (type != null) {
                        fieldKey = FieldKey.getInformationFieldKey(type);
                        ficheItemType = getFicheItemType(fieldKey, el);
                    }
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
                if (fieldKey != null) {
                    try {
                        corpusMetadataEditor.createCorpusField(fieldKey, ficheItemType);
                    } catch (ExistingFieldKeyException efke) {
                        errorHandler.fieldKeyDoublonError(fieldKey);
                    }
                }
            }
        }
    }

    private String getType(Element element_xml) {
        String type = element_xml.getAttribute("type");
        if (type.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "type");
            return null;
        }
        if (StringUtils.isTechnicalName(type, false)) {
            return type;
        } else {
            errorHandler.wrongTypeAttributeError(element_xml.getTagName(), type);
            return null;
        }
    }

    private short getFicheItemType(FieldKey fieldKey, Element element_xml) {
        String ficheItem = element_xml.getAttribute("fiche-item");
        if (ficheItem.length() > 0) {
            try {
                return CorpusMetadataUtils.ficheItemTypeToShort(ficheItem);
            } catch (IllegalArgumentException iae) {
                errorHandler.wrongFicheItemTypeError(fieldKey, ficheItem);
                return CorpusField.ITEM_FIELD;
            }
        }
        /*
         * Vieille compatibilité (avant 0.9.8.4)
         */
        String croisement = element_xml.getAttribute("croisement");
        if (croisement.length() > 0) {
            if (croisement.equals("sphere")) {
                return CorpusField.PERSONNE_FIELD;
            }
            if (croisement.equals("pays")) {
                return CorpusField.PAYS_FIELD;
            }
            if (croisement.equals("langue")) {
                return CorpusField.LANGUE_FIELD;
            }
            if (croisement.equals("datation")) {
                return CorpusField.DATATION_FIELD;
            }
            return CorpusField.ITEM_FIELD;
        }
        /*
         * Vieille compatibilité (avant 0.9.8.4)
         */
        Short itg = (Short) ficheItemMap.get(fieldKey);
        if (itg != null) {
            return itg.shortValue();
        }
        return CorpusField.ITEM_FIELD;
    }

    /*
     * Vieille compatibilité (avant 0.9.8.4)
     */
    private void checkFicheItemType(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("option")) {
                    String name = el.getAttribute("name");
                    if (name.equals("acceptedFicheItemMask")) {
                        try {
                            FieldKey fieldKey = FieldKey.parse(el.getAttribute("field-key"));
                            short fit = getFicheItemType(el.getAttribute("value"));
                            ficheItemMap.put(fieldKey, new Short(fit));
                        } catch (ParseException pe) {
                        }
                    }
                }
            }
        }
    }

    /*
     * Vieille compatibilité (avant 0.9.8.4)
     */
    private short getFicheItemType(String s) {
        if (s.contains("personne")) {
            return CorpusField.PERSONNE_FIELD;
        }
        if (s.contains("langue")) {
            return CorpusField.LANGUE_FIELD;
        }
        if (s.contains("pays")) {
            return CorpusField.PAYS_FIELD;
        }
        if (s.contains("datation")) {
            return CorpusField.DATATION_FIELD;
        }
        if (s.contains("courriel")) {
            return CorpusField.COURRIEL_FIELD;
        }
        if (s.contains("link")) {
            return CorpusField.LINK_FIELD;
        }
        return CorpusField.ITEM_FIELD;
    }

    private void initOptions(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("option")) {
                    String name = el.getAttribute("name");
                    if (name.length() == 0) {
                        errorHandler.emptyAttributeError("option", "name");
                        continue;
                    }
                    CorpusField corpusField = getCorpusField(el);
                    if (corpusField == null) {
                        continue;
                    }
                    if (!FieldOptionsUtils.testFieldOption(name, corpusField)) {
                        errorHandler.norelevantOptionWarning(corpusField.getFieldKey(), name);
                        continue;
                    }
                    if (name.equals(INFORMATIONDISPLAY_OPTION)) {
                        initInformationDisplayOption(corpusField, el);
                    } else if (name.equals(DEFAULTSPHEREKEY_OPTION)) {
                        initDefaultSphereKeyOption(corpusField, el);
                    } else if ((name.equals("mainPersonne") || (name.equals(SUBFIELDDISPLAY_OPTION)))) {
                        initSubfieldDisplayOption(corpusField, el);
                    } else if (name.equals(TITREGENERATIONFORMAT_OPTION)) {
                        initTitreGenerationFormatOption(corpusField, el);
                    } else if (name.equals(CURRENCYARRAY_OPTION)) {
                        initCurrencyArrayOption(corpusField, el);
                    } else if (name.equals(GEOLOCALISATIONFIELD_OPTION)) {
                        initGeolocalisationField(corpusField);
                    } else if (name.equals(ADDRESSFIELDARRAY_OPTION)) {
                        initAddressFieldArray(corpusField, el);
                    } else {
                        initStringOption(corpusField, el, name);
                    }
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private void initInformationDisplayOption(CorpusField corpusField, Element el) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if (!fieldKey.isInformationFieldKey()) {
            errorHandler.norelevantOptionWarning(fieldKey, INFORMATIONDISPLAY_OPTION);
            return;
        }
        String value = el.getAttribute("value");
        short infdisp = -999;
        try {
            infdisp = FieldOptionsUtils.informationDisplayToShort(value);
        } catch (IllegalArgumentException iae) {
            errorHandler.wrongOptionValueError(fieldKey, INFORMATIONDISPLAY_OPTION, value);
        }
        if (infdisp != -999) {
            try {
                corpusMetadataEditor.setFieldOption(corpusField, INFORMATIONDISPLAY_OPTION, new Short(infdisp));
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }
    }

    private void initGeolocalisationField(CorpusField corpusField) {
        try {
            corpusMetadataEditor.setGeolocalisationField(corpusField);
        } catch (IllegalArgumentException iae) {
            errorHandler.norelevantOptionWarning(corpusField.getFieldKey(), GEOLOCALISATIONFIELD_OPTION);
            return;
        }
    }

    private void initDefaultSphereKeyOption(CorpusField corpusField, Element el) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if (!fieldKey.isFicheItemField(0)) {
            errorHandler.norelevantOptionWarning(fieldKey, DEFAULTSPHEREKEY_OPTION);
            return;
        }
        String value = el.getAttribute("value");
        SubsetKey sphereKey = null;
        if (!value.equals("null")) {
            try {
                sphereKey = SubsetKey.parse(value);

            } catch (java.text.ParseException pe) {
                errorHandler.wrongOptionValueError(fieldKey, DEFAULTSPHEREKEY_OPTION, value);
                return;
            }
            if (!sphereKey.isSphereSubset()) {
                errorHandler.wrongOptionValueError(fieldKey, DEFAULTSPHEREKEY_OPTION, value);
                return;
            }

        }
        try {
            corpusMetadataEditor.setFieldOption(corpusField, DEFAULTSPHEREKEY_OPTION, sphereKey);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
    }

    private void initSubfieldDisplayOption(CorpusField corpusField, Element el) {
        String value = el.getAttribute("value");
        if (value.equals("1")) {
            try {
                corpusMetadataEditor.setFieldOption(corpusField, SUBFIELDDISPLAY_OPTION, Boolean.TRUE);
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }
    }

    private void initTitreGenerationFormatOption(CorpusField corpusField, Element el) {
        FieldKey fieldKey = corpusField.getFieldKey();
        if (!fieldKey.equals(FieldKey.TITRE)) {
            errorHandler.norelevantOptionWarning(fieldKey, TITREGENERATIONFORMAT_OPTION);
            return;
        }
        String[] array = getValueArray(fieldKey, TITREGENERATIONFORMAT_OPTION, el);
        if (array == null) {
            return;
        }
        BasedefichesFormatDefErrorHandler ffeh = new BasedefichesFormatDefErrorHandler();
        BasedefichesFormatDef basedefichesFormatDef = BasedefichesFormatDefParser.parse(array, ffeh);
        if (basedefichesFormatDef != null) {
            try {
                corpusMetadataEditor.setFieldOption(corpusField, TITREGENERATIONFORMAT_OPTION, basedefichesFormatDef);
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }
    }

    private void initAddressFieldArray(CorpusField corpusField, Element el) {
        FieldKey fieldKey = corpusField.getFieldKey();
        String[] array = getValueArray(fieldKey, ADDRESSFIELDARRAY_OPTION, el);
        if (array == null) {
            return;
        }
        List<FieldKey> fieldKeyList = new ArrayList<FieldKey>();
        int length = array.length;
        for (int i = 0; i < length; i++) {
            boolean done = false;
            try {
                FieldKey otherFieldKey = FieldKey.parse(array[i]);
                CorpusField otherCorpusField = ((CorpusMetadata) corpusMetadataEditor.getMetadata()).getCorpusFieldByFieldKey(otherFieldKey);
                if (otherCorpusField != null) {
                    fieldKeyList.add(otherFieldKey);
                    done = true;
                } else {
                    done = false;
                }
            } catch (ParseException pe) {
                done = false;
            }
            if (!done) {
                errorHandler.wrongOptionValueError(fieldKey, ADDRESSFIELDARRAY_OPTION, array[i]);
            }
        }
        try {
            corpusMetadataEditor.setFieldOption(corpusField, ADDRESSFIELDARRAY_OPTION, fieldKeyList.toArray(new FieldKey[fieldKeyList.size()]));
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
    }

    private void initStringOption(CorpusField corpusField, Element el, String name) {
        if (name.equals("imageSource") || name.equals("albumKey")) {
            return;
        }
        String value = el.getAttribute("value");
        if (value.length() > 0) {
            try {
                corpusMetadataEditor.setFieldOption(corpusField, name, value);
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }
    }

    private void initCurrencyArrayOption(CorpusField corpusField, Element el) {
        FieldKey fieldKey = corpusField.getFieldKey();
        String[] array = getValueArray(fieldKey, CURRENCYARRAY_OPTION, el);
        if (array == null) {
            return;
        }
        int length = array.length;
        int[] currencyArray = new int[length];
        int p = 0;
        for (int i = 0; i < length; i++) {
            try {
                int currencyInteger = CurrencyInteger.parse(array[i]);
                currencyArray[p] = currencyInteger;
                p++;
            } catch (CurrencyIntegerException cie) {
                errorHandler.wrongOptionValueError(fieldKey, CURRENCYARRAY_OPTION, array[i]);
            }
        }
        if (p == 0) {
            return;
        }
        if (p < length) {
            int[] tempArray = new int[p];
            System.arraycopy(currencyArray, 0, tempArray, 0, p);
            currencyArray = tempArray;
        }
        try {
            corpusMetadataEditor.setFieldOption(corpusField, CURRENCYARRAY_OPTION, currencyArray);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
    }

    private String[] getValueArray(FieldKey fieldKey, String optionName, Element element_xml) {
        List<String> valeurList = new ArrayList<String>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("value")) {
                    valeurList.add(XMLUtils.getData(el));
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
        int size = valeurList.size();
        if (size == 0) {
            errorHandler.missingValueElementsError(fieldKey, optionName);
            return null;
        }
        return valeurList.toArray(new String[size]);
    }

    private void initL10n(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("field")) {
                    CorpusField corpusField = getCorpusField(el);
                    if (corpusField == null) {
                        continue;
                    }
                    NodeList libelleList = el.getChildNodes();
                    for (int j = 0; j < libelleList.getLength(); j++) {
                        Node nd2 = libelleList.item(j);
                        if ((nd2.getNodeType() == Node.ELEMENT_NODE) && (((Element) nd2).getTagName().equals("lib"))) {
                            try {
                                Libelle libelle = LibelleUtils.readLib((Element) nd2);
                                if (libelle != null) {
                                    corpusMetadataEditor.putLibelle(corpusField, libelle);
                                }
                            } catch (LangIntegerException ile) {
                            }
                        }
                    }
                } else if (tagname.equals("intitule")) {
                    addIntitule(el, corpusMetadataEditor);
                }
            }
        }
    }

    private CorpusField getCorpusField(Element element_xml) {
        String fieldkey_string = element_xml.getAttribute("field-key");
        if (fieldkey_string.length() == 0) {
            errorHandler.missingFieldKeyAttributeError(element_xml.getTagName());
            return null;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(fieldkey_string);
        } catch (ParseException pe) {
            errorHandler.wrongFieldKeyAttributeError(element_xml.getTagName(), fieldkey_string);
            return null;
        }
        CorpusField corpusField = ((CorpusMetadata) corpusMetadataEditor.getMetadata()).getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            errorHandler.unknownCorpusFieldWarning(element_xml.getTagName(), fieldKey);
        }
        return corpusField;
    }


    private class BasedefichesFormatDefErrorHandler implements BasedefichesFormatDefParser.ErrorHandler {

        public void nullFormatSourceError() {
            handle("nullFormatSourceError");
        }

        public void wrongFormatSourceError() {
            handle("wrongFormatSourceError");
        }

        public void wrongFormatSourcePartError(String formatSourcePart) {
            handle("wrongFormatSourcePartError" + " : " + formatSourcePart);
        }

        public void invalidAsciiCharacterInstructionError(String formatInstructionPart) {
            handle("invalidAsciiCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void invalidEndCharacterInstructionError(String formatInstructionPart) {
            handle("invalidEndCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void invalidSeparatorCharacterInstructionError(String formatInstructionPart) {
            handle("invalidSeparatorCharacterInstructionError" + " : " + formatInstructionPart);
        }

        public void unknownArgumentKeyInstructionWarning(String argumentKey) {
            handle("unknownArgumentKeyInstructionWarning" + " : " + argumentKey);
        }

        public void missingArgumentValueInstructionError(String argumentKey) {
            handle("missingArgumentValueInstructionError" + " : " + argumentKey);
        }

        public void wrongArgumentValueInstructionError(String argumentKey, String argumentValue) {
            handle("wrongArgumentValueInstructionError" + " : " + argumentKey + " / " + argumentValue);
        }

        private void handle(String s) {
            errorHandler.wrongOptionValueError(FieldKey.TITRE, TITREGENERATIONFORMAT_OPTION, s);
        }

    }

}
