/* BasedefichesLib_Xml - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusFieldList;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptions;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsConstants;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.IntegerList;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class CorpusMetadataXMLWriter extends UtilXMLWriter implements FieldOptionsConstants {

    public CorpusMetadataXMLWriter() {
    }

    public void appendCorpusMetadata(CorpusMetadata corpusMetadata) throws IOException {
        openTag("corpus-metadata");
        appendFields(corpusMetadata);
        appendOptions(corpusMetadata);
        appendL10n(corpusMetadata);
        closeTag("corpus-metadata");
    }

    private void appendFields(CorpusMetadata corpusMetadata) throws IOException {
        openTag("fields");
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            addEmptyElement("soustitre");
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            startOpenTag("propriete");
            addAttribute("type", corpusField.getFieldKey().getTypeString());
            appendFicheItemAttribute(corpusField);
            closeEmptyTag();

        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            startOpenTag("information");
            addAttribute("type", corpusField.getFieldKey().getTypeString());
            appendFicheItemAttribute(corpusField);
            closeEmptyTag();
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        for (int i = 0; i < sectionCount; i++) {
            CorpusField corpusField = corpusMetadata.getSectionField(i);
            startOpenTag("section");
            addAttribute("type", corpusField.getFieldKey().getTypeString());
            closeEmptyTag();
        }
        closeTag("fields");
    }

    private void appendFicheItemAttribute(CorpusField corpusField) throws IOException {
        addAttribute("fiche-item", CorpusMetadataUtils.ficheItemTypeToString(corpusField.getFicheItemType()));
    }

    private void appendL10n(CorpusMetadata corpusMetadata) throws IOException {
        openTag("l10n");
        List<CorpusField> list = CorpusMetadataUtils.getCorpusFieldList(corpusMetadata);
        for (CorpusField corpusField : list) {
            startOpenTag("field");
            addAttribute("field-key", corpusField.getFieldKey().toString());
            endOpenTag();
            addLibelleList(corpusField);
            closeTag("field");
        }
        BasedefichesXMLUtils.appendIntituleList(corpusMetadata, this);
        closeTag("l10n");
    }

    private void appendOptions(CorpusMetadata corpusMetadata) throws IOException {
        openTag("options");
        CorpusField geolocalisationField = corpusMetadata.getGeolocalisationField();
        if (geolocalisationField != null) {
            String value = null;
            appendOption(FieldOptionsUtils.GEOLOCALISATIONFIELD_OPTION, value, geolocalisationField.getFieldKey());
        }
        appendOptions(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE));
        appendOptions(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.LANG));
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            appendOptions(corpusField);
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            appendOptions(corpusField);
        }
        closeTag("options");
    }

    private void appendOptions(CorpusField corpusField) throws IOException {
        FieldOptions fieldOptions = corpusField.getFieldOptions();
        FieldKey fieldKey = corpusField.getFieldKey();
        Short informationDisplayShort = fieldOptions.getInformationDisplay();
        if (informationDisplayShort != null) {
            short informationDisplay = informationDisplayShort.shortValue();
            if (informationDisplay != FieldOptionsUtils.getDefaultInformationDisplay()) {
                appendOption(INFORMATIONDISPLAY_OPTION, FieldOptionsUtils.informationDisplayToString(informationDisplay), fieldKey);
            }
        }
        SubsetKey sphereKey = fieldOptions.getDefaultSphereKey();
        if (sphereKey != null) {
            appendOption(DEFAULTSPHEREKEY_OPTION, sphereKey.toString(), fieldKey);
        }
        boolean bool = fieldOptions.getSubfieldDisplay();
        if (bool) {
            appendOption(SUBFIELDDISPLAY_OPTION, "1", fieldKey);
        }
        IntegerList currencyList = fieldOptions.getCurrencyList();
        if (currencyList != null) {
            int length = currencyList.getIntegerCount();
            String[] currencyArray = new String[length];
            for (int i = 0; i < length; i++) {
                currencyArray[i] = CurrencyInteger.toString(currencyList.getInteger(i));
            }
            appendOption(CURRENCYARRAY_OPTION, currencyArray, fieldKey);
        }
        BasedefichesFormatDef generationFormatDef = fieldOptions.getGenerationFormatDef();
        if (generationFormatDef != null) {
            appendOption(TITREGENERATIONFORMAT_OPTION, BasedefichesFormatUtils.toStringArray(generationFormatDef), fieldKey);
        }
        CorpusFieldList corpusFieldList = fieldOptions.getAddressFieldList();
        if (corpusFieldList != null) {
            int length = corpusFieldList.getCorpusFieldCount();
            String[] fieldArray = new String[length];
            for (int i = 0; i < length; i++) {
                fieldArray[i] = corpusFieldList.getCorpusField(i).getFieldKey().toString();
            }
            appendOption(ADDRESSFIELDARRAY_OPTION, fieldArray, fieldKey);
        }
        String[] stringOptionNameArray = corpusField.getStringOptionNameArray();
        int length = stringOptionNameArray.length;
        for (int i = 0; i < length; i++) {
            appendStringOption(corpusField, stringOptionNameArray[i]);
        }
    }

    private void appendStringOption(CorpusField corpusField, String optionName) throws IOException {
        String optionValue = corpusField.getStringOption(optionName);
        if (optionValue != null) {
            appendOption(optionName, optionValue, corpusField.getFieldKey());
        }
    }

    private void appendOption(String name, String value, FieldKey fieldKey) throws IOException {
        startOpenTag("option");
        addAttribute("name", name);
        addAttribute("value", value);
        addAttribute("field-key", fieldKey.toString());
        closeEmptyTag();
    }

    private void appendOption(String name, String[] values, FieldKey fieldKey) throws IOException {
        startOpenTag("option");
        addAttribute("name", name);
        addAttribute("field-key", fieldKey.toString());
        endOpenTag();
        int length = values.length;
        for (int i = 0; i < length; i++) {
            String value = values[i];
            if ((value != null) && (value.length() > 0)) {
                addSimpleElement("value", value);
            } else {
                addEmptyElement("value");
            }
        }
        closeTag("option");
    }

}
