/* BasedefichesLib_Xml - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.CroisementEntry;
import net.mapeadores.basedefiches.croisement.NotEditableCroisementException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CroisementDOMReader extends BasedefichesDOMReader {

    private CroisementEditor croisementEditor;

    public CroisementDOMReader(CroisementEditor croisementsEditor, BasedefichesXmlErrorHandler basedefichesXmlErrorHandler) {
        super(basedefichesXmlErrorHandler);
        this.croisementEditor = croisementsEditor;
    }

    public boolean initCroisement(SubsetCouple subsetCouple, Element element_xml) {
        List<CroisementEntry> croisementEntryList = new ArrayList<CroisementEntry>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("ci")) {
                    try {
                        CroisementEntry croisementEntry = getCroisementEntry(el);
                        croisementEntryList.add(croisementEntry);
                    } catch (IllegalArgumentException iae) {
                    }
                } else {
                    basedefichesXmlErrorHandler.unknownTagWarning(tagname);
                }
            }
        }
        if (croisementEntryList.size() == 0) {
            return false;
        }
        try {
            croisementEditor.initCroisements(subsetCouple, croisementEntryList);
            return true;
        } catch (NotEditableCroisementException nece) {
            return false;
        }
    }

    private CroisementEntry getCroisementEntry(Element element) {
        InternalCroisementEntry internalCroisementEntry = new InternalCroisementEntry();
        internalCroisementEntry.setIdsubset1(getAttribute(element, "id1", false));
        internalCroisementEntry.setIdsubset2(getAttribute(element, "id2", false));
        internalCroisementEntry.setPoids(getAttribute(element, "poids", true));
        internalCroisementEntry.setPosition1(getAttribute(element, "pos1", true));
        internalCroisementEntry.setPosition2(getAttribute(element, "pos2", true));
        return internalCroisementEntry;
    }

    private int getAttribute(Element element, String name, boolean emptyallowed) {
        String s = element.getAttribute(name);
        if (s.length() == 0) {
            if (emptyallowed) {
                return -1;
            }
            basedefichesXmlErrorHandler.emptyAttributeError(element.getTagName(), name);
            throw new IllegalArgumentException();
        }
        try {
            int entier = Integer.parseInt(s);
            if (entier < 0) {
                entier = -1;
            }
            return entier;
        } catch (NumberFormatException nfe) {
            basedefichesXmlErrorHandler.wrongIntegerAttributeError(element.getTagName(), name, s);
            throw new IllegalArgumentException();
        }
    }


    private class InternalCroisementEntry implements CroisementEntry {

        private int idsubset1;
        private int idsubset2;
        private int poids = 1;
        private int position1 = -1;
        private int position2 = -1;

        public int getIdsubset1() {
            return idsubset1;
        }

        public void setIdsubset1(int idsubset1) {
            this.idsubset1 = idsubset1;
        }

        public int getIdsubset2() {
            return idsubset2;
        }

        public void setIdsubset2(int idsubset2) {
            this.idsubset2 = idsubset2;
        }

        public int getPoids() {
            return poids;
        }

        public void setPoids(int poids) {
            if (poids < 1) {
                this.poids = 1;
            } else {
                this.poids = poids;
            }
        }

        public int getPosition1() {
            return position1;
        }

        public void setPosition1(int position1) {
            this.position1 = position1;
        }

        public int getPosition2() {
            return position2;
        }

        public void setPosition2(int position2) {
            this.position2 = position2;
        }

    }

}
