/* BasedefichesLib_Xml - Copyright (c) 2006-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.corpus.fiche.CdataDiv;
import net.mapeadores.basedefiches.corpus.fiche.Code;
import net.mapeadores.basedefiches.corpus.fiche.Courriel;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Div;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlock;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.H;
import net.mapeadores.basedefiches.corpus.fiche.IdentifiedFicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.Image;
import net.mapeadores.basedefiches.corpus.fiche.Insert;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Li;
import net.mapeadores.basedefiches.corpus.fiche.Link;
import net.mapeadores.basedefiches.corpus.fiche.Ln;
import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.fiche.MutableFicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.Nombre;
import net.mapeadores.basedefiches.corpus.fiche.P;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.ParagraphBlock;
import net.mapeadores.basedefiches.corpus.fiche.Pays;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.fiche.S;
import net.mapeadores.basedefiches.corpus.fiche.Table;
import net.mapeadores.basedefiches.corpus.fiche.Td;
import net.mapeadores.basedefiches.corpus.fiche.TextContentBuilder;
import net.mapeadores.basedefiches.corpus.fiche.Tr;
import net.mapeadores.basedefiches.corpus.fiche.Ul;
import net.mapeadores.basedefiches.corpus.fiche.ZoneBlock;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.CurrencyIntegerException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


/**
 *
 * @author Vincent Calame
 */
public class FicheDOMReader {

    private Fiche currentFiche;
    private boolean allowFlat = false;
    private RootHandler rootHandler = new RootHandler();
    private EnteteHandler enteteHandler = new EnteteHandler();
    private boolean append = true;
    private CorpsdeficheHandler corpsdeficheHandler = new CorpsdeficheHandler();

    public FicheDOMReader() {
    }

    public void setAllowFlat(boolean allowFlat) {
        this.allowFlat = allowFlat;
    }

    public void setAppend(boolean append) {
        this.append = append;
    }

    public Fiche readFiche(Element element_xml) {
        Fiche fiche = new Fiche();
        try {
            int langInteger = LangInteger.parse(element_xml.getAttribute("xml:lang"));
            fiche.setLangInteger(langInteger);
        } catch (LangIntegerException clie) {
        }
        readFiche(fiche, element_xml);
        return fiche;
    }

    public void readFiche(Fiche fiche, Element element_xml) {
        this.currentFiche = fiche;
        DOMUtils.readChildren(element_xml, rootHandler);
    }


    private class RootHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("entete")) {
                DOMUtils.readChildren(element, enteteHandler);
            } else if (tagName.equals("corpsdefiche")) {
                DOMUtils.readChildren(element, corpsdeficheHandler);
            } else if (tagName.equals("source")) {
                DOMUtils.readChildren(element, enteteHandler);
            } else {
                if (allowFlat) {
                    boolean isEnteteElement = enteteHandler.readEnteteElement(element);
                    if (!isEnteteElement) {
                        corpsdeficheHandler.readCorpsdeficheElement(element);
                    }
                }
            }
        }

    }


    private class EnteteHandler implements ElementHandler {

        public void readElement(Element element) {
            readEnteteElement(element);
        }

        private boolean readEnteteElement(Element el) {
            String tagname = el.getTagName();
            if (tagname.equals("titre")) {
                currentFiche.setTitre(XMLUtils.getData(el));
                return true;
            } else if (tagname.equals("soustitre")) {
                currentFiche.setSoustitre(readPara(el));
                return true;
            } else if (tagname.equals("propriete")) {
                readPropriete(currentFiche, el);
                return true;
            } else if (tagname.equals("information")) {
                readInformation(currentFiche, el);
                return true;
            } else if (tagname.equals("redacteurs")) {
                readRedacteurs(currentFiche, el);
                return true;
            }
            return false;
        }

    }


    private class CorpsdeficheHandler implements ElementHandler {

        public void readElement(Element element) {
            readCorpsdeficheElement(element);
        }

        private boolean readCorpsdeficheElement(Element el) {
            String tagname = el.getTagName();
            if (tagname.equals("texte")) {
                FieldKey fieldKey = FieldKey.build(FieldKey.SECTION_CATEGORY, "texte");
                MutableFicheBlockList ficheBlockList = new MutableFicheBlockList();
                readFicheBlockList(ficheBlockList, el);
                if (append) {
                    currentFiche.appendSection(fieldKey, ficheBlockList);
                } else {
                    currentFiche.setSection(fieldKey, ficheBlockList);
                }
                return true;
            } else if ((tagname.equals("section")) || (tagname.equals("annexe"))) {
                readSection(currentFiche, el);
                return true;
            }
            return false;
        }

    }

    public void readPropriete(Fiche fiche, Element element) {
        String type = element.getAttribute("type");
        if (type.length() == 0) {
            return;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(FieldKey.PROPRIETE_CATEGORY, type);
        } catch (ParseException pe) {
            return;
        }
        FicheItem ficheItem = null;
        NodeList liste = element.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                ficheItem = readFicheItem(el);
                break;
            }
        }
        fiche.setPropriete(fieldKey, ficheItem);
    }

    public void readInformation(Fiche fiche, Element element) {
        String type = element.getAttribute("type");
        if (type.length() == 0) {
            return;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(FieldKey.INFORMATION_CATEGORY, type);
        } catch (ParseException pe) {
            return;
        }
        MutableFicheItemList ficheItemList = new MutableFicheItemList();
        readFicheItemList(ficheItemList, element);
        if (append) {
            fiche.appendInformation(fieldKey, ficheItemList);
        } else {
            fiche.setInformation(fieldKey, ficheItemList);
        }
    }

    public void readRedacteurs(Fiche fiche, Element element) {
        MutableFicheItemList ficheItemList = new MutableFicheItemList();
        readFicheItemList(ficheItemList, element);
        if (append) {
            fiche.appendRedacteurs(ficheItemList);
        } else {
            fiche.setRedacteurs(ficheItemList);
        }
    }

    public FicheItem readFicheItem(Element element) {
        FicheItem ficheItem = null;
        String tagname = element.getTagName();
        if (tagname.equals("item")) {
            String value = XMLUtils.getData(element);
            if (value.length() == 0) {
                return null;
            } else {
                return new Item(value);
            }
        }
        if (tagname.equals("personne")) {
            ficheItem = readPersonne(element);
        } else if (tagname.equals("langue")) {
            ficheItem = readLangue(element);
        } else if (tagname.equals("pays")) {
            ficheItem = readPays(element);
        } else if (tagname.equals("datation")) {
            ficheItem = readDatation(element);
        } else if (tagname.equals("link")) {
            ficheItem = readLink(element);
        } else if (tagname.equals("courriel")) {
            ficheItem = readCourriel(element);
        } else if (tagname.equals("entite")) {
            ficheItem = readEntite(element);
        } else if (tagname.equals("biblio")) {
            ficheItem = readBiblio(element);
        } else if (tagname.equals("montant")) {
            ficheItem = readMontant(element);
        } else if (tagname.equals("nombre")) {
            ficheItem = readNombre(element);
        } else if (tagname.equals("geopoint")) {
            ficheItem = readGeopoint(element);
        } else if (tagname.equals("para")) {
            ficheItem = readPara(element);
        } else if (tagname.equals("image")) {
            ficheItem = readImage(element);
        }
        return ficheItem;
    }

    public Personne readPersonne(Element element_xml) {
        String strg_idsphere = element_xml.getAttribute("idsphere");
        String sphere_string = element_xml.getAttribute("sphere");
        if ((sphere_string.length() != 0) && (strg_idsphere.length() != 0)) {
            try {
                RedacteurKey redacteurKey = RedacteurKey.parse(sphere_string + "_" + strg_idsphere);
                return new Personne(redacteurKey.toString());
            } catch (ParseException pe) {
                return null;
            }
        }
        String nom = "";
        String prenom = "";
        String organisme = "";
        String original = "";
        NodeList liste = element_xml.getChildNodes();
        boolean nomAvant = false;
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("nom")) {
                    nom = XMLUtils.getData(el);
                    String avantString = el.getAttribute("avant");
                    if (avantString.equals("true")) {
                        nomAvant = true;
                    }
                    //deprecié
                    String strg_style = el.getAttribute("style");
                    if (strg_style.equals("zh") || strg_style.equals("zh-latin")) {
                        nomAvant = true;
                    }
                } else if (tagname.equals("prenom")) {
                    prenom = XMLUtils.getData(el);
                } else if (tagname.equals("original")) {
                    original = XMLUtils.getData(el);
                } else if (tagname.equals("organisme")) {
                    organisme = XMLUtils.getData(el);
                }
            }
        }
        PersonneBean personneBean = new PersonneBean(nom, prenom);
        personneBean.setOriginal(original);
        personneBean.setNomAvant(nomAvant);
        return new Personne(personneBean, organisme);
    }

    public Langue readLangue(Element element_xml) {
        String strg_intlang = element_xml.getAttribute("lang");
        try {
            int langInteger = LangInteger.parse(strg_intlang, true);
            return new Langue(langInteger);
        } catch (LangIntegerException mcle) {
            return null;
        }
    }

    public Pays readPays(Element element_xml) {
        String codePays = element_xml.getAttribute("country");
        try {
            Pays pays = new Pays(codePays);
            return pays;
        } catch (IllegalArgumentException iae) {
            return null;
        }
    }

    public Nombre readNombre(Element element_xml) {
        String valString = element_xml.getAttribute("val");
        try {
            Decimal decimal = StringUtils.parseStrictDecimal(valString);
            return new Nombre(decimal);
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    public Geopoint readGeopoint(Element element_xml) {
        String latitudeString = element_xml.getAttribute("lat");
        String longitudeString = element_xml.getAttribute("lon");
        if (longitudeString.length() == 0) {
            longitudeString = element_xml.getAttribute("long"); //Ancienne version avant 0.9.8.7-beta07
        }
        try {
            Decimal latitude = StringUtils.parseStrictDecimal(latitudeString);
            Decimal longitude = StringUtils.parseStrictDecimal(longitudeString);
            return new Geopoint(new DegreDecimal(latitude), new DegreDecimal(longitude));
        } catch (NumberFormatException nfe) {
            return null;
        }
    }

    public Para readPara(Element element_xml) {
        Para.Builder builder = new Para.Builder();
        readContentList(builder, element_xml);
        return builder.toPara();
    }

    public Montant readMontant(Element element_xml) {
        String valString = element_xml.getAttribute("val");
        String curString = element_xml.getAttribute("cur");
        try {
            Decimal decimal = StringUtils.parseStrictDecimal(valString);
            int currencyInteger = CurrencyInteger.parse(curString);
            return new Montant(decimal, currencyInteger);
        } catch (NumberFormatException nfe) {
            return null;
        } catch (CurrencyIntegerException cie) {
            return null;
        }
    }

    public Datation readDatation(Element element_xml) {
        String type = element_xml.getAttribute("type");
        String a = element_xml.getAttribute("a");
        String s = element_xml.getAttribute("s");
        String t = element_xml.getAttribute("t");
        String m = element_xml.getAttribute("m");
        String j = element_xml.getAttribute("j");
        int dateInteger = 0;
        try {
            int annee = Integer.parseInt(a);
            if (type.equals("s")) {
                int semestre = Integer.parseInt(s);
                dateInteger = DateInteger.fromSemestre(annee, semestre);
            } else if (type.equals("t")) {
                int trimestre = Integer.parseInt(t);
                dateInteger = DateInteger.fromTrimestre(annee, trimestre);
            } else if (type.equals("m")) {
                int mois = Integer.parseInt(m);
                dateInteger = DateInteger.fromMois(annee, mois);
            } else if (type.equals("j")) {
                int mois = Integer.parseInt(m);
                int jour = Integer.parseInt(j);
                dateInteger = DateInteger.fromJour(annee, mois, jour);
            } else {
                dateInteger = DateInteger.fromAnnee(annee); // seule ligne à conserver
            }
        } catch (NumberFormatException nfe) {
            return null;
        } catch (DateIntegerException de) {
            return null;
        }
        return new Datation(dateInteger);
    }

    public Link readLink(Element element_xml) {
        String href = element_xml.getAttribute("href");
        String title = "";
        String comment = "";
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("title")) {
                    title = XMLUtils.getData(element);
                } else if (tagname.equals("comment")) {
                    comment = XMLUtils.getData(element);
                }
            }
        }
        return new Link(href, title, comment);
    }

    public Image readImage(Element element_xml) {
        String src = element_xml.getAttribute("src");
        String alt = "";
        String title = "";
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("alt")) {
                    alt = XMLUtils.getData(element);
                } else if (tagname.equals("title")) {
                    title = XMLUtils.getData(element);
                }
            }
        }
        return new Image(src, alt, title);
    }

    public Courriel readCourriel(Element element_xml) {
        CourrielBean courrielBean = new CourrielBean();
        String addrSpec = element_xml.getAttribute("addr-spec");
        courrielBean.setAddrSpec(addrSpec);
        String realName = element_xml.getAttribute("real-name");
        if (realName.length() == 0) {
            String strg = XMLUtils.getData(element_xml);
            if (strg.length() > 0) {
                try {
                    CourrielBean c2 = CourrielBean.parse(strg);
                    realName = c2.getNomComplet();
                } catch (ParseException pe) {
                }
            }
        }
        courrielBean.setNomComplet(realName);
        return new Courriel(courrielBean);
    }

    public Item readEntite(Element element_xml) {
        String nom = "";
        String localisation = "";
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("nom")) {
                    nom = XMLUtils.getData(el);
                } else if (tagname.equals("localisation")) {
                    localisation = XMLUtils.getData(el);
                }
            }
        }
        return new Item(nom);
    }

    public Item readBiblio(Element element_xml) {
        String titre = "";
        String auteurs = "";
        String collection = "";
        String editeur = "";
        String ville = "";
        String pays = "";
        String annee = "";
        String nbrepages = "";
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("titre")) {
                    titre = XMLUtils.getData(el);
                } else if (tagname.equals("auteurs")) {
                    auteurs = XMLUtils.getData(el);
                } else if (tagname.equals("collection")) {
                    collection = XMLUtils.getData(el);
                } else if (tagname.equals("editeur")) {
                    editeur = XMLUtils.getData(el);
                } else if (tagname.equals("ville")) {
                    ville = XMLUtils.getData(el);
                } else if (tagname.equals("pays")) {
                    pays = XMLUtils.getData(el);
                } else if (tagname.equals("annee")) {
                    annee = XMLUtils.getData(el);
                } else if (tagname.equals("nbrepages")) {
                    nbrepages = XMLUtils.getData(el);
                }
            }
        }
        StringBuilder buf = new StringBuilder();
        buf.append(titre);
        appendString(buf, auteurs);
        appendString(buf, collection);
        appendString(buf, editeur);
        appendString(buf, ville);
        appendString(buf, pays);
        appendString(buf, annee);
        appendString(buf, nbrepages);
        return new Item(buf.toString());
    }

    private void appendString(StringBuilder buf, String s) {
        if (s.length() > 0) {
            buf.append(", ");
            buf.append(s);
        }
    }

    public void readSection(Fiche fiche, Element element) {
        String type = element.getAttribute("type");
        if (type.length() == 0) {
            return;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(FieldKey.SECTION_CATEGORY, type);
        } catch (ParseException pe) {
            return;
        }
        MutableFicheBlockList ficheBlockList = new MutableFicheBlockList();
        readFicheBlockList(ficheBlockList, element);
        if (append) {
            fiche.appendSection(fieldKey, ficheBlockList);
        } else {
            fiche.setSection(fieldKey, ficheBlockList);
        }
    }

    public FicheBlock readFicheBlock(Element element) {
        String tagname = element.getTagName();
        if (tagname.equals("p")) {
            return readP(element);
        } else if (tagname.equals("ul")) {
            return readUl(element);
        } else if (tagname.equals("h")) {
            return readH(element);
        } else if (tagname.equals("code")) {
            return readCode(element);
        } else if (tagname.equals("table")) {
            return readTable(element);
        } else if (tagname.equals("insert")) {
            return readInsert(element);
        } else if (tagname.equals("div")) {
            return readDiv(element);
        } else if (tagname.equals("cdatadiv")) {
            return readCDataDiv(element);
        } else {
            return null;
        }
    }

    public P readP(Element element_xml) {
        short type = P.getTypeToShort(element_xml.getAttribute("type"));
        P p = new P(type);
        p.setSource(element_xml.getAttribute("source"));
        readParagraphBlock(p, element_xml);
        return p;
    }

    public H readH(Element element_xml) {
        int level = 1;
        try {
            level = Integer.parseInt(element_xml.getAttribute("level"));
        } catch (NumberFormatException nfe) {
        }
        H h = new H(level);
        readParagraphBlock(h, element_xml);
        return h;
    }

    public Ul readUl(Element element_xml) {
        Ul ul = new Ul();
        ul.setClassName(element_xml.getAttribute("class"));
        ul.setHtmlId(element_xml.getAttribute("htmlid"));
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("li")) {
                    ul.addLi(readLi(el));
                }
            }
        }
        return ul;
    }

    public Li readLi(Element element_xml) {
        Li li = new Li();
        readIdentifiedFicheBlockList(li, element_xml);
        readFicheBlockList(li, element_xml);
        return li;
    }

    public Code readCode(Element element_xml) {
        short type = Code.typeToShort(element_xml.getAttribute("type"));
        Code code = new Code(type);
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("ln")) {
                    code.addLn(readLn(el));
                }
            }
        }
        readZoneBlock(code, element_xml);
        return code;
    }

    public Table readTable(Element element_xml) {
        Table table = new Table();
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("tr")) {
                    table.addTr(readTr(el));
                } else if (tagname.equals("caption")) {
                    String caption = XMLUtils.getData(el);
                    table.getLegende().addText(caption);
                }
            }
        }
        readZoneBlock(table, element_xml);
        return table;
    }

    public Tr readTr(Element element_xml) {
        Tr tr = new Tr();
        readIdentifiedFicheBlockList(tr, element_xml);
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("td")) {
                    tr.addTd(readTd(el));
                }
            }
        }
        return tr;
    }

    public Ln readLn(Element element_xml) {
        int indentation = 0;
        try {
            indentation = Integer.parseInt(element_xml.getAttribute("indent"));
            if (indentation < 0) {
                indentation = 0;
            }
        } catch (NumberFormatException nfe) {
        }
        Ln ln = new Ln(XMLUtils.getData(element_xml, false));
        ln.setIndentation(indentation);
        ln.setClassName(element_xml.getAttribute("class"));
        ln.setHtmlId(element_xml.getAttribute("htmlid"));
        return ln;
    }

    public Td readTd(Element element_xml) {
        String typeString = element_xml.getAttribute("type");
        short type = Td.typeToShort(typeString);
        Td td = null;
        if (type == Td.HEADER_TYPE) {
            td = Td.getHeaderTdInstance();
        } else {
            td = new Td();
        }
        try {
            int colspan = Integer.parseInt(element_xml.getAttribute("colspan"));
            td.setColSpan(colspan);
        } catch (NumberFormatException nfe) {
        }
        try {
            int rowspan = Integer.parseInt(element_xml.getAttribute("rowspan"));
            td.setRowSpan(rowspan);
        } catch (NumberFormatException nfe) {
        }
        readParagraphBlock(td, element_xml);
        return td;
    }

    public Insert readInsert(Element element_xml) {
        Insert insert = null;
        try {
            short type = Insert.typeToShort(element_xml.getAttribute("type"));
            insert = new Insert(type);
        } catch (IllegalArgumentException iae) {
            return null;
        }
        insert.setSrc(element_xml.getAttribute("src"));
        insert.setRef(element_xml.getAttribute("ref"));
        insert.setCredit(element_xml.getAttribute("credit"));
        insert.setPosition(Insert.positionToString(element_xml.getAttribute("position")));
        try {
            int width = Integer.parseInt(element_xml.getAttribute("width"));
            insert.setWidth(width);
        } catch (NumberFormatException nfe) {
        }
        try {
            int height = Integer.parseInt(element_xml.getAttribute("height"));
            insert.setHeight(height);
        } catch (NumberFormatException nfe) {
        }
        String albumSubsetName = element_xml.getAttribute("album");
        if (albumSubsetName.length() > 0) {
            try {
                SubsetKey albumSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, albumSubsetName);
                String idalbumString = element_xml.getAttribute("idalbum");
                if (idalbumString.length() > 0) {
                    try {
                        int idalbum = Integer.parseInt(idalbumString);
                        String albumDimName = element_xml.getAttribute("albumdim");
                        insert.setIllustration(albumSubsetKey, idalbum, albumDimName);
                    } catch (NumberFormatException nfe) {
                    }
                }
            } catch (ParseException pe) {
            }
        }
        readZoneBlock(insert, element_xml);
        return insert;
    }

    public Div readDiv(Element element_xml) {
        Div div = new Div();
        readZoneBlock(div, element_xml);
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("fbl")) {
                    readFicheBlockList(div.getFicheBlockList(), el);
                }
            }
        }

        return div;
    }

    public CdataDiv readCDataDiv(Element element_xml) {
        CdataDiv cdataDiv = new CdataDiv();
        readZoneBlock(cdataDiv, element_xml);
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("cdata")) {
                    String cdata = XMLUtils.getRawData(el);
                    cdataDiv.setCdata(cdata);
                    break;
                }
            }
        }
        return cdataDiv;
    }

    private void readZoneBlock(ZoneBlock zoneBlock, Element element_xml) {
        zoneBlock.setClassName(element_xml.getAttribute("class"));
        zoneBlock.setHtmlId(element_xml.getAttribute("htmlid"));
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                String tagname = el.getTagName();
                if (tagname.equals("numero")) {
                    readParagraphBlock(zoneBlock.getNumero(), el);
                } else if (tagname.equals("legende")) {
                    readParagraphBlock(zoneBlock.getLegende(), el);
                } else {
                    break;
                }
            }
        }
    }

    public S readS(Element element_xml) {
        String typeString = element_xml.getAttribute("type");
        short type = 0;
        String subType = null;
        if (typeString.equals("xml")) {
            type = S.TYPE_DFN;
            subType = "XML";
        } else if (typeString.equals("cmd")) {
            type = S.TYPE_DFN;
            subType = "CMD";
        } else if (typeString.equals("ui")) {
            type = S.TYPE_SAMP;
            subType = "UI";
        } else {
            type = S.getTypeToShort(typeString);
        }
        S s = new S(type);
        s.setValue(XMLUtils.getData(element_xml, false));
        int mask = s.getAttributeMask();
        s.setClassName(element_xml.getAttribute("class"));
        s.setHtmlId(element_xml.getAttribute("htmlid"));
        if ((mask & S.REF_BITVALUE) != 0) {
            s.setRef(element_xml.getAttribute("ref"));
        }
        if ((mask & S.SRC_BITVALUE) != 0) {
            s.setSrc(element_xml.getAttribute("src"));
        }
        if ((mask & S.TITLE_BITVALUE) != 0) {
            s.setTitle(element_xml.getAttribute("title"));
        }
        if ((mask & S.LANG_BITVALUE) != 0) {
            s.setLang(element_xml.getAttribute("lang"));
        }
        if ((mask & S.SUBTYPE_BITVALUE) != 0) {
            if (subType == null) {
                subType = element_xml.getAttribute("subtype");
            }
            s.setSubtype(subType);
        }
        return s;
    }

    private void readIdentifiedFicheBlockList(IdentifiedFicheBlockList identifiedFicheBlockList, Element element_xml) {
        identifiedFicheBlockList.setClassName(element_xml.getAttribute("class"));
        identifiedFicheBlockList.setHtmlId(element_xml.getAttribute("htmlid"));
    }

    private void readParagraphBlock(ParagraphBlock pb, Element element_xml) {
        pb.setClassName(element_xml.getAttribute("class"));
        pb.setHtmlId(element_xml.getAttribute("htmlid"));
        readContentList(pb, element_xml);
    }

    private void readContentList(TextContentBuilder pb, Element element_xml) {
        NodeList list = element_xml.getChildNodes();
        for (int i = 0; i < list.getLength(); i++) {
            short type = list.item(i).getNodeType();
            if (type == Node.TEXT_NODE) {
                String data = ((Text) list.item(i)).getData();
                data = XMLUtils.cleanString(data, false);
                if (data.length() > 0) {
                    pb.addText(data);
                }
            } else if (type == Node.ELEMENT_NODE) {
                Element element = (Element) list.item(i);
                if (element.getTagName().equals("s")) {
                    pb.addS(readS(element));
                }
            }
        }
    }

    private void readFicheItemList(MutableFicheItemList ficheItemList, Element element) {
        NodeList liste = element.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                FicheItem ficheItem = readFicheItem(el);
                try {
                    ficheItemList.addFicheItem(ficheItem);
                } catch (IllegalArgumentException iae) {
                }
            }
        }
    }

    public void readFicheBlockList(MutableFicheBlockList ficheBlockList, Element element) {
        NodeList liste = element.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(j);
                FicheBlock ficheBlock = readFicheBlock(el);
                if (ficheBlockList.isValidFicheBlock(ficheBlock)) {
                    ficheBlockList.addFicheBlock(ficheBlock);
                }
            }
        }
    }

}
