/* BasedefichesLib_Xml - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml;

import java.text.ParseException;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.ParentRecursivityException;
import net.mapeadores.basedefiches.thesaurus.ThesaurusEditor;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadataEditor;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.langinteger.LangIntegerUtils;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusDOMReader extends BasedefichesDOMReader {

    private ThesaurusEditor thesaurusEditor;
    private ThesaurusMetadataEditor thesaurusMetadataEditor;
    private ThesaurusXmlErrorHandler errorHandler;

    public ThesaurusDOMReader(ThesaurusXmlErrorHandler errorHandler) {
        super(errorHandler);
        this.errorHandler = errorHandler;
    }

    public void setThesaurusEditor(ThesaurusEditor thesaurusEditor) {
        this.thesaurusEditor = thesaurusEditor;
        this.thesaurusMetadataEditor = thesaurusEditor.getThesaurusMetadataEditor();
    }

    public static short getThesaurusType(Element element_xml) {
        String typeString = element_xml.getAttribute("thesaurus-type");
        return ThesaurusUtils.thesaurusTypeToShort(typeString);
    }

    public void fillThesaurus(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        boolean metadone = false;
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if ((!metadone) && (tagname.equals("metadata"))) {
                    metadone = true;
                    fillMetadata(el);
                } else if (tagname.equals("motcle")) {
                    addMotcle(el, null);
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private void fillMetadata(Element element_xml) {
        if (((ThesaurusMetadata) thesaurusMetadataEditor.getMetadata()).getThesaurusType() != ThesaurusMetadata.BABELIEN_TYPE) {
            String langs = element_xml.getAttribute("thesaurus-langs");
            if (langs.length() == 0) {
                errorHandler.emptyAttributeError(element_xml.getTagName(), "thesaurus-langs");
            } else {
                int[] array = LangIntegerUtils.toCleanLangIntegerArray(langs);
                if (array.length > 0) {
                    thesaurusMetadataEditor.setAuthorizedLangIntegerArray(array);
                } else {
                    errorHandler.wrongLangArrayError(langs);
                }
            }
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("intitule")) {
                    addIntitule(el, thesaurusMetadataEditor);
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

    private void addMotcle(Element element_xml, Motcle parent) {
        String idthsString = element_xml.getAttribute("idths");
        if (idthsString.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "idths");
            return;
        }
        int idths = 0;
        try {
            idths = Integer.parseInt(idthsString);
        } catch (NumberFormatException nfe) {
            errorHandler.wrongIntegerAttributeError(element_xml.getTagName(), "idths", idthsString);
            return;
        }
        String idalpha = null;
        if (ThesaurusUtils.isIdalphaType(thesaurusEditor.getThesaurus())) {
            idalpha = element_xml.getAttribute("idalpha");
            if (idalpha.length() == 0) {
                errorHandler.emptyAttributeError(element_xml.getTagName(), "idalpha");
                return;
            }
        }
        Motcle motcle = null;
        try {
            motcle = thesaurusEditor.createMotcle(idths, idalpha);
        } catch (ExistingIdException eie) {
            errorHandler.motcleDoublonError(idths, idalpha);
            return;
        } catch (ParseException pe) {
            errorHandler.idalphaParseError(idalpha);
            return;
        }
        if (parent != null) {
            try {
                thesaurusEditor.setParent(motcle, parent);
            } catch (ParentRecursivityException pre) {
                throw new ImplementationException(pre);
            }
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("lib")) {
                    try {
                        Libelle libelle = LibelleUtils.readLib(el);
                        if (libelle != null) {
                            thesaurusEditor.putLibelle(motcle, libelle);
                        }
                    } catch (LangIntegerException e) {
                        errorHandler.wrongLangAttributeError("lib", el.getAttribute("xml:lang"));
                    }
                } else if (tagname.equals("motcle")) {
                    addMotcle(el, motcle);
                } else {
                    errorHandler.unknownTagWarning(tagname);
                }
            }
        }
    }

}
