/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.Locale;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.FileLength;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.extraction.DocumentExtractInfo;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.api.DocumentFilter;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class DocumentXMLPart extends XMLPart {

    private ExtractionContext extractionContext;

    public DocumentXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
    }

    public void addDocument(DocumentExtractInfo documentExtractInfo) throws IOException {
        DocumentFilter documentFilter = documentExtractInfo.getDocumentFilter();
        Document document = documentExtractInfo.getDocument();
        String tagName = documentFilter.getTagName();
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("basename", document.getBasename());
            endOpenTag();
        }
        for (int i = 0; i < document.getVersionCount(); i++) {
            Version version = document.getVersion(i);
            addVersion(version);
        }
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void addVersion(Version version) throws IOException {
        FileLength fileLength = version.getFileLength();
        float value = fileLength.getRoundedValue();
        short roundType = fileLength.getRoundType();
        boolean isKibiOctet = (fileLength.getRoundType() == FileLength.KB_ROUND);
        startOpenTag("version");
        addAttribute("extension", version.getVersionName().getExtension());
        endOpenTag();
        startOpenTag("size");
        addAttribute("value", String.valueOf(value));
        String unit = (isKibiOctet) ? "Ki" : "Mi";
        addAttribute("unit", unit);
        endOpenTag();
        addSizeLib(value, isKibiOctet);
        closeTag("size");
        closeTag("version");
    }

    private void addSizeLib(float value, boolean isKibiOctet) throws IOException {
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            NumberFormat format = NumberFormat.getInstance(locale);
            StringBuilder buf = new StringBuilder();
            buf.append(format.format(value));
            buf.append(' ');
            if (isKibiOctet) {
                buf.append(LocalisationUtils.getKibiOctet(localeInteger));
            } else {
                buf.append(LocalisationUtils.getMebiOctet(localeInteger));
            }
            XMLUtils.addLibElement(this, localeInteger, buf.toString());
        }
    }

}
