/* BasedefichesLib_Xml - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.extraction.api.ExtractionConstants;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.api.GroupClause;
import net.mapeadores.basedefiches.extraction.api.IntituleClause;
import net.mapeadores.basedefiches.extraction.api.PackClause;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.extraction.builders.DocumentFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtractionDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitAddendaDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitAlbumDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitCorpusDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitThesaurusDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.FicheFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.GroupClauseBuilder;
import net.mapeadores.basedefiches.extraction.builders.IllustrationFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.IntituleClauseBuilder;
import net.mapeadores.basedefiches.extraction.builders.MotcleFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.PackClauseBuilder;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitList;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitListBuilder;
import net.mapeadores.basedefiches.selection.DocumentSelectOptions;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeParser;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.conditions.ConditionsXMLStorage;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionDOMReader implements ExtractionConstants {

    public ExtractionDOMReader() {
    }

    public ExtractionDef readExtraction(Element element_xml) {
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        ExtraitCorpusDef dynamicExtraitCorpusDef = null;
        ExtraitThesaurusDef dynamicExtraitThesaurusDef = null;
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        tagNameAttribute = BuilderUtils.testTagName(tagNameAttribute, EXTRACTION_DEFAULT_TAGNAME);
        extractionDefBuilder.setTagName(tagNameAttribute);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("static")) {
                    initStaticFilterList(extractionDefBuilder, element);
                } else if ((tagname.equals("extraitcorpus")) && (dynamicExtraitCorpusDef == null)) {
                    dynamicExtraitCorpusDef = initExtraitCorpusDef(element);
                } else if ((tagname.equals("extraitthesaurus")) && (dynamicExtraitThesaurusDef == null)) {
                    dynamicExtraitThesaurusDef = initExtraitThesaurusDef(element);
                }
            }
        }
        extractionDefBuilder.setDynamicExtraitCorpusDef(dynamicExtraitCorpusDef);
        extractionDefBuilder.setDynamicExtraitThesaurusDef(dynamicExtraitThesaurusDef);
        return extractionDefBuilder.toExtractionDef();
    }

    private void initStaticFilterList(ExtractionDefBuilder extractionDefBuilder, Element element_xml) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        tagNameAttribute = BuilderUtils.testTagName(tagNameAttribute, STATIC_DEFAULT_TAGNAME);
        extractionDefBuilder.setStaticTagName(tagNameAttribute);
        List<ExtraitCorpusDef> extraitCorpusDefList = new ArrayList<ExtraitCorpusDef>();
        List<ExtraitThesaurusDef> extraitThesaurusDefList = new ArrayList<ExtraitThesaurusDef>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("extraitcorpus")) {
                    extraitCorpusDefList.add(initExtraitCorpusDef(element));
                } else if (tagname.equals("extraitthesaurus")) {
                    extraitThesaurusDefList.add(initExtraitThesaurusDef(element));
                }
            }
        }
        extractionDefBuilder.setStaticExtraitCorpusDefList(BuilderUtils.toExtraitCorpusDefList(extraitCorpusDefList));
        extractionDefBuilder.setStaticExtraitThesaurusDefList(BuilderUtils.toExtraitThesaurusDefList(extraitThesaurusDefList));
    }

    private FicheFilter initFicheFilter(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("fiche")) {
                    String tagNameAttribute = element.getAttribute("tag-name");
                    FilterUnitListBuilder ficheInfoListBuilder = new FilterUnitListBuilder();
                    initFilterUnitList(ficheInfoListBuilder, element);
                    FicheFilterBuilder builder = new FicheFilterBuilder();
                    builder.updateFilterUnitList(ficheInfoListBuilder.toFilterUnitList());
                    builder.parseTagName(tagNameAttribute);
                    return builder.toFicheFilter();
                }
            }
        }
        return FicheFilterBuilder.TITRE_FICHEFILTER;
    }

    public static FicheSelectOptions getFicheSelectOptions(String s) {
        SubsetKey[] corpusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_CORPUS, s);
        if (corpusKeyArray != null) {
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            return ficheSelectOptions;
        } else {
            return null;
        }
    }

    public static FicheSelectOptions getFicheSelectOptions(Element element_xml) {
        FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
        List motcleSelectOptionsList = new ArrayList();
        List redacteurKeyList = new ArrayList();
        List subsetKeyList = new ArrayList();
        if (isEmpty(element_xml)) {
            return ficheSelectOptions;
        }
        StringBuilder corpusNameBuffer = new StringBuilder();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("corpus")) {
                    String corpusName = XMLUtils.getData(element);
                    if (corpusName.length() > 0) {
                        if (corpusNameBuffer.length() > 0) {
                            corpusNameBuffer.append(';');
                        }
                        corpusNameBuffer.append(corpusName);
                    }
                } else if (tagname.equals("range")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        ficheSelectOptions.setRangeList(rangeList);
                    }
                } else if (tagname.equals("field-select")) {
                    Condition fieldCondition = initEntete(element);
                    if ((fieldCondition != null) && (fieldCondition.getState() != ConditionConstants.U)) {
                        short fieldConditionScope = FicheSelectOptions.TITRE_SCOPE;
                        String scopeString = element.getAttribute("scope");
                        try {
                            fieldConditionScope = FicheSelectOptions.fieldConditionToShort(scopeString);
                        } catch (IllegalArgumentException iae) {
                        }
                        FieldKey[] fieldKeyArray = null;
                        if (fieldConditionScope == FicheSelectOptions.SELECTION_SCOPE) {
                            fieldKeyArray = getFieldKeyArray(element);
                            if (fieldKeyArray == null) {
                                fieldConditionScope = FicheSelectOptions.TITRE_SCOPE;
                            }
                        }
                        ficheSelectOptions.setFieldCondition(fieldCondition, fieldConditionScope, fieldKeyArray);
                    }
                } else if (tagname.equals("motcle-select")) {
                    MotcleSelectOptions motcleSelectOptions = getMotcleSelectOptions(element);
                    motcleSelectOptionsList.add(motcleSelectOptions);
                } else if (tagname.equals("motcle-select-mode")) {
                    short mode = getMotcleSelectMode(element);
                    if (mode != 0) {
                        ficheSelectOptions.setMotcleSelectMode(mode);
                    }
                } else if (tagname.equals("redacteurs")) {
                    initRedacteurList(element, redacteurKeyList, subsetKeyList);
                } else if (tagname.equals("poids")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        if (rangeList != null) {
                            boolean isExclude = false;
                            String mode = element.getAttribute("mode");
                            if (mode.equals("exclude")) {
                                isExclude = true;
                            }
                            ficheSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(rangeList, isExclude));
                        }
                    }
                } else if (tagname.equals("current")) {
                    ficheSelectOptions.setWithCurrentCorpus(true);
                }
            }
        }
        int motcleSize = motcleSelectOptionsList.size();
        if (motcleSize > 0) {
            MotcleSelectOptions[] motcleSelectOptionsArray = new MotcleSelectOptions[motcleSize];
            for (int i = 0; i < motcleSize; i++) {
                motcleSelectOptionsArray[i] = (MotcleSelectOptions) motcleSelectOptionsList.get(i);
            }
            ficheSelectOptions.setMotcleSelectOptionsArray(motcleSelectOptionsArray);
        }
        if (corpusNameBuffer.length() > 0) {
            SubsetKey[] corpusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_CORPUS, corpusNameBuffer.toString());
            if (corpusKeyArray != null) {
                ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            }
        }
        int size = redacteurKeyList.size();
        if (size > 0) {
            RedacteurKey[] redacteurKeyArray = new RedacteurKey[size];
            for (int i = 0; i < size; i++) {
                redacteurKeyArray[i] = (RedacteurKey) redacteurKeyList.get(i);
            }
            ficheSelectOptions.setRedacteurKeyArray(redacteurKeyArray);
        }
        int subsetSize = subsetKeyList.size();
        if (subsetSize > 0) {
            SubsetKey[] subsetKeyArray = new SubsetKey[subsetSize];
            for (int i = 0; i < subsetSize; i++) {
                subsetKeyArray[i] = (SubsetKey) subsetKeyList.get(i);
            }
            ficheSelectOptions.setSphereKeyArray(subsetKeyArray);
        }
        return ficheSelectOptions;
    }

    private static void initRedacteurList(Element element_xml, List redacteurKeyList, List subsetKeyList) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("redacteur")) {
                    String sphere = element.getAttribute("sphere");
                    String idsphereString = element.getAttribute("idsphere");
                    if (sphere.length() == 0) {
                        sphere = null;
                    }
                    try {
                        redacteurKeyList.add(RedacteurKey.parse(sphere + "_" + idsphereString));
                    } catch (java.text.ParseException pe) {
                    }
                } else if (tagname.equals("sphere")) {
                    String sphereName = XMLUtils.getData(element);
                    if (sphereName.length() > 0) {
                        try {
                            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName);
                            subsetKeyList.add(sphereKey);
                        } catch (java.text.ParseException pe) {
                        }
                    }
                }
            }
        }
    }

    private static short getMotcleSelectMode(Element element_xml) {
        String modeString = element_xml.getAttribute("mode");
        if (modeString.equals("and")) {
            return ConditionUtils.MODE_AND;
        } else if (modeString.equals("or")) {
            return ConditionUtils.MODE_OR;
        } else {
            return 0;
        }
    }

    private static Condition initEntete(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (ConditionsXMLStorage.isConditionElement(tagname)) {
                    return ConditionsXMLStorage.getCondition(element);
                }
            }
        }
        return null;
    }

    private static FieldKey[] getFieldKeyArray(Element element_xml) {
        Set<FieldKey> fieldKeySet = new LinkedHashSet<FieldKey>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("field")) {
                    try {
                        FieldKey fieldKey = FieldKey.parse(XMLUtils.getData(element));
                        fieldKeySet.add(fieldKey);
                    } catch (ParseException pe) {
                    }
                }
            }
        }
        int size = fieldKeySet.size();
        if (size == 0) {
            return null;
        }
        return fieldKeySet.toArray(new FieldKey[size]);
    }

    public static MotcleSelectOptions getMotcleSelectOptions(String s) {
        SubsetKey[] thesaurusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_THESAURUS, s);
        if (thesaurusKeyArray != null) {
            MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
            motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
            return motcleSelectOptions;
        } else {
            return null;
        }
    }

    public static IllustrationSelectOptions getIllustrationSelectOptions(String s) {
        SubsetKey[] albumKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_ALBUM, s);
        if (albumKeyArray != null) {
            IllustrationSelectOptions illustrationSelectOptions = new IllustrationSelectOptions();
            illustrationSelectOptions.setAlbumKeyArray(albumKeyArray);
            return illustrationSelectOptions;
        } else {
            return null;
        }
    }

    public static IllustrationSelectOptions getIllustrationSelectOptions(Element element_xml) {
        IllustrationSelectOptions illustrationSelectOptions = new IllustrationSelectOptions();
        if (isEmpty(element_xml)) {
            return illustrationSelectOptions;
        }
        StringBuilder albumNameBuffer = new StringBuilder();
        List<String> albumDimNameList = new ArrayList<String>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("album")) {
                    String albumName = XMLUtils.getData(element);
                    if (albumName.length() > 0) {
                        if (albumNameBuffer.length() > 0) {
                            albumNameBuffer.append(';');
                        }
                        albumNameBuffer.append(albumName);
                    }
                } else if (tagname.equals("albumdim")) {
                    String albumDimName = XMLUtils.getData(element);
                    if (albumDimName.length() > 0) {
                        albumDimNameList.add(albumDimName);
                    }
                }
            }
        }
        if (albumNameBuffer.length() > 0) {
            SubsetKey[] albumKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_ALBUM, albumNameBuffer.toString());
            if (albumKeyArray != null) {
                illustrationSelectOptions.setAlbumKeyArray(albumKeyArray);
            }
        }
        int size = albumDimNameList.size();
        if (size > 0) {
            illustrationSelectOptions.setAlbumDimNameArray(albumDimNameList.toArray(new String[size]));
        }
        return illustrationSelectOptions;
    }

    private void initIllustrationFilter(IllustrationFilterBuilder illustrationFilterBuilder, Element element_xml, String defaultTagName) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        illustrationFilterBuilder.parseIllustrationTagName(tagNameAttribute, defaultTagName);
    }

    public static MotcleSelectOptions getMotcleSelectOptions(Element element_xml) {
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        if (isEmpty(element_xml)) {
            return motcleSelectOptions;
        }
        StringBuilder thesaurusNameBuffer = new StringBuilder();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("thesaurus")) {
                    String nomthesaurus = XMLUtils.getData(element);
                    if (nomthesaurus.length() > 0) {
                        if (thesaurusNameBuffer.length() > 0) {
                            thesaurusNameBuffer.append(';');
                        }
                        thesaurusNameBuffer.append(nomthesaurus);
                    }
                } else if (tagname.equals("range")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        motcleSelectOptions.setIdthsRangeList(rangeList);
                    }
                } else if (tagname.equals("level")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        motcleSelectOptions.setLevelRangeList(rangeList);
                    }
                } else if (tagname.equals("poids")) {
                    String range = XMLUtils.getData(element);
                    if (range.length() > 0) {
                        RangeList rangeList = RangeUtils.positiveRangeParse(range);
                        if (rangeList != null) {
                            boolean isExclude = false;
                            String mode = element.getAttribute("mode");
                            if (mode.equals("exclude")) {
                                isExclude = true;
                            }
                            motcleSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(rangeList, isExclude));
                        }
                    }
                } else if (ConditionsXMLStorage.isConditionElement(tagname)) {
                    Condition condition = ConditionsXMLStorage.getCondition(element);
                    if (condition.getState() != ConditionConstants.U) {
                        motcleSelectOptions.setFieldCondition(condition);
                    }
                }
            }
        }
        if (thesaurusNameBuffer.length() > 0) {
            SubsetKey[] thesaurusKeyArray = BasedefichesUtils.toSubsetKeyArray(SubsetKey.CATEGORY_THESAURUS, thesaurusNameBuffer.toString());
            if (thesaurusKeyArray != null) {
                motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
            }
        }
        return motcleSelectOptions;
    }

    private void initMotcleFilter(MotcleFilterBuilder motcleFilterBuilder, Element element_xml, String defaultTagName) {
        String tagNameAttribute = element_xml.getAttribute("tag-name");
        motcleFilterBuilder.parseMotcleTagName(tagNameAttribute, defaultTagName);
        String recursiveAttribute = element_xml.getAttribute("recursive");
        if ((recursiveAttribute.equals("1")) || (recursiveAttribute.equalsIgnoreCase("true"))) {
            if (defaultTagName.equals(MOTCLE_DEFAULT_TAGNAME)) {
                motcleFilterBuilder.setChildrenRecursive();
            } else if (defaultTagName.equals(PARENT_DEFAULT_TAGNAME)) {
                motcleFilterBuilder.setParentRecursive();
            }
        }
        if (isEmpty(element_xml)) {
            return;
        }
        FicheSelectorBundle ficheSelectorBundle = new FicheSelectorBundle();
        NodeList liste = element_xml.getChildNodes();
        for (int j = 0; j < liste.getLength(); j++) {
            if (liste.item(j).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(j);
                String tagname = element.getTagName();
                if (tagname.equals("lib")) {
                    motcleFilterBuilder.setWithLib(true);
                } else if (tagname.equals("level")) {
                    motcleFilterBuilder.setWithLevel(true);
                } else if (tagname.equals("next")) {
                    MotcleFilterBuilder nextFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(nextFilterBuilder, element, NEXT_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setNextFilter(nextFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("previous")) {
                    MotcleFilterBuilder previousFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(previousFilterBuilder, element, PREVIOUS_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setPreviousFilter(previousFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("motcle")) {
                    MotcleFilterBuilder childrenFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(childrenFilterBuilder, element, MOTCLE_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setChildrenFilter(childrenFilterBuilder.toMotcleFilter());
                } else if (tagname.equals("parent")) {
                    MotcleFilterBuilder parentFilterBuilder = new MotcleFilterBuilder();
                    initMotcleFilter(parentFilterBuilder, element, PARENT_DEFAULT_TAGNAME);
                    motcleFilterBuilder.setParentFilter(parentFilterBuilder.toMotcleFilter());
                } else {
                    ficheSelectorBundle.readExtraitCorpus(element);
                }
            }
        }
        if (ficheSelectorBundle.isNotEmpty()) {
            motcleFilterBuilder.setIndexationExtraitCorpusDefList(ficheSelectorBundle.toExtraitCorpusDefList());
        }
    }

    private void initFilterUnitList(FilterUnitListBuilder builder, Element element_xml) {
        if (isEmpty(element_xml)) {
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                testFilterUnit(builder, element);
            }
        }
    }

    private void testFilterUnit(FilterUnitListBuilder builder, Element element) {
        String paramsString = element.getAttribute("params");
        AttributeHolder paramsAttributes = null;
        if (paramsString.length() > 0) {
            paramsAttributes = AttributeParser.parseAttributeHolder(paramsString, ExtractionAttributes.EXTRACTION_NAMESPACE, null);
        }
        String tagname = element.getTagName();
        if (tagname.equals("entete")) {
            builder.addEnteteFilter();
        } else if (tagname.equals("corpsdefiche")) {
            builder.addCorpsdeficheFilter();
        } else if (tagname.equals("titre")) {
            builder.addFieldKey(FieldKey.TITRE, paramsAttributes);
        } else if (tagname.equals("soustitre")) {
            builder.addFieldKey(FieldKey.SOUSTITRE, paramsAttributes);
        } else if (tagname.equals("propriete")) {
            String type = element.getAttribute("type");
            if (type.length() > 0) {
                builder.addTypeFilter(FieldKey.PROPRIETE_CATEGORY, type, paramsAttributes);
            }
        } else if (tagname.equals("redacteurs")) {
            builder.addFieldKey(FieldKey.REDACTEURS, paramsAttributes);
        } else if (tagname.equals("information")) {
            String type = element.getAttribute("type");
            if (type.length() > 0) {
                builder.addTypeFilter(FieldKey.INFORMATION_CATEGORY, type, paramsAttributes);
            }
        } else if (tagname.equals("texte")) {
            builder.addTypeFilter(FieldKey.SECTION_CATEGORY, "texte", paramsAttributes);
        } else if ((tagname.equals("section")) || (tagname.equals("annexe"))) {
            String type = element.getAttribute("type");
            if (type.length() > 0) {
                builder.addTypeFilter(FieldKey.SECTION_CATEGORY, type, paramsAttributes);
            }
        } else if (tagname.equals("lot")) {
            String tagName = element.getAttribute("tag-name");
            FilterUnitListBuilder builder2 = new FilterUnitListBuilder();
            initFilterUnitList(builder2, element);
            FilterUnitList filterUnitList = builder2.toFilterUnitList();
            if (filterUnitList.getFilterUnitCount() > 0) {
                builder.addLotFilter(tagName, filterUnitList);
            }
        } else if ((tagname.equals("langue")) || (tagname.equals("lang"))) {
            builder.addFieldKey(FieldKey.LANG, paramsAttributes);
        } else if (tagname.equals("chrono")) {
            builder.addChronoFilter();
        } else if (tagname.equals("association")) {
            builder.addExtraitAddendaFilter(ExtraitAddendaDefBuilder.ASSOCIATION_EXTRAITADDENDADEF);
        } else if (tagname.equals("extraitaddenda")) {
            ExtraitAddendaDef extraitAddendaDef = initExtraitAddendaDef(element);
            builder.addExtraitAddendaFilter(extraitAddendaDef);
        } else if (tagname.equals("extraitcorpus")) {
            ExtraitCorpusDef extraitCorpusDef = initExtraitCorpusDef(element);
            builder.addExtraitCorpusFilter(extraitCorpusDef);
        } else if (tagname.equals("extraitthesaurus")) {
            ExtraitThesaurusDef extraitThesaurusDef = initExtraitThesaurusDef(element);
            builder.addExtraitThesaurusFilter(extraitThesaurusDef);
        } else if (tagname.equals("extraitalbum")) {
            ExtraitAlbumDef extraitAlbumDef = initExtraitAlbumDef(element);
            builder.addExtraitAlbumFilter(extraitAlbumDef);
        } else if (tagname.equals("intitule")) {
            builder.addIntituleFicheFilter();
        }
    }

    private static boolean isEmpty(Element element) {
        int size = element.getChildNodes().getLength();
        if (size == 0) {
            return true;
        }
        if (size > 1) {
            return false;
        }
        Node node = element.getFirstChild();
        if (node.getNodeType() != Node.TEXT_NODE) {
            return false;
        }
        String data = ((Text) node).getData().trim();
        if (data.length() == 0) {
            return true;
        } else {
            return false;
        }
    }


    private class FicheSelectorBundle implements ElementHandler {

        private List<ExtraitCorpusDef> list = new ArrayList<ExtraitCorpusDef>();

        void addAllLiage() {
            ExtraitCorpusDefBuilder extraitCorpusDefBuilder = new ExtraitCorpusDefBuilder();
            extraitCorpusDefBuilder.setFicheFilter(FicheFilterBuilder.TITRE_FICHEFILTER);
            list.add(extraitCorpusDefBuilder.toExtraitCorpusDef());
        }

        boolean isNotEmpty() {
            return (list.size() != 0);
        }

        public ExtraitCorpusDefList toExtraitCorpusDefList() {
            return BuilderUtils.toExtraitCorpusDefList(list);
        }

        public void readElement(Element element) {
            readExtraitCorpus(element);
        }

        public boolean readExtraitCorpus(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("extraitcorpus")) {
                ExtraitCorpusDef extraitCorpusDef = initExtraitCorpusDef(element);
                list.add(extraitCorpusDef);
                return true;
            }
            return false;
        }

    }

    private ExtraitAddendaDef initExtraitAddendaDef(Element element_xml) {
        ExtraitAddendaDefBuilder extraitAddendaDefBuilder = new ExtraitAddendaDefBuilder();
        extraitAddendaDefBuilder.setExtraitName(element_xml.getAttribute("extrait-name"));
        extraitAddendaDefBuilder.setTagName(element_xml.getAttribute("tag-name"));
        DocumentFilterBuilder documentFilterBuilder = new DocumentFilterBuilder();
        List<DocumentSelectOptions> selectOptionsList = new ArrayList<DocumentSelectOptions>();
        extraitAddendaDefBuilder.setDocumentFilter(documentFilterBuilder.toDocumentFilter());
        extraitAddendaDefBuilder.setDocumentSelectOptionsList(selectOptionsList);
        return extraitAddendaDefBuilder.toExtraitAddendaDef();
    }

    private ExtraitAlbumDef initExtraitAlbumDef(Element element_xml) {
        ExtraitAlbumDefBuilder extraitAlbumDefBuilder = new ExtraitAlbumDefBuilder();
        extraitAlbumDefBuilder.setExtraitName(element_xml.getAttribute("extrait-name"));
        extraitAlbumDefBuilder.setTagName(element_xml.getAttribute("tag-name"));
        IllustrationSelectOptions attributeSelectionOptions = null;
        String albumAttribute = element_xml.getAttribute("album");
        if (albumAttribute.length() > 0) {
            attributeSelectionOptions = getIllustrationSelectOptions(albumAttribute);
        }
        IllustrationFilterBuilder illustrationFilterBuilder = new IllustrationFilterBuilder();
        NodeList liste = element_xml.getChildNodes();
        List<IllustrationSelectOptions> selectOptionsList = new ArrayList<IllustrationSelectOptions>();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("illustration")) {
                    initIllustrationFilter(illustrationFilterBuilder, element, ILLUSTRATION_DEFAULT_TAGNAME);
                } else if (tagname.equals("illustration-select")) {
                    selectOptionsList.add(getIllustrationSelectOptions(element));
                }
            }
        }
        int optionsSize = selectOptionsList.size();
        if ((optionsSize == 0) && (attributeSelectionOptions != null)) {
            selectOptionsList.add(attributeSelectionOptions);
        }
        extraitAlbumDefBuilder.setIllustrationFilter(illustrationFilterBuilder.toIllustrationFilter());
        extraitAlbumDefBuilder.setIllustrationSelectOptionsList(selectOptionsList);
        return extraitAlbumDefBuilder.toExtraitAlbumDef();
    }

    private ExtraitThesaurusDef initExtraitThesaurusDef(Element element_xml) {
        ExtraitThesaurusDefBuilder extraitThesaurusDefBuilder = new ExtraitThesaurusDefBuilder();
        extraitThesaurusDefBuilder.setExtraitName(element_xml.getAttribute("extrait-name"));
        extraitThesaurusDefBuilder.setTagName(element_xml.getAttribute("tag-name"));
        MotcleSelectOptions attributeSelectionOptions = null;
        String thesaurusAttribute = element_xml.getAttribute("thesaurus");
        if (thesaurusAttribute.length() > 0) {
            attributeSelectionOptions = getMotcleSelectOptions(thesaurusAttribute);
        }
        MotcleFilterBuilder motcleFilterBuilder = new MotcleFilterBuilder();
        NodeList liste = element_xml.getChildNodes();
        List<MotcleSelectOptions> selectOptionsList = new ArrayList<MotcleSelectOptions>();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("motcle")) {
                    initMotcleFilter(motcleFilterBuilder, element, MOTCLE_DEFAULT_TAGNAME);
                } else if (tagname.equals("motcle-select")) {
                    selectOptionsList.add(getMotcleSelectOptions(element));
                } else if (tagname.equals("intitule")) {
                    motcleFilterBuilder.setWithIntituleThesaurus(true);
                }
            }
        }
        int optionsSize = selectOptionsList.size();
        if ((optionsSize == 0) && (attributeSelectionOptions != null)) {
            selectOptionsList.add(attributeSelectionOptions);
        }
        extraitThesaurusDefBuilder.setMotcleFilter(motcleFilterBuilder.toMotcleFilter());
        extraitThesaurusDefBuilder.setMotcleSelectOptionsList(selectOptionsList);
        return extraitThesaurusDefBuilder.toExtraitThesaurusDef();
    }

    private ExtraitCorpusDef initExtraitCorpusDef(Element element_xml) {
        ExtraitCorpusDefBuilder extraitCorpusDefBuilder = new ExtraitCorpusDefBuilder();
        extraitCorpusDefBuilder.setExtraitName(element_xml.getAttribute("extrait-name"));
        extraitCorpusDefBuilder.setTagName(element_xml.getAttribute("tag-name"));
        FicheSelectOptions attributeSelectionOptions = null;
        String corpusAttribute = element_xml.getAttribute("corpus");
        if (corpusAttribute.length() > 0) {
            attributeSelectionOptions = getFicheSelectOptions(corpusAttribute);
        }
        GroupClause attributeGroupClause = null;
        String sortAttribute = element_xml.getAttribute("sort");
        if (sortAttribute.length() > 0) {
            attributeGroupClause = GroupClauseBuilder.getGroupClause(sortAttribute, true);
        }
        GroupClause groupClause = null;
        PackClause packClause = null;
        IntituleClause intituleClause = null;
        List<FicheSelectOptions> selectOptionsList = new ArrayList<FicheSelectOptions>();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("group")) {
                    groupClause = initGroupClause(element);
                } else if (tagname.equals("fiche-select")) {
                    selectOptionsList.add(getFicheSelectOptions(element));
                } else if (tagname.equals("intitule")) {
                    intituleClause = initIntituleClause(element);
                } else if (tagname.equals("pack")) {
                    packClause = initPackClause(element);
                }
            }
        }
        int optionsSize = selectOptionsList.size();
        if ((optionsSize == 0) && (attributeSelectionOptions != null)) {
            selectOptionsList.add(attributeSelectionOptions);
        }
        if (groupClause == null) {
            groupClause = attributeGroupClause;
        }
        FicheFilter ficheFilter = initFicheFilter(element_xml);
        extraitCorpusDefBuilder.setGroupClause(groupClause);
        extraitCorpusDefBuilder.setPackClause(packClause);
        extraitCorpusDefBuilder.setIntituleClause(intituleClause);
        extraitCorpusDefBuilder.setFicheFilter(ficheFilter);
        extraitCorpusDefBuilder.setFicheSelectOptionsList(selectOptionsList);
        return extraitCorpusDefBuilder.toExtraitCorpusDef();
    }

    private static PackClause initPackClause(Element element_xml) {
        String sizeString = element_xml.getAttribute("size");
        PackClauseBuilder builder = PackClauseBuilder.newInstance(sizeString);
        if (builder == null) {
            return null;
        }
        String moduloString = element_xml.getAttribute("modulo");
        if (moduloString.length() > 0) {
            builder.parseModulo(moduloString);
        }
        String tagName = element_xml.getAttribute("tag-name");
        if (tagName.length() > 0) {
            builder.parseTagName(tagName);
        }
        return builder.toPackClause();
    }

    private static IntituleClause initIntituleClause(Element element_xml) {
        return IntituleClauseBuilder.DEFAULT;
    }

    private static GroupClause initGroupClause(Element element_xml) {
        String typeString = element_xml.getAttribute("type");
        GroupClauseBuilder builder = GroupClauseBuilder.newInstance(typeString);
        if (builder == null) {
            return null;
        }
        String params = element_xml.getAttribute("params");
        if (params.length() == 0) {
            params = element_xml.getAttribute("mode");
        }
        if (params.length() > 0) {
            builder.parseGroupParams(params);
        }
        String sortString = element_xml.getAttribute("sort");
        if (sortString.length() > 0) {
            builder.parseSortOrder(sortString);
        }
        String tagName = element_xml.getAttribute("tag-name");
        if (tagName.length() > 0) {
            builder.parseTagName(tagName);
        }
        GroupClause subGroupClause = null;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("group")) {
                    subGroupClause = initGroupClause(element);
                }
            }
        }
        if (subGroupClause != null) {
            builder.setSubGroupClause(subGroupClause);
        }
        GroupClause groupClause = builder.toGroupClause();
        return groupClause;
    }

}
