/* BasedefichesLib_Xml - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.PoidsCondition;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.conditions.ConditionsXMLStorage;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionUtils {

    private ExtractionUtils() {
    }

    public static void writeFicheSelectOptions(XMLWriter xmlWriter, FicheSelectOptions ficheSelectOptions, String tagName) throws IOException {
        xmlWriter.openTag(tagName);
        writeFicheSelectOptions(xmlWriter, ficheSelectOptions);
        xmlWriter.closeTag(tagName);
    }

    public static void writeFicheSelectOptions(XMLWriter xmlWriter, FicheSelectOptions ficheSelectOptions) throws IOException {
        SubsetKey[] corpusKeyArray = ficheSelectOptions.getCorpusKeyArray();
        if (corpusKeyArray != null) {
            for (int i = 0; i < corpusKeyArray.length; i++) {
                xmlWriter.addSimpleElement("corpus", corpusKeyArray[i].getSubsetName());
            }
        }
        RangeList rangeList = ficheSelectOptions.getRangeList();
        if (rangeList != null) {
            xmlWriter.addSimpleElement("range", RangeUtils.positiveRangetoString(rangeList));
        }
        PoidsCondition poidsCondition = ficheSelectOptions.getPoidsCondition();
        if (poidsCondition != null) {
            xmlWriter.startOpenTag("poids");
            if (poidsCondition.isExclusionRangeList()) {
                xmlWriter.addAttribute("mode", "exclude");
            }
            xmlWriter.append(RangeUtils.positiveRangetoString(poidsCondition.getPoidsRangeList()));
            xmlWriter.endOpenTag();
            xmlWriter.closeTag("poids");
        }
        Condition fieldCondition = ficheSelectOptions.getFieldCondition();
        if (fieldCondition != null) {
            short fieldConditionScope = ficheSelectOptions.getFieldConditionScope();
            xmlWriter.startOpenTag("field-select");
            xmlWriter.addAttribute("scope", FicheSelectOptions.fieldConditionToString(fieldConditionScope));
            xmlWriter.endOpenTag();
            if (fieldConditionScope == FicheSelectOptions.SELECTION_SCOPE) {
                FieldKey[] fieldKeyArray = ficheSelectOptions.getFieldKeyArray();
                int length = fieldKeyArray.length;
                for (int i = 0; i < length; i++) {
                    xmlWriter.addSimpleElement("field", fieldKeyArray[i].toString());
                }
            }
            ConditionsXMLStorage.appendCondition(xmlWriter, fieldCondition);
            xmlWriter.closeTag("field-select");
        }

        MotcleSelectOptions[] motcleSelectOptionsArray = ficheSelectOptions.getMotcleSelectOptionsArray();
        if (motcleSelectOptionsArray != null) {
            int length = motcleSelectOptionsArray.length;
            if (length > 1) {
                xmlWriter.startOpenTag("motcle-select-mode");
                xmlWriter.addAttribute("mode", ConditionUtils.modeToString(ficheSelectOptions.getMotcleSelectMode()));
                xmlWriter.closeEmptyTag();
            }
            for (int i = 0; i < length; i++) {
                xmlWriter.openTag("motcle-select");
                writeMotcleSelectOptions(xmlWriter, motcleSelectOptionsArray[i]);
                xmlWriter.closeTag("motcle-select");
            }
        }
        RedacteurKey[] redacteurKeyArray = ficheSelectOptions.getRedacteurKeyArray();
        SubsetKey[] sphereKeyArray = ficheSelectOptions.getSphereKeyArray();
        if ((redacteurKeyArray != null) || (sphereKeyArray != null)) {
            xmlWriter.openTag("redacteurs");
            if (redacteurKeyArray != null) {
                for (int i = 0; i < redacteurKeyArray.length; i++) {
                    RedacteurKey redacteurKey = redacteurKeyArray[i];
                    xmlWriter.startOpenTag("redacteur");
                    SubsetKey sphereKey = redacteurKey.getSphereKey();
                    if (sphereKey != null) {
                        xmlWriter.addAttribute("sphere", sphereKey.getSubsetName());
                    }
                    xmlWriter.addAttribute("idsphere", redacteurKey.getIdsphereString());
                    xmlWriter.closeEmptyTag();
                }
            }
            if (sphereKeyArray != null) {
                for (int i = 0; i < sphereKeyArray.length; i++) {
                    xmlWriter.addSimpleElement("sphere", sphereKeyArray[i].getSubsetName());
                }
            }
            xmlWriter.closeTag("redacteurs");
        }
    }

    public static void writeMotcleSelectOptions(XMLWriter xmlWriter, MotcleSelectOptions motcleSelectOptions) throws IOException {
        SubsetKey[] thesaurusKeyArray = motcleSelectOptions.getThesaurusKeyArray();
        if (thesaurusKeyArray != null) {
            for (int i = 0; i < thesaurusKeyArray.length; i++) {
                xmlWriter.addSimpleElement("thesaurus", thesaurusKeyArray[i].getSubsetName());
            }
        }
        RangeList idthsRangeList = motcleSelectOptions.getIdthsRangeList();
        if (idthsRangeList != null) {
            xmlWriter.addSimpleElement("range", RangeUtils.positiveRangetoString(idthsRangeList));
        }
        PoidsCondition poidsCondition = motcleSelectOptions.getPoidsCondition();
        if (poidsCondition != null) {
            xmlWriter.startOpenTag("poids");
            if (poidsCondition.isExclusionRangeList()) {
                xmlWriter.addAttribute("mode", "exclude");
            }
            xmlWriter.append(RangeUtils.positiveRangetoString(poidsCondition.getPoidsRangeList()));
            xmlWriter.endOpenTag();
            xmlWriter.closeTag("poids");
        }
        RangeList levelRangeList = motcleSelectOptions.getLevelRangeList();
        if (levelRangeList != null) {
            xmlWriter.addSimpleElement("level", RangeUtils.positiveRangetoString(levelRangeList));
        }
        Condition sourceCondition = motcleSelectOptions.getFieldCondition();
        if (sourceCondition != null) {
            ConditionsXMLStorage.appendCondition(xmlWriter, sourceCondition);
        }
    }

}
