/* BasedefichesLib_Xml - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.api.ExtractionSource;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.extraction.api.CorpusExtractorList;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.extraction.api.ThesaurusExtractorList;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelectOptionsList;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptionsList;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionXMLPart extends XMLPart {

    private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
    private ExtractionContext extractionContext;
    private ExtraitCorpusXMLPart extraitCorpusXMLPart;
    private ExtraitThesaurusXMLPart extraitThesaurusXMLPart;

    public ExtractionXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
        extraitCorpusXMLPart = new ExtraitCorpusXMLPart(xmlWriter, extractionContext);
        extraitThesaurusXMLPart = new ExtraitThesaurusXMLPart(xmlWriter, extractionContext);
    }

    public void addExtraction(ExtractionDef extractionDef, Date date, ExtractionSource extractionSource, boolean documentRoot) throws IOException {
        String tagName = extractionDef.getTagName();
        if ((tagName == null) && (documentRoot)) {
            tagName = "NULL";
        }
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("date", simpleDateFormat.format(new Date()));
            endOpenTag();
        }
        Object source = extractionSource.getDynamicObject();
        if (source != null) {
            addDynamic(extractionDef, source);
        }
        addStatic(extractionDef.getStaticTagName(), extractionSource);
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void addDynamic(ExtractionDef extractionDef, Object source) throws IOException {
        if (source instanceof AbregeFiche) {
            addAbregeFicheDynamic(extractionDef.getDynamicExtraitCorpusDef(), (AbregeFiche) source);
        } else if (source instanceof Corpus) {
            addCorpusDynamic(extractionDef, (Corpus) source);
        } else if (source instanceof Motcle) {
            addMotcleDynamic(extractionDef.getDynamicExtraitThesaurusDef(), (Motcle) source);
        } else if (source instanceof Thesaurus) {
            addThesaurusDynamic(extractionDef.getDynamicExtraitThesaurusDef(), (Thesaurus) source);
        } else {
            throw new ClassCastException("unable to deal with the class " + source.getClass().getName());
        }
    }

    private void addAbregeFicheDynamic(ExtraitCorpusDef dynamicExtraitCorpusDef, AbregeFiche abregeFiche) throws IOException {
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = new CorpusExtractor(dynamicExtraitCorpusDef);
        corpusExtractor.add(abregeFiche, 0);
        extraitCorpusXMLPart.addExtraitcorpus(corpusExtractor);
    }

    private void addCorpusDynamic(ExtractionDef extractionDef, Corpus corpus) throws IOException {
        ExtraitCorpusDef dynamicExtraitCorpusDef = extractionDef.getDynamicExtraitCorpusDef();
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = new CorpusExtractor(dynamicExtraitCorpusDef);
        FicheSelectOptionsList ficheSelectOptionsList = dynamicExtraitCorpusDef.getFicheSelectOptionsList();
        int count = ficheSelectOptionsList.getFicheSelectOptionsCount();
        if (count > 0) {
            SubsetKey corpusKey = corpus.getSubsetKey();
            SubsetKey[] corpusKeyArray = {corpusKey};
            List<FicheSelectOptions> newOptionsList = new ArrayList<FicheSelectOptions>();
            for (int i = 0; i < count; i++) {
                FicheSelectOptions options = ficheSelectOptionsList.getFicheSelectOptions(i);
                SubsetKey[] array = options.getCorpusKeyArray();
                boolean present = false;
                if (array == null) {
                    present = true;
                } else {
                    for (int j = 0; j < array.length; j++) {
                        if (array[j].equals(corpusKey)) {
                            present = true;
                            break;
                        }
                    }
                }
                if (present) {
                    FicheSelectOptions nvSO = new FicheSelectOptions(options);
                    nvSO.setCorpusKeyArray(corpusKeyArray);
                    newOptionsList.add(nvSO);
                }
            }
            if (newOptionsList.size() == 0) {
                return;
            }
            FicheSelector ficheSelector = new FicheSelector(extractionContext.getBasedefiches(), extractionContext.getLocalisationLists().getWorkingLangInteger(0));
            ficheSelector.setFicheEligibility(extractionContext.getFicheEligibility());
            ficheSelector.addFicheSelectOptionsList(SelectionUtils.toFicheSelectOptionsList(newOptionsList));
            for (int i = 0; i < corpus.getTicketLength(); i++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
                if (abregeFiche != null) {
                    if (ficheSelector.isSelected(abregeFiche)) {
                        corpusExtractor.add(abregeFiche, 0);
                    }
                }
            }
        } else {
            FicheEligibility ficheEligibility = extractionContext.getFicheEligibility();
            for (int i = 0; i < corpus.getTicketLength(); i++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
                if (abregeFiche != null) {
                    if ((ficheEligibility == null) || (ficheEligibility.accept(abregeFiche))) {
                        corpusExtractor.add(abregeFiche, 0);
                    }
                }
            }
        }
        extraitCorpusXMLPart.addExtraitcorpus(corpusExtractor);
    }

    private void addMotcleDynamic(ExtraitThesaurusDef dynamicExtraitThesaurusDef, Motcle motcle) throws IOException {
        if (dynamicExtraitThesaurusDef == null) {
            return;
        }
        ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(dynamicExtraitThesaurusDef, null);
        thesaurusExtractor.add(motcle, 0);
        extraitThesaurusXMLPart.addExtraitthesaurus(thesaurusExtractor);
    }

    private void addThesaurusDynamic(ExtraitThesaurusDef dynamicExtraitThesaurusDef, Thesaurus thesaurus) throws IOException {
        if (dynamicExtraitThesaurusDef == null) {
            return;
        }
        ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(dynamicExtraitThesaurusDef, null);
        MotcleSelectOptionsList optionsList = dynamicExtraitThesaurusDef.getMotcleSelectOptionsList();
        int optionsCount = optionsList.getMotcleSelectOptionsCount();
        if (optionsCount > 0) {
            SubsetKey thesaurusKey = thesaurus.getSubsetKey();
            SubsetKey[] thesaurusKeyArray = {thesaurusKey};
            List<MotcleSelectOptions> newOptionsList = new ArrayList<MotcleSelectOptions>();
            for (int i = 0; i < optionsCount; i++) {
                MotcleSelectOptions options = optionsList.getMotcleSelectOptions(i);
                SubsetKey[] array = options.getThesaurusKeyArray();
                boolean present = false;
                if (array == null) {
                    present = true;
                } else {
                    for (int j = 0; j < array.length; j++) {
                        if (array[j].equals(thesaurusKey)) {
                            present = true;
                            break;
                        }
                    }
                }
                if (present) {
                    MotcleSelectOptions nvSO = new MotcleSelectOptions(options);
                    nvSO.setThesaurusKeyArray(thesaurusKeyArray);
                    newOptionsList.add(nvSO);
                }
            }
            if (newOptionsList.size() == 0) {
                return;
            }
            MotcleSelector motcleSelector = new MotcleSelector(extractionContext.getBasedefiches(), extractionContext.getLocalisationLists().getWorkingLangInteger(0));
            motcleSelector.addMotcleSelectOptionsList(SelectionUtils.toMotcleSelectOptionsList(newOptionsList));
            for (int i = 0; i < thesaurus.getTicketLength(); i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if (motcle != null) {
                    if (motcleSelector.isSelected(motcle)) {
                        thesaurusExtractor.add(motcle, 0);
                    }
                }
            }
        } else {
            MotcleList levelList = thesaurus.getFirstLevelList();
            for (int i = 0; i < levelList.getMotcleCount(); i++) {
                Motcle motcle = levelList.getMotcle(i);
                thesaurusExtractor.add(motcle, 0);
            }
        }
        extraitThesaurusXMLPart.addExtraitthesaurus(thesaurusExtractor);
    }

    private void addStatic(String staticTagName, ExtractionSource extractionSource) throws IOException {
        CorpusExtractorList corpusExtractorList = extractionSource.getStaticCorpusExtractorList();
        boolean containsExtraitcorpus = corpusExtractorList.containsFiche();
        ThesaurusExtractorList thesaurusExtractorList = extractionSource.getStaticThesaurusExtractorList();
        boolean containsExtraitthesaurus = thesaurusExtractorList.containsMotcle();
        if ((!containsExtraitcorpus) && (!containsExtraitthesaurus)) {
            return;
        }
        if (staticTagName != null) {
            openTag(staticTagName);
        }
        if (containsExtraitcorpus) {
            int count = corpusExtractorList.getCorpusExtractorCount();
            for (int i = 0; i < count; i++) {
                CorpusExtractor corpusExtractor = corpusExtractorList.getCorpusExtractor(i);
                if (corpusExtractor.containsFiche()) {
                    extraitCorpusXMLPart.addExtraitcorpus(corpusExtractor);
                }
            }
        }
        if (containsExtraitthesaurus) {
            int count = thesaurusExtractorList.getThesaurusExtractorCount();
            for (int i = 0; i < count; i++) {
                ThesaurusExtractor thesaurusExtractor = thesaurusExtractorList.getThesaurusExtractor(i);
                if (thesaurusExtractor.containsMotcle()) {
                    extraitThesaurusXMLPart.addExtraitthesaurus(thesaurusExtractor);
                }
            }
        }
        if (staticTagName != null) {
            closeTag(staticTagName);
        }
    }

    /**
     * Convertit une extraction en chaîne.
     * l'argument <code>prettyXml</code> indique s'il faut mettre tabulations et sauts de ligne.
     */
    public static String toString(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource, boolean prettyXml) {
        StringBuilder buf = new StringBuilder();
        XMLWriter xmlWriter = XMLUtils.getXMLWriter(buf, (prettyXml) ? 0 : -999);
        ExtractionXMLPart extractionXMLPart = new ExtractionXMLPart(xmlWriter, extractionContext);
        try {
            xmlWriter.appendXMLDeclaration();
            extractionXMLPart.addExtraction(extractionDef, new Date(), extractionSource, true);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    /**
     * Convertit une extraction en objet StreamSource permettant une transformation XSLT
     */
    public static StreamSource toStreamSource(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource) {
        String extractionString = toString(extractionContext, extractionDef, extractionSource, false);
        StringReader stringReader = new StringReader(extractionString);
        return new StreamSource(stringReader);
    }

}
