/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.extraction.AddendaExtractor;
import net.mapeadores.basedefiches.extraction.DocumentExtractInfo;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitAddendaXMLPart extends XMLPart {

    private ExtractionContext extractionContext;
    private DocumentXMLPart documentXMLPart;

    public ExtraitAddendaXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
        documentXMLPart = new DocumentXMLPart(xmlWriter, extractionContext);
    }

    public void addExtraitAddenda(AddendaExtractor addendaExtractor) throws IOException {
        ExtraitAddendaDef extraitAddendaDef = addendaExtractor.getExtraitAddendaDef();
        String tagName = extraitAddendaDef.getTagName();
        boolean withTag = (tagName != null);
        int addendaCount = addendaExtractor.getAddendaCount();
        String extraitName = extraitAddendaDef.getExtraitName();
        if (extraitName != null) {
            if (withTag) {
                startOpenTag(tagName);
                addAttribute("extrait-name", extraitName);
                endOpenTag();
            }
            for (int j = 0; j < addendaCount; j++) {
                List<DocumentExtractInfo> list = addendaExtractor.getDocumentExtractInfoList(j);
                for (DocumentExtractInfo info : list) {
                    documentXMLPart.addDocument(info);
                }
            }
            if (withTag) {
                closeTag(tagName);
            }
        } else {
            for (int j = 0; j < addendaCount; j++) {
                Addenda thesaurus = addendaExtractor.getAddenda(j);
                if (withTag) {
                    startOpenTag(tagName);
                    addAttribute("addenda", thesaurus.getSubsetKey().getSubsetName());
                    endOpenTag();
                }
                List<DocumentExtractInfo> list = addendaExtractor.getDocumentExtractInfoList(j);
                for (DocumentExtractInfo info : list) {
                    documentXMLPart.addDocument(info);
                }
                if (withTag) {
                    closeTag(tagName);
                }
            }
        }
    }

}
