/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.extraction.AlbumExtractor;
import net.mapeadores.basedefiches.extraction.IllustrationExtractInfo;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitAlbumXMLPart extends XMLPart {

    private IllustrationXMLPart illustrationXMLPart;

    public ExtraitAlbumXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
        illustrationXMLPart = new IllustrationXMLPart(xmlWriter);
    }

    public void addExtraitAlbum(AlbumExtractor albumExtractor) throws IOException {
        ExtraitAlbumDef extraitAlbumDef = albumExtractor.getExtraitAlbumDef();
        String tagName = extraitAlbumDef.getTagName();
        boolean withTag = (tagName != null);
        int albumCount = albumExtractor.getAlbumCount();
        String extraitName = extraitAlbumDef.getExtraitName();
        if (extraitName != null) {
            if (withTag) {
                startOpenTag(tagName);
                addAttribute("extrait-name", extraitName);
                endOpenTag();
            }
            for (int j = 0; j < albumCount; j++) {
                List<IllustrationExtractInfo> list = albumExtractor.getIllustrationExtractInfoList(j);
                for (IllustrationExtractInfo info : list) {
                    illustrationXMLPart.addIllustration(info);
                }
            }
            if (withTag) {
                closeTag(tagName);
            }
        } else {
            for (int j = 0; j < albumCount; j++) {
                Album thesaurus = albumExtractor.getAlbum(j);
                if (withTag) {
                    startOpenTag(tagName);
                    addAttribute("album", thesaurus.getSubsetKey().getSubsetName());
                    endOpenTag();
                }
                List<IllustrationExtractInfo> list = albumExtractor.getIllustrationExtractInfoList(j);
                for (IllustrationExtractInfo info : list) {
                    illustrationXMLPart.addIllustration(info);
                }
                if (withTag) {
                    closeTag(tagName);
                }
            }
        }
    }

}
