/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.FicheExtractInfo;
import net.mapeadores.basedefiches.extraction.api.PackClause;
import net.mapeadores.basedefiches.extraction.groups.FicheGroup;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLPart;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitCorpusXMLPart extends XMLPart {

    private FicheXMLPart ficheXMLPart;
    private FicheItemXMLPart ficheItemXMLPart;
    private PackWriter packWriter = new PackWriter();
    private ExtractionContext extractionContext;

    public ExtraitCorpusXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.ficheXMLPart = new FicheXMLPart(xmlWriter, extractionContext);
        this.ficheItemXMLPart = new FicheItemXMLPart(xmlWriter);
        ficheItemXMLPart.setCompleteMode(extractionContext);
        this.extractionContext = extractionContext;
    }

    public void addExtraitcorpus(CorpusExtractor corpusExtractor) throws IOException {
        int size = corpusExtractor.getExtraitCount();
        for (int j = 0; j < size; j++) {
            CorpusExtractor.Extrait extrait = corpusExtractor.getExtrait(j);
            String tagName = extrait.getTagName();
            if (tagName != null) {
                startOpenTag(tagName);
                String extraitName = extrait.getName();
                if (extraitName != null) {
                    addAttribute("extrait-name", extraitName);
                } else {
                    addAttribute("corpus", extrait.getCorpusKey().getSubsetName());
                }
                endOpenTag();
            }
            packWriter.setCurrentPackClause(extrait.getPackClause());
            FicheGroup[] array = extrait.getGroupArray();
            for (int i = 0; i < array.length; i++) {
                addGroup(array[i]);
            }
            packWriter.flush();
            if (extrait.withCorpusIntitule()) {
                addCorpusIntitule(extrait.getCorpusKey());
            }
            if (tagName != null) {
                closeTag(tagName);
            }
        }
    }

    private void addGroup(FicheGroup group) throws IOException {
        String tagName = group.getTagName();
        if (tagName != null) {
            String name = group.getName();
            startOpenTag(tagName);
            addAttribute("name", name);
            Map<String, String> attributesMap = group.getAttributesMap();
            if (attributesMap != null) {
                for (Map.Entry<String, String> mapEntry : attributesMap.entrySet()) {
                    addAttribute(mapEntry.getKey(), mapEntry.getValue());
                }
            }
            endOpenTag();
            FicheItem[] matchingFicheItemArray = group.getMatchingFicheItemArray();
            if (matchingFicheItemArray != null) {
                for (int i = 0; i < matchingFicheItemArray.length; i++) {
                    FicheItem ficheItem = matchingFicheItemArray[i];
                    ficheItemXMLPart.addFicheItem(ficheItem);
                }
            }
        }
        if (group.isBottomGroup()) {
            FicheExtractInfo[] array = group.getFicheExtractInfoArray();
            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    packWriter.ficheStart();
                    ficheXMLPart.addFiche(array[i]);
                    packWriter.ficheEnd();
                }
            }
        } else {
            FicheGroup[] subgroupArray = group.getSubgroupArray();
            if (subgroupArray != null) {
                for (int i = 0; i < subgroupArray.length; i++) {
                    addGroup(subgroupArray[i]);
                }
            }
        }
        if (tagName != null) {
            packWriter.groupFlush();
            closeTag(tagName);
        }
    }

    private void addCorpusIntitule(SubsetKey corpusKey) throws IOException {
        Corpus corpus = (Corpus) extractionContext.getBasedefiches().getSubset(corpusKey);
        openTag("intitule");
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int workingLang = localisationLists.getWorkingLangInteger(i);
            String lib = CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLang);
            XMLUtils.addLibElement(this, workingLang, lib);
        }
        closeTag("intitule");
    }


    private class PackWriter {

        private PackClause packClause;
        private String tagName;
        private int packSize;
        private int counter = 0;
        private int modulo = 0;
        private int packCounter = 1;

        PackWriter() {
        }

        void setCurrentPackClause(PackClause newPackClause) {
            reset();
            if (newPackClause != null) {
                String tn = newPackClause.getTagName();
                if (tn != null) {
                    int ps = newPackClause.getPackSize();
                    if (ps > 0) {
                        this.packClause = newPackClause;
                        this.tagName = tn;
                        this.packSize = ps;
                        int m = newPackClause.getModulo();
                        if (m < 2) {
                            m = 0;
                        }
                        this.modulo = m;
                    }
                }
            }
        }

        public void ficheStart() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter == 0) {
                startOpenTag(tagName);
                if (modulo > 0) {
                    addAttribute("modulo-val", packCounter);
                    if (packCounter == modulo) {
                        packCounter = 1;
                    } else {
                        packCounter++;
                    }
                }
                endOpenTag();
            }
            counter++;
        }

        public void ficheEnd() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter == packSize) {
                closeTag(tagName);
                counter = 0;
            }
        }

        public void groupFlush() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter > 0) {
                closePack();
            }
        }

        public void flush() throws IOException {
            if (!testPack()) {
                return;
            }
            if (counter > 0) {
                closePack();
            }
            reset();
        }

        private boolean testPack() {
            return (packClause != null);
        }

        private void closePack() throws IOException {
            closeTag(tagName);
            counter = 0;
        }

        private void reset() {
            packClause = null;
            tagName = null;
            counter = 0;
            packSize = 0;
            modulo = 0;
            packCounter = 1;
        }

    }

}
