/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.MotcleExtractInfo;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLPart;


/**
 *
 * @author Vincent Calame
 */
public class ExtraitThesaurusXMLPart extends XMLPart {

    private ExtractionContext extractionContext;
    private MotcleXMLPart motcleXMLPart;

    public ExtraitThesaurusXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
        motcleXMLPart = new MotcleXMLPart(xmlWriter, extractionContext);
    }

    public void addExtraitthesaurus(ThesaurusExtractor thesaurusExtractor) throws IOException {
        ExtraitThesaurusDef extraitThesaurusDef = thesaurusExtractor.getExtraitThesaurusDef();
        String tagName = extraitThesaurusDef.getTagName();
        boolean withTag = (tagName != null);
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int thesaurusCount = thesaurusExtractor.getThesaurusCount();
        String extraitName = extraitThesaurusDef.getExtraitName();
        if (extraitName != null) {
            if (withTag) {
                startOpenTag(tagName);
                addAttribute("extrait-name", extraitName);
                endOpenTag();
            }
            for (int j = 0; j < thesaurusCount; j++) {
                List<MotcleExtractInfo> list = thesaurusExtractor.getMotcleExtractInfoList(j);
                for (MotcleExtractInfo info : list) {
                    motcleXMLPart.addMotcle(info);
                }
            }
            if (withTag) {
                closeTag(tagName);
            }
        } else {
            boolean withIntituleThesaurus = extraitThesaurusDef.getMotcleFilter().isWithIntituleThesaurus();
            for (int j = 0; j < thesaurusCount; j++) {
                Thesaurus thesaurus = thesaurusExtractor.getThesaurus(j);
                if (withTag) {
                    startOpenTag(tagName);
                    addAttribute("thesaurus", thesaurus.getSubsetKey().getSubsetName());
                    addAttribute("rattachement", thesaurusExtractor.getRattachementType(j));
                    endOpenTag();
                }
                List<MotcleExtractInfo> list = thesaurusExtractor.getMotcleExtractInfoList(j);
                for (MotcleExtractInfo info : list) {
                    motcleXMLPart.addMotcle(info);
                }
                if (withIntituleThesaurus) {
                    openTag("intitule");
                    int length = localisationLists.getFormatLocaleCount();
                    for (int i = 0; i < length; i++) {
                        int workingLang = localisationLists.getWorkingLangInteger(i);
                        String lib = ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLang);
                        XMLUtils.addLibElement(this, workingLang, lib);
                    }
                    closeTag("intitule");
                }
                if (withTag) {
                    closeTag(tagName);
                }
            }
        }
    }

}
