/* BasedefichesLib_Xml - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.primitives.DegreSexagesimal;
import net.mapeadores.util.primitives.PrimL10nUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import net.mapeadores.util.text.DateFormatBundle;
import net.mapeadores.util.xml.XMLWriter;
import java.util.Locale;
import java.text.DecimalFormatSymbols;
import java.io.IOException;
import net.mapeadores.basedefiches.corpus.fiche.Courriel;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.fiche.Image;
import net.mapeadores.basedefiches.corpus.fiche.Item;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Link;
import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.fiche.Nombre;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.Pays;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLPart;


/**
 *
 * @author Vincent Calame
 */
public class FicheItemXMLPart extends XMLPart {

    private boolean complete = false;
    private ExtractionContext extractionContext;
    private FicheBlockXMLPart ficheBlockXMLPart;

    public FicheItemXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
        ficheBlockXMLPart = new FicheBlockXMLPart(xmlWriter);
    }

    public void setCompleteMode(ExtractionContext extractionContext) {
        this.extractionContext = extractionContext;
        this.complete = (extractionContext != null);
        ficheBlockXMLPart.setCompleteMode(extractionContext);
    }

    public void addFicheItemList(FicheItemList ficheItemList) throws IOException {
        int size = ficheItemList.getFicheItemCount();
        for (int i = 0; i < size; i++) {
            addFicheItem(ficheItemList.getFicheItem(i), null);
        }
    }

    public void addFicheItemList(FicheItemList ficheItemList, Object fieldOptionObject) throws IOException {
        int size = ficheItemList.getFicheItemCount();
        for (int i = 0; i < size; i++) {
            addFicheItem(ficheItemList.getFicheItem(i), fieldOptionObject);
        }
    }

    public void addFicheItem(FicheItem ficheItem) throws IOException {
        addFicheItem(ficheItem, null);
    }

    public void addFicheItem(FicheItem ficheItem, Object fieldOptionObject) throws IOException {
        if (ficheItem instanceof Item) {
            addItem((Item) ficheItem);
        } else if (ficheItem instanceof Personne) {
            addPersonne((Personne) ficheItem);
        } else if (ficheItem instanceof Datation) {
            addDatation((Datation) ficheItem, fieldOptionObject);
        } else if (ficheItem instanceof Langue) {
            addLangue((Langue) ficheItem);
        } else if (ficheItem instanceof Pays) {
            addPays((Pays) ficheItem);
        } else if (ficheItem instanceof Link) {
            addLink((Link) ficheItem);
        } else if (ficheItem instanceof Courriel) {
            addCourriel((Courriel) ficheItem);
        } else if (ficheItem instanceof Montant) {
            addMontant((Montant) ficheItem);
        } else if (ficheItem instanceof Nombre) {
            addNombre((Nombre) ficheItem);
        } else if (ficheItem instanceof Geopoint) {
            addGeopoint((Geopoint) ficheItem);
        } else if (ficheItem instanceof Para) {
            addPara((Para) ficheItem);
        } else if (ficheItem instanceof Image) {
            addImage((Image) ficheItem, fieldOptionObject);
        } else {
            throw new ClassCastException("unknown instance of FicheItem = " + ficheItem.getClass().getName());
        }
    }

    public void addItem(Item item) throws IOException {
        addSimpleElement("item", item.getValue());
    }

    public void addCourriel(Courriel courriel) throws IOException {
        String addrSpec = courriel.getAddrSpec();
        startOpenTag("courriel");
        addAttribute("addr-spec", addrSpec);
        if (courriel.hasNomComplet()) {
            addAttribute("real-name", courriel.getNomComplet());
        }
        if (complete) {
            int idx = addrSpec.indexOf('@');
            addAttribute("user", addrSpec.substring(0, idx));
            addAttribute("domain", addrSpec.substring(idx + 1));
            endOpenTag();
            escape(courriel.toString());
            closeTag("courriel", false, true);
        } else {
            closeEmptyTag();
        }
    }

    public void addDatation(Datation datation, Object fieldOptionObject) throws IOException {
        short typedatation = datation.getTypeDatation();
        startOpenTag("datation");
        addAttribute("a", String.valueOf(datation.getAnnee()));
        switch (typedatation) {
            case DateInteger.TYPE_SEMESTRE:
                addAttribute("type", "s");
                addAttribute("s", String.valueOf(datation.getSemestre()));
                break;
            case DateInteger.TYPE_TRIMESTRE:
                addAttribute("type", "t");
                addAttribute("t", String.valueOf(datation.getTrimestre()));
                break;
            case DateInteger.TYPE_MOIS:
                addAttribute("type", "m");
                addAttribute("m", String.valueOf(datation.getMois()));
                break;
            case DateInteger.TYPE_JOUR:
                addAttribute("type", "j");
                addAttribute("m", String.valueOf(datation.getMois()));
                addAttribute("j", String.valueOf(datation.getJour()));
                break;
            default:
                addAttribute("type", "a");
        }
        if (complete) {
            addAttribute("sort", DateInteger.toSortString(datation.getDateInteger()));
            endOpenTag();
            addDatationLib(datation.getDateInteger(), fieldOptionObject);
            closeTag("datation");
        } else {
            closeEmptyTag();
        }
    }

    private void addDatationLib(int dateInteger, Object fieldOptionObject) throws IOException {
        if ((fieldOptionObject != null) && (fieldOptionObject instanceof Short)) {
            short libType = ((Short) fieldOptionObject).shortValue();
            dateInteger = DateInteger.troncateDateInteger(dateInteger, libType);
        }
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            DateFormatBundle dateFormatBundle = DateFormatBundle.getDateFormatBundle(locale);
            XMLUtils.addLibElement(this, localeInteger, DateInteger.getDateLibelle(dateInteger, dateFormatBundle));
        }
    }

    public void addLangue(Langue langue) throws IOException {
        startOpenTag("langue");
        addAttribute("lang", LangInteger.toString(langue.getLangInteger()));
        if (complete) {
            endOpenTag();
            addLangueLib(langue.getLangInteger());
            closeTag("langue");
        } else {
            closeEmptyTag();
        }
    }

    private void addLangueLib(int langInteger) throws IOException {
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            XMLUtils.addLibElement(this, localeInteger, LangIntegerLocalisation.getLangLibelle(langInteger, locale));
        }
    }

    public void addNombre(Nombre nombre) throws IOException {
        startOpenTag("nombre");
        Decimal decimal = nombre.getDecimal();
        addDecimalAttributes(decimal);
        if (complete) {
            endOpenTag();
            addNombreLib(decimal);
            closeTag("nombre");
        } else {
            closeEmptyTag();
        }
    }

    public void addGeopoint(Geopoint geopoint) throws IOException {
        startOpenTag("geopoint");
        DegreDecimal latitude = geopoint.getLatitude();
        DegreDecimal longitude = geopoint.getLongitude();
        addAttribute("lat", latitude.toString());
        addAttribute("lon", longitude.toString());
        if (complete) {
            endOpenTag();
            DegreSexagesimal latitudeSexa = DegreSexagesimal.fromDegreDecimal(latitude);
            DegreSexagesimal longitudeSexa = DegreSexagesimal.fromDegreDecimal(longitude);
            startOpenTag("latitude");
            addSexagesimalAttributes(latitudeSexa);
            endOpenTag();
            addSexagesimalLib(latitudeSexa, true);
            closeTag("latitude");
            startOpenTag("longitude");
            addSexagesimalAttributes(longitudeSexa);
            endOpenTag();
            addSexagesimalLib(longitudeSexa, false);
            closeTag("longitude");
            closeTag("geopoint");
        } else {
            closeEmptyTag();
        }
    }

    private void addSexagesimalAttributes(DegreSexagesimal sexa) throws IOException {
        addAttribute("deg", sexa.getDegre());
        addAttribute("mn", sexa.getMinute());
        addAttribute("sec", sexa.getSeconde());
    }

    private void addSexagesimalLib(DegreSexagesimal sexa, boolean latitude) throws IOException {
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            XMLUtils.addLibElement(this, localeInteger, PrimL10nUtils.getLib(sexa, locale, latitude));
        }
    }

    private void addNombreLib(Decimal decimal) throws IOException {
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            DecimalFormatSymbols symbols = localisationLists.getDecimalFormatSymbols(i);
            String strg = decimal.toString(symbols);
            XMLUtils.addLibElement(this, localeInteger, strg);
        }

    }

    public void addMontant(Montant montant) throws IOException {
        startOpenTag("montant");
        Decimal decimal = montant.getDecimal();
        addDecimalAttributes(decimal);
        addAttribute("cur", CurrencyInteger.toString(montant.getCurrencyInteger()));
        if (complete) {
            endOpenTag();
            addMontantLib(decimal, montant.getCurrencyInteger());
            closeTag("montant");
        } else {
            closeEmptyTag();
        }
    }

    private void addMontantLib(Decimal decimal, int currencyInteger) throws IOException {
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            DecimalFormatSymbols symbols = localisationLists.getDecimalFormatSymbols(i);
            String strg = CurrencyInteger.toString(decimal, currencyInteger, symbols);
            XMLUtils.addLibElement(this, localeInteger, strg);
        }

    }

    private void addDecimalAttributes(Decimal decimal) throws IOException {
        String partieEntiere = decimal.getPartieEntiereString();
        String partieDecimale = decimal.getPartieDecimaleString();
        if (complete) {
            addAttribute("int", partieEntiere);
            addAttribute("dec", partieDecimale);
        }
        String val = (partieDecimale.length() != 0) ? partieEntiere + "." + partieDecimale : partieEntiere;
        addAttribute("val", val);
    }

    public void addLink(Link link) throws IOException {
        startOpenTag("link");
        addAttribute("href", link.getHref());
        if ((!complete) && (link.hasHrefOnly())) {
            closeEmptyTag();
        } else {
            endOpenTag();
            String title = link.getTitle();
            if ((title.length() == 0) && (complete)) {
                title = link.getHref();
                if ((title.startsWith("http://")) && (title.length() > 8)) {
                    title = title.substring(7);
                }
            }
            addSimpleElement("title", title);
            addSimpleElement("comment", link.getComment());
            closeTag("link");
        }
    }

    public void addImage(Image image, Object fieldOptionObject) throws IOException {
        startOpenTag("image");
        addAttribute("src", image.getSrc());
        if ((!complete) && (image.hasSrcOnly())) {
            closeEmptyTag();
        } else {
            if ((complete) && (fieldOptionObject != null) && (fieldOptionObject instanceof String)) {
                if (!StringUtils.isAbsoluteUrlString(image.getSrc())) {
                    addAttribute("base-url", (String) fieldOptionObject);
                }
            }
            endOpenTag();
            addSimpleElement("alt", image.getAlt());
            addSimpleElement("title", image.getTitle());
            closeTag("image");
        }
    }

    public void addPays(Pays pays) throws IOException {
        startOpenTag("pays");
        addAttribute("country", pays.getCountryCode());
        if (complete) {
            endOpenTag();
            addPaysLib(pays.getCountryInteger());
            closeTag("pays");
        } else {
            closeEmptyTag();
        }
    }

    private void addPaysLib(int paysLangInteger) throws IOException {
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        int length = localisationLists.getFormatLocaleCount();
        for (int i = 0; i < length; i++) {
            int localeInteger = localisationLists.getFormatLocaleInteger(i);
            Locale locale = localisationLists.getFormatLocale(i);
            XMLUtils.addLibElement(this, localeInteger, LangIntegerLocalisation.getCountryLibelle(paysLangInteger, locale));
        }

    }

    public void addPersonne(Personne personne) throws IOException {
        startOpenTag("personne");
        if (personne.isRedacteurSphere()) {
            try {
                RedacteurKey redacteurKey = RedacteurKey.parse(personne.getRedacteurKeyString());
                addAttribute("sphere", redacteurKey.getSphereKey().getSubsetName());
                addAttribute("idsphere", redacteurKey.getIdsphereString());
                if (complete) {
                    Redacteur redacteur = SphereUtils.getRedacteur(extractionContext.getBasedefiches(), redacteurKey);
                    if (redacteur != null) {
                        endOpenTag();
                        addPersonneBean(redacteur.getPersonneBean());
                        closeTag("personne");
                    } else {
                        closeEmptyTag();
                    }
                } else {
                    closeEmptyTag();
                }
            } catch (java.text.ParseException pe) {
                closeEmptyTag();
            }
        } else {
            endOpenTag();
            addPersonneBean(personne.getPersonneBean());
            addSimpleElement("organisme", personne.getOrganisme());
            closeTag("personne");
        }
    }

    private void addPersonneBean(PersonneBean personneBean) throws IOException {
        String nom = personneBean.getNom();
        if (nom.length() > 0) {
            startOpenTag("nom");
            boolean nomAvant = personneBean.isNomAvant();
            if (nomAvant) {
                addAttribute("avant", "true");
            }
            endOpenTag(false);
            escape(nom);
            closeTag("nom", false, false);
        }
        addSimpleElement("prenom", personneBean.getPrenom());
        addSimpleElement("original", personneBean.getOriginal());
    }

    public void addPara(Para para) throws IOException {
        startOpenTag("para");
        endOpenTag(false);
        ficheBlockXMLPart.addTextContentList(para);
        closeTag("para", false, false);
    }

}
