/* BasedefichesLib_Xml - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import net.mapeadores.basedefiches.corpus.fiche.Section;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class FicheStorageXMLWriter extends UtilXMLWriter {

    private FicheItemXMLPart ficheItemXMLPart;
    private FicheBlockXMLPart ficheBlockXMLPart;

    public FicheStorageXMLWriter() {
        ficheItemXMLPart = new FicheItemXMLPart(this);
        ficheBlockXMLPart = new FicheBlockXMLPart(this);
    }

    public void appendFiche(Fiche fiche) throws IOException {
        startOpenTag("fiche");
        addXmlLangAttribute(fiche.getLangInteger());
        endOpenTag();
        addEntete(fiche);
        addCorpsdefiche(fiche);
        closeTag("fiche");
    }

    private void addEntete(Fiche fiche) throws IOException {
        openTag("entete");
        addSimpleElement("titre", fiche.getTitre());
        Para soustitrePara = fiche.getSoustitre();
        if (soustitrePara != null) {
            startOpenTag("soustitre");
            endOpenTag(false);
            ficheBlockXMLPart.addTextContentList(soustitrePara);
            closeTag("soustitre", false, false);
        }
        int proprieteCount = fiche.getProprieteCount();
        for (int i = 0; i < proprieteCount; i++) {
            addPropriete(fiche.getPropriete(i));
        }
        int informationCount = fiche.getInformationCount();
        for (int i = 0; i < informationCount; i++) {
            addInformation(fiche.getInformation(i));
        }
        FicheItemList redacteursItemList = fiche.getRedacteurs();
        if (redacteursItemList != null) {
            openTag("redacteurs");
            ficheItemXMLPart.addFicheItemList(redacteursItemList);
            closeTag("redacteurs");
        }
        closeTag("entete");
    }

    private void addCorpsdefiche(Fiche fiche) throws IOException {
        int sectionCount = fiche.getSectionCount();
        if (sectionCount == 0) {
            return;
        }
        openTag("corpsdefiche");
        for (int i = 0; i < sectionCount; i++) {
            addSection(fiche.getSection(i));
        }
        closeTag("corpsdefiche");

    }

    private void addPropriete(Propriete propriete) throws IOException {
        FicheItem ficheItem = propriete.getFicheItem();
        startOpenTag("propriete");
        addAttribute("type", propriete.getFieldKey().getTypeString());
        endOpenTag();
        ficheItemXMLPart.addFicheItem(ficheItem);
        closeTag("propriete");
    }

    private void addInformation(Information information) throws IOException {
        startOpenTag("information");
        addAttribute("type", information.getFieldKey().getTypeString());
        endOpenTag();
        ficheItemXMLPart.addFicheItemList(information);
        closeTag("information");
    }

    private void addSection(Section section) throws IOException {
        startOpenTag("section");
        addAttribute("type", section.getFieldKey().getTypeString());
        endOpenTag();
        ficheBlockXMLPart.addFicheBlockList(section);
        closeTag("section");
    }

}
