/* BasedefichesLib_Xml - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import java.util.Locale;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Datation;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Langue;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsConstants;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.extraction.AddendaExtractor;
import net.mapeadores.basedefiches.extraction.AlbumExtractor;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.FicheExtractInfo;
import net.mapeadores.basedefiches.extraction.FieldTypeInfo;
import net.mapeadores.basedefiches.extraction.ThesaurusExtractor;
import net.mapeadores.basedefiches.extraction.api.ExtraitAddendaDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.filterunit.ChronoFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.CorpsdeficheFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.EnteteFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.ExtraitAddendaFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.ExtraitAlbumFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.ExtraitCorpusFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.ExtraitThesaurusFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.FieldKeyFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitList;
import net.mapeadores.basedefiches.extraction.filterunit.IntituleFicheFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.LotFilterUnit;
import net.mapeadores.basedefiches.extraction.filterunit.TypePrefixFilterUnit;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.selection.DocumentSelector;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.IllustrationSelector;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.alphabet.Alphabet;
import net.mapeadores.util.xml.DefaultXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLPart;


/**
 *
 * @author Vincent Calame
 */
public class FicheXMLPart extends XMLPart {

    private FilterUnitXMLPart filterUnitPart;
    private FicheFilter ficheFilter;
    private FichePointeur fichePointeur;

    public FicheXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        filterUnitPart = new FilterUnitXMLPart(xmlWriter, extractionContext);
    }

    public void addFiche(FicheExtractInfo ficheExtractInfo) throws IOException {
        updatePointeur(ficheExtractInfo);
        ficheFilter = ficheExtractInfo.getFicheFilter();
        AbregeFiche abregeFiche = ficheExtractInfo.getAbregeFiche();

        String tagName = ficheFilter.getTagName();
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("idcorpus", String.valueOf(abregeFiche.getIdcorpus()));
            XMLUtils.addXmlLangAttribute(this, abregeFiche.getLangInteger());
            int poids = ficheExtractInfo.getPoids();
            if (poids > 0) {
                addAttribute("poids", String.valueOf(poids));
            }
            addAttribute("corpus", abregeFiche.getCorpus().getSubsetKey().getSubsetName());
            if (ficheFilter.isEmptyFicheFilter()) {
                closeEmptyTag();
                return;
            }
            endOpenTag();
        }
        addGroupClauseObject(ficheExtractInfo.getGroupClauseObject());
        filterUnitPart.addFilterUnitList(ficheFilter);
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void updatePointeur(FicheExtractInfo ficheExtractInfo) {
        boolean ficheComplete = (ficheExtractInfo.getFicheFilter().isWithCorpsdefiche());
        AbregeFiche abregeFiche = ficheExtractInfo.getAbregeFiche();
        Corpus corpus = abregeFiche.getCorpus();
        if (fichePointeur != null) {
            if (!fichePointeur.getSubset().equals(corpus)) {
                fichePointeur = null;
            } else if (fichePointeur.isFicheComplete() != ficheComplete) {
                fichePointeur = null;
            }
        }
        if (fichePointeur == null) {
            fichePointeur = new FichePointeur(corpus, ficheComplete);
        }
        fichePointeur.setCurrentSubsetItem(abregeFiche);
        filterUnitPart.updatePointeur(fichePointeur);
    }

    public void addGroupClauseObject(Object groupClauseObject) throws IOException {
        if (groupClauseObject == null) {
            return;
        }
        if (groupClauseObject instanceof Alphabet) {
            Alphabet alphabet = (Alphabet) groupClauseObject;
            startOpenTag("alphabet");
            if (alphabet.isWithArticle()) {
                addAttribute("article", alphabet.getArticleString());
                addAttribute("espace", (alphabet.isWithEspace()) ? 1 : 0);
            }
            addAttribute("titre", alphabet.getAlphabetString());
            closeEmptyTag();
        }
    }


    private static class FilterUnitXMLPart extends XMLPart {

        private ExtractionContext extractionContext;
        FicheItemXMLPart ficheItemXMLPart;
        FicheBlockXMLPart ficheBlockXMLPart;
        ExtraitCorpusXMLPart extraitCorpusXMLPart = null;
        ExtraitThesaurusXMLPart extraitThesaurusXMLPart = null;
        ExtraitAlbumXMLPart extraitAlbumXMLPart = null;
        ExtraitAddendaXMLPart extraitAddendaXMLPart = null;
        FichePointeur fichePointeur;

        FilterUnitXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
            super(xmlWriter);
            this.extractionContext = extractionContext;
            ficheItemXMLPart = new FicheItemXMLPart(xmlWriter);
            ficheBlockXMLPart = new FicheBlockXMLPart(xmlWriter);
            ficheItemXMLPart.setCompleteMode(extractionContext);
            ficheBlockXMLPart.setCompleteMode(extractionContext);
        }

        private void updatePointeur(FichePointeur fichePointeur) {
            this.fichePointeur = fichePointeur;
            ficheBlockXMLPart.setDefaultCorpus((Corpus) fichePointeur.getSubset());
        }

        private void addFilterUnitList(FilterUnitList filterUnitList) throws IOException {
            int filterUnitCount = filterUnitList.getFilterUnitCount();
            for (int i = 0; i < filterUnitCount; i++) {
                FilterUnit filterUnit = filterUnitList.getFilterUnit(i);
                if (filterUnit instanceof EnteteFilterUnit) {
                    addEntete();
                } else if (filterUnit instanceof CorpsdeficheFilterUnit) {
                    addCorpsdefiche();
                } else if (filterUnit instanceof FieldKeyFilterUnit) {
                    FieldKeyFilterUnit fieldKeyFilterUnit = (FieldKeyFilterUnit) filterUnit;
                    addFieldKey(fieldKeyFilterUnit.getFieldKey(), fieldKeyFilterUnit.getAttributes());
                } else if (filterUnit instanceof TypePrefixFilterUnit) {
                    addPrefixedField((TypePrefixFilterUnit) filterUnit);
                } else if (filterUnit instanceof LotFilterUnit) {
                    addLot((LotFilterUnit) filterUnit);
                } else if (filterUnit instanceof ChronoFilterUnit) {
                    addChrono();
                } else if (filterUnit instanceof IntituleFicheFilterUnit) {
                    addIntituleFiche();
                } else if (filterUnit instanceof ExtraitAddendaFilterUnit) {
                    addExtraitAddenda(((ExtraitAddendaFilterUnit) filterUnit).getExtraitAddendaDef());
                } else if (filterUnit instanceof ExtraitCorpusFilterUnit) {
                    addExtraitCorpus(((ExtraitCorpusFilterUnit) filterUnit).getExtraitCorpusDef());
                } else if (filterUnit instanceof ExtraitThesaurusFilterUnit) {
                    addExtraitThesaurus(((ExtraitThesaurusFilterUnit) filterUnit).getExtraitThesaurusDef());
                } else if (filterUnit instanceof ExtraitAlbumFilterUnit) {
                    addExtraitAlbum(((ExtraitAlbumFilterUnit) filterUnit).getExtraitAlbumDef());
                }
            }
        }

        private void addLot(LotFilterUnit lotFilterUnit) throws IOException {
            StringBuilder buf = new StringBuilder();
            DefaultXMLWriter xw = new DefaultXMLWriter();
            xw.setAppendable(buf);
            xw.setIndentLength(getCurrentIndentValue() + 1);
            FilterUnitXMLPart child = new FilterUnitXMLPart(xw, extractionContext);
            child.updatePointeur(fichePointeur);
            child.addFilterUnitList(lotFilterUnit.getFilterUnitList());
            String result = buf.toString();
            if (result.length() > 0) {
                String tagName = lotFilterUnit.getTagName();
                if (tagName == null) {
                    tagName = "lot";
                }
                boolean withTagName = (!tagName.equals("NULL"));
                if (withTagName) {
                    openTag(tagName);
                }
                append(result);
                if (withTagName) {
                    closeTag(tagName);
                }
            }
        }

        private void addEntete() throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            addTitre();
            addSoustitre();
            int proprieteCount = corpusMetadata.getProprieteFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                addPropriete(corpusField.getFieldKey(), null, getFieldOptionObject(corpusField, null));
            }
            int informationCount = corpusMetadata.getInformationFieldCount();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                addInformation(corpusField.getFieldKey(), null, getFieldOptionObject(corpusField, null));
            }
            addRedacteurs();
        }

        private Object getFieldOptionObject(CorpusField corpusField, AttributeHolder attributes) {
            short type = corpusField.getFicheItemType();
            if (type == CorpusField.IMAGE_FIELD) {
                return corpusField.getStringOption(FieldOptionsConstants.BASEURL_OPTION);
            } else if (type == CorpusField.DATATION_FIELD) {
                if (attributes != null) {
                    Attribute attribute = attributes.getAttributeByKey(ExtractionAttributes.LIBTYPE_ATTRIBUTEKEY);
                    if (attribute != null) {
                        String value = attribute.getValue(0);
                        try {
                            short libType = DateInteger.typeToShort(value);
                            return new Short(libType);
                        } catch (IllegalArgumentException iae) {
                        }
                    }
                }
            }
            return null;
        }

        private void addCorpsdefiche() throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            int sectionCount = corpusMetadata.getSectionFieldCount();
            for (int i = 0; i < sectionCount; i++) {
                addSection(corpusMetadata.getSectionField(i).getFieldKey(), null);
            }
        }

        private void addFieldKey(FieldKey fieldKey, AttributeHolder attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (corpusField == null) {
                return;
            }
            if (fieldKey.equals(FieldKey.TITRE)) {
                addTitre();
            } else if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                addSoustitre();
            } else if (fieldKey.equals(FieldKey.REDACTEURS)) {
                addRedacteurs();
            } else if (fieldKey.equals(FieldKey.LANG)) {
                addLang();
            } else if (fieldKey.isProprieteFieldKey()) {
                addPropriete(fieldKey, null, getFieldOptionObject(corpusField, attributes));
            } else if (fieldKey.isInformationFieldKey()) {
                addInformation(fieldKey, null, getFieldOptionObject(corpusField, attributes));
            } else if (fieldKey.isSectionFieldKey()) {
                addSection(fieldKey, null);
            }
        }

        private void addPrefixedField(TypePrefixFilterUnit filterUnit) throws IOException {
            AttributeHolder attributes = filterUnit.getAttributes();
            String prefix = filterUnit.getPrefix();
            switch (filterUnit.getCategory()) {
                case FieldKey.PROPRIETE_CATEGORY:
                    addPrefixedPropriete(prefix, attributes);
                    break;
                case FieldKey.INFORMATION_CATEGORY:
                    addPrefixeInformation(prefix, attributes);
                    break;
                case FieldKey.SECTION_CATEGORY:
                    addPrefixedSection(prefix, attributes);
                    break;
            }
        }

        private void addPrefixedPropriete(String prefix, AttributeHolder attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            int proprieteCount = corpusMetadata.getProprieteFieldCount();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldTypeInfo fieldTypeInfo = FieldTypeInfo.testPrefix(fieldKey, prefix);
                if (fieldTypeInfo != null) {
                    addPropriete(fieldKey, fieldTypeInfo, getFieldOptionObject(corpusField, attributes));
                }
            }
        }

        private void addPrefixeInformation(String prefix, AttributeHolder attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            int informationCount = corpusMetadata.getInformationFieldCount();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                FieldKey fieldKey = corpusField.getFieldKey();
                FieldTypeInfo fieldTypeInfo = FieldTypeInfo.testPrefix(fieldKey, prefix);
                if (fieldTypeInfo != null) {
                    addInformation(fieldKey, fieldTypeInfo, getFieldOptionObject(corpusField, attributes));
                }
            }
        }

        private void addPrefixedSection(String prefix, AttributeHolder attributes) throws IOException {
            CorpusMetadata corpusMetadata = getCorpusMetatdata();
            int sectionCount = corpusMetadata.getSectionFieldCount();
            for (int i = 0; i < sectionCount; i++) {
                FieldKey fieldKey = corpusMetadata.getSectionField(i).getFieldKey();
                FieldTypeInfo fieldTypeInfo = FieldTypeInfo.testPrefix(fieldKey, prefix);
                if (fieldTypeInfo != null) {
                    addSection(fieldKey, fieldTypeInfo);
                }
            }
        }

        private void addTitre() throws IOException {
            String titre = (String) fichePointeur.getValue(FieldKey.TITRE);
            if (titre == null) {
                return;
            }
            openTag("titre");
            addSimpleElement("item", titre);
            closeTag("titre");
        }

        private void addSoustitre() throws IOException {
            Para para = (Para) fichePointeur.getValue(FieldKey.SOUSTITRE);
            if (para == null) {
                return;
            }
            openTag("soustitre");
            ficheItemXMLPart.addPara(para);
            closeTag("soustitre");
        }

        private void addRedacteurs() throws IOException {
            FicheItemList ficheItemList = (FicheItemList) fichePointeur.getValue(FieldKey.REDACTEURS);
            if (ficheItemList == null) {
                return;
            }
            openTag("redacteurs");
            ficheItemXMLPart.addFicheItemList(ficheItemList);
            closeTag("redacteurs");
        }

        private void addPropriete(FieldKey fieldKey, FieldTypeInfo fieldTypeInfo, Object fieldOptionObject) throws IOException {
            FicheItem ficheItem = (FicheItem) fichePointeur.getValue(fieldKey);
            if (ficheItem == null) {
                return;
            }
            startOpenTag("propriete");
            addAttribute("type", fieldKey.getTypeString());
            if (fieldTypeInfo != null) {
                addAttribute("part1", fieldTypeInfo.getPart1());
                addAttribute("part2", fieldTypeInfo.getPart2());
            }
            endOpenTag();
            ficheItemXMLPart.addFicheItem(ficheItem, fieldOptionObject);
            closeTag("propriete");
        }

        private void addInformation(FieldKey fieldKey, FieldTypeInfo fieldTypeInfo, Object fieldOptionObject) throws IOException {
            FicheItemList ficheItemList = (FicheItemList) fichePointeur.getValue(fieldKey);
            if (ficheItemList == null) {
                return;
            }
            startOpenTag("information");
            addAttribute("type", fieldKey.getTypeString());
            if (fieldTypeInfo != null) {
                addAttribute("part1", fieldTypeInfo.getPart1());
                addAttribute("part2", fieldTypeInfo.getPart2());
            }
            endOpenTag();
            ficheItemXMLPart.addFicheItemList(ficheItemList, fieldOptionObject);
            closeTag("information");
        }

        private void addSection(FieldKey fieldKey, FieldTypeInfo fieldTypeInfo) throws IOException {
            FicheBlockList blockList = (FicheBlockList) fichePointeur.getValue(fieldKey);
            if (blockList == null) {
                return;
            }
            startOpenTag("section");
            addAttribute("type", fieldKey.getTypeString());
            if (fieldTypeInfo != null) {
                addAttribute("part1", fieldTypeInfo.getPart1());
                addAttribute("part2", fieldTypeInfo.getPart2());
            }
            endOpenTag();
            ficheBlockXMLPart.addFicheBlockList(blockList);
            closeTag("section");
        }

        private void addChrono() throws IOException {
            AbregeFiche abregeFiche = getAbregeFiche();
            int creationDate = abregeFiche.getCreationDate();
            if (creationDate != 0) {
                openTag("chrono");
                ficheItemXMLPart.addDatation(new Datation(creationDate), null);
                int modificationDate = abregeFiche.getModificationDate();
                if (modificationDate != creationDate) {
                    ficheItemXMLPart.addDatation(new Datation(modificationDate), null);
                }
                closeTag("chrono");
            }
        }

        private void addLang() throws IOException {
            openTag("lang");
            ficheItemXMLPart.addLangue(new Langue(getAbregeFiche().getLangInteger()));
            closeTag("lang");
        }

        private void addIntituleFiche() throws IOException {
            LocalisationLists localisationLists = extractionContext.getLocalisationLists();
            openTag("intitule");
            int length = localisationLists.getFormatLocaleCount();
            for (int i = 0; i < length; i++) {
                int workingLang = localisationLists.getWorkingLangInteger(i);
                Locale locale = localisationLists.getFormatLocale(i);
                String lib = CorpusMetadataUtils.getFicheNumberString(getAbregeFiche(), workingLang, locale);
                XMLUtils.addLibElement(this, workingLang, lib);
            }
            closeTag("intitule");
        }

        private void addExtraitCorpus(ExtraitCorpusDef extraitCorpusDef) throws IOException {
            Corpus currentCorpus = (Corpus) fichePointeur.getSubset();
            Basedefiches basedefiches = extractionContext.getBasedefiches();
            SubsetKey subsetKey1 = currentCorpus.getSubsetKey();
            int ticket1 = getAbregeFiche().getTicket();
            CorpusExtractor corpusExtractor = new CorpusExtractor(extraitCorpusDef);
            FicheSelector ficheSelector = new FicheSelector(basedefiches, extractionContext.getLocalisationLists().getWorkingLangInteger(0));
            ficheSelector.setCurrentCorpus(currentCorpus);
            ficheSelector.setFicheEligibility(extractionContext.getFicheEligibility());
            ficheSelector.addFicheSelectOptionsList(extraitCorpusDef.getFicheSelectOptionsList());
            int corpusCount = ficheSelector.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                Corpus other = ficheSelector.getCorpus(j);
                SubsetKey subsetKey2 = other.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(subsetKey1, subsetKey2);
                CroisementItem croisementItem = croisement.getCroisementItem(subsetKey1, ticket1);
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket2 = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    AbregeFiche abrege2 = (AbregeFiche) other.getSubsetItemByTicket(ticket2);
                    if (abrege2 != null) {
                        if (ficheSelector.isSelected(abrege2, poids)) {
                            corpusExtractor.add(abrege2, poids);
                        }
                    }
                }
            }
            if (!corpusExtractor.containsFiche()) {
                return;
            }
            if (extraitCorpusXMLPart == null) {
                extraitCorpusXMLPart = new ExtraitCorpusXMLPart(getXMLWriter(), extractionContext);
            }
            extraitCorpusXMLPart.addExtraitcorpus(corpusExtractor);
        }

        private void addExtraitThesaurus(ExtraitThesaurusDef extraitThesaurusDef) throws IOException {
            Corpus currentCorpus = (Corpus) fichePointeur.getSubset();
            Basedefiches basedefiches = extractionContext.getBasedefiches();
            SubsetKey corpusKey = currentCorpus.getSubsetKey();
            int ticket1 = getAbregeFiche().getTicket();
            ThesaurusExtractor thesaurusExtractor = new ThesaurusExtractor(extraitThesaurusDef, currentCorpus);
            MotcleSelector motcleSelector = new MotcleSelector(basedefiches, extractionContext.getLocalisationLists().getWorkingLangInteger(0));
            motcleSelector.addMotcleSelectOptionsList(extraitThesaurusDef.getMotcleSelectOptionsList());
            int thesaurusCount = motcleSelector.getThesaurusCount();
            for (int j = 0; j < thesaurusCount; j++) {
                Thesaurus thesaurus = motcleSelector.getThesaurus(j);
                SubsetKey thesaurusKey = thesaurus.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(corpusKey, thesaurusKey);
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ticket1);
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket2 = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(ticket2);
                    if (motcle != null) {
                        if (motcleSelector.isSelected(motcle, poids)) {
                            thesaurusExtractor.add(motcle, poids);
                        }
                    }
                }
            }
            if (!thesaurusExtractor.containsMotcle()) {
                return;
            }
            if (extraitThesaurusXMLPart == null) {
                extraitThesaurusXMLPart = new ExtraitThesaurusXMLPart(getXMLWriter(), extractionContext);
            }
            extraitThesaurusXMLPart.addExtraitthesaurus(thesaurusExtractor);
        }

        private void addExtraitAlbum(ExtraitAlbumDef extraitAlbumDef) throws IOException {
            Corpus currentCorpus = (Corpus) fichePointeur.getSubset();
            Basedefiches basedefiches = extractionContext.getBasedefiches();
            SubsetKey corpusKey = currentCorpus.getSubsetKey();
            int ticket1 = getAbregeFiche().getTicket();
            AlbumExtractor albumExtractor = new AlbumExtractor(extraitAlbumDef);
            IllustrationSelector illustrationSelector = new IllustrationSelector(basedefiches);
            illustrationSelector.addIllustrationSelectOptionsList(extraitAlbumDef.getIllustrationSelectOptionsList());
            int albumCount = illustrationSelector.getAlbumCount();
            for (int j = 0; j < albumCount; j++) {
                Album album = illustrationSelector.getAlbum(j);
                SubsetKey albumKey = album.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(corpusKey, albumKey);
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ticket1);
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket2 = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    Illustration illustration = (Illustration) album.getSubsetItemByTicket(ticket2);
                    if (illustration != null) {
                        if (illustrationSelector.isSelected(illustration, poids)) {
                            albumExtractor.add(illustration, poids);
                        }
                    }
                }
            }
            if (!albumExtractor.containsIllustration()) {
                return;
            }
            if (extraitAlbumXMLPart == null) {
                extraitAlbumXMLPart = new ExtraitAlbumXMLPart(getXMLWriter());
            }
            extraitAlbumXMLPart.addExtraitAlbum(albumExtractor);
        }

        private void addExtraitAddenda(ExtraitAddendaDef extraitAddendaDef) throws IOException {
            Corpus currentCorpus = (Corpus) fichePointeur.getSubset();
            Basedefiches basedefiches = extractionContext.getBasedefiches();
            SubsetKey corpusKey = currentCorpus.getSubsetKey();
            int ticket1 = getAbregeFiche().getTicket();
            AddendaExtractor addendaExtractor = new AddendaExtractor(extraitAddendaDef);
            DocumentSelector documentSelector = new DocumentSelector(basedefiches);
            documentSelector.addDocumentSelectOptionsList(extraitAddendaDef.getDocumentSelectOptionsList());
            int addendaCount = documentSelector.getAddendaCount();
            for (int j = 0; j < addendaCount; j++) {
                Addenda addenda = documentSelector.getAddenda(j);
                SubsetKey addendaKey = addenda.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(corpusKey, addendaKey);
                CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, ticket1);
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket2 = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    Document document = (Document) addenda.getSubsetItemByTicket(ticket2);
                    if (document != null) {
                        if (documentSelector.isSelected(document, poids)) {
                            addendaExtractor.add(document, poids);
                        }
                    }
                }
            }
            if (!addendaExtractor.containsDocument()) {
                return;
            }
            if (extraitAddendaXMLPart == null) {
                extraitAddendaXMLPart = new ExtraitAddendaXMLPart(getXMLWriter(), extractionContext);
            }
            extraitAddendaXMLPart.addExtraitAddenda(addendaExtractor);
        }

        private AbregeFiche getAbregeFiche() {
            return (AbregeFiche) fichePointeur.getCurrentSubsetItem();
        }

        private CorpusMetadata getCorpusMetatdata() {
            return ((Corpus) fichePointeur.getSubset()).getCorpusMetadata();
        }

    }

}
