/* BasedefichesLib_Xml - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.extraction;

import java.io.IOException;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.extraction.CorpusExtractor;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.MotcleExtractInfo;
import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDefList;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLPart;


/**
 *
 * @author Vincent Calame
 */
public class MotcleXMLPart extends XMLPart {

    private ExtractionContext extractionContext;
    private ExtraitCorpusXMLPart extraitCorpusXMLPart = null;

    public MotcleXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
    }

    public void addMotcle(MotcleExtractInfo motcleExtractInfo) throws IOException {
        Motcle motcle = motcleExtractInfo.getMotcle();
        MotcleFilter motcleFilter = motcleExtractInfo.getMotcleFilter();
        String tagName = motcleFilter.getTagName();
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("idths", String.valueOf(motcle.getIdths()));
            String idalpha = motcle.getIdalpha();
            if (idalpha != null) {
                addAttribute("idalpha", idalpha);
            }
            int poids = motcleExtractInfo.getPoids();
            if (poids > 0) {
                addAttribute("poids", String.valueOf(poids));
            }
            addAttribute("thesaurus", motcle.getThesaurus().getSubsetKey().getSubsetName());
            if (motcleFilter.isNoneFiltering()) {
                closeEmptyTag();
                return;
            }
            endOpenTag();
        } else if (motcleFilter.isNoneFiltering()) {
            return;
        }
        if (motcleFilter.withLevel()) {
            addSimpleElement("level", String.valueOf(motcle.getLevel()));
        }
        if (motcleFilter.withLib()) {
            XMLUtils.addLibelleList(this, motcle, null);
        }
        MotcleFilter childrenFilter = motcleFilter.getChildrenFilter();
        if (childrenFilter != null) {
            MotcleList childList = motcle.getChildList();
            for (int i = 0; i < childList.getMotcleCount(); i++) {
                Motcle child = childList.getMotcle(i);
                MotcleExtractInfo childExtractInfo = new InternalMotcleExtractInfo(child, childrenFilter);
                addMotcle(childExtractInfo);
            }
        }
        MotcleFilter parentFilter = motcleFilter.getParentFilter();
        if (parentFilter != null) {
            Motcle parentMotcle = motcle.getParent();
            addLinkedMotcle(parentMotcle, motcleFilter.getParentFilter());
        }
        MotcleFilter previousFilter = motcleFilter.getPreviousFilter();
        if (previousFilter != null) {
            Motcle previousMotcle = ThesaurusUtils.getPrevious(motcle);
            addLinkedMotcle(previousMotcle, previousFilter);
        }
        MotcleFilter nextFilter = motcleFilter.getNextFilter();
        if (nextFilter != null) {
            Motcle nextMotcle = ThesaurusUtils.getNext(motcle);
            addLinkedMotcle(nextMotcle, nextFilter);
        }
        addIndexation(motcle, motcleFilter);
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void addLinkedMotcle(Motcle linkedMotcle, MotcleFilter linkedMotcleFilter) throws IOException {
        if (linkedMotcle == null) {
            return;
        }
        addMotcle(new InternalMotcleExtractInfo(linkedMotcle, linkedMotcleFilter));
    }

    private void addIndexation(Motcle motcle, MotcleFilter motcleFilter) throws IOException {
        ExtraitCorpusDefList extraitCorpusDefList = motcleFilter.getIndexationExtraitCorpusDefList();
        int count = extraitCorpusDefList.getExtraitCorpusDefCount();
        if (count == 0) {
            return;
        }
        Basedefiches basedefiches = extractionContext.getBasedefiches();
        LocalisationLists localisationLists = extractionContext.getLocalisationLists();
        SubsetKey thesaurusKey = motcle.getThesaurus().getSubsetKey();
        FicheEligibility ficheEligibility = extractionContext.getFicheEligibility();
        for (int k = 0; k < count; k++) {
            ExtraitCorpusDef extraitCorpusDef = extraitCorpusDefList.getExtraitCorpusDef(k);
            CorpusExtractor corpusExtractor = new CorpusExtractor(extraitCorpusDef);
            FicheSelector indexationFicheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            indexationFicheSelector.setFicheEligibility(ficheEligibility);
            indexationFicheSelector.addFicheSelectOptionsList(extraitCorpusDef.getFicheSelectOptionsList());
            for (int j = 0; j < indexationFicheSelector.getCorpusCount(); j++) {
                Corpus corpus = indexationFicheSelector.getCorpus(j);
                SubsetKey corpusKey = corpus.getSubsetKey();
                Croisement croisement = basedefiches.getCroisement(corpusKey, thesaurusKey);
                CroisementItem croisementItem = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
                int ciCount = croisementItem.getCount();
                for (int i = 0; i < ciCount; i++) {
                    int ticket = croisementItem.getTicket(i);
                    int poids = croisementItem.getPoids(i);
                    AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ticket);
                    if (abregeFiche != null) {
                        if (indexationFicheSelector.isSelected(abregeFiche, poids)) {
                            corpusExtractor.add(abregeFiche, poids);
                        }
                    }
                }
            }
            if (!corpusExtractor.containsFiche()) {
                continue;
            }
            if (extraitCorpusXMLPart == null) {
                extraitCorpusXMLPart = new ExtraitCorpusXMLPart(getXMLWriter(), extractionContext);
            }
            extraitCorpusXMLPart.addExtraitcorpus(corpusExtractor);
        }
    }


    private static class InternalMotcleExtractInfo implements MotcleExtractInfo {

        private Motcle motcle;
        private MotcleFilter motcleFilter;

        private InternalMotcleExtractInfo(Motcle motcle, MotcleFilter motcleFilter) {
            this.motcle = motcle;
            this.motcleFilter = motcleFilter;
        }

        public Motcle getMotcle() {
            return motcle;
        }

        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        public int getPoids() {
            return 0;
        }

    }

}
