/* BasedefichesLib_Xml - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.libs;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.Metadata;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.DefaultXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class LibsXMLWriter extends DefaultXMLWriter {

    public final static String SPECIAL_INTITULES = "intitules";
    private int langInteger;

    public LibsXMLWriter(int langInteger, Appendable appendable, int nbredetabulations) {
        super();
        setAppendable(appendable);
        setIndentLength(nbredetabulations);
        this.langInteger = langInteger;
    }

    public void appendIntituleLibs(Basedefiches basedefiches) throws IOException {
        startOpenTag("libs");
        endOpenTag();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            appendCorpusIntituleLibs(corpus, true);
        }
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            appendThesaurusIntituleLibs(thesaurus, true);
        }
        closeTag("libs");
    }

    public void appendSubsetLibs(Subset subset) throws IOException {
        if (subset instanceof Thesaurus) {
            appendThesaurusLibs((Thesaurus) subset);
        } else if (subset instanceof Corpus) {
            appendCorpusLibs(((Corpus) subset).getCorpusMetadata());
        }
    }

    public void appendThesaurusLibs(Thesaurus thesaurus) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "thesaurus");
        addAttribute("thesaurus", thesaurus.getSubsetKey().getSubsetName());
        endOpenTag();
        appendThesaurusIntituleLibs(thesaurus, false);
        MotcleList firstLevel = thesaurus.getFirstLevelList();
        for (int i = 0; i < firstLevel.getMotcleCount(); i++) {
            Motcle motcle = firstLevel.getMotcle(i);
            appendMotcleLib(motcle, true);
        }
        closeTag("libs");
    }

    private void appendThesaurusIntituleLibs(Thesaurus thesaurus, boolean withThesaurusName) throws IOException {
        String thesaurusName = (withThesaurusName) ? thesaurus.getSubsetKey().getSubsetName() : null;
        appendIntituleLibs(thesaurus.getThesaurusMetadata(), "thesaurus", thesaurusName);
    }

    public void appendCorpusLibs(CorpusMetadata corpusMetadata) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "metadata");
        addAttribute("corpus", corpusMetadata.getCorpus().getSubsetKey().getSubsetName());
        endOpenTag();
        appendCorpusIntituleLibs(corpusMetadata.getCorpus(), false);
        appendCorpusFieldLib(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.IDCORPUS));
        appendCorpusFieldLib(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.LANG));
        appendCorpusFieldLib(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.REDACTEURS));
        appendCorpusFieldLib(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE));
        appendCorpusFieldLib(corpusMetadata.getCorpusFieldByFieldKey(FieldKey.SOUSTITRE));
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            appendCorpusFieldLib(corpusMetadata.getProprieteField(i));
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            appendCorpusFieldLib(corpusMetadata.getInformationField(i));
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        for (int i = 0; i < sectionCount; i++) {
            appendCorpusFieldLib(corpusMetadata.getSectionField(i));
        }
        closeTag("libs");
    }

    private void appendCorpusIntituleLibs(Corpus corpus, boolean withCorpusName) throws IOException {
        String corpusName = (withCorpusName) ? corpus.getSubsetKey().getSubsetName() : null;
        appendIntituleLibs(corpus.getCorpusMetadata(), "corpus", corpusName);
    }

    private void appendIntituleLibs(Metadata metadata, String categoryString, String subsetId) throws IOException {
        int intituleCount = metadata.getIntituleCount();
        for (int i = 0; i < intituleCount; i++) {
            Intitule intitule = metadata.getIntitule(i);
            appendIntituleLib(intitule, categoryString, subsetId);
        }
    }

    public void appendLangueLibs(int[] langIntegerArray, Locale locale, boolean onlyMain) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "langue");
        endOpenTag();
        for (int i = 0; i < langIntegerArray.length; i++) {
            appendLangueLib(langIntegerArray[i], locale, onlyMain);
        }
        closeTag("libs");
    }

    public void appendPaysLibs(int[] langIntegerArray, Locale locale, boolean onlyMain) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "pays");
        endOpenTag();
        for (int i = 0; i < langIntegerArray.length; i++) {
            appendPaysLib(langIntegerArray[i], locale, onlyMain);
        }
        closeTag("libs");
    }

    public void appendLangueLib(int langInteger, Locale locale, boolean onlyMain) throws IOException {
        startLib();
        addAttribute("code", LangInteger.toString(langInteger));
        endOpenTag(false);
        escape(LangIntegerLocalisation.getLangLibelle(langInteger, locale, onlyMain));
        closeTag("lib", false, false);

    }

    public void appendFicheLibs(Collection values) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "corpus");
        endOpenTag();
        for (Iterator it = values.iterator(); it.hasNext();) {
            AbregeFiche abregeFiche = (AbregeFiche) it.next();
            appendFicheLib(abregeFiche);
        }
        closeTag("libs");
    }

    public void appendMotcleLibs(Collection values) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "thesaurus");
        endOpenTag();
        for (Iterator it = values.iterator(); it.hasNext();) {
            Motcle motcle = (Motcle) it.next();
            appendMotcleLib(motcle, false);
        }
        closeTag("libs");
    }

    public void appendFicheLib(AbregeFiche abregeFiche) throws IOException {
        startLib();
        addAttribute("corpus", abregeFiche.getCorpus().getSubsetKey().getSubsetName());
        addAttribute("idcorpus", String.valueOf(abregeFiche.getIdcorpus()));
        endOpenTag(false);
        escape(abregeFiche.getTitre());
        closeTag("lib", false, false);

    }

    public void appendPaysLib(int langInteger, Locale locale, boolean onlyMain) throws IOException {
        startLib();
        addAttribute("code", LangInteger.getCountry(langInteger));
        endOpenTag(false);
        escape(LangIntegerLocalisation.getCountryLibelle(langInteger, locale, onlyMain));
        closeTag("lib", false, false);

    }

    private void appendIntituleLib(Intitule intitule, String subsetCategory, String corpusName) throws IOException {
        startLib();
        addAttribute(subsetCategory, corpusName);
        addAttribute("intitule-name", intitule.getName());
        endLib(intitule);
    }

    private void appendCorpusFieldLib(CorpusField corpusField) throws IOException {
        if (corpusField == null) {
            return;
        }
        startLib();
        addAttribute("field-key", corpusField.getFieldKey().toString());
        endLib(corpusField);
    }

    private void appendMotcleLib(Motcle motcle, boolean recursive) throws IOException {
        startLib();
        addAttribute("idths", String.valueOf(motcle.getIdths()));
        addAttribute("thesaurus", motcle.getThesaurus().getSubsetKey().getSubsetName());
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            addAttribute("idalpha", idalpha);
        }
        endLib(motcle);
        if (recursive) {
            MotcleList childList = motcle.getChildList();
            for (int i = 0; i < childList.getMotcleCount(); i++) {
                appendMotcleLib(childList.getMotcle(i), recursive);
            }
        }
    }

    private void startLib() throws IOException {
        startOpenTag("lib");
    }

    private void endLib(LibelleHolder libelleHolder) throws IOException {
        endOpenTag(false);
        String lib = libelleHolder.getLibelleString(langInteger, false);
        if (lib != null) {
            escape(lib);
        }
        closeTag("lib", false, false);
    }

    private void unknonwn(String key) throws IOException {
        startOpenTag("libs");
        addAttribute("unknown", key);
        closeEmptyTag();
    }

    public static String getLibsXML(int langInteger, Basedefiches basedefiches, SubsetKey subsetKey, boolean withTabs) {
        StringBuilder buf = new StringBuilder(256);
        try {
            LibsXMLWriter libsXMLWriter = new LibsXMLWriter(langInteger, buf, (withTabs) ? 0 : -999);
            libsXMLWriter.appendXMLDeclaration();
            Subset subset = basedefiches.getSubset(subsetKey);
            if (subset == null) {
                libsXMLWriter.unknonwn(subsetKey.toString());
            } else {
                libsXMLWriter.appendSubsetLibs(subset);
            }
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static String getLibsXML(int langInteger, Basedefiches basedefiches, String special, boolean withTabs) {
        StringBuilder buf = new StringBuilder(256);
        try {
            LibsXMLWriter libsXMLWriter = new LibsXMLWriter(langInteger, buf, (withTabs) ? 0 : -999);
            libsXMLWriter.appendXMLDeclaration();
            if (special.equals(SPECIAL_INTITULES)) {
                libsXMLWriter.appendIntituleLibs(basedefiches);
            } else {
                libsXMLWriter.unknonwn(special);
            }
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
