/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.scrutariexport;

import java.io.IOException;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDef;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.tableexport.ColDef;
import net.mapeadores.basedefiches.tableexport.ColDefList;
import net.mapeadores.basedefiches.xml.extraction.ExtractionUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportXMLPart extends XMLPart {

    public ScrutariExportXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void addScrutariExportDef(ScrutariExportDef scrutariExportDef) throws IOException {
        openTag("scrutariexport");
        addSimpleElement("base-name", scrutariExportDef.getBaseName());
        addSimpleElement("base-icon", scrutariExportDef.getBaseIcon());
        int langUiCount = scrutariExportDef.getLangUICount();
        if (langUiCount > 0) {
            openTag("langs-ui");
            for (int i = 0; i < langUiCount; i++) {
                addSimpleElement("lang", LangInteger.toString(scrutariExportDef.getLangUI(i)));
            }
            closeTag("langs-ui");
        }
        addPattern(scrutariExportDef.getFicheHrefPattern(), "fiche-pattern");
        addPattern(scrutariExportDef.getMotcleHrefPattern(), "motcle-pattern");
        int corpusCount = scrutariExportDef.getCorpusScrutariDefCount();
        for (int i = 0; i < corpusCount; i++) {
            addCorpusScrutariDef(scrutariExportDef.getCorpusScrutariDef(i));
        }
        int thesaurusCount = scrutariExportDef.getThesaurusScrutariDefCount();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusScrutariDef thesaurusScrutariDef = scrutariExportDef.getThesaurusScrutariDef(i);
            if (thesaurusScrutariDef.isSelected()) {
                addThesaurusScrutariDef(thesaurusScrutariDef);
            }
        }
        FicheSelectOptions ficheSelectOptions = scrutariExportDef.getFicheSelectOptions();
        if (ficheSelectOptions != null) {
            openTag("fiche-select");
            ExtractionUtils.writeFicheSelectOptions(this, ficheSelectOptions);
            closeTag("fiche-select");
        }
        closeTag("scrutariexport");
    }

    private void addPattern(AccoladePattern accoladePattern, String tagName) throws IOException {
        if (accoladePattern == null) {
            return;
        }
        addSimpleElement(tagName, accoladePattern.toString());
    }

    private void addCorpusScrutariDef(CorpusScrutariDef corpusScrutariDef) throws IOException {
        startOpenTag("corpus-def");
        addAttribute("corpus", corpusScrutariDef.getCorpusSubsetKey().getSubsetName());
        endOpenTag();
        addSimpleElement("date", corpusScrutariDef.getDateFormatSourceKey().toString());
        addBasedefichesFormatDef(corpusScrutariDef.getTitreFormatDef(), "titre-format");
        addBasedefichesFormatDef(corpusScrutariDef.getSoustitreFormatDef(), "soustitre-format");
        ColDefList colDefList = corpusScrutariDef.getComplementColDefList();
        int count = colDefList.getColDefCount();
        for (int i = 0; i < count; i++) {
            ColDef colDef = colDefList.getColDef(i);
            addColDef(colDef);
        }
        addPattern(corpusScrutariDef.getHrefPattern(), "href-pattern");
        if (corpusScrutariDef.isMultiLang()) {
            addSimpleElement("multilang-key", corpusScrutariDef.getMultiLangKey().toString());
        }
        closeTag("corpus-def");
    }

    private void addThesaurusScrutariDef(ThesaurusScrutariDef thesaurusScrutariDef) throws IOException {
        startOpenTag("thesaurus-def");
        addAttribute("thesaurus", thesaurusScrutariDef.getThesaurusSubsetKey().getSubsetName());
        endOpenTag();
        String motcleFormatPattern = thesaurusScrutariDef.getMotcleFormatPattern();
        if (motcleFormatPattern != null) {
            addSimpleElement("motcle-format", motcleFormatPattern);
        }
        closeTag("thesaurus-def");
    }

    private void addColDef(ColDef colDef) throws IOException {
        startOpenTag("complement");
        String name = colDef.getColName();
        if (!name.startsWith("comp_")) {
            addAttribute("name", name);
        }
        endOpenTag();
        LibelleHolder customLibelle = colDef.getCustomLibelle();
        if (customLibelle != null) {
            XMLUtils.addLibelleList(this, customLibelle);
        }
        addBasedefichesFormatDef(colDef.getBasedefichesFormatDef(), null);
        closeTag("complement");
    }

    private void addBasedefichesFormatDef(BasedefichesFormatDef basedefichesFormatDef, String tagName) throws IOException {
        if (basedefichesFormatDef == null) {
            return;
        }
        if (tagName != null) {
            openTag(tagName);
        }
        String[] values = BasedefichesFormatUtils.toStringArray(basedefichesFormatDef);
        int length = values.length;
        for (int i = 0; i < length; i++) {
            String value = values[i];
            if ((value != null) && (value.length() > 0)) {
                addSimpleElement("value", value);
            } else {
                addEmptyElement("value");
            }
        }
        if (tagName != null) {
            closeTag(tagName);
        }
    }

}
