/* BasedefichesLib_Xml - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.basedefiches.xml.sqlexport;

import net.mapeadores.basedefiches.sqlexport.SqlExportDefBuilder;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportDOMReader {

    private SqlExportDefBuilder sqlExportDefBuilder;
    private SqlExportXmlErrorHandler sqlExportXmlErrorHandler;

    public SqlExportDOMReader(SqlExportDefBuilder sqlExportDefBuilder, SqlExportXmlErrorHandler sqlExportXmlErrorHandler) {
        this.sqlExportDefBuilder = sqlExportDefBuilder;
        this.sqlExportXmlErrorHandler = sqlExportXmlErrorHandler;
    }

    public void readSqlExportDef(Element element_xml) {
        DOMUtils.readChildren(element_xml, new RootHandler());
    }


    private class RootHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("param")) {
                String name = element.getAttribute("name").trim();
                if (name.length() == 0) {
                    sqlExportXmlErrorHandler.emptyAttributeError("param", "name");
                    return;
                }
                String value = element.getAttribute("value").trim();
                sqlExportDefBuilder.addParam(name, value);
            } else {
                String contenu = DOMUtils.readSimpleElement(element);
                if (contenu.length() == 0) {
                    return;
                }
                if (tagName.equals("path")) {
                    sqlExportDefBuilder.setPath(contenu);
                } else if (tagName.equals("url")) {
                    sqlExportDefBuilder.setUrl(contenu);
                } else if (tagName.equals("tableexport-name")) {
                    sqlExportDefBuilder.setTableExportName(contenu);
                } else if (tagName.equals("sqlexport-classname")) {
                    sqlExportDefBuilder.setSqlExportClassName(contenu);
                } else if (tagName.equals("post-command")) {
                    sqlExportDefBuilder.setPostCommand(contenu);
                } else {
                    sqlExportXmlErrorHandler.unknownTagWarning(tagName);
                }
            }
        }

    }

}
