/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;


/**
 *
 * @author Vincent Calame
 */
public class AvailableFormatLocales {

    private static Locale defaultLocale;
    private static Map<Integer, Locale> availableLocale;
    private static int[] availableArray;

    static {
        Initializer init = new Initializer();
        init.add("fr-FR", "fr");
        init.add("es-ES", "es");
        init.add("es-AR");
        init.add("en-US", "en");
        init.add("it");
        init.add("ar");
        init.add("pt-BR", "pt");
        init.add("zh-CN", "zh");
        defaultLocale = init.getDefLoc();
        availableLocale = init.getMapByLang();
        availableArray = init.getAvaiArray();
    }

    public static int[] getAvailableArray() {
        int length = availableArray.length;
        int[] result = new int[length];
        System.arraycopy(availableArray, 0, result, 0, length);
        return result;
    }

    public static Locale getNearestAvailableLocale(int langInteger) {
        Locale loc = availableLocale.get(new Integer(langInteger));
        if (loc != null) {
            return loc;
        }
        int langPart = LangInteger.getLangPart(langInteger);
        loc = availableLocale.get(new Integer(langPart));
        return loc;
    }

    public static Locale getDefaultLocale() {
        return defaultLocale;
    }


    private static class Initializer {

        private Locale defaultLocale;
        private SortedMap<String, Integer> sortedLocale = new TreeMap<String, Integer>();
        private Map<Integer, Locale> mapBylang = new HashMap<Integer, Locale>();

        Initializer() {
            try {
                defaultLocale = LangInteger.getLocale(LangInteger.parse("fr-FR"));
            } catch (LangIntegerException e) {
                throw new ShouldNotOccurException(e);
            }
        }

        public Locale getDefLoc() {
            return defaultLocale;
        }

        public int[] getAvaiArray() {
            int size = sortedLocale.size();
            int p = 0;
            int[] result = new int[size];
            for (Integer itg : sortedLocale.values()) {
                result[p] = itg.intValue();
                p++;
            }
            return result;
        }

        public Map<Integer, Locale> getMapByLang() {
            return mapBylang;
        }

        private void add(String s) {
            add(s, null);
        }

        private void add(String s, String onlyLang) {
            try {
                int localeLangInteger = LangInteger.parse(s);
                Locale locale = LangInteger.getLocale(localeLangInteger);
                Integer itg = new Integer(localeLangInteger);
                sortedLocale.put(s, itg);
                mapBylang.put(itg, locale);
                if (onlyLang != null) {
                    int onlyLangInteger = LangInteger.parse(onlyLang);
                    mapBylang.put(new Integer(onlyLangInteger), locale);
                }
            } catch (LangIntegerException e) {
                throw new ShouldNotOccurException(e);
            }
        }

    }

}
