/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages;

import fr.exemole.bdfserver.balayages.api.SiteMapOption;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class BalayageBuilder implements BalayageConstants {

    private InternalBalayage balayage;
    private ArrayList<FicheSelectOptions> globalFicheSelectOptionsList = new ArrayList<FicheSelectOptions>();
    private Map<String, InternalBalayageMode> modeMap = new LinkedHashMap<String, InternalBalayageMode>();
    private List<BalayageUnitBuilder> unitList = new ArrayList<BalayageUnitBuilder>();

    public BalayageBuilder(String name) {
        balayage = new InternalBalayage(name);
    }

    public Balayage toBalayage() {
        int size = globalFicheSelectOptionsList.size();
        if (size > 0) {
            balayage.globalFicheSelectOptionsArray = globalFicheSelectOptionsList.toArray(new FicheSelectOptions[size]);
        }
        int size2 = modeMap.size();
        if (size2 > 0) {
            BalayageMode[] array = new BalayageMode[size2];
            int p = 0;
            for (BalayageMode balayageMode : modeMap.values()) {
                array[p] = balayageMode;
                p++;
            }
            balayage.balayageModeArray = array;
        }
        int size3 = unitList.size();
        if (size3 > 0) {
            BalayageUnit[] balayageUnitArray = new BalayageUnit[size3];
            for (int i = 0; i < size3; i++) {
                balayageUnitArray[i] = unitList.get(i).toBalayageUnit();
            }
            balayage.balayageUnitArray = balayageUnitArray;
        }
        return balayage;
    }

    public void addGlobalFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        globalFicheSelectOptionsList.add(ficheSelectOptions);
    }

    public void setRootName(String rootName) {
        if (rootName != null) {
            rootName = rootName.intern();
        }
        balayage.rootName = rootName;
    }

    public void setPath(String path) {
        balayage.path = path;
    }

    public void setWithTransformation(boolean withTransformation) {
        balayage.withTransformation = withTransformation;
    }

    public void setPostscriptumCommand(String command) {
        balayage.postscriptum.setCommand(command);
    }

    public void addPostscriptumScrutariExport(String name) {
        balayage.postscriptum.addScrutariExport(name);
    }

    public void setDefaultLangOption(short defaultLangOption) {
        balayage.defaultLangOption = defaultLangOption;
    }

    public void setLocalisationLists(LocalisationLists localisationLists) {
        balayage.localisationLists = localisationLists;
    }

    public void setSiteMapOption(SiteMapOption siteMapOption) {
        balayage.siteMapOption = siteMapOption;
    }

    public BalayageMode addBalayageMode(String name) {
        InternalBalayageMode ibe = modeMap.get(name.intern());
        if (ibe == null) {
            ibe = new InternalBalayageMode(name);
            modeMap.put(name, ibe);
        }
        return ibe;
    }

    public void setWithCurrentDateLimit(BalayageMode balayageMode, boolean b) {
        ((InternalBalayageMode) balayageMode).setWithCurrentDateLimit(b);
    }

    public BalayageUnitBuilder addBalayageUnitBuilder(short type) {
        BalayageUnitBuilder unitBuilder = new BalayageUnitBuilder(type);
        unitList.add(unitBuilder);
        return unitBuilder;
    }


    private static class InternalBalayage implements Balayage {

        private short defaultLangOption = LANG_OPTION_NONE;
        private String name;
        private LocalisationLists localisationLists;
        private boolean withTransformation;
        private String rootName;
        private String path;
        private FicheSelectOptions[] globalFicheSelectOptionsArray;
        private BalayageMode[] balayageModeArray;
        private BalayageUnit[] balayageUnitArray;
        private SiteMapOption siteMapOption;
        private InternalPostscriptum postscriptum;

        private InternalBalayage(String name) {
            this.name = name;
            this.postscriptum = new InternalPostscriptum();
        }

        public String getName() {
            return name;
        }

        public LocalisationLists getLocalisationLists() {
            return localisationLists;
        }

        public String getRootName() {
            return rootName;
        }

        public String getPath() {
            return path;
        }

        public boolean isWithTransformation() {
            return withTransformation;
        }

        public int getBalayageUnitCount() {
            if (balayageUnitArray == null) {
                return 0;
            }
            return balayageUnitArray.length;
        }

        public BalayageUnit getBalayageUnit(int i) {
            return balayageUnitArray[i];
        }

        public int getGlobalFicheSelectOptionsCount() {
            if (globalFicheSelectOptionsArray == null) {
                return 0;
            }
            return globalFicheSelectOptionsArray.length;
        }

        public FicheSelectOptions getGlobalFicheSelectOptions(int i) {
            return globalFicheSelectOptionsArray[i];
        }

        public int getBalayageModeCount() {
            if (balayageModeArray == null) {
                return 0;
            }
            return balayageModeArray.length;
        }

        public BalayageMode getBalayageMode(int i) {
            return balayageModeArray[i];
        }

        public short getDefaultLangOption() {
            return defaultLangOption;
        }

        public SiteMapOption getSiteMapOption() {
            return siteMapOption;
        }

        public Postscriptum getPostscriptum() {
            return postscriptum;
        }

    }


    private static class InternalPostscriptum implements Balayage.Postscriptum {

        private String command;
        private List<String> scrutariExportList = new ArrayList<String>();

        private InternalPostscriptum() {
        }

        public String getCommand() {
            return command;
        }

        public int getScrutariExportCount() {
            return scrutariExportList.size();
        }

        public String getScrutariExportName(int i) {
            return scrutariExportList.get(i);
        }

        private void setCommand(String command) {
            this.command = command;
        }

        private void addScrutariExport(String name) {
            scrutariExportList.add(name);
        }

    }


    private static class InternalBalayageMode implements BalayageMode {

        private String name;
        private boolean withCurrentDateLimit = false;

        InternalBalayageMode(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public boolean isWithCurrentDateLimit() {
            return withCurrentDateLimit;
        }

        void setWithCurrentDateLimit(boolean withCurrentDateLimit) {
            this.withCurrentDateLimit = withCurrentDateLimit;
        }

    }

}
