/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;


/**
 *
 * @author Vincent Calame
 */
public class BalayageUnitBuilder implements BalayageConstants {

    private InternalBalayageUnit balayageUnit;
    private LinkedHashSet<String> modeSet = new LinkedHashSet<String>();
    private LinkedHashSet<String> extensionSet = new LinkedHashSet<String>();
    private List<OutputBuilder> outputList = new ArrayList<OutputBuilder>();

    public BalayageUnitBuilder(short type) {
        balayageUnit = new InternalBalayageUnit(type);
    }

    public void setName(String name) {
        balayageUnit.setName(name);
    }

    public void setGlobalSelect(boolean globalSelect) {
        balayageUnit.setGlobalSelect(globalSelect);
    }

    public void setExtractionPath(String extractionPath) {
        balayageUnit.setExtractionPath(extractionPath);
    }

    public void addMode(String mode) {
        modeSet.add(mode.intern());
    }

    public void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
        balayageUnit.setFicheSelectOptions(ficheSelectOptions);
    }

    public void setMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
        balayageUnit.setMotcleSelectOptions(motcleSelectOptions);
    }

    public void setIllustrationSelectOptions(IllustrationSelectOptions illustrationSelectOptions) {
        balayageUnit.setIllustrationSelectOptions(illustrationSelectOptions);
    }

    public void addExtension(String s) {
        extensionSet.add(s.intern());
    }

    public OutputBuilder addOutputBuilder() {
        OutputBuilder iob = new OutputBuilder();
        outputList.add(iob);
        return iob;
    }

    public BalayageUnit toBalayageUnit() {
        int size = modeSet.size();
        if (size > 0) {
            String[] modeArray = new String[size];
            int p = 0;
            for (String mode : modeSet) {
                modeArray[p] = mode;
                p++;
            }
            balayageUnit.setModeArray(modeArray);
        }
        int size2 = extensionSet.size();
        if (size2 > 0) {
            String[] extensionArray = new String[size2];
            int p = 0;
            for (String extension : extensionSet) {
                extensionArray[p] = extension;
                p++;
            }
            balayageUnit.setExtensionArray(extensionArray);
        }
        int size3 = outputList.size();
        if (size3 > 0) {
            Output[] outputArray = new Output[size3];
            for (int i = 0; i < size3; i++) {
                outputArray[i] = outputList.get(i).toOutput(balayageUnit);
            }
            balayageUnit.setOutputArray(outputArray);
        }
        return balayageUnit;
    }


    private static class InternalBalayageUnit implements BalayageUnit {

        private short type;
        private String extractionPath;
        private FicheSelectOptions ficheSelectOptions;
        private MotcleSelectOptions motcleSelectOptions;
        private Output[] outputArray;
        private boolean globalSelect = true;
        private String[] modeArray;
        private String[] extensionArray;
        private String name;
        private IllustrationSelectOptions illustrationSelectOptions;

        public InternalBalayageUnit(short type) {
            this.type = type;
        }

        public String getName() {
            return name;
        }

        public short getType() {
            return type;
        }

        public boolean isGlobalSelect() {
            return globalSelect;
        }

        public FicheSelectOptions getFicheSelectOptions() {
            return ficheSelectOptions;
        }

        public MotcleSelectOptions getMotcleSelectOptions() {
            return motcleSelectOptions;
        }

        public int getOutputCount() {
            if (outputArray == null) {
                return 0;
            }
            return outputArray.length;
        }

        public Output getOutput(int i) {
            return outputArray[i];
        }

        public boolean hasMode(String s) {
            if (modeArray == null) {
                return false;
            }
            int size = modeArray.length;
            if (size == 0) {
                return false;
            }
            for (int i = 0; i < size; i++) {
                if (modeArray[i].equals(s)) {
                    return true;
                }
            }
            return false;
        }

        public int getExtensionCount() {
            if (extensionArray == null) {
                return 0;
            }
            return extensionArray.length;
        }

        public String getExtension(int i) {
            return extensionArray[i];
        }

        public String getExtractionPath() {
            return extractionPath;
        }

        public IllustrationSelectOptions getIllustrationSelectOptions() {
            return illustrationSelectOptions;
        }

        private void setName(String name) {
            this.name = name;
        }

        private void setExtractionPath(String extractionPath) {
            this.extractionPath = extractionPath;
        }

        private void setModeArray(String[] modeArray) {
            this.modeArray = modeArray;
        }

        private void setFicheSelectOptions(FicheSelectOptions ficheSelectOptions) {
            this.ficheSelectOptions = ficheSelectOptions;
        }

        private void setMotcleSelectOptions(MotcleSelectOptions motcleSelectOptions) {
            this.motcleSelectOptions = motcleSelectOptions;
        }

        private void setExtensionArray(String[] extensionArray) {
            this.extensionArray = extensionArray;
        }

        private void setGlobalSelect(boolean globalSelect) {
            this.globalSelect = globalSelect;
        }

        private void setOutputArray(Output[] outputArray) {
            this.outputArray = outputArray;
        }

        private void setIllustrationSelectOptions(IllustrationSelectOptions illustrationSelectOptions) {
            this.illustrationSelectOptions = illustrationSelectOptions;
        }

    }

}
