/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.LocalisationLists;
import net.mapeadores.util.text.AccoladePattern;


/**
 *
 * @author Vincent Calame
 */
public class OutputBuilder implements BalayageConstants {

    private InternalOutput output = new InternalOutput();
    private List<InternalOutputParam> outputParamList = new ArrayList<InternalOutputParam>();

    public OutputBuilder() {
    }

    public void setName(String name) {
        output.setName(name);
    }

    public void setXsltPath(String xsltPath) {
        output.setXsltPath(xsltPath);
    }

    public void setMode(String mode) {
        output.setMode(mode);
    }

    public void setCleanBefore(boolean cleanBefore) {
        output.setCleanBefore(cleanBefore);
    }

    public void setLangOption(short langOption) {
        output.setLangOption(langOption);
    }

    public void setIncludeOutput(boolean includeOutput) {
        output.setIncludeOutput(includeOutput);
    }

    public void setPattern(AccoladePattern pattern) {
        output.setPattern(pattern);
    }

    public void setOutputPathPattern(AccoladePattern outputPathPattern) {
        output.setOutputPathPattern(outputPathPattern);
    }

    public void setLocalisationLists(LocalisationLists localisationLists) {
        output.setLocalisationLists(localisationLists);
    }

    public void addOutputParam(String name, String value) {
        outputParamList.add(new InternalOutputParam(name, value));
    }

    public Output toOutput(BalayageUnit balayageUnit) {
        output.setBalayageUnit(balayageUnit);
        int size = outputParamList.size();
        if (size > 0) {
            OutputParam[] outputParamArray = new OutputParam[size];
            output.setOutputParamArray(outputParamList.toArray(outputParamArray));
        }
        return output;
    }


    private static class InternalOutput implements Output {

        private String name;
        private short langOption = LANG_OPTION_UNKNOWN;
        private String xsltPath;
        private AccoladePattern pattern;
        private AccoladePattern outputPathPattern;
        private String reversePath = "";
        private OutputParam[] outputParamArray;
        private String mode;
        private boolean cleanBefore = false;
        private boolean includeOutput = false;
        private LocalisationLists localisationLists;
        private BalayageUnit balayageUnit;

        private InternalOutput() {
        }

        public String getName() {
            return name;
        }

        public String getXsltPath() {
            return xsltPath;
        }

        public AccoladePattern getPattern() {
            return pattern;
        }

        public AccoladePattern getOutputPath() {
            return outputPathPattern;
        }

        public int getOutputParamCount() {
            if (outputParamArray == null) {
                return 0;
            }
            return outputParamArray.length;
        }

        public OutputParam getOutputParam(int i) {
            return outputParamArray[i];
        }

        public String getMode() {
            return mode;
        }

        public boolean isCleanBefore() {
            return cleanBefore;
        }

        public short getLangOption() {
            return langOption;
        }

        public String getReversePath() {
            return reversePath;
        }

        public boolean isIncludeOutput() {
            return includeOutput;
        }

        public LocalisationLists getLocalisationLists() {
            return localisationLists;
        }

        public BalayageUnit getBalayageUnit() {
            return balayageUnit;
        }

        private void setName(String name) {
            this.name = name;
        }

        private void setXsltPath(String xsltPath) {
            this.xsltPath = xsltPath;
        }

        private void setMode(String mode) {
            this.mode = mode;
        }

        private void setCleanBefore(boolean cleanBefore) {
            this.cleanBefore = cleanBefore;
        }

        private void setLangOption(short langOption) {
            this.langOption = langOption;
        }

        private void setIncludeOutput(boolean includeOutput) {
            this.includeOutput = includeOutput;
        }

        private void setPattern(AccoladePattern pattern) {
            this.pattern = pattern;
        }

        private void setOutputPathPattern(AccoladePattern outputPathPattern) {
            this.outputPathPattern = outputPathPattern;
            initReversePath();
        }

        private void setOutputParamArray(OutputParam[] outputParamArray) {
            this.outputParamArray = outputParamArray;
        }

        private void setLocalisationLists(LocalisationLists localisationLists) {
            this.localisationLists = localisationLists;
        }

        private void initReversePath() {
            if (outputPathPattern != null) {
                String path = outputPathPattern.toString();
                int length = path.length();
                StringBuffer buf = new StringBuffer();
                for (int i = 0; i < length; i++) {
                    if (path.charAt(i) == '/') {
                        buf.append("../");
                    }
                }
                reversePath = buf.toString();
            } else {
                reversePath = "";
            }
        }

        private void setBalayageUnit(BalayageUnit balayageUnit) {
            this.balayageUnit = balayageUnit;
        }

    }


    private static class InternalOutputParam implements OutputParam {

        String name;
        String value;

        InternalOutputParam(String name, String value) {
            this.name = name;
            this.value = value;
        }

        public String getName() {
            return name;
        }

        public String getValue() {
            return value;
        }

    }

}
