/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageMode;
import fr.exemole.bdfserver.balayages.api.SiteMapOption;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import java.io.File;


/**
 *
 * @author Vincent Calame
 */
public class BalayageInitializer {

    private BdfServer bdfServer;
    private Balayage balayage;
    private File balayageRootDirectory;
    private String errorKey;
    private Object errorValue;
    private BalayageMode balayageMode;

    public BalayageInitializer(BdfServer bdfServer, Balayage balayage) {
        this.bdfServer = bdfServer;
        this.balayage = balayage;
        init();
    }

    private void init() {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        String rootName = balayage.getRootName();
        if (rootName == null) {
            errorKey = "balayage_err_undefinedroot";
            return;
        }
        File rootDirectory = configurationManager.getBalayageRootDirectory(rootName);
        if (!rootDirectory.exists()) {
            errorKey = "balayage_err_notexistingroot";
            String[] err = {balayage.getRootName(), rootDirectory.getAbsolutePath()};
            errorValue = err;
            return;
        }
        if (!rootDirectory.isDirectory()) {
            errorKey = "balayage_err_notdirectoryroot";
            String[] err = {balayage.getRootName(), rootDirectory.getAbsolutePath()};
            errorValue = err;
            return;
        }
        String path = balayage.getPath();
        if (path != null) {
            if (!pathTest(path)) {
                errorKey = "balayage_err_wrongrelativepath";
                errorValue = path;
                return;
            }
            balayageRootDirectory = new File(rootDirectory, path);
            if (!balayageRootDirectory.exists()) {
                balayageRootDirectory.mkdirs();
            } else if (!balayageRootDirectory.isDirectory()) {
                errorKey = "balayage_err_notdirectorypath";
                errorValue = balayageRootDirectory.getAbsolutePath();
                return;
            }
        } else {
            balayageRootDirectory = rootDirectory;
        }
        if (balayage.getBalayageUnitCount() == 0) {
            errorKey = "balayage_err_unitempty";
        }
    }

    public boolean hasError() {
        return (errorKey != null);
    }

    public String getErrorKey() {
        return errorKey;
    }

    public Object getErrorValue() {
        return errorValue;
    }

    public void setBalayageMode(BalayageMode balayageMode) {
        this.balayageMode = balayageMode;
    }

    public BalayageEngine getBalayageEngine(BalayageLog balayageLog) {
        if (hasError()) {
            return null;
        }
        BalayageParameters balayageParameters = new BalayageParameters(balayage, bdfServer);
        balayageParameters.setBalayageMode(balayageMode);
        balayageParameters.setBalayageLog(balayageLog);
        balayageParameters.setBalayageRootDirectory(balayageRootDirectory);
        SiteMapOption siteMapOption = balayage.getSiteMapOption();
        if (siteMapOption != null) {
            SiteMapBuilder siteMapBuilder = new SiteMapBuilder(siteMapOption);
            balayageParameters.setFileGenerationListener(siteMapBuilder);
        }
        BalayageEngine balayageEngine = new BalayageEngine(balayageParameters);
        return balayageEngine;
    }

    static private boolean pathTest(String path) {
        return (path.indexOf("..") == -1);
    }

}
