/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageMode;
import fr.exemole.bdfserver.balayages.FileGenerationListener;
import java.io.File;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.misc.LinkAnalyser;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class BalayageParameters {

    private BalayageMode balayageMode;
    private BalayageLog balayageLog;
    private Balayage balayage;
    private BdfServer bdfServer;
    private File balayageRootDirectory;
    private String[] langArray;
    private FileGenerationListener fileGenerationListener;
    private boolean extractionContextInit = false;
    private ExtractionContext extractionContext;

    public BalayageParameters(Balayage balayage, BdfServer bdfServer) {
        this.balayage = balayage;
        this.bdfServer = bdfServer;
    }

    public BalayageMode getBalayageMode() {
        return balayageMode;
    }

    public void setBalayageMode(BalayageMode balayageMode) {
        this.balayageMode = balayageMode;
    }

    public BalayageLog getBalayageLog() {
        return balayageLog;
    }

    public void setBalayageLog(BalayageLog balayageLog) {
        this.balayageLog = balayageLog;
    }

    public Balayage getBalayage() {
        return balayage;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public File getBalayageRootDirectory() {
        return balayageRootDirectory;
    }

    public void setBalayageRootDirectory(File balayageRootDirectory) {
        this.balayageRootDirectory = balayageRootDirectory;
    }

    public ExtractionContext getExtractionContext() {
        if (extractionContextInit) {
            return extractionContext;
        }
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        LocalisationLists localisationLists = balayage.getLocalisationLists();
        if (localisationLists == null) {
            localisationLists = LocalisationLists.newInstance(bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray());
        }
        FicheEligibility ficheEligibility = null;
        int optionsCount = balayage.getGlobalFicheSelectOptionsCount();
        if (optionsCount > 0) {
            FicheSelector ficheSelector = new FicheSelector(basedefiches, localisationLists.getWorkingLangInteger(0));
            for (int i = 0; i < optionsCount; i++) {
                ficheSelector.addFicheSelectOptions(balayage.getGlobalFicheSelectOptions(i));
            }
            AbregeFicheSortedList abregeFicheSortedList = SelectionUtils.noSort(ficheSelector);
            ficheEligibility = abregeFicheSortedList;
        }
        extractionContext = new InternalExtractionContext(basedefiches, localisationLists, ficheEligibility);
        extractionContextInit = true;
        return extractionContext;
    }

    /**
     * @return the fileGenerationListener
     */
    public FileGenerationListener getFileGenerationListener() {
        return fileGenerationListener;
    }

    /**
     * @param fileGenerationListener the fileGenerationListener to set
     */
    public void setFileGenerationListener(FileGenerationListener fileGenerationListener) {
        this.fileGenerationListener = fileGenerationListener;
    }


    private static class InternalExtractionContext implements ExtractionContext {

        private Basedefiches basedefiches;
        private LocalisationLists localisationLists;
        private FicheEligibility ficheEligibility;
        private LinkAnalyser linkAnalyser = new DefaultLinkAnalyser();

        private InternalExtractionContext(Basedefiches basedefiches, LocalisationLists localisationLists, FicheEligibility ficheEligibility) {
            this.basedefiches = basedefiches;
            this.localisationLists = localisationLists;
            this.ficheEligibility = ficheEligibility;
        }

        public Basedefiches getBasedefiches() {
            return basedefiches;
        }

        public LocalisationLists getLocalisationLists() {
            return localisationLists;
        }

        public FicheEligibility getFicheEligibility() {
            return ficheEligibility;
        }

        public FicheEligibility getStaticFicheEligibility() {
            return ficheEligibility;
        }

        public LinkAnalyser getLinkAnalyser() {
            return linkAnalyser;
        }

    }


    private static class DefaultLinkAnalyser implements LinkAnalyser {

        public boolean isExternalLink(String href) {
            if (href == null) {
                return false;
            }
            return href.startsWith("http://");
        }

    }

}
