/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.balayages.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class ExtensionFilterFactory {

    private final static ExtensionFilter ACCEPT_ALL_FILTER = new AcceptAllExtensionFilter();

    private ExtensionFilterFactory() {
    }

    public static ExtensionFilter fromBalayageUnit(BalayageUnit balayageUnit) {
        int count = balayageUnit.getExtensionCount();
        if (count == 0) {
            return ACCEPT_ALL_FILTER;
        }
        HashSet<String> set = new HashSet<String>();
        for (int i = 0; i < count; i++) {
            set.add(balayageUnit.getExtension(i));
        }
        return new SetExtensionFilter(set);
    }


    private static class AcceptAllExtensionFilter implements ExtensionFilter {

        public boolean acceptExtension(String extension) {
            return true;
        }

    }


    private static class SetExtensionFilter implements ExtensionFilter {

        private Set<String> set;

        SetExtensionFilter(Set<String> set) {
            this.set = set;
        }

        public boolean acceptExtension(String extension) {
            return set.contains(extension);
        }

    }

}
