/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.balayages.engine;

import fr.exemole.bdfserver.balayages.api.SiteMapOption;
import fr.exemole.bdfserver.balayages.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import java.io.*;


/**
 *
 * @author Vincent Calame
 */
public class SiteMapBuilder implements FileGenerationListener {

    private List<String> urlList = new ArrayList<String>();
    private SiteMapOption siteMapOption;

    public SiteMapBuilder(SiteMapOption siteMapOption) {
        this.siteMapOption = siteMapOption;
    }

    public void generateFile(String langDir, String path, String fileName) {
        if (!fileName.endsWith(".html")) {
            return;
        }
        StringBuffer buf = new StringBuffer();
        if (langDir != null) {
            buf.append(langDir);
            buf.append('/');
        }
        if ((path != null) && (path.length() > 0)) {
            buf.append(path);
            buf.append('/');
        }
        buf.append(fileName);
        urlList.add(buf.toString());
    }

    public String writeSiteMapFile(File directory) throws IOException {
        String path = siteMapOption.getPath();
        if (path != null) {
            directory = new File(directory, path);
        }
        File destination = new File(directory, siteMapOption.getFileName());
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(destination), "UTF-8"));
        SiteMapWriter siteMapWriter = new SiteMapWriter(siteMapOption.getBaseUrl());
        siteMapWriter.setAppendable(writer);
        siteMapWriter.appendXMLDeclaration();
        siteMapWriter.setIndentLength(-999);
        siteMapWriter.start();
        for (String url : urlList) {
            siteMapWriter.addUrl(url);
        }
        siteMapWriter.end();
        writer.close();
        return destination.getPath();
    }


    private static class SiteMapWriter extends DefaultXMLWriter {

        private String baseUrl;

        SiteMapWriter(String baseUrl) {
            if (baseUrl == null) {
                this.baseUrl = "";
            } else {
                this.baseUrl = baseUrl;
            }
        }

        public void start() throws IOException {
            startOpenTag("urlset");
            addAttribute("xmlns", "http://www.sitemaps.org/schemas/sitemap/0.9");
            endOpenTag();
        }

        public void addUrl(String url) throws IOException {
            openTag("url");
            addSimpleElement("loc", baseUrl + url);
            closeTag("url");
        }

        public void end() throws IOException {
            closeTag("urlset");
        }

    }

}
