/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.Output;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import fr.exemole.bdfserver.balayages.engine.EngineUtils;
import net.mapeadores.util.text.ValueResolver;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.selection.FicheEligibility;


/**
 *
 * @author Vincent Calame
 */
public class DocumentBalayageUnitRunner extends AbstractBalayageUnitRunner {

    private DocumentOutputRunner[] outputRunnerArray;

    public DocumentBalayageUnitRunner(BalayageParameters balayageParameters, int balayageIndex, BalayageUnit balayageUnit) {
        super(balayageParameters, balayageIndex, balayageUnit);
        List<DocumentOutputRunner> outputList = new ArrayList<DocumentOutputRunner>();
        int outputCount = balayageUnit.getOutputCount();
        int p = 0;
        for (int i = 0; i < outputCount; i++) {
            Output output = balayageUnit.getOutput(i);
            DocumentOutputRunner computedOutput = new DocumentOutputRunner(balayageParameters, p, output, balayageIndex);
            if (computedOutput.isValid()) {
                outputList.add(computedOutput);
                p++;
            }
        }
        outputRunnerArray = outputList.toArray(new DocumentOutputRunner[outputList.size()]);
    }

    protected void run(short balayageUnitType) {
        Basedefiches basedefiches = extractionContext.getBasedefiches();
        FicheEligibility ficheEligibility = extractionContext.getFicheEligibility();
        FicheSelectOptions ficheSelectOptions = balayageUnit.getFicheSelectOptions();
        Addenda addenda = (Addenda) basedefiches.getSubset(SubsetKey.ADDENDA);
        boolean globalSelect = balayageUnit.isGlobalSelect();
        if ((ficheSelectOptions == null) && ((!globalSelect) || (ficheEligibility == null))) {
            for (int i = 0; i < addenda.getTicketLength(); i++) {
                Document document = (Document) addenda.getSubsetItemByTicket(i);
                if (document != null) {
                    documentBalayage(document);
                }
            }
        } else {
            FicheSelector ficheSelector = new FicheSelector(basedefiches, defaultWorkingLangInteger);
            if (ficheSelectOptions == null) {
                ficheSelector.addFicheSelectOptions(new FicheSelectOptions());
            } else {
                ficheSelector.addFicheSelectOptions(ficheSelectOptions);
                globalSelect = true;
            }
            if (globalSelect) {
                ficheSelector.setFicheEligibility(ficheEligibility);
            }
            Set<Document> documentSet = new HashSet<Document>();
            AbregeFicheSortedList abregeFicheSortedList = SelectionUtils.sortByIdcorpus(ficheSelector);
            for (int k = 0; k < abregeFicheSortedList.getCorpusCount(); k++) {
                Corpus corpus = abregeFicheSortedList.getCorpus(k);
                SubsetKey corpusKey = corpus.getSubsetKey();
                SubsetItemList abregeFicheList = abregeFicheSortedList.getCorpusAbregeFicheList(k);
                Croisement croisement = basedefiches.getCroisement(corpusKey, SubsetKey.ADDENDA);
                int ficheCount = abregeFicheList.getSubsetItemCount();
                for (int j = 0; j < ficheCount; j++) {
                    AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                    CroisementItem croisementItem = croisement.getCroisementItem(corpusKey, abrege.getTicket());
                    int ciCount = croisementItem.getCount();
                    for (int l = 0; l < ciCount; l++) {
                        int docTicket = croisementItem.getTicket(l);
                        Document document = (Document) addenda.getSubsetItemByTicket(docTicket);
                        if (document != null) {
                            documentSet.add(document);
                        }

                    }
                }
            }
            for (Document document : documentSet) {
                documentBalayage(document);
            }
        }
    }

    private void documentBalayage(Document document) {
        ValueResolver patternValueSet = EngineUtils.getValueResolver(document);
        long startTime = System.currentTimeMillis();
        int length = outputRunnerArray.length;
        for (int i = 0; i < length; i++) {
            DocumentOutputRunner outputRunner = outputRunnerArray[i];
            outputRunner.copy(patternValueSet, document, startTime);
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        int length = outputRunnerArray.length;
        for (int i = 0; i < length; i++) {
            outputRunnerArray[i].dispose();
            outputRunnerArray[i] = null;
        }
        outputRunnerArray = null;
    }

    protected boolean isEmpty() {
        return (outputRunnerArray.length == 0);
    }

}
