/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.Output;
import fr.exemole.bdfserver.balayages.engine.BalayageLog;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import fr.exemole.bdfserver.balayages.engine.EngineUtils;
import fr.exemole.bdfserver.balayages.engine.ExtensionFilter;
import fr.exemole.bdfserver.balayages.engine.ExtensionFilterFactory;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.association.VersionName;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class DocumentOutputRunner extends AbstractOutputRunner {

    private ExtensionFilter extensionFilter;

    public DocumentOutputRunner(BalayageParameters balayageParameters, int outputIndex, Output output, int balayageUnitIndex) {
        super(balayageParameters, outputIndex, output, balayageUnitIndex);
        extensionFilter = ExtensionFilterFactory.fromBalayageUnit(output.getBalayageUnit());
    }

    public void copy(ValueResolver valueResolver, Document document, long startTime) {
        File directory = rootDirectory;
        String path = EngineUtils.getPath(output, valueResolver);
        if (path != null) {
            directory = new File(directory, path);
        }
        prepareDirectory(directory);
        for (int i = 0; i < document.getVersionCount(); i++) {
            Version version = document.getVersion(i);
            VersionName versionName = version.getVersionName();
            if (extensionFilter.acceptExtension(versionName.getExtension())) {
                File destination = new File(directory, versionName.toString());
                try {
                    OutputStream outputStream = new FileOutputStream(destination);
                    InputStream inputStream = version.getInputStream();
                    IOUtils.copy(inputStream, outputStream);
                    inputStream.close();
                    outputStream.close();
                    balayageLog.addFileGeneration(balayageUnitIndex, outputIndex, destination, System.currentTimeMillis() - startTime);
                } catch (IOException ioe) {
                    String msg = ioe.getMessage();
                    String message = ioe.getClass().getName();
                    if (msg != null) {
                        message = message + " : " + msg;
                    }
                    balayageLog.addOutputError(balayageUnitIndex, outputIndex, output, BalayageLog.IO_EXCEPTION, destination.getName(), message);
                }
            }
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        extensionFilter = null;
    }

}
