/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.BalayageUnit;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.AlbumList;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptions;
import net.mapeadores.basedefiches.selection.SelectionUtils;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationBalayageUnitRunner extends AbstractBalayageUnitRunner {

    private IllustrationOutputRunner[] outputRunnerArray;

    public IllustrationBalayageUnitRunner(BalayageParameters balayageParameters, int balayageIndex, BalayageUnit balayageUnit) {
        super(balayageParameters, balayageIndex, balayageUnit);
        List<IllustrationOutputRunner> outputList = new ArrayList<IllustrationOutputRunner>();
        int outputCount = balayageUnit.getOutputCount();
        int p = 0;
        for (int i = 0; i < outputCount; i++) {
            IllustrationOutputRunner outputRunner = new IllustrationOutputRunner(balayageParameters, p, balayageUnit.getOutput(i), balayageIndex);
            if (outputRunner.isValid()) {
                outputList.add(outputRunner);
                p++;
            }
        }
        outputRunnerArray = outputList.toArray(new IllustrationOutputRunner[outputList.size()]);
    }

    protected void run(short balayageUnitType) {
        IllustrationSelectOptions illustrationSelectOptions = balayageUnit.getIllustrationSelectOptions();
        AlbumList albumList = SelectionUtils.getAlbumList(extractionContext.getBasedefiches(), illustrationSelectOptions);
        String[] albumDimNameArray = null;
        if (illustrationSelectOptions != null) {
            albumDimNameArray = illustrationSelectOptions.getAlbumDimNameArray();
        }
        int albumCount = albumList.getAlbumCount();
        int outputRunnerLength = outputRunnerArray.length;
        for (int j = 0; j < albumCount; j++) {
            long startTime = System.currentTimeMillis();
            Album album = albumList.getAlbum(j);
            for (int i = 0; i < outputRunnerLength; i++) {
                if (i > 0) {
                    startTime = System.currentTimeMillis();
                }
                IllustrationOutputRunner outputRunner = outputRunnerArray[i];
                outputRunner.copy(album, albumDimNameArray, startTime);
            }
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        int length = outputRunnerArray.length;
        for (int i = 0; i < length; i++) {
            outputRunnerArray[i].dispose();
            outputRunnerArray[i] = null;
        }
        outputRunnerArray = null;
    }

    protected boolean isEmpty() {
        return (outputRunnerArray.length == 0);
    }

}
