/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.balayages.engine.runners;

import fr.exemole.bdfserver.balayages.BalayageConstants;
import fr.exemole.bdfserver.balayages.BalayageUtils;
import fr.exemole.bdfserver.balayages.Output;
import fr.exemole.bdfserver.balayages.engine.BalayageLog;
import fr.exemole.bdfserver.balayages.engine.BalayageParameters;
import fr.exemole.bdfserver.balayages.engine.EngineUtils;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.AccoladePattern;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationOutputRunner extends AbstractOutputRunner {

    private final static short NOSPECIALDIM = 0;
    private AccoladePattern pattern;

    public IllustrationOutputRunner(BalayageParameters balayageParameters, int outputIndex, Output output, int balayageUnitIndex) {
        super(balayageParameters, outputIndex, output, balayageUnitIndex);
        pattern = BalayageUtils.getPattern(output, BalayageConstants.ILLUSTRATION_TYPE);
    }

    public void copy(Album album, String[] albumDimArray, long startTime) {
        if (albumDimArray == null) {
            copyFiles(album, null, AlbumConstants.ORIGINAL_SPECIALDIM, startTime);
        } else {
            int length = albumDimArray.length;
            for (int i = 0; i < length; i++) {
                String albumDimName = albumDimArray[i];
                if (albumDimName.equals(AlbumConstants.MINI_DIMNAME)) {
                    copyFiles(album, null, AlbumConstants.MINI_SPECIALDIM, startTime);
                } else {
                    AlbumDim albumDim = album.getAlbumMetadata().getAlbumDimByName(albumDimName);
                    if (albumDim != null) {
                        copyFiles(album, albumDim, NOSPECIALDIM, startTime);
                    }
                }
            }
        }
    }

    private void copyFiles(Album album, AlbumDim albumDim, short specialDim, long startTime) {
        IllustrationValueResolver valueResolver = new IllustrationValueResolver(album, albumDim, specialDim);
        File directory = rootDirectory;
        String path = EngineUtils.getPath(output, valueResolver);
        if (path != null) {
            directory = new File(directory, path);
        }
        prepareDirectory(directory);
        int length = album.getTicketLength();
        for (int i = 0; i < length; i++) {
            Illustration illustration = (Illustration) album.getSubsetItemByTicket(i);
            if (illustration != null) {
                valueResolver.setIllustration(illustration);
                String fileName = pattern.format(valueResolver) + "." + AlbumUtils.formatTypeToString(illustration.getFormatType());
                File destination = new File(directory, fileName);
                try {
                    OutputStream outputStream = new FileOutputStream(destination);
                    InputStream inputStream = null;
                    if (albumDim != null) {
                        inputStream = illustration.getInputStream(albumDim);
                    } else {
                        inputStream = illustration.getInputStream(specialDim);
                    }
                    IOUtils.copy(inputStream, outputStream);
                    inputStream.close();
                    outputStream.close();
                    balayageLog.addFileGeneration(balayageUnitIndex, outputIndex, destination, System.currentTimeMillis() - startTime);
                } catch (IOException ioe) {
                    String msg = ioe.getMessage();
                    String message = ioe.getClass().getName();
                    if (msg != null) {
                        message = message + " : " + msg;
                    }
                    balayageLog.addOutputError(balayageUnitIndex, outputIndex, output, BalayageLog.IO_EXCEPTION, destination.getName(), message);
                }
            }
        }
    }


    private static class IllustrationValueResolver implements ValueResolver {

        private Album album;
        private AlbumDim albumDim;
        private short specialDim;
        private Illustration illustration;

        private IllustrationValueResolver(Album album, AlbumDim albumDim, short specialDim) {
            this.album = album;
            this.albumDim = albumDim;
            this.specialDim = specialDim;
        }

        public String getValue(AccoladeArgument patternArgument) {
            String patternArgumentName = patternArgument.getName();
            if (patternArgumentName.equals("album")) {
                return album.getSubsetKey().getSubsetName();
            }
            if (patternArgumentName.equals("albumdim")) {
                if (albumDim == null) {
                    return AlbumUtils.specialDimToString(specialDim);
                } else {
                    return albumDim.getName();
                }
            }
            if (patternArgumentName.equals("idalbum")) {
                if (illustration != null) {
                    return String.valueOf(illustration.getIdalbum());
                }
            }
            return AccoladePattern.toString(patternArgument);
        }

        private void setIllustration(Illustration illustration) {
            this.illustration = illustration;
        }

    }

}
