/* BdfServer
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.commands;

import org.w3c.dom.*;


/**
 *
 * @author Vincent Calame
 */
public class CommandListDOMReader {

    private ErrorHandler errorHandler;
    private Builder builder;

    public CommandListDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }

    public void readCommandList(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("command")) {
                    addCommand(element);
                } else {
                    //errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    public void addCommand(Element element_xml) {
        String commandName = element_xml.getAttribute("name");
        if (commandName.length() == 0) {
            return;
        }
        String keyString = element_xml.getAttribute("key");
        if (keyString.length() == 0) {
            return;
        }
        CommandDefBuilder commandDefBuilder = builder.createCommand(commandName);
        commandDefBuilder.setKey(keyString);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("param")) {
                    String name = element.getAttribute("name");
                    if (name.length() == 0) {
                        continue;
                    }
                    commandDefBuilder.addParameter(name);
                } else {
                    //errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }


    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

    }


    public static interface Builder {

        public CommandDefBuilder createCommand(String name);

    }


    public static interface CommandDefBuilder {

        public void setKey(String key);

        public void addParameter(String name);

    }

}
