/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.implementations;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.storage.BdfStorage;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class PasswordManagerImpl implements PasswordManager {

    private BdfServer bdfServer;
    private BdfStorage.PasswordStorage passwordStorage;
    private Map<String, String> md5PasswordMap = new LinkedHashMap<String, String>();

    public PasswordManagerImpl(BdfServer bdfServer, BdfStorage.PasswordStorage passwordStorage) {
        this.bdfServer = bdfServer;
        this.passwordStorage = passwordStorage;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public synchronized void check(List<String> passwordList) {
        md5PasswordMap.clear();
        md5PasswordMap.put(CONFIRM_PASSWORDKEY, StringUtils.toMD5("test"));
        String key = null;
        for (String s : passwordList) {
            if (key != null) {
                md5PasswordMap.put(key, s);
                key = null;
            } else {
                key = s;
            }
        }
    }

    public boolean checkPassword(String passwordKey, String passwordValue) {
        String currentMd5Value = md5PasswordMap.get(passwordKey);
        if (currentMd5Value == null) {
            return false;
        }
        String md5Value = StringUtils.toMD5(passwordValue);
        return currentMd5Value.equals(md5Value);
    }

    public synchronized void setPassword(String passwordKey, String passwordValue) {
        if ((passwordValue == null) || (passwordValue.length() == 0)) {
            if (passwordKey.equals(CONFIRM_PASSWORDKEY)) {
                throw new IllegalArgumentException(passwordKey + " password cannot be null or empty");
            } else {
                md5PasswordMap.remove(passwordKey);
            }
        } else {
            md5PasswordMap.put(passwordKey, StringUtils.toMD5(passwordValue));
        }
        save();
    }

    public String getMD5Password(String passwordKey) {
        return md5PasswordMap.get(passwordKey);
    }

    public String[] toPasswordArray() {
        int size = md5PasswordMap.size();
        String[] array = new String[size * 2];
        int p = 0;
        for (Map.Entry<String, String> entry : md5PasswordMap.entrySet()) {
            array[2 * p] = entry.getKey();
            array[2 * p + 1] = entry.getValue();
            p++;
        }
        return array;
    }

    private void save() {
        String[] array = toPasswordArray();
        passwordStorage.savePasswordArray(array);
    }

}
