/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.implementations;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ScrutariExportManager;
import fr.exemole.bdfserver.storage.BdfStorage;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportManagerImpl implements ScrutariExportManager {

    private BdfServer bdfServer;
    private BdfStorage.ScrutariExportStorage scrutariExportStorage;
    private SortedMap<String, ScrutariExportDef> scrutariExportDefMap = new TreeMap<String, ScrutariExportDef>();
    private ScrutariExportDef[] scrutariExportDefArray;

    public ScrutariExportManagerImpl(BdfServer bdfServer, BdfStorage.ScrutariExportStorage scrutariExportStorage) {
        this.bdfServer = bdfServer;
        this.scrutariExportStorage = scrutariExportStorage;
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public synchronized void check(List<ScrutariExportDef> scrutariExportDefList) {
        scrutariExportDefMap.clear();
        for (ScrutariExportDef scrutariExportDef : scrutariExportDefList) {
            scrutariExportDefMap.put(scrutariExportDef.getName(), scrutariExportDef);
        }
        updateArray();
    }

    public int getScrutariExportDefCount() {
        return scrutariExportDefArray.length;
    }

    public ScrutariExportDef getScrutariExportDef(int i) {
        return scrutariExportDefArray[i];
    }

    public ScrutariExportDef getScrutariExportDef(String name) {
        return scrutariExportDefMap.get(name);
    }

    public synchronized void putScrutariExportDef(ScrutariExportDef scrutariExportDef) {
        String name = scrutariExportDef.getName();
        scrutariExportDefMap.put(name, scrutariExportDef);
        updateArray();
        scrutariExportStorage.saveScrutariExportDef(scrutariExportDef);
    }

    public synchronized void removeScrutariExportDef(String name) {
        boolean here = scrutariExportDefMap.containsKey(name);
        if (!here) {
            return;
        }
        scrutariExportDefMap.remove(name);
        scrutariExportStorage.removeScrutariExportDef(name);
        updateArray();
    }

    private void updateArray() {
        int size = scrutariExportDefMap.size();
        scrutariExportDefArray = new ScrutariExportDef[size];
        int p = 0;
        for (ScrutariExportDef scrutariExportDef : scrutariExportDefMap.values()) {
            scrutariExportDefArray[p] = scrutariExportDef;
            p++;
        }
    }

}
