/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.implementations;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.SqlExportManager;
import fr.exemole.bdfserver.storage.BdfStorage;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportManagerImpl implements SqlExportManager {

    private BdfServer bdfServer;
    private BdfStorage.SqlExportStorage sqlExportStorage;
    private SortedMap<String, SqlExportDef> sqlExportDefMap = new TreeMap<String, SqlExportDef>();
    private SqlExportDef[] sqlExportDefArray;

    public SqlExportManagerImpl(BdfServer bdfServer, BdfStorage.SqlExportStorage sqlExportStorage) {
        this.bdfServer = bdfServer;
        this.sqlExportStorage = sqlExportStorage;
    }

    public synchronized void check(List<SqlExportDef> sqlExportDefList) {
        sqlExportDefMap.clear();
        for (SqlExportDef scrutariExportDef : sqlExportDefList) {
            sqlExportDefMap.put(scrutariExportDef.getName(), scrutariExportDef);
        }
        updateArray();
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public int getSqlExportDefCount() {
        return sqlExportDefArray.length;
    }

    public SqlExportDef getSqlExportDef(int i) {
        return sqlExportDefArray[i];
    }

    public SqlExportDef getSqlExportDef(String name) {
        return sqlExportDefMap.get(name);
    }

    public synchronized void putSqlExportDef(SqlExportDef sqlExportDef) {
        String name = sqlExportDef.getName();
        sqlExportDefMap.put(name, sqlExportDef);
        updateArray();
        sqlExportStorage.saveSqlExportDef(sqlExportDef);
    }

    public synchronized void removeSqlExportDef(String name) {
        boolean here = sqlExportDefMap.containsKey(name);
        if (!here) {
            return;
        }
        sqlExportDefMap.remove(name);
        sqlExportStorage.removeSqlExportDef(name);
        updateArray();
    }

    private void updateArray() {
        int size = sqlExportDefMap.size();
        sqlExportDefArray = new SqlExportDef[size];
        int p = 0;
        for (SqlExportDef scrutariExportDef : sqlExportDefMap.values()) {
            sqlExportDefArray[p] = scrutariExportDef;
            p++;
        }
    }

}
