/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.implementations;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.xsltfactory.XsltResource;
import java.io.File;
import java.io.IOException;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.URIResolver;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.DocStream;


/**
 *
 * @author Vincent Calame
 */
public class TransformationURIResolver implements URIResolver {

    private final static String INC_BDF_DIR = "bdf:_inc_/";
    private final static String ETC_BDF_DIR = "bdf:_etc_/";
    private final static String VAR_BDF_DIR = "bdf:_var_/";
    private final static String BDF_PROTOCOL = "bdf:";
    private BdfServer bdfServer;

    public TransformationURIResolver(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public Source resolve(String href, String base) throws TransformerException {
        String resourceName = href;
        if (resourceName.startsWith(INC_BDF_DIR)) {
            File incDir = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.BALAYAGEINCLUDE_OUTPUTDIRECTORY);
            String filePath = resourceName.substring(INC_BDF_DIR.length());
            File f = new File(incDir, filePath);
            if (f.exists() && (!f.isDirectory())) {
                return new StreamSource(f);
            } else {
                throw new TransformerException(filePath + " (No such file or directory in bdf:_inc_/ )");
            }
        }
        if (resourceName.startsWith(ETC_BDF_DIR)) {
            String filePath = resourceName.substring(ETC_BDF_DIR.length());
            File f = bdfServer.getTransformationManager().getEtcCommonsFile(filePath);
            if (f.exists() && (!f.isDirectory())) {
                return new StreamSource(f);
            } else {
                throw new TransformerException(filePath + " (No such file in Etc Commons )");
            }
        }
        if (resourceName.startsWith(VAR_BDF_DIR)) {
            String filePath = resourceName.substring(VAR_BDF_DIR.length());
            File f = bdfServer.getTransformationManager().getVarCommonsFile(filePath);
            if (f.exists() && (!f.isDirectory())) {
                return new StreamSource(f);
            } else {
                throw new TransformerException(filePath + " (No such file in Var Commons )");
            }
        }
        if (!resourceName.startsWith(BDF_PROTOCOL)) {
            if (!resourceName.startsWith("_")) {
                return null;
            } else if (resourceName.lastIndexOf("_") < 1) {
                return null;
            } else if (resourceName.lastIndexOf("/") != -1) {
                return null;
            }
        } else {
            resourceName = resourceName.substring(BDF_PROTOCOL.length());
        }
        XsltResource innerResource = XsltResource.parse(resourceName);
        if (innerResource == null) {
            throw new TransformerException(resourceName + " (Unknown bdf resource)");
        }
        DocStream docStream = bdfServer.getInnerResourceDocStream(innerResource, true);
        if (docStream == null) {
            throw new TransformerException(resourceName + " (No such bdf resource)");
        }
        try {
            return new StreamSource(docStream.getInputStream());
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

}
