/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.l10n;

import java.text.DecimalFormatSymbols;
import java.util.AbstractList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.RandomAccess;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.DateFormatBundle;


/**
 *
 * @author Vincent Calame
 */
public final class LocaleBundle {

    public final static String KILOBYTE_KEY = "kilobyte";
    public final static String MEGABYTE_KEY = "megabyte";
    public final static String CREATION_KEY = "date_creation";
    public final static String MODIFICATION_KEY = "date_modification";
    public final static String ID_FORMAT_KEY = "id-format";
    public final static String FILELENGTH_FORMAT_KEY = "filelength-format";
    public final static List KEYLIST = new KeyList();
    private static HashMap hashMap = new HashMap();
    private Locale locale;
    private DecimalFormatSymbols symbols;
    private DateFormatBundle dateFormatBundle;

    private LocaleBundle(Locale locale) {
        this.locale = locale;
        symbols = new DecimalFormatSymbols(locale);
        dateFormatBundle = DateFormatBundle.getDateFormatBundle(locale);
    }

    public Locale getLocale() {
        return locale;
    }

    public DecimalFormatSymbols getDecimalFormatSymbols() {
        return symbols;
    }

    public String getLocalizedString(String key) {
        if (key.equals(ID_FORMAT_KEY)) {
            return getIdFormat();
        }
        if (key.equals(FILELENGTH_FORMAT_KEY)) {
            return getFileLenghtFormat();
        }
        return key;
    }

    public DateFormatBundle getDateFormatBundle() {
        return dateFormatBundle;
    }

    public String getFileLenghtFormat() {
        return "0" + symbols.getDecimalSeparator() + "#";
    }

    public String getIdFormat() {
        return "#" + symbols.getGroupingSeparator() + "##0";
    }

    public static LocaleBundle getLocaleBundle(Locale locale) {
        LocaleBundle localeBundle = (LocaleBundle) hashMap.get(locale);
        if (localeBundle == null) {
            synchronized (hashMap) {
                localeBundle = new LocaleBundle(locale);
                hashMap.put(locale, localeBundle);
            }
        }
        return localeBundle;
    }


    private static class KeyList extends AbstractList implements RandomAccess {

        String[] keyArray = {CREATION_KEY, MODIFICATION_KEY, KILOBYTE_KEY, MEGABYTE_KEY, ID_FORMAT_KEY, FILELENGTH_FORMAT_KEY};

        KeyList() {
        }

        public int size() {
            return keyArray.length;
        }

        public Object get(int i) {
            return keyArray[i];
        }

    }

    public String getLangString() {
        return LangInteger.toString(LangInteger.fromLocale(locale));
    }

}
