/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.libscol;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUIUtils;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUILibelleMap;
import java.io.IOException;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeLibelleMap;
import net.mapeadores.basedefiches.xml.libs.LibsXMLWriter;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.xml.DefaultXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class LibsColXMLWriter extends DefaultXMLWriter {

    LibsXMLWriter libsXMLWriter;
    private int langInteger;
    private BdfServer bdfServer;
    private Basedefiches basedefiches;

    public LibsColXMLWriter(BdfServer bdfServer, int langInteger, Appendable appendable, int indentLength) {
        super();
        setAppendable(appendable);
        setIndentLength(indentLength);
        this.langInteger = langInteger;
        libsXMLWriter = new LibsXMLWriter(langInteger, appendable, indentLength + 1);
        this.bdfServer = bdfServer;
        this.basedefiches = bdfServer.getBasedefiches();
    }

    public void appendLibsCol(LibsColKey libsColKey) throws IOException {
        openTag("libscol");
        if (libsColKey.isFromSubset()) {
            SubsetKey subsetKey = libsColKey.getSubsetKey();
            Subset subset = bdfServer.getBasedefiches().getSubset(subsetKey);
            if (subset instanceof Corpus) {
                appendCorpus((Corpus) subset);
            }
        } else if (libsColKey.equals(LibsColKey.ALLCORPUS)) {
            appendAllCorpus();
        } else {
            throw new SwitchException("Unknown libsColKey = " + libsColKey.toString());
        }
        closeTag("libscol");
    }

    private void appendAllCorpus() throws IOException {
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            appendCorpus(basedefiches.getCorpus(i));
        }
    }

    private void appendCorpus(Corpus corpus) throws IOException {
        libsXMLWriter.appendCorpusLibs(corpus.getCorpusMetadata());
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        appendCorpusUI(corpusUI);
    }

    private void appendCorpusUI(CorpusUI corpusUI) throws IOException {
        startOpenTag("libs");
        addAttribute("category", "corpusui");
        addAttribute("corpus", corpusUI.getCorpusSubsetKey().getSubsetName());
        endOpenTag();
        IncludeUILibelleMap libelleSourceCorpusInclude = bdfServer.getLibelleManager().getIncludeUILibelleMap(corpusUI.getCorpusSubsetKey());
        int count = corpusUI.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = corpusUI.getComponentUI(i);
            if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                String lib = libelleSourceCorpusInclude.getLibelleString(includeUI, langInteger);
                appendIncludeUILib(includeUI.getIncludeKey().toString(), lib);
            }
        }
        appendIncludeKeyLib(libelleSourceCorpusInclude, IncludeKey.DATE_CREATION);
        appendIncludeKeyLib(libelleSourceCorpusInclude, IncludeKey.DATE_MODIFICATION);
        List<CommentUI> commentList = CorpusUIUtils.getCommentUIList(corpusUI);
        for (CommentUI commentUI : commentList) {
            appendCommentUILib(commentUI.getName(), commentUI.getHtmlStringByLang(langInteger));
        }
        closeTag("libs");
    }

    private void appendIncludeKeyLib(IncludeLibelleMap includeLibelleMap, IncludeKey includeKey) throws IOException {
        String lib = includeLibelleMap.getLibelleString(includeKey, langInteger);
        appendIncludeUILib(includeKey.toString(), lib);
    }

    private void appendIncludeUILib(String includeKey, String lib) throws IOException {
        startOpenTag("lib");
        addAttribute("include-key", includeKey.toString());
        endOpenTag(false);
        escape(lib);
        closeTag("lib", false, false);
    }

    private void appendCommentUILib(String commentName, String lib) throws IOException {
        startOpenTag("lib");
        addAttribute("comment-name", commentName);
        endOpenTag(false);
        escape(lib);
        closeTag("lib", false, false);
    }

    public static String getLibsColXML(int langInteger, BdfServer bdfServer, LibsColKey libsColKey, boolean withTabs) {
        StringBuilder buf = new StringBuilder(1024);
        try {
            LibsColXMLWriter libsXMLWriter = new LibsColXMLWriter(bdfServer, langInteger, buf, (withTabs) ? 0 : -999);
            libsXMLWriter.appendXMLDeclaration();
            libsXMLWriter.appendLibsCol(libsColKey);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
