/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageConstants;
import fr.exemole.bdfserver.storage.BalayagesStorage;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.extraction.builders.ExtractionDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.FicheFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.IntituleClauseBuilder;
import net.mapeadores.basedefiches.extraction.builders.MotcleFilterBuilder;
import net.mapeadores.basedefiches.xml.extraction.ExtractionDOMReader;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.exceptions.NestedSAXException;
import net.mapeadores.util.exceptions.SwitchException;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class BalayageManager implements BalayageConstants {

    private BdfServer bdfServer;
    private BalayagesStorage balayageStorage;
    private SortedMap<String, Balayage> balayageMap = new TreeMap<String, Balayage>();
    private Balayage[] balayageArray;

    public BalayageManager(BdfServer bdfServer, BalayagesStorage balayageStorage) {
        this.bdfServer = bdfServer;
        this.balayageStorage = balayageStorage;
        update();
    }

    public void update() {
        balayageMap.clear();
        List<Balayage> list = balayageStorage.checkStorage();
        for (Balayage balayage : list) {
            balayageMap.put(balayage.getName(), balayage);
        }
        int size = balayageMap.size();
        balayageArray = new Balayage[size];
        int p = 0;
        for (Balayage balayage : balayageMap.values()) {
            balayageArray[p] = balayage;
            p++;
        }
    }

    public int getBalayageCount() {
        return balayageArray.length;
    }

    public Balayage getBalayage(int i) {
        return balayageArray[i];
    }

    public Balayage getBalayage(String name) {
        return balayageMap.get(name);
    }

    /**
     * Retourne <em>null</em> si la ressource n'existe pas et une exception s'il y a un problème
     * pour y accéder.
     *
     */
    public ExtractionDef getExtractionDef(Balayage balayage, String extractionPath) {
        try {
            InputStream inputStream = balayageStorage.getExtractionInputStream(balayage.getName(), extractionPath);
            if (inputStream == null) {
                return null;
            }
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docbuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = docbuilder.parse(inputStream);
            ExtractionDOMReader extractionDomReader = new ExtractionDOMReader();
            ExtractionDef extractionDef = extractionDomReader.readExtraction(document.getDocumentElement());
            return extractionDef;
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        } catch (org.xml.sax.SAXException se) {
            throw new NestedSAXException(se);
        } catch (ParserConfigurationException pce) {
            throw new NestedLibraryException(pce);
        }
    }

    public ExtractionDef getDefaultExtractionDef(short balayageUnitType) {
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        switch (balayageUnitType) {
            case CORPUS_TYPE:
                extractionDefBuilder.setDynamicExtraitCorpusDef(BuilderUtils.getExtraitCorpusDef(FicheFilterBuilder.TITRE_FICHEFILTER, null));
                break;
            case FICHE_TYPE:
                FicheFilter ficheFilter = FicheFilterBuilder.newFicheInstance(null);
                extractionDefBuilder.setDynamicExtraitCorpusDef(BuilderUtils.getExtraitCorpusDef(ficheFilter, IntituleClauseBuilder.DEFAULT));
                break;
            case MOTCLE_TYPE:
                MotcleFilter aloneFilter = MotcleFilterBuilder.getAloneInstance();
                extractionDefBuilder.setDynamicExtraitThesaurusDef(BuilderUtils.getExtraitThesaurusDef(aloneFilter));
                break;
            case THESAURUS_TYPE:
                MotcleFilter thesaurusFilter = MotcleFilterBuilder.getThesaurusInstance();
                extractionDefBuilder.setDynamicExtraitThesaurusDef(BuilderUtils.getExtraitThesaurusDef(thesaurusFilter));
                break;
            case UNIQUE_TYPE:
                break;
            default:
                throw new SwitchException("wrong balayageUnitType = " + balayageUnitType);
        }
        return extractionDefBuilder.toExtractionDef();
    }

    public Source getTransformerSource(Balayage balayage, String xsltPath) {
        return balayageStorage.getTransformerSource(balayage.getName(), xsltPath);
    }

}
