/* BdfServer
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.managers;

import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.localisation.*;
import fr.exemole.bdfserver.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public final class LangConfig {

    private boolean withOriginalName = false;
    private boolean withoutNomAvant = false;
    private boolean allLanguages = false;
    private int[] workingLangIntegerArray = new int[1];

    {
        workingLangIntegerArray[0] = LangInteger.FRENCH;
    }

    private int[] supplementaryLangIntegerArray = new int[0];

    public LangConfig() {
    }

    /*public LangConfig(LangConfig oldInstance) {
    this.allLanguages = oldInstance.allLanguages;
    setWorkingLangIntegerArray(oldInstance.workingLangIntegerArray);
    setSupplementaryLangIntegerArray(oldInstance.supplementaryLangIntegerArray);
    //this.withoutNomAvant =
    }*/
    public boolean isAllLanguages() {
        return allLanguages;
    }

    public void setAllLanguages(boolean allLanguages) {
        this.allLanguages = allLanguages;
    }

    public int[] getWorkingLangIntegerArray() {
        return workingLangIntegerArray;
    }

    /*
     *
     * @throws IllegalArgumentException si le tableau est de longueur nulle.
     */
    public void setWorkingLangIntegerArray(int[] intArray) {
        if (intArray.length == 0) {
            throw new IllegalArgumentException("intArray.length == 0");
        }
        workingLangIntegerArray = ArrayUtils.cleanCopy(intArray);
    }

    public void setWorkingLangIntegerArray(String[] langStringArray) {
        if (langStringArray == null) {
            return;
        }
        int[] newArray = ArrayUtils.cleanCopy(LangIntegerUtils.toLangIntegerArray(langStringArray));
        if (newArray.length > 0) {
            workingLangIntegerArray = newArray;
        }
    }

    public int[] getSupplementaryLangIntegerArray() {
        return supplementaryLangIntegerArray;
    }

    public void setSupplementaryLangIntegerArray(int[] intArray) {
        supplementaryLangIntegerArray = ArrayUtils.cleanCopy(intArray);
    }

    public void setSupplementaryLangIntegerArray(String[] langStringArray) {
        if (langStringArray == null) {
            return;
        }
        int[] newArray = ArrayUtils.cleanCopy(LangIntegerUtils.toLangIntegerArray(langStringArray));
        supplementaryLangIntegerArray = newArray;
    }

    public LocalisationInfo getDefaultLocalisationInfo() {
        return new LocalisationInfo(getDefaultWorkingLangInteger(), getDefaultFormatLocale());
    }

    public boolean containsWorkingLangInteger(int workingLangInteger) {
        int length = workingLangIntegerArray.length;
        for (int i = 0; i < length; i++) {
            if (workingLangInteger == workingLangIntegerArray[i]) {
                return true;
            }
        }
        return false;
    }

    public int getDefaultWorkingLangInteger() {
        return workingLangIntegerArray[0];
    }

    public Locale getDefaultFormatLocale() {
        Locale formatLocale = AvailableFormatLocales.getNearestAvailableLocale(workingLangIntegerArray[0]);
        if (formatLocale == null) {
            formatLocale = AvailableFormatLocales.getDefaultLocale();
        }
        return formatLocale;
    }

    public Collection getAllLocaleSet() {
        HashSet itgSet = new HashSet();
        for (int i = 0; i < workingLangIntegerArray.length; i++) {
            itgSet.add(new Integer(workingLangIntegerArray[i]));
        }
        for (int i = 0; i < supplementaryLangIntegerArray.length; i++) {
            itgSet.add(new Integer(supplementaryLangIntegerArray[i]));
        }
        int[] array = AvailableFormatLocales.getAvailableArray();
        for (int i = 0; i < array.length; i++) {
            int itg = array[i];
            itgSet.add(new Integer(itg));
            itgSet.add(new Integer(LangInteger.getLangPart(itg)));
        }
        TreeMap resultMap = new TreeMap();
        for (Iterator it = itgSet.iterator(); it.hasNext();) {
            int langInteger = ((Integer) it.next()).intValue();
            resultMap.put(LangInteger.toString(langInteger), LangInteger.getLocale(langInteger));
        }
        return resultMap.values();
    }

    public boolean isWithOriginalName() {
        return withOriginalName;
    }

    public void setWithOriginalName(boolean withOriginalName) {
        this.withOriginalName = withOriginalName;
    }

    public int[] getAvailableLangIntegerArray() {
        return ArrayUtils.merge(workingLangIntegerArray, supplementaryLangIntegerArray);
    }

    public boolean isWithoutNomAvant() {
        return withoutNomAvant;
    }

    public void setWithoutNomAvant(boolean withoutNomAvant) {
        this.withoutNomAvant = withoutNomAvant;
    }

}
