/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;

import java.util.*;
import javax.xml.transform.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.localisation.*;


/**
 *
 * @author Vincent Calame
 */
public class TransformerParameters implements TransformationConstants {

    private Map<String, String> priorParamMap = new LinkedHashMap<String, String>();
    private Map<String, String> paramMap = new LinkedHashMap<String, String>();

    public void setPriorParam(String paramName, String paramValue) {
        if (paramValue == null) {
            priorParamMap.remove(paramName);
        } else {
            priorParamMap.put(paramName, paramValue);
            if (paramMap.containsKey(paramName)) {
                paramMap.remove(paramName);
            }
        }
    }

    public boolean setParam(String paramName, String paramValue) {
        if (paramValue == null) {
            paramMap.remove(paramName);
            return true;
        } else {
            if (!priorParamMap.containsKey(paramName)) {
                paramMap.put(paramName, paramValue);
                return true;
            } else {
                return false;
            }
        }
    }

    public void initParameters(Transformer transformer) {
        for (Map.Entry<String, String> entry : priorParamMap.entrySet()) {
            String paramName = entry.getKey();
            String paramValue = entry.getValue();
            initParameter(transformer, paramName, paramValue);
        }
        for (Map.Entry<String, String> entry : paramMap.entrySet()) {
            String paramName = entry.getKey();
            String paramValue = entry.getValue();
            initParameter(transformer, paramName, paramValue);
        }
    }

    private void initParameter(Transformer transformer, String paramName, String paramValue) {
        transformer.setParameter(paramName, paramValue);
        if (paramName.equals(BDF_FILENAME_PARAMETER)) {
            String fileBaseName = paramValue;
            String fileExtension = "";
            int idx = paramValue.lastIndexOf('.');
            if (idx != -1) {
                fileBaseName = paramValue.substring(0, idx);
                fileExtension = paramValue.substring(idx + 1);
            }
            transformer.setParameter(BDF_FILEBASENAME_PARAMETER, fileBaseName);
            transformer.setParameter(BDF_FILEEXTENSION_PARAMETER, fileExtension);
        }
    }

    public void setLocalisationInfo(LocalisationInfo localisationInfo) {
        setParam(WORKINGLANG_PARAMETER, LangInteger.toString(localisationInfo.getWorkingLangInteger()));
        setParam(FORMATLOCALE_PARAMETER, LangInteger.toString(localisationInfo.getFormatLocaleInteger()));
    }

    public Map<String, String> toMap() {
        Map<String, String> map = new HashMap<String, String>();
        map.putAll(priorParamMap);
        map.putAll(paramMap);
        return map;
    }

}
