/* BdfServer - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.transformation;


/**
 *
 * @author Vincent Calame
 */
public final class XslTransformerKey {

    private TransformationKey transformationKey;
    private String transformerName;
    private int hashCode;

    /**
     *
     *@throws IllegalArgumentException si sourceKey est nul
     */
    public XslTransformerKey(TransformationKey transformationKey, String templateName) {
        if (transformationKey == null) {
            throw new IllegalArgumentException("transformationKey is null");
        }
        this.transformerName = templateName;
        this.transformationKey = transformationKey;
        initHashCode();
    }

    private void initHashCode() {
        hashCode = transformationKey.hashCode();
        if (transformerName != null) {
            hashCode = hashCode + transformerName.hashCode();
        }
    }

    public String getTransformerName() {
        return transformerName;
    }

    public TransformationKey getTransformationKey() {
        return transformationKey;
    }

    @Override
    public int hashCode() {
        return hashCode;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj.hashCode() != this.hashCode) {
            return false;
        }
        if (!(obj instanceof XslTransformerKey)) {
            return false;
        }
        XslTransformerKey other = (XslTransformerKey) obj;
        if (!other.transformationKey.equals(this.transformationKey)) {
            return false;
        }
        if (this.transformerName == null) {
            return (other.transformerName == null);
        }
        if (other.transformerName == null) {
            return false;
        }
        return (other.transformerName.equals(this.transformerName));
    }

    public boolean isDefaultTransformerKey() {
        return (transformerName == null);
    }

    public XslTransformerKey toDefaultTransformerKey() {
        if (transformerName == null) {
            return this;
        }
        return new XslTransformerKey(transformationKey, null);
    }

    public static XslTransformerKey getDefaultXslTransformerKey(TransformationKey transformationKey) {
        return new XslTransformerKey(transformationKey, null);
    }

}
