/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPath;
import fr.exemole.bdfserver.managers.configuration.LinkPathGroup;
import fr.exemole.bdfserver.managers.configuration.PageSubstitution;
import java.io.IOException;
import java.util.List;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class ConfigurationManagerXMLWriter extends ManagerXMLWriter {

    public ConfigurationManagerXMLWriter() {
    }

    public void appendManager(Object manager) throws IOException {
        appendConfigurationManager((ConfigurationManager) manager);
    }

    public void appendConfigurationManager(ConfigurationManager configurationManager) throws IOException {
        openTag("configuration-manager");
        appendLangConfig(configurationManager.getLangConfig());
        appendPathList(configurationManager.getConfigurationPathList());
        appendPageSubstitutionList(configurationManager.getPageSubstitutionList());
        List<LinkPathGroup> linkPathGroupList = configurationManager.getLinkPathGroupList();
        for (LinkPathGroup linkPathGroup : linkPathGroupList) {
            appendLinkPathGroup(linkPathGroup);
        }
        closeTag("configuration-manager");
    }

    private void appendPathList(List<ConfigurationPath> list) throws IOException {
        if (list.isEmpty()) {
            return;
        }
        openTag("path-list");
        for (ConfigurationPath configurationPath : list) {
            appendPath(configurationPath);
        }
        closeTag("path-list");
    }

    private void appendPageSubstitutionList(List<PageSubstitution> list) throws IOException {
        if (list.isEmpty()) {
            return;
        }
        openTag("page-list");
        for (PageSubstitution pageSubstitution : list) {
            appendPageSubstitution(pageSubstitution);
        }
        closeTag("page-list");
    }

    private void appendPath(ConfigurationPath configurationPath) throws IOException {
        startOpenTag("path");
        addAttribute("name", configurationPath.getName());
        addAttribute("public", configurationPath.getPublicURLString());
        addAttribute("system", configurationPath.getSystemDirectoryString());
        closeEmptyTag();
    }

    private void appendPageSubstitution(PageSubstitution pageSubstitution) throws IOException {
        startOpenTag("page");
        addAttribute("original", pageSubstitution.getOriginalName());
        addAttribute("substitution", pageSubstitution.getSubstitutionName());
        closeEmptyTag();
    }

    private void appendLangConfig(LangConfig langConfig) throws IOException {
        openTag("lang-config");
        int[] workingLangArray = langConfig.getWorkingLangIntegerArray();
        for (int i = 0; i < workingLangArray.length; i++) {
            startOpenTag("working-lang");
            addAttribute("lang", LangInteger.toString(workingLangArray[i]));
            closeEmptyTag();
        }
        int[] supplementaryLangArray = langConfig.getSupplementaryLangIntegerArray();
        for (int i = 0; i < supplementaryLangArray.length; i++) {
            startOpenTag("supplementary-lang");
            addAttribute("lang", LangInteger.toString(supplementaryLangArray[i]));
            closeEmptyTag();
        }
        if (langConfig.isAllLanguages()) {
            addEmptyElement("all-languages");
        }
        if (langConfig.isWithOriginalName()) {
            addEmptyElement("with-original-name");
        }
        if (langConfig.isWithoutNomAvant()) {
            addEmptyElement("without-nom-avant");
        }
        closeTag("lang-config");
    }

    private void appendLinkPathGroup(LinkPathGroup linkPathGroup) throws IOException {
        startOpenTag("linkpath-group");
        addAttribute("name", linkPathGroup.getName());
        endOpenTag();
        int count = linkPathGroup.getLinkPathCount();
        for (int i = 0; i < count; i++) {
            String linkPath = linkPathGroup.getLinkPath(i);
            startOpenTag("linkpath");
            addAttribute("value", linkPath);
            closeEmptyTag();
        }
        closeTag("linkpath-group");
    }

}
