/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import net.mapeadores.basedefiches.sphere.RedacteurKey;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public abstract class ManagerDOMReader {

    private ErrorHandler errorHandler;

    public void setErrorHandler(ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }

    public abstract void readManager(Object manager, Element element_xml);

    public RedacteurKey getRedacteurKey(Element element_xml) {
        String idsphereString = element_xml.getAttribute("idsphere");
        String sphere = element_xml.getAttribute("sphere");
        try {
            RedacteurKey redacteurKey = RedacteurKey.parse(sphere + "_" + idsphereString);
            return redacteurKey;
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    public void unknownTagWarning(String tagName) {
        if (errorHandler != null) {
            errorHandler.unknownTagWarning(tagName);
        }
    }

    public void malformedURLError(String tagName, String attribute, String urlString) {
        if (errorHandler != null) {
            errorHandler.malformedURLError(tagName, attribute, urlString);
        }
    }

    public void fileNotFoundError(String tagName, String attribute, String filePath) {
        if (errorHandler != null) {
            errorHandler.fileNotFoundError(tagName, attribute, filePath);
        }
    }

    public void emptyAttributeError(String tagName, String attributeName) {
        if (errorHandler != null) {
            errorHandler.emptyAttributeError(tagName, attributeName);
        }
    }


    public static interface ErrorHandler {

        public void unknownTagWarning(String tagName);

        public void malformedURLError(String tagName, String attribute, String urlString);

        public void fileNotFoundError(String tagName, String attribute, String filePath);

        public void emptyAttributeError(String tagName, String attributeName);

    }

}
