/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.policies.ThesaurusPolicy;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class PolicyManagerDOMReader extends ManagerDOMReader {

    private PolicyManager currentPolicyManager;

    public PolicyManagerDOMReader() {
    }

    public void readManager(Object manager, Element element_xml) {
        readPolicyManager((PolicyManager) manager, element_xml);
    }

    public void readPolicyManager(PolicyManager policyManager, Element element_xml) {
        this.currentPolicyManager = policyManager;
        DOMUtils.readChildren(element_xml, new RootHandler());
        this.currentPolicyManager = null;
    }


    private class RootHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("thesaurus-policy-list")) {
                DOMUtils.readChildren(element, new ThesaurusPolicyListHandler());
            } else if (tagName.equals("user-allow-policy")) {
                DOMUtils.readChildren(element, new UserAllowPolicyHandler());
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class UserAllowPolicyHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("data")) {
                currentPolicyManager.setDataModifAllowed(true);
            } else if (tagName.equals("password")) {
                currentPolicyManager.setPasswordModifAllowed(true);
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class ThesaurusPolicyListHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("thesaurus-policy")) {
                String name = element.getAttribute("thesaurus-name");
                ThesaurusPolicy thesaurusPolicy = null;
                try {
                    SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, name);
                    thesaurusPolicy = currentPolicyManager.getThesaurusPolicy(thesaurusKey);
                } catch (ParseException pe) {
                    return;
                }
                if (thesaurusPolicy == null) {
                    return;
                }
                short ajout = 0;
                try {
                    ajout = ThesaurusPolicy.ajoutPolicyToShort(element.getAttribute("ajout"));
                } catch (IllegalArgumentException iae) {
                    return;
                }
                ThesaurusHandler thesaurusHandler = new ThesaurusHandler();
                DOMUtils.readChildren(element, thesaurusHandler);
                List<SubsetKey> thesaurusList = thesaurusHandler.getThesaurusList();
                int size = thesaurusList.size();
                if (ajout == ThesaurusPolicy.AJOUT_VERIFICATION) {
                    if (size > 0) {
                        thesaurusPolicy.setAjoutPolicy(ThesaurusPolicy.AJOUT_VERIFICATION, thesaurusList);
                    }
                } else if (ajout == ThesaurusPolicy.AJOUT_TRANSFERT) {
                    if (size > 0) {
                        thesaurusPolicy.setAjoutPolicy(ThesaurusPolicy.AJOUT_TRANSFERT, thesaurusList.get(0));
                    }
                } else {
                    thesaurusPolicy.setAjoutPolicy(ajout, null);
                }
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class ThesaurusHandler implements ElementHandler {

        List<SubsetKey> thesaurusList = new ArrayList<SubsetKey>();

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("thesaurus")) {
                String name = DOMUtils.readSimpleElement(element);
                if (name.length() > 0) {
                    try {
                        thesaurusList.add(SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, name));
                    } catch (ParseException pe) {
                    }

                }
            } else {
                unknownTagWarning(tagName);
            }
        }

        public List<SubsetKey> getThesaurusList() {
            return thesaurusList;
        }

    }

}
