/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.policies.ThesaurusPolicy;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class PolicyManagerXMLWriter extends ManagerXMLWriter {

    public PolicyManagerXMLWriter() {
    }

    public void appendManager(Object manager) throws IOException {
        appendPolicyManager((PolicyManager) manager);
    }

    public void appendPolicyManager(PolicyManager policyManager) throws IOException {
        openTag("policy-manager");
        appendThesaurusPolicyList(policyManager);
        appendUserAllowPolicy(policyManager);
        closeTag("policy-manager");
    }

    public void appendUserAllowPolicy(PolicyManager policyManager) throws IOException {
        boolean data = policyManager.isDataModifAllowed();
        boolean password = policyManager.isPasswordModifAllowed();
        if ((!data) && (!password)) {
            return;
        }
        openTag("user-allow-policy");
        if (data) {
            addEmptyElement("data");
        }
        if (password) {
            addEmptyElement("password");
        }
        closeTag("user-allow-policy");
    }

    public void appendThesaurusPolicyList(PolicyManager policyManager) throws IOException {
        Basedefiches basedefiches = policyManager.getBdfServer().getBasedefiches();
        int thesaurusCount = basedefiches.getThesaurusCount();
        openTag("thesaurus-policy-list");
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            ThesaurusPolicy thesaurusPolicy = policyManager.getThesaurusPolicy(thesaurus.getSubsetKey());
            appendThesaurusPolicy(thesaurusPolicy);
        }
        closeTag("thesaurus-policy-list");
    }

    public void appendThesaurusPolicy(ThesaurusPolicy thesaurusPolicy) throws IOException {
        if (thesaurusPolicy.getAjoutPolicy() == ThesaurusPolicy.AJOUT_AUCUN) {
            return;
        }
        startOpenTag("thesaurus-policy");
        addAttribute("thesaurus-name", thesaurusPolicy.getThesaurusKey().getSubsetName());
        addAttribute("ajout", ThesaurusPolicy.ajoutPolicyToString(thesaurusPolicy.getAjoutPolicy()));
        List<SubsetKey> list = getThesaurusKeyList(thesaurusPolicy);
        if (!list.isEmpty()) {
            endOpenTag();
            for (SubsetKey thesaurusKey : list) {
                appendThesaurus(thesaurusKey);
            }
            closeTag("thesaurus-policy");
        } else {
            closeEmptyTag();
        }
    }

    private void appendThesaurus(SubsetKey thesaurusKey) throws IOException {
        addSimpleElement("thesaurus", thesaurusKey.getSubsetName());
    }

    private List<SubsetKey> getThesaurusKeyList(ThesaurusPolicy thesaurusPolicy) {
        short ajout = thesaurusPolicy.getAjoutPolicy();
        if (ajout == ThesaurusPolicy.AJOUT_VERIFICATION) {
            return thesaurusPolicy.getVerifThesaurusKeyList();
        } else if (ajout == ThesaurusPolicy.AJOUT_TRANSFERT) {
            return Collections.singletonList(thesaurusPolicy.getTransferThesaurusKey());
        } else {
            return Collections.emptyList();
        }
    }

}
