/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.managers.xml;

import fr.exemole.bdfserver.managers.SqlExportManager;
import java.text.ParseException;
import net.mapeadores.basedefiches.sqlexport.SqlExportDefBuilder;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportManagerDOMReader extends ManagerDOMReader {

    private SqlExportManager currentSqlExportManager;

    public SqlExportManagerDOMReader() {
    }

    public void readManager(Object manager, Element element_xml) {
        readSqlExportManager((SqlExportManager) manager, element_xml);
    }

    public void readSqlExportManager(SqlExportManager sqlExportManager, Element element_xml) {
        this.currentSqlExportManager = sqlExportManager;
        DOMUtils.readChildren(element_xml, new RootHandler());
        this.currentSqlExportManager = null;

    }


    private class RootHandler implements ElementHandler {

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("sqlexport")) {
                String name = element.getAttribute("name");
                if (!StringUtils.isTechnicalName(name, true)) {
                    return;
                }
                try {
                    SqlExportDefBuilder sqlExportDefBuilder = SqlExportDefBuilder.newInstance(name);
                    SqlExportHandler sqlExportHandler = new SqlExportHandler(sqlExportDefBuilder);
                    DOMUtils.readChildren(element, sqlExportHandler);
                    currentSqlExportManager.putSqlExportDef(sqlExportDefBuilder.toSqlExportDef());
                } catch (ParseException pe) {
                }
            } else {
                unknownTagWarning(tagName);
            }
        }

    }


    private class SqlExportHandler implements ElementHandler {

        private SqlExportDefBuilder sqlExportDefBuilder;

        SqlExportHandler(SqlExportDefBuilder sqlExportDefBuilder) {
            this.sqlExportDefBuilder = sqlExportDefBuilder;
        }

        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("param")) {
                String name = element.getAttribute("name").trim();
                if (name.length() == 0) {
                    return;
                }
                String value = element.getAttribute("value").trim();
                sqlExportDefBuilder.addParam(name, value);
            } else {
                String contenu = DOMUtils.readSimpleElement(element);
                if (contenu.length() == 0) {
                    return;
                }
                if (tagName.equals("path")) {
                    sqlExportDefBuilder.setPath(contenu);
                } else if (tagName.equals("url")) {
                    sqlExportDefBuilder.setUrl(contenu);
                } else if (tagName.equals("tableexport-name")) {
                    sqlExportDefBuilder.setTableExportName(contenu);
                } else if (tagName.equals("sqlexport-classname")) {
                    sqlExportDefBuilder.setSqlExportClassName(contenu);
                } else if (tagName.equals("post-command")) {
                    sqlExportDefBuilder.setPostCommand(contenu);
                } else {
                    unknownTagWarning(tagName);
                }
            }
        }

    }

}
