/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formfactory;

import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.tostring.FicheBlockToString;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.SeparatorOptions;


/**
 *
 * @author Vincent Calame
 */
public class FicheFormUtils {

    private FicheFormUtils() {
    }

    public static String getStringValue(CorpusField corpusField, FicheFormParameters ficheFormParameters) {
        FicheItemToString.Parameters parameters = getParameters(ficheFormParameters, corpusField);
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        FieldKey fieldKey = corpusField.getFieldKey();
        Object obj = fichePointeur.getValue(fieldKey);
        if (obj == null) {
            return "";
        }
        if (obj instanceof Integer) {
            return obj.toString();
        }
        if (obj instanceof String) {
            return TextParser.escapeString((String) obj);
        }
        if (obj instanceof FicheItem) {
            return FicheItemToString.toString((FicheItem) obj, parameters);
        }
        if (obj instanceof FicheItemList) {
            boolean isBlock = CorpusMetadataUtils.isBlockDisplayInformationField(corpusField);
            SeparatorOptions separatorOptions = (isBlock) ? ficheFormParameters.getBlockSeparatorOptions() : ficheFormParameters.getInlineSeparatorOptions();
            return FicheItemToString.toString((FicheItemList) obj, separatorOptions, parameters);
        }
        if (obj instanceof FicheBlockList) {
            return FicheBlockToString.toString((FicheBlockList) obj, ficheFormParameters.isWithSpecialParseChars());
        }
        throw new SwitchException("Unexpected instance of " + obj.getClass().getName());
    }

    public static FicheItemToString.Parameters getParameters(FicheFormParameters ficheFormParameters, CorpusField corpusField) {
        SubsetKey defaultSphereKey = corpusField.getFieldOptions().getDefaultSphereKey();
        if (defaultSphereKey == null) {
            return ficheFormParameters.getFicheItemToStringParameters();
        }
        FicheItemToString.Parameters newParam = new FicheItemToString.Parameters(ficheFormParameters.getFicheItemToStringParameters());
        newParam.setDefautSphereKey(defaultSphereKey);
        return newParam;
    }

    public static String getStringValue(Thesaurus thesaurus, MotcleCroisementList motcleList, boolean ignorePoids, FicheFormParameters ficheFormParameters) {
        if (motcleList == null) {
            return "";
        }
        boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        boolean isBabelien = ThesaurusUtils.isBabelienType(thesaurus);
        int langInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, ficheFormParameters.getWorkingLangInteger());
        SeparatorOptions separatorOptions = ficheFormParameters.getInlineSeparatorOptions();
        int size = motcleList.getSubsetItemCount();
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < size; i++) {
            if (i > 0) {
                appendSeparator(buf, separatorOptions);
            }
            Motcle motcle = motcleList.getMotcle(i);
            int poids = motcleList.getPoids(i);
            if (withIdalpha) {
                buf.append(motcle.getIdalpha());
            } else {
                String lib = motcle.getLibelleString(langInteger, false);
                if ((lib == null) || (lib.length() == 0)) {
                    if (!isBabelien) {
                        lib = String.valueOf(motcle.getIdths());
                    } else {
                        lib = motcle.getLibelleString(0, true);
                    }
                }
                buf.append(lib);
            }
            if (!ignorePoids) {
                appendPoids(buf, motcleList.getPoids(i));
            }
        }
        if (separatorOptions.isLastInclude()) {
            appendSeparator(buf, separatorOptions);
        }
        return buf.toString();
    }

    private static void appendPoids(StringBuffer stringBuffer, int poids) {
        if (poids > 1) {
            if (poids == 2) {
                stringBuffer.append('+');
            } else if (poids == 3) {
                stringBuffer.append("++");
            } else if (poids == 4) {
                stringBuffer.append("+++");
            } else {
                stringBuffer.append(" <");
                stringBuffer.append(poids);
                stringBuffer.append('>');
            }
        }
    }

    private static void appendSeparator(StringBuffer stringBuffer, SeparatorOptions separatorOptions) {
        stringBuffer.append(separatorOptions.getSeparator());
        if (separatorOptions.isSpaceInclude()) {
            stringBuffer.append(' ');
        }
    }

}
