/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp;

import fr.exemole.bdfserver.request.form.CorpusFieldElement;
import fr.exemole.bdfserver.request.form.FicheForm;
import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.ficheform.AlbumIncludeElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.FieldElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.GeopointSubfieldsElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.ImageSubfieldsElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.ThesaurusIncludeElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.LangElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.CorpusIncludeElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.MontantSubfieldsElementBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.PersonneSubfieldBuilder;
import fr.exemole.bdfserver.request.formimp.ficheform.SpecialIncludeElementBuilder;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;


/**
 *
 * @author Vincent Calame
 */
public class DefaultFicheForm implements FicheForm {

    private FicheFormParameters ficheFormParameters;

    public DefaultFicheForm(FicheFormParameters ficheFormParameters) {
        this.ficheFormParameters = ficheFormParameters;
    }

    public IncludeElement getFormElement(IncludeUI includeUI) {
        IncludeKey includeKey = includeUI.getIncludeKey();
        if (includeKey.isSpecialInclude()) {
            if (includeKey.equals(IncludeKey.LIAGE)) {
                return CorpusIncludeElementBuilder.toIncludeElement(ficheFormParameters, includeUI);
            }
            return SpecialIncludeElementBuilder.toIncludeElement(ficheFormParameters, includeUI);
        } else if (includeKey.isThesaurusInclude()) {
            return ThesaurusIncludeElementBuilder.toIncludeElement(ficheFormParameters, includeUI);
        } else if (includeKey.isCorpusInclude()) {
            return CorpusIncludeElementBuilder.toIncludeElement(ficheFormParameters, includeUI);
        } else if (includeKey.isAlbumInclude()) {
            return AlbumIncludeElementBuilder.toIncludeElement(ficheFormParameters, includeUI);
        }
        return null;
    }

    public CorpusFieldElement getFormElement(FieldUI fieldUI) {
        CorpusField corpusField = fieldUI.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        if (fieldKey == FieldKey.IDCORPUS) {
            return null;
        }
        if ((fieldKey == FieldKey.TITRE) && (CorpusMetadataUtils.isTitreGenerated(corpusField))) {
            return null;
        }
        if (fieldKey == FieldKey.LANG) {
            return LangElementBuilder.toCorpusFieldElement(ficheFormParameters, corpusField);
        }
        if (CorpusMetadataUtils.hasPersonneSubfields(corpusField)) {
            return PersonneSubfieldBuilder.toCorpusFieldElement(ficheFormParameters, fieldUI);
        }
        if (CorpusMetadataUtils.hasGeopointSubfields(corpusField)) {
            return GeopointSubfieldsElementBuilder.toCorpusFieldElement(ficheFormParameters, fieldUI);
        }
        if (CorpusMetadataUtils.isCurrencyArrayField(corpusField)) {
            return MontantSubfieldsElementBuilder.toCorpusFieldElement(ficheFormParameters, fieldUI);
        }
        if (CorpusMetadataUtils.hasImageSubfields(corpusField)) {
            return ImageSubfieldsElementBuilder.toCorpusFieldElement(ficheFormParameters, fieldUI);
        }
        return FieldElementBuilder.toCorpusFieldElement(ficheFormParameters, fieldUI);
    }

}
