/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.form.AlbumIncludeElement;
import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractIncludeElement;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.IllustrationCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class AlbumIncludeElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private IncludeUI includeUI;
    private IncludeKey includeKey;
    private Album album;

    private AlbumIncludeElementBuilder(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.includeUI = includeUI;
        this.includeKey = includeUI.getIncludeKey();
        SubsetKey albumKey = includeKey.getSubsetKey();
        album = (Album) ficheFormParameters.getBdfServer().getBasedefiches().getSubset(albumKey);
    }

    public static IncludeElement toIncludeElement(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        AlbumIncludeElementBuilder albumIncludeElementBuilder = new AlbumIncludeElementBuilder(ficheFormParameters, includeUI);

        return albumIncludeElementBuilder.toIncludeElement();
    }

    private IncludeElement toIncludeElement() {
        if (album == null) {
            return null;
        }
        InternalAlbumIncludeElement albumIncludeElement = new InternalAlbumIncludeElement(includeKey);
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        IllustrationCroisementList illustrationCroisementList = fichePointeur.getIllustrationCroisementList(album);
        if (includeKey.hasPoidsFilter()) {
            illustrationCroisementList = CroisementUtils.poidsFilter(illustrationCroisementList, includeKey.getPoidsFilter());
        } else {
            IncludeSummary.SubsetInfo info = ficheFormParameters.getIncludeSummary().getSubsetInfo(album.getSubsetKey());
            RangeList rangeList = null;
            if (info != null) {
                rangeList = info.getPoidsFilterRangeList();
                if (rangeList != null) {
                    illustrationCroisementList = CroisementUtils.excludePoids(illustrationCroisementList, rangeList);
                }
            }
        }
        if ((illustrationCroisementList != null) && (illustrationCroisementList.getSubsetItemCount() == 0)) {
            illustrationCroisementList = null;
        }
        albumIncludeElement.setIllustrationCroisementList(illustrationCroisementList);
        FicheFormImpUtils.setIntitule(ficheFormParameters, albumIncludeElement, includeUI);
        return albumIncludeElement;
    }


    private static class InternalAlbumIncludeElement extends AbstractIncludeElement implements AlbumIncludeElement {

        private IllustrationCroisementList illustrationCroisementList = null;

        private InternalAlbumIncludeElement(IncludeKey includeKey) {
            super(includeKey);
        }

        public int getIllustrationCount() {
            if (illustrationCroisementList == null) {
                return 0;
            }
            return illustrationCroisementList.getSubsetItemCount();
        }

        public Illustration getIllustration(int i) {
            if (illustrationCroisementList == null) {
                throw new IndexOutOfBoundsException();
            }
            return illustrationCroisementList.getIllustration(i);
        }

        private void setIllustrationCroisementList(IllustrationCroisementList illustrationCroisementList) {
            this.illustrationCroisementList = illustrationCroisementList;
        }

    }

}
