/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractIncludeElement;
import fr.exemole.bdfserver.request.formimp.DefaultCorpusIncludeElement;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.LimitOption;
import java.util.ArrayList;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.PointeurUtils;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusIncludeElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private IncludeUI includeUI;
    private IncludeKey includeKey;
    private boolean hasPoidsFilter;
    private ArrayList<AbregeFiche> abrList = new ArrayList<AbregeFiche>();
    private ArrayList<Integer> poidsList;

    private CorpusIncludeElementBuilder(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.includeUI = includeUI;
        this.includeKey = includeUI.getIncludeKey();
        this.hasPoidsFilter = includeKey.hasPoidsFilter();
    }

    public static IncludeElement toIncludeElement(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        CorpusIncludeElementBuilder builder = new CorpusIncludeElementBuilder(ficheFormParameters, includeUI);
        AbstractIncludeElement includeElement = builder.toIncludeElement();
        if (includeElement != null) {
            FicheFormImpUtils.setIntitule(ficheFormParameters, includeElement, includeUI);
            includeElement.setMandatory(FicheFormImpUtils.isMandatory(includeUI));
        }
        return includeElement;
    }

    private AbstractIncludeElement toIncludeElement() {
        DefaultCorpusIncludeElement element = new DefaultCorpusIncludeElement(includeKey);
        element.setLocalisationInfo(ficheFormParameters.getLocalisationInfo());
        if (includeKey.equals(IncludeKey.LIAGE)) {
            if (!ficheFormParameters.isNewFiche()) {
                initListsForLiage();
            }
            LimitOption limitOption = includeUI.getLimitOption();
            boolean noNew = limitOption.isNoNewAllowed();
            if (noNew) {
                if (ficheFormParameters.isNewFiche()) {
                    return null;
                } else {
                    if (abrList.size() == 0) {
                        return null;
                    }
                }
            }
        } else {
            Corpus corpus = (Corpus) ficheFormParameters.getBdfServer().getBasedefiches().getSubset(includeKey.getSubsetKey());
            if (corpus == null) {
                return null;
            }
            if (!ficheFormParameters.isNewFiche()) {
                initListsForCorpus(corpus);
            }
        }
        int size = abrList.size();
        if (size > 0) {
            AbregeFiche[] abregeFicheArray = new AbregeFiche[size];
            int[] poidsArray = new int[size];
            for (int i = 0; i < size; i++) {
                abregeFicheArray[i] = abrList.get(i);
                if (poidsList != null) {
                    poidsArray[i] = poidsList.get(i).intValue();
                } else {
                    poidsArray[i] = -1;
                }
            }
            element.setAbregeFicheList(abregeFicheArray, poidsArray);
        }
        String motcleSelectOptionsString = includeUI.getMotcleSelectOptionsString();
        if (motcleSelectOptionsString != null) {
            motcleSelectOptionsString = FicheFormImpUtils.replaceGenericParameters(ficheFormParameters, motcleSelectOptionsString);
            element.setMotcleSelectOptionsString(motcleSelectOptionsString);
        }
        return element;
    }

    private void initListsForLiage() {
        poidsList = new ArrayList<Integer>();
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        FicheCroisementList[] array = PointeurUtils.getNoList(fichePointeur, ficheFormParameters.getIncludeSummary());
        int length = array.length;
        for (int j = 0; j < length; j++) {
            FicheCroisementList ficheCroisementList = array[j];
            int size = ficheCroisementList.getSubsetItemCount();
            for (int i = 0; i < size; i++) {
                AbregeFiche abrg2 = ficheCroisementList.getAbregeFiche(i);
                int poids = ficheCroisementList.getPoids(i);
                abrList.add(abrg2);
                poidsList.add(new Integer(poids));
            }
        }
    }

    private void initListsForCorpus(Corpus corpus) {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        FicheCroisementList ficheCroisementList = fichePointeur.getFicheCroisementList(corpus);
        if (hasPoidsFilter) {
            int poidsFilter = includeKey.getPoidsFilter();
            int size = ficheCroisementList.getSubsetItemCount();
            for (int i = 0; i < size; i++) {
                int poids = ficheCroisementList.getPoids(i);
                if (poids == poidsFilter) {
                    abrList.add(ficheCroisementList.getAbregeFiche(i));
                }
            }
        } else {
            IncludeSummary.SubsetInfo info = ficheFormParameters.getIncludeSummary().getSubsetInfo(corpus.getSubsetKey());
            RangeList rangeList = null;
            if (info != null) {
                rangeList = info.getPoidsFilterRangeList();
            }
            poidsList = new ArrayList<Integer>();
            int size = ficheCroisementList.getSubsetItemCount();
            for (int i = 0; i < size; i++) {
                int poids = ficheCroisementList.getPoids(i);
                if ((rangeList == null) || (!rangeList.contains(poids))) {
                    AbregeFiche other = ficheCroisementList.getAbregeFiche(i);
                    abrList.add(other);
                    poidsList.add(poids);
                }
            }
        }
    }

}
