/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formfactory.FicheFormUtils;
import fr.exemole.bdfserver.request.formimp.AbstractCorpusFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultHiddenFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultItemFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultTextFieldElement;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.options.InputOption;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;


/**
 *
 * @author Vincent Calame
 */
public class FieldElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;
    private FieldKey fieldKey;

    public FieldElementBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
        this.fieldKey = corpusField.getFieldKey();
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        FieldElementBuilder builder = new FieldElementBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        if (fieldKey.ownsToCorpsdefiche()) {
            return toTextFieldElement();
        }
        InputOption inputOption = fieldUI.getInputOption();
        switch (inputOption.getInputType()) {
            case InputOption.INPUT_HIDDEN:
                return toHiddenFieldElement();
            default:
                return toItemFieldElement();
        }
    }

    private DefaultHiddenFieldElement toHiddenFieldElement() {
        DefaultHiddenFieldElement formElement = new DefaultHiddenFieldElement(corpusField);
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, fieldUI);
        } else {
            value = FicheFormUtils.getStringValue(corpusField, ficheFormParameters);
        }
        formElement.setValue(value);
        return formElement;
    }

    private DefaultTextFieldElement toTextFieldElement() {
        DefaultTextFieldElement formElement = new DefaultTextFieldElement(corpusField);
        formElement.setMandatory(isMandatory);
        formElement.setRows(fieldUI.getInputOption().getRows());
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, fieldUI);
        } else {
            value = FicheFormUtils.getStringValue(corpusField, ficheFormParameters);
        }
        formElement.setFormattedText(value);
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

    private DefaultItemFieldElement toItemFieldElement() {
        DefaultItemFieldElement formElement = new DefaultItemFieldElement(corpusField);
        formElement.setMandatory(isMandatory);
        InputOption inputOption = fieldUI.getInputOption();
        boolean redimAllowed = getDefaultRedimAllowed();
        formElement.setRedimAllowed(redimAllowed);
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, fieldUI);
        } else {
            value = FicheFormUtils.getStringValue(corpusField, ficheFormParameters);
        }
        formElement.setValue(value);
        int rows = inputOption.getRows();
        if (redimAllowed) {
            rows = FicheFormImpUtils.checkRows(rows, value.length());
        }
        if ((rows == 1) && (CorpusMetadataUtils.isBlockDisplayInformationField(corpusField))) {
            rows = 3;
        }
        formElement.setRows(rows);
        formElement.setWidthType(inputOption.getWidthType());
        if (corpusField.getFicheItemType() == CorpusField.PERSONNE_FIELD) {
            SubsetKey subsetKey = corpusField.getFieldOptions().getDefaultSphereKey();
            if (subsetKey == null) {
                subsetKey = ficheFormParameters.getDefaultSphereKey();
            }
            formElement.setSphereKey(subsetKey);
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

    private boolean getDefaultRedimAllowed() {
        if (!fieldKey.isProprieteFieldKey()) {
            return true;
        }
        switch (corpusField.getFicheItemType()) {
            case CorpusField.ITEM_FIELD:
            case CorpusField.PARA_FIELD:
                return true;
            default:
                return false;
        }
    }

}
