/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractCorpusFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultGeopointSubfieldsElement;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusFieldList;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;


/**
 *
 * @author Vincent Calame
 */
public class GeopointSubfieldsElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;

    private GeopointSubfieldsElementBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        GeopointSubfieldsElementBuilder builder = new GeopointSubfieldsElementBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FicheItemToString.Parameters parameters = ficheFormParameters.getFicheItemToStringParameters();
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultGeopointSubfieldsElement formElement = new DefaultGeopointSubfieldsElement(corpusField);
        formElement.setMandatory(isMandatory);
        CorpusFieldList corpusFieldArray = corpusField.getFieldOptions().getAddressFieldList();
        if ((corpusFieldArray != null) && (corpusFieldArray.getCorpusFieldCount() > 0)) {
            int length = corpusFieldArray.getCorpusFieldCount();
            String array[] = new String[length];
            for (int i = 0; i < length; i++) {
                array[i] = corpusFieldArray.getCorpusField(i).getFieldKey().toString();
            }
            formElement.setAdresseFieldNameArray(array);
        }
        FicheItem ficheItem = (FicheItem) fichePointeur.getValue(corpusField.getFieldKey());
        if (ficheItem != null) {
            if (ficheItem instanceof Geopoint) {
                Geopoint geopoint = (Geopoint) ficheItem;
                formElement.setLatitude(FicheItemToString.toString(geopoint.getLatitude(), parameters));
                formElement.setLongitude(FicheItemToString.toString(geopoint.getLongitude(), parameters));
            } else {
                formElement.setLatitude(FicheItemToString.toString(ficheItem, parameters));
            }
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
