/* BdfServer- Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.request.form.LangFieldElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractCorpusFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultLangFieldElement;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;


/**
 *
 * @author Vincent Calame
 */
public class LangElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;

    private LangElementBuilder(FicheFormParameters ficheFormParameters, CorpusField corpusField) {
        this.ficheFormParameters = ficheFormParameters;
        this.corpusField = corpusField;
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, CorpusField corpusField) {
        LangElementBuilder builder = new LangElementBuilder(ficheFormParameters, corpusField);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultLangFieldElement formElement = new DefaultLangFieldElement(corpusField);
        LangConfig langConfig = ficheFormParameters.getBdfServer().getConfigurationManager().getLangConfig();
        if (langConfig.isAllLanguages()) {
            formElement.setType(LangFieldElement.ANY_TYPE);
            if (ficheFormParameters.isNewFiche()) {
                formElement.setLangInteger(ficheFormParameters.getWorkingLangInteger());
            } else {
                formElement.setLangInteger(((AbregeFiche) fichePointeur.getCurrentSubsetItem()).getLangInteger());
            }
        } else {
            int[] array = langConfig.getAvailableLangIntegerArray();
            if (array.length == 1) {
                formElement.setType(LangFieldElement.UNIQUE_TYPE);
                formElement.setLangInteger(array[0]);
            } else {
                formElement.setType(LangFieldElement.ARRAY_TYPE);
                formElement.setAvailableLangArray(array);
                if (ficheFormParameters.isNewFiche()) {
                    formElement.setLangInteger(ficheFormParameters.getWorkingLangInteger());
                } else {
                    formElement.setLangInteger(((AbregeFiche) fichePointeur.getCurrentSubsetItem()).getLangInteger());
                }
            }
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
