/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractCorpusFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultMontantSubfieldsElement;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Montant;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.IntegerList;
import net.mapeadores.util.primitives.IntegerListUtils;


/**
 *
 * @author Vincent Calame
 */
public class MontantSubfieldsElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;

    private MontantSubfieldsElementBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        MontantSubfieldsElementBuilder builder = new MontantSubfieldsElementBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FicheItemToString.Parameters parameters = ficheFormParameters.getFicheItemToStringParameters();
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultMontantSubfieldsElement formElement = new DefaultMontantSubfieldsElement(corpusField);
        formElement.setMandatory(isMandatory);
        FicheItem ficheItem = (FicheItem) fichePointeur.getValue(corpusField.getFieldKey());
        IntegerList currencyList = corpusField.getFieldOptions().getCurrencyList();
        formElement.setCurrencyArray(IntegerListUtils.toArray(currencyList));
        formElement.setUnique((currencyList.getIntegerCount() == 1));
        String firstCur = CurrencyInteger.toString(currencyList.getInteger(0));
        if (ficheItem == null) {
            formElement.setCur(firstCur);
        } else if (ficheItem instanceof Montant) {
            Montant montant = (Montant) ficheItem;
            String curcur = CurrencyInteger.toString(montant.getCurrencyInteger());
            if (!curcur.equals(firstCur)) {
                formElement.setUnique(false);
            }
            formElement.setCur(curcur);
            formElement.setNum(montant.getDecimal().toStringWithBlank(parameters.getDecimalChar()));
        } else {
            formElement.setUnique(false);
            formElement.setCur("XXX");
            formElement.setNum(FicheItemToString.toString(ficheItem, parameters));
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
