/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractCorpusFieldElement;
import fr.exemole.bdfserver.request.formimp.DefaultPersonneSubfieldsElement;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Personne;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;


/**
 *
 * @author Vincent Calame
 */
public class PersonneSubfieldBuilder {

    private FicheFormParameters ficheFormParameters;
    private CorpusField corpusField;
    private boolean isMandatory;
    private FieldUI fieldUI;

    private PersonneSubfieldBuilder(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.fieldUI = fieldUI;
        this.corpusField = fieldUI.getCorpusField();
        this.isMandatory = FicheFormImpUtils.isMandatory(fieldUI);
    }

    public static AbstractCorpusFieldElement toCorpusFieldElement(FicheFormParameters ficheFormParameters, FieldUI fieldUI) {
        PersonneSubfieldBuilder builder = new PersonneSubfieldBuilder(ficheFormParameters, fieldUI);
        return builder.toCorpusFieldElement();
    }

    private AbstractCorpusFieldElement toCorpusFieldElement() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        DefaultPersonneSubfieldsElement formElement = new DefaultPersonneSubfieldsElement(corpusField);
        LangConfig langConfig = ficheFormParameters.getBdfServer().getConfigurationManager().getLangConfig();
        formElement.setMandatory(isMandatory);
        formElement.setWithOriginal(langConfig.isWithOriginalName());
        boolean withoutNomAvant = langConfig.isWithoutNomAvant();
        if (withoutNomAvant) {
            withoutNomAvant = !(BdfUserUtils.forceNomAvant(ficheFormParameters.getBdfUser()));
        }
        formElement.setWithoutNomAvant(withoutNomAvant);
        FicheItem ficheItem = (FicheItem) fichePointeur.getValue(corpusField.getFieldKey());
        if (ficheItem != null) {
            if (ficheItem instanceof Personne) {
                Personne personne = (Personne) ficheItem;
                if (personne.isRedacteurSphere()) {
                    SubsetKey defaultSphereKey = null;
                    formElement.setNom(FicheItemToString.toString(personne, defaultSphereKey));
                } else {
                    formElement.setFromPersonneBean(personne.getPersonneBean());
                }
            } else {
                formElement.setNom(FicheItemToString.toString(ficheItem, null));
            }
        }
        FicheFormImpUtils.setIntitule(ficheFormParameters, formElement, corpusField);
        return formElement;
    }

}
