/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formimp.AbstractIncludeElement;
import fr.exemole.bdfserver.request.formimp.DefaultEditableAssociationElement;
import fr.exemole.bdfserver.request.formimp.DefaultNotEditableAssociationElement;
import fr.exemole.bdfserver.request.formimp.DefaultRattachementToSlaveElement;
import fr.exemole.bdfserver.request.formimp.DefautRattachementToMasterElement;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.LimitOption;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.PointeurUtils;


/**
 *
 * @author Vincent Calame
 */
public class SpecialIncludeElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private IncludeKey includeKey;
    private boolean isMandatory;
    private IncludeUI includeUI;

    private SpecialIncludeElementBuilder(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.includeKey = includeUI.getIncludeKey();
        this.isMandatory = FicheFormImpUtils.isMandatory(includeUI);
        this.includeUI = includeUI;
    }

    public static AbstractIncludeElement toIncludeElement(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        SpecialIncludeElementBuilder builder = new SpecialIncludeElementBuilder(ficheFormParameters, includeUI);
        return builder.toIncludeElement();
    }

    private AbstractIncludeElement toIncludeElement() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
            Subset masterSubset = fichePointeur.getSubset().getMasterSubset();
            if (masterSubset != null) {
                SubsetItem masterSubsetItem = fichePointeur.getCurrentMasterSubsetItem();
                if (masterSubsetItem == null) {
                    masterSubsetItem = ficheFormParameters.getMasterSubsetItem();
                    if (masterSubsetItem == null) {
                        return null;
                    }
                }
                DefautRattachementToMasterElement formElement = new DefautRattachementToMasterElement(includeKey);
                formElement.setMasterSubset(masterSubset);
                formElement.setMasterSubsetItem(masterSubsetItem);
                initIntitule(formElement);
                return formElement;
            } else {
                FicheCroisementList[] array = PointeurUtils.getRattachementList(fichePointeur);
                DefaultRattachementToSlaveElement formElement = new DefaultRattachementToSlaveElement(includeKey);
                formElement.setSlaveAbregeFicheListArray(array);
                initIntitule(formElement);
                return formElement;
            }
        } else if (includeKey.equals(IncludeKey.ASSOCIATION)) {
            LimitOption limitOption = includeUI.getLimitOption();
            if (limitOption.isNoNewAllowed()) {
                if (ficheFormParameters.isNewFiche()) {
                    return null;
                }
                DefaultNotEditableAssociationElement formElement = new DefaultNotEditableAssociationElement(includeKey);
                formElement.setDocumentCroisementList(fichePointeur.getDocumentCroisementList());
                initIntitule(formElement);
                return formElement;
            } else {
                DefaultEditableAssociationElement formElement = new DefaultEditableAssociationElement(includeKey);
                formElement.setDocumentCroisementList(fichePointeur.getDocumentCroisementList());
                initIntitule(formElement);
                return formElement;
            }
        } else {
            return null;
        }
    }

    private void initIntitule(AbstractIncludeElement includeElement) {
        FicheFormImpUtils.setIntitule(ficheFormParameters, includeElement, includeUI);
    }

}
