/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.formimp.ficheform;

import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.request.formfactory.FicheFormUtils;
import fr.exemole.bdfserver.request.formimp.AbstractIncludeElement;
import fr.exemole.bdfserver.request.formimp.DefaultHiddenIncludeElement;
import fr.exemole.bdfserver.request.formimp.DefaultListThesaurusIncludeElement;
import fr.exemole.bdfserver.request.formimp.DefaultNotEditableIndexationElement;
import fr.exemole.bdfserver.request.formimp.DefaultTextThesaurusIncludeElement;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.InputOption;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.croisement.CroisementUtils;
import net.mapeadores.basedefiches.croisement.MotcleCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.RangeList;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusIncludeElementBuilder {

    private FicheFormParameters ficheFormParameters;
    private Thesaurus thesaurus;
    private IncludeUI includeUI;
    private IncludeKey includeKey;
    private MotcleCroisementList motcleCroisementList = null;
    private boolean isMandatory;
    private boolean hasPoidsFilter;
    private InputOption inputOption;

    private ThesaurusIncludeElementBuilder(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        this.ficheFormParameters = ficheFormParameters;
        this.includeUI = includeUI;
        this.includeKey = includeUI.getIncludeKey();
        this.isMandatory = FicheFormImpUtils.isMandatory(includeUI);
        this.hasPoidsFilter = includeKey.hasPoidsFilter();
        this.inputOption = includeUI.getInputOption();
        SubsetKey thesaurusKey = includeKey.getSubsetKey();
        thesaurus = (Thesaurus) ficheFormParameters.getBdfServer().getBasedefiches().getSubset(thesaurusKey);
    }

    public static IncludeElement toIncludeElement(FicheFormParameters ficheFormParameters, IncludeUI includeUI) {
        ThesaurusIncludeElementBuilder builder = new ThesaurusIncludeElementBuilder(ficheFormParameters, includeUI);
        return builder.toIncludeElement();
    }

    private AbstractIncludeElement toIncludeElement() {
        if (thesaurus == null) {
            return null;
        }
        initMotcleList();
        switch (inputOption.getInputType()) {
            case InputOption.INPUT_CHECK:
                return toListIndexationElement();
            case InputOption.INPUT_RADIO:
            case InputOption.INPUT_SELECT:
                if ((motcleCroisementList == null) || (motcleCroisementList.getSubsetItemCount() == 1)) {
                    return toListIndexationElement();
                } else {
                    return toTextIndexationElement();
                }
            case InputOption.INPUT_TEXT:
                return toTextIndexationElement();
            case InputOption.INPUT_HIDDEN:
                return toHiddenInclude();
            case InputOption.INPUT_NOTEDITABLE:
                return toNotEditableIndexation();
            default:
                throw new SwitchException("wrong inputOption.getInputType() = " + inputOption.getInputType());
        }
    }

    private DefaultHiddenIncludeElement toHiddenInclude() {
        DefaultHiddenIncludeElement formElement = new DefaultHiddenIncludeElement(includeKey);
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, includeUI);
        } else {
            value = FicheFormUtils.getStringValue(thesaurus, motcleCroisementList, hasPoidsFilter, ficheFormParameters);
        }
        formElement.setValue(value);
        return formElement;
    }

    private DefaultNotEditableIndexationElement toNotEditableIndexation() {
        DefaultNotEditableIndexationElement formElement = new DefaultNotEditableIndexationElement(includeKey);
        formElement.setIdalphaSeparator(" – ");
        formElement.setWorkingLangInteger(ficheFormParameters.getWorkingLangInteger());
        Set<Motcle> selectedMotcleSet = null;
        if (ficheFormParameters.isNewFiche()) {
            selectedMotcleSet = getDefaultMotcleSet();
        } else {
            selectedMotcleSet = getMotcleSet();
        }
        formElement.setSelectedMotcleSet(selectedMotcleSet);
        initIntitule(formElement);
        return formElement;
    }

    private DefaultListThesaurusIncludeElement toListIndexationElement() {
        DefaultListThesaurusIncludeElement formElement = new DefaultListThesaurusIncludeElement(includeKey);
        formElement.setMandatory(isMandatory);
        formElement.setListType(inputOption.getInputType());
        formElement.setThesaurus(thesaurus);
        formElement.setIdalphaSeparator(" – ");
        formElement.setNoneAllowed(!isMandatory);
        formElement.setWorkingLangInteger(ficheFormParameters.getWorkingLangInteger());
        formElement.setWithIdalpha(includeUI.prefixWithIdalpha());
        boolean newIndexation = false;
        Set<Motcle> selectedMotcleSet = null;
        if (ficheFormParameters.isNewFiche()) {
            selectedMotcleSet = getDefaultMotcleSet();
            newIndexation = true;
        } else {
            selectedMotcleSet = getMotcleSet();
        }
        formElement.setNewIndexation(newIndexation);
        formElement.setSelectedMotcleSet(selectedMotcleSet);
        initIntitule(formElement);
        return formElement;
    }

    private DefaultTextThesaurusIncludeElement toTextIndexationElement() {
        DefaultTextThesaurusIncludeElement formElement = new DefaultTextThesaurusIncludeElement(includeKey);
        boolean redimAllowed = true;
        formElement.setMandatory(isMandatory);
        formElement.setRedimAllowed(redimAllowed);
        String value = "";
        if (ficheFormParameters.isNewFiche()) {
            value = FicheFormImpUtils.getDefVal(ficheFormParameters, includeUI);
        } else {
            value = FicheFormUtils.getStringValue(thesaurus, motcleCroisementList, hasPoidsFilter, ficheFormParameters);
        }
        formElement.setValue(value);
        int rows = inputOption.getRows();
        if (redimAllowed) {
            rows = FicheFormImpUtils.checkRows(rows, value.length());
        }
        formElement.setRows(rows);
        formElement.setWidthType(inputOption.getWidthType());
        initIntitule(formElement);
        return formElement;
    }

    private void initMotcleList() {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        motcleCroisementList = fichePointeur.getMotcleCroisementList(thesaurus);
        if (hasPoidsFilter) {
            motcleCroisementList = CroisementUtils.poidsFilter(motcleCroisementList, includeKey.getPoidsFilter());
        } else {
            IncludeSummary.SubsetInfo info = ficheFormParameters.getIncludeSummary().getSubsetInfo(thesaurus.getSubsetKey());
            RangeList rangeList = null;
            if (info != null) {
                rangeList = info.getPoidsFilterRangeList();
                if (rangeList != null) {
                    motcleCroisementList = CroisementUtils.excludePoids(motcleCroisementList, rangeList);
                }
            }
        }
        if ((motcleCroisementList != null) && (motcleCroisementList.getSubsetItemCount() == 0)) {
            motcleCroisementList = null;
        }
    }

    private Set<Motcle> getDefaultMotcleSet() {
        String defaultValue = FicheFormImpUtils.getDefVal(ficheFormParameters, includeUI);
        if (defaultValue.length() == 0) {
            return Collections.emptySet();
        }
        String[] tokens = StringUtils.getTokens(defaultValue, ';', StringUtils.EMPTY_EXCLUDE);
        Set<Motcle> defaultSet = new HashSet<Motcle>();
        boolean withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        for (int i = 0; i < tokens.length; i++) {
            String token = tokens[i];
            if (withIdalpha) {
                Motcle motcle = thesaurus.getMotcleByIdalpha(token);
                if (motcle != null) {
                    defaultSet.add(motcle);
                }
            } else {
                try {
                    int idths = Integer.parseInt(token);
                    Motcle motcle = thesaurus.getMotcleByIdths(idths);
                    if (motcle != null) {
                        defaultSet.add(motcle);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }
        return defaultSet;
    }

    private Set<Motcle> getMotcleSet() {
        if (motcleCroisementList == null) {
            return Collections.emptySet();
        }
        int size = motcleCroisementList.getSubsetItemCount();
        Set<Motcle> set = new HashSet<Motcle>();
        for (int i = 0; i < size; i++) {
            set.add(motcleCroisementList.getMotcle(i));
        }
        return set;
    }

    private void initIntitule(AbstractIncludeElement includeElement) {
        FicheFormImpUtils.setIntitule(ficheFormParameters, includeElement, includeUI);
    }

}
