/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.IllustrationCroisementList;
import net.mapeadores.basedefiches.croisement.MutableIllustrationCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;


/**
 *
 * @author Vincent Calame
 */
public class AlbumIncludeParser {

    private AbregeFiche abregeFiche;
    private List<TmpFileToken> tmpFileTokenList = new ArrayList<TmpFileToken>();
    private List<Illustration> removeList = new ArrayList<Illustration>();
    private List<UpdateToken> updateTokenList = new ArrayList<UpdateToken>();
    private Map<SubsetKey, MutableIllustrationCroisementList> createIllustrationMap = new HashMap<SubsetKey, MutableIllustrationCroisementList>();
    private File tmpDirectory;

    public AlbumIncludeParser(AbregeFiche abregeFiche, File tmpDirectory) {
        this.abregeFiche = abregeFiche;
        this.tmpDirectory = tmpDirectory;
    }

    public void parse(String paramName, String[] tokens) {
        int idx = paramName.indexOf(':');
        if (idx == -1) {
            throw new IllegalArgumentException("Missing ':', not an album Parameter : " + paramName);
        }
        String action = paramName.substring(idx + 1);
        String includeKeyString = paramName.substring(0, idx);
        IncludeKey includeKey = null;
        try {
            includeKey = IncludeKey.parse(includeKeyString);
        } catch (ParseException pe) {
            throw new IllegalArgumentException("not an album Parameter : " + paramName);
        }
        SubsetKey albumSubsetKey = includeKey.getSubsetKey();
        if (!albumSubsetKey.isAlbumSubset()) {
            throw new IllegalArgumentException("not an album Parameter : " + paramName);
        }
        Album album = (Album) abregeFiche.getCorpus().getBasedefiches().getSubset(albumSubsetKey);
        if (album == null) {
            return;
        }
        if (action.equals("create")) {
            int poidsFilter = includeKey.getPoidsFilter();
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                if (token.length() > 0) {
                    TmpFileToken tmpFileToken = new TmpFileToken(token, albumSubsetKey, poidsFilter);
                    tmpFileTokenList.add(tmpFileToken);
                }
            }
        } else if (action.equals("remove")) {
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                if (token.length() > 0) {
                    try {
                        int idalbum = Integer.parseInt(token);
                        Illustration illustration = album.getIllustrationByIdalbum(idalbum);
                        if (illustration != null) {
                            removeList.add(illustration);
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
            }
        } else if (action.equals("update")) {
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                if (token.length() > 0) {
                    int sepIndex = token.indexOf('=');
                    if (sepIndex == -1) {
                        continue;
                    }
                    try {
                        int idalbum = Integer.parseInt(token.substring(0, sepIndex));
                        Illustration illustration = album.getIllustrationByIdalbum(idalbum);
                        if (illustration != null) {
                            String tmpFileName = token.substring(sepIndex + 1);
                            updateTokenList.add(new UpdateToken(illustration, tmpFileName));
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
            }
        } else {
            throw new IllegalArgumentException("Unknown ation ':" + action + "', not an album Parameter : " + paramName);
        }
    }

    public void save(BasedefichesEditor basedefichesEditor) {
        CroisementEditor croisementEditor = basedefichesEditor.getCroisementEditor();
        for (Illustration illustration : removeList) {
            croisementEditor.removeCroisement(abregeFiche, illustration);
            basedefichesEditor.getAlbumEditor(illustration.getAlbum().getSubsetKey()).removeIllustration(illustration);
        }
        for (UpdateToken updateToken : updateTokenList) {
            updateToken.updateIllustration(basedefichesEditor);
        }
        if (tmpFileTokenList.size() > 0) {
            for (TmpFileToken tmpFileToken : tmpFileTokenList) {
                tmpFileToken.createIllustration(basedefichesEditor);
            }
            for (IllustrationCroisementList createCroisementList : createIllustrationMap.values()) {
                croisementEditor.updateCroisement(abregeFiche, createCroisementList, CroisementEditor.APPEND_AND_POIDS_NOCHANGE);
            }
        }

    }

    private MutableIllustrationCroisementList getCreateCroisementList(SubsetKey albumSubsetKey) {
        MutableIllustrationCroisementList illustrationCroisementList = createIllustrationMap.get(albumSubsetKey);
        if (illustrationCroisementList == null) {
            illustrationCroisementList = new MutableIllustrationCroisementList(albumSubsetKey, 4);
            createIllustrationMap.put(albumSubsetKey, illustrationCroisementList);
        }
        return illustrationCroisementList;
    }


    private class UpdateToken {

        private Illustration illustration;
        private String tmpFileName;

        private UpdateToken(Illustration illustration, String tmpFileName) {
            this.illustration = illustration;
            this.tmpFileName = tmpFileName;
        }

        private void updateIllustration(BasedefichesEditor basedefichesEditor) {
            AlbumEditor albumEditor = basedefichesEditor.getAlbumEditor(illustration.getAlbum().getSubsetKey());
            int idx = tmpFileName.lastIndexOf(".");
            String extension = tmpFileName.substring(idx + 1);
            File f = new File(tmpDirectory, tmpFileName);
            if (!f.exists()) {
                return;
            }
            short formatType = AlbumUtils.formatTypeToShort(extension);
            try {
                InputStream inputStream = new FileInputStream(f);
                albumEditor.updateIllustration(illustration, inputStream, formatType);
                inputStream.close();
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
        }

    }


    private class TmpFileToken {

        private int poids = 1;
        private String tmpFileName;
        private SubsetKey albumSubsetKey;
        private Illustration illustration;

        private TmpFileToken(String tmpFileName, SubsetKey albumSubsetKey, int poids) {
            if (poids > 1) {
                this.poids = poids;
            }
            this.albumSubsetKey = albumSubsetKey;
            this.tmpFileName = tmpFileName;
        }

        private void createIllustration(BasedefichesEditor basedefichesEditor) {
            AlbumEditor albumEditor = basedefichesEditor.getAlbumEditor(albumSubsetKey);
            if (albumEditor == null) {
                return;
            }
            int idx = tmpFileName.lastIndexOf(".");
            String extension = tmpFileName.substring(idx + 1);
            File f = new File(tmpDirectory, tmpFileName);
            if (!f.exists()) {
                return;
            }
            short formatType = AlbumUtils.formatTypeToShort(extension);
            try {
                illustration = albumEditor.createIllustration(-1, formatType);
            } catch (ExistingIdException eie) {
            }
            try {
                InputStream inputStream = new FileInputStream(f);
                albumEditor.updateIllustration(illustration, inputStream, formatType);
                inputStream.close();
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
            MutableIllustrationCroisementList mutableIllustrationCroisementList = getCreateCroisementList(albumSubsetKey);
            mutableIllustrationCroisementList.add(illustration, poids);
        }

    }

}
