/* BdfServer - Copyright (c) 2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import net.mapeadores.util.request.FileValue;
import net.mapeadores.basedefiches.*;
import net.mapeadores.basedefiches.association.*;
import java.util.*;
import net.mapeadores.basedefiches.corpus.*;
import net.mapeadores.basedefiches.croisement.*;
import java.io.*;
import net.mapeadores.util.exceptions.*;


/**
 *
 * @author Vincent Calame
 */
public class AssociationParser {

    private AbregeFiche abregeFiche;
    private Map<String, Document> currentMap = new HashMap<String, Document>();
    private Map<String, Action> actionMap = new LinkedHashMap<String, Action>();
    private Addenda addenda;
    private CroisementEditor croisementEditor;
    private AddendaEditor documentSubsetEditor;

    public AssociationParser(AbregeFiche abregeFiche) {
        this.abregeFiche = abregeFiche;
        Corpus corpus = abregeFiche.getCorpus();
        addenda = (Addenda) corpus.getBasedefiches().getSubset(SubsetKey.ADDENDA);
        Croisement croisement = corpus.getBasedefiches().getCroisement(corpus.getSubsetKey(), SubsetKey.ADDENDA);
        CroisementItem croisementItem = croisement.getCroisementItem(corpus.getSubsetKey(), abregeFiche.getTicket());
        int currentCount = croisementItem.getCount();
        for (int i = 0; i < currentCount; i++) {
            int ticket = croisementItem.getTicket(i);
            Document document = (Document) addenda.getSubsetItemByTicket(ticket);
            if (document != null) {
                String baseName = document.getBasename();
                currentMap.put(baseName, document);
            }
        }
    }

    public void add(FileValue[] fileValues) {
        int count = fileValues.length;
        for (int i = 0; i < count; i++) {
            FileValue fileValue = fileValues[i];
            String fileName = fileValue.getName();
            VersionName.ParseResult parseResult = VersionName.parse(fileName, true);
            if (parseResult == null) {
                continue;
            }
            String extension = "";
            if (!parseResult.hasExtension()) {
                extension = "null";
            } else {
                extension = parseResult.getExtension();
            }
            String baseName = AssociationUtils.checkBasename(parseResult.getBasename());
            Action action = actionMap.get(baseName);
            if (action == null) {
                Document currentDocument = currentMap.get(baseName);
                if (currentDocument != null) {
                    action = new Action(currentDocument);
                } else {
                    String newBaseName = AssociationUtils.checkBasename(baseName, addenda);
                    action = new Action(newBaseName);
                }
                actionMap.put(baseName, action);
            }
            action.add(extension, fileValue);
        }

    }

    public void save(BasedefichesEditor basedefichesEditor) {
        croisementEditor = basedefichesEditor.getCroisementEditor();
        documentSubsetEditor = basedefichesEditor.getAddendaEditor();
        for (Action action : actionMap.values()) {
            action.save();
        }

    }


    private class Action {

        Map<String, FileValue> versionMap = new LinkedHashMap<String, FileValue>();
        Document currentDocument;
        String newBaseName;
        String firstExtension;
        FileValue firstFileValue;

        Action(Document currentDocument) {
            this.currentDocument = currentDocument;
        }

        Action(String newBaseName) {
            this.newBaseName = newBaseName;
        }

        private void add(String extension, FileValue fileValue) {
            if ((currentDocument == null) && (firstExtension == null)) {
                firstExtension = extension;
                firstFileValue = fileValue;
            } else {
                versionMap.put(extension, fileValue);
            }
        }

        private void save() {
            if (currentDocument == null) {
                VersionName versionName = VersionName.getInstance(newBaseName, firstExtension);
                try {
                    currentDocument = documentSubsetEditor.createDocument(versionName);
                    Version version = currentDocument.getVersionByExtension(firstExtension);
                    try {
                        documentSubsetEditor.updateVersion(version, firstFileValue.getInputStream());
                        firstFileValue.free();
                    } catch (IOException ioe) {
                        throw new NestedIOException(ioe);
                    }
                    DefaultDocumentCroisementList documentList = new DefaultDocumentCroisementList(currentDocument);
                    croisementEditor.updateCroisement(abregeFiche, documentList, CroisementEditor.APPEND_AND_POIDS_REPLACE);

                } catch (ExistingBasenameException ebe) {
                    throw new ImplementationException(ebe);
                }
            }
            updateDocument();
        }

        private void updateDocument() {
            for (Map.Entry<String, FileValue> entry : versionMap.entrySet()) {
                String extension = entry.getKey();
                FileValue fileValue = entry.getValue();
                Version version = currentDocument.getVersionByExtension(extension);
                if (version == null) {
                    try {
                        version = documentSubsetEditor.createVersion(currentDocument, extension);
                    } catch (ExistingVersionException eve) {
                        throw new ImplementationException(eve);
                    } catch (IllegalArgumentException iae) {
                    }
                }
                if (version != null) {
                    try {
                        documentSubsetEditor.updateVersion(version, fileValue.getInputStream());
                    } catch (IOException ioe) {
                        throw new NestedIOException(ioe);
                    }
                }
                fileValue.free();
            }
        }

    }

}
