/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.DefaultFicheCroisementList;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;


/**
 *
 * @author Vincent Calame
 */
public class LiageParser {

    private Map<SubsetKey, DefaultFicheCroisementList> listMap = new HashMap<SubsetKey, DefaultFicheCroisementList>();
    private AbregeFiche abregeFiche;
    private SubsetKey currentCorpusKey;
    private Basedefiches basedefiches;
    private Subset masterSubset;
    private Subset[] slaveSubsetArray;

    public LiageParser(AbregeFiche abregeFiche) {
        this.abregeFiche = abregeFiche;
        currentCorpusKey = abregeFiche.getCorpus().getSubsetKey();
        basedefiches = abregeFiche.getCorpus().getBasedefiches();
        this.masterSubset = abregeFiche.getCorpus().getMasterSubset();
        this.slaveSubsetArray = abregeFiche.getCorpus().getSlaveSubsetArray();
    }

    public void parse(String paramName, String[] tokens) {
        if (paramName.equals(IncludeKey.LIAGE.toString())) {
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                try {
                    FicheToken ficheToken = new FicheToken(token);
                    addFiche(ficheToken);
                } catch (ParseException pe) {
                }
            }
        } else {
            IncludeKey includeKey = null;
            try {
                includeKey = IncludeKey.parse(paramName);
            } catch (ParseException pe) {
                throw new IllegalArgumentException("not a corpus Parameter : " + paramName);
            }
            SubsetKey corpusKey = includeKey.getSubsetKey();
            if (!corpusKey.isCorpusSubset()) {
                throw new IllegalArgumentException("not a corpus Parameter : " + paramName);
            }
            int poidsFilter = includeKey.getPoidsFilter();
            for (int i = 0; i < tokens.length; i++) {
                String token = tokens[i].trim();
                try {
                    FicheToken ficheToken = new FicheToken(token, corpusKey, poidsFilter);
                    addFiche(ficheToken);
                } catch (ParseException pe) {
                }
            }
        }
    }

    private void addFiche(FicheToken ficheToken) {
        AbregeFiche abregeFiche = getAbregeFiche(ficheToken);
        if (abregeFiche == null) {
            return;
        }
        DefaultFicheCroisementList abregeFicheList = getAbregeFicheList(ficheToken.getCorpusKey());
        abregeFicheList.add(abregeFiche, ficheToken.getPoids());
    }

    private DefaultFicheCroisementList getAbregeFicheList(SubsetKey crpsKey) {
        DefaultFicheCroisementList abregeFicheList = listMap.get(crpsKey);
        if (abregeFicheList == null) {
            abregeFicheList = new DefaultFicheCroisementList(crpsKey, 4);
            listMap.put(crpsKey, abregeFicheList);
        }
        return abregeFicheList;
    }

    private AbregeFiche getAbregeFiche(FicheToken ficheToken) {
        Corpus corpus = (Corpus) basedefiches.getSubset(ficheToken.getCorpusKey());
        if (corpus == null) {
            return null;
        }
        if (isNotEditableCroisement(corpus)) {
            return null;
        }
        return corpus.getAbregeFicheByIdcorpus(ficheToken.getIdcorpus());
    }

    public void save(CroisementEditor croisementEditor) {
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            if (!isNotEditableCroisement(corpus)) {
                FicheCroisementList abregeFicheList = getAbregeFicheList(corpus.getSubsetKey());
                croisementEditor.updateCroisement(abregeFiche, abregeFicheList, CroisementEditor.CLEAR_EXISTING);
            }
        }
    }

    private boolean isNotEditableCroisement(Subset subset) {
        if (masterSubset != null) {
            return (masterSubset.equals(subset));
        }
        if (slaveSubsetArray != null) {
            int length = slaveSubsetArray.length;
            for (int i = 0; i < length; i++) {
                if (slaveSubsetArray[i].equals(subset)) {
                    return true;
                }
            }
            return false;
        }
        return false;
    }


    private class FicheToken {

        private int poids = 1;
        private int idcorpus;
        private SubsetKey corpusKey;

        FicheToken(String s2) throws ParseException {
            PoidsToken poidsToken = PoidsToken.parse(s2);
            poids = poidsToken.getPoids();
            String text = poidsToken.getText();
            int idx = text.indexOf('/');
            if (idx == 0) {
                if (text.length() == 1) {
                    throw new ParseException(s2, 0);
                }
                idx = -1;
                text = text.substring(1);
            }
            if (idx == -1) {
                corpusKey = currentCorpusKey;
                try {
                    idcorpus = Integer.parseInt(text);
                } catch (NumberFormatException nfe) {
                    throw new ParseException(s2, 0);
                }
            } else {
                if (idx == text.length() - 1) {
                    throw new ParseException(s2, 0);
                }
                String corpusSubid = text.substring(0, idx);
                corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, corpusSubid);
                try {
                    idcorpus = Integer.parseInt(text.substring(idx + 1));

                } catch (NumberFormatException nfe) {
                    throw new ParseException(s2, idx);
                }
            }
            if (idcorpus < 0) {
                throw new ParseException(s2, 0);
            }
        }

        FicheToken(String s, SubsetKey corpusKey, int poidsFilter) throws ParseException {
            PoidsToken poidsToken = PoidsToken.parse(s);
            this.corpusKey = corpusKey;
            if (poidsFilter < 1) {
                poids = poidsToken.getPoids();
            } else {
                poids = poidsFilter;
            }
            try {
                idcorpus = Integer.parseInt(poidsToken.getText());
            } catch (NumberFormatException nfe) {
                throw new ParseException(s, 0);
            }
            if (idcorpus < 0) {
                throw new ParseException(s, 0);
            }
        }

        public int getPoids() {
            return poids;
        }

        public int getIdcorpus() {
            return idcorpus;
        }

        public SubsetKey getCorpusKey() {
            return corpusKey;
        }

    }

}
