/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.text.CleanedString;


/**
 *
 * @author Vincent Calame
 */
public class PoidsMotcleToken {

    private boolean idthsBundle = false;
    private int poids = 1;
    private int idths;
    private CleanedString text;

    private PoidsMotcleToken(int idths, int poids) {
        this.idths = idths;
        this.poids = poids;
        idthsBundle = true;
    }

    private PoidsMotcleToken(CleanedString text, int poids) {
        this.text = CleanedString.newInstance(text);
        this.poids = poids;
        idthsBundle = false;

    }

    public static PoidsMotcleToken parse(String s, boolean withIdalpha) {
        PoidsToken poidsToken = PoidsToken.parse(s);
        String text = poidsToken.getText();
        int poids = poidsToken.getPoids();
        if (!withIdalpha) {
            try {
                int idths = Integer.parseInt(text);
                return new PoidsMotcleToken(idths, poids);
            } catch (NumberFormatException nfe) {
            }

        }
        CleanedString cleanedString = CleanedString.newInstance(text);
        if (cleanedString == null) {
            return null;
        }
        return new PoidsMotcleToken(cleanedString, poids);
    }

    public boolean isIdthsBundle() {
        return idthsBundle;
    }

    public int getPoids() {
        return poids;
    }

    public int getIdths() {
        return idths;
    }

    public void setPoids(int poids) {
        this.poids = poids;
    }

    /**
     * Null si et seulement si isIdthsBundle() = true
     */
    public CleanedString getText() {
        return text;
    }

    public static Motcle getMotcle(PoidsMotcleToken motcleToken, Thesaurus thesaurus, int langInteger) {
        if (motcleToken.isIdthsBundle()) {
            return thesaurus.getMotcleByIdths(motcleToken.getIdths());
        }
        CleanedString cleanedString = motcleToken.getText();
        if (cleanedString == null) {
            return null;
        }
        String text = cleanedString.toString();
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            return thesaurus.getMotcleByIdalpha(text);
        } else {
            return thesaurus.getMotcleByLibelle(text, langInteger);
        }
    }

}
