/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers;

import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.beans.PersonneBean;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurItemParser {

    private Basedefiches basedefiches;
    private TextParser textParser;

    public RedacteurItemParser(Basedefiches basedefiches, TextParser textParser) {
        this.basedefiches = basedefiches;
        this.textParser = textParser;
    }

    public void parseRedacteurItemList(String[] tokens, MutableFicheItemList ficheItemList, SubsetKey defaultSphereKey) {
        for (int i = 0; i < tokens.length; i++) {
            String token = tokens[i];
            RedacteurItem redacteurItem = parseRedacteurItem(token, defaultSphereKey);
            if (redacteurItem != null) {
                ficheItemList.addFicheItem(redacteurItem);
            }
        }
    }

    public RedacteurItem parseRedacteurItem(String token, SubsetKey defaultSphereKey) {
        RedacteurKey redacteurKey = RedacteurKey.parseFromUserString(token, defaultSphereKey);
        if (redacteurKey != null) {
            SubsetKey sphereKey = redacteurKey.getSphereKey();
            Sphere sphere = (Sphere) basedefiches.getSubset(sphereKey);
            if ((sphere != null) && (sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString()) != null)) {
                return new Personne(redacteurKey.toString());
            } else {
                return new Item(textParser.parseString(token));
            }
        }
        Personne personne = parsePersonne(token, textParser);
        if (personne != null) {
            return personne;
        }
        return new Item(textParser.parseString(token));
    }

    public RedacteurItem parseRedacteurItem(String nom, String prenom, String original, boolean nomAvant) {
        if ((prenom.length() == 0) && (original.length() == 0)) {
            RedacteurKey redacteurKey = RedacteurKey.parseFromUserString(nom, null);
            if (redacteurKey != null) {
                SubsetKey sphereKey = redacteurKey.getSphereKey();
                Sphere sphere = (Sphere) basedefiches.getSubset(sphereKey);
                if ((sphere != null) && (sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString()) != null)) {
                    return new Personne(redacteurKey.toString());
                }
            }
            if (nom.length() == 0) {
                return null;
            }
            if (!nomAvant) {
                return new Item(textParser.parseString(nom));
            }
        }
        PersonneBean personneBean = new PersonneBean();
        personneBean.setNom(textParser.parseString(nom));
        personneBean.setPrenom(textParser.parseString(prenom));
        personneBean.setOriginal(textParser.parseString(original));
        personneBean.setNomAvant(nomAvant);
        return new Personne(personneBean, null);
    }

    public static Personne parsePersonne(String s, TextParser textParser) {
        int separateurinterne = '*';
        int posvirg = s.indexOf(separateurinterne);
        if (posvirg == -1) {
            separateurinterne = '|';
            posvirg = s.indexOf(separateurinterne);
        }
        if (posvirg > -1) {
            PersonneBean personneBean = new PersonneBean();
            personneBean.setNom(textParser.parseString(s.substring(0, posvirg)));
            String prenom = "";
            String organisme = null;
            if (posvirg < (s.length() - 1)) {
                String suite = s.substring(posvirg + 1).trim();
                int pos2 = suite.indexOf(separateurinterne);
                if (pos2 > -1) {
                    prenom = suite.substring(0, pos2);
                    if (pos2 < (suite.length() - 1)) {
                        organisme = suite.substring(pos2 + 1);
                    }
                } else {
                    prenom = suite;
                }
            }
            boolean nomavant = false;
            if ((prenom.length() > 0) && (prenom.charAt(0) == '%')) {
                prenom = prenom.substring(1);
                personneBean.setNomAvant(true);
            }
            personneBean.setPrenom(textParser.parseString(prenom));
            return new Personne(personneBean, textParser.parseString(organisme));
        } else {
            return null;
        }
    }

    /*public static RedacteurKey[] parseRedacteurKeyArray(String[] tokens, Basedefiches basedefiches, RedacteurKey userRedacteurKey, TextParser textParser) {
        RedacteurItemParser redacteurItemParser = new RedacteurItemParser(basedefiches, userRedacteurKey, textParser);
        int length = tokens.length;
        if (length == 0) {
            return null;
        }
        int p = 0;
        RedacteurKey[] array = new RedacteurKey[length];
        for (int i = 0; i < length; i++) {
            RedacteurKey redacteurKey = RedacteurKey.parseFromUserString(tokens[i], userRedacteurKey.getSphereKey());
            if (redacteurKey != null) {
                SubsetKey sphereKey = redacteurKey.getSphereKey();
                Sphere sphere = (Sphere) basedefiches.getSubset(sphereKey);
                if ((sphere != null) && (sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString()) != null)) {
                    array[p] = redacteurKey;
                    p++;
                }
            }
        }
        if (p == 0) {
            return null;
        }
        if (p < length) {
            RedacteurKey[] nv = new RedacteurKey[p];
            System.arraycopy(array, 0, nv, 0, p);
            return nv;
        }
        return array;
    }*/

}
