/* BdfServer
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.util.text.*;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class BlockParser {

    private int endIndex;
    private FicheBlock ficheBlock;
    private String[] lineArray;
    protected TextParser textParser;

    public BlockParser(TextParser textParser) {
        this.textParser = textParser;
    }

    public int getEndIndex() {
        return endIndex;
    }

    protected void setLineArray(String[] lineArray) {
        this.lineArray = lineArray;
    }

    protected int getLineArrayLength() {
        return lineArray.length;
    }

    public String getCleanedLine(int lineIndex) {
        return StringUtils.cleanString(lineArray[lineIndex]);
    }

    public LineInfo getLineInfo(int lineIndex) {
        return LineInfo.parse(StringUtils.cleanString(lineArray[lineIndex]));
    }

    public String getLine(int lineIndex) {
        return lineArray[lineIndex];
    }

    public TextParser getTextParser() {
        return textParser;
    }

    public void setTextParser(TextParser textParser) {
        this.textParser = textParser;
    }

    protected void setParseResult(int endIndex, FicheBlock ficheBlock) {
        this.endIndex = endIndex;
        this.ficheBlock = ficheBlock;
    }

    public abstract void parse(String[] lineArray, int startIndex);

    public abstract boolean isStartLine(String line);

    public FicheBlock getFicheBlock() {
        return ficheBlock;
    }

    public static BlockParser getTableParser(TextParser textParser) {
        return new TableParser(textParser);
    }

    public static BlockParser getCodeParser(TextParser textParser) {
        return new CodeParser(textParser);
    }

    public static BlockParser getUlParser(TextParser textParser) {
        return new UlParser(textParser);
    }

    public static BlockParser getInsertParser(TextParser textParser) {
        return new InsertParser(textParser);
    }

    public static BlockParser getDivParser(TextParser textParser, TypoOptions typoOptions) {
        return new DivParser(textParser, typoOptions);
    }

}
