/* BdfServer
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.*;
import fr.exemole.bdfserver.request.parsers.*;


/**
 *
 * @author Vincent Calame
 */
public class DivParser extends ZoneBlockParser {

    public final static short DIV = 1;
    public final static short CDATA_DIV = 2;
    private TypoOptions typoOptions;
    private short divType = DIV;

    DivParser(TextParser textParser, TypoOptions typoOptions) {
        super(textParser);
        this.typoOptions = typoOptions;
    }

    public boolean isStartLine(String line) {
        if (!line.startsWith("!!!")) {
            return false;
        }
        if (line.startsWith("!!!c")) {
            divType = CDATA_DIV;
        } else {
            divType = DIV;
        }
        return true;
    }

    public void parse(String[] lineArray, int startIndex) {
        setLineArray(lineArray);
        LineParser lineParser = null;
        switch (divType) {
            case DIV:
                lineParser = new DivLineParser(typoOptions);
                break;
            case CDATA_DIV:
                lineParser = new CdataDivLineParser();
                break;
        }
        int beginIndex = parseZoneBlockElements(lineParser.getZoneBlock(), startIndex + 1);
        int stop = getLineArrayLength() - 1;
        for (int i = beginIndex; i <= stop; i++) {
            String line = getLine(i);
            if (line.trim().startsWith("!!!")) {
                stop = i;
                break;
            } else {
                i = lineParser.parseLine(lineArray, i);
            }
        }
        lineParser.flush();
        setParseResult(stop, lineParser.getZoneBlock());
    }


    private interface LineParser {

        public ZoneBlock getZoneBlock();

        public void flush();

        public int parseLine(String[] lineArray, int i);

    }


    private static class DivLineParser implements LineParser {

        private Div div;
        private MutableFicheBlockList ficheBlockList;
        private FicheBlockParser ficheBlockParser;

        DivLineParser(TypoOptions typoOptions) {
            div = new Div();
            ficheBlockList = div.getFicheBlockList();
            ficheBlockParser = new FicheBlockParser(typoOptions);
        }

        public ZoneBlock getZoneBlock() {
            return div;
        }

        public void flush() {
        }

        public int parseLine(String[] lineArray, int i) {
            return ficheBlockParser.parseLine(lineArray, i, ficheBlockList);
        }

    }


    private static class CdataDivLineParser implements LineParser {

        private CdataDiv cdataDiv;
        private StringBuffer cdataBuffer;

        CdataDivLineParser() {
            cdataDiv = new CdataDiv();
            cdataBuffer = new StringBuffer();
        }

        public ZoneBlock getZoneBlock() {
            return cdataDiv;
        }

        public void flush() {
            cdataDiv.setCdata(cdataBuffer.toString());
        }

        public int parseLine(String[] lineArray, int i) {
            if (cdataBuffer.length() > 0) {
                cdataBuffer.append('\n');
            }
            cdataBuffer.append(lineArray[i]);
            return i;
        }

    }

}
