/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.request.*;
import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
class InsertParser extends ZoneBlockParser {

    InsertParser(TextParser textParser) {
        super(textParser);
    }

    public boolean isStartLine(String line) {
        return line.startsWith(":::");
    }

    public void parse(String[] lineArray, int startIndex) {
        setLineArray(lineArray);
        LineInfo lineInfo = getLineInfo(startIndex);
        String line1 = lineInfo.getLine();
        short type = Insert.IMAGE_TYPE;
        short position = Insert.POSITION_UNDETERMINED;
        if (line1.length() > 3) {
            type = Insert.typeToShort(line1.charAt(3));
            if (type == Insert.LETTRINE_TYPE) {
                if (line1.length() > 4) {
                    position = Insert.positionToShort(line1.charAt(4));
                }
            }
        }
        Insert insert = new Insert(type);
        insert.setPosition(position);
        addIdentifiersInfo(insert, lineInfo);
        int stop = getLineArrayLength() - 1;
        for (int i = startIndex + 1; i <= stop; i++) {
            String line = getCleanedLine(i);
            if (line.startsWith(":::")) {
                stop = i;
                break;
            } else {
                if (line.length() > 0) {
                    parseLine(insert, line);
                }
            }
        }
        setParseResult(stop, insert);
    }

    private void parseLine(Insert insert, String line) {
        LineInfo lineInfo = LineInfo.parse(line);
        line = lineInfo.getLine();
        int idx = line.indexOf('=');
        if (idx == -1) {
            return;
        }
        String txt = line.substring(idx + 1).trim();
        lineInfo.setLine(txt);
        char carac = line.charAt(0);
        switch (carac) {
            case 'i':
                boolean illustrationDone = parseIllustration(insert, txt);
                if (!illustrationDone) {
                    if (insert.getSrc().length() == 0) {
                        insert.setSrc(txt);
                    }
                }
                break;
            case 's':
                insert.setSrc(txt);
                break;
            case 'a':
                insert.setRef(txt);
                break;
            case 'c':
                insert.setCredit(txt);
                break;
            case 'w':
                try {
                    int width = Integer.parseInt(txt);
                    insert.setWidth(width);
                } catch (NumberFormatException nfe) {
                }
                break;
            case 'h':
                try {
                    int height = Integer.parseInt(txt);
                    insert.setHeight(height);
                } catch (NumberFormatException nfe) {
                }
                break;
            case 'l':
                parseParagraph(insert.getLegende(), lineInfo);
                break;
            case 'n':
                parseParagraph(insert.getNumero(), lineInfo);
                break;
        }
    }

    private boolean parseIllustration(Insert insert, String txt) {
        int idx = txt.indexOf('/');
        if (idx < 1) {
            return false;
        }
        try {
            SubsetKey albumSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, txt.substring(0, idx));
            int idalbum = Integer.parseInt(txt.substring(idx + 1));
            insert.setIllustration(albumSubsetKey, idalbum, "");
            return true;
        } catch (ParseException pe) {
            return false;
        } catch (NumberFormatException nfe) {
            return false;
        }
    }

}
