/* BdfServer - Copyright (c) 2006-2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import java.util.*;
import fr.exemole.bdfserver.request.parsers.ParagraphParser;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
class UlParser extends BlockParser {

    private DepthParser depthParser = new DepthParser();
    ParagraphParser paragraphBlockParser;
    UlBuffer firstUlBuffer;
    UlBuffer lastUlBuffer;

    UlParser(TextParser textParser) {
        super(textParser);
        paragraphBlockParser = new ParagraphParser(textParser);
    }

    public boolean isStartLine(String line) {
        if ((line.startsWith("-")) || (line.startsWith("_"))) {
            return true;
        } else {
            return false;
        }
    }

    public void parse(String[] lineArray, int startIndex) {
        setLineArray(lineArray);
        firstUlBuffer = null;
        lastUlBuffer = null;
        int stop = getLineArrayLength() - 1;
        for (int i = startIndex; i <= stop; i++) {
            String line = getCleanedLine(i);
            boolean adddone = add(line);
            if (!adddone) {
                stop = i - 1;
                break;
            }
        }
        setParseResult(stop, firstUlBuffer.ul);
    }

    private boolean add(String line) {
        LineInfo lineInfo = LineInfo.parse(line);
        line = lineInfo.getLine();
        if (line.length() == 0) {
            return true;
        }
        if (line.startsWith("-")) {
            addTiret(lineInfo);
            return true;
        }
        if (line.startsWith("_")) {
            addSouligne(lineInfo);
            return true;
        }
        return false;
    }

    void addTiret(LineInfo lineInfo) {
        depthParser.parseLine(lineInfo.getLine().substring(1).trim());
        int depth = depthParser.getDepth();
        UlBuffer ulBuffer = getUlBuffer(depth, depthParser.getLineInfoBundleList(), lineInfo);

        LineInfoBundle last = depthParser.getLastLineInfoBundle();
        Li li = createLi(last.getLiLineInfo());
        P p = new P(P.TYPE_NORMAL);
        paragraphBlockParser.parseParagraphBlock(p, last.getFicheBlockLineInfo());
        li.addP(p);
        ulBuffer.addLi(li);
        lastUlBuffer = ulBuffer;
    }

    void addSouligne(LineInfo lineInfo) {
        String token = lineInfo.getLine();
        int depth = 1;
        int taille = token.length();
        while ((depth < taille) && (token.charAt(depth) == '_')) {
            depth++;
        }
        lineInfo.setLine(token.substring(depth).trim());
        UlBuffer strate = getUlBuffer(depth, null, null);
        P p = new P(P.TYPE_NORMAL);
        paragraphBlockParser.parseParagraphBlock(p, lineInfo);
        strate.addP(p);
        lastUlBuffer = strate;
    }

    UlBuffer getUlBuffer(int depth, List lineInfoBundleList, LineInfo firstLineInfo) {
        if (firstUlBuffer == null) {
            firstUlBuffer = new UlBuffer(1, firstLineInfo);
            lastUlBuffer = firstUlBuffer;
        }
        if (depth == 1) {
            return firstUlBuffer;
        }
        if (depth == lastUlBuffer.depth) {
            return lastUlBuffer;
        } else if (depth < lastUlBuffer.depth) {
            UlBuffer result = firstUlBuffer;
            for (int i = 1; i < depth; i++) {
                result = result.subUlBuffer;
            }
            return result;
        } else {
            UlBuffer previousUlBuffer = lastUlBuffer;
            for (int i = lastUlBuffer.depth + 1; i <= depth; i++) {
                LineInfo ulLineInfo = null;
                LineInfo liLineInfo = null;
                if (lineInfoBundleList != null) {
                    LineInfoBundle bundle = (LineInfoBundle) lineInfoBundleList.get(i - 2);
                    ulLineInfo = bundle.getFicheBlockLineInfo();
                    if (!ulLineInfo.hasIdentifiers()) {
                        ulLineInfo = bundle.getLiLineInfo();
                    }
                    bundle = (LineInfoBundle) lineInfoBundleList.get(i - i);
                    liLineInfo = bundle.getLiLineInfo();
                }
                UlBuffer newUlBuffer = new UlBuffer(i, ulLineInfo);
                previousUlBuffer.setSubUlBuffer(newUlBuffer, liLineInfo);
                previousUlBuffer = newUlBuffer;
            }
            lastUlBuffer = previousUlBuffer;
            return lastUlBuffer;
        }
    }


    class UlBuffer {

        int depth;
        Ul ul;
        Li dernierli;
        UlBuffer subUlBuffer;

        UlBuffer(int depth, LineInfo lineInfo) {
            ul = new Ul();
            if (lineInfo != null) {
                ul.setClassName(lineInfo.getClassName());
                ul.setHtmlId(lineInfo.getHtmlId());
            }
            this.depth = depth;
        }

        void setSubUlBuffer(UlBuffer ulBuffer, LineInfo potentialLiLineInfo) {
            if (dernierli == null) {
                dernierli = createLi(potentialLiLineInfo);
                ul.addLi(dernierli);
            }
            dernierli.addUl(ulBuffer.ul);
            subUlBuffer = ulBuffer;
        }

        void addLi(Li li) {
            ul.addLi(li);
            dernierli = li;
            subUlBuffer = null;
        }

        void addP(P p) {
            if (dernierli == null) {
                dernierli = new Li();
                ul.addLi(dernierli);
            }
            dernierli.addP(p);
            subUlBuffer = null;
        }

    }

    private Li createLi(LineInfo lineInfo) {
        Li li = new Li();
        if (lineInfo != null) {
            li.setClassName(lineInfo.getClassName());
            li.setHtmlId(lineInfo.getHtmlId());
        }
        return li;
    }


    private class DepthParser {

        ArrayList infoList = new ArrayList();
        String paragraphLine;

        public void parseLine(String line) {
            infoList.clear();
            parseLoop(line);
        }

        private void parseLoop(String line) {
            LineInfoBundle bundle = new LineInfoBundle(line);
            infoList.add(bundle);
            line = bundle.getLine();
            if (line.startsWith("-")) {
                parseLoop(line.substring(1).trim());
            } else {
                paragraphLine = line;
            }

        }

        public int getDepth() {
            return infoList.size();
        }

        public LineInfoBundle getLastLineInfoBundle() {
            return (LineInfoBundle) infoList.get(infoList.size() - 1);
        }

        public List getLineInfoBundleList() {
            return infoList;
        }

    }


    private class LineInfoBundle {

        LineInfo liLineInfo = null;
        LineInfo ficheBlockLineInfo = null;
        String line;

        LineInfoBundle(String s) {
            liLineInfo = LineInfo.parse(s);
            ficheBlockLineInfo = LineInfo.parse(liLineInfo.getLine());
            line = ficheBlockLineInfo.getLine();
        }

        public String getLine() {
            return line;
        }

        public LineInfo getFicheBlockLineInfo() {
            return ficheBlockLineInfo;
        }

        public LineInfo getLiLineInfo() {
            return liLineInfo;
        }

    }

}
