/* BdfServer
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.request.parsers.blockparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.parsers.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class ZoneBlockParser extends BlockParser {

    ParagraphParser parser;

    public ZoneBlockParser(TextParser textParser) {
        super(textParser);
        parser = new ParagraphParser(textParser);
    }

    protected int parseZoneBlockElements(ZoneBlock zoneBlock, int beginIndex) {
        int length = getLineArrayLength();
        while (beginIndex < length) {
            String line = getCleanedLine(beginIndex);
            LineInfo lineInfo = LineInfo.parse(line);
            line = lineInfo.getLine();
            if (line.length() < 2) {
                break;
            }
            if (line.startsWith("n=")) {
                lineInfo.setLine(line.substring(2).trim());
                parseParagraph(zoneBlock.getNumero(), lineInfo);
                beginIndex++;
            } else if (line.startsWith("l=")) {
                lineInfo.setLine(line.substring(2).trim());
                parseParagraph(zoneBlock.getLegende(), lineInfo);
                beginIndex++;
            } else {
                break;
            }
        }
        return beginIndex;
    }

    protected void parseParagraph(ParagraphBlock paragraphBlock, LineInfo lineInfo) {
        parser.parseParagraphBlock(paragraphBlock, lineInfo);
    }

    protected void addIdentifiersInfo(ZoneBlock zoneBlock, LineInfo lineInfo) {
        zoneBlock.setClassName(lineInfo.getClassName());
        zoneBlock.setHtmlId(lineInfo.getHtmlId());
    }

}
