/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.spanparsers;

import net.mapeadores.basedefiches.corpus.fiche.S;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.SpanRules;
import java.util.List;
import java.util.ArrayList;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
class BracketsParser extends SpanParser {

    private BracketTokens tokens = new BracketTokens();
    private SpanRules currentRules;
    private StopIndices globalStopIndices = new StopIndices();
    private char startChar = '[';
    private char endChar = ']';

    BracketsParser(TextParser textParser) {
        super(textParser);
        setStartCharCount(1);
    }

    BracketsParser(TextParser textParser, char startChar, char endChar) {
        this(textParser);
        this.startChar = startChar;
        this.endChar = endChar;
    }

    public char getStartChar() {
        return startChar;
    }

    public boolean parse(String bout, int startBracketIndex) {
        reinit(bout);
        globalStopIndices.clear();
        currentRules = null;
        short spanType = 0;
        if (startBracketIndex > (bout.length() - 3)) {
            return false;
        }
        char initialCarac = bout.charAt(startBracketIndex + 1);
        if (initialCarac == '%') {
            return parseBrRef(startBracketIndex + 2);
        } else if (initialCarac == '>') {
            spanType = S.TYPE_IREF;
            return parseInternalRef(spanType, startBracketIndex + 2);
        } else if (initialCarac == '#') {
            spanType = S.TYPE_ANCHOR;
            return parseInternalRef(spanType, startBracketIndex + 2);
        }
        try {
            spanType = S.getInitialToShort(initialCarac);
            currentRules = SpanRules.getSpanRules(spanType);
        } catch (IllegalArgumentException iae) {
            return false;
        }
        String subtype = null;
        if (S.hasAttribute(spanType, S.SUBTYPE_BITVALUE)) {
            subtype = S.getInitialToSubtype(spanType, bout.charAt(startBracketIndex + 2));
            if (subtype != null) {
                startBracketIndex++;
            }
        }
        char secondBracket = bout.charAt(startBracketIndex + 2);
        if (secondBracket != startChar) {
            return false;
        }
        return parseCommon(spanType, subtype, startBracketIndex + 3);
    }

    private boolean parseBrRef(int beginIndex) {
        if (sourceString.charAt(beginIndex) == endChar) {
            setEndIndex(beginIndex);
            initSpan(S.TYPE_BR);
            return true;
        } else {
            return false;
        }
    }

    private boolean parseInternalRef(short spanType, int beginIndex) {
        checkStopIndices(beginIndex, globalStopIndices);
        if (globalStopIndices.isWrong()) {
            return false;
        }
        StringBuilder refbuf = new StringBuilder();
        int lastStringIndex = globalStopIndices.getLastStringIndex();
        try {
            for (int i = beginIndex; i <= lastStringIndex; i++) {
                i = textParser.parseInSpanChar(sourceString, i, refbuf, false);
            }
        } catch (IOException ioe) {
        }
        String ref = refbuf.toString();
        String value = ref;
        if (ref.endsWith("!")) {
            ref = ref.substring(0, ref.length() - 1);
            value = "";
        }
        setEndIndex(globalStopIndices.getEndIndex());
        S s = initSpan(spanType);
        s.setValue(value);
        s.setRef(ref);
        return true;
    }

    private boolean parseCommon(short spanType, String subtype, int beginIndex) {
        tokens.parseTokens(beginIndex);
        if (globalStopIndices.isWrong()) {
            return false;
        }
        setEndIndex(globalStopIndices.getEndIndex());
        S s = initSpan(spanType);
        if (subtype != null) {
            s.setSubtype(subtype);
        }
        int count = currentRules.getAttributesCount();
        for (int i = 0; i < count; i++) {
            String token = tokens.getToken(i);
            s.setAttribute(currentRules.getAttributeBitValue(i), token);
        }
        s.checkHttpValue(true);
        return true;
    }

    private void checkStopIndices(int beginIndex, StopIndices stopIndices) {
        int firstSpace = -1;
        for (int i = beginIndex; i < sourceStringLength; i++) {
            char carac = sourceString.charAt(i);
            if (carac == ' ') {
                if (firstSpace == -1) {
                    firstSpace = i;
                }
            } else if (carac == startChar) {
                if (firstSpace == -1) {
                    stopIndices.setWrong();
                } else {
                    stopIndices.setEndIndex(firstSpace - 1);
                    stopIndices.setLastStringIndex(firstSpace - 1);
                }
                return;
            } else if (carac == endChar) {
                stopIndices.setEndIndex(i);
                stopIndices.setLastStringIndex(i - 1);
                return;
            } else if (carac == '\\') {
                i++;
            }
        }

        if (firstSpace == -1) {
            stopIndices.setEndIndex(sourceStringLength - 1);
            stopIndices.setLastStringIndex(sourceStringLength - 1);
        } else {
            stopIndices.setEndIndex(firstSpace - 1);
            stopIndices.setLastStringIndex(firstSpace - 1);
        }
    }


    private class BracketTokens {

        private List<String> tokenList = new ArrayList<String>();
        private boolean currentIsTypo;
        private int currentTokenBeginIndex;
        private StopIndices innerStopIndices = new StopIndices();

        public void parseTokens(int tokensBeginIndex) {
            tokenList.clear();
            innerStopIndices.clear();
            this.currentTokenBeginIndex = tokensBeginIndex;
            while (true) {
                initToken();
                parseToken();
                if (innerStopIndices.isWrong()) {
                    globalStopIndices.setWrong();
                    return;
                }
                int idxFin = testFollowing(innerStopIndices.getEndIndex());
                if (idxFin > 0) {
                    globalStopIndices.setEndIndex(idxFin);
                    return;
                }
            }
        }

        private void initToken() {
            int size = tokenList.size();
            if (size < currentRules.getAttributesCount()) {
                currentIsTypo = currentRules.isTypoAttribute(size);
            } else {
                currentIsTypo = false;
            }
        }

        private void parseToken() {
            checkStopIndices(currentTokenBeginIndex, innerStopIndices);
            if (innerStopIndices.isWrong()) {
                return;
            }
            StringBuilder buf = new StringBuilder();
            try {
                for (int i = currentTokenBeginIndex; i <= innerStopIndices.getLastStringIndex(); i++) {
                    i = textParser.parseInSpanChar(sourceString, i, buf, currentIsTypo);
                }
            } catch (IOException ioe) {
            }
            tokenList.add(buf.toString().trim());
        }

        public String getToken(int i) {
            if (i >= tokenList.size()) {
                return "";
            }
            return tokenList.get(i);
        }

        private int testFollowing(int lastStopIdx) {
            int p = 0;
            for (int i = lastStopIdx + 1; i < sourceStringLength; i++) {
                char carac = sourceString.charAt(i);
                if (Character.isWhitespace(carac)) {
                    p++;
                    continue;
                }
                if (carac == startChar) {
                    if (i == (sourceStringLength - 1)) {
                        return lastStopIdx;
                    } else {
                        currentTokenBeginIndex = i + 1;
                        return 0;
                    }
                } else if (carac == endChar) {
                    return i;
                } else {
                    return lastStopIdx;
                }
            }
            return lastStopIdx + p;
        }

    }


    private static class StopIndices {

        int lastStringIndex;
        int endIndex;
        boolean isWrong = false;

        public void clear() {
            isWrong = false;
        }

        public int getLastStringIndex() {
            return lastStringIndex;
        }

        public int getEndIndex() {
            return endIndex;
        }

        public boolean isWrong() {
            return isWrong;
        }

        public void setEndIndex(int endIndex) {
            this.endIndex = endIndex;
        }

        public void setLastStringIndex(int lastStringIndex) {
            this.lastStringIndex = lastStringIndex;
        }

        public void setWrong() {
            isWrong = true;
            lastStringIndex = -1;
            endIndex = -1;
        }

    }

}
