/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.spanparsers;

import net.mapeadores.basedefiches.corpus.fiche.S;
import fr.exemole.bdfserver.request.TextParser;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
class EmphasisParser extends SpanParser {

    private short type;

    public EmphasisParser(TextParser textParser) {
        super(textParser);
    }

    public char getStartChar() {
        return '{';
    }

    public boolean parse(String bout, int startIndex) {
        reinit(bout);
        int beginIndex = initBraceCount(startIndex);
        if (beginIndex == (sourceStringLength - 1)) {
            return false;
        }
        int stopIndex = getStopIndex(beginIndex);
        if (stopIndex == -1) {
            return false;
        }
        StringBuilder buftexte = new StringBuilder();
        try {
            for (int i = beginIndex; i < stopIndex; i++) {
                i = textParser.parseInSpanChar(sourceString, i, buftexte, true);
            }
        } catch (IOException ioe) {
        }
        checkEndIndex(stopIndex);
        S span = initSpan(type);
        span.setValue(buftexte.toString().trim());
        return true;
    }

    private int initBraceCount(int startIndex) {
        int braceCount = 1;
        int realStart = startIndex + 1;
        for (int i = startIndex + 1; i < sourceStringLength; i++) {
            char carac = sourceString.charAt(i);
            if (carac == '{') {
                braceCount++;
                realStart++;
                if (braceCount == 3) {
                    break;
                }
            } else {
                break;
            }
        }
        if (braceCount == 1) {
            type = S.TYPE_EM;
        } else if (braceCount == 2) {
            type = S.TYPE_STRG;
        } else if (braceCount == 3) {
            type = S.TYPE_EMSTRG;
        }
        setStartCharCount(braceCount);
        return realStart;
    }

    private int getStopIndex(int beginIndex) {
        for (int i = beginIndex; i < sourceStringLength; i++) {
            char carac = sourceString.charAt(i);
            switch (carac) {
                case '}':
                    return i;
                case '\\':
                    i++;
            }
        }
        return -1;
    }

    private void checkEndIndex(int stopIndex) {
        for (int i = stopIndex + 1; i < sourceStringLength; i++) {
            if (sourceString.charAt(i) == '}') {
                stopIndex++;
            } else {
                break;
            }
        }
        setEndIndex(stopIndex);
    }

}
