/* BdfServer - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.parsers.spanparsers;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.*;


/**
 *
 * @author Vincent Calame
 */
public abstract class SpanParser {

    protected String sourceString;
    protected int sourceStringLength;
    private S parsedSpan;
    private int endIndex;
    private int startCharCount = 1;
    protected TextParser textParser;

    public SpanParser() {
    }

    public SpanParser(TextParser textParser) {
        this.textParser = textParser;
    }

    public static SpanParser getEmphasisParser(TextParser textParser) {
        return new EmphasisParser(textParser);
    }

    public static SpanParser getBracketsParser(TextParser textParser) {
        return new BracketsParser(textParser);
    }

    public static SpanParser getParenthesisParser(TextParser textParser) {
        return new BracketsParser(textParser, '(', ')');
    }

    public static SpanParser getHttpParser() {
        return new HttpParser();
    }

    public static SpanParser getWwwParser() {
        return new WwwParser();
    }

    public abstract boolean parse(String sourceString, int startIndex);

    public abstract char getStartChar();

    public S getS() {
        return parsedSpan;
    }

    public int getStartCharCount() {
        return startCharCount;
    }

    public int getEndIndex() {
        return endIndex;
    }

    public TextParser getTextParser() {
        return textParser;
    }

    public void setTextParser(TextParser textParser) {
        this.textParser = textParser;
    }

    protected S initSpan(short type) {
        parsedSpan = new S(type);
        return parsedSpan;
    }

    protected void reinit(String sourceString) {
        endIndex = -1;
        parsedSpan = null;
        this.sourceString = sourceString;
        this.sourceStringLength = sourceString.length();
    }

    protected void setEndIndex(int endIndex) {
        this.endIndex = endIndex;
    }

    protected void setStartCharCount(int startCharCount) {
        this.startCharCount = startCharCount;
    }

    /*
     *Valide si un caractère peut-être dans un url ou non lorsqu'on fait une reconnaissance avec http. La règle est assez restrictive.
     * Il est à noter que rentrer un url avec [[][]] n'impose, lui aucune restriction.
     * le  point . et le point d'interrogation ? doivent être traités séparément
     */
    static boolean checkCharInUrl(char c) {
        if (c < '#' || c > '~') {
            return false;
        }
        if (c == '#') {
            return true;
        }
        if (c == '%') {
            return true;
        }
        if (c == '&') {
            return true;
        }
        if (c == '+') {
            return true;
        }
        if (c == '-') {
            return true;
        }
        if (c >= '/' && c <= ':') {
            return true;
        }
        if (c == '=') {
            return true;
        }
        if (c >= '@' && c <= 'Z') {
            return true;
        }
        if (c == '_') {
            return true;
        }
        if (c >= 'a' && c <= 'z') {
            return true;
        }
        if (c == '~') {
            return true;
        }
        return false;
    }

    int appendUrl(StringBuffer buftexte, int beginIndex) {
        int stopIndex = beginIndex - 1;
        for (int i = beginIndex; i < sourceStringLength; i++) {
            char c = sourceString.charAt(i);
            if (c == '?') {
                if (i == (sourceStringLength - 1)) {
                    break;
                }
                char suivant = sourceString.charAt(i + 1);
                if (checkCharInUrl(suivant)) {
                    buftexte.append('?');
                } else {
                    break;
                }
            } else if (c == '.') {
                if (i == (sourceStringLength - 1)) {
                    break;
                }
                char suivant = sourceString.charAt(i + 1);
                if (suivant == '.') {
                    buftexte.append("..");
                    i = i + 1;
                } else if (checkCharInUrl(suivant)) {
                    buftexte.append('.');
                } else {
                    break;
                }
            } else if (checkCharInUrl(c)) {
                buftexte.append(c);
            } else {
                break;
            }
            stopIndex = i;
        }
        return stopIndex;
    }

}
