/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.tools;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class MutableRequestMap implements RequestMap {

    private ValueBuffer[] valueBufferArray;
    private HashMap<String, ValueBuffer> valueBufferMap = new HashMap<String, ValueBuffer>();

    public MutableRequestMap(String[] parameterNameArray) {
        int length = parameterNameArray.length;
        valueBufferArray = new ValueBuffer[length];
        for (int i = 0; i < length; i++) {
            String name = parameterNameArray[i];
            ValueBuffer valuesBuffer = valueBufferMap.get(name);
            if (valuesBuffer == null) {
                valuesBuffer = new ValueBuffer();
                valueBufferMap.put(name, valuesBuffer);
            }
            valueBufferArray[i] = valuesBuffer;
        }
    }

    public Document getDocument() {
        return null;
    }

    public FileValue getFileValue(String name) {
        return null;
    }

    public FileValue[] getFileValues(String name) {
        return null;
    }

    public String getParameter(String name) {
        ValueBuffer valueBuffer = valueBufferMap.get(name);
        if (valueBuffer == null) {
            return null;
        }
        return valueBuffer.getValue();
    }

    public String[] getParameterValues(String name) {
        ValueBuffer valueBuffer = valueBufferMap.get(name);
        if (valueBuffer == null) {
            return null;
        }
        return valueBuffer.getValueArray();
    }

    public Set<String> getParameterNameSet() {
        return valueBufferMap.keySet();
    }

    public short getResponseDataType() {
        return 0;
    }

    public void update(String[] valueArray) {
        clear();
        int length = Math.min(valueArray.length, valueBufferArray.length);
        for (int i = 0; i < length; i++) {
            valueBufferArray[i].add(valueArray[i]);
        }
    }

    public void clear() {
        for (ValueBuffer valueBuffer : valueBufferMap.values()) {
            valueBuffer.clear();
        }
    }


    private static class ValueBuffer {

        private List<String> values = new ArrayList<String>();

        private ValueBuffer() {
        }

        private void clear() {
            values.clear();
        }

        private void add(String value) {
            values.add(value);
        }

        private String getValue() {
            if (values.size() == 0) {
                return "";
            } else {
                return (String) values.get(0);
            }
        }

        private String[] getValueArray() {
            int size = values.size();
            if (size == 0) {
                String[] result = new String[1];
                result[0] = "";
                return result;
            } else {
                String[] result = new String[size];
                for (int i = 0; i < size; i++) {
                    result[i] = (String) values.get(i);
                }
                return result;
            }
        }

    }

}

