/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.tostring;

import net.mapeadores.basedefiches.corpus.fiche.*;
import fr.exemole.bdfserver.request.TextParser;
import java.io.IOException;
import net.mapeadores.basedefiches.SubsetKey;


/**
 *
 * @author Vincent Calame
 */
public class FicheBlockToString {

    private boolean withSpecialParseChars = true;
    private UlWriter ulWriter = new UlWriter();
    private StringBuilder buf;
    private char endBracketChar = ']';
    private char startBracketChar = '[';

    private FicheBlockToString() {
        buf = new StringBuilder(1000);
    }

    private void setWithSpecialParseChars(boolean withSpecialParseChars) {
        this.withSpecialParseChars = withSpecialParseChars;
    }

    public static String toString(FicheBlockList ficheBlockList, boolean withSpecialParseChars) {
        if (ficheBlockList.getFicheBlockCount() == 0) {
            return "";
        }
        FicheBlockToString fbts = new FicheBlockToString();
        fbts.appendFicheBlockList(ficheBlockList);
        fbts.setWithSpecialParseChars(withSpecialParseChars);
        return fbts.buf.toString();
    }

    private void appendFicheBlockList(FicheBlockList ficheBlockList) {
        ulWriter.clear();
        int size = ficheBlockList.getFicheBlockCount();
        boolean hPrevious = false;
        for (int j = 0; j < size; j++) {
            FicheBlock ficheBlock = ficheBlockList.getFicheBlock(j);
            boolean isH = false;
            if (j > 0) {
                buf.append('\n');
            }
            if (ficheBlock instanceof H) {
                isH = true;
                H h = (H) ficheBlock;
                int level = h.getLevel();
                if (!hPrevious) {
                    buf.append('\n');
                }
                appendIdentifiers(h);
                for (int i = 0; i < level; i++) {
                    buf.append("#");
                }
                appendParagraphBlock((H) ficheBlock);
                buf.append('\n');
            } else if (ficheBlock instanceof Ul) {
                ulWriter.appendUl((Ul) ficheBlock);
            } else if (ficheBlock instanceof Code) {
                appendCode((Code) ficheBlock);
            } else if (ficheBlock instanceof Table) {
                appendTable((Table) ficheBlock);
            } else if (ficheBlock instanceof Insert) {
                appendInsert((Insert) ficheBlock);
            } else if (ficheBlock instanceof Div) {
                appendDiv((Div) ficheBlock);
            } else if (ficheBlock instanceof CdataDiv) {
                appendCdataDiv((CdataDiv) ficheBlock);
            } else {
                P p = (P) ficheBlock;
                appendIdentifiers(p);
                boolean rajout = false;
                short type = p.getType();
                if (type == P.TYPE_CITATION) {
                    buf.append('>');
                    rajout = true;
                }
                String source = p.getSource();
                if (type == P.TYPE_QUESTION) {
                    buf.append('<');
                    buf.append(source);
                    buf.append(" ?:");
                    rajout = true;
                } else if (type == P.TYPE_REMARQUE) {
                    buf.append('<');
                    buf.append(source);
                    buf.append(" !:");
                    rajout = true;
                } else if (source.length() > 0) {
                    buf.append('<');
                    buf.append(source);
                    buf.append(" :");
                    rajout = true;
                }
                if (rajout) {
                    buf.append(' ');
                }
                boolean done = appendParagraphBlock(p);
                if ((done) || (rajout)) {
                    buf.append('\n');
                }
            }
            hPrevious = isH;
        }

    }

    private boolean appendParagraphBlock(ParagraphBlock prg) {
        try {
            return appendContentList(buf, prg, withSpecialParseChars);
        } catch (IOException ioe) {
            throw new IllegalStateException("Sould not occur");
        }
    }

    public static boolean appendContentList(Appendable buf, TextContentList contentList, boolean withSpecialParseChars) throws IOException {
        boolean rajout = false;
        int count = contentList.getTextContentCount();
        for (int i = 0; i < count; i++) {
            Object obj = contentList.getTextContent(i);
            if (obj instanceof String) {
                TextParser.escapeText(buf, (String) obj, !rajout);
                if ((!rajout) && (((String) obj).length() > 0)) {
                    rajout = true;
                }
            } else if (obj instanceof S) {
                rajout = true;
                TextParser.escapeSpan(buf, (S) obj, withSpecialParseChars);
            }
        }
        return rajout;
    }

    private void appendCode(Code code) {
        short type = code.getType();
        appendIdentifiers(code);
        buf.append("+++");
        if (type != Code.SANS_TYPE) {
            buf.append(Code.typeToInitiale(type));
        }
        buf.append('\n');
        appendZoneBlockElements(code);
        String indentString = "  ";
        switch (type) {
            case Code.SANS_TYPE:
                indentString = "_";
        }
        for (int i = 0; i < code.getFicheBlockCount(); i++) {
            Ln ln = code.getLn(i);
            appendIdentifiers(ln);
            String s = ln.getValue();
            int indent = ln.getIndentation();
            for (int j = 0; j < indent; j++) {
                buf.append(indentString);
            }
            if (s.length() > 0) {
                char carac = s.charAt(0);
                switch (carac) {
                    case '+':
                    case '\\':
                    case '[':
                        buf.append('\\');
                        break;
                    case '_':
                        if (type == Code.SANS_TYPE) {
                            buf.append('\\');
                        }
                        break;
                }
            }
            buf.append(s);
            buf.append('\n');
        }
        buf.append("+++++++++++");
        buf.append('\n');
    }

    private void appendDiv(Div div) {
        appendIdentifiers(div);
        buf.append("!!!");
        buf.append('\n');
        appendZoneBlockElements(div);
        appendFicheBlockList(div.getFicheBlockList());
        buf.append("!!!!!!!!!!!");
        buf.append('\n');
    }

    private void appendCdataDiv(CdataDiv cdataDiv) {
        appendIdentifiers(cdataDiv);
        buf.append("!!!cdata");
        buf.append('\n');
        appendZoneBlockElements(cdataDiv);
        buf.append(cdataDiv.getCdata());
        buf.append('\n');
        buf.append("!!!!!!!!!!!");
        buf.append('\n');
    }

    private void appendInsert(Insert insert) {
        short type = insert.getType();
        appendIdentifiers(insert);
        buf.append(":::");
        buf.append(Insert.typeToInitiale(type));
        if (type == Insert.LETTRINE_TYPE) {
            buf.append(Insert.positionToInitiale(insert.getPosition()));
        }
        buf.append('\n');
        appendZoneBlockElements(insert);
        SubsetKey albumSubsetKey = insert.getAlbumSubsetKey();
        String credit = insert.getCredit();
        if (albumSubsetKey != null) {
            buf.append("i= ");
            buf.append(albumSubsetKey.getSubsetName());
            buf.append("/");
            buf.append(insert.getIdalbum());
            buf.append('\n');
            if (credit.length() > 0) {
                buf.append("c= ");
                buf.append(credit);
                buf.append('\n');
            }
        } else {
            buf.append("s= ");
            buf.append(insert.getSrc());
            buf.append('\n');
            buf.append("c= ");
            buf.append(credit);
            buf.append('\n');
            appendInt('w', insert.getWidth());
            appendInt('h', insert.getHeight());
        }
        String ref = insert.getRef();
        if (ref.length() > 0) {
            buf.append("a= ");
            buf.append(ref);
            buf.append('\n');
        }
        buf.append(":::::::::::");
        buf.append('\n');
    }

    private void appendInt(char carac, int value) {
        if (value < 0) {
            return;
        }
        buf.append(carac);
        buf.append("= ");
        buf.append(value);
        buf.append('\n');
    }

    private void appendZoneBlockElements(ZoneBlock zoneBlock) {
        appendParagraph('n', zoneBlock.getNumero());
        appendParagraph('l', zoneBlock.getLegende());
        buf.append('\n');
    }

    private void appendParagraph(char carac, ParagraphBlock paragraphBlock) {
        appendIdentifiers(paragraphBlock);
        buf.append(carac);
        buf.append("= ");
        appendParagraphBlock(paragraphBlock);
        buf.append('\n');
    }

    private void appendTable(Table table) {
        appendIdentifiers(table);
        buf.append("===");
        int headerMask = table.getHeaderMask();
        if ((headerMask & Table.HORIZONTALHEADER_BITVALUE) != 0) {
            buf.append('h');
        }
        if ((headerMask & Table.VERTICALHEADER_BITVALUE) != 0) {
            buf.append('v');
        }
        buf.append('\n');
        appendZoneBlockElements(table);
        int tableSize = table.getTrCount();
        for (int i = 0; i < tableSize; i++) {
            Tr tr = table.getTr(i);
            int trSize = tr.getFicheBlockCount();
            if (trSize == 0) {
                continue;
            }
            boolean done = appendIdentifiers(tr);
            if (done) {
                buf.append('\n');
            }
            for (int k = 0; k < trSize; k++) {
                Td td = tr.getTd(k);
                appendTd(td);

            }
            buf.append('\n');
        }

        buf.append("===========");
        buf.append('\n');
    }

    private void appendTd(Td td) {
        appendIdentifiers(td);
        boolean rajout = appendParagraphBlock(td);
        if (!rajout) {
            buf.append('-');
        }
        int rowspan = td.getRowSpan();
        if (rowspan > 1) {
            buf.append(" [+");
            buf.append(rowspan - 1);
            buf.append("]");
        }
        buf.append('\n');
        int colspan = td.getColSpan();
        if (colspan > 1) {
            for (int i = 1; i < colspan; i++) {
                buf.append('=');
                buf.append('\n');
            }
        }
    }

    private boolean appendIdentifiers(FicheBlock ficheBlock) {
        int result = appendIdentifiers(ficheBlock, 0);
        return (result == 0);
    }

    private int appendIdentifiers(FicheBlock ficheBlock, int noIndentifiedCount) {
        String className = ficheBlock.getClassName();
        String htmlId = ficheBlock.getHtmlId();
        if ((className.length() > 0) || (htmlId.length() > 0)) {
            if (noIndentifiedCount > 0) {
                for (int i = 0; i < noIndentifiedCount; i++) {
                    buf.append(startBracketChar);
                    buf.append(endBracketChar);
                }
            }
            buf.append(startBracketChar);
            buf.append(className);
            if (htmlId.length() > 0) {
                buf.append('!');
                buf.append(htmlId);
            }
            buf.append(endBracketChar);
            return 0;
        } else {
            return noIndentifiedCount + 1;
        }
    }

    private int appendIdentifiers(IdentifiedFicheBlockList ficheBlockList, int noIndentifiedCount) {
        String className = ficheBlockList.getClassName();
        String htmlId = ficheBlockList.getHtmlId();
        if ((className.length() > 0) || (htmlId.length() > 0)) {
            if (noIndentifiedCount > 0) {
                for (int i = 0; i < noIndentifiedCount; i++) {
                    buf.append(startBracketChar);
                    buf.append(endBracketChar);
                }
            }
            buf.append(startBracketChar);
            buf.append(className);
            if (htmlId.length() > 0) {
                buf.append('!');
                buf.append(htmlId);
            }
            buf.append(endBracketChar);
            return 0;
        } else {
            return noIndentifiedCount + 1;
        }
    }

    private boolean appendIdentifiers(IdentifiedFicheBlockList ficheBlockList) {
        int result = appendIdentifiers(ficheBlockList, 0);
        return (result == 0);
    }


    private class UlWriter {

        int liCount = 0;

        public void clear() {
            liCount = 0;
        }

        public void appendUl(Ul ul) {
            if (ul.getLiCount() == 0) {
                return;
            }
            liCount = 0;
            appendIdentifiers(ul, 0);
            startAppend(ul, 1);
        }

        private void startAppend(Ul ul, int currentLevel) {
            buf.append('-');
            appendFirstLi(ul.getLi(0), currentLevel);
            for (int i = 1; i < ul.getLiCount(); i++) {
                appendFollowing(ul.getLi(i), currentLevel);
            }
        }

        private void appendFirstLi(Li li, int currentLevel) {
            liCount = appendIdentifiers(li, 0);
            for (int i = 0; i < li.getFicheBlockCount(); i++) {
                FicheBlock fb = li.getFicheBlock(i);
                if (fb instanceof P) {
                    if (i == 0) {
                        appendFirst((P) fb);
                    } else {
                        appendFollowing((P) fb, currentLevel);
                    }
                } else if (fb instanceof Ul) {
                    if (i == 0) {
                        appendFirst((Ul) fb, currentLevel);
                    } else {
                        appendFollowing((Ul) fb, currentLevel);
                    }
                }
            }
        }

        private void appendFollowing(Li li, int currentLevel) {
            if (li.getFicheBlockCount() == 0) {
                return;
            }
            for (int i = 0; i < currentLevel; i++) {
                buf.append('-');
            }
            appendFirstLi(li, currentLevel);
        }

        private void appendFirst(P p) {
            appendIdentifiers(p, liCount);
            buf.append(' ');
            appendParagraphBlock(p);
            buf.append('\n');
        }

        private void appendFollowing(P p, int currentLevel) {
            appendIdentifiers(p);
            for (int i = 0; i < currentLevel; i++) {
                buf.append('_');
            }
            buf.append(' ');
            appendParagraphBlock(p);
            buf.append('\n');
        }

        private void appendFirst(Ul ul, int currentLevel) {
            if (ul.getLiCount() == 0) {
                buf.append('\n');
                return;
            }
            appendIdentifiers(ul, liCount);
            startAppend(ul, currentLevel + 1);
        }

        private void appendFollowing(Ul ul, int currentLevel) {
            if (ul.getLiCount() == 0) {
                return;
            }
            for (int i = 0; i < currentLevel; i++) {
                buf.append('-');
            }
            appendIdentifiers(ul, 0);
            startAppend(ul, currentLevel + 1);
        }

    }

}
