/* BdfServer - Copyright (c) 2006-2009 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.request.tostring;

import net.mapeadores.basedefiches.corpus.fiche.*;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.*;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.text.SeparatorOptions;
import fr.exemole.bdfserver.request.*;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class FicheItemToString {

    public FicheItemToString() {
    }

    public static String toString(FicheItemList ficheItemList, SeparatorOptions separatorOptions, Parameters parameters) {
        int size = ficheItemList.getFicheItemCount();
        if (size == 0) {
            return "";
        }
        StringBuilder buf = new StringBuilder(128);
        for (int k = 0; k < size; k++) {
            FicheItem ficheItem = ficheItemList.getFicheItem(k);
            buf.append(toString(ficheItem, parameters));
            if (k < (size - 1)) {
                buf.append(separatorOptions.getSeparator());
                if (separatorOptions.isSpaceInclude()) {
                    buf.append(' ');
                }
            } else if (separatorOptions.isLastInclude()) {
                buf.append(separatorOptions.getSeparator());
                if (separatorOptions.isSpaceInclude()) {
                    buf.append(' ');
                }
            }
        }
        return buf.toString();
    }

    public static String toString(FicheItem item, Parameters parameters) {
        if (item instanceof Item) {
            return toString((Item) item);
        } else if (item instanceof Personne) {
            return toString((Personne) item, getDefaultSphereKey(parameters));
        } else if (item instanceof Langue) {
            return toString((Langue) item);
        } else if (item instanceof Pays) {
            return toString((Pays) item);
        } else if (item instanceof Datation) {
            return toString((Datation) item);
        } else if (item instanceof Courriel) {
            return toString((Courriel) item);
        } else if (item instanceof Link) {
            return toString((Link) item);
        } else if (item instanceof Nombre) {
            return toString((Nombre) item, getDecimalChar(parameters));
        } else if (item instanceof Montant) {
            return toString((Montant) item, getDecimalChar(parameters));
        } else if (item instanceof Geopoint) {
            return toString((Geopoint) item, getDecimalChar(parameters));
        } else if (item instanceof Para) {
            return toString((Para) item);
        } else if (item instanceof Image) {
            return toString((Image) item);
        } else {
            throw new IllegalArgumentException("unknown FicheItem implementation");
        }
    }

    public static String toString(DegreDecimal degreDecimal, Parameters parameters) {
        return degreDecimal.toDecimal().toStringWithBlank(getDecimalChar(parameters));
    }

    public static String toString(Para para) {
        StringBuilder buf = new StringBuilder();
        try {
            FicheBlockToString.appendContentList(buf, para, true);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static String toString(Item item) {
        String value = item.getValue();
        return TextParser.escapeString(value);
    }

    public static String toString(Personne personne, SubsetKey defaultSphereKey) {
        if (personne.isRedacteurSphere()) {
            try {
                RedacteurKey redacteurKey = RedacteurKey.parse(personne.getRedacteurKeyString());
                return toString(redacteurKey, defaultSphereKey);
            } catch (java.text.ParseException pe) {
                return personne.getRedacteurKeyString();
            }
        } else {
            StringBuilder buf = new StringBuilder(64);
            try {
                TextParser.escapeString(buf, personne.getNom());
                buf.append('*');
                if (personne.isNomAvant()) {
                    buf.append('%');
                }
                TextParser.escapeString(buf, personne.getPrenom());
                String organisme = personne.getOrganisme();
                if (organisme != null) {
                    buf.append('*');
                    TextParser.escapeString(buf, organisme);
                }
            } catch (IOException ioe) {
            }
            return buf.toString();
        }
    }

    public static String toString(RedacteurKey redacteurKey, SubsetKey defaultSphereKey) {
        if ((defaultSphereKey != null) && (defaultSphereKey.equals(redacteurKey.getSphereKey()))) {
            return redacteurKey.getIdsphereString();
        } else {
            return redacteurKey.toString(RedacteurKey.BRACKET_STYLE);
        }
    }

    public static String toString(Langue langue) {
        return LangInteger.toString(langue.getLangInteger());
    }

    public static String toString(Pays pays) {
        return pays.getCountryCode();
    }

    public static String toString(Datation datation) {
        return datation.toString();
    }

    public static String toString(Courriel courriel) {
        return courriel.toString();
    }

    public static String toString(Link link) {
        StringBuilder buf = new StringBuilder(128);
        String href = link.getHref();
        buf.append(href);
        String title = link.getTitle();
        if (title.length() == 0) {
            title = null;
        } else if (title.equals(href)) {
            title = null;
        } else if ((href.length() > 8) && (title.equals(href.substring(7)))) {
            title = null;
        } else {
            buf.append('*');
            try {
                TextParser.escapeString(buf, title);
            } catch (IOException ioe) {
            }
        }
        String comment = link.getComment();
        if (comment.length() > 0) {
            if (title == null) {
                buf.append('*');
            }
            buf.append('*');
            try {
                TextParser.escapeString(buf, comment);
            } catch (IOException ioe) {
            }
        }
        return buf.toString();
    }

    public static String toString(Image image) {
        StringBuilder buf = new StringBuilder(128);
        String src = image.getSrc();
        String alt = image.getAlt();
        String title = image.getTitle();
        buf.append(src);
        if (alt.length() > 0) {
            buf.append('*');
            try {
                TextParser.escapeString(buf, alt);
            } catch (IOException ioe) {
            }
        }
        if (title.length() > 0) {
            if (alt.length() == 0) {
                buf.append('*');
            }
            buf.append('*');
            try {
                TextParser.escapeString(buf, alt);
            } catch (IOException ioe) {
            }
        }
        return buf.toString();
    }

    public static String toString(Nombre nombre, char decimalChar) {
        String valString = nombre.getDecimal().toStringWithBlank(decimalChar);
        return valString;
    }

    public static String toString(Montant montant, char decimalChar) {
        String valString = montant.getDecimal().toStringWithBlank(decimalChar);
        String cur = CurrencyInteger.toString(montant.getCurrencyInteger());
        return valString + cur;
    }

    public static String toString(Geopoint geopoint, char decimalChar) {
        StringBuilder buf = new StringBuilder();
        buf.append(geopoint.getLatitude().toDecimal().toStringWithBlank(decimalChar));
        buf.append("* ");
        buf.append(geopoint.getLongitude().toDecimal().toStringWithBlank(decimalChar));
        return buf.toString();
    }

    private static char getDecimalChar(Parameters parameters) {
        if (parameters == null) {
            return '.';
        }
        return parameters.getDecimalChar();
    }

    private static SubsetKey getDefaultSphereKey(Parameters parameters) {
        if (parameters == null) {
            return null;
        }
        return parameters.getDefautSphereKey();
    }


    public static class Parameters {

        private SubsetKey defautSphereKey = null;
        private char decimalChar = '.';

        public Parameters() {
        }

        public Parameters(Parameters other) {
            this.decimalChar = other.decimalChar;
            this.defautSphereKey = other.defautSphereKey;
        }

        public SubsetKey getDefautSphereKey() {
            return defautSphereKey;
        }

        public void setDefautSphereKey(SubsetKey defautSphereKey) {
            this.defautSphereKey = defautSphereKey;
        }

        public char getDecimalChar() {
            return decimalChar;
        }

        public void setDecimalChar(char decimalChar) {
            this.decimalChar = decimalChar;
        }

    }

}
