<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0" xmlns:xlink="http://www.w3.org/1999/xlink">

    <!-- nécessite la variable $FORMATLOCALE (pour <xsl:template name="subtemplate_LOCLIB">) -->

<!-- accessible de l'extérieur : MODE_INLINE MODE_BLOCK -->

    <xsl:template match="*" mode="MODE_BLOCK">
        <xsl:param name="PARAM_CLASS" select=""/>
        <text:p text:style-name="{$PARAM_CLASS}"><xsl:apply-templates select="."/></text:p>
    </xsl:template>

    <xsl:template match="*" mode="MODE_LISTITEM">
        <xsl:param name="PARAM_CLASS" select=""/>
        <text:list-item><text:p text:style-name="{$PARAM_CLASS}"><xsl:apply-templates select="."/></text:p></text:list-item>
    </xsl:template>

    <xsl:template match="*" mode="MODE_INLINE">
        <xsl:param name="PARAM_SEP" select="', '"/>
        <xsl:if test="position() &gt; 1"><xsl:value-of select="$PARAM_SEP"/></xsl:if>
        <xsl:apply-templates select="."/>
    </xsl:template>

    <xsl:template match="soustitre">
        <xsl:call-template name="subtemplate_PRGCONTENT"/>
    </xsl:template>

    <xsl:template match="item">
        <xsl:value-of select="."/>
    </xsl:template>

    <xsl:template match="personne[original = true()]">
        <xsl:value-of select="original"/>
        <xsl:if test="(nom = true()) or (prenom=true())">
            <xsl:text> (</xsl:text>
            <xsl:apply-templates select="." mode="submode_PERSONNE"/>
            <xsl:text>)</xsl:text>
        </xsl:if>
        <xsl:if test="string-length(organisme) &gt; 0"><xsl:text> (</xsl:text><xsl:value-of select="organisme"/><xsl:text>)</xsl:text></xsl:if>
    </xsl:template>

    <xsl:template match="personne">
        <xsl:apply-templates select="." mode="submode_PERSONNE"/>
        <xsl:if test="string-length(organisme) &gt; 0"><xsl:text> (</xsl:text><xsl:value-of select="organisme"/><xsl:text>)</xsl:text></xsl:if>
    </xsl:template>

    <xsl:template match="personne[nom/@avant = 'true']" mode="submode_PERSONNE">
        <xsl:value-of select="nom"/><xsl:if test="string-length(prenom) &gt; 0"><xsl:text> </xsl:text><xsl:value-of select="prenom"/></xsl:if>
    </xsl:template>

    <xsl:template match="personne[nom = false()]" mode="submode_PERSONNE">
        <xsl:value-of select="prenom"/>
    </xsl:template>

    <xsl:template match="personne" mode="submode_PERSONNE">
        <xsl:if test="string-length(prenom) &gt; 0"><xsl:value-of select="prenom"/><xsl:text> </xsl:text></xsl:if><xsl:value-of select="nom"/>
    </xsl:template>

    <xsl:template match="entite">
        <xsl:value-of select="nom"/>
        <xsl:if test="string-length(localisation) &gt; 0"><xsl:text>, </xsl:text><xsl:value-of select="localisation"/></xsl:if>
    </xsl:template>

    <xsl:template match="langue">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="pays">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="image">
        <xsl:value-of select="@base-url"/><xsl:value-of select="@src"/>
    </xsl:template>

    <xsl:template match="para">
        <xsl:call-template name="subtemplate_PRGCONTENT"/>
    </xsl:template>

    <xsl:template match="datation">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="montant">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="nombre">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="geopoint">
        <xsl:apply-templates select="latitude"/><xsl:text> </xsl:text><xsl:apply-templates select="longitude"/>
    </xsl:template>

    <xsl:template match="latitude">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="longitude">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>

    <xsl:template match="courriel">
        <text:a xlink:type="simple" xlink:href="mailto:{.}"><xsl:value-of select="."/></text:a>
    </xsl:template>

    <xsl:template match="link">
        <text:a xlink:type="simple" xlink:href="{@href}"><xsl:value-of select="title"/></text:a>
        <xsl:if test="string-length(comment) &gt; 0"><xsl:text> (</xsl:text><xsl:value-of select="comment"/>)</xsl:if>
    </xsl:template>

    <xsl:template name="subtemplate_LOCLIB">
        <xsl:variable name="var_LIB" select="string(lib[@xml:lang = $FORMATLOCALE])"/>
        <xsl:choose>
            <xsl:when test="string-length($var_LIB) &gt; 0"><xsl:value-of select="$var_LIB"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="lib[1]"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

</xsl:stylesheet>