<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

    <xsl:template match="document">
        <xsl:apply-templates select="." mode="MODE_DOCUMENTLINK"/>
    </xsl:template>


    <xsl:template match="extraitcorpus" mode="MODE_BLOCK">
        <xsl:param name="PARAM_CLASS" select=""/>
        <xsl:apply-templates select="fiche" mode="MODE_BLOCK">
            <xsl:with-param name="PARAM_CLASS" select="$PARAM_CLASS"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="extraitcorpus" mode="MODE_LISTITEM">
        <xsl:param name="PARAM_CLASS" select=""/>
        <xsl:apply-templates select="fiche" mode="MODE_LISTITEM">
            <xsl:with-param name="PARAM_CLASS" select="$PARAM_CLASS"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="extraitcorpus" mode="MODE_INLINE">
        <xsl:param name="PARAM_SEP" select="' ; '"/>
        <xsl:apply-templates select="fiche" mode="MODE_INLINE">
            <xsl:with-param name="PARAM_SEP" select="$PARAM_SEP"/>
        </xsl:apply-templates>
    </xsl:template>


    <xsl:template match="fiche" mode="MODE_BLOCK">
        <xsl:param name="PARAM_CLASS" select=""/>
        <p class="{$PARAM_CLASS}"><xsl:apply-templates select="." mode="MODE_FICHELINK"/></p>
    </xsl:template>

    <xsl:template match="fiche" mode="MODE_LISTITEM">
        <xsl:param name="PARAM_CLASS" select=""/>
        <li>
            <p class="{$PARAM_CLASS}"><xsl:apply-templates select="." mode="MODE_FICHELINK"/></p>
        </li>
    </xsl:template>

    <xsl:template match="fiche" mode="MODE_INLINE">
        <xsl:param name="PARAM_SEP" select="' ; '"/>
        <xsl:if test="position() &gt; 1"><xsl:value-of select="$PARAM_SEP"/></xsl:if>
        <xsl:apply-templates select="." mode="MODE_FICHELINK"/>
    </xsl:template>

    <xsl:template match="fiche" mode="MODE_FICHELINK">
        <a>
            <xsl:attribute name="href">
                <xsl:call-template name="TEMPLATE_FICHEURL">
                    <xsl:with-param name="PARAM_CORPUS" select="@corpus"/>
                    <xsl:with-param name="PARAM_IDCORPUS" select="@idcorpus"/>
                </xsl:call-template>
            </xsl:attribute>
            <xsl:apply-templates select="." mode="MODE_INTITULE"/>
        </a>
    </xsl:template>


    <xsl:template match="document" mode="MODE_BLOCK">
        <xsl:param name="PARAM_CLASS" select=""/>
        <xsl:param name="PARAM_VERSIONSEP" select="' / '"/>
        <p class="{$PARAM_CLASS}">
            <xsl:apply-templates select="." mode="MODE_DOCUMENTLINK">
                <xsl:with-param name="PARAM_VERSIONSEP" select="$PARAM_VERSIONSEP"/>
            </xsl:apply-templates>
        </p>
    </xsl:template>

    <xsl:template match="document" mode="MODE_LISTITEM">
        <xsl:param name="PARAM_CLASS" select=""/>
        <xsl:param name="PARAM_VERSIONSEP" select="' / '"/>
        <li>
            <p class="{$PARAM_CLASS}">
                <xsl:apply-templates select="." mode="MODE_DOCUMENTLINK">
                    <xsl:with-param name="PARAM_VERSIONSEP" select="$PARAM_VERSIONSEP"/>
                </xsl:apply-templates>
            </p>
        </li>
    </xsl:template>

    <xsl:template match="document" mode="MODE_INLINE">
        <xsl:param name="PARAM_SEP" select="' ; '"/>
        <xsl:param name="PARAM_VERSIONSEP" select="' / '"/>
        <xsl:if test="position() &gt; 1"><xsl:value-of select="$PARAM_SEP"/></xsl:if>
        <xsl:apply-templates select="." mode="MODE_DOCUMENTLINK">
            <xsl:with-param name="PARAM_VERSIONSEP" select="$PARAM_VERSIONSEP"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="document" mode="MODE_DOCUMENTLINK">
        <xsl:param name="PARAM_VERSIONSEP" select="' / '"/>
        <xsl:value-of select="@basename"/><xsl:text> [&#x00A0;</xsl:text>
        <xsl:apply-templates select="version" mode="MODE_VERSIONLINK">
            <xsl:with-param name="PARAM_VERSIONSEP" select="$PARAM_VERSIONSEP"/>
        </xsl:apply-templates>
        <xsl:text>&#x00A0;]</xsl:text>
    </xsl:template>

    <xsl:template match="version" mode="MODE_VERSIONLINK">
        <xsl:param name="PARAM_VERSIONSEP" select="' / '"/>
        <xsl:if test="position() &gt; 1"><xsl:value-of select="$PARAM_VERSIONSEP"/></xsl:if>
        <xsl:variable name="var_BASENAME" select="../@basename"/>
        <xsl:variable name="var_EXTENSION" select="@extension"/>
        <a>
            <xsl:attribute name="href">
                <xsl:call-template name="TEMPLATE_VERSIONURL">
                    <xsl:with-param name="PARAM_BASENAME" select="$var_BASENAME"/>
                    <xsl:with-param name="PARAM_EXTENSION" select="$var_EXTENSION"/>
                </xsl:call-template>
            </xsl:attribute>
            <xsl:text>.</xsl:text><xsl:value-of select="$var_EXTENSION"/>
        </a>
        <small><xsl:text> (</xsl:text><xsl:apply-templates select="." mode="MODE_VERSIONSIZE"/>)</small>
    </xsl:template>

    <xsl:template match="version" mode="MODE_VERSIONSIZE">
        <xsl:apply-templates select="size" mode="MODE_VERSIONSIZE"/>
    </xsl:template>

    <xsl:template match="size" mode="MODE_VERSIONSIZE">
        <xsl:call-template name="subtemplate_LOCLIB"/>
    </xsl:template>


    <xsl:template match="extraitalbum" mode="MODE_BLOCK">
        <xsl:param name="PARAM_CLASS" select=""/>
        <xsl:apply-templates select="illustration" mode="MODE_BLOCK">
            <xsl:with-param name="PARAM_CLASS" select="$PARAM_CLASS"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="extraitalbum" mode="MODE_LISTITEM">
        <xsl:param name="PARAM_CLASS" select=""/>
        <xsl:apply-templates select="illustration" mode="MODE_LISTITEM">
            <xsl:with-param name="PARAM_CLASS" select="$PARAM_CLASS"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="extraitalbum" mode="MODE_INLINE">
        <xsl:param name="PARAM_SEP" select="' &#x00A0; '"/>
        <xsl:apply-templates select="illustration" mode="MODE_INLINE">
            <xsl:with-param name="PARAM_SEP" select="$PARAM_SEP"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="illustration" mode="MODE_BLOCK">
        <xsl:param name="PARAM_CLASS" select=""/>
        <p class="{$PARAM_CLASS}"><xsl:apply-templates select="." mode="MODE_FICHELINK"/></p>
    </xsl:template>

    <xsl:template match="illustration" mode="MODE_LISTITEM">
        <xsl:param name="PARAM_CLASS" select=""/>
        <li>
            <p class="{$PARAM_CLASS}"><xsl:apply-templates select="." mode="MODE_FICHELINK"/></p>
        </li>
    </xsl:template>

    <xsl:template match="illustration" mode="MODE_INLINE">
        <xsl:param name="PARAM_SEP" select="' &#x00A0; '"/>
        <xsl:if test="position() &gt; 1"><xsl:value-of select="$PARAM_SEP"/></xsl:if>
        <xsl:apply-templates select="." mode="MODE_FICHELINK"/>
    </xsl:template>

    <xsl:template match="illustration" mode="MODE_FICHELINK">
        <a>
            <xsl:attribute name="href">
                <xsl:call-template name="TEMPLATE_ILLUSTRATIONURL">
                    <xsl:with-param name="PARAM_ALBUM" select="@album"/>
                    <xsl:with-param name="PARAM_IDALBUM" select="@idalbum"/>
                    <xsl:with-param name="PARAM_FORMAT" select="@format"/>
                    <xsl:with-param name="PARAM_DIM" select="''"/>
                </xsl:call-template>
            </xsl:attribute>
            <img>
                <xsl:attribute name="src">
                    <xsl:call-template name="TEMPLATE_ILLUSTRATIONURL">
                        <xsl:with-param name="PARAM_ALBUM" select="@album"/>
                        <xsl:with-param name="PARAM_IDALBUM" select="@idalbum"/>
                        <xsl:with-param name="PARAM_FORMAT" select="@format"/>
                        <xsl:with-param name="PARAM_DIM" select="'_mini'"/>
                    </xsl:call-template>
                </xsl:attribute>
            </img>
        </a>
    </xsl:template>

</xsl:stylesheet>