/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.roles;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.text.DefaultLibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class Role {

    private String name;
    private Map<SubsetKey, CorpusPermission> corpusPermissionMap = new HashMap<SubsetKey, CorpusPermission>();
    private RoleLibelleHolder roleLibelleHolder;

    public Role(String name) {
        this.name = name;
        this.roleLibelleHolder = new RoleLibelleHolder(name);
    }

    public CorpusPermission getCorpusPermission(SubsetKey corpusKey) {
        return corpusPermissionMap.get(corpusKey);
    }

    public void setCorpusPermission(SubsetKey corpusKey, CorpusPermission corpusPermission) {
        corpusPermissionMap.put(corpusKey, corpusPermission);
    }

    public void removeCorpusPermission(SubsetKey corpusKey) {
        corpusPermissionMap.remove(corpusKey);
    }

    public Map<SubsetKey, CorpusPermission> getCorpusPermissionMap() {
        return Collections.unmodifiableMap(corpusPermissionMap);
    }

    public String getName() {
        return name;
    }

    public LibelleHolder getRoleLibelleHolder() {
        return roleLibelleHolder;
    }

    public void putLibelle(Libelle libelle) {
        roleLibelleHolder.putLibelle(libelle);
    }

    public void removeLibelle(int langInteger) {
        roleLibelleHolder.removeLibelle(langInteger);
    }


    private class RoleLibelleHolder extends DefaultLibelleHolder {

        private RoleLibelleHolder(String name) {
            super(name);
        }

        private void putLibelle(Libelle libelle) {
            innerPutLibelle(libelle);
        }

        private void removeLibelle(int langInteger) {
            innerRemoveLibelle(langInteger);
        }

    }

}
