/* BdfServer - Copyright (c) 2008-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.roles.xml;

import fr.exemole.bdfserver.roles.CorpusPermission;
import fr.exemole.bdfserver.roles.FicheActionPermission;
import fr.exemole.bdfserver.roles.Role;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;
import java.io.IOException;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class RoleXMLWriter extends UtilXMLWriter {

    public RoleXMLWriter() {
    }

    public void appendRole(Role role) throws IOException {
        startOpenTag("role");
        endOpenTag();
        appendLibelleList(role.getRoleLibelleHolder());
        appendCorpusPermissionMap(role.getCorpusPermissionMap());

        closeTag("role");
    }

    private void appendLibelleList(LibelleHolder libelleHolder) throws IOException {
        openTag("lib-list");
        int libelleCount = libelleHolder.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = libelleHolder.getLibelleByIndex(i);
            addLibElement(libelle.getLangInteger(), libelle.getLibelleString());
        }
        closeTag("lib-list");
    }

    private void appendCorpusPermissionMap(Map<SubsetKey, CorpusPermission> map) throws IOException {
        for (Map.Entry<SubsetKey, CorpusPermission> entry : map.entrySet()) {
            SubsetKey subsetKey = entry.getKey();
            CorpusPermission corpusPermission = entry.getValue();
            appendCorpusPermission(subsetKey, corpusPermission);
        }
    }

    private void appendCorpusPermission(SubsetKey corpusKey, CorpusPermission corpusPermission) throws IOException {
        startOpenTag("corpus-permission");
        addAttribute("corpus", corpusKey.getSubsetName());
        endOpenTag();
        if (corpusPermission.canCreateFiche()) {
            addEmptyElement("create-fiche");
        }
        appendFicheActionPermission("read-fiche", corpusPermission.getReadFichePermission());
        appendFicheActionPermission("write-fiche", corpusPermission.getWriteFichePermission());
        appendFicheActionPermission("send-fiche", corpusPermission.getSendFichePermission());
        closeTag("corpus-permission");
    }

    private void appendFicheActionPermission(String tagname, FicheActionPermission ficheActionPermission) throws IOException {
        startOpenTag(tagname);
        addAttribute("type", FicheActionPermission.typeToString(ficheActionPermission.getType()));
        closeEmptyTag();
    }

}
