/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.runners;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageMode;
import fr.exemole.bdfserver.balayages.BalayageUtils;
import fr.exemole.bdfserver.balayages.engine.BalayageEngine;
import fr.exemole.bdfserver.balayages.engine.BalayageInitializer;
import fr.exemole.bdfserver.balayages.engine.BalayageLog;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import java.io.File;
import net.mapeadores.util.logging.CommandMessage;


/**
 *
 * @author Vincent Calame
 */
public class BalayageRunner {

    /**
     * Retourne soit une instance de CommandMessage en cas d'erreur soit l'instance de BalayageLog en argument.
     */
    public static Result run(Balayage balayage, String mode, BdfServer bdfServer) {
        Result result = new Result();
        BalayageInitializer balayageInitializer = new BalayageInitializer(bdfServer, balayage);
        if (balayageInitializer.hasError()) {
            result.commandMessage = CommandMessage.error(balayageInitializer.getErrorKey(), balayageInitializer.getErrorValue());
            return result;
        }
        if (mode != null) {
            BalayageMode balayageMode = BalayageUtils.getMode(balayage, mode);
            if (balayageMode != null) {
                balayageInitializer.setBalayageMode(balayageMode);
            }
        }
        String balayageName = balayage.getName();
        BalayageLog balayageLog = new BalayageLog(balayageName, getBalayageLogFile(bdfServer, balayageName, mode));
        result.balayageLog = balayageLog;
        BalayageEngine balayageEngine = balayageInitializer.getBalayageEngine(balayageLog);
        balayageEngine.run();
        balayageLog.close();
        return result;
    }

    private static File getBalayageLogFile(BdfServer bdfServer, String balayageName, String mode) {
        File logsDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.LOGS_OUTPUTDIRECTORY);
        if (mode == null) {
            mode = "";
        } else {
            mode = "_" + mode;
        }
        return new File(logsDirectory, "log_balayage_" + balayageName + mode + ".txt");
    }


    public static class Result {

        private CommandMessage commandMessage;
        private BalayageLog balayageLog;

        public boolean hasInitError() {
            return (commandMessage != null);
        }

        public CommandMessage getInitError() {
            return commandMessage;
        }

        public BalayageLog getBalayageLog() {
            return balayageLog;
        }

    }

}
