/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.runners;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.tools.scrutariexport.ScrutariExportEngine;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.xml.DefaultXMLWriter;
import net.mapeadores.util.xml.XMLUtils;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportRunner {

    public final static String INFO_SUFFIX = ".scrutari-info";
    public final static String DATA_SUFFIX = ".scrutari-data";
    private BdfServer bdfServer;
    private ScrutariExportDef scrutariExportDef;

    public ScrutariExportRunner(ScrutariExportDef scrutariExportDef, BdfServer bdfServer) {
        this.scrutariExportDef = scrutariExportDef;
        this.bdfServer = bdfServer;
    }

    public static void run(ScrutariExportDef scrutariExportDef, BdfServer bdfServer) throws IOException {
        ScrutariExportRunner runner = new ScrutariExportRunner(scrutariExportDef, bdfServer);
        runner.run();
    }

    private void run() throws IOException {
        writeHeader();
        writeBase();
    }

    private File getFile(String suffix) {
        File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PUBLIC_OUTPUTDIRECTORY);
        File scrutariDirectory = new File(outputDirectory, "scrutari");
        scrutariDirectory.mkdir();
        String fileName = scrutariExportDef.getName() + suffix + ".xml";
        File file = new File(scrutariDirectory, fileName);
        return file;
    }

    private void writeHeader() throws IOException {
        File file = getFile(INFO_SUFFIX);
        BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
        XMLUtils.writeXMLDeclaration(bufwriter);
        HeaderWriter headerWriter = new HeaderWriter(bufwriter);
        headerWriter.write();
        bufwriter.close();
    }

    private void writeBase() throws IOException {
        File file = getFile(DATA_SUFFIX);
        BufferedWriter bufwriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
        XMLUtils.writeXMLDeclaration(bufwriter);
        ScrutariExportEngine scrutariExportEngine = new ScrutariExportEngine(bdfServer.getBasedefiches(), bufwriter, -999, bdfServer.getLibelleManager().getFormatSourceLibelleProvider());
        scrutariExportEngine.addBase(scrutariExportDef, bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray());
        bufwriter.close();
    }


    private class HeaderWriter extends DefaultXMLWriter {

        HeaderWriter(Appendable appendable) {
            super();
            setAppendable(appendable);
            setIndentLength(-999);
        }

        void write() throws IOException {
            openTag("header");
            addSimpleElement("date", DateInteger.toString(DateInteger.current()));
            addSimpleElement("path", scrutariExportDef.getName() + DATA_SUFFIX + ".xml");
            closeTag("header");
        }

    }

}
