/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.runners;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.externalscript.ExternalScript;
import fr.exemole.bdfserver.managers.tableexport.TableExport;
import fr.exemole.bdfserver.sqlexport.BdfServerSqlExport;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import net.mapeadores.basedefiches.sqlexport.SqlExport;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.basedefiches.tableexport.SubsetTableExport;
import net.mapeadores.basedefiches.tableexport.croisement.CroisementTableExport;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ProcessLog;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportRunner {

    public static CommandMessage run(BdfServer bdfServer, SqlExportDef sqlExportDef) throws IOException {
        LocalisationInfo localisationInfo = null;
        String className = sqlExportDef.getSqlExportClassName();
        if (className.length() == 0) {
            className = "net.mapeadores.basedefiches.tools.sqlexport.DefaultSqlExport";
        }
        SqlExport sqlExport = null;
        try {
            Class classe = Class.forName(className);
            Object obj = classe.newInstance();
            sqlExport = (SqlExport) obj;
        } catch (ClassNotFoundException cnfe) {
            return CommandMessage.error("exportation_err_sqlexport_classnotfound", className);
        } catch (IllegalAccessException iae) {
            iae.printStackTrace();
            return CommandMessage.error("exportation_err_sqlexport_illegalaccess", className);
        } catch (InstantiationException ie) {
            ie.printStackTrace();
            return CommandMessage.error("exportation_err_sqlexport_instantiation", className);
        } catch (ClassCastException cce) {
            return CommandMessage.error("exportation_err_sqlexport_classcast", className);
        }
        String tableExportName = sqlExportDef.getTableExportName();
        if (tableExportName != null) {
            TableExport tableExport = bdfServer.getTableExportManager().getTableExport(tableExportName);
            if (tableExport != null) {
                SubsetTableExport[] subsetTableExportArray = tableExport.getSubsetTableExportArray(localisationInfo);
                sqlExport.setSubsetTableExportArray(subsetTableExportArray);
                CroisementTableExport[] croisementTableExport = tableExport.getCroisementTableExportArray();
                sqlExport.setCroisementTableExportArray(croisementTableExport);
            }
        }
        int paramCount = sqlExportDef.getParamCount();
        if (paramCount > 0) {
            for (int i = 0; i < paramCount; i++) {
                SqlExportDef.Param param = sqlExportDef.getParam(i);
                sqlExport.setParameter(param.getName(), param.getValue());
            }
        }
        String path = sqlExportDef.getPath();
        OutputStream outputStream = null;
        try {
            outputStream = new FileOutputStream(new File(path));
        } catch (IOException ioe) {
            return CommandMessage.error(new ResourceItem("exportation_err_sqlexport_path", path));
        }
        sqlExport.setBasedefiches(bdfServer.getBasedefiches());
        if (sqlExport instanceof BdfServerSqlExport) {
            ((BdfServerSqlExport) sqlExport).setBdfServer(bdfServer);
        }
        ProcessLog processLog = sqlExport.exportDump(outputStream);
        outputStream.close();
        String postCommand = sqlExportDef.getPostCommand();
        if (postCommand != null) {
            ExternalScript externalScript = bdfServer.getExternalScriptManager().getExternalScript(postCommand);
            if (externalScript != null) {
                try {
                    externalScript.exec();
                } catch (IOException ioe) {
                }
            }
        }
        return CommandMessage.done(new ResourceItem("exportation_done_sqlrun"), processLog);
    }

}
