/* BdfServer - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage;

import java.net.URL;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.docstream.ClassResourceDocStream;


/**
 *
 * @author Vincent Calame
 */
public class BdfResourceUtils {

    private BdfResourceUtils() {
    }

    public static boolean containsBdfResource(BdfResourceStorage bdfResourceStorage, short resourceType, String resourcePath, boolean withDefault) {
        if (bdfResourceStorage.containsBdfResource(resourceType, resourcePath)) {
            return true;
        }
        if (!withDefault) {
            return false;
        }
        Class referenceClass = bdfResourceStorage.getDefaultBdfResourceReferenceClass(resourceType);
        if (referenceClass == null) {
            return false;
        }
        resourcePath = computeBdfResource(resourceType, resourcePath);
        URL url = referenceClass.getResource(resourcePath);
        return (url != null);
    }

    public static DocStream getBdfResourceDocStream(BdfResourceStorage bdfResourceStorage, short resourceType, String resourcePath) {
        DocStream docStream = bdfResourceStorage.getBdfResourceDocStream(resourceType, resourcePath);
        if (docStream != null) {
            return docStream;
        }
        Class referenceClass = bdfResourceStorage.getDefaultBdfResourceReferenceClass(resourceType);
        if (referenceClass == null) {
            return null;
        }
        resourcePath = computeBdfResource(resourceType, resourcePath);
        ClassResourceDocStream classResourceDocStream = ClassResourceDocStream.newInstance(referenceClass, resourcePath);
        if (classResourceDocStream == null) {
            return null;
        }
        String mimeType = bdfResourceStorage.getMimeTypeResolver().getMimeTypeFromFileName(resourcePath);
        classResourceDocStream.setMimeType(mimeType);
        if (mimeType.startsWith("text/")) {
            classResourceDocStream.setCharset("UTF-8");
        }
        return classResourceDocStream;
    }

    private static String computeBdfResource(short resourceType, String resourcePath) {
        switch (resourceType) {
            case BdfResourceConstants.THEME_RESOURCE:
                if (resourcePath.endsWith(".css")) {
                    if (!resourcePath.endsWith("_1.css")) {
                        resourcePath = resourcePath.substring(0, resourcePath.length() - 4) + "_1.css";
                    }
                }
                return "resources/theme/" + resourcePath;
            case BdfResourceConstants.JQUERY_RESOURCE:
                return "resources/jquery/" + resourcePath;
            case BdfResourceConstants.JS_RESOURCE:
                return "resources/js/" + resourcePath;
            case BdfResourceConstants.CSS_RESOURCE:
                return "resources/css/" + resourcePath;
            case BdfResourceConstants.OTHER_RESOURCE:
                return "resources/rsc/" + resourcePath;
            default:
                throw new SwitchException("resourceType = " + resourceType);
        }
    }

}
