/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui;

import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.actions.ReadFiche;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusUtils;
import net.mapeadores.basedefiches.sphere.RedacteurKey;


/**
 *
 * @author Vincent Calame
 */
public class SubsetGroupUtils implements SubsetGroupConstants {

    private SubsetGroupUtils() {
    }

    public static SubsetGroupList checkCorpusGroupList(BdfUser bdfUser, SubsetGroupList subsetGroupList) {
        if ((bdfUser == null) || BdfUserUtils.isAdmin(bdfUser)) {
            return subsetGroupList;
        }
        CheckedSubsetGroupList checked = new CheckedSubsetGroupList(bdfUser, subsetGroupList);
        return checked;

    }

    public static SubsetGroupList checkNoSlaveSubset(Basedefiches basedefiches, SubsetGroupList subsetGroupList) {
        NoSlaveSubsetGroupList noSlaveSubsetGroupList = new NoSlaveSubsetGroupList();
        noSlaveSubsetGroupList.init(basedefiches, subsetGroupList);
        return noSlaveSubsetGroupList;
    }

    public static SubsetGroupList checkNoRattachementSubset(Corpus corpus, SubsetGroupList subsetGroupList) {
        Set<SubsetKey> rattachementSet = CorpusUtils.getRattachementSet(corpus);
        if (rattachementSet.size() == 0) {
            return subsetGroupList;
        }
        ExclusionSubsetGroupList noRattachementSubsetGroupList = new ExclusionSubsetGroupList();
        noRattachementSubsetGroupList.init(rattachementSet, subsetGroupList);
        return noRattachementSubsetGroupList;
    }

    public final static short toShortType(String typeString) {
        if (typeString.equals("grpcorpus")) {
            return CORPUS_GROUP;
        } else if (typeString.equals("grpthesaurus")) {
            return THESAURUS_GROUP;
        } else if (typeString.equals("grpsphere")) {
            return SPHERE_GROUP;
        } else if (typeString.equals("grpalbum")) {
            return ALBUM_GROUP;
        } else {
            throw new IllegalArgumentException("wrong typeString value : " + typeString);
        }
    }

    public final static String toStringType(short type) {
        switch (type) {
            case CORPUS_GROUP:
                return "grpcorpus";
            case THESAURUS_GROUP:
                return "grpthesaurus";
            case SPHERE_GROUP:
                return "grpsphere";
            case ALBUM_GROUP:
                return "grpalbum";
            default:
                throw new IllegalArgumentException("wrong type = " + type);
        }
    }


    private static class ExclusionSubsetGroupList implements SubsetGroupList {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();

        private ExclusionSubsetGroupList() {
        }

        public int getSubsetGroupCount() {
            return groupList.size();
        }

        public SubsetKeyGroup getSubsetGroup(int i) {
            return groupList.get(i);
        }

        private void init(Set<SubsetKey> exclusionSet, SubsetGroupList subsetGroupList) {
            int count = subsetGroupList.getSubsetGroupCount();
            for (int i = 0; i < count; i++) {
                SubsetKeyGroup newGroup = check(exclusionSet, subsetGroupList.getSubsetGroup(i));
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public SubsetKeyGroup check(Set<SubsetKey> exclusionSet, SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                if (!exclusionSet.contains(subsetKey)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }


    private static class NoSlaveSubsetGroupList implements SubsetGroupList {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();

        private NoSlaveSubsetGroupList() {
        }

        public int getSubsetGroupCount() {
            return groupList.size();
        }

        public SubsetKeyGroup getSubsetGroup(int i) {
            return groupList.get(i);
        }

        private void init(Basedefiches basedefiches, SubsetGroupList subsetGroupList) {
            int count = subsetGroupList.getSubsetGroupCount();
            for (int i = 0; i < count; i++) {
                SubsetKeyGroup newGroup = check(basedefiches, subsetGroupList.getSubsetGroup(i));
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public SubsetKeyGroup check(Basedefiches basedefiches, SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                Subset subset = basedefiches.getSubset(subsetKey);
                if ((subset != null) && (subset.getMasterSubset() == null)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }


    private static class CheckedSubsetGroupList implements SubsetGroupList {

        private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();
        private String[] roleNameArray;
        private PermissionManager permissionManager;
        private Basedefiches basedefiches;
        private BdfUser bdfUser;
        private RedacteurKey redacteurKey;

        CheckedSubsetGroupList(BdfUser bdfUser, SubsetGroupList subsetGroupList) {
            this.bdfUser = bdfUser;
            this.redacteurKey = bdfUser.getRedacteurKey();
            permissionManager = bdfUser.getBdfServer().getPermissionManager();
            basedefiches = bdfUser.getBdfServer().getBasedefiches();
            roleNameArray = bdfUser.getRoleNameArray();
            int count = subsetGroupList.getSubsetGroupCount();
            for (int i = 0; i < count; i++) {
                SubsetKeyGroup newGroup = check(subsetGroupList.getSubsetGroup(i));
                if (newGroup != null) {
                    groupList.add(newGroup);
                }
            }
        }

        public int getSubsetGroupCount() {
            return groupList.size();
        }

        public SubsetKeyGroup getSubsetGroup(int i) {
            return groupList.get(i);
        }

        public SubsetKeyGroup check(SubsetKeyGroup subsetKeyGroup) {
            List<SubsetKey> newList = new ArrayList<SubsetKey>();
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                ReadFiche readFiche = FicheAction.getReadFiche(corpus);
                if (!permissionManager.cannotDo(redacteurKey, roleNameArray, readFiche)) {
                    newList.add(subsetKey);
                }
            }
            if (newList.isEmpty()) {
                return null;
            }
            return subsetKeyGroup.copy(newList);
        }

    }

}
