/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class SubsetKeyGroup {

    private String templateName;
    private List<SubsetKey> subsetKeyList = new ArrayList<SubsetKey>();
    private List<SubsetKey> unmodifiableList = Collections.unmodifiableList(subsetKeyList);
    private LibelleHolder libelleHolder;

    public SubsetKeyGroup(String templateName) {
        this.templateName = templateName;
    }

    public SubsetKeyGroup copy(List<SubsetKey> newSubsetKeyList) {
        SubsetKeyGroup newGroup = new SubsetKeyGroup(this.templateName);
        newGroup.subsetKeyList.addAll(newSubsetKeyList);
        newGroup.libelleHolder = this.libelleHolder;
        return newGroup;
    }

    public boolean isNoTemplateGroup() {
        return (templateName == null);
    }

    public String getTemplateName() {
        return templateName;
    }

    public boolean equalsToTemplateName(String otherName) {
        if (otherName == null) {
            return (templateName == null);
        }
        if (templateName == null) {
            return false;
        }
        return (otherName.equals(templateName));
    }

    public String getLibelleString(int langInteger) {
        if (templateName == null) {
            return "<NULL>";
        }
        if (libelleHolder == null) {
            return "grp_" + templateName;
        }
        return libelleHolder.getLibelleString(langInteger, true);
    }

    void setLibelleHolder(LibelleHolder libelleHolder) {
        this.libelleHolder = libelleHolder;
    }

    void add(SubsetKey subsetKey) {
        subsetKeyList.add(subsetKey);
    }

    void remove(SubsetKey subsetKey) {
        subsetKeyList.remove(subsetKey);
    }

    void convertToNoTemplateGroup() {
        libelleHolder = null;
        templateName = null;
    }

    public List<SubsetKey> getSubsetKeyList() {
        return unmodifiableList;
    }

    public int getSubsetCount() {
        return subsetKeyList.size();
    }

}
