/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.text.DefaultLibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class SubsetKeyGroupTemplates {

    private Map<String, GroupLibelleHolder> groupLibelleHolderMap = new HashMap<String, GroupLibelleHolder>();
    private List<String> groupNameList = new ArrayList<String>();
    private List<SubsetKeyTree> subsetKeyTreeList = new ArrayList<SubsetKeyTree>();

    public SubsetKeyGroupTemplates() {
    }

    void addSubsetKeyTree(SubsetKeyTree subsetKeyTree) {
        subsetKeyTreeList.add(subsetKeyTree);
    }

    public void createTemplate(String groupName) {
        if (!groupLibelleHolderMap.containsKey(groupName)) {
            groupLibelleHolderMap.put(groupName, new GroupLibelleHolder(groupName));
            groupNameList.add(groupName);
        }
    }

    public boolean removeTemplate(String groupName) {
        boolean here = (groupLibelleHolderMap.remove(groupName) != null);
        groupNameList.remove(groupName);
        if (here) {
            for (int i = 0; i < subsetKeyTreeList.size(); i++) {
                SubsetKeyTree tree = (SubsetKeyTree) subsetKeyTreeList.get(i);
                tree.removeGroup(groupName);
            }
        }
        return here;
    }

    public boolean containsGroupTemplate(String groupName) {
        return groupLibelleHolderMap.containsKey(groupName);
    }

    public LibelleHolder getLibelleHolder(String groupName) {
        GroupLibelleHolder groupLibelleHolder = (GroupLibelleHolder) groupLibelleHolderMap.get(groupName);
        return groupLibelleHolder;
    }

    public void putLibelle(String groupName, Libelle libelle) {
        GroupLibelleHolder groupLibelleHolder = (GroupLibelleHolder) groupLibelleHolderMap.get(groupName);
        if (groupLibelleHolder != null) {
            groupLibelleHolder.putLibelle(libelle);
        }
    }

    public void removeLibelle(String groupName, int langInteger) {
        GroupLibelleHolder groupLibelleHolder = (GroupLibelleHolder) groupLibelleHolderMap.get(groupName);
        if (groupLibelleHolder != null) {
            groupLibelleHolder.removeLibelle(langInteger);
        }
    }

    /**
     * Retourne une chaîne vide si la langue n'est pas défini.
     */
    public String getLibelleString(String groupName, int langInteger) {
        LibelleHolder libelleHolder = (LibelleHolder) groupLibelleHolderMap.get(groupName);
        if (libelleHolder == null) {
            return "";
        }
        return libelleHolder.getLibelleString(langInteger, true);
    }

    public int size() {
        return groupNameList.size();
    }

    public String getGroupName(int i) {
        return (String) groupNameList.get(i);
    }


    private class GroupLibelleHolder extends DefaultLibelleHolder {

        private String groupName;

        private GroupLibelleHolder(String groupName) {
            super("?");
            this.groupName = groupName;
        }

        private void putLibelle(Libelle libelle) {
            innerPutLibelle(libelle);
        }

        private void removeLibelle(int langInteger) {
            innerRemoveLibelle(langInteger);
        }

    }

}
