/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class SubsetKeyTree implements SubsetGroupList, SubsetGroupConstants {

    private short type;
    private Map<SubsetKey, SubsetKeyGroup> subsetKeyMap = new HashMap<SubsetKey, SubsetKeyGroup>();
    private List<SubsetKeyGroup> groupList = new ArrayList<SubsetKeyGroup>();
    private SubsetKeyGroupTemplates subsetKeyGroupTemplates;

    public SubsetKeyTree(SubsetKeyGroupTemplates subsetKeyGroupTemplates, short type) {
        if ((type < 1) || (type > 4)) {
            throw new IllegalArgumentException("wrong type = " + type);
        }
        this.type = type;
        this.subsetKeyGroupTemplates = subsetKeyGroupTemplates;
        subsetKeyGroupTemplates.addSubsetKeyTree(this);
    }

    public int getSubsetGroupCount() {
        return groupList.size();
    }

    public SubsetKeyGroup getSubsetGroup(int i) {
        return groupList.get(i);
    }

    public void clear() {
        subsetKeyMap.clear();
        groupList.clear();
    }

    void removeGroup(String groupName) {
        for (int i = 0; i < groupList.size(); i++) {
            SubsetKeyGroup group = groupList.get(i);
            if ((!group.isNoTemplateGroup()) && (group.getTemplateName().equals(groupName))) {
                group.convertToNoTemplateGroup();
                break;
            }
        }
    }

    public void addSubsetKey(SubsetKey subsetKey, String groupName) {
        testSubsetKey(subsetKey);
        if (subsetKeyMap.containsKey(subsetKey)) {
            return;
        }
        if (groupName != null) {
            if (!subsetKeyGroupTemplates.containsGroupTemplate(groupName)) {
                groupName = null;
            }
        }
        SubsetKeyGroup currentSubsetKeyGroup = null;
        int size = groupList.size();
        if (size > 0) {
            SubsetKeyGroup last = groupList.get(size - 1);
            if (last.equalsToTemplateName(groupName)) {
                currentSubsetKeyGroup = last;
            }
        }
        if (currentSubsetKeyGroup == null) {
            if (groupName != null) {
                if (subsetKeyGroupTemplates.containsGroupTemplate(groupName)) {
                    currentSubsetKeyGroup = new SubsetKeyGroup(groupName);
                    currentSubsetKeyGroup.setLibelleHolder(subsetKeyGroupTemplates.getLibelleHolder(groupName));
                } else {
                    groupName = null;
                }
            }
            if (groupName == null) {
                currentSubsetKeyGroup = new SubsetKeyGroup(null);
            }
            groupList.add(currentSubsetKeyGroup);
        }
        currentSubsetKeyGroup.add(subsetKey);
        subsetKeyMap.put(subsetKey, currentSubsetKeyGroup);
    }

    public void removeSubsetKey(SubsetKey subsetKey) {
        SubsetKeyGroup subsetKeyGroup = subsetKeyMap.get(subsetKey);
        if (subsetKeyGroup == null) {
            return;
        }
        subsetKeyGroup.remove(subsetKey);
        subsetKeyMap.remove(subsetKey);
    }

    public void checkBasedefiches(Basedefiches basedefiches) {
        List<SubsetKey> forbiddenSubset = new ArrayList<SubsetKey>();
        for (SubsetKey subsetKey : subsetKeyMap.keySet()) {
            if (basedefiches.getSubset(subsetKey) == null) {
                forbiddenSubset.add(subsetKey);
            }
        }
        for (SubsetKey subsetKey : forbiddenSubset) {
            removeSubsetKey(subsetKey);
        }
        if (type == CORPUS_GROUP) {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                addSubsetKey(corpus.getSubsetKey(), null);
            }
        } else if (type == THESAURUS_GROUP) {
            int thesaurusCount = basedefiches.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = basedefiches.getThesaurus(i);
                addSubsetKey(thesaurus.getSubsetKey(), null);
            }
        } else if (type == SPHERE_GROUP) {
            int sphereCount = basedefiches.getSphereCount();
            for (int i = 0; i < sphereCount; i++) {
                Sphere sphere = basedefiches.getSphere(i);
                addSubsetKey(sphere.getSubsetKey(), null);
            }
        } else if (type == ALBUM_GROUP) {
            int albumCount = basedefiches.getAlbumCount();
            for (int i = 0; i < albumCount; i++) {
                Album album = basedefiches.getAlbum(i);
                addSubsetKey(album.getSubsetKey(), null);
            }
        }
    }

    public boolean isLegalSubsetKey(SubsetKey subsetKey) {
        try {
            testSubsetKey(subsetKey);
            return true;
        } catch (IllegalArgumentException iae) {
            return false;
        }
    }

    public SubsetKeyGroupTemplates getSubsetKeyGroupTemplates() {
        return subsetKeyGroupTemplates;
    }

    private void testSubsetKey(SubsetKey subsetKey) {
        if ((type == CORPUS_GROUP) && (!subsetKey.isCorpusSubset())) {
            throw new IllegalArgumentException("only corpus subsetKey allowed");
        }
        if ((type == THESAURUS_GROUP) && (!subsetKey.isThesaurusSubset())) {
            throw new IllegalArgumentException("only thesaurus subsetKey allowed");
        }
        if ((type == SPHERE_GROUP) && (!subsetKey.isSphereSubset())) {
            throw new IllegalArgumentException("only sphere subsetKey allowed");
        }
        if ((type == ALBUM_GROUP) && (!subsetKey.isAlbumSubset())) {
            throw new IllegalArgumentException("only album subsetKey allowed");
        }
    }

}
