/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIFilter;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.MutableInputOption;
import fr.exemole.bdfserver.ui.options.MutableLimitOption;
import fr.exemole.bdfserver.ui.options.MutableValueOption;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.CorpusList;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.extraction.api.IllustrationFilter;
import net.mapeadores.basedefiches.extraction.api.ExtraitAlbumDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitCorpusDef;
import net.mapeadores.basedefiches.extraction.api.ExtraitThesaurusDef;
import net.mapeadores.basedefiches.extraction.api.FicheFilter;
import net.mapeadores.basedefiches.extraction.api.MotcleFilter;
import net.mapeadores.basedefiches.extraction.builders.ExtraitAddendaDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.IllustrationFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitAlbumDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitCorpusDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.ExtraitThesaurusDefBuilder;
import net.mapeadores.basedefiches.extraction.builders.FicheFilterBuilder;
import net.mapeadores.basedefiches.extraction.builders.MotcleFilterBuilder;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitList;
import net.mapeadores.basedefiches.extraction.filterunit.FilterUnitListBuilder;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeSummary;
import net.mapeadores.basedefiches.include.IncludeSummaryBuilder;
import net.mapeadores.basedefiches.selection.IllustrationSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.primitives.RangeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUIUtils {

    private final static InputOption DEFAULT_PROPRIETE = new DefaultInputOption(InputOption.WIDTH_MEDIUM, 1, InputOption.INPUT_TEXT);
    private final static InputOption DEFAULT_SECTION = new DefaultInputOption(InputOption.WIDTH_LARGE, 8, InputOption.INPUT_TEXT);
    private final static InputOption DEFAULT_INFORMATION = new DefaultInputOption(InputOption.WIDTH_LARGE, 1, InputOption.INPUT_TEXT);
    private final static InputOption DEFAULT_TITRE = new DefaultInputOption(InputOption.WIDTH_LARGE, 1, InputOption.INPUT_TEXT);
    private final static InputOption DEFAULT_REDACTEURS = new DefaultInputOption(InputOption.WIDTH_SMALL, 1, InputOption.INPUT_TEXT);
    private final static InputOption DEFAULT_INDEXATION = new DefaultInputOption(InputOption.WIDTH_LARGE, 1, InputOption.INPUT_TEXT);
    private final static InputOption DEFAULT_LANG = new DefaultInputOption(InputOption.WIDTH_SMALL, 1, InputOption.INPUT_SELECT);
    public final static ComponentUIList EMPTY_COMPONENTUILIST = new EmptyComponentUIList();

    private CorpusUIUtils() {
    }

    public static MutableInputOption getDefaultInputOption(FieldKey fieldKey) {
        if (fieldKey.isProprieteFieldKey()) {
            return new MutableInputOption(DEFAULT_PROPRIETE);
        }
        if (fieldKey.isSectionFieldKey()) {
            return new MutableInputOption(DEFAULT_SECTION);
        }
        if (fieldKey.isInformationFieldKey()) {
            return new MutableInputOption(DEFAULT_INFORMATION);
        }
        if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
            return new MutableInputOption(DEFAULT_TITRE);
        }
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return new MutableInputOption(DEFAULT_REDACTEURS);
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return new MutableInputOption(DEFAULT_LANG);
        }
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            throw new IllegalArgumentException("No InputOption for FieldKey.IDCORPUS");
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

    /* Peut-être nul */
    public static MutableValueOption getDefaultValueOption(FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return null;
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return null;
        }
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            return null;
        }
        return new MutableValueOption();
    }

    public static boolean equalsToDefault(FieldKey fieldKey, InputOption inputOption) {
        if (fieldKey.isProprieteFieldKey()) {
            return inputOptionEquality(inputOption, DEFAULT_PROPRIETE);
        }
        if (fieldKey.isSectionFieldKey()) {
            return inputOptionEquality(inputOption, DEFAULT_SECTION);
        }
        if (fieldKey.isInformationFieldKey()) {
            return inputOptionEquality(inputOption, DEFAULT_INFORMATION);
        }
        if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
            return inputOptionEquality(inputOption, DEFAULT_TITRE);
        }
        if (fieldKey.equals(FieldKey.REDACTEURS)) {
            return inputOptionEquality(inputOption, DEFAULT_REDACTEURS);
        }
        if (fieldKey.equals(FieldKey.LANG)) {
            return inputOptionEquality(inputOption, DEFAULT_LANG);
        }
        if (fieldKey.equals(FieldKey.IDCORPUS)) {
            throw new IllegalArgumentException("No htmlInputBean for FieldKey.IDCORPUS");
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

    public static MutableInputOption getDefaultIndexationInputOption() {
        return new MutableInputOption(DEFAULT_INDEXATION);
    }

    public static MutableLimitOption getDefaultSpecialLiageLimitOption() {
        MutableLimitOption mutableLimitOption = new MutableLimitOption();
        return mutableLimitOption;
    }

    public static MutableLimitOption getDefaultSpecialAssociationLimitOption() {
        MutableLimitOption mutableLimitOption = new MutableLimitOption();
        mutableLimitOption.setNoNewAllowed(true);
        return mutableLimitOption;
    }

    public static boolean inputOptionEquality(InputOption inputOption1, InputOption inputOption2) {
        if (inputOption1.getInputType() != inputOption2.getInputType()) {
            return false;
        }
        if (inputOption1.getRows() != inputOption2.getRows()) {
            return false;
        }
        if (inputOption1.getWidthType() != inputOption2.getWidthType()) {
            return false;
        }
        return true;
    }

    public static boolean equalstoDefaultIndexation(InputOption inputOption) {
        return inputOptionEquality(inputOption, DEFAULT_INDEXATION);
    }

    public static boolean isCommentUIKey(String key) {
        return (key.startsWith("comment_"));
    }

    public static boolean isIndexationUIKey(String key) {
        return (key.startsWith("thesaurus_"));
    }

    public static boolean isLiageUIKey(String key) {
        return (key.startsWith("corpus_"));
    }

    public static FieldKey isFieldKey(String key) {
        try {
            FieldKey fieldKey = FieldKey.parse(key);
            return fieldKey;
        } catch (ParseException pe) {
            return null;
        }
    }

    public static IncludeSummary getIncludeSummary(ComponentUIList componentUIList) {
        IncludeSummaryBuilder builder = new IncludeSummaryBuilder();
        int count = componentUIList.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof IncludeUI) {
                builder.append(((IncludeUI) componentUI).getIncludeKey());
            }
        }
        return builder.getIncludeSummary();
    }

    public static FilterUnitList toFilterUnitList(Corpus corpus, ComponentUIList componentUIList, boolean addChrono, boolean addIntituleFiche) {
        FicheFilter liageFicheFilter = FicheFilterBuilder.TITRE_FICHEFILTER;
        MotcleFilter indexationMotcleFilter = MotcleFilterBuilder.INDEXATION_MOTCLEFILTER;
        IllustrationFilter ficheIllustrationFilter = IllustrationFilterBuilder.FICHE_ILLUSTRATIONFILTER;
        IncludeSummary includeSummary = getIncludeSummary(componentUIList);
        FilterUnitListBuilder builder = new FilterUnitListBuilder();
        int count = componentUIList.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof FieldUI) {
                builder.addFieldKey(((FieldUI) componentUI).getCorpusField().getFieldKey(), null);
            } else if (componentUI instanceof IncludeUI) {
                IncludeKey includeKey = ((IncludeUI) componentUI).getIncludeKey();
                if (includeKey.isChronoInclude()) {
                    builder.addChronoFilter();
                } else if (includeKey.isCorpusInclude()) {
                    if (includeKey.hasPoidsFilter()) {
                        builder.addExtraitCorpusFilter(getPoidsExtraitCorpusDef(includeKey, liageFicheFilter));
                    } else {
                        builder.addExtraitCorpusFilter(getNoPoidsExtraitCorpusDef(includeKey.getSubsetKey(), includeSummary, liageFicheFilter));
                    }

                } else if (includeKey.isThesaurusInclude()) {
                    if (includeKey.hasPoidsFilter()) {
                        builder.addExtraitThesaurusFilter(getPoidsExtraitThesaurusDef(includeKey, indexationMotcleFilter));
                    } else {
                        builder.addExtraitThesaurusFilter(getNoPoidsExtraitThesaurusDef(includeKey.getSubsetKey(), includeSummary, indexationMotcleFilter));
                    }
                } else if (includeKey.isAlbumInclude()) {
                    if (includeKey.hasPoidsFilter()) {
                        builder.addExtraitAlbumFilter(getPoidsExtraitAlbumDef(includeKey, ficheIllustrationFilter));
                    } else {
                        builder.addExtraitAlbumFilter(getNoPoidsExtraitAlbumDef(includeKey.getSubsetKey(), includeSummary, ficheIllustrationFilter));
                    }
                } else if (includeKey.isSpecialInclude()) {
                    if (includeKey.equals(IncludeKey.ASSOCIATION)) {
                        builder.addExtraitAddendaFilter(ExtraitAddendaDefBuilder.ASSOCIATION_EXTRAITADDENDADEF);
                    } else if (includeKey.equals(IncludeKey.LIAGE)) {
                        ExtraitCorpusDef liageExtraitCorpusDef = getLiageExtraitCorpusDef(corpus, includeSummary, liageFicheFilter);
                        if (liageExtraitCorpusDef != null) {
                            builder.addExtraitCorpusFilter(liageExtraitCorpusDef);
                        }
                    } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                        Subset masterSubset = corpus.getMasterSubset();
                        if (masterSubset != null) {
                            if (masterSubset instanceof Thesaurus) {
                                builder.addExtraitThesaurusFilter(getRattachementExtraitThesaurusDef((Thesaurus) masterSubset, indexationMotcleFilter));
                            } else if (masterSubset instanceof Corpus) {
                                builder.addExtraitCorpusFilter(getRattachementExtraitCorpusDef((Corpus) masterSubset, liageFicheFilter));
                            }
                        }
                    }
                }
            }
        }
        if (addChrono) {
            builder.addChronoFilter();
        }
        if (addIntituleFiche) {
            builder.addIntituleFicheFilter();
        }
        return builder.toFilterUnitList();
    }

    public static List<IncludeUI> getCorpusIncludeUIList(ComponentUIList componentUIList) {
        int count = componentUIList.getComponentUICount();
        if (count == 0) {
            return Collections.emptyList();
        }
        List<IncludeUI> includeUIList = new ArrayList<IncludeUI>();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                SubsetKey subsetKey = includeUI.getIncludeKey().getSubsetKey();
                if ((subsetKey != null) && (subsetKey.isCorpusSubset())) {
                    includeUIList.add(includeUI);
                }
            }
        }
        return includeUIList;
    }

    public static List<IncludeUI> getThesaurusIncludeUIList(ComponentUIList componentUIList) {
        int count = componentUIList.getComponentUICount();
        if (count == 0) {
            return Collections.emptyList();
        }
        List<IncludeUI> includeUIList = new ArrayList<IncludeUI>();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                SubsetKey subsetKey = includeUI.getIncludeKey().getSubsetKey();
                if ((subsetKey != null) && (subsetKey.isThesaurusSubset())) {
                    includeUIList.add(includeUI);
                }
            }
        }
        return includeUIList;
    }

    public static List<IncludeUI> getAlbumIncludeUIList(ComponentUIList componentUIList) {
        int count = componentUIList.getComponentUICount();
        if (count == 0) {
            return Collections.emptyList();
        }
        List<IncludeUI> includeUIList = new ArrayList<IncludeUI>();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                SubsetKey subsetKey = includeUI.getIncludeKey().getSubsetKey();
                if ((subsetKey != null) && (subsetKey.isAlbumSubset())) {
                    includeUIList.add(includeUI);
                }
            }
        }
        return includeUIList;
    }

    public static List<CommentUI> getCommentUIList(ComponentUIList componentUIList) {
        int count = componentUIList.getComponentUICount();
        if (count == 0) {
            return Collections.emptyList();
        }
        List<CommentUI> commentUIList = new ArrayList<CommentUI>();
        Set<String> doneSet = new HashSet<String>();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof CommentUI) {
                CommentUI commentUI = (CommentUI) componentUI;
                if (!doneSet.contains(commentUI.getKeyString())) {
                    doneSet.add(commentUI.getKeyString());
                    commentUIList.add(commentUI);
                }
            }
        }
        return commentUIList;
    }

    public static ComponentUIList filter(ComponentUIList componentUIList, ComponentUIFilter componentUIFilter) {
        if (componentUIFilter == null) {
            return componentUIList;
        }
        List<ComponentUI> result = new ArrayList<ComponentUI>();
        int count = componentUIList.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            componentUI = componentUIFilter.filter(componentUI);
            if (componentUI != null) {
                result.add(componentUI);
            }
        }
        int size = result.size();
        if (size == 0) {
            return EMPTY_COMPONENTUILIST;
        }
        return new ArrayComponentUIList(result.toArray(new ComponentUI[size]));
    }

    private static ExtraitCorpusDef getPoidsExtraitCorpusDef(IncludeKey corpusIncludeKey, FicheFilter ficheFilter) {
        SubsetKey[] corpusArray = new SubsetKey[1];
        corpusArray[0] = corpusIncludeKey.getSubsetKey();
        int poids = corpusIncludeKey.getPoidsFilter();
        FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
        ficheSelectOptions.setCorpusKeyArray(corpusArray);
        ficheSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poids));
        ExtraitCorpusDefBuilder builder = new ExtraitCorpusDefBuilder();
        builder.setFicheFilter(ficheFilter);
        builder.setExtraitName(corpusIncludeKey.toString());
        builder.setFicheSelectOptionsList(ficheSelectOptions);
        return builder.toExtraitCorpusDef();
    }

    private static ExtraitCorpusDef getNoPoidsExtraitCorpusDef(SubsetKey corpusKey, IncludeSummary includeSummary, FicheFilter ficheFilter) {
        SubsetKey[] corpusArray = new SubsetKey[1];
        corpusArray[0] = corpusKey;
        RangeList poidsRangeList = null;
        IncludeSummary.SubsetInfo includeKeyInfo = includeSummary.getSubsetInfo(corpusKey);
        if (includeKeyInfo != null) {
            poidsRangeList = includeKeyInfo.getPoidsFilterRangeList();
        }
        FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
        ficheSelectOptions.setCorpusKeyArray(corpusArray);
        if (poidsRangeList != null) {
            ficheSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poidsRangeList, true));
        }
        ExtraitCorpusDefBuilder builder = new ExtraitCorpusDefBuilder();
        builder.setFicheFilter(ficheFilter);
        builder.setExtraitName(corpusKey.toString());
        builder.setFicheSelectOptionsList(ficheSelectOptions);
        return builder.toExtraitCorpusDef();
    }

    private static ExtraitCorpusDef getLiageExtraitCorpusDef(Corpus referenceCorpus, IncludeSummary summary, FicheFilter ficheFilter) {
        SubsetKey masterSubsetKey = null;
        Subset masterSubset = referenceCorpus.getMasterSubset();
        if ((masterSubset != null) && (masterSubset instanceof Corpus)) {
            masterSubsetKey = masterSubset.getSubsetKey();
        }
        CorpusList corpusList = referenceCorpus.getBasedefiches();
        int corpusCount = corpusList.getCorpusCount();
        ExtraitCorpusDefBuilder builder = new ExtraitCorpusDefBuilder();
        builder.setExtraitName(IncludeKey.LIAGE.toString());
        builder.setFicheFilter(ficheFilter);
        List<SubsetKey> allCorpusList = new ArrayList<SubsetKey>();
        List<FicheSelectOptions> ficheSelectOptionsList = new ArrayList<FicheSelectOptions>();
        for (int i = 0; i < corpusCount; i++) {
            SubsetKey otherKey = corpusList.getCorpus(i).getSubsetKey();
            if (otherKey.equals(masterSubsetKey)) {
                continue;
            }
            IncludeSummary.SubsetInfo info = summary.getSubsetInfo(otherKey);
            if (info == null) {
                allCorpusList.add(otherKey);
            } else if (info.hasAnyPoidsInclude()) {
                continue;
            } else {
                RangeList poidsRangeList = info.getPoidsFilterRangeList();
                FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
                SubsetKey[] corpusArray = new SubsetKey[1];
                corpusArray[0] = otherKey;
                ficheSelectOptions.setCorpusKeyArray(corpusArray);
                ficheSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poidsRangeList, true));
                ficheSelectOptionsList.add(ficheSelectOptions);
            }
        }
        int allCount = allCorpusList.size();
        if (allCount > 0) {
            SubsetKey[] corpusArray = new SubsetKey[allCount];
            corpusArray = allCorpusList.toArray(corpusArray);
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            ficheSelectOptions.setCorpusKeyArray(corpusArray);
            ficheSelectOptionsList.add(ficheSelectOptions);
        }
        if (ficheSelectOptionsList.isEmpty()) {
            return null;
        }
        builder.setFicheSelectOptionsList(ficheSelectOptionsList);
        return builder.toExtraitCorpusDef();
    }

    private static ExtraitCorpusDef getRattachementExtraitCorpusDef(Corpus corpus, FicheFilter ficheFilter) {
        SubsetKey[] corpusArray = new SubsetKey[1];
        corpusArray[0] = corpus.getSubsetKey();
        FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
        ficheSelectOptions.setCorpusKeyArray(corpusArray);
        ExtraitCorpusDefBuilder builder = new ExtraitCorpusDefBuilder();
        builder.setFicheFilter(ficheFilter);
        builder.setExtraitName(IncludeKey.RATTACHEMENT.toString());
        builder.setFicheSelectOptionsList(ficheSelectOptions);
        return builder.toExtraitCorpusDef();
    }

    private static ExtraitThesaurusDef getPoidsExtraitThesaurusDef(IncludeKey thesaurusIncludeKey, MotcleFilter motcleFilter) {
        SubsetKey[] thesaurusArray = new SubsetKey[1];
        thesaurusArray[0] = thesaurusIncludeKey.getSubsetKey();
        int poids = thesaurusIncludeKey.getPoidsFilter();
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        motcleSelectOptions.setThesaurusKeyArray(thesaurusArray);
        motcleSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poids));
        ExtraitThesaurusDefBuilder builder = new ExtraitThesaurusDefBuilder();
        builder.setMotcleFilter(motcleFilter);
        builder.setExtraitName(thesaurusIncludeKey.toString());
        builder.setMotcleSelectOptionsList(motcleSelectOptions);
        return builder.toExtraitThesaurusDef();
    }

    private static ExtraitThesaurusDef getNoPoidsExtraitThesaurusDef(SubsetKey thesaurusKey, IncludeSummary includeSummary, MotcleFilter motcleFilter) {
        SubsetKey[] thesaurusArray = new SubsetKey[1];
        thesaurusArray[0] = thesaurusKey;
        RangeList poidsRangeList = null;
        IncludeSummary.SubsetInfo includeKeyInfo = includeSummary.getSubsetInfo(thesaurusKey);
        if (includeKeyInfo != null) {
            poidsRangeList = includeKeyInfo.getPoidsFilterRangeList();
        }
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        motcleSelectOptions.setThesaurusKeyArray(thesaurusArray);
        if (poidsRangeList != null) {
            motcleSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poidsRangeList, true));
        }
        ExtraitThesaurusDefBuilder builder = new ExtraitThesaurusDefBuilder();
        builder.setMotcleFilter(motcleFilter);
        builder.setExtraitName(thesaurusKey.toString());
        builder.setMotcleSelectOptionsList(motcleSelectOptions);
        return builder.toExtraitThesaurusDef();
    }

    private static ExtraitThesaurusDef getRattachementExtraitThesaurusDef(Thesaurus thesaurus, MotcleFilter motcleFilter) {
        SubsetKey[] thesaurusArray = new SubsetKey[1];
        thesaurusArray[0] = thesaurus.getSubsetKey();
        MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
        motcleSelectOptions.setThesaurusKeyArray(thesaurusArray);
        ExtraitThesaurusDefBuilder builder = new ExtraitThesaurusDefBuilder();
        builder.setMotcleFilter(motcleFilter);
        builder.setExtraitName(IncludeKey.RATTACHEMENT.toString());
        builder.setMotcleSelectOptionsList(motcleSelectOptions);
        return builder.toExtraitThesaurusDef();
    }

    private static ExtraitAlbumDef getPoidsExtraitAlbumDef(IncludeKey albumIncludeKey,IllustrationFilter illustrationFilter) {
        SubsetKey[] albumArray = new SubsetKey[1];
        albumArray[0] = albumIncludeKey.getSubsetKey();
        int poids = albumIncludeKey.getPoidsFilter();
        IllustrationSelectOptions illustrationSelectOptions = new IllustrationSelectOptions();
        illustrationSelectOptions.setAlbumKeyArray(albumArray);
        illustrationSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poids));
        ExtraitAlbumDefBuilder builder = new ExtraitAlbumDefBuilder();
        builder.setIllustrationFilter(illustrationFilter);
        builder.setExtraitName(albumIncludeKey.toString());
        builder.setIllustrationSelectOptionsList(illustrationSelectOptions);
        return builder.toExtraitAlbumDef();
    }

    private static ExtraitAlbumDef getNoPoidsExtraitAlbumDef(SubsetKey albumKey, IncludeSummary includeSummary, IllustrationFilter illustrationFilter) {
        SubsetKey[] albumArray = new SubsetKey[1];
        albumArray[0] = albumKey;
        RangeList poidsRangeList = null;
        IncludeSummary.SubsetInfo includeKeyInfo = includeSummary.getSubsetInfo(albumKey);
        if (includeKeyInfo != null) {
            poidsRangeList = includeKeyInfo.getPoidsFilterRangeList();
        }
        IllustrationSelectOptions illustrationSelectOptions = new IllustrationSelectOptions();
        illustrationSelectOptions.setAlbumKeyArray(albumArray);
        if (poidsRangeList != null) {
            illustrationSelectOptions.setPoidsCondition(SelectionUtils.newPoidsCondition(poidsRangeList, true));
        }
        ExtraitAlbumDefBuilder builder = new ExtraitAlbumDefBuilder();
        builder.setIllustrationFilter(illustrationFilter);
        builder.setExtraitName(albumKey.toString());
        builder.setIllustrationSelectOptionsList(illustrationSelectOptions);
        return builder.toExtraitAlbumDef();
    }


    private static class EmptyComponentUIList implements ComponentUIList {

        private EmptyComponentUIList() {
        }

        public int getComponentUICount() {
            return 0;
        }

        public ComponentUI getComponentUI(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayComponentUIList implements ComponentUIList {

        private ComponentUI[] componentUIArray;

        private ArrayComponentUIList(ComponentUI[] componentUIArray) {
            this.componentUIArray = componentUIArray;
        }

        public int getComponentUICount() {
            return componentUIArray.length;
        }

        public ComponentUI getComponentUI(int i) {
            return componentUIArray[i];
        }

    }

}
