/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCommentUI implements CommentUI {

    private LinkedIntHashMap htmlStringMap = new LinkedIntHashMap();
    private String name;
    private int location = ALL_MASK;

    public DefaultCommentUI(String name) {
        this.name = name;
    }

    public DefaultCommentUI(CommentUI commentUI) {
        this.name = commentUI.getName();
        this.location = commentUI.getLocation();
        int libelleCount = commentUI.getHtmlStringCount();
        for (int i = 0; i < libelleCount; i++) {
            int langInteger = commentUI.getHtmlLangInteger(i);
            String htmlString = commentUI.getHtmlString(i);
            innerPutHtmlString(langInteger, htmlString);
        }
    }

    public int getLocation() {
        return location;
    }

    public String getName() {
        return name;
    }

    public String getKeyString() {
        return "comment_" + name;
    }

    public int getHtmlStringCount() {
        return htmlStringMap.size();
    }

    public String getHtmlString(int i) {
        return (String) htmlStringMap.getValueAt(i);
    }

    public int getHtmlLangInteger(int i) {
        return htmlStringMap.getKeyAt(i);
    }

    public String getHtmlStringByLang(int langInteger) {
        return (String) htmlStringMap.get(langInteger);
    }

    protected boolean innerSetLocation(int location) {
        if ((location < 1) || (location > ALL_MASK)) {
            location = ALL_MASK;
        }
        if (this.location == location) {
            return false;
        }
        this.location = location;
        return true;
    }

    protected boolean innerPutHtmlString(int langInteger, String htmlString) {
        boolean keyHere = htmlStringMap.containsKey(langInteger);
        if ((htmlString == null) || (htmlString.length() == 0)) {
            htmlStringMap.remove(langInteger);
            return keyHere;
        } else {
            Object old = htmlStringMap.put(langInteger, htmlString);
            if (keyHere) {
                return (!htmlString.equals(old));
            } else {
                return true;
            }
        }
    }

    public static String maskToString(int position) {
        MaskBuffer maskBuffer = new MaskBuffer(position);
        maskBuffer.append(FORM_BITVALUE, "form");
        maskBuffer.append(AFFICHAGE_BITVALUE, "affichage");
        return maskBuffer.toString();
    }

    public static int maskStringToInt(String s) {
        int location = 0;
        String[] array = StringUtils.getTokens(s, '+', StringUtils.EMPTY_EXCLUDE);
        for (int i = 0; i < array.length; i++) {
            String token = array[i];
            if (token.equals("form")) {
                location = location | FORM_BITVALUE;
            } else if (token.equals("affichage")) {
                location = location | AFFICHAGE_BITVALUE;
            }
        }
        if (location == 0) {
            location = ALL_MASK;
        }
        return location;
    }


    private static class MaskBuffer {

        private StringBuffer buf = new StringBuffer();
        private int location;
        private boolean premier = true;

        private MaskBuffer(int location) {
            this.location = location;
        }

        private void append(int mask, String maskString) {
            if ((location & mask) != 0) {
                if (premier) {
                    premier = false;
                } else {
                    buf.append("+");
                }
                buf.append(maskString);
            }
        }

        @Override
        public String toString() {
            return buf.toString();
        }

    }

}
