/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.DefaultValueOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;


/**
 *
 * @author Vincent Calame
 */
public class DefaultFieldUI implements FieldUI {

    private CorpusField corpusField;
    private InternalInputOption internalInputOption;
    private InternalValueOption internalValueOption;

    public DefaultFieldUI(CorpusField corpusField) {
        this.internalInputOption = new InternalInputOption(CorpusUIUtils.getDefaultInputOption(corpusField.getFieldKey()));
        this.corpusField = corpusField;
        ValueOption defValueOption = CorpusUIUtils.getDefaultValueOption(corpusField.getFieldKey());
        if (defValueOption == null) {
            this.internalValueOption = null;
        } else {
            this.internalValueOption = new InternalValueOption(defValueOption);
        }
    }

    public DefaultFieldUI(FieldUI fieldUI) {
        this.corpusField = fieldUI.getCorpusField();
        this.internalInputOption = new InternalInputOption(fieldUI.getInputOption());
        ValueOption valueOption = fieldUI.getValueOption();
        if (valueOption != null) {
            this.internalValueOption = new InternalValueOption();
        } else {
            this.internalValueOption = null;
        }
    }

    public CorpusField getCorpusField() {
        return corpusField;
    }

    public InputOption getInputOption() {
        return internalInputOption;
    }

    public ValueOption getValueOption() {
        return internalValueOption;
    }

    public String getKeyString() {
        return corpusField.getFieldKey().toString();
    }

    protected void innerUpdateInputOption(InputOption inputOption) {
        internalInputOption.update(inputOption);
    }

    protected void innerUpdateValueOption(ValueOption valueOption) {
        if (internalValueOption != null) {
            internalValueOption.update(valueOption);
        }
    }


    private class InternalInputOption extends DefaultInputOption {

        InternalInputOption(InputOption inputOption) {
            super(inputOption);
        }

        private void update(InputOption inputOption) {
            innerUpdate(inputOption);
        }

    }


    private class InternalValueOption extends DefaultValueOption {

        InternalValueOption() {
            innerSetMandatory(false);
            innerSetDefaultValue("");
        }

        InternalValueOption(ValueOption valueOption) {
            super(valueOption);
        }

        private void update(ValueOption valueOption) {
            innerUpdate(valueOption);
        }

    }

}
