/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.corpus;

import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.DefaultLimitOption;
import fr.exemole.bdfserver.ui.options.DefaultValueOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.text.DefaultLibelleHolder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class DefaultIncludeUI implements IncludeUI {

    private IncludeKey includeKey;
    private boolean defaultLibelle = true;
    private boolean withIdalpha = true;
    private CustomLibelleHolder customLibelleHolder;
    private InternalInputOption internalInputOption;
    private InternalLimitOption internalLimitOption;
    private InternalValueOption internalValueOption;
    private String motcleSelectOptionsString;

    public DefaultIncludeUI(IncludeKey includeKey) {
        this.includeKey = includeKey;
        init();
    }

    private void init() {
        if (includeKey.isThesaurusInclude()) {
            internalInputOption = new InternalInputOption(CorpusUIUtils.getDefaultIndexationInputOption());
        }
        if (includeKey.equals(IncludeKey.LIAGE)) {
            internalLimitOption = new InternalLimitOption(CorpusUIUtils.getDefaultSpecialLiageLimitOption());
        }
        if (includeKey.equals(IncludeKey.ASSOCIATION)) {
            internalLimitOption = new InternalLimitOption(CorpusUIUtils.getDefaultSpecialAssociationLimitOption());
        }
        this.internalValueOption = new InternalValueOption();
    }

    public DefaultIncludeUI(IncludeUI includeUI) {
        this.includeKey = includeUI.getIncludeKey();
        this.internalValueOption = new InternalValueOption(includeUI.getValueOption());
        LimitOption limitOption = includeUI.getLimitOption();
        if (limitOption != null) {
            this.internalLimitOption = new InternalLimitOption(limitOption);
        } else {
            this.internalLimitOption = null;
        }
        InputOption inputOption = includeUI.getInputOption();
        if (inputOption != null) {
            this.internalInputOption = new InternalInputOption(inputOption);
        } else {
            this.internalInputOption = null;
        }
        this.motcleSelectOptionsString = includeUI.getMotcleSelectOptionsString();
        boolean otherUsage = includeUI.useDefaultLibelle();
        this.defaultLibelle = otherUsage;
        if (!otherUsage) {
            customLibelleHolder = new CustomLibelleHolder();
            LibelleHolder holder = includeUI.getCustomLibelle();
            int libelleCount = holder.getLibelleCount();
            for (int i = 0; i < libelleCount; i++) {
                Libelle libelle = holder.getLibelleByIndex(i);
                customLibelleHolder.putLibelle(libelle);
            }
        }
    }

    public InputOption getInputOption() {
        return internalInputOption;
    }

    public LimitOption getLimitOption() {
        return internalLimitOption;
    }

    public ValueOption getValueOption() {
        return internalValueOption;
    }

    public String getMotcleSelectOptionsString() {
        return motcleSelectOptionsString;
    }

    public String getKeyString() {
        return includeKey.toString();
    }

    public LibelleHolder getCustomLibelle() {
        return customLibelleHolder;
    }

    public IncludeKey getIncludeKey() {
        return includeKey;
    }

    public boolean useDefaultLibelle() {
        return defaultLibelle;
    }

    public boolean prefixWithIdalpha() {
        return withIdalpha;
    }

    protected void innerUpdateInputOption(InputOption inputOption) {
        if (internalInputOption != null) {
            internalInputOption.update(inputOption);
        }
    }

    protected void innerUpdateLimitOption(LimitOption limitOption) {
        if (internalLimitOption != null) {
            internalLimitOption.update(limitOption);
        }
    }

    protected void innerSetMotcleSelectOptionsString(String motcleSelectOptionsString) {
        this.motcleSelectOptionsString = motcleSelectOptionsString;
    }

    protected void innerUpdateValueOption(ValueOption valueOption) {
        internalValueOption.update(valueOption);
    }

    protected boolean innerSetDefaultLibelleUsage(boolean def) {
        if (def == defaultLibelle) {
            return false;
        }
        this.defaultLibelle = def;
        if (def) {
            customLibelleHolder = null;
        } else {
            customLibelleHolder = new CustomLibelleHolder();
        }
        return true;
    }

    protected boolean innerSetWithIdalpha(boolean withIdalpha) {
        if (this.withIdalpha == withIdalpha) {
            return false;
        }
        this.withIdalpha = withIdalpha;
        return true;
    }

    protected boolean innerPutLibelle(Libelle libelle) {
        if (customLibelleHolder == null) {
            throw new IllegalArgumentException("defaultLibelleUsage is set");
        }
        return customLibelleHolder.putLibelle(libelle);
    }

    protected boolean innerRemoveLibelle(int langInteger) {
        if (customLibelleHolder == null) {
            throw new IllegalArgumentException("defaultLibelleUsage is set");
        }
        return customLibelleHolder.removeLibelle(langInteger);
    }


    private class CustomLibelleHolder extends DefaultLibelleHolder {

        private boolean putLibelle(Libelle libelle) {
            return innerPutLibelle(libelle);
        }

        private boolean removeLibelle(int langInteger) {
            return innerRemoveLibelle(langInteger);
        }

    }


    private class InternalInputOption extends DefaultInputOption {

        InternalInputOption(InputOption inputOption) {
            super(inputOption);
        }

        private void update(InputOption inputOption) {
            innerUpdate(inputOption);
        }

    }


    private class InternalLimitOption extends DefaultLimitOption {

        InternalLimitOption(LimitOption limitOption) {
            super(limitOption);
        }

        private void update(LimitOption limitOption) {
            innerUpdate(limitOption);
        }

    }


    private class InternalValueOption extends DefaultValueOption {

        InternalValueOption() {
            innerSetMandatory(false);
            innerSetDefaultValue("");
        }

        InternalValueOption(ValueOption valueOption) {
            super(valueOption);
        }

        private void update(ValueOption valueOption) {
            innerUpdate(valueOption);
        }

    }

}
