/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.options;


/**
 *
 * @author Vincent Calame
 */
public class DefaultInputOption implements InputOption {

    private short inputType = INPUT_TEXT;
    private short widthType = WIDTH_LARGE;
    private int rows = 1;

    public DefaultInputOption() {
    }

    public DefaultInputOption(InputOption inputOption) {
        innerUpdate(inputOption);
    }

    public DefaultInputOption(short widthType, int rows, short type) {
        innerSetWidthType(widthType);
        innerSetRows(rows);
        innerSetInputType(type);
    }

    public short getInputType() {
        return inputType;
    }

    public short getWidthType() {
        return widthType;
    }

    public int getRows() {
        return rows;
    }

    protected void innerSetInputType(short inputType) {
        switch (inputType) {
            case INPUT_TEXT:
            case INPUT_SELECT:
            case INPUT_RADIO:
            case INPUT_CHECK:
            case INPUT_NOTEDITABLE:
            case INPUT_HIDDEN:
                this.inputType = inputType;
                break;
            default:
                throw new IllegalArgumentException("inputType has Wrong value");
        }
    }

    protected void innerUpdate(InputOption inputOption) {
        innerSetWidthType(inputOption.getWidthType());
        innerSetRows(inputOption.getRows());
        innerSetInputType(inputOption.getInputType());
    }

    protected void innerSetWidthType(short widthType) {
        switch (widthType) {
            case WIDTH_SMALL:
            case WIDTH_MEDIUM:
            case WIDTH_LARGE:
                this.widthType = widthType;
                break;
            default:
                throw new IllegalArgumentException("wrong value : " + widthType);
        }
    }

    protected void innerSetRows(int rows) {
        if (rows <= 0) {
            throw new IllegalArgumentException("rows <= 0");
        }
        this.rows = rows;
    }

    public final static String inputTypeToString(short inputType) {
        switch (inputType) {
            case INPUT_TEXT:
                return "text";
            case INPUT_SELECT:
                return "select";
            case INPUT_CHECK:
                return "check";
            case INPUT_RADIO:
                return "radio";
            case INPUT_NOTEDITABLE:
                return "noteditable";
            case INPUT_HIDDEN:
                return "hidden";
            default:
                throw new IllegalArgumentException("wrong value :" + inputType);
        }
    }

    public final static short inputTypeToShort(String s) {
        if (s.equals("text")) {
            return INPUT_TEXT;
        }
        if (s.equals("select")) {
            return INPUT_SELECT;
        }
        if (s.equals("check")) {
            return INPUT_CHECK;
        }
        if (s.equals("radio")) {
            return INPUT_RADIO;
        }
        if (s.equals("noteditable")) {
            return INPUT_NOTEDITABLE;
        }
        if (s.equals("hidden")) {
            return INPUT_HIDDEN;
        }
        throw new IllegalArgumentException("wrong value : " + s);
    }

    public final static String widthTypeToString(short widthType) {
        switch (widthType) {
            case WIDTH_SMALL:
                return "small";
            case WIDTH_MEDIUM:
                return "medium";
            case WIDTH_LARGE:
                return "large";
            default:
                throw new IllegalArgumentException("wrong value : " + widthType);
        }
    }

    public final static short widthTypeToShort(String s) {
        if (s.equals("small")) {
            return WIDTH_SMALL;
        }
        if (s.equals("medium")) {
            return WIDTH_MEDIUM;
        }
        if (s.equals("large")) {
            return WIDTH_LARGE;
        }
        try {
            int size = Integer.parseInt(s);
            if (size < 10) {
                return WIDTH_SMALL;
            }
            if (size < 50) {
                return WIDTH_MEDIUM;
            }
            return WIDTH_LARGE;
        } catch (NumberFormatException nfe) {
            throw new IllegalArgumentException("wrong value : " + s);
        }
    }

}
