/* BdfServer - Copyright (c) 2009-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.corpus.MutableCommentUI;
import fr.exemole.bdfserver.ui.corpus.MutableFieldUI;
import fr.exemole.bdfserver.ui.corpus.MutableIncludeUI;
import net.mapeadores.util.exceptions.SwitchException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class ComponentUiFilterDOMReader extends AbstractUiDOMReader {

    private Builder builder;

    public ComponentUiFilterDOMReader(Builder builder, UiErrorHandler errorHandler) {
        super(builder.getComponentUIEditor(), errorHandler);
        this.builder = builder;
    }

    public void readComponentUiFilter(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("exclude")) {
                    readExclude(element);
                } else if (tagName.equals("replace")) {
                    readReplace(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void readReplace(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                ComponentUI componentUI = getComponentUI(element);
                if (componentUI != null) {
                    componentUI = readComponentUI(element, componentUI);
                    builder.replaceComponentUI(componentUI);
                }
            }
        }
    }

    private void readExclude(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                ComponentUI componentUI = getComponentUI(element);
                if (componentUI != null) {
                    builder.excludeComponentUI(componentUI);
                }
            }
        }
    }

    private ComponentUI readComponentUI(Element element_xml, ComponentUI componentUI) {
        if (componentUI instanceof FieldUI) {
            MutableFieldUI mutableFieldUI = new MutableFieldUI((FieldUI) componentUI);
            readFieldUI(element_xml, mutableFieldUI);
            return mutableFieldUI;
        } else if (componentUI instanceof IncludeUI) {
            MutableIncludeUI mutableIncludeUI = new MutableIncludeUI((IncludeUI) componentUI);
            readIncludeUI(element_xml, mutableIncludeUI);
            return mutableIncludeUI;
        } else if (componentUI instanceof CommentUI) {
            MutableCommentUI mutableCommentUI = new MutableCommentUI((CommentUI) componentUI);
            readCommentUI(element_xml, mutableCommentUI);
            return mutableCommentUI;
        }
        throw new SwitchException("Unkown implementation : " + componentUI.getClass().getName());
    }

    private ComponentUI getComponentUI(Element element) {
        String tagName = element.getTagName();
        if (tagName.equals("field-ui")) {
            return getFieldUI(element);
        } else if (tagName.equals("subset-ui")) {
            return getSubsetIncludeUI(element, false);
        } else if (tagName.equals("comment-def")) {
            return getCommentUI(element, false);
        } else if (tagName.equals("special-ui")) {
            return getSpecialIncludeUI(element, false);
        } else {
            errorHandler.unknownTagWarning(tagName);
            return null;
        }
    }


    public static interface Builder {

        public ComponentUIEditor getComponentUIEditor();

        public void excludeComponentUI(ComponentUI componentUI);

        public void replaceComponentUI(ComponentUI componentUI);

    }

}
