/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class CorpusUiDOMReader extends AbstractUiDOMReader {

    private String corpusName;

    public CorpusUiDOMReader(ComponentUIEditor editor, UiErrorHandler errorHandler, SubsetKey corpusKey) {
        super(editor, errorHandler);
        this.corpusName = corpusKey.getSubsetName();
    }

    public void readCorpusUI(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        int length = liste.getLength();
        for (int i = 0; i < length; i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("metadata")) {
                    readMetadata(element);
                }
            }
        }
        List<String> keyList = new ArrayList<String>();
        for (int i = 0; i < length; i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("metadata")) {
                    continue;
                } else if (tagName.equals("comment")) {
                    String key = readComment(element);
                    if (key != null) {
                        keyList.add(key);
                    }
                } else if (tagName.equals("field-ui")) {
                    String key = readFieldUI(element);
                    if (key != null) {
                        keyList.add(key);
                    }
                } else if (tagName.equals("subset-ui")) {
                    String key = readSubsetIncludeUI(element);
                    if (key != null) {
                        keyList.add(key);
                    }
                } else if (tagName.equals("special-ui")) {
                    String key = readSpecialIncludeUI(element);
                    if (key != null) {
                        keyList.add(key);
                    }
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
        int size = keyList.size();
        if (size > 0) {
            String[] array = new String[size];
            for (int i = 0; i < size; i++) {
                array[i] = (String) keyList.get(i);
            }
            editor.setPositionArray(array);
        }
    }

    private String readComment(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError("comment", "name");
            return null;
        }
        name = checkName(name);
        CommentUI comment = editor.getCommentUI(name, false);
        if (comment == null) {
            errorHandler.unknownKeyError("comment", name);
            return null;
        }
        return "comment_" + name;
    }

    private String readFieldUI(Element element_xml) {
        FieldUI fieldUI = getFieldUI(element_xml);
        if (fieldUI == null) {
            return null;
        }
        readFieldUI(element_xml, fieldUI);
        return fieldUI.getCorpusField().getFieldKey().toString();
    }

    private String readSubsetIncludeUI(Element element_xml) {
        IncludeUI includeUI = getSubsetIncludeUI(element_xml, true);
        if (includeUI == null) {
            return null;
        }
        readIncludeUI(element_xml, includeUI);
        return includeUI.getIncludeKey().toString();
    }

    private String readSpecialIncludeUI(Element element_xml) {
        IncludeUI includeUI = getSpecialIncludeUI(element_xml, true);
        if (includeUI == null) {
            return null;
        }
        readIncludeUI(element_xml, includeUI);
        return includeUI.getIncludeKey().toString();
    }

    private void readMetadata(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("comment-def")) {
                    readCommentDef(element);
                } else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }

    private void readCommentDef(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "name");
            return;
        }
        name = checkName(name);
        CommentUI commentUI = editor.getCommentUI(name, true);
        if (commentUI == null) {
            return;
        }
        readCommentUI(element_xml, commentUI);
    }

    /**
     * Conversion de l'ancien mode de création automatique des noms qui utilisait le nom du corpus
     */
    private String checkName(String name) {
        if (name.startsWith(corpusName)) {
            try {
                int number = Integer.parseInt(name.substring(corpusName.length()));
                return "c" + number;
            } catch (NumberFormatException nfe) {
                return name;
            }
        }
        return name;
    }

}
