/* BdfServer - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.ui.xml;

import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIFilter;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.corpus.MutableCommentUI;
import fr.exemole.bdfserver.ui.corpus.MutableFieldUI;
import fr.exemole.bdfserver.ui.corpus.MutableIncludeUI;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class DefaultUIFilterBuilder implements ComponentUiFilterDOMReader.Builder {

    private InternalComponentUIEditor internalComponentUIEditor;
    private Set<String> excludeSet = new HashSet<String>();
    private Map<String, ComponentUI> replaceMap = new HashMap<String, ComponentUI>();
    private CorpusUI corpusUI;

    public DefaultUIFilterBuilder(CorpusUI corpusUI) {
        this.corpusUI = corpusUI;
        internalComponentUIEditor = new InternalComponentUIEditor();
    }

    public ComponentUIEditor getComponentUIEditor() {
        return internalComponentUIEditor;
    }

    public void excludeComponentUI(ComponentUI componentUI) {
        excludeSet.add(componentUI.getKeyString());
    }

    /**
     * Sont des instances de MutableIncludeUI, MutableFieldUI, MutableCommentUI
     */
    public void replaceComponentUI(ComponentUI componentUI) {
        replaceMap.put(componentUI.getKeyString(), componentUI);
    }

    public ComponentUIFilter toComponentUIFilter() {
        return new InternalComponentUIFilter(excludeSet, replaceMap);
    }


    private static class InternalComponentUIFilter implements ComponentUIFilter {

        private Set<String> excludeSet;
        private Map<String, ComponentUI> replaceMap;

        InternalComponentUIFilter(Set<String> excludeSet, Map<String, ComponentUI> replaceMap) {
            this.excludeSet = excludeSet;
            this.replaceMap = replaceMap;
        }

        public ComponentUI filter(ComponentUI componentUI) {
            String keyString = componentUI.getKeyString();
            if (excludeSet.contains(keyString)) {
                return null;
            }
            ComponentUI other = replaceMap.get(keyString);
            if (other != null) {
                return other;
            }
            return componentUI;
        }

    }


    private class InternalComponentUIEditor implements ComponentUIEditor {

        public FieldUI getFieldUI(FieldKey fieldKey) {
            return corpusUI.getFieldUI(fieldKey);
        }

        public IncludeUI getIncludeUI(IncludeKey includeKey, boolean askForCreation) {
            return corpusUI.getIncludeUI(includeKey);
        }

        public CommentUI getCommentUI(String name, boolean askForCreation) {
            return corpusUI.getCommentUI(name);
        }

        public boolean setInputOption(ComponentUI componentUI, InputOption inputOption) {
            if (componentUI instanceof MutableFieldUI) {
                ((MutableFieldUI) componentUI).updateInputOption(inputOption);
                return true;
            } else if (componentUI instanceof MutableIncludeUI) {
                ((MutableIncludeUI) componentUI).updateInputOption(inputOption);
                return true;
            }
            return false;
        }

        public boolean setLimitOption(ComponentUI componentUI, LimitOption limitOption) {
            if (componentUI instanceof MutableIncludeUI) {
                ((MutableIncludeUI) componentUI).updateLimitOption(limitOption);
                return true;
            }
            return false;
        }

        public boolean setValueOption(ComponentUI componentUI, ValueOption valueOption) {
            if (componentUI instanceof MutableFieldUI) {
                ((MutableFieldUI) componentUI).updateValueOption(valueOption);
                return true;
            } else if (componentUI instanceof MutableIncludeUI) {
                ((MutableIncludeUI) componentUI).updateValueOption(valueOption);
                return true;
            }
            return false;
        }

        public boolean setMotcleSelectOptionsString(ComponentUI componentUI, String motcleSelectOptionsString) {
            if (componentUI instanceof MutableIncludeUI) {
                ((MutableIncludeUI) componentUI).setMotcleSelectOptionsString(motcleSelectOptionsString);
                return true;
            }
            return false;
        }

        public boolean putLibelle(IncludeUI includeUI, Libelle libelle) {
            if (includeUI instanceof MutableIncludeUI) {
                return ((MutableIncludeUI) includeUI).putLibelle(libelle);
            }
            return false;
        }

        public boolean removeLibelle(IncludeUI includeUI, int langInteger) {
            if (includeUI instanceof MutableIncludeUI) {
                return ((MutableIncludeUI) includeUI).removeLibelle(langInteger);
            }
            return false;
        }

        public boolean setDefaultLibelleUsage(ComponentUI componentUI, boolean b) {
            if (componentUI instanceof MutableIncludeUI) {
                return ((MutableIncludeUI) componentUI).setDefaultLibelleUsage(b);
            }
            return false;
        }

        public boolean setLocation(ComponentUI componentUI, int newLocation) {
            if (componentUI instanceof MutableCommentUI) {
                return ((MutableCommentUI) componentUI).setLocation(newLocation);
            }
            return false;
        }

        public boolean setPositionArray(String[] positionArray) {
            return false;
        }

        public boolean putHtmlString(CommentUI commentUI, int langInteger, String htmlString) {
            if (commentUI instanceof MutableCommentUI) {
                return ((MutableCommentUI) commentUI).putHtmlString(langInteger, htmlString);
            }
            return false;
        }

    }

}
