/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.users;

import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class BdfUserDefBuilder {

    private InternalBdfUserDef internalBdfUserDef;

    public BdfUserDefBuilder(RedacteurKey redacteurKey) {
        internalBdfUserDef = new InternalBdfUserDef(redacteurKey);
    }

    public BdfUserDef toBdfUserDef() {
        return internalBdfUserDef;
    }

    public void setLocalisationInfo(LocalisationInfo localisationInfo) {
        internalBdfUserDef.localisationInfo = localisationInfo;
    }

    public void setDefaultFicheSelectOptions(FicheSelectOptions defaultFicheSelectOptions) {
        internalBdfUserDef.defaultFicheSelectOptions = defaultFicheSelectOptions;
    }

    public void setAbregeFicheSortType(short abregeFicheSortType) {
        if (!SelectionUtils.isValidSortType(abregeFicheSortType)) {
            throw new IllegalArgumentException("abregeFicheSortType is not valid");
        }
        internalBdfUserDef.abregeFicheSortType = abregeFicheSortType;
    }

    public void setRoleNameArray(String[] roleNameArray) {
        internalBdfUserDef.roleNameArray = roleNameArray;
    }

    public void setTableExportParametersBean(TableExportParametersBean tableExportParametersBean) {
        internalBdfUserDef.tableExportParametersBean = tableExportParametersBean;
    }

    public void setXSLTemplateName(TransformationKey transformationKey, String templateName) {
        internalBdfUserDef.xslTemplateDefList.add(new InternalXslTemplateDef(templateName, transformationKey));
    }

    public void putStreamTransformerKey(StreamTransformerKey streamTransformerKey) {
        internalBdfUserDef.streamTransformerKeyList.add(streamTransformerKey);
    }


    private static class InternalBdfUserDef implements BdfUserDef {

        private RedacteurKey redacteurKey;
        private LocalisationInfo localisationInfo;
        private FicheSelectOptions defaultFicheSelectOptions;
        private short abregeFicheSortType;
        private String[] roleNameArray;
        private TableExportParametersBean tableExportParametersBean;
        private List<StreamTransformerKey> streamTransformerKeyList = new ArrayList<StreamTransformerKey>();
        private List<XslTemplateDef> xslTemplateDefList = new ArrayList<XslTemplateDef>();

        private InternalBdfUserDef(RedacteurKey redacteurKey) {
            this.redacteurKey = redacteurKey;
        }

        public RedacteurKey getRedacteurKey() {
            return redacteurKey;
        }

        public LocalisationInfo getLocalisationInfo() {
            return localisationInfo;
        }

        public FicheSelectOptions getDefaultFicheSelectOptions() {
            return defaultFicheSelectOptions;
        }

        public short getAbregeFicheSortType() {
            return abregeFicheSortType;
        }

        public int getRoleNameCount() {
            if (roleNameArray == null) {
                return 0;
            }
            return roleNameArray.length;
        }

        public String getRoleName(int i) {
            return roleNameArray[i];
        }

        public TableExportParametersBean getTableExportParametersBean() {
            return tableExportParametersBean;
        }

        public int getStreamTransformerKeyCount() {
            return streamTransformerKeyList.size();
        }

        public StreamTransformerKey getStreamTransformerKey(int i) {
            return streamTransformerKeyList.get(i);
        }

        public int getXslTemplateDefCount() {
            return xslTemplateDefList.size();
        }

        public XslTemplateDef getXslTemplateDef(int i) {
            return xslTemplateDefList.get(i);
        }

    }


    private static class InternalXslTemplateDef implements BdfUserDef.XslTemplateDef {

        private String templateName;
        private TransformationKey transformationKey;

        private InternalXslTemplateDef(String templateName, TransformationKey transformationKey) {
            this.templateName = templateName;
            this.transformationKey = transformationKey;
        }

        public String getTemplateName() {
            return templateName;
        }

        public TransformationKey getTransformationKey() {
            return transformationKey;
        }

    }

}
