/* BdfServer - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.users;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.request.tostring.FicheItemToString;
import java.text.DecimalFormatSymbols;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.text.Libelle;


/**
 *
 * @author Vincent Calame
 */
public class BdfUserUtils implements BdfUserConstants {

    private BdfUserUtils() {
    }

    public static Redacteur getRedacteur(BdfUser bdfUser) {
        return getRedacteur(bdfUser.getBdfServer().getBasedefiches(), bdfUser.getRedacteurKey());
    }

    public static Redacteur getRedacteur(BdfServer bdfServer, RedacteurKey redacteurKey) {
        return getRedacteur(bdfServer.getBasedefiches(), redacteurKey);
    }

    public static Redacteur getRedacteur(Basedefiches basedefiches, RedacteurKey redacteurKey) {
        Sphere sphere = (Sphere) basedefiches.getSubset(redacteurKey.getSphereKey());
        if (sphere == null) {
            return null;
        }
        return sphere.getRedacteurByIdsphere(redacteurKey.getIdsphereString());
    }

    public static Sphere getSphere(BdfUser bdfUser) {
        RedacteurKey redacteurKey = bdfUser.getRedacteurKey();
        return (Sphere) bdfUser.getBdfServer().getBasedefiches().getSubset(redacteurKey.getSphereKey());
    }

    public static boolean isAdmin(BdfServer bdfServer, RedacteurKey redacteurKey) {
        return bdfServer.getPermissionManager().isAdmin(redacteurKey);
    }

    public static boolean isAdmin(BdfUser bdfUser) {
        if (bdfUser == null) {
            return false;
        }
        return bdfUser.getBdfServer().getPermissionManager().isAdmin(bdfUser.getRedacteurKey());
    }

    public static void saveBdfUser(BdfUser bdfUser) {
        bdfUser.getBdfServer().getBdfStorage().getBdfUserStorage().saveBdfUserDef(BdfUser.toBdfUserDef(bdfUser));
    }

    public static String getAbregeFicheString(BdfUser bdfUser, AbregeFiche abregeFiche) {
        return CorpusMetadataUtils.getFicheIntituleString(abregeFiche, bdfUser.getLocalisationInfo());
    }

    public static String getMotcleString(BdfUser bdfUser, Motcle motcle) {
        boolean withSmallId = true;
        String idalpha = motcle.getIdalpha();
        StringBuffer buf = new StringBuffer();
        if (idalpha != null) {
            buf.append(idalpha);
            buf.append(" - ");
            withSmallId = false;
        }
        buf.append(motcle.getLibelleString(bdfUser.getLocalisationInfo().getWorkingLangInteger(), true));
        if (withSmallId) {
            buf.append(" (");
            buf.append("idths=");
            buf.append(String.valueOf(motcle.getIdths()));
            buf.append(")");
        }
        return buf.toString();
    }

    public static String getMotcleCompleteString(BdfUser bdfUser, Motcle motcle) {
        boolean withSmallId = true;
        String idalpha = motcle.getIdalpha();
        StringBuilder buf = new StringBuilder();
        if (idalpha != null) {
            buf.append(idalpha);
            buf.append(" - ");
            withSmallId = false;
        }
        int libelleCount = motcle.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = motcle.getLibelleByIndex(i);
            if (i > 0) {
                buf.append(" / ");
            }
            buf.append(libelle.getLibelleString());
        }
        if (withSmallId) {
            buf.append(" (");
            buf.append("idths=");
            buf.append(String.valueOf(motcle.getIdths()));
            buf.append(")");
        }
        return buf.toString();
    }

    public static FicheItemToString.Parameters getFicheItemToStringParameters(BdfUser bdfUser) {
        FicheItemToString.Parameters params = new FicheItemToString.Parameters();
        params.setDefautSphereKey(bdfUser.getRedacteurKey().getSphereKey());
        DecimalFormatSymbols dfs = new DecimalFormatSymbols(bdfUser.getLocalisationInfo().getFormatLocale());
        params.setDecimalChar(dfs.getDecimalSeparator());
        return params;
    }

    public static FicheItemToString.Parameters getFicheItemToStringParameters(LocalisationInfo localisationInfo) {
        FicheItemToString.Parameters params = new FicheItemToString.Parameters();
        DecimalFormatSymbols dfs = new DecimalFormatSymbols(localisationInfo.getFormatLocale());
        params.setDecimalChar(dfs.getDecimalSeparator());
        return params;
    }

    public static short checkScreenWidth(String s) {
        if (s == null) {
            return SCREENWIDTH_1024;
        }
        try {
            int val = Integer.parseInt(s);
            if (val < 900) {
                return SCREENWIDTH_800;
            }
            if (val < 1100) {
                return SCREENWIDTH_1024;
            }
            return SCREENWIDTH_1280;
        } catch (NumberFormatException nfe) {
            return SCREENWIDTH_1024;
        }
    }

    public static short getScreenWidth(BdfUser bdfUser) {
        if (bdfUser == null) {
            return SCREENWIDTH_1024;
        }
        return bdfUser.getScreenWidth();
    }

    public static boolean forceNomAvant(BdfUser bdfUser) {
        if (bdfUser == null) {
            return false;
        }
        return isAdmin(bdfUser);
    }

}
