/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.users.xml;

import fr.exemole.bdfserver.beans.TableExportParametersBean;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.users.BdfUserDef;
import java.io.IOException;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.basedefiches.tableexport.TableExportCommon;
import net.mapeadores.basedefiches.xml.extraction.ExtractionUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.xml.UtilXMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class UserXMLWriter extends UtilXMLWriter {

    public UserXMLWriter() {
    }

    public void appendUser(BdfUserDef bdfUserDef) throws IOException {
        startOpenTag("user");
        endOpenTag();
        appendLocalisationInfo(bdfUserDef);
        appendRoles(bdfUserDef);
        appendTransformation(bdfUserDef);
        appendSelection(bdfUserDef);
        appendTableExportParameters(bdfUserDef);
        closeTag("user");
    }

    private void appendRoles(BdfUserDef bdfUserDef) throws IOException {
        int roleNameCount = bdfUserDef.getRoleNameCount();
        if (roleNameCount == 0) {
            return;
        }
        openTag("roles");
        for (int i = 0; i < roleNameCount; i++) {
            startOpenTag("role");
            addAttribute("name", bdfUserDef.getRoleName(i));
            closeEmptyTag();
        }
        closeTag("roles");
    }

    private void appendTransformation(BdfUserDef bdfUserDef) throws IOException {
        int xslTemplateDefCount = bdfUserDef.getXslTemplateDefCount();
        int streamTransformerKeyCount = bdfUserDef.getStreamTransformerKeyCount();
        if ((xslTemplateDefCount == 0) && (streamTransformerKeyCount == 0)) {
            return;
        }
        openTag("transformation");
        for (int i = 0; i < xslTemplateDefCount; i++) {
            BdfUserDef.XslTemplateDef xslTemplateDef = bdfUserDef.getXslTemplateDef(i);
            startOpenTag("xsl");
            addAttribute("key", xslTemplateDef.getTransformationKey().toString());
            addAttribute("name", xslTemplateDef.getTemplateName());
            closeEmptyTag();
        }
        for (int i = 0; i < streamTransformerKeyCount; i++) {
            StreamTransformerKey key = bdfUserDef.getStreamTransformerKey(i);
            startOpenTag("stream");
            addAttribute("key", key.getTransformationKey().toString());
            addAttribute("extension", key.getExtension());
            addAttribute("name", key.getTransformerName());
            closeEmptyTag();
        }
        closeTag("transformation");

    }

    private void appendLocalisationInfo(BdfUserDef bdfUserDef) throws IOException {
        LocalisationInfo localisationInfo = bdfUserDef.getLocalisationInfo();
        if (localisationInfo == null) {
            return;
        }
        startOpenTag("loc-info");
        addAttribute("working-lang", LangInteger.toString(localisationInfo.getWorkingLangInteger()));
        if (!localisationInfo.isSame()) {
            addAttribute("format-locale", LangInteger.toString(localisationInfo.getFormatLocaleInteger()));
        }
        closeEmptyTag();
    }

    private void appendSelection(BdfUserDef bdfUserDef) throws IOException {
        FicheSelectOptions ficheSelectOptions = bdfUserDef.getDefaultFicheSelectOptions();
        if (ficheSelectOptions == null) {
            return;
        }
        startOpenTag("selection");
        addAttribute("sort", SelectionUtils.sortTypeToString(bdfUserDef.getAbregeFicheSortType()));
        endOpenTag();
        ExtractionUtils.writeFicheSelectOptions(this, ficheSelectOptions);
        closeTag("selection");
    }

    private void appendTableExportParameters(BdfUserDef bdfUserDef) throws IOException {
        TableExportParametersBean tableExportParametersBean = bdfUserDef.getTableExportParametersBean();
        if (tableExportParametersBean == null) {
            return;
        }
        startOpenTag("table-export-parameters");
        addAttribute("name", tableExportParametersBean.getTableExportName());
        appendBoolean("with-textes", tableExportParametersBean.isWithTextes());
        appendBoolean("with-association", tableExportParametersBean.isWithAssociation());
        appendBoolean("with-liage", tableExportParametersBean.isWithLiage());
        appendBoolean("with-chrono", tableExportParametersBean.isWithChrono());
        addAttribute("header-type", TableExportCommon.headerTypeToString(tableExportParametersBean.getHeaderType()));
        addAttribute("indexation-include-type", TableExportCommon.indexationIncludeTypeToString(tableExportParametersBean.getIndexationIncludeType()));
        closeEmptyTag();
    }

    private void appendBoolean(String name, boolean booleanValue) throws IOException {
        String value = (booleanValue) ? "1" : "0";
        addAttribute(name, value);
    }

}
