/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.xsltfactory.odcss.BdfCssSourceUtils;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import net.mapeadores.opendocument.css.elements.ElementMaps;
import net.mapeadores.opendocument.css.output.StylesOutput;
import net.mapeadores.opendocument.css.parse.CssParser;
import net.mapeadores.opendocument.css.parse.CssSource;
import net.mapeadores.opendocument.transform.ODStreamTransformer;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.io.StreamTransformerErrorHandler;


/**
 *
 * @author Vincent Calame
 */
public class DefaultOdStreamTransformerBuilder {

    private ODStreamTransformer.Builder odStreamTransformerBuilder;
    private BdfServer bdfServer;

    public DefaultOdStreamTransformerBuilder(BdfServer bdfServer, String extension) {
        odStreamTransformerBuilder = new ODStreamTransformer.Builder(extension);
        this.bdfServer = bdfServer;
    }

    private void setOdCss(String odCssRessourcUrl) {
        CssSource cssSource = BdfCssSourceUtils.getBdfCssSource(odCssRessourcUrl, bdfServer);
        if (cssSource == null) {
            throw new InternalResourceException("missing resource = " + odCssRessourcUrl);
        }
        try {
            ElementMaps elementMaps = CssParser.parse(cssSource, CssParser.OUT_ERRORHANDLER);
            ByteArrayOutputStream stylesOutputStream = new ByteArrayOutputStream();
            ByteArrayOutputStream contentAutomaticStylesOutputStream = new ByteArrayOutputStream();
            StylesOutput.write(stylesOutputStream, elementMaps);
            StylesOutput.writeAutomaticStyles(contentAutomaticStylesOutputStream, elementMaps);
            odStreamTransformerBuilder.setStyles(stylesOutputStream.toByteArray());
            odStreamTransformerBuilder.setContentAutomaticStyles(contentAutomaticStylesOutputStream.toByteArray());
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }

    }

    public StreamTransformer toStreamTransformer(String odCssRessourceUrl, String contentStyleSheet) {
        setOdCss(odCssRessourceUrl);
        odStreamTransformerBuilder.setURIResolver(bdfServer.getURIResolver());
        odStreamTransformerBuilder.setContentURI("_default_");
        odStreamTransformerBuilder.setStreamTransformerErrorHandler(new InternalStreamTransformerErrorHandler());
        try {
            odStreamTransformerBuilder.setContentXsl(contentStyleSheet.getBytes("UTF-8"));
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        return odStreamTransformerBuilder.toODStreamTransformer();
    }


    private static class InternalStreamTransformerErrorHandler implements StreamTransformerErrorHandler {

        public void missingResourceError(String resourceURI) {
            throw new InternalResourceException("StreamTransformerErrorHandler - missingResourceError : " + resourceURI);
        }

        public void wrongResourceError(String resourceURI, String message) {
            throw new InternalResourceException("StreamTransformerErrorHandler - wrongResourceError : " + resourceURI + " / " + message);
        }

    }

}
