/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.xsltfactory.xslpattern.PatternUtils;
import fr.exemole.bdfserver.xsltfactory.xslpattern.XSLTPatternBuilder;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.text.insertpattern.api.InstructionErrorHandler;


/**
 *
 * @author Vincent Calame
 */
public class DefaultXSLTFactory implements XSLTFactoryConstants {

    private DefaultXSLTFactory() {
    }

    public static String getDefaultStyleSheet(BdfServer bdfServer, TransformationKey transformationKey, short format) {
        String pattern = XSLTPatternBuilder.getXSLTPattern(bdfServer, transformationKey, false, format);
        if (pattern == null) {
            return null;
        }
        return PatternUtils.getStyleSheet(bdfServer, pattern, new XSLTErrorHandler(), null);
    }


    static private class XSLTErrorHandler implements InstructionErrorHandler {

        public void characterError(char carac, String key, int row, int col) {
            throw new InternalResourceException("XSLTErrorHandler characterError = " + carac + " / " + key + " / " + row + "," + col);
        }

        public void warning(String key, int row, int col) {
            throw new InternalResourceException("XSLTErrorHandler warning = " + key + " / " + row + "," + col);
        }

    }

    public static StreamTransformer getDefaultStreamTransformer(BdfServer bdfServer, TransformationKey transformationKey, short format) {
        switch (format) {
            case ODT_FORMAT:
                DefaultOdStreamTransformerBuilder odBuilder = new DefaultOdStreamTransformerBuilder(bdfServer, "odt");
                return odBuilder.toStreamTransformer(getOdCssResourceUrl(transformationKey), getDefaultStyleSheet(bdfServer, transformationKey, format));
            case XSL_FORMAT:
                return null;
            default:
                throw new SwitchException("format = " + format);
        }
    }

    private static String getOdCssResourceUrl(TransformationKey transformationKey) {
        if (transformationKey.isCorpusTransformationKey() || (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY))) {
            return "bdf:fiche_odt.css";
        }
        throw new SwitchException("transformationKey = " + transformationKey.toString());
    }

}
