/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPath;
import fr.exemole.bdfserver.managers.transformation.TransformationConstants;
import java.io.IOException;
import java.net.URL;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.xml.XSLTWriter;


/**
 *
 * @author Vincent Calame
 */
public class ImportXSLTWriter extends XSLTWriter implements XSLTFactoryConstants, TransformationConstants {

    public static final short FICHE_IMPORT = 1;
    public static final short STATTHESAURUS_IMPORT = 2;
    private short importType = 1;
    private BdfServer bdfServer;

    public ImportXSLTWriter(short importType, BdfServer bdfServer) {
        super();
        this.bdfServer = bdfServer;
        this.importType = importType;
    }

    public void addImportXSLT(short format) throws IOException {
        openXslStyleSheet();
        switch (format) {
            case XSL_FORMAT:
                switch (importType) {
                    case FICHE_IMPORT:
                        addXslImport("bdf:_common_ficheitem.xsl");
                        addXslImport("bdf:_common_ficheblock.xsl");
                        addXslImport("bdf:_common_field.xsl");
                        addXslImport("bdf:_common_include.xsl");
                        addXslImport("bdf:_common_intitules.xsl");
                        addXslImport("bdf:_common_motcle.xsl");
                        break;
                    case STATTHESAURUS_IMPORT:
                        addXslImport("bdf:_common_intitules.xsl");
                        addXslImport("bdf:_common_motcle.xsl");
                        break;
                    default:
                        throw new SwitchException("importtype = " + importType);
                }
                break;
            case ODT_FORMAT:
                switch (importType) {
                    case FICHE_IMPORT:
                        addXslImport("bdf:_common_ficheitem_odt.xsl");
                        addXslImport("bdf:_common_ficheblock_odt.xsl");
                        addXslImport("bdf:_common_field.xsl");
                        addXslImport("bdf:_common_include_odt.xsl");
                        addXslImport("bdf:_common_intitules.xsl");
                        addXslImport("bdf:_common_motcle_odt.xsl");
                        break;
                    default:
                        throw new SwitchException("importtype = " + importType);
                }
                break;
            default:
                throw new SwitchException("format = " + format);
        }

        addXslParam(WORKINGLANG_PARAMETER, "fr", true);
        addXslParam(FORMATLOCALE_PARAMETER, "fr", true);
        addXslParam(BDF_BASEDEFICHESPATH_PARAMETER, "", true);

        addFicheURLTemplate();
        addMotcleURLTemplate();
        addIllustrationURLTemplate();
        addVersionURLTemplate();
        closeXslStyleSheet();
    }

    private void addFicheURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_FICHEURL");
        addXslParam("PARAM_CORPUS");
        addXslParam("PARAM_IDCORPUS");
        addXslParam("PARAM_FRAGMENT", "", true);
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        addXslText("fiches/");
        addXslValueOf("$PARAM_CORPUS");
        addXslText("-");
        addXslValueOf("$PARAM_IDCORPUS");
        addXslText(".html");
        openXslIf("string-length($PARAM_FRAGMENT) > 0");
        appendIndent();
        addXslText("#");
        addXslValueOf("$PARAM_FRAGMENT");
        closeXslIf();
        closeXslTemplate();
    }

    private void addMotcleURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_MOTCLEURL");
        addXslParam("PARAM_THESAURUS");
        addXslParam("PARAM_IDTHS");
        addXslParam("PARAM_FRAGMENT", "", true);
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        append("Thesaurus?thesaurus=");
        addXslValueOf("$PARAM_THESAURUS");
        escape("&idths=");
        addXslValueOf("$PARAM_IDTHS");
        closeXslTemplate();
    }

    private void addIllustrationURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_ILLUSTRATIONURL");
        addXslParam("PARAM_ALBUM");
        addXslParam("PARAM_IDALBUM");
        addXslParam("PARAM_FORMAT");
        addXslParam("PARAM_DIM", "", true);
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        addXslText("illustrations/");
        openXslIf("string-length($PARAM_DIM) > 0");
        addXslValueOf("$PARAM_DIM");
        addXslText("/");
        closeXslIf();
        addXslValueOf("$PARAM_ALBUM");
        addXslText("-");
        addXslValueOf("$PARAM_IDALBUM");
        addXslText(".");
        addXslValueOf("$PARAM_FORMAT");
        closeXslTemplate();
    }

    private void addVersionURLTemplate() throws IOException {
        openNameXslTemplate("TEMPLATE_VERSIONURL");
        addXslParam("PARAM_BASENAME");
        addXslParam("PARAM_EXTENSION");
        appendIndent();
        addXslValueOf("$" + BDF_BASEDEFICHESPATH_PARAMETER);
        append("docs/");
        addXslValueOf("$PARAM_BASENAME");
        append(".");
        addXslValueOf("$PARAM_EXTENSION");
        closeXslTemplate();
    }

    /*public static String getImportXSLTName(short importType) {
    StringBuffer buf = new StringBuffer();
    buf.append("_import_");
    buf.append(importTypeToString(importType));
    buf.append(".xsl");
    return buf.toString();
    }*/
    /**
     * retourne nul si importType incorrect.
     */
    public static String getImportXSLT(String name, BdfServer bdfServer, boolean withTabs) {
        short importType = 0;
        short format = XSL_FORMAT;
        int index = name.indexOf('_');
        if (index != -1) {
            String formatString = name.substring(index + 1);
            if (formatString.equals("odt")) {
                format = ODT_FORMAT;
            } else if (!formatString.equals("xsl")) {
                return null;
            }
            name = name.substring(0, index);
        }
        if (name.equals("fiche")) {
            importType = FICHE_IMPORT;
        } else if (name.equals("statthesaurus")) {
            importType = STATTHESAURUS_IMPORT;
        } else {
            return null;
        }
        return getImportXSLT(importType, bdfServer, format, withTabs);
    }

    /**
     *@throw IllegalArgumentException si le type est incorrect
     */
    public static String getImportXSLT(short importType, BdfServer bdfServer, short format, boolean withTabs) {
        StringBuilder buf = new StringBuilder();
        try {
            ImportXSLTWriter importXSLTWriter = new ImportXSLTWriter(importType, bdfServer);
            importXSLTWriter.setAppendable(buf);
            importXSLTWriter.setIndentLength((withTabs) ? 0 : -999);
            importXSLTWriter.appendXMLDeclaration();
            importXSLTWriter.addImportXSLT(format);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    /*public static String importTypeToString(short importType) {
    switch(importType) {
    case FICHE_IMPORT :
    return "fiche";
    case STATTHESAURUS_IMPORT :
    return "statthesaurus";
    default :
    throw new IllegalArgumentException("wrong importType : " + importType);
    }
    }*/
}
