/* BdfServer - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.odcss;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceUtils;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import net.mapeadores.opendocument.css.parse.CssSource;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.io.DocStream;


/**
 *
 * @author Vincent Calame
 */
public class BdfCssSourceUtils {

    private final static String ETC_BDF_DIR = "bdf:_etc_/";
    private final static String VAR_BDF_DIR = "bdf:_var_/";
    private final static String BDF_PROTOCOL = "bdf:";

    public static boolean isBdfCssSource(String resourceUri) {
        return resourceUri.startsWith(BDF_PROTOCOL);
    }

    public static CssSource getBdfCssSource(String resourceUri, BdfServer bdfServer) {
        if (!isBdfCssSource(resourceUri)) {
            return null;
        }
        if (resourceUri.startsWith(ETC_BDF_DIR)) {
            String filePath = resourceUri.substring(ETC_BDF_DIR.length());
            File f = bdfServer.getTransformationManager().getEtcCommonsFile(filePath);
            if (f.exists() && (!f.isDirectory())) {
                return new FileBdfCssSource(getBaseUri(resourceUri), f, bdfServer);
            } else {
                return null;
            }
        }
        if (resourceUri.startsWith(VAR_BDF_DIR)) {
            String filePath = resourceUri.substring(VAR_BDF_DIR.length());
            File f = bdfServer.getTransformationManager().getVarCommonsFile(filePath);
            if (f.exists() && (!f.isDirectory())) {
                return new FileBdfCssSource(getBaseUri(resourceUri), f, bdfServer);
            } else {
                return null;
            }
        }
        String resourcePath = resourceUri.substring(BDF_PROTOCOL.length());
        if (!BdfResourceUtils.containsBdfResource(bdfServer.getBdfResourceStorage(), BdfResourceConstants.CSS_RESOURCE, resourcePath, true)) {
            return null;
        }
        return new BdfResourceCssSource(resourcePath, bdfServer);
    }

    private static String getBaseUri(String resourceUri) {
        int idx = resourceUri.lastIndexOf('/');
        if (idx == -1) {
            return "";
        }
        return resourceUri.substring(0, idx + 1);
    }


    private static class BdfResourceCssSource implements CssSource {

        private BdfServer bdfServer;
        private String resourcePath;

        private BdfResourceCssSource(String resourcePath, BdfServer bdfServer) {
            this.resourcePath = resourcePath;
            this.bdfServer = bdfServer;
        }

        public Reader getReader() throws IOException {
            try {
                DocStream docStream = BdfResourceUtils.getBdfResourceDocStream(bdfServer.getBdfResourceStorage(), BdfResourceConstants.CSS_RESOURCE, resourcePath);
                if (docStream == null) {
                    throw new ShouldNotOccurException("test of " + resourcePath + " done before");
                }
                Reader reader = new InputStreamReader(docStream.getInputStream(), "UTF-8");
                return reader;
            } catch (IOException ioe) {
                throw new InternalResourceException(ioe);
            }
        }

        public CssSource getImportCssSource(String url) {
            if ((url.startsWith("bdf:")) && (url.endsWith(".css"))) {
                return getBdfCssSource(url, bdfServer);
            }
            return null;
        }

        public String getCssSourceURI() {
            return "bdf:" + resourcePath;
        }

    }

}
