/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.LimitOption;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;


/**
 *
 * @author Vincent Calame
 */
public class ComponentInfo {

    public final static short MODE_INLINE = 1;
    public final static short MODE_BLOCK = 2;
    public final static short TEXTE = 3;
    public final static short MODE_LISTITEM = 4;
    private String variableString;
    private short mode;
    private boolean forceWithIf = false;

    public ComponentInfo() {
    }

    public static ComponentInfo newInstance(CorpusField corpusField) {
        FieldKey fieldKey = corpusField.getFieldKey();
        ComponentInfo componentInfo = new ComponentInfo();
        if (fieldKey.ownsToCorpsdefiche()) {
            componentInfo.setMode(TEXTE);
        } else if (fieldKey.isInformationFieldKey()) {
            if (CorpusMetadataUtils.isBlockDisplayInformationField(corpusField)) {
                switch (corpusField.getFicheItemType()) {
                    case CorpusField.ITEM_FIELD:
                    case CorpusField.PARA_FIELD:
                    case CorpusField.IMAGE_FIELD:
                        componentInfo.setMode(MODE_BLOCK);
                        break;
                    default:
                        componentInfo.setMode(MODE_LISTITEM);
                }
            } else {
                componentInfo.setMode(MODE_INLINE);
            }
        } else {
            componentInfo.setMode(MODE_INLINE);
        }
        componentInfo.setVariableString(fieldKey.toString());
        return componentInfo;
    }

    public static ComponentInfo newInstance(IncludeUI includeUI) {
        IncludeKey includeKey = includeUI.getIncludeKey();
        ComponentInfo componentInfo = new ComponentInfo();
        componentInfo.setVariableString(includeKey.toString());
        if ((includeKey.isThesaurusInclude()) || (includeKey.isAlbumInclude())) {
            componentInfo.setMode(MODE_INLINE);
        } else {
            componentInfo.setMode(MODE_LISTITEM);
        }
        LimitOption limitOption = includeUI.getLimitOption();
        if (limitOption != null) {
            componentInfo.setForceWithIf(limitOption.isNoNewAllowed());
        }
        return componentInfo;
    }

    public boolean isInLine() {
        return (mode == MODE_INLINE);
    }

    public short getMode() {
        return mode;
    }

    public void setMode(short mode) {
        this.mode = mode;
    }

    public String getVariableString() {
        return variableString;
    }

    public void setVariableString(String variableString) {
        this.variableString = variableString;
    }

    public boolean isForceWithIf() {
        return forceWithIf;
    }

    public void setForceWithIf(boolean forceWithIf) {
        this.forceWithIf = forceWithIf;
    }

}
