/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;


/**
 *
 * @author Vincent Calame
 */
public abstract class FichePattern extends PatternBuffer {

    private boolean uniqueLang;
    private boolean previousDiv;
    private boolean withIf = false;
    private boolean multiple = false;
    private boolean currentForceWithIf = false;
    private BdfServer bdfServer;

    public FichePattern() {
    }

    public void appendPatternStart(String name) {
        appendResource(name);
    }

    public void setWithIf(boolean withIf) {
        this.withIf = withIf;
    }

    public void setBdfServer(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        this.uniqueLang = langTest();
    }

    public void setPreviousDiv(boolean previousDiv) {
        this.previousDiv = previousDiv;
    }

    public abstract boolean insertComment();

    public abstract void appendEntete(boolean withSoustitre);

    public boolean isPreviousDiv() {
        return previousDiv;
    }

    public void testNewLine() {
        if (!previousDiv) {
            newLine();
        }
        previousDiv = true;
    }

    public void appendCorpusPattern(Corpus corpus) {
        appendComponentUIListPattern(corpus.getSubsetKey(), bdfServer.getUIManager().getComponentUIList(corpus, null));
    }

    private void appendComponentUIListPattern(SubsetKey corpusKey, ComponentUIList componentUIList) {
        int count = componentUIList.getComponentUICount();
        boolean withSoustitre = false;
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof FieldUI) {
                FieldKey fieldKey = ((FieldUI) componentUI).getCorpusField().getFieldKey();
                if (fieldKey.equals(FieldKey.SOUSTITRE)) {
                    withSoustitre = true;
                    break;
                }
            }
        }
        newLine();
        newLine();
        appendSimpleInstruction("fiche", corpusKey.getSubsetName());
        newLine();
        newLine();
        appendEntete(withSoustitre);
        newLine();
        setPreviousDiv(true);
        int p = 0;
        for (int i = 0; i < count; i++) {
            ComponentUI obj = componentUIList.getComponentUI(i);
            if (obj instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) obj;
                FieldKey fieldKey = fieldUI.getCorpusField().getFieldKey();
                if ((fieldKey.equals(FieldKey.TITRE)) || (fieldKey.equals(FieldKey.SOUSTITRE))) {
                    continue;
                } else if (fieldKey.equals(FieldKey.LANG)) {
                    if (!uniqueLang) {
                        ComponentInfo componentInfo = ComponentInfo.newInstance(fieldUI.getCorpusField());
                        appendComponentInfo(componentInfo);
                    } else {
                        continue;
                    }
                } else {
                    ComponentInfo componentInfo = ComponentInfo.newInstance(fieldUI.getCorpusField());
                    appendComponentInfo(componentInfo);
                }
            } else if (obj instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) obj;
                ComponentInfo componentInfo = ComponentInfo.newInstance(includeUI);
                appendComponentInfo(componentInfo);
            } else if (obj instanceof CommentUI) {
                if (insertComment()) {
                    CommentUI commentUI = (CommentUI) obj;
                    if ((commentUI.getLocation() & CommentUI.AFFICHAGE_BITVALUE) != 0) {
                        testNewLine();
                        appendInstruction("comment_" + commentUI.getName());
                        newLine();
                        newLine();
                    }
                }
            }
            p++;
            if (p == 50) {
                testNewLine();
                appendInstruction("fichesection");
                newLine();
                newLine();
                p = 0;
            }
        }
        newLine();
        appendChrono();
    }

    public abstract void appendChrono();

    public abstract void appendInLine(ComponentInfo componentInfo);

    public abstract void appendBlock(ComponentInfo componentInfo);

    public abstract void appendListItem(ComponentInfo componentInfo);

    public abstract void appendTexte(ComponentInfo componentInfo);

    @Override
    public void startIf(String clause) {
        if ((!withIf) && (!currentForceWithIf)) {
            return;
        }
        super.startIf(clause);
    }

    @Override
    public void endIf() {
        if ((!withIf) && (!currentForceWithIf)) {
            return;
        }
        super.endIf();
    }

    private void appendComponentInfo(ComponentInfo componentInfo) {
        String variableString = componentInfo.getVariableString();
        boolean old = currentForceWithIf;
        currentForceWithIf = componentInfo.isForceWithIf();
        short mode = componentInfo.getMode();
        switch (mode) {
            case ComponentInfo.MODE_INLINE:
                startIf(variableString);
                appendInLine(componentInfo);
                endIf();
                setPreviousDiv(false);
                break;
            case ComponentInfo.MODE_BLOCK:
                testNewLine();
                startIf(variableString);
                appendBlock(componentInfo);
                endIf();
                break;
            case ComponentInfo.MODE_LISTITEM:
                testNewLine();
                startIf(variableString);
                appendListItem(componentInfo);
                endIf();
                break;
            case ComponentInfo.TEXTE:
                testNewLine();
                startIf(variableString);
                appendTexte(componentInfo);
                endIf();
                break;
        }
        currentForceWithIf = old;
    }

    private boolean langTest() {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        int[] supplementaryArray = langConfig.getSupplementaryLangIntegerArray();
        if ((!langConfig.isAllLanguages()) && (workingArray.length + supplementaryArray.length == 1)) {
            return true;
        } else {
            return false;
        }
    }

    public boolean isMultiple() {
        return multiple;
    }

    public void setMultiple(boolean multiple) {
        this.multiple = multiple;
    }

}
