/* BdfServer - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.xsltfactory.XSLTFactoryConstants;
import fr.exemole.bdfserver.xsltfactory.XSLTFactoryUtils;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.util.exceptions.SwitchException;


/**
 *
 * @author Vincent Calame
 */
public class XSLTPatternBuilder implements XSLTFactoryConstants {

    private XSLTPatternBuilder() {
    }

    /**
     * Retourne null si le corpus est inconnu.
     */
    public static String getXSLTPattern(BdfServer bdfServer, TransformationKey transformationKey, boolean withIf, short formatType) {
        if (transformationKey.isCorpusTransformationKey()) {
            FichePattern fichePattern = getFichePattern(formatType);
            fichePattern.setBdfServer(bdfServer);
            fichePattern.setWithIf(withIf);
            fichePattern.appendResource(getFichePatternStart("fiche", formatType));
            fichePattern.setMultiple(false);
            SubsetKey corpusKey = transformationKey.toCorpusKey();
            Corpus corpus = (Corpus) bdfServer.getBasedefiches().getSubset(corpusKey);
            if (corpus == null) {
                return null;
            }
            fichePattern.appendCorpusPattern(corpus);
            return fichePattern.toString();
        } else if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
            FichePattern fichePattern = getFichePattern(formatType);
            fichePattern.setBdfServer(bdfServer);
            fichePattern.setWithIf(withIf);
            fichePattern.setMultiple(false);
            fichePattern.appendResource(getFichePatternStart("compilation", formatType));
            Basedefiches basedefiches = bdfServer.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                fichePattern.appendCorpusPattern(corpus);
            }
            return fichePattern.toString();
        } else {
            ResourceFichePattern resourceFichePattern = new ResourceFichePattern(getResourceName(transformationKey, formatType));
            return resourceFichePattern.toString();
        }
    }

    private static String getFichePatternStart(String prefix, short formatType) {
        String name = prefix + XSLTFactoryUtils.toFormatTypeSuffix(formatType) + "_patternstart.txt";
        return name;
    }

    private static FichePattern getFichePattern(short formatType) {
        switch (formatType) {
            case XSL_FORMAT:
                return new XslFichePattern();
            case ODT_FORMAT:
                return new OdtFichePattern();
            default:
                throw new SwitchException("formatType = " + formatType);

        }
    }

    private static String getResourceName(TransformationKey transformationKey, short formatType) {
        String prefix = getPrefix(transformationKey);
        String name = prefix + XSLTFactoryUtils.toFormatTypeSuffix(formatType) + ".txt";
        return name;
    }

    private static String getPrefix(TransformationKey transformationKey) {
        if (transformationKey.equals(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY)) {
            return "statthesaurus";
        } else if (transformationKey.equals(TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY)) {
            return "listepermutee";
        } else {
            throw new SwitchException("unknown tranformationKey = " + transformationKey.toString());
        }
    }

}

