/* BdfServer - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern;


/**
 *
 * @author Vincent Calame
 */
public class XslFichePattern extends FichePattern {

    public XslFichePattern() {
    }

    public boolean insertComment() {
        return true;
    }

    public void appendEntete(boolean withSoustitre) {
        openDiv("entete");
        boolean mult = isMultiple();
        String tagName = (mult) ? "h2" : "h1";
        openTag(tagName, null, false);
        appendModeInstruction("titre", "_intitule", null, null);
        closeTag(tagName, true);
        if (withSoustitre) {
            startIf("soustitre");
            openP("soustitre");
            appendInstruction("soustitre");
            closeP();
            endIf();
        }
        closeDiv();
    }

    public void appendChrono() {
        openP("chrono");
        appendLibInstruction("date_creation");
        append(" : ");
        appendInstruction("date_creation");
        append(" — ");
        appendLibInstruction("date_modification");
        append(" : ");
        appendInstruction("date_modification");
        closeP();
    }

    public void appendInLine(ComponentInfo componentInfo) {
        String variableString = componentInfo.getVariableString();
        openP("champ");
        openSpan("intitule");
        appendLibInstruction(variableString);
        append(" : ");
        closeSpan();
        appendInstruction(variableString);
        closeP();
    }

    public void appendListItem(ComponentInfo componentInfo) {
        String variableString = componentInfo.getVariableString();
        appendIntituleP(variableString, "intitule");
        openUl("block");
        appendModeInstruction(variableString, "_listitem", null, null);
        closeUl();
    }

    public void appendBlock(ComponentInfo componentInfo) {
        appendBlock(componentInfo.getVariableString(), "intitule", "block", "_block");
    }

    public void appendTexte(ComponentInfo componentInfo) {
        appendBlock(componentInfo.getVariableString(), "intituletexte", "texte", null);
    }

    private void appendBlock(String variableString, String className, String divClassName, String mode) {
        appendIntituleP(variableString, className);
        openDiv(divClassName);
        if (mode == null) {
            appendInstruction(variableString);
        } else {
            appendModeInstruction(variableString, mode, null, null);
        }
        newLine();
        closeDiv();
    }

    private void appendIntituleP(String variableString, String className) {
        openP(className);
        appendLibInstruction(variableString);
        append(" :");
        closeP();
    }

    public void openP(String className) {
        openHtmlP(className);
    }

    public void closeP() {
        closeHtmlP();
    }

    public void openSpan(String className) {
        openHtmlSpan(className);
    }

    public void closeSpan() {
        closeHtmlSpan();
    }

    public void openUl(String className) {
        openHtmlUl(className);
    }

    public void closeUl() {
        closeHtmlUl();
    }

}
