/* BdfServer - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xsltfactory.xslpattern.bih;

import fr.exemole.bdfserver.xsltfactory.xslpattern.InsertionUnitTranslator;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.include.IncludeUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.insertpattern.api.Argument;
import net.mapeadores.util.text.insertpattern.api.InsertionUnit;
import net.mapeadores.util.text.insertpattern.api.Instruction;
import net.mapeadores.util.xml.PatternXSLTResolver;
import net.mapeadores.util.xml.XSLTWriter;


/**
 *
 * @author Vincent Calame
 */
public class FicheFieldAndIncludeTranslator implements InsertionUnitTranslator {

    private PatternXSLTResolver xsltResolver;
    private int ficheIndex;
    private Map<Integer, Object> itemMap = new HashMap<Integer, Object>();
    private FicheSection firstFicheSection = new FicheSection(0);
    private FicheSection currentFicheSection = firstFicheSection;
    private String corpusSubId;
    private String mode = null;
    private boolean insertLibsVars = false;

    public FicheFieldAndIncludeTranslator(PatternXSLTResolver xsltResolver, int ficheIndex) {
        this.xsltResolver = xsltResolver;
        this.ficheIndex = ficheIndex;
    }

    public void setInsertLibsVar(boolean insertLibsVars) {
        this.insertLibsVars = insertLibsVars;
    }

    public void translate(InsertionUnit insertionUnit) throws IOException {
        int index = insertionUnit.getIndex();
        if (index == ficheIndex) {
            resolveFiche(insertionUnit);
        } else {
            Object obj = itemMap.get(index);
            if (obj == null) {
                XSLTWriter xw = xsltResolver.getCurrentXSLTWriter();
                xw.append("#UNKNOWN INSTRUCTION : {");
                xw.append(insertionUnit.getInstructionArray()[0].getArgument(0).getKey());
                xw.append(" ...} #");
            } else if (obj instanceof FicheSection) {
                resolveSection((FicheSection) obj);
            } else {
                KeyParam keyParam = (KeyParam) obj;
                if (keyParam.hasWithParam()) {
                    XSLTWriter currentXSLTWriter = xsltResolver.getCurrentXSLTWriter();
                    currentXSLTWriter.openXslApplyTemplates(keyParam.getPath(), keyParam.getMode());
                    keyParam.addParameters(currentXSLTWriter);
                    currentXSLTWriter.closeXslApplyTemplates();
                } else {
                    xsltResolver.getCurrentXSLTWriter().addXslApplyTemplates(keyParam.getPath(), keyParam.getMode());
                }
            }
        }
    }

    private void resolveFiche(InsertionUnit insertionUnit) throws IOException {
        Instruction[] instructionArray = insertionUnit.getInstructionArray();
        StringBuffer xpathBuffer = new StringBuffer();
        xpathBuffer.append("fiche");
        corpusSubId = instructionArray[0].getArgument(0).getValue();
        if ((corpusSubId != null) && (corpusSubId.length() > 0)) {
            xpathBuffer.append("[");

            xpathBuffer.append("@corpus='");
            xpathBuffer.append(corpusSubId);
            xpathBuffer.append("'");

            xpathBuffer.append("]");
        } else {
            corpusSubId = "";
        }
        int length = instructionArray.length;
        for (int i = 1; i < length; i++) {
            Instruction instruction = instructionArray[i];
            Argument argument0 = instruction.getArgument(0);
            String firstKey = argument0.getKey();
            if (firstKey.equals("mode")) {
                mode = argument0.getValue();
            }
        }
        XSLTWriter currentXSLTWriter = xsltResolver.getCurrentXSLTWriter();
        currentXSLTWriter.openMatchXslTemplate(xpathBuffer.toString(), mode);
        addCommonVars(currentXSLTWriter, firstFicheSection);

    }

    private void resolveSection(FicheSection ficheSection) throws IOException {
        XSLTWriter currentXSLTWriter = xsltResolver.getCurrentXSLTWriter();
        String name = "subtemplate_" + corpusSubId + "_" + String.valueOf(ficheSection.getFicheSectionIndex());
        currentXSLTWriter.addXslCallTemplate(name, mode);
        currentXSLTWriter.closeXslTemplate();
        currentXSLTWriter.openNameXslTemplate(name, mode);
        addCommonVars(currentXSLTWriter, ficheSection);
    }

    private void addCommonVars(XSLTWriter currentXSLTWriter, FicheSection ficheSection) throws IOException {
        StringBuffer buf = new StringBuffer();
        if (insertLibsVars) {
            buf.append("$LIBSCOL/libs[");
            if (corpusSubId.length() > 0) {
                buf.append("@corpus='");
                buf.append(corpusSubId);
                buf.append("' and ");
            }
            String start = buf.toString();
            currentXSLTWriter.addXslVariable("libs_metadata", start + "@category='metadata']", false);
            currentXSLTWriter.addXslVariable("libs_corpusui", start + "@category='corpusui']", false);
        }
    }

    public boolean isTemplateInstruction(InsertionUnit insertionUnit) {
        if (insertionUnit.getIndex() == ficheIndex) {
            return true;
        } else {
            return false;
        }
    }

    public void addKey(int index, Instruction[] instructionArray) {
        Instruction instruction = instructionArray[0];
        Set<Object> keySet = getKeySet(instruction);
        if (keySet.size() == 0) {
            return;
        }
        KeyParam keyParam = new KeyParam(keySet);
        if (instructionArray.length > 1) {
            Instruction secondInstruction = instructionArray[1];
            int lg2 = secondInstruction.getLength();
            for (int i = 0; i < lg2; i++) {
                Argument argument = secondInstruction.getArgument(i);
                String key = argument.getKey();
                String value = BihUtils.checkValue(argument.getValue());
                if (key.equals("mode")) {
                    keyParam.setMode(value);
                } else if (key.equals("class")) {
                    keyParam.setClassName(value);
                } else if (key.equals("sep")) {
                    keyParam.setSep(value);
                }
            }
        }
        itemMap.put(index, keyParam);
    }

    public void checkFicheSection(int index) {
        FicheSection ficheSection = new FicheSection(currentFicheSection.getFicheSectionIndex() + 1);
        currentFicheSection = ficheSection;
        itemMap.put(index, ficheSection);
    }

    private Set<Object> getKeySet(Instruction instruction) {
        LinkedHashSet<Object> result = new LinkedHashSet<Object>();
        int length = instruction.getLength();
        for (int i = 0; i < length; i++) {
            Object key = IncludeUtils.parseKey(instruction.getArgument(i).getKey());
            if (key != null) {
                if (key instanceof FieldKey) {
                    FieldKey fieldKey = (FieldKey) key;
                    if (fieldKey.ownsToCorpsdefiche()) {
                        continue;
                    } else if (fieldKey.isSpecialFieldKey()) {
                        if ((!fieldKey.equals(FieldKey.REDACTEURS)) && (!fieldKey.equals(FieldKey.LANG)) && (!fieldKey.equals(FieldKey.SOUSTITRE))) {
                            continue;
                        }
                    }
                }
                result.add(key);
            }
        }
        return result;
    }


    private class KeyParam {

        private Set<Object> keySet;
        private String mode = "MODE_INLINE"; //mode par défaut
        private String sep;
        private String className;
        private FicheSection ficheSection;

        KeyParam(Set<Object> keySet) {
            this.ficheSection = currentFicheSection;
            this.keySet = keySet;
        }

        void setMode(String mode) {
            this.mode = BihUtils.checkMode(mode);
        }

        void setSep(String sep) {
            this.sep = sep;
        }

        void setClassName(String className) {
            this.className = className;
        }

        public String getMode() {
            return mode;
        }

        public String getSep() {
            return sep;
        }

        public String getClassName() {
            return className;
        }

        public boolean hasWithParam() {
            return ((className != null) || (sep != null));
        }

        void addParameters(XSLTWriter currentXSLTWriter) throws IOException {
            if (className != null) {
                currentXSLTWriter.addXslWithParam("PARAM_CLASS", className, true);
            }
            if (sep != null) {
                currentXSLTWriter.addXslWithParam("PARAM_SEP", sep, true);
            }
        }

        String getPath() {
            if (keySet.size() == 1) {
                return getUniquePath();
            }
            StringBuilder pathBuffer = new StringBuilder();
            boolean premier = true;
            for (Object obj : keySet) {
                if (premier) {
                    premier = false;
                } else {
                    pathBuffer.append(" | ");
                }
                appendPath(obj, pathBuffer, true);
            }
            return pathBuffer.toString();
        }

        private void appendPath(Object obj, StringBuilder pathBuffer, boolean withChildren) {
            if (obj instanceof FieldKey) {
                BihUtils.appendPath(pathBuffer, (FieldKey) obj, false, withChildren);
            } else if (obj instanceof IncludeKey) {
                IncludeKey includeKey = (IncludeKey) obj;
                if (includeKey.isSubsetInclude()) {
                    BihUtils.appendSubsetIncludeKey(pathBuffer, includeKey, withChildren);
                } else {
                    if (includeKey.isChronoInclude()) {
                        BihUtils.appendChronoIncludeKey(pathBuffer, includeKey);
                    } else if (includeKey.isSpecialInclude()) {
                        if (includeKey.equals(IncludeKey.ASSOCIATION)) {
                            BihUtils.appendAssocation(pathBuffer);
                        } else if (includeKey.equals(IncludeKey.LIAGE)) {
                            pathBuffer.append("extraitcorpus[@extrait-name='liage']/fiche");
                        } else if (includeKey.equals(IncludeKey.RATTACHEMENT)) {
                            BihUtils.appendRattachementPath(pathBuffer, withChildren);
                        } else {
                            throw new SwitchException("unknown special key = " + obj.toString());
                        }
                    }
                }
            } else {
                throw new SwitchException("unknown key = " + obj.toString());
            }
        }

        private String getUniquePath() {
            StringBuilder pathBuffer = new StringBuilder();
            Object key = keySet.iterator().next();
            appendPath(key, pathBuffer, false);
            return pathBuffer.toString();
        }

    }


    private class FicheSection {

        int ficheSectionIndex;

        FicheSection(int ficheSectionIndex) {
            this.ficheSectionIndex = ficheSectionIndex;
        }

        public int getFicheSectionIndex() {
            return ficheSectionIndex;
        }

    }

}
