/* BdfServer - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.zip;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.ScrutariExportManager;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.managers.xml.ConfigurationManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.ManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PermissionManagerXMLWriter;
import fr.exemole.bdfserver.managers.xml.PolicyManagerXMLWriter;
import fr.exemole.bdfserver.roles.Role;
import fr.exemole.bdfserver.roles.xml.RoleXMLWriter;
import fr.exemole.bdfserver.managers.SqlExportManager;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceList;
import fr.exemole.bdfserver.storage.BdfResourceStorage;
import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.StorageUtils;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.xml.CorpusUiXMLWriter;
import fr.exemole.bdfserver.ui.xml.TreesXMLWriter;
import fr.exemole.bdfserver.users.BdfUserDef;
import fr.exemole.bdfserver.users.xml.UserXMLWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import net.mapeadores.basedefiches.AlbumList;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.CorpusList;
import net.mapeadores.basedefiches.SphereList;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.ThesaurusList;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.filters.SubsetFilter;
import net.mapeadores.basedefiches.filters.SubsetItemFilter;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.selection.util.MutableAlbumList;
import net.mapeadores.basedefiches.selection.util.MutableCorpusList;
import net.mapeadores.basedefiches.selection.util.MutableSphereList;
import net.mapeadores.basedefiches.selection.util.MutableThesaurusList;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.xml.*;
import net.mapeadores.basedefiches.xml.extraction.FicheStorageXMLWriter;
import net.mapeadores.basedefiches.xml.scrutariexport.ScrutariExportXMLPart;
import net.mapeadores.basedefiches.xml.sqlexport.SqlExportXMLPart;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class BdfServerZip {

    private SubsetFilter subsetFilter;
    private BdfServer bdfServer;
    private Basedefiches basedefiches;
    private CorpusList corpusList;
    private ThesaurusList thesaurusList;
    private SphereList sphereList;
    private AlbumList albumList;
    private String racineBasedefiches = "var/bdfdata/basedefiches/";
    private String racineConf = "var/bdfdata/conf/";
    private String racineRoles = "var/bdfdata/roles/";
    private String racineUsers = "var/bdfdata/users/";
    private String racineResources = "var/rscdata/";
    private boolean withAddenda = false;
    private ZipOutputStream zipOutputStream;
    private BufferedWriter bufWriter;

    public BdfServerZip(BdfServer bdfServer, SubsetFilter subsetFilter) {
        this.bdfServer = bdfServer;
        this.subsetFilter = subsetFilter;
        init(bdfServer.getBasedefiches());
    }

    public void zip(ZipOutputStream zipOutputStream) throws IOException {
        this.bufWriter = new BufferedWriter(new OutputStreamWriter(zipOutputStream, "UTF-8"));
        this.zipOutputStream = zipOutputStream;
        zipBasedefiches();
        zipManager();
        zipRoles();
        zipUsers();
        zipResources();
    }

    private void init(Basedefiches basedefiches) {
        this.basedefiches = basedefiches;
        if (subsetFilter == null) {
            corpusList = basedefiches;
            thesaurusList = basedefiches;
            sphereList = basedefiches;
            albumList = basedefiches;
            withAddenda = true;
        } else {
            withAddenda = subsetFilter.containsSubset(SubsetKey.ADDENDA);
            MutableThesaurusList mutThesaurusList = new MutableThesaurusList();
            int thesaurusCount = basedefiches.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = basedefiches.getThesaurus(i);
                if (subsetFilter.containsSubset(thesaurus.getSubsetKey())) {
                    mutThesaurusList.add(thesaurus);
                }
            }
            MutableSphereList mutSphereList = new MutableSphereList();
            int sphereCount = basedefiches.getSphereCount();
            for (int i = 0; i < sphereCount; i++) {
                Sphere sphere = basedefiches.getSphere(i);
                if (subsetFilter.containsSubset(sphere.getSubsetKey())) {
                    mutSphereList.add(sphere);
                }
            }
            MutableCorpusList mutCorpusList = new MutableCorpusList();
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                if (subsetFilter.containsSubset(corpus.getSubsetKey())) {
                    Subset masterSubset = corpus.getMasterSubset();
                    if ((masterSubset != null) && (!subsetFilter.containsSubset(masterSubset.getSubsetKey()))) {
                        continue;
                    }
                    mutCorpusList.add(corpus);
                }
            }
            MutableAlbumList mutAlbumList = new MutableAlbumList();
            int albumCount = basedefiches.getAlbumCount();
            for (int i = 0; i < albumCount; i++) {
                Album album = basedefiches.getAlbum(i);
                if (subsetFilter.containsSubset(album.getSubsetKey())) {
                    mutAlbumList.add(album);
                }
            }
            corpusList = mutCorpusList;
            thesaurusList = mutThesaurusList;
            sphereList = mutSphereList;
            albumList = mutAlbumList;
        }
    }

    private void zipResources() throws IOException {
        BdfResourceStorage resourceStorage = bdfServer.getBdfResourceStorage();
        zipResources(resourceStorage.getVarBdfResourceList(), racineResources);
    }

    private void zipResources(BdfResourceList bdfResourceList, String rscPath) throws IOException {
        BdfResourceStorage resourceStorage = bdfServer.getBdfResourceStorage();
        int resourceCount = bdfResourceList.getResourceCount();
        for (int i = 0; i < resourceCount; i++) {
            short resourceType = bdfResourceList.getResourceType(i);
            String resourcePath = bdfResourceList.getResourcePath(i);
            zipOutputStream.putNextEntry(new ZipEntry(getParentPath(rscPath, resourceType) + resourcePath));
            DocStream docStream = resourceStorage.getBdfResourceDocStream(resourceType, resourcePath);
            InputStream inputStream = docStream.getInputStream();
            IOUtils.copy(inputStream, zipOutputStream);
            inputStream.close();
        }
    }

    private String getParentPath(String rscPath, short resourceType) {
        switch (resourceType) {
            case BdfResourceConstants.JQUERY_RESOURCE:
                return rscPath + "_jquery/";
            case BdfResourceConstants.JS_RESOURCE:
                return rscPath + "_js/";
            case BdfResourceConstants.THEME_RESOURCE:
                return rscPath + "_theme/";
            case BdfResourceConstants.CSS_RESOURCE:
                return rscPath + "_css/";
            case BdfResourceConstants.OTHER_RESOURCE:
            default:
                return rscPath;
        }
    }

    private void zipRoles() throws IOException {
        Role[] roleArray = bdfServer.getPermissionManager().getRoleArray(true);
        RoleXMLWriter roleXMLWriter = new RoleXMLWriter();
        roleXMLWriter.setAppendable(bufWriter);
        int roleCount = roleArray.length;
        for (int i = 0; i < roleCount; i++) {
            Role role = roleArray[i];
            zipOutputStream.putNextEntry(new ZipEntry(racineRoles + role.getName() + ".xml"));
            roleXMLWriter.setIndentLength(0);
            roleXMLWriter.appendXMLDeclaration();
            roleXMLWriter.appendRole(role);
            bufWriter.flush();
        }
    }

    private void zipUsers() throws IOException {
        BdfStorage.BdfUserStorage bdfUserStorage = bdfServer.getBdfStorage().getBdfUserStorage();
        UserXMLWriter userXMLWriter = new UserXMLWriter();
        userXMLWriter.setAppendable(bufWriter);
        int sphereCount = sphereList.getSphereCount();
        for (int i = 0; i < sphereCount; i++) {
            Sphere sphere = sphereList.getSphere(i);
            int ticketLength = sphere.getTicketLength();
            for (int j = 0; j < ticketLength; j++) {
                Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(j);
                if (redacteur != null) {
                    RedacteurKey redacteurKey = redacteur.getRedacteurKey();
                    BdfUserDef bdfUserDef = bdfUserStorage.getBdfUserDef(redacteurKey);
                    if (bdfUserDef != null) {
                        zipOutputStream.putNextEntry(new ZipEntry(racineUsers + redacteurKey.toString(RedacteurKey.UNDERSCORE_STYLE) + ".xml"));
                        userXMLWriter.setIndentLength(0);
                        userXMLWriter.appendXMLDeclaration();
                        userXMLWriter.appendUser(bdfUserDef);
                        bufWriter.flush();
                    }
                }
            }
        }
    }

    private void zipManager() throws IOException {
        XMLWriter xmlWriter = XMLUtils.getXMLWriter(bufWriter, 0);
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        zipManager("configurationmanager.xml", new ConfigurationManagerXMLWriter(), configurationManager);
        PasswordManager passwordManager = bdfServer.getPasswordManager();
        String[] passwordArray = passwordManager.toPasswordArray();
        if (passwordArray.length > 0) {
            zipOutputStream.putNextEntry(new ZipEntry(racineConf + "passwd"));
            StorageUtils.writePasswordArray(bufWriter, passwordArray);
            bufWriter.flush();
        }
        PermissionManager permissionManager = bdfServer.getPermissionManager();
        zipManager("permissionmanager.xml", new PermissionManagerXMLWriter(), permissionManager);
        PolicyManager policyManager = bdfServer.getPolicyManager();
        zipManager("policymanager.xml", new PolicyManagerXMLWriter(), policyManager);
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        int sqlExportDefCount = sqlExportManager.getSqlExportDefCount();
        if (sqlExportDefCount > 0) {
            SqlExportXMLPart sqlExportXMLPart = new SqlExportXMLPart(xmlWriter);
            for (int i = 0; i < sqlExportDefCount; i++) {
                SqlExportDef sqlExportDef = sqlExportManager.getSqlExportDef(i);
                zipOutputStream.putNextEntry(new ZipEntry(racineConf + "sqlexport/" + sqlExportDef.getName() + ".xml"));
                sqlExportXMLPart.appendXMLDeclaration();
                sqlExportXMLPart.addSqlExportDef(sqlExportDef);
                bufWriter.flush();
            }
        }
        ScrutariExportManager scrutariExportManager = bdfServer.getScrutariExportManager();
        int scrutariExportCount = scrutariExportManager.getScrutariExportDefCount();
        if (scrutariExportCount > 0) {
            ScrutariExportXMLPart scrutariExportXMLPart = new ScrutariExportXMLPart(xmlWriter);
            for (int i = 0; i < scrutariExportCount; i++) {
                ScrutariExportDef scrutariExportDef = scrutariExportManager.getScrutariExportDef(i);
                zipOutputStream.putNextEntry(new ZipEntry(racineConf + "scrutariexport/" + scrutariExportDef.getName() + ".xml"));
                scrutariExportXMLPart.appendXMLDeclaration();
                scrutariExportXMLPart.addScrutariExportDef(scrutariExportDef);
                bufWriter.flush();
            }
        }
        UIManager uiManager = bdfServer.getUIManager();
        zipOutputStream.putNextEntry(new ZipEntry(racineConf + "ui/trees.xml"));
        TreesXMLWriter treesXMLWriter = new TreesXMLWriter();
        treesXMLWriter.setAppendable(bufWriter);
        treesXMLWriter.setIndentLength(0);
        treesXMLWriter.appendXMLDeclaration();
        treesXMLWriter.appendTrees(uiManager);
        bufWriter.flush();
        CorpusUiXMLWriter corpusUiXMLWriter = new CorpusUiXMLWriter();
        corpusUiXMLWriter.setAppendable(bufWriter);
        int corpusCount = corpusList.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = corpusList.getCorpus(i);
            CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
            zipOutputStream.putNextEntry(new ZipEntry(racineConf + "ui/" + corpus.getSubsetKey().toString() + ".xml"));
            corpusUiXMLWriter.setIndentLength(0);
            corpusUiXMLWriter.appendXMLDeclaration();
            corpusUiXMLWriter.appendCorpusUI(corpusUI);
            bufWriter.flush();
        }
    }

    private void zipManager(String fileName, ManagerXMLWriter managerXMLWriter, Object obj) throws IOException {
        zipOutputStream.putNextEntry(new ZipEntry(racineConf + fileName));
        managerXMLWriter.setAppendable(bufWriter);
        managerXMLWriter.setIndentLength(0);
        managerXMLWriter.appendXMLDeclaration();
        managerXMLWriter.appendManager(obj);
        bufWriter.flush();
    }

    private void zipBasedefiches() throws IOException {
        zipOutputStream.putNextEntry(new ZipEntry(racineBasedefiches + "metadata.xml"));
        BasedefichesMetadataXMLWriter basedefichesMetadataXMLWriter = new BasedefichesMetadataXMLWriter();
        basedefichesMetadataXMLWriter.setAppendable(bufWriter);
        basedefichesMetadataXMLWriter.setIndentLength(0);
        basedefichesMetadataXMLWriter.appendXMLDeclaration();
        basedefichesMetadataXMLWriter.appendBasedefichesMetadata(basedefiches.getBasedefichesMetadata());
        bufWriter.flush();
        int sphereCount = sphereList.getSphereCount();
        SphereXMLWriter sphereXMLWriter = new SphereXMLWriter();
        for (int i = 0; i < sphereCount; i++) {
            Sphere sphere = sphereList.getSphere(i);
            SubsetItemFilter subsetItemFilter = getSubsetItemFilter(sphere);
            zipOutputStream.putNextEntry(new ZipEntry(racineBasedefiches + "sphere/" + sphere.getSubsetKey().getSubsetName() + ".xml"));
            sphereXMLWriter.setAppendable(bufWriter);
            sphereXMLWriter.setIndentLength(0);
            sphereXMLWriter.appendXMLDeclaration();
            sphereXMLWriter.appendSphere(sphere, subsetItemFilter);
            bufWriter.flush();
        }
        int thesaurusCount = thesaurusList.getThesaurusCount();
        ThesaurusXMLWriter thesaurusXMLWriter = new ThesaurusXMLWriter();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = thesaurusList.getThesaurus(i);
            SubsetItemFilter subsetItemFilter = getSubsetItemFilter(thesaurus);
            zipOutputStream.putNextEntry(new ZipEntry(racineBasedefiches + "thesaurus/" + thesaurus.getSubsetKey().getSubsetName() + ".xml"));
            thesaurusXMLWriter.setAppendable(bufWriter);
            thesaurusXMLWriter.setIndentLength(0);
            thesaurusXMLWriter.appendXMLDeclaration();
            thesaurusXMLWriter.appendThesaurus(thesaurus, subsetItemFilter);
            bufWriter.flush();
        }
        int albumCount = albumList.getAlbumCount();
        AlbumXMLWriter albumXMLWriter = new AlbumXMLWriter();
        for (int i = 0; i < albumCount; i++) {
            Album album = albumList.getAlbum(i);
            SubsetItemFilter subsetItemFilter = getSubsetItemFilter(album);
            String subsetName = album.getSubsetKey().getSubsetName();
            zipOutputStream.putNextEntry(new ZipEntry(racineBasedefiches + "album/" + subsetName + ".xml"));
            albumXMLWriter.setAppendable(bufWriter);
            albumXMLWriter.setIndentLength(0);
            albumXMLWriter.appendXMLDeclaration();
            albumXMLWriter.appendAlbum(album);
            bufWriter.flush();
            int imageCount = album.getTicketLength();
            for (int j = 0; j < imageCount; j++) {
                Illustration illustration = (Illustration) album.getSubsetItemByTicket(j);
                if (testSubsetItem(illustration, subsetItemFilter)) {
                    int idalbum = illustration.getIdsubset();
                    String illustrationracine = racineBasedefiches + "album/" + subsetName + "/i." + getMillier(idalbum) + "/" + idalbum + "/";
                    zipOutputStream.putNextEntry(new ZipEntry(illustrationracine + "img." + AlbumUtils.formatTypeToString(illustration.getFormatType())));
                    InputStream imageStream = illustration.getInputStream(AlbumConstants.ORIGINAL_SPECIALDIM);
                    IOUtils.copy(imageStream, zipOutputStream);
                    imageStream.close();
                }
            }
        }
        if (withAddenda) {
            Addenda addenda = (Addenda) basedefiches.getSubset(SubsetKey.ADDENDA);
            if (addenda.size() > 0) {
                SubsetItemFilter subsetItemFilter = getSubsetItemFilter(addenda);
                zipOutputStream.putNextEntry(new ZipEntry(racineBasedefiches + "addenda/x.xml"));
                AddendaXMLWriter addendaXMLWriter = new AddendaXMLWriter();
                addendaXMLWriter.setAppendable(bufWriter);
                addendaXMLWriter.setIndentLength(0);
                addendaXMLWriter.appendXMLDeclaration();
                addendaXMLWriter.appendAddenda(addenda, subsetItemFilter);
                bufWriter.flush();
                int documentLength = addenda.getTicketLength();
                for (int i = 0; i < documentLength; i++) {
                    Document document = (Document) addenda.getSubsetItemByTicket(i);
                    if (testSubsetItem(document, subsetItemFilter)) {
                        int idaddenda = document.getIdaddenda();
                        String docracine = racineBasedefiches + "addenda/x/d." + getMillier(idaddenda) + "/" + idaddenda + "/";
                        int versionCount = document.getVersionCount();
                        for (int j = 0; j < versionCount; j++) {
                            Version version = document.getVersion(j);
                            zipOutputStream.putNextEntry(new ZipEntry(docracine + "doc." + version.getVersionName().getExtension()));
                            InputStream versionStream = version.getInputStream();
                            IOUtils.copy(versionStream, zipOutputStream);
                            versionStream.close();
                        }
                    }
                }
            }
        }
        int corpusCount = corpusList.getCorpusCount();
        CorpusMetadataXMLWriter corpusMetadataXMLWriter = new CorpusMetadataXMLWriter();
        ChronoXMLWriter chronoXMLWriter = new ChronoXMLWriter();
        FicheStorageXMLWriter ficheStorageXMLWriter = new FicheStorageXMLWriter();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = corpusList.getCorpus(i);
            SubsetItemFilter subsetItemFilter = getSubsetItemFilter(corpus);
            String corpusName = corpus.getSubsetKey().getSubsetName();
            String entryPath = racineBasedefiches + "corpus/";
            Subset masterSubset = corpus.getMasterSubset();
            if (masterSubset != null) {
                String masterSubsetName = masterSubset.getSubsetKey().getSubsetName();
                if (masterSubset instanceof Corpus) {
                    entryPath = entryPath + "_mc_" + masterSubsetName + "_" + corpusName + ".xml";
                } else if (masterSubset instanceof Thesaurus) {
                    entryPath = entryPath + "_mt_" + masterSubsetName + "_" + corpusName + ".xml";
                } else {
                    throw new UnsupportedOperationException("Unkwnown Master Subset implementation : " + masterSubset.getClass().getName());
                }
            } else {
                entryPath = entryPath + corpusName + ".xml";
            }
            zipOutputStream.putNextEntry(new ZipEntry(entryPath));
            corpusMetadataXMLWriter.setAppendable(bufWriter);
            corpusMetadataXMLWriter.setIndentLength(0);
            corpusMetadataXMLWriter.appendXMLDeclaration();
            corpusMetadataXMLWriter.appendCorpusMetadata(corpus.getCorpusMetadata());
            bufWriter.flush();
            if (corpus.size() > 0) {
                String chronoEntryPath = racineBasedefiches + "chrono/" + corpus.getSubsetKey().toString() + ".xml";
                zipOutputStream.putNextEntry(new ZipEntry(chronoEntryPath));
                chronoXMLWriter.setAppendable(bufWriter);
                chronoXMLWriter.setIndentLength(0);
                chronoXMLWriter.appendXMLDeclaration();
                chronoXMLWriter.appendChrono(corpus, subsetItemFilter);
                bufWriter.flush();
            }
            if (withAddenda) {
                appendCroisement(corpus, basedefiches.getSubset(SubsetKey.ADDENDA));
            }
            for (int j = 0; j < thesaurusCount; j++) {
                Thesaurus thesaurus = thesaurusList.getThesaurus(j);
                if ((masterSubset != null) && (masterSubset.equals(thesaurus))) {
                    continue;
                }
                appendCroisement(corpus, thesaurus);
            }
            for (int j = i; j < corpusCount; j++) {
                Corpus otherCorpus = corpusList.getCorpus(j);
                if ((masterSubset != null) && (masterSubset.equals(otherCorpus))) {
                    continue;
                }
                appendCroisement(corpus, otherCorpus);
            }
            for (int j = 0; j < albumCount; j++) {
                Album album = albumList.getAlbum(j);
                appendCroisement(corpus, album);
            }
            int ficheLength = corpus.getTicketLength();
            for (int j = 0; j < ficheLength; j++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                if (testSubsetItem(abregeFiche, subsetItemFilter)) {
                    int idcorpus = abregeFiche.getIdcorpus();
                    Fiche fiche = corpus.getFiche(abregeFiche);
                    String ficheEntryPath = racineBasedefiches + "corpus/" + corpusName + "/f." + getMillier(idcorpus) + "/" + idcorpus + ".xml";
                    zipOutputStream.putNextEntry(new ZipEntry(ficheEntryPath));
                    ficheStorageXMLWriter.setAppendable(bufWriter);
                    ficheStorageXMLWriter.setIndentLength(0);
                    ficheStorageXMLWriter.appendXMLDeclaration();
                    ficheStorageXMLWriter.appendFiche(fiche);
                    bufWriter.flush();
                }
            }
        }

    }

    private SubsetItemFilter getSubsetItemFilter(Subset subset) {
        if (subsetFilter == null) {
            return null;
        }
        return subsetFilter.getSubsetItemFilter(subset.getSubsetKey());
    }

    private SubsetItemFilter getSubsetItemFilter(SubsetKey subsetKey) {
        if (subsetFilter == null) {
            return null;
        }
        return subsetFilter.getSubsetItemFilter(subsetKey);
    }

    private boolean testSubsetItem(SubsetItem subsetItem, SubsetItemFilter subsetItemFilter) {
        if (subsetItem == null) {
            return false;
        }
        if (subsetItemFilter == null) {
            return true;
        }
        return subsetItemFilter.containsSubsetItem(subsetItem.getIdsubset());
    }

    private void appendCroisement(Subset subset1, Subset subset2) throws IOException {
        SubsetCouple subsetCouple = new SubsetCouple(subset1.getSubsetKey(), subset2.getSubsetKey());
        Croisement croisement = basedefiches.getCroisement(subsetCouple.getSubsetKey1(), subsetCouple.getSubsetKey2());
        if (croisement == null) {
            return;
        }
        SubsetItemFilter subsetItemFilter1 = getSubsetItemFilter(subsetCouple.getSubsetKey1());
        SubsetItemFilter subsetItemFilter2 = getSubsetItemFilter(subsetCouple.getSubsetKey2());
        StringBuilder buf = new StringBuilder();
        CroisementXMLWriter croisementXMLWriter = new CroisementXMLWriter();
        croisementXMLWriter.setAppendable(buf);
        croisementXMLWriter.setIndentLength(0);
        croisementXMLWriter.appendXMLDeclaration();
        int count = croisementXMLWriter.appendCroisement(croisement, subsetItemFilter1, subsetItemFilter2);
        if (count == 0) {
            return;
        }
        String entryPath = subsetCouple.getSubsetKey1().getSubsetName() + "_" + subsetCouple.getSubsetKey2().getSubsetName() + ".xml";
        if (subsetCouple.isCorpusCorpusCouple()) {
            entryPath = racineBasedefiches + "corpus_corpus/" + entryPath;
        } else if (subsetCouple.isCorpusThesaurusCouple()) {
            entryPath = racineBasedefiches + "corpus_thesaurus/" + entryPath;
        }
        if (subsetCouple.isCorpusAddendaCouple()) {
            entryPath = racineBasedefiches + "corpus_addenda/" + entryPath;
        }
        if (subsetCouple.isCorpusAlbumCouple()) {
            entryPath = racineBasedefiches + "corpus_album/" + entryPath;
        }
        zipOutputStream.putNextEntry(new ZipEntry(entryPath));
        bufWriter.write(buf.toString());
        bufWriter.flush();
    }

    private String getMillier(int id) {
        int millier = id / 1000;
        if (millier < 10) {
            return "00" + millier;
        }
        if (millier < 100) {
            return "0" + millier;
        }
        return String.valueOf(millier);
    }

}
