/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.response.BdfConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesEditor;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractBdfCommand2 implements BdfCommand2, BdfInstructionConstants, BdfConstants {

    private final String commandName;
    protected final BdfContext bdfContext;
    protected final RequestMap requestMap;
    protected final BdfServer bdfServer;
    protected final Basedefiches basedefiches;
    protected final BasedefichesEditor basedefichesEditor;
    protected BdfUser bdfUser;
    private InternalBdfCommand2Result bdfCommandResult;

    public AbstractBdfCommand2(BdfContext bdfContext, RequestMap requestMap, String commandName) {
        this.commandName = commandName;
        this.bdfContext = bdfContext;
        this.requestMap = requestMap;
        this.bdfServer = bdfContext.getBdfServer();
        this.basedefiches = bdfServer.getBasedefiches();
        this.basedefichesEditor = bdfServer.getBasedefichesEditor();
    }

    public String getCommandName() {
        return commandName;
    }

    public CommandMessage testCommand(BdfUser bdfUser) throws BdfCommand2Exception {
        this.bdfUser = bdfUser;
        bdfCommandResult = new InternalBdfCommand2Result();
        checkParameters();
        return bdfCommandResult.getCommandMessage();
    }

    public BdfCommand2Result doCommand(BdfUser bdfUser) throws BdfCommand2Exception {
        this.bdfUser = bdfUser;
        bdfCommandResult = new InternalBdfCommand2Result();
        if (checkParameters()) {
            doCommand();
        }
        return bdfCommandResult;
    }

    protected abstract boolean checkParameters() throws BdfCommand2Exception;

    protected abstract void doCommand() throws BdfCommand2Exception;

    protected void putResultObject(String name, Object obj) {
        bdfCommandResult.resultObjectMap.put(name, obj);
    }

    protected String getMandatoryParameter(String paramName) throws ParameterBdfCommand2Exception {
        return BdfCommand2Utils.getMandatoryParameter(requestMap, paramName);
    }

    protected int getMandatoryIntegerParameter(String paramName) throws ParameterBdfCommand2Exception {
        return BdfCommand2Utils.getMandatoryIntegerParameter(requestMap, paramName);
    }

    protected void setError(String errorKey, Object messageValue) {
        bdfCommandResult.setError(errorKey, messageValue);
    }

    protected void addLog(ProcessLog processLog) {
        bdfCommandResult.addLog(processLog);
    }

    protected void setDone(String doneKey, Object messageValue) {
        bdfCommandResult.setDone(doneKey, messageValue);
    }

    protected boolean hasMultiError() {
        return bdfCommandResult.hasMultiError();
    }

    protected void addMultiError(String errorKey, Object messageValue) {
        bdfCommandResult.addMultiError(errorKey, messageValue);
    }

    protected void setMultiErrorTitle(String errorKey, Object messageValue) {
        bdfCommandResult.setMultiErrorTitle(errorKey, messageValue);
    }


    private class InternalBdfCommand2Result implements BdfCommand2Result {

        private Map<String, Object> resultObjectMap = new HashMap<String, Object>();
        private CommandMessage commandMessage;
        private List<ResourceItem> multiErrorList;
        private ProcessLog processLog;
        private Set<String> resultObjectNameSet = Collections.unmodifiableSet(resultObjectMap.keySet());

        public Object getResultObject(String name) {
            return resultObjectMap.get(name);
        }

        public CommandMessage getCommandMessage() {
            return commandMessage;
        }

        public Set<String> getResultObjectNameSet() {
            return resultObjectNameSet;
        }

        private void setError(String errorKey, Object messageValue) {
            testCommandMessage();
            commandMessage = CommandMessage.error(errorKey, messageValue);
        }

        private void addLog(ProcessLog processLog) {
            testCommandMessage();
            this.processLog = processLog;
        }

        private void setDone(String doneKey, Object messageValue) {
            testCommandMessage();
            if (processLog != null) {
                commandMessage = CommandMessage.done(new ResourceItem(doneKey, messageValue), processLog);
            } else {
                commandMessage = CommandMessage.done(doneKey, messageValue);
            }
        }

        private boolean hasMultiError() {
            return (multiErrorList != null);
        }

        private void addMultiError(String errorKey, Object messageValue) {
            testCommandMessage();
            if (multiErrorList == null) {
                multiErrorList = new ArrayList<ResourceItem>();
            }
            multiErrorList.add(new ResourceItem(errorKey, messageValue));
        }

        private void setMultiErrorTitle(String errorKey, Object messageValue) {
            testCommandMessage();
            if (multiErrorList == null) {
                throw new IllegalStateException("multiErrorList is empty");
            }
            commandMessage = CommandMessage.error(new ResourceItem(errorKey, messageValue), multiErrorList);
        }

        private void testCommandMessage() {
            if (commandMessage != null) {
                throw new IllegalStateException("commandMessage is already defined");
            }
        }

    }

}
