/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.actions.CreateFiche;
import fr.exemole.bdfserver.actions.FicheAction;
import fr.exemole.bdfserver.actions.ReadFiche;
import fr.exemole.bdfserver.actions.WriteFiche;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.users.BdfUser;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.MetadataEditor;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.text.ChangeLibelleHolder;
import net.mapeadores.util.text.ChangeLibelleHolderBuilder;
import net.mapeadores.util.text.LocalizedStringList;


/**
 *
 * @author Vincent Calame
 */
public class BdfCommand2Utils {

    public final static String CONFIRMPASSWORD_PARAMNAME = "confirmpassword";
    public final static String LOCATION_FORM_PARAMVALUE = "form";
    public final static String LOCATION_AFFICHAGE_PARAMVALUE = "affichage";
    public final static String LOCATION_BOTH_PARAMVALUE = "both";

    private BdfCommand2Utils() {
    }

    public static String getMandatoryParameter(RequestMap requestMap, String paramName) throws ParameterBdfCommand2Exception {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.MANDATORY, paramName);
        }
        return paramValue;
    }

    public static int getMandatoryIntegerParameter(RequestMap requestMap, String paramName) throws ParameterBdfCommand2Exception {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.MANDATORY, paramName);
        }
        try {
            int value = Integer.parseInt(paramValue);
            return value;
        } catch (NumberFormatException nfe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.NOTINTEGER, paramName, paramValue);
        }
    }

    public static boolean getBooleanParameter(RequestMap requestMap, String paramName) throws ParameterBdfCommand2Exception {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            return false;
        }
        paramValue = paramValue.toLowerCase();
        if (paramValue.equals("1") || paramValue.equals("true")) {
            return true;
        }
        if (paramValue.equals("0") || paramValue.equals("false")) {
            return false;
        }
        throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.NOTBOOLEAN, paramName, paramValue);
    }

    public static Corpus getMandatoryCorpus(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return (Corpus) getMandatorySubset(basedefiches, requestMap, SubsetKey.CATEGORY_CORPUS);
    }

    public static Thesaurus getMandatoryThesaurus(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return (Thesaurus) getMandatorySubset(basedefiches, requestMap, SubsetKey.CATEGORY_THESAURUS);
    }

    public static Sphere getMandatorySphere(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return (Sphere) getMandatorySubset(basedefiches, requestMap, SubsetKey.CATEGORY_SPHERE);
    }

    public static Album getMandatoryAlbum(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return (Album) getMandatorySubset(basedefiches, requestMap, SubsetKey.CATEGORY_ALBUM);
    }

    public static Subset getMandatorySubset(Basedefiches basedefiches, RequestMap requestMap, short subsetCategory) throws ParameterBdfCommand2Exception {
        return getMandatorySubset(basedefiches, requestMap, subsetCategory, SubsetKey.categoryToString(subsetCategory));
    }

    public static Subset getMandatorySubset(Basedefiches basedefiches, RequestMap requestMap, short subsetCategory, String paramName) throws ParameterBdfCommand2Exception {
        String paramValue = getMandatoryParameter(requestMap, paramName);
        try {
            SubsetKey subsetKey = SubsetKey.parse(subsetCategory, paramValue);
            Subset subset = basedefiches.getSubset(subsetKey);
            if (subset == null) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSET, paramName, paramValue);
            }
            return subset;
        } catch (ParseException pe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGSUBSETNAME, paramName, paramValue);
        }
    }

    public static AlbumDim getMandatoryAlbumDim(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return getMandatoryAlbumDim(basedefiches, requestMap, "albumdim");
    }

    public static AlbumDim getMandatoryAlbumDim(Basedefiches basedefiches, RequestMap requestMap, String paramName) throws ParameterBdfCommand2Exception {
        Album album = getMandatoryAlbum(basedefiches, requestMap);
        return getMandatoryAlbumDim(basedefiches, requestMap, album, paramName);
    }

    public static AlbumDim getMandatoryAlbumDim(Basedefiches basedefiches, RequestMap requestMap, Album album, String paramName) throws ParameterBdfCommand2Exception {
        String paramValue = getMandatoryParameter(requestMap, paramName);
        AlbumDim albumDim = album.getAlbumMetadata().getAlbumDimByName(paramValue);
        if (albumDim == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNALBUMDIM, paramName, paramValue);
        }
        return albumDim;
    }

    public static int getMandatoryIdsubset(Basedefiches basedefiches, RequestMap requestMap, short subsetCategory) throws ParameterBdfCommand2Exception {
        String paramName = BdfHtmlUtils.getDefaultIdsubsetParamName(subsetCategory);
        int idsubset = getMandatoryIntegerParameter(requestMap, paramName);
        if (idsubset < 1) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGIDSUBSET, paramName, paramName);
        }
        return idsubset;
    }

    public static AbregeFiche getMandatoryAbregeFiche(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return (AbregeFiche) getMandatorySubsetItem(basedefiches, requestMap, SubsetKey.CATEGORY_CORPUS);
    }

    public static AbregeFiche getMandatoryAbregeFiche(Basedefiches basedefiches, RequestMap requestMap, Corpus corpus) throws ParameterBdfCommand2Exception {
        return (AbregeFiche) getMandatorySubsetItem(basedefiches, requestMap, corpus);
    }

    public static Motcle getMandatoryMotcle(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        String idmtcl = requestMap.getParameter("idmtcl");
        if (idmtcl == null) {
            return (Motcle) getMandatorySubsetItem(basedefiches, requestMap, SubsetKey.CATEGORY_THESAURUS);
        }
        Thesaurus thesaurus = getMandatoryThesaurus(basedefiches, requestMap);
        Motcle motcle = null;
        if (ThesaurusUtils.isIdalphaType(thesaurus)) {
            motcle = thesaurus.getMotcleByIdalpha(idmtcl);
        } else {
            try {
                int idths = Integer.parseInt(idmtcl);
                motcle = thesaurus.getMotcleByIdths(idths);
            } catch (NumberFormatException nfe) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.NOTINTEGER, "idmtcl", idmtcl);
            }
        }
        if (motcle == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSETITEM, "idmtcl", idmtcl);
        }
        return motcle;
    }

    public static Redacteur getMandatoryRedacteur(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        Sphere sphere = getMandatorySphere(basedefiches, requestMap);
        String idsphereString = getMandatoryParameter(requestMap, BdfHtmlUtils.getDefaultIdsubsetParamName(SubsetKey.CATEGORY_SPHERE));
        Redacteur redacteur = sphere.getRedacteurByIdsphere(idsphereString);
        if (redacteur == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSETITEM, BdfHtmlUtils.getDefaultIdsubsetParamName(SubsetKey.CATEGORY_SPHERE), idsphereString);
        }
        return redacteur;
    }

    public static Illustration getMandatoryIllustration(Basedefiches basedefiches, RequestMap requestMap) throws ParameterBdfCommand2Exception {
        return (Illustration) getMandatorySubsetItem(basedefiches, requestMap, SubsetKey.CATEGORY_ALBUM);
    }

    public static Illustration getMandatoryIllustration(Basedefiches basedefiches, RequestMap requestMap, Album album) throws ParameterBdfCommand2Exception {
        return (Illustration) getMandatorySubsetItem(basedefiches, requestMap, album);
    }

    public static SubsetItem getMandatorySubsetItem(Basedefiches basedefiches, RequestMap requestMap, short subsetCategory) throws ParameterBdfCommand2Exception {
        int idsubset = getMandatoryIdsubset(basedefiches, requestMap, subsetCategory);
        Subset subset = getMandatorySubset(basedefiches, requestMap, subsetCategory);
        SubsetItem subsetItem = subset.getSubsetItemById(idsubset);
        if (subsetItem == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSETITEM, BdfHtmlUtils.getDefaultIdsubsetParamName(subsetCategory), String.valueOf(idsubset));
        }
        return subsetItem;
    }

    public static SubsetItem getMandatorySubsetItem(Basedefiches basedefiches, RequestMap requestMap, Subset subset) throws ParameterBdfCommand2Exception {
        short subsetCategory = subset.getSubsetKey().getCategory();
        int idsubset = getMandatoryIdsubset(basedefiches, requestMap, subsetCategory);
        SubsetItem subsetItem = subset.getSubsetItemById(idsubset);
        if (subsetItem == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSETITEM, BdfHtmlUtils.getDefaultIdsubsetParamName(subsetCategory), String.valueOf(idsubset));
        }
        return subsetItem;
    }

    public static void testCreateFichePermission(BdfServer bdfServer, BdfUser bdfUser, Corpus corpus, int idcorpus) throws PermissionBdfCommand2Exception {
        CreateFiche createFiche = CreateFiche.newInstance(corpus, idcorpus);
        if (!bdfServer.getPermissionManager().canDo(bdfUser, createFiche)) {
            String corpusString = corpus.getSubsetKey().toString();
            CommandMessage commandMessage = CommandMessage.error("global_err_createfiche", corpusString);
            PermissionBdfCommand2Exception permissionException = new PermissionBdfCommand2Exception(commandMessage);
            throw permissionException;
        }
    }

    public static void testWriteFichePermission(BdfServer bdfServer, BdfUser bdfUser, AbregeFiche abregeFiche) throws PermissionBdfCommand2Exception {
        WriteFiche writeFiche = FicheAction.getWriteFiche(abregeFiche);
        if (!bdfServer.getPermissionManager().canDo(bdfUser, writeFiche)) {
            String intitule = CorpusMetadataUtils.getFicheIntituleString(abregeFiche, bdfUser.getLocalisationInfo());
            CommandMessage commandMessage = CommandMessage.error("global_err_writefiche", intitule);
            PermissionBdfCommand2Exception permissionException = new PermissionBdfCommand2Exception(commandMessage);
            throw permissionException;
        }
    }

    public static void testReadFichePermission(BdfServer bdfServer, BdfUser bdfUser, AbregeFiche abregeFiche) throws PermissionBdfCommand2Exception {
        ReadFiche readFiche = FicheAction.getReadFiche(abregeFiche);
        if (!bdfServer.getPermissionManager().canDo(bdfUser, readFiche)) {
            String intitule = CorpusMetadataUtils.getFicheIntituleString(abregeFiche, bdfUser.getLocalisationInfo());
            CommandMessage commandMessage = CommandMessage.error("global_err_readfiche", intitule);
            PermissionBdfCommand2Exception permissionException = new PermissionBdfCommand2Exception(commandMessage);
            throw permissionException;
        }
    }

    public static boolean checkConfirmPassword(RequestMap requestMap, BdfServer bdfServer) throws ParameterBdfCommand2Exception {
        String param_password = getMandatoryParameter(requestMap, CONFIRMPASSWORD_PARAMNAME);
        return bdfServer.getPasswordManager().checkPassword(PasswordManager.CONFIRM_PASSWORDKEY, param_password);
    }

    public static int getCommentPositionMask(RequestMap requestMap, String paramName) throws ParameterBdfCommand2Exception {
        String positionString = getMandatoryParameter(requestMap, paramName);
        if (positionString.equals(LOCATION_FORM_PARAMVALUE)) {
            return CommentUI.FORM_BITVALUE;
        } else if (positionString.equals(LOCATION_AFFICHAGE_PARAMVALUE)) {
            return CommentUI.AFFICHAGE_BITVALUE;
        } else if (positionString.equals(LOCATION_BOTH_PARAMVALUE)) {
            return CommentUI.ALL_MASK;
        } else {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, paramName, positionString);
        }
    }

    public static ChangeLibelleHolder getChangeLibelleHolder(RequestMap requestMap, String paramPrefix) {
        ChangeLibelleHolderBuilder result = new ChangeLibelleHolderBuilder();
        int prefixLength = paramPrefix.length();
        for (String paramName : requestMap.getParameterNameSet()) {
            if (!paramName.startsWith(paramPrefix)) {
                continue;
            }
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(prefixLength));
                String nvlibelle = requestMap.getParameter(paramName);
                result.putLibelle(currentLangInteger, nvlibelle);

            } catch (LangIntegerException ile) {
            }
        }
        return result.toChangeLibelleHolder();
    }

    public static LocalizedStringList getLocalizedStringList(RequestMap requestMap, String paramPrefix) {
        InternalLocalizedStringList result = new InternalLocalizedStringList();
        int prefixLength = paramPrefix.length();
        for (String paramName : requestMap.getParameterNameSet()) {
            if (!paramName.startsWith(paramPrefix)) {
                continue;
            }
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(prefixLength));
                String value = requestMap.getParameter(paramName);
                result.add(currentLangInteger, value);

            } catch (LangIntegerException ile) {
            }
        }
        return result;
    }

    public static boolean changeIntituleLibelles(MetadataEditor metadataEditor, Intitule intitule, ChangeLibelleHolder changeLibelleHolder) {
        boolean done = false;
        int libelleCount = changeLibelleHolder.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            boolean stepdone = metadataEditor.putLibelle(intitule, changeLibelleHolder.getLibelleByIndex(i));
            if (stepdone) {
                done = true;
            }
        }
        int removedCount = changeLibelleHolder.getRemovedLangCount();
        for (int i = 0; i < removedCount; i++) {
            boolean stepdone = metadataEditor.removeLibelle(intitule, changeLibelleHolder.getRemovedLangInteger(i));
            if (stepdone) {
                done = true;
            }
        }
        return done;
    }

    public static CommandMessage toCommandMessage(BdfCommand2Exception bce) {
        String msg = bce.getMessage();
        CommandMessage errorMessage = null;
        if (bce instanceof ParameterBdfCommand2Exception) {
            errorMessage = CommandMessage.error("global_err_parameterexception", msg);
        } else if (bce instanceof PermissionBdfCommand2Exception) {
            errorMessage = ((PermissionBdfCommand2Exception) bce).getCommandMessage();
        } else {
            errorMessage = CommandMessage.error("global_err_commandexception", msg);
        }
        return errorMessage;
    }


    private static class InternalLocalizedStringList implements LocalizedStringList {

        private List<Integer> langIntegerList = new ArrayList<Integer>();
        private List<String> stringList = new ArrayList<String>();

        private InternalLocalizedStringList() {
        }

        public int getCount() {
            return langIntegerList.size();
        }

        public int getLangInteger(int i) {
            return langIntegerList.get(i);
        }

        public String getString(int i) {
            return stringList.get(i);
        }

        protected void add(int langInteger, String string) {
            langIntegerList.add(langInteger);
            stringList.add(string);
        }

    }

}
