/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import java.io.File;
import java.io.IOException;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.util.awt.ResizeInfo;
import net.mapeadores.util.images.ImageResizing;


/**
 *
 * @author Vincent Calame
 */
public class AlbumCommandUtils {

    public static final String ORIGINAL_TMPFILENAME = "_ORIGINAL";

    private AlbumCommandUtils() {
    }

    public static void minimizeTmpFile(File tmpFile, Album album) throws IOException {
        ResizeInfo resizeInfo = album.getAlbumMetadata().getResizeInfo(AlbumConstants.MINI_SPECIALDIM);
        String fileName = tmpFile.getName();
        String extension = getExtension(fileName);
        File miniFile = new File(tmpFile.getParentFile(), "mini-" + fileName);
        ImageResizing.resize(tmpFile, miniFile, resizeInfo, extension);
    }

    public static File getTmpFile(BdfServer bdfServer, String extension) {
        synchronized (bdfServer) {
            File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PRIVATE_OUTPUTDIRECTORY);
            File tmpDirectory = new File(outputDirectory, "tmp");
            tmpDirectory.mkdir();
            String number = Long.toHexString(System.currentTimeMillis());
            File f = new File(tmpDirectory, "img-" + number + "." + extension);
            if (!f.exists()) {
                return f;
            } else {
                int p = 1;
                while (true) {
                    f = new File(tmpDirectory, "img-" + number + "_" + p + "." + extension);
                    if (!f.exists()) {
                        return f;
                    } else {
                        p++;
                    }
                }
            }
        }
    }

    public static String getExtension(File file) {
        return getExtension(file.getName());
    }

    public static String getExtension(String fileName) {
        int idx = fileName.lastIndexOf('.');
        return fileName.substring(idx + 1);
    }

    public static String checkExtension(String fileName) {
        int idx = fileName.lastIndexOf(".");
        if (idx == -1) {
            return null;
        }
        String extension = fileName.substring(idx + 1);
        if (extension.equalsIgnoreCase("png")) {
            return "png";
        }
        if (extension.equalsIgnoreCase("jpg")) {
            return "jpg";
        }
        if (extension.equalsIgnoreCase("jpeg")) {
            return "jpg";
        }
        if (extension.equalsIgnoreCase("bmp")) {
            return "bmp";
        }
        if (extension.equalsIgnoreCase("gif")) {
            return "gif";
        }
        if (extension.equalsIgnoreCase("pdf")) {
            return "pdf";
        }
        return null;
    }

    public static boolean convertToPng(String extension) {
        if (extension.equals("gif")) {
            return true;
        }
        if (extension.equals("bmp")) {
            return true;
        }
        return false;
    }

}
