/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import java.text.ParseException;
import net.mapeadores.basedefiches.ExistingSubsetException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.AlbumEditor;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class AlbumCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "AlbumCreation";
    public final static String COMMANDKEY = "ALB-01";
    public final static String NEWALBUM_PARAMNAME = "newalbum";
    private SubsetKey newSubsetKey;

    public AlbumCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        try {
            AlbumEditor albumEditor = basedefichesEditor.createAlbum(newSubsetKey);
            setDone("album_done_albumcreation", newSubsetKey.toString());
            putResultObject(ALBUM_OBJ, albumEditor.getAlbum());
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test done before", ese);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String newAlbumName = getMandatoryParameter(NEWALBUM_PARAMNAME);
        newAlbumName = newAlbumName.trim();
        if (newAlbumName.length() == 0) {
            setError("album_err_missingalbumname", null);
            return false;
        }
        try {
            newSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, newAlbumName);
        } catch (ParseException pe) {
            setError("album_err_wrongalbumname", newAlbumName);
            return false;
        }
        if (basedefiches.getSubset(newSubsetKey) != null) {
            setError("album_err_existingalbumname", newAlbumName);
            return false;
        }
        return true;
    }

}
