/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.album.metadata.AlbumMetadataEditor;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class AlbumDimModifCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "AlbumDimModif";
    public final static String COMMANDKEY = "ALB-04";
    public final static String ALBUMDIM_WIDTH_PARAMNAME = "albumdimwidth";
    public final static String ALBUMDIM_HEIGHT_PARAMNAME = "albumdimheight";
    private AlbumDim albumDim;
    private int width = 0;
    private int height = 0;

    public AlbumDimModifCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        Album album = albumDim.getAlbumMetadata().getAlbum();
        AlbumMetadataEditor albumMetadataEditor = basedefichesEditor.getAlbumEditor(album.getSubsetKey()).getAlbumMetadataEditor();
        albumMetadataEditor.setDim(albumDim, width, height);
        putResultObject(ALBUM_OBJ, album);
        putResultObject(ALBUMDIM_OBJ, albumDim);
        setDone("album_done_albumdimmodif", albumDim.getName());
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        albumDim = BdfCommand2Utils.getMandatoryAlbumDim(basedefiches, requestMap);
        short type = albumDim.getDimType();
        if (AlbumUtils.needWidth(type)) {
            String widthString = getMandatoryParameter(ALBUMDIM_WIDTH_PARAMNAME);
            widthString = widthString.trim();
            if (widthString.length() == 0) {
                setError("album_err_missingwidth", null);
                return false;
            }
            try {
                width = Integer.parseInt(widthString);
                if (width < 1) {
                    setError("album_err_wrongwidth", widthString);
                    return false;
                }
            } catch (NumberFormatException nfe) {
                setError("album_err_wrongwidth", widthString);
                return false;
            }

        }
        if (AlbumUtils.needHeight(type)) {
            String heightString = getMandatoryParameter(ALBUMDIM_HEIGHT_PARAMNAME);
            heightString = heightString.trim();
            if (heightString.length() == 0) {
                setError("album_err_missingheight", null);
                return false;
            }
            try {
                height = Integer.parseInt(heightString);
                if (height < 1) {
                    setError("album_err_wrongheight", heightString);
                    return false;
                }
            } catch (NumberFormatException nfe) {
                setError("album_err_wrongheight", heightString);
                return false;
            }

        }
        return true;
    }

}
