/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.imageio.IIOException;
import javax.imageio.ImageIO;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.util.images.ImagesUtils;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationFileUploadCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "IllustrationFileUpload";
    public final static String COMMANDKEY = "ALB-07";
    public final static String FILE_PARAMNAME = "file";
    private File tmpFile;
    private Album album;

    public IllustrationFileUploadCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return false;
    }

    protected void doCommand() throws BdfCommand2Exception {
        FileValue fileValue = requestMap.getFileValue(FILE_PARAMNAME);
        if (fileValue == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.MANDATORY, FILE_PARAMNAME);
        }
        boolean done = false;
        try {
            done = saveFile(fileValue);
        } catch (IOException ioe) {
            setError("album_err_ioexception", ioe.getMessage());
        }
        if (done) {
            setDone("album_done_fileupload", null);
            putResultObject(TMPFILE_OBJ, tmpFile);
            putResultObject(ALBUM_OBJ, album);
        }
        fileValue.free();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        album = BdfCommand2Utils.getMandatoryAlbum(basedefiches, requestMap);
        return true;
    }

    private boolean saveFile(FileValue fileValue) throws IOException {
        long length = fileValue.length();
        if (length < 2) {
            setError("album_err_emptyfile", null);
            return false;
        }
        String extension = AlbumCommandUtils.checkExtension(fileValue.getName());
        if (extension == null) {
            setError("album_err_noimagefile", null);
            return false;
        }
        if (extension.equals("pdf")) {
            extension = "png";
            tmpFile = AlbumCommandUtils.getTmpFile(bdfServer, "png");
            InputStream inputStream = fileValue.getInputStream();
            ImagesUtils.convertPdf(inputStream, 1, "png", tmpFile);
            inputStream.close();

        } else if (AlbumCommandUtils.convertToPng(extension)) {
            extension = "png";
            tmpFile = AlbumCommandUtils.getTmpFile(bdfServer, "png");
            InputStream inputStream = fileValue.getInputStream();
            BufferedImage srcImg = ImageIO.read(inputStream);
            inputStream.close();
            if (srcImg == null) {
                setError("album_err_unsupportedimagetype", null);
                return false;
            } else {
                ImageIO.write(srcImg, "png", tmpFile);
            }

        } else {
            try {
                InputStream inputStream = fileValue.getInputStream();
                BufferedImage srcImg = ImageIO.read(inputStream);
                inputStream.close();
                if (srcImg == null) {
                    setError("album_err_unsupportedimagetype", null);
                    return false;
                }
            } catch (IOException ioe) {
                if (ioe instanceof IIOException) {
                    setError("album_err_unsupportedimagetype", null);
                    return false;
                } else {
                    setError("album_err_ioexception", ioe.getMessage());
                    return false;
                }
            }
            tmpFile = AlbumCommandUtils.getTmpFile(bdfServer, extension);
            OutputStream outputStream = new FileOutputStream(tmpFile);
            InputStream inputStream = fileValue.getInputStream();
            IOUtils.copy(inputStream, outputStream);
            inputStream.close();
            outputStream.close();
        }
        AlbumCommandUtils.minimizeTmpFile(tmpFile, album);
        return true;
    }

}
