/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.util.awt.ResizeInfo;
import net.mapeadores.util.images.ImageResizing;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class ToolResizeCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "ToolResize";
    public final static String TMPFILE_PARAMNAME = "tmpfile";
    public final static String WIDTH_PARAMNAME = "width";
    public final static String HEIGHT_PARAMNAME = "height";
    private Album album;
    private File tmpFile;
    private Illustration illustration;
    private int width, height;
    private String extension;

    public ToolResizeCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return false;
    }

    protected void doCommand() throws BdfCommand2Exception {
        File newTmpFile = AlbumCommandUtils.getTmpFile(bdfServer, extension);
        ResizeInfo resizeInfo = ResizeInfo.newFixedDimInstance(width, height);
        try {
            InputStream inputStream = getInputStream();
            ImageResizing.resize(inputStream, newTmpFile, resizeInfo, extension);
            inputStream.close();
            AlbumCommandUtils.minimizeTmpFile(newTmpFile, album);
        } catch (IOException ioe) {
            setError("album_err_ioexception", ioe.getMessage());
            return;
        }
        putResultObject(TMPFILE_OBJ, newTmpFile);
        putResultObject(ALBUM_OBJ, album);
        setDone("album_done_tool_resize", newTmpFile.getName());
    }

    private InputStream getInputStream() throws IOException {
        if (tmpFile != null) {
            return new FileInputStream(tmpFile);
        } else {
            return illustration.getInputStream(AlbumConstants.ORIGINAL_SPECIALDIM);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        album = BdfCommand2Utils.getMandatoryAlbum(basedefiches, requestMap);
        width = BdfCommand2Utils.getMandatoryIntegerParameter(requestMap, WIDTH_PARAMNAME);
        height = BdfCommand2Utils.getMandatoryIntegerParameter(requestMap, HEIGHT_PARAMNAME);
        String tmpFileName = requestMap.getParameter(TMPFILE_PARAMNAME);
        if ((tmpFileName != null) && (!tmpFileName.equals(AlbumCommandUtils.ORIGINAL_TMPFILENAME))) {
            File outputDirectory = bdfServer.getConfigurationManager().getOutputDirectory(ConfigurationManager.PRIVATE_OUTPUTDIRECTORY);
            File tmpDirectory = new File(outputDirectory, "tmp");
            tmpFile = new File(tmpDirectory, tmpFileName);
            if (!tmpFile.exists()) {
                this.setError("album_err_tmpfile", tmpFileName);
                return false;
            }
            extension = AlbumCommandUtils.getExtension(tmpFile);
        } else {
            illustration = BdfCommand2Utils.getMandatoryIllustration(basedefiches, requestMap, album);
            extension = AlbumUtils.formatTypeToString(illustration.getFormatType());
        }
        return true;
    }

}
