/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.request.RequestUtils;
import net.mapeadores.basedefiches.BasedefichesConstants;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadataEditor;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.ChangeLibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class IntitulesCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Intitules";
    public final static String COMMANDKEY = "CNF-04";
    public final static String SHORT_PARAMPREFIX = "short_";
    public final static String LONG_PARAMPREFIX = "long_";
    private ChangeLibelleHolder shortChangeLibelleHolder;
    private ChangeLibelleHolder longChangeLibelleHolder;

    public IntitulesCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        boolean done = false;
        if (update(BasedefichesConstants.SHORT, shortChangeLibelleHolder)) {
            done = true;
        }
        if (update(BasedefichesConstants.LONG, longChangeLibelleHolder)) {
            done = true;
        }
        if (done) {
            setDone("configuration_done_intitules", null);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingLangArray = langConfig.getWorkingLangIntegerArray();
        shortChangeLibelleHolder = RequestUtils.getChangeLibelleHolder(requestMap, SHORT_PARAMPREFIX, workingLangArray);
        longChangeLibelleHolder = RequestUtils.getChangeLibelleHolder(requestMap, LONG_PARAMPREFIX, workingLangArray);
        return true;
    }

    private boolean update(String intituleName, ChangeLibelleHolder changeLibelleHolder) {
        BasedefichesMetadataEditor basedefichesMetadataEditor = bdfServer.getBasedefichesEditor().getBasedefichesMetadataEditor();
        Intitule intitule = BasedefichesUtils.getOrCreateIntitule(basedefichesMetadataEditor, intituleName);
        return BdfCommand2Utils.changeIntituleLibelles(basedefichesMetadataEditor, intitule, changeLibelleHolder);
    }

}
