/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.LangConfig;
import net.mapeadores.util.langinteger.LangIntegerUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class LangConfigCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "LangConfig";
    public final static String COMMANDKEY = "CNF-03";
    public final static String WORKINGLANG_PARAMNAME = "workinglang";
    public final static String SUPPLEMENTARYLANG_PARAMNAME = "supplementarylang";
    public final static String ALLLANG_PARAMNAME = "alllang";
    public final static String ORIGINALALLOWED_PARAMNAME = "originalallowed";
    public final static String WITHOUTNOMAVANT_PARAMNAME = "withoutnomavant";
    private int[] workingArray;
    private int[] supplementaryArray;
    private boolean allLanguage;
    private boolean withOriginalName;
    private boolean withoutNomAvant;

    public LangConfigCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        LangConfig langConfig = new LangConfig();
        langConfig.setWorkingLangIntegerArray(workingArray);
        langConfig.setAllLanguages(allLanguage);
        if (!allLanguage) {
            langConfig.setSupplementaryLangIntegerArray(supplementaryArray);
        }
        langConfig.setWithOriginalName(withOriginalName);
        langConfig.setWithoutNomAvant(withoutNomAvant);
        bdfServer.getConfigurationManager().setLangConfig(langConfig);
        bdfServer.saveConfigurationManager();
        bdfServer.getTransformationManager().update();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String param_languesbase = BdfCommand2Utils.getMandatoryParameter(requestMap, WORKINGLANG_PARAMNAME);
        workingArray = LangIntegerUtils.toCleanLangIntegerArray(param_languesbase);
        if (workingArray.length == 0) {
            setError("configuration_err_languesbase", null);
            return false;
        }
        allLanguage = BdfCommand2Utils.getBooleanParameter(requestMap, ALLLANG_PARAMNAME);
        if (!allLanguage) {
            String param_languescomplement = BdfCommand2Utils.getMandatoryParameter(requestMap, SUPPLEMENTARYLANG_PARAMNAME);
            supplementaryArray = LangIntegerUtils.toCleanLangIntegerArray(param_languescomplement);
        }
        withOriginalName = BdfCommand2Utils.getBooleanParameter(requestMap, ORIGINALALLOWED_PARAMNAME);
        withoutNomAvant = BdfCommand2Utils.getBooleanParameter(requestMap, WITHOUTNOMAVANT_PARAMNAME);
        return true;
    }

}
