/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.configuration.PageSubstitution;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.request.RequestMap;

/**
 *
 * @author Vincent Calame
 */
public class PageSubstitutionCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "PageSubstitution";
    public final static String COMMANDKEY = "CNF-18";
    public final static String PAGES_PARAMNAME = "pages";
    private List<PageSubstitution> pageSubstitutionList;

    public PageSubstitutionCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        configurationManager.setPageSubstitutionList(pageSubstitutionList);
        bdfServer.saveConfigurationManager();
        setDone("configuration_done_pagesubstitution", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String pages = BdfCommand2Utils.getMandatoryParameter(requestMap, PAGES_PARAMNAME);
        pageSubstitutionList = new ArrayList<PageSubstitution>();
        BufferedReader bufReader = new BufferedReader(new StringReader(pages));
        String ligne = null;
        try {
            while((ligne = bufReader.readLine()) != null) {
                ligne = ligne.trim();
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf('=');
                    if (idx > 0) {
                        String original = ligne.substring(0, idx).trim();
                        String substitution = ligne.substring(idx+1).trim();
                        if ((original.length() > 0) && (substitution.length() > 0)) {
                            pageSubstitutionList.add(new PageSubstitution(original, substitution));
                        }
                    }
                }
            }
        } catch(IOException ioe) {

        }
        return true;
    }
}
