/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.managers.ConfigurationManager;
import net.mapeadores.basedefiches.BasedefichesAttributes;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeBuilder;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.CleanedString;


/**
 *
 * @author Vincent Calame
 */
public class UrlCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Url";
    public final static String COMMANDKEY = "CNF-02";
    public final static String INSERTURL_PARAMNAME = "inserturl";
    private Attribute imageBaseUrlAttribute;

    public UrlCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        BasedefichesMetadata metadata = bdfServer.getBasedefiches().getBasedefichesMetadata();
        ConfigurationManager configurationManager = bdfServer.getConfigurationManager();
        boolean done = false;
        if (imageBaseUrlAttribute == null) {
            done = bdfServer.getBasedefichesEditor().removeAttribute(metadata, BasedefichesAttributes.IMAGEBASEURL_KEY);
        } else {
            done = bdfServer.getBasedefichesEditor().putAttribute(metadata, imageBaseUrlAttribute);
        }
        if (done) {
            setDone("configuration_done_core", null);
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String insertUrlString = getMandatoryParameter(INSERTURL_PARAMNAME);
        CleanedString cleanedString = CleanedString.newInstance(insertUrlString);
        if (cleanedString != null) {
            imageBaseUrlAttribute = AttributeBuilder.toAttribute(BasedefichesAttributes.IMAGEBASEURL_KEY, cleanedString);
        } else {
            imageBaseUrlAttribute = null;
        }
        return true;
    }

}
