/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUIUtils;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import java.util.List;
import net.mapeadores.basedefiches.ExistingSubsetException;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.ExistingFieldKeyException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionException;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptions;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsConstants;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.primitives.IntegerList;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class CloneCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Clone";
    public final static String COMMANDKEY = "CRP-21";
    public static String NEWCORPUS_PARAMNAME = "newcorpus";
    private Corpus corpus;
    private CorpusMetadata corpusMetadata;
    private SubsetKey newCorpusKey;
    private CorpusEditor newCorpusEditor;
    private CorpusMetadataEditor newCorpusMetadataEditor;
    private CorpusMetadata newCorpusMetadata;
    private CorpusUI corpusUI;
    private CorpusUI newCorpusUI;

    public CloneCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        corpusMetadata = corpus.getCorpusMetadata();
        bdfServer.bufferBasedefichesListener();
        try {
            newCorpusEditor = bdfServer.getBasedefichesEditor().createCorpus(newCorpusKey, null);
            newCorpusMetadataEditor = newCorpusEditor.getCorpusMetadataEditor();
            newCorpusMetadata = (CorpusMetadata) newCorpusMetadataEditor.getMetadata();
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
        int intituleCount = corpusMetadata.getIntituleCount();
        for (int i = 0; i < intituleCount; i++) {
            Intitule intitule = corpusMetadata.getIntitule(i);
            Intitule newIntitule = newCorpusMetadataEditor.createIntitule(intitule.getName());
            int libelleCount = intitule.getLibelleCount();
            for (int j = 0; j < libelleCount; j++) {
                Libelle libelle = intitule.getLibelleByIndex(j);
                CleanedString cleanedString = CleanedString.newInstance(libelle.getLibelleString() + " (2)");
                newCorpusMetadataEditor.putLibelle(newIntitule, LibelleUtils.toLibelle(libelle.getLangInteger(), cleanedString));
            }
        }
        copyMandatoryField(FieldKey.TITRE);
        copyMandatoryField(FieldKey.LANG);
        copyMandatoryField(FieldKey.REDACTEURS);
        copySpecialField(FieldKey.SOUSTITRE);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            copyField(corpusField);
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            copyField(corpusField);
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        for (int i = 0; i < sectionCount; i++) {
            CorpusField corpusField = corpusMetadata.getSectionField(i);
            copyField(corpusField);
        }
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        newCorpusUI = bdfServer.getUIManager().getCorpusUI(newCorpusEditor.getCorpus());
        copyCorpusUI();
        bdfServer.flushBasedefichesListener();
        String[] val = new String[2];
        val[0] = corpus.getSubsetKey().toString();
        val[1] = newCorpusKey.toString();
        putResultObject(CORPUS_OBJ, newCorpusEditor.getCorpus());
        setDone("corpus_done_clone", val);

    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String newCorpusName = BdfCommand2Utils.getMandatoryParameter(requestMap, NEWCORPUS_PARAMNAME);
        try {
            newCorpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, newCorpusName);
            if (bdfServer.getBasedefiches().getSubset(newCorpusKey) != null) {
                setError("corpus_err_existingcorpusname", newCorpusKey.toString());
                return false;
            }
        } catch (java.text.ParseException pe) {
            setError("corpus_err_wrongcorpusname", newCorpusName);
            return false;
        }
        return true;
    }

    private void copyMandatoryField(FieldKey fieldKey) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        CorpusField newCorpusField = newCorpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        copyField(corpusField, newCorpusField);
    }

    private void copySpecialField(FieldKey fieldKey) {
        CorpusField corpusField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
        if (corpusField == null) {
            return;
        }
        try {
            CorpusField newCorpusField = newCorpusMetadataEditor.createCorpusField(fieldKey, CorpusField.NO_FICHEITEM_FIELD);
            copyField(corpusField, newCorpusField);
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("new corpus");
        }
    }

    private void copyField(CorpusField corpusField) {
        try {
            CorpusField newCorpusField = newCorpusMetadataEditor.createCorpusField(corpusField.getFieldKey(), corpusField.getFicheItemType());
            copyField(corpusField, newCorpusField);
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("new corpus");
        }
    }

    private void copyField(CorpusField corpusField, CorpusField newCorpusField) {
        int libelleCount = corpusField.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = corpusField.getLibelleByIndex(i);
            newCorpusMetadataEditor.putLibelle(newCorpusField, libelle);
        }
        FieldOptions fieldOptions = corpusField.getFieldOptions();
        try {
            Short informationDisplayShort = fieldOptions.getInformationDisplay();
            if (informationDisplayShort != null) {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, FieldOptionsConstants.INFORMATIONDISPLAY_OPTION, informationDisplayShort);
            }
            SubsetKey sphereKey = fieldOptions.getDefaultSphereKey();
            if (sphereKey != null) {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION, sphereKey);
            }
            boolean bool = fieldOptions.getSubfieldDisplay();
            if (bool) {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, FieldOptionsConstants.SUBFIELDDISPLAY_OPTION, Boolean.TRUE);
            }
            IntegerList currencyList = fieldOptions.getCurrencyList();
            if (currencyList != null) {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, FieldOptionsConstants.CURRENCYARRAY_OPTION, currencyList);
            }
            BasedefichesFormatDef generationFormatDef = fieldOptions.getGenerationFormatDef();
            if (generationFormatDef != null) {
                newCorpusMetadataEditor.setFieldOption(newCorpusField, FieldOptionsConstants.TITREGENERATIONFORMAT_OPTION, generationFormatDef);
            }
        } catch (FieldOptionException foe) {
            throw new ShouldNotOccurException("clone corpus");
        }
    }

    private void copyCorpusUI() {
        ComponentUIEditor newEditor = newCorpusUI.getComponentUIEditor();
        int count = corpusUI.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = corpusUI.getComponentUI(i);
            if (componentUI instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) componentUI;
                FieldUI newFieldUI = (FieldUI) newCorpusUI.getFieldUI(fieldUI.getCorpusField().getFieldKey());
                newEditor.setInputOption(newFieldUI, fieldUI.getInputOption());
            } else if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                IncludeUI newIncludeUI = newEditor.getIncludeUI(includeUI.getIncludeKey(), true);
                newEditor.setInputOption(newIncludeUI, includeUI.getInputOption());
            }
        }
        if (corpusUI.isAssociationActive()) {
            newCorpusUI.setAssociationActive(true);
        }
        List<CommentUI> commentUIList = CorpusUIUtils.getCommentUIList(corpusUI);
        for (CommentUI commentUI : commentUIList) {
            CommentUI newCommentUI = (CommentUI) newEditor.getCommentUI(commentUI.getName(), true);
            newEditor.setLocation(newCommentUI, commentUI.getLocation());
            int htmlStringCount = commentUI.getHtmlStringCount();
            for (int i = 0; i < htmlStringCount; i++) {
                int langInteger = commentUI.getHtmlLangInteger(i);
                String htmlString = commentUI.getHtmlString(i);
                newEditor.putHtmlString(newCommentUI, langInteger, htmlString);
            }
        }
        String[] array = corpusUI.getPositionArray();
        newEditor.setPositionArray(array);
    }

}
