/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.LocalizedStringList;


/**
 *
 * @author Vincent Calame
 */
public class CommentChangeCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "CommentChange";
    public final static String COMMANDKEY = "CRP-16";
    public final static String NAME_PARAMNAME = "name";
    public final static String LOCATION_PARAMNAME = "location";
    public final static String COMMENTLIB_PREFIX = "comment/";
    private Corpus corpus;
    private CommentUI commentUI;
    private int positionMask;
    private LocalizedStringList htmlStringList;

    public CommentChangeCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        ComponentUIEditor componentUIEditor = corpusUI.getComponentUIEditor();
        componentUIEditor.setLocation(commentUI, positionMask);
        int count = htmlStringList.getCount();
        for (int i = 0; i < count; i++) {
            int langInteger = htmlStringList.getLangInteger(i);
            String htmlString = htmlStringList.getString(i);
            componentUIEditor.putHtmlString(commentUI, langInteger, htmlString);
        }
        setDone("corpus_done_commentchange", commentUI.getName());
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String name = BdfCommand2Utils.getMandatoryParameter(requestMap, NAME_PARAMNAME);
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        commentUI = corpusUI.getComponentUIEditor().getCommentUI(name, false);
        if (commentUI == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, NAME_PARAMNAME, name);
        }
        positionMask = BdfCommand2Utils.getCommentPositionMask(requestMap, LOCATION_PARAMNAME);
        htmlStringList = BdfCommand2Utils.getLocalizedStringList(requestMap, COMMENTLIB_PREFIX);
        return true;
    }

}
