/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CommentRemoveCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "CommentRemove";
    public final static String COMMANDKEY = "CRP-17";
    public final static String NAME_PARAMNAME = "name";
    private Corpus corpus;
    private String name;

    public CommentRemoveCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ,corpus);
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        corpusUI.removeComment(name);
        setDone("corpus_done_commentchange", name);
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        if (!BdfCommand2Utils.checkConfirmPassword(requestMap, bdfServer)) {
            setError("global_err_password", null);
            return false;
        }
        name = BdfCommand2Utils.getMandatoryParameter(requestMap, NAME_PARAMNAME);
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        CommentUI commentUI = corpusUI.getComponentUIEditor().getCommentUI(name, false);
        if (commentUI == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, NAME_PARAMNAME, name);
        }
        return true;
    }

}
