/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.MutableInputOption;
import fr.exemole.bdfserver.ui.options.MutableLimitOption;
import fr.exemole.bdfserver.ui.options.MutableValueOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class ComponentInputCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "ComponentInput";
    public final static String COMMANDKEY = "CRP-19";
    public final static String WIDTH_PARAMSUFFIX = "|width";
    public final static String ROWS_PARAMSUFFIX = "|rows";
    public final static String INPUT_PARAMSUFFIX = "|input";
    public final static String NONEW_PARAMSUFFIX = "|nonew";
    public final static String DEFAULT_PARAMSUFFIX = "|default";
    public final static String MANDATORY_PARAMSUFFIX = "|mandatory";
    private Corpus corpus;
    private CorpusUI corpusUI;
    private List<UICouple> uiCoupleList;

    public ComponentInputCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        ComponentUIEditor editor = corpusUI.getComponentUIEditor();
        for (UICouple o : uiCoupleList) {
            ComponentUI componentUI = o.getComponentUI();
            if (componentUI instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) componentUI;
                editor.setInputOption(fieldUI, o.getInputOption());
                if (o.hasValueOption()) {
                    editor.setValueOption(fieldUI, o.getValueOption());
                }
            } else if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                if (o.hasInputOption()) {
                    editor.setInputOption(includeUI, o.getInputOption());
                }
                if (o.hasLimitOption()) {
                    editor.setLimitOption(includeUI, o.getLimitOption());
                }
                if (o.hasValueOption()) {
                    editor.setValueOption(includeUI, o.getValueOption());
                }
            }
        }
        setDone("corpus_done_componentinput", null);
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        int count = corpusUI.getComponentUICount();
        uiCoupleList = new ArrayList<UICouple>();
        for (int i = 0; i < count; i++) {
            ComponentUI obj = corpusUI.getComponentUI(i);
            InputOption inputOption = null;
            LimitOption limitOption = null;
            ValueOption valueOption = null;
            if (obj instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) obj;
                String racine = fieldUI.getCorpusField().getFieldKey().toString();
                inputOption = update(requestMap, racine, fieldUI.getInputOption(), false);
                valueOption = update(requestMap, racine, fieldUI.getValueOption());
            } else if (obj instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) obj;
                IncludeKey includeKey = includeUI.getIncludeKey();
                String racine = includeKey.toString();
                if (includeKey.isThesaurusInclude()) {
                    inputOption = update(requestMap, racine, includeUI.getInputOption(), true);
                    valueOption = update(requestMap, racine, includeUI.getValueOption());
                } else if (includeKey.equals(IncludeKey.LIAGE)) {
                    limitOption = update(requestMap, racine, includeUI.getLimitOption());
                }
            }
            UICouple o = new UICouple();
            o.setComponentUI(obj);
            o.setInputOption(inputOption);
            o.setLimitOption(limitOption);
            o.setValueOption(valueOption);
            uiCoupleList.add(o);
        }
        return true;
    }

    private LimitOption update(RequestMap requestMap, String key, LimitOption limitOption) throws BdfCommand2Exception {
        MutableLimitOption mutableLimitOption = new MutableLimitOption(limitOption);
        boolean nonewAllowed = BdfCommand2Utils.getBooleanParameter(requestMap, key + NONEW_PARAMSUFFIX);
        mutableLimitOption.setNoNewAllowed(nonewAllowed);
        return mutableLimitOption;
    }

    private InputOption update(RequestMap requestMap, String key, InputOption inputOption, boolean withInputType) throws BdfCommand2Exception {
        MutableInputOption newInputOption = new MutableInputOption(inputOption);
        String width = requestMap.getParameter(key + WIDTH_PARAMSUFFIX);
        if (width != null) {
            try {
                short widthType = DefaultInputOption.widthTypeToShort(width);
                newInputOption.setWidthType(widthType);
            } catch (IllegalArgumentException iae) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, key + WIDTH_PARAMSUFFIX, width);
            }
        }
        String rows = requestMap.getParameter(key + ROWS_PARAMSUFFIX);
        if (rows != null) {
            try {
                int r = Integer.parseInt(rows);
                if (r > 0) {
                    newInputOption.setRows(r);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        if (withInputType) {
            String param = requestMap.getParameter(key + INPUT_PARAMSUFFIX);
            if (param != null) {
                try {
                    short inputType = DefaultInputOption.inputTypeToShort(param);
                    newInputOption.setInputType(inputType);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, key + INPUT_PARAMSUFFIX, param);
                }
            }
        }
        return newInputOption;
    }

    private ValueOption update(RequestMap requestMap, String key, ValueOption valueOption) throws BdfCommand2Exception {
        if (valueOption == null) {
            return null;
        }
        MutableValueOption mutableValueOption = new MutableValueOption();
        String mandatory = requestMap.getParameter(key + MANDATORY_PARAMSUFFIX);
        if ((mandatory != null) && (mandatory.equals("1"))) {
            mutableValueOption.setMandatory(true);
        } else {
            mutableValueOption.setMandatory(false);
        }
        String defaultValue = requestMap.getParameter(key + DEFAULT_PARAMSUFFIX);
        if (defaultValue != null) {
            defaultValue = StringUtils.cleanString(defaultValue);
            mutableValueOption.setDefaultValue(defaultValue);
        }
        return mutableValueOption;
    }


    private class UICouple {

        private ComponentUI componentUI;
        private InputOption inputOption;
        private LimitOption limitOption;
        private ValueOption valueOption;

        public ComponentUI getComponentUI() {
            return componentUI;
        }

        public void setComponentUI(ComponentUI componentUI) {
            this.componentUI = componentUI;
        }

        public boolean hasInputOption() {
            return (inputOption != null);
        }

        public boolean hasLimitOption() {
            return (limitOption != null);
        }

        public boolean hasValueOption() {
            return (valueOption != null);
        }

        public InputOption getInputOption() {
            return inputOption;
        }

        public void setInputOption(InputOption inputOption) {
            this.inputOption = inputOption;
        }

        public LimitOption getLimitOption() {
            return limitOption;
        }

        public void setLimitOption(LimitOption limitOption) {
            this.limitOption = limitOption;
        }

        public ValueOption getValueOption() {
            return valueOption;
        }

        public void setValueOption(ValueOption valueOption) {
            this.valueOption = valueOption;
        }

    }

}
