/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import java.text.ParseException;
import net.mapeadores.basedefiches.conversion.FicheConverter;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.ExistingFieldKeyException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionException;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.util.collections.ArrayUtils;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class ConversionCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Conversion";
    public final static String COMMANDKEY = "CRP-22";
    public final static String FIELD_PARAMNAME = "field";
    public final static String NEWFIELD_PARAMNAME = "newfield";
    private Corpus corpus;
    private CorpusField sourceField;
    private FieldKey newFieldKey;

    public ConversionCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        CorpusField newCorpusField = null;
        CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey());
        CorpusMetadataEditor corpusMetadataEditor = corpusEditor.getCorpusMetadataEditor();
        bdfServer.bufferBasedefichesListener();
        try {
            short ficheItemType = getFicheItemType();
            newCorpusField = corpusMetadataEditor.createCorpusField(newFieldKey, ficheItemType);
            if ((newFieldKey.isInformationFieldKey()) && (sourceField.getFieldKey().ownsToCorpsdefiche())) {
                corpusMetadataEditor.setFieldOption(newCorpusField, FieldOptionsUtils.INFORMATIONDISPLAY_OPTION, new Short(FieldOptionsUtils.BLOCK_DISPLAY));
            }
        } catch (ExistingFieldKeyException efke) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        bdfServer.flushBasedefichesListener();
        FicheConverter ficheConverter = FicheConverter.getInstance(sourceField, newCorpusField);
        int size = corpus.getTicketLength();
        int ficheCount = 0;
        for (int i = 0; i < size; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
            if (abregeFiche != null) {
                Fiche fiche = corpus.getFiche(abregeFiche);
                ficheConverter.convert(abregeFiche, fiche);
                corpusEditor.saveFiche(abregeFiche, fiche);
                ficheCount++;
            }
        }
        Object[] resultObject = ArrayUtils.toObjectArray(sourceField.getFieldKey().toString(), newFieldKey.toString(), new Integer(ficheCount));
        addLog(ficheConverter.getProcessLog());
        setDone("corpus_done_conversion", resultObject);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        String fieldParamName = BdfCommand2Utils.getMandatoryParameter(requestMap, FIELD_PARAMNAME);
        try {
            FieldKey fieldKey = FieldKey.parse(fieldParamName);
            sourceField = corpusMetadata.getCorpusFieldByFieldKey(fieldKey);
            if (sourceField == null) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNFIELDKEY, FIELD_PARAMNAME, fieldParamName);
            }
        } catch (ParseException pe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGFIELDKEY, FIELD_PARAMNAME, fieldParamName);
        }
        String newFieldParamName = BdfCommand2Utils.getMandatoryParameter(requestMap, NEWFIELD_PARAMNAME);
        try {
            newFieldKey = FieldKey.parse(newFieldParamName);
            if (corpusMetadata.getCorpusFieldByFieldKey(newFieldKey) != null) {
                setError("corpus_err_existingtype", newFieldKey.toString());
                return false;
            }
            if (newFieldKey.isSpecialFieldKey()) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGFIELDKEY, NEWFIELD_PARAMNAME, newFieldParamName);
            }
        } catch (ParseException pe) {
            setError("corpus_err_badtype", newFieldParamName);
            return false;
        }
        return true;
    }

    private short getFicheItemType() {
        if (newFieldKey.isSectionFieldKey()) {
            return CorpusField.NO_FICHEITEM_FIELD;
        }
        FieldKey sourceFieldKey = sourceField.getFieldKey();
        if (sourceFieldKey.isSectionFieldKey()) {
            return CorpusField.PARA_FIELD;
        }
        short current = sourceField.getFicheItemType();
        if (current == CorpusField.NO_FICHEITEM_FIELD) {
            return CorpusField.ITEM_FIELD;
        }
        return current;
    }

}
