/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.ExistingSubsetException;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.CorpusEditor;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CorpusCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "CorpusCreation";
    public final static String COMMANDKEY = "CRP-01";
    public final static String NEWCORPUS_PARAMNAME = "newcorpus";
    public final static String CORPUSTYPE_PARAMNAME = "corpustype";
    public final static String NORMAL_PARAMVALUE = "normal";
    public final static String CORPUSSLAVE_PARAMVALUE = "corpusslave";
    public final static String THESAURUSSLAVE_PARAMVALUE = "thesaurusslave";
    public final static String MASTERTHESAURUS_PARAMNAME = "masterthesaurus";
    public final static String MASTERCORPUS_PARAMNAME = "mastercorpus";
    private SubsetKey newCorpusKey;
    private Subset masterSubset;

    public CorpusCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    public void doCommand() throws BdfCommand2Exception {
        try {
            CorpusEditor corpusEditor = bdfServer.getBasedefichesEditor().createCorpus(newCorpusKey, masterSubset);
            putResultObject(CORPUS_OBJ, corpusEditor.getCorpus());
            setDone("corpus_done_corpuscreation", newCorpusKey.toString());
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String newCorpusName = getMandatoryParameter(NEWCORPUS_PARAMNAME);
        initSubsetMaster();
        newCorpusName = newCorpusName.trim();
        if (newCorpusName.length() == 0) {
            setError("corpus_err_missingcorpusname", newCorpusName);
            return false;
        }
        try {
            newCorpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, newCorpusName);
            if (bdfServer.getBasedefiches().getSubset(newCorpusKey) != null) {
                setError("corpus_err_existingcorpusname", newCorpusKey.toString());
                return false;
            }
        } catch (java.text.ParseException pe) {
            setError("corpus_err_wrongcorpusname", newCorpusName);
            return false;
        }
        return true;
    }

    private void initSubsetMaster() throws ParameterBdfCommand2Exception {
        String corpusType = getMandatoryParameter(CORPUSTYPE_PARAMNAME);
        if (corpusType.equals(NORMAL_PARAMVALUE)) {
            masterSubset = null;
        } else if (corpusType.equals(THESAURUSSLAVE_PARAMVALUE)) {
            String masterthesaurusId = getMandatoryParameter(MASTERTHESAURUS_PARAMNAME);
            masterSubset = BasedefichesUtils.getThesaurus(bdfServer.getBasedefiches(), masterthesaurusId);
            if (masterSubset == null) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSET, MASTERTHESAURUS_PARAMNAME, masterthesaurusId);
            }
        } else if (corpusType.equals(CORPUSSLAVE_PARAMVALUE)) {
            String mastercorpusId = getMandatoryParameter(MASTERCORPUS_PARAMNAME);
            masterSubset = BasedefichesUtils.getCorpus(bdfServer.getBasedefiches(), mastercorpusId);
            if (masterSubset == null) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSET, MASTERCORPUS_PARAMNAME, mastercorpusId);
            }
            if (masterSubset.getMasterSubset() != null) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.SLAVESUBSET, MASTERCORPUS_PARAMNAME, mastercorpusId);
            }
        } else {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, CORPUSTYPE_PARAMNAME, corpusType);
        }
    }

}
