/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import net.mapeadores.basedefiches.NoRemoveableSubsetException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class CorpusRemoveCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "CorpusRemove";
    public final static String COMMANDKEY = "CRP-09";
    private Corpus corpus;

    public CorpusRemoveCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        SubsetKey corpusKey = corpus.getSubsetKey();
        try {
            bdfServer.getBasedefichesEditor().removeCorpus(corpus);
            setDone("corpus_done_corpusremove", corpusKey.getSubsetName());
            putResultObject(SUBSETKEY_OBJ, corpusKey);
        } catch (NoRemoveableSubsetException nrse) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        if (!BdfCommand2Utils.checkConfirmPassword(requestMap, bdfServer)) {
            setError("global_err_password", null);
            return false;
        }
        if (!basedefiches.isRemoveable(corpus)) {
            setError("corpus_err_corpusnotremoveable", null);
            return false;
        }
        return true;
    }

}
