/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.ExistingFieldKeyException;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FieldCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "FieldCreation";
    public final static String CATEGORY_PARAMNAME = "category";
    public final static String TYPE_PARAMNAME = "type";
    public final static String FICHEITEMTYPE_PARAMNAME = "ficheitemtype";
    private Corpus corpus;
    private FieldKey newFieldKey;
    private short ficheItemType = CorpusField.NO_FICHEITEM_FIELD;

    public FieldCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        CorpusMetadataEditor corpusMetadataEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        if (newFieldKey.isSpecialFieldKey()) {
            try {
                corpusMetadataEditor.createCorpusField(newFieldKey, CorpusField.NO_FICHEITEM_FIELD);
                setDone("corpus_done_activation_" + newFieldKey.toString(), null);
            } catch (ExistingFieldKeyException efke) {
                throw new ShouldNotOccurException("test done before");
            }
        } else {
            try {
                corpusMetadataEditor.createCorpusField(newFieldKey, ficheItemType);
                setDone("corpus_done_fieldcreation", newFieldKey.toString());
            } catch (ExistingFieldKeyException efke) {
                throw new ShouldNotOccurException("test done before");
            }
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String categoryParameter = BdfCommand2Utils.getMandatoryParameter(requestMap, CATEGORY_PARAMNAME);
        short category = 0;
        try {
            category = FieldKey.categoryToShort(categoryParameter);
        } catch (IllegalArgumentException iae) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, CATEGORY_PARAMNAME, categoryParameter);
        }
        String typeParameter = BdfCommand2Utils.getMandatoryParameter(requestMap, TYPE_PARAMNAME);
        typeParameter = typeParameter.trim();
        if (typeParameter.length() == 0) {
            setError("corpus_err_missingfieldtype", null);
            return false;
        }
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        if (category == FieldKey.SPECIAL_CATEGORY) {
            if (typeParameter.equals("soustitre")) {
                newFieldKey = FieldKey.SOUSTITRE;
            } else {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, TYPE_PARAMNAME, typeParameter);
            }
            if (corpusMetadata.getCorpusFieldByFieldKey(newFieldKey) != null) {
                setError("corpus_err_stillactivated_" + newFieldKey.toString(), null);
                return false;
            }
        } else {
            if (!StringUtils.isTechnicalName(typeParameter, false)) {
                setError("corpus_err_badtype", typeParameter);
                return false;
            }
            switch (category) {
                case FieldKey.PROPRIETE_CATEGORY:
                    newFieldKey = FieldKey.getProprieteFieldKey(typeParameter);
                    break;
                case FieldKey.INFORMATION_CATEGORY:
                    newFieldKey = FieldKey.getInformationFieldKey(typeParameter);
                    break;
                case FieldKey.SECTION_CATEGORY:
                    newFieldKey = FieldKey.getSectionFieldKey(typeParameter);
                    break;
                default:
                    throw new SwitchException("category = " + category);
            }
            if (corpusMetadata.getCorpusFieldByFieldKey(newFieldKey) != null) {
                setError("corpus_err_existingtype", newFieldKey.toString());
                return false;
            }
            if ((category == FieldKey.INFORMATION_CATEGORY) || (category == FieldKey.PROPRIETE_CATEGORY)) {
                String ficheItemTypeParameter = BdfCommand2Utils.getMandatoryParameter(requestMap, FICHEITEMTYPE_PARAMNAME);
                if (ficheItemTypeParameter.length() == 0) {
                    setError("corpus_err_noficheitemtype", typeParameter);
                    return false;
                }
                try {
                    ficheItemType = CorpusMetadataUtils.ficheItemTypeToShort(ficheItemTypeParameter);
                } catch (IllegalArgumentException iae) {
                    throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, FICHEITEMTYPE_PARAMNAME, ficheItemTypeParameter);
                }
            }
        }
        return true;
    }

    public static String getCommandKey(short category) {
        switch (category) {
            case FieldKey.PROPRIETE_CATEGORY:
                return "CRP-04";
            case FieldKey.INFORMATION_CATEGORY:
                return "CRP-05";
            case FieldKey.SECTION_CATEGORY:
                return "CRP-06";

            default:
                throw new SwitchException("category = " + category);
        }
    }

    public static String getCommandKey(FieldKey fieldKey) {
        if (fieldKey.equals(FieldKey.SOUSTITRE)) {
            return "CRP-02";
        }
        throw new SwitchException("fieldKey = " + fieldKey.toString());
    }

}
