/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.MandatoryFieldException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class FieldRemoveCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "FieldRemove";
    public final static String COMMANDKEY = "CRP-08";
    public final static String FIELDKEY_PARAMNAME = "fieldkey";
    private Corpus corpus;
    private CorpusField corpusField;

    public FieldRemoveCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ,corpus);
        CorpusMetadataEditor corpusMetadataEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        try {
            String key = corpusField.getFieldKey().toString();
            corpusMetadataEditor.removeCorpusField(corpusField);
            setDone("corpus_done_fieldremove", key);
        } catch (MandatoryFieldException mfe) {
            throw new ShouldNotOccurException("test done before");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String fieldKeyValue = BdfCommand2Utils.getMandatoryParameter(requestMap, FIELDKEY_PARAMNAME);
        fieldKeyValue = fieldKeyValue.trim();
        if (fieldKeyValue.length() == 0) {
            setError("corpus_err_missingfieldtype", null);
            return false;
        }
        FieldKey fieldKey = null;
        try {
            fieldKey = FieldKey.parse(fieldKeyValue);
        } catch (java.text.ParseException pe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGFIELDKEY, FIELDKEY_PARAMNAME, fieldKeyValue);
        }
        corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
        if ((corpusField == null) || (fieldKey.isMandatoryFieldKey())) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGFIELDKEY, FIELDKEY_PARAMNAME, fieldKeyValue);
        }
        if (!BdfCommand2Utils.checkConfirmPassword(requestMap, bdfServer)) {
            setError("global_err_password", null);
            return false;
        }
        return true;
    }

}
