/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.TypoOptions;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import java.text.ParseException;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.ChangeLibelleHolder;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class IncludeChangeCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "IncludeChange";
    public final static String COMMANDKEY = "CRP-13";
    public final static String DEFAULTLIBELLEUSAGE_PARAMNAME = "defaultlibelleusage";
    public final static String KEY_PARAMNAME = "key";
    public final static String LIB_PARAMPREFIX = "lib/";
    private CorpusUI corpusUI;
    private IncludeUI includeUI;
    private Corpus corpus;
    private boolean useDefaultLibelle = false;
    private ChangeLibelleHolder changeLibelleHolder;

    public IncludeChangeCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        boolean done = updateIncludeUI(includeUI);
        if (done) {
            setDone("corpus_done_includeuichange", includeUI.getIncludeKey().toString());
        }
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String key = BdfCommand2Utils.getMandatoryParameter(requestMap, KEY_PARAMNAME);
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        try {
            IncludeKey includeKey = IncludeKey.parse(key);
            includeUI = corpusUI.getIncludeUI(includeKey);
            if (includeUI == null) {
                throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNINCLUDEKEY, KEY_PARAMNAME, key);
            }
        } catch (ParseException pe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGINCLUDEKEY, KEY_PARAMNAME, key);
        }
        useDefaultLibelle = BdfCommand2Utils.getBooleanParameter(requestMap, DEFAULTLIBELLEUSAGE_PARAMNAME);
        if (!useDefaultLibelle) {
            changeLibelleHolder = BdfCommand2Utils.getChangeLibelleHolder(requestMap, LIB_PARAMPREFIX);
        }
        return true;
    }

    private boolean updateIncludeUI(IncludeUI includeUI) {
        ComponentUIEditor editor = corpusUI.getComponentUIEditor();
        if (useDefaultLibelle) {
            if (includeUI.useDefaultLibelle()) {
                return false;
            } else {
                editor.setDefaultLibelleUsage(includeUI, true);
                return true;
            }
        }
        editor.setDefaultLibelleUsage(includeUI, false);
        TextParser textParser = new TextParser();
        boolean done = false;
        int libelleCount = changeLibelleHolder.getLibelleCount();
        for (int i = 0; i < libelleCount; i++) {
            Libelle libelle = changeLibelleHolder.getLibelleByIndex(i);
            int currentLangInteger = libelle.getLangInteger();
            TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLangInteger);
            textParser.setQuoteOptions(typoOptions.getQuoteOptions());
            String libelleString = libelle.getLibelleString();
            String checkedLibelleString = textParser.parseString(libelleString);
            if (!checkedLibelleString.equals(libelleString)) {
                CleanedString cleanedString = CleanedString.newInstance(checkedLibelleString);
                if (cleanedString == null) {
                    boolean b = editor.removeLibelle(includeUI, currentLangInteger);
                    if (b) {
                        done = true;
                    }
                    continue;
                } else {
                    libelle = LibelleUtils.toLibelle(currentLangInteger, cleanedString);
                }
            }
            boolean b = editor.putLibelle(includeUI, libelle);
            if (b) {
                done = true;
            }
        }
        int removedCount = changeLibelleHolder.getRemovedLangCount();
        for (int i = 0; i < removedCount; i++) {
            boolean b = editor.removeLibelle(includeUI, changeLibelleHolder.getRemovedLangInteger(i));
            if (b) {
                done = true;
            }
        }
        return done;
    }

}
