/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class IncludeCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "IncludeCreation";
    public final static String COMMANDKEY = "CRP-12";
    public final static String SUBSET_PARAMNAME = "subset";
    public final static String POIDSFILTER_PARAMNAME = "poidsfilter";
    public final static String NO_POIDSFILTER_PARAMVALUE = "0";
    public final static String YES_POIDSFILTER_PARAMVALUE = "1";
    public final static String POIDSFILTERVALUE_PARAMNAME = "poidsfiltervalue";
    private Corpus corpus;
    private CorpusUI corpusUI;
    private Subset subset;
    private IncludeKey includeKey;

    public IncludeCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ,corpus);
        corpusUI.getComponentUIEditor().getIncludeUI(includeKey, true);
        setDone("corpus_done_includeuiadd", includeKey);
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String subsetString = getMandatoryParameter(SUBSET_PARAMNAME);
        SubsetKey subsetKey = null;
        try {
            subsetKey = SubsetKey.parse(subsetString);
        } catch (java.text.ParseException pe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGSUBSETNAME, SUBSET_PARAMNAME, subsetString);
        }
        subset = bdfServer.getBasedefiches().getSubset(subsetKey);
        if (subset == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSET, SUBSET_PARAMNAME, subsetString);
        }
        String poidsFilterString = getMandatoryParameter(POIDSFILTER_PARAMNAME);
        int poidsFilter = -1;
        if (poidsFilterString.equals(YES_POIDSFILTER_PARAMVALUE)) {
            String poidsFilterValue = BdfCommand2Utils.getMandatoryParameter(requestMap, POIDSFILTERVALUE_PARAMNAME);
            boolean error = false;
            try {
                poidsFilter = Integer.parseInt(poidsFilterValue);
                if (poidsFilter < 1) {
                    error = true;
                }
            } catch (NumberFormatException nfe) {
                error = true;
            }
            if (error) {
                setError("corpus_err_poidsfiltervalue", poidsFilterValue);
                return false;
            }
        } else if (!poidsFilterString.equals(NO_POIDSFILTER_PARAMVALUE)) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, POIDSFILTER_PARAMNAME, poidsFilterString);
        }
        try {
            includeKey = IncludeKey.newInstance(subsetKey, poidsFilter);
        } catch(IllegalArgumentException iae) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNSUBSET, SUBSET_PARAMNAME, subsetString);
        }
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        if (corpusUI.ownsToRattachement(subsetKey)) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.SLAVESUBSET, SUBSET_PARAMNAME, subsetString);
        }
        if (corpusUI.getIncludeUI(includeKey) != null) {
            setError("corpus_err_existingincludekey", includeKey.toString());
            return false;
        }
        return true;
    }

}
