/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class IncludeRemoveCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "IncludeRemove";
    public final static String COMMANDKEY = "CRP-14";
    public final static String KEY_PARAMNAME = "key";
    private Corpus corpus;
    private IncludeKey includeKey;
    private CorpusUI corpusUI;

    public IncludeRemoveCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ,corpus);
        corpusUI.removeInclude(includeKey);
        setDone("corpus_done_includeuiremove", includeKey.toString());
        bdfServer.saveUIManager();
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        String key = BdfCommand2Utils.getMandatoryParameter(requestMap, KEY_PARAMNAME);
        try {
            includeKey = IncludeKey.parse(key);
        } catch (java.text.ParseException pe) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.WRONGINCLUDEKEY, KEY_PARAMNAME, key);
        }
        corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        if (corpusUI.getIncludeUI(includeKey) == null) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNINCLUDEKEY, KEY_PARAMNAME, key);
        }
        return true;
    }

}
