/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.request.TypoOptions;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataEditor;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class IntitulesCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Intitules";
    public final static String COMMANDKEY = "CRP-10";
    private Corpus corpus;

    public IntitulesCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() {
        putResultObject(CORPUS_OBJ, corpus);
        CorpusMetadataEditor corpusMetadataEditor = bdfServer.getBasedefichesEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        boolean done = false;
        TextParser textParser = new TextParser();
        for (String paramName : requestMap.getParameterNameSet()) {
            int idx = paramName.lastIndexOf('/');
            if (idx == -1) {
                continue;
            }
            String fieldName = paramName.substring(0, idx);
            try {
                int currentLangInteger = LangInteger.parse(paramName.substring(idx + 1));
                TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLangInteger);
                textParser.setQuoteOptions(typoOptions.getQuoteOptions());
                String nvlibelle = textParser.parseString(requestMap.getParameter(paramName));
                CleanedString cleanedString = CleanedString.newInstance(nvlibelle);
                if (fieldName.startsWith("intitule_")) {
                    String intituleName = fieldName.substring(9);
                    Intitule intitule = BasedefichesUtils.getOrCreateIntitule(corpusMetadataEditor, intituleName);
                    boolean stepDone = false;
                    if (cleanedString == null) {
                        stepDone = corpusMetadataEditor.removeLibelle(intitule, currentLangInteger);
                    } else {
                        stepDone = corpusMetadataEditor.putLibelle(intitule, LibelleUtils.toLibelle(currentLangInteger, cleanedString));
                    }
                    if (stepDone) {
                        done = true;
                    }
                } else {
                    try {
                        FieldKey fieldKey = FieldKey.parse(fieldName);
                        CorpusField corpusField = corpus.getCorpusMetadata().getCorpusFieldByFieldKey(fieldKey);
                        if (corpusField != null) {
                            boolean stepDone = false;
                            if (cleanedString == null) {
                                stepDone = corpusMetadataEditor.removeLibelle(corpusField, currentLangInteger);
                            } else {
                                stepDone = corpusMetadataEditor.putLibelle(corpusField, LibelleUtils.toLibelle(currentLangInteger, cleanedString));
                            }
                            if (stepDone) {
                                done = true;
                            }
                        }
                    } catch (java.text.ParseException pe) {
                    }
                }
            } catch (LangIntegerException ile) {
            }
        }
        if (done) {
            setDone("corpus_done_intitules", null);
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        corpus = BdfCommand2Utils.getMandatoryCorpus(basedefiches, requestMap);
        return true;
    }

}
