/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.envoi;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.email.EnvoiParameters;
import fr.exemole.bdfserver.email.FicheAttachmentParameters;
import fr.exemole.bdfserver.email.SendEngine;
import fr.exemole.bdfserver.email.SendReport;
import fr.exemole.bdfserver.email.SmtpManager;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.Addenda;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.association.VersionName;
import net.mapeadores.basedefiches.association.VersionName.ParseResult;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class SendCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "Send";
    public final static String BCC_PARAMNAME = "bcc";
    public final static String TYPE_PARAMNAME = "type";
    public final static String SUBJECT_PARAMNAME = "subject";
    public final static String TO_PARAMNAME = "to";
    public final static String MESSAGE_PARAMNAME = "message";
    public final static String DOCUMENTVERSION_PARAMNAME = "documentversion";
    public final static String FICHEVERSION_PARAMNAME = "ficheversion";
    public final static String HTML_FICHEVERSION_PARAMVALUE = "html";
    public final static String ODT_FICHEVERSION_PARAMVALUE = "odt";
    private SmtpManager smtpManager;
    private EnvoiParameters envoiParameters;

    public SendCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return false;
    }

    protected void doCommand() throws BdfCommand2Exception {
        if (envoiParameters.getWrongAddressList() != null) {
            setError("envoi_err_wrongaddress", null);
            return;
        }
        SendEngine sendEngine = new SendEngine(smtpManager, bdfServer, bdfUser);
        SendReport sendReport = sendEngine.sendEmail(envoiParameters);
        switch (sendReport.getState()) {
            case SendReport.OK_STATE:
                setDone(getDoneKey(), null);
                break;
            case SendReport.EMAILBUILD_ERROR_STATE:
                setError("envoi_err_emailbuild", sendReport.getErrorMessage());
                break;
            case SendReport.SEND_ERROR_STATE:
                setError("envoi_err_send", sendReport.getErrorMessage());
                break;
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        smtpManager = testSmtpManager(bdfContext);
        envoiParameters = buildFromRequest(bdfServer, bdfUser, requestMap, false);
        if (envoiParameters.getEnvoiType() == EnvoiParameters.ENVOI_SELECTION) {
            Set<AbregeFiche> setAbreges = new LinkedHashSet<AbregeFiche>();
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                String[] param_selection = requestMap.getParameterValues("selection_" + corpus.getSubsetKey().getSubsetName());
                if ((param_selection != null) && (param_selection.length > 0)) {
                    for (int j = 0; j < param_selection.length; j++) {
                        try {
                            int idcorpus = Integer.parseInt(param_selection[j]);
                            AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                            if (abregeFiche != null) {
                                setAbreges.add(abregeFiche);
                            }
                        } catch (NumberFormatException nfe) {
                        }
                    }
                }
            }
            int size = setAbreges.size();
            if (size > 0) {
                AbregeFiche[] array = new AbregeFiche[size];
                int p = 0;
                for (AbregeFiche abregeFiche : setAbreges) {
                    array[p] = abregeFiche;
                    p++;
                }
                envoiParameters.setAbregeFicheArray(array);
            }
        }
        putResultObject(ENVOIPARAMETERS_OBJ, envoiParameters);
        if (envoiParameters.getSubject() == null) {
            setError("envoi_err_nosubject", null);
            return false;
        }
        if (envoiParameters.getMessage() == null) {
            setError("envoi_err_nomessage", null);
            return false;
        }
        if (envoiParameters.hasNoTo()) {
            setError("envoi_err_noto", null);
            return false;
        }
        return true;
    }

    private String getDoneKey() {
        switch (envoiParameters.getEnvoiType()) {
            case EnvoiParameters.ENVOI_FICHEUNIQUE:
                return "envoi_done_send_fiche";
            case EnvoiParameters.ENVOI_COMPILATION:
                return "envoi_done_send_compilation";
            case EnvoiParameters.ENVOI_SELECTION:
                return "envoi_done_send_selection";
            default:
                return null;
        }
    }

    public static EnvoiParameters buildFromRequest(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap, boolean withDefaultValue) throws BdfCommand2Exception {
        CourrielBean fromCourrielBean = BdfUserUtils.getRedacteur(bdfUser).getCourrielBean();
        if (fromCourrielBean == null) {
            throw new BdfCommand2Exception("BdfUser has no email address");
        }
        String typeString = BdfCommand2Utils.getMandatoryParameter(requestMap, TYPE_PARAMNAME);
        short envoiType = 0;
        try {
            envoiType = EnvoiParameters.typeToShort(typeString);
        } catch (IllegalArgumentException iae) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, TYPE_PARAMNAME + " = " + typeString);
        }
        EnvoiParameters envoiParameters = null;
        if (envoiType == EnvoiParameters.ENVOI_FICHEUNIQUE) {
            AbregeFiche abregeFiche = BdfCommand2Utils.getMandatoryAbregeFiche(bdfServer.getBasedefiches(), requestMap);
            BdfCommand2Utils.testReadFichePermission(bdfServer, bdfUser, abregeFiche);
            envoiParameters = EnvoiParameters.newFicheUniqueInstance(fromCourrielBean, abregeFiche);
            if ((withDefaultValue) && (envoiParameters.getSubject() == null)) {
                envoiParameters.setSubject(BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
            }
            FicheAttachmentParameters ficheAttachmentParameters = new FicheAttachmentParameters();
            if (withDefaultValue) {
                ficheAttachmentParameters.setWithHtml(true);
            } else {
                initFicheAttachmentParameters(bdfServer.getBasedefiches(), ficheAttachmentParameters, requestMap);
            }
            envoiParameters.setFicheAttachmentParameters(ficheAttachmentParameters);
        } else if (envoiType == EnvoiParameters.ENVOI_COMPILATION) {
            envoiParameters = EnvoiParameters.newCompilationInstance(fromCourrielBean);
        } else if (envoiType == EnvoiParameters.ENVOI_SELECTION) {
            envoiParameters = EnvoiParameters.newSelectionInstance(fromCourrielBean);
        }
        String[] toValues = requestMap.getParameterValues(TO_PARAMNAME);
        if (toValues != null) {
            List<String> toList = new ArrayList<String>();
            int length = toValues.length;
            for (int i = 0; i < length; i++) {
                String[] tokens = StringUtils.getTechnicalTokens(toValues[i], false);
                int ln = tokens.length;
                for (int j = 0; j < ln; j++) {
                    toList.add(tokens[j]);
                }
            }
            int size = toList.size();
            if (size > 0) {
                envoiParameters.parseToArray(bdfServer.getBasedefiches(), toList, bdfUser.getRedacteurKey().getSphereKey());
            }
        }
        String subject = requestMap.getParameter(SUBJECT_PARAMNAME);
        if (subject != null) {
            envoiParameters.setSubject(subject);
        }
        boolean withBcc = false;
        if (withDefaultValue) {
            withBcc = true;
        } else {
            withBcc = BdfCommand2Utils.getBooleanParameter(requestMap, BCC_PARAMNAME);
        }
        envoiParameters.setWithRedacteurBcc(withBcc);
        envoiParameters.setMessage(requestMap.getParameter(MESSAGE_PARAMNAME));
        return envoiParameters;
    }

    private static void initFicheAttachmentParameters(Basedefiches basedefiches, FicheAttachmentParameters ficheAttachmentParameters, RequestMap requestMap) {
        String[] values = requestMap.getParameterValues(FICHEVERSION_PARAMNAME);
        if (values != null) {
            int length = values.length;
            for (int i = 0; i < length; i++) {
                String value = values[i];
                if (value.equals(HTML_FICHEVERSION_PARAMVALUE)) {
                    ficheAttachmentParameters.setWithHtml(true);
                } else if (value.equals(ODT_FICHEVERSION_PARAMVALUE)) {
                    ficheAttachmentParameters.setWithOdt(true);
                }
            }
        }
        String[] versionArray = requestMap.getParameterValues(DOCUMENTVERSION_PARAMNAME);
        if (versionArray != null) {
            Addenda addenda = (Addenda) basedefiches.getSubset(SubsetKey.ADDENDA);
            int length = versionArray.length;
            for (int i = 0; i < length; i++) {
                String name = versionArray[i];
                ParseResult parseResult = VersionName.parse(name, true);
                if (parseResult != null) {
                    Document document = addenda.getDocumentByBasename(parseResult.getBasename());
                    if (document != null) {
                        Version version = document.getVersionByExtension(parseResult.getExtension());
                        if (version != null) {
                            ficheAttachmentParameters.addVersion(version);
                        }
                    }
                }
            }
        }
    }

    public static SmtpManager testSmtpManager(BdfContext bdfContext) throws BdfCommand2Exception {
        SmtpManager smtpManager = (SmtpManager) bdfContext.getContextObject(BdfContextConstants.SMTPMANAGER_NAME);
        if (!smtpManager.isInit()) {
            StringBuilder buf = new StringBuilder();
            buf.append("Smtp parameters are not initialized (InitError: ");
            buf.append(smtpManager.getInitState());
            List<String> errorList = smtpManager.getErrorList();
            if (errorList != null) {
                for (String error : errorList) {
                    buf.append(" / ");
                    buf.append(error);
                }
            }
            buf.append(")");
            throw new BdfCommand2Exception(buf.toString());
        }
        return smtpManager;
    }

}

