/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.saisie;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementEditor;
import net.mapeadores.basedefiches.croisement.DefaultFicheCroisementList;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class SelectionIndexationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "SelectionIndexation";
    public final static String COMMANDKEY = "SAI-04";
    private Motcle motcle;
    private List<CorpusInfo> corpusInfoList;

    public SelectionIndexationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        int count = corpusInfoList.size();
        CroisementEditor croisementEditor = bdfServer.getBasedefichesEditor().getCroisementEditor();
        int totalajout = 0;
        int totalsupp = 0;
        for (int i = 0; i < count; i++) {
            CorpusInfo corpusInfo = corpusInfoList.get(i);
            totalajout = totalajout + corpusInfo.updateAjout(croisementEditor);
            totalsupp = totalsupp + corpusInfo.updateSupp(croisementEditor);
        }
        int[] totaux = {totalajout, totalsupp};
        putResultObject(INTARRAY_OBJ, totaux);
        putResultObject(MOTCLE_OBJ, motcle);
        setDone("saisie_done_massindexation", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        motcle = BdfCommand2Utils.getMandatoryMotcle(basedefiches, requestMap);
        corpusInfoList = new ArrayList<CorpusInfo>();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            Subset masterSubset = corpus.getMasterSubset();
            if ((masterSubset != null) && (masterSubset.equals(motcle.getThesaurus()))) {
                continue;
            }
            SubsetKey corpusKey = corpus.getSubsetKey();
            List<AbregeFiche> suppList = getSuppList(corpus, requestMap.getParameterValues("supp_" + corpusKey.getSubsetName()));
            FicheCroisementList ajoutList = getAjoutList(corpus, requestMap.getParameterValues("ajout_" + corpusKey.getSubsetName()));
            if ((suppList != null) || (ajoutList != null)) {
                corpusInfoList.add(new CorpusInfo(motcle, corpus, ajoutList, suppList));
            }
        }
        if (corpusInfoList.isEmpty()) {
            return false;
        } else {
            return true;
        }
    }

    private FicheCroisementList getAjoutList(Corpus corpus, String[] ajouts) {
        if (ajouts == null) {
            return null;
        }
        int length = ajouts.length;
        DefaultFicheCroisementList abregeFicheList = new DefaultFicheCroisementList(corpus.getSubsetKey(), length);
        for (int i = 0; i < length; i++) {
            try {
                int idcorpus = Integer.parseInt(ajouts[i]);
                AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche != null) {
                    int poids = getPoids(abregeFiche);
                    abregeFicheList.add(abregeFiche, poids);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        if (abregeFicheList.getSubsetItemCount() == 0) {
            return null;
        }
        return abregeFicheList;
    }

    private List<AbregeFiche> getSuppList(Corpus corpus, String[] supps) {
        if (supps == null) {
            return null;
        }
        List<AbregeFiche> suppList = new ArrayList<AbregeFiche>();
        int length = supps.length;
        for (int i = 0; i < length; i++) {
            try {
                int idcorpus = Integer.parseInt(supps[i]);
                AbregeFiche abregeFiche = corpus.getAbregeFicheByIdcorpus(idcorpus);
                if (abregeFiche != null) {
                    suppList.add(abregeFiche);
                }
            } catch (NumberFormatException nfe) {
            }
        }
        if (suppList.size() == 0) {
            return null;
        }
        return suppList;
    }

    private int getPoids(AbregeFiche abregeFiche) {
        String param = requestMap.getParameter("poids_" + abregeFiche.getCorpus().getSubsetKey().getSubsetName() + "_" + String.valueOf(abregeFiche.getIdcorpus()));
        if ((param == null) || (param.length() == 0)) {
            return 1;
        }
        try {
            int poids = Integer.parseInt(param);
            if (poids < 1) {
                poids = 1;
            }
            return poids;
        } catch (NumberFormatException nfe) {
            return 1;
        }
    }


    private static class CorpusInfo {

        private Corpus corpus;
        private FicheCroisementList ajoutList;
        private List<AbregeFiche> suppList;
        private Croisement croisement;
        private SubsetKey corpusKey;
        private SubsetKey thesaurusKey;
        private Motcle motcle;

        CorpusInfo(Motcle motcle, Corpus corpus, FicheCroisementList ajoutList, List<AbregeFiche> suppList) {
            this.corpus = corpus;
            this.ajoutList = ajoutList;
            this.suppList = suppList;
            this.motcle = motcle;
            this.corpusKey = corpus.getSubsetKey();
            this.thesaurusKey = motcle.getThesaurus().getSubsetKey();
            this.croisement = corpus.getBasedefiches().getCroisement(corpusKey, thesaurusKey);
        }

        int updateAjout(CroisementEditor croisementEditor) {
            if (ajoutList == null) {
                return 0;
            }
            int count = ajoutList.getSubsetItemCount();
            DefaultFicheCroisementList newList = new DefaultFicheCroisementList(corpusKey, count);
            for (int i = 0; i < count; i++) {
                AbregeFiche abregeFiche = ajoutList.getAbregeFiche(i);
                if (croisement.getPoids(corpusKey, abregeFiche.getTicket(), thesaurusKey, motcle.getTicket()) == 0) {
                    int poids = ajoutList.getPoids(i);
                    newList.add(abregeFiche, poids);
                }
            }
            int result = newList.getSubsetItemCount();
            if (result > 0) {
                croisementEditor.updateCroisement(motcle, newList, CroisementEditor.APPEND_AND_POIDS_REPLACE);
            }
            return result;
        }

        int updateSupp(CroisementEditor croisementEditor) {
            if (suppList == null) {
                return 0;
            }
            int count = suppList.size();
            int result = 0;
            for (int i = 0; i < count; i++) {
                AbregeFiche abregeFiche = suppList.get(i);
                if (croisement.getPoids(corpusKey, abregeFiche.getTicket(), thesaurusKey, motcle.getTicket()) != 0) {
                    croisementEditor.removeCroisement(abregeFiche, motcle);
                    result++;
                }
            }
            return result;
        }

    }

}
