/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import net.mapeadores.basedefiches.ExistingIdException;
import net.mapeadores.basedefiches.sphere.Idsphere;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurCreation";
    public final static String COMMANDKEY = "SPH-04";
    public static String NEWIDSPHERE_PARAMNAME = "newidsphere";
    private int newIdsphere;
    private Sphere sphere;

    public RedacteurCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        SphereEditor sphereEditor = bdfServer.getBasedefichesEditor().getSphereEditor(sphere.getSubsetKey());
        try {
            Redacteur redacteur = sphereEditor.createRedacteur(newIdsphere);
            setDone("sphere_done_newidsphere", Idsphere.toString(newIdsphere));
            putResultObject(SPHERE_OBJ, redacteur.getSphere());
            putResultObject(REDACTEUR_OBJ, redacteur);
        } catch (ExistingIdException iee) {
            throw new ShouldNotOccurException("test before ExistingIdsphereException");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        sphere = BdfCommand2Utils.getMandatorySphere(basedefiches, requestMap);
        String param_new = getMandatoryParameter(NEWIDSPHERE_PARAMNAME);
        try {
            newIdsphere = Idsphere.parse(param_new);
        } catch (java.text.ParseException pe) {
            setError("sphere_err_idspheremalforme", param_new);
            return false;
        }
        if (sphere.getRedacteurByIdsphere(newIdsphere) != null) {
            setError("sphere_err_idsphereexistant", param_new);
            return false;
        }
        return true;
    }

}
