/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.ParameterBdfCommand2Exception;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.Locale;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.request.RequestMap;

/**
 *
 * @author Vincent Calame
 */
public class RedacteurLangCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurLang";
    public final static String COMMANDKEY = "USR-02";
    public final static String WORKINGLANG_PARAMNAME = "workinglang";
    public final static String FORMATLOCALE_PARAMNAME = "formatlang";
    private LocalisationInfo locInfo;
    private BdfUser redacBdfUser;

    public RedacteurLangCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
            return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    public void doCommand() throws BdfCommand2Exception {
        redacBdfUser.setLocalisationInfo(locInfo);
        BdfUserUtils.saveBdfUser(redacBdfUser);
        setDone("user_done_lang", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        Redacteur redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        if (!BdfUserUtils.isAdmin(bdfUser)) {
            if (!bdfUser.getRedacteurKey().equals(redacteur.getRedacteurKey())) {
                return false;
            }
        }
        if (bdfUser.getRedacteurKey().equals(redacteur.getRedacteurKey())) {
            redacBdfUser = bdfUser;
        } else {
            redacBdfUser = new BdfUser(bdfServer, redacteur.getRedacteurKey());
        }
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        String workingLangParam = getMandatoryParameter(WORKINGLANG_PARAMNAME);
        int workingLang = 0;
        try {
            workingLang = LangInteger.parse(workingLangParam);
        } catch (LangIntegerException lie) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.NOTLANG, WORKINGLANG_PARAMNAME, workingLangParam);
        }
        if (!langConfig.containsWorkingLangInteger(workingLang)) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.UNKNOWNVALUE, WORKINGLANG_PARAMNAME, workingLangParam);
        }
        String formatLang = getMandatoryParameter(FORMATLOCALE_PARAMNAME);
        Locale formatLocale = null;
        try {
            int formatInteger = LangInteger.parse(formatLang);
            formatLocale = LangInteger.getLocale(formatInteger);
        } catch (LangIntegerException lie) {
            throw new ParameterBdfCommand2Exception(ParameterBdfCommand2Exception.NOTLANG, FORMATLOCALE_PARAMNAME, formatLang);
        }
        locInfo = new LocalisationInfo(workingLang, formatLocale);
        return true;
    }
}
