/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationInfo;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurTemplatesCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "RedacteurTemplates";
    public final static String COMMANDKEY = "USR-04";
    private TransformationManager transformationManager;
    private BdfUser redacBdfUser;

    public RedacteurTemplatesCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    public void doCommand() throws BdfCommand2Exception {
        transformationManager = bdfServer.getTransformationManager();
        saveKeys();
        BdfUserUtils.saveBdfUser(redacBdfUser);
        setDone("user_done_templates", null);
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        Redacteur redacteur = BdfCommand2Utils.getMandatoryRedacteur(basedefiches, requestMap);
        if (!BdfUserUtils.isAdmin(bdfUser)) {
            if (!bdfUser.getRedacteurKey().equals(redacteur.getRedacteurKey())) {
                return false;
            }
        }
        if (bdfUser.getRedacteurKey().equals(redacteur.getRedacteurKey())) {
            redacBdfUser = bdfUser;
        } else {
            redacBdfUser = new BdfUser(bdfServer, redacteur.getRedacteurKey());
        }
        return true;
    }

    private void saveKeys() {
        saveTransformationKey(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
        saveTransformationKey(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY);
        saveTransformationKey(TransformationKey.MOTCLE_TRANSFORMATIONKEY);
        saveTransformationKey(TransformationKey.INVERSETHESAURUS_TRANSFORMATIONKEY);
        Basedefiches basedefiches = bdfServer.getBasedefiches();
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            TransformationKey transformationKey = new TransformationKey(corpus.getSubsetKey());
            saveTransformationKey(transformationKey);
        }
    }

    private void saveTransformationKey(TransformationKey transformationKey) {
        saveXSLTemplateKey(transformationKey);
        TransformationInfo transformationInfo = transformationManager.getTransformationInfo(transformationKey);
        int count = transformationInfo.getExtensionListCount();
        for (int i = 0; i < count; i++) {
            String extension = transformationInfo.getExtension(i);
            saveStreamTransformerKey(transformationKey, extension);
        }
    }

    private void saveXSLTemplateKey(TransformationKey transformationKey) {
        String value = requestMap.getParameter(transformationKey.toString());
        if (value != null) {
            if (value.equals("#null#")) {
                redacBdfUser.setXSLTemplateName(transformationKey, null);
            } else {
                redacBdfUser.setXSLTemplateName(transformationKey, value);
            }
        }
    }

    private void saveStreamTransformerKey(TransformationKey transformationKey, String extension) {
        String value = requestMap.getParameter(StreamTransformerKey.toStringKey(transformationKey, extension));
        if (value != null) {
            try {
                StreamTransformerKey streamTransformerKey = StreamTransformerKey.parse(value);
                redacBdfUser.putStreamTransformerKey(streamTransformerKey);
            } catch (java.text.ParseException pe) {
            }
        }
    }

}
