/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands2.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.AbstractBdfCommand2;
import fr.exemole.bdfserver.commands2.BdfCommand2Exception;
import net.mapeadores.basedefiches.ExistingSubsetException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.SphereEditor;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;

/**
 *
 * @author Vincent Calame
 */
public class SphereCreationCommand extends AbstractBdfCommand2 {

    public final static String COMMANDNAME = "SphereCreation";
    public final static String COMMANDKEY = "SPH-01";
    public final static String NEWSPHERE_PARAMNAME = "newsphere";
    private SubsetKey sphereKey;

    public SphereCreationCommand(BdfContext bdfContext, RequestMap requestMap) {
        super(bdfContext, requestMap, COMMANDNAME);
    }

    public short getBdfUserNeed() {
        return ADMIN_ONLY_BDFUSER;
    }

    public boolean needSynchronisation() {
        return true;
    }

    protected void doCommand() throws BdfCommand2Exception {
        try {
            SphereEditor sphereEditor = bdfServer.getBasedefichesEditor().createSphere(sphereKey);
            setDone("sphere_done_spherecreation", sphereKey.toString());
            putResultObject(SPHERE_OBJ, sphereEditor.getSphere());
        } catch (ExistingSubsetException ese) {
            throw new ShouldNotOccurException("test before ExistingSubsetException");
        }
    }

    protected boolean checkParameters() throws BdfCommand2Exception {
        String nameParam = getMandatoryParameter(NEWSPHERE_PARAMNAME);
         nameParam = nameParam.trim();
        if (nameParam.length() == 0) {
            setError("sphere_err_missingspherename", null);
            return false;
        }
        try {
            sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, nameParam);
        } catch (java.text.ParseException pe) {
            setError("sphere_err_wrongspherename", nameParam);
            return false;
        }
        if (basedefiches.getSubset(sphereKey) != null) {
            setError("sphere_err_existingspherename", nameParam);
            return false;
        }
        return true;
    }
}
