/*  BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers;

import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.roles.Role;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class CommandBoxUtils {

    private CommandBoxUtils() {
    }

    public static void insertRole(CommandBox commandBox, Role role) {
        commandBox.putHiddenAttribute("role", role.getName());
    }

    public static void insertCorpus(CommandBox commandBox, Corpus corpus) {
        putSubset(commandBox, corpus);
    }

    public static void insertSphere(CommandBox commandBox, Sphere sphere) {
        putSubset(commandBox, sphere);
    }

    public static void insertRedacteur(CommandBox commandBox, Redacteur redacteur) {
        insertRedacteur(commandBox, redacteur.getRedacteurKey());
    }

    public static void insertRedacteur(CommandBox commandBox, RedacteurKey redacteurKey) {
        putSubset(commandBox, redacteurKey.getSphereKey());
        commandBox.putHiddenAttribute(BdfHtmlUtils.getDefaultIdsubsetParamName(SubsetKey.CATEGORY_SPHERE), redacteurKey.getIdsphereString());
    }

    public static void insertAlbum(CommandBox commandBox, Album album) {
        putSubset(commandBox, album);
    }

    public static void insertAlbumDim(CommandBox commandBox, AlbumDim albumDim) {
        insertAlbum(commandBox, albumDim.getAlbumMetadata().getAlbum());
        commandBox.putHiddenAttribute("albumdim", albumDim.getName());
    }

    public static void insertIllustration(CommandBox commandBox, Illustration illustration) {
        insertAlbum(commandBox, illustration.getAlbum());
        commandBox.putHiddenAttribute(BdfHtmlUtils.getDefaultIdsubsetParamName(SubsetKey.CATEGORY_ALBUM), String.valueOf(illustration.getIdsubset()));
    }

    private static void putSubset(CommandBox commandBox, Subset subset) {
        putSubset(commandBox, subset.getSubsetKey());
    }

    private static void putSubset(CommandBox commandBox, SubsetKey subsetKey) {
        commandBox.putHiddenAttribute(SubsetKey.categoryToString(subsetKey.getCategory()), subsetKey.getSubsetName());
    }

}
