/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.affichage;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.sphere.SphereCreationCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.htmlproducers.sphere.SphereHtmlConstants;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class RecensementsHtmlProducer extends BdfHtmlProducer implements AffichageHtmlConstants {

    private Basedefiches basedefiches;

    public RecensementsHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        basedefiches = bdfServer.getBasedefiches();
        setCssThemeFile("affichage_recensements.css");
        addJavascriptFiles(GlobalCatalog.BDF_JAVASCRIPT);
    }

    public void printHtml() {
        start();

        printH2("global_list_thesaurus");
        if (isAdmin()) {
            startP(GlobalCatalog.GLB_P_SMALLLINKS);
            // fr.exemole.bdfserver.servlets.thesaurus.DefaultCommand.THESAURUSCREATIONFORM_PAGE_PARAMVALUE
            appendA("Thesaurus?page=" + "thesauruscreation", "thesaurus_title_thesauruscreation", "Saisie");
            endP();
        }
        startUl(getThesaurusAttr());
        printSubsetGroupList(bdfServer.getUIManager().getThesaurusSubsetKeyTree(), new ThesaurusPrinter());
        endUl();

        printH2("global_list_sphere");
        if (isAdmin()) {
            startP(GlobalCatalog.GLB_P_SMALLLINKS);
            BdfHtmlUtils.appendCommandLink(this, "Sphere?page=" + SphereHtmlConstants.SPHERECREATIONFORM_PAGE, SphereCreationCommand.COMMANDKEY, "Saisie");
            BdfHtmlUtils.printTiret(this);
            BdfHtmlUtils.printOds(this, "redacteurs/_all");
            endP();
        }
        startUl(getSphereAttr());
        printSubsetGroupList(bdfServer.getUIManager().getSphereSubsetKeyTree(), new SpherePrinter());
        endUl();
        HTMLAttributes liAttributes = new HTMLAttributes();
        appendH2("global_list_iso");
        startUl();
        liAttributes.setId("global_subset_langue");
        startLi(liAttributes);
        appendA("iso/" + "languages" + ".html", "global_subset_langue");
        endLi();
        liAttributes.setId("global_subset_pays");
        startLi(liAttributes);
        appendA("iso/" + "countries" + ".html", "global_subset_pays");
        endLi();
        endUl();
        end();
    }

    private void printSubsetGroupList(SubsetGroupList subsetGroupList, SubsetPrinter subsetPrinter) {
        int count = subsetGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = subsetGroupList.getSubsetGroup(i);
            boolean none = group.isNoTemplateGroup();
            if (!none) {
                startLi();
                startP("Group");
                escape(group.getLibelleString(workingLangInteger));
                endP();
                startUl(getThesaurusAttr());
            }
            for (SubsetKey subsetKey : group.getSubsetKeyList()) {
                subsetPrinter.printSubset(subsetKey);
            }
            if (!none) {
                endUl();
                endLi();
            }
        }
    }

    private void printH2(String key) {
        HTMLAttributes imgAtt = new HTMLAttributes();
        imgAtt.setSrc("theme/icons/16x16/" + key + ".png");
        imgAtt.setWidth("16");
        imgAtt.setHeight("16");
        imgAtt.setAlt("");
        startH2();
        appendImg(imgAtt);
        space();
        localize(key);
        endH2();
    }


    private abstract class SubsetPrinter {

        abstract void printSubset(SubsetKey subsetKey);

    }


    private class ThesaurusPrinter extends SubsetPrinter {

        HTMLAttributes aAttributes = new HTMLAttributes();

        void printSubset(SubsetKey subsetKey) {
            Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(subsetKey);
            if (thesaurus == null) {
                return;
            }
            aAttributes.setHref("Thesaurus?" + RequestUtils.toParameterString(thesaurus));
            String thesaurusName = thesaurus.getSubsetKey().getSubsetName();
            startLi();
            startA(aAttributes);
            escape(ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
            endA();
            if (isAdmin()) {
                space();
                startCode();
                append("[");
                append(thesaurusName);
                append("]");
                endCode();
            }
            space();
            startSpan(GlobalCatalog.GLB_P_SMALLLINKS);
            BdfHtmlUtils.printOds(RecensementsHtmlProducer.this, "motscles/" + thesaurusName + "-table");
            appendBr();
            BdfHtmlUtils.printDeployTitle(RecensementsHtmlProducer.this, thesaurusName, "global_link_statistiques");
            endSpan();
            BdfHtmlUtils.printThesaurusLinksDeployBlock(RecensementsHtmlProducer.this, thesaurusName, "motscles/");
            endLi();
        }

    }


    private class SpherePrinter extends SubsetPrinter {

        HTMLAttributes aAttributes = new HTMLAttributes();

        void printSubset(SubsetKey subsetKey) {
            Sphere sphere = (Sphere) basedefiches.getSubset(subsetKey);
            if (sphere == null) {
                return;
            }
            aAttributes.setHref("Sphere?" + RequestUtils.toParameterString(sphere) + "&page=" + SphereHtmlConstants.SPHERE_PAGE);
            String sphereName = sphere.getSubsetKey().getSubsetName();
            startLi();
            startA(aAttributes);
            escape(SphereUtils.getSphereIntitule(sphere, workingLangInteger));
            endA();
            space();
            startCode();
            append("[");
            append(sphereName);
            append("]");
            endCode();
            endLi();
        }

    }

    private static HTMLAttributes getThesaurusAttr() {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setClasses("Thesaurus");
        return htmlAttributes;
    }

    private static HTMLAttributes getSphereAttr() {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setClasses("Sphere");
        return htmlAttributes;
    }

}
