/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.album;

import fr.exemole.bdfserver.commands2.album.AlbumCreationCommand;
import fr.exemole.bdfserver.commands2.album.AlbumDimModifCommand;
import fr.exemole.bdfserver.commands2.album.AlbumDimCreationCommand;
import fr.exemole.bdfserver.commands2.album.DimUsageModifCommand;
import fr.exemole.bdfserver.commands2.album.IllustrationFileUploadCommand;
import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.util.awt.ResizeInfo;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.UtilHtmlPrinter;


/**
 *
 * @author Vincent Calame
 */
public class AlbumCommandBoxUtils implements AlbumConstants {

    public static void printAlbumCreationBox(UtilHtmlPrinter printer, CommandBox commandBox) {
        commandBox = commandBox.derive(AlbumCreationCommand.COMMANDNAME, AlbumCreationCommand.COMMANDKEY);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "album_lib_albumcreation");
        printer.appendMediumInput(AlbumCreationCommand.NEWALBUM_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("album_submit_albumcreation");
    }

    public static void printAlbumDimModifBox(UtilHtmlPrinter printer, CommandBox commandBox, AlbumDim albumDim) {
        commandBox = commandBox.derive(AlbumDimModifCommand.COMMANDNAME, AlbumDimModifCommand.COMMANDKEY);
        CommandBoxUtils.insertAlbumDim(commandBox, albumDim);
        printer.startCommandBox(commandBox);
        ResizeInfo resizeInfo = albumDim.getResizeInfo();
        short type = albumDim.getDimType();
        if (AlbumUtils.needWidth(type)) {
            printer.startP();
            HtmlPrinterUtils.printLibColon(printer, "album_lib_width");
            printer.appendSmallInput(AlbumDimModifCommand.ALBUMDIM_WIDTH_PARAMNAME, String.valueOf(resizeInfo.getWidth()));
            printer.endP();
        }
        if (AlbumUtils.needHeight(type)) {
            printer.startP();
            HtmlPrinterUtils.printLibColon(printer, "album_lib_height");
            printer.appendSmallInput(AlbumDimModifCommand.ALBUMDIM_HEIGHT_PARAMNAME, String.valueOf(resizeInfo.getHeight()));
            printer.endP();
        }
        printer.endCommandBox("album_submit_albumdimmodif");
    }

    public static void printDimUsageModifBox(UtilHtmlPrinter printer, CommandBox commandBox, Album album) {
        commandBox = commandBox.derive(DimUsageModifCommand.COMMANDNAME, DimUsageModifCommand.COMMANDKEY);
        String vignetteName = "";
        AlbumDim vignetteDim = album.getAlbumMetadata().getAlbumDimForUsage(VIGNETTE_USAGE);
        if (vignetteDim != null) {
            vignetteName = vignetteDim.getName();
        }
        String detailName = "";
        AlbumDim detailDim = album.getAlbumMetadata().getAlbumDimForUsage(DETAIL_USAGE);
        if (detailDim != null) {
            detailName = detailDim.getName();
        }
        CommandBoxUtils.insertAlbum(commandBox, album);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "album_lib_dimusage_vignette");
        printer.appendMediumInput(DimUsageModifCommand.VIGNETTE_PARAMNAME, vignetteName);
        printer.endP();
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "album_lib_dimusage_detail");
        printer.appendMediumInput(DimUsageModifCommand.DETAIL_PARAMNAME, detailName);
        printer.endP();
        printer.endCommandBox("album_submit_dimusagemodif");
    }

    public static void printAlbumDimRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, AlbumDim albumDim) {
        //printer.startCommandBox(AlbumDimRemoveCommand.COMMANDNAME,AlbumDimRemoveCommand.COMMANDKEY);
        /*printer.startP();
        printer.printLibColon("sphere_lib_albumcreation");
        printer.appendMediumInput(AlbumCreationCommand.NEWALBUM_PARAMNAME, "");
        printer.endP();*/
        //printer.endCommandBox("album_submit_albumcreation");
    }

    public static void printAlbumDimCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, Album album) {
        commandBox = commandBox.derive(AlbumDimCreationCommand.COMMANDNAME, AlbumDimCreationCommand.COMMANDKEY);
        CommandBoxUtils.insertAlbum(commandBox, album);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "sphere_lib_newalbumdim");
        printer.appendMediumInput(AlbumDimCreationCommand.NEWALBUMDIM_PARAMNAME, "");
        printer.endP();
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "sphere_lib_albumdimtype");
        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setName(AlbumDimCreationCommand.ALBUMDIMTYPE_PARAMNAME);
        printer.startSelect(selectAttributes);
        printOption(printer, FIXEDWIDTH_DIMTYPE);
        printOption(printer, FIXEDHEIGHT_DIMTYPE);
        printOption(printer, MAXWIDTH_DIMTYPE);
        printOption(printer, MAXHEIGHT_DIMTYPE);
        printOption(printer, MAXDIM_DIMTYPE);
        printer.endSelect();
        printer.endP();
        printer.endCommandBox("album_submit_albumdimcreation");
    }

    private static void printOption(UtilHtmlPrinter printer, short albumDimType) {
        String value = AlbumUtils.dimTypeToString(albumDimType);
        printer.appendOption(value, false, "album_albumdimtype_" + value);
    }

    public static void printIllustrationFileUploadBox(UtilHtmlPrinter printer, CommandBox commandBox, Album album, Illustration illustration) {
        commandBox = commandBox.derive(IllustrationFileUploadCommand.COMMANDNAME, IllustrationFileUploadCommand.COMMANDKEY);
        commandBox.setMultipart(true);
        if (illustration != null) {
            CommandBoxUtils.insertIllustration(commandBox, illustration);
        } else {
            CommandBoxUtils.insertAlbum(commandBox, album);
        }
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "album_lib_illustrationfile");
        printer.endP();
        printer.startP();
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setType("file");
        htmlAttributes.setName(IllustrationFileUploadCommand.FILE_PARAMNAME);
        htmlAttributes.setSize("40");
        printer.appendInput(htmlAttributes);
        printer.endP();
        printer.endCommandBox("album_submit_illustrationfileupload");
    }

}
