/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.album.AlbumCommandUtils;
import fr.exemole.bdfserver.commands2.album.ToolCropCommand;
import fr.exemole.bdfserver.commands2.album.ToolResizeCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.catalogs.IllustrationCatalog;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.servlets.CommandRequestMap;
import net.mapeadores.util.servlets.ResponseDataType;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationChangeHtmlProducer extends BdfHtmlProducer implements AlbumHtmlConstants, IllustrationCatalog {

    private Album album;
    private Illustration illustration;
    private String appelant;
    private String tmpFileName;
    private String changeType;

    private IllustrationChangeHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant) {
        init(bdfContext, bdfUser);
        setCssThemeFile("illustration.css");
        this.album = album;
        this.appelant = appelant;
        addJqueryExtensions("form");
        addJqueryExtensions("imgareaselect");
        addJavascriptFiles(ILLUSTRATION_JAVASCRIPT_CORE);
        addJavascriptFiles(ILLUSTRATION_JAVASCRIPT_CHANGE);
        localizeTitle("album_title_illustration", null);
    }

    public static IllustrationChangeHtmlProducer newTmpFileCreate(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant, String tmpFileName) {
        IllustrationChangeHtmlProducer htmlProducer = new IllustrationChangeHtmlProducer(bdfContext, bdfUser, album, appelant);
        htmlProducer.tmpFileName = tmpFileName;
        htmlProducer.changeType = CREATE_CHANGETYPE;
        return htmlProducer;
    }

    public static IllustrationChangeHtmlProducer newTmpFileChange(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant, String tmpFileName) {
        IllustrationChangeHtmlProducer htmlProducer = new IllustrationChangeHtmlProducer(bdfContext, bdfUser, album, appelant);
        htmlProducer.tmpFileName = tmpFileName;
        htmlProducer.changeType = TMPFILECHANGE_CHANGETYPE;
        return htmlProducer;
    }

    public static IllustrationChangeHtmlProducer newIllustrationChange(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant, Illustration illustration, String tmpFileName) {
        IllustrationChangeHtmlProducer htmlProducer = new IllustrationChangeHtmlProducer(bdfContext, bdfUser, album, appelant);
        htmlProducer.illustration = illustration;
        htmlProducer.tmpFileName = tmpFileName;
        htmlProducer.changeType = ILLUSTRATIONCHANGE_CHANGETYPE;
        return htmlProducer;
    }

    public void printHtml() {
        start();
        startScript(null);
        append("\n ILL_Change.changeType=\"");
        escapeScriptString(changeType);
        append("\";");
        append("\n ILL_Change.appelant=\"");
        escapeScriptString(appelant);
        append("\";");
        append("\n ILL_Change.tmpFileName=\"");
        escapeScriptString(checkTmpFileName());
        append("\";");
        append("\n ILL_Change.album=\"");
        append(album.getSubsetKey().getSubsetName());
        append("\";");
        if (illustration != null) {
            append("\n ILL_Change.idalbum=");
            append(String.valueOf(illustration.getIdalbum()));
            append(";");
            append("\n ILL_Change.originalFileName=\"");
            append(String.valueOf(AlbumUtils.toFileName(illustration)));
            append("\";");
        }
        endScript();
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId(ID_ILL_BLOC_GLOBAL);
        startDiv(divAttributes);
        printBlocToolsIcons();
        printBlocToolsInfos();
        printBlocImage();
        printBlocValidation();
        endDiv();
        end();
    }

    private String checkTmpFileName() {
        if (tmpFileName != null) {
            return tmpFileName;
        }
        return AlbumCommandUtils.ORIGINAL_TMPFILENAME;
    }

    private void printBlocToolsIcons() {
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId(ID_ILL_BLOC_TOOLS_ICONS);
        startDiv(divAttributes);
        addIcon(ID_ILL_CROP_ICON, "illustration_tool_crop.png", "album_lib_tool_crop");
        addIcon(ID_ILL_RESIZE_ICON, "illustration_tool_resize.png", "album_lib_tool_resize");
        addIcon(ID_ILL_REPLACE_ICON, "illustration_tool_replace.png", "album_lib_tool_replace");
        addIcon(ID_ILL_PREVIOUS_ICON, "illustration_tool_previous.png", "album_lib_tool_previous");
        addIcon(ID_ILL_NEXT_ICON, "illustration_tool_next.png", "album_lib_tool_next");
        if ((illustration != null) && (tmpFileName != null)) {
            addIcon(ID_ILL_RESET_ICON, "illustration_tool_reset.png", "album_lib_tool_reset");
        }
        endDiv();
    }

    private void addIcon(String id, String img, String locKey) {
        HTMLAttributes iconAttributes = new HTMLAttributes();
        iconAttributes.setWidth("16");
        iconAttributes.setHeight("16");
        iconAttributes.setId(id);
        iconAttributes.setSrc("theme/icons/16x16/" + img);
        iconAttributes.setTitle(getLocalization(locKey));
        appendImg(iconAttributes);
    }

    private void printBlocToolsInfos() {
        HTMLAttributes submitAttributes = new HTMLAttributes();
        submitAttributes.setType(HTMLAttributes.SUBMIT_TYPE);
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId(ID_ILL_BLOC_TOOLS_INFOS);
        startDiv(divAttributes);
        divAttributes.setId(ID_ILL_CROP_INFO);
        startDiv(divAttributes);
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setMethod("GET");
        formAttributes.setAction("Album");
        startForm(formAttributes);
        appendHiddenInput("cmd", ToolCropCommand.COMMANDNAME);
        appendHiddenInput("album", album.getSubsetKey().getSubsetName());
        if (illustration != null) {
            appendHiddenInput("idalbum", String.valueOf(illustration.getIdalbum()));
        }
        appendHiddenInput(ToolCropCommand.X_PARAMNAME, "0");
        appendHiddenInput(ToolCropCommand.Y_PARAMNAME, "0");
        appendHiddenInput(ToolCropCommand.WIDTH_PARAMNAME, "8");
        appendHiddenInput(ToolCropCommand.HEIGHT_PARAMNAME, "8");
        appendHiddenInput(ToolCropCommand.TMPFILE_PARAMNAME, checkTmpFileName());
        appendHiddenInput(CommandRequestMap.DATATYPE_PARAMETER, ResponseDataType.JSON_DATATYPE_STRING);
        HtmlPrinterUtils.printLibColon(this, "album_lib_croparea");
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.setId(ID_ILL_CROPAREA);
        startSpan(spanAttributes);
        endSpan();
        submitAttributes.setValue(getLocalization("album_submit_tool_crop"));
        submitAttributes.setId(ID_ILL_CROP_SUBMIT);
        appendInput(submitAttributes);
        endForm();
        endDiv();
        divAttributes.setId(ID_ILL_RESIZE_INFO);
        startDiv(divAttributes);
        startForm(formAttributes);
        appendHiddenInput("cmd", ToolResizeCommand.COMMANDNAME);
        appendHiddenInput("album", album.getSubsetKey().getSubsetName());
        if (illustration != null) {
            appendHiddenInput("idalbum", String.valueOf(illustration.getIdalbum()));
        }
        appendHiddenInput(ToolResizeCommand.TMPFILE_PARAMNAME, checkTmpFileName());
        appendHiddenInput(CommandRequestMap.DATATYPE_PARAMETER, ResponseDataType.JSON_DATATYPE_STRING);
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setType(HTMLAttributes.TEXT_TYPE);
        inputAttributes.setSize("4");
        HtmlPrinterUtils.printLibColon(this, "album_lib_width");
        inputAttributes.setId(ID_ILL_WIDTHINPUT);
        inputAttributes.setName(ToolResizeCommand.WIDTH_PARAMNAME);
        appendInput(inputAttributes);
        space();
        HtmlPrinterUtils.printLibColon(this, "album_lib_height");
        inputAttributes.setId(ID_ILL_HEIGHTINPUT);
        inputAttributes.setName(ToolResizeCommand.HEIGHT_PARAMNAME);
        appendInput(inputAttributes);
        submitAttributes.setValue(getLocalization("album_submit_tool_resize"));
        submitAttributes.setId(ID_ILL_RESIZE_SUBMIT);
        appendInput(submitAttributes);
        endForm();
        endDiv();
        endDiv();
    }

    private void printBlocImage() {
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId(ID_ILL_BLOC_IMAGE);
        startDiv(divAttributes);
        HTMLAttributes imgAttributes = new HTMLAttributes();
        imgAttributes.setId(ID_ILL_IMAGE);
        if (tmpFileName != null) {
            imgAttributes.setSrc("prv/tmp/" + tmpFileName);
        } else {
            imgAttributes.setSrc("illustrations/" + AlbumUtils.toFileName(illustration));
        }
        appendImg(imgAttributes);
    }

    private void printBlocValidation() {
        HTMLAttributes buttonsAttributes = new HTMLAttributes();
        buttonsAttributes.setId(ID_ILL_BLOC_VALIDATION);
        startDiv(buttonsAttributes);
        HTMLAttributes buttonAttributes = new HTMLAttributes();
        buttonAttributes.setType("button");
        buttonAttributes.setId(ID_ILL_BUTTON_OK);
        buttonAttributes.setValue(getLocalization("global_submit_ok"));
        appendInput(buttonAttributes);
        space();
        buttonAttributes.setId(ID_ILL_BUTTON_CANCEL);
        buttonAttributes.setValue(getLocalization("global_submit_cancel"));
        appendInput(buttonAttributes);
        endDiv();
    }

}
