/* BdfServer_Command - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.album;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.album.IllustrationFileUploadCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.catalogs.IllustrationCatalog;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationUploadHtmlProducer extends BdfHtmlProducer implements AlbumHtmlConstants, IllustrationCatalog {

    private Album album;
    private String appelant;
    private boolean newUpload = false;

    private IllustrationUploadHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant) {
        init(bdfContext, bdfUser);
        setCssThemeFile("illustration.css");
        this.album = album;
        this.appelant = appelant;
        addJavascriptFiles(ILLUSTRATION_JAVASCRIPT_CORE);
        addJavascriptFiles(ILLUSTRATION_JAVASCRIPT_UPLOAD);
    }

    public static IllustrationUploadHtmlProducer newNewUpload(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant) {
        IllustrationUploadHtmlProducer producer = new IllustrationUploadHtmlProducer(bdfContext, bdfUser, album, appelant);
        producer.localizeTitle("album_title_illustration", null);
        producer.newUpload = true;
        return producer;
    }

    public static IllustrationUploadHtmlProducer newReplaceUpload(BdfContext bdfContext, BdfUser bdfUser, Album album, String appelant) {
        IllustrationUploadHtmlProducer producer = new IllustrationUploadHtmlProducer(bdfContext, bdfUser, album, appelant);
        producer.localizeTitle("album_title_replaceupload", null);
        producer.newUpload = false;
        return producer;
    }

    public void printHtml() {
        start();
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.addClass(ILL_GLOBAL);
        startDiv(divAttributes);
        appendH1((newUpload) ? "album_title_newupload" : "album_title_replaceupload");
        printCommandMessage();
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setMethod("POST");
        formAttributes.setAction("Album");
        formAttributes.setEnctype(HTMLAttributes.MULTIPART_ENCTYPE);
        startForm(formAttributes);
        appendHiddenInput("cmd", IllustrationFileUploadCommand.COMMANDNAME);
        appendHiddenInput("album", album.getSubsetKey().getSubsetName());
        if (newUpload) {
            appendHiddenInput("page-err", AlbumHtmlConstants.ILLUSTRATIONUPLOAD_NEW_PAGE);
            appendHiddenInput("page", AlbumHtmlConstants.ILLUSTRATIONCHANGE_PAGE);
        } else {
            appendHiddenInput("page-err", AlbumHtmlConstants.ILLUSTRATIONUPLOAD_REPLACE_PAGE);
            appendHiddenInput("page", AlbumHtmlConstants.ILLUSTRATIONUPLOAD_CONFIRM_PAGE);
        }
        appendHiddenInput("appelant", appelant);
        startP();
        HtmlPrinterUtils.printLibColon(this, "album_lib_illustrationfile");
        endP();
        startP();
        HTMLAttributes fileAttributes = new HTMLAttributes();
        fileAttributes.setType(HTMLAttributes.FILE_TYPE);
        fileAttributes.setName(IllustrationFileUploadCommand.FILE_PARAMNAME);
        fileAttributes.setSize("40");
        appendInput(fileAttributes);
        endP();
        printButtonsDiv();
        endForm();
        endDiv();
        end();
    }

    private void printButtonsDiv() {
        HTMLAttributes buttonsAttributes = new HTMLAttributes();
        buttonsAttributes.addClass(IllustrationCatalog.ILL_BUTTONS);
        startDiv(buttonsAttributes);
        HTMLAttributes submitAttributes = new HTMLAttributes();
        submitAttributes.setType(HTMLAttributes.SUBMIT_TYPE);
        submitAttributes.setValue(getLocalization("album_submit_fileupload"));
        appendInput(submitAttributes);
        space();
        HTMLAttributes cancelAttributes = new HTMLAttributes();
        cancelAttributes.setType(HTMLAttributes.BUTTON_TYPE);
        cancelAttributes.setId(ID_ILL_BUTTON_CANCEL);
        cancelAttributes.setValue(getLocalization("global_submit_cancel"));
        appendInput(cancelAttributes);
        endDiv();
    }

}
