/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.configuration.IdentificationCommand;
import fr.exemole.bdfserver.commands2.configuration.LangConfigCommand;
import fr.exemole.bdfserver.commands2.configuration.UrlCommand;
import fr.exemole.bdfserver.commands2.configuration.UserAllowChangeCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.BasedefichesAttributes;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class CoreFormHtmlProducer extends BdfHtmlProducer implements ConfigurationHtmlConstants {

    public CoreFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        BdfHtmlUtils.addTestJavascript(this);
    }

    public void printHtml() {
        start();
        ConfigurationHtmlUtils.printTitre(this, "configuration_title_core", COREFORM_PAGE);
        printCommandMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Configuration");
        commandBox.setCommandFamily("CNF");
        commandBox.setPage(COREFORM_PAGE);

        printIdentification(commandBox);
        printUrl(commandBox);
        printLangConfig(commandBox);
        printUserAllow(commandBox);
        end();
    }

    private void printIdentification(CommandBox commandBox) {
        BasedefichesMetadata basedefichesMetadata = bdfServer.getBasedefiches().getBasedefichesMetadata();
        commandBox = commandBox.derive(IdentificationCommand.COMMANDNAME, IdentificationCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startP();
        BdfHtmlUtils.printLibColon(this, "configuration_lib_authorityuuid");
        appendLargeInput(IdentificationCommand.AUTHORITYUUID_PARAMNAME, basedefichesMetadata.getAuthorityUUID());
        endP();
        startP();
        BdfHtmlUtils.printLibColon(this, "configuration_lib_basename");
        appendMediumInput(IdentificationCommand.BASENAME_PARAMNAME, basedefichesMetadata.getBasedefichesName());
        endP();
        endCommandBox("configuration_submit_core");
    }

    private void printUrl(CommandBox commandBox) {
        commandBox = commandBox.derive(UrlCommand.COMMANDNAME, UrlCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startP();
        BdfHtmlUtils.printLibColon(this, "configuration_lib_inserturl");
        String value = "";
        Attribute attribute = bdfServer.getBasedefiches().getBasedefichesMetadata().getAttributeByKey(BasedefichesAttributes.IMAGEBASEURL_KEY);
        if (attribute != null) {
            value = attribute.getValue(0);
        }
        appendLargeInput(UrlCommand.INSERTURL_PARAMNAME, value);
        endP();
        endCommandBox("configuration_submit_core");
    }

    private void printLangConfig(CommandBox commandBox) {
        commandBox = commandBox.derive(LangConfigCommand.COMMANDNAME, LangConfigCommand.COMMANDKEY);
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingArray = langConfig.getWorkingLangIntegerArray();
        HTMLAttributes radioAttributes = new HTMLAttributes();
        radioAttributes.setName(LangConfigCommand.ALLLANG_PARAMNAME);
        boolean allLanguages = langConfig.isAllLanguages();
        startCommandBox(commandBox);
        startP();
        BdfHtmlUtils.printLibColon(this, "configuration_lib_workinglang");
        appendSmallInput(LangConfigCommand.WORKINGLANG_PARAMNAME, getArray(workingArray));
        endP();
        startP();
        radioAttributes.setValue("0");
        radioAttributes.setChecked(!allLanguages);
        addRadio(radioAttributes, "configuration_lib_alllang_no", null);
        appendBr();
        startSmall();
        doublespace();
        BdfHtmlUtils.printLibColon(this, "configuration_lib_supplementarylang");
        appendSmallInput(LangConfigCommand.SUPPLEMENTARYLANG_PARAMNAME, getArray(langConfig.getSupplementaryLangIntegerArray()));
        endSmall();
        endP();
        startP();
        radioAttributes.setValue("1");
        radioAttributes.setChecked(allLanguages);
        addRadio(radioAttributes, "configuration_lib_alllang_yes", null);
        endP();
        HTMLAttributes checkAttributes = new HTMLAttributes();
        checkAttributes.setValue("1");
        startP();
        checkAttributes.setName(LangConfigCommand.ORIGINALALLOWED_PARAMNAME);
        checkAttributes.setChecked(langConfig.isWithOriginalName());
        addCheck(checkAttributes, "configuration_lib_originalallowed", null);
        endP();
        startP();
        checkAttributes.setName(LangConfigCommand.WITHOUTNOMAVANT_PARAMNAME);
        checkAttributes.setChecked(langConfig.isWithoutNomAvant());
        addCheck(checkAttributes, "configuration_lib_withoutnomavant", null);
        endP();
        endCommandBox("configuration_submit_core");
    }

    private void printUserAllow(CommandBox commandBox) {
        commandBox = commandBox.derive(UserAllowChangeCommand.COMMANDNAME, UserAllowChangeCommand.COMMANDKEY);
        PolicyManager policyManager = bdfServer.getPolicyManager();
        startCommandBox(commandBox);
        HTMLAttributes checkAttributes = new HTMLAttributes();
        checkAttributes.setValue("1");
        startP();
        checkAttributes.setName(UserAllowChangeCommand.DATA_PARAMNAME);
        checkAttributes.setChecked(policyManager.isDataModifAllowed());
        addCheck(checkAttributes, "configuration_lib_userallow_data", null);
        endP();
        startP();
        checkAttributes.setName(UserAllowChangeCommand.PASSWORD_PARAMNAME);
        checkAttributes.setChecked(policyManager.isPasswordModifAllowed());
        addCheck(checkAttributes, "configuration_lib_userallow_password", null);
        endP();
        startP();
        endP();
        endCommandBox("configuration_submit_userallowchange");
    }

    private String getArray(int[] array) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < array.length; i++) {
            if (i > 0) {
                buf.append(';');
            }
            LangInteger.appendString(buf, array[i]);
        }
        return buf.toString();
    }

}
