/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.configuration.BdfResourceUpdateCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.storage.BdfResourceConstants;
import fr.exemole.bdfserver.storage.BdfResourceStorage;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;


/**
 *
 * @author Vincent Calame
 */
public class LogosFormHtmlProducer extends BdfHtmlProducer implements ConfigurationHtmlConstants {

    public LogosFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
    }

    public void printHtml() {
        HTMLAttributes imageAttributes = new HTMLAttributes();
        BdfResourceStorage bdfResourceStorage = bdfServer.getBdfResourceStorage();
        start();
        ConfigurationHtmlUtils.printTitre(this, "configuration_title_logos", LOGOSFORM_PAGE);
        printCommandMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Configuration");
        commandBox.setCommandFamily("CNF");
        commandBox.setPage(LOGOSFORM_PAGE);
        commandBox.setCommandString(BdfResourceUpdateCommand.COMMANDNAME);
        commandBox.setMultipart(true);
        commandBox.setCommandKey(BdfResourceUpdateCommand.ICON16_COMMANDKEY);
        commandBox.putHiddenAttribute(BdfResourceUpdateCommand.RSCPATH_PARAMNAME, BdfResourceStorage.ICON);
        startCommandBox(commandBox);
        startP();
        boolean contains = bdfResourceStorage.containsBdfResource(BdfResourceConstants.OTHER_RESOURCE, BdfResourceStorage.ICON);
        String key = (contains) ? "configuration_lib_icon_current" : "configuration_lib_icon_default";
        HtmlPrinterUtils.printLibColon(this, key);
        imageAttributes.setSrc("rsc/icon.png");
        imageAttributes.setAlt("icon 16x16");
        imageAttributes.setHeight("16");
        imageAttributes.setWidth("16");
        appendImg(imageAttributes);
        endP();
        printFileInput("configuration_lib_icon_file");
        endCommandBox("configuration_submit_upload");

        commandBox.setCommandKey(BdfResourceUpdateCommand.ICON32_COMMANDKEY);
        commandBox.putHiddenAttribute(BdfResourceUpdateCommand.RSCPATH_PARAMNAME, BdfResourceStorage.ICON32);
        startCommandBox(commandBox);
        startP();
        boolean contains32 = bdfResourceStorage.containsBdfResource(BdfResourceConstants.OTHER_RESOURCE, BdfResourceStorage.ICON32);
        String key32 = (contains32) ? "configuration_lib_icon_current" : "configuration_lib_icon_default";
        HtmlPrinterUtils.printLibColon(this, key32);
        imageAttributes.setSrc("rsc/icon32.png");
        imageAttributes.setAlt("icon 32x32");
        imageAttributes.setHeight("32");
        imageAttributes.setWidth("32");
        appendImg(imageAttributes);
        endP();
        printFileInput("configuration_lib_icon_file");
        endCommandBox("configuration_submit_upload");

        commandBox.setCommandKey(BdfResourceUpdateCommand.LOGO_COMMANDKEY);
        commandBox.putHiddenAttribute(BdfResourceUpdateCommand.RSCPATH_PARAMNAME, BdfResourceStorage.LOGO);
        startCommandBox(commandBox);
        startP();
        boolean containsLogo = bdfResourceStorage.containsBdfResource(BdfResourceConstants.OTHER_RESOURCE, BdfResourceStorage.LOGO);
        if (containsLogo) {
            HtmlPrinterUtils.printLibColon(this, "configuration_lib_logo_current");
            imageAttributes.setSrc("rsc/logo.png");
            imageAttributes.setAlt("logo");
            imageAttributes.setHeight(null);
            imageAttributes.setWidth(null);
            appendImg(imageAttributes);
        } else {
            localize("configuration_info_logo_none");
        }
        endP();
        printFileInput("configuration_lib_logo_file");
        endCommandBox("configuration_submit_upload");

        end();
    }

    private void printFileInput(String libkey) {
        startP();
        HtmlPrinterUtils.printLibColon(this, libkey);
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setType("file");
        htmlAttributes.setName(BdfResourceUpdateCommand.RSCFILE_PARAMNAME);
        htmlAttributes.setSize("75");
        appendInput(htmlAttributes);
        endP();
    }

}
