/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.configuration;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.configuration.GroupChangeCommand;
import fr.exemole.bdfserver.commands2.configuration.GroupCreationCommand;
import fr.exemole.bdfserver.commands2.configuration.GroupRemoveCommand;
import fr.exemole.bdfserver.commands2.configuration.PositionCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetGroupConstants;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetGroupUtils;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class SubsetKeyTreeFormHtmlProducer extends BdfHtmlProducer implements ConfigurationHtmlConstants {

    private int[] workingArray;
    private SubsetKeyGroupTemplates subsetKeyGroupTemplates;

    public SubsetKeyTreeFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        BdfHtmlUtils.addTestJavascript(this);
        workingArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        subsetKeyGroupTemplates = bdfServer.getUIManager().getSubsetKeyGroupTemplates();
    }

    public void printHtml() {
        UIManager uiManager = bdfServer.getUIManager();
        start();
        ConfigurationHtmlUtils.printTitre(this, "configuration_title_listes", SUBSETKEYTREEFORM_PAGE);
        printCommandMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Configuration");
        commandBox.setCommandFamily("CNF");
        commandBox.setPage(SUBSETKEYTREEFORM_PAGE);
        printForm(commandBox, SubsetGroupConstants.CORPUS_GROUP, PositionCommand.CORPUS_COMMANDKEY, uiManager.getCorpusSubsetKeyTree(), basedefiches.getCorpusCount());
        printForm(commandBox, SubsetGroupConstants.THESAURUS_GROUP, PositionCommand.THESAURUS_COMMANDKEY, uiManager.getThesaurusSubsetKeyTree(), basedefiches.getThesaurusCount());
        printForm(commandBox, SubsetGroupConstants.SPHERE_GROUP, PositionCommand.SPHERE_COMMANDKEY, uiManager.getSphereSubsetKeyTree(), basedefiches.getSphereCount());
        printForm(commandBox, SubsetGroupConstants.ALBUM_GROUP, PositionCommand.ALBUM_COMMANDKEY, uiManager.getAlbumSubsetKeyTree(), basedefiches.getAlbumCount());
        printGroupChangeForms(commandBox);
        printGroupCreationForm(commandBox);
        end();
    }

    private void printForm(CommandBox commandBox, short type, String commandKey, SubsetGroupList subsetKeyTree, int subsetCount) {
        commandBox = commandBox.derive(PositionCommand.COMMANDNAME, commandKey);
        String typeString = SubsetGroupUtils.toStringType(type);
        commandBox.putHiddenAttribute(PositionCommand.GROUPTYPE_PARAMNAME, typeString);
        startCommandBox(commandBox);
        startTextArea(PositionCommand.SUBSETPOSITION_PARAMNAME, 40, subsetCount + 3);
        int count = subsetKeyTree.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = subsetKeyTree.getSubsetGroup(i);
            boolean noneGroup = group.isNoTemplateGroup();
            if (!noneGroup) {
                append("\n");
                append("grp_");
                append(group.getTemplateName());
                append("\n");
            }
            List<SubsetKey> subsetKeyList = group.getSubsetKeyList();
            for (SubsetKey subsetKey : subsetKeyList) {
                append(subsetKey.toString());
                append("\n");
            }
            if (!noneGroup) {
                append("---");
                append("\n\n");
            }
        }
        endTextArea();
        endCommandBox("configuration_submit_ordre");
    }

    private void printGroupChangeForms(CommandBox commandBox) {
        CommandBox changeCommandBox = commandBox.derive(GroupChangeCommand.COMMANDNAME, GroupChangeCommand.COMMANDKEY);
        CommandBox removeCommandBox = commandBox.derive(GroupRemoveCommand.COMMANDNAME, GroupRemoveCommand.COMMANDKEY);
        int size = subsetKeyGroupTemplates.size();
        if (size == 0) {
            return;
        }
        appendH2("configuration_title_groupchange");
        for (int j = 0; j < size; j++) {
            String groupName = subsetKeyGroupTemplates.getGroupName(j);
            startH3();
            startCode();
            append("[grp_");
            append(groupName);
            append("]");
            endCode();
            BdfHtmlUtils.printTiret(this);
            escape(subsetKeyGroupTemplates.getLibelleString(groupName, workingLangInteger));
            endH3();
            changeCommandBox.putHiddenAttribute(GroupChangeCommand.GROUPNAME_PARAMNAME, groupName);
            startCommandBox(changeCommandBox);
            startTable();
            for (int i = 0; i < workingArray.length; i++) {
                int langInteger = workingArray[i];
                String lib = null;
                LibelleHolder libelleHolder = subsetKeyGroupTemplates.getLibelleHolder(groupName);
                if (libelleHolder != null) {
                    lib = libelleHolder.getLibelleString(langInteger, false);
                }
                if (lib == null) {
                    lib = "";
                }
                BdfHtmlUtils.printLangFieldRow(this, GroupChangeCommand.GROUP_PARAMPREFIX, langInteger, lib);
            }
            endTable();
            endCommandBox("configuration_submit_groupchange");
            removeCommandBox.putHiddenAttribute(GroupRemoveCommand.GROUPNAME_PARAMNAME, groupName);
            startCommandBox(removeCommandBox);
            BdfHtmlUtils.printConfirmPassword(this);
            endCommandBox("configuration_submit_groupremove");
        }
    }

    private void printGroupCreationForm(CommandBox commandBox) {
        commandBox = commandBox.derive(GroupCreationCommand.COMMANDNAME, GroupCreationCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startTable();
        for (int i = 0; i < workingArray.length; i++) {
            int langInteger = workingArray[i];
            BdfHtmlUtils.printLangFieldRow(this, GroupChangeCommand.GROUP_PARAMPREFIX, langInteger, "");
        }
        endTable();
        endCommandBox("configuration_submit_groupcreation");
    }

}
