/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.commands2.BdfCommand2Utils;
import fr.exemole.bdfserver.commands2.corpus.AssociationActivationCommand;
import fr.exemole.bdfserver.commands2.corpus.CloneCommand;
import fr.exemole.bdfserver.commands2.corpus.CommentChangeCommand;
import fr.exemole.bdfserver.commands2.corpus.CommentCreationCommand;
import fr.exemole.bdfserver.commands2.corpus.CommentRemoveCommand;
import fr.exemole.bdfserver.commands2.corpus.ComponentInputCommand;
import fr.exemole.bdfserver.commands2.corpus.ComponentPositionCommand;
import fr.exemole.bdfserver.commands2.corpus.ConversionCommand;
import fr.exemole.bdfserver.commands2.corpus.CorpusCreationCommand;
import fr.exemole.bdfserver.commands2.corpus.CorpusRemoveCommand;
import fr.exemole.bdfserver.commands2.corpus.DuplicationCommand;
import fr.exemole.bdfserver.commands2.corpus.FieldCreationCommand;
import fr.exemole.bdfserver.commands2.corpus.FieldRemoveCommand;
import fr.exemole.bdfserver.commands2.corpus.IncludeChangeCommand;
import fr.exemole.bdfserver.commands2.corpus.IncludeCreationCommand;
import fr.exemole.bdfserver.commands2.corpus.IncludeRemoveCommand;
import fr.exemole.bdfserver.commands2.corpus.IntitulesCommand;
import fr.exemole.bdfserver.commands2.corpus.ReloadCommand;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.html.utils.SubsetGroupListPrinter;
import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetGroupUtils;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import fr.exemole.bdfserver.ui.options.LimitOption;
import fr.exemole.bdfserver.ui.options.ValueOption;
import java.util.List;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.CorpusConstants;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.UtilHtmlPrinter;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class CorpusCommandBoxUtils {

    public static void printCorpusCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, BdfServer bdfServer, int workingLangInteger) {
        SubsetGroupListPrinter subsetGroupListPrinter = new SubsetGroupListPrinter(bdfServer.getBasedefiches(), printer, workingLangInteger);
        subsetGroupListPrinter.setOnlySubIdValues(true);
        commandBox = commandBox.derive(CorpusCreationCommand.COMMANDNAME, CorpusCreationCommand.COMMANDKEY);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_corpuscreation");
        printer.appendMediumInput(CorpusCreationCommand.NEWCORPUS_PARAMNAME, "");
        printer.endP();
        printCreationCorpusTypeRadio(printer, CorpusCreationCommand.NORMAL_PARAMVALUE, true);
        if (bdfServer.getBasedefiches().getThesaurusCount() > 0) {
            SubsetGroupList thesaurusGroupList = bdfServer.getUIManager().getThesaurusSubsetKeyTree();
            subsetGroupListPrinter.setSubsetGroupList(thesaurusGroupList);
            HTMLAttributes selectAttributes = new HTMLAttributes();
            selectAttributes.setName(CorpusCreationCommand.MASTERTHESAURUS_PARAMNAME);
            printCreationCorpusTypeRadio(printer, CorpusCreationCommand.THESAURUSSLAVE_PARAMVALUE, false);
            printer.startUl();
            printer.startLi();
            printer.startP();
            HtmlPrinterUtils.printLibColon(printer, "global_type_thesaurus");
            subsetGroupListPrinter.printSelect(selectAttributes);
            printer.endP();
            printer.endLi();
            printer.endUl();
        }
        if (bdfServer.getBasedefiches().getCorpusCount() > 0) {
            SubsetGroupList corpusGroupList = SubsetGroupUtils.checkNoSlaveSubset(bdfServer.getBasedefiches(), bdfServer.getUIManager().getCorpusSubsetKeyTree());
            if (corpusGroupList.getSubsetGroupCount() > 0) {
                subsetGroupListPrinter.setSubsetGroupList(corpusGroupList);
                HTMLAttributes selectAttributes = new HTMLAttributes();
                selectAttributes.setName(CorpusCreationCommand.MASTERCORPUS_PARAMNAME);
                printCreationCorpusTypeRadio(printer, CorpusCreationCommand.CORPUSSLAVE_PARAMVALUE, false);
                printer.startUl();
                printer.startLi();
                printer.startP();
                HtmlPrinterUtils.printLibColon(printer, "global_type_corpus");
                subsetGroupListPrinter.printSelect(selectAttributes);
                printer.endP();
                printer.endLi();
                printer.endUl();
            }
        }
        printer.endCommandBox("corpus_submit_corpuscreation");
    }

    private static void printCreationCorpusTypeRadio(HtmlPrinter printer, String value, boolean checked) {
        printer.startP();
        printer.appendRadioInput(CorpusCreationCommand.CORPUSTYPE_PARAMNAME, value, checked, "corpus_lib_corpustype_" + value);
        printer.endP();
    }

    public static void printIntitulesBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, BdfServer bdfServer) {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int[] workingLangArray = langConfig.getWorkingLangIntegerArray();
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        commandBox = commandBox.derive(IntitulesCommand.COMMANDNAME, IntitulesCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printCorpusIntituleTable(printer, corpusMetadata, CorpusConstants.CORPUS, workingLangArray);
        printCorpusIntituleTable(printer, corpusMetadata, CorpusConstants.FICHE_EXISTING, workingLangArray);
        printCorpusIntituleTable(printer, corpusMetadata, CorpusConstants.FICHE_NEW, workingLangArray);
        printer.appendH3("corpus_lib_intitules_special");
        List<CorpusField> specialList = CorpusMetadataUtils.getSpecialCorpusFieldList(corpusMetadata, true);
        for (CorpusField corpusField : specialList) {
            printCorpusFieldTable(printer, corpusField, workingLangArray);
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        if (proprieteCount > 0) {
            printer.appendH3("corpus_lib_intitules_propriete");
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                printCorpusFieldTable(printer, corpusField, workingLangArray);
            }
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        if (informationCount > 0) {
            printer.appendH3("corpus_lib_intitules_information");
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                printCorpusFieldTable(printer, corpusField, workingLangArray);
            }
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        if (sectionCount > 0) {
            printer.appendH3("corpus_lib_intitules_section");
            for (int i = 0; i < sectionCount; i++) {
                CorpusField corpusField = corpusMetadata.getSectionField(i);
                printCorpusFieldTable(printer, corpusField, workingLangArray);
            }
        }
        printer.endCommandBox("corpus_submit_intitules");
    }

    private static void printCorpusFieldTable(HtmlPrinter printer, CorpusField corpusField, int[] workingLangArray) {
        printer.startP();
        printer.startCode();
        printer.append("[");
        printer.append(corpusField.getFieldKey().toString());
        printer.append("]");
        printer.endCode();
        printer.endP();
        printer.startTable();
        for (int i = 0; i < workingLangArray.length; i++) {
            int langInteger = workingLangArray[i];
            String intituleString = corpusField.getLibelleString(langInteger, false);
            if (intituleString == null) {
                intituleString = "";
            }
            BdfHtmlUtils.printLangFieldRow(printer, corpusField.getFieldKey().toString() + "/", langInteger, intituleString);
        }
        printer.endTable();
    }

    private static void printCorpusIntituleTable(HtmlPrinter printer, CorpusMetadata corpusMetadata, String name, int[] workingLangArray) {
        printer.startP();
        printer.localize("corpus_corpusintitule_" + name);
        printer.endP();
        printer.startTable();
        for (int i = 0; i < workingLangArray.length; i++) {
            int langInteger = workingLangArray[i];
            String intituleString = null;
            Intitule intitule = corpusMetadata.getIntituleByName(name);
            if (intitule != null) {
                intituleString = intitule.getLibelleString(langInteger, false);
            }
            if (intituleString == null) {
                intituleString = "";
            }
            BdfHtmlUtils.printLangFieldRow(printer, "intitule_" + name + "/", langInteger, intituleString);
        }
        printer.endTable();
    }

    public static void printSpecialFieldActivation(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, FieldKey fieldKey) {
        commandBox = commandBox.derive(FieldCreationCommand.COMMANDNAME, FieldCreationCommand.getCommandKey(fieldKey));
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.putHiddenAttribute(FieldCreationCommand.CATEGORY_PARAMNAME, FieldKey.categoryToString(FieldKey.SPECIAL_CATEGORY));
        commandBox.putHiddenAttribute(FieldCreationCommand.TYPE_PARAMNAME, fieldKey.toString());
        printer.startCommandBox(commandBox);
        printer.endCommandBox("corpus_submit_fieldactivation");
    }

    public static void printFieldCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, short fieldcategory) {
        commandBox = commandBox.derive(FieldCreationCommand.COMMANDNAME, FieldCreationCommand.getCommandKey(fieldcategory));
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.putHiddenAttribute(FieldCreationCommand.CATEGORY_PARAMNAME, FieldKey.categoryToString(fieldcategory));
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_fieldcreation_type");
        printer.appendMediumInput(FieldCreationCommand.TYPE_PARAMNAME, "");
        printer.endP();
        if (fieldcategory != FieldKey.SECTION_CATEGORY) {
            printSelect(printer);
        }
        printer.endCommandBox("corpus_submit_fieldcreation");
    }

    private static void printSelect(HtmlPrinter printer) {
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_fieldcreation_ficheitemtype");
        printer.startSelect(FieldCreationCommand.FICHEITEMTYPE_PARAMNAME);
        HTMLAttributes attr = new HTMLAttributes();
        attr.setValue("");
        attr.setSelected(true);
        printer.startOption(attr);
        printer.append("---");
        printer.endOption();
        printFicheItemTypeOption(printer, CorpusField.ITEM_FIELD);
        printFicheItemTypeOption(printer, CorpusField.PERSONNE_FIELD);
        printFicheItemTypeOption(printer, CorpusField.LANGUE_FIELD);
        printFicheItemTypeOption(printer, CorpusField.PAYS_FIELD);
        printFicheItemTypeOption(printer, CorpusField.DATATION_FIELD);
        printFicheItemTypeOption(printer, CorpusField.LINK_FIELD);
        printFicheItemTypeOption(printer, CorpusField.COURRIEL_FIELD);
        printFicheItemTypeOption(printer, CorpusField.NOMBRE_FIELD);
        printFicheItemTypeOption(printer, CorpusField.MONTANT_FIELD);
        printFicheItemTypeOption(printer, CorpusField.GEOPOINT_FIELD);
        printFicheItemTypeOption(printer, CorpusField.PARA_FIELD);
        printFicheItemTypeOption(printer, CorpusField.IMAGE_FIELD);
        printer.endSelect();
        printer.endP();
    }

    private static void printFicheItemTypeOption(HtmlPrinter printer, short ficheItemType) {
        String ficheItemString = CorpusMetadataUtils.ficheItemTypeToString(ficheItemType);
        HTMLAttributes attr = new HTMLAttributes();
        attr.setSelected(false);
        attr.setValue(ficheItemString);
        printer.startOption(attr);
        printer.append("[");
        printer.append(ficheItemString);
        printer.append("] - ");
        printer.localize("corpus_ficheitem_" + ficheItemString);
        printer.endOption();

    }

    public static void printFieldRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus) {

        commandBox = commandBox.derive(FieldRemoveCommand.COMMANDNAME, FieldRemoveCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_fieldremove");
        printer.startSelect(FieldRemoveCommand.FIELDKEY_PARAMNAME);
        HTMLAttributes attr = new HTMLAttributes();
        attr.setValue("");
        attr.setSelected(true);
        printer.startOption(attr);
        printer.append("---");
        printer.endOption();
        printFieldKeyOptionList(printer, corpus, false);
        printer.endSelect();
        printer.endP();
        BdfHtmlUtils.printConfirmPassword(printer);
        printer.endCommandBox("corpus_submit_fieldremove");
    }

    private static void printFieldKeyOptionList(UtilHtmlPrinter printer, Corpus corpus, boolean withNoRemoveable) {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        if (withNoRemoveable) {
            printFieldKeyOption(printer, FieldKey.TITRE);
            printFieldKeyOption(printer, FieldKey.LANG);
        }
        if (CorpusMetadataUtils.hasSoustitreField(corpusMetadata)) {
            printFieldKeyOption(printer, FieldKey.SOUSTITRE);
        }
        if (withNoRemoveable) {
            printFieldKeyOption(printer, FieldKey.REDACTEURS);
        }
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        for (int i = 0; i < proprieteCount; i++) {
            CorpusField corpusField = corpusMetadata.getProprieteField(i);
            printFieldKeyOption(printer, corpusField.getFieldKey());
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        for (int i = 0; i < informationCount; i++) {
            CorpusField corpusField = corpusMetadata.getInformationField(i);
            printFieldKeyOption(printer, corpusField.getFieldKey());
        }
        int sectionCount = corpusMetadata.getSectionFieldCount();
        for (int i = 0; i < sectionCount; i++) {
            CorpusField corpusField = corpusMetadata.getSectionField(i);
            printFieldKeyOption(printer, corpusField.getFieldKey());
        }
    }

    private static void printFieldKeyOption(HtmlPrinter printer, FieldKey fieldKey) {
        printer.appendOption(fieldKey.toString(), false);
    }

    public static void printComponentPositionBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, ComponentUIList componentUIList) {
        commandBox = commandBox.derive(ComponentPositionCommand.COMMANDNAME, ComponentPositionCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.startTextArea(ComponentPositionCommand.COMPONENTPOSITION_PARAMNAME, 60, 20);
        int count = componentUIList.getComponentUICount();
        for (int i = 0; i < count; i++) {
            printer.append(componentUIList.getComponentUI(i).getKeyString());
            printer.append("\n");
        }
        printer.endTextArea();
        printer.endCommandBox("corpus_submit_componentposition");
    }

    public static void printComponentInputBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, ComponentUIList componentUIList) {
        commandBox = commandBox.derive(ComponentInputCommand.COMMANDNAME, ComponentInputCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        int count = componentUIList.getComponentUICount();
        printer.startUl();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof FieldUI) {
                FieldUI fieldUI = (FieldUI) componentUI;
                if (!fieldUI.getCorpusField().getFieldKey().equals(FieldKey.LANG)) {
                    printInputLi(printer, fieldUI.getCorpusField().getFieldKey().toString(), fieldUI.getInputOption(), fieldUI.getValueOption(), false);
                }
            } else if (componentUI instanceof IncludeUI) {
                IncludeUI includeUI = (IncludeUI) componentUI;
                IncludeKey includeKey = includeUI.getIncludeKey();
                String key = includeKey.toString();
                if (includeKey.isThesaurusInclude()) {
                    printInputLi(printer, key, includeUI.getInputOption(), includeUI.getValueOption(), true);
                } else if (includeKey.equals(IncludeKey.LIAGE)) {
                    printInputLi(printer, key, includeUI.getLimitOption());
                }
            }
        }
        printer.endUl();
        printer.endCommandBox("corpus_submit_componentinput");
    }

    private static void printInputLi(HtmlPrinter printer, String key, InputOption inputOption, ValueOption valueOption, boolean allInputAllowed) {
        printer.startLi();
        printer.startP();
        printer.startCode();
        if (allInputAllowed) {
            printer.append("[");
        }
        printer.append(key);
        if (allInputAllowed) {
            printer.append("]");
        }
        printer.endCode();
        printer.endP();
        printer.startSmall();
        printer.startUl();
        printer.startLi();
        printer.startP();
        short currentWidthType = inputOption.getWidthType();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_size");
        printer.startSelect(key + ComponentInputCommand.WIDTH_PARAMSUFFIX);
        printWidthOption(printer, InputOption.WIDTH_SMALL, currentWidthType);
        printWidthOption(printer, InputOption.WIDTH_MEDIUM, currentWidthType);
        printWidthOption(printer, InputOption.WIDTH_LARGE, currentWidthType);
        printer.endSelect();
        printer.endP();
        printer.endLi();
        printer.startLi();
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_rows");
        printer.appendSmallestInput(key + ComponentInputCommand.ROWS_PARAMSUFFIX, String.valueOf(inputOption.getRows()));
        printer.endP();
        printer.endLi();
        if (allInputAllowed) {
            short currentValue = inputOption.getInputType();
            printer.startLi();
            printer.startP();
            HtmlPrinterUtils.printLibColon(printer, "corpus_lib_inputtype");
            printer.startSelect(key + ComponentInputCommand.INPUT_PARAMSUFFIX);
            printInputOption(printer, InputOption.INPUT_TEXT, currentValue);
            printInputOption(printer, InputOption.INPUT_SELECT, currentValue);
            printInputOption(printer, InputOption.INPUT_CHECK, currentValue);
            printInputOption(printer, InputOption.INPUT_RADIO, currentValue);
            printer.endSelect();
            printer.endP();
            printer.endLi();
        }
        if (valueOption != null) {
            printer.startLi();
            printer.startP();
            HTMLAttributes checkAttributes = new HTMLAttributes();
            checkAttributes.setName(key + ComponentInputCommand.MANDATORY_PARAMSUFFIX);
            checkAttributes.setValue("1");
            checkAttributes.setChecked(valueOption.isMandatory());
            printer.addCheck(checkAttributes, "corpus_lib_mandatoryvalue", null);
            printer.endP();
            printer.endLi();
            printer.startLi();
            printer.startP();
            HtmlPrinterUtils.printLibColon(printer, "corpus_lib_defaulvalue");
            printer.appendLargeInput(key + ComponentInputCommand.DEFAULT_PARAMSUFFIX, valueOption.getDefaultValue());
            printer.endP();
            printer.endLi();
        }
        printer.endUl();
        printer.endSmall();
        printer.endLi();
    }

    private static void printInputLi(HtmlPrinter printer, String key, LimitOption limitOption) {
        printer.startLi();
        printer.startP();
        printer.startCode();
        printer.append("[");
        printer.append(key);
        printer.append("]");
        printer.endCode();
        printer.endP();
        printer.startSmall();
        printer.startUl();
        printer.startLi();
        printer.startP();
        HTMLAttributes nonewAttributes = new HTMLAttributes();
        nonewAttributes.setChecked(limitOption.isNoNewAllowed());
        nonewAttributes.setValue("1");
        nonewAttributes.setName(key + ComponentInputCommand.NONEW_PARAMSUFFIX);
        printer.addCheck(nonewAttributes, "corpus_lib_nonew", null);
        printer.endP();
        printer.endLi();
        printer.endUl();
        printer.endSmall();
        printer.endLi();
    }

    private static void printInputOption(HtmlPrinter printer, short value, short currentValue) {
        String string = DefaultInputOption.inputTypeToString(value);
        boolean selected = (value == currentValue);
        printer.appendOption(string, selected, "corpus_inputtype_" + string);
    }

    private static void printWidthOption(HtmlPrinter printer, short value, short currentValue) {
        String string = DefaultInputOption.widthTypeToString(value);
        boolean selected = (value == currentValue);
        printer.appendOption(string, selected, "corpus_size_" + string);
    }

    public static void printCorpusRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus) {
        commandBox = commandBox.derive(CorpusRemoveCommand.COMMANDNAME, CorpusRemoveCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        BdfHtmlUtils.printConfirmPassword(printer);
        printer.endCommandBox("corpus_submit_corpusremove");
    }

    public static void printAssociationActivationBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, CorpusUI corpusUI) {
        commandBox = commandBox.derive(AssociationActivationCommand.COMMANDNAME, AssociationActivationCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        boolean active = corpusUI.isAssociationActive();
        String infoKey = (active) ? "corpus_info_association_active" : "corpus_info_association_inactive";
        String submitKey = (active) ? "corpus_submit_association_inactivate" : "corpus_submit_association_activate";
        printer.startCommandBox(commandBox);
        printer.startP(GlobalCatalog.GLB_P_SUBTITLE);
        printer.localize(infoKey);
        printer.endP();
        printer.endCommandBox(submitKey);
    }

    public static void printIncludeChangeBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, IncludeUI includeUI, int[] workingLangArray, boolean withJQuery) {
        String key = includeUI.getIncludeKey().toString();
        LibelleHolder libelleHolder = includeUI.getCustomLibelle();
        boolean defaultLibUsage = (libelleHolder == null);
        commandBox = commandBox.derive(IncludeChangeCommand.COMMANDNAME, IncludeChangeCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.putHiddenAttribute(IncludeChangeCommand.KEY_PARAMNAME, includeUI.getKeyString());
        printer.startCommandBox(commandBox);
        printer.startP();
        HTMLAttributes checkAttributes = new HTMLAttributes();
        if (withJQuery) {
            checkAttributes.setOnClick("BDF.deployFromInvertCheck(this,'libtable_" + key + "');");
        }
        checkAttributes.setName(IncludeChangeCommand.DEFAULTLIBELLEUSAGE_PARAMNAME);
        checkAttributes.setValue("1");
        checkAttributes.setChecked((libelleHolder == null));
        printer.addCheck(checkAttributes, "corpus_lib_defaultlibelleusage", null);
        printer.endP();
        HTMLAttributes tableAttributes = new HTMLAttributes();
        tableAttributes.setId("libtable_" + key);
        if (withJQuery) {
            if (defaultLibUsage) {
                tableAttributes.addClass(GlobalCatalog.INVISIBLE);
            }
        }
        int length = workingLangArray.length;
        printer.startTable(tableAttributes);
        for (int i = 0; i < length; i++) {
            int langInteger = workingLangArray[i];
            String lib = null;
            if (libelleHolder != null) {
                lib = libelleHolder.getLibelleString(langInteger, false);
            }
            if (lib == null) {
                lib = "";
            }
            BdfHtmlUtils.printLangFieldRow(printer, IncludeChangeCommand.LIB_PARAMPREFIX, langInteger, lib);
        }
        printer.endTable();
        printer.endCommandBox("corpus_submit_includeuichange");
    }

    public static void printIncludeCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, BdfServer bdfServer, int workingLangInteger, SubsetGroupList subsetGroupList) {
        subsetGroupList = SubsetGroupUtils.checkNoRattachementSubset(corpus, subsetGroupList);
        if (subsetGroupList.getSubsetGroupCount() == 0) {
            return;
        }
        SubsetGroupListPrinter stp = new SubsetGroupListPrinter(bdfServer.getBasedefiches(), printer, workingLangInteger);
        commandBox = commandBox.derive(IncludeCreationCommand.COMMANDNAME, IncludeCreationCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.startP();
        printer.localize("corpus_lib_includeuiadd");
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setName(IncludeCreationCommand.SUBSET_PARAMNAME);
        stp.setSubsetGroupList(subsetGroupList);
        stp.setSubsetStringPrint(true);
        stp.printSelect(selectAtt);
        printer.endP();
        printer.startP();
        printer.appendRadioInput(IncludeCreationCommand.POIDSFILTER_PARAMNAME, IncludeCreationCommand.NO_POIDSFILTER_PARAMVALUE, true, "corpus_lib_poidsfilter_no");
        printer.endP();
        printer.startP();
        printer.appendRadioInput(IncludeCreationCommand.POIDSFILTER_PARAMNAME, IncludeCreationCommand.YES_POIDSFILTER_PARAMVALUE, false, "corpus_lib_poidsfilter_yes");
        printer.space();
        printer.appendSmallestInput(IncludeCreationCommand.POIDSFILTERVALUE_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("corpus_submit_includeuiadd");
    }

    public static void printIncludeRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, IncludeUI includeUI) {
        commandBox = commandBox.derive(IncludeRemoveCommand.COMMANDNAME, IncludeRemoveCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.putHiddenAttribute(IncludeRemoveCommand.KEY_PARAMNAME, includeUI.getKeyString());
        printer.startCommandBox(commandBox);
        printer.endCommandBox("corpus_submit_includeuiremove");
    }

    public static void printCommentChangeBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, CommentUI commentUI, int[] workingLangArray) {
        commandBox = commandBox.derive(CommentChangeCommand.COMMANDNAME, CommentChangeCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.putHiddenAttribute(CommentChangeCommand.NAME_PARAMNAME, commentUI.getName());
        printer.startCommandBox(commandBox);
        printCommentLocationRadioList(printer, commentUI.getLocation());
        printer.startTable();
        int length = workingLangArray.length;
        for (int i = 0; i < length; i++) {
            int langInteger = workingLangArray[i];
            String lib = commentUI.getHtmlStringByLang(langInteger);
            if (lib == null) {
                lib = "";
            }
            BdfHtmlUtils.printLangFieldRow(printer, CommentChangeCommand.COMMENTLIB_PREFIX, langInteger, lib, 77, 9, false);
        }
        printer.endTable();
        printer.endCommandBox("corpus_submit_commentchange");
    }

    public static void printCommentCreationBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, int[] workingLangArray) {
        commandBox = commandBox.derive(CommentCreationCommand.COMMANDNAME, CommentCreationCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printCommentLocationRadioList(printer, CommentUI.ALL_MASK);
        printer.startTable();
        int length = workingLangArray.length;
        for (int i = 0; i < length; i++) {
            int langInteger = workingLangArray[i];
            BdfHtmlUtils.printLangFieldRow(printer, CommentCreationCommand.COMMENTLIB_PREFIX, langInteger, "", 77, 9, false);
        }
        printer.endTable();
        printer.endCommandBox("corpus_submit_commentcreation");
    }

    private static void printCommentLocationRadioList(HtmlPrinter printer, int position) {
        String paramName = CommentChangeCommand.LOCATION_PARAMNAME;
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_commentlocation");
        printer.appendRadioInput(paramName, BdfCommand2Utils.LOCATION_FORM_PARAMVALUE, (position == CommentUI.FORM_BITVALUE), "corpus_commentlocation_form");
        printer.space();
        printer.appendRadioInput(paramName, BdfCommand2Utils.LOCATION_AFFICHAGE_PARAMVALUE, (position == CommentUI.AFFICHAGE_BITVALUE), "corpus_commentlocation_affichage");
        printer.space();
        printer.appendRadioInput(paramName, BdfCommand2Utils.LOCATION_BOTH_PARAMVALUE, (position == CommentUI.ALL_MASK), "corpus_commentlocation_both");
        printer.endP();
    }

    public static void printCommentRemoveBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus, CommentUI commentUI) {
        commandBox = commandBox.derive(CommentRemoveCommand.COMMANDNAME, CommentRemoveCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        commandBox.putHiddenAttribute(CommentRemoveCommand.NAME_PARAMNAME, commentUI.getName());
        printer.startCommandBox(commandBox);
        BdfHtmlUtils.printConfirmPassword(printer);
        printer.endCommandBox("corpus_submit_commentremove");
    }

    public static void printReloadBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus) {
        commandBox = commandBox.derive(ReloadCommand.COMMANDNAME, ReloadCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.endCommandBox("global_submit_ok");
    }

    public static void printCloneBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus) {
        commandBox = commandBox.derive(CloneCommand.COMMANDNAME, CloneCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_corpuscreation");
        printer.appendMediumInput(CloneCommand.NEWCORPUS_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("global_submit_ok");
    }

    public static void printConversionBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus) {
        commandBox = commandBox.derive(ConversionCommand.COMMANDNAME, ConversionCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_fieldtoclone");
        printer.startSelect(ConversionCommand.FIELD_PARAMNAME);
        printFieldKeyOptionList(printer, corpus, true);
        printer.endSelect();
        printer.endP();
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_newfieldkey");
        printer.appendMediumInput(ConversionCommand.NEWFIELD_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("global_submit_ok");
    }

    public static void printDuplicationBox(UtilHtmlPrinter printer, CommandBox commandBox, Corpus corpus) {
        commandBox = commandBox.derive(DuplicationCommand.COMMANDNAME, DuplicationCommand.COMMANDKEY);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        printer.startCommandBox(commandBox);
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_idcorpustokens");
        printer.appendBr();
        printer.startTextArea(DuplicationCommand.IDCORPUSTOKENS_PARAMNAME, 80, 5);
        printer.endTextArea();
        printer.endP();
        printer.startP();
        HtmlPrinterUtils.printLibColon(printer, "corpus_lib_poidswithorigin");
        printer.appendSmallInput(DuplicationCommand.POIDSWITHORIGIN_PARAMNAME, "");
        printer.endP();
        printer.endCommandBox("global_submit_ok");
    }

}
