/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.commands2.corpus.CorpusRemoveCommand;
import fr.exemole.bdfserver.commands2.corpus.FieldOptionsCommand;
import fr.exemole.bdfserver.commands2.corpus.FieldRemoveCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class CorpusHtmlUtils implements CorpusHtmlConstants {

    private CorpusHtmlUtils() {
    }

    public static void printTitre(BdfHtmlProducer bhp, Corpus corpus, String currentPage) {
        printTitre(bhp, corpus, currentPage, true);
    }

    public static void printTitre(BdfHtmlProducer bhp, Corpus corpus, String currentPage, boolean withCorpusSelect) {
        if (!bhp.isWithJQuery()) {
            withCorpusSelect = false;
        }
        bhp.startP(GlobalCatalog.GLB_P_TOOLBARLINKS);
        bhp.appendA("Corpus?page=" + CORPUS_INDEX_PAGE, "corpus_title_index");
        BdfHtmlUtils.printTiret(bhp);
        printLinks(bhp, corpus, currentPage);
        bhp.endP();
        bhp.startH1();
        bhp.startCode();
        bhp.append("[");
        bhp.append(corpus.getSubsetKey().toString());
        bhp.append("]");
        bhp.endCode();
        BdfHtmlUtils.printTiret(bhp);
        if (withCorpusSelect) {
            BdfHtmlUtils.printChangeSubsetSelect(bhp, corpus, currentPage);
        } else {
            bhp.escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, bhp.getWorkingLangInteger()));
        }
        bhp.endH1();
        bhp.printCommandMessage();
    }

    public static void printIncludePageTitre(BdfHtmlProducer bhp, Corpus corpus, String includePage) {
        bhp.startP(GlobalCatalog.GLB_P_TOOLBARLINKS);
        bhp.appendA("Corpus?page=" + CORPUS_INDEX_PAGE, "corpus_title_index");
        BdfHtmlUtils.printTiret(bhp);
        printLinks(bhp, corpus, CORPUSINCLUDE_INDEX_PAGE);
        bhp.endP();
        bhp.startH1();
        bhp.startCode();
        bhp.append("[");
        bhp.append(corpus.getSubsetKey().toString());
        bhp.append("]");
        bhp.endCode();
        if (bhp.isWithJQuery()) {
            BdfHtmlUtils.printTiret(bhp);
            BdfHtmlUtils.printChangeSubsetSelect(bhp, corpus, includePage);
        } else {
            bhp.escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, bhp.getWorkingLangInteger()));
        }
        bhp.endH1();
        bhp.startP(GlobalCatalog.GLB_P_TOOLBARLINKS);
        printIncludeLinks(bhp, corpus, includePage);
        bhp.endP();
        bhp.printCommandMessage();

    }

    private static void printIncludeLinks(BdfHtmlProducer bhp, Corpus currentCorpus, String pageActu) {
        printLink(bhp, currentCorpus, CORPUSINCLUDE_SPECIALFORM_PAGE, "corpus_title_include_special", pageActu);
        BdfHtmlUtils.printTiret(bhp);
        printLink(bhp, currentCorpus, CORPUSINCLUDE_THESAURUSFORM_PAGE, "corpus_title_include_thesaurus", pageActu);
        BdfHtmlUtils.printTiret(bhp);
        printLink(bhp, currentCorpus, CORPUSINCLUDE_CORPUSFORM_PAGE, "corpus_title_include_corpus", pageActu);
        BdfHtmlUtils.printTiret(bhp);
        printLink(bhp, currentCorpus, CORPUSINCLUDE_ALBUMFORM_PAGE, "corpus_title_include_album", pageActu);
        BdfHtmlUtils.printTiret(bhp);
        printLink(bhp, currentCorpus, CORPUSINCLUDE_COMMENTFORM_PAGE, "corpus_title_include_comment", pageActu);
    }

    public static void printLinks(BdfHtmlProducer hp, Corpus currentCorpus, String pageActu) {
        printLink(hp, currentCorpus, CORPUS_INTITULESFORM_PAGE, "corpus_title_intitules", pageActu);
        BdfHtmlUtils.printTiret(hp);
        printLink(hp, currentCorpus, CORPUSFIELD_CREATIONFORM_PAGE, "corpus_title_fieldcreation", pageActu);
        BdfHtmlUtils.printTiret(hp);
        printCommandLink(hp, currentCorpus, CORPUSFIELD_OPTIONSFORM_PAGE, FieldOptionsCommand.COMMANDKEY, pageActu);
        if (CorpusMetadataUtils.containsRemoveableField(currentCorpus.getCorpusMetadata())) {
            BdfHtmlUtils.printTiret(hp);
            printCommandLink(hp, currentCorpus, CORPUSFIELD_REMOVEFORM_PAGE, FieldRemoveCommand.COMMANDKEY, pageActu);
        }
        BdfHtmlUtils.printTiret(hp);
        printLink(hp, currentCorpus, CORPUSINCLUDE_INDEX_PAGE, "corpus_title_include", pageActu);
        BdfHtmlUtils.printTiret(hp);
        printLink(hp, currentCorpus, CORPUS_COMPONENTFORM_PAGE, "corpus_title_ui", pageActu);
        BdfHtmlUtils.printTiret(hp);
        printLink(hp, currentCorpus, CORPUS_ADVANCEDCOMMANDS_PAGE, "global_link_advancedcommands", pageActu);
        if (BasedefichesUtils.isRemoveable(currentCorpus)) {
            BdfHtmlUtils.printTiret(hp);
            printCommandLink(hp, currentCorpus, CORPUS_REMOVEFORM_PAGE, CorpusRemoveCommand.COMMANDKEY, pageActu);
        }
    }

    private static void printCommandLink(BdfHtmlProducer hp, Corpus currentCorpus, String page, String commandKey, String pageActu) {
        if (pageActu.equals(page)) {
            hp.startSpan(GlobalCatalog.GLB_SPAN_CURRENTPAGE);
            BdfHtmlUtils.printCommandLocString(hp, commandKey);
            hp.endSpan();
        } else {
            String href = "Corpus?" + RequestUtils.toParameterString(currentCorpus) + "&amp;page=" + page;
            BdfHtmlUtils.appendCommandLink(hp, href, commandKey, null);
        }
    }

    public static void printLink(HtmlPrinter hp, Corpus currentCorpus, String page, String l10nKey, String pageActu) {
        if (pageActu.equals(page)) {
            hp.startSpan(GlobalCatalog.GLB_SPAN_CURRENTPAGE);
            hp.localize(l10nKey);
            hp.endSpan();
        } else {
            String href = "Corpus?" + RequestUtils.toParameterString(currentCorpus) + "&amp;page=" + page;
            hp.appendA(href, l10nKey);
        }
    }

    public static void printCorpusState(HtmlPrinter hp, Corpus currentCorpus) {
        CorpusMetadata currentCorpusMetadata = currentCorpus.getCorpusMetadata();
        boolean containsSoustitre = CorpusMetadataUtils.hasSoustitreField(currentCorpusMetadata);
        int proprieteCount = currentCorpusMetadata.getProprieteFieldCount();
        int informationCount = currentCorpusMetadata.getInformationFieldCount();
        int sectionCount = currentCorpusMetadata.getSectionFieldCount();
        hp.startP();
        hp.startCode();
        if ((!containsSoustitre) && (proprieteCount == 0) && (sectionCount == 0) && (informationCount == 0)) {
            hp.localize("corpus_info_state_onlymandatory");
        } else {
            hp.localize("corpus_info_state_mandatory");
            if (containsSoustitre) {
                hp.appendBr();
                hp.appendBr();
                hp.append(FieldKey.SOUSTITRE.toString());
            }
            if (proprieteCount > 0) {
                hp.appendBr();
                for (int i = 0; i < proprieteCount; i++) {
                    CorpusField corpusField = currentCorpusMetadata.getProprieteField(i);
                    hp.appendBr();
                    hp.append(corpusField.getFieldKey().toString());
                }
            }
            if (sectionCount > 0) {
                hp.appendBr();
                for (int i = 0; i < sectionCount; i++) {
                    CorpusField corpusField = currentCorpusMetadata.getSectionField(i);
                    hp.appendBr();
                    hp.append(corpusField.getFieldKey().toString());
                }
            }
            if (informationCount > 0) {
                hp.appendBr();
                for (int i = 0; i < informationCount; i++) {
                    CorpusField corpusField = currentCorpusMetadata.getInformationField(i);
                    hp.appendBr();
                    hp.append(corpusField.getFieldKey().toString());
                }
            }
        }
        hp.endCode();
        hp.endP();
    }

    public static void printIncludeChangeDeployGroup(BdfHtmlProducer hp, CommandBox commandBox, Corpus corpus, IncludeUI includeUI, int[] workingLangArray) {
        String key = "Change_" + includeUI.getIncludeKey().toString();
        BdfHtmlUtils.printDeployTitle(hp, key, "global_link_change");
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setId(GlobalCatalog.DEPLOYDEFAULT_IDPREFIX + key);
        divAtt.addClass(GlobalCatalog.DIV_GLB_DEPLOYBLOCK);
        hp.startDiv(divAtt);
        LibelleHolder libelleHolder = includeUI.getCustomLibelle();
        boolean defaultLibUsage = (libelleHolder == null);
        if (defaultLibUsage) {
            hp.startP(GlobalCatalog.GLB_P_SUBTITLE);
            hp.localize("corpus_info_defaultlibelleusage");
            hp.endP();
        } else {
            hp.startTable();
            int length = workingLangArray.length;
            for (int i = 0; i < length; i++) {
                int langInteger = workingLangArray[i];
                String lib = libelleHolder.getLibelleString(langInteger, false);
                if (lib == null) {
                    lib = "";
                }
                hp.startRow();
                hp.startCellWithTopAlign();
                hp.append(LangInteger.toString(langInteger));
                HtmlPrinterUtils.printColon(hp);
                hp.append("&nbsp;");
                hp.endCell();
                hp.startCell();
                hp.escape(lib);
                hp.endCell();
                hp.endRow();
            }
            hp.endTable();
        }
        hp.endDiv();
        BdfHtmlUtils.startDeployBlock(hp, key);
        CorpusCommandBoxUtils.printIncludeChangeBox(hp, commandBox, corpus, includeUI, workingLangArray, defaultLibUsage);
        hp.endDiv();
    }

    public static void printIncludeRemoveDeployGroup(BdfHtmlProducer hp, CommandBox commandBox, Corpus corpus, IncludeUI includeUI) {
        String key = "Remove_" + includeUI.getIncludeKey().toString();
        BdfHtmlUtils.printDeployTitle(hp, key, "global_link_remove");
        BdfHtmlUtils.startDeployBlock(hp, key);
        CorpusCommandBoxUtils.printIncludeRemoveBox(hp, commandBox, corpus, includeUI);
        hp.endDiv();
    }

    public static void printCommentChangeDeployGroup(BdfHtmlProducer hp, CommandBox commandBox, Corpus corpus, CommentUI commentUI, int[] workingLangArray) {
        String key = "Change_" + commentUI.getName();
        BdfHtmlUtils.printDeployTitle(hp, key, "global_link_change");
        HTMLAttributes divAtt = new HTMLAttributes();
        divAtt.setId(GlobalCatalog.DEPLOYDEFAULT_IDPREFIX + key);
        divAtt.addClass(GlobalCatalog.DIV_GLB_DEPLOYBLOCK);
        hp.startDiv(divAtt);
        hp.startP(GlobalCatalog.GLB_P_SUBTITLE);
        HtmlPrinterUtils.printLibColon(hp, "corpus_lib_commentlocation");
        int position = commentUI.getLocation();
        boolean suivant = false;
        if ((position & CommentUI.FORM_BITVALUE) != 0) {
            suivant = true;
            hp.localize("corpus_commentlocation_form");
        }
        if ((position & CommentUI.AFFICHAGE_BITVALUE) != 0) {
            if (suivant) {
                hp.append(", ");
            }
            hp.localize("corpus_commentlocation_affichage");
        }
        hp.endP();
        hp.startTable();
        int length = workingLangArray.length;
        for (int i = 0; i < length; i++) {
            int langInteger = workingLangArray[i];
            String lib = commentUI.getHtmlStringByLang(langInteger);
            if (lib == null) {
                lib = "";
            }
            hp.startRow();
            hp.startCellWithTopAlign();
            hp.append(LangInteger.toString(langInteger));
            HtmlPrinterUtils.printColon(hp);
            hp.append("&nbsp;");
            hp.endCell();
            hp.startCell();
            hp.startPre();
            hp.escape(lib, true);
            hp.endPre();
            hp.endCell();
            hp.endRow();
        }
        hp.endTable();
        hp.endDiv();
        BdfHtmlUtils.startDeployBlock(hp, key);
        CorpusCommandBoxUtils.printCommentChangeBox(hp, commandBox, corpus, commentUI, workingLangArray);
        hp.endDiv();
    }

    public static void printCommentRemoveDeployGroup(BdfHtmlProducer hp, CommandBox commandBox, Corpus corpus, CommentUI commentUI) {
        String key = "Remove_" + commentUI.getName();
        BdfHtmlUtils.printDeployTitle(hp, key, "global_link_remove");
        BdfHtmlUtils.startDeployBlock(hp, key);
        CorpusCommandBoxUtils.printCommentRemoveBox(hp, commandBox, corpus, commentUI);
        hp.endDiv();
    }

}
