/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.corpus.CorpusCreationCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;


/**
 *
 * @author Vincent Calame
 */
public class CorpusIndexHtmlProducer extends BdfHtmlProducer implements CorpusHtmlConstants {

    private SubsetGroupList corpusGroupList;

    public CorpusIndexHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        setCssThemeFile("corpus.css");
        corpusGroupList = bdfServer.getUIManager().getCorpusSubsetKeyTree();
    }

    public void printHtml() {
        start();
        printCommandMessage();
        appendH1("corpus_title_index");
        if (bdfServer.getBasedefiches().getCorpusCount() > 0) {
            printCorpusList();
            printCorpusState();
        }
        startH2();
        BdfHtmlUtils.appendCommandLink(this, "Corpus?page=" + CORPUS_CREATIONFORM_PAGE, CorpusCreationCommand.COMMANDKEY, null);
        endH2();
        end();
    }

    private void printCorpusList() {
        startUl();
        int count = corpusGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = corpusGroupList.getSubsetGroup(i);
            boolean isTemplate = !group.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startH3();
                escape(group.getLibelleString(workingLangInteger));
                endH3();
                startUl();
            }
            for (SubsetKey subsetKey : group.getSubsetKeyList()) {
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                startLi();
                printCorpusInfo(corpus);
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpusState() {
        appendH2("corpus_title_state");
        startUl();
        int count = corpusGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = corpusGroupList.getSubsetGroup(i);
            boolean isTemplate = !group.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startH3();
                escape(group.getLibelleString(workingLangInteger));
                endH3();
                startUl();
            }
            for (SubsetKey subsetKey : group.getSubsetKeyList()) {
                Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
                startLi();
                printCurrentState(corpus);
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpusInfo(Corpus corpus) {
        startP();
        startCode();
        append("[");
        append(corpus.getSubsetKey().toString());
        append("]");
        endCode();
        BdfHtmlUtils.printTiret(this);
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        Subset masterSubset = corpus.getMasterSubset();
        if (masterSubset != null) {
            space();
            startSmall();
            append("(");
            if (masterSubset instanceof Thesaurus) {
                BdfHtmlUtils.printLibColon(this, "corpus_lib_corpustype_thesaurusslave");
                escape(ThesaurusUtils.getThesaurusIntitule((Thesaurus) masterSubset, workingLangInteger));
            } else if (masterSubset instanceof Corpus) {
                BdfHtmlUtils.printLibColon(this, "corpus_lib_corpustype_corpusslave");
                escape(CorpusMetadataUtils.getCorpusIntituleString((Corpus) masterSubset, workingLangInteger));
            }
            append(")");
            endSmall();
        }
        endP();
        startP(GlobalCatalog.GLB_P_SMALLLINKS);
        CorpusHtmlUtils.printLinks(this, corpus, "");
        endP();
    }

    void printCurrentState(Corpus corpus) {
        startP();
        startCode();
        append("[");
        append(corpus.getSubsetKey().toString());
        append("]");
        endCode();
        BdfHtmlUtils.printTiret(this);
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        endP();
        CorpusHtmlUtils.printCorpusState(this, corpus);
    }

}
