/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.corpus.FieldOptionsCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.htmlproducers.CommandBoxUtils;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusFieldList;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.FieldOptionsUtils;
import net.mapeadores.basedefiches.format.BasedefichesFormatDef;
import net.mapeadores.basedefiches.format.BasedefichesFormatUtils;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.primitives.CurrencyInteger;
import net.mapeadores.util.primitives.IntegerList;


/**
 *
 * @author Vincent Calame
 */
public class FieldOptionsFormHtmlProducer extends BdfHtmlProducer implements CorpusHtmlConstants {

    private Corpus corpus;
    private List<CorpusField> geopointFieldList = new ArrayList<CorpusField>();

    public FieldOptionsFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Corpus corpus) {
        init(bdfContext, bdfUser);
        this.corpus = corpus;
        BdfHtmlUtils.addTestJavascript(this);
        addJavascriptFiles(GlobalCatalog.BDF_JAVASCRIPT);
        setCssThemeFile("corpus.css");
    }

    public void printHtml() {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        start();
        CorpusHtmlUtils.printTitre(this, corpus, CORPUSFIELD_OPTIONSFORM_PAGE);
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Corpus");
        commandBox.setCommandFamily("CRP");
        commandBox.setCommandString(FieldOptionsCommand.COMMANDNAME);
        commandBox.setCommandKey(FieldOptionsCommand.COMMANDKEY);
        commandBox.setPage(CORPUSFIELD_OPTIONSFORM_PAGE);
        CommandBoxUtils.insertCorpus(commandBox, corpus);
        startCommandBox(commandBox);
        int proprieteCount = corpusMetadata.getProprieteFieldCount();
        if (proprieteCount > 0) {
            appendH3("corpus_title_options_propriete");
            startUl();
            for (int i = 0; i < proprieteCount; i++) {
                CorpusField corpusField = corpusMetadata.getProprieteField(i);
                printCorpusFieldLi(corpusField);
            }
            endUl();
        }
        int informationCount = corpusMetadata.getInformationFieldCount();
        if (informationCount > 0) {
            appendH3("corpus_title_options_information");
            startUl();
            for (int i = 0; i < informationCount; i++) {
                CorpusField corpusField = corpusMetadata.getInformationField(i);
                printCorpusFieldLi(corpusField);
            }
            endUl();
        }
        appendH3("corpus_title_options_titregeneration");
        CorpusField titreField = corpusMetadata.getCorpusFieldByFieldKey(FieldKey.TITRE);
        BasedefichesFormatDef ficheFormatDef = titreField.getFieldOptions().getGenerationFormatDef();
        printTitreGenerationFormat(ficheFormatDef);
        if (geopointFieldList.size() > 0) {
            appendH3("corpus_title_options_geolocalisation");
            printGeolocalisationFieldSelect();
        }
        endCommandBox("corpus_submit_options");
        end();
    }

    private void printCorpusFieldLi(CorpusField corpusField) {
        startLi();
        printIntitule(corpusField);
        printDefaultSphereKeyList(corpusField);
        printDisplay(corpusField);
        printCurrencyArray(corpusField);
        printAddressFieldArray(corpusField);
        printBaseUrl(corpusField);
        printSubfieldDisplay(corpusField);
        endLi();
    }

    private void printTitreGenerationFormat(BasedefichesFormatDef ficheFormatDef) {
        String formatSource = null;
        String formatPattern = null;
        String formatInstruction = null;
        if (ficheFormatDef != null) {
            String[] array = BasedefichesFormatUtils.toStringArray(ficheFormatDef);
            formatSource = array[0];
            formatPattern = array[1];
            formatInstruction = array[2];
        }
        startTable();
        startRow();
        HtmlPrinterUtils.printLibCells(this, "corpus_lib_options_titregeneration_source", false);
        startCell();
        appendLargeInput(FieldOptionsCommand.TITREGENERATION_SOURCE_PARAMNAME, formatSource);
        endCell();
        endRow();
        startRow();

        HtmlPrinterUtils.printLibCells(this, "corpus_lib_options_titregeneration_pattern", false);
        startCell();
        appendLargeInput(FieldOptionsCommand.TITREGENERATION_PATTERN_PARAMNAME, formatPattern);
        endCell();
        endRow();
        startRow();
        HtmlPrinterUtils.printLibCells(this, "corpus_lib_options_titregeneration_instruction", false);
        startCell();
        appendLargeInput(FieldOptionsCommand.TITREGENERATION_INSTRUCTION_PARAMNAME, formatInstruction);
        endCell();
        endRow();

        endTable();
    }

    private void printGeolocalisationFieldSelect() {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        CorpusField currentField = corpus.getCorpusMetadata().getGeolocalisationField();
        startP();
        HtmlPrinterUtils.printLibColon(this, "corpus_lib_options_geolocalisation_field");
        startSelect(FieldOptionsCommand.GEOLOCALISATIONFIELD_PARAMNAME);
        htmlAttributes.setValue("");
        htmlAttributes.setSelected((currentField == null));
        startOption(htmlAttributes);
        localize("global_lib_none");
        endOption();
        for (CorpusField otherCorpusField : geopointFieldList) {
            FieldKey fieldKey = otherCorpusField.getFieldKey();
            htmlAttributes.setValue(fieldKey.toString());
            boolean selected = ((currentField != null) && (currentField.equals(otherCorpusField)));
            htmlAttributes.setSelected(selected);
            startOption(htmlAttributes);
            printIntituleValue(otherCorpusField);
            endOption();
        }
        endSelect();
        endP();
    }

    private void printIntitule(CorpusField corpusField) {
        startP();
        printIntituleValue(corpusField);
        endP();
        startP(GlobalCatalog.GLB_P_SUBTITLE);
        HtmlPrinterUtils.printLibColon(this, "corpus_lib_fieldcreation_ficheitemtype");
        append(" ");
        String ficheItemString = CorpusMetadataUtils.ficheItemTypeToString(corpusField.getFicheItemType());
        localize("corpus_ficheitem_" + ficheItemString);
        endP();
    }

    private void printIntituleValue(CorpusField corpusField) {
        append("[");
        append(corpusField.getFieldKey().toString());
        append("] - ");
        String lib = corpusField.getLibelleString(workingLangInteger, false);
        if (lib != null) {
            escape(lib);
        }
    }

    private void printBaseUrl(CorpusField corpusField) {
        if (!FieldOptionsUtils.testFieldOption(FieldOptionsUtils.BASEURL_OPTION, corpusField)) {
            return;
        }
        String value = corpusField.getStringOption(FieldOptionsUtils.BASEURL_OPTION);
        if (value == null) {
            value = "";
        }
        String paramName = corpusField.getFieldKey().toString() + FieldOptionsCommand.BASEURL_SUFFIX;
        startP();
        startSmall();
        HtmlPrinterUtils.printLibColon(this, "corpus_lib_options_baseurl");
        appendLargeInput(paramName, value);
        endSmall();
        endP();
    }

    private void printDefaultSphereKeyList(CorpusField corpusField) {
        if (!FieldOptionsUtils.testFieldOption(FieldOptionsUtils.DEFAULTSPHEREKEY_OPTION, corpusField)) {
            return;
        }
        SubsetKey currentDefault = corpusField.getFieldOptions().getDefaultSphereKey();
        startP();
        startSmall();
        HtmlPrinterUtils.printLibColon(this, "corpus_lib_options_defaultsphere");
        startSelect(corpusField.getFieldKey().toString() + FieldOptionsCommand.DEFAULTSPHERE_SUFFIX);
        appendOption("", (currentDefault == null), "corpus_lib_options_defaultsphere_user");
        int sphereCount = basedefiches.getSphereCount();
        for (int i = 0; i < sphereCount; i++) {
            Sphere sphere = basedefiches.getSphere(i);
            SubsetKey sphereKey = sphere.getSubsetKey();
            boolean selected = ((currentDefault != null) && (currentDefault.equals(sphereKey)));
            startOption(sphereKey.getSubsetName(), selected);
            append("[");
            append(sphereKey.toString());
            append("]");
            BdfHtmlUtils.printTiret(this);
            escape(SphereUtils.getSphereIntitule(sphere, workingLangInteger));
            endOption();
        }
        endSelect();
        endSmall();
        endP();
    }

    private void printSubfieldDisplay(CorpusField corpusField) {
        if (!FieldOptionsUtils.testFieldOption(FieldOptionsUtils.SUBFIELDDISPLAY_OPTION, corpusField)) {
            return;
        }
        String paramName = corpusField.getFieldKey().toString() + FieldOptionsCommand.SUBFIELDDISPLAY_SUFFIX;
        boolean checked = corpusField.getFieldOptions().getSubfieldDisplay();
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(paramName);
        htmlAttributes.setValue("1");
        htmlAttributes.setChecked(checked);
        startP();
        startSmall();
        addCheck(htmlAttributes, "corpus_lib_options_subfielddisplay", null);
        endSmall();
        endP();
    }

    private void printCurrencyArray(CorpusField corpusField) {
        if (!FieldOptionsUtils.testFieldOption(FieldOptionsUtils.CURRENCYARRAY_OPTION, corpusField)) {
            return;
        }
        String paramName = corpusField.getFieldKey().toString() + FieldOptionsCommand.CURRENCYARRAY_SUFFIX;
        IntegerList currencyList = corpusField.getFieldOptions().getCurrencyList();
        String value = "";
        if (currencyList != null) {
            int count = currencyList.getIntegerCount();
            StringBuilder buf = new StringBuilder();
            for (int i = 0; i < count; i++) {
                int currencyInteger = currencyList.getInteger(i);
                buf.append(CurrencyInteger.toString(currencyInteger));
                buf.append("; ");
            }
            value = buf.toString();
        }
        startP();
        startSmall();
        HtmlPrinterUtils.printLibColon(this, "corpus_lib_options_currencyarray");
        appendMediumInput(paramName, value);
        endSmall();
        endP();
    }

    private void printAddressFieldArray(CorpusField corpusField) {
        if (!FieldOptionsUtils.testFieldOption(FieldOptionsUtils.ADDRESSFIELDARRAY_OPTION, corpusField)) {
            return;
        }
        geopointFieldList.add(corpusField);
        String paramName = corpusField.getFieldKey().toString() + FieldOptionsCommand.ADDRESSFIELDARRAY_SUFFIX;
        CorpusFieldList corpusFieldList = corpusField.getFieldOptions().getAddressFieldList();
        String value = "";
        if (corpusFieldList != null) {
            int count = corpusFieldList.getCorpusFieldCount();
            StringBuilder buf = new StringBuilder();
            for (int i = 0; i < count; i++) {
                CorpusField otherCorpusField = corpusFieldList.getCorpusField(i);
                if (i > 0) {
                    buf.append(", ");
                }
                buf.append(otherCorpusField.getFieldKey().toString());
            }
            value = buf.toString();
        }
        startP();
        startSmall();
        HtmlPrinterUtils.printLibColon(this, "corpus_lib_options_addressfieldarray");
        appendLargeInput(paramName, value);
        endSmall();
        endP();
    }

    private void printDisplay(CorpusField corpusField) {
        if (!FieldOptionsUtils.testFieldOption(FieldOptionsUtils.INFORMATIONDISPLAY_OPTION, corpusField)) {
            return;
        }
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(corpusField.getFieldKey().toString() + FieldOptionsCommand.BLOCKDISPLAY_SUFFIX);
        htmlAttributes.setValue("1");
        htmlAttributes.setChecked(CorpusMetadataUtils.isBlockDisplayInformationField(corpusField));
        startP();
        startSmall();
        addCheck(htmlAttributes, "corpus_lib_options_display", null);
        endSmall();
        endP();
    }

}
