/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.corpus;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.html.CommandBox;


/**
 *
 * @author Vincent Calame
 */
public class IncludeSpecialFormHtmlProducer extends BdfHtmlProducer implements CorpusHtmlConstants {

    private Corpus corpus;

    public IncludeSpecialFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Corpus corpus) {
        init(bdfContext, bdfUser);
        this.corpus = corpus;
        BdfHtmlUtils.addTestJavascript(this);
        addJavascriptFiles(GlobalCatalog.BDF_JAVASCRIPT);
        setCssThemeFile("corpus.css");
    }

    public void printHtml() {
        CorpusUI corpusUI = bdfServer.getUIManager().getCorpusUI(corpus);
        int[] workingLangArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        start();
        CorpusHtmlUtils.printIncludePageTitre(this, corpus, CORPUSINCLUDE_SPECIALFORM_PAGE);
        appendH2("corpus_title_include_special");
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Corpus");
        commandBox.setCommandFamily("CRP");
        commandBox.setPage(CORPUSINCLUDE_SPECIALFORM_PAGE);
        startUl();
        printLi(corpusUI, commandBox, IncludeKey.LIAGE, workingLangArray);
        printLi(corpusUI, commandBox, IncludeKey.RATTACHEMENT, workingLangArray);
        printLi(corpusUI, commandBox, IncludeKey.ASSOCIATION, workingLangArray);
        endUl();

        CorpusCommandBoxUtils.printAssociationActivationBox(this, commandBox, corpus, corpusUI);
        end();
    }

    private void printLi(CorpusUI corpusUI, CommandBox commandBox, IncludeKey includeKey, int[] workingLangArray) {
        IncludeUI includeUI = corpusUI.getIncludeUI(includeKey);
        if (includeUI != null) {
            startLi();
            startP();
            startCode();
            append("[");
            append(includeKey.toString());
            append("]");
            endCode();
            endP();
            CorpusHtmlUtils.printIncludeChangeDeployGroup(this, commandBox, corpus, includeUI, workingLangArray);
            endLi();
        }
    }

}
