/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.envoi;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.envoi.SendCommand;
import fr.exemole.bdfserver.email.WrongAddress;
import fr.exemole.bdfserver.email.EnvoiParameters;
import fr.exemole.bdfserver.email.FicheAttachmentParameters;
import fr.exemole.bdfserver.email.ValidAddress;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.users.BdfUser;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.croisement.DocumentCroisementList;
import net.mapeadores.basedefiches.pointeurs.FichePointeur;
import net.mapeadores.basedefiches.pointeurs.PointeurUtils;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class SendFormHtmlProducer extends BdfHtmlProducer implements EnvoiHtmlConstants {

    private short envoiType;
    private EnvoiParameters envoiParameters;
    private AbregeFiche uniqueAbregeFiche;
    private Set<AbregeFiche> abregeFicheSet;

    public SendFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, EnvoiParameters envoiParameters) {
        init(bdfContext, bdfUser);
        this.envoiType = envoiParameters.getEnvoiType();
        this.envoiParameters = envoiParameters;
        setCssThemeFile("envoi.css");
        AbregeFiche[] abregeFicheArray = envoiParameters.getAbregeFicheArray();
        if (envoiType == EnvoiParameters.ENVOI_FICHEUNIQUE) {
            uniqueAbregeFiche = abregeFicheArray[0];
        } else if (envoiType == EnvoiParameters.ENVOI_SELECTION) {
            if (abregeFicheArray != null) {
                abregeFicheSet = new HashSet<AbregeFiche>();
                int length = abregeFicheArray.length;
                for (int i = 0; i < length; i++) {
                    abregeFicheSet.add(abregeFicheArray[i]);
                }
            }
        }
    }

    public void printHtml() {
        start();
        appendH1(getTitleKey());
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction("Envoi");
        formAttributes.setMethod("POST");
        Map<String, String> hiddenParametersMap = new HashMap<String, String>();
        BdfHtmlUtils.addCommandParameter(hiddenParametersMap, SendCommand.COMMANDNAME);
        BdfHtmlUtils.addPageParameter(hiddenParametersMap, SENDCONFIRMATION_PAGE);
        BdfHtmlUtils.addErrorPageParameter(hiddenParametersMap, SENDFORM_PAGE);
        hiddenParametersMap.put(SendCommand.TYPE_PARAMNAME, EnvoiParameters.typeToString(envoiType));
        if (envoiType == EnvoiParameters.ENVOI_FICHEUNIQUE) {
            BdfHtmlUtils.addSubsetParameter(hiddenParametersMap, uniqueAbregeFiche.getCorpus());
            BdfHtmlUtils.addSubsetItemParameter(hiddenParametersMap, uniqueAbregeFiche);
        }
        BdfHtmlUtils.startForm(this, formAttributes, hiddenParametersMap);
        printCommandMessage();
        startP();
        BdfHtmlUtils.printLibColon(this, "envoi_lib_to");
        appendBr();
        HTMLAttributes toAttributes = new HTMLAttributes();
        toAttributes.setName(SendCommand.TO_PARAMNAME);
        toAttributes.setCols(60);
        toAttributes.setRows(4);
        startTextArea(toAttributes);
        List<ValidAddress> validAddressList = envoiParameters.getValidAddressList();
        if (validAddressList != null) {
            for (ValidAddress validAddress : validAddressList) {
                escape(validAddress.getOrginalString(), true);
                append("\n");
            }
        }
        endTextArea();
        endP();
        startP();
        startSmall();
        appendCheckInput(SendCommand.BCC_PARAMNAME, "1", envoiParameters.isWithRedacteurBcc(), "envoi_lib_bcc");
        endSmall();
        endP();
        startP();
        BdfHtmlUtils.printLibColon(this, "envoi_lib_subject");
        appendLargeInput(SendCommand.SUBJECT_PARAMNAME, envoiParameters.getSubject());
        endP();
        startP();
        BdfHtmlUtils.printLibColon(this, "envoi_lib_message");
        appendBr();
        HTMLAttributes messageAttributes = new HTMLAttributes();
        messageAttributes.setName(SendCommand.MESSAGE_PARAMNAME);
        messageAttributes.setCols(72);
        messageAttributes.setRows(10);
        startTextArea(messageAttributes);
        String message = envoiParameters.getMessage();
        if (message != null) {
            escape(message, true);
        }
        endTextArea();
        endP();
        if (envoiType == EnvoiParameters.ENVOI_COMPILATION) {
            XslTransformerKey templateKey = bdfUser.getXslTransformerKey(TransformationKey.COMPILATION_TRANSFORMATIONKEY);
            String templateName = templateKey.getTransformerName();
            startP();
            appendBr();
            if (templateName == null) {
                localize("envoi_info_modeledefaut");
            } else {
                localize("envoi_info_modelexslt", templateName);
            }
            endP();
        }
        if (envoiType == EnvoiParameters.ENVOI_FICHEUNIQUE) {
            printFicheUnique();
        } else {
            boolean selection = (envoiType == EnvoiParameters.ENVOI_SELECTION);
            if (selection) {
                appendH2("envoi_title_fiches_selection");
            } else {
                appendH2("envoi_title_fiches_compilation");
            }
            AbregeFicheSortedList triAbregesFiches = bdfUser.getAbregeFicheSortedList();
            HTMLAttributes checkAttributes = new HTMLAttributes();
            checkAttributes.setType(HTMLAttributes.CHECKBOX_TYPE);
            for (int i = 0; i < triAbregesFiches.getCorpusCount(); i++) {
                Corpus corpus = triAbregesFiches.getCorpus(i);
                SubsetItemList abregeFicheList = triAbregesFiches.getCorpusAbregeFicheList(i);
                startH3();
                escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
                endH3();
                startUl();
                int ficheCount = abregeFicheList.getSubsetItemCount();
                for (int j = 0; j < ficheCount; j++) {
                    AbregeFiche abrg = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                    startLi();
                    startP();
                    if (selection) {
                        startLabel(null);
                        checkAttributes.setName("selection_" + corpus.getSubsetKey().getSubsetName());
                        checkAttributes.setValue(String.valueOf(abrg.getIdcorpus()));
                        boolean checked = ((abregeFicheSet != null) && (abregeFicheSet.contains(abrg)));
                        checkAttributes.setChecked(checked);
                        appendInput(checkAttributes);
                    }
                    append(String.valueOf(abrg.getIdcorpus()));
                    BdfHtmlUtils.printTiret(this);
                    escape(abrg.getTitre());
                    if (selection) {
                        endLabel();
                    }
                    endP();
                    endLi();
                }
                endUl();
            }
        }
        BdfHtmlUtils.endForm(this, getSubmitKey());
        end();
    }

    private String getSubmitKey() {
        switch (envoiType) {
            case EnvoiParameters.ENVOI_FICHEUNIQUE:
                return "envoi_submit_envoifiche";
            case EnvoiParameters.ENVOI_COMPILATION:
                return "envoi_submit_envoicompilation";
            case EnvoiParameters.ENVOI_SELECTION:
                return "envoi_submit_envoiselection";
            default:
                return null;
        }
    }

    private String getTitleKey() {
        switch (envoiType) {
            case EnvoiParameters.ENVOI_FICHEUNIQUE:
                return "envoi_title_ficheunique";
            case EnvoiParameters.ENVOI_COMPILATION:
                return "envoi_title_compilation";
            case EnvoiParameters.ENVOI_SELECTION:
                return "envoi_title_selection";
            default:
                return null;
        }
    }

    public void printFicheUnique() {
        FicheAttachmentParameters ficheAttachmentParameters = envoiParameters.getFicheAttachmentParameters();
        startH2();
        escape(BdfUserUtils.getAbregeFicheString(bdfUser, uniqueAbregeFiche));
        endH2();
        appendH3("global_type_fiche");
        HTMLAttributes checkFicheAttributes = new HTMLAttributes();
        checkFicheAttributes.setType(HTMLAttributes.CHECKBOX_TYPE);
        checkFicheAttributes.setName(SendCommand.FICHEVERSION_PARAMNAME);
        startUl();
        startLi();
        startP();
        startLabel(null);
        checkFicheAttributes.setValue(SendCommand.HTML_FICHEVERSION_PARAMVALUE);
        checkFicheAttributes.setChecked(ficheAttachmentParameters.isWithHtml());
        appendInput(checkFicheAttributes);
        append(" ");
        localize("envoi_lib_ficheversion_html");
        endLabel();
        endP();
        endLi();
        startLi();
        startP();
        startLabel(null);
        checkFicheAttributes.setValue(SendCommand.ODT_FICHEVERSION_PARAMVALUE);
        checkFicheAttributes.setChecked(ficheAttachmentParameters.isWithOdt());
        appendInput(checkFicheAttributes);
        append(" ");
        localize("envoi_lib_ficheversion_odt");
        endLabel();
        endP();
        endLi();
        endUl();
        FichePointeur fichePointeur = PointeurUtils.toFichePointeur(uniqueAbregeFiche);
        DocumentCroisementList documentCroisementList = fichePointeur.getDocumentCroisementList();
        int length = documentCroisementList.getSubsetItemCount();
        if (length > 0) {
            HTMLAttributes checkDocAttributes = new HTMLAttributes();
            checkDocAttributes.setType(HTMLAttributes.CHECKBOX_TYPE);
            checkDocAttributes.setName(SendCommand.DOCUMENTVERSION_PARAMNAME);
            appendH3("listefiches_docfiche");
            startUl();
            for (int i = 0; i < length; i++) {
                Document document = documentCroisementList.getDocument(i);
                startLi();
                startP();
                startEm();
                append(document.getBasename());
                endEm();
                endP();
                startUl();
                int vc = document.getVersionCount();
                for (int j = 0; j < vc; j++) {
                    Version version = document.getVersion(j);
                    startLi();
                    startP();
                    startLabel(null);
                    String versionName = version.getVersionName().toString();
                    checkDocAttributes.setValue(versionName);
                    checkDocAttributes.setChecked(ficheAttachmentParameters.containsVersion(versionName));
                    appendInput(checkDocAttributes);
                    append(" ");
                    append(versionName);
                    endLabel();
                    startSmall();
                    append(" (");
                    BdfHtmlUtils.printDocumentVersionSize(this, version);
                    append(")");
                    endSmall();
                    endP();
                    endLi();
                }
                endUl();
                endLi();
            }
            endUl();
        }
    }

    @Override
    public void printCommandMessage() {
        super.printCommandMessage();
        List<WrongAddress> wrongAddressList = envoiParameters.getWrongAddressList();
        if (wrongAddressList != null) {
            HTMLAttributes tdAttributes = new HTMLAttributes();
            tdAttributes.setStyle("color: blue;");
            startTable();
            for (WrongAddress wrongAddress : wrongAddressList) {
                short errorType = wrongAddress.getErrorType();
                String codeerr = null;
                String txterr = null;
                if (errorType == WrongAddress.MALFORMED_ADDRESS_ERROR) {
                    codeerr = "envoi_err_incorrect";
                    txterr = wrongAddress.getMalformedString();
                } else {
                    txterr = wrongAddress.getRedacteurKey().toString(RedacteurKey.BRACKET_STYLE);
                    switch (errorType) {
                        case WrongAddress.REDACTEUR_INACTIVE_ERROR:
                            codeerr = "envoi_err_inactif";
                            break;
                        case WrongAddress.REDACTEUR_NOADRESS_ERROR:
                            codeerr = "envoi_err_inconnu";
                            break;
                        case WrongAddress.REDACTEUR_UNKNOWN_ERROR:
                            codeerr = "envoi_err_unknown";
                            break;
                        default:
                            throw new SwitchException("Unknown errorType : " + errorType);
                    }
                }
                startRow();
                startCell(tdAttributes);
                localize(codeerr);
                endCell();
                startCell();
                appendLargeInput(SendCommand.TO_PARAMNAME, txterr);
                endCell();
                endRow();
            }
            endTable();
        }
    }

}
