/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.saisie;

import fr.exemole.bdfserver.commands2.saisie.IndexationUpdateCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.html.forms.CommonFormHtml;
import fr.exemole.bdfserver.html.forms.IncludeFormHtml;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.request.form.FicheForm;
import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.form.TextThesaurusIncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormFactory;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.ui.corpus.DefaultIncludeUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.users.BdfUserUtils;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class FicheIndexationHtmlProducer extends BdfHtmlProducer implements FicheFormCatalog, SaisieHtmlConstants, GlobalCatalog {

    private AbregeFiche abregeFiche;
    private Thesaurus uniqueThesaurus;
    private FicheForm ficheForm;

    public FicheIndexationHtmlProducer() {
    }

    public void init(FicheFormParameters ficheFormParameters, AbregeFiche abregeFiche, Thesaurus uniqueThesaurus) {
        init(ficheFormParameters.getBdfContext(), ficheFormParameters.getBdfUser());
        addJavascriptFiles(FICHEFORM_JAVASCRIPT_LIB);
        this.abregeFiche = abregeFiche;
        this.uniqueThesaurus = uniqueThesaurus;
        setTitle("BDF - " + FICHEINDEXATION_PAGE);
        setCssThemeFile("saisie.css");
        ficheFormParameters.getFichePointeur().setCurrentSubsetItem(abregeFiche);
        ficheForm = FicheFormFactory.newInstance(ficheFormParameters);
    }

    public void printHtml() {
        start();
        if (isWithJQuery()) {
            CommonFormHtml.printFFCoreLocValues(this, getLocalisationInfo());
            IncludeFormHtml.printFFCoreLocValues(this);
        }

        String title = BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche);
        start();
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction("Saisie");
        formAttributes.setMethod("POST");
        Map<String, String> hiddenParametersMap = new HashMap<String, String>();
        BdfHtmlUtils.addCommandParameter(hiddenParametersMap, IndexationUpdateCommand.COMMANDNAME);
        BdfHtmlUtils.addPageParameter(hiddenParametersMap, FICHEINDEXATION_PAGE);
        BdfHtmlUtils.addSubsetParameter(hiddenParametersMap, abregeFiche.getCorpus());
        BdfHtmlUtils.addSubsetItemParameter(hiddenParametersMap, abregeFiche);
        BdfHtmlUtils.startForm(this, formAttributes, hiddenParametersMap);
        if (uniqueThesaurus == null) {
            BdfHtmlUtils.printFixedFormTitle(this, BdfUserUtils.getAbregeFicheString(bdfUser, abregeFiche));
            HTMLAttributes formBodyAttributes = new HTMLAttributes();
            formBodyAttributes.addClass(DIV_GLB_FORMBODY);
            startDiv(formBodyAttributes);
            printCommandMessage();
            printThesaurusTree();
            endDiv();
            BdfHtmlUtils.endForm(this, "global_submit_ok");
        } else {
            printCommandMessage();
            startH1();
            escape(title);
            endH1();
            IncludeUI includeUI = new DefaultIncludeUI(IncludeKey.newInstance(uniqueThesaurus.getSubsetKey()));
            IncludeElement includeElement = ficheForm.getFormElement(includeUI);
            IncludeFormHtml.printTextThesaurusInclude(this, (TextThesaurusIncludeElement) includeElement, null);
            append(RequestUtils.toHiddenParameter(uniqueThesaurus));
            BdfHtmlUtils.endForm(this, "global_submit_ok");
            String fichePath = "fiches/" + RequestUtils.toBasename(abregeFiche) + ".html";
            HTMLAttributes iframeAttributes = new HTMLAttributes();
            iframeAttributes.setWidth("95%");
            iframeAttributes.setHeight("500px");
            iframeAttributes.setSrc(fichePath);
            iframeAttributes.setScrolling("yes");
            startIframe(iframeAttributes);
            startP();
            appendA(fichePath, "saisie_link_affichage");
            endP();
            endIframe();
        }
        end();
    }

    private void printThesaurusTree() {
        Subset masterSubset = abregeFiche.getCorpus().getMasterSubset();
        SubsetKey masterSubsetKey = null;
        if ((masterSubset != null) && (masterSubset instanceof Thesaurus)) {
            masterSubsetKey = masterSubset.getSubsetKey();
        }
        SubsetGroupList subsetGroupList = bdfServer.getUIManager().getThesaurusSubsetKeyTree();
        int count = subsetGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = subsetGroupList.getSubsetGroup(i);
            boolean isTemplate = !group.isNoTemplateGroup();
            if (isTemplate) {
                startH2();
                escape(group.getLibelleString(workingLangInteger));
                endH2();
            }
            List<SubsetKey> subsetKeyList = group.getSubsetKeyList();
            for (SubsetKey thesaurusKey : subsetKeyList) {
                if (masterSubsetKey != null) {
                    if (thesaurusKey.equals(masterSubsetKey)) {
                        continue;
                    }
                }
                Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
                if (thesaurus != null) {
                    IncludeUI includeUI = new DefaultIncludeUI(IncludeKey.newInstance(thesaurus.getSubsetKey()));
                    IncludeElement includeElement = ficheForm.getFormElement(includeUI);
                    IncludeFormHtml.printTextThesaurusInclude(this, (TextThesaurusIncludeElement) includeElement, null);
                }
            }
            if (isTemplate) {
                startP();
                doublespace();
                endP();
            }
        }
    }

}
