/* BdfServer_Commands - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.saisie;

import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.html.forms.CommonFormHtml;
import fr.exemole.bdfserver.request.form.CorpusFieldElement;
import fr.exemole.bdfserver.request.form.FicheForm;
import fr.exemole.bdfserver.request.form.IncludeElement;
import fr.exemole.bdfserver.request.formfactory.FicheFormFactory;
import fr.exemole.bdfserver.request.formfactory.FicheFormParameters;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.ComponentUIList;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;


/**
 *
 * @author Vincent Calame
 */
public class FormExportHtmlProducer extends BdfHtmlProducer implements SaisieHtmlConstants, FicheFormCatalog {

    private FicheForm ficheForm;
    private ComponentUIList componentUIList;

    public FormExportHtmlProducer(FicheFormParameters ficheFormParameters, ComponentUIList componentUIList) {
        init(ficheFormParameters.getBdfContext(), ficheFormParameters.getBdfUser());
        ficheForm = FicheFormFactory.newInstance(ficheFormParameters);
        this.componentUIList = componentUIList;
    }

    public void printHtml() {
        int count = componentUIList.getComponentUICount();
        for (int i = 0; i < count; i++) {
            ComponentUI componentUI = componentUIList.getComponentUI(i);
            if (componentUI instanceof FieldUI) {
                printFieldUI((FieldUI) componentUI);
            } else if (componentUI instanceof IncludeUI) {
                printIncludeUI((IncludeUI) componentUI);
            } else if (componentUI instanceof CommentUI) {
            } else {
                throw new IllegalArgumentException("unknown ComponentUI implementation : " + componentUI.getClass().getName());
            }
        }
    }

    private void printFieldUI(FieldUI fieldUI) {
        CorpusFieldElement formElement = ficheForm.getFormElement(fieldUI);
        if (formElement == null) {
            return;
        }
        printStartComment(formElement.getCorpusField().getFieldKey().toString());
        CommonFormHtml.printFormElement(this, formElement, null);
        printEndComment();
    }

    private void printIncludeUI(IncludeUI includeUI) {
        IncludeElement includeElement = ficheForm.getFormElement(includeUI);
        if (includeElement == null) {
            return;
        }
        printStartComment(includeElement.getIncludeKey().toString());
        CommonFormHtml.printFormElement(this, includeElement, null);
        printEndComment();
    }

    private void printStartComment(String key) {
        append("<!-- StartFormExport = ");
        append(key);
        append(" -->\n");
    }

    private void printEndComment() {
        append("<!-- EndFormExport -->\n");
    }

}
