/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.selection;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.commands2.selection.SelectionChangeCommand;
import fr.exemole.bdfserver.commands2.selection.SelectionDefaultCommand;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.html.catalogs.SelectFormCatalog;
import fr.exemole.bdfserver.html.forms.SelectionFormPrinter;
import fr.exemole.bdfserver.htmlproducers.affichage.AffichageHtmlConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.SelectionConstants;
import net.mapeadores.basedefiches.selection.SelectionUtils;
import net.mapeadores.util.html.HTMLAttributes;


/**
 *
 * @author Vincent Calame
 */
public class SelectionFormHtmlProducer extends BdfHtmlProducer implements SelectionHtmlConstants {

    private FicheSelectOptions ficheSelectOptions;

    public SelectionFormHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, String selectionName) {
        init(bdfContext, bdfUser);
        setCssThemeFile("selection.css");
        addJavascriptFiles(SelectFormCatalog.SELECTFORM_JAVASCRIPT_LIB);
        addJavascriptFiles(SelectFormCatalog.SELECTFORM_FICHE);
        initFicheSelectOptions(selectionName);
    }

    private void initFicheSelectOptions(String selectionName) {
        if (selectionName != null) {
            if (selectionName.equals("_default")) {
                ficheSelectOptions = bdfUser.getDefaultFicheSelectOptions();
            }
        }
        if (ficheSelectOptions == null) {
            ficheSelectOptions = bdfUser.getFicheSelectOptions();
        }
    }

    public void printHtml() {
        start();
        HTMLAttributes toolBarAttributes = new HTMLAttributes();
        toolBarAttributes.addClass(GlobalCatalog.GLB_P_TOOLBARLINKS);

        startP(toolBarAttributes);
        printPageLink("selection_link_default_save", "&cmd=" + SelectionDefaultCommand.COMMANDNAME);
        space();
        printPageLink("selection_link_default_load", "&selectionname=_default");
        endP();

        appendH1("selection_title_selection");

        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setTarget("Liste");
        formAttributes.setMethod("GET");
        Map<String, String> hiddenParametersMap = new HashMap<String, String>();
        BdfHtmlUtils.addCommandParameter(hiddenParametersMap, SelectionChangeCommand.COMMANDNAME);
        BdfHtmlUtils.addRedirectParameter(hiddenParametersMap, "Affichage?page=" + AffichageHtmlConstants.FICHES_PAGE);
        BdfHtmlUtils.startForm(this, formAttributes, hiddenParametersMap);
        startP("Prg_Submit");
        printHtmlSelectOrdreTri();
        space();
        addSubmit("selection_submit_selection", false);
        endP();

        SelectionFormPrinter selectionFormWriter = new SelectionFormPrinter(bdfServer, this, workingLangInteger);
        selectionFormWriter.setFicheSelectOptions(ficheSelectOptions);
        selectionFormWriter.setBdfUser(bdfUser);
        selectionFormWriter.printFieldSets();
        selectionFormWriter.printSeparator();

        endForm();
        end();
    }

    private void printPageLink(String key, String extraParams) {
        HTMLAttributes imgAttributes = get32pxIconAttributes(key);
        String title = getLocalization(key);
        imgAttributes.setTitle(title);
        imgAttributes.setAlt(title);
        startA("Selection?page=" + SELECTIONFORM_PAGE + extraParams);
        appendImg(imgAttributes);
        endA();
    }

    private void printHtmlSelectOrdreTri() {
        localize("selection_ordretri");
        startSelect("ordre-tri");
        short ordretri = bdfUser.getAbregeFicheSortType();
        printOptionTri(SelectionConstants.TITRE_SORTTYPE, ordretri, "selection_tri_titre");
        printOptionTri(SelectionConstants.IDCORPUS_SORTTYPE, ordretri, "selection_tri_idcorpus");
        printOptionTri(SelectionConstants.CREATIONDATE_SORTTYPE, ordretri, "selection_tri_creationdate");
        printOptionTri(SelectionConstants.MODIFICATIONDATE_SORTTYPE, ordretri, "selection_tri_modificationdate");
        endSelect();
    }

    private void printOptionTri(short sortType, short valeuractu, String libKey) {
        String sort_value = SelectionUtils.sortTypeToString(sortType);
        boolean selected = (sortType == valeuractu);
        appendOption(sort_value, selected, libKey);
    }

    private HTMLAttributes get32pxIconAttributes(String iconKey) {
        HTMLAttributes imgAtt = new HTMLAttributes();
        imgAtt.setSrc("theme/icons/32x16/" + iconKey + ".png");
        imgAtt.setWidth("32");
        imgAtt.setHeight("16");
        imgAtt.setClasses("Icon");
        return imgAtt;
    }

}
