/* BdfServer_Commands - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.sphere;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.response.BdfConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.List;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurStats;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurStateHtmlProducer extends BdfHtmlProducer implements SphereHtmlConstants {

    private Redacteur redacteur;

    public RedacteurStateHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Redacteur redacteur) {
        init(bdfContext, bdfUser);
        BdfHtmlUtils.addTestJavascript(this);
        this.redacteur = redacteur;
        setCssThemeFile("sphere.css");
    }

    public void printHtml() {
        start();
        SphereHtmlUtils.printToolbar(this, REDACTEURSTATE_PAGE, redacteur);
        printCommandMessage();
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction("Sphere");
        commandBox.setCommandFamily("SPH");
        commandBox.setPage(REDACTEURSTATE_PAGE);
        if (!bdfServer.getPermissionManager().isAdmin(redacteur.getRedacteurKey())) {
            SphereCommandBoxUtils.printRedacteurActifBox(this, commandBox, redacteur);
        }
        RedacteurStats redacteurStats = new RedacteurStats(redacteur);
        if (!redacteurStats.isEmpty()) {
            SphereCommandBoxUtils.printRedacteurReplaceBox(this, commandBox, redacteur);
            printRedacteurStats(redacteurStats);
        } else {
            appendH2("sphere_title_stats");
            startP();
            localize("sphere_info_nofiche");
            endP();
        }
        if (redacteurStats.isEmpty()) {
            commandBox.setPage(BdfConstants.MESSAGE_PAGE);
            SphereCommandBoxUtils.printRedacteurRemoveBox(this, commandBox, redacteur);
        }
        end();
    }

    private void printRedacteurStats(RedacteurStats redacteurStats) {
        appendH2("sphere_title_stats");
        int count = redacteurStats.getFicheCount();
        startP();
        if (count == 1) {
            localize("sphere_info_unefiche");
        } else {
            localize("sphere_info_fiches", StringUtils.toString(count, bdfUser.getLocalisationInfo().getFormatLocale()));
        }
        endP();
        List<RedacteurStats.ByCorpusStats> byCorpusStatsList = redacteurStats.getByCorpusStatsList();
        startUl();
        for (RedacteurStats.ByCorpusStats byCorpusStats : byCorpusStatsList) {
            Corpus corpus = byCorpusStats.getCorpus();
            SubsetKey corpusKey = corpus.getSubsetKey();
            startLi();
            startP();
            startCode();
            append("[");
            append(corpusKey.toString());
            append("]");
            endCode();
            endP();
            startUl();
            List<RedacteurStats.ByFicheStats> list = byCorpusStats.getByFicheStatsList();
            for (RedacteurStats.ByFicheStats byFicheStats : list) {
                int idcorpus = byFicheStats.getIdcorpus();
                startLi();
                startP();
                append(corpusKey.getSubsetName());
                append("/");
                append(String.valueOf(idcorpus));
                append(" ");
                startSpan(GlobalCatalog.GLB_SPAN_SMALLLINKS);
                append("[");
                int poids = byFicheStats.getPoids();
                boolean premier = true;
                if ((poids & 1) != 0) {
                    premier = false;
                    append("redacteurs");
                }
                if ((poids & 2) != 0) {
                    if (premier) {
                        premier = false;
                    } else {
                        append(" - ");
                    }
                    append("propriete");
                }
                if ((poids & 4) != 0) {
                    if (premier) {
                        premier = false;
                    } else {
                        append(" - ");
                    }
                    append("information");
                }
                append("] [");
                String href = "Saisie?" + RequestUtils.toParameterString(corpusKey, idcorpus);
                appendA(href, "listefiches_modifier");
                BdfHtmlUtils.printTiret(this);
                String href2 = "fiches/" + corpusKey.getSubsetName() + "-" + idcorpus + ".html";
                appendA(href2, "listefiches_afficher");
                append("]");
                endSpan();
                endP();
                endLi();
            }
            endUl();
            endLi();
        }
        endUl();
    }

}
