/* BdfServer_Commands - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers;

import fr.exemole.bdfserver.commands2.BdfCommand2Result;
import fr.exemole.bdfserver.response.BdfConstants;
import java.io.File;
import java.io.IOException;
import java.util.Set;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.localisation.ResourceLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class ResultJsonProducer implements JsonProducer {
    
    private BdfCommand2Result bdfCommand2Result;
    private ResourceLocalisation resourceLocalisation;

    public ResultJsonProducer(BdfCommand2Result bdfCommand2Result, ResourceLocalisation resourceLocalisation) {
        this.bdfCommand2Result = bdfCommand2Result;
        this.resourceLocalisation = resourceLocalisation;
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            ResourceItem titleResourceItem = bdfCommand2Result.getCommandMessage().getTitleResourceItem();
            jsonWriter.object();
            jsonWriter.key("commandMessage");
            jsonWriter.object();
            jsonWriter.key("type");
            jsonWriter.value("done");
            jsonWriter.key("key");
            jsonWriter.value(titleResourceItem.getResourceKey());
            jsonWriter.key("text");
            jsonWriter.value(resourceLocalisation.toString(titleResourceItem));
            jsonWriter.endObject();
            jsonWriter.key("result");
            jsonWriter.object();
            writeResultObjects(jsonWriter, bdfCommand2Result.getResultObjectNameSet());
            jsonWriter.endObject();
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }

    }
    
    private void writeResultObjects(JSONWriter jsonWriter, Set<String> resultObjectNameSet) throws JSONException {
        for(String name : resultObjectNameSet) {
            Object obj = bdfCommand2Result.getResultObject(name);
            if (name.equals(BdfConstants.TMPFILE_OBJ)) {
                jsonWriter.key("tmpFileName");
                jsonWriter.value(((File) obj).getName());
            }
        }
    }



}
