/* BdfServer_Commands
 *
 * Copyright (C) 2006-2010 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.bdfserver.response;

import fr.exemole.bdfserver.request.*;
import java.util.*;
import fr.exemole.bdfserver.*;
import fr.exemole.bdfserver.users.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.servlets.*;


/**
 *
 * @author Vincent Calame
 */
public class BdfResponseParameters extends ResponseParameters implements BdfConstants {

    public static final short ERROR_MESSAGE = 1;
    private BdfServer bdfServer;
    private BdfUser bdfUser;
    private boolean admin;
    private LocalisationInfo localisationInfo;
    private String simpleText;
    private GenericParameters genericParameters;

    public BdfResponseParameters(BdfServer bdfServer, LocalisationInfo localisationInfo) {
        this.bdfServer = bdfServer;
        this.bdfUser = null;
        this.admin = false;
        this.localisationInfo = localisationInfo;
    }

    public BdfResponseParameters(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        this.localisationInfo = null;
        this.bdfUser = null;
        this.admin = false;
    }

    public BdfResponseParameters(BdfServer bdfServer, BdfUser bdfUser) {
        this.bdfServer = bdfServer;
        this.bdfUser = bdfUser;
        this.admin = BdfUserUtils.isAdmin(bdfUser);
    }

    public BdfServer getBdfServer() {
        return bdfServer;
    }

    public BdfUser getBdfUser() {
        return bdfUser;
    }

    public LocalisationInfo getLocalisationInfo() {
        if (localisationInfo != null) {
            return localisationInfo;
        }
        if (bdfUser != null) {
            return bdfUser.getLocalisationInfo();
        }
        return bdfServer.getConfigurationManager().getLangConfig().getDefaultLocalisationInfo();
    }

    public void setLocalisationInfo(LocalisationInfo localisationInfo) {
        this.localisationInfo = localisationInfo;
    }

    public boolean isAdmin() {
        return admin;
    }

    public String getSimpleText() {
        return simpleText;
    }

    public void setSimpleText(String simpleText) {
        this.simpleText = simpleText;
    }

    public GenericParameters getGenericParameters() {
        return genericParameters;
    }

    public void setGenericParameters(GenericParameters genericParameters) {
        this.genericParameters = genericParameters;
    }

    public void setType(short type) {
        if (type > 2000) {
            setContentStreamCode(type);
        } else if (type > 1000) {
            setHtmlPageCode(type);
        }
    }

}
