/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.balayage;

import fr.exemole.bdfserver.balayages.Balayage;
import fr.exemole.bdfserver.balayages.BalayageBuilder;
import fr.exemole.bdfserver.balayages.xml.BalayageDOMReader;
import fr.exemole.bdfserver.storage.BalayagesStorage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.util.text.StringUtils;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryBalayagesStorage implements BalayagesStorage {

    private File balayagesDirectory;
    private Map<String, BalayageBundle> bundleMap = new HashMap<String, BalayageBundle>();

    public DirectoryBalayagesStorage(File balayagesDirectory) {
        this.balayagesDirectory = balayagesDirectory;
        checkStorage();
    }

    public synchronized List<Balayage> checkStorage() {
        bundleMap.clear();
        if ((!balayagesDirectory.exists()) || (!balayagesDirectory.isDirectory())) {
            return Collections.emptyList();
        }
        File[] files = balayagesDirectory.listFiles();
        for (int i = 0; i < files.length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                createBalayageBundle(f);
            }
        }
        List<Balayage> result = new ArrayList<Balayage>();
        for (BalayageBundle balayageBundle : bundleMap.values()) {
            result.add(balayageBundle.getBalayage());
        }
        return result;
    }

    public InputStream getExtractionInputStream(String name, String extractionPath) throws IOException {
        BalayageBundle balayageBundle = bundleMap.get(name);
        if (balayageBundle == null) {
            return null;
        }
        File file = new File(balayageBundle.extractionDirectory, extractionPath);
        if ((!file.exists()) || (file.isDirectory())) {
            return null;
        }
        return new FileInputStream(file);
    }

    public Source getTransformerSource(String name, String xsltPath) {
        BalayageBundle balayageBundle = (BalayageBundle) bundleMap.get(name);
        if (balayageBundle == null) {
            return null;
        }
        File file = new File(balayageBundle.xsltDirectory, xsltPath);
        if ((!file.exists()) || (file.isDirectory())) {
            return null;
        }
        return new StreamSource(file);
    }

    private boolean createBalayageBundle(File directory) {
        String name = directory.getName();
        if (!StringUtils.isTechnicalName(name, true)) {
            return false;
        }
        File balayageFile = new File(directory, "balayage.xml");
        if (!balayageFile.exists()) {
            return false;
        }
        Document document = null;
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docbuilder = documentBuilderFactory.newDocumentBuilder();
            document = docbuilder.parse(balayageFile);

        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        BalayageBuilder balayageBuilder = new BalayageBuilder(name);
        BalayageDOMReader balayageReader = new BalayageDOMReader(balayageBuilder);
        balayageReader.readBalayage(document.getDocumentElement());
        Balayage balayage = balayageBuilder.toBalayage();
        BalayageBundle balayageBundle = new BalayageBundle(directory, balayage);
        if (!balayageBundle.directoryTest()) {
            return false;
        }
        bundleMap.put(name, balayageBundle);
        return true;
    }


    private class BalayageBundle {

        private File directory;
        private File xsltDirectory;
        private File extractionDirectory;
        private Balayage balayage;

        private BalayageBundle(File directory, Balayage balayage) {
            this.directory = directory;
            this.balayage = balayage;
            xsltDirectory = new File(directory, "xslt");
            extractionDirectory = new File(directory, "extraction");
        }

        private Balayage getBalayage() {
            return balayage;
        }

        private boolean directoryTest() {
            if ((!xsltDirectory.exists()) || (!xsltDirectory.isDirectory())) {
                return false;
            }
            if ((!extractionDirectory.exists()) || (!extractionDirectory.isDirectory())) {
                return false;
            }
            return true;
        }

    }

}
