/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.io.IOUtils;


/**
 *
 * @author Vincent Calame
 */
public class BdfDirectories implements BdfDirectoryConstants {

    private File baseDirectory;
    private File backupDirectory;
    private File cacheDirectory;

    public BdfDirectories(File baseDirectory, File cacheDirectory, File backupDirectory) {
        if (!baseDirectory.exists()) {
            copyDefaultFiles(baseDirectory);
        }
        this.baseDirectory = baseDirectory;
        if (cacheDirectory != null) {
            testDirectory(cacheDirectory);
            this.cacheDirectory = cacheDirectory;
        }
        if (backupDirectory != null) {
            testDirectory(backupDirectory);
            this.backupDirectory = backupDirectory;
        }
        BdfDirectoriesConversion.checkConversion(baseDirectory);
        testRelativePath(CONF);
        testRelativePath(USERS);
        testRelativePath(BASEDEFICHES);
        testRelativePath(ROLES);
    }

    /**
     * Les répertoires par catégorie sont sous la forme basedefiches/{categorie}.
     * Cette méthode crèe le répertoire pour la catégorie si elle n'existe pas.
     */
    public void testCategoryDirectory(short category) {
        String categoryString = SubsetKey.categoryToString(category);
        File directory = new File(getBaseDirectory(), BASEDEFICHES + File.separator + categoryString);
        directory.mkdirs();
    }

    public void testCroisementDirectory(SubsetCouple subsetCouple) {
        SubsetKey subsetKey1 = subsetCouple.getSubsetKey1();
        SubsetKey subsetKey2 = subsetCouple.getSubsetKey2();
        String category1 = SubsetKey.categoryToString(subsetKey1.getCategory());
        String category2 = SubsetKey.categoryToString(subsetKey2.getCategory());
        File directory = new File(getBaseDirectory(), BASEDEFICHES + File.separator + category1 + "_" + category2);
        directory.mkdirs();
    }

    public void testChronoDirectory() {
        File directory = new File(getBaseDirectory(), BdfStorageUtils.getChronoPath());
        directory.mkdirs();
    }

    public boolean isWithBackup() {
        return (backupDirectory != null);
    }

    protected void testDirectory(File directory) {
        if (!directory.exists()) {
            directory.mkdirs();
        } else if (!directory.isDirectory()) {
            throw new BdfStorageException(directory.getPath() + " is not a directory");
        }
    }

    protected void testRelativePath(String path) {
        File testDirectory = new File(baseDirectory, path);
        testDirectory(testDirectory);
    }

    public File getBaseDirectory() {
        return baseDirectory;
    }

    public File getBackupDirectory() {
        return backupDirectory;
    }

    public File getCacheDirectory() {
        return cacheDirectory;
    }

    private void copyDefaultFiles(File baseDirectory) {
        baseDirectory.mkdirs();
        File basedefichesDir = new File(baseDirectory, "basedefiches");
        File sphereDir = new File(basedefichesDir, "sphere");
        sphereDir.mkdirs();
        File addendaDir = new File(basedefichesDir, "addenda");
        addendaDir.mkdirs();
        copyDefaultFile(sphereDir, "admin.xml");
        File confDir = new File(baseDirectory, "conf");
        confDir.mkdir();
        copyDefaultFile(confDir, "passwordmanager.xml");
        copyDefaultFile(confDir, "permissionmanager.xml");
    }

    private void copyDefaultFile(File dir, String name) {
        File adminFile = new File(dir, name);
        InputStream inputStream = getClass().getResourceAsStream("defaultfiles/" + name);
        try {
            FileOutputStream fos = new FileOutputStream(adminFile);
            IOUtils.copy(inputStream, fos);
            fos.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

}
