/* BdfServer_DirectoryStorage - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.impl.AlbumDataSource;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.images.ImageResizing;
import net.mapeadores.util.awt.ResizeInfo;
import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.IIOException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.util.io.FileUtils;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryAlbumDataSource implements AlbumDataSource {

    private BdfDirectories bdfDirectories;

    public DirectoryAlbumDataSource(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public InputStream getInputStream(Illustration illustration, String dimName, ResizeInfo resizeInfo) {
        IllustrationDirectory imageDirectory = IllustrationDirectory.newInstance(bdfDirectories, illustration);
        if (resizeInfo == null) {
            return imageDirectory.getImage();
        }
        SubsetKey albumSubsetKey = illustration.getAlbum().getSubsetKey();
        String fileName = AlbumUtils.toFileName(illustration);
        File dimDir = getAlbumDirectory(albumSubsetKey, dimName);
        File dimFile = new File(dimDir, fileName);
        if (!dimFile.exists()) {
            InputStream originalInputStream = imageDirectory.getImage();
            if (originalInputStream == null) {
                return null;
            }
            try {
                ImageResizing.resize(originalInputStream, dimFile, resizeInfo, toImageIOFormat(illustration.getFormatType()));
                originalInputStream.close();
            } catch (IOException ioe) {
                if (ioe instanceof IIOException) {
                    try {
                        InputStream inputStream = this.getClass().getResourceAsStream("ecran-noir.png");
                        if (inputStream == null) {
                            throw new InternalResourceException("ecran-noir.png not found");
                        }
                        ImageResizing.resize(inputStream, dimFile, resizeInfo, toImageIOFormat(illustration.getFormatType()));
                        inputStream.close();
                    } catch (IOException ioe2) {
                        throw new BdfStorageException("file : " + dimFile.getAbsolutePath(), ioe2);
                    }
                } else {
                    throw new BdfStorageException("file : " + dimFile.getAbsolutePath(), ioe);
                }
            }
        }
        try {
            return new FileInputStream(dimFile);
        } catch (IOException ioe) {
            throw new BdfStorageException("file : " + dimFile.getAbsolutePath(), ioe);
        }
    }

    public void update(Illustration illustration, InputStream inputStream) {
        IllustrationDirectory imageDirectory = IllustrationDirectory.newInstance(bdfDirectories, illustration);
        imageDirectory.updateImage(inputStream, illustration.getFormatType());
        cleanAlbumDimDirectories(illustration.getAlbum().getSubsetKey(), illustration.getIdalbum());
    }

    public void clearDimCache(SubsetKey albumSubsetKey, String dimName) {
        File dimCacheDirectory = getAlbumDirectory(albumSubsetKey, dimName);
        if (dimCacheDirectory.exists()) {
            try {
                FileUtils.deleteDirectory(dimCacheDirectory);
            } catch (IOException ioe) {
                throw new BdfStorageException("dir : " + dimCacheDirectory.getAbsolutePath(), ioe);
            }
        }
    }

    public boolean deleteIllustration(SubsetKey albumSubsetKey, int idalbum) {
        IllustrationDirectory imageDirectory = IllustrationDirectory.newInstance(bdfDirectories, albumSubsetKey, idalbum);
        cleanAlbumDimDirectories(albumSubsetKey, idalbum);
        return imageDirectory.delete();
    }

    private void cleanAlbumDimDirectories(SubsetKey albumSubsetKey, int idalbum) {
        File cacheDir = new File(bdfDirectories.getCacheDirectory(), albumSubsetKey.toString());
        if ((cacheDir.exists())) {
            String baseName = albumSubsetKey.getSubsetName() + "-" + idalbum;
            File[] files = cacheDir.listFiles();
            int length = files.length;
            for (int i = 0; i < length; i++) {
                File d = files[i];
                if (d.isDirectory()) {
                    File f = new File(d, baseName + ".png");
                    if (f.exists()) {
                        f.delete();
                    }
                    f = new File(d, baseName + ".jpg");
                    if (f.exists()) {
                        f.delete();
                    }
                }
            }
        }
    }

    private File getAlbumDirectory(SubsetKey albumKey, String albumDimName) {
        return getAlbumDirectory(bdfDirectories, albumKey, albumDimName);
    }

    public static File getAlbumDirectory(BdfDirectories bdfDirectories, SubsetKey albumKey, String albumDimName) {
        if (albumDimName == null) {
            String albumPath = BdfDirectoryConstants.BASEDEFICHES + File.separator + "album" + File.separator + albumKey.getSubsetName();
            return new File(bdfDirectories.getBaseDirectory(), albumPath);
        } else {
            File dimDir = new File(bdfDirectories.getCacheDirectory(), albumKey.toString() + File.separator + albumDimName);
            if (!dimDir.exists()) {
                dimDir.mkdirs();
            } else if (!dimDir.isDirectory()) {
                dimDir.delete();
                dimDir.mkdir();
            }
            return dimDir;
        }
    }

    private static String toImageIOFormat(short type) {
        switch (type) {
            case AlbumConstants.PNG_FORMATTYPE:
                return "png";
            case AlbumConstants.JPG_FORMATTYPE:
                return "jpeg";
            default:
                throw new IllegalArgumentException("unknown type " + type);
        }
    }

}
