/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.old.CastorUtils;
import fr.exemole.bdfserver.users.BdfUserDef;
import fr.exemole.bdfserver.users.xml.UserDOMReader;
import fr.exemole.bdfserver.users.xml.UserXMLWriter;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryBdfUserStorage implements BdfStorage.BdfUserStorage {

    private BdfDirectories bdfDirectories;

    public DirectoryBdfUserStorage(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public void saveBdfUserDef(BdfUserDef bdfUserDef) {
        StorageFile storageFile = BdfStorageUtils.getUserStorageFile(bdfDirectories, bdfUserDef.getRedacteurKey());
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            UserXMLWriter xmlWriter = new UserXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.setIndentLength(0);
            xmlWriter.appendUser(bdfUserDef);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void removeBdfUserDef(RedacteurKey redacteurKey) {
        StorageFile storageFile = getUserStorageFile(redacteurKey);
        storageFile.delete();
    }

    public BdfUserDef getBdfUserDef(RedacteurKey redacteurKey) {
        StorageFile storageFile = getUserStorageFile(redacteurKey);
        File file = storageFile.getFile();
        if (!file.exists()) {
            return null;
        }
        Document doc = storageFile.readDocument();
        if (doc == null) {
            return null;
        }
        String tagName = doc.getDocumentElement().getTagName();
        if (tagName.equals("bdf-user-bean")) {
            BdfUserDef bdfUserDef = CastorUtils.toBdfUserDef(file, redacteurKey);
            if (bdfUserDef != null) {
                saveBdfUserDef(bdfUserDef);
            } else {
                storageFile.delete();
            }
            return bdfUserDef;
        } else {
            UserDOMReader domReader = new UserDOMReader();
            BdfUserDef bdfUserDef = domReader.readBdfUser(doc.getDocumentElement(), redacteurKey);
            return bdfUserDef;
        }
    }

    public List<BdfUserDef> check() {
        List<BdfUserDef> result = new ArrayList<BdfUserDef>();
        File usersDir = new File(bdfDirectories.getBaseDirectory(), BdfDirectoryConstants.USERS);
        if (usersDir.exists()) {
            if (!usersDir.isDirectory()) {
                throw new BdfStorageException(usersDir.getPath() + " is not a directory");
            }
        } else {
            return result;
        }
        File[] fileList = usersDir.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            String fileName = f.getName();
            if (fileName.endsWith(".xml")) {
                String redacteurKeyString = fileName.substring(0, fileName.length() - 4);
                try {
                    RedacteurKey redacteurKey = RedacteurKey.parse(redacteurKeyString);
                    BdfUserDef bdfUserDef = getBdfUserDef(redacteurKey);
                    if (bdfUserDef != null) {
                        result.add(bdfUserDef);
                    }
                } catch (ParseException pe) {
                }
            }
        }
        return result;
    }

    private StorageFile getUserStorageFile(RedacteurKey redacteurKey) {
        String path = BdfDirectoryConstants.USERS + File.separator + redacteurKey.toString(RedacteurKey.UNDERSCORE_STYLE) + ".xml";
        return new StorageFile(bdfDirectories, path);
    }

}
