/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.StorageUtils;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryPasswordStorage implements BdfStorage.PasswordStorage, BdfDirectoryConstants {

    private BdfDirectories bdfDirectories;

    public DirectoryPasswordStorage(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public List<String> check() {
        List<String> result = new ArrayList<String>();
        File file = getPasswordFile().getFile();
        if (file.exists()) {
            try {
                BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(file), "UTF-8"));
                String ligne = null;
                while ((ligne = reader.readLine()) != null) {
                    int idx = ligne.indexOf(':');
                    if (idx < 1) {
                        continue;
                    }
                    if (idx == (ligne.length() - 1)) {
                        continue;
                    }
                    String passwordKey = ligne.substring(0, idx);
                    String passwordValue = ligne.substring(idx + 1);
                    result.add(passwordKey);
                    result.add(passwordValue);
                }
                reader.close();
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
        }
        return result;
    }

    public void savePasswordArray(String[] passwordArray) {
        File file = getPasswordFile().getFile();
        try {
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
            StorageUtils.writePasswordArray(writer, passwordArray);
            writer.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    private StorageFile getPasswordFile() {
        return new StorageFile(bdfDirectories, CONF + File.separator + "passwd");
    }

}
