/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogScrutariExportXmlErrorHandler;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDefBuilder;
import net.mapeadores.basedefiches.xml.scrutariexport.ScrutariExportDOMReader;
import net.mapeadores.basedefiches.xml.scrutariexport.ScrutariExportXMLPart;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryScrutariExportStorage implements BdfStorage.ScrutariExportStorage, BdfDirectoryConstants {

    private BdfDirectories bdfDirectories;
    private String rootPath = CONF + File.separator + "scrutariexport";

    public DirectoryScrutariExportStorage(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public List<ScrutariExportDef> check(MutableProcessLog storageProcessLog) {
        List<ScrutariExportDef> result = new ArrayList<ScrutariExportDef>();
        File scrutariExportDirectory = new File(bdfDirectories.getBaseDirectory(), rootPath);
        if (scrutariExportDirectory.exists()) {
            if (!scrutariExportDirectory.isDirectory()) {
                throw new BdfStorageException(scrutariExportDirectory.getPath() + " is not a directory");
            }
        } else {
            return result;
        }
        File[] fileList = scrutariExportDirectory.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            String fileName = f.getName();
            if (fileName.endsWith(".xml")) {
                String exportName = fileName.substring(0, fileName.length() - 4);
                storageProcessLog.setCurrentURI(ProcessURI.fromFile(f));
                try {
                    ScrutariExportDefBuilder builder = ScrutariExportDefBuilder.newInstance(exportName);
                    StorageFile storageFile = new StorageFile(bdfDirectories, rootPath + File.separator + fileName);
                    Document doc = storageFile.readDocument();
                    ScrutariExportDOMReader domReader = new ScrutariExportDOMReader(builder, new LogScrutariExportXmlErrorHandler(storageProcessLog));
                    domReader.readScrutariExportDef(doc.getDocumentElement());
                    result.add(builder.toScrutariExportDef());
                } catch (ParseException pe) {
                    ProcessLogItem rsl = storageProcessLog.newLogItemInstance();
                    rsl.setCategory("WARNING");
                    rsl.setKey("wrongScrutariExportFileName");
                    rsl.setObject(fileName);
                }

            }
        }
        return result;
    }

    public void saveScrutariExportDef(ScrutariExportDef scrutariExportDef) {
        StorageFile storageFile = getScrutariExportFile(scrutariExportDef.getName());
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(buf, 0);
            ScrutariExportXMLPart xmlPart = new ScrutariExportXMLPart(xmlWriter);
            xmlPart.addScrutariExportDef(scrutariExportDef);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void removeScrutariExportDef(String name) {
        StorageFile storageFile = getScrutariExportFile(name);
        storageFile.delete();
    }

    private StorageFile getScrutariExportFile(String name) {
        String fileName = name + ".xml";
        return new StorageFile(bdfDirectories, rootPath + File.separator + fileName);
    }

}
