/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorage;
import fr.exemole.bdfserver.storage.BdfStorageException;
import fr.exemole.bdfserver.storage.directory.bdf.utils.LogSqlExportXmlErrorHandler;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.sqlexport.SqlExportDef;
import net.mapeadores.basedefiches.sqlexport.SqlExportDefBuilder;
import net.mapeadores.basedefiches.xml.sqlexport.SqlExportDOMReader;
import net.mapeadores.basedefiches.xml.sqlexport.SqlExportXMLPart;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class DirectorySqlExportStorage implements BdfStorage.SqlExportStorage, BdfDirectoryConstants {

    private BdfDirectories bdfDirectories;
    private String rootPath = CONF + File.separator + "sqlexport";

    public DirectorySqlExportStorage(BdfDirectories bdfDirectories) {
        this.bdfDirectories = bdfDirectories;
    }

    public List<SqlExportDef> check(MutableProcessLog storageProcessLog) {
        List<SqlExportDef> result = new ArrayList<SqlExportDef>();
        File sqlExportDirectory = new File(bdfDirectories.getBaseDirectory(), rootPath);
        if (sqlExportDirectory.exists()) {
            if (!sqlExportDirectory.isDirectory()) {
                throw new BdfStorageException(sqlExportDirectory.getPath() + " is not a directory");
            }
        } else {
            return result;
        }
        File[] fileList = sqlExportDirectory.listFiles();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            String fileName = f.getName();
            if (fileName.endsWith(".xml")) {
                String exportName = fileName.substring(0, fileName.length() - 4);
                storageProcessLog.setCurrentURI(ProcessURI.fromFile(f));
                try {
                    SqlExportDefBuilder builder = SqlExportDefBuilder.newInstance(exportName);
                    StorageFile storageFile = new StorageFile(bdfDirectories, rootPath + File.separator + fileName);
                    Document doc = storageFile.readDocument();
                    SqlExportDOMReader domReader = new SqlExportDOMReader(builder, new LogSqlExportXmlErrorHandler(storageProcessLog));
                    domReader.readSqlExportDef(doc.getDocumentElement());
                    result.add(builder.toSqlExportDef());
                } catch (ParseException pe) {
                    ProcessLogItem rsl = storageProcessLog.newLogItemInstance();
                    rsl.setCategory("WARNING");
                    rsl.setKey("wrongSqlExportFileName");
                    rsl.setObject(fileName);
                }

            }
        }
        return result;
    }

    public void saveSqlExportDef(SqlExportDef sqlExportDef) {
        StorageFile storageFile = getSqlExportFile(sqlExportDef.getName());
        try {
            BufferedWriter buf = storageFile.getXMLWriter();
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(buf, 0);
            SqlExportXMLPart xmlPart = new SqlExportXMLPart(xmlWriter);
            xmlPart.addSqlExportDef(sqlExportDef);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    public void removeSqlExportDef(String name) {
        StorageFile storageFile = getSqlExportFile(name);
        storageFile.delete();
    }

    private StorageFile getSqlExportFile(String name) {
        String fileName = name + ".xml";
        return new StorageFile(bdfDirectories, rootPath + File.separator + fileName);
    }

}
