/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.xml.FicheDOMReader;
import net.mapeadores.basedefiches.xml.extraction.FicheStorageXMLWriter;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class FicheStorageFile extends StorageFile {

    public FicheStorageFile(BdfDirectories bdfDirectories, String path) {
        super(bdfDirectories, path);
    }

    public static FicheStorageFile getFicheStorageFile(BdfDirectories bdfDirectories, AbregeFiche abregeFiche) {
        return getInternalStorageFile(bdfDirectories, abregeFiche.getCorpus().getSubsetKey(), abregeFiche.getIdcorpus());
    }

    /**
     *@throws IllegalArgumentException si corpusKey est incorrect et idcorpus < 1
     */
    public static FicheStorageFile getFicheStorageFile(BdfDirectories bdfDirectories, SubsetKey corpusKey, int idcorpus) {
        if (!corpusKey.isCorpusSubset()) {
            throw new IllegalArgumentException("!corpusKey.isCorpusSubset()");
        }
        if (idcorpus < 0) {
            throw new IllegalArgumentException("idcorpus < 0");
        }
        return getInternalStorageFile(bdfDirectories, corpusKey, idcorpus);
    }

    private static FicheStorageFile getInternalStorageFile(BdfDirectories bdfDirectories, SubsetKey corpusKey, int idcorpus) {
        String path = getPath(corpusKey, idcorpus);
        return new FicheStorageFile(bdfDirectories, path);
    }

    Fiche getFiche(Corpus corpus, boolean updateFields) {
        Document document = readDocument();
        if (document == null) {
            throw new BdfStorageException(new FileNotFoundException(this.getFile().getAbsolutePath()));
        }
        FicheDOMReader ficheDOMReader = new FicheDOMReader();
        Fiche fiche = ficheDOMReader.readFiche(document.getDocumentElement());
        return fiche;
    }

    public static void saveFiche(BdfDirectories bdfDirectories, AbregeFiche abregeFiche, Fiche fiche) {
        Corpus corpus = abregeFiche.getCorpus();
        FicheStorageFile ficheStorageFile = getFicheStorageFile(bdfDirectories, corpus.getSubsetKey(), abregeFiche.getIdcorpus());
        try {
            BufferedWriter buf = ficheStorageFile.getXMLWriter();
            FicheStorageXMLWriter xmlWriter = new FicheStorageXMLWriter();
            xmlWriter.setAppendable(buf);
            xmlWriter.appendFiche(fiche);
            buf.close();
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

    private static String getPath(SubsetKey corpusKey, int idcorpus) {
        StringBuilder buf = new StringBuilder();
        buf.append("basedefiches");
        buf.append(File.separatorChar);
        buf.append("corpus");
        buf.append(File.separatorChar);
        buf.append(corpusKey.getSubsetName());
        buf.append(File.separatorChar);
        buf.append("f.");
        if (idcorpus < 1000) {
            buf.append("000");
        } else {
            if (idcorpus < 10000) {
                buf.append("00");
            } else if (idcorpus < 100000) {
                buf.append("0");
            }
            buf.append(idcorpus / 1000);
        }
        buf.append(File.separatorChar);
        buf.append(idcorpus);
        buf.append(".xml");
        return buf.toString();
    }

}
