/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.bdf;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.logging.ProcessURI;
import org.w3c.dom.Document;


/**
 * Désigne un fichier destiné à stocker des informations de la base.
 *
 * @author Vincent Calame
 */
public class StorageFile {

    public final static String ENCODING = "UTF-8";
    private File file;
    private File backupFile;

    public StorageFile(BdfDirectories bdfDirectories, String path) {
        file = new File(bdfDirectories.getBaseDirectory(), path);
        if (bdfDirectories.isWithBackup()) {
            backupFile = new File(bdfDirectories.getBackupDirectory(), path);
        }
    }

    public boolean exists() {
        if (file.exists()) {
            return true;
        }
        if (backupExist()) {
            return true;
        }
        return false;
    }

    public boolean backupExist() {
        return (backupFile != null) && (backupFile.exists());
    }

    public File getFile() {
        return file;
    }

    public ProcessURI toProcessURI() {
        return ProcessURI.fromFile(file);
    }

    public void delete() {
        backup();
        try {
            if (file.exists()) {
                FileUtils.forceDelete(file);
            }
        } catch (IOException ioe) {
            throw new BdfStorageException(file, ioe);
        }
    }

    public Document readDocument() {
        if (!file.exists()) {
            return null;
        }
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docbuilder = null;
        Document document = null;
        try {
            docbuilder = documentBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new BdfStorageException(pce);
        }
        try {
            InputStream is = new FileInputStream(file);
            document = docbuilder.parse(is);
            is.close();
        } catch (Exception e) {
            if (!backupExist()) {
                throw new BdfStorageException(file, e);
            }
            try {
                InputStream is = new FileInputStream(backupFile);
                document = docbuilder.parse(is);
                is.close();
            } catch (Exception e2) {
                throw new BdfStorageException(backupFile, e);
            }
            restore();
        }
        return document;
    }

    public boolean restore() {
        if (backupExist()) {
            try {
                FileUtils.copyFile(backupFile, file);
            } catch (IOException ioe) {
                throw new BdfStorageException(file, ioe);
            }
            return true;
        }
        return false;
    }

    public BufferedWriter getXMLWriter() throws IOException {
        testPath();
        backup();
        OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
        BufferedWriter bufWriter = new BufferedWriter(writer);
        bufWriter.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        return bufWriter;
    }

    private void testPath() {
        file.getParentFile().mkdirs();
    }

    private void backup() {
        if ((file.exists()) && (backupFile != null) && (!file.isDirectory())) {
            try {
                FileUtils.copyFile(file, backupFile);
            } catch (IOException ioe) {
                throw new BdfStorageException(backupFile, ioe);
            }
        }
    }

}
