/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.conversion_0_20;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Image;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.xml.FicheDOMReader;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;


/**
 *
 * @author Vincent Calame
 */
public class Conversion {

    private File baseDir;

    public Conversion(File bdfDir) {
        this.baseDir = bdfDir;
    }

    public void run() throws IOException {
        conversionCorpus();
        conversionAddenda();
        analyseCorpus();
        conversionAlbum();
    }

    private void conversionAddenda() {
        File addendaDir = new File(baseDir, "basedefiches" + File.separator + "addenda");
        addendaDir.mkdirs();
        File documentDir = new File(baseDir, "basedefiches" + File.separator + "document");
        if (!documentDir.exists()) {
            return;
        }
        File xDir = new File(addendaDir, "x");
        xDir.mkdir();
        File[] list = documentDir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File f = list[i];
            String name = f.getName();
            if (f.isDirectory()) {
                File[] idalbumDirList = f.listFiles();
                int ln2 = idalbumDirList.length;
                for (int j = 0; j < ln2; j++) {
                    File idalbumDir = idalbumDirList[j];
                    if (idalbumDir.isDirectory()) {
                        File[] docList = idalbumDir.listFiles();
                        int ln3 = docList.length;
                        for (int k = 0; k < ln3; k++) {
                            File doc = docList[k];
                            String docName = doc.getName();
                            int idx = docName.indexOf('.');
                            if (idx > 0) {
                                try {
                                    Integer itg = Integer.parseInt(docName.substring(0, idx));
                                    doc.renameTo(new File(doc.getParentFile(), "doc." + docName.substring(idx + 1)));
                                } catch (NumberFormatException nfe) {
                                }
                            }
                        }
                    }
                }
                f.renameTo(new File(xDir, name));
            } else if (name.equals("document.xml")) {
                f.renameTo(new File(addendaDir, "x.xml"));
            }
        }
        documentDir.delete();
        File croisementDir = new File(baseDir, "basedefiches" + File.separator + "corpus_document");
        if (croisementDir.exists()) {
            croisementDir.renameTo(new File(croisementDir.getParentFile(), "corpus_addenda"));
        }
    }

    private void conversionCorpus() {
        File corpusDir = new File(baseDir, "basedefiches" + File.separator + "corpus");
        if (!corpusDir.exists()) {
            return;
        }
        File[] list = corpusDir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File dir = list[i];
            if (dir.isDirectory()) {
                File[] fichesDirList = dir.listFiles();
                int ln2 = fichesDirList.length;
                for (int j = 0; j < ln2; j++) {
                    File fichesDir = fichesDirList[j];
                    if (fichesDir.isDirectory()) {
                        File[] fichesList = fichesDir.listFiles();
                        int ln3 = fichesList.length;
                        for (int k = 0; k < ln3; k++) {
                            File fiche = fichesList[k];
                            String name = fiche.getName();
                            if ((name.startsWith("fiche-")) && (name.endsWith(".xml"))) {
                                try {
                                    int idcorpus = Integer.parseInt(name.substring(6, 12));
                                    fiche.renameTo(new File(fichesDir, idcorpus + ".xml"));
                                } catch (NumberFormatException nfe) {
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private void analyseCorpus() throws IOException {
        FicheDOMReader ficheDOMReader = new FicheDOMReader();
        File corpusDir = new File(baseDir, "basedefiches" + File.separator + "corpus");
        if (!corpusDir.exists()) {
            return;
        }
        File[] list = corpusDir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File f = list[i];
            if (!f.isDirectory()) {
                String name = f.getName();
                int idxfin = name.indexOf('.');
                int idxdebut = 0;
                int idxbas = name.lastIndexOf('_');
                if (idxbas != -1) {
                    idxdebut = idxbas + 1;
                }
                SubsetKey corpusKey = SubsetKey.build(SubsetKey.CATEGORY_CORPUS, name.substring(idxdebut, idxfin));
                Document document = DOMUtils.readDocument(f);
                AlbumKeyDOMReader domReader = new AlbumKeyDOMReader();
                domReader.read(document.getDocumentElement());
                Map<FieldKey, SubsetKey> map = domReader.getMap();
                if (map.size() > 0) {
                    File crpDir = new File(corpusDir, corpusKey.getSubsetName());
                    File[] groupeFichesList = crpDir.listFiles();
                    int lng2 = groupeFichesList.length;
                    for (int j = 0; j < lng2; j++) {
                        File groupeFichesDir = groupeFichesList[j];
                        if (groupeFichesDir.isDirectory()) {
                            File[] fichesList = groupeFichesDir.listFiles();
                            int fichesLength = fichesList.length;
                            for (int k = 0; k < fichesLength; k++) {
                                File ficheFile = fichesList[k];
                                int idcorpus = getIdcorpus(ficheFile.getName());
                                Fiche fiche = ficheDOMReader.readFiche(DOMUtils.readDocument(ficheFile).getDocumentElement());
                                for (Map.Entry<FieldKey, SubsetKey> entry : map.entrySet()) {
                                    FieldKey fieldKey = entry.getKey();
                                    SubsetKey albumKey = entry.getValue();
                                    Propriete propriete = fiche.getPropriete(fieldKey);
                                    if (propriete != null) {
                                        FicheItem ficheItem = propriete.getFicheItem();
                                        if (ficheItem instanceof Image) {
                                            String src = ((Image) ficheItem).getSrc();
                                            int idalbum = getIdalbum(src);
                                            if (idalbum == 0) {
                                                System.out.println("src = " + src);
                                            } else {
                                                SimpleCroisement simpleCroisement = SimpleCroisement.getSimpleCroisement(corpusKey, albumKey);
                                                simpleCroisement.add(idcorpus, idalbum);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        File dir = new File(baseDir, "basedefiches" + File.separator + "corpus_album");
        dir.mkdirs();
        SimpleCroisement.saveXml(dir);
    }

    private void conversionAlbum() throws IOException {
        File albumDir = new File(baseDir, "basedefiches" + File.separator + "album");
        if (!albumDir.exists()) {
            return;
        }
        File[] list = albumDir.listFiles();
        int length = list.length;
        for (int i = 0; i < length; i++) {
            File dir = list[i];
            if (dir.isDirectory()) {
                File[] dirList = dir.listFiles();
                int ln = dirList.length;
                for (int j = 0; j < ln; j++) {
                    File f = dirList[j];
                    String name = f.getName();
                    if (f.isDirectory()) {
                        if (!name.startsWith("i.")) {
                            FileUtils.deleteDirectory(f);
                        }
                    } else {
                        if (name.startsWith("bdf-")) {
                            int idx = name.indexOf('.');
                            if (idx == -1) {
                                System.out.println("fichier inconnu : " + name);
                            } else {
                                try {
                                    int idalbum = Integer.parseInt(name.substring(4, idx));
                                    File destDir = null;
                                    if (idalbum >= 1000) {
                                        destDir = new File(dir, "i.001");
                                    } else {
                                        destDir = new File(dir, "i.000");
                                    }
                                    File idalbumDir = new File(destDir, String.valueOf(idalbum));
                                    idalbumDir.mkdirs();
                                    String extension = name.substring(idx + 1);
                                    if ((!extension.equals("png")) && (!extension.equals("jpg"))) {
                                        System.out.println("fichier inconnu : " + name);
                                    } else {
                                        f.renameTo(new File(idalbumDir, "img." + extension));
                                    }
                                } catch (NumberFormatException nfe) {
                                    System.out.println("fichier inconnu : " + name);
                                }
                            }
                        } else {
                            System.out.println("fichier inconnu : " + name);
                        }
                    }
                }
            }
        }
    }

    private int getIdcorpus(String fileName) {
        int idx1 = fileName.indexOf('-');
        int idx2 = fileName.indexOf('.');
        return Integer.parseInt(fileName.substring(idx1 + 1, idx2));
    }

    private int getIdalbum(String src) {
        if (src.startsWith("bdf-")) {
            int idx = src.indexOf('.');
            try {
                int idalbum = Integer.parseInt(src.substring(4, idx));
                return idalbum;
            } catch (NumberFormatException nfe) {
            }
        }
        return 0;
    }

}
