/* BdfServer_DirectoryStorage - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.conversion_0_20;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetCouple;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.primitives.PrimUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public class SimpleCroisement {

    static private Map<SubsetCouple, SimpleCroisement> map = new HashMap<SubsetCouple, SimpleCroisement>();
    private List<Long> longList = new ArrayList<Long>();
    private SubsetCouple couple;

    private SimpleCroisement(SubsetCouple couple) {
        this.couple = couple;
    }

    public void add(int idcorpus, int idalbum) {
        longList.add(PrimUtils.toLong(idcorpus, idalbum));
    }

    public static SimpleCroisement getSimpleCroisement(SubsetKey corpusKey, SubsetKey albumKey) {
        SubsetCouple couple = new SubsetCouple(corpusKey, albumKey);
        SimpleCroisement simpleCroisement = map.get(couple);
        if (simpleCroisement == null) {
            simpleCroisement = new SimpleCroisement(couple);
            map.put(couple, simpleCroisement);
        }
        return simpleCroisement;
    }

    private void save(File dir) throws IOException {
        String name = couple.getSubsetKey1().getSubsetName() + "_" + couple.getSubsetKey2().getSubsetName() + ".xml";
        File f = new File(dir, name);
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f), "UTF-8"));
        XMLWriter xmlWriter = XMLUtils.getXMLWriter(writer, 0);
        xmlWriter.appendXMLDeclaration();
        xmlWriter.openTag("croisement");
        for (long l : longList) {
            xmlWriter.startOpenTag("ci");
            xmlWriter.addAttribute("id1", PrimUtils.getInt1(l));
            xmlWriter.addAttribute("id2", PrimUtils.getInt2(l));
            xmlWriter.closeEmptyTag();
        }
        xmlWriter.closeTag("croisement");
        writer.close();
    }

    public static void saveXml(File dir) throws IOException {
        for (SimpleCroisement simpleCroisement : map.values()) {
            simpleCroisement.save(dir);
        }
    }

}
