/* BdfServer_DirectoryStorage - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.jdbm;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import jdbm.helper.Serializer;
import net.mapeadores.basedefiches.corpus.fiche.Section;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheBlockList;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.srz.FichePrimitivesStorage;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;

/**
 *
 * @author Vincent Calame
 */
public class CorpsFicheSerializer implements Serializer {

    private FieldKeyCoder fieldKeyCoder;

    public CorpsFicheSerializer(FieldKeyCoder fieldKeyCoder) {
        this.fieldKeyCoder = fieldKeyCoder;
    }

    public byte[] serialize(Object obj) throws IOException {
        CorpsFiche corpsFiche = (CorpsFiche) obj;
        Fiche fiche = corpsFiche.getFiche();
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        DataOutputStream dataOutput = new DataOutputStream(output);
        DataOutputPrimitivesWriter primWriter = new DataOutputPrimitivesWriter(dataOutput);
        int sectionCount = fiche.getSectionCount();
        dataOutput.writeInt(sectionCount);
        for(int i = 0; i < sectionCount; i++) {
            Section section = fiche.getSection(i);
            int fieldKeyCode = fieldKeyCoder.askForCode(section);
            dataOutput.writeInt(fieldKeyCode);
            FichePrimitivesStorage.writeFicheBlockList(primWriter, section);
        }
        dataOutput.flush();
        return output.toByteArray();
    }


    public Object deserialize(byte[] serialized) {
        ByteArrayInputStream input = new ByteArrayInputStream(serialized);
        DataInputStream dataInput = new DataInputStream(input);
        DataInputPrimitivesReader primReader = new DataInputPrimitivesReader(dataInput);
        try {
            Fiche fiche = new Fiche();
            int sectionCount = dataInput.readInt();
            for(int i = 0; i < sectionCount; i++) {
                int fieldKeyCode = dataInput.readInt();
                FicheBlockList ficheBlockList = FichePrimitivesStorage.readFicheBlockList(primReader);
                if (ficheBlockList != null) {
                    FieldKey fieldKey = fieldKeyCoder.getFieldKey(fieldKeyCode);
                    if ((fieldKey != null) && (fieldKey.isSectionFieldKey())) {
                        fiche.setSection(fieldKey, ficheBlockList);
                    }
                }
            }
            return new CorpsFiche(fiche);
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
     }
}
