/* BdfServer_DirectoryStorage - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.jdbm;

import fr.exemole.bdfserver.storage.BdfStorageException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import jdbm.helper.Serializer;
import net.mapeadores.basedefiches.corpus.fiche.Fiche;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.FicheItemList;
import net.mapeadores.basedefiches.corpus.fiche.Information;
import net.mapeadores.basedefiches.corpus.fiche.Para;
import net.mapeadores.basedefiches.corpus.fiche.Propriete;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.srz.FichePrimitivesStorage;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public class EnteteFicheSerializer implements Serializer {

    private FieldKeyCoder fieldKeyCoder;

    public EnteteFicheSerializer(FieldKeyCoder fieldKeyCoder) {
        this.fieldKeyCoder = fieldKeyCoder;
    }

    public byte[] serialize(Object obj) throws IOException {
        EnteteFiche enteteFiche = (EnteteFiche) obj;
        Fiche fiche = enteteFiche.getFiche();
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        DataOutputStream dataOutput = new DataOutputStream(output);
        DataOutputPrimitivesWriter primWriter = new DataOutputPrimitivesWriter(dataOutput);
        dataOutput.writeInt(enteteFiche.getIdcorpus());
        dataOutput.writeInt(fiche.getLangInteger());
        dataOutput.writeUTF(fiche.getTitre());
        FichePrimitivesStorage.writeFicheItem(primWriter, fiche.getSoustitre());
        FichePrimitivesStorage.writeFicheItemList(primWriter, fiche.getRedacteurs());
        int proprieteCount = fiche.getProprieteCount();
        dataOutput.writeInt(proprieteCount);
        for (int i = 0; i < proprieteCount; i++) {
            Propriete propriete = fiche.getPropriete(i);
            int fieldKeyCode = fieldKeyCoder.askForCode(propriete);
            dataOutput.writeInt(fieldKeyCode);
            FichePrimitivesStorage.writeProprieteFicheItem(primWriter, propriete);
        }
        int informationCount = fiche.getInformationCount();
        dataOutput.writeInt(informationCount);
        for (int i = 0; i < informationCount; i++) {
            Information information = fiche.getInformation(i);
            int fieldKeyCode = fieldKeyCoder.askForCode(information);
            dataOutput.writeInt(fieldKeyCode);
            FichePrimitivesStorage.writeFicheItemList(primWriter, information);
        }
        dataOutput.flush();
        return output.toByteArray();
    }

    public Object deserialize(byte[] serialized) {
        ByteArrayInputStream input = new ByteArrayInputStream(serialized);
        DataInputStream dataInput = new DataInputStream(input);
        DataInputPrimitivesReader primReader = new DataInputPrimitivesReader(dataInput);
        try {
            int idcorpus = dataInput.readInt();
            Fiche fiche = new Fiche();
            EnteteFiche enteteFiche = new EnteteFiche(idcorpus, fiche);
            int langInteger = dataInput.readInt();
            fiche.setLangInteger(langInteger);
            String titre = dataInput.readUTF();
            fiche.setTitre(titre);
            Para soustitrePara = (Para) FichePrimitivesStorage.readFicheItem(primReader);
            fiche.setSoustitre(soustitrePara);
            FicheItemList redacteurList = FichePrimitivesStorage.readFicheItemList(primReader);
            fiche.setRedacteurs(redacteurList);
            int proprieteCount = dataInput.readInt();
            for(int i = 0; i < proprieteCount; i++) {
                int fieldKeyCode = dataInput.readInt();
                FicheItem ficheItem = FichePrimitivesStorage.readFicheItem(primReader);
                if (ficheItem != null) {
                    FieldKey fieldKey = fieldKeyCoder.getFieldKey(fieldKeyCode);
                    if ((fieldKey != null) && (fieldKey.isProprieteFieldKey())) {
                        fiche.setPropriete(fieldKey, ficheItem);
                    }
                }
            }
            int informationCount = dataInput.readInt();
            for(int i = 0; i < informationCount; i++) {
                int fieldKeyCode = dataInput.readInt();
                FicheItemList ficheItemList = FichePrimitivesStorage.readFicheItemList(primReader);
                if (ficheItemList != null) {
                    FieldKey fieldKey = fieldKeyCoder.getFieldKey(fieldKeyCode);
                    if ((fieldKey != null) && (fieldKey.isInformationFieldKey())) {
                        fiche.setInformation(fieldKey, ficheItemList);
                    }
                }
            }
            return enteteFiche;
        } catch (IOException ioe) {
            throw new BdfStorageException(ioe);
        }
    }

}
