/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.old;

import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.users.BdfUserDef;
import fr.exemole.bdfserver.users.BdfUserDefBuilder;
import java.text.ParseException;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.LocalisationInfoBean;


/**
 *
 * @author Vincent Calame
 */
public class BdfUserBeanStorage {

    private BdfUserBeanStorage() {
    }

    public static BdfUserDef toBdfUserDef(BdfUserBean bdfUserBean, RedacteurKey redacteurKey) {
        BdfUserDefBuilder bdfUserDefBuilder = new BdfUserDefBuilder(redacteurKey);
        XslTemplateBean[] array = bdfUserBean.getXslTemplateBeanArray();
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                XslTemplateBean bean = array[i];
                if (bean == null) {
                    continue;
                }
                String transformationKey = bean.getTransformationKey();
                if (transformationKey != null) {
                    try {
                        bdfUserDefBuilder.setXSLTemplateName(TransformationKey.parse(transformationKey), bean.getTemplateName());
                    } catch (ParseException pe) {
                    }
                }
            }
        }
        String[] transformerArray = bdfUserBean.getStreamTransformerKeyArray();
        if (transformerArray != null) {
            for (int i = 0; i < transformerArray.length; i++) {
                try {
                    StreamTransformerKey streamTransformerKey = StreamTransformerKey.parse(transformerArray[i]);
                    bdfUserDefBuilder.putStreamTransformerKey(streamTransformerKey);
                } catch (ParseException pe) {
                }
            }
        }
        FicheSelectOptionsBean optionsBean = bdfUserBean.getDefaultFicheSelectOptionsBean();
        if (optionsBean != null) {
            FicheSelectOptions options = new FicheSelectOptions();
            SelectionBeanStorage.fill(options, optionsBean);
            bdfUserDefBuilder.setDefaultFicheSelectOptions(options);
        }
        bdfUserDefBuilder.setAbregeFicheSortType(bdfUserBean.getAbregeFicheSortType());
        bdfUserDefBuilder.setTableExportParametersBean(bdfUserBean.getTableExportParametersBean());
        LocalisationInfoBean locBean = bdfUserBean.getLocalisationInfoBean();
        if (locBean != null) {
            bdfUserDefBuilder.setLocalisationInfo(new LocalisationInfo(locBean));
        }
        return bdfUserDefBuilder.toBdfUserDef();
    }

}
