/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.old;

import fr.exemole.bdfserver.managers.ConfigurationManager;
import fr.exemole.bdfserver.managers.LangConfig;
import fr.exemole.bdfserver.managers.PasswordManager;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.managers.PolicyManager;
import fr.exemole.bdfserver.managers.ScrutariExportManager;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.managers.configuration.ConfigurationPathException;
import fr.exemole.bdfserver.managers.policies.ThesaurusPolicy;
import fr.exemole.bdfserver.managers.SqlExportManager;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;


/**
 *
 * @author Vincent Calame
 */
public class ManagerBeanStorage {

    private ManagerBeanStorage() {
    }

    public static void fillManager(Object manager, ManagerBean managerBean) {
        if (manager instanceof PasswordManager) {
            fillPasswordManager((PasswordManager) manager, (PasswordManagerBean) managerBean);
        } else if (manager instanceof PermissionManager) {
            fillPermissionManager((PermissionManager) manager, (PermissionManagerBean) managerBean);

        } else if (manager instanceof UIManager) {
            fillUIManager((UIManager) manager, (UiManagerBean) managerBean);
        } else if (manager instanceof ConfigurationManager) {
            fillConfigurationManager((ConfigurationManager) manager, (ConfigurationManagerBean) managerBean);
        } else if (manager instanceof PolicyManager) {
            fillPolicyManager((PolicyManager) manager, (PolicyManagerBean) managerBean);
        } else if (manager instanceof ScrutariExportManager) {
            fillRchfExportManager((ScrutariExportManager) manager, (RchfExportManagerBean) managerBean);
        } else {
        }
    }

    private static void fillUIManager(UIManager uiManager, UiManagerBean uiManagerBean) {
        CorpusUiBean[] array = uiManagerBean.getCorpusUiBeanArray();
        Basedefiches basedefiches = uiManager.getBdfServer().getBasedefiches();
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                UIBeanStorage.addCorpusUIBean(uiManager, array[i], basedefiches);
            }
        }
        GroupTemplateBean[] groupTemplateBeanArray = uiManagerBean.getGroupTemplateBeanArray();
        if (groupTemplateBeanArray != null) {
            for (int i = 0; i < groupTemplateBeanArray.length; i++) {
                UIBeanStorage.addGroupTemplateBean(uiManager.getSubsetKeyGroupTemplates(), groupTemplateBeanArray[i]);
            }
        }
        UIBeanStorage.fillSubsetKeyTree(uiManager.getCorpusSubsetKeyTree(), uiManagerBean.getCorpusSubsetKeyTreeTokens(), basedefiches);
        UIBeanStorage.fillSubsetKeyTree(uiManager.getThesaurusSubsetKeyTree(), uiManagerBean.getThesaurusSubsetKeyTreeTokens(), basedefiches);
        UIBeanStorage.fillSubsetKeyTree(uiManager.getSphereSubsetKeyTree(), uiManagerBean.getSphereSubsetKeyTreeTokens(), basedefiches);
    }

    private static void fillPermissionManager(PermissionManager permissionManager, PermissionManagerBean permissionManagerBean) {
        RoleRedacteurBean[] array = permissionManagerBean.getRoleRedacteurBeanArray();
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                RoleRedacteurBean bean = array[i];
                try {
                    RedacteurKey redacteurKey = RedacteurKey.parse(bean.getRedacteurKey());
                    String roleName = bean.getRoleName();
                    if (roleName.equals("admin")) {
                        Redacteur redacteur = permissionManager.getRedacteur(redacteurKey);
                        if (redacteur != null) {
                            permissionManager.setAdmin(redacteur, true);
                        }
                    }
                } catch (java.text.ParseException pe) {
                }
            }
        }
    }

    private static void fillPasswordManager(PasswordManager passwordManager, PasswordManagerBean passwordManagerBean) {
        passwordManager.setPassword(PasswordManager.CONFIRM_PASSWORDKEY, passwordManagerBean.getConfirmPassword());
        PasswordRedacteurBean[] array = passwordManagerBean.getPasswordRedacteurBeanArray();
        for (int i = 0; i < array.length; i++) {
            PasswordRedacteurBean bean = array[i];
            try {
                RedacteurKey redacteurKey = RedacteurKey.parse(bean.getRedacteurKey());
                Redacteur redacteur = SphereUtils.getRedacteur(passwordManager.getBdfServer().getBasedefiches(), redacteurKey);
                if (redacteur != null) {
                    passwordManager.setPassword(redacteurKey.toString(), bean.getPassword());
                }
            } catch (java.text.ParseException pe) {
            }
        }
    }

    private static void fillConfigurationManager(ConfigurationManager configurationManager, ConfigurationManagerBean configurationManagerBean) {
        LangConfig langConfig = fromBean(configurationManagerBean.getLangConfigBean());
        langConfig.setWithOriginalName(configurationManagerBean.isOriginalAllowed());
        configurationManager.setLangConfig(langConfig);
        /*try {
        configurationManager.setConfigurationPath(configurationManager.CONTEXT_URL, configurationManagerBean.getContextUrl(), null);
        } catch(ConfigurationPathException cpe) {

        }*/
        /*try {
        configurationManager.setConfigurationPath(ConfigurationManager.INSERT_URL, configurationManagerBean.getInsertUrl(), null);
        } catch (ConfigurationPathException cpe) {
        }*/
        try {
            configurationManager.setConfigurationPath(ConfigurationManager.WWW_ROOT, null, configurationManagerBean.getWwwRootPath());
        } catch (ConfigurationPathException cpe) {
        }
        OutputDirectoryBean[] outputDirectoryBeanArray = configurationManagerBean.getOutputDirectoryBeanArray();
        if (outputDirectoryBeanArray != null) {
            for (int i = 0; i < outputDirectoryBeanArray.length; i++) {
                OutputDirectoryBean bean = outputDirectoryBeanArray[i];
                try {
                    configurationManager.setConfigurationPath(bean.getName(), bean.getExternalUrl(), bean.getDirectoryPath());
                } catch (ConfigurationPathException cpe) {
                }
            }
        }
    }

    private static void fillPolicyManager(PolicyManager policyManager, PolicyManagerBean policyManagerBean) {
        ThesaurusPolicyBean[] array = policyManagerBean.getThesaurusPolicyBeanArray();
        if (array != null) {
            for (int i = 0; i < array.length; i++) {
                ThesaurusPolicyBean bean = array[i];
                try {
                    SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, bean.getThesaurusSubId());
                    ThesaurusPolicy thesaurusPolicy = policyManager.getThesaurusPolicy(thesaurusKey);
                    if (thesaurusPolicy != null) {
                        fillThesaurusPolicy(policyManager.getBdfServer().getBasedefiches(), thesaurusPolicy, bean);
                    }
                } catch (ParseException pe) {
                }
            }
        }
    }

    private static void fillRchfExportManager(ScrutariExportManager scrutariExportManager, RchfExportManagerBean rchfExportManagerBean) {
        RchfExportBean[] rchfExportBeanArray = rchfExportManagerBean.getRchfExportBeanArray();
        if (rchfExportBeanArray != null) {
            for (int i = 0; i < rchfExportBeanArray.length; i++) {
                RchfExportBean rchfExportBean = rchfExportBeanArray[i];
                ScrutariExportDef scrutariExportDef = ScrutariExportBeanStorage.toScrutariExportDef(scrutariExportManager.getBdfServer().getBasedefiches(), rchfExportBean);
                if (scrutariExportDef != null) {
                    scrutariExportManager.putScrutariExportDef(scrutariExportDef);
                }
            }
        }
    }

    private static void fillThesaurusPolicy(Basedefiches basedefiches, ThesaurusPolicy thesaurusPolicy, ThesaurusPolicyBean bean) {
        try {
            short ajoutPolicy = bean.getAjoutPolicy();
            Object policyParameter = null;
            if (ajoutPolicy == ThesaurusPolicy.AJOUT_TRANSFERT) {
                String trsSubId = bean.getTransferThesaurusSubId();
                if (trsSubId == null) {
                    ajoutPolicy = ThesaurusPolicy.AJOUT_AUCUN;
                } else {
                    Thesaurus thesaurus = BasedefichesUtils.getThesaurus(basedefiches, trsSubId);
                    if ((thesaurus != null) && (ThesaurusUtils.isBabelienType(thesaurus))) {
                        policyParameter = thesaurus.getSubsetKey();
                    } else {
                        ajoutPolicy = ThesaurusPolicy.AJOUT_AUCUN;
                    }
                }
            } else if (ajoutPolicy == ThesaurusPolicy.AJOUT_VERIFICATION) {
                String[] array = bean.getVerifThesaurusSubIdArray();
                if ((array == null) || (array.length == 0)) {
                    ajoutPolicy = ThesaurusPolicy.AJOUT_AUTORISE;
                }
                List<SubsetKey> list = new ArrayList<SubsetKey>();
                for (int i = 0; i < array.length; i++) {
                    String subId = array[i];
                    if (subId != null) {
                        Thesaurus thesaurus = BasedefichesUtils.getThesaurus(basedefiches, subId);
                        if ((thesaurus != null) && (!thesaurus.getSubsetKey().equals(thesaurusPolicy.getThesaurusKey())) && (!ThesaurusUtils.isIdalphaType(thesaurus))) {
                            list.add(thesaurus.getSubsetKey());
                        }
                    }
                }
                if (list.size() > 0) {
                    policyParameter = list;
                } else {
                    ajoutPolicy = ThesaurusPolicy.AJOUT_AUTORISE;
                }
            }
            thesaurusPolicy.setAjoutPolicy(ajoutPolicy, policyParameter);
        } catch (IllegalArgumentException iae) {
            iae.printStackTrace();
        } catch (IllegalStateException iae) {
            iae.printStackTrace();
        }
    }

    private static LangConfig fromBean(LangConfigBean langConfigBean) {
        LangConfig langConfig = new LangConfig();
        if (langConfigBean != null) {
            langConfig.setAllLanguages(langConfigBean.isAllLanguages());
            langConfig.setWorkingLangIntegerArray(langConfigBean.getWorkingLangArray());
            langConfig.setSupplementaryLangIntegerArray(langConfigBean.getSupplementaryLangArray());
        }
        return langConfig;
    }

    /**
     * Retourne le nom du fichier dans lequel a été enregistré le « bean du manager »
     */
    public static String getBeanFileName(Object manager) {
        if (manager instanceof PasswordManager) {
            return "passwordmanager";
        } else if (manager instanceof PermissionManager) {
            return "permissionmanager";
        } else if (manager instanceof UIManager) {
            return "uimanager";
        } else if (manager instanceof ConfigurationManager) {
            return "configurationmanager";
        } else if (manager instanceof PolicyManager) {
            return "policymanager";
        } else if (manager instanceof ScrutariExportManager) {
            return "rchfexportmanager";
        } else if (manager instanceof SqlExportManager) {
            return "sqlexportmanager";
        }
        throw new IllegalArgumentException("Unknown manager class : " + manager.getClass().getName());
    }

    public static Class getBeanClass(Object manager) {
        if (manager instanceof PasswordManager) {
            return PasswordManagerBean.class;
        } else if (manager instanceof PermissionManager) {
            return PermissionManagerBean.class;
        } else if (manager instanceof UIManager) {
            return UiManagerBean.class;
        } else if (manager instanceof ConfigurationManager) {
            return ConfigurationManagerBean.class;
        } else if (manager instanceof PolicyManager) {
            return PolicyManagerBean.class;
        } else if (manager instanceof ScrutariExportManager) {
            return RchfExportManagerBean.class;
        }
        throw new IllegalArgumentException("Unknown manager class : " + manager.getClass().getName());
    }

    /**
     * Retourne le nom du fichier dans lequel doit être enregistré le « bean du manager »
     */
    public static String getBeanFileName(ManagerBean managerBean) {
        if (managerBean instanceof PasswordManagerBean) {
            return "passwordmanager";
        } else if (managerBean instanceof PermissionManagerBean) {
            return "permissionmanager";
        } else if (managerBean instanceof UiManagerBean) {
            return "uimanager";
        } else if (managerBean instanceof ConfigurationManagerBean) {
            return "configurationmanager";
        } else if (managerBean instanceof PolicyManagerBean) {
            return "policymanager";
        } else if (managerBean instanceof RchfExportManagerBean) {
            return "rchfexportmanager";
        }
        throw new IllegalArgumentException("Unknown manager class : " + managerBean.getClass().getName());
    }

}
