/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.old;

import java.text.ParseException;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.format.patterns.MotcleFormat;
import net.mapeadores.basedefiches.include.IncludeUtils;
import net.mapeadores.basedefiches.scrutariexport.CorpusScrutariDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDef;
import net.mapeadores.basedefiches.scrutariexport.ScrutariExportDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.ThesaurusScrutariDefBuilder;
import net.mapeadores.basedefiches.scrutariexport.WrongFormatSourceKeyException;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.text.AccoladePattern;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariExportBeanStorage {

    private ScrutariExportBeanStorage() {
    }

    public static ScrutariExportDef toScrutariExportDef(Basedefiches basedefiches, RchfExportBean rchfExportBean) {
        ScrutariExportDefBuilder scrutariExportDefBuilder = null;
        try {
            scrutariExportDefBuilder = ScrutariExportDefBuilder.newInstance(rchfExportBean.getName());
        } catch (java.text.ParseException pe) {
            return null;
        }
        try {
            scrutariExportDefBuilder.setBaseName(rchfExportBean.getBaseName());
        } catch (java.text.ParseException pe) {
        }
        scrutariExportDefBuilder.setLangUiArray(rchfExportBean.getLangUiArray());
        FicheSelectOptionsBean optionsBean = rchfExportBean.getFicheSelectOptionsBean();
        if (optionsBean != null) {
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            SelectionBeanStorage.fill(ficheSelectOptions, optionsBean);
            scrutariExportDefBuilder.setFicheSelectOptions(ficheSelectOptions);
        }
        CorpusRchfInfoBean[] corpusRchfInfoBeanArray = rchfExportBean.getCorpusRchfInfoBeanArray();
        if (corpusRchfInfoBeanArray != null) {
            for (int i = 0; i < corpusRchfInfoBeanArray.length; i++) {
                CorpusRchfInfoBean corpusRchfInfoBean = corpusRchfInfoBeanArray[i];
                if (corpusRchfInfoBean != null) {
                    String corpusSubId = corpusRchfInfoBean.getCorpusSubId();
                    if (corpusSubId != null) {
                        Corpus corpus = BasedefichesUtils.getCorpus(basedefiches, corpusSubId);
                        if (corpus != null) {
                            CorpusScrutariDefBuilder corpusScrutariDefBuilder = new CorpusScrutariDefBuilder(corpus.getSubsetKey());
                            fill(corpusScrutariDefBuilder, corpusRchfInfoBean);
                            scrutariExportDefBuilder.addCorpusScrutariDef(corpusScrutariDefBuilder.toCorpusScrutariDef());
                        }
                    }
                }
            }
        }
        ThesaurusRchfInfoBean[] thesaurusRchfInfoBeanArray = rchfExportBean.getThesaurusRchfInfoBeanArray();
        if (thesaurusRchfInfoBeanArray != null) {
            for (int i = 0; i < thesaurusRchfInfoBeanArray.length; i++) {
                ThesaurusRchfInfoBean thesaurusRchfInfoBean = thesaurusRchfInfoBeanArray[i];
                if (thesaurusRchfInfoBean != null) {
                    String thesaurusSubId = thesaurusRchfInfoBean.getThesaurusSubId();
                    if (thesaurusSubId != null) {
                        Thesaurus thesaurus = BasedefichesUtils.getThesaurus(basedefiches, thesaurusSubId);
                        if (thesaurus != null) {
                            ThesaurusScrutariDefBuilder thesaurusScrutariDefBuilder = new ThesaurusScrutariDefBuilder(thesaurus.getSubsetKey());
                            fill(thesaurusScrutariDefBuilder, thesaurusRchfInfoBean);
                            scrutariExportDefBuilder.addThesaurusScrutariDef(thesaurusScrutariDefBuilder.toThesaurusScrutariDef());
                        }
                    }
                }
            }
        }
        String fichePatternString = rchfExportBean.getFicheGlobalPattern();
        if (fichePatternString != null) {
            try {
                scrutariExportDefBuilder.setFicheHrefPattern(new AccoladePattern(fichePatternString));
            } catch (java.text.ParseException pe) {
            }
        }
        String motclePatternString = rchfExportBean.getMotcleGlobalPattern();
        if (motclePatternString != null) {
            try {
                scrutariExportDefBuilder.setMotcleHrefPattern(new AccoladePattern(motclePatternString));
            } catch (java.text.ParseException pe) {
            }
        }
        scrutariExportDefBuilder.setBaseIcon(rchfExportBean.getBaseIcon());
        return scrutariExportDefBuilder.toScrutariExportDef();
    }

    private static void fill(CorpusScrutariDefBuilder corpusScrutariDefBuilder, CorpusRchfInfoBean corpusRchfInfoBean) {
        String[] keyArray = corpusRchfInfoBean.getComplementKeyArray();
        if (keyArray != null) {
            for (int i = 0; i < keyArray.length; i++) {
                Object key = IncludeUtils.parseKey(keyArray[i]);
                if (key != null) {
                    corpusScrutariDefBuilder.addComplementKey(key);
                }
            }
        }
        String dateKeyString = corpusRchfInfoBean.getDateKey();
        if (dateKeyString != null) {
            try {
                corpusScrutariDefBuilder.setDateKey(dateKeyString);
            } catch (ParseException pe) {
            } catch (WrongFormatSourceKeyException wfske) {
            }
        }
    }

    private static void fill(ThesaurusScrutariDefBuilder thesaurusScrutariDefBuilder, ThesaurusRchfInfoBean thesaurusRchfInfoBean) {
        thesaurusScrutariDefBuilder.setSelected(thesaurusRchfInfoBean.isSelected());
        String pat = thesaurusRchfInfoBean.getLibellePattern();
        if (pat != null) {
            try {
                MotcleFormat motcleFormat = new MotcleFormat();
                motcleFormat.setPattern(pat);
                thesaurusScrutariDefBuilder.setMotcleFormatPattern(pat);
            } catch (java.text.ParseException pe) {
            }
        }
    }

}
