/* BdfServer_DirectoryStorage - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.storage.directory.old;

import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetKeyGroupTemplates;
import fr.exemole.bdfserver.ui.SubsetKeyTree;
import fr.exemole.bdfserver.ui.SubsetKeyTreeUtils;
import fr.exemole.bdfserver.ui.corpus.CommentUI;
import fr.exemole.bdfserver.ui.corpus.ComponentUIEditor;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUIUtils;
import fr.exemole.bdfserver.ui.corpus.DefaultCommentUI;
import fr.exemole.bdfserver.ui.corpus.FieldUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import fr.exemole.bdfserver.ui.options.DefaultInputOption;
import fr.exemole.bdfserver.ui.options.InputOption;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleBean;


/**
 *
 * @author Vincent Calame
 */
public class UIBeanStorage {

    private UIBeanStorage() {
    }

    public static void fillSubsetKeyTree(SubsetKeyTree subsetKeyTree, String[] tokens, Basedefiches basedefiches) {
        if (tokens == null) {
            return;
        }
        subsetKeyTree.clear();
        SubsetKeyTreeUtils.parseTokens(subsetKeyTree, tokens);
        subsetKeyTree.checkBasedefiches(basedefiches);
    }

    public static void addGroupTemplateBean(SubsetKeyGroupTemplates templates, GroupTemplateBean groupTemplateBean) {
        if (groupTemplateBean == null) {
            return;
        }
        String groupName = groupTemplateBean.getGroupName();
        if (groupName == null) {
            return;
        }
        templates.createTemplate(groupName);
        LibelleBean[] libelleBeanArray = groupTemplateBean.getLibelleBeanArray();
        if (libelleBeanArray != null) {
            for (int i = 0; i < libelleBeanArray.length; i++) {
                Libelle libelle = libelleBeanArray[i].toLibelle();
                templates.putLibelle(groupName, libelle);
            }
        }
    }

    public static void addCorpusUIBean(UIManager bdfUI, CorpusUiBean corpusUIBean, Basedefiches basedefiches) {
        if (corpusUIBean == null) {
            return;
        }
        String nomCorpus = corpusUIBean.getCorpusSubId();
        if (nomCorpus == null) {
            return;
        }
        Corpus corpus = BasedefichesUtils.getCorpus(basedefiches, nomCorpus);
        if (corpus == null) {
            return;
        }
        CorpusUI corpusUI = bdfUI.getCorpusUI(corpus);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        ComponentUiBean[] array = corpusUIBean.getComponentUiBeanArray();
        if (array != null) {
            List keyList = new ArrayList();
            for (int i = 0; i < array.length; i++) {
                String key = addComponentUIBean(corpusUI, array[i], basedefiches);
                if (key != null) {
                    keyList.add(key);
                }
            }
            int size = keyList.size();
            String[] positionArray = new String[size];
            for (int i = 0; i < size; i++) {
                positionArray[i] = (String) keyList.get(i);
            }
            corpusUI.getComponentUIEditor().setPositionArray(positionArray);
        }
    }

    public static String addComponentUIBean(CorpusUI corpusUI, ComponentUiBean componentUIBean, Basedefiches basedefiches) {
        if (componentUIBean == null) {
            return null;
        }
        ComponentUIEditor editor = corpusUI.getComponentUIEditor();
        String key = checkKey(componentUIBean, corpusUI.getCorpusSubsetKey().getSubsetName());
        if (key == null) {
            return null;
        }
        if (componentUIBean.isDuplicate()) {
            if (CorpusUIUtils.isCommentUIKey(key)) {
                return key;
            } else {
                return null;
            }
        }
        HtmlInputBean htmlInputBean = componentUIBean.getHtmlInputBean();
        InputOption inputOption = null;
        if (htmlInputBean != null) {
            short widthType = InputOption.WIDTH_LARGE;
            switch (htmlInputBean.getSize()) {
                case -1:
                    widthType = InputOption.WIDTH_SMALL;
                    break;
                case - 2:
                    widthType = InputOption.WIDTH_MEDIUM;
                    break;
                case -3:
                    widthType = InputOption.WIDTH_LARGE;
                    break;
            }
            inputOption = new DefaultInputOption(widthType, htmlInputBean.getRows(), htmlInputBean.getInputType());
        }
        LibelleBean[] libelleBeanArray = componentUIBean.getLibelleBeanArray();
        String options = componentUIBean.getOptions();
        if (CorpusUIUtils.isCommentUIKey(key)) {
            String name = key.substring(8);
            CommentUI commentUI = editor.getCommentUI(name, true);
            if (libelleBeanArray != null) {
                for (int i = 0; i < libelleBeanArray.length; i++) {
                    Libelle libelle = libelleBeanArray[i].toLibelle();
                    editor.putHtmlString(commentUI, libelle.getLangInteger(), libelle.getLibelleString());
                }
            }
            if (options != null) {
                int location = DefaultCommentUI.maskStringToInt(options);
                editor.setLocation(commentUI, location);
            }
            return key;
        }
        FieldKey fieldKey = CorpusUIUtils.isFieldKey(key);
        if (fieldKey != null) {
            FieldUI fieldUI = corpusUI.getFieldUI(fieldKey);
            if (fieldUI == null) {
                return null;
            }
            if (inputOption == null) {
                inputOption = CorpusUIUtils.getDefaultInputOption(fieldUI.getCorpusField().getFieldKey());
            }
            editor.setInputOption(fieldUI, inputOption);
            return key;
        }
        IncludeKey includeKey = null;
        try {
            includeKey = IncludeKey.parse(key);
        } catch (java.text.ParseException pe) {
            return null;
        }
        IncludeUI includeUI = editor.getIncludeUI(includeKey, true);
        if ((options != null) && (includeUI != null)) {
            if (options.equals("defaultlib")) {
                editor.setDefaultLibelleUsage(includeUI, true);
            } else if (options.equals("definedlib")) {
                editor.setDefaultLibelleUsage(includeUI, false);
                if (libelleBeanArray != null) {
                    for (int i = 0; i < libelleBeanArray.length; i++) {
                        Libelle libelle = libelleBeanArray[i].toLibelle();
                        if (libelle != null) {
                            editor.putLibelle(includeUI, libelle);
                        }
                    }
                }
            }
        }
        if ((includeUI != null) && (includeKey.isThesaurusInclude())) {
            if (inputOption == null) {
                inputOption = CorpusUIUtils.getDefaultIndexationInputOption();
            }
            editor.setInputOption(includeUI, inputOption);
        }
        return key;
    }

    /*
     * Compatibilité ancienne version
     */
    private static String checkKey(ComponentUiBean componentUIBean, String corpusSubid) {
        String key = componentUIBean.getKey();
        if (key == null) {
            return null;
        }
        if (key.indexOf('_') != -1) {
            return key;
        }
        if (key.startsWith(corpusSubid)) {
            try {
                int itg = Integer.parseInt(key.substring(corpusSubid.length()));
                key = "comment_" + key;
            } catch (NumberFormatException nfe) {
            }
        }
        return key;
    }

    /*public static CorpusUiBean toCorpusUIBean(CorpusUI corpusUI) {
    CorpusUiBean corpusUIBean = new CorpusUiBean();
    corpusUIBean.setCorpusSubId(corpusUI.getCorpusSubsetKey().getSubId());
    List list = corpusUI.getOrderedComponentUIList(CorpusUI.ALL_LIST);
    int size = list.size();
    ComponentUiBean[] array = new ComponentUiBean[size];
    HashSet duplicateSet = new HashSet();
    for(int i = 0; i < size;i++) {
    ComponentUI componentUI = (ComponentUI)list.get(i);
    if (componentUI instanceof CommentUI) {
    CommentUI commentUI = (CommentUI) componentUI;
    boolean duplicate = duplicateSet.contains(commentUI.getName());
    array[i] = toComponentUIBean(commentUI,duplicate);
    if (!duplicate) duplicateSet.add(commentUI.getName());
    } else if (componentUI instanceof FieldUI) {
    array[i] = toComponentUIBean((FieldUI)componentUI);
    } else if (componentUI instanceof IncludeUI) {
    array[i] = toComponentUIBean((IncludeUI)componentUI);
    }
    }
    corpusUIBean.setComponentUiBeanArray(array);
    return corpusUIBean;
    }*/
    /*public static ComponentUiBean toComponentUIBean(CommentUI commentUI, boolean duplicate) {
    ComponentUiBean componentUIBean = new ComponentUiBean();
    componentUIBean.setKey(commentUI.getKeyString());
    componentUIBean.setDuplicate(duplicate);
    if (!duplicate) {
    List list = commentUI.getLibelleList();
    int size = list.size();
    LibelleBean[] libelleBeanArray = new LibelleBean[size];
    for(int i = 0; i < size;i++) {
    libelleBeanArray[i] = new LibelleBean((Libelle)list.get(i));
    }
    componentUIBean.setLibelleBeanArray(libelleBeanArray);
    componentUIBean.setOptions(DefaultCommentUI.maskToString(commentUI.getLocation()));
    }
    return componentUIBean;
    }*/
    /*public static ComponentUiBean toComponentUIBean(FieldUI fieldUI) {
    ComponentUiBean componentUIBean = new ComponentUiBean();
    componentUIBean.setKey(fieldUI.getKeyString());
    HtmlInputBean htmlInputBean = fieldUI.getHtmlInputBean();
    if (!CorpusUIUtils.equalsToDefault(fieldUI.getCorpusField().getFieldKey(),htmlInputBean)) componentUIBean.setHtmlInputBean(htmlInputBean);
    return componentUIBean;
    }*/
    /*public static ComponentUiBean toComponentUIBean(IncludeUI includeUI) {
    ComponentUiBean componentUIBean = new ComponentUiBean();
    componentUIBean.setKey(includeUI.getKeyString());
    if (includeUI.useDefaultLibelle()) {
    componentUIBean.setOptions("defaultlib");
    } else {
    componentUIBean.setOptions("definedlib");
    //@todo componentUIBean.setLibelleBeanArray(includeUI.getLibelleByLangMap().toLibelleBeanArray());
    }
    if (includeUI instanceof HtmlInputIncludeUI) {
    HtmlInputBean htmlInputBean = ((HtmlInputIncludeUI) includeUI).getHtmlInputBean();
    if (!CorpusUIUtils.equalstoDefaultIndexation(htmlInputBean)) componentUIBean.setHtmlInputBean(htmlInputBean);
    }
    return componentUIBean;
    }*/
    /*public static GroupTemplateBean[] toGroupTemplateBeanArray(SubsetKeyGroupTemplates templates) {
    int size = templates.size();
    GroupTemplateBean[]  groupTemplateBeanArray = new GroupTemplateBean[size];
    for (int i = 0; i < size;i++) {
    String groupName = templates.getGroupName(i);
    GroupTemplateBean bean = new GroupTemplateBean();
    bean.setGroupName(groupName);
    LibelleByLangMap map = templates.getLibelleByLangMap(groupName);
    if (map != null) {
    bean.setLibelleBeanArray(map.toLibelleBeanArray());
    }
    groupTemplateBeanArray[i] = bean;
    }
    return groupTemplateBeanArray;
    }*/
}
