/* BdfServer_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.email.datasource.StreamDataSource;
import fr.exemole.bdfserver.email.datasource.VersionDataSource;
import fr.exemole.bdfserver.email.datasource.XslDataSource;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.Collection;
import java.util.List;
import javax.activation.DataSource;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.util.email.Email;
import net.mapeadores.util.email.EmailException;
import net.mapeadores.util.email.MultiPartEmail;
import net.mapeadores.util.email.tools.EmailUtils;


/**
 *
 * @author Vincent Calame
 */
public class SendEngine {

    private SmtpManager smtpManager;
    private BdfServer bdfServer;
    private BdfUser bdfUser;

    public SendEngine(SmtpManager smtpManager, BdfServer bdfServer, BdfUser bdfUser) {
        this.smtpManager = smtpManager;
        this.bdfServer = bdfServer;
        this.bdfUser = bdfUser;
    }

    public SendEngine(BdfContext bdfContext, BdfUser bdfUser) {
        this.smtpManager = (SmtpManager) bdfContext.getContextObject(BdfContextConstants.SMTPMANAGER_NAME);
        this.bdfServer = bdfContext.getBdfServer();
        this.bdfUser = bdfUser;
    }

    public MultiPartEmail createMultiPartEmail() {
        return EmailUtils.newMultipartEmail(smtpManager.getSmtpParameters());
    }

    public SendReport sendEmail(EnvoiParameters envoiParameters) {
        MultiPartEmail email = createMultiPartEmail();
        try {
            initEmail(email, envoiParameters);
        } catch (EmailException emailException) {
            return SendReport.emailBuildErrorReport(emailException.getMessage());
        }
        try {
            AbregeFiche[] abregeFicheArray = envoiParameters.getAbregeFicheArray();
            switch (envoiParameters.getEnvoiType()) {
                case EnvoiParameters.ENVOI_COMPILATION:
                    DataSource compilationDataSource = XslDataSource.newCompilationInstance(bdfServer, bdfUser, bdfUser.getAbregeFicheSortedList());
                    email = email.attach(compilationDataSource, compilationDataSource.getName(), "");
                    break;
                case EnvoiParameters.ENVOI_SELECTION:
                    if (abregeFicheArray != null) {
                        int length = abregeFicheArray.length;
                        for (int i = 0; i < length; i++) {
                            DataSource ficheDataSource = XslDataSource.newFicheInstance(bdfServer, bdfUser, abregeFicheArray[i]);
                            email = email.attach(ficheDataSource, ficheDataSource.getName(), "");
                        }
                    }
                    break;
                case EnvoiParameters.ENVOI_FICHEUNIQUE:
                    AbregeFiche abregeFiche = abregeFicheArray[0];
                    ficheAttachements(email, abregeFiche, envoiParameters.getFicheAttachmentParameters());
                    break;

            }
        } catch (EmailException emailException) {
            return SendReport.emailBuildErrorReport(emailException.getMessage());
        }
        try {
            email.send();
            return SendReport.okReport();
        } catch (EmailException emailException) {
            return SendReport.sendErrorReport(emailException.getMessage());
        }
    }

    private void ficheAttachements(MultiPartEmail email, AbregeFiche abregeFiche, FicheAttachmentParameters ficheAttachmentParameters) throws EmailException {
        if (ficheAttachmentParameters.isWithHtml()) {
            DataSource ficheDataSource = XslDataSource.newFicheInstance(bdfServer, bdfUser, abregeFiche);
            email = email.attach(ficheDataSource, ficheDataSource.getName(), "");
        }
        if (ficheAttachmentParameters.isWithOdt()) {
            DataSource odtFicheDataSource = StreamDataSource.newFicheInstance(bdfServer, bdfUser, abregeFiche, "odt");
            email = email.attach(odtFicheDataSource, odtFicheDataSource.getName(), "");
        }
        Collection<Version> versionCollection = ficheAttachmentParameters.getVersionCollection();
        for (Version version : versionCollection) {
            VersionDataSource dataSource = VersionDataSource.newInstance(bdfServer, version);
            email = email.attach(dataSource, dataSource.getName(), "");
        }
    }

    private void initEmail(Email email, EnvoiParameters envoiParameters) throws EmailException {
        CourrielBean fromCourrielBean = envoiParameters.getFromCourrielBean();
        String fromNomcomplet = fromCourrielBean.getNomComplet();
        if (envoiParameters.isWithRedacteurBcc()) {
            email.addBcc(fromCourrielBean.getAddrSpec(), fromNomcomplet);
        }
        email.setFrom(fromCourrielBean.getAddrSpec(), fromNomcomplet);
        email.setSubject(envoiParameters.getSubject());
        email.setMsg(envoiParameters.getMessage());
        List<ValidAddress> validAddressList = envoiParameters.getValidAddressList();
        for (ValidAddress validAddress : validAddressList) {
            CourrielBean courrielBean = validAddress.getCourrielBean();
            String nomcomplet = courrielBean.getNomComplet();
            if (nomcomplet == null) {
                email.addTo(courrielBean.getAddrSpec());
            } else {
                email.addTo(courrielBean.getAddrSpec(), nomcomplet);
            }
        }
    }

}
