/* BdfServer_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email.datasource;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationConstants;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.managers.transformation.TransformerParameters;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.activation.DataSource;
import javax.xml.transform.TransformerException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.ExtractionContextUtils;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtractionSource;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class StreamDataSource implements DataSource {

    private ExtractionContext extractionContext;
    private ExtractionDef extractionDef;
    private ExtractionSource extractionSource;
    private String contentType;
    private String name;
    private StreamTransformer streamTransformer;
    private TransformerParameters transformerParameters;

    private StreamDataSource() {
    }

    public String getContentType() {
        return contentType;
    }

    public String getName() {
        return name;
    }

    public InputStream getInputStream() throws IOException {
        try {
            ByteArrayOutputStream byteoutput = new ByteArrayOutputStream();
            streamTransformer.transform(ExtractionXMLPart.toStreamSource(extractionContext, extractionDef, extractionSource), byteoutput, transformerParameters.toMap());
            return new ByteArrayInputStream(byteoutput.toByteArray());
        } catch (TransformerException te) {
            throw new NestedTransformerException(te);
        }
    }

    public OutputStream getOutputStream() throws IOException {
        throw new IOException("no editable source");
    }

    private void initTransformer(BdfServer bdfServer, BdfUser bdfUser, TransformationKey transformationKey, String baseName, String extension, Object dynamicObject) {
        extractionContext = ExtractionContextUtils.newExtractionContext(bdfServer.getBasedefiches(), LocalisationLists.newInstance(bdfUser.getLocalisationInfo()));
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        StreamTransformerKey streamTransformerKey = bdfUser.getStreamTransformerKey(transformationKey, extension);
        if (streamTransformerKey == null) {
            streamTransformerKey = StreamTransformerKey.getDefaultStreamTransformerKey(transformationKey, extension);
        }
        extractionDef = transformationManager.getExtractionDef(streamTransformerKey);
        extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        streamTransformer = transformationManager.getStreamTransformer(streamTransformerKey);
        contentType = streamTransformer.getMimeType();
        name = baseName + "." + extension;
        transformerParameters = new TransformerParameters();
        transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
        transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, bdfUser.getBaseDeFichesURL());
    }

    public static StreamDataSource newFicheInstance(BdfServer bdfServer, BdfUser bdfUser, AbregeFiche abregeFiche, String extension) {
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        String baseName = corpusKey.getSubsetName() + "-" + String.valueOf(abregeFiche.getIdcorpus());
        TransformationKey transformationKey = new TransformationKey(corpusKey);
        StreamDataSource streamDataSource = new StreamDataSource();
        streamDataSource.initTransformer(bdfServer, bdfUser, transformationKey, baseName, extension, abregeFiche);
        return streamDataSource;
    }

}
