/* BdfServer_Email - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.email.datasource;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.TransformationConstants;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.managers.transformation.TransformerParameters;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.activation.DataSource;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.ExtractionContextUtils;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtractionSource;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.localisation.LocalisationLists;


/**
 *
 * @author Vincent Calame
 */
public class XslDataSource implements DataSource {

    private ExtractionContext extractionContext;
    private ExtractionDef extractionDef;
    private ExtractionSource extractionSource;
    private String contentType;
    private Transformer transformer;
    private String name;

    private XslDataSource() {
    }

    private void initTransformer(BdfServer bdfServer, BdfUser bdfUser, TransformationKey transformationKey, String baseName, ExtractionContext extractionContext, Object dynamicObject) {
        this.extractionContext = extractionContext;
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        extractionDef = transformationManager.getExtractionDef(xslTransformerKey);
        extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        transformer = transformationManager.getXslTransformer(xslTransformerKey);
        TransformerParameters transformerParameters = new TransformerParameters();
        transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
        transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, bdfUser.getBaseDeFichesURL());
        transformerParameters.initParameters(transformer);
        String method = transformer.getOutputProperty("method");
        if (method == null) {
            method = "xml";
        }
        String encoding = transformer.getOutputProperty("encoding");
        if (encoding == null) {
            encoding = "UTF-8";
        }
        if (method.equals("xml")) {
            contentType = "text/xml;charset=" + encoding;
            name = baseName + ".xml";
        } else if (method.equals("html")) {
            contentType = "text/html;charset=" + encoding;
            name = baseName + ".html";
        } else {
            contentType = "text/plain;charset=" + encoding;
            name = baseName + ".txt";
        }

    }

    public String getContentType() {
        return contentType;
    }

    public String getName() {
        return name;
    }

    public InputStream getInputStream() throws IOException {
        try {
            ByteArrayOutputStream byteoutput = new ByteArrayOutputStream();
            transformer.transform(ExtractionXMLPart.toStreamSource(extractionContext, extractionDef, extractionSource), new StreamResult(byteoutput));
            return new ByteArrayInputStream(byteoutput.toByteArray());
        } catch (TransformerException te) {
            throw new NestedTransformerException(te);
        }
    }

    public OutputStream getOutputStream() throws IOException {
        throw new IOException("no editable source");
    }

    public static XslDataSource newFicheInstance(BdfServer bdfServer, BdfUser bdfUser, AbregeFiche abregeFiche) {
        SubsetKey corpusKey = abregeFiche.getCorpus().getSubsetKey();
        String baseName = corpusKey.getSubsetName() + "-" + String.valueOf(abregeFiche.getIdcorpus());
        TransformationKey transformationKey = new TransformationKey(corpusKey);
        ExtractionContext extractionContext = ExtractionContextUtils.newExtractionContext(bdfServer.getBasedefiches(), LocalisationLists.newInstance(bdfUser.getLocalisationInfo()));
        XslDataSource xslDataSource = new XslDataSource();
        xslDataSource.initTransformer(bdfServer, bdfUser, transformationKey, baseName, extractionContext, abregeFiche);
        return xslDataSource;
    }

    public static XslDataSource newCompilationInstance(BdfServer bdfServer, BdfUser bdfUser, AbregeFicheSortedList abregeFicheSortedList) {
        TransformationKey transformationKey = TransformationKey.COMPILATION_TRANSFORMATIONKEY;
        String baseName = "compilation";
        ExtractionContext extractionContext = ExtractionContextUtils.newExtractionContext(bdfServer.getBasedefiches(), LocalisationLists.newInstance(bdfUser.getLocalisationInfo()), null, abregeFicheSortedList);
        XslDataSource xslDataSource = new XslDataSource();
        xslDataSource.initTransformer(bdfServer, bdfUser, transformationKey, baseName, extractionContext, null);
        return xslDataSource;
    }

}
