/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.get.instructions.bdfcontent.AdminInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.BalayagesInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.CroisementsInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.FichesInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.IllustrationsInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.IsoInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.MotsclesInstruction;
import fr.exemole.bdfserver.get.instructions.bdfcontent.RedacteursInstruction;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheInstruction;
import fr.exemole.bdfserver.get.instructions.stream.StreamInstruction;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.instruction.InstructionUtils;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class GetInstructionFactory {

    private GetInstructionFactory() {
    }

    public static BdfInstruction getBdfInstruction(BdfContext bdfContext, BdfRequestMap requestMap) {
        String type = requestMap.getParameter("type");
        if (type == null) {
            return new ErrorBdfInstruction(bdfContext, requestMap, "global_err_parameter_mandatory", "type");
        } else if (type.equals("pioche")) {
            return new PiocheInstruction(bdfContext, requestMap);
        } else {
            Object[] valueArray = {"type", type};
            return new ErrorBdfInstruction(bdfContext, requestMap, "global_err_parameter_wrong", valueArray);
        }

    }

    public static BdfInstruction getBdfInstruction(BdfContext bdfContext, BdfRequestMap requestMap, String dir, String filePath) {
        short streamDir = StreamInstruction.testDir(dir);
        if (streamDir != 0) {
            if ((filePath.length() == 0) || (filePath.endsWith("/"))) {
                return new ErrorBdfInstruction(bdfContext, requestMap, "global_err_dir_list", dir + '/' + filePath);
            }
            return new StreamInstruction(bdfContext, streamDir, filePath);
        }
        if (dir.equals("balayages")) {
            return new BalayagesInstruction(bdfContext, filePath);
        } else if (dir.equals("iso")) {
            return new IsoInstruction(bdfContext, filePath);
        } else if (dir.equals("redacteurs")) {
            return new RedacteursInstruction(bdfContext, filePath);
        } else if (dir.equals("admin")) {
            return new AdminInstruction(bdfContext, filePath);
        } else if (dir.equals("fiches")) {
            return new FichesInstruction(bdfContext, filePath);
        } else if (dir.equals("motscles")) {
            return new MotsclesInstruction(bdfContext, filePath);
        } else if (dir.equals("croisements")) {
            return new CroisementsInstruction(bdfContext, filePath);
        } else if (dir.equals("illustrations")) {
            return new IllustrationsInstruction(bdfContext, filePath);
        } else {
            return null;
        }
    }


    private static class ErrorBdfInstruction implements BdfInstruction {

        private BdfRequestMap requestMap;
        private BdfServer bdfServer;
        private ResourceLocalisationProvider resourceLocalisationProvider;
        private String errorKey;
        private Object errorValue;

        private ErrorBdfInstruction(BdfContext bdfContext, BdfRequestMap requestMap, String errorKey, Object errorValue) {
            this.requestMap = requestMap;
            this.bdfServer = bdfContext.getBdfServer();
            this.resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
            this.errorKey = errorKey;
            this.errorValue = errorValue;
        }

        public short getBdfUserNeed() {
            return BdfInstructionConstants.IF_ANY_BDFUSER;
        }

        public ResponseHandler runInstruction(BdfUser bdfUser) {
            LocalisationInfo localisationInfo = null;
            if (bdfUser != null) {
                localisationInfo = bdfUser.getLocalisationInfo();
            } else {
                localisationInfo = InstructionUtils.getLocalisationInfo(requestMap, bdfServer);
            }
            ResourceLocalisation resourceLocalisation = resourceLocalisationProvider.getResourceLocalisation(localisationInfo.getFormatLocale());

            return CommandMessageResponseHandler.getInstance(CommandMessage.error(errorKey, errorValue), resourceLocalisation, requestMap);
        }

    }

}
