/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;


/**
 *
 * @author Vincent Calame
 */
public class CorpusThesaurusHtmlProducer extends BdfHtmlProducer {

    private Corpus corpus;
    private InfoMotcle[] firstlevelInfos;
    private Croisement croisement;

    public CorpusThesaurusHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, Corpus corpus, Thesaurus thesaurus) {
        init(bdfContext, bdfUser);
        this.corpus = corpus;
        croisement = bdfServer.getBasedefiches().getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
        MotcleList firstlevel = thesaurus.getFirstLevelList();
        firstlevelInfos = new InfoMotcle[firstlevel.getMotcleCount()];
        for (int i = 0; i < firstlevel.getMotcleCount(); i++) {
            firstlevelInfos[i] = new InfoMotcle(firstlevel.getMotcle(i));
        }
        setReversePath("../..");
        setCssThemeFile("affichage.css");
    }

    public void printHtml() {
        start();
        startTable();
        printLigneEnteteTexte();
        AbregeFicheSortedList abregeFicheSortedList = bdfUser.getAbregeFicheSortedList();
        SubsetItemList abregeFicheList = abregeFicheSortedList.getAbregeFicheListByCorpus(corpus);
        int ficheCount = abregeFicheList.getSubsetItemCount();
        for (int i = 0; i < ficheCount; i++) {
            AbregeFiche abregeFiche = (AbregeFiche) abregeFicheList.getSubsetItem(i);
            printLigne(abregeFiche);
        }
        printLigneEnteteCode();
        endTable();
        end();
    }

    private void printLigneEnteteTexte() {
        startRow();
        startCell();
        append("id");
        endCell();
        appendEmptyCell();
        for (int i = 0; i < firstlevelInfos.length; i++) {
            firstlevelInfos[i].printLigneEnteteTexte();
        }
        endRow();
    }

    private void printLigneEnteteCode() {
        startRow();
        appendEmptyCell();
        appendEmptyCell();
        for (int i = 0; i < firstlevelInfos.length; i++) {
            firstlevelInfos[i].printLigneEnteteCode();
        }
        endRow();
    }

    private void printLigne(AbregeFiche abrege) {
        Map<Integer, Integer> mapitems = new HashMap<Integer, Integer>();
        CroisementItem croisementItem = croisement.getCroisementItem(corpus.getSubsetKey(), abrege.getTicket());
        int ciCount = croisementItem.getCount();
        for (int i = 0; i < ciCount; i++) {
            int ticket = croisementItem.getTicket(i);
            int poids = croisementItem.getPoids(i);
            mapitems.put(ticket, poids);
        }
        startRow();
        startCell();
        append(String.valueOf(abrege.getIdcorpus()));
        endCell();
        startCell();
        escape(abrege.getTitre());
        endCell();
        for (int i = 0; i < firstlevelInfos.length; i++) {
            firstlevelInfos[i].printLigne(mapitems);
        }
        endRow();
    }


    private class InfoMotcle {

        private Motcle motcle;
        private InfoMotcle[] infos;
        private int taille;

        private InfoMotcle(Motcle motcle) {
            this.motcle = motcle;
            MotcleList childList = motcle.getChildList();
            taille = childList.getMotcleCount();
            infos = new InfoMotcle[taille];
            for (int i = 0; i < taille; i++) {
                infos[i] = new InfoMotcle(childList.getMotcle(i));
            }
        }

        private void printLigneEnteteTexte() {
            startCell();
            String idalpha = motcle.getIdalpha();
            escape(motcle.getLibelleString(workingLangInteger, true));
            endCell();
            for (int i = 0; i < taille; i++) {
                infos[i].printLigneEnteteTexte();
            }
        }

        private void printLigneEnteteCode() {
            startCell();
            String idalpha = motcle.getIdalpha();
            if (idalpha != null) {
                append(idalpha);
            } else {
                append(String.valueOf(motcle.getIdths()));
            }
            endCell();
            for (int i = 0; i < taille; i++) {
                infos[i].printLigneEnteteCode();
            }
        }

        private void printLigne(Map<Integer, Integer> mapitems) {
            startCell();
            Integer poids = mapitems.get(motcle.getTicket());
            if (poids != null) {
                append(poids.toString());
            }
            endCell();
            for (int i = 0; i < taille; i++) {
                infos[i].printLigne(mapitems);
            }
        }

    }

}
