/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import net.mapeadores.basedefiches.SubsetItemList;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.fiche.FicheItem;
import net.mapeadores.basedefiches.corpus.fiche.Geopoint;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.pointeurs.PointeurUtils;
import net.mapeadores.basedefiches.selection.AbregeFicheSortedList;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.html.GabaritHtmlProducer;
import net.mapeadores.util.html.HTMLUtils;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class GeolocalisationHtmlProducer extends GabaritHtmlProducer {

    private static String gabaritString;
    private InternalValueResolver valueResolver = new InternalValueResolver();
    private ResourceLocalisation resourceLocalisation;
    private BdfUser bdfUser;

    static {
        try {
            InputStream inputStream = GeolocalisationHtmlProducer.class.getResourceAsStream("geolocalisation.html");
            gabaritString = IOUtils.toString(new InputStreamReader(inputStream, "UTF-8"));
            inputStream.close();
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }

    }

    public GeolocalisationHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(gabaritString, valueResolver);
        this.bdfUser = bdfUser;
        ResourceLocalisationProvider resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
        resourceLocalisation = resourceLocalisationProvider.getResourceLocalisation(bdfUser.getFormatLocale());
    }

    private String getEscapedLocalisation(String key, Object value) {
        return HTMLUtils.escapeHTML(resourceLocalisation.toString(key, value));
    }


    private class InternalValueResolver implements ValueResolver {

        public String getValue(AccoladeArgument accoladeArgument) {
            String key = accoladeArgument.getName();
            if (key.equals("TITRE")) {
                return getEscapedLocalisation("listefiches_geolocalisation", null);
            } else if (key.equals("PATH")) {
                return "../";
            } else if (key.equals("CREATION_MARQUEURS")) {
                StringBuffer buf = new StringBuffer();
                AbregeFicheSortedList abregeFicheSortedList = bdfUser.getAbregeFicheSortedList();
                int corpusCount = abregeFicheSortedList.getCorpusCount();
                for (int i = 0; i < corpusCount; i++) {
                    Corpus corpus = abregeFicheSortedList.getCorpus(i);
                    CorpusField geolocalisationField = corpus.getCorpusMetadata().getGeolocalisationField();
                    if (geolocalisationField != null) {
                        SubsetItemList abregeFicheList = abregeFicheSortedList.getCorpusAbregeFicheList(i);
                        int ficheCount = abregeFicheList.getSubsetItemCount();
                        for (int j = 0; j < ficheCount; j++) {
                            AbregeFiche abrege = (AbregeFiche) abregeFicheList.getSubsetItem(j);
                            FicheItem ficheItem = (FicheItem) PointeurUtils.getValue(abrege, geolocalisationField.getFieldKey());
                            if ((ficheItem != null) && (ficheItem instanceof Geopoint)) {
                                Geopoint geopoint = (Geopoint) ficheItem;
                                buf.append("GEO_CreateMarqueur(");
                                buf.append(geopoint.getLatitude().toString());
                                buf.append(",");
                                buf.append(geopoint.getLongitude().toString());
                                buf.append(",\"");
                                StringUtils.escapeDoubleQuote(CorpusMetadataUtils.getFicheIntituleString(abrege, bdfUser.getLocalisationInfo()), buf);
                                buf.append("\");\n");
                            }
                        }
                    }
                }
                return buf.toString();
            } else {
                return getEscapedLocalisation(key, null);
            }
        }

    }

}
