/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.SphereUtils;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.html.HtmlPrinterUtils;


/**
 *
 * @author Vincent Calame
 */
public class GlobalStatsHtmlProducer extends BdfHtmlProducer {

    public GlobalStatsHtmlProducer(BdfContext bdfContext, BdfUser bdfUser) {
        init(bdfContext, bdfUser);
        setReversePath("..");
        setCssThemeFile("affichage.css");
        localizeTitle("admin_title_globalstats", null);
    }

    public void printHtml() {
        UIManager uiManager = bdfServer.getUIManager();
        start();
        appendH1("admin_title_globalstats");
        printSubsetStats(uiManager.getCorpusSubsetKeyTree(), "global_list_corpus", 1);
        printSubsetStats(uiManager.getThesaurusSubsetKeyTree(), "global_list_thesaurus", 2);
        printSubsetStats(uiManager.getSphereSubsetKeyTree(), "global_list_sphere", 3);
        end();
    }

    private void printSubsetStats(SubsetGroupList subsetGroupList, String titleKey, int type) {
        startH2();
        localize(titleKey);
        HtmlPrinterUtils.printColon(this);
        append(String.valueOf(basedefiches.getCorpusCount()));
        endH2();
        startUl();
        int count = subsetGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup subsetKeyGroup = subsetGroupList.getSubsetGroup(i);
            boolean isTemplate = !subsetKeyGroup.isNoTemplateGroup();
            if (isTemplate) {
                startLi();
                startH3();
                escape(subsetKeyGroup.getLibelleString(workingLangInteger));
                endH3();
                startUl();
            }
            for (SubsetKey subsetKey : subsetKeyGroup.getSubsetKeyList()) {
                startLi();
                switch (type) {
                    case 1:
                        printCorpus(subsetKey);
                        break;
                    case 2:
                        printThesaurus(subsetKey);
                        break;
                    case 3:
                        printSphere(subsetKey);
                        break;
                }
                endLi();
            }
            if (isTemplate) {
                endUl();
                endLi();
            }
        }
        endUl();
    }

    private void printCorpus(SubsetKey subsetKey) {
        Corpus corpus = (Corpus) basedefiches.getSubset(subsetKey);
        escape(CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger));
        HtmlPrinterUtils.printColon(this);
        append(String.valueOf(corpus.size()));
    }

    private void printThesaurus(SubsetKey subsetKey) {
        Thesaurus thesaurus = (Thesaurus) basedefiches.getSubset(subsetKey);
        escape(ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        HtmlPrinterUtils.printColon(this);
        append(String.valueOf(thesaurus.size()));
    }

    private void printSphere(SubsetKey subsetKey) {
        Sphere sphere = (Sphere) basedefiches.getSubset(subsetKey);
        escape(SphereUtils.getSphereIntitule(sphere, workingLangInteger));
        HtmlPrinterUtils.printColon(this);
        append(String.valueOf(sphere.size()));
    }

}
