/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.pioche;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheConstants;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheParameters;
import fr.exemole.bdfserver.get.jsonproducers.pioche.PiocheJsonConstants;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.catalogs.PiocheCatalog;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.List;
import net.mapeadores.basedefiches.Subset;


/**
 *
 * @author Vincent Calame
 */
public class FichePiocheHtmlProducer extends BdfHtmlProducer implements PiocheHtmlConstants, PiocheConstants {

    private PiocheParameters piocheParameters;
    private List<Subset> corpusList;

    public FichePiocheHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, PiocheParameters piocheParameters) {
        init(bdfContext, bdfUser);
        this.piocheParameters = piocheParameters;
        corpusList = piocheParameters.getSubsetList();
        setCssThemeFile("pioche.css");
        addJqueryExtensions("form");
        addJavascriptFiles(PiocheCatalog.PIOCHE_JAVASCRIPT_CORE);
        addJavascriptFiles(PiocheCatalog.PIOCHE_JAVASCRIPT_RECHERCHE);
        localizeTitle("pioche_title_fiche", null);
    }

    public void printHtml() {
        String wanted = IDALPHAKEY_INTKEY_WANTED;
        if ((corpusList != null) && (corpusList.size() == 1)) {
            wanted = INTKEY_WANTED;
        }
        int total = getTotal();
        start();
        PiocheHtmlUtils.printScriptVar(this, piocheParameters, total, " – ", wanted);
        PiocheHtmlUtils.printRechercheDiv(this, piocheParameters, PiocheJsonConstants.FICHE_JSON, PiocheHtmlUtils.getSubsetsString(corpusList), null);
        PiocheHtmlUtils.printSelectZone(this, piocheParameters, total);
        PiocheHtmlUtils.printButtonDiv(this);
        end();
    }

    private int getTotal() {
        int count = 0;
        if (corpusList == null) {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                count = count + basedefiches.getCorpus(i).size();
            }
        } else {
            for (Subset subset : corpusList) {
                count = count + subset.size();
            }
        }
        return count;
    }

}
