/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.pioche;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheConstants;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheParameters;
import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.html.GabaritHtmlProducer;
import net.mapeadores.util.html.HTMLUtils;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.text.AccoladeArgument;
import net.mapeadores.util.text.ValueResolver;


/**
 *
 * @author Vincent Calame
 */
public class GeocodagePiocheHtmlProducer extends GabaritHtmlProducer implements PiocheHtmlConstants, PiocheConstants {

    private static String gabaritString;
    private PiocheParameters piocheParameters;
    private DegreDecimal latDecimal;
    private DegreDecimal lonDecimal;
    private String appelant;
    private CorpusField corpusField;
    private InternalValueResolver valueResolver = new InternalValueResolver();
    private ResourceLocalisation resourceLocalisation;

    static {
        try {
            InputStream inputStream = GeocodagePiocheHtmlProducer.class.getResourceAsStream("geocodage.html");
            gabaritString = IOUtils.toString(new InputStreamReader(inputStream, "UTF-8"));
            inputStream.close();
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }

    }

    public GeocodagePiocheHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, PiocheParameters piocheParameters) {
        init(gabaritString, valueResolver);
        ResourceLocalisationProvider resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
        resourceLocalisation = resourceLocalisationProvider.getResourceLocalisation(bdfUser.getFormatLocale());
        this.piocheParameters = piocheParameters;
        appelant = piocheParameters.getAppelant();
        latDecimal = (DegreDecimal) piocheParameters.getParameter(LAT_PARAMNAME);
        lonDecimal = (DegreDecimal) piocheParameters.getParameter(LONG_PARAMNAME);
        corpusField = (CorpusField) piocheParameters.getParameter(FIELD_PARAMNAME);
    }

    private String getEscapedLocalisation(String key, Object value) {
        return HTMLUtils.escapeHTML(resourceLocalisation.toString(key, value));
    }


    private class InternalValueResolver implements ValueResolver {

        public String getValue(AccoladeArgument accoladeArgument) {
            String key = accoladeArgument.getName();
            if (key.equals("TITRE")) {
                return getEscapedLocalisation("pioche_title_geocodage", null);
            } else if (key.equals("APPELANT")) {
                return appelant;
            } else if (key.equals("LAT")) {
                if (latDecimal != null) {
                    return latDecimal.toString();
                } else {
                    return "-999";
                }
            } else if (key.equals("LON")) {
                if (lonDecimal != null) {
                    return lonDecimal.toString();
                } else {
                    return "-999";
                }
            } else if (key.equals("IDPREFIX")) {
                return FicheFormCatalog.ID_FF_CHAMP_PREFIX;
            } else if (key.equals("ICONPATH")) {
                return "theme/icons/48x16/";
            } else {
                return getEscapedLocalisation(key, null);
            }
        }

    }

}
