/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.pioche;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheConstants;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheParameters;
import fr.exemole.bdfserver.get.jsonproducers.pioche.PiocheJsonConstants;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.catalogs.PiocheCatalog;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;


/**
 *
 * @author Vincent Calame
 */
public class MotclePiocheHtmlProducer extends BdfHtmlProducer implements PiocheHtmlConstants, PiocheConstants {

    private PiocheParameters piocheParameters;
    private List<Subset> thesaurusList;
    private String param;
    private boolean wantedIdths = false;

    public MotclePiocheHtmlProducer(BdfContext bdfContext, BdfUser bdfUser, PiocheParameters piocheParameters) {
        init(bdfContext, bdfUser);
        this.piocheParameters = piocheParameters;
        thesaurusList = piocheParameters.getSubsetList();
        param = (String) piocheParameters.getParam();
        if ((param != null) && (param.equals(IDTHS_PARAMVALUE))) {
            wantedIdths = true;
        }
        setCssThemeFile("pioche.css");
        addJqueryExtensions("form");
        addJavascriptFiles(PiocheCatalog.PIOCHE_JAVASCRIPT_CORE);
        addJavascriptFiles(PiocheCatalog.PIOCHE_JAVASCRIPT_RECHERCHE);
        if ((thesaurusList != null) && (thesaurusList.size() == 1)) {
            Thesaurus thesaurus = (Thesaurus) thesaurusList.get(0);
            localizeTitle("pioche_title_motcle_one", ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger));
        } else {
            localizeTitle("pioche_title_motcle", null);
        }
    }

    public void printHtml() {
        int total = getTotal();
        start();
        PiocheHtmlUtils.printScriptVar(this, piocheParameters, total, " – ", (wantedIdths) ? IDALPHAKEY_INTKEY_WANTED : IDALPHAKEY_TEXT_WANTED);
        PiocheHtmlUtils.printRechercheDiv(this, piocheParameters, PiocheJsonConstants.MOTCLE_JSON, PiocheHtmlUtils.getSubsetsString(thesaurusList), null);
        PiocheHtmlUtils.printSelectZone(this, piocheParameters, total);
        PiocheHtmlUtils.printButtonDiv(this);
        end();
    }

    private int getTotal() {
        int count = 0;
        if (thesaurusList == null) {
            int thesaurusCount = basedefiches.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                count = count + basedefiches.getThesaurus(i).size();
            }
        } else {
            for (Subset subset : thesaurusList) {
                count = count + subset.size();
            }
        }
        return count;
    }

}
