/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.htmlproducers.pioche;

import fr.exemole.bdfserver.get.instructions.pioche.PiocheConstants;
import fr.exemole.bdfserver.get.instructions.pioche.PiocheParameters;
import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.PiocheCatalog;
import java.util.List;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.servlets.CommandRequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class PiocheHtmlUtils implements PiocheHtmlConstants, PiocheConstants {

    public static String getSubsetsString(List<Subset> subsetList) {
        if (subsetList == null) {
            return "";
        }
        StringBuilder buf = new StringBuilder();
        int size = subsetList.size();
        for (int i = 0; i < size; i++) {
            Subset subset = subsetList.get(i);
            if (i > 0) {
                buf.append(",");
            }
            buf.append(subset.getSubsetKey().getSubsetName());
        }
        return buf.toString();
    }

    public static void printAppelant(HtmlPrinter htmlPrinter, String appelant) {
        htmlPrinter.append("PCH_Core.setAppelantId(\"");
        htmlPrinter.append(appelant);
        htmlPrinter.append("\");");
    }

    public static void printLimite(HtmlPrinter htmlPrinter, int limite) {
        htmlPrinter.append("PCH_Core.setLimite(");
        htmlPrinter.append(String.valueOf(limite));
        htmlPrinter.append(");");
    }

    public static void printTotal(HtmlPrinter htmlPrinter, int total) {
        htmlPrinter.append("PCH_Core.setTotal(");
        htmlPrinter.append(String.valueOf(total));
        htmlPrinter.append(");");
    }

    public static void printLoc(HtmlPrinter htmlPrinter, String key, Object value) {
        htmlPrinter.append("PCH_Core.setLoc(\"");
        htmlPrinter.append(key);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(StringUtils.escapeDoubleQuote(htmlPrinter.getLocalization(key, value)));
        htmlPrinter.append("\");");
    }

    public static void printSeparator(HtmlPrinter htmlPrinter, String value) {
        htmlPrinter.append("PCH_Core.setSeparator(\"");
        htmlPrinter.append(StringUtils.escapeDoubleQuote(value));
        htmlPrinter.append("\");");
    }

    public static void printWanted(HtmlPrinter htmlPrinter, String value) {
        htmlPrinter.append("PCH_Core.setWanted(\"");
        htmlPrinter.append(StringUtils.escapeDoubleQuote(value));
        htmlPrinter.append("\");");
    }

    public static void printParam(HtmlPrinter htmlPrinter, String key, String value) {
        htmlPrinter.append("PCH_Core.setParam(\"");
        htmlPrinter.append(key);
        htmlPrinter.append("\",\"");
        htmlPrinter.append(StringUtils.escapeDoubleQuote(value));
        htmlPrinter.append("\");");
    }

    public static void printScriptVar(HtmlPrinter htmlPrinter, PiocheParameters piocheParameters, int total, String separator, String wanted) {
        htmlPrinter.startScript(null);
        printAppelant(htmlPrinter, piocheParameters.getAppelant());
        printLimite(htmlPrinter, piocheParameters.getLimite());
        printLoc(htmlPrinter, "pioche_info_aucunresultat", null);
        printSeparator(htmlPrinter, separator);
        printTotal(htmlPrinter, total);
        printWanted(htmlPrinter, wanted);
        htmlPrinter.endScript();
    }

    public static void printRechercheDiv(BdfHtmlProducer bdfHtmlProducer, PiocheParameters piocheParameters, String json, String subsetsString, String param) {
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId("ID_PIOCHE_FORMDIV");
        bdfHtmlProducer.startDiv(divAttributes);
        HTMLAttributes formAttributes = new HTMLAttributes();
        formAttributes.setAction("Get");
        formAttributes.setMethod("get");
        bdfHtmlProducer.startForm(formAttributes);
        bdfHtmlProducer.appendHiddenInput("type", "pioche");
        bdfHtmlProducer.appendHiddenInput(CommandRequestMap.JSON_PARAMETER, json);
        bdfHtmlProducer.appendHiddenInput(SUBSETS_PARAMNAME, subsetsString);
        if (param != null) {
            bdfHtmlProducer.appendHiddenInput(PARAM_PARAMNAME, param);
        }
        String motcleSelectString = piocheParameters.getMotcleSelectOptionsString();
        if (motcleSelectString != null) {
            bdfHtmlProducer.appendHiddenInput(MOTCLESELECTOPTIONS_PARAMNAME, motcleSelectString);
        }
        bdfHtmlProducer.startP();
        BdfHtmlUtils.printLibColon(bdfHtmlProducer, "pioche_lib_sequence");
        bdfHtmlProducer.appendBr();
        HTMLAttributes rechercheAttributes = new HTMLAttributes();
        rechercheAttributes.setId("ID_PIOCHE_RECHERCHE");
        rechercheAttributes.setSize("35");
        rechercheAttributes.setType("text");
        rechercheAttributes.setName(RECHERCHE_PARAMNAME);
        bdfHtmlProducer.appendInput(rechercheAttributes);
        bdfHtmlProducer.space();
        bdfHtmlProducer.addSubmit("global_submit_search", false);
        bdfHtmlProducer.endP();
        bdfHtmlProducer.endForm();
        bdfHtmlProducer.endDiv();
    }

    public static void printSelectZone(HtmlPrinter htmlPrinter, PiocheParameters piocheParameters, int total) {
        HTMLAttributes totalAttributes = new HTMLAttributes();
        HTMLAttributes selectAttributes = new HTMLAttributes();
        selectAttributes.setStyle("width: 100%;");
        htmlPrinter.startP();
        BdfHtmlUtils.printLibColon(htmlPrinter, "pioche_lib_proposition");
        htmlPrinter.space();
        htmlPrinter.append("[");
        totalAttributes.setId("ID_PIOCHE_PROP_TOTAL");
        htmlPrinter.startSpan(totalAttributes);
        htmlPrinter.append("0");
        htmlPrinter.endSpan();
        htmlPrinter.append("/");
        htmlPrinter.append(String.valueOf(total));
        htmlPrinter.append("]");
        HTMLAttributes ongletAttributes = new HTMLAttributes();
        ongletAttributes.setId("ID_PIOCHE_PROP_ONGLETS");
        htmlPrinter.startSpan(ongletAttributes);
        htmlPrinter.endSpan();
        htmlPrinter.appendBr();
        selectAttributes.setId("ID_PIOCHE_PROP_SELECT");
        selectAttributes.setSize("11");
        htmlPrinter.startSelect(selectAttributes);
        htmlPrinter.endSelect();
        htmlPrinter.endP();
        htmlPrinter.startP();
        BdfHtmlUtils.printLibColon(htmlPrinter, "pioche_lib_selection");
        htmlPrinter.space();
        htmlPrinter.append("[");
        totalAttributes.setId("ID_PIOCHE_SEL_TOTAL");
        htmlPrinter.startSpan(totalAttributes);
        htmlPrinter.append("0");
        htmlPrinter.endSpan();
        htmlPrinter.append("]");
        htmlPrinter.appendBr();
        selectAttributes.setId("ID_PIOCHE_SEL_SELECT");
        selectAttributes.setSize(getSelectSize(piocheParameters));
        htmlPrinter.startSelect(selectAttributes);
        htmlPrinter.endSelect();
        htmlPrinter.endP();
    }

    public static void printButtonDiv(HtmlPrinter htmlPrinter) {
        HTMLAttributes buttonsAttributes = new HTMLAttributes();
        buttonsAttributes.addClass(PiocheCatalog.PCH_BUTTONSDIV);
        htmlPrinter.startDiv(buttonsAttributes);
        HTMLAttributes buttonAttributes = new HTMLAttributes();
        buttonAttributes.setType("button");
        buttonAttributes.setId("ID_PIOCHE_OK");
        buttonAttributes.setValue(htmlPrinter.getLocalization("global_submit_ok"));
        htmlPrinter.appendInput(buttonAttributes);
        htmlPrinter.space();
        buttonAttributes.setId("ID_PIOCHE_CANCEL");
        buttonAttributes.setValue(htmlPrinter.getLocalization("global_submit_cancel"));
        htmlPrinter.appendInput(buttonAttributes);
        htmlPrinter.endDiv();
    }

    private static String getSelectSize(PiocheParameters piocheParameters) {
        int limite = piocheParameters.getLimite();
        if (limite == 1) {
            return "2";
        } else {
            return String.valueOf(limite);
        }
    }

}
