/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.get.htmlproducers.bdfcontent.GlobalStatsHtmlProducer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.UiOdsProducer;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.logging.ProcessLogUtils;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.HtmlResponseHandler;
import net.mapeadores.util.servlets.handlers.SimpleResponseHandler;
import net.mapeadores.util.servlets.handlers.StreamResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class AdminInstruction implements BdfInstruction, BdfInstructionConstants {

    private BdfContext bdfContext;
    private String filePath;

    public AdminInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.filePath = filePath;
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        FilePathInfo filePathInfo = FilePathInfo.getSimpleFile(filePath);
        if (filePathInfo == null) {
            return null;
        }
        String baseName = filePathInfo.getBaseName();
        String extension = filePathInfo.getExtension();
        if (baseName.equals("globalstats")) {
            if (extension.equals("html")) {
                HtmlProducer htmlProducer = new GlobalStatsHtmlProducer(bdfContext, bdfUser);
                return new HtmlResponseHandler(htmlProducer);
            } else {
                return null;
            }
        } else if (baseName.equals("storageprocesslog")) {
            if (extension.equals("txt")) {
                return new SimpleResponseHandler(ProcessLogUtils.toString(bdfContext.getBdfServer().getStorageProcessLog()));
            } else {
                return null;
            }
        } else if (baseName.equals("ui")) {
            if (extension.equals("ods")) {
                StreamProducer streamProducer = new UiOdsProducer(bdfContext.getBdfServer());
                return new StreamResponseHandler(streamProducer);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

}
