/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.get.htmlproducers.bdfcontent.CorpusThesaurusHtmlProducer;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.text.ParseException;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.HtmlResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class CroisementsInstruction implements BdfInstruction, BdfInstructionConstants {

    private BdfContext bdfContext;
    private String filePath;

    public CroisementsInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.filePath = filePath;
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        int idx1 = filePath.indexOf('/');
        if (idx1 == -1) {
            return null;
        }
        String subDir = filePath.substring(0, idx1);
        if (!subDir.equals("corpus_thesaurus")) {
            return null;
        }
        FilePathInfo filePathInfo = FilePathInfo.getSimpleFile(filePath.substring(idx1 + 1));
        if (filePathInfo == null) {
            return null;
        }
        String baseName = filePathInfo.getBaseName();
        String extension = filePathInfo.getExtension();
        if (!extension.equals("html")) {
            return null;
        }
        int idx = baseName.indexOf('_');
        if ((idx < 1) || (idx == (baseName.length() - 1))) {
            return null;
        }
        Corpus corpus = null;
        Thesaurus thesaurus = null;
        Basedefiches basedefiches = bdfContext.getBdfServer().getBasedefiches();
        try {
            SubsetKey corpusKey = SubsetKey.parse(SubsetKey.CATEGORY_CORPUS, baseName.substring(0, idx));
            corpus = (Corpus) basedefiches.getSubset(corpusKey);
            if (corpus == null) {
                return null;
            }
        } catch (ParseException pe) {
            return null;
        }
        try {
            SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, baseName.substring(idx + 1));
            thesaurus = (Thesaurus) basedefiches.getSubset(thesaurusKey);
            if (thesaurus == null) {
                return null;
            }
        } catch (ParseException pe) {
            return null;
        }
        HtmlProducer htmlProducer = new CorpusThesaurusHtmlProducer(bdfContext, bdfUser, corpus, thesaurus);
        return new HtmlResponseHandler(htmlProducer);
    }

}
