/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.IllustrationStreamProducer;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.users.BdfUser;
import java.text.ParseException;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.album.Album;
import net.mapeadores.basedefiches.album.AlbumConstants;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.album.metadata.AlbumDim;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.RedirectResponseHandler;
import net.mapeadores.util.servlets.handlers.StreamResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class IllustrationsInstruction implements BdfInstruction, BdfInstructionConstants {

    private BdfContext bdfContext;
    private BdfServer bdfServer;
    private String filePath;

    public IllustrationsInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.filePath = filePath;
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        int idx1 = filePath.indexOf('/');
        String dimString = null;
        String illustrationName = null;
        if (idx1 != -1) {
            if (filePath.indexOf('/', idx1 + 1) != -1) {
                return null;
            }
            dimString = filePath.substring(0, idx1);
            illustrationName = filePath.substring(idx1 + 1);
        } else {
            illustrationName = filePath;
        }
        int idx2 = illustrationName.indexOf('-');
        if (idx2 == -1) {
            return null;
        }
        int idx3 = illustrationName.indexOf('.');
        if (idx3 == -1) {
            return null;
        }
        if (idx3 < idx2) {
            return null;
        }
        SubsetKey albumKey = null;
        try {
            albumKey = SubsetKey.parse(SubsetKey.CATEGORY_ALBUM, illustrationName.substring(0, idx2));
        } catch (ParseException pe) {
            return null;
        }
        int idalbum = 0;
        try {
            idalbum = Integer.parseInt(illustrationName.substring(idx2 + 1, idx3));
        } catch (NumberFormatException nfe) {
            return null;
        }
        short formatType = 0;
        try {
            formatType = AlbumUtils.formatTypeToShort(illustrationName.substring(idx3 + 1));
        } catch (IllegalArgumentException iae) {
            return null;
        }
        Album album = (Album) bdfServer.getBasedefiches().getSubset(albumKey);
        if (album == null) {
            return null;
        }
        Illustration illustration = album.getIllustrationByIdalbum(idalbum);
        if (illustration == null) {
            return null;
        }
        AlbumDim albumDim = null;
        short specialDim = 0;
        if (dimString == null) {
            specialDim = AlbumConstants.ORIGINAL_SPECIALDIM;
        } else {
            if (dimString.equals("_mini")) {
                specialDim = AlbumConstants.MINI_SPECIALDIM;
            } else {
                albumDim = album.getAlbumMetadata().getAlbumDimByName(dimString);
                if (albumDim == null) {
                    return null;
                }
            }
        }
        if (illustration.getFormatType() != formatType) {
            return new RedirectResponseHandler(AlbumUtils.toFileName(illustration));
        }
        IllustrationStreamProducer illustrationStreamProducer = null;
        if (albumDim != null) {
            illustrationStreamProducer = new IllustrationStreamProducer(illustration, albumDim);
        } else {
            illustrationStreamProducer = new IllustrationStreamProducer(illustration, specialDim);
        }
        return new StreamResponseHandler(illustrationStreamProducer);
    }

}
