/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.bdfcontent;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.get.streamproducers.bdfcontent.SphereOdsProducer;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.users.BdfUser;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.StreamResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class RedacteursInstruction implements BdfInstruction, BdfInstructionConstants {

    private BdfContext bdfContext;
    private BdfServer bdfServer;
    private String filePath;

    public RedacteursInstruction(BdfContext bdfContext, String filePath) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.filePath = filePath;
    }

    public short getBdfUserNeed() {
        return MANDATORY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        FilePathInfo filePathInfo = FilePathInfo.getSimpleFile(filePath);
        if (filePathInfo == null) {
            return null;
        }
        if (!filePathInfo.getExtension().equals("ods")) {
            return null;
        }
        String baseName = filePathInfo.getBaseName();
        Sphere activeSphere = null;
        if (!baseName.equals("_all")) {
            try {
                SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, baseName);
                activeSphere = (Sphere) bdfServer.getBasedefiches().getSubset(sphereKey);
            } catch (java.text.ParseException pe) {
                return null;
            }
            if (activeSphere == null) {
                return null;
            }
        }
        StreamProducer streamProducer = new SphereOdsProducer(bdfServer, activeSphere);
        return new StreamResponseHandler(streamProducer);
    }

}
