/* BdfServer_Get - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.pioche;

import fr.exemole.bdfserver.BdfContext;
import fr.exemole.bdfserver.BdfContextConstants;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.get.htmlproducers.pioche.FichePiocheHtmlProducer;
import fr.exemole.bdfserver.get.htmlproducers.pioche.GeocodagePiocheHtmlProducer;
import fr.exemole.bdfserver.get.htmlproducers.pioche.LanguePiocheHtmlProducer;
import fr.exemole.bdfserver.get.htmlproducers.pioche.MotclePiocheHtmlProducer;
import fr.exemole.bdfserver.get.htmlproducers.pioche.PaysPiocheHtmlProducer;
import fr.exemole.bdfserver.get.htmlproducers.pioche.PiocheHtmlConstants;
import fr.exemole.bdfserver.get.htmlproducers.pioche.RedacteurPiocheHtmlProducer;
import fr.exemole.bdfserver.get.jsonproducers.pioche.FichePiocheData;
import fr.exemole.bdfserver.get.jsonproducers.pioche.LanguePiocheData;
import fr.exemole.bdfserver.get.jsonproducers.pioche.MotclePiocheData;
import fr.exemole.bdfserver.get.jsonproducers.pioche.PaysPiocheData;
import fr.exemole.bdfserver.get.jsonproducers.pioche.PiocheJsonConstants;
import fr.exemole.bdfserver.get.jsonproducers.pioche.RedacteurPiocheData;
import fr.exemole.bdfserver.instruction.BdfInstruction;
import fr.exemole.bdfserver.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.instruction.BdfRequestMap;
import fr.exemole.bdfserver.instruction.InstructionUtils;
import fr.exemole.bdfserver.users.BdfUser;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.util.SelectOptionsParser;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionUtils;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceLocalisationProvider;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.servlets.ResponseDataType;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.handlers.HtmlResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class PiocheInstruction implements BdfInstruction, BdfInstructionConstants, ResponseDataType, PiocheConstants, PiocheHtmlConstants, PiocheJsonConstants {

    private Basedefiches basedefiches;
    private BdfContext bdfContext;
    private BdfServer bdfServer;
    private BdfRequestMap requestMap;
    private BdfUser bdfUser;
    private ResourceLocalisationProvider resourceLocalisationProvider;
    private LocalisationInfo localisationInfo;
    private PiocheParameters piocheParameters = new PiocheParameters();

    public PiocheInstruction(BdfContext bdfContext, BdfRequestMap requestMap) {
        this.bdfContext = bdfContext;
        this.bdfServer = bdfContext.getBdfServer();
        this.basedefiches = bdfServer.getBasedefiches();
        this.requestMap = requestMap;
        this.resourceLocalisationProvider = (ResourceLocalisationProvider) bdfContext.getContextObject(BdfContextConstants.RESOURCELOCALISATIONPROVIDER_NAME);
    }

    public short getBdfUserNeed() {
        return IF_ANY_BDFUSER;
    }

    public ResponseHandler runInstruction(BdfUser bdfUser) {
        this.bdfUser = bdfUser;
        if (bdfUser != null) {
            localisationInfo = bdfUser.getLocalisationInfo();
        } else {
            localisationInfo = InstructionUtils.getLocalisationInfo(requestMap, bdfServer);
        }
        ResourceLocalisation resourceLocalisation = resourceLocalisationProvider.getResourceLocalisation(localisationInfo.getFormatLocale());
        String pageParam = requestMap.getPageParameter();
        String jsonParam = requestMap.getJsonParameter();
        if (pageParam != null) {
            HtmlProducer htmlProducer = null;
            try {
                htmlProducer = getPageHtmlProducer(pageParam);
            } catch (PiocheException pe) {
                return CommandMessageResponseHandler.getHtmlInstance(pe.getErrorMessage(), resourceLocalisation);
            }
            if (htmlProducer == null) {
                return CommandMessageResponseHandler.getHtmlInstance(CommandMessage.error("global_err_unknownpage", pageParam), resourceLocalisation);
            }
            return new HtmlResponseHandler(htmlProducer);
        } else if (jsonParam != null) {
            JsonProducer jsonProducer = null;
            try {
                jsonProducer = getJsonProducer(jsonParam);
            } catch (PiocheException pe) {
                return CommandMessageResponseHandler.getJsonInstance(pe.getErrorMessage(), resourceLocalisation, requestMap);
            }
            if (jsonProducer == null) {
                return CommandMessageResponseHandler.getJsonInstance(CommandMessage.error("global_err_unknownjson", jsonParam), resourceLocalisation, requestMap);
            }
            return new JsonResponseHandler(jsonProducer, requestMap.getResponseDataType(), requestMap.getCallbackParameter());
        } else {
            return CommandMessageResponseHandler.getInstance(CommandMessage.error("global_err_nopagenojson", null), resourceLocalisation, requestMap);
        }

    }

    private HtmlProducer getPageHtmlProducer(String pageName) throws PiocheException {
        piocheParameters.setAppelant(getMandatoryParameter(APPELANT_PARAMNAME));
        if (pageName.equals(GEOCODAGE_PAGE)) {
            String latString = getMandatoryParameter(LAT_PARAMNAME).trim();
            String lonString = getMandatoryParameter(LONG_PARAMNAME).trim();
            DegreDecimal latDecimal = null;
            DegreDecimal longDecimal = null;
            if ((latString.length() > 0) && (lonString.length() > 0)) {
                List errorList = new ArrayList();
                try {
                    latDecimal = new DegreDecimal(StringUtils.parseDecimal(latString));
                } catch (NumberFormatException nfe) {
                    errorList.add(new ResourceItem("saisie_err_geopoint_lat", latString));
                }
                try {
                    longDecimal = new DegreDecimal(StringUtils.parseDecimal(lonString));
                } catch (NumberFormatException nfe) {
                    errorList.add(new ResourceItem("saisie_err_geopoint_lon", lonString));
                }
                if (errorList.size() > 0) {
                    CommandMessage errorMessage = CommandMessage.error(new ResourceItem("saisie_err_geopoint"), errorList);
                    throw new PiocheException(errorMessage);
                }
            }
            piocheParameters.putParameter(LAT_PARAMNAME, latDecimal);
            piocheParameters.putParameter(LONG_PARAMNAME, longDecimal);
            return new GeocodagePiocheHtmlProducer(bdfContext, bdfUser, piocheParameters);
        }
        piocheParameters.setLimite(getLimite());
        if (pageName.equals(FICHE_PAGE)) {
            piocheParameters.setSubsetList(getCorpusList());
            String motcleSelectString = requestMap.getParameter(MOTCLESELECTOPTIONS_PARAMNAME);
            if (motcleSelectString != null) {
                piocheParameters.setMotcleSelectOptionsString(motcleSelectString);
            }
            return new FichePiocheHtmlProducer(bdfContext, bdfUser, piocheParameters);
        } else if (pageName.equals(MOTCLE_PAGE)) {
            String param = requestMap.getParameter(PARAM_PARAMNAME);
            if ((param != null) && (param.equals(IDTHS_PARAMVALUE))) {
                piocheParameters.setParam(PiocheParameters.IDTHS_PARAMVALUE);
            }
            piocheParameters.setSubsetList(getThesaurusList());
            return new MotclePiocheHtmlProducer(bdfContext, bdfUser, piocheParameters);
        } else if (pageName.equals(REDACTEUR_PAGE)) {
            piocheParameters.setParam(getDefaultSphereKey());
            piocheParameters.setSubsetList(getSphereList());
            return new RedacteurPiocheHtmlProducer(bdfContext, bdfUser, piocheParameters);
        } else if (pageName.equals(LANGUE_PAGE)) {
            return new LanguePiocheHtmlProducer(bdfContext, bdfUser, piocheParameters);
        } else if (pageName.equals(PAYS_PAGE)) {
            return new PaysPiocheHtmlProducer(bdfContext, bdfUser, piocheParameters);
        } else {
            return null;
        }
    }

    private JsonProducer getJsonProducer(String json) throws PiocheException {
        int langInteger = bdfUser.getWorkingLangInteger();
        try {
            langInteger = LangInteger.parse(requestMap.getParameter("lang"));
        } catch (LangIntegerException mcle) {
        }
        Condition condition = ConditionUtils.parseSimpleCondition(getMandatoryParameter("recherche"));
        if (json.equals(FICHE_JSON)) {
            List<Subset> list = getCorpusList();
            FichePiocheData fichePiocheData = new FichePiocheData(basedefiches, langInteger);
            fichePiocheData.setSubsetList(list);
            fichePiocheData.setFicheEligibility(bdfServer.getPermissionManager().getFicheEligibility(bdfUser));
            fichePiocheData.setCondition(condition);
            String motcleSelectString = requestMap.getParameter(MOTCLESELECTOPTIONS_PARAMNAME);
            if (motcleSelectString != null) {
                MotcleSelectOptions motcleSelectOptions = SelectOptionsParser.parse(motcleSelectString);
                if (motcleSelectOptions != null) {
                    MotcleSelectOptions[] array = {motcleSelectOptions};
                    fichePiocheData.setMotcleSelectOptionsArray(array);
                }
            }
            fichePiocheData.init();
            return fichePiocheData;
        } else if (json.equals(MOTCLE_JSON)) {
            List<Subset> list = getThesaurusList();
            if ((list != null) && (list.size() == 1)) {
                Thesaurus thesaurus = (Thesaurus) list.get(0);
                langInteger = ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, langInteger);
            }
            MotclePiocheData motclePiocheData = new MotclePiocheData(basedefiches, langInteger);
            motclePiocheData.setSubsetList(list);
            motclePiocheData.setCondition(condition);
            motclePiocheData.init();
            return motclePiocheData;
        } else if (json.equals(REDACTEUR_JSON)) {
            List<Subset> list = getSphereList();
            RedacteurPiocheData redacteurPiocheData = new RedacteurPiocheData(basedefiches, langInteger);
            redacteurPiocheData.setSubsetList(list);
            redacteurPiocheData.setCondition(condition);
            redacteurPiocheData.setDefaultSphereKey(getDefaultSphereKey());
            redacteurPiocheData.init();
            return redacteurPiocheData;
        } else if (json.equals(LANGUE_JSON)) {
            LanguePiocheData languePiocheData = new LanguePiocheData(localisationInfo.getFormatLocale());
            languePiocheData.setCondition(condition);
            languePiocheData.init();
            return languePiocheData;
        } else if (json.equals(PAYS_JSON)) {
            PaysPiocheData paysPiocheData = new PaysPiocheData(localisationInfo.getFormatLocale());
            paysPiocheData.setCondition(condition);
            paysPiocheData.init();
            return paysPiocheData;
        } else {
            return null;
        }
    }

    private int getLimite() {
        String param_limite = requestMap.getParameter(LIMITE_PARAMNAME);
        int limite = 5;
        if (param_limite != null) {
            try {
                limite = Integer.parseInt(param_limite);
                if (limite < 1) {
                    limite = 5;
                }
            } catch (NumberFormatException nfe) {
            }
        }
        return limite;
    }

    private SubsetKey getDefaultSphereKey() {
        String param = requestMap.getParameter(PARAM_PARAMNAME);
        if (param == null) {
            return null;
        }
        try {
            SubsetKey sphereKey = SubsetKey.parse(param);
            if (sphereKey.isSphereSubset()) {
                return sphereKey;
            } else {
                return null;
            }
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    private List<Subset> getThesaurusList() throws PiocheException {
        String thesaurusParam = getMandatoryParameter("subsets");
        if (thesaurusParam.length() == 0) {
            return null;
        } else {
            String[] thesaurusArray = StringUtils.getTechnicalTokens(thesaurusParam);
            ArrayList<Subset> thsList = new ArrayList<Subset>();
            for (int i = 0; i < thesaurusArray.length; i++) {
                Subset subset = (Thesaurus) BasedefichesUtils.getThesaurus(basedefiches, thesaurusArray[i]);
                if (subset != null) {
                    thsList.add(subset);
                }
            }
            if (thsList.size() == 0) {
                Object[] valueArray = {"subsets", thesaurusParam};
                throw new PiocheException(CommandMessage.error("global_err_parameter_wrong", valueArray));
            }
            return thsList;
        }
    }

    private List<Subset> getCorpusList() throws PiocheException {
        String subsetsParam = getMandatoryParameter("subsets");
        if (subsetsParam.length() == 0) {
            return null;
        } else {
            String[] corpusArray = StringUtils.getTechnicalTokens(subsetsParam);
            ArrayList<Subset> crpList = new ArrayList<Subset>();
            for (int i = 0; i < corpusArray.length; i++) {
                Subset subset = BasedefichesUtils.getCorpus(basedefiches, corpusArray[i]);
                if (subset != null) {
                    crpList.add(subset);
                }
            }
            if (crpList.size() == 0) {
                Object[] valueArray = {"subsets", subsetsParam};
                throw new PiocheException(CommandMessage.error("global_err_parameter_wrong", valueArray));
            }
            return crpList;
        }
    }

    private List<Subset> getSphereList() throws PiocheException {
        String subsetsParam = getMandatoryParameter("subsets");
        if (subsetsParam.length() == 0) {
            return null;
        } else {
            String[] array = StringUtils.getTechnicalTokens(subsetsParam);
            ArrayList<Subset> sphList = new ArrayList<Subset>();
            for (int i = 0; i < array.length; i++) {
                Subset subset = BasedefichesUtils.getSphere(basedefiches, array[i]);
                if (subset != null) {
                    sphList.add(subset);
                }
            }
            if (sphList.size() == 0) {
                Object[] valueArray = {"subsets", subsetsParam};
                throw new PiocheException(CommandMessage.error("global_err_parameter_wrong", valueArray));
            }
            return sphList;
        }
    }

    public String getMandatoryParameter(String paramName) throws PiocheException {
        String s = requestMap.getParameter(paramName);
        if (s == null) {
            throw new PiocheException("global_err_parameter_mandatory", paramName);
        }
        return s;
    }

}
