/* BdfServer_Get - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.jsonproducers.pioche;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.CorpusList;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.selection.FicheSelectOptions;
import net.mapeadores.basedefiches.selection.FicheSelector;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.util.MutableCorpusList;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;


/**
 *
 * @author Vincent Calame
 */
public class FichePiocheData implements JsonProducer {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private CorpusList corpusList;
    private Condition condition;
    private Collection<AbregeFiche> result;
    private FicheEligibility ficheEligibility;
    private MotcleSelectOptions[] motcleSelectOptionsArray;

    public FichePiocheData(Basedefiches basedefiches, int workingLangInteger) {
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
        corpusList = basedefiches;
    }

    public void setFicheEligibility(FicheEligibility ficheEligibility) {
        this.ficheEligibility = ficheEligibility;
    }

    public void setSubsetList(List<Subset> subsetList) {
        if (subsetList == null) {
            corpusList = basedefiches;
        } else {
            MutableCorpusList mutableCorpusList = new MutableCorpusList();
            for (Subset subset : subsetList) {
                mutableCorpusList.add((Corpus) subset);
            }
            corpusList = mutableCorpusList;
        }
    }

    public void setCondition(Condition condition) {
        this.condition = condition;
    }

    public void setMotcleSelectOptionsArray(MotcleSelectOptions[] motcleSelectOptionsArray) {
        this.motcleSelectOptionsArray = motcleSelectOptionsArray;
    }

    public void init() {
        if ((condition != null) || (motcleSelectOptionsArray != null)) {
            int corpusCount = corpusList.getCorpusCount();
            SubsetKey[] corpusKeyArray = new SubsetKey[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                corpusKeyArray[i] = corpusList.getCorpus(i).getSubsetKey();
            }
            FicheSelectOptions ficheSelectOptions = new FicheSelectOptions();
            ficheSelectOptions.setCorpusKeyArray(corpusKeyArray);
            if (condition != null) {
                ficheSelectOptions.setFieldCondition(condition, FicheSelectOptions.TITRE_SCOPE, null);
            }
            if (motcleSelectOptionsArray != null) {
                ficheSelectOptions.setMotcleSelectOptionsArray(motcleSelectOptionsArray);
            }
            FicheSelector ficheSelector = new FicheSelector(basedefiches, workingLangInteger);
            ficheSelector.addFicheSelectOptions(ficheSelectOptions);
            ficheSelector.setFicheEligibility(ficheEligibility);
            result = selection(ficheSelector);
        } else {
            result = new ArrayList<AbregeFiche>();
            int corpusCount = corpusList.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = corpusList.getCorpus(i);
                int length = corpus.getTicketLength();
                for (int j = 0; j < length; j++) {
                    AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(j);
                    if (abregeFiche != null) {
                        if ((ficheEligibility == null) || (ficheEligibility.accept(abregeFiche))) {
                            result.add(abregeFiche);
                        }
                    }
                }
            }
        }
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.size();
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (AbregeFiche abregeFiche : result) {
                    int idcorpus = abregeFiche.getIdcorpus();
                    String corpusName = abregeFiche.getCorpus().getSubsetKey().getSubsetName();
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(idcorpus);
                    jsonWriter.key("idalphakey");
                    jsonWriter.value(corpusName + '/' + idcorpus);
                    String titre = abregeFiche.getTitre();
                    if (titre.length() > 0) {
                        jsonWriter.key("text");
                        jsonWriter.value(titre);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private Collection<AbregeFiche> selection(FicheSelector ficheSelector) {
        TreeSet<AbregeFiche> treeset = new TreeSet<AbregeFiche>(new FicheComparator());
        int corpusCount = corpusList.getCorpusCount();
        for (int j = 0; j < corpusCount; j++) {
            Corpus corpus = corpusList.getCorpus(j);
            int length = corpus.getTicketLength();
            for (int i = 0; i < length; i++) {
                AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(i);
                if (abregeFiche == null) {
                    continue;
                }
                if (ficheSelector.isSelected(abregeFiche)) {
                    treeset.add(abregeFiche);
                }
            }
        }
        return treeset;
    }


    private static class FicheComparator implements Comparator<AbregeFiche> {

        public int compare(AbregeFiche af1, AbregeFiche af2) {
            int result = af1.getCorpus().getSubsetKey().compareTo(af2.getCorpus().getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = af1.getIdcorpus();
            int id2 = af2.getIdcorpus();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

}
