/* BdfServer_Get - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.jsonproducers.pioche;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.ThesaurusList;
import net.mapeadores.basedefiches.selection.MotcleSelectOptions;
import net.mapeadores.basedefiches.selection.MotcleSelector;
import net.mapeadores.basedefiches.selection.util.MutableThesaurusList;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;


/**
 *
 * @author Vincent Calame
 */
public class MotclePiocheData implements JsonProducer {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private ThesaurusList thesaurusList;
    private Condition condition;
    private Collection<Motcle> result;

    public MotclePiocheData(Basedefiches basedefiches, int workingLangInteger) {
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
        thesaurusList = basedefiches;
    }

    public void setSubsetList(List<Subset> subsetList) {
        if (subsetList == null) {
            thesaurusList = basedefiches;
        } else {
            MutableThesaurusList mutableThesaurusList = new MutableThesaurusList();
            for (Subset subset : subsetList) {
                mutableThesaurusList.add((Thesaurus) subset);
            }
            thesaurusList = mutableThesaurusList;
        }
    }

    public void setCondition(Condition condition) {
        this.condition = condition;
    }

    public void init() {
        if (condition != null) {
            int thesaurusCount = thesaurusList.getThesaurusCount();
            SubsetKey[] thesaurusKeyArray = new SubsetKey[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                thesaurusKeyArray[i] = thesaurusList.getThesaurus(i).getSubsetKey();
            }
            MotcleSelectOptions motcleSelectOptions = new MotcleSelectOptions();
            motcleSelectOptions.setThesaurusKeyArray(thesaurusKeyArray);
            motcleSelectOptions.setFieldCondition(condition);
            motcleSelectOptions.setIdalphaOnly(false);
            MotcleSelector motcleSelector = new MotcleSelector(basedefiches, workingLangInteger);
            motcleSelector.addMotcleSelectOptions(motcleSelectOptions);
            result = selection(motcleSelector);
        } else {
            result = new ArrayList<Motcle>();
            int thesaurusCount = thesaurusList.getThesaurusCount();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = thesaurusList.getThesaurus(i);
                appendMotcle(thesaurus.getFirstLevelList());
            }
        }
    }

    private void appendMotcle(MotcleList list) {
        for (int i = 0; i < list.getMotcleCount(); i++) {
            Motcle motcle = list.getMotcle(i);
            result.add(motcle);
            appendMotcle(motcle.getChildList());
        }
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.size();
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (Motcle motcle : result) {
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(motcle.getIdths());
                    String idalpha = motcle.getIdalpha();
                    if (idalpha != null) {
                        jsonWriter.key("idalphakey");
                        jsonWriter.value(idalpha);
                    }
                    String lib = motcle.getLibelleString(workingLangInteger, false);
                    if (lib != null) {
                        jsonWriter.key("text");
                        jsonWriter.value(lib);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private Collection<Motcle> selection(MotcleSelector motcleSelector) {
        TreeSet<Motcle> treeSet = new TreeSet<Motcle>(new MotcleComparator(workingLangInteger));
        int thesaurusCount = thesaurusList.getThesaurusCount();
        for (int j = 0; j < thesaurusCount; j++) {
            Thesaurus thesaurus = thesaurusList.getThesaurus(j);
            int length = thesaurus.getTicketLength();
            for (int i = 0; i < length; i++) {
                Motcle motcle = (Motcle) thesaurus.getSubsetItemByTicket(i);
                if (motcle == null) {
                    continue;
                }
                if (motcleSelector.isSelected(motcle)) {
                    treeSet.add(motcle);
                }
            }
        }
        return treeSet;
    }


    private static class MotcleComparator implements Comparator<Motcle> {

        private int langInteger;

        public MotcleComparator(int langInteger) {
            this.langInteger = langInteger;
        }

        public int compare(Motcle mc1, Motcle mc2) {
            String a1 = mc1.getIdalpha();
            String a2 = mc2.getIdalpha();
            String lib1 = mc1.getLibelleString(langInteger, true);
            String lib2 = mc2.getLibelleString(langInteger, true);
            if (a1 != null) {
                lib1 = a1 + " – " + lib1;
            }
            if (a2 != null) {
                lib2 = a2 + " – " + lib2;
            }
            int result = 0;
            result = lib1.compareTo(lib2);
            if (result != 0) {
                return result;
            }
            result = mc1.getThesaurus().getSubsetKey().compareTo(mc2.getThesaurus().getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = mc1.getIdths();
            int id2 = mc2.getIdths();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

    }

}
