/* BdfServer_Get - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.jsonproducers.pioche;

import java.io.IOException;
import java.util.Locale;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerLocalisation;
import net.mapeadores.util.langinteger.LocaleItem;
import net.mapeadores.util.text.TextConstants;


/**
 *
 * @author Vincent Calame
 */
public class PaysPiocheData implements JsonProducer {

    private Condition condition;
    private LocaleItem localeItem;
    private int[] result;

    public PaysPiocheData(Locale locale) {
        this.localeItem = LangIntegerLocalisation.getLocaleItem(locale);
    }

    public void setCondition(Condition condition) {
        this.condition = condition;
    }

    public void init() {
        if (condition != null) {
            result = selection();
        } else {
            result = localeItem.getCountryIntegerArray();
        }
    }

    private int[] selection() {
        String texte = condition.getIncludingTextTest(0).getText();
        return localeItem.getCountryIntegerArray(texte, TextConstants.CONTAINS);
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.length;
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (int i = 0; i < length; i++) {
                    int id = result[i];
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(id);
                    jsonWriter.key("idalphakey");
                    jsonWriter.value(LangInteger.getCountry(id));
                    String lib = localeItem.getCountryLibelle(id, false);
                    if (lib.length() > 0) {
                        jsonWriter.key("text");
                        jsonWriter.value(lib);
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
