/* BdfServer_Get - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.jsonproducers.pioche;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SphereList;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.selection.RedacteurSelectOptions;
import net.mapeadores.basedefiches.selection.RedacteurSelector;
import net.mapeadores.basedefiches.selection.util.MutableSphereList;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurPiocheData implements JsonProducer {

    private Basedefiches basedefiches;
    private int workingLangInteger;
    private SphereList sphereList;
    private Collection<Redacteur> result;
    private Condition condition;
    private short style;
    private SubsetKey defaultSphereKey;

    public RedacteurPiocheData(Basedefiches basedefiches, int workingLangInteger) {
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
        sphereList = basedefiches;
    }

    public void setSubsetList(List<Subset> subsetList) {
        if (subsetList == null) {
            sphereList = basedefiches;
        } else {
            MutableSphereList mutableSphereList = new MutableSphereList();
            for (Subset subset : subsetList) {
                mutableSphereList.add((Sphere) subset);
            }
            sphereList = mutableSphereList;
        }
    }

    public void setCondition(Condition condition) {
        this.condition = condition;
    }

    public void setDefaultSphereKey(SubsetKey defaultSphereKey) {
        this.defaultSphereKey = defaultSphereKey;
    }

    public void init() {
        style = PersonneBean.BIBLIO_STYLE;
        if (condition != null) {
            RedacteurSelectOptions redacteurSelectOptions = new RedacteurSelectOptions();
            redacteurSelectOptions.setNomcompletCondition(condition);
            RedacteurSelector redacteurSelector = new RedacteurSelector(basedefiches, workingLangInteger);
            redacteurSelector.addRedacteurSelectOptions(redacteurSelectOptions);
            result = selection(redacteurSelector);
        } else {
            result = new ArrayList<Redacteur>();
            int sphereCount = sphereList.getSphereCount();
            for (int i = 0; i < sphereCount; i++) {
                Sphere corpus = sphereList.getSphere(i);
                int length = corpus.getTicketLength();
                for (int j = 0; j < length; j++) {
                    Redacteur redacteur = (Redacteur) corpus.getSubsetItemByTicket(j);
                    if (redacteur != null) {
                        result.add(redacteur);
                    }
                }
            }
        }
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            int length = result.size();
            jsonWriter.key("length");
            jsonWriter.value(length);
            if (length > 0) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (Redacteur redacteur : result) {
                    RedacteurKey redacteurKey = redacteur.getRedacteurKey();
                    int intkey = redacteur.getIdsphere();
                    jsonWriter.object();
                    jsonWriter.key("intkey");
                    jsonWriter.value(intkey);
                    jsonWriter.key("idalphakey");
                    if ((defaultSphereKey != null) && (redacteurKey.getSphereKey().equals(defaultSphereKey))) {
                        jsonWriter.value(redacteurKey.getIdsphereString());
                    } else {
                        jsonWriter.value(redacteurKey.toString(RedacteurKey.BRACKET_STYLE));
                    }
                    PersonneBean personneBean = redacteur.getPersonneBean();
                    jsonWriter.key("text");
                    jsonWriter.value(personneBean.getNomComplet(style));
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private Collection<Redacteur> selection(RedacteurSelector redacteurSelector) {
        TreeSet<Redacteur> redacSet = new TreeSet<Redacteur>(new RedacteurComparator());
        int sphereCount = sphereList.getSphereCount();
        for (int j = 0; j < sphereCount; j++) {
            Sphere sphere = sphereList.getSphere(j);
            int length = sphere.getTicketLength();
            for (int i = 0; i < length; i++) {
                Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
                if (redacteur == null) {
                    continue;
                }
                if (!redacteur.isActif()) {
                    continue;
                }
                if (redacteurSelector.isSelected(redacteur)) {
                    redacSet.add(redacteur);
                }
            }
        }
        return redacSet;
    }


    private static class RedacteurComparator implements Comparator<Redacteur> {

        public int compare(Redacteur r1, Redacteur r2) {
            String nc1 = r1.getPersonneBean().getNomComplet(PersonneBean.ANNUAIRE_UP_STYLE);
            String nc2 = r2.getPersonneBean().getNomComplet(PersonneBean.ANNUAIRE_UP_STYLE);
            int result = nc1.compareTo(nc2);
            if (result != 0) {
                return result;
            }
            String id1 = r1.getRedacteurKey().getIdsphereString();
            String id2 = r2.getRedacteurKey().getIdsphereString();
            result = id1.compareTo(id2);
            if (result != 0) {
                return result;
            }
            return r1.getRedacteurKey().getSphereKey().compareTo(r2.getRedacteurKey().getSphereKey());
        }

    }

}
