/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers.bdfcontent;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.transformation.StreamTransformerKey;
import fr.exemole.bdfserver.managers.transformation.TransformationConstants;
import fr.exemole.bdfserver.managers.transformation.TransformationInfo;
import fr.exemole.bdfserver.managers.transformation.TransformationKey;
import fr.exemole.bdfserver.managers.transformation.TransformationManager;
import fr.exemole.bdfserver.managers.transformation.TransformerParameters;
import fr.exemole.bdfserver.managers.transformation.XslTransformerKey;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringReader;
import java.util.Date;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.basedefiches.extraction.ExtractionContext;
import net.mapeadores.basedefiches.extraction.api.ExtractionDef;
import net.mapeadores.basedefiches.extraction.api.ExtractionSource;
import net.mapeadores.basedefiches.extraction.builders.BuilderUtils;
import net.mapeadores.basedefiches.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.RedirectResponseHandler;
import net.mapeadores.util.servlets.handlers.StreamResponseHandler;
import net.mapeadores.util.servlets.handlers.XmlResponseHandler;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XmlProducer;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionStream {

    private ExtractionStream() {
    }

    public static ResponseHandler toRedirectResponseHandler(BdfServer bdfServer, BdfUser bdfUser, TransformationKey transformationKey) {
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
        if (!xslTransformerKey.isDefaultTransformerKey()) {
            TransformationInfo transformationInfo = bdfServer.getTransformationManager().getTransformationInfo(xslTransformerKey.getTransformationKey());
            if (!transformationInfo.containsXslTransformer(xslTransformerKey.getTransformerName())) {
                xslTransformerKey = xslTransformerKey.toDefaultTransformerKey();
            }
        }
        return new RedirectResponseHandler("../" + getTemplatePath(xslTransformerKey));
    }

    public static String getTemplatePath(XslTransformerKey xslTransformerKey) {
        String templateName = xslTransformerKey.getTransformerName();
        StringBuilder buf = new StringBuilder();
        buf.append("xslt/");
        if (templateName == null) {
            buf.append("_default_");
            buf.append(xslTransformerKey.getTransformationKey().toString());
        } else {
            buf.append(xslTransformerKey.getTransformationKey().toString());
            buf.append("/");
            buf.append(templateName);
        }
        buf.append(".xsl");
        return buf.toString();
    }

    public static ResponseHandler toStreamTransformationResponseHandler(BdfServer bdfServer, BdfUser bdfUser, TransformationKey transformationKey, String extension, ExtractionContext extractionContext, Object dynamicObject) {
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        StreamTransformerKey streamTransformerKey = getStreamTransformerKey(bdfUser, transformationKey, extension);
        StreamTransformer streamTransformer = transformationManager.getStreamTransformer(streamTransformerKey);
        ExtractionDef extractionDef = transformationManager.getExtractionDef(streamTransformerKey);
        if (streamTransformer == null) {
            throw new HttpAccessException(HttpAccessException.NOT_FOUND);
        }
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        Source xmlSource = ExtractionXMLPart.toStreamSource(extractionContext, extractionDef, extractionSource);
        TransformerParameters transformerParameters = new TransformerParameters();
        transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
        transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, bdfUser.getBaseDeFichesURL());
        StreamTransformerStreamProducer streamProducer = new StreamTransformerStreamProducer(xmlSource, streamTransformer, transformerParameters);
        return new StreamResponseHandler(streamProducer);
    }

    public static ResponseHandler toXmlResponseHandler(BdfServer bdfServer, BdfUser bdfUser, TransformationKey transformationKey, ExtractionContext extractionContext, Object dynamicObject, String stylesheetPath) {
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
        ExtractionDef extractionDef = bdfServer.getTransformationManager().getExtractionDef(xslTransformerKey);
        return toXmlResponseHandler(bdfServer, bdfUser, extractionContext, extractionDef, dynamicObject, stylesheetPath);
    }

    public static ResponseHandler toXmlResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, ExtractionDef extractionDef, Object dynamicObject, String stylesheetPath) {
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        ExtractionXmlProducer extractionXmlProducer = new ExtractionXmlProducer(extractionContext, extractionDef, extractionSource, stylesheetPath);
        return new XmlResponseHandler(extractionXmlProducer);
    }

    public static ResponseHandler toHtmlTransformationResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, Object dynamicObject, TransformationKey transformationKey) {
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
        ExtractionDef extractionDef = transformationManager.getExtractionDef(xslTransformerKey);
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        return toHtmlTransformationResponseHandler(bdfServer, bdfUser, ExtractionXMLPart.toStreamSource(extractionContext, extractionDef, extractionSource), xslTransformerKey);
    }

    public static ResponseHandler toHtmlTransformationResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, ExtractionDef extractionDef, Object dynamicObject, Transformer transformer) {
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        StreamProducer streamProducer = new TransformerStreamProducer(ExtractionXMLPart.toStreamSource(extractionContext, extractionDef, extractionSource), transformer);
        return new StreamResponseHandler(streamProducer);
    }

    public static ResponseHandler toHtmlTransformationResponseHandler(BdfServer bdfServer, BdfUser bdfUser, String sourceString, TransformationKey transformationKey) {
        XslTransformerKey xslTransformerKey = bdfUser.getXslTransformerKey(transformationKey);
        StringReader stringReader = new StringReader(sourceString);
        StreamSource xmlSource = new StreamSource(stringReader);
        return toHtmlTransformationResponseHandler(bdfServer, bdfUser, xmlSource, xslTransformerKey);
    }

    private static ResponseHandler toHtmlTransformationResponseHandler(BdfServer bdfServer, BdfUser bdfUser, StreamSource streamSource, XslTransformerKey xslTransformerKey) {
        Transformer transformer = bdfServer.getTransformationManager().getXslTransformer(xslTransformerKey);
        TransformerParameters transformerParameters = new TransformerParameters();
        transformerParameters.setLocalisationInfo(bdfUser.getLocalisationInfo());
        transformerParameters.setParam(TransformationConstants.BDF_BASEDEFICHESPATH_PARAMETER, "../");
        transformerParameters.initParameters(transformer);
        StreamProducer streamProducer = new TransformerStreamProducer(streamSource, transformer);
        return new StreamResponseHandler(streamProducer);
    }

    public static StreamTransformerKey getStreamTransformerKey(BdfUser bdfUser, TransformationKey transformationKey, String extension) {
        StreamTransformerKey streamTransformerKey = bdfUser.getStreamTransformerKey(transformationKey, extension);
        if (streamTransformerKey == null) {
            streamTransformerKey = StreamTransformerKey.getDefaultStreamTransformerKey(transformationKey, extension);
        }
        return streamTransformerKey;
    }


    private static class TransformerStreamProducer implements StreamProducer {

        private Source xmlSource;
        private Transformer transformer;
        private String mimeType;
        private String charset;

        private TransformerStreamProducer(Source xmlSource, Transformer transformer) {
            this.xmlSource = xmlSource;
            this.transformer = transformer;
            String method = transformer.getOutputProperty("method");
            if (method == null) {
                method = "xml";
            }
            String encoding = transformer.getOutputProperty("encoding");
            if (encoding == null) {
                encoding = "UTF-8";
            }
            this.charset = encoding;
            if (method.equals("xml")) {
                this.mimeType = "text/xml";
            } else if (method.equals("html")) {
                this.mimeType = "text/html";
            } else {
                this.mimeType = "text/plain";
            }
        }

        public String getMimeType() {
            return mimeType;
        }

        public String getCharset() {
            return charset;
        }

        public void writeStream(OutputStream outputStream) throws IOException {
            try {
                transformer.transform(xmlSource, new StreamResult(outputStream));
            } catch (TransformerException te) {
                throw new NestedTransformerException(te);
            }
        }

    }


    private static class StreamTransformerStreamProducer implements StreamProducer {

        private Source xmlSource;
        private StreamTransformer streamTransformer;
        private TransformerParameters transformerParameters;

        private StreamTransformerStreamProducer(Source xmlSource, StreamTransformer transformer, TransformerParameters transformerParameters) {
            this.xmlSource = xmlSource;
            this.streamTransformer = transformer;
            this.transformerParameters = transformerParameters;
        }

        public String getMimeType() {
            return streamTransformer.getMimeType();
        }

        public String getCharset() {
            return null;
        }

        public void writeStream(OutputStream outputStream) throws IOException {
            try {
                streamTransformer.transform(xmlSource, outputStream, transformerParameters.toMap());
            } catch (TransformerException te) {
                throw new NestedTransformerException(te);
            }
        }

    }


    private static class ExtractionXmlProducer implements XmlProducer {

        private ExtractionContext extractionContext;
        private ExtractionDef extractionDef;
        private ExtractionSource extractionSource;
        private String stylesheetPath;

        private ExtractionXmlProducer(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource, String stylesheetPath) {
            this.extractionContext = extractionContext;
            this.extractionDef = extractionDef;
            this.extractionSource = extractionSource;
            this.stylesheetPath = stylesheetPath;

        }

        public void writeXml(Appendable appendable) throws IOException {
            int nbretabulations = (stylesheetPath == null) ? 0 : -999;
            XMLWriter xmlWriter = XMLUtils.getXMLWriter(appendable, nbretabulations);
            xmlWriter.appendXMLDeclaration();
            if (stylesheetPath != null) {
                xmlWriter.appendStylesheet("../" + stylesheetPath, "text/xsl");
            }
            ExtractionXMLPart extractionXMLPart = new ExtractionXMLPart(xmlWriter, extractionContext);
            extractionXMLPart.addExtraction(extractionDef, new Date(), extractionSource, true);
        }

    }

}
