/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers.bdfcontent;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetXMLWriter;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetZipEntrySource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.text.lexie.LexieMap;
import net.mapeadores.util.text.lexie.LexieSource;
import net.mapeadores.util.text.lexie.Occurence;
import net.mapeadores.util.text.lexie.TextLexieUnit;


/**
 *
 * @author Vincent Calame
 */
public class LexieMapOdsProducer implements StreamProducer {

    private LexieMap lexieMap;
    private Thesaurus thesaurus;

    public LexieMapOdsProducer(LexieMap lexieMap, Thesaurus thesaurus) {
        this.lexieMap = lexieMap;
        this.thesaurus = thesaurus;
    }

    public String getMimeType() {
        return OdUtils.getMimeType(OdUtils.OD_SPREADSHEET);
    }

    public String getCharset() {
        return null;
    }

    public void writeStream(OutputStream outputStream) throws IOException {
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        LexieOdsWriter xmlWriter = new LexieOdsWriter(thesaurus, lexieMap);
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(xmlWriter);
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
    }


    private static class LexieOdsWriter extends SpreadSheetXMLWriter {

        private LexieMap lexieMap;
        private Thesaurus thesaurus;
        private boolean withIdalpha;

        private LexieOdsWriter(Thesaurus thesaurus, LexieMap lexieMap) {
            this.lexieMap = lexieMap;
            this.thesaurus = thesaurus;
            withIdalpha = ThesaurusUtils.isIdalphaType(thesaurus);
        }

        public void printTable() throws IOException {
            String thesaurusName = thesaurus.getSubsetKey().getSubsetName();
            openTable(thesaurusName);
            printFirstRow();
            for (Iterator it = lexieMap.getCollatedKeyMap().values().iterator(); it.hasNext();) {
                TextLexieUnit lexieUnit = (TextLexieUnit) it.next();
                printLexieUnit(lexieUnit);
            }
            closeTable();
        }

        private void printLexieUnit(TextLexieUnit lexieUnit) throws IOException {
            String canonical = lexieUnit.getCanonicalLexie();
            List list = lexieUnit.getOccurenceList();
            boolean premier = true;
            LexieSource previousLexieSource = null;
            for (Iterator it = list.iterator(); it.hasNext();) {
                Occurence occurence = (Occurence) it.next();
                LexieSource lexieSource = occurence.getParsedLexieSource().getLexieSource();
                if (previousLexieSource != null) {
                    if (previousLexieSource.equals(lexieSource)) {
                        continue;
                    }
                }
                previousLexieSource = lexieSource;
                openTableRow();
                if (premier) {
                    addStringTableCell(canonical);
                    premier = false;
                } else {
                    addEmptyTableCell();
                }
                addStringTableCell(lexieSource.getSourceText());
                Motcle motcle = (Motcle) lexieSource.getSourceObject();
                addIntegerTableCell(motcle.getIdths());
                if (withIdalpha) {
                    addStringTableCell(motcle.getIdalpha());
                }
                addStringTableCell(canonical);
                closeTableRow();
            }
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("lexie");
            addStringTableCell("lib");
            addStringTableCell("idths");
            if (withIdalpha) {
                addStringTableCell("idalpha");
            }
            addStringTableCell("lexie_rep");
            closeTableRow();
        }

    }

}
