/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers.bdfcontent;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.PermissionManager;
import fr.exemole.bdfserver.users.BdfUser;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;
import java.util.TreeMap;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.beans.CourrielBean;
import net.mapeadores.basedefiches.beans.PersonneBean;
import net.mapeadores.basedefiches.sphere.Redacteur;
import net.mapeadores.basedefiches.sphere.RedacteurKey;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetXMLWriter;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetZipEntrySource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.util.io.StreamProducer;


/**
 *
 * @author Vincent Calame
 */
public class SphereOdsProducer implements StreamProducer {

    private BdfServer bdfServer;
    private Sphere sphere;

    public SphereOdsProducer(BdfServer bdfServer, Sphere sphere) {
        this.bdfServer = bdfServer;
        this.sphere = sphere;
    }

    public String getMimeType() {
        return OdUtils.getMimeType(OdUtils.OD_SPREADSHEET);
    }

    public String getCharset() {
        return null;
    }

    public void writeStream(OutputStream outputStream) throws IOException {
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(new SphereXMLWriter(sphere, bdfServer));
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
    }


    private static class SphereXMLWriter extends SpreadSheetXMLWriter {

        private Sphere activeSphere;
        private BdfServer bdfServer;
        private TreeMap<String, Redacteur> actifMap = new TreeMap<String, Redacteur>();
        private TreeMap<String, Redacteur> inactifMap = new TreeMap<String, Redacteur>();
        private PermissionManager permissionManager;
        private boolean all = false;

        SphereXMLWriter(Sphere activeSphere, BdfServer bdfServer) {
            this.activeSphere = activeSphere;
            this.bdfServer = bdfServer;
            this.permissionManager = bdfServer.getPermissionManager();
            this.all = (activeSphere == null);
            init();
        }

        private void init() {
            if (all) {
                Basedefiches basedefiches = bdfServer.getBasedefiches();
                int sphereCount = basedefiches.getSphereCount();
                for (int j = 0; j < sphereCount; j++) {
                    Sphere sphere = basedefiches.getSphere(j);
                    for (int i = 0; i < sphere.getTicketLength(); i++) {
                        Redacteur redacteur = (Redacteur) sphere.getSubsetItemByTicket(i);
                        if (redacteur != null) {
                            Map map = (redacteur.isActif()) ? actifMap : inactifMap;
                            map.put(redacteur.getRedacteurKey().toString(RedacteurKey.UNDERSCORE_STYLE), redacteur);
                        }
                    }
                }
            } else {
                for (int i = 0; i < activeSphere.getTicketLength(); i++) {
                    Redacteur redacteur = (Redacteur) activeSphere.getSubsetItemByTicket(i);
                    if (redacteur != null) {
                        Map map = (redacteur.isActif()) ? actifMap : inactifMap;
                        map.put(redacteur.getRedacteurKey().getIdsphereString(), redacteur);
                    }
                }
            }
        }

        public void printTable() throws IOException {
            String name = (activeSphere != null) ? activeSphere.getSubsetKey().getSubsetName() : "_all";
            openTable(name);
            printFirstRow();
            printMap(actifMap);
            printMap(inactifMap);
            closeTable();
        }

        private void printMap(Map<String, Redacteur> map) throws IOException {
            for (Map.Entry<String, Redacteur> mapEntry : map.entrySet()) {
                String strg_idsphere = mapEntry.getKey();
                Redacteur redacteur = mapEntry.getValue();
                printRedacteur(redacteur);
            }
        }

        private void printRedacteur(Redacteur redacteur) throws IOException {
            RedacteurKey redacteurKey = redacteur.getRedacteurKey();
            openTableRow();
            int actif = (redacteur.isActif()) ? 1 : 0;
            addIntegerTableCell(actif);
            if (all) {
                addStringTableCell(redacteurKey.getSphereKey().getSubsetName());
            }
            String idsphere = redacteurKey.getIdsphereString();
            addStringTableCell(idsphere);
            PersonneBean personneBean = redacteur.getPersonneBean();
            addStringTableCell(personneBean.getNom());
            addStringTableCell(personneBean.getPrenom());
            int nomavant = (personneBean.isNomAvant()) ? 1 : 0;
            addIntegerTableCell(nomavant);
            CourrielBean courrielBean = redacteur.getCourrielBean();
            if (courrielBean != null) {
                addStringTableCell(courrielBean.getAddrSpec());
            } else {
                addEmptyTableCell();
            }
            if (actif == 1) {
                int admin = (permissionManager.isAdmin(redacteurKey)) ? 1 : 0;
                addIntegerTableCell(admin);

                BdfUser redacUser = new BdfUser(bdfServer, redacteurKey);
                String[] roleArray = redacUser.getRoleNameArray();
                if (roleArray == null) {
                    addStringTableCell("_default");
                } else if (roleArray.length == 0) {
                    addEmptyTableCell();
                } else {
                    StringBuffer buf = new StringBuffer();
                    for (int i = 0; i < roleArray.length; i++) {
                        if (i > 0) {
                            buf.append("; ");
                        }
                        buf.append(roleArray[i]);
                    }
                    addStringTableCell(buf.toString());
                }
            }
            closeTableRow();
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("actif");
            if (all) {
                addStringTableCell("sphere");
            }
            addStringTableCell("idsphere");
            addStringTableCell("nom");
            addStringTableCell("prenom");
            addStringTableCell("nomavant");
            addStringTableCell("courriel");
            addStringTableCell("admin");
            addStringTableCell("roles");
            closeTableRow();
        }

    }

}
