/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers.bdfcontent;

import java.io.IOException;
import java.io.OutputStream;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.croisement.Croisement;
import net.mapeadores.basedefiches.croisement.CroisementItem;
import net.mapeadores.basedefiches.selection.FicheEligibility;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.MotcleList;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetXMLWriter;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetZipEntrySource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.langinteger.LangInteger;

/**
 *
 * @author Vincent Calame
 */
public class StatsThesaurusOdsProducer implements StreamProducer {

    private Thesaurus thesaurus;
    private FicheEligibility ficheEligibility;
    private int workingLangInteger;

    public StatsThesaurusOdsProducer(Thesaurus thesaurus, FicheEligibility ficheEligibility, int workingLangInteger) {
        this.thesaurus = thesaurus;
        this.ficheEligibility = ficheEligibility;
        this.workingLangInteger = workingLangInteger;
    }
    
    public String getMimeType() {
        return OdUtils.getMimeType(OdUtils.OD_SPREADSHEET);
    }

    public String getCharset() {
        return null;
    }

    public void writeStream(OutputStream outputStream) throws IOException {
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        StatsThesaurusXMLWriter xmlWriter = new StatsThesaurusXMLWriter(thesaurus, ficheEligibility, ThesaurusUtils.checkLangIntegerDisponibility(thesaurus, workingLangInteger));
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(xmlWriter);
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
    }


    private static class StatsThesaurusXMLWriter extends SpreadSheetXMLWriter {

        private Thesaurus thesaurus;
        private FicheEligibility ficheEligibility;
        private SubsetKey thesaurusKey;
        private Croisement[] croisements;
        private Corpus[] corpusArray;
        private boolean withIdalpha;
        private boolean isBabelien;
        private int thesaurusLangInteger;
        private MotcleStats motcleStats;

        StatsThesaurusXMLWriter(Thesaurus thesaurus, FicheEligibility ficheEligibility, int thesaurusLangInteger) {
            this.thesaurus = thesaurus;
            this.ficheEligibility = ficheEligibility;
            thesaurusKey = thesaurus.getSubsetKey();
            ThesaurusMetadata thesaurusMetaData = thesaurus.getThesaurusMetadata();
            short thesaurusType = thesaurusMetaData.getThesaurusType();
            withIdalpha = (thesaurusType == ThesaurusMetadata.IDALPHA_TYPE);
            isBabelien = (thesaurusType == ThesaurusMetadata.BABELIEN_TYPE);
            this.thesaurusLangInteger = thesaurusLangInteger;
            initCroisements();
            motcleStats = new MotcleStats();
        }

        private void initCroisements() {
            Basedefiches basedefiches = thesaurus.getBasedefiches();
            int corpusCount = basedefiches.getCorpusCount();
            croisements = new Croisement[corpusCount];
            corpusArray = new Corpus[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                Corpus corpus = basedefiches.getCorpus(i);
                Croisement croisement = basedefiches.getCroisement(corpus.getSubsetKey(), thesaurus.getSubsetKey());
                croisements[i] = croisement;
                corpusArray[i] = corpus;
            }
        }

        public void printTable() throws IOException {
            String thesaurusName = thesaurusKey.getSubsetName();
            openTable(thesaurusName);
            printFirstRow();
            MotcleList motcleList = thesaurus.getFirstLevelList();
            for (int i = 0; i < motcleList.getMotcleCount(); i++) {
                printMotcleRow(motcleList.getMotcle(i));
            }
            closeTable();
        }

        private void printMotcleRow(Motcle motcle) throws IOException {
            motcleStats.updateStats(motcle);
            int total = motcleStats.getTotal();
            openTableRow();
            addIntegerTableCell(motcle.getIdths());
            if (withIdalpha) {
                addStringTableCell(motcle.getIdalpha());
            }
            if (isBabelien) {
                addStringTableCell(motcle.getLibelleString(thesaurusLangInteger, true));
            } else {
                addStringTableCell(motcle.getLibelleString(thesaurusLangInteger, true));
            }
            addIntegerTableCell(total);
            int[] parCroisement = motcleStats.getParCroisement();
            for (int i = 0; i < parCroisement.length; i++) {
                int ct = parCroisement[i];
                if (ct == 0) {
                    addEmptyTableCell();
                } else {
                    addIntegerTableCell(ct);
                }
            }
            closeTableRow();
            MotcleList motcleList = motcle.getChildList();
            for (int i = 0; i < motcleList.getMotcleCount(); i++) {
                printMotcleRow(motcleList.getMotcle(i));
            }
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("idths");
            if (withIdalpha) {
                addStringTableCell("idalpha");
            }
            if (isBabelien) {
                addStringTableCell("lib");
            } else {
                addStringTableCell("lib_" + LangInteger.toString(thesaurusLangInteger));
            }
            addStringTableCell("_total");
            int croisementLength = croisements.length;
            for (int i = 0; i < croisementLength; i++) {
                String s = croisements[i].getSubsetCouple().getSubsetKey1().getSubsetName();
                addStringTableCell(s);
            }
            closeTableRow();
        }


        private class MotcleStats {

            private int total = 0;
            private int[] parCroisement;

            MotcleStats() {
                parCroisement = new int[croisements.length];
            }

            void updateStats(Motcle motcle) {
                total = 0;
                int length = croisements.length;
                for (int i = 0; i < length; i++) {
                    Croisement croisement = croisements[i];
                    CroisementItem ci = croisement.getCroisementItem(thesaurusKey, motcle.getTicket());
                    int ct = 0;
                    int size = ci.getCount();
                    if (ficheEligibility == null) {
                        ct = size;
                    } else {
                        Corpus corpus = corpusArray[i];
                        for (int j = 0; j < size; j++) {
                            AbregeFiche abregeFiche = (AbregeFiche) corpus.getSubsetItemByTicket(ci.getTicket(j));
                            if (abregeFiche != null) {
                                if (ficheEligibility.accept(abregeFiche)) {
                                    ct++;
                                }
                            }
                        }
                    }
                    parCroisement[i] = ct;
                    total = total + ct;
                }
            }

            public int getTotal() {
                return total;
            }

            public int[] getParCroisement() {
                return parCroisement;
            }

        }

    }
}
