/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers.bdfcontent;

import java.io.IOException;
import java.io.OutputStream;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.tableexport.thesaurus.ThesaurusTableExport;
import net.mapeadores.basedefiches.tableexport.thesaurus.ThesaurusTableWriter;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusFieldKey;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetXMLWriter;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetZipEntrySource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.langinteger.LangInteger;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusOdsProducer implements StreamProducer {

    private Thesaurus thesaurus;

    public ThesaurusOdsProducer(Thesaurus thesaurus) {
        this.thesaurus = thesaurus;
    }

    public String getMimeType() {
        return OdUtils.getMimeType(OdUtils.OD_SPREADSHEET);
    }

    public String getCharset() {
        return null;
    }

    public void writeStream(OutputStream outputStream) throws IOException {
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        SpreadSheetZipEntrySource contentEntrySource = new SpreadSheetZipEntrySource(new ThesaurusXMLWriter(thesaurus));
        odZip.setContentZipEntrySource(contentEntrySource);
        odZip.zip(outputStream);
    }


    private static class ThesaurusXMLWriter extends SpreadSheetXMLWriter implements ThesaurusTableWriter {

        private SubsetKey thesaurusKey;
        private ThesaurusTableExport thesaurusTableExport;

        private ThesaurusXMLWriter(Thesaurus thesaurus) {
            thesaurusTableExport = ThesaurusTableExport.getDefault(thesaurus);
            thesaurusKey = thesaurus.getSubsetKey();
        }

        public void printTable() throws IOException {
            String name = thesaurusKey.getSubsetName();
            openTable(name);
            printFirstRow();
            try {
                thesaurusTableExport.export(this);
            } catch (NestedIOException ioe) {
                throw ioe.getIOException();
            }
            closeTable();
        }

        private void printFirstRow() throws IOException {
            ThesaurusFieldKey[] thesaurusFieldKeyArray = thesaurusTableExport.getThesaurusFieldKeyArray();
            openTableRow();
            for (int i = 0; i < thesaurusFieldKeyArray.length; i++) {
                addStringTableCell(thesaurusFieldKeyArray[i].toString());
            }
            closeTableRow();
        }

        public void startRow(Motcle motcle) {
            try {
                openTableRow();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addIdthsCell(ThesaurusFieldKey thesaurusFieldKey, Integer itg) {
            try {
                if (itg != null) {
                    addIntegerTableCell(itg.intValue());
                } else {
                    addEmptyTableCell();
                }
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addIdalphaCell(ThesaurusFieldKey thesaurusFieldKey, String s) {
            try {
                if (s != null) {
                    addStringTableCell(s);
                } else {
                    addEmptyTableCell();
                }
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addLibCell(ThesaurusFieldKey thesaurusFieldKey, String s) {
            try {
                if (s != null) {
                    addStringTableCell(s);
                } else {
                    addEmptyTableCell();
                }
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void addBabelienLangCell(int langInteger) {
            try {
                String lg = LangInteger.toString(langInteger);
                addStringTableCell(lg);
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

        public void endRow() {
            try {
                closeTableRow();
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        }

    }

}
