/* BdfServer_Get - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers.bdfcontent;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.ui.ComponentUI;
import fr.exemole.bdfserver.ui.corpus.CorpusUI;
import fr.exemole.bdfserver.ui.corpus.IncludeUI;
import java.io.IOException;
import java.io.OutputStream;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.Intitule;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadata;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.metadata.BasedefichesMetadata;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.sphere.metadata.SphereMetadata;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.metadata.ThesaurusMetadata;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetXMLWriter;
import net.mapeadores.basedefiches.tools.odsexport.SpreadSheetZipEntrySource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class UiOdsProducer implements StreamProducer {

    private BdfServer bdfServer;

    public UiOdsProducer(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
    }

    public String getMimeType() {
        return OdUtils.getMimeType(OdUtils.OD_SPREADSHEET);
    }

    public String getCharset() {
        return null;
    }

    public void writeStream(OutputStream outputStream) throws IOException {
        OdZip odZip = new OdZip();
        odZip.setOdtype(OdUtils.OD_SPREADSHEET);
        odZip.setContentZipEntrySource(new SpreadSheetZipEntrySource(new UiXMLWriter()));
        odZip.zip(outputStream);
    }


    private class UiXMLWriter extends SpreadSheetXMLWriter {

        private UIManager uiManager;
        private Basedefiches basedefiches;
        private int[] langIntegerArray;

        private UiXMLWriter() {
            uiManager = bdfServer.getUIManager();
            basedefiches = bdfServer.getBasedefiches();
            langIntegerArray = bdfServer.getConfigurationManager().getLangConfig().getWorkingLangIntegerArray();
        }

        public void printTable() throws IOException {
            openTable("libs");
            printFirstRow();
            printMetadata();
            printThesaurus();
            printSphere();
            printCorpus();
            closeTable();
        }

        private void printMetadata() throws IOException {
            BasedefichesMetadata metadata = basedefiches.getBasedefichesMetadata();
            int intituleCount = metadata.getIntituleCount();
            for (int i = 0; i < intituleCount; i++) {
                Intitule intitule = metadata.getIntitule(i);
                String key = "basedefiches/intitule_" + intitule.getName();
                printRow(key, intitule);
            }
        }

        private void printThesaurus() throws IOException {
            int thesaurusCount = basedefiches.getThesaurusCount();
            for (int j = 0; j < thesaurusCount; j++) {
                Thesaurus thesaurus = basedefiches.getThesaurus(j);
                String rootKey = thesaurus.getSubsetKey().toString();
                ThesaurusMetadata metadata = thesaurus.getThesaurusMetadata();
                int intituleCount = metadata.getIntituleCount();
                for (int i = 0; i < intituleCount; i++) {
                    Intitule intitule = metadata.getIntitule(i);
                    String key = rootKey + "/intitule_" + intitule.getName();
                    printRow(key, intitule);
                }
            }
        }

        private void printSphere() throws IOException {
            int sphereCount = basedefiches.getSphereCount();
            for (int j = 0; j < sphereCount; j++) {
                Sphere sphere = basedefiches.getSphere(j);
                String rootKey = sphere.getSubsetKey().toString();
                SphereMetadata metadata = sphere.getSphereMetadata();
                int intituleCount = metadata.getIntituleCount();
                for (int i = 0; i < intituleCount; i++) {
                    Intitule intitule = metadata.getIntitule(i);
                    String key = rootKey + "/intitule_" + intitule.getName();
                    printRow(key, intitule);

                }
            }
        }

        private void printCorpus() throws IOException {
            int corpusCount = basedefiches.getCorpusCount();
            for (int i = 0; i < corpusCount; i++) {
                printCorpus(basedefiches.getCorpus(i));
            }
        }

        private void printCorpus(Corpus corpus) throws IOException {
            String rootKey = corpus.getSubsetKey().toString();
            CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
            int intituleCount = corpusMetadata.getIntituleCount();
            for (int i = 0; i < intituleCount; i++) {
                Intitule intitule = corpusMetadata.getIntitule(i);
                String key = rootKey + "/intitule_" + intitule.getName();
                printRow(key, intitule);

            }
            for (CorpusField corpusField : CorpusMetadataUtils.getCorpusFieldList(corpusMetadata)) {
                String key = rootKey + "/" + corpusField.getFieldKey().toString();
                printRow(key, corpusField);
            }
            CorpusUI corpusUI = uiManager.getCorpusUI(corpus);
            int count = corpusUI.getComponentUICount();
            for (int i = 0; i < count; i++) {
                ComponentUI componentUI = corpusUI.getComponentUI(i);
                if (componentUI instanceof IncludeUI) {
                    IncludeUI includeUI = (IncludeUI) componentUI;
                    if (!includeUI.useDefaultLibelle()) {
                        String key = rootKey + "/" + includeUI.getIncludeKey().toString();
                        printRow(key, includeUI.getCustomLibelle());
                    }
                }
            }
        }

        private void printFirstRow() throws IOException {
            openTableRow();
            addStringTableCell("key");
            for (int i = 0; i < langIntegerArray.length; i++) {
                int langInteger = langIntegerArray[i];
                addStringTableCell("lib_" + LangInteger.toString(langInteger));
            }
            closeTableRow();
        }

        private void printRow(String key, LibelleHolder libelleHolder) throws IOException {
            openTableRow();
            addStringTableCell(key);
            for (int i = 0; i < langIntegerArray.length; i++) {
                int langInteger = langIntegerArray[i];
                Libelle libelle = libelleHolder.getLibelleByLang(langInteger);
                if (libelle == null) {
                    addEmptyTableCell();
                } else {
                    addStringTableCell(libelle.getLibelleString());
                }
            }
            closeTableRow();
        }

    }

}
