/* BdfServer_Html - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html;

import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.html.catalogs.GlobalCatalog;
import fr.exemole.bdfserver.html.utils.SubsetGroupListPrinter;
import fr.exemole.bdfserver.managers.UIManager;
import fr.exemole.bdfserver.request.RequestUtils;
import fr.exemole.bdfserver.request.TextParser;
import fr.exemole.bdfserver.roles.Role;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.Map;
import net.mapeadores.basedefiches.Subset;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.association.FileLength;
import net.mapeadores.basedefiches.association.Version;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.sphere.Sphere;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.SubmitParameters;
import net.mapeadores.util.html.UtilHtmlPrinter;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.ResourceItem;


/**
 *
 * @author Vincent Calame
 */
public class BdfHtmlUtils implements GlobalCatalog {

    private BdfHtmlUtils() {
    }

    public static void printTiret(HtmlPrinter htmlPrinter) {
        htmlPrinter.append(" – ");
    }

    public static void printLibColon(HtmlPrinter htmlPrinter, String libKey) {
        htmlPrinter.localize(libKey);
        htmlPrinter.append("&nbsp;: ");
    }

    public static void printMotcleLibelle(BdfHtmlProducer bhp, Motcle motcle) {
        printMotcleLibelle(bhp, motcle, ThesaurusUtils.checkLangIntegerDisponibility(motcle.getThesaurus(), bhp.getWorkingLangInteger()));
    }

    public static void printMotcleLibelle(HtmlPrinter htmlPrinter, Motcle motcle, int langInteger) {
        String idalpha = motcle.getIdalpha();
        if (idalpha != null) {
            htmlPrinter.append(idalpha);
            htmlPrinter.append(" \u2013 ");
        }
        String lib = motcle.getLibelleString(langInteger, ThesaurusUtils.isBabelienType(motcle.getThesaurus()));
        if (lib == null) {
            htmlPrinter.append("?");
        } else {
            htmlPrinter.escape(lib);
        }
        if (idalpha == null) {
            htmlPrinter.startSmall();
            htmlPrinter.append(" (idths=");
            htmlPrinter.append(String.valueOf(motcle.getIdths()));
            htmlPrinter.append(")");
            htmlPrinter.endSmall();
        }
    }

    public static void printRoleIntitule(BdfHtmlProducer bhp, Role role) {
        printRoleIntitule(bhp, role, bhp.getBdfServer(), bhp.getWorkingLangInteger());
    }

    public static void printRoleIntitule(HtmlPrinter htmlPrinter, Role role, BdfServer bdfServer, int workingLangInteger) {
        if (bdfServer.getPermissionManager().isDefaultRole(role)) {
            htmlPrinter.startEm();
            htmlPrinter.localize("admin_lib_defaultrole");
            htmlPrinter.endEm();
        } else {
            htmlPrinter.startCode();
            htmlPrinter.append("[");
            htmlPrinter.append(role.getName());
            htmlPrinter.append("]");
            htmlPrinter.endCode();
        }
        printTiret(htmlPrinter);
        htmlPrinter.escape(role.getRoleLibelleHolder().getLibelleString(workingLangInteger, true));
    }

    public static void startForm(HtmlPrinter htmlPrinter, HTMLAttributes formAttributes, Map<String, String> hiddenParametersMap) {
        htmlPrinter.startForm(formAttributes);
        for (Map.Entry<String, String> entry : hiddenParametersMap.entrySet()) {
            htmlPrinter.appendHiddenInput(entry.getKey(), entry.getValue());
        }
    }

    public static void endForm(HtmlPrinter htmlPrinter, String submitResourceKey) {
        htmlPrinter.startP();
        HTMLAttributes inputAttr = new HTMLAttributes();
        inputAttr.setType("submit");
        inputAttr.setValue(htmlPrinter.getLocalization(submitResourceKey));
        htmlPrinter.appendInput(inputAttr);
        htmlPrinter.endP();
        htmlPrinter.endForm();
    }

    public static void addSubsetParameter(Map<String, String> parametersMap, Subset subset) {
        addSubsetParameter(parametersMap, subset.getSubsetKey());
    }

    public static void addSubsetParameter(Map<String, String> parametersMap, SubsetKey subsetKey) {
        parametersMap.put(SubsetKey.categoryToString(subsetKey.getCategory()), subsetKey.getSubsetName());
    }

    public static void addSubsetItemParameter(Map<String, String> parametersMap, SubsetItem subsetItem) {
        addSubsetItemParameter(parametersMap, subsetItem.getSubset().getSubsetKey().getCategory(), subsetItem.getIdsubset());
    }

    public static void addSubsetItemParameter(Map<String, String> parametersMap, short subsetCategory, int idsubset) {
        parametersMap.put(getDefaultIdsubsetParamName(subsetCategory), String.valueOf(idsubset));
    }

    public static void addPageParameter(Map<String, String> parametersMap, String pageName) {
        parametersMap.put("page", pageName);
    }

    public static void addRedirectParameter(Map<String, String> parametersMap, String redirectPath) {
        parametersMap.put("redirect", redirectPath);
    }

    public static void addErrorPageParameter(Map<String, String> parametersMap, String pageName) {
        parametersMap.put("page-err", pageName);
    }

    public static void addCommandParameter(Map<String, String> parametersMap, String commandName) {
        parametersMap.put("cmd", commandName);
    }

    public static void printFixedFormTitle(UtilHtmlPrinter htmlPrinter, String title) {
        HTMLAttributes enteteAttributes = new HTMLAttributes();
        enteteAttributes.addClass(DIV_GLB_FORMHEADER);
        htmlPrinter.startDiv(enteteAttributes);
        SubmitParameters submitParameters = new SubmitParameters();
        submitParameters.setEncloseInDiv(true);
        HTMLAttributes divSubmitAttributes = new HTMLAttributes();
        divSubmitAttributes.addClass(DIV_GLB_FIXEDSUBMIT);
        submitParameters.setDivAttributes(divSubmitAttributes);
        htmlPrinter.addSubmit(new ResourceItem("global_submit_ok"), submitParameters);
        htmlPrinter.startH1();
        htmlPrinter.escape(title);
        htmlPrinter.endH1();
        htmlPrinter.endDiv();
    }

    public static String getDefaultIdsubsetParamName(short subsetCategory) {
        switch (subsetCategory) {
            case SubsetKey.CATEGORY_CORPUS:
                return "idcorpus";
            case SubsetKey.CATEGORY_THESAURUS:
                return "idths";
            case SubsetKey.CATEGORY_SPHERE:
                return "idsphere";
            case SubsetKey.CATEGORY_ALBUM:
                return "idalbum";
            default:
                throw new SwitchException("Unknown category : " + subsetCategory);
        }
    }

    public static void printChangeSubsetSelect(BdfHtmlProducer bhp, Subset currentSubset, String currentPage) {
        if (!bhp.isWithJQuery()) {
            return;
        }
        UIManager uiManager = bhp.getBdfServer().getUIManager();
        if (currentPage == null) {
            currentPage = "";
        }
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setOnChange("BDF.ChangeSubset(this,'" + currentPage + "');");
        String name = null;
        SubsetGroupList subsetGroupList = null;
        if (currentSubset instanceof Corpus) {
            name = "corpus";
            subsetGroupList = uiManager.getCorpusSubsetKeyTree();
        } else if (currentSubset instanceof Thesaurus) {
            name = "thesaurus";
            subsetGroupList = uiManager.getThesaurusSubsetKeyTree();
        } else if (currentSubset instanceof Sphere) {
            name = "sphere";
            subsetGroupList = uiManager.getSphereSubsetKeyTree();
        }
        selectAtt.setName(name);
        printSubsetSelect(bhp, subsetGroupList, selectAtt, currentSubset.getSubsetKey());

    }

    public static void printSubsetSelect(BdfHtmlProducer bhp, SubsetGroupList subsetGroupList, HTMLAttributes selectAttributes, SubsetKey selectedKey) {
        SubsetGroupListPrinter subsetGroupListPrinter = new SubsetGroupListPrinter(bhp.getBdfServer().getBasedefiches(), bhp, bhp.getWorkingLangInteger());
        subsetGroupListPrinter.setSubsetGroupList(subsetGroupList);
        subsetGroupListPrinter.setOnlySubIdValues(true);
        subsetGroupListPrinter.setSelectedSubsetKey(selectedKey);
        subsetGroupListPrinter.printSelect(selectAttributes);
    }

    public static void printSubsetSelect(BdfHtmlProducer bhp, SubsetGroupList subsetGroupList, String name, SubsetKey selectedKey) {
        HTMLAttributes selectAtt = new HTMLAttributes();
        selectAtt.setName(name);
        printSubsetSelect(bhp, subsetGroupList, selectAtt, selectedKey);
    }

    public static void printLangFieldRow(HtmlPrinter htmlPrinter, String paramPrefix, int langInteger, String value) {
        printLangFieldRow(htmlPrinter, paramPrefix, langInteger, value, 50, 1, true);
    }

    public static void printLangFieldRow(HtmlPrinter htmlPrinter, String paramPrefix, int langInteger, String value, int cols, int rows, boolean typoText) {
        if (typoText) {
            value = TextParser.escapeString(value);
        }
        String langString = LangInteger.toString(langInteger);
        htmlPrinter.startRow();
        if (rows == 1) {
            htmlPrinter.startCell();
            htmlPrinter.append(langString);
            htmlPrinter.endCell();
            htmlPrinter.startCell();
            htmlPrinter.append("&nbsp;:&nbsp;");
            htmlPrinter.endCell();
            htmlPrinter.startCell();
            HTMLAttributes inputAttributes = new HTMLAttributes();
            inputAttributes.setType("text");
            inputAttributes.setSize("50");
            inputAttributes.setName(paramPrefix + langString);
            inputAttributes.setValue(value);
            htmlPrinter.appendInput(inputAttributes);
            htmlPrinter.endCell();
        } else {
            htmlPrinter.startCellWithTopAlign();
            htmlPrinter.append(langString);
            htmlPrinter.endCell();
            htmlPrinter.startCellWithTopAlign();
            htmlPrinter.append("&nbsp;:&nbsp;");
            htmlPrinter.endCell();
            htmlPrinter.startCell();
            HTMLAttributes textAreaAttributes = new HTMLAttributes();
            textAreaAttributes.setCols(cols);
            textAreaAttributes.setRows(rows);
            textAreaAttributes.setName(paramPrefix + langString);
            htmlPrinter.startTextArea(textAreaAttributes);
            htmlPrinter.escape(value, true);
            htmlPrinter.endTextArea();
            htmlPrinter.endCell();
        }
        htmlPrinter.endRow();
    }

    public static void print16pxStaticIcon(HtmlPrinter htmlPrinter, String iconKey, String reversePath) {
        HTMLAttributes imgAtt = new HTMLAttributes();
        String src = "theme/icons/16x16/" + iconKey + ".png";
        if (reversePath.length() > 0) {
            src = reversePath + "/" + src;
        }
        imgAtt.setSrc(src);
        imgAtt.setWidth("16");
        imgAtt.setHeight("16");
        imgAtt.setAlt("");
        htmlPrinter.appendImg(imgAtt);
    }

    public static void printOds(HtmlPrinter htmlPrinter, String href) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setHref(href + ".ods");
        htmlPrinter.append("(");
        htmlPrinter.startA(htmlAttributes);
        htmlPrinter.append("ods");
        htmlPrinter.endA();
        htmlPrinter.append(")");
    }

    public static void addTestJavascript(BdfHtmlProducer bhp) {
        bhp.addJqueryExtensions("form");
        bhp.addJavascriptFiles(GlobalCatalog.TEST_JAVASCRIPT);
    }

    public static void printConfirmPassword(HtmlPrinter htmlPrinter) {
        htmlPrinter.startP();
        htmlPrinter.startEm();
        HtmlPrinterUtils.printLibColon(htmlPrinter, "global_lib_confirmpassword");
        htmlPrinter.endEm();
        htmlPrinter.appendPasswordInput(RequestUtils.CONFIRMPASSWORD_PARAMNAME, 20);
        htmlPrinter.endP();
    }

    /**
     * Les blocs à déployer doivent avoir comme identifiant DEPLOYBLOCK_ + key
     */
    public static void printDeployTitle(BdfHtmlProducer bhp, String key, String locKey) {
        if (!bhp.isWithJQuery()) {
            return;
        }
        String deployIconId = GlobalCatalog.DEPLOYICON_IDPREFIX + key;
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.addClass(DIV_GLB_DEPLOYTITLE);
        bhp.startDiv(divAttributes);
        bhp.startA("javascript:BDF.deploy('" + key + "');");
        HTMLAttributes deployBoxAttributes = new HTMLAttributes();
        deployBoxAttributes.addClass(SPAN_GLB_DEPLOYICON);
        deployBoxAttributes.setId(deployIconId);
        bhp.startSpan(deployBoxAttributes);
        bhp.append("+");
        bhp.endSpan();
        bhp.append(" ");
        bhp.localize(locKey);
        bhp.endA();
        bhp.endDiv();
    }

    public static void startDeployBlock(BdfHtmlProducer bhp, String key) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.addClass(DIV_GLB_DEPLOYBLOCK);
        if (bhp.isWithJQuery()) {
            htmlAttributes.addClass(INVISIBLE);
        }
        htmlAttributes.setId(GlobalCatalog.DEPLOYBLOCK_IDPREFIX + key);
        bhp.startDiv(htmlAttributes);
    }

    public static void printThesaurusLinksDeployBlock(BdfHtmlProducer bhp, String thesaurusSubId, String hrefBase) {
        HTMLAttributes statsAttributes = new HTMLAttributes();
        HTMLAttributes ulAttributes = new HTMLAttributes();
        ulAttributes.setId(GlobalCatalog.DEPLOYBLOCK_IDPREFIX + thesaurusSubId);
        ulAttributes.addClass(UL_GLB_SMALLLINKLIST);
        if (bhp.isWithJQuery()) {
            ulAttributes.addClass(INVISIBLE);
        }
        bhp.startUl(ulAttributes);
        /*appendLi(page, thesaurusSubId, MotsclesInstruction.STATS_OPTION, "thesaurus_link_stats", hrefBase);
        appendLi(page, thesaurusSubId, MotsclesInstruction.STATSSEL_OPTION, "thesaurus_link_statssel", hrefBase);
        appendLi(page, thesaurusSubId, MotsclesInstruction.INVERSE_OPTION, "thesaurus_link_listepermutee", hrefBase);*/
        printLi(bhp, thesaurusSubId, "stats", "thesaurus_link_stats", hrefBase);
        printLi(bhp, thesaurusSubId, "statssel", "thesaurus_link_statssel", hrefBase);
        printLi(bhp, thesaurusSubId, "inverse", "thesaurus_link_listepermutee", hrefBase);
        bhp.endUl();
    }

    private static void printLi(BdfHtmlProducer bhp, String thesaurusSubId, String option, String locKey, String hrefBase) {
        String href = hrefBase + thesaurusSubId + "-" + option;
        HTMLAttributes aAttributes = new HTMLAttributes();
        aAttributes.setHref(href + ".html");
        aAttributes.setTarget("Saisie");
        bhp.startLi();
        bhp.startA(aAttributes);
        bhp.localize(locKey);
        bhp.endA();
        bhp.space();
        printOds(bhp, href);
        bhp.endLi();
    }

    public static void appendCommandLink(BdfHtmlProducer bhp, String href, String commandKey, String target) {
        bhp.startA(href, target);
        printCommandLocString(bhp, commandKey);
        bhp.endA();
    }

    public static void printCommandLocString(BdfHtmlProducer bhp, String commandKey) {
        bhp.escape(bhp.getCommandKeyLocalisation().toString(commandKey, null));
    }

    public static void printDocumentVersionSize(BdfHtmlProducer bhp, Version version) {
        FileLength fileLength = version.getFileLength();
        Locale formatLocale = bhp.getLocalisationInfo().getFormatLocale();
        NumberFormat format = NumberFormat.getInstance(formatLocale);
        float value = fileLength.getRoundedValue();
        bhp.escape(format.format(value));
        bhp.append("&nbsp;");
        if (fileLength.getRoundType() == FileLength.KB_ROUND) {
            bhp.escape(LocalisationUtils.getKibiOctet(formatLocale));
        } else {
            bhp.escape(LocalisationUtils.getMebiOctet(formatLocale));
        }
    }

}
