/* BdfServer_Html - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.request.form.GeopointSubfieldsElement;
import fr.exemole.bdfserver.request.form.HiddenFieldElement;
import fr.exemole.bdfserver.request.form.ImageSubfieldsElement;
import fr.exemole.bdfserver.request.form.ItemFieldElement;
import fr.exemole.bdfserver.request.form.LangFieldElement;
import fr.exemole.bdfserver.request.form.MontantSubfieldsElement;
import fr.exemole.bdfserver.request.form.PersonneSubfieldsElement;
import fr.exemole.bdfserver.request.form.TextFieldElement;
import fr.exemole.bdfserver.ui.options.InputOption;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.basedefiches.corpus.metadata.CorpusField;
import net.mapeadores.basedefiches.corpus.metadata.FieldKey;
import net.mapeadores.basedefiches.corpus.metadata.SubfieldKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.primitives.CurrencyInteger;


/**
 *
 * @author Vincent Calame
 */
public class CorpusFieldFormHtml implements FicheFormCatalog {

    private CorpusFieldFormHtml() {
    }

    public static void printComment(HtmlPrinter htmlPrinter, String comment) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.addClass(FF_COMMENT_CLASS);
        htmlPrinter.startDiv(htmlAttributes);
        htmlPrinter.breakLine();
        htmlPrinter.append(comment);
        htmlPrinter.breakLine();
        htmlPrinter.endDiv();
    }

    public static void printImageSubfields(BdfHtmlProducer bdfHtmlProducer, ImageSubfieldsElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        String name = getName(corpusField, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        String src = formElement.getSrc();
        SubfieldKey srcKey = new SubfieldKey(fieldKey, SubfieldKey.SRC_SUBTYPE);
        SubfieldKey altKey = new SubfieldKey(fieldKey, SubfieldKey.ALT_SUBTYPE);
        SubfieldKey titleKey = new SubfieldKey(fieldKey, SubfieldKey.TITLE_SUBTYPE);
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.startDiv(getChampSpanAttributes());
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setSize("45");
        inputAttributes.setType(HTMLAttributes.TEXT_TYPE);
        bdfHtmlProducer.startTable();
        startSubFieldRow(bdfHtmlProducer, "global_lib_src");
        String srcName = getPrefixedName(srcKey.toString(), namePrefix);
        inputAttributes.setName(srcName);
        inputAttributes.setId(getChampId(srcName));
        inputAttributes.setValue(src);
        bdfHtmlProducer.appendInput(inputAttributes);
        endSubfieldRow(bdfHtmlProducer);
        String altString = formElement.getAlt();
        startSubFieldRow(bdfHtmlProducer, "global_lib_alt");
        String altName = getPrefixedName(altKey.toString(), namePrefix);
        inputAttributes.setName(altName);
        inputAttributes.setId(getChampId(altName));
        inputAttributes.setValue(altString);
        bdfHtmlProducer.appendInput(inputAttributes);
        endSubfieldRow(bdfHtmlProducer);
        String titleString = formElement.getTitle();
        startSubFieldRow(bdfHtmlProducer, "global_lib_title");
        String titleName = getPrefixedName(titleKey.toString(), namePrefix);
        inputAttributes.setName(titleName);
        inputAttributes.setId(getChampId(titleName));
        inputAttributes.setValue(titleString);
        bdfHtmlProducer.appendInput(inputAttributes);
        endSubfieldRow(bdfHtmlProducer);
        bdfHtmlProducer.endTable();
        bdfHtmlProducer.endDiv();
        bdfHtmlProducer.endDiv();
    }

    public static void printLangField(BdfHtmlProducer bdfHtmlProducer, LangFieldElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getName(corpusField, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        int langInteger = formElement.getLangInteger();
        String langString = LangInteger.toString(langInteger);
        short langFieldType = formElement.getType();
        if (langFieldType == LangFieldElement.UNIQUE_TYPE) {
            bdfHtmlProducer.startDiv(null);
            bdfHtmlProducer.appendHiddenInput(name, langString);
            bdfHtmlProducer.endDiv();
            return;
        }
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        if (langFieldType == LangFieldElement.ANY_TYPE) {
            blocAttributes.addClass(FF_XSMALL);
        } else {
            blocAttributes.addClass(FF_XSMALL);
        }
        if ((bdfHtmlProducer.isWithJQuery()) && (langFieldType == LangFieldElement.ANY_TYPE)) {
            blocAttributes.addClass(JQ_FF_LANGUE);
        }
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setName(name);
        inputAttributes.setId(getChampId(name));
        if (langFieldType == LangFieldElement.ARRAY_TYPE) {
            int[] array = formElement.getAvailableLangArray();
            bdfHtmlProducer.startSelect(inputAttributes);
            for (int i = 0; i < array.length; i++) {
                int c = array[i];
                String currLangString = LangInteger.toString(c);
                bdfHtmlProducer.startOption(currLangString, (c == langInteger));
                bdfHtmlProducer.append(currLangString);
                bdfHtmlProducer.endOption();
            }
            bdfHtmlProducer.endSelect();
        } else if (langFieldType == LangFieldElement.ANY_TYPE) {
            inputAttributes.setType("text");
            inputAttributes.setSize("4");
            inputAttributes.setValue(langString);
            inputAttributes.addClass(FF_COMPLET);
            bdfHtmlProducer.appendInput(inputAttributes);
        } else {
            throw new SwitchException("Wrong type = " + formElement.getType());
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        if ((bdfHtmlProducer.isWithJQuery()) && (langFieldType == LangFieldElement.ANY_TYPE)) {
            bdfHtmlProducer.startScript(null);
            CommonFormHtml.printFFCoreLimiteValue(bdfHtmlProducer, name, "1");
            bdfHtmlProducer.breakLine();
            bdfHtmlProducer.endScript();
        }
        bdfHtmlProducer.endDiv();
    }

    public static void printHiddenField(HtmlPrinter htmlPrinter, HiddenFieldElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getName(corpusField, namePrefix);
        htmlPrinter.startDiv(null);
        htmlPrinter.appendHiddenInput(name, formElement.getValue());
        htmlPrinter.endDiv();
    }

    public static void printItemField(BdfHtmlProducer bdfHtmlProducer, ItemFieldElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getName(corpusField, namePrefix);
        String piocheClass = null;
        if (bdfHtmlProducer.isWithJQuery()) {
            piocheClass = getPiocheClass(corpusField);
        }
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        int defaultSize = 0;
        switch (formElement.getWidthType()) {
            case InputOption.WIDTH_SMALL:
                blocAttributes.addClass(FF_SMALL);
                defaultSize = 21;
                break;
            case InputOption.WIDTH_MEDIUM:
                blocAttributes.addClass(FF_MEDIUM);
                defaultSize = 39;
                break;
            case InputOption.WIDTH_LARGE:
                blocAttributes.addClass(FF_LARGE);
                defaultSize = 77;
                break;
        }
        if (piocheClass != null) {
            blocAttributes.addClass(piocheClass);
        }
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setName(name);
        inputAttributes.setId(getChampId(name));
        inputAttributes.addClass(FF_COMPLET);
        int rows = formElement.getRows();
        if (rows == 1) {
            inputAttributes.setType("text");
            inputAttributes.setSize(String.valueOf(defaultSize));
            inputAttributes.setValue(formElement.getValue());
            bdfHtmlProducer.appendInput(inputAttributes);
        } else {
            inputAttributes.setCols(defaultSize);
            inputAttributes.setRows(rows);
            bdfHtmlProducer.startTextArea(inputAttributes);
            bdfHtmlProducer.escape(formElement.getValue(), true);
            bdfHtmlProducer.endTextArea();
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        if (piocheClass != null) {
            bdfHtmlProducer.startScript(null);
            SubsetKey subsetKey = formElement.getSphereKey();

            if (subsetKey != null) {
                CommonFormHtml.printFFCoreParamValue(bdfHtmlProducer, name, subsetKey.toString());
            }
            CommonFormHtml.printFFCoreLimiteValue(bdfHtmlProducer, name, getPiocheLimite(corpusField));
            bdfHtmlProducer.breakLine();
            bdfHtmlProducer.endScript();
        }
        bdfHtmlProducer.endDiv();
    }

    public static void printCurrencyArrayField(BdfHtmlProducer bdfHtmlProducer, MontantSubfieldsElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getName(corpusField, namePrefix);
        String currentCur = formElement.getCur();
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        FieldKey fieldKey = corpusField.getFieldKey();
        SubfieldKey curKey = new SubfieldKey(fieldKey, SubfieldKey.CUR_SUBTYPE);
        SubfieldKey numKey = new SubfieldKey(fieldKey, SubfieldKey.VALUE_SUBTYPE);
        if (formElement.isUnique()) {
            bdfHtmlProducer.startEm();
            bdfHtmlProducer.append(currentCur);
            bdfHtmlProducer.endEm();
            bdfHtmlProducer.appendHiddenInput(curKey.toString(), currentCur);
        } else {
            boolean inSelection = false;
            HTMLAttributes selectAttributes = new HTMLAttributes();
            selectAttributes.setName(getPrefixedName(curKey.toString(), namePrefix));
            HTMLAttributes htmlAttributes = new HTMLAttributes();
            int[] currencyArray = formElement.getCurrencyArray();
            bdfHtmlProducer.startSelect(selectAttributes);
            for (int i = 0; i < currencyArray.length; i++) {
                int currencyInteger = currencyArray[i];
                String curString = CurrencyInteger.toString(currencyInteger);
                boolean selected = curString.equals(formElement.getCur());
                htmlAttributes.setSelected(selected);
                htmlAttributes.setValue(curString);
                if (selected) {
                    inSelection = true;
                }
                bdfHtmlProducer.startOption(htmlAttributes);
                bdfHtmlProducer.append(curString);
                bdfHtmlProducer.endOption();
            }
            if (!inSelection) {
                htmlAttributes.setSelected(true);
                htmlAttributes.setValue(formElement.getCur());
                bdfHtmlProducer.startOption(htmlAttributes);
                bdfHtmlProducer.append("?");
                bdfHtmlProducer.append(formElement.getCur());
                bdfHtmlProducer.append("?");
                bdfHtmlProducer.endOption();
            }
            bdfHtmlProducer.endSelect();
        }
        bdfHtmlProducer.space();
        bdfHtmlProducer.appendMediumInput(getPrefixedName(numKey.toString(), namePrefix), formElement.getNum());
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
    }

    public static void printGeopointSubfields(BdfHtmlProducer page, GeopointSubfieldsElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getName(corpusField, namePrefix);
        FieldKey fieldKey = corpusField.getFieldKey();
        boolean isMandatory = formElement.isMandatory();
        SubfieldKey latitudeKey = new SubfieldKey(fieldKey, SubfieldKey.LATITUDE_SUBTYPE);
        SubfieldKey longitudeKey = new SubfieldKey(fieldKey, SubfieldKey.LONGITUDE_SUBTYPE);

        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        blocAttributes.addClass(JQ_FF_GEOCODAGE);
        page.startDiv(blocAttributes);

        printIntituleSpans(page, formElement.getIntitule());

        page.startDiv(getChampSpanAttributes());

        page.startTable();
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setSize("15");
        inputAttributes.setType("text");

        startSubFieldRow(page, "global_lib_latitude");
        String latName = getPrefixedName(latitudeKey.toString(), namePrefix);
        inputAttributes.setName(latName);
        inputAttributes.setId(getChampId(latName));
        inputAttributes.setValue(formElement.getLatitude());
        page.appendInput(inputAttributes);
        endSubfieldRow(page);

        startSubFieldRow(page, "global_lib_longitude");
        String lonName = getPrefixedName(longitudeKey.toString(), namePrefix);
        inputAttributes.setName(lonName);
        inputAttributes.setId(getChampId(lonName));
        inputAttributes.setValue(formElement.getLongitude());
        page.appendInput(inputAttributes);
        endSubfieldRow(page);
        page.endTable();
        page.endDiv();

        if (page.isWithJQuery()) {
            String[] array = formElement.getAdresseFieldNameArray();
            if (array != null) {
                page.startScript(null);
                page.breakLine();
                for (int i = 0; i < array.length; i++) {
                    page.appendScriptLine("FF_Core.addParamInArray(\"" + name + "\",\"" + getPrefixedName(array[i], namePrefix) + "\");");
                }
                page.endScript();
            }
        }
        page.endDiv();
    }

    public static void printPersonneSubfields(BdfHtmlProducer bdfHtmlProducer, PersonneSubfieldsElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        FieldKey fieldKey = corpusField.getFieldKey();
        boolean isMandatory = formElement.isMandatory();
        boolean withOriginal = formElement.isWithOriginal();
        if (formElement.getOriginal().length() > 0) {
            withOriginal = true;
        }
        SubfieldKey nomKey = new SubfieldKey(fieldKey, SubfieldKey.NOM_SUBTYPE);
        SubfieldKey prenomKey = new SubfieldKey(fieldKey, SubfieldKey.PRENOM_SUBTYPE);
        SubfieldKey originalKey = new SubfieldKey(fieldKey, SubfieldKey.ORIGINAL_SUBTYPE);
        SubfieldKey nomAvantKey = new SubfieldKey(fieldKey, SubfieldKey.NOMAVANT_SUBTYPE);
        HTMLAttributes blocAttributes;
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setSize("30");
        inputAttributes.setType("text");
        inputAttributes.addClass(FF_COMPLET);
        String nomName = getPrefixedName(nomKey.toString(), namePrefix);
        blocAttributes = getBlocAttributes(nomName, isMandatory);
        blocAttributes.addClass(FF_MEDIUM);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, bdfHtmlProducer.getLocalization((withOriginal) ? "sphere_lib_nom_latin" : "sphere_lib_nom"));
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        inputAttributes.setName(nomName);
        inputAttributes.setId(getChampId(nomName));
        inputAttributes.setValue(formElement.getNom());
        bdfHtmlProducer.appendInput(inputAttributes);
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.endDiv();
        String prenomName = getPrefixedName(prenomKey.toString(), namePrefix);
        blocAttributes = getBlocAttributes(prenomName, isMandatory);
        blocAttributes.addClass(FF_MEDIUM);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, bdfHtmlProducer.getLocalization((withOriginal) ? "sphere_lib_prenom_latin" : "sphere_lib_prenom"));
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        inputAttributes.setName(prenomName);
        inputAttributes.setId(getChampId(prenomName));
        inputAttributes.setValue(formElement.getPrenom());
        bdfHtmlProducer.appendInput(inputAttributes);
        if ((!formElement.isWithoutNomAvant()) || (formElement.isNomAvant())) {
            bdfHtmlProducer.appendBr();
            String avantName = getPrefixedName(nomAvantKey.toString(), namePrefix);
            HTMLAttributes avantAttributes = new HTMLAttributes();
            avantAttributes.setName(avantName);
            avantAttributes.setId(getChampId(avantName));
            avantAttributes.setValue("1");
            avantAttributes.setChecked(formElement.isNomAvant());
            HTMLAttributes cbSpanAttributes = new HTMLAttributes();
            cbSpanAttributes.addClass(FF_CHECKBOX_CLASS);
            bdfHtmlProducer.startSpan(cbSpanAttributes);
            bdfHtmlProducer.addCheck(avantAttributes, "sphere_lib_nomavant", null);
            bdfHtmlProducer.endSpan();
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.endDiv();
        if (withOriginal) {
            String originalName = getPrefixedName(originalKey.toString(), namePrefix);
            blocAttributes = getBlocAttributes(originalName, isMandatory);
            blocAttributes.addClass(FF_MEDIUM);
            bdfHtmlProducer.startDiv(blocAttributes);
            printIntituleSpans(bdfHtmlProducer, bdfHtmlProducer.getLocalization("sphere_lib_original"));
            bdfHtmlProducer.startSpan(getChampSpanAttributes());
            inputAttributes.setName(originalName);
            inputAttributes.setId(getChampId(originalName));
            inputAttributes.setValue(formElement.getOriginal());
            bdfHtmlProducer.appendInput(inputAttributes);
            bdfHtmlProducer.endSpan();
            bdfHtmlProducer.endDiv();
        }
    }

    public static void printTextField(BdfHtmlProducer bdfHtmlProducer, TextFieldElement formElement, String namePrefix) {
        CorpusField corpusField = formElement.getCorpusField();
        String name = getName(corpusField, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes divAttributes = new HTMLAttributes();
        HTMLAttributes blocAttributes = new HTMLAttributes();
        blocAttributes.setId(ID_FF_BLOC_PREFIX + name);
        blocAttributes.addClass(FF_TEXTFIELD);
        blocAttributes.addClass(JQ_FF_TEXTE);
        if (isMandatory) {
            blocAttributes.addClass(JQ_FF_MANDATORY);
        }
        bdfHtmlProducer.startDiv(blocAttributes);
        bdfHtmlProducer.breakLine();
        divAttributes.setClasses(FF_TEXTFIELD_INTITULE);
        bdfHtmlProducer.startDiv(divAttributes);
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.addClass(JQ_FF_INTITULE);
        bdfHtmlProducer.startSpan(spanAttributes);
        bdfHtmlProducer.escape(formElement.getIntitule());
        bdfHtmlProducer.endSpan();
        spanAttributes.setClasses(FF_TEXTFIELD_DEUXPOINTS);
        bdfHtmlProducer.startSpan(spanAttributes);
        HtmlPrinterUtils.printColon(bdfHtmlProducer);
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.endDiv();
        divAttributes.addClass(FF_TEXTFIELD_CHAMP);
        divAttributes.addClass(JQ_FF_TEXTEBLOC);
        bdfHtmlProducer.startDiv(divAttributes);
        bdfHtmlProducer.breakLine();
        HTMLAttributes textAreaAttributes = new HTMLAttributes();
        textAreaAttributes.setCols(77);
        textAreaAttributes.setRows(formElement.getRows());
        textAreaAttributes.setName(name);
        textAreaAttributes.setId(getChampId(name));
        bdfHtmlProducer.startTextArea(textAreaAttributes);
        bdfHtmlProducer.escape(formElement.getFormattedText(), true);
        bdfHtmlProducer.endTextArea();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
        bdfHtmlProducer.endDiv();
    }

    private static String getName(CorpusField corpusField, String namePrefix) {
        FieldKey fieldKey = corpusField.getFieldKey();
        String name = fieldKey.toString();
        return getPrefixedName(name, namePrefix);
    }

    private static String getPrefixedName(String name, String namePrefix) {
        if ((namePrefix != null) && (namePrefix.length() > 0)) {
            name = namePrefix + "_" + name;
        }
        return name;
    }

    public static void printCommonLocValues(HtmlPrinter htmlPrinter) {
        htmlPrinter.startScript(null);
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_area_increase");
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_area_decrease");
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_zoomedit");
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_pioche_langue");
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_pioche_pays");
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_pioche_redacteur");
        CommonFormHtml.printFFCoreLocValue(htmlPrinter, "global_link_pioche_geocodage");
        htmlPrinter.append("\n");
        htmlPrinter.endScript();
    }

    private static String getPiocheClass(CorpusField corpusField) {
        switch (corpusField.getFicheItemType()) {
            case CorpusField.PERSONNE_FIELD:
                return JQ_FF_SPHERE;
            case CorpusField.LANGUE_FIELD:
                return JQ_FF_LANGUE;
            case CorpusField.PAYS_FIELD:
                return JQ_FF_PAYS;
            default:
                return null;
        }
    }

    private static String getPiocheLimite(CorpusField corpusField) {
        if (corpusField.getFieldKey().isProprieteFieldKey()) {
            return "1";
        } else {
            return "-1";
        }
    }

    private static HTMLAttributes getBlocAttributes(String name, boolean isMandatory) {
        name = name.replace(':', '_');
        HTMLAttributes blocAttributes = new HTMLAttributes();
        blocAttributes.setId(ID_FF_BLOC_PREFIX + name);
        blocAttributes.addClass(FF_ITEMFIELD);
        if (isMandatory) {
            blocAttributes.addClass(JQ_FF_MANDATORY);
        }
        return blocAttributes;
    }

    public static HTMLAttributes getChampSpanAttributes() {
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.setClasses(FF_ITEMFIELD_CHAMP);
        return spanAttributes;
    }

    private static String getChampId(String name) {
        name = name.replace(':', '_');
        return ID_FF_CHAMP_PREFIX + name;
    }

    private static void startSubFieldRow(HtmlPrinter htmlPrinter, String libKey) {
        htmlPrinter.startRow();
        htmlPrinter.startCell();
        htmlPrinter.doublespace();
        htmlPrinter.startSmall();
        htmlPrinter.localize(libKey);
        htmlPrinter.endSmall();
        htmlPrinter.endCell();
        htmlPrinter.startCell();
    }

    private static void endSubfieldRow(HtmlPrinter htmlPrinter) {
        htmlPrinter.endCell();
        htmlPrinter.endRow();
    }

    public static void printIntituleSpans(HtmlPrinter htmlPrinter, String intitule) {
        HTMLAttributes spanAttributes = new HTMLAttributes();
        htmlPrinter.breakLine();
        spanAttributes.addClass(FF_ITEMFIELD_INTITULE);
        spanAttributes.addClass(JQ_FF_INTITULE);
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.escape(intitule);
        htmlPrinter.endSpan();
        htmlPrinter.breakLine();
        spanAttributes.setClasses(FF_ITEMFIELD_DEUXPOINTS);
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.append(":");
        htmlPrinter.endSpan();
        htmlPrinter.breakLine();
    }

}

