/* BdfServer_Html - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.html.BdfHtmlProducer;
import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.html.catalogs.FicheFormCatalog;
import fr.exemole.bdfserver.request.form.AlbumIncludeElement;
import fr.exemole.bdfserver.request.form.EditableAssociationElement;
import fr.exemole.bdfserver.request.form.HiddenIncludeElement;
import fr.exemole.bdfserver.request.form.CorpusIncludeElement;
import fr.exemole.bdfserver.request.form.ListThesaurusIncludeElement;
import fr.exemole.bdfserver.request.form.NotEditableAssociationElement;
import fr.exemole.bdfserver.request.form.NotEditableIndexationElement;
import fr.exemole.bdfserver.request.form.RattachementToMasterElement;
import fr.exemole.bdfserver.request.form.RattachementToSlaveElement;
import fr.exemole.bdfserver.request.form.TextThesaurusIncludeElement;
import fr.exemole.bdfserver.ui.options.InputOption;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.SubsetItem;
import net.mapeadores.basedefiches.album.AlbumUtils;
import net.mapeadores.basedefiches.album.Illustration;
import net.mapeadores.basedefiches.association.Document;
import net.mapeadores.basedefiches.corpus.AbregeFiche;
import net.mapeadores.basedefiches.corpus.Corpus;
import net.mapeadores.basedefiches.corpus.metadata.CorpusMetadataUtils;
import net.mapeadores.basedefiches.croisement.DocumentCroisementList;
import net.mapeadores.basedefiches.croisement.FicheCroisementList;
import net.mapeadores.basedefiches.include.IncludeKey;
import net.mapeadores.basedefiches.thesaurus.Motcle;
import net.mapeadores.basedefiches.thesaurus.Thesaurus;
import net.mapeadores.basedefiches.thesaurus.ThesaurusUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CheckboxPrinter;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.InputListPrinter;
import net.mapeadores.util.html.RadioPrinter;
import net.mapeadores.util.html.SelectPrinter;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class IncludeFormHtml implements FicheFormCatalog {

    private IncludeFormHtml() {
    }

    public static void printFFCoreLocValues(BdfHtmlProducer bdfHtmlProducer) {
        Basedefiches basedefiches = bdfHtmlProducer.getBdfServer().getBasedefiches();
        int workingLangInteger = bdfHtmlProducer.getWorkingLangInteger();
        bdfHtmlProducer.startScript(null);
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "global_link_pioche_fiche_all");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "global_link_poids_on");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "global_link_poids_off");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "global_link_illustration_create");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "global_link_illustration_change");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "global_link_illustration_remove");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "album_info_removewarning_illustration");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "album_info_removewarning_tmpfile");
        CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, "album_info_tmpfile");
        int thesaurusCount = basedefiches.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            Thesaurus thesaurus = basedefiches.getThesaurus(i);
            String intitule = ThesaurusUtils.getThesaurusIntitule(thesaurus, workingLangInteger);
            CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, thesaurus.getSubsetKey().toString(), "global_link_pioche_fiche_one", intitule);
        }
        int corpusCount = basedefiches.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            Corpus corpus = basedefiches.getCorpus(i);
            String intitule = CorpusMetadataUtils.getCorpusIntituleString(corpus, workingLangInteger);
            CommonFormHtml.printFFCoreLocValue(bdfHtmlProducer, corpus.getSubsetKey().toString(), "global_link_pioche_fiche_one", intitule);
        }
        bdfHtmlProducer.append("\n");
        bdfHtmlProducer.endScript();
    }

    public static void printNotEditableAssociation(BdfHtmlProducer bdfHtmlProducer, NotEditableAssociationElement formElement) {
        DocumentCroisementList documentCroisementList = formElement.getDocumentCroisementList();
        int length = documentCroisementList.getSubsetItemCount();
        HTMLAttributes blocAttributes = getNotEditableDivAttributes();
        bdfHtmlProducer.startDiv(blocAttributes);
        printNotEditableIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        HTMLAttributes spanAttributes = getNotEditableChampSpanAttributes();
        bdfHtmlProducer.startSpan(spanAttributes);
        if (length == 0) {
            bdfHtmlProducer.startEm();
            bdfHtmlProducer.localize("saisie_info_noassociation");
            bdfHtmlProducer.endEm();
        } else {
            for (int i = 0; i < length; i++) {
                if (i > 0) {
                    bdfHtmlProducer.appendBr();
                }
                printSimpleDocument(bdfHtmlProducer, documentCroisementList.getDocument(i));
            }
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
    }

    public static void printEditableAssociation(BdfHtmlProducer bdfHtmlProducer, EditableAssociationElement formElement, String namePrefix) {
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        blocAttributes.addClass(FF_LARGE);
        blocAttributes.addClass(JQ_FF_ASSOCIATION);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.breakLine();
        DocumentCroisementList documentCroisementList = formElement.getDocumentCroisementList();
        int length = documentCroisementList.getSubsetItemCount();
        HTMLAttributes libSpanAttributes = new HTMLAttributes();
        libSpanAttributes.addClass(FF_ITEMFIELD_CHAMP);
        libSpanAttributes.addClass(FF_SOUSBLOC_INTITULE);
        if (length > 0) {
            HTMLAttributes currentListAttributes = new HTMLAttributes();
            currentListAttributes.addClass(FF_SOUSBLOC);
            currentListAttributes.addClass(FF_LARGE);
            bdfHtmlProducer.startDiv(currentListAttributes);
            bdfHtmlProducer.breakLine();
            printLocSpan(bdfHtmlProducer, "association_lib_file_current", libSpanAttributes);
            bdfHtmlProducer.breakLine();
            bdfHtmlProducer.startDiv(getChampSpanAttributes());
            bdfHtmlProducer.breakLine();
            HTMLAttributes currentAttributes = new HTMLAttributes();
            for (int i = 0; i < length; i++) {
                bdfHtmlProducer.startDiv(currentAttributes);
                printSimpleDocument(bdfHtmlProducer, documentCroisementList.getDocument(i));
                bdfHtmlProducer.endDiv();
            }
            bdfHtmlProducer.endDiv();
            bdfHtmlProducer.endDiv();
        }
        HTMLAttributes newListAttributes = new HTMLAttributes();
        newListAttributes.addClass(FF_SOUSBLOC);
        newListAttributes.addClass(FF_LARGE);
        if (length > 0) {
            newListAttributes.addClass(FF_NEXT);
        }
        bdfHtmlProducer.startDiv(newListAttributes);
        bdfHtmlProducer.breakLine();
        String key = (length == 0) ? "association_lib_file_new" : "association_lib_file_update";
        printLocSpan(bdfHtmlProducer, key, libSpanAttributes);
        bdfHtmlProducer.breakLine();
        HTMLAttributes fileInputAttributes = new HTMLAttributes();
        fileInputAttributes.addClass(FF_COMPLET);
        fileInputAttributes.setType("file");
        fileInputAttributes.setSize("35");
        fileInputAttributes.setName(name);
        bdfHtmlProducer.startDiv(getChampSpanAttributes());
        bdfHtmlProducer.appendInput(fileInputAttributes);
        bdfHtmlProducer.endDiv();
        bdfHtmlProducer.endDiv();
        bdfHtmlProducer.endDiv();
    }

    private static void printLocSpan(HtmlPrinter bdfHtmlProducer, String key, HTMLAttributes spanAttributes) {
        bdfHtmlProducer.startSpan(spanAttributes);
        bdfHtmlProducer.localize(key);
        bdfHtmlProducer.endSpan();
    }

    public static void printHiddenInclude(BdfHtmlProducer bdfHtmlProducer, HiddenIncludeElement formElement, String namePrefix) {
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        bdfHtmlProducer.startDiv(null);
        bdfHtmlProducer.appendHiddenInput(name, formElement.getValue());
        bdfHtmlProducer.endDiv();
    }

    public static void printNotEditableIndexation(BdfHtmlProducer bdfHtmlProducer, NotEditableIndexationElement formElement, String namePrefix) {
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        HTMLAttributes blocAttributes = getBlocAttributes(name, false);
        blocAttributes.addClass(FF_LARGE);
        bdfHtmlProducer.startDiv(blocAttributes);
        printNotEditableIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        HTMLAttributes spanAttributes = getNotEditableChampSpanAttributes();
        StringBuilder valueBuffer = new StringBuilder();
        StringBuilder libBuffer = new StringBuilder();
        Set<Motcle> selectedMotcleSet = formElement.getSelectedMotcleSet();
        if (!selectedMotcleSet.isEmpty()) {
            int workingLangInteger = formElement.getWorkingLangInteger();
            String idalphaSeparator = formElement.getIdalphaSeparator();
            boolean premier = true;
            for (Motcle motcle : selectedMotcleSet) {
                if (premier) {
                    premier = false;
                } else {
                    valueBuffer.append(";");
                    libBuffer.append(" ; ");
                }
                String idalpha = motcle.getIdalpha();
                if (idalpha == null) {
                    valueBuffer.append(motcle.getIdths());
                } else {
                    valueBuffer.append(idalpha);
                }
                String lib = motcle.getLibelleString(workingLangInteger, false);
                if ((lib == null) || (lib.length() == 0)) {
                    if (ThesaurusUtils.isBabelienType(motcle.getThesaurus())) {
                        lib = motcle.getLibelleString(0, true);
                    } else if (idalpha != null) {
                        lib = motcle.getIdalpha();
                    } else {
                        lib = String.valueOf(motcle.getIdths());
                    }
                } else if (idalpha != null) {
                    lib = motcle.getIdalpha() + idalphaSeparator + lib;
                }
                libBuffer.append(lib);
            }
        }
        bdfHtmlProducer.appendHiddenInput(name, valueBuffer.toString());
        bdfHtmlProducer.startSpan(spanAttributes);
        bdfHtmlProducer.escape(libBuffer.toString());
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
    }

    public static void printCorpusInclude(BdfHtmlProducer bdfHtmlProducer, CorpusIncludeElement formElement, String namePrefix) {
        boolean globalLiage = formElement.isGlobalLiage();
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        blocAttributes.addClass(FF_LARGE);
        if (globalLiage) {
            blocAttributes.addClass(JQ_FF_LIAGE);
        } else {
            blocAttributes.addClass(JQ_FF_CORPUSINCLUDE);
        }
        if (includeKey.getPoidsFilter() == -1) {
            blocAttributes.addClass(JQ_FF_WITHPOIDS);
        }
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.breakLine();
        HTMLAttributes checkBoxListAttributes = new HTMLAttributes();
        checkBoxListAttributes.addClass(FF_SOUSBLOC);
        checkBoxListAttributes.addClass(FF_LARGE);
        checkBoxListAttributes.addClass(JQ_FF_LISTSOUSBLOC);
        bdfHtmlProducer.startDiv(checkBoxListAttributes);
        bdfHtmlProducer.breakLine();
        int count = formElement.getAbregeFicheCount();
        if (count > 0) {
            bdfHtmlProducer.startDiv(getChampSpanAttributes());
            bdfHtmlProducer.breakLine();
            LocalisationInfo localisationInfo = formElement.getLocalisationInfo();
            HTMLAttributes checkBoxAttributes = new HTMLAttributes();
            checkBoxAttributes.addClass(FF_CHECKBOX_CLASS);
            if (bdfHtmlProducer.isWithJQuery()) {
                HTMLAttributes labelAttributes = new HTMLAttributes();
                HTMLAttributes labelTextSpanAttributes = new HTMLAttributes();
                labelTextSpanAttributes.addClass(JQ_FF_LABELTEXT);
                HTMLAttributes poidsSpanAttributes = new HTMLAttributes();
                poidsSpanAttributes.addClass(JQ_FF_POIDS);
                for (int i = 0; i < count; i++) {
                    AbregeFiche abregeFiche = formElement.getAbregeFiche(i);
                    int poids = formElement.getPoids(i);
                    String value = formElement.getValue(i);
                    bdfHtmlProducer.startDiv(checkBoxAttributes);
                    bdfHtmlProducer.startLabel(labelAttributes);
                    bdfHtmlProducer.appendCheckInput(name, value, true, null);
                    bdfHtmlProducer.startSpan(labelTextSpanAttributes);
                    if (poids > 0) {
                        bdfHtmlProducer.append(" ");
                        bdfHtmlProducer.startSpan(poidsSpanAttributes);
                        if (poids > 1) {
                            bdfHtmlProducer.escape("<");
                            bdfHtmlProducer.append(String.valueOf(poids));
                            bdfHtmlProducer.escape(">");
                        }
                        bdfHtmlProducer.endSpan();
                    }
                    bdfHtmlProducer.append(" ");
                    bdfHtmlProducer.escape(CorpusMetadataUtils.getFicheIntituleString(abregeFiche, localisationInfo));
                    bdfHtmlProducer.endSpan();
                    bdfHtmlProducer.endLabel();
                    bdfHtmlProducer.endDiv();
                }
            } else {
                for (int i = 0; i < count; i++) {
                    AbregeFiche abregeFiche = formElement.getAbregeFiche(i);
                    String value = formElement.getValue(i);
                    bdfHtmlProducer.startDiv(checkBoxAttributes);
                    bdfHtmlProducer.appendSmallestInput(name, value);
                    bdfHtmlProducer.append(" ");
                    bdfHtmlProducer.escape(CorpusMetadataUtils.getFicheIntituleString(abregeFiche, localisationInfo));
                    bdfHtmlProducer.endDiv();
                }
            }
            bdfHtmlProducer.endDiv();
        }
        bdfHtmlProducer.endDiv();
        HTMLAttributes sousblocAttributes = new HTMLAttributes();
        sousblocAttributes.addClass(FF_SOUSBLOC);
        if (globalLiage) {
            sousblocAttributes.addClass(FF_MEDIUM);
        } else {
            sousblocAttributes.addClass(FF_SMALL);
        }
        if (bdfHtmlProducer.isWithJQuery()) {
            sousblocAttributes.addClass(JQ_FF_INPUTSOUSBLOC);
        }
        bdfHtmlProducer.startDiv(sousblocAttributes);
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setName(name);
        inputAttributes.setId(ID_FF_CHAMP_PREFIX + name);
        inputAttributes.addClass(FF_COMPLET);
        int rows = formElement.getRows();
        if (rows == 1) {
            inputAttributes.setType("text");
            inputAttributes.setSize("29");
            bdfHtmlProducer.appendInput(inputAttributes);
        } else {
            inputAttributes.setCols(39);
            inputAttributes.setRows(rows);
            bdfHtmlProducer.startTextArea(inputAttributes);
            bdfHtmlProducer.endTextArea();
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
        if (bdfHtmlProducer.isWithJQuery()) {
            bdfHtmlProducer.startScript(null);
            CommonFormHtml.printFFCoreLimiteValue(bdfHtmlProducer, name, "-1");
            String motcleSelectOptionsString = formElement.getMotcleSelectOptionsString();
            if (motcleSelectOptionsString != null) {
                CommonFormHtml.printFFCoreMotcleSelectOptionsString(bdfHtmlProducer, name, motcleSelectOptionsString);
            }
            bdfHtmlProducer.breakLine();
            bdfHtmlProducer.endScript();
        }
        bdfHtmlProducer.endDiv();
    }

    public static void printAlbumInclude(BdfHtmlProducer bdfHtmlProducer, AlbumIncludeElement formElement, String namePrefix) {
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        blocAttributes.addClass(FF_LARGE);
        blocAttributes.addClass(JQ_FF_ALBUMINCLUDE);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        HTMLAttributes illustrationListAttributes = new HTMLAttributes();
        illustrationListAttributes.addClass(FF_ITEMFIELD_CHAMP);
        illustrationListAttributes.addClass(JQ_FF_ILLUSTRATIONLIST);
        bdfHtmlProducer.startDiv(illustrationListAttributes);
        bdfHtmlProducer.breakLine();
        int illustrationCount = formElement.getIllustrationCount();
        if (illustrationCount > 0) {
            String albumSubsetName = includeKey.getSubsetKey().getSubsetName();
            HTMLAttributes illustrationDivAttributes = new HTMLAttributes();
            illustrationDivAttributes.addClass(FF_ILLUSTRATION_DIV);
            illustrationDivAttributes.addClass(JQ_FF_ILLUSTRATION);
            HTMLAttributes imgAttributes = new HTMLAttributes();
            imgAttributes.addClass(JQ_FF_ILLUSTRATION_IMG);
            HTMLAttributes imgDivAttributes = new HTMLAttributes();
            imgDivAttributes.addClass(FF_ILLUSTRATION_IMG);
            HTMLAttributes uriDivAttributes = new HTMLAttributes();
            uriDivAttributes.addClass(FF_ILLUSTRATION_URI);
            uriDivAttributes.addClass(JQ_FF_ILLUSTRATION_URI);
            for (int i = 0; i < illustrationCount; i++) {
                Illustration illustration = formElement.getIllustration(i);
                int idalbum = illustration.getIdalbum();
                String extension = AlbumUtils.formatTypeToString(illustration.getFormatType());
                illustrationDivAttributes.setId(ID_FF_ILLUSTRATION_PREFIX + name + "__" + idalbum);
                bdfHtmlProducer.startDiv(illustrationDivAttributes);
                imgAttributes.setSrc("illustrations/_mini/" + AlbumUtils.toFileName(illustration));
                bdfHtmlProducer.startDiv(imgDivAttributes);
                bdfHtmlProducer.appendImg(imgAttributes);
                bdfHtmlProducer.endDiv();
                bdfHtmlProducer.startDiv(uriDivAttributes);
                bdfHtmlProducer.append(albumSubsetName + "/" + idalbum);
                bdfHtmlProducer.endDiv();
                bdfHtmlProducer.endDiv();
            }
        }
        bdfHtmlProducer.endDiv();
        HTMLAttributes illustrationInputAttributes = new HTMLAttributes();
        illustrationInputAttributes.addClass(FF_ITEMFIELD_CHAMP);
        illustrationInputAttributes.addClass(JQ_FF_ILLUSTRATIONINPUT);
        bdfHtmlProducer.startDiv(illustrationInputAttributes);
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setType(HTMLAttributes.HIDDEN_TYPE);
        inputAttributes.setName(name + ":create");
        inputAttributes.setId(ID_FF_CHAMP_PREFIX + name + "_create");
        bdfHtmlProducer.appendInput(inputAttributes);
        inputAttributes.setName(name + ":remove");
        inputAttributes.setId(ID_FF_CHAMP_PREFIX + name + "_remove");
        bdfHtmlProducer.appendInput(inputAttributes);
        inputAttributes.setName(name + ":update");
        inputAttributes.setId(ID_FF_CHAMP_PREFIX + name + "_update");
        bdfHtmlProducer.appendInput(inputAttributes);
        bdfHtmlProducer.endDiv();
        bdfHtmlProducer.endDiv();
    }

    public static void printTextThesaurusInclude(BdfHtmlProducer bdfHtmlProducer, TextThesaurusIncludeElement formElement, String namePrefix) {
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        int defaultSize = 0;
        switch (formElement.getWidthType()) {
            case InputOption.WIDTH_SMALL:
                blocAttributes.addClass(FF_SMALL);
                defaultSize = 21;
                break;
            case InputOption.WIDTH_MEDIUM:
                blocAttributes.addClass(FF_MEDIUM);
                defaultSize = 39;
                break;
            case InputOption.WIDTH_LARGE:
                blocAttributes.addClass(FF_LARGE);
                defaultSize = 77;
                break;
        }
        blocAttributes.addClass(JQ_FF_THESAURUSINCLUDE);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        HTMLAttributes inputAttributes = new HTMLAttributes();
        inputAttributes.setName(name);
        inputAttributes.setId(ID_FF_CHAMP_PREFIX + name);
        inputAttributes.addClass(FF_COMPLET);
        int rows = formElement.getRows();
        if (rows == 1) {
            inputAttributes.setType("text");
            inputAttributes.setSize(String.valueOf(defaultSize));
            inputAttributes.setValue(formElement.getValue());
            bdfHtmlProducer.appendInput(inputAttributes);
        } else {
            inputAttributes.setCols(defaultSize);
            inputAttributes.setRows(rows);
            bdfHtmlProducer.startTextArea(inputAttributes);
            bdfHtmlProducer.escape(formElement.getValue(), true);
            bdfHtmlProducer.endTextArea();
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        if (bdfHtmlProducer.isWithJQuery()) {
            bdfHtmlProducer.startScript(null);
            CommonFormHtml.printFFCoreLimiteValue(bdfHtmlProducer, name, "-1");
            bdfHtmlProducer.breakLine();
            bdfHtmlProducer.endScript();
        }
        bdfHtmlProducer.endDiv();
    }

    public static void printListThesaurusInclude(BdfHtmlProducer bdfHtmlProducer, ListThesaurusIncludeElement formElement, String namePrefix) {
        IncludeKey includeKey = formElement.getIncludeKey();
        String name = getName(includeKey, namePrefix);
        boolean isMandatory = formElement.isMandatory();
        HTMLAttributes blocAttributes = getBlocAttributes(name, isMandatory);
        blocAttributes.addClass(FF_LARGE);
        bdfHtmlProducer.startDiv(blocAttributes);
        printIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        bdfHtmlProducer.startSpan(getChampSpanAttributes());
        Thesaurus currentThesaurus = formElement.getThesaurus();
        int listSize = currentThesaurus.size();
        InputListPrinter inputListPrinter = getInputListPrinter(bdfHtmlProducer, name, formElement.getListType(), formElement.isNoneAllowed(), listSize);
        ThesaurusUtils.getFirst(currentThesaurus);
        boolean isIdalphaType = ThesaurusUtils.isIdalphaType(currentThesaurus);
        boolean isBabelien = ThesaurusUtils.isBabelienType(currentThesaurus);
        boolean withIdalpha = false;
        if ((isIdalphaType) && (formElement.isWithIdalpha())) {
            withIdalpha = true;
        }
        String idalphaSeparator = formElement.getIdalphaSeparator();
        int workingLangInteger = formElement.getWorkingLangInteger();
        Set<Motcle> selectedMotcleSet = formElement.getSelectedMotcleSet();
        short valueType = 0;
        if (selectedMotcleSet.isEmpty()) {
            if (formElement.isNewIndexation()) {
                valueType = InputListPrinter.VALUE_NEW;
            } else {
                valueType = InputListPrinter.VALUE_NO;
            }
        } else {
            valueType = InputListPrinter.VALUE_YES;
        }
        inputListPrinter.start(valueType);
        Motcle motcle = ThesaurusUtils.getFirst(currentThesaurus);
        while (motcle != null) {
            String valcourant = null;
            if (isIdalphaType) {
                valcourant = motcle.getIdalpha();
            } else {
                valcourant = String.valueOf(motcle.getIdths());
            }
            boolean selected = selectedMotcleSet.contains(motcle);
            String lib = motcle.getLibelleString(workingLangInteger, false);
            if ((lib == null) || (lib.length() == 0)) {
                if (isBabelien) {
                    lib = motcle.getLibelleString(0, true);
                } else if (isIdalphaType) {
                    lib = motcle.getIdalpha();
                } else {
                    lib = String.valueOf(motcle.getIdths());
                }
            } else if (withIdalpha) {
                lib = motcle.getIdalpha() + idalphaSeparator + lib;
            }
            inputListPrinter.add(valcourant, lib, motcle.getLevel(), selected);
            motcle = ThesaurusUtils.getNext(motcle);
        }
        inputListPrinter.end();
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
    }

    public static void printRattachementToSlave(BdfHtmlProducer bdfHtmlProducer, RattachementToSlaveElement formElement) {
        FicheCroisementList[] array = formElement.getSlaveAbregeFicheListArray();
        int length = array.length;
        HTMLAttributes blocAttributes = getNotEditableDivAttributes();
        bdfHtmlProducer.startDiv(blocAttributes);
        printNotEditableIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
        HTMLAttributes spanAttributes = getNotEditableChampSpanAttributes();
        bdfHtmlProducer.startSpan(spanAttributes);
        if (length == 0) {
            bdfHtmlProducer.startEm();
            bdfHtmlProducer.localize("saisie_info_norattachement");
            bdfHtmlProducer.endEm();
        } else {
            boolean premier = true;
            for (int j = 0; j < array.length; j++) {
                FicheCroisementList abregeFicheList = array[j];
                int size = abregeFicheList.getSubsetItemCount();
                for (int i = 0; i < size; i++) {
                    AbregeFiche abrg2 = abregeFicheList.getAbregeFiche(i);
                    if (premier) {
                        premier = false;
                    } else {
                        bdfHtmlProducer.appendBr();
                    }
                    bdfHtmlProducer.escape(CorpusMetadataUtils.getFicheIntituleString(abrg2, bdfHtmlProducer.getLocalisationInfo()));
                }
            }
        }
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        bdfHtmlProducer.endDiv();
    }

    public static void printRattachementToMaster(BdfHtmlProducer bdfHtmlProducer, RattachementToMasterElement formElement) {
        SubsetItem subsetItem = formElement.getMasterSubsetItem();
        if (subsetItem instanceof Motcle) {
            HTMLAttributes blocAttributes = getNotEditableDivAttributes();
            bdfHtmlProducer.startDiv(blocAttributes);
            printNotEditableIntituleSpans(bdfHtmlProducer, formElement.getIntitule());
            HTMLAttributes spanAttributes = getNotEditableChampSpanAttributes();
            bdfHtmlProducer.startSpan(spanAttributes);
            bdfHtmlProducer.startStrong();
            BdfHtmlUtils.printMotcleLibelle(bdfHtmlProducer, (Motcle) subsetItem);
            bdfHtmlProducer.endStrong();
            bdfHtmlProducer.endSpan();
            bdfHtmlProducer.breakLine();
        } else if (subsetItem instanceof AbregeFiche) {
            HTMLAttributes blocAttributes = new HTMLAttributes();
            blocAttributes.addClass(FF_IFRAME);
            bdfHtmlProducer.startDiv(blocAttributes);
            bdfHtmlProducer.breakLine();
            String idcorpusString = String.valueOf(subsetItem.getIdsubset());
            String corpusName = subsetItem.getSubset().getSubsetKey().getSubsetName();
            String href = "fiches/" + corpusName + "-" + idcorpusString + ".html";
            HTMLAttributes divAttributes = new HTMLAttributes();
            divAttributes.setClasses(FF_IFRAME_INTITULE);
            bdfHtmlProducer.startDiv(divAttributes);
            bdfHtmlProducer.escape(formElement.getIntitule());
            HtmlPrinterUtils.printColon(bdfHtmlProducer);
            bdfHtmlProducer.endDiv();
            divAttributes.setClasses(FF_IFRAME_CONTENU);
            bdfHtmlProducer.startDiv(divAttributes);
            bdfHtmlProducer.breakLine();
            HTMLAttributes iframeAttributes = new HTMLAttributes();
            iframeAttributes.setWidth("100%");
            iframeAttributes.setHeight("330px");
            iframeAttributes.setSrc(href);
            iframeAttributes.setScrolling("yes");
            bdfHtmlProducer.startIframe(iframeAttributes);
            bdfHtmlProducer.escape(href);
            bdfHtmlProducer.endIframe();
            bdfHtmlProducer.endDiv();
        }
        bdfHtmlProducer.endDiv();
    }

    public static void printNotEditableIntituleSpans(BdfHtmlProducer bdfHtmlProducer, String intitule) {
        HTMLAttributes spanAttributes = new HTMLAttributes();
        bdfHtmlProducer.breakLine();
        spanAttributes.setClasses(FF_NOTEDITABLE_INTITULE);
        bdfHtmlProducer.startSpan(spanAttributes);
        bdfHtmlProducer.escape(intitule);
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
        spanAttributes.setClasses(FF_NOTEDITABLE_DEUXPOINTS);
        bdfHtmlProducer.startSpan(spanAttributes);
        bdfHtmlProducer.append(":");
        bdfHtmlProducer.endSpan();
        bdfHtmlProducer.breakLine();
    }

    public static HTMLAttributes getNotEditableChampSpanAttributes() {
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.setClasses(FF_NOTEDITABLE_CHAMP_CLASS);
        return spanAttributes;
    }

    private static HTMLAttributes getNotEditableDivAttributes() {
        HTMLAttributes blocAttributes = new HTMLAttributes();
        blocAttributes.addClass(FF_NOTEDITABLE);
        blocAttributes.addClass(FF_LARGE);
        return blocAttributes;
    }

    public static void printSimpleDocument(HtmlPrinter htmlPrinter, Document document) {
        htmlPrinter.append(document.getBasename());
        int count = document.getVersionCount();
        if (count == 1) {
            htmlPrinter.append(".");
            htmlPrinter.append(document.getVersion(0).getVersionName().getExtension());
        } else {
            for (int i = 0; i < count; i++) {
                htmlPrinter.append("[.");
                htmlPrinter.append(document.getVersion(i).getVersionName().getExtension());
                htmlPrinter.append("]");
            }
        }
    }

    private static String getName(IncludeKey includeKey, String namePrefix) {
        String name = includeKey.toString();
        return getPrefixedName(name, namePrefix);
    }

    private static String getPrefixedName(String name, String namePrefix) {
        if ((namePrefix != null) && (namePrefix.length() > 0)) {
            name = namePrefix + "_" + name;
        }
        return name;
    }

    private static HTMLAttributes getBlocAttributes(String name, boolean isMandatory) {
        HTMLAttributes blocAttributes = new HTMLAttributes();
        blocAttributes.setId(ID_FF_BLOC_PREFIX + name);
        blocAttributes.addClass(FF_ITEMFIELD);
        if (isMandatory) {
            blocAttributes.addClass(JQ_FF_MANDATORY);
        }
        return blocAttributes;
    }

    public static void printIntituleSpans(HtmlPrinter htmlPrinter, String intitule) {
        HTMLAttributes spanAttributes = new HTMLAttributes();
        htmlPrinter.breakLine();
        spanAttributes.addClass(FF_ITEMFIELD_INTITULE);
        spanAttributes.addClass(JQ_FF_INTITULE);
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.escape(intitule);
        htmlPrinter.endSpan();
        htmlPrinter.breakLine();
        spanAttributes.setClasses(FF_ITEMFIELD_DEUXPOINTS);
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.append(":");
        htmlPrinter.endSpan();
        htmlPrinter.breakLine();
    }

    public static HTMLAttributes getChampSpanAttributes() {
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.setClasses(FF_ITEMFIELD_CHAMP);
        return spanAttributes;
    }

    private static InputListPrinter getInputListPrinter(BdfHtmlProducer bdfHtmlProducer, String name, short type, boolean noneAllowed, int listSize) {
        InputListPrinter inputListPrinter = null;
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(name);
        switch (type) {
            case InputOption.INPUT_SELECT:
                inputListPrinter = new SelectPrinter(bdfHtmlProducer, htmlAttributes);
                break;
            case InputOption.INPUT_RADIO:
                if (bdfHtmlProducer.isWithJQuery()) {
                    htmlAttributes.addClass(JQ_FF_LABELINPUT);
                }
                inputListPrinter = new RadioPrinter(bdfHtmlProducer, null, htmlAttributes);
                break;
            case InputOption.INPUT_CHECK:
                if (bdfHtmlProducer.isWithJQuery()) {
                    htmlAttributes.addClass(JQ_FF_LABELINPUT);
                }
                inputListPrinter = new CheckboxPrinter(bdfHtmlProducer, null, htmlAttributes);
                break;
            default:
                throw new SwitchException("Wrong type = " + type);
        }
        inputListPrinter.setMaxStringLength(50);
        inputListPrinter.setWithEmptyValue(noneAllowed);
        inputListPrinter.setListSize(listSize);
        return inputListPrinter;
    }

}
