/* BdfServer_Html - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.utils;

import fr.exemole.bdfserver.html.BdfHtmlUtils;
import fr.exemole.bdfserver.ui.SubsetGroupList;
import fr.exemole.bdfserver.ui.SubsetKeyGroup;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import net.mapeadores.basedefiches.Basedefiches;
import net.mapeadores.basedefiches.BasedefichesUtils;
import net.mapeadores.basedefiches.SubsetKey;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;


/**
 *
 * @author Vincent Calame
 */
public class SubsetGroupListPrinter {

    private HtmlPrinter hp;
    private Basedefiches basedefiches;
    private int workingLangInteger;
    private boolean onlySubIdValues = false;
    private boolean subsetStringPrint = false;
    private Set<SubsetKey> selectedSet = Collections.emptySet();
    private Set<SubsetKey> disabledSet = Collections.emptySet();
    private SubsetGroupList subsetGroupList;

    public SubsetGroupListPrinter(Basedefiches basedefiches, HtmlPrinter htmlPrinter, int workingLangInteger) {
        this.hp = htmlPrinter;
        this.basedefiches = basedefiches;
        this.workingLangInteger = workingLangInteger;
    }

    public void setSubsetGroupList(SubsetGroupList subsetGroupList) {
        this.subsetGroupList = subsetGroupList;
    }

    public void setOnlySubIdValues(boolean onlySubIdValues) {
        this.onlySubIdValues = onlySubIdValues;
    }

    public void setSubsetStringPrint(boolean subsetStringPrint) {
        this.subsetStringPrint = subsetStringPrint;
    }

    public void setSelectedSubsetKey(SubsetKey subsetKey) {
        if (subsetKey == null) {
            selectedSet = Collections.emptySet();
        } else {
            selectedSet = Collections.singleton(subsetKey);
        }
    }

    public void setDisabledSet(Set<SubsetKey> disabledSet) {
        if (disabledSet == null) {
            this.disabledSet = Collections.emptySet();
        } else {
            this.disabledSet = disabledSet;
        }
    }

    public void setSelectedSet(Set<SubsetKey> selectedSet) {
        if (selectedSet == null) {
            this.selectedSet = Collections.emptySet();
        } else {
            this.selectedSet = selectedSet;
        }
    }

    public void printSelect(HTMLAttributes selectAttributes) {
        HTMLAttributes optGroupAttr = new HTMLAttributes();
        hp.startSelect(selectAttributes);
        boolean selectPrims = false;
        if (!selectAttributes.isMultiple()) {
            if (selectedSet.size() == 0) {
                selectPrims = true;
            }
        }
        int count = subsetGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = subsetGroupList.getSubsetGroup(i);
            boolean isTemplate = !group.isNoTemplateGroup();
            if (isTemplate) {
                optGroupAttr.setLabel(group.getLibelleString(workingLangInteger));
                hp.startOptGroup(optGroupAttr);
            }
            List<SubsetKey> subsetKeyList = group.getSubsetKeyList();
            for (SubsetKey subsetKey : subsetKeyList) {
                HTMLAttributes htmlAttributes = new HTMLAttributes();
                if (selectPrims) {
                    selectPrims = false;
                    htmlAttributes.setSelected(true);
                } else {
                    htmlAttributes.setSelected(selectedSet.contains(subsetKey));
                }
                htmlAttributes.setDisabled(disabledSet.contains(subsetKey));
                String value = (onlySubIdValues) ? subsetKey.getSubsetName() : subsetKey.toString();
                htmlAttributes.setValue(value);
                hp.startOption(htmlAttributes);
                if (subsetStringPrint) {
                    hp.append("[");
                    hp.append(subsetKey.toString());
                    hp.append("]");
                    BdfHtmlUtils.printTiret(hp);
                }
                hp.escape(BasedefichesUtils.getSubsetIntitule(basedefiches, subsetKey, workingLangInteger));
                hp.endOption();
            }
            if (isTemplate) {
                hp.endOptGroup();
            }
        }
        hp.endSelect();
    }

    public void printCheckBoxList(HTMLAttributes checkAttributes, HTMLAttributes labelAttributes, HTMLAttributes groupTitleAttributes, HTMLAttributes groupDivAttributes) {
        int count = subsetGroupList.getSubsetGroupCount();
        for (int i = 0; i < count; i++) {
            SubsetKeyGroup group = subsetGroupList.getSubsetGroup(i);
            boolean isTemplate = !group.isNoTemplateGroup();
            if (isTemplate) {
                hp.startSpan(groupTitleAttributes);
                hp.escape(group.getLibelleString(workingLangInteger));
                hp.endSpan();
                hp.startDiv(groupDivAttributes);
            }
            List<SubsetKey> subsetKeyList = group.getSubsetKeyList();
            for (SubsetKey subsetKey : subsetKeyList) {
                String value = (onlySubIdValues) ? subsetKey.getSubsetName() : subsetKey.toString();
                checkAttributes.setValue(value);
                checkAttributes.setChecked(selectedSet.contains(subsetKey));
                checkAttributes.setDisabled(disabledSet.contains(subsetKey));
                StringBuffer buf = new StringBuffer();
                if (subsetStringPrint) {
                    buf.append("[");
                    buf.append(subsetKey.toString());
                    buf.append("]");
                    buf.append(" - ");
                }
                buf.append(BasedefichesUtils.getSubsetIntitule(basedefiches, subsetKey, workingLangInteger));
                hp.addCheck(checkAttributes, buf.toString(), labelAttributes);
            }
            if (isTemplate) {
                hp.endDiv();
            }
        }
    }

}
