/* BdfServer_Interaction - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.instruction;

import fr.exemole.bdfserver.AvailableFormatLocales;
import fr.exemole.bdfserver.BdfServer;
import fr.exemole.bdfserver.managers.LangConfig;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.localisation.LocalisationInfo;


/**
 *
 * @author Vincent Calame
 */
public class InstructionUtils {

    public static LocalisationInfo getLocalisationInfo(BdfRequestMap bdfRequestMap, BdfServer bdfServer) {
        LangConfig langConfig = bdfServer.getConfigurationManager().getLangConfig();
        int workingLangInteger = langConfig.getDefaultWorkingLangInteger();
        Locale formatLocale = langConfig.getDefaultFormatLocale();
        boolean workingDone = false;
        boolean formatDone = false;

        int[] localeArray = getLocaleIntegerArray(bdfRequestMap);
        for (int i = 0; i < localeArray.length; i++) {
            int localeInteger = localeArray[i];
            int onlyLangInteger = LangInteger.getLangPart(localeInteger);
            if (!workingDone) {
                if (langConfig.containsWorkingLangInteger(localeInteger)) {
                    workingLangInteger = localeInteger;
                    workingDone = true;
                } else if (onlyLangInteger != localeInteger) {
                    if (langConfig.containsWorkingLangInteger(onlyLangInteger)) {
                        workingLangInteger = onlyLangInteger;
                        workingDone = true;
                    }
                }
            }
            if (!formatDone) {
                Locale locale = AvailableFormatLocales.getNearestAvailableLocale(localeInteger);
                if (locale != null) {
                    formatLocale = locale;
                    formatDone = true;
                }
            }
            if ((formatDone) && (workingDone)) {
                break;
            }
        }
        return new LocalisationInfo(workingLangInteger, formatLocale);

    }

    public static int[] getLocaleIntegerArray(BdfRequestMap bdfRequestMap) {
        List<Integer> list = new ArrayList<Integer>();
        String localeParameter = bdfRequestMap.getLocaleParameter();
        if (localeParameter != null) {
            try {
                int langInteger = LangInteger.parse(localeParameter);
                list.add(langInteger);
            } catch (LangIntegerException lie) {
            }
        }
        for (Enumeration enumeration = bdfRequestMap.getRequest().getLocales(); enumeration.hasMoreElements();) {
            Locale locale = (Locale) enumeration.nextElement();
            int langInteger = LangInteger.fromLocale(locale);
            if (langInteger != LangInteger.UNDETERMINED) {
                list.add(langInteger);
            }
        }
        int size = list.size();
        int[] result = new int[size];
        for (int i = 0; i < size; i++) {
            result[i] = list.get(i);
        }
        return result;
    }

    public static Locale getDefaultFormatLocale(BdfServer bdfServer, String localeParam) {
        Locale uiLocale = null;
        if ((localeParam != null) && (localeParam.length() > 1)) {
            String lang = localeParam.substring(0, 2);
            String country = "";
            if (localeParam.length() == 5) {
                country = localeParam.substring(3, 5);
            }
            uiLocale = new Locale(lang, country);
        }
        if (uiLocale == null) {
            uiLocale = LangInteger.getLocale(bdfServer.getDefaultWorkingLangInteger());
        }
        return uiLocale;
    }

}
