/* OdLib_Css - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.css.elements;

import java.util.Map;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.LinkedHashMap;


/**
 *
 * @author Vincent Calame
 */
public class ElementMaps {

    private Map<String, Map<String, StyleElement>> mapOfMaps = new HashMap<String, Map<String, StyleElement>>();
    private Map<Integer, StyleElement> hMap = new TreeMap<Integer, StyleElement>();
    private Map<String, ListStyleElement> listStyleMap = new HashMap<String, ListStyleElement>();

    public ElementMaps() {
    }

    public Map<Integer, StyleElement> getHMap() {
        return hMap;
    }

    public Map<String, StyleElement> getFamilyMap(String family) {
        return mapOfMaps.get(family);
    }

    public OdElement getElement(String elementName, String styleName) {
        int level = StyleElement.isHLevel(elementName);
        if (level == 0) {
            return getFromFamily(StyleElement.PARAGRAPH_FAMILY, "Heading");
        } else if (level > 0) {
            return getHStyle(level);
        }
        if (elementName.equals("p")) {
            return getFromFamily(StyleElement.PARAGRAPH_FAMILY, styleName);
        } else if (elementName.equals("span")) {
            return getFromFamily(StyleElement.TEXT_FAMILY, styleName);
        } else if (elementName.equals("table")) {
            return getFromFamily(StyleElement.TABLE_FAMILY, styleName);
        } else if (elementName.equals("column")) {
            return getFromFamily(StyleElement.TABLECOLUMN_FAMILY, styleName);
        } else if ((elementName.equals("td")) || (elementName.equals("cell"))) {
            return getFromFamily(StyleElement.TABLECELL_FAMILY, styleName);
        } else if ((elementName.equals("tr")) || (elementName.equals("row"))) {
            return getFromFamily(StyleElement.TABLEROW_FAMILY, styleName);
        } else {
            return null;
        }
    }

    public OdElement getElement(String elementName, String styleName, String attributeName, String attributeValue) {
        if ((elementName.equals("ul")) || (elementName.equals("ol"))) {
            if (attributeName.equals("level")) {
                try {
                    int level = Integer.parseInt(attributeValue);
                    if ((level < 1) || (level > 10)) {
                        return null;
                    }
                    short type = ListLevelElement.BULLET_TYPE;
                    if (elementName.equals("ol")) {
                        type = ListLevelElement.NUMBER_TYPE;
                    }
                    ListStyleElement listStyleElement = getListStyleElement(styleName, type);
                    return listStyleElement.getOrCreateListLevelElement(level);
                } catch (NumberFormatException nfe) {
                    return null;
                }
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    public Map<String, ListStyleElement> getListStyleMap() {
        return listStyleMap;
    }

    ListStyleElement getListStyleElement(String styleName, short listLevelType) {
        ListStyleElement listStyleElement = (ListStyleElement) listStyleMap.get(styleName);
        if (listStyleElement == null) {
            listStyleElement = new ListStyleElement(styleName, listLevelType);
            listStyleMap.put(styleName, listStyleElement);
        }
        return listStyleElement;
    }

    StyleElement getFromFamily(String family, String styleName) {
        if (styleName == null) {
            return null;
        }
        Map<String, StyleElement> map = mapOfMaps.get(family);
        if (map == null) {
            map = new LinkedHashMap<String, StyleElement>();
            mapOfMaps.put(family, map);
        }
        StyleElement styleElement = (StyleElement) map.get(styleName);
        if (styleElement == null) {
            styleElement = StyleElement.newInstance(family, styleName);
            map.put(styleName, styleElement);
        }
        return styleElement;
    }

    StyleElement getHStyle(int level) {
        Integer itg = new Integer(level);
        StyleElement hStyle = (StyleElement) hMap.get(itg);
        if (hStyle == null) {
            hStyle = StyleElement.newHInstance(level);
            hMap.put(itg, hStyle);
        }
        return hStyle;
    }

    public void append(ElementMaps importElementMaps) {
        importH(importElementMaps);
        for (Map.Entry<String, Map<String, StyleElement>> entry : importElementMaps.mapOfMaps.entrySet()) {
            String family = (String) entry.getKey();
            Map<String, StyleElement> map = entry.getValue();
            if (!mapOfMaps.containsKey(family)) {
                Map<String, StyleElement> newMap = new HashMap<String, StyleElement>(map);
                mapOfMaps.put(family, newMap);
            } else {
                append(map, (Map<String, StyleElement>) mapOfMaps.get(family));
            }
        }
        importListStyle(importElementMaps);
    }

    private void importListStyle(ElementMaps importElementMaps) {
        for (Map.Entry<String, ListStyleElement> entry : importElementMaps.listStyleMap.entrySet()) {
            String styleName = entry.getKey();
            ListStyleElement importStyle = entry.getValue();
            ListStyleElement current = listStyleMap.get(styleName);
            if (current == null) {
                listStyleMap.put(styleName, importStyle);
            } else {
                current.importAttributes(importStyle);
            }
        }
    }

    private void importH(ElementMaps importElementMaps) {
        for (Map.Entry<Integer, StyleElement> entry : importElementMaps.hMap.entrySet()) {
            Integer level = entry.getKey();
            StyleElement importStyle = entry.getValue();
            StyleElement hStyle = hMap.get(level);
            if (hStyle == null) {
                hMap.put(level, importStyle);
            } else {
                hStyle.importAttributes(importStyle);
            }
        }
    }

    private void append(Map<String, StyleElement> importMap, Map<String, StyleElement> destinationMap) {
        for (Map.Entry<String, StyleElement> entry : importMap.entrySet()) {
            String styleName = entry.getKey();
            StyleElement importStyle = entry.getValue();
            StyleElement destinationStyle = destinationMap.get(styleName);
            if (destinationStyle == null) {
                destinationMap.put(styleName, importStyle);
            } else {
                destinationStyle.importAttributes(importStyle);
            }
        }
    }

}
