/* OdLib_Css - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.css.elements;

import net.mapeadores.opendocument.css.Constants;
import java.util.Map;
import java.util.HashMap;
import java.util.Collections;


/**
 *
 * @author Vincent Calame
 */
public abstract class OdElement implements Constants {

    @SuppressWarnings({"unchecked"})
    private Map<OdAttribute, String>[] attributeMapArray = new Map[ATTRIBUTES_LENGTH];

    public OdElement() {
    }

    private Map<OdAttribute, String> getOrCreateAttributeMap(int attributeType) {
        Map<OdAttribute, String> map = attributeMapArray[attributeType - 1];
        if (map == null) {
            map = new HashMap<OdAttribute, String>();
            attributeMapArray[attributeType - 1] = map;
        }
        return map;
    }

    public void importAttributes(OdElement importElement) {
        for (int i = 0; i < ATTRIBUTES_LENGTH; i++) {
            Map<OdAttribute, String> map = importElement.attributeMapArray[i];
            if (map == null) {
                continue;
            }
            Map<OdAttribute, String> destinationMap = attributeMapArray[i];
            if (destinationMap == null) {
                destinationMap = new HashMap<OdAttribute, String>(map);
                attributeMapArray[i] = destinationMap;
            } else {
                for (Map.Entry<OdAttribute, String> entry : map.entrySet()) {
                    OdAttribute key = entry.getKey();
                    if (!destinationMap.containsKey(entry.getKey())) {
                        destinationMap.put(key, entry.getValue());
                    }
                }
            }
        }
    }

    public void putAttribute(String name, String value) {
        int[] avalaibleTypeArray = getAvalaibleTypeArray();
        OdAttribute attribute = OdAttributeMap.getAttribute(avalaibleTypeArray, name);
        if (attribute == null) {
            return;
        }
        Map<OdAttribute, String> map = getOrCreateAttributeMap(attribute.getAttributeType());
        if (isStyleNameAttribute(name)) {
            value = checkStyleName(value);
        }
        map.put(attribute, value);
    }

    public Map<OdAttribute, String> getAttributeMap(int attributeType) {
        Map<OdAttribute, String> map = attributeMapArray[attributeType - 1];
        if (map == null) {
            return Collections.emptyMap();
        }
        return map;
    }

    public abstract int[] getAvalaibleTypeArray();

    private boolean isStyleNameAttribute(String name) {
        if (name.equals("parent-style-name")) {
            return true;
        }
        if (name.equals("next-style-name")) {
            return true;
        }
        if (name.equals("style-name")) {
            return true;
        }
        if (name.equals("list-style-name")) {
            return true;
        }
        if (name.equals("master-page-name")) {
            return true;
        }
        return false;
    }

    public static String checkDisplayName(String name) {
        StringBuffer displayBuf = new StringBuffer();
        int length = name.length();
        char previous = name.charAt(0);
        displayBuf.append(previous);
        for (int i = 1; i < length; i++) {
            char carac = name.charAt(i);
            if (charTest(carac) && (!charTest(previous))) {
                displayBuf.append(" ");
            }
            previous = carac;
            displayBuf.append(carac);
        }
        return displayBuf.toString();
    }

    public static String checkStyleName(String name) {
        int hLevel = isHLevel(name);
        if (hLevel == 0) {
            return "Heading";
        }
        if (hLevel > 0) {
            return "Heading_20_" + String.valueOf(hLevel);
        }
        int length = name.length();
        if (length == 0) {
            return "";
        }
        StringBuilder buf = new StringBuilder();
        char previous = name.charAt(0);
        buf.append(previous);
        for (int i = 1; i < length; i++) {
            char carac = name.charAt(i);
            if (charTest(carac) && (!charTest(previous))) {
                buf.append("_20_");
            }
            previous = carac;
            buf.append(carac);
        }
        return buf.toString();
    }

    public static boolean charTest(char carac) {
        if ((carac >= '0') && (carac <= '9')) {
            return true;
        }
        if (Character.isUpperCase(carac)) {
            return true;
        }
        return false;
    }

    public static int isHLevel(String name) {
        if (!name.startsWith("h")) {
            return -1;
        }
        int length = name.length();
        if (length == 1) {
            return 0;
        }
        try {
            int level = Integer.parseInt(name.substring(1));
            if (level < 1) {
                return -2;
            }
            if (level > 10) {
                return -3;
            }
            return level;
        } catch (NumberFormatException nfe) {
            return -4;
        }
    }

}
