/* OdLib_Css- Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.css.elements;

import java.util.Map;
import java.util.HashMap;


/**
 *
 * @author Vincent Calame
 */
public class StyleElement extends OdElement {

    public final static String PARAGRAPH_FAMILY = "paragraph";
    public final static String TEXT_FAMILY = "text";
    public final static String TABLE_FAMILY = "table";
    public final static String TABLECELL_FAMILY = "table-cell";
    public final static String TABLEROW_FAMILY = "table-row";
    public final static String TABLECOLUMN_FAMILY = "table-column";
    private final static Map<String, int[]> availableMap = new HashMap<String, int[]>();

    static {
        int[] avprg = {STYLE_ATTRIBUTES, PARAGRAPH_PROPERTIES_ATTRIBUTES, TEXT_PROPERTIES_ATTRIBUTES};
        availableMap.put(PARAGRAPH_FAMILY, avprg);
        int[] avtxt = {STYLE_ATTRIBUTES, TEXT_PROPERTIES_ATTRIBUTES};
        availableMap.put(TEXT_FAMILY, avtxt);
        int[] avtb = {STYLE_ATTRIBUTES, TABLE_PROPERTIES_ATTRIBUTES};
        availableMap.put(TABLE_FAMILY, avtb);
        int[] avtd = {STYLE_ATTRIBUTES, TABLECELL_PROPERTIES_ATTRIBUTES};
        availableMap.put(TABLECELL_FAMILY, avtd);
        int[] avtr = {STYLE_ATTRIBUTES, TABLEROW_PROPERTIES_ATTRIBUTES};
        availableMap.put(TABLEROW_FAMILY, avtr);
        int[] avcol = {STYLE_ATTRIBUTES, TABLECOLUMN_PROPERTIES_ATTRIBUTES};
        availableMap.put(TABLECOLUMN_FAMILY, avcol);
    }

    private String family;
    private int[] availableTypeArray;
    private String styleName;

    private StyleElement(String family, String styleName, int[] availableTypeArray) {
        this.family = family;
        this.availableTypeArray = availableTypeArray;
        this.styleName = styleName;
    }

    public String getStyleName() {
        return styleName;
    }

    public int[] getAvalaibleTypeArray() {
        return availableTypeArray;
    }

    public String getFamily() {
        return family;
    }

    public static StyleElement newInstance(String family, String styleName) {
        String displayName = checkDisplayName(styleName);
        styleName = checkStyleName(styleName);
        int[] result = availableMap.get(family);
        StyleElement styleElement = new StyleElement(family, styleName, result);
        styleElement.putAttribute("display-name", displayName);
        return styleElement;
    }

    public static StyleElement newHInstance(int level) {
        String levelString = String.valueOf(level);
        String styleName = "Heading_20_" + levelString;
        int[] result = {STYLE_ATTRIBUTES, PARAGRAPH_PROPERTIES_ATTRIBUTES, TEXT_PROPERTIES_ATTRIBUTES};
        StyleElement styleElement = new StyleElement(PARAGRAPH_FAMILY, styleName, result);
        styleElement.putAttribute("default-outline-level", levelString);
        styleElement.putAttribute("parent-style-name", "Heading");
        styleElement.putAttribute("display-name", "Heading " + levelString);
        return styleElement;
    }

}
