/* OdLib_Css - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.css.parse;

import java.util.ArrayList;
import java.util.List;
import org.w3c.css.sac.CSSException;
import org.w3c.css.sac.CSSParseException;


/**
 *
 * @author Vincent Calame
 */
public class LogCssErrorHandler implements CssErrorHandler {

    private String currentURI;
    private LogItem fatalErrorLogItem;
    private List<LogItem> logItemList = new ArrayList<LogItem>();

    public LogCssErrorHandler() {
    }

    public void setCurrentURI(String uri) {
        this.currentURI = uri;
    }

    public void error(CSSParseException exception) throws CSSException {
        logItemList.add(new LogItem(currentURI, "css error: " + exception.getMessage()));
    }

    public void fatalError(CSSParseException exception) throws CSSException {
        fatalErrorLogItem = new LogItem(currentURI, exception.getMessage());
    }

    public void warning(CSSParseException exception) throws CSSException {
        logItemList.add(new LogItem(currentURI, "css warning: " + exception.getMessage()));
    }

    public boolean hasFatalError() {
        return (fatalErrorLogItem != null);
    }

    public LogItem getFatalErrorLogItem() {
        return fatalErrorLogItem;
    }

    public int getLogItemCount() {
        return logItemList.size();
    }

    public LogItem getLogItem(int i) {
        return logItemList.get(i);
    }


    public static class LogItem {

        private String uri;
        private String message;

        private LogItem(String uri, String message) {
            this.uri = uri;
            this.message = message;
        }

        public String getURI() {
            return uri;
        }

        public String getMessage() {
            return message;
        }

        @Override
        public String toString() {
            return uri + " / " + message;
        }

    }

}
