/* OdLib_Io - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io;

import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.xml.DefaultXMLWriter;
import java.util.Date;
import java.util.List;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class OdContentWriter extends DefaultXMLWriter {
    
    public OdContentWriter() {
        
    }
    
    public void openDocumentContent() throws IOException {
        startOpenTag("office:document-content");
        addAttribute("xmlns:office","urn:oasis:names:tc:opendocument:xmlns:office:1.0");
        addAttribute("xmlns:style","urn:oasis:names:tc:opendocument:xmlns:style:1.0");
        addAttribute("xmlns:text","urn:oasis:names:tc:opendocument:xmlns:text:1.0");
        addAttribute("xmlns:table","urn:oasis:names:tc:opendocument:xmlns:table:1.0");
        addAttribute("xmlns:draw","urn:oasis:names:tc:opendocument:xmlns:drawing:1.0");
        addAttribute("xmlns:fo","urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0");
        addAttribute("xmlns:xlink","http://www.w3.org/1999/xlink");
        addAttribute("xmlns:dc","http://purl.org/dc/elements/1.1/");
        addAttribute("xmlns:meta","urn:oasis:names:tc:opendocument:xmlns:meta:1.0");
        addAttribute("xmlns:number","urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0");
        addAttribute("xmlns:svg","urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0");
        addAttribute("xmlns:chart","urn:oasis:names:tc:opendocument:xmlns:chart:1.0");
        addAttribute("xmlns:dr3d","urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0");
        addAttribute("xmlns:math","http://www.w3.org/1998/Math/MathML");
        addAttribute("xmlns:form","urn:oasis:names:tc:opendocument:xmlns:form:1.0");
        addAttribute("xmlns:script","urn:oasis:names:tc:opendocument:xmlns:script:1.0");
        addAttribute("xmlns:ooo","http://openoffice.org/2004/office");
        addAttribute("xmlns:ooow","http://openoffice.org/2004/writer");
        addAttribute("xmlns:oooc","http://openoffice.org/2004/calc");
        addAttribute("xmlns:dom","http://www.w3.org/2001/xml-events");
        addAttribute("xmlns:xforms","http://www.w3.org/2002/xforms");
        addAttribute("xmlns:xsd","http://www.w3.org/2001/XMLSchema");
        addAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
        addAttribute("office:version","1.0");
        endOpenTag();
    }
    
    public void closeDocumentContent() throws IOException {
        closeTag("office:document-content");
    }
    
    

    
    public void openAutomaticStyles() throws IOException {
        openTag("office:automatic-styles");
    }
    
    public void closeAutomaticStyles() throws IOException {
        closeTag("office:automatic-styles");
    }
    
    public void openBody() throws IOException {
        openTag("office:body");
    }
    
    public void closeBody() throws IOException {
        closeTag("office:body");
    }
    
    public void openSpreadsheet() throws IOException {
        openTag("office:spreadsheet");
    }
    
    public void closeSpreadsheet() throws IOException {
        closeTag("office:spreadsheet");
    }
    
    public void openTable(String name) throws IOException {
        startOpenTag("table:table");
        addAttribute("table:name",name);
        endOpenTag();
    }
    
    public void closeTable() throws IOException {
        closeTag("table:table");
    }
    
    public void openTableRow() throws IOException {
        openTag("table:table-row");
    }
    
    public void closeTableRow() throws IOException {
        closeTag("table:table-row");
    }
    
    public void openTableCell(String valueType) throws IOException {
        startOpenTag("table:table-cell");
        addAttribute("office:value-type",valueType);
        endOpenTag();
    }
    
    public void closeTableCell() throws IOException {
        closeTag("table:table-cell");
    }
    
    public void openP() throws IOException {
        openTag("text:p");
    }
    
    public void closeP() throws IOException {
        closeTag("text:p");
    }
    
    public void addEmptyTableCell() throws IOException {
        startOpenTag("table:table-cell");
        closeEmptyTag();
    }
    
    public void addIntegerTableCell(int itg) throws IOException {
        startOpenTag("table:table-cell");
        addAttribute("office:value-type","float");
        addAttribute("office:value", String.valueOf(itg));
        closeEmptyTag();
    }
    
    public void addIntegerTableCell(long lg) throws IOException {
        startOpenTag("table:table-cell");
        addAttribute("office:value-type","float");
        addAttribute("office:value", String.valueOf(lg));
        closeEmptyTag();
    }
    
    public void addDecimalTableCell(Decimal decimal) throws IOException {
        startOpenTag("table:table-cell");
        addAttribute("office:value-type","float");
        addAttribute("office:value", decimal.toString());
        closeEmptyTag();
    }
    
    public void addDoubleTableCell(double dbl) throws IOException {
        startOpenTag("table:table-cell");
        addAttribute("office:value-type","float");
        addAttribute("office:value", String.valueOf(dbl));
        closeEmptyTag();
    }
    
    public void addDateTableCell(Date date) throws IOException {
        addDateTableCell(DateInteger.fromDate(date));
    }
    
    public void addDateTableCell(int dateInteger) throws IOException {
        startOpenTag("table:table-cell");
        addAttribute("office:value-type","date");
        addAttribute("office:date-value", DateInteger.toISOString(dateInteger));
        closeEmptyTag();
    }

    public void addStringTableCell(String value) throws IOException {
        openTableCell("string");
        addSimpleElement("text:p",value);
        closeTableCell();
    }
    
    public void addLineTableCell(List<String> list) throws IOException {
        openTableCell("string");
        for(String s : list) {
            addSimpleElement("text:p",s);
        }
        closeTableCell();
    }
    
    public void startTableColumnOpenTag(String styleName, Integer numberColumnsRepeated, String defaultCellStyleName) throws IOException {
        startOpenTag("table:table-column");
        addAttribute("table:style-name",styleName);
        if (numberColumnsRepeated != null) addAttribute("table:number-columns-repeated", numberColumnsRepeated.toString());
        addAttribute("table:default-cell-style-name",defaultCellStyleName);
    }
     
    
    public void openStyle(String styleName, String styleFamily, String parentStyleName) throws IOException {
        startStyleOpenTag(styleName,styleFamily,parentStyleName);
        endOpenTag();
    }
    
    public void startStyleOpenTag(String styleName, String styleFamily, String parentStyleName) throws IOException {
        startOpenTag("style:style");
        addAttribute("style:name", styleName);
        addAttribute("style:family", styleFamily);
        addAttribute("style:parent-style-name", parentStyleName);
    }
    
    public void startDateStyleOpenTag(String styleName, String styleFamily, Boolean automaticOrder) throws IOException {
        startOpenTag("number:date-style");
        addAttribute("style:name", styleName);
        addAttribute("style:family", styleFamily);
        if (automaticOrder != null) addAttribute("number:automatic-order", automaticOrder.toString());
    }
    
    public void closeDateStyle() throws IOException {
        closeTag("number:date-style");
    }
    
    public void addISONumber() throws IOException {
        startOpenTag("number:year");
        addAttribute("number:style", "long");
        closeEmptyTag();
        addNumberText("-");
        startOpenTag("number:month");
        addAttribute("number:style", "long");
        closeEmptyTag();
        addNumberText("-");
        startOpenTag("number:day");
        addAttribute("number:style", "long");
        closeEmptyTag();
    }
    
    public void addNumberText(String s) throws IOException {
        addSimpleElement("number:text", s);
    }

    
    public void closeStyle() throws IOException {
        closeTag("style:style");
    }
    
}
