/* OdLib_Io - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io;


/**
 *
 * @author Vincent Calame
 */
public class OdUtils {
    public final static short OD_TEXT = 1;
    public final static short OD_SPREADSHEET = 2;
    public final static short OD_DATABASE = 3;
    public final static short OD_CHART = 4;
    public final static short OD_FORMULA = 5;
    public final static short OD_DRAWING = 6;
    public final static short OD_IMAGE = 7;
    public final static short OD_MASTER = 8;
    public final static short OD_PRESENTATION = 9;
    
    private OdUtils() {
    }
    
    public static String getMimeType(short odType) {
        switch(odType) {
            case OD_TEXT :
                return "application/vnd.oasis.opendocument.text";
            case OD_SPREADSHEET :
                return "application/vnd.oasis.opendocument.spreadsheet";
            case  OD_DATABASE :
                return "application/vnd.oasis.opendocument.database";
            case  OD_CHART :
                return "application/vnd.oasis.opendocument.chart";
            case OD_FORMULA :
                return "application/vnd.oasis.opendocument.formula";
            case OD_DRAWING:
                return "application/vnd.oasis.opendocument.graphics";
            case OD_IMAGE :
                return "application/vnd.oasis.opendocument.image";
            case OD_MASTER :
                return "application/vnd.oasis.opendocument.text-master";
            case OD_PRESENTATION:
                return "application/vnd.oasis.opendocument.presentation";
            default :
                throw new IllegalArgumentException("unknown odType = " + odType);
        }
    }
    
    public static String getExtension(short odType) {
        switch(odType) {
            case OD_TEXT :
                return "odt";
            case OD_SPREADSHEET :
                return "ods";
            case  OD_DATABASE :
                return "odb";
            case  OD_CHART :
                return "odc";
            case OD_FORMULA :
                return "odf";
            case OD_DRAWING:
                return "odg";
            case OD_IMAGE :
                return "odi";
            case OD_MASTER :
                return "odm";
            case OD_PRESENTATION:
                return "odp";
            default :
                throw new IllegalArgumentException("unknown odType = " + odType);
        }
    }
    
    public static boolean isOdExtension(String extension) {
        try {
            getOdType(extension);
            return true;
        }
        catch(IllegalArgumentException iae) {
            return false;
        }
    }
    
    public static short getOdType(String extension) {
        if (extension.equals("odt")) {
            return OD_TEXT;
        }
        else if (extension.equals("ods")) {
            return OD_SPREADSHEET;
        }
        else if (extension.equals("odb")) {
            return OD_DATABASE;
        }
        else if (extension.equals("odc")) {
            return OD_CHART;
        }
        else if (extension.equals("odf")) {
            return OD_FORMULA;
        }
        else if (extension.equals("odg")) {
            return OD_DRAWING;
        }
        else if (extension.equals("odi")) {
            return OD_IMAGE;
        }
        else if (extension.equals("odm")) {
            return OD_MASTER;
        }
        else if (extension.equals("odp")) {
            return OD_PRESENTATION;
        }
        else {
            throw new IllegalArgumentException("wrong extension = " + extension);
        }
    }
    
    /*
    switch(odType) {
        case OD_TEXT :
        case OD_SPREADSHEET :
        case  OD_DATABASE :
        case  OD_CHART :
        case OD_FORMULA :
        case OD_DRAWING:
        case OD_IMAGE :
        case OD_MASTER :
        case OD_PRESENTATION:
        default :
            throw new SwitchException("unknown odType = " + odType);
    }
     
     */
       
            
}
