/* OdLib_Io - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.io;

import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.exceptions.InternalResourceException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;




/**
 *
 * @author Vincent Calame
 */
public class OdZip {
    private ZipEntrySource contentZipEntrySource;
    private ZipEntrySource stylesZipEntrySource;
    private short odType = OdUtils.OD_TEXT;
    
    public OdZip() {
    }
    
    public void setOdtype(short odType) {
        this.odType = odType;
    }
    
    public void setContentZipEntrySource(ZipEntrySource contentZipEntrySource) {
        this.contentZipEntrySource = contentZipEntrySource;
    }
    
    public void setStylesZipEntrySource(ZipEntrySource stylesZipEntrySource) {
        this.stylesZipEntrySource = stylesZipEntrySource;
    }
    
    public void zip(OutputStream stream) throws IOException {
        ZipOutputStream ecriturezip = new ZipOutputStream(stream);
        Zipper zipper = new Zipper(ecriturezip);
        zipper.zip();
        ecriturezip.close();
    }
    
    private class Zipper {
        ZipOutputStream zip;
        
        Zipper(ZipOutputStream zipOutputStream) {
            this.zip = zipOutputStream;
        }
        
        void zip() throws IOException {
            zipManifest();
            zipContent();
            zipStyles();
        }
        
        private void zipManifest() throws IOException {
            ZipEntry zipEntry = new ZipEntry("META-INF/manifest.xml");
            zip.putNextEntry(zipEntry);
            InputStream inputStream = getManifestStream();
            IOUtils.copy(inputStream,zip);
            inputStream.close();
            zip.closeEntry();
        }
        
        private void zipContent() throws IOException {
            ZipEntry zipEntry = new ZipEntry("content.xml");
            zip.putNextEntry(zipEntry);
            if (contentZipEntrySource != null) {
                contentZipEntrySource.writeEntry(zip);
            }
            else {
                InputStream inputStream = getResourceStream("empty-content.xml");
                IOUtils.copy(inputStream,zip);
                inputStream.close();
            }
            zip.closeEntry();
        }
        
        private void zipStyles() throws IOException {
            ZipEntry zipEntry = new ZipEntry("styles.xml");
            zip.putNextEntry(zipEntry);
            if (stylesZipEntrySource != null) {
                stylesZipEntrySource.writeEntry(zip);
            }
            else {
                InputStream inputStream = getResourceStream("empty-styles.xml");
                IOUtils.copy(inputStream,zip);
                inputStream.close();
            }
            zip.closeEntry();
        }
        
    }
    
    private InputStream getResourceStream(String name) {
        InputStream inputStream = getClass().getResourceAsStream("resources/" + name);
        if (inputStream == null) throw new InternalResourceException("resources/" + name);
        return inputStream;
    }
    
    private InputStream getManifestStream() {
        String resourceName = null;
        switch(odType) {
            case OdUtils.OD_TEXT :
               resourceName = "manifest-text.xml";
               break;
            case OdUtils.OD_SPREADSHEET :
                resourceName = "manifest-spreadsheet.xml";
                break;
            default :
                throw new SwitchException("unknown odType = " + odType);
                
        }
        return getResourceStream(resourceName);
    }
 
}
