/* ScrutariDataExport - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.dataexport.xml;

import net.scrutari.dataexport.api.*;
import java.util.*;


/**
 *
 * @author Vincent Calame
 */
public class XmlBaseMetadataExport extends XmlBuilder implements BaseMetadataExport {

    private String authorityUUID = null;
    private String baseName = null;
    private String baseIcon = null;
    private List<String> langUIList = new ArrayList<String>();
    private Map<String, String> shortMap = new LinkedHashMap<String, String>();
    private Map<String, String> longMap = new LinkedHashMap<String, String>();

    public void setAuthorityUUID(String authorityUUID) {
        this.authorityUUID = authorityUUID;
    }

    public void setBaseName(String baseName) {
        this.baseName = baseName;
    }

    public void setBaseIcon(String baseIcon) {
        this.baseIcon = baseIcon;
    }

    public void setIntitule(int intituleType, String lang, String intituleValue) {
        Map<String, String> map = getMap(intituleType);
        map.put(lang, intituleValue);
    }

    public void addLangUI(String lang) {
        langUIList.add(lang);
    }

    public void writeXML(XmlWriter xmlWriter) {
        xmlWriter.openTag("base-metadata");
        xmlWriter.addSimpleElement("authority-uuid", authorityUUID);
        xmlWriter.addSimpleElement("base-name", baseName);
        xmlWriter.addSimpleElement("base-icon", baseIcon);
        addMap(INTITULE_SHORT, xmlWriter);
        addMap(INTITULE_LONG, xmlWriter);
        int size = langUIList.size();
        if (size > 0) {
            xmlWriter.openTag("langs-ui");
            for (int i = 0; i < size; i++) {
                xmlWriter.addSimpleElement("lang", langUIList.get(i));
            }
            xmlWriter.closeTag("langs-ui");
        }
        xmlWriter.closeTag("base-metadata");
    }

    private void addMap(int intituleType, XmlWriter xmlWriter) {
        Map<String, String> map = getMap(intituleType);
        if (map.isEmpty()) {
            return;
        }
        String suffix = getSuffix(intituleType);
        xmlWriter.openTag("intitule-" + suffix);
        for (Map.Entry<String, String> entry : map.entrySet()) {
            xmlWriter.addLibElement(entry.getKey(), entry.getValue());
        }
        xmlWriter.closeTag("intitule-" + suffix);
    }

    private Map<String, String> getMap(int type) {
        switch (type) {
            case INTITULE_SHORT:
                return shortMap;
            case INTITULE_LONG:
                return longMap;
            default:
                throw new IllegalArgumentException("Wrong intituleType = " + type);
        }
    }

    private static String getSuffix(int type) {
        switch (type) {
            case INTITULE_SHORT:
                return "short";
            case INTITULE_LONG:
                return "long";
            default:
                throw new IllegalArgumentException("Wrong intituleType = " + type);
        }
    }

}
