/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.text.CleanedString;


/**
 *
 * @author Vincent Calame
 */
public class AttributeBuilder {

    private AttributeKey attributeKey;
    private List<String> valList = new ArrayList<String>();

    public AttributeBuilder(AttributeKey attributeKey) {
        if (attributeKey == null) {
            throw new IllegalArgumentException("attributeKey is null");
        }
        this.attributeKey = attributeKey;
    }

    public Attribute toAttribute() {
        int size = valList.size();
        if (size == 0) {
            return null;
        }
        if (size == 1) {
            return new SingletonAttribute(attributeKey, valList.get(0));
        }
        String[] valueArray = valList.toArray(new String[size]);
        return new ArrayAttribute(attributeKey, valueArray);
    }

    public boolean addValue(String value) {
        if (value == null) {
            return false;
        }
        CleanedString cleanedString = CleanedString.newInstance(value);
        if (cleanedString == null) {
            return false;
        }
        valList.add(cleanedString.toString());
        return true;
    }

    public void addValue(CleanedString cleanedString) {
        if (cleanedString == null) {
            throw new IllegalArgumentException("cleanedString is null");
        }
        valList.add(cleanedString.toString());
    }

    public static Attribute toAttribute(AttributeKey key, CleanedString cleanedString) {
        if (key == null) {
            throw new IllegalArgumentException("key is null");
        }
        if (cleanedString == null) {
            throw new IllegalArgumentException("cleanedString is null");
        }
        return new SingletonAttribute(key, cleanedString.toString());
    }


    private static class SingletonAttribute implements Attribute {

        private AttributeKey key;
        private String value;

        private SingletonAttribute(AttributeKey key, String value) {
            this.key = key;
            this.value = value;
        }

        public AttributeKey getAttributeKey() {
            return key;
        }

        public String getValue() {
            return value;
        }

        public int getValueCount() {
            return 1;
        }

        public String getValue(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return value;
        }

    }


    private static class ArrayAttribute implements Attribute {

        private AttributeKey key;
        private String[] valueArray;

        private ArrayAttribute(AttributeKey key, String[] valueArray) {
            this.key = key;
            this.valueArray = valueArray;
        }

        public AttributeKey getAttributeKey() {
            return key;
        }

        public int getValueCount() {
            return valueArray.length;
        }

        public String getValue(int i) {
            return valueArray[i];
        }

    }

}
