/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.util.text.ChangeLibelleHolderBuilder;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.LibelleHolder;


/**
 *
 * @author Vincent Calame
 */
public class AttributeDefBuilder {

    private Set<String> scopeSet = new LinkedHashSet<String>();
    private InternalAttributeDef attributeDef;
    private ChangeLibelleHolderBuilder libelleHolderBuilder;

    public AttributeDefBuilder(AttributeKey attributeKey) {
        if (attributeKey == null) {
            throw new IllegalArgumentException("attributeKey is null");
        }
        attributeDef = new InternalAttributeDef(attributeKey);
        libelleHolderBuilder = new ChangeLibelleHolderBuilder(attributeKey.toString());
    }

    public AttributeKey getAttributeKey() {
        return attributeDef.attributeKey;
    }

    public AttributeDef toAttributeDef() {
        int size = scopeSet.size();
        String[] scopeArray = new String[size];
        int p = 0;
        for (String scope : scopeSet) {
            scopeArray[p] = scope;
            p++;
        }
        attributeDef.scopeArray = scopeArray;
        attributeDef.libelleHolder = libelleHolderBuilder.toChangeLibelleHolder();
        return attributeDef;
    }

    public void addScope(String s) {
        scopeSet.add(s);
    }

    public void setFormatType(String formatType) {
        if (formatType == null) {
            formatType = "";
        }
        attributeDef.formatType = formatType;
    }

    public void addLibelle(Libelle libelle) {
        libelleHolderBuilder.putLibelle(libelle);
    }

    public final static AttributeDefBuilder derive(AttributeDef attributeDef) {
        AttributeDefBuilder builder = new AttributeDefBuilder(attributeDef.getAttributeKey());
        int scopeCount = attributeDef.getScopeCount();
        for (int i = 0; i < scopeCount; i++) {
            builder.addScope(attributeDef.getScope(i));
        }
        builder.setFormatType(attributeDef.getFormatType());
        LibelleHolder libelleHolder = attributeDef.getLibelleHolder();
        int libCount = libelleHolder.getLibelleCount();
        for (int i = 0; i < libCount; i++) {
            builder.addLibelle(libelleHolder.getLibelleByIndex(i));
        }
        return builder;
    }


    private static class InternalAttributeDef implements AttributeDef {

        private AttributeKey attributeKey;
        private String[] scopeArray;
        private String formatType = "";
        private Object formatObject;
        private LibelleHolder libelleHolder;

        private InternalAttributeDef(AttributeKey attributeKey) {
            this.attributeKey = attributeKey;
        }

        public AttributeKey getAttributeKey() {
            return attributeKey;
        }

        public int getScopeCount() {
            return scopeArray.length;
        }

        public String getScope(int i) {
            return scopeArray[i];
        }

        public String getFormatType() {
            return formatType;
        }

        public Object getFormatObject() {
            return formatObject;
        }

        public LibelleHolder getLibelleHolder() {
            return libelleHolder;
        }

    }

}
