/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.io.Serializable;
import java.text.ParseException;
import net.mapeadores.util.text.CleanedString;


/**
 *
 * @author Vincent Calame
 */
public final class AttributeKey implements Serializable {

    public final static String TRANSIENT_NAMESPACE = "_transient";
    private String nameSpace;
    private String localKey;

    private AttributeKey(String nameSpace, String localKey) {
        this.nameSpace = nameSpace;
        this.localKey = localKey;
    }

    public static AttributeKey newInstance(CharSequence nameSpace, CharSequence localKey) {
        if (localKey == null) {
            throw new IllegalArgumentException("localKey is null");
        }
        NameSpaceString nameSpaceString = NameSpaceString.newInstance(nameSpace);
        return newInstance(nameSpaceString, localKey);
    }

    public static AttributeKey newInstance(NameSpaceString nameSpace, CharSequence localKey) {
        if (localKey == null) {
            throw new IllegalArgumentException("localKey is null");
        }
        CleanedString cleanString = CleanedString.newInstance(localKey);
        if (cleanString == null) {
            throw new IllegalArgumentException("Illegal localKey: " + localKey);
        }
        return newInstance(nameSpace, cleanString);
    }

    public static AttributeKey newInstance(NameSpaceString nameSpace, CleanedString localKey) {
        if (localKey == null) {
            throw new IllegalArgumentException("localKey is null");
        }
        if (nameSpace == null) {
            return new AttributeKey("", localKey.toString());
        } else {
            return new AttributeKey(nameSpace.toString(), localKey.toString());
        }
    }

    public static AttributeKey parse(String s, NameSpaceString defaultNameSpace) throws ParseException {
        int idx = s.indexOf(':');
        if (idx == -1) {
            s = s.trim();
            if (s.length() == 0) {
                throw new ParseException("s is empty", 0);
            }
            if (defaultNameSpace != null) {
                return new AttributeKey(defaultNameSpace.toString(), s);
            } else {
                return new AttributeKey("", s);
            }
        }
        String nameSpace = s.substring(0, idx).trim();
        String localKey = s.substring(idx + 1).trim();
        if (localKey.length() == 0) {
            throw new ParseException("localKey is empty", 0);
        }
        return new AttributeKey(nameSpace, localKey);
    }

    /**
     * N'est jamais nul mais peut être vide (pas d'espace de nom)
     */
    public String getNameSpace() {
        return nameSpace;
    }

    /**
     * N'est ni nul ni de longueur nulle
     */
    public String getLocalKey() {
        return localKey;
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        AttributeKey otherAttributeKey = (AttributeKey) other;
        if (!otherAttributeKey.nameSpace.equals(this.nameSpace)) {
            return false;
        }
        if (!otherAttributeKey.localKey.equals(this.localKey)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return nameSpace.hashCode() + localKey.hashCode();
    }

    @Override
    public String toString() {
        return nameSpace + ":" + localKey;
    }

}
