/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class ChangeAttributeHolderBuilder {

    private Set<AttributeKey> removedKeySet = new LinkedHashSet<AttributeKey>();
    private Map<AttributeKey, Attribute> attributeMap = new LinkedHashMap<AttributeKey, Attribute>();

    public ChangeAttributeHolderBuilder() {
    }

    public void putAttribute(Attribute attribute) {
        AttributeKey attributeKey = attribute.getAttributeKey();
        attributeMap.put(attributeKey, attribute);
        removedKeySet.remove(attributeKey);
    }

    public void putRemovedAttributeKey(AttributeKey attributeKey) {
        removedKeySet.add(attributeKey);
        attributeMap.remove(attributeKey);
    }

    public ChangeAttributeHolder toChangeAttributeHolder() {
        int removedSize = removedKeySet.size();
        AttributeKey[] removedAttributeKeyArray = new AttributeKey[removedSize];
        if (removedSize > 0) {
            int p = 0;
            for (AttributeKey attributeKey : removedKeySet) {
                removedAttributeKeyArray[p] = attributeKey;
                p++;
            }
        }
        int attributeSize = attributeMap.size();
        Attribute[] attributeArray = new Attribute[attributeSize];
        if (attributeSize > 0) {
            int p = 0;
            for (Attribute attribute : attributeMap.values()) {
                attributeArray[p] = attribute;
                p++;
            }
        }
        return new InternalChangeAttributeHolder(attributeMap, attributeArray, removedAttributeKeyArray);
    }


    private static class InternalChangeAttributeHolder implements ChangeAttributeHolder {

        private AttributeKey[] removedAttributeKeyArray;
        private Attribute[] attributeArray;
        private Map<AttributeKey, Attribute> attributeMap;

        private InternalChangeAttributeHolder(Map<AttributeKey, Attribute> attributeMap, Attribute[] attributeArray, AttributeKey[] removedAttributeKeyArray) {
            this.removedAttributeKeyArray = removedAttributeKeyArray;
            this.attributeArray = attributeArray;
            this.attributeMap = attributeMap;
        }

        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeMap.get(key);
        }

        public int getAttributeCount() {
            return attributeArray.length;
        }

        public Attribute getAttributeByIndex(int i) {
            return attributeArray[i];
        }

        public int getRemovedAttributeCount() {
            return removedAttributeKeyArray.length;
        }

        public AttributeKey getRemovedAttributeKey(int i) {
            return removedAttributeKeyArray[i];
        }

    }

}
