/* UtilLib - Copyright (c) 2005-2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.collections;


/**
 * Une interface qui permet un double accès à des objets par des clés entières et une position d'index de liste.
 * <code>IndexAndIntKeyMap</code> n'est pas une extension de l'interface <code>Map</code> du <em>Java Collection Framework</em>.
 ** <p>Cette interface est utilisée pour des classes qui indexe des objets avec des valeurs entières (typiquement des identifiants comme dans les bases de données relationnelles)
 ** et qui veluent un accès similaire à celle d'une liste.<:p>
 **<p> Elle repose sur l'idée que les clés entières sont elles-mêmes regroupées dans une instance de {@link StableIntList}.
 ** C'est cette instance qui détermine les valeurs d'index pour les objets.</p>
 ** Il y a donc un dialogue entre l'interface <code>IndexAndIntKeyMap</code> et l'objet renvoyé par <code>getIntKeyList()</code>.
 ** Il peut même s'agir du même objet.
 *
 * @author  Vincent Calame - Exemole
 */
public interface IndexAndIntKeyMap {

    /**
     * Retourne la clé entière correspondant à l'index. Envoi l'exception DisabledIndexException si l'index a été déactivé.
     */
    public int getKeyByIndex(int index) throws DisabledIndexException;

    /**
     * Retourne l'index de la clé entière. Retourne -1 si la clé n'existe pas, c'est à dire si elle n'a jamais été entrée ou si elle a été désactivée antérieurment.
     */
    public int getIndexByKey(int key);

    /**
     * Retourne la valeur à l'index <code>index</code>.
     * Retourne <em>null</em> si l'index n'existe pas ou a été désactivé.
     */
    public Object getValueByIndex(int index);

    /**
     * Retourne la valeur correspondant à la clé entière <code>key</code>. Retourne <em>null</em> s'il n'y a pas de valeur pour cette clé.
     **/
    public Object getValueByKey(int key);

    /**
     * Nombre total d'indices;
     */
    public int length();

    /**
     * Nombre total d'indices actifs
     */
    public int getEnabledIndexCount();

}
