/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.text.RuleBasedCollator;
import net.mapeadores.util.text.collation.CollationUnit;


/**
 *
 * @author Vincent Calame
 */
public class CollatedStringTestUnit implements ConditionConstants {

    private TextTest textTest;
    private boolean isExcludingPartTestType;
    private String searchCollatedString;

    private CollatedStringTestUnit(TextTest textTest, String searchCollatedString) {
        this.textTest = textTest;
        this.isExcludingPartTestType = ConditionUtils.isExcludingPartTestType(textTest.getTestType());
        this.searchCollatedString = searchCollatedString;
    }

    public TextTest getTextTest() {
        return textTest;
    }

    public boolean accept(String collatedString) {
        short testType = textTest.getTestType();
        switch (testType) {
            case EST_VIDE:
                return (collatedString.length() == 0);
            case EST_NON_VIDE:
                return (collatedString.length() > 0);
        }
        if (isExcludingPartTestType) {
            switch (testType) {
                case NE_CORRESPOND_PAS_A:
                    return !collatedString.equals(searchCollatedString);
                case NE_COMMENCE_PAS_PAR:
                    return !collatedString.startsWith(searchCollatedString);
                case NE_FINIT_PAS_PAR:
                    return !collatedString.endsWith(searchCollatedString);
                default:
                    return !collatedString.contains(searchCollatedString);
            }
        } else {
            switch (testType) {
                case CORRESPOND_A:
                    return collatedString.equals(searchCollatedString);
                case COMMENCE_PAR:
                    return collatedString.startsWith(searchCollatedString);
                case FINIT_PAR:
                    return collatedString.endsWith(searchCollatedString);
                default:
                    return collatedString.contains(searchCollatedString);
            }
        }
    }

    public static CollatedStringTestUnit newInstance(TextTest textTest, RuleBasedCollator collator) {
        String searchCollatedString = CollationUnit.collate(textTest.getText(), collator);
        return new CollatedStringTestUnit(textTest, searchCollatedString);
    }

}
