/* UtilLib - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import net.mapeadores.util.xml.XMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import java.io.IOException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;


/**
 *
 * @author Vincent Calame
 */
public class ConditionsXMLStorage {

    private ConditionsXMLStorage() {
    }

    public static void appendCondition(XMLWriter xmlWriter, Condition condition) throws IOException {
        xmlWriter.startOpenTag("condition");
        xmlWriter.addAttribute("mode", ConditionUtils.modeToString(condition.getMode()));
        xmlWriter.endOpenTag();
        TextTest[] array = ConditionUtils.getContentConditionArray(condition);
        for (int i = 0; i < array.length; i++) {
            appendTestText(array[i], xmlWriter);
        }
        xmlWriter.closeTag("condition");
    }

    public static void appendTestText(TextTest textTest, XMLWriter xmlWriter) throws IOException {
        xmlWriter.startOpenTag("text-test");
        xmlWriter.addAttribute("type", ConditionUtils.testTypeToString(textTest.getTestType()));
        xmlWriter.endOpenTag(false);
        xmlWriter.escape(textTest.getText());
        xmlWriter.closeTag("text-test", false, false);
    }

    public static boolean isConditionElement(String tagName) {
        return (tagName.equals("condition")) || (tagName.equals("condition-group"));
    }

    public static Condition getCondition(Element element_xml) {
        String modeString = element_xml.getAttribute("mode");
        short mode = (modeString.equals("and")) ? ConditionConstants.MODE_AND : ConditionConstants.MODE_OR;
        String q = element_xml.getAttribute("q");
        if (q.length() == 0) {
            q = element_xml.getAttribute("raw");
        }
        if (q.length() > 0) {
            return ConditionUtils.parseCondition(q, mode);
        }
        DefaultCondition condition = new DefaultCondition(mode);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if ((tagname.equals("text-test")) || (tagname.equals("condition"))) {
                    TextTest textTest = getTextTest(el);
                    if (textTest != null) {
                        condition.innerAddTextTest(textTest);
                    }
                }
            }
        }
        return condition;
    }

    private static TextTest getTextTest(Element element_xml) {
        String text = XMLUtils.getData(element_xml);
        String type = element_xml.getAttribute("type");
        short testType = 0;
        if (type.length() == 0) {
            testType = ConditionUtils.CONTIENT;
        } else {
            try {
                testType = ConditionUtils.testTypeToShort(type);
            } catch (IllegalArgumentException iae) {
                return null;
            }
        }
        return new DefaultTextTest(testType, text);
    }

}
