/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class DefaultCondition implements Condition, ConditionConstants {

    private short mode;
    private short state = U;
    private List<TextTest> excludingList = new ArrayList<TextTest>();
    private List<TextTest> includingList = new ArrayList<TextTest>();

    public DefaultCondition(short mode) {
        this.mode = mode;
    }

    public short getMode() {
        return mode;
    }

    public short getState() {
        return state;
    }

    public int getExcludingTextTestCount() {
        return excludingList.size();
    }

    public int getIncludingTextTestCount() {
        return includingList.size();
    }

    public TextTest getExcludingTextTest(int i) {
        return excludingList.get(i);
    }

    public TextTest getIncludingTextTest(int i) {
        return includingList.get(i);
    }

    protected void innerSetState(short state) {
        this.state = state;
    }

    protected void innerAddTextTest(TextTest textTest) {
        short testType = textTest.getTestType();
        short addingState = ConditionUtils.getState(testType);
        state = ConditionUtils.merge(mode, state, addingState);
        if (!ConditionUtils.isPartState(state)) {
            excludingList.clear();
            includingList.clear();
        } else {
            if (ConditionUtils.isExcludingPartTestType(testType)) {
                if (excludingList.size() < 31) {
                    excludingList.add(new DefaultTextTest(textTest)); //@todo cover
                }
            } else if (ConditionUtils.isIncludingPartTestType(testType)) {
                if (includingList.size() < 31) {
                    includingList.add(new DefaultTextTest(textTest)); //@todo cover
                }
            }
        }
    }

}
