/* UtilLib - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.format;

import net.mapeadores.util.primitives.PrimUtils;
import java.util.Map;
import java.util.HashMap;


/**
 *
 * @author Vincent Calame
 */
public abstract class FormatDefBuilder implements FormatConstants {

    private boolean formatPatternListInit = false;
    private Map<Integer, InternalInnerSeparator> innerSeparatorMap = new HashMap<Integer, InternalInnerSeparator>();
    private Map<Long, InternalSourceSeparator> sourceSeparatorMap = new HashMap<Long, InternalSourceSeparator>();

    public FormatDefBuilder() {
    }

    public void addFormatPattern(String fP) {
        if (fP != null) {
            fP = fP.trim();
            if (fP.length() == 0) {
                fP = null;
            }
        }
        if ((fP == null) && (!formatPatternListInit)) {
            return;
        }
        formatPatternListInit = true;
        addCleanedFormatPattern(fP);
    }

    public void setCastType(short castType) {
        put(CAST_PARAMETER, new Short(castType));
    }

    public void setLimit(int limit) {
        if (limit < 0) {
            limit = -1;
        }
        put(LIMIT_PARAMETER, new Integer(limit));
    }

    public void setPosition(int pos) {
        if (pos < 0) {
            pos = -1;
        }
        put(POSITION_PARAMETER, new Integer(pos));
        if (pos == FormatConstants.MAX_POSITION) {
            put(FILLBYLAST_PARAMETER, Boolean.TRUE);
        }
    }

    public void setFillByLast(boolean fillByLast) {
        put(FILLBYLAST_PARAMETER, Boolean.valueOf(fillByLast));
    }

    public void setConvertEmptyToNull(boolean convertEmptyToNull) {
        put(EMPTYTONULL_PARAMETER, Boolean.valueOf(convertEmptyToNull));
    }

    public void setDefaultString(String defaultValue) {
        if (defaultValue == null) {
            remove(DEFAULTVALUE_PARAMETER);
        } else {
            put(DEFAULTVALUE_PARAMETER, defaultValue);
        }
    }

    public void setCalcul(Calcul calcul) {
        if (calcul == null) {
            remove(CALCUL_PARAMETER);
        } else {
            put(CALCUL_PARAMETER, calcul);
        }
    }

    public void setSeparator(String separator) {
        if (separator == null) {
            remove(SEPARATOR_PARAMETER);
        } else {
            put(SEPARATOR_PARAMETER, separator);
        }
    }

    public void setInternalSeparator(int sourceIndex, String separator) {
        if (separator == null) {
            throw new IllegalArgumentException("separator is null");
        }
        InternalInnerSeparator innerSeparator = innerSeparatorMap.get(sourceIndex);
        if (innerSeparator == null) {
            innerSeparator = new InternalInnerSeparator(sourceIndex, separator);
            innerSeparatorMap.put(sourceIndex, innerSeparator);
            addInnerSeparator(innerSeparator);
        } else {
            innerSeparator.setSeparator(separator);
        }
    }

    public void setSourceSeparator(int sourceIndex1, int sourceIndex2, String separator) {
        if (separator == null) {
            throw new IllegalArgumentException("separator is null");
        }
        if (sourceIndex1 == sourceIndex2) {
            throw new IllegalArgumentException("sourceIndex1 == sourceIndex2");
        }
        if (sourceIndex1 > sourceIndex2) {
            int tmp = sourceIndex1;
            sourceIndex1 = sourceIndex2;
            sourceIndex2 = tmp;
        }
        long l = PrimUtils.toLong(sourceIndex1, sourceIndex2);
        InternalSourceSeparator sourceSeparator = sourceSeparatorMap.get(l);
        if (sourceSeparator == null) {
            sourceSeparator = new InternalSourceSeparator(sourceIndex1, sourceIndex2, separator);
            sourceSeparatorMap.put(l, sourceSeparator);
            addSourceSeparator(sourceSeparator);
        } else {
            sourceSeparator.setSeparator(separator);
        }
    }

    protected abstract void put(String paramName, Object paramValue);

    protected abstract void remove(String paramName);

    protected abstract void addInnerSeparator(FormatDef.InnerSeparator innerSeparator);

    protected abstract void addSourceSeparator(FormatDef.SourceSeparator sourceSeparator);

    protected abstract void addCleanedFormatPattern(String formatPattern);


    private static class InternalInnerSeparator implements FormatDef.InnerSeparator {

        private int sourceIndex;
        private String separator;

        private InternalInnerSeparator(int sourceIndex, String separator) {
            this.sourceIndex = sourceIndex;
            this.separator = separator;
        }

        public int getSourceIndex() {
            return sourceIndex;
        }

        public String getSeparator() {
            return separator;
        }

        private void setSeparator(String separator) {
            this.separator = separator;
        }

    }


    private static class InternalSourceSeparator implements FormatDef.SourceSeparator {

        private int sourceIndex1;
        private int sourceIndex2;
        private String separator;

        private InternalSourceSeparator(int sourceIndex1, int sourceIndex2, String separator) {
            this.sourceIndex1 = sourceIndex1;
            this.sourceIndex2 = sourceIndex2;
            this.separator = separator;
        }

        public int getSourceIndex1() {
            return sourceIndex1;
        }

        public int getSourceIndex2() {
            return sourceIndex2;
        }

        public String getSeparator() {
            return separator;
        }

        private void setSeparator(String separator) {
            this.separator = separator;
        }

    }

}
