/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.util.List;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ProcessLogUtils;


/**
 *
 * @author Vincent Calame
 */
public class HtmlPrinterUtils {

    private HtmlPrinterUtils() {
    }

    public static void printColon(HtmlPrinter htmlPrinter) {
        htmlPrinter.append("&nbsp;: ");
    }

    public static void printLibColon(HtmlPrinter htmlPrinter, String libKey) {
        htmlPrinter.localize(libKey);
        printColon(htmlPrinter);
    }

    public static void printLibCells(HtmlPrinter htmlPrinter, String libKey, boolean topAlign) {
        if (topAlign) {
            htmlPrinter.startCellWithTopAlign();
        } else {
            htmlPrinter.startCell();
        }
        htmlPrinter.localize(libKey);
        htmlPrinter.endCell();
        if (topAlign) {
            htmlPrinter.startCellWithTopAlign();
        } else {
            htmlPrinter.startCell();
        }
        htmlPrinter.append("&nbsp;:&nbsp;");
        htmlPrinter.endCell();
    }

    public static void printCommandMessage(HtmlPrinter htmlPrinter, CommandMessage commandMessage, String doneMessageSpanClass, String errorMessageSpanClass) {
        String spanClass = doneMessageSpanClass;
        if (commandMessage.isErrorMessage()) {
            spanClass = errorMessageSpanClass;
        }
        htmlPrinter.startP();
        htmlPrinter.startSpan(spanClass);
        htmlPrinter.localize(commandMessage.getTitleResourceItem());
        htmlPrinter.endSpan();
        if (commandMessage.hasMultiError()) {
            List<ResourceItem> list = commandMessage.getMultiErrorList();
            htmlPrinter.startSpan(errorMessageSpanClass);
            htmlPrinter.startSmall();
            for (ResourceItem resourceItem : list) {
                htmlPrinter.appendBr();
                htmlPrinter.localize(resourceItem);
            }
            htmlPrinter.endSmall();
            htmlPrinter.endSpan();
        } else if (commandMessage.hasLog()) {
            htmlPrinter.appendBr();
            htmlPrinter.startPre();
            htmlPrinter.escape(ProcessLogUtils.toString(commandMessage.getProcessLog()), true);
            htmlPrinter.endPre();
        }
        htmlPrinter.endP();
    }

}
