/* UtilLib - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.io;

import java.util.List;
import java.util.ArrayList;
import java.io.File;


/**
 *
 * @author Vincent Calame
 */
public class ResourceInfoUtils {

    private ResourceInfoUtils() {
    }

    public static ResourceInfo toResourceInfo(File f) {
        return new InternalResourceInfo(f);
    }

    public static ResourceInfo[] toResourceInfoArray(File dir) {
        File[] fileList = dir.listFiles();
        List<ResourceInfo> list = new ArrayList<ResourceInfo>();
        for (int i = 0; i < fileList.length; i++) {
            File f = fileList[i];
            if (!f.isDirectory()) {
                list.add(toResourceInfo(f));
            }
        }
        return list.toArray(new ResourceInfo[list.size()]);
    }


    private static class InternalResourceInfo implements ResourceInfo {

        private String name;
        private long length;
        private long lastModified;

        InternalResourceInfo(File file) {
            name = file.getName();
            length = file.length();
            lastModified = file.lastModified();
        }

        public String getName() {
            return name;
        }

        public long length() {
            return length;
        }

        public long lastModified() {
            return lastModified;
        }

    }

}
