/* UtilLib - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.io;

import net.mapeadores.util.exceptions.ExceptionsUtils;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.TransformerException;
import org.xml.sax.SAXParseException;


/**
 *
 * @author Vincent Calame
 */
public class TransformerErrorListener implements ErrorListener {

    private ErrorListener customErrorListener;
    private StringBuffer errorBuffer = new StringBuffer();
    private int fatalErrorCode = 0;

    public TransformerErrorListener() {
    }

    public void setCustomErrorListener(ErrorListener customErrorListener) {
        this.customErrorListener = customErrorListener;
    }

    public void checkFatalTransformerException(TransformerException te) {
        if (te.hashCode() != fatalErrorCode) {
            addError(te);
        }
    }

    public boolean hasError() {
        return (errorBuffer.length() != 0);
    }

    public String getErrorMessage() {
        return errorBuffer.toString();
    }

    public void error(TransformerException transformerException) throws TransformerException {
        addError(transformerException);
        if (customErrorListener != null) {
            customErrorListener.error(transformerException);
        } else {
            throw transformerException;
        }
    }

    public void fatalError(TransformerException transformerException) throws TransformerException {
        addError(transformerException);
        fatalErrorCode = transformerException.hashCode();
        if (customErrorListener != null) {
            customErrorListener.fatalError(transformerException);
        } else {
            throw transformerException;
        }
    }

    public void warning(TransformerException transformerException) throws TransformerException {
        if (customErrorListener != null) {
            customErrorListener.warning(transformerException);
        } else {
            throw transformerException;
        }
    }

    private void addError(TransformerException transformerException) {
        String message = getTransformerMessage(transformerException);
        if (errorBuffer.length() != 0) {
            errorBuffer.append(" / ");
        }
        errorBuffer.append(message);
    }

    private String getTransformerMessage(TransformerException te) {
        Throwable e = te.getCause();
        if ((e != null) && (e instanceof TransformerException)) {
            return getTransformerMessage((TransformerException) e);
        }
        StringBuilder buf = new StringBuilder();
        SourceLocator sourceLocator = te.getLocator();
        if (sourceLocator != null) {
            ExceptionsUtils.append(buf, sourceLocator.getPublicId(), sourceLocator.getSystemId(), sourceLocator.getLineNumber(), sourceLocator.getColumnNumber());
        } else if ((e != null) && (e instanceof SAXParseException)) {
            SAXParseException spe = (SAXParseException) e;
            ExceptionsUtils.append(buf, spe);
        }
        if (e != null) {
            buf.append(e.getLocalizedMessage());
        } else {
            buf.append(te.getMessageAndLocation());
        }
        return buf.toString();
    }

}
