/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import net.mapeadores.util.langinteger.LangInteger;
import java.text.DecimalFormatSymbols;
import java.util.Locale;


/**
 *
 * @author Vincent Calame
 */
public class LocalisationLists {

    private int[] workingLangArray;
    private Locale[] formatLocaleArray;
    private int[] formatLocaleIntegerArray;
    private DecimalFormatSymbols[] decimalFormatSymbolsArray;

    /*public LocalisationLists() {
    }*/
    private LocalisationLists(int[] workingLangArray, Locale[] formatLocaleArray, int[] formatLocaleIntegerArray) {
        this.workingLangArray = workingLangArray;
        this.formatLocaleArray = formatLocaleArray;
        this.formatLocaleIntegerArray = formatLocaleIntegerArray;
        int length = formatLocaleArray.length;
        decimalFormatSymbolsArray = new DecimalFormatSymbols[length];
        for (int i = 0; i < length; i++) {
            decimalFormatSymbolsArray[i] = new DecimalFormatSymbols(formatLocaleArray[i]);
        }
    }

    public int getWorkingLangCount() {
        return workingLangArray.length;
    }

    public int getWorkingLangInteger(int i) {
        return workingLangArray[i];
    }

    public int getFormatLocaleCount() {
        return formatLocaleArray.length;
    }

    public Locale getFormatLocale(int i) {
        return formatLocaleArray[i];
    }

    public int getFormatLocaleInteger(int i) {
        return formatLocaleIntegerArray[i];
    }

    public DecimalFormatSymbols getDecimalFormatSymbols(int i) {
        return decimalFormatSymbolsArray[i];
    }

    public static LocalisationLists newInstance(LocalisationInfo localisationInfo) {
        int[] wl = new int[1];
        Locale[] loc = new Locale[1];
        int[] loci = new int[1];
        wl[0] = localisationInfo.getWorkingLangInteger();
        loc[0] = localisationInfo.getFormatLocale();
        loci[0] = localisationInfo.getFormatLocaleInteger();
        return new LocalisationLists(wl, loc, loci);
    }

    public static LocalisationLists newInstance(int workingLangInteger) {
        int[] wl = new int[1];
        Locale[] loc = new Locale[1];
        int[] loci = new int[1];
        wl[0] = workingLangInteger;
        loc[0] = LangInteger.getLocale(workingLangInteger);
        loci[0] = LangInteger.fromLocale(loc[0]);
        return new LocalisationLists(wl, loc, loci);
    }

    public static LocalisationLists newInstance(int[] workingLangIntegerArray) {
        int length = workingLangIntegerArray.length;
        int[] wl = new int[length];
        Locale[] loc = new Locale[length];
        int[] loci = new int[length];
        for (int i = 0; i < length; i++) {
            int langInteger = workingLangIntegerArray[i];
            Locale locale = LangInteger.getLocale(langInteger);
            wl[i] = langInteger;
            loc[i] = locale;
            loci[i] = LangInteger.fromLocale(locale);
        }
        return new LocalisationLists(wl, loc, loci);
    }

}
