/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.Set;
import java.util.Collections;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractProcessLog implements ProcessLog {

    private Map<String, URILog> uriLogMap = new LinkedHashMap<String, URILog>();
    private Set<String> unmodifiableSet = Collections.unmodifiableSet(uriLogMap.keySet());

    public AbstractProcessLog() {
    }

    public Set<String> getProcessSourceURISet() {
        return unmodifiableSet;
    }

    public List<ProcessLogItem> getProcessLogItemList(String processSourceURI) {
        URILog sourceLog = uriLogMap.get(processSourceURI);
        if (sourceLog == null) {
            return Collections.emptyList();
        }
        return sourceLog.getLogItemList();
    }

    public boolean hasLog() {
        for (URILog uriLog : uriLogMap.values()) {
            if (uriLog.hasLog()) {
                return true;
            }
        }
        return false;
    }

    public boolean hasLog(String processURIString) {
        URILog uriLog = getURILog(processURIString);
        if (uriLog == null) {
            return false;
        }
        return uriLog.hasLog();
    }

    protected URILog getURILog(String processURIString) {
        return uriLogMap.get(processURIString);
    }

    protected void putURILog(URILog uriLog) {
        uriLogMap.put(uriLog.processURI.getProcessURIString(), uriLog);
    }


    public class URILog {

        private List<ProcessLogItem> liste;
        protected ProcessURI processURI;

        public URILog(ProcessURI processURI) {
            this.processURI = processURI;
            liste = new ArrayList<ProcessLogItem>();
        }

        public boolean hasLog() {
            return !liste.isEmpty();
        }

        public List<ProcessLogItem> getLogItemList() {
            return liste;
        }

        public void add(ProcessLogItem logItem) {
            liste.add(logItem);
        }

        public ProcessLogItem newProcessLogItem() {
            return new ProcessLogItem(processURI.getProcessURIString());
        }

    }

}
