/* UtilLib - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.ResourceItem;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessage {

    private ResourceItem titleResourceItem;
    private ProcessLog processLog;
    private List<ResourceItem> multiErrorList;
    private boolean errorMessage = false;

    private CommandMessage(String key, Object value) {
        if (key == null) {
            throw new IllegalArgumentException("key is null");
        }
        titleResourceItem = new ResourceItem(key, value);
    }

    private CommandMessage(ResourceItem keyResourceItem) {
        if (keyResourceItem == null) {
            throw new IllegalArgumentException("keyResourceItem is null");
        }
        this.titleResourceItem = keyResourceItem;
    }

    private void initIsErrorMessage(boolean errorMessage) {
        this.errorMessage = errorMessage;
    }

    private void initMultiErrorList(List<ResourceItem> multiErrorList) {
        this.multiErrorList = multiErrorList;
    }

    private void initProcessLog(ProcessLog processLog) {
        this.processLog = processLog;
    }

    public boolean isErrorMessage() {
        return errorMessage;
    }

    public ResourceItem getTitleResourceItem() {
        return titleResourceItem;
    }

    public boolean hasMultiError() {
        return (multiErrorList != null);
    }

    public List<ResourceItem> getMultiErrorList() {
        return multiErrorList;
    }

    public boolean hasLog() {
        return ((processLog != null) && (processLog.hasLog()));
    }

    public ProcessLog getProcessLog() {
        return processLog;
    }

    public static CommandMessage error(String key, Object value) {
        CommandMessage commandMessage = new CommandMessage(key, value);
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(ResourceItem titleResourceItem) {
        CommandMessage commandMessage = new CommandMessage(titleResourceItem);
        commandMessage.initIsErrorMessage(true);
        return commandMessage;
    }

    public static CommandMessage error(ResourceItem titleResourceItem, List<ResourceItem> multiErrorList) {
        CommandMessage commandMessage = new CommandMessage(titleResourceItem);
        commandMessage.initIsErrorMessage(true);
        commandMessage.initMultiErrorList(multiErrorList);
        return commandMessage;
    }

    public static CommandMessage done(String key, Object value) {
        CommandMessage commandMessage = new CommandMessage(key, value);
        return commandMessage;
    }

    public static CommandMessage done(ResourceItem titleResourceItem) {
        CommandMessage commandMessage = new CommandMessage(titleResourceItem);
        return commandMessage;
    }

    public static CommandMessage done(ResourceItem titleResourceItem, ProcessLog processLog) {
        CommandMessage commandMessage = new CommandMessage(titleResourceItem);
        commandMessage.initProcessLog(processLog);
        return commandMessage;
    }

}
