/* UtilLib - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.localisation.ResourceItem;
import java.io.IOException;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessageJsonProducer implements JsonProducer {

    private CommandMessage commandMessage;
    private ResourceLocalisation resourceLocalisation;
    private boolean errorMessage = false;
    private String messageKey;
    private String messageText;

    public CommandMessageJsonProducer(CommandMessage commandMessage, ResourceLocalisation resourceLocalisation) {
        if (commandMessage == null) {
            throw new IllegalArgumentException("commandMessage is null");
        }
        this.errorMessage = commandMessage.isErrorMessage();
        this.messageKey = commandMessage.getTitleResourceItem().getResourceKey();
        this.commandMessage = commandMessage;
        this.resourceLocalisation = resourceLocalisation;
    }

    public CommandMessageJsonProducer(String messageKey, String messageText, boolean errorMessage) {
        this.errorMessage = errorMessage;
        this.messageKey = messageKey;
        this.messageText = messageText;
    }

    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("commandMessage");
            jsonWriter.object();
            writeType(jsonWriter);
            writeRessourceItem(jsonWriter);
            jsonWriter.endObject();
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private void writeType(JSONWriter jsonWriter) throws JSONException {
        jsonWriter.key("type");
        if (errorMessage) {
            jsonWriter.value("error");
        } else {
            jsonWriter.value("done");
        }
    }

    private void writeRessourceItem(JSONWriter jsonWriter) throws JSONException {
        jsonWriter.key("key");
        jsonWriter.value(messageKey);
        if (messageText != null) {
            jsonWriter.key("text");
            jsonWriter.value(messageText);
        } else if (commandMessage != null) {
            ResourceItem keyResourceItem = commandMessage.getTitleResourceItem();
            if (resourceLocalisation != null) {
                jsonWriter.key("text");
                jsonWriter.value(resourceLocalisation.toString(keyResourceItem));
            }
            List<ResourceItem> multiErrorList = commandMessage.getMultiErrorList();
            if (multiErrorList != null) {
                jsonWriter.key("array");
                jsonWriter.array();
                for (ResourceItem resourceItem : multiErrorList) {
                    jsonWriter.object();
                    jsonWriter.key("key");
                    jsonWriter.value(resourceItem.getResourceKey());
                    if (resourceLocalisation != null) {
                        jsonWriter.key("text");
                        jsonWriter.value(resourceLocalisation.toString(resourceItem));
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
        }
    }

}
