/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.text.StringUtils;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.util.logging.LogRecord;
import java.util.logging.Level;
import java.util.logging.Formatter;


/**
 *
 * @author Vincent Calame
 */
public class LocalizedFormatter extends Formatter {

    private static final DateFormat jour = new SimpleDateFormat("yyyy-MM-dd");
    private static final DateFormat heure = new SimpleDateFormat("HH:mm:ss");
    private Date dat = new Date();
    private Locale locale;

    public LocalizedFormatter(Locale locale) {
        this.locale = locale;
    }

    public synchronized String format(LogRecord logRecord) {
        dat.setTime(logRecord.getMillis());
        StringBuilder buf = new StringBuilder();
        appendLetter(buf, logRecord.getLevel());
        buf.append("[");
        buf.append(jour.format(dat));
        buf.append("][");
        buf.append(heure.format(dat));
        buf.append("]");
        appendMessage(buf, logRecord);
        buf.append('\n');
        if (logRecord.getThrown() != null) {
            try {
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                logRecord.getThrown().printStackTrace(pw);
                pw.close();
                buf.append(sw.toString());
            } catch (Exception ex) {
            }
        }
        return buf.toString();
    }

    private void appendMessage(StringBuilder buf, LogRecord logRecord) {
        String key = logRecord.getMessage();
        ResourceBundle resourceBundle = logRecord.getResourceBundle();
        if (resourceBundle == null) {
            appendSimple(buf, logRecord);
        } else if (resourceBundle != null) {
            try {
                String s = StringUtils.getMessage(resourceBundle, key, logRecord.getParameters(), locale);
                buf.append("[");
                buf.append(key);
                buf.append("] ");
                buf.append(s);

            } catch (Exception ex) {
                appendSimple(buf, logRecord);
                return;
            }
        }
    }

    private void appendSimple(StringBuilder buf, LogRecord logRecord) {
        buf.append(" ");
        buf.append(logRecord.getMessage());
        Object[] objs = logRecord.getParameters();
        if (objs != null) {
            for (int i = 0; i < objs.length; i++) {
                buf.append(" - ");
                buf.append(objs[i].toString());
            }
        }
    }

    private void appendLetter(StringBuilder buf, Level level) {
        if (level.equals(Level.SEVERE)) {
            buf.append("s");
        } else if (level.equals(Level.WARNING)) {
            buf.append("w");
        } else if (level.equals(Level.INFO)) {
            buf.append("i");
        } else if (level.equals(Level.CONFIG)) {
            buf.append("c");
        } else {
            buf.append("f");
        }
    }

}
