/* UtilLib - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.SimpleTimeZone;
import java.util.Date;
import java.io.BufferedWriter;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.io.File;
import java.text.SimpleDateFormat;


/**
 *
 * @author Vincent Calame
 */
public class TimeLog {

    private static String SEPARATEUR = "\t";
    private static long startTime;
    private static long previousTime;
    private static BufferedWriter writer;
    private static OutputStream os;

    private TimeLog() {
    }

    public static void startLog(File f, String localisation, String action) throws IOException {
        startLog(new FileOutputStream(f), localisation, action);
    }

    public static void startLog(OutputStream outputStream, String localisation, String action) throws IOException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
        dateFormat.setTimeZone(new SimpleTimeZone(0, "GMT"));
        os = outputStream;
        writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
        startTime = System.currentTimeMillis();
        writer.write(dateFormat.format(new Date(startTime)));
        writer.write("\n");
        writer.write(localisation);
        writer.write(SEPARATEUR);
        writer.write(action);
        writer.write("\n");
        previousTime = System.currentTimeMillis();
    }

    public static boolean isStarted() {
        return (writer != null);
    }

    public static void endLog(String localisation, String action) throws IOException {
        if (!testWriter(localisation, action)) {
            return;
        }
        log(localisation, action);
        writer.flush();
        writer.close();
        os.close();
        writer = null;
    }

    public static void log(String localisation, String action) {
        if (!testWriter(localisation, action)) {
            return;
        }
        try {
            long currentTime = System.currentTimeMillis();
            writer.write(localisation);
            writer.write(SEPARATEUR);
            writer.write(action);
            writer.write(SEPARATEUR);
            writer.write("+ ");
            printIntervalle(currentTime, previousTime);
            writer.write(SEPARATEUR);
            printIntervalle(currentTime, startTime);
            writer.write("\n");
            previousTime = currentTime;
        } catch (IOException ioe) {
        }
    }

    private static boolean testWriter(String localisation, String action) {
        if (writer != null) {
            return true;
        }
        System.out.println("+--------------------------------------------------------+");
        System.out.println("Time Log : null writer / loc = " + localisation + " / action = " + action);
        return false;
    }

    private static void printIntervalle(long currentTime, long oldTime) throws IOException {
        int diff = (int) (currentTime - oldTime);
        if (diff == 0) {
            writer.write("0ms");
        } else {
            double ms = ((double) diff) / 1000;
            double s = Math.floor(ms);
            ms = (ms - s) * 1000;
            if (s > 0) {
                double mn = Math.floor(s / 60);
                s = s - (mn * 60);
                if (mn > 0) {
                    writer.write(String.valueOf((int) mn));
                    writer.write("mn ");
                }
            }
            writer.write(String.valueOf((int) s));
            writer.write("s ");
            writer.write(String.valueOf((int) ms));
            writer.write("ms");
        }
    }

}
