/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.util.Collection;


/**
 *
 * @author Vincent Calame
 */
public class IntegerListUtils {

    public final static IntegerList EMPTY_INTEGERLIST = new EmptyIntegerList();

    private IntegerListUtils() {
    }

    public static int[] toArray(IntegerList integerList) {
        int length = integerList.getIntegerCount();
        int[] result = new int[length];
        for (int i = 0; i < length; i++) {
            result[i] = integerList.getInteger(i);
        }
        return result;
    }

    public static IntegerList fromLangInteger(int langInteger) {
        return new SingletonIntegerList(langInteger);
    }

    public static IntegerList fromArray(int[] array, boolean copy) {
        if (copy) {
            int length = array.length;
            int[] temp = new int[length];
            System.arraycopy(array, 0, temp, 0, length);
            array = temp;
        }
        return new ArrayIntegerList(array);
    }

    public static IntegerList fromCollection(Collection<Integer> collection) {
        int size = collection.size();
        int[] array = new int[size];
        int p = 0;
        for (int i : collection) {
            array[p] = i;
            p++;
        }
        return new ArrayIntegerList(array);
    }

    public static boolean isEqual(IntegerList integerList, int[] array) {
        int length = array.length;
        if (length != integerList.getIntegerCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            if (array[i] != integerList.getInteger(i)) {
                return false;
            }
        }
        return true;
    }


    private static class EmptyIntegerList implements IntegerList {

        private EmptyIntegerList() {
        }

        public int getIntegerCount() {
            return 0;
        }

        public int getInteger(int i) {
            throw new IndexOutOfBoundsException("integerCount = 0");
        }

    }


    private static class SingletonIntegerList implements IntegerList {

        private int langInteger;

        private SingletonIntegerList(int langInteger) {
            this.langInteger = langInteger;
        }

        private SingletonIntegerList() {
        }

        public int getIntegerCount() {
            return 1;
        }

        public int getInteger(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("integerCount = 1");
            }
            return langInteger;
        }

    }


    private static class ArrayIntegerList implements IntegerList {

        private int[] array;

        private ArrayIntegerList(int[] array) {
            this.array = array;
        }

        public int getIntegerCount() {
            return array.length;
        }

        public int getInteger(int i) {
            return array[i];
        }

    }

}
