/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class ChangeLibelleHolderBuilder {

    private String defaultLib;
    private Set<Integer> removedLangSet = new LinkedHashSet<Integer>();
    private Map<Integer, Libelle> libelleMap = new LinkedHashMap<Integer, Libelle>();

    public ChangeLibelleHolderBuilder() {
        this.defaultLib = "?";
    }

    public ChangeLibelleHolderBuilder(CleanedString cleanedString) {
        if (cleanedString == null) {
            throw new IllegalArgumentException("cleanedString is null");
        }
        this.defaultLib = cleanedString.toString();
    }

    public ChangeLibelleHolderBuilder(CharSequence charSequence) {
        CleanedString cleanedString = CleanedString.newInstance(charSequence);
        if (cleanedString == null) {
            throw new IllegalArgumentException("Illegal default String : " + charSequence.toString());
        }
        this.defaultLib = cleanedString.toString();
    }

    public void putLibelle(Libelle libelle) {
        int langInteger = libelle.getLangInteger();
        libelleMap.put(langInteger, libelle);
        removedLangSet.remove(langInteger);
    }

    public Libelle putLibelle(int langInteger, CharSequence charSequence) {
        return putLibelle(langInteger, CleanedString.newInstance(charSequence));
    }

    public Libelle putLibelle(int langInteger, CleanedString cleanedString) {
        if (cleanedString == null) {
            removedLangSet.add(langInteger);
            libelleMap.remove(langInteger);
            return null;
        } else {
            Libelle libelle = LibelleUtils.toLibelle(langInteger, cleanedString);
            libelleMap.put(langInteger, libelle);
            removedLangSet.remove(langInteger);
            return libelle;
        }
    }

    public Libelle getLibelle(int langInteger) {
        return libelleMap.get(langInteger);
    }

    public void putRemovedLangInteger(int langInteger) {
        removedLangSet.add(langInteger);
        libelleMap.remove(langInteger);
    }

    public ChangeLibelleHolder toChangeLibelleHolder() {
        int removedSize = removedLangSet.size();
        int[] removedLangIntegerArray = new int[removedSize];
        if (removedSize > 0) {
            int p = 0;
            for (int langInteger : removedLangSet) {
                removedLangIntegerArray[p] = langInteger;
                p++;
            }
        }
        int libelleSize = libelleMap.size();
        Libelle[] libelleArray = new Libelle[libelleSize];
        if (libelleSize > 0) {
            int p = 0;
            for (Libelle libelle : libelleMap.values()) {
                libelleArray[p] = libelle;
                p++;
            }
        }
        return new InternalChangeLibelleHolder(defaultLib, libelleMap, libelleArray, removedLangIntegerArray);
    }


    private static class InternalChangeLibelleHolder implements ChangeLibelleHolder {

        private String defaultLib;
        private int[] removedLangIntegerArray;
        private Libelle[] libelleArray;
        private Map<Integer, Libelle> libelleMap;

        private InternalChangeLibelleHolder(String defaultLib, Map<Integer, Libelle> libelleMap, Libelle[] libelleArray, int[] removedLangIntegerArray) {
            this.removedLangIntegerArray = removedLangIntegerArray;
            this.libelleArray = libelleArray;
            this.libelleMap = libelleMap;
            this.defaultLib = defaultLib;
        }

        public String getLibelleString(int langInteger, boolean defaut) {
            Libelle libelle = getLibelleByLang(langInteger);
            if (libelle == null) {
                if (defaut) {
                    return defaultLib;
                } else {
                    return null;
                }
            }
            return libelle.getLibelleString();
        }

        public int getLibelleCount() {
            return libelleArray.length;
        }

        public Libelle getLibelleByIndex(int i) {
            return libelleArray[i];
        }

        public Libelle getLibelleByLang(int langInteger) {
            return libelleMap.get(langInteger);
        }

        public int getRemovedLangCount() {
            return removedLangIntegerArray.length;
        }

        public int getRemovedLangInteger(int i) {
            return removedLangIntegerArray[i];
        }

    }

}
