/* UtilLib - Copyright (c) 2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;


/**
 *
 * @author Vincent Calame
 */
public final class CleanedString implements CharSequence {

    private String string;

    private CleanedString(String string) {
        this.string = string;
    }

    public char charAt(int index) {
        return string.charAt(index);
    }

    public int length() {
        return string.length();
    }

    public CharSequence subSequence(int start, int end) {
        return string.subSequence(start, end);
    }

    @Override
    public String toString() {
        return string;
    }

    @Override
    public int hashCode() {
        return string.hashCode();
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        CleanedString otherAttributeKey = (CleanedString) other;
        return otherAttributeKey.string.equals(this.string);
    }

    public static CleanedString newInstance(CharSequence cs) {
        return newInstance(cs, false);
    }

    public static CleanedString newInstance(CharSequence cs, boolean internString) {
        if (cs == null) {
            return null;
        }
        String s = StringUtils.cleanString(cs);
        if (s.length() == 0) {
            return null;
        }
        if (internString) {
            s = s.intern();
        }
        return new CleanedString(s);
    }

}
