/* UtilLib - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import net.mapeadores.util.collections.LinkedIntHashMap;


/**
 *
 * @author Vincent Calame
 */
public class DefaultLibelleHolder implements LibelleHolder {

    private static final String DEFAULT_LIBELLESTRING = "?";
    private String defaultLibelle = DEFAULT_LIBELLESTRING;
    private LinkedIntHashMap libelleMap = new LinkedIntHashMap();

    public DefaultLibelleHolder() {
    }

    public DefaultLibelleHolder(CharSequence defaultLibelle) {
        CleanedString cleanedString = CleanedString.newInstance(defaultLibelle);
        if (cleanedString != null) {
            this.defaultLibelle = cleanedString.toString();
        }
    }

    public Libelle getLibelleByLang(int langInteger) {
        return (Libelle) libelleMap.get(langInteger);
    }

    public int getLibelleCount() {
        return libelleMap.size();
    }

    public Libelle getLibelleByIndex(int i) {
        return (Libelle) libelleMap.getValueAt(i);
    }

    public String getLibelleString(int langInteger, boolean defaut) {
        Libelle libelle = getLibelleByLang(langInteger);
        if (libelle != null) {
            return libelle.getLibelleString();
        }
        if (defaut) {
            return defaultLibelle;
        } else {
            return null;
        }
    }

    protected boolean innerPutLibelle(Libelle libelle) {
        int langInteger = libelle.getLangInteger();
        Libelle oldLibelle = getLibelleByLang(langInteger);
        if (oldLibelle == null) {
            libelleMap.put(langInteger, libelle);
            return true;
        } else {
            if (LibelleUtils.areEquals(oldLibelle, libelle)) {
                return false;
            }
            libelleMap.put(langInteger, libelle);
            return true;
        }
    }

    protected boolean innerRemoveLibelle(int langInteger) {
        Object oldlib = libelleMap.remove(langInteger);
        return (oldlib != null);
    }

}
