/* UtilLib - Copyright (c) 2006-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import java.io.Serializable;


/**
 * Implémentation de l'interface Libelle qui suit les spécifications Java Beans.
 *
 * @author Vincent Calame
 */
public final class LibelleBean implements Serializable {

    private String lang = "und";
    private String libelleString;

    public LibelleBean() {
    }

    public LibelleBean(Libelle libelle) {
        setLangInteger(libelle.getLangInteger());
        setLibelleString(libelle.getLibelleString());
    }

    public LibelleBean(int langInteger, String libelleString) {
        this.lang = LangInteger.toString(langInteger);
        this.libelleString = libelleString;
    }

    public String getLang() {
        return lang;
    }

    public void setLang(String lang) {
        if (lang == null) {
            throw new IllegalArgumentException("lang cannot be null");
        }
        this.lang = lang;
    }

    public void setLangInteger(int langInteger) {
        this.lang = LangInteger.toString(langInteger);
    }

    public String getLibelleString() {
        return libelleString;
    }

    public void setLibelleString(String libelleString) {
        this.libelleString = libelleString;
    }

    public Libelle toLibelle() {
        CleanedString cleanedString = CleanedString.newInstance(libelleString);
        if (cleanedString == null) {
            return null;
        }
        int langInteger = 0;
        try {
            langInteger = LangInteger.parse(lang);
        } catch (LangIntegerException lie) {
            langInteger = LangInteger.UNDETERMINED;
        }
        return LibelleUtils.toLibelle(langInteger, cleanedString);
    }

}
