/* UtilLib - Copyright (c) -2005 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;


/**
 *
 * @author Vincent Calame
 */
public class SearchSequenceUtils {

    private SearchSequenceUtils() {
    }

    public static String toSQLCondition(String fieldName, SearchSequence searchSequence, short combinaisonType) {
        int tokenCount = searchSequence.getTokenCount();
        if (tokenCount == 0) {
            return "";
        }
        StringBuffer buf = new StringBuffer();
        buf.append('(');
        for (int i = 0; i < tokenCount; i++) {
            if (i > 0) {
                if (combinaisonType == TextConstants.AND) {
                    buf.append(" AND ");
                } else if (combinaisonType == TextConstants.OR) {
                    buf.append(" OR ");
                }
            }
            SearchSequence.Token token = searchSequence.getToken(i);
            if (token.isMultiToken()) {
                SearchSequence.SimpleToken[] array = ((SearchSequence.MultiToken) token).getSubtokenArray();
                for (int j = 0; j < array.length; j++) {
                    if (j > 0) {
                        if (combinaisonType == TextConstants.AND) {
                            buf.append(" AND ");
                        } else if (combinaisonType == TextConstants.OR) {
                            buf.append(" OR ");
                        }
                    }
                    SearchSequence.SimpleToken simpleToken = array[j];
                    appendCondition(buf, fieldName, simpleToken.toString(false), simpleToken.getSearchType());
                }
            } else {
                SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                appendCondition(buf, fieldName, simpleToken.toString(false), simpleToken.getSearchType());
            }
        }
        buf.append(')');
        return buf.toString();
    }

    private static void appendCondition(StringBuffer buf, String fieldName, String token, int type) {
        buf.append('(');
        buf.append(fieldName);
        switch (type) {
            case TextConstants.MATCHES:
                buf.append(" LIKE '");
                appendString(buf, token);
                buf.append("'");
                break;
            case TextConstants.CONTAINS:
                buf.append(" LIKE '%");
                appendString(buf, token);
                buf.append("%'");
                break;
            case TextConstants.STARTSWITH:
                buf.append(" LIKE '");
                appendString(buf, token);
                buf.append("%'");
                break;
            case TextConstants.ENDSWITH:
                buf.append(" LIKE '%");
                appendString(buf, token);
                buf.append("'");
                break;
        }
        buf.append(')');
    }

    private static void appendString(StringBuffer buf, String token) {
        int length = token.length();
        for (int i = 0; i < length; i++) {
            char carac = token.charAt(i);
            if (carac == '\'') {
                buf.append("''");
            } else {
                buf.append(carac);
            }
        }
    }

}
