/* UtilLib - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.net.URL;


/**
 *
 * @author Vincent Calame
 */
public class UrlKey {

    private URL url;
    private String keyString;

    public UrlKey(URL url) {
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        this.url = url;
        this.keyString = toKey(url);
    }

    public URL getUrl() {
        return url;
    }

    public String getKeyString() {
        return keyString;
    }

    @Override
    public String toString() {
        return keyString;
    }

    @Override
    public int hashCode() {
        return keyString.hashCode();
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        UrlKey otherKey = (UrlKey) other;
        return (this.keyString.equals(otherKey.keyString));
    }

    public static String toKey(URL url) {
        return toKey(url.toString());
    }

    public static String toKey(String urlString) {
        int length = urlString.length();
        int start = 0;
        if (urlString.startsWith("http://")) {
            start = 7;
        }
        StringBuffer keyBuffer = new StringBuffer(length);
        for (int i = start; i < length; i++) {
            char carac = urlString.charAt(i);
            switch (carac) {
                case ':':
                case '?':
                case '%':
                case '/':
                case '=':
                case '&':
                case ';':
                case ',':
                case ' ':
                case '+':
                case '.':
                    keyBuffer.append('_');
                    break;
                default:
                    keyBuffer.append(carac);
            }
        }
        return keyBuffer.toString();
    }

}
