/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.collation.map;

import net.mapeadores.util.text.collation.CollationUnit;
import java.util.Locale;
import java.util.Collection;
import java.text.Collator;


/**
 *
 * @author Vincent Calame
 */
public interface CollatedKeyMap {

    public void clear();

    /**
     * Crèe un objet CollationUnit avec comme source s et en se basant sur le Collator de l'instance de CollatedKeyHashMap
     * Équivalent de new CollationUnit(s,getCollator().
     */
    public CollationUnit collateString(String s);

    /**
     * Retourne l'instance de Collator utilisée pour la création des « collatedKey » et la recherche.
     */
    public Collator getCollator();

    /**
     * Retourne l'objet Locale qui a servi à la construction de l'objet.
     */
    public Locale getLocale();

    /**
     * Effectue une recherche sur les clés de l'instance de CollatedKeyHashMap et retourne le résultat sous la forme d'une instance de SearchResultUnitList.
     * Cette méthode vérifie la présence de searchText dans les clés suivant trois conditions : TextConstants.CONTAINS, TextConstants.ENDSWITH, TextConstants.STARTSWITH
     * searchText subit évidemment une « collation » pour que les conditions répondent aux règles du Collator.
     */
    public SearchResultUnitList search(String searchText, int type, ValueFilter valueFilter);

    /**
     * Retourne la valeur stockée à la clé « s ». La chaîne s subit d'abord une « collation ».
     * Cette méthode est équivalente à getValueByCollatedKey(toCollateKey(key))
     */
    public Object getValue(String key);

    /**
     * Retourne la valeur stockée à la clé déjà « collatée », collatedKey.
     */
    public Object getValueByCollatedKey(String collatedKey);

    /**
     * Stocke la valeur à la clé Key. Équivalent de putValueByCollatedKey(toCollatedKey(key),value).
     */
    public void putValue(String key, Object value);

    /**
     * Stocke la valeur à la clé déjà « collatée » collatedKey.
     */
    public void putValueByCollatedKey(String collatedKey, Object value);

    /**
     * Supprime la valeur référencée à la clé key.
     */
    public Object removeValue(String key);

    /**
     * Supprime la valeur à la clé déjà « collatée » collatedKey.
     */
    public Object removeValueByCollatedKey(String collatedKey);

    /**
     * Retourne le nombre de valeurs stockées dans l'instance de CollatedKeyHashMap.
     */
    public int size();

    /**
     * Retourne la forme « collatée » d'une clé.
     */
    public String toCollatedKey(String key);

    public Collection values();

}
