/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.insertpattern.api;

import java.util.List;
import java.util.ArrayList;
import java.util.Stack;


/**
 *
 * @author Vincent Calame
 */
public class InsertionBuffer implements InsertionHandler, InsertionUnitType {

    private IfStack ifStack = new IfStack();
    private List<InsertionUnit> insertionUnitList = new ArrayList<InsertionUnit>();

    public InsertionBuffer() {
    }

    public void appendString(String s) {
        insertionUnitList.add(new InsertionUnit(insertionUnitList.size(), s));
    }

    public void doInsertion(Instruction[] instructionArray) {
        insertionUnitList.add(new InsertionUnit(insertionUnitList.size(), instructionArray));
    }

    public void startIf(String clause, boolean unique) {
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(), START_IF);
        insertionUnit.setUnique(true);
        insertionUnit.setClause(clause);
        insertionUnitList.add(insertionUnit);
        ifStack.increase(insertionUnit);
    }

    public void startElseIf(String clause) {
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(), START_ELSEIF);
        insertionUnit.setClause(clause);
        insertionUnitList.add(insertionUnit);
        ifStack.checkNotUnique();
    }

    public void startElse() {
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(), START_ELSE);
        insertionUnitList.add(insertionUnit);
        ifStack.checkNotUnique();
    }

    public void endIf(boolean unique, boolean onElse) {
        boolean isUnique = ifStack.decrease();
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(), END_IF);
        insertionUnit.setUnique(isUnique);
        insertionUnit.setOnElse(onElse);
        insertionUnitList.add(insertionUnit);
    }

    public InsertionUnit getLastInsertionUnit() {
        return insertionUnitList.get(insertionUnitList.size() - 1);
    }

    public List<InsertionUnit> getInsertionUnitList() {
        return insertionUnitList;
    }


    private class IfStack {

        Stack stack = new Stack();

        boolean decrease() {
            InsertionUnit insertionUnit = (InsertionUnit) stack.pop();
            return insertionUnit.isUnique();
        }

        void increase(InsertionUnit insertionUnit) {
            stack.add(insertionUnit);
        }

        void checkNotUnique() {
            InsertionUnit insertionUnit = (InsertionUnit) stack.peek();
            insertionUnit.setUnique(false);
        }

    }

}
