/* UtilLib - Copyright (c) 2007-2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.lexie.xml;

import net.mapeadores.util.text.*;
import net.mapeadores.util.text.lexie.LexieMap;
import net.mapeadores.util.text.lexie.ParsedLexieSource;
import net.mapeadores.util.text.lexie.TextLexieUnit;
import net.mapeadores.util.text.lexie.Occurence;
import net.mapeadores.util.text.lexie.LexieSource;
import net.mapeadores.util.xml.DefaultXMLWriter;
import java.util.List;
import java.util.Iterator;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class LexieMapXMLWriter extends DefaultXMLWriter {

    public LexieMapXMLWriter() {
        super();
    }

    public void appendLexieMap(LexieMap lexieMap) throws IOException {
        openTag("lexiemap");
        openTag("lexies");
        for (Iterator it = lexieMap.getCollatedKeyMap().values().iterator(); it.hasNext();) {
            TextLexieUnit lexieUnit = (TextLexieUnit) it.next();
            appendLexieUnit(lexieUnit);
        }
        closeTag("lexies");
        openTag("sources");
        for (ParsedLexieSource parsedLexieSource : lexieMap.getParsedLexieSourceMap().values()) {
            appendParsedLexieSource(parsedLexieSource);
        }
        closeTag("sources");
        closeTag("lexiemap");
    }

    public void appendLexieUnit(TextLexieUnit lexieUnit) throws IOException {
        startOpenTag("lexie");
        addAttribute("lx-id", lexieUnit.getLexieId());
        addAttribute("value", lexieUnit.getCanonicalLexie());
        endOpenTag();
        List<Occurence> occurenceList = lexieUnit.getOccurenceList();
        for (Occurence occurence : occurenceList) {
            ParsedLexieSource parsedLexieSource = occurence.getParsedLexieSource();
            startOpenTag("occurence");
            addAttribute("src-id", parsedLexieSource.getLexieSource().getSourceId());
            addAttribute("pos", String.valueOf(occurence.getOccurencePosition()));
            closeEmptyTag();
        }
        closeTag("lexie");
    }

    public void appendParsedLexieSource(ParsedLexieSource parsedLexieSource) throws IOException {
        LexieSource lexieSource = parsedLexieSource.getLexieSource();
        startOpenTag("source");
        addAttribute("src-id", lexieSource.getSourceId());
        endOpenTag();
        openTag("text");
        appendText(lexieSource.getSourceText(), parsedLexieSource.getOccurenceList());
        closeTag("text", false, true);
        closeTag("source");
    }

    private void appendText(String text, List<Occurence> occurenceList) throws IOException {
        int length = text.length();
        int size = occurenceList.size();
        int startLexieIndex = -1;
        int endLexieIndex = -1;
        Occurence currentOccurence = null;
        int currentOccurenceIndex = -1;
        if (size > 0) {
            currentOccurenceIndex = 0;
            currentOccurence = occurenceList.get(0);
            SubstringPosition substringPosition = currentOccurence.getSubstringPosition();
            startLexieIndex = substringPosition.getBeginIndex();
            endLexieIndex = substringPosition.getEndIndex();
        }
        for (int i = 0; i < length; i++) {
            if (i == startLexieIndex) {
                startOpenTag("lx", false);
                addAttribute("lx-id", currentOccurence.getLexieUnit().getLexieId());
                addAttribute("pos", String.valueOf(currentOccurence.getOccurencePosition()));
                endOpenTag(false);
            }
            escape(text.charAt(i));
            if (i == endLexieIndex) {
                closeTag("lx", false, false);
                currentOccurenceIndex++;
                if (currentOccurenceIndex < size) {
                    currentOccurence = occurenceList.get(currentOccurenceIndex);
                    SubstringPosition substringPosition = currentOccurence.getSubstringPosition();
                    startLexieIndex = substringPosition.getBeginIndex();
                    endLexieIndex = substringPosition.getEndIndex();
                } else {
                    currentOccurence = null;
                    startLexieIndex = -1;
                    endLexieIndex = -1;
                }
            }

        }

    }

    public static String getLexieMapXML(LexieMap lexieMap, boolean withTabs) {
        StringBuilder buf = new StringBuilder(512);
        try {
            LexieMapXMLWriter lexieMapXMLWriter = new LexieMapXMLWriter();
            lexieMapXMLWriter.setAppendable(buf);
            lexieMapXMLWriter.setIndentLength((withTabs) ? 0 : -999);
            lexieMapXMLWriter.appendXMLDeclaration();
            lexieMapXMLWriter.appendLexieMap(lexieMap);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
