/* UtilLib - Copyright (c) 2007 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.tableparser;

import java.util.List;
import java.util.ArrayList;


/**
 *
 * @author Vincent Calame
 */
public class MysqlParser {

    private MysqlParser() {
    }

    public static String[][] parse(String s, MysqlParameters mysqlParameters) {
        char linesTerminatedBy = mysqlParameters.getLinesTerminatedBy();
        char escapedBy = mysqlParameters.getEscapedBy();
        char fieldsTerminatedBy = mysqlParameters.getFieldsTerminatedBy();
        int length = s.length();
        LineBuffer lineBuffer = new LineBuffer();
        for (int i = 0; i < length; i++) {
            char carac = s.charAt(i);
            if (carac == escapedBy) {
                if (i < (length - 1)) {
                    char next = s.charAt(i + 1);
                    if (next == 'N') {
                    } else if (next == '0') {
                        lineBuffer.append((char) 0);
                    } else {
                        lineBuffer.append(next);
                    }
                    if (next == '\r') {
                        if (i < (length - 2)) {
                            if (s.charAt(i + 2) == '\n') {
                                lineBuffer.append('\n');
                                i++;
                            }
                        }
                    }
                    i++;
                } else {
                    lineBuffer.append(escapedBy);
                }
            } else if (carac == fieldsTerminatedBy) {
                lineBuffer.endField();
            } else if (carac == '\r') {
                if (linesTerminatedBy == '\n') {
                    if (i < (length - 1)) {
                        if (s.charAt(i + 1) == '\n') {
                            i++;
                        }
                    }
                    lineBuffer.endLine();
                } else {
                    lineBuffer.append(carac);
                }
            } else if (carac == linesTerminatedBy) {
                lineBuffer.endLine();
            } else {
                lineBuffer.append(carac);
            }
        }
        lineBuffer.endLine();
        return lineBuffer.flush();
    }


    static class LineBuffer {

        List<String[]> lineList = new ArrayList<String[]>();
        List<String> fieldList = new ArrayList<String>();
        StringBuffer fieldBuffer;

        LineBuffer() {
            fieldBuffer = new StringBuffer();
        }

        void append(char carac) {
            fieldBuffer.append(carac);
        }

        void endField() {
            fieldList.add(fieldBuffer.toString());
            fieldBuffer = new StringBuffer();
        }

        void endLine() {
            endField();
            int size = fieldList.size();
            String[] line = new String[size];
            for (int i = 0; i < size; i++) {
                line[i] = fieldList.get(i);
            }
            lineList.add(line);
            fieldList.clear();
        }

        String[][] flush() {
            int size = lineList.size();
            String[][] result = new String[size][];
            for (int i = 0; i < size; i++) {
                result[i] = lineList.get(i);
            }
            lineList.clear();
            return result;
        }

    }

}
