/* UtilLib - Copyright (c) 2007-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.tableparser;

import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.exceptions.NestedClassLoaderException;


/**
 *
 * @author Vincent Calame
 */
public class TableParser {

    
    private static CsvParser csvParser = null;

    private TableParser() {
    }

    public static String[][] parse(String s, Parameters parameters) {
        if (parameters instanceof OOCopyParameters) {
            return parseOOCopy(s, (OOCopyParameters) parameters);
        }
        if (parameters instanceof CsvParameters) {
            if (csvParser != null) {
                return csvParser.parse(s, (CsvParameters) parameters);
            }
            SimpleCsvParser scp = new SimpleCsvParser();
            return scp.parse(s, (CsvParameters) parameters);
        }
        if (parameters instanceof MysqlParameters) {
            return MysqlParser.parse(s, (MysqlParameters) parameters);
        }
        throw new IllegalArgumentException("unknown TableParser.Parameters implementations = " + parameters.getClass().getName());
    }

    private static String[][] parseOOCopy(String s, OOCopyParameters parameters) {
        String[] lines = StringUtils.getLineTokens(s, StringUtils.NOTCLEAN);
        int length = lines.length;
        String[][] result = new String[length][];
        for (int i = 0; i < length; i++) {
            String line = lines[i];
            String[] tokens = StringUtils.getTokens(line, '\t', StringUtils.NOTCLEAN);
            result[i] = tokens;
        }
        return result;
    }

    public static boolean isCsvParserInit() {
        return (csvParser != null);
    }

    public static void initCsvParser(String className) {
        try {
            Class csvParserClass = Class.forName(className);
            csvParser = (CsvParser) csvParserClass.newInstance();
        } catch (Exception e) {
            throw new NestedClassLoaderException(e);
        }
    }


    public static interface Parameters {
    }

        

}
