/* UtilLib - Copyright (c) 2006 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.terme;


/**
 *
 * @author  Vincent Calame
 */
public class DefaultTermeDomainEditor implements TermeDomainEditor {

    private InternalTermeDomain internalTermeDomain = new InternalTermeDomain();
    private TermeEntry[] entries = new TermeEntry[128];
    private int size = 0;

    public DefaultTermeDomainEditor() {
    }

    public int getNewCode(Object owner) {
        if (owner == null) {
            throw new IllegalArgumentException("owner argument is null");
        }
        int newcode = size++;
        checkSize();
        entries[newcode] = new TermeEntry(owner, newcode);
        return newcode;
    }

    public TermeDomain getTermeDomain() {
        return internalTermeDomain;
    }

    public void setTerme(int code, Terme terme, Object owner) throws IllegalOwnerException, IllegalCodeException {
        if (owner == null) {
            throw new IllegalArgumentException("owner argument is null");
        }
        if ((code < 0) || (code >= size)) {
            throw new IllegalCodeException(IllegalCodeException.MISSING_CODE, code);
        }
        if ((terme != null) && (code != terme.getCode())) {
            throw new IllegalCodeException(IllegalCodeException.NOTCORRESPONDING_CODE, code);
        }
        TermeEntry entry = entries[code];
        if (entry == null) {
            throw new IllegalCodeException(IllegalCodeException.DISABLED_CODE, code);
        }
        if (!entry.checkOwner(owner)) {
            throw new IllegalOwnerException();
        }
        entry.setTerme(terme);
    }

    private synchronized void checkSize() {
        if (size >= entries.length) {
            TermeEntry[] nv = new TermeEntry[(entries.length * 3) / 2 + 1];
            System.arraycopy(entries, 0, nv, 0, entries.length);
            entries = nv;
        }
    }

    public synchronized void removeOwner(Object owner) {
        for (int i = 0; i < size; i++) {
            TermeEntry entry = entries[i];
            if (entry != null) {
                if (entry.checkOwner(owner)) {
                    entries[i] = null;
                }
            }
        }
    }

    public boolean checkOwner(Object owner, int code) throws IllegalCodeException {
        if ((code < 0) || (code >= size)) {
            throw new IllegalCodeException(IllegalCodeException.MISSING_CODE, code);
        }
        TermeEntry entry = entries[code];
        if (entry == null) {
            throw new IllegalCodeException(IllegalCodeException.DISABLED_CODE, code);
        }
        return entry.checkOwner(owner);
    }


    class TermeEntry {

        Object owner;
        int code;
        Terme terme;

        TermeEntry(Object owner, int code) {
            this.owner = owner;
            this.code = code;
        }

        void setTerme(Terme terme) {
            this.terme = terme;
        }

        boolean checkOwner(Object obj) {
            return owner.equals(obj);
        }

    }


    class InternalTermeDomain implements TermeDomain {

        public Terme getTermeByCode(int code) throws IllegalCodeException {
            if ((code < 0) || (code >= size)) {
                throw new IllegalCodeException(IllegalCodeException.MISSING_CODE, code);
            }
            TermeEntry entry = entries[code];
            if (entry == null) {
                throw new IllegalCodeException(IllegalCodeException.DISABLED_CODE, code);
            }
            Terme terme = entry.terme;
            if (terme == null) {
                throw new IllegalCodeException(IllegalCodeException.DISABLED_CODE, code);
            }
            return terme;
        }

    }

}
