/* UtilLib - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.exceptions.NestedSAXException;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class DOMUtils {

    private final static DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();

    private DOMUtils() {
    }

    public static void readChildren(Element element_xml, ElementHandler elementHandler) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                elementHandler.readElement(element);
            }
        }
    }

    /**
     * N'est jamais nul.
     */
    public static String readSimpleElement(Element element_xml) {
        return readSimpleElement(element_xml, true);
    }

    /**
     * N'est jamais nul.
     */
    public static String readSimpleElement(Element el, boolean trim) {
        Node nd = el.getFirstChild();
        if ((nd != null) && ((nd.getNodeType() == Node.TEXT_NODE) || (nd.getNodeType() == Node.CDATA_SECTION_NODE))) {
            String result = ((Text) nd).getData();
            if (result == null) {
                return "";
            } else {
                return StringUtils.cleanString(result, trim);
            }
        } else {
            return "";
        }
    }

    /**
     * Retourne nul si vide
     */
    public static CleanedString contentToCleanedString(Element element) {
        Node nd = element.getFirstChild();
        if ((nd != null) && ((nd.getNodeType() == Node.TEXT_NODE) || (nd.getNodeType() == Node.CDATA_SECTION_NODE))) {
            String result = ((Text) nd).getData();
            return CleanedString.newInstance(result);
        } else {
            return null;
        }
    }

    public static Document readDocument(File f) {
        DocumentBuilder docBuilder = newDocumentBuilder();
        try {
            Document document = docBuilder.parse(f);
            return document;
        } catch (SAXException saxe) {
            throw new NestedSAXException(saxe);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public static Document readDocument(InputStream inputStream) {
        DocumentBuilder docBuilder = newDocumentBuilder();
        try {
            Document document = docBuilder.parse(inputStream);
            return document;
        } catch (SAXException saxe) {
            throw new NestedSAXException(saxe);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public static Document readDocument(InputSource inputSource) {
        DocumentBuilder docBuilder = newDocumentBuilder();
        try {
            Document document = docBuilder.parse(inputSource);
            return document;
        } catch (SAXException saxe) {
            throw new NestedSAXException(saxe);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
    }

    public synchronized static DocumentBuilder newDocumentBuilder() {
        try {
            return documentBuilderFactory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new NestedLibraryException(pce);
        }
    }

}
