/* UtilLib - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml;

import net.mapeadores.util.xml.ns.NameSpace;
import java.io.IOException;


/**
 *
 * @author Vincent Calame
 */
public class XMLPart implements XMLWriter {

    private XMLWriter xmlWriter;

    public XMLPart(XMLWriter xmlWriter) {
        this.xmlWriter = xmlWriter;
    }

    public XMLWriter getXMLWriter() {
        return xmlWriter;
    }

    public Appendable getAppendable() {
        return xmlWriter.getAppendable();
    }

    public boolean isAposEscaped() {
        return xmlWriter.isAposEscaped();
    }

    public int getCurrentIndentValue() {
        return xmlWriter.getCurrentIndentValue();
    }

    public void appendIndent() throws IOException {
        xmlWriter.appendIndent();
    }

    public void escape(CharSequence charSequence) throws IOException {
        xmlWriter.escape(charSequence);
    }

    public void escape(char carac) throws IOException {
        xmlWriter.escape(carac);
    }

    public void openTag(String name) throws IOException {
        xmlWriter.openTag(name);
    }

    public void openTag(String name, boolean indentBefore, boolean indentIncrease) throws IOException {
        xmlWriter.openTag(name, indentBefore, indentIncrease);
    }

    public void startOpenTag(String name) throws IOException {
        xmlWriter.startOpenTag(name);
    }

    public void startOpenTag(String name, boolean indentBefore) throws IOException {
        xmlWriter.startOpenTag(name, indentBefore);
    }

    public void closeTag(String name) throws IOException {
        xmlWriter.closeTag(name);
    }

    public void closeTag(String name, boolean indentBefore, boolean indentDecrease) throws IOException {
        xmlWriter.closeTag(name, indentBefore, indentDecrease);
    }

    public void endOpenTag() throws IOException {
        xmlWriter.endOpenTag();
    }

    public void endOpenTag(boolean increaseIndent) throws IOException {
        xmlWriter.endOpenTag(increaseIndent);
    }

    public void addAttribute(String name, String value) throws IOException {
        xmlWriter.addAttribute(name, value);
    }

    public void addAttribute(String name, int value) throws IOException {
        xmlWriter.addAttribute(name, value);
    }

    public void addAttribute(String attributeName, char c) throws IOException {
        xmlWriter.addAttribute(attributeName, c);
    }

    public void closeEmptyTag() throws IOException {
        xmlWriter.closeEmptyTag();
    }

    public void addNameSpaceAttribute(NameSpace nameSpace) throws IOException {
        xmlWriter.addNameSpaceAttribute(nameSpace);
    }

    public void addNameSpaceAttribute(String prefix, String nameSpace) throws IOException {
        xmlWriter.addNameSpaceAttribute(prefix, nameSpace);
    }

    public void addSimpleElement(String tagname, String value) throws IOException {
        xmlWriter.addSimpleElement(tagname, value);
    }

    public void addSimpleElement(String tagname, String attribute, String attributeValue, String value) throws IOException {
        xmlWriter.addSimpleElement(tagname, attribute, attributeValue, value);
    }

    public void addEmptyElement(String tagname) throws IOException {
        xmlWriter.addEmptyElement(tagname);
    }

    public void appendXMLDeclaration() throws IOException {
        xmlWriter.appendXMLDeclaration();
    }

    public void appendStylesheet(String href, String type) throws IOException {
        xmlWriter.appendStylesheet(href, type);
    }

    public void appendDocType(String doctype) throws IOException {
        xmlWriter.appendDocType(doctype);
    }

    public void append(char c) throws IOException {
        xmlWriter.append(c);
    }

    public void append(CharSequence charSequence) throws IOException {
        xmlWriter.append(charSequence);
    }

    public void appendCData(CharSequence charSequence) throws IOException {
        xmlWriter.appendCData(charSequence);
    }

}
