/* UtilLib - Copyright (c) 2008 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml.xmlpull;

import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractElementHandler implements XmlPullElementHandler {

    XmlPullElementHandler subElementHandler;

    public AbstractElementHandler() {
    }

    public void processStartElement(String tagname, Map<String, String> attributeMap) {
        if (subElementHandler != null) {
            subElementHandler.processStartElement(tagname, attributeMap);
        } else {
            subElementHandler = startSubElement(tagname, attributeMap);
            if (subElementHandler == null) {
                subElementHandler = new NullHandler();
            }
        }
    }

    public boolean processEndElement(String tagname) {
        if (subElementHandler != null) {
            boolean endSubElement = subElementHandler.processEndElement(tagname);
            if (endSubElement) {
                if (subElementHandler instanceof SimpleElementHandler) {
                    SimpleElementHandler simple = (SimpleElementHandler) subElementHandler;
                    endSimpleSubElement(simple.getType(), simple.getText());
                } else {
                    endSubElement(subElementHandler);
                }
                subElementHandler = null;
            }
            return false;
        } else {
            return true;
        }
    }

    public void processText(String text) {
        if (subElementHandler != null) {
            subElementHandler.processText(text);
        } else {
            appendText(text);
        }
    }

    public abstract XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap);

    public abstract void endSubElement(XmlPullElementHandler subElementHandler);

    public abstract void endSimpleSubElement(int type, String text);

    public abstract void appendText(String text);

    public static String getAttributeString(Map<String, String> attributeMap, String name) {
        if (attributeMap == null) {
            return null;
        }
        String result = (String) attributeMap.get(name);
        if (result == null) {
            return null;
        }
        if (result.length() == 0) {
            return null;
        }
        return result;
    }

}
