/* UtilLib - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.xml.xmlpull;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.Libelle;
import java.util.Map;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class LibelleElementHandler implements XmlPullElementHandler {

    private int langInteger = LangInteger.UNDETERMINED;
    private StringBuilder buf = new StringBuilder();
    private int subCount = 0;

    public LibelleElementHandler(Map<String, String> attributeMap) throws LangIntegerException {
        if (attributeMap != null) {
            String langIntegerString = (String) attributeMap.get("xml:lang");
            if ((langIntegerString != null) && (langIntegerString.length() > 0)) {
                langInteger = LangInteger.parse(langIntegerString);
            }
        }
    }

    public void processStartElement(String tagname, Map<String, String> attributeMap) {
        subCount++;
        buf.append("<");
        buf.append(tagname);
        buf.append(">");
    }

    public boolean processEndElement(String tagname) {
        if (subCount == 0) {
            return true;
        }
        subCount--;
        buf.append("</");
        buf.append(tagname);
        buf.append(">");
        return false;
    }

    public void processText(String text) {
        buf.append(text);
    }

    public XmlPullElementHandler initSubElementHandler(String tagname, Map attributeMap) {
        return null;
    }

    public void flushSubElementHandler(String tagname, XmlPullElementHandler elementHandler) {
    }

    /**
     * Peut être nul
     */
    public Libelle toLibelle() {
        CleanedString cleanedString = CleanedString.newInstance(buf);
        if (cleanedString == null) {
            return null;
        }
        return LibelleUtils.toLibelle(langInteger, cleanedString);
    }

}
