/* UtilLib_Image - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.images;

import java.awt.Graphics;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.OutputStream;
import javax.imageio.ImageIO;
import org.icepdf.core.exceptions.PDFException;
import org.icepdf.core.exceptions.PDFSecurityException;
import org.icepdf.core.pobjects.Document;
import org.icepdf.core.pobjects.Page;
import org.icepdf.core.util.GraphicsRenderingHints;


/**
 *
 * @author Vincent Calame
 */
public class ImagesUtils {

    public static void convert(InputStream input, String destinationFormatName, File destination) throws IOException {
        BufferedImage bufferedImage = ImageIO.read(input);
        ImageIO.write(bufferedImage, destinationFormatName, destination);
    }

    public static void convertPdf(InputStream input, int pageNumber, String destinationFormatName, File destination) throws IOException {
        Document document = new Document();
        try {
            document.setInputStream(input, "test.pdf");
        } catch (PDFException ex) {
            System.out.println("Error parsing PDF document " + ex);
        } catch (PDFSecurityException ex) {
            System.out.println("Error encryption not supported " + ex);
        } catch (FileNotFoundException ex) {
            System.out.println("Error file not found " + ex);
        } catch (IOException ex) {
            System.out.println("Error IOException " + ex);
        }

        float scale = 1.0f;
        float rotation = 0f;

        int nbrePages = document.getNumberOfPages();
        if (pageNumber > nbrePages) {
            pageNumber = 1;
        }
        BufferedImage bufferedImage = (BufferedImage) document.getPageImage(pageNumber - 1, GraphicsRenderingHints.PRINT, Page.BOUNDARY_CROPBOX, rotation, scale);
        document.dispose();
        ImageIO.write(bufferedImage, destinationFormatName, destination);
    }

    public static void cropImage(InputStream inputStream, String format, OutputStream outputStream, int x, int y, int width, int height) throws IOException {
        BufferedImage srcImg = ImageIO.read(inputStream);
        int type = (srcImg.getTransparency() == Transparency.OPAQUE) ? BufferedImage.TYPE_INT_RGB : BufferedImage.TYPE_INT_ARGB;
        BufferedImage destImg = new BufferedImage(width, height, type);
        Graphics g = destImg.getGraphics();
        g.drawImage(srcImg, 0, 0, width, height, x, y, x + width, y + height, null);
        g.dispose();
        ImageIO.write(destImg, format, outputStream);
    }

    public static boolean testImage(File file) {
        try {
            FileInputStream inputStream = new FileInputStream(file);
            boolean test = testImage(inputStream);
            inputStream.close();
            return test;
        } catch (IOException ioe) {
            return false;
        }
    }

    public static boolean testImage(InputStream inputStream) {
        try {
            BufferedImage srcImg = ImageIO.read(inputStream);
            return true;
        } catch (IOException ioe) {
            return false;
        }
    }

}
