/* UtilLib_Servlet - Copyright (c) 2008-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * @author Vincent Calame
 */
public class CommandRequestMap extends DefaultRequestMap {

    public final static String LOCALE_PARAMETER = "locale";
    public final static String COMMAND_PARAMETER = "cmd";
    public final static String PAGE_PARAMETER = "page";
    public final static String JSON_PARAMETER = "json";
    public final static String REDIRECT_PARAMETER = "redirect";
    public final static String TEST_PARAMETER = "test";
    public final static String RESPONSEFACTORY_PARAMETER = "responsefactory";
    public final static String DATATYPE_PARAMETER = "datatype";
    public final static String CALLBACK_PARAMETER = "callback";
    private boolean localeInit = false;
    private boolean commandInit = false;
    private boolean pageInit = false;
    private boolean responseFactoryInit = false;
    private boolean redirectInit = false;
    private String command;
    private String locale;
    private String page;
    private String redirect;
    private String responseFactory;
    private short dataType = UNDETERMINED_DATATYPE;

    public CommandRequestMap(HttpServletRequest request) {
        super(request);
        initDataType();
    }

    public CommandRequestMap(HttpServletRequest request, MultiPartParser multiPartParser) {
        super(request, multiPartParser);
        initDataType();
    }

    private void initDataType() {
        String dataTypeParameter = getParameter(DATATYPE_PARAMETER);
        if (dataTypeParameter != null) {
            try {
                dataType = ResponseDataTypeUtils.dataTypeToShort(dataTypeParameter);
            } catch (IllegalArgumentException iae) {
            }
        }
    }

    public String getLocaleParameter() {
        if (localeInit) {
            return locale;
        }
        locale = getParameterString(LOCALE_PARAMETER);
        localeInit = true;
        return locale;
    }

    public String getCommandParameter() {
        if (commandInit) {
            return command;
        }
        command = getParameterString(COMMAND_PARAMETER);
        commandInit = true;
        return command;
    }

    public boolean isPageDefined() {
        String pageVal = getPageParameter();
        if ((pageVal == null) || (pageVal.length() == 0)) {
            return false;
        }
        return true;
    }

    public String getPageParameter() {
        if (pageInit) {
            return page;
        }
        page = getParameterString(PAGE_PARAMETER);
        pageInit = true;
        return page;
    }

    public String getJsonParameter() {
        return getParameterString(JSON_PARAMETER);
    }

    public String getCallbackParameter() {
        return getParameterString(CALLBACK_PARAMETER);
    }

    public boolean isRedirectDefined() {
        String redirectVal = getRedirectParameter();
        if ((redirectVal == null) || (redirectVal.length() == 0)) {
            return false;
        }
        return true;
    }

    public String getRedirectParameter() {
        if (redirectInit) {
            return redirect;
        }
        redirect = getParameterString(REDIRECT_PARAMETER);
        redirectInit = true;
        return redirect;
    }

    public boolean isTest() {
        String testString = getParameterString(TEST_PARAMETER);
        if (testString == null) {
            return false;
        }
        testString = testString.toLowerCase();
        if ((testString.equals("1")) || (testString.equals("true"))) {
            return true;
        }
        return false;
    }

    public String getResponseFactoryParameter() {
        if (responseFactoryInit) {
            return responseFactory;
        }
        responseFactory = getParameterString(RESPONSEFACTORY_PARAMETER);
        responseFactoryInit = true;
        return responseFactory;
    }

    private String getParameterString(String parameterName) {
        return getParameter(parameterName);
    }

    public boolean asksForXml() {
        return (dataType == XML_DATATYPE);
    }

    public short getResponseDataType() {
        return dataType;
    }

}
