/* UtilLib_Servlet  - Copyright (c) 2010-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets.handlers;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.ResourceItem;
import net.mapeadores.util.localisation.ResourceLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.CommandMessageJsonProducer;
import net.mapeadores.util.logging.ProcessLogUtils;
import net.mapeadores.util.servlets.CommandRequestMap;
import net.mapeadores.util.servlets.ResponseDataType;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessageResponseHandler implements ResponseHandler {

    private long lastModified = ResponseHandler.LASTMODIFIED_NOCACHE;
    private CommandMessage commandMessage;
    private ResourceLocalisation resourceLocalisation;
    private boolean isHtml = false;

    public CommandMessageResponseHandler(CommandMessage commandMessage, ResourceLocalisation resourceLocalisation) {
        this.commandMessage = commandMessage;
        this.resourceLocalisation = resourceLocalisation;
    }

    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter pw = response.getWriter();
        HtmlPrinter htmlPrinter = new HtmlPrinter();
        htmlPrinter.initPrinter(pw);
        htmlPrinter.addResourceLocalisation(resourceLocalisation);
        htmlPrinter.startHtml(0);
        htmlPrinter.startHead();
        htmlPrinter.startTitle();
        htmlPrinter.escape(commandMessage.getTitleResourceItem().getResourceKey());
        htmlPrinter.endTitle();
        htmlPrinter.endHead();
        htmlPrinter.startBody();
        htmlPrinter.startP();
        HTMLAttributes spanAttributes = new HTMLAttributes();
        if (commandMessage.isErrorMessage()) {
            spanAttributes.setStyle("color: red");
        } else {
            spanAttributes.setStyle("color: green");
        }
        htmlPrinter.startSpan(spanAttributes);
        htmlPrinter.localize(commandMessage.getTitleResourceItem());
        htmlPrinter.endSpan();
        htmlPrinter.endP();
        if (commandMessage.hasMultiError()) {
            htmlPrinter.startUl();
            List<ResourceItem> list = commandMessage.getMultiErrorList();
            for (ResourceItem resourceItem : list) {
                htmlPrinter.startLi();
                htmlPrinter.localize(resourceItem);
                htmlPrinter.endLi();
            }
            htmlPrinter.endUl();
        }
        if (commandMessage.hasLog()) {
            htmlPrinter.startPre();
            htmlPrinter.escape(ProcessLogUtils.toString(commandMessage.getProcessLog()), true);
            htmlPrinter.endPre();
        }
        htmlPrinter.endBody();
        pw.close();
    }

    public static ResponseHandler getInstance(CommandMessage commandMessage, ResourceLocalisation resourceLocalisation, CommandRequestMap requestMap) {
        short responseDataType = requestMap.getResponseDataType();
        switch (responseDataType) {
            case ResponseDataType.JSON_DATATYPE:
            case ResponseDataType.JSON_IN_HTML_DATATYPE:
            case ResponseDataType.JSON_P_DATATYPE:
                return getJsonInstance(commandMessage, resourceLocalisation, requestMap);
        }
        return getHtmlInstance(commandMessage, resourceLocalisation);
    }

    public static ResponseHandler getJsonInstance(CommandMessage commandMessage, ResourceLocalisation resourceLocalisation, CommandRequestMap requestMap) {
        JsonProducer jsonProducer = null;
        if (commandMessage != null) {
            jsonProducer = new CommandMessageJsonProducer(commandMessage, resourceLocalisation);
        }
        return new JsonResponseHandler(jsonProducer, requestMap.getResponseDataType(), requestMap.getCallbackParameter());
    }

    public static ResponseHandler getHtmlInstance(CommandMessage commandMessage, ResourceLocalisation resourceLocalisation) {
        return new CommandMessageResponseHandler(commandMessage, resourceLocalisation);
    }

}
