/* UtilLib_Servlet - Copyright (c) 2010 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.servlets.handlers;

import java.io.IOException;
import java.io.OutputStream;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class StreamResponseHandler implements ResponseHandler {

    private long lastModified = ResponseHandler.LASTMODIFIED_NOCACHE;
    private StreamProducer streamProducer;
    private String mimeType = "application/octet-stream";
    private String charset = null;
    private int length = -1;

    public StreamResponseHandler(StreamProducer streamProducer) {
        this.streamProducer = streamProducer;
        String mimeType = streamProducer.getMimeType();
        if (mimeType != null) {
            this.mimeType = mimeType;
        }
        this.charset = streamProducer.getCharset();
    }

    public long getLastModified() {
        return lastModified;
    }

    public void setLastModified(long lastModified) {
        this.lastModified = lastModified;
    }

    public void setMimeType(String mimeType) {
        this.mimeType = mimeType;
    }

    public void setCharset(String charset) {
        this.charset = charset;
    }

    public void setLength(int length) {
        this.length = length;
    }

    public void handleResponse(HttpServletResponse response) throws IOException {
        String contentType = mimeType;
        if (charset != null) {
            contentType = contentType + ";charset=" + charset;
        }
        response.setContentType(contentType);
        if (length > -1) {
            response.setContentLength(length);
        }
        OutputStream output = response.getOutputStream();
        streamProducer.writeStream(output);
        output.close();
    }

}
